@@ -28,6 +28,10 @@
    tracking between which failed */
 #undef DUMP_FAILURE
 
+/* define this to generate PNG images with content of search areas
+   on every itteration of tracking */
+#undef DUMP_ALWAYS
+
 #include "libmv-capi.h"
 
 #include "third_party/gflags/gflags/gflags.h"
@@ -45,6 +49,7 @@
 #include "libmv/tracking/trklt_region_tracker.h"
 #include "libmv/tracking/lmicklt_region_tracker.h"
 #include "libmv/tracking/pyramid_region_tracker.h"
+#include "libmv/tracking/track_region.h"
 
 #include "libmv/simple_pipeline/callbacks.h"
 #include "libmv/simple_pipeline/tracks.h"
@@ -54,11 +59,12 @@
 #include "libmv/simple_pipeline/pipeline.h"
 #include "libmv/simple_pipeline/camera_intrinsics.h"
 #include "libmv/simple_pipeline/rigid_registration.h"
+#include "libmv/simple_pipeline/modal_solver.h"
 
 #include <stdlib.h>
 #include <assert.h>
 
-#ifdef DUMP_FAILURE
+#if defined(DUMP_FAILURE) || defined (DUMP_ALWAYS)
 #  include <png.h>
 #endif
 
@@ -96,7 +102,7 @@ void libmv_initLogging(const char *argv0)
 void libmv_startDebugLogging(void)
 {
 	google::SetCommandLineOption("logtostderr", "1");
-	google::SetCommandLineOption("v", "0");
+	google::SetCommandLineOption("v", "2");
 	google::SetCommandLineOption("stderrthreshold", "1");
 	google::SetCommandLineOption("minloglevel", "0");
 	V3D::optimizerVerbosenessLevel = 1;
@@ -157,21 +163,36 @@ libmv_RegionTracker *libmv_bruteRegionTrackerNew(int half_window_size, double mi
 	return (libmv_RegionTracker *)brute_region_tracker;
 }
 
-static void floatBufToImage(const float *buf, int width, int height, libmv::FloatImage *image)
+static void floatBufToImage(const float *buf, int width, int height, int channels, libmv::FloatImage *image)
 {
-	int x, y, a = 0;
+	int x, y, k, a = 0;
 
-	image->resize(height, width);
+	image->Resize(height, width, channels);
 
 	for (y = 0; y < height; y++) {
 		for (x = 0; x < width; x++) {
-			(*image)(y, x, 0) = buf[a++];
+			for (k = 0; k < channels; k++) {
+				(*image)(y, x, k) = buf[a++];
+			}
 		}
 	}
 }
 
-#ifdef DUMP_FAILURE
-void savePNGImage(png_bytep *row_pointers, int width, int height, int depth, int color_type, char *file_name)
+static void imageToFloatBuf(const libmv::FloatImage *image, int channels, float *buf)
+{
+	int x, y, k, a = 0;
+
+	for (y = 0; y < image->Height(); y++) {
+		for (x = 0; x < image->Width(); x++) {
+			for (k = 0; k < channels; k++) {
+				buf[a++] = (*image)(y, x, k);
+			}
+		}
+	}
+}
+
+#if defined(DUMP_FAILURE) || defined (DUMP_ALWAYS)
+static void savePNGImage(png_bytep *row_pointers, int width, int height, int depth, int color_type, char *file_name)
 {
 	png_infop info_ptr;
 	png_structp png_ptr;
@@ -222,7 +243,7 @@ void savePNGImage(png_bytep *row_pointers, int width, int height, int depth, int
 	fclose(fp);
 }
 
-static void saveImage(char *prefix, libmv::FloatImage image, int x0, int y0)
+static void saveImage(const char *prefix, libmv::FloatImage image, int x0, int y0)
 {
 	int x, y;
 	png_bytep *row_pointers;
@@ -233,14 +254,14 @@ static void saveImage(char *prefix, libmv::FloatImage image, int x0, int y0)
 		row_pointers[y]= (png_bytep)malloc(sizeof(png_byte)*4*image.Width());
 
 		for (x = 0; x < image.Width(); x++) {
-			if (x0 == x && y0 == y) {
+			if (x0 == x && image.Height() - y0 - 1 == y) {
 				row_pointers[y][x*4+0]= 255;
 				row_pointers[y][x*4+1]= 0;
 				row_pointers[y][x*4+2]= 0;
 				row_pointers[y][x*4+3]= 255;
 			}
 			else {
-				float pixel = image(y, x, 0);
+				float pixel = image(image.Height() - y - 1, x, 0);
 				row_pointers[y][x*4+0]= pixel*255;
 				row_pointers[y][x*4+1]= pixel*255;
 				row_pointers[y][x*4+2]= pixel*255;
@@ -262,7 +283,7 @@ static void saveImage(char *prefix, libmv::FloatImage image, int x0, int y0)
 	free(row_pointers);
 }
 
-static void saveBytesImage(char *prefix, unsigned char *data, int width, int height)
+static void saveBytesImage(const char *prefix, unsigned char *data, int width, int height)
 {
 	int x, y;
 	png_bytep *row_pointers;
@@ -301,19 +322,23 @@ int libmv_regionTrackerTrack(libmv_RegionTracker *libmv_tracker, const float *im
 	libmv::RegionTracker *region_tracker = (libmv::RegionTracker *)libmv_tracker;
 	libmv::FloatImage old_patch, new_patch;
 
-	floatBufToImage(ima1, width, height, &old_patch);
-	floatBufToImage(ima2, width, height, &new_patch);
+	floatBufToImage(ima1, width, height, 1, &old_patch);
+	floatBufToImage(ima2, width, height, 1, &new_patch);
 
-#ifndef DUMP_FAILURE
+#if !defined(DUMP_FAILURE) && !defined(DUMP_ALWAYS)
 	return region_tracker->Track(old_patch, new_patch, x1, y1, x2, y2);
 #else
 	{
-		double sx2 = *x2, sy2 = *y2;
+		/* double sx2 = *x2, sy2 = *y2; */
 		int result = region_tracker->Track(old_patch, new_patch, x1, y1, x2, y2);
 
+#if defined(DUMP_ALWAYS)
+		{
+#else
 		if (!result) {
+#endif
 			saveImage("old_patch", old_patch, x1, y1);
-			saveImage("new_patch", new_patch, sx2, sy2);
+			saveImage("new_patch", new_patch, *x2, *y2);
 		}
 
 		return result;
@@ -328,6 +353,123 @@ void libmv_regionTrackerDestroy(libmv_RegionTracker *libmv_tracker)
 	delete region_tracker;
 }
 
+/* ************ Planar tracker ************ */
+
+/* TrackRegion (new planar tracker) */
+int libmv_trackRegion(const struct libmv_trackRegionOptions *options,
+                      const float *image1, int image1_width, int image1_height,
+                      const float *image2, int image2_width, int image2_height,
+                      const double *x1, const double *y1,
+                      struct libmv_trackRegionResult *result,
+                      double *x2, double *y2)
+{
+	double xx1[5], yy1[5];
+	double xx2[5], yy2[5];
+	bool tracking_result = false;
+
+	/* Convert to doubles for the libmv api. The four corners and the center. */
+	for (int i = 0; i < 5; ++i) {
+		xx1[i] = x1[i];
+		yy1[i] = y1[i];
+		xx2[i] = x2[i];
+		yy2[i] = y2[i];
+	}
+
+	libmv::TrackRegionOptions track_region_options;
+	libmv::FloatImage image1_mask;
+
+	switch (options->motion_model) {
+#define LIBMV_CONVERT(the_model) \
+    case libmv::TrackRegionOptions::the_model: \
+		track_region_options.mode = libmv::TrackRegionOptions::the_model; \
+		break;
+		LIBMV_CONVERT(TRANSLATION)
+		LIBMV_CONVERT(TRANSLATION_ROTATION)
+		LIBMV_CONVERT(TRANSLATION_SCALE)
+		LIBMV_CONVERT(TRANSLATION_ROTATION_SCALE)
+		LIBMV_CONVERT(AFFINE)
+		LIBMV_CONVERT(HOMOGRAPHY)
+#undef LIBMV_CONVERT
+	}
+
+	track_region_options.minimum_correlation = options->minimum_correlation;
+	track_region_options.max_iterations = options->num_iterations;
+	track_region_options.sigma = options->sigma;
+	track_region_options.num_extra_points = 1;
+	track_region_options.image1_mask = NULL;
+	track_region_options.use_brute_initialization = options->use_brute;
+	track_region_options.use_normalized_intensities = options->use_normalization;
+
+	if (options->image1_mask) {
+		floatBufToImage(options->image1_mask, image1_width, image1_height, 1, &image1_mask);
+
+		track_region_options.image1_mask = &image1_mask;
+	}
+
+	/* Convert from raw float buffers to libmv's FloatImage. */
+	libmv::FloatImage old_patch, new_patch;
+	floatBufToImage(image1, image1_width, image1_height, 1, &old_patch);
+	floatBufToImage(image2, image2_width, image2_height, 1, &new_patch);
+
+	libmv::TrackRegionResult track_region_result;
+	libmv::TrackRegion(old_patch, new_patch, xx1, yy1, track_region_options, xx2, yy2, &track_region_result);
+
+	/* Convert to floats for the blender api. */
+	for (int i = 0; i < 5; ++i) {
+		x2[i] = xx2[i];
+		y2[i] = yy2[i];
+	}
+
+	/* TODO(keir): Update the termination string with failure details. */
+	if (track_region_result.termination == libmv::TrackRegionResult::PARAMETER_TOLERANCE ||
+	    track_region_result.termination == libmv::TrackRegionResult::FUNCTION_TOLERANCE  ||
+	    track_region_result.termination == libmv::TrackRegionResult::GRADIENT_TOLERANCE  ||
+	    track_region_result.termination == libmv::TrackRegionResult::NO_CONVERGENCE)
+	{
+		tracking_result = true;
+	}
+
+#if defined(DUMP_FAILURE) || defined(DUMP_ALWAYS)
+#if defined(DUMP_ALWAYS)
+	{
+#else
+	if (!tracking_result) {
+#endif
+		saveImage("old_patch", old_patch, x1[4], y1[4]);
+		saveImage("new_patch", new_patch, x2[4], y2[4]);
+
+		if (options->image1_mask)
+			saveImage("mask", image1_mask, x2[4], y2[4]);
+	}
+#endif
+
+	return tracking_result;
+}
+
+void libmv_samplePlanarPatch(const float *image, int width, int height,
+                             int channels, const double *xs, const double *ys,
+                             int num_samples_x, int num_samples_y,
+                             const float *mask, float *patch,
+                             double *warped_position_x, double *warped_position_y)
+{
+	libmv::FloatImage libmv_image, libmv_patch, libmv_mask;
+	libmv::FloatImage *libmv_mask_for_sample = NULL;
+
+	floatBufToImage(image, width, height, channels, &libmv_image);
+
+	if (mask) {
+		floatBufToImage(mask, width, height, 1, &libmv_mask);
+
+		libmv_mask_for_sample = &libmv_mask;
+	}
+
+	libmv::SamplePlanarPatch(libmv_image, xs, ys, num_samples_x, num_samples_y,
+	                         libmv_mask_for_sample, &libmv_patch,
+	                         warped_position_x, warped_position_y);
+
+	imageToFloatBuf(&libmv_patch, channels, patch);
+}
+
 /* ************ Tracks ************ */
 
 libmv_Tracks *libmv_tracksNew(void)
@@ -384,9 +526,89 @@ int libmv_refineParametersAreValid(int parameters) {
 	                       LIBMV_REFINE_RADIAL_DISTORTION_K1));
 }
 
+static void libmv_solveRefineIntrinsics(libmv::Tracks *tracks, libmv::CameraIntrinsics *intrinsics,
+			libmv::EuclideanReconstruction *reconstruction, int refine_intrinsics,
+			reconstruct_progress_update_cb progress_update_callback, void *callback_customdata)
+{
+	/* only a few combinations are supported but trust the caller */
+	int libmv_refine_flags = 0;
+
+	if (refine_intrinsics & LIBMV_REFINE_FOCAL_LENGTH) {
+		libmv_refine_flags |= libmv::BUNDLE_FOCAL_LENGTH;
+	}
+	if (refine_intrinsics & LIBMV_REFINE_PRINCIPAL_POINT) {
+		libmv_refine_flags |= libmv::BUNDLE_PRINCIPAL_POINT;
+	}
+	if (refine_intrinsics & LIBMV_REFINE_RADIAL_DISTORTION_K1) {
+		libmv_refine_flags |= libmv::BUNDLE_RADIAL_K1;
+	}
+	if (refine_intrinsics & LIBMV_REFINE_RADIAL_DISTORTION_K2) {
+		libmv_refine_flags |= libmv::BUNDLE_RADIAL_K2;
+	}
+
+	progress_update_callback(callback_customdata, 1.0, "Refining solution");
+
+	libmv::EuclideanBundleCommonIntrinsics(*(libmv::Tracks *)tracks, libmv_refine_flags,
+		reconstruction, intrinsics);
+}
 
 libmv_Reconstruction *libmv_solveReconstruction(libmv_Tracks *tracks, int keyframe1, int keyframe2,
-			int refine_intrinsics, double focal_length, double principal_x, double principal_y, double k1, double k2, double k3,
+			int refine_intrinsics, double focal_length, double principal_x, double principal_y,
+			double k1, double k2, double k3, struct libmv_reconstructionOptions *options,
+			reconstruct_progress_update_cb progress_update_callback, void *callback_customdata)
+{
+	/* Invert the camera intrinsics. */
+	libmv::vector<libmv::Marker> markers = ((libmv::Tracks*)tracks)->AllMarkers();
+	libmv_Reconstruction *libmv_reconstruction = new libmv_Reconstruction();
+	libmv::EuclideanReconstruction *reconstruction = &libmv_reconstruction->reconstruction;
+	libmv::CameraIntrinsics *intrinsics = &libmv_reconstruction->intrinsics;
+	libmv::ReconstructionOptions reconstruction_options;
+
+	ReconstructUpdateCallback update_callback =
+		ReconstructUpdateCallback(progress_update_callback, callback_customdata);
+
+	intrinsics->SetFocalLength(focal_length, focal_length);
+	intrinsics->SetPrincipalPoint(principal_x, principal_y);
+	intrinsics->SetRadialDistortion(k1, k2, k3);
+
+	reconstruction_options.success_threshold = options->success_threshold;
+	reconstruction_options.use_fallback_reconstruction = options->use_fallback_reconstruction;
+
+	for (int i = 0; i < markers.size(); ++i) {
+		intrinsics->InvertIntrinsics(markers[i].x,
+			markers[i].y,
+			&(markers[i].x),
+			&(markers[i].y));
+	}
+
+	libmv::Tracks normalized_tracks(markers);
+
+	LG << "frames to init from: " << keyframe1 << " " << keyframe2;
+	libmv::vector<libmv::Marker> keyframe_markers =
+		normalized_tracks.MarkersForTracksInBothImages(keyframe1, keyframe2);
+	LG << "number of markers for init: " << keyframe_markers.size();
+
+	update_callback.invoke(0, "Initial reconstruction");
+
+	libmv::EuclideanReconstructTwoFrames(keyframe_markers, reconstruction);
+	libmv::EuclideanBundle(normalized_tracks, reconstruction);
+	libmv::EuclideanCompleteReconstruction(reconstruction_options, normalized_tracks,
+	                                       reconstruction, &update_callback);
+
+	if (refine_intrinsics) {
+		libmv_solveRefineIntrinsics((libmv::Tracks *)tracks, intrinsics, reconstruction,
+			refine_intrinsics, progress_update_callback, callback_customdata);
+	}
+
+	progress_update_callback(callback_customdata, 1.0, "Finishing solution");
+	libmv_reconstruction->tracks = *(libmv::Tracks *)tracks;
+	libmv_reconstruction->error = libmv::EuclideanReprojectionError(*(libmv::Tracks *)tracks, *reconstruction, *intrinsics);
+
+	return (libmv_Reconstruction *)libmv_reconstruction;
+}
+
+struct libmv_Reconstruction *libmv_solveModal(struct libmv_Tracks *tracks, double focal_length,
+			double principal_x, double principal_y, double k1, double k2, double k3,
 			reconstruct_progress_update_cb progress_update_callback, void *callback_customdata)
 {
 	/* Invert the camera intrinsics. */
@@ -411,37 +633,7 @@ libmv_Reconstruction *libmv_solveReconstruction(libmv_Tracks *tracks, int keyfra
 
 	libmv::Tracks normalized_tracks(markers);
 
-	LG << "frames to init from: " << keyframe1 << " " << keyframe2;
-	libmv::vector<libmv::Marker> keyframe_markers =
-		normalized_tracks.MarkersForTracksInBothImages(keyframe1, keyframe2);
-	LG << "number of markers for init: " << keyframe_markers.size();
-
-	update_callback.invoke(0, "Initial reconstruction");
-
-	libmv::EuclideanReconstructTwoFrames(keyframe_markers, reconstruction);
-	libmv::EuclideanBundle(normalized_tracks, reconstruction);
-	libmv::EuclideanCompleteReconstruction(normalized_tracks, reconstruction, &update_callback);
-
-	if (refine_intrinsics) {
-		/* only a few combinations are supported but trust the caller */
-		int libmv_refine_flags = 0;
-		if (refine_intrinsics & LIBMV_REFINE_FOCAL_LENGTH) {
-			libmv_refine_flags |= libmv::BUNDLE_FOCAL_LENGTH;
-		}
-		if (refine_intrinsics & LIBMV_REFINE_PRINCIPAL_POINT) {
-			libmv_refine_flags |= libmv::BUNDLE_PRINCIPAL_POINT;
-		}
-		if (refine_intrinsics & LIBMV_REFINE_RADIAL_DISTORTION_K1) {
-			libmv_refine_flags |= libmv::BUNDLE_RADIAL_K1;
-		}
-		if (refine_intrinsics & LIBMV_REFINE_RADIAL_DISTORTION_K2) {
-			libmv_refine_flags |= libmv::BUNDLE_RADIAL_K2;
-		}
-
-		progress_update_callback(callback_customdata, 1.0, "Refining solution");
-		libmv::EuclideanBundleCommonIntrinsics(*(libmv::Tracks *)tracks, libmv_refine_flags,
-			reconstruction, intrinsics);
-	}
+	libmv::ModalSolver(normalized_tracks, reconstruction, &update_callback);
 
 	progress_update_callback(callback_customdata, 1.0, "Finishing solution");
 	libmv_reconstruction->tracks = *(libmv::Tracks *)tracks;
@@ -708,7 +900,7 @@ void libmv_CameraIntrinsicsUpdate(struct libmv_CameraIntrinsics *libmvIntrinsics
 		intrinsics->SetFocalLength(focal_length, focal_length);
 
 	if (intrinsics->principal_point_x() != principal_x || intrinsics->principal_point_y() != principal_y)
-		intrinsics->SetFocalLength(focal_length, focal_length);
+		intrinsics->SetPrincipalPoint(principal_x, principal_y);
 
 	if (intrinsics->k1() != k1 || intrinsics->k2() != k2 || intrinsics->k3() != k3)
 		intrinsics->SetRadialDistortion(k1, k2, k3);
@@ -844,7 +1036,7 @@ void libmv_InvertIntrinsics(double focal_length, double principal_x, double prin
 
 /* ************ point clouds ************ */
 
-void libmvTransformToMat4(libmv::Mat3 &R, libmv::Vec3 &S, libmv::Vec3 &t, double M[4][4])
+static void libmvTransformToMat4(libmv::Mat3 &R, libmv::Vec3 &S, libmv::Vec3 &t, double M[4][4])
 {
 	for (int j = 0; j < 3; ++j)
 		for (int k = 0; k < 3; ++k)
