@@ -47,7 +47,7 @@ protected:
 	/**
 	 * If there is no reader it is created out of this factory.
 	 */
-	AUD_Reference<AUD_IFactory> m_factory;
+	boost::shared_ptr<AUD_IFactory> m_factory;
 
 	/**
 	 * Returns the reader created out of the factory.
@@ -55,7 +55,7 @@ protected:
 	 * classes.
 	 * \return The reader created out of the factory.
 	 */
-	inline AUD_Reference<AUD_IReader> getReader() const
+	inline boost::shared_ptr<AUD_IReader> getReader() const
 	{
 		return m_factory->createReader();
 	}
@@ -65,7 +65,7 @@ public:
 	 * Creates a new factory.
 	 * \param factory The input factory.
 	 */
-	AUD_EffectFactory(AUD_Reference<AUD_IFactory> factory);
+	AUD_EffectFactory(boost::shared_ptr<AUD_IFactory> factory);
 
 	/**
 	 * Destroys the factory.
@@ -76,7 +76,7 @@ public:
 	 * Returns the saved factory.
 	 * \return The factory or NULL if there has no factory been saved.
 	 */
-	AUD_Reference<AUD_IFactory> getFactory() const;
+	boost::shared_ptr<AUD_IFactory> getFactory() const;
 };
 
 #endif //__AUD_EFFECTFACTORY_H__
