@@ -31,7 +31,6 @@
 
 
 /**
-
  * Implementation of external api for CSG part of BSP lib interface.
  */
 
@@ -39,7 +38,7 @@
 #include "BSP_CSGMesh_CFIterator.h"
 #include "MEM_RefCountPtr.h"
 
-#include "../../boolop/extern/BOP_Interface.h"
+#include "BOP_Interface.h"
 #include <iostream>
 using namespace std;
 
@@ -89,16 +88,16 @@ CSG_PerformBooleanOperation(
 
 	BoolOpType boolType;
 	
-	switch( op_type ) {
-	case e_csg_union:
-		boolType = BOP_UNION;
-		break;
-	case e_csg_difference:
-		boolType = BOP_DIFFERENCE;
-		break;
-	default:
-		boolType = BOP_INTERSECTION;
-		break;
+	switch (op_type) {
+		case e_csg_union:
+			boolType = BOP_UNION;
+			break;
+		case e_csg_difference:
+			boolType = BOP_DIFFERENCE;
+			break;
+		default:
+			boolType = BOP_INTERSECTION;
+			break;
 	}
 
 	BoolOpState boolOpResult;
