@@ -233,16 +233,16 @@ static void xml_read_film(const XMLReadState& state, pugi::xml_node node)
 	float aspect = (float)cam->width/(float)cam->height;
 
 	if(cam->width >= cam->height) {
-		cam->left = -aspect;
-		cam->right = aspect;
-		cam->bottom = -1.0f;
-		cam->top = 1.0f;
+		cam->viewplane.left = -aspect;
+		cam->viewplane.right = aspect;
+		cam->viewplane.bottom = -1.0f;
+		cam->viewplane.top = 1.0f;
 	}
 	else {
-		cam->left = -1.0f;
-		cam->right = 1.0f;
-		cam->bottom = -1.0f/aspect;
-		cam->top = 1.0f/aspect;
+		cam->viewplane.left = -1.0f;
+		cam->viewplane.right = 1.0f;
+		cam->viewplane.bottom = -1.0f/aspect;
+		cam->viewplane.top = 1.0f/aspect;
 	}
 
 	cam->need_update = true;
@@ -254,6 +254,16 @@ static void xml_read_film(const XMLReadState& state, pugi::xml_node node)
 static void xml_read_integrator(const XMLReadState& state, pugi::xml_node node)
 {
 	Integrator *integrator = state.scene->integrator;
+	
+	xml_read_bool(&integrator->progressive, node, "progressive");
+	
+	if(!integrator->progressive) {
+		xml_read_int(&integrator->diffuse_samples, node, "diffuse_samples");
+		xml_read_int(&integrator->glossy_samples, node, "glossy_samples");
+		xml_read_int(&integrator->transmission_samples, node, "transmission_samples");
+		xml_read_int(&integrator->ao_samples, node, "ao_samples");
+		xml_read_int(&integrator->mesh_light_samples, node, "mesh_light_samples");
+	}
 
 	xml_read_int(&integrator->min_bounce, node, "min_bounce");
 	xml_read_int(&integrator->max_bounce, node, "max_bounce");
@@ -267,8 +277,10 @@ static void xml_read_integrator(const XMLReadState& state, pugi::xml_node node)
 	
 	xml_read_bool(&integrator->transparent_shadows, node, "transparent_shadows");
 	xml_read_bool(&integrator->no_caustics, node, "no_caustics");
+	xml_read_float(&integrator->filter_glossy, node, "blur_glossy");
 	
 	xml_read_int(&integrator->seed, node, "seed");
+	xml_read_float(&integrator->sample_clamp, node, "sample_clamp");
 }
 
 /* Camera */
@@ -284,15 +296,28 @@ static void xml_read_camera(const XMLReadState& state, pugi::xml_node node)
 	xml_read_float(&cam->farclip, node, "farclip");
 	xml_read_float(&cam->aperturesize, node, "aperturesize"); // 0.5*focallength/fstop
 	xml_read_float(&cam->focaldistance, node, "focaldistance");
-	xml_read_float(&cam->shutteropen, node, "shutteropen");
-	xml_read_float(&cam->shutterclose, node, "shutterclose");
+	xml_read_float(&cam->shuttertime, node, "shuttertime");
 
 	if(xml_equal_string(node, "type", "orthographic"))
 		cam->type = CAMERA_ORTHOGRAPHIC;
 	else if(xml_equal_string(node, "type", "perspective"))
 		cam->type = CAMERA_PERSPECTIVE;
-	else if(xml_equal_string(node, "type", "environment"))
-		cam->type = CAMERA_ENVIRONMENT;
+	else if(xml_equal_string(node, "type", "panorama"))
+		cam->type = CAMERA_PANORAMA;
+
+	if(xml_equal_string(node, "panorama_type", "equirectangular"))
+		cam->panorama_type = PANORAMA_EQUIRECTANGULAR;
+	else if(xml_equal_string(node, "panorama_type", "fisheye_equidistant"))
+		cam->panorama_type = PANORAMA_FISHEYE_EQUIDISTANT;
+	else if(xml_equal_string(node, "panorama_type", "fisheye_equisolid"))
+		cam->panorama_type = PANORAMA_FISHEYE_EQUISOLID;
+
+	xml_read_float(&cam->fisheye_fov, node, "fisheye_fov");
+	xml_read_float(&cam->fisheye_lens, node, "fisheye_lens");
+
+	xml_read_float(&cam->sensorwidth, node, "sensorwidth");
+	xml_read_float(&cam->sensorheight, node, "sensorheight");
+
 
 	cam->matrix = state.tfm;
 
@@ -354,6 +379,9 @@ static void xml_read_shader_graph(const XMLReadState& state, Shader *shader, pug
 		else if(string_iequals(node.name(), "checker_texture")) {
 			snode = new CheckerTextureNode();
 		}
+		else if(string_iequals(node.name(), "brick_texture")) {
+			snode = new BrickTextureNode();
+		}
 		else if(string_iequals(node.name(), "gradient_texture")) {
 			GradientTextureNode *blend = new GradientTextureNode();
 			xml_read_enum(&blend->type, GradientTextureNode::type_enum, node, "type");
@@ -379,9 +407,9 @@ static void xml_read_shader_graph(const XMLReadState& state, Shader *shader, pug
 			snode = dist;
 		}
 		else if(string_iequals(node.name(), "wave_texture")) {
-			WaveTextureNode *wood = new WaveTextureNode();
-			xml_read_enum(&wood->type, WaveTextureNode::type_enum, node, "type");
-			snode = wood;
+			WaveTextureNode *wave = new WaveTextureNode();
+			xml_read_enum(&wave->type, WaveTextureNode::type_enum, node, "type");
+			snode = wave;
 		}
 		else if(string_iequals(node.name(), "normal")) {
 			snode = new NormalNode();
@@ -459,6 +487,7 @@ static void xml_read_shader_graph(const XMLReadState& state, Shader *shader, pug
 		else if(string_iequals(node.name(), "mix")) {
 			MixNode *mix = new MixNode();
 			xml_read_enum(&mix->type, MixNode::type_enum, node, "type");
+			xml_read_bool(&mix->use_clamp, node, "use_clamp");
 			snode = mix;
 		}
 		else if(string_iequals(node.name(), "gamma")) {
@@ -490,6 +519,7 @@ static void xml_read_shader_graph(const XMLReadState& state, Shader *shader, pug
 		else if(string_iequals(node.name(), "math")) {
 			MathNode *math = new MathNode();
 			xml_read_enum(&math->type, MathNode::type_enum, node, "type");
+			xml_read_bool(&math->use_clamp, node, "use_clamp");
 			snode = math;
 		}
 		else if(string_iequals(node.name(), "vector_math")) {
@@ -561,6 +591,7 @@ static void xml_read_shader_graph(const XMLReadState& state, Shader *shader, pug
 					if(string_iequals(in->name, attr.name())) {
 						switch(in->type) {
 							case SHADER_SOCKET_FLOAT:
+							case SHADER_SOCKET_INT:
 								xml_read_float(&in->value.x, node, attr.name());
 								break;
 							case SHADER_SOCKET_COLOR:
@@ -660,7 +691,7 @@ static void xml_read_mesh(const XMLReadState& state, pugi::xml_node node)
 				for(int j = 0; j < nverts[i]-2; j++) {
 					int v0 = verts[index_offset];
 					int v1 = verts[index_offset + j + 1];
-					int v2 = verts[index_offset + j + 2];;
+					int v2 = verts[index_offset + j + 2];
 
 					sdmesh.add_face(v0, v1, v2);
 				}
@@ -705,7 +736,7 @@ static void xml_read_mesh(const XMLReadState& state, pugi::xml_node node)
 	}
 
 	/* temporary for test compatibility */
-	mesh->attributes.remove(Attribute::STD_VERTEX_NORMAL);
+	mesh->attributes.remove(ATTR_STD_VERTEX_NORMAL);
 }
 
 /* Patch */
@@ -766,7 +797,7 @@ static void xml_read_patch(const XMLReadState& state, pugi::xml_node node)
 		delete patch;
 
 		/* temporary for test compatibility */
-		mesh->attributes.remove(Attribute::STD_VERTEX_NORMAL);
+		mesh->attributes.remove(ATTR_STD_VERTEX_NORMAL);
 	}
 }
 
