@@ -20,6 +20,7 @@
 #include "graph.h"
 #include "light.h"
 #include "nodes.h"
+#include "osl.h"
 #include "scene.h"
 #include "shader.h"
 
@@ -43,6 +44,7 @@ void BlenderSync::find_shader(BL::ID id, vector<uint>& used_shaders, int default
 	for(size_t i = 0; i < scene->shaders.size(); i++) {
 		if(scene->shaders[i] == shader) {
 			used_shaders.push_back(i);
+			scene->shaders[i]->tag_used(scene);
 			break;
 		}
 	}
@@ -75,6 +77,65 @@ static float get_node_output_value(BL::Node b_node, const string& name)
 	return sock.default_value();
 }
 
+static ShaderSocketType convert_socket_type(BL::NodeSocket::type_enum b_type)
+{
+	switch (b_type) {
+	case BL::NodeSocket::type_VALUE:
+		return SHADER_SOCKET_FLOAT;
+	case BL::NodeSocket::type_INT:
+		return SHADER_SOCKET_INT;
+	case BL::NodeSocket::type_VECTOR:
+		return SHADER_SOCKET_VECTOR;
+	case BL::NodeSocket::type_RGBA:
+		return SHADER_SOCKET_COLOR;
+	case BL::NodeSocket::type_SHADER:
+		return SHADER_SOCKET_CLOSURE;
+	case BL::NodeSocket::type_STRING:
+		return SHADER_SOCKET_STRING;
+	
+	case BL::NodeSocket::type_BOOLEAN:
+	case BL::NodeSocket::type_MESH:
+	default:
+		return SHADER_SOCKET_FLOAT;
+	}
+}
+
+static void set_default_value(ShaderInput *input, BL::NodeSocket sock, BL::BlendData b_data, BL::ID b_id)
+{
+	/* copy values for non linked inputs */
+	switch(input->type) {
+	case SHADER_SOCKET_FLOAT: {
+		BL::NodeSocketFloatNone value_sock(sock);
+		input->set(value_sock.default_value());
+		break;
+	}
+	case SHADER_SOCKET_INT: {
+		BL::NodeSocketIntNone value_sock(sock);
+		input->set((float)value_sock.default_value());
+		break;
+	}
+	case SHADER_SOCKET_COLOR: {
+		BL::NodeSocketRGBA rgba_sock(sock);
+		input->set(get_float3(rgba_sock.default_value()));
+		break;
+	}
+	case SHADER_SOCKET_NORMAL:
+	case SHADER_SOCKET_POINT:
+	case SHADER_SOCKET_VECTOR: {
+		BL::NodeSocketVectorNone vec_sock(sock);
+		input->set(get_float3(vec_sock.default_value()));
+		break;
+	}
+	case SHADER_SOCKET_STRING: {
+		BL::NodeSocketStringNone string_sock(sock);
+		input->set((ustring)blender_absolute_path(b_data, b_id, string_sock.default_value()));
+		break;
+	}
+	case SHADER_SOCKET_CLOSURE:
+		break;
+	}
+}
+
 static void get_tex_mapping(TextureMapping *mapping, BL::TexMapping b_mapping)
 {
 	if(!b_mapping)
@@ -97,28 +158,52 @@ static void get_tex_mapping(TextureMapping *mapping, BL::ShaderNodeMapping b_map
 	mapping->translation = get_float3(b_mapping.translation());
 	mapping->rotation = get_float3(b_mapping.rotation());
 	mapping->scale = get_float3(b_mapping.scale());
+
+	mapping->use_minmax = b_mapping.use_min() || b_mapping.use_max();
+
+	if(b_mapping.use_min())
+		mapping->min = get_float3(b_mapping.min());
+	if(b_mapping.use_max())
+		mapping->max = get_float3(b_mapping.max());
 }
 
-static ShaderNode *add_node(BL::BlendData b_data, ShaderGraph *graph, BL::ShaderNode b_node)
+static ShaderNode *add_node(Scene *scene, BL::BlendData b_data, BL::Scene b_scene, ShaderGraph *graph, BL::ShaderNodeTree b_ntree, BL::ShaderNode b_node)
 {
 	ShaderNode *node = NULL;
 
 	switch(b_node.type()) {
 		/* not supported */
-		case BL::ShaderNode::type_CURVE_VEC: break;
 		case BL::ShaderNode::type_GEOMETRY: break;
 		case BL::ShaderNode::type_MATERIAL: break;
 		case BL::ShaderNode::type_MATERIAL_EXT: break;
 		case BL::ShaderNode::type_OUTPUT: break;
-		case BL::ShaderNode::type_SCRIPT: break;
 		case BL::ShaderNode::type_SQUEEZE: break;
 		case BL::ShaderNode::type_TEXTURE: break;
+		case BL::ShaderNode::type_FRAME: break;
 		/* handled outside this function */
 		case BL::ShaderNode::type_GROUP: break;
 		/* existing blender nodes */
+		case BL::ShaderNode::type_REROUTE: {
+			BL::Node::inputs_iterator b_input;
+			b_node.inputs.begin(b_input);
+			BL::Node::outputs_iterator b_output;
+			b_node.outputs.begin(b_output);
+			ProxyNode *proxy = new ProxyNode(convert_socket_type(b_input->type()), convert_socket_type(b_output->type()));
+			node = proxy;
+			break;
+		}
+		case BL::ShaderNode::type_CURVE_VEC: {
+			BL::ShaderNodeVectorCurve b_curve_node(b_node);
+			VectorCurvesNode *curves = new VectorCurvesNode();
+			curvemapping_color_to_array(b_curve_node.mapping(), curves->curves, RAMP_TABLE_SIZE, false);
+			node = curves;
+			break;
+		}
 		case BL::ShaderNode::type_CURVE_RGB: {
-			RGBCurvesNode *ramp = new RGBCurvesNode();
-			node = ramp;
+			BL::ShaderNodeRGBCurve b_curve_node(b_node);
+			RGBCurvesNode *curves = new RGBCurvesNode();
+			curvemapping_color_to_array(b_curve_node.mapping(), curves->curves, RAMP_TABLE_SIZE, true);
+			node = curves;
 			break;
 		}
 		case BL::ShaderNode::type_VALTORGB: {
@@ -160,6 +245,7 @@ static ShaderNode *add_node(BL::BlendData b_data, ShaderGraph *graph, BL::Shader
 			BL::ShaderNodeMixRGB b_mix_node(b_node);
 			MixNode *mix = new MixNode();
 			mix->type = MixNode::type_enum[b_mix_node.blend_type()];
+			mix->use_clamp = b_mix_node.use_clamp();
 			node = mix;
 			break;
 		}
@@ -183,6 +269,7 @@ static ShaderNode *add_node(BL::BlendData b_data, ShaderGraph *graph, BL::Shader
 			BL::ShaderNodeMath b_math_node(b_node);
 			MathNode *math = new MathNode();
 			math->type = MathNode::type_enum[b_math_node.operation()];
+			math->use_clamp = b_math_node.use_clamp();
 			node = math;
 			break;
 		}
@@ -252,6 +339,10 @@ static ShaderNode *add_node(BL::BlendData b_data, ShaderGraph *graph, BL::Shader
 			node = new HoldoutNode();
 			break;
 		}
+		case BL::ShaderNode::type_BSDF_ANISOTROPIC: {
+			node = new WardBsdfNode();
+			break;
+		}
 		case BL::ShaderNode::type_BSDF_DIFFUSE: {
 			node = new DiffuseBsdfNode();
 			break;
@@ -291,6 +382,23 @@ static ShaderNode *add_node(BL::BlendData b_data, ShaderGraph *graph, BL::Shader
 			node = glass;
 			break;
 		}
+		case BL::ShaderNode::type_BSDF_REFRACTION: {
+			BL::ShaderNodeBsdfRefraction b_refraction_node(b_node);
+			RefractionBsdfNode *refraction = new RefractionBsdfNode();
+			switch(b_refraction_node.distribution()) {
+				case BL::ShaderNodeBsdfRefraction::distribution_SHARP:
+					refraction->distribution = ustring("Sharp");
+					break;
+				case BL::ShaderNodeBsdfRefraction::distribution_BECKMANN:
+					refraction->distribution = ustring("Beckmann");
+					break;
+				case BL::ShaderNodeBsdfRefraction::distribution_GGX:
+					refraction->distribution = ustring("GGX");
+					break;
+			}
+			node = refraction;
+			break;
+		}
 		case BL::ShaderNode::type_BSDF_TRANSLUCENT: {
 			node = new TranslucentBsdfNode();
 			break;
@@ -307,6 +415,10 @@ static ShaderNode *add_node(BL::BlendData b_data, ShaderGraph *graph, BL::Shader
 			node = new EmissionNode();
 			break;
 		}
+		case BL::ShaderNode::type_AMBIENT_OCCLUSION: {
+			node = new AmbientOcclusionNode();
+			break;
+		}
 		case BL::ShaderNode::type_VOLUME_ISOTROPIC: {
 			node = new IsotropicVolumeNode();
 			break;
@@ -323,14 +435,112 @@ static ShaderNode *add_node(BL::BlendData b_data, ShaderGraph *graph, BL::Shader
 			node = new LightPathNode();
 			break;
 		}
+		case BL::ShaderNode::type_LIGHT_FALLOFF: {
+			node = new LightFalloffNode();
+			break;
+		}
+		case BL::ShaderNode::type_OBJECT_INFO: {
+			node = new ObjectInfoNode();
+			break;
+		}
+		case BL::ShaderNode::type_PARTICLE_INFO: {
+			node = new ParticleInfoNode();
+			break;
+		}
+		case BL::ShaderNode::type_HAIR_INFO: {
+			node = new HairInfoNode();
+			break;
+		}
+		case BL::ShaderNode::type_BUMP: {
+			node = new BumpNode();
+			break;
+		}
+		case BL::ShaderNode::type_SCRIPT: {
+#ifdef WITH_OSL
+			if(!scene->shader_manager->use_osl())
+				break;
+
+			/* create script node */
+			BL::ShaderNodeScript b_script_node(b_node);
+			OSLScriptNode *script_node = new OSLScriptNode();
+			
+			/* Generate inputs/outputs from node sockets
+			 *
+			 * Note: the node sockets are generated from OSL parameters,
+			 * so the names match those of the corresponding parameters exactly.
+			 *
+			 * Note 2: ShaderInput/ShaderOutput store shallow string copies only!
+			 * Socket names must be stored in the extra lists instead. */
+			BL::Node::inputs_iterator b_input;
+
+			for (b_script_node.inputs.begin(b_input); b_input != b_script_node.inputs.end(); ++b_input) {
+				script_node->input_names.push_back(ustring(b_input->name()));
+				ShaderInput *input = script_node->add_input(script_node->input_names.back().c_str(), convert_socket_type(b_input->type()));
+				set_default_value(input, *b_input, b_data, b_ntree);
+			}
+
+			BL::Node::outputs_iterator b_output;
+
+			for (b_script_node.outputs.begin(b_output); b_output != b_script_node.outputs.end(); ++b_output) {
+				script_node->output_names.push_back(ustring(b_output->name()));
+				script_node->add_output(script_node->output_names.back().c_str(), convert_socket_type(b_output->type()));
+			}
+
+			/* load bytecode or filepath */
+			OSLShaderManager *manager = (OSLShaderManager*)scene->shader_manager;
+			string bytecode_hash = b_script_node.bytecode_hash();
+
+			if(!bytecode_hash.empty()) {
+				/* loaded bytecode if not already done */
+				if(!manager->shader_test_loaded(bytecode_hash))
+					manager->shader_load_bytecode(bytecode_hash, b_script_node.bytecode());
+
+				script_node->bytecode_hash = bytecode_hash;
+			}
+			else {
+				/* set filepath */
+				script_node->filepath = blender_absolute_path(b_data, b_ntree, b_script_node.filepath());
+			}
+			
+			node = script_node;
+#endif
+
+			break;
+		}
 		case BL::ShaderNode::type_TEX_IMAGE: {
 			BL::ShaderNodeTexImage b_image_node(b_node);
 			BL::Image b_image(b_image_node.image());
 			ImageTextureNode *image = new ImageTextureNode();
-			/* todo: handle generated/builtin images */
-			if(b_image)
-				image->filename = blender_absolute_path(b_data, b_image, b_image.filepath());
+			if(b_image) {
+				/* builtin images will use callback-based reading because
+				 * they could only be loaded correct from blender side
+				 */
+				bool is_builtin = b_image.packed_file() ||
+				                  b_image.source() == BL::Image::source_GENERATED ||
+				                  b_image.source() == BL::Image::source_MOVIE;
+
+				if(is_builtin) {
+					/* for builtin images we're using image datablock name to find an image to
+					 * read pixels from later
+					 *
+					 * also store frame number as well, so there's no differences in handling
+					 * builtin names for packed images and movies
+					 */
+					int scene_frame = b_scene.frame_current();
+					int image_frame = image_user_frame_number(b_image_node.image_user(), scene_frame);
+					image->filename = b_image.name() + "@" + string_printf("%d", image_frame);
+					image->builtin_data = b_image.ptr.data;
+				}
+				else {
+					image->filename = image_user_file_path(b_image_node.image_user(), b_image, b_scene.frame_current());
+					image->builtin_data = NULL;
+				}
+
+				image->animated = b_image_node.image_user().use_auto_refresh();
+			}
 			image->color_space = ImageTextureNode::color_space_enum[(int)b_image_node.color_space()];
+			image->projection = ImageTextureNode::projection_enum[(int)b_image_node.projection()];
+			image->projection_blend = b_image_node.projection_blend();
 			get_tex_mapping(&image->tex_mapping, b_image_node.texture_mapping());
 			node = image;
 			break;
@@ -339,8 +549,23 @@ static ShaderNode *add_node(BL::BlendData b_data, ShaderGraph *graph, BL::Shader
 			BL::ShaderNodeTexEnvironment b_env_node(b_node);
 			BL::Image b_image(b_env_node.image());
 			EnvironmentTextureNode *env = new EnvironmentTextureNode();
-			if(b_image)
-				env->filename = blender_absolute_path(b_data, b_image, b_image.filepath());
+			if(b_image) {
+				bool is_builtin = b_image.packed_file() ||
+				                  b_image.source() == BL::Image::source_GENERATED ||
+				                  b_image.source() == BL::Image::source_MOVIE;
+
+				if(is_builtin) {
+					int scene_frame = b_scene.frame_current();
+					int image_frame = image_user_frame_number(b_env_node.image_user(), scene_frame);
+					env->filename = b_image.name() + "@" + string_printf("%d", image_frame);
+					env->builtin_data = b_image.ptr.data;
+				}
+				else {
+					env->filename = image_user_file_path(b_env_node.image_user(), b_image, b_scene.frame_current());
+					env->animated = b_env_node.image_user().use_auto_refresh();
+					env->builtin_data = NULL;
+				}
+			}
 			env->color_space = EnvironmentTextureNode::color_space_enum[(int)b_env_node.color_space()];
 			env->projection = EnvironmentTextureNode::projection_enum[(int)b_env_node.projection()];
 			get_tex_mapping(&env->tex_mapping, b_env_node.texture_mapping());
@@ -386,6 +611,17 @@ static ShaderNode *add_node(BL::BlendData b_data, ShaderGraph *graph, BL::Shader
 			node = checker;
 			break;
 		}
+		case BL::ShaderNode::type_TEX_BRICK: {
+			BL::ShaderNodeTexBrick b_brick_node(b_node);
+			BrickTextureNode *brick = new BrickTextureNode();
+			brick->offset = b_brick_node.offset();
+			brick->offset_frequency = b_brick_node.offset_frequency();
+			brick->squash = b_brick_node.squash();
+			brick->squash_frequency = b_brick_node.squash_frequency();
+			get_tex_mapping(&brick->tex_mapping, b_brick_node.texture_mapping());
+			node = brick;
+			break;
+		}
 		case BL::ShaderNode::type_TEX_NOISE: {
 			BL::ShaderNodeTexNoise b_noise_node(b_node);
 			NoiseTextureNode *noise = new NoiseTextureNode();
@@ -402,7 +638,10 @@ static ShaderNode *add_node(BL::BlendData b_data, ShaderGraph *graph, BL::Shader
 			break;
 		}
 		case BL::ShaderNode::type_TEX_COORD: {
-			node = new TextureCoordinateNode();;
+			BL::ShaderNodeTexCoord b_tex_coord_node(b_node);
+			TextureCoordinateNode *tex_coord = new TextureCoordinateNode();
+			tex_coord->from_dupli = b_tex_coord_node.from_dupli();
+			node = tex_coord;
 			break;
 		}
 		case BL::ShaderNode::type_TEX_SKY: {
@@ -414,6 +653,23 @@ static ShaderNode *add_node(BL::BlendData b_data, ShaderGraph *graph, BL::Shader
 			node = sky;
 			break;
 		}
+		case BL::ShaderNode::type_NORMAL_MAP: {
+			BL::ShaderNodeNormalMap b_normal_map_node(b_node);
+			NormalMapNode *nmap = new NormalMapNode();
+			nmap->space = NormalMapNode::space_enum[(int)b_normal_map_node.space()];
+			nmap->attribute = b_normal_map_node.uv_map();
+			node = nmap;
+			break;
+		}
+		case BL::ShaderNode::type_TANGENT: {
+			BL::ShaderNodeTangent b_tangent_node(b_node);
+			TangentNode *tangent = new TangentNode();
+			tangent->direction_type = TangentNode::direction_type_enum[(int)b_tangent_node.direction_type()];
+			tangent->axis = TangentNode::axis_enum[(int)b_tangent_node.axis()];
+			tangent->attribute = b_tangent_node.uv_map();
+			node = tangent;
+			break;
+		}
 	}
 
 	if(node && node != graph->output())
@@ -470,53 +726,7 @@ static SocketPair node_socket_map_pair(PtrNodeMap& node_map, BL::Node b_node, BL
 	return SocketPair(node_map[b_node.ptr.data], name);
 }
 
-static ShaderSocketType convert_socket_type(BL::NodeSocket::type_enum b_type)
-{
-	switch (b_type) {
-	case BL::NodeSocket::type_VALUE:
-		return SHADER_SOCKET_FLOAT;
-	case BL::NodeSocket::type_VECTOR:
-		return SHADER_SOCKET_VECTOR;
-	case BL::NodeSocket::type_RGBA:
-		return SHADER_SOCKET_COLOR;
-	case BL::NodeSocket::type_SHADER:
-		return SHADER_SOCKET_CLOSURE;
-	
-	case BL::NodeSocket::type_BOOLEAN:
-	case BL::NodeSocket::type_MESH:
-	case BL::NodeSocket::type_INT:
-	default:
-		return SHADER_SOCKET_FLOAT;
-	}
-}
-
-static void set_default_value(ShaderInput *input, BL::NodeSocket sock)
-{
-	/* copy values for non linked inputs */
-	switch(input->type) {
-	case SHADER_SOCKET_FLOAT: {
-		BL::NodeSocketFloatNone value_sock(sock);
-		input->set(value_sock.default_value());
-		break;
-	}
-	case SHADER_SOCKET_COLOR: {
-		BL::NodeSocketRGBA rgba_sock(sock);
-		input->set(get_float3(rgba_sock.default_value()));
-		break;
-	}
-	case SHADER_SOCKET_NORMAL:
-	case SHADER_SOCKET_POINT:
-	case SHADER_SOCKET_VECTOR: {
-		BL::NodeSocketVectorNone vec_sock(sock);
-		input->set(get_float3(vec_sock.default_value()));
-		break;
-	}
-	case SHADER_SOCKET_CLOSURE:
-		break;
-	}
-}
-
-static void add_nodes(BL::BlendData b_data, ShaderGraph *graph, BL::ShaderNodeTree b_ntree, PtrSockMap& sockets_map)
+static void add_nodes(Scene *scene, BL::BlendData b_data, BL::Scene b_scene, ShaderGraph *graph, BL::ShaderNodeTree b_ntree, PtrSockMap& sockets_map)
 {
 	/* add nodes */
 	BL::ShaderNodeTree::nodes_iterator b_node;
@@ -524,7 +734,34 @@ static void add_nodes(BL::BlendData b_data, ShaderGraph *graph, BL::ShaderNodeTr
 	PtrSockMap proxy_map;
 
 	for(b_ntree.nodes.begin(b_node); b_node != b_ntree.nodes.end(); ++b_node) {
-		if(b_node->is_a(&RNA_NodeGroup)) {
+		if(b_node->mute()) {
+			BL::Node::inputs_iterator b_input;
+			BL::Node::outputs_iterator b_output;
+			bool found_match = false;
+
+			/* this is slightly different than blender logic, we just connect a
+			 * single pair for of input/output, but works ok for the node we have */
+			for(b_node->inputs.begin(b_input); b_input != b_node->inputs.end(); ++b_input) {
+				if(b_input->is_linked()) {
+					for(b_node->outputs.begin(b_output); b_output != b_node->outputs.end(); ++b_output) {
+						if(b_output->is_linked() && b_input->type() == b_output->type()) {
+							ProxyNode *proxy = new ProxyNode(convert_socket_type(b_input->type()), convert_socket_type(b_output->type()));
+							graph->add(proxy);
+
+							proxy_map[b_input->ptr.data] = SocketPair(proxy, proxy->inputs[0]->name);
+							proxy_map[b_output->ptr.data] = SocketPair(proxy, proxy->outputs[0]->name);
+							found_match = true;
+
+							break;
+						}
+					}
+				}
+
+				if(found_match)
+					break;
+			}
+		}
+		else if(b_node->is_a(&RNA_NodeGroup)) {
 			/* add proxy converter nodes for inputs and outputs */
 			BL::NodeGroup b_gnode(*b_node);
 			BL::ShaderNodeTree b_group_ntree(b_gnode.node_tree());
@@ -547,7 +784,7 @@ static void add_nodes(BL::BlendData b_data, ShaderGraph *graph, BL::ShaderNodeTr
 				group_sockmap[b_input->group_socket().ptr.data] = SocketPair(proxy, proxy->outputs[0]->name);
 				
 				/* default input values of the group node */
-				set_default_value(proxy->inputs[0], *b_input);
+				set_default_value(proxy->inputs[0], *b_input, b_data, b_group_ntree);
 			}
 			
 			for(b_node->outputs.begin(b_output); b_output != b_node->outputs.end(); ++b_output) {
@@ -561,13 +798,13 @@ static void add_nodes(BL::BlendData b_data, ShaderGraph *graph, BL::ShaderNodeTr
 				group_sockmap[b_output->group_socket().ptr.data] = SocketPair(proxy, proxy->inputs[0]->name);
 				
 				/* default input values of internal, unlinked group outputs */
-				set_default_value(proxy->inputs[0], b_output->group_socket());
+				set_default_value(proxy->inputs[0], b_output->group_socket(), b_data, b_group_ntree);
 			}
 			
-			add_nodes(b_data, graph, b_group_ntree, group_sockmap);
+			add_nodes(scene, b_data, b_scene, graph, b_group_ntree, group_sockmap);
 		}
 		else {
-			ShaderNode *node = add_node(b_data, graph, BL::ShaderNode(*b_node));
+			ShaderNode *node = add_node(scene, b_data, b_scene, graph, b_ntree, BL::ShaderNode(*b_node));
 			
 			if(node) {
 				BL::Node::inputs_iterator b_input;
@@ -581,7 +818,7 @@ static void add_nodes(BL::BlendData b_data, ShaderGraph *graph, BL::ShaderNodeTr
 					assert(input);
 					
 					/* copy values for non linked inputs */
-					set_default_value(input, *b_input);
+					set_default_value(input, *b_input, b_data, b_ntree);
 				}
 			}
 		}
@@ -604,7 +841,7 @@ static void add_nodes(BL::BlendData b_data, ShaderGraph *graph, BL::ShaderNodeTr
 
 		/* from sock */
 		if(b_from_node) {
-			if (b_from_node.is_a(&RNA_NodeGroup))
+			if (b_from_node.mute() || b_from_node.is_a(&RNA_NodeGroup))
 				from_pair = proxy_map[b_from_sock.ptr.data];
 			else
 				from_pair = node_socket_map_pair(node_map, b_from_node, b_from_sock);
@@ -614,7 +851,7 @@ static void add_nodes(BL::BlendData b_data, ShaderGraph *graph, BL::ShaderNodeTr
 
 		/* to sock */
 		if(b_to_node) {
-			if (b_to_node.is_a(&RNA_NodeGroup))
+			if (b_to_node.mute() || b_to_node.is_a(&RNA_NodeGroup))
 				to_pair = proxy_map[b_to_sock.ptr.data];
 			else
 				to_pair = node_socket_map_pair(node_map, b_to_node, b_to_sock);
@@ -623,7 +860,7 @@ static void add_nodes(BL::BlendData b_data, ShaderGraph *graph, BL::ShaderNodeTr
 			to_pair = sockets_map[b_to_sock.ptr.data];
 
 		/* either node may be NULL when the node was not exported, typically
-		   because the node type is not supported */
+		 * because the node type is not supported */
 		if(from_pair.first && to_pair.first) {
 			ShaderOutput *output = from_pair.first->output(from_pair.second.c_str());
 			ShaderInput *input = to_pair.first->input(to_pair.second.c_str());
@@ -635,7 +872,7 @@ static void add_nodes(BL::BlendData b_data, ShaderGraph *graph, BL::ShaderNodeTr
 
 /* Sync Materials */
 
-void BlenderSync::sync_materials()
+void BlenderSync::sync_materials(bool update_all)
 {
 	shader_map.set_default(scene->shaders[scene->default_surface]);
 
@@ -646,7 +883,7 @@ void BlenderSync::sync_materials()
 		Shader *shader;
 		
 		/* test if we need to sync */
-		if(shader_map.sync(&shader, *b_mat)) {
+		if(shader_map.sync(&shader, *b_mat) || update_all) {
 			ShaderGraph *graph = new ShaderGraph();
 
 			shader->name = b_mat->name().c_str();
@@ -657,7 +894,7 @@ void BlenderSync::sync_materials()
 				PtrSockMap sock_to_node;
 				BL::ShaderNodeTree b_ntree(b_mat->node_tree());
 
-				add_nodes(b_data, graph, b_ntree, sock_to_node);
+				add_nodes(scene, b_data, b_scene, graph, b_ntree, sock_to_node);
 			}
 			else {
 				ShaderNode *closure, *out;
@@ -682,14 +919,14 @@ void BlenderSync::sync_materials()
 
 /* Sync World */
 
-void BlenderSync::sync_world()
+void BlenderSync::sync_world(bool update_all)
 {
 	Background *background = scene->background;
 	Background prevbackground = *background;
 
 	BL::World b_world = b_scene.world();
 
-	if(world_recalc || b_world.ptr.data != world_map) {
+	if(world_recalc || update_all || b_world.ptr.data != world_map) {
 		Shader *shader = scene->shaders[scene->default_background];
 		ShaderGraph *graph = new ShaderGraph();
 
@@ -698,7 +935,7 @@ void BlenderSync::sync_world()
 			PtrSockMap sock_to_node;
 			BL::ShaderNodeTree b_ntree(b_world.node_tree());
 
-			add_nodes(b_data, graph, b_ntree, sock_to_node);
+			add_nodes(scene, b_data, b_scene, graph, b_ntree, sock_to_node);
 		}
 		else if(b_world) {
 			ShaderNode *closure, *out;
@@ -736,7 +973,7 @@ void BlenderSync::sync_world()
 
 /* Sync Lamps */
 
-void BlenderSync::sync_lamps()
+void BlenderSync::sync_lamps(bool update_all)
 {
 	shader_map.set_default(scene->shaders[scene->default_light]);
 
@@ -747,7 +984,7 @@ void BlenderSync::sync_lamps()
 		Shader *shader;
 		
 		/* test if we need to sync */
-		if(shader_map.sync(&shader, *b_lamp)) {
+		if(shader_map.sync(&shader, *b_lamp) || update_all) {
 			ShaderGraph *graph = new ShaderGraph();
 
 			/* create nodes */
@@ -757,7 +994,7 @@ void BlenderSync::sync_lamps()
 				PtrSockMap sock_to_node;
 				BL::ShaderNodeTree b_ntree(b_lamp->node_tree());
 
-				add_nodes(b_data, graph, b_ntree, sock_to_node);
+				add_nodes(scene, b_data, b_scene, graph, b_ntree, sock_to_node);
 			}
 			else {
 				ShaderNode *closure, *out;
@@ -766,7 +1003,9 @@ void BlenderSync::sync_lamps()
 				if(b_lamp->type() == BL::Lamp::type_POINT ||
 				   b_lamp->type() == BL::Lamp::type_SPOT ||
 				   b_lamp->type() == BL::Lamp::type_AREA)
+				{
 					strength = 100.0f;
+				}
 
 				closure = graph->add(new EmissionNode());
 				closure->input("Color")->value = get_float3(b_lamp->color());
@@ -784,11 +1023,20 @@ void BlenderSync::sync_lamps()
 
 void BlenderSync::sync_shaders()
 {
+	/* for auto refresh images */
+	bool auto_refresh_update = false;
+
+	if(preview) {
+		ImageManager *image_manager = scene->image_manager;
+		int frame = b_scene.frame_current();
+		auto_refresh_update = image_manager->set_animation_frame_update(frame);
+	}
+
 	shader_map.pre_sync();
 
-	sync_world();
-	sync_lamps();
-	sync_materials();
+	sync_world(auto_refresh_update);
+	sync_lamps(auto_refresh_update);
+	sync_materials(auto_refresh_update);
 
 	/* false = don't delete unused shaders, not supported */
 	shader_map.post_sync(false);
