@@ -22,8 +22,10 @@
 #include <stdlib.h>
 
 #include "device_memory.h"
+#include "device_task.h"
 
 #include "util_list.h"
+#include "util_stats.h"
 #include "util_string.h"
 #include "util_thread.h"
 #include "util_types.h"
@@ -32,6 +34,7 @@
 CCL_NAMESPACE_BEGIN
 
 class Progress;
+class RenderTile;
 
 /* Device Types */
 
@@ -52,6 +55,7 @@ public:
 	int num;
 	bool display_device;
 	bool advanced_shading;
+	bool pack_images;
 	vector<DeviceInfo> multi_devices;
 
 	DeviceInfo()
@@ -61,41 +65,15 @@ public:
 		num = 0;
 		display_device = false;
 		advanced_shading = true;
+		pack_images = false;
 	}
 };
 
-/* Device Task */
-
-class DeviceTask {
-public:
-	typedef enum { PATH_TRACE, TONEMAP, SHADER } Type;
-	Type type;
-
-	int x, y, w, h;
-	device_ptr rng_state;
-	device_ptr rgba;
-	device_ptr buffer;
-	int sample;
-	int resolution;
-	int offset, stride;
-
-	device_ptr shader_input;
-	device_ptr shader_output;
-	int shader_eval_type;
-	int shader_x, shader_w;
-
-	DeviceTask(Type type = PATH_TRACE);
-
-	void split(list<DeviceTask>& tasks, int num);
-	void split(ThreadQueue<DeviceTask>& tasks, int num);
-	void split_max_size(list<DeviceTask>& tasks, int max_size);
-};
-
 /* Device */
 
 class Device {
 protected:
-	Device() {}
+	Device(Stats &stats_) : stats(stats_) {}
 
 	bool background;
 	string error_msg;
@@ -106,6 +84,10 @@ public:
 	/* info */
 	DeviceInfo info;
 	virtual const string& error_message() { return error_msg; }
+	bool have_error() { return !error_message().empty(); }
+
+	/* statistics */
+	Stats &stats;
 
 	/* regular memory */
 	virtual void mem_alloc(device_memory& mem, MemoryType type) = 0;
@@ -148,8 +130,12 @@ public:
 	void server_run();
 #endif
 
+	/* multi device */
+	virtual void map_tile(Device *sub_device, RenderTile& tile) {}
+	virtual int device_number(Device *sub_device) { return 0; }
+
 	/* static */
-	static Device *create(DeviceInfo& info, bool background = true, int threads = 0);
+	static Device *create(DeviceInfo& info, Stats &stats, bool background = true);
 
 	static DeviceType type_from_string(const char *name);
 	static string string_from_type(DeviceType type);
