@@ -29,6 +29,11 @@ class Scene;
 
 class Background {
 public:
+	float ao_factor;
+	float ao_distance;
+
+	bool use;
+
 	bool transparent;
 	bool need_update;
 
