@@ -21,6 +21,7 @@
 
 #include "kernel_types.h"
 
+#include "util_boundbox.h"
 #include "util_transform.h"
 #include "util_types.h"
 
@@ -28,6 +29,7 @@ CCL_NAMESPACE_BEGIN
 
 class Device;
 class DeviceScene;
+class Scene;
 
 /* Camera
  *
@@ -37,8 +39,7 @@ class DeviceScene;
 class Camera {
 public:
 	/* motion blur */
-	float shutteropen;
-	float shutterclose;
+	float shuttertime;
 
 	/* depth of field */
 	float focaldistance;
@@ -50,24 +51,46 @@ public:
 	CameraType type;
 	float fov;
 
+	/* panorama */
+	PanoramaType panorama_type;
+	float fisheye_fov;
+	float fisheye_lens;
+
+	/* sensor */
+	float sensorwidth;
+	float sensorheight;
+
 	/* clipping */
 	float nearclip;
 	float farclip;
 
 	/* screen */
 	int width, height;
-	float left, right, bottom, top;
+	BoundBox2D viewplane;
+
+	/* border */
+	BoundBox2D border;
 
 	/* transformation */
 	Transform matrix;
 
+	/* motion */
+	MotionTransform motion;
+	bool use_motion;
+
 	/* computed camera parameters */
-    Transform screentoworld;
+	Transform screentoworld;
 	Transform rastertoworld;
 	Transform ndctoworld;
-	Transform rastertocamera;
 	Transform cameratoworld;
+
 	Transform worldtoraster;
+	Transform worldtoscreen;
+	Transform worldtondc;
+	Transform worldtocamera;
+
+	Transform rastertocamera;
+	Transform cameratoraster;
 
 	float3 dx;
 	float3 dy;
@@ -75,6 +98,7 @@ public:
 	/* update */
 	bool need_update;
 	bool need_device_update;
+	int previous_need_motion;
 
 	/* functions */
 	Camera();
@@ -82,10 +106,11 @@ public:
 
 	void update();
 
-	void device_update(Device *device, DeviceScene *dscene);
+	void device_update(Device *device, DeviceScene *dscene, Scene *scene);
 	void device_free(Device *device, DeviceScene *dscene);
 
 	bool modified(const Camera& cam);
+	bool motion_modified(const Camera& cam);
 	void tag_update();
 };
 
