@@ -22,14 +22,25 @@
 #include "device_memory.h"
 
 #include "util_string.h"
+#include "util_thread.h"
 #include "util_vector.h"
 
+#include "kernel_types.h"  /* for TEX_NUM_FLOAT_IMAGES */
+
 CCL_NAMESPACE_BEGIN
 
-#define TEX_NUM_FLOAT_IMAGES	5
 #define TEX_NUM_IMAGES			95
-#define TEX_IMAGE_MAX			(TEX_NUM_IMAGES + TEX_NUM_FLOAT_IMAGES)
-#define TEX_IMAGE_FLOAT_START	TEX_NUM_IMAGES
+#define TEX_IMAGE_BYTE_START	TEX_NUM_FLOAT_IMAGES
+
+#define TEX_EXTENDED_NUM_FLOAT_IMAGES	5
+#define TEX_EXTENDED_NUM_IMAGES			512
+#define TEX_EXTENDED_IMAGE_BYTE_START	TEX_EXTENDED_NUM_FLOAT_IMAGES
+
+/* color to use when textures are not found */
+#define TEX_IMAGE_MISSING_R 1
+#define TEX_IMAGE_MISSING_G 0
+#define TEX_IMAGE_MISSING_B 1
+#define TEX_IMAGE_MISSING_A 1
 
 class Device;
 class DeviceScene;
@@ -40,33 +51,51 @@ public:
 	ImageManager();
 	~ImageManager();
 
-	int add_image(const string& filename, bool& is_float);
-	void remove_image(const string& filename);
+	int add_image(const string& filename, void *builtin_data, bool animated, bool& is_float, bool& is_linear);
+	void remove_image(const string& filename, void *builtin_data);
+	bool is_float_image(const string& filename, void *builtin_data, bool& is_linear);
 
 	void device_update(Device *device, DeviceScene *dscene, Progress& progress);
 	void device_free(Device *device, DeviceScene *dscene);
 
 	void set_osl_texture_system(void *texture_system);
+	void set_pack_images(bool pack_images_);
+	void set_extended_image_limits(void);
+	bool set_animation_frame_update(int frame);
 
 	bool need_update;
 
+	boost::function<void(const string &filename, void *data, bool &is_float, int &width, int &height, int &channels)> builtin_image_info_cb;
+	boost::function<bool(const string &filename, void *data, unsigned char *pixels)> builtin_image_pixels_cb;
+	boost::function<bool(const string &filename, void *data, float *pixels)> builtin_image_float_pixels_cb;
 private:
+	int tex_num_images;
+	int tex_num_float_images;
+	int tex_image_byte_start;
+	thread_mutex device_mutex;
+	int animation_frame;
+
 	struct Image {
 		string filename;
+		void *builtin_data;
 
 		bool need_load;
+		bool animated;
 		int users;
 	};
 
 	vector<Image*> images;
 	vector<Image*> float_images;
 	void *osl_texture_system;
+	bool pack_images;
 
 	bool file_load_image(Image *img, device_vector<uchar4>& tex_img);
 	bool file_load_float_image(Image *img, device_vector<float4>& tex_img);
 
-	void device_load_image(Device *device, DeviceScene *dscene, int slot);
+	void device_load_image(Device *device, DeviceScene *dscene, int slot, Progress *progess);
 	void device_free_image(Device *device, DeviceScene *dscene, int slot);
+
+	void device_pack_images(Device *device, DeviceScene *dscene, Progress& progess);
 };
 
 CCL_NAMESPACE_END
