@@ -48,9 +48,14 @@ public:
 
 	int map_resolution;
 
+	float spot_angle;
+	float spot_smooth;
+
 	bool cast_shadow;
+	bool use_mis;
 
 	int shader;
+	int samples;
 
 	void tag_update(Scene *scene);
 };
