@@ -41,16 +41,26 @@ public:
 	Transform tfm;
 	BoundBox bounds;
 	ustring name;
+	uint random_id;
 	int pass_id;
 	vector<ParamValue> attributes;
 	uint visibility;
+	MotionTransform motion;
+	bool use_motion;
+	bool use_holdout;
+	bool curverender;
+
+	float3 dupli_generated;
+	float2 dupli_uv;
+
+	int particle_id;
 
 	Object();
 	~Object();
 
 	void tag_update(Scene *scene);
 
-	void compute_bounds();
+	void compute_bounds(bool motion_blur, float shuttertime);
 	void apply_transform();
 };
 
@@ -64,12 +74,12 @@ public:
 	~ObjectManager();
 
 	void device_update(Device *device, DeviceScene *dscene, Scene *scene, Progress& progress);
-	void device_update_transforms(Device *device, DeviceScene *dscene, Scene *scene, Progress& progress);
+	void device_update_transforms(Device *device, DeviceScene *dscene, Scene *scene, uint *object_flag, Progress& progress);
 	void device_free(Device *device, DeviceScene *dscene);
 
 	void tag_update(Scene *scene);
 
-	void apply_static_transforms(Scene *scene, Progress& progress);
+	void apply_static_transforms(Scene *scene, uint *object_flag, Progress& progress);
 };
 
 CCL_NAMESPACE_END
