@@ -28,6 +28,8 @@
 #include "shader.h"
 #include "mesh.h"
 #include "object.h"
+#include "particles.h"
+#include "curves.h"
 #include "scene.h"
 #include "svm.h"
 #include "osl.h"
@@ -37,7 +39,7 @@
 
 CCL_NAMESPACE_BEGIN
 
-Scene::Scene(const SceneParams& params_)
+Scene::Scene(const SceneParams& params_, const DeviceInfo& device_info_)
 : params(params_)
 {
 	device = NULL;
@@ -52,38 +54,26 @@ Scene::Scene(const SceneParams& params_)
 	object_manager = new ObjectManager();
 	integrator = new Integrator();
 	image_manager = new ImageManager();
-	shader_manager = ShaderManager::create(this);
+	particle_system_manager = new ParticleSystemManager();
+	curve_system_manager = new CurveSystemManager();
+
+	/* OSL only works on the CPU */
+	if(device_info_.type == DEVICE_CPU)
+		shader_manager = ShaderManager::create(this, params.shadingsystem);
+	else
+		shader_manager = ShaderManager::create(this, SceneParams::SVM);
+
+	if (device_info_.type == DEVICE_CPU)
+		image_manager->set_extended_image_limits();
 }
 
 Scene::~Scene()
 {
-	if(device) camera->device_free(device, &dscene);
-	delete camera;
-
-	if(device) filter->device_free(device, &dscene);
-	delete filter;
-
-	if(device) film->device_free(device, &dscene);
-	delete film;
-
-	if(device) background->device_free(device, &dscene);
-	delete background;
-
-	if(device) mesh_manager->device_free(device, &dscene);
-	delete mesh_manager;
-
-	if(device) object_manager->device_free(device, &dscene);
-	delete object_manager;
-
-	if(device) integrator->device_free(device, &dscene);
-	delete integrator;
-
-	if(device) shader_manager->device_free(device, &dscene);
-	delete shader_manager;
-
-	if(device) light_manager->device_free(device, &dscene);
-	delete light_manager;
+	free_memory(true);
+}
 
+void Scene::free_memory(bool final)
+{
 	foreach(Shader *s, shaders)
 		delete s;
 	foreach(Mesh *m, meshes)
@@ -92,9 +82,48 @@ Scene::~Scene()
 		delete o;
 	foreach(Light *l, lights)
 		delete l;
+	foreach(ParticleSystem *p, particle_systems)
+		delete p;
 
-	if(device) image_manager->device_free(device, &dscene);
-	delete image_manager;
+	shaders.clear();
+	meshes.clear();
+	objects.clear();
+	lights.clear();
+	particle_systems.clear();
+
+	if(device) {
+		camera->device_free(device, &dscene);
+		filter->device_free(device, &dscene);
+		film->device_free(device, &dscene);
+		background->device_free(device, &dscene);
+		integrator->device_free(device, &dscene);
+
+		object_manager->device_free(device, &dscene);
+		mesh_manager->device_free(device, &dscene);
+		shader_manager->device_free(device, &dscene);
+		light_manager->device_free(device, &dscene);
+
+		particle_system_manager->device_free(device, &dscene);
+		curve_system_manager->device_free(device, &dscene);
+
+		if(!params.persistent_data || final)
+			image_manager->device_free(device, &dscene);
+	}
+
+	if(final) {
+		delete filter;
+		delete camera;
+		delete film;
+		delete background;
+		delete integrator;
+		delete object_manager;
+		delete mesh_manager;
+		delete shader_manager;
+		delete light_manager;
+		delete particle_system_manager;
+		delete curve_system_manager;
+		delete image_manager;
+	}
 }
 
 void Scene::device_update(Device *device_, Progress& progress)
@@ -111,6 +140,8 @@ void Scene::device_update(Device *device_, Progress& progress)
 	 * - Displacement shader must have all shader data available.
 	 * - Light manager needs final mesh data to compute emission CDF.
 	 */
+	
+	image_manager->set_pack_images(device->info.pack_images);
 
 	progress.set_status("Updating Background");
 	background->device_update(device, &dscene, this);
@@ -128,7 +159,7 @@ void Scene::device_update(Device *device_, Progress& progress)
 	if(progress.get_cancel()) return;
 
 	progress.set_status("Updating Camera");
-	camera->device_update(device, &dscene);
+	camera->device_update(device, &dscene, this);
 
 	if(progress.get_cancel()) return;
 
@@ -137,6 +168,11 @@ void Scene::device_update(Device *device_, Progress& progress)
 
 	if(progress.get_cancel()) return;
 
+	progress.set_status("Updating Hair Systems");
+	curve_system_manager->device_update(device, &dscene, this, progress);
+
+	if(progress.get_cancel()) return;
+
 	progress.set_status("Updating Meshes");
 	mesh_manager->device_update(device, &dscene, this, progress);
 
@@ -147,6 +183,11 @@ void Scene::device_update(Device *device_, Progress& progress)
 
 	if(progress.get_cancel()) return;
 
+	progress.set_status("Updating Particle Systems");
+	particle_system_manager->device_update(device, &dscene, this, progress);
+
+	if(progress.get_cancel()) return;
+
 	progress.set_status("Updating Filter");
 	filter->device_update(device, &dscene);
 
@@ -158,7 +199,7 @@ void Scene::device_update(Device *device_, Progress& progress)
 	if(progress.get_cancel()) return;
 
 	progress.set_status("Updating Integrator");
-	integrator->device_update(device, &dscene);
+	integrator->device_update(device, &dscene, this);
 
 	if(progress.get_cancel()) return;
 
@@ -166,6 +207,33 @@ void Scene::device_update(Device *device_, Progress& progress)
 	device->const_copy_to("__data", &dscene.data, sizeof(dscene.data));
 }
 
+Scene::MotionType Scene::need_motion(bool advanced_shading)
+{
+	if(integrator->motion_blur)
+		return (advanced_shading)? MOTION_BLUR: MOTION_NONE;
+	else if(Pass::contains(film->passes, PASS_MOTION))
+		return MOTION_PASS;
+	else
+		return MOTION_NONE;
+}
+
+bool Scene::need_global_attribute(AttributeStandard std)
+{
+	if(std == ATTR_STD_UV)
+		return Pass::contains(film->passes, PASS_UV);
+	if(std == ATTR_STD_MOTION_PRE || ATTR_STD_MOTION_POST)
+		return need_motion() == MOTION_PASS;
+	
+	return false;
+}
+
+void Scene::need_global_attributes(AttributeRequestSet& attributes)
+{
+	for(int std = ATTR_STD_NONE; std < ATTR_STD_NUM; std++)
+		if(need_global_attribute((AttributeStandard)std))
+			attributes.add((AttributeStandard)std);
+}
+
 bool Scene::need_update()
 {
 	return (need_reset() || film->need_update);
@@ -181,7 +249,27 @@ bool Scene::need_reset()
 		|| light_manager->need_update
 		|| filter->need_update
 		|| integrator->need_update
-		|| shader_manager->need_update);
+		|| shader_manager->need_update
+		|| particle_system_manager->need_update
+		|| curve_system_manager->need_update);
+}
+
+void Scene::reset()
+{
+	shader_manager->reset(this);
+	shader_manager->add_default(this);
+
+	/* ensure all objects are updated */
+	camera->tag_update();
+	filter->tag_update(this);
+	film->tag_update(this);
+	background->tag_update(this);
+	integrator->tag_update(this);
+}
+
+void Scene::device_free()
+{
+	free_memory(false);
 }
 
 CCL_NAMESPACE_END
