@@ -40,6 +40,10 @@ SVMShaderManager::~SVMShaderManager()
 {
 }
 
+void SVMShaderManager::reset(Scene *scene)
+{
+}
+
 void SVMShaderManager::device_update(Device *device, DeviceScene *dscene, Scene *scene, Progress& progress)
 {
 	if(!need_update)
@@ -48,6 +52,9 @@ void SVMShaderManager::device_update(Device *device, DeviceScene *dscene, Scene
 	/* test if we need to update */
 	device_free(device, dscene);
 
+	/* determine which shaders are in use */
+	device_update_shaders_used(scene);
+
 	/* svm_nodes */
 	vector<int4> svm_nodes;
 	size_t i;
@@ -119,6 +126,8 @@ int SVMCompiler::stack_size(ShaderSocketType type)
 {
 	if(type == SHADER_SOCKET_FLOAT)
 		return 1;
+	else if(type == SHADER_SOCKET_INT)
+		return 1;
 	else if(type == SHADER_SOCKET_COLOR)
 		return 3;
 	else if(type == SHADER_SOCKET_VECTOR)
@@ -212,10 +221,13 @@ void SVMCompiler::stack_assign(ShaderInput *input)
 			if(input->type == SHADER_SOCKET_FLOAT) {
 				add_node(NODE_VALUE_F, __float_as_int(input->value.x), input->stack_offset);
 			}
+			else if(input->type == SHADER_SOCKET_INT) {
+				add_node(NODE_VALUE_F, (int)input->value.x, input->stack_offset);
+			}
 			else if(input->type == SHADER_SOCKET_VECTOR ||
-				input->type == SHADER_SOCKET_NORMAL ||
-				input->type == SHADER_SOCKET_POINT ||
-				input->type == SHADER_SOCKET_COLOR) {
+			        input->type == SHADER_SOCKET_NORMAL ||
+			        input->type == SHADER_SOCKET_POINT ||
+			        input->type == SHADER_SOCKET_COLOR) {
 
 				add_node(NODE_VALUE_V, input->stack_offset);
 				add_node(NODE_VALUE_V, input->value);
@@ -251,11 +263,11 @@ void SVMCompiler::stack_link(ShaderInput *input, ShaderOutput *output)
 void SVMCompiler::stack_clear_users(ShaderNode *node, set<ShaderNode*>& done)
 {
 	/* optimization we should add:
-	   find and lower user counts for outputs for which all inputs are done.
-	   this is done before the node is compiled, under the assumption that the
-	   node will first load all inputs from the stack and then writes its
-	   outputs. this used to work, but was disabled because it gave trouble
-	   with inputs getting stack positions assigned */
+	 * find and lower user counts for outputs for which all inputs are done.
+	 * this is done before the node is compiled, under the assumption that the
+	 * node will first load all inputs from the stack and then writes its
+	 * outputs. this used to work, but was disabled because it gave trouble
+	 * with inputs getting stack positions assigned */
 
 	foreach(ShaderInput *input, node->inputs) {
 		ShaderOutput *output = input->link;
@@ -337,7 +349,7 @@ uint SVMCompiler::attribute(ustring name)
 	return shader_manager->get_attribute_id(name);
 }
 
-uint SVMCompiler::attribute(Attribute::Standard std)
+uint SVMCompiler::attribute(AttributeStandard std)
 {
 	return shader_manager->get_attribute_id(std);
 }
@@ -418,8 +430,8 @@ void SVMCompiler::generate_closure(ShaderNode *node, set<ShaderNode*>& done)
 			add_node(NODE_ADD_CLOSURE, 0, 0, 0);
 
 		/* generate code for closure 1
-		   note we backup all compiler state and restore it afterwards, so one
-		   closure choice doesn't influence the other*/
+		 * note we backup all compiler state and restore it afterwards, so one
+		 * closure choice doesn't influence the other*/
 		if(cl1in->link) {
 			StackBackup backup;
 			stack_backup(backup, done);
@@ -448,7 +460,7 @@ void SVMCompiler::generate_closure(ShaderNode *node, set<ShaderNode*>& done)
 			add_node(NODE_END, 0, 0, 0);
 
 		/* set jump for mix node, -1 because offset is already
-		   incremented when this jump is added to it */
+		 * incremented when this jump is added to it */
 		svm_nodes[mix_offset].z = cl2_offset - mix_offset - 1;
 
 		done.insert(node);
@@ -470,65 +482,37 @@ void SVMCompiler::generate_closure(ShaderNode *node, set<ShaderNode*>& done)
 		stack_clear_users(node, done);
 		stack_clear_temporary(node);
 
-		if(node->name == ustring("emission"))
+		if(node->has_surface_emission())
 			current_shader->has_surface_emission = true;
-		if(node->name == ustring("transparent"))
+		if(node->has_surface_transparent())
 			current_shader->has_surface_transparent = true;
 
 		/* end node is added outside of this */
 	}
 }
 
-void SVMCompiler::generate_multi_closure(ShaderNode *node, set<ShaderNode*>& done, uint in_offset)
+void SVMCompiler::generate_multi_closure(ShaderNode *node, set<ShaderNode*>& done, set<ShaderNode*>& closure_done)
 {
 	/* todo: the weaks point here is that unlike the single closure sampling 
-	   we will evaluate all nodes even if they are used as input for closures
-	   that are unused. it's not clear what would be the best way to skip such
-	   nodes at runtime, especially if they are tangled up  */
+	 * we will evaluate all nodes even if they are used as input for closures
+	 * that are unused. it's not clear what would be the best way to skip such
+	 * nodes at runtime, especially if they are tangled up  */
+	
+	/* only generate once */
+	if(closure_done.find(node) != closure_done.end())
+		return;
+
+	closure_done.insert(node);
 
 	if(node->name == ustring("mix_closure") || node->name == ustring("add_closure")) {
-		ShaderInput *fin = node->input("Fac");
+		/* weighting is already taken care of in ShaderGraph::transform_multi_closure */
 		ShaderInput *cl1in = node->input("Closure1");
 		ShaderInput *cl2in = node->input("Closure2");
 
-		uint out1_offset = SVM_STACK_INVALID;
-		uint out2_offset = SVM_STACK_INVALID;
-
-		if(fin) {
-			/* mix closure */
-			set<ShaderNode*> dependencies;
-			find_dependencies(dependencies, done, fin);
-			generate_svm_nodes(dependencies, done);
-
-			stack_assign(fin);
-
-			if(cl1in->link)
-				out1_offset = stack_find_offset(SHADER_SOCKET_FLOAT);
-			if(cl2in->link)
-				out2_offset = stack_find_offset(SHADER_SOCKET_FLOAT);
-
-			add_node(NODE_MIX_CLOSURE, 
-				encode_uchar4(fin->stack_offset, in_offset, out1_offset, out2_offset));
-		}
-		else {
-			/* add closure */
-			out1_offset = in_offset;
-			out2_offset = in_offset;
-		}
-
-		if(cl1in->link) {
-			generate_multi_closure(cl1in->link->parent, done, out1_offset);
-
-			if(fin)
-				stack_clear_offset(SHADER_SOCKET_FLOAT, out1_offset);
-		}
-
-		if(cl2in->link) {
-			generate_multi_closure(cl2in->link->parent, done, out2_offset);
-
-			if(fin)
-				stack_clear_offset(SHADER_SOCKET_FLOAT, out2_offset);
-		}
+		if(cl1in->link)
+			generate_multi_closure(cl1in->link->parent, done, closure_done);
+		if(cl2in->link)
+			generate_multi_closure(cl2in->link->parent, done, closure_done);
 	}
 	else {
 		/* execute dependencies for closure */
@@ -540,7 +524,16 @@ void SVMCompiler::generate_multi_closure(ShaderNode *node, set<ShaderNode*>& don
 			}
 		}
 
-		mix_weight_offset = in_offset;
+		/* closure mix weight */
+		const char *weight_name = (current_type == SHADER_TYPE_VOLUME)? "VolumeMixWeight": "SurfaceMixWeight";
+		ShaderInput *weight_in = node->input(weight_name);
+
+		if(weight_in && (weight_in->link || weight_in->value.x != 1.0f)) {
+			stack_assign(weight_in);
+			mix_weight_offset = weight_in->stack_offset;
+		}
+		else
+			mix_weight_offset = SVM_STACK_INVALID;
 
 		/* compile closure itself */
 		node->compile(*this);
@@ -549,13 +542,13 @@ void SVMCompiler::generate_multi_closure(ShaderNode *node, set<ShaderNode*>& don
 
 		mix_weight_offset = SVM_STACK_INVALID;
 
-		if(node->name == ustring("emission"))
+		if(node->has_surface_emission())
 			current_shader->has_surface_emission = true;
-		if(node->name == ustring("transparent"))
+		if(node->has_surface_transparent())
 			current_shader->has_surface_transparent = true;
-
-		/* end node is added outside of this */
 	}
+
+	done.insert(node);
 }
 
 
@@ -597,44 +590,48 @@ void SVMCompiler::compile_type(Shader *shader, ShaderGraph *graph, ShaderType ty
 	memset(&active_stack, 0, sizeof(active_stack));
 	svm_nodes.clear();
 
-	foreach(ShaderNode *node, graph->nodes) {
-		foreach(ShaderInput *input, node->inputs)
+	foreach(ShaderNode *node_iter, graph->nodes) {
+		foreach(ShaderInput *input, node_iter->inputs)
 			input->stack_offset = SVM_STACK_INVALID;
-		foreach(ShaderOutput *output, node->outputs)
+		foreach(ShaderOutput *output, node_iter->outputs)
 			output->stack_offset = SVM_STACK_INVALID;
 	}
 
-	if(clin->link) {
-		bool generate = false;
-		if(type == SHADER_TYPE_SURFACE) {
-			/* generate surface shader */
-			generate = true;
-			shader->has_surface = true;
-		}
-		else if(type == SHADER_TYPE_VOLUME) {
-			/* generate volume shader */
-			generate = true;
-			shader->has_volume = true;
-		}
-		else if(type == SHADER_TYPE_DISPLACEMENT) {
-			/* generate displacement shader */
-			generate = true;
-			shader->has_displacement = true;
+	if(shader->used) {
+		if(clin->link) {
+			bool generate = false;
+			if(type == SHADER_TYPE_SURFACE) {
+				/* generate surface shader */
+				generate = true;
+				shader->has_surface = true;
+			}
+			else if(type == SHADER_TYPE_VOLUME) {
+				/* generate volume shader */
+				generate = true;
+				shader->has_volume = true;
+			}
+			else if(type == SHADER_TYPE_DISPLACEMENT) {
+				/* generate displacement shader */
+				generate = true;
+				shader->has_displacement = true;
+			}
+
+			if(generate) {
+				set<ShaderNode*> done;
+
+				if(use_multi_closure) {
+					set<ShaderNode*> closure_done;
+					generate_multi_closure(clin->link->parent, done, closure_done);
+				}
+				else
+					generate_closure(clin->link->parent, done);
+			}
 		}
 
-		if(generate) {
-			set<ShaderNode*> done;
-
-			if(use_multi_closure)
-				generate_multi_closure(clin->link->parent, done, SVM_STACK_INVALID);
-			else
-				generate_closure(clin->link->parent, done);
-		}
+		/* compile output node */
+		node->compile(*this);
 	}
 
-	/* compile output node */
-	node->compile(*this);
-
 	add_node(NODE_END, 0, 0, 0);
 }
 
@@ -648,9 +645,9 @@ void SVMCompiler::compile(Shader *shader, vector<int4>& global_svm_nodes, int in
 			shader->graph_bump = shader->graph->copy();
 
 	/* finalize */
-	shader->graph->finalize(false, false);
+	shader->graph->finalize(false, false, use_multi_closure);
 	if(shader->graph_bump)
-		shader->graph_bump->finalize(true, false);
+		shader->graph_bump->finalize(true, false, use_multi_closure);
 
 	current_shader = shader;
 
