@@ -36,31 +36,26 @@
 
 
 // 3d point with integer coordinates
-typedef struct
-{
+typedef struct {
 	int x, y, z;
 } Point3i;
 
-typedef struct
-{
+typedef struct {
 	Point3i begin;
 	Point3i end;
 } BoundingBox;
 
 // triangle that points to three vertices
-typedef struct 
-{
-	float vt[3][3] ;
+typedef struct {
+	float vt[3][3];
 } Triangle;
 
 // 3d point with float coordinates
-typedef struct
-{
+typedef struct {
 	float x, y, z;
 } Point3f;
 
-typedef struct
-{
+typedef struct {
 	Point3f begin;
 	Point3f end;
 } BoundingBoxf;
