@@ -87,6 +87,10 @@ public:
 	setCursorPosition(GHOST_TInt32 x,
 	                  GHOST_TInt32 y);
 
+	void
+	getAllDisplayDimensions(GHOST_TUns32& width,
+	                        GHOST_TUns32& height) const;
+
 	void
 	getMainDisplayDimensions(GHOST_TUns32& width,
 	                         GHOST_TUns32& height) const;
@@ -110,7 +114,7 @@ private:
 	             );
 
 	/* SDL specific */
-	GHOST_WindowSDL * findGhostWindow(SDL_Window *sdl_win);
+	GHOST_WindowSDL *findGhostWindow(SDL_Window *sdl_win);
 
 	bool
 	generateWindowExposeEvents();
