@@ -43,14 +43,14 @@ class STR_String;
  * upper-left corner of the screen.
  * Implements part of the GHOST_IWindow interface and adds some methods to
  * be implemented by childs of this class.
- * @author	Maarten Gribnau
- * @date	May 7, 2001
+ * \author	Maarten Gribnau
+ * \date	May 7, 2001
  */
 class GHOST_Window : public GHOST_IWindow
 {
 public:
 	/**
-	 * @section Interface inherited from GHOST_IWindow left for derived class
+	 * \section Interface inherited from GHOST_IWindow left for derived class
 	 * implementation.
 	 * virtual	bool getValid() const = 0;
 	 * virtual void setTitle(const STR_String& title) = 0;
@@ -75,23 +75,23 @@ public:
 	 * Constructor.
 	 * Creates a new window and opens it.
 	 * To check if the window was created properly, use the getValid() method.
-	 * @param width				The width the window.
-	 * @param heigh				The height the window.
-	 * @param state				The state the window is initially opened with.
-	 * @param type				The type of drawing context installed in this window.
-	 * @param stereoVisual		Stereo visual for quad buffered stereo.
-	 * @param numOfAASamples	Number of samples used for AA (zero if no AA)
+	 * \param width				The width the window.
+	 * \param heigh				The height the window.
+	 * \param state				The state the window is initially opened with.
+	 * \param type				The type of drawing context installed in this window.
+	 * \param stereoVisual		Stereo visual for quad buffered stereo.
+	 * \param numOfAASamples	Number of samples used for AA (zero if no AA)
 	 */
 	GHOST_Window(
-		GHOST_TUns32 width,
-		GHOST_TUns32 height,
-		GHOST_TWindowState state,
-		GHOST_TDrawingContextType type = GHOST_kDrawingContextTypeNone,
-		const bool stereoVisual = false,
-		const GHOST_TUns16 numOfAASamples = 0);
+	    GHOST_TUns32 width,
+	    GHOST_TUns32 height,
+	    GHOST_TWindowState state,
+	    GHOST_TDrawingContextType type = GHOST_kDrawingContextTypeNone,
+	    const bool stereoVisual = false,
+	    const GHOST_TUns16 numOfAASamples = 0);
 
 	/**
-	 * @section Interface inherited from GHOST_IWindow left for derived class
+	 * \section Interface inherited from GHOST_IWindow left for derived class
 	 * implementation.
 	 * virtual	bool getValid() const = 0;
 	 * virtual void setTitle(const STR_String& title) = 0;
@@ -119,65 +119,66 @@ public:
 
 	/**
 	 * Returns the associated OS object/handle
-	 * @return The associated OS object/handle
+	 * \return The associated OS object/handle
 	 */
-	virtual void* getOSWindow() const;
+	virtual void *getOSWindow() const;
 	
 	/**
 	 * Returns the current cursor shape.
-	 * @return	The current cursor shape.
+	 * \return	The current cursor shape.
 	 */
 	inline virtual GHOST_TStandardCursor getCursorShape() const;
 
 	/**
 	 * Set the shape of the cursor.
-	 * @param	cursor	The new cursor shape type id.
-	 * @return	Indication of success.
+	 * \param	cursor	The new cursor shape type id.
+	 * \return	Indication of success.
 	 */
 	virtual GHOST_TSuccess setCursorShape(GHOST_TStandardCursor cursorShape);
 
 	/**
 	 * Set the shape of the cursor to a custom cursor.
-	 * @param	bitmap	The bitmap data for the cursor.
-	 * @param	mask	The mask data for the cursor.
-	 * @param	hotX	The X coordinate of the cursor hotspot.
-	 * @param	hotY	The Y coordinate of the cursor hotspot.
-	 * @return	Indication of success.
+	 * \param	bitmap	The bitmap data for the cursor.
+	 * \param	mask	The mask data for the cursor.
+	 * \param	hotX	The X coordinate of the cursor hotspot.
+	 * \param	hotY	The Y coordinate of the cursor hotspot.
+	 * \return	Indication of success.
 	 */
 	virtual GHOST_TSuccess setCustomCursorShape(GHOST_TUns8 bitmap[16][2], 
-												GHOST_TUns8 mask[16][2], 
-												int hotX, 
-												int hotY);
+	                                            GHOST_TUns8 mask[16][2],
+	                                            int hotX,
+	                                            int hotY);
 												
 	virtual GHOST_TSuccess setCustomCursorShape(GHOST_TUns8 *bitmap, 
-												GHOST_TUns8 *mask, 
-												int sizex, int sizey,
-												int hotX,  int hotY,
-												int fg_color, int bg_color);
+	                                            GHOST_TUns8 *mask,
+	                                            int sizex, int sizey,
+	                                            int hotX,  int hotY,
+	                                            int fg_color, int bg_color);
 	
 	/**
 	 * Returns the visibility state of the cursor.
-	 * @return	The visibility state of the cursor.
+	 * \return	The visibility state of the cursor.
 	 */
 	inline virtual bool getCursorVisibility() const;
 	inline virtual GHOST_TGrabCursorMode getCursorGrabMode() const;
+	inline virtual bool getCursorGrabModeIsWarp() const;
 	inline virtual void getCursorGrabInitPos(GHOST_TInt32 &x, GHOST_TInt32 &y) const;
 	inline virtual void getCursorGrabAccum(GHOST_TInt32 &x, GHOST_TInt32 &y) const;
 	inline virtual void setCursorGrabAccum(GHOST_TInt32 x, GHOST_TInt32 y);
 
 	/**
 	 * Shows or hides the cursor.
-	 * @param	visible The new visibility state of the cursor.
-	 * @return	Indication of success.
+	 * \param	visible The new visibility state of the cursor.
+	 * \return	Indication of success.
 	 */
 	virtual GHOST_TSuccess setCursorVisibility(bool visible);
 
 	/**
 	 * Sets the cursor grab.
-	 * @param	mode The new grab state of the cursor.
-	 * @return	Indication of success.
+	 * \param	mode The new grab state of the cursor.
+	 * \return	Indication of success.
 	 */
-	virtual GHOST_TSuccess setCursorGrab(GHOST_TGrabCursorMode mode, GHOST_Rect *bounds);
+	virtual GHOST_TSuccess setCursorGrab(GHOST_TGrabCursorMode mode, GHOST_Rect *bounds, GHOST_TInt32 mouse_ungrab_xy[2]);
 
 	/**
 	 * Gets the cursor grab region, if unset the window is used.
@@ -186,15 +187,19 @@ public:
 	virtual GHOST_TSuccess getCursorGrabBounds(GHOST_Rect& bounds);
 
 	/**
-     * Sets the progress bar value displayed in the window/application icon
-	 * @param progress The progress % (0.0 to 1.0)
+	 * Sets the progress bar value displayed in the window/application icon
+	 * \param progress The progress % (0.0 to 1.0)
 	 */
-	virtual GHOST_TSuccess setProgressBar(float progress) {return GHOST_kFailure;};
+	virtual GHOST_TSuccess setProgressBar(float progress) {
+		return GHOST_kFailure;
+	}
 	
 	/**
 	 * Hides the progress bar in the icon
 	 */
-	virtual GHOST_TSuccess endProgressBar() {return GHOST_kFailure;};
+	virtual GHOST_TSuccess endProgressBar() {
+		return GHOST_kFailure;
+	}
 	
 	/**
 	 * Tells if the ongoing drag'n'drop object can be accepted upon mouse drop
@@ -209,20 +214,20 @@ public:
 	
 	/**
 	 * Sets the window "modified" status, indicating unsaved changes
-	 * @param isUnsavedChanges Unsaved changes or not
-	 * @return Indication of success.
+	 * \param isUnsavedChanges Unsaved changes or not
+	 * \return Indication of success.
 	 */
 	virtual GHOST_TSuccess setModifiedState(bool isUnsavedChanges);
 	
 	/**
 	 * Gets the window "modified" status, indicating unsaved changes
-	 * @return True if there are unsaved changes
+	 * \return True if there are unsaved changes
 	 */
 	virtual bool getModifiedState();
 	
 	/**
 	 * Returns the type of drawing context used in this window.
-	 * @return The current type of drawing context.
+	 * \return The current type of drawing context.
 	 */
 	inline virtual GHOST_TDrawingContextType getDrawingContextType();
 
@@ -230,14 +235,14 @@ public:
 	 * Tries to install a rendering context in this window.
 	 * Child classes do not need to overload this method.
 	 * They should overload the installDrawingContext and removeDrawingContext instead.
-	 * @param type	The type of rendering context installed.
-	 * @return Indication as to whether installation has succeeded.
+	 * \param type	The type of rendering context installed.
+	 * \return Indication as to whether installation has succeeded.
 	 */
 	virtual GHOST_TSuccess setDrawingContextType(GHOST_TDrawingContextType type);
 
 	/**
 	 * Returns the window user data.
-	 * @return The window user data.
+	 * \return The window user data.
 	 */
 	inline virtual GHOST_TUserDataPtr getUserData() const
 	{
@@ -246,24 +251,31 @@ public:
 	
 	/**
 	 * Changes the window user data.
-	 * @param data The window user data.
+	 * \param data The window user data.
 	 */
 	virtual void setUserData(const GHOST_TUserDataPtr userData)
 	{
 		m_userData = userData;
 	}
+	
+	virtual float getNativePixelSize(void)
+	{
+		if (m_nativePixelSize > 0.0f)
+			return m_nativePixelSize;
+		return 1.0f;
+	}
 
 protected:
 	/**
 	 * Tries to install a rendering context in this window.
-	 * @param type	The type of rendering context installed.
-	 * @return Indication as to whether installation has succeeded.
+	 * \param type	The type of rendering context installed.
+	 * \return Indication as to whether installation has succeeded.
 	 */
 	virtual GHOST_TSuccess installDrawingContext(GHOST_TDrawingContextType type) = 0;
 
 	/**
 	 * Removes the current drawing context.
-	 * @return Indication as to whether removal has succeeded.
+	 * \return Indication as to whether removal has succeeded.
 	 */
 	virtual GHOST_TSuccess removeDrawingContext() = 0;
 
@@ -277,7 +289,9 @@ protected:
 	 * Sets the cursor grab on the window using
 	 * native window system calls.
 	 */
-	virtual GHOST_TSuccess setWindowCursorGrab(GHOST_TGrabCursorMode mode) { return GHOST_kSuccess; };
+	virtual GHOST_TSuccess setWindowCursorGrab(GHOST_TGrabCursorMode mode) {
+		return GHOST_kSuccess;
+	}
 	
 	/**
 	 * Sets the cursor shape on the window using
@@ -294,7 +308,7 @@ protected:
 	                                                  int hotX, int hotY) = 0;
 	
 	virtual GHOST_TSuccess setWindowCustomCursorShape(GHOST_TUns8 *bitmap, GHOST_TUns8 *mask, 
-						int szx, int szy, int hotX, int hotY, int fg, int bg) = 0;
+	                                                  int szx, int szy, int hotX, int hotY, int fg, int bg) = 0;
 	/** The the of drawing context installed in this window. */
 	GHOST_TDrawingContextType m_drawingContextType;
 	
@@ -328,7 +342,7 @@ protected:
 	/** Modified state : are there unsaved changes */
 	bool m_isUnsavedChanges;
 	
-	/** Stores wether this is a full screen window. */
+	/** Stores whether this is a full screen window. */
 	bool m_fullScreen;
 
 	/** Stereo visual created. Only necessary for 'real' stereo support,
@@ -344,6 +358,9 @@ protected:
 	GHOST_TUns32 m_fullScreenWidth;
 	/** Full-screen height */
 	GHOST_TUns32 m_fullScreenHeight;
+	
+	/* OSX only, retina screens */
+	float m_nativePixelSize;
 };
 
 
@@ -362,6 +379,12 @@ inline GHOST_TGrabCursorMode GHOST_Window::getCursorGrabMode() const
 	return m_cursorGrab;
 }
 
+inline bool GHOST_Window::getCursorGrabModeIsWarp() const
+{
+	return (m_cursorGrab == GHOST_kGrabWrap) ||
+	       (m_cursorGrab == GHOST_kGrabHide);
+}
+
 inline void GHOST_Window::getCursorGrabInitPos(GHOST_TInt32 &x, GHOST_TInt32 &y) const
 {
 	x = m_cursorGrabInitPos[0];
@@ -370,14 +393,14 @@ inline void GHOST_Window::getCursorGrabInitPos(GHOST_TInt32 &x, GHOST_TInt32 &y)
 
 inline void GHOST_Window::getCursorGrabAccum(GHOST_TInt32 &x, GHOST_TInt32 &y) const
 {
-	x= m_cursorGrabAccumPos[0];
-	y= m_cursorGrabAccumPos[1];
+	x = m_cursorGrabAccumPos[0];
+	y = m_cursorGrabAccumPos[1];
 }
 
 inline void GHOST_Window::setCursorGrabAccum(GHOST_TInt32 x, GHOST_TInt32 y)
 {
-	m_cursorGrabAccumPos[0]= x;
-	m_cursorGrabAccumPos[1]= y;
+	m_cursorGrabAccumPos[0] = x;
+	m_cursorGrabAccumPos[1] = y;
 }
 
 inline GHOST_TStandardCursor GHOST_Window::getCursorShape() const
