@@ -53,8 +53,8 @@ class GHOST_SystemCocoa;
  * which is called the gutter.
  * When OpenGL contexts are active, GHOST will use AGL_BUFFER_RECT to prevent
  * OpenGL drawing outside the reduced client rectangle.
- * @author	Maarten Gribnau
- * @date	May 23, 2001
+ * \author	Maarten Gribnau
+ * \date	May 23, 2001
  */
 class GHOST_WindowCocoa : public GHOST_Window {
 public:
@@ -62,29 +62,29 @@ public:
 	 * Constructor.
 	 * Creates a new window and opens it.
 	 * To check if the window was created properly, use the getValid() method.
-	 * @param systemCocoa		The associated system class to forward events to
-	 * @param title				The text shown in the title bar of the window.
-	 * @param left				The coordinate of the left edge of the window.
-	 * @param top				The coordinate of the top edge of the window.
-	 * @param width				The width the window.
-	 * @param height			The height the window.
-	 * @param state				The state the window is initially opened with.
-	 * @param type				The type of drawing context installed in this window.
-	 * @param stereoVisual		Stereo visual for quad buffered stereo.
-	 * @param numOfAASamples	Number of samples used for AA (zero if no AA)
+	 * \param systemCocoa		The associated system class to forward events to
+	 * \param title				The text shown in the title bar of the window.
+	 * \param left				The coordinate of the left edge of the window.
+	 * \param top				The coordinate of the top edge of the window.
+	 * \param width				The width the window.
+	 * \param height			The height the window.
+	 * \param state				The state the window is initially opened with.
+	 * \param type				The type of drawing context installed in this window.
+	 * \param stereoVisual		Stereo visual for quad buffered stereo.
+	 * \param numOfAASamples	Number of samples used for AA (zero if no AA)
 	 */
 	GHOST_WindowCocoa(
-		GHOST_SystemCocoa *systemCocoa,
-		const STR_String& title,
-		GHOST_TInt32 left,
-		GHOST_TInt32 bottom,
-		GHOST_TUns32 width,
-		GHOST_TUns32 height,
-		GHOST_TWindowState state,
-		GHOST_TDrawingContextType type = GHOST_kDrawingContextTypeNone,
-		const bool stereoVisual = false,
-		const GHOST_TUns16 numOfAASamples = 0
-	);
+	    GHOST_SystemCocoa *systemCocoa,
+	    const STR_String& title,
+	    GHOST_TInt32 left,
+	    GHOST_TInt32 bottom,
+	    GHOST_TUns32 width,
+	    GHOST_TUns32 height,
+	    GHOST_TWindowState state,
+	    GHOST_TDrawingContextType type = GHOST_kDrawingContextTypeNone,
+	    const bool stereoVisual = false,
+	    const GHOST_TUns16 numOfAASamples = 0
+	    );
 
 	/**
 	 * Destructor.
@@ -94,163 +94,167 @@ public:
 
 	/**
 	 * Returns indication as to whether the window is valid.
-	 * @return The validity of the window.
+	 * \return The validity of the window.
 	 */
-	virtual	bool getValid() const;
+	virtual bool getValid() const;
 	
 	/**
 	 * Returns the associated NSWindow object
-	 * @return The associated NSWindow object
+	 * \return The associated NSWindow object
 	 */
-	virtual void* getOSWindow() const;
+	virtual void *getOSWindow() const;
 
 	/**
 	 * Sets the title displayed in the title bar.
-	 * @param title	The title to display in the title bar.
+	 * \param title	The title to display in the title bar.
 	 */
 	virtual void setTitle(const STR_String& title);
 
 	/**
 	 * Returns the title displayed in the title bar.
-	 * @param title	The title displayed in the title bar.
+	 * \param title	The title displayed in the title bar.
 	 */
 	virtual void getTitle(STR_String& title) const;
 
 	/**
 	 * Returns the window rectangle dimensions.
 	 * The dimensions are given in screen coordinates that are relative to the upper-left corner of the screen. 
-	 * @param bounds The bounding rectangle of the window.
+	 * \param bounds The bounding rectangle of the window.
 	 */
-	virtual	void getWindowBounds(GHOST_Rect& bounds) const;
+	virtual void getWindowBounds(GHOST_Rect& bounds) const;
 	
 	/**
 	 * Returns the client rectangle dimensions.
 	 * The left and top members of the rectangle are always zero.
-	 * @param bounds The bounding rectangle of the cleient area of the window.
+	 * \param bounds The bounding rectangle of the cleient area of the window.
 	 */
-	virtual	void getClientBounds(GHOST_Rect& bounds) const;
+	virtual void getClientBounds(GHOST_Rect& bounds) const;
 
 	/**
 	 * Resizes client rectangle width.
-	 * @param width The new width of the client area of the window.
+	 * \param width The new width of the client area of the window.
 	 */
-	virtual	GHOST_TSuccess setClientWidth(GHOST_TUns32 width);
+	virtual GHOST_TSuccess setClientWidth(GHOST_TUns32 width);
 
 	/**
 	 * Resizes client rectangle height.
-	 * @param height The new height of the client area of the window.
+	 * \param height The new height of the client area of the window.
 	 */
-	virtual	GHOST_TSuccess setClientHeight(GHOST_TUns32 height);
+	virtual GHOST_TSuccess setClientHeight(GHOST_TUns32 height);
 
 	/**
 	 * Resizes client rectangle.
-	 * @param width		The new width of the client area of the window.
-	 * @param height	The new height of the client area of the window.
+	 * \param width		The new width of the client area of the window.
+	 * \param height	The new height of the client area of the window.
 	 */
-	virtual	GHOST_TSuccess setClientSize(GHOST_TUns32 width, GHOST_TUns32 height);
+	virtual GHOST_TSuccess setClientSize(GHOST_TUns32 width, GHOST_TUns32 height);
 
 	/**
 	 * Returns the state of the window (normal, minimized, maximized).
-	 * @return The state of the window.
+	 * \return The state of the window.
 	 */
 	virtual GHOST_TWindowState getState() const;
 
 	/**
 	 * Sets the window "modified" status, indicating unsaved changes
-	 * @param isUnsavedChanges Unsaved changes or not
-	 * @return Indication of success.
+	 * \param isUnsavedChanges Unsaved changes or not
+	 * \return Indication of success.
 	 */
 	virtual GHOST_TSuccess setModifiedState(bool isUnsavedChanges);
 	
 	/**
 	 * Converts a point in screen coordinates to client rectangle coordinates
-	 * @param inX	The x-coordinate on the screen.
-	 * @param inY	The y-coordinate on the screen.
-	 * @param outX	The x-coordinate in the client rectangle.
-	 * @param outY	The y-coordinate in the client rectangle.
+	 * \param inX	The x-coordinate on the screen.
+	 * \param inY	The y-coordinate on the screen.
+	 * \param outX	The x-coordinate in the client rectangle.
+	 * \param outY	The y-coordinate in the client rectangle.
 	 */
-	virtual	void screenToClient(GHOST_TInt32 inX, GHOST_TInt32 inY, GHOST_TInt32& outX, GHOST_TInt32& outY) const;
+	virtual void screenToClient(GHOST_TInt32 inX, GHOST_TInt32 inY, GHOST_TInt32& outX, GHOST_TInt32& outY) const;
 
 	/**
 	 * Converts a point in screen coordinates to client rectangle coordinates
-	 * @param inX	The x-coordinate in the client rectangle.
-	 * @param inY	The y-coordinate in the client rectangle.
-	 * @param outX	The x-coordinate on the screen.
-	 * @param outY	The y-coordinate on the screen.
+	 * \param inX	The x-coordinate in the client rectangle.
+	 * \param inY	The y-coordinate in the client rectangle.
+	 * \param outX	The x-coordinate on the screen.
+	 * \param outY	The y-coordinate on the screen.
 	 */
-	virtual	void clientToScreen(GHOST_TInt32 inX, GHOST_TInt32 inY, GHOST_TInt32& outX, GHOST_TInt32& outY) const;
+	virtual void clientToScreen(GHOST_TInt32 inX, GHOST_TInt32 inY, GHOST_TInt32& outX, GHOST_TInt32& outY) const;
 
 	/**
 	 * Converts a point in screen coordinates to client rectangle coordinates
 	 * but without the y coordinate conversion needed for ghost compatibility.
-	 * @param inX	The x-coordinate in the client rectangle.
-	 * @param inY	The y-coordinate in the client rectangle.
-	 * @param outX	The x-coordinate on the screen.
-	 * @param outY	The y-coordinate on the screen.
+	 * \param inX	The x-coordinate in the client rectangle.
+	 * \param inY	The y-coordinate in the client rectangle.
+	 * \param outX	The x-coordinate on the screen.
+	 * \param outY	The y-coordinate on the screen.
 	 */
 	void clientToScreenIntern(GHOST_TInt32 inX, GHOST_TInt32 inY, GHOST_TInt32& outX, GHOST_TInt32& outY) const;
 
 	/**
 	 * Converts a point in screen coordinates to client rectangle coordinates,
 	 * but without the y coordinate conversion needed for ghost compatibility.
-	 * @param inX	The x-coordinate in the client rectangle.
-	 * @param inY	The y-coordinate in the client rectangle.
-	 * @param outX	The x-coordinate on the screen.
-	 * @param outY	The y-coordinate on the screen.
+	 * \param inX	The x-coordinate in the client rectangle.
+	 * \param inY	The y-coordinate in the client rectangle.
+	 * \param outX	The x-coordinate on the screen.
+	 * \param outY	The y-coordinate on the screen.
 	 */
 	void screenToClientIntern(GHOST_TInt32 inX, GHOST_TInt32 inY, GHOST_TInt32& outX, GHOST_TInt32& outY) const;
 
 	/**
 	 * Gets the screen the window is displayed in
-	 * @return The NSScreen object
+	 * \return The NSScreen object
 	 */
-	NSScreen* getScreen();
+	NSScreen *getScreen();
 	
 	/**
 	 * Sets the state of the window (normal, minimized, maximized).
-	 * @param state The state of the window.
-	 * @return Indication of success.
+	 * \param state The state of the window.
+	 * \return Indication of success.
 	 */
 	virtual GHOST_TSuccess setState(GHOST_TWindowState state);
 
 	/**
 	 * Sets the order of the window (bottom, top).
-	 * @param order The order of the window.
-	 * @return Indication of success.
+	 * \param order The order of the window.
+	 * \return Indication of success.
 	 */
 	virtual GHOST_TSuccess setOrder(GHOST_TWindowOrder order);
 
 	/**
 	 * Swaps front and back buffers of a window.
-	 * @return	A boolean success indicator.
+	 * \return	A boolean success indicator.
 	 */
 	virtual GHOST_TSuccess swapBuffers();
 
 	/**
 	 * Updates the drawing context of this window. Needed
 	 * whenever the window is changed.
-	 * @return Indication of success.
+	 * \return Indication of success.
 	 */
 	GHOST_TSuccess updateDrawingContext();
 
 	/**
 	 * Activates the drawing context of this window.
-	 * @return	A boolean success indicator.
+	 * \return	A boolean success indicator.
 	 */
 	virtual GHOST_TSuccess activateDrawingContext();
 
 	virtual void loadCursor(bool visible, GHOST_TStandardCursor cursor) const;
     
 
-	const GHOST_TabletData* GetTabletData()
-	{ return &m_tablet; }
+	const GHOST_TabletData *GetTabletData()
+	{
+		return &m_tablet;
+	}
 
 	GHOST_TabletData& GetCocoaTabletData()
-	{ return m_tablet; }
+	{
+		return m_tablet;
+	}
 	
 	/**
 	 * Sets the progress bar value displayed in the window/application icon
-	 * @param progress The progress % (0.0 to 1.0)
+	 * \param progress The progress % (0.0 to 1.0)
 	 */
 	virtual GHOST_TSuccess setProgressBar(float progress);
 	
@@ -258,23 +262,28 @@ public:
 	 * Hides the progress bar icon
 	 */
 	virtual GHOST_TSuccess endProgressBar();
+	
+	
+	virtual void setNativePixelSize(void);
+
+	
 protected:
 	/**
 	 * Tries to install a rendering context in this window.
-	 * @param type	The type of rendering context installed.
-	 * @return Indication as to whether installation has succeeded.
+	 * \param type	The type of rendering context installed.
+	 * \return Indication as to whether installation has succeeded.
 	 */
 	virtual GHOST_TSuccess installDrawingContext(GHOST_TDrawingContextType type);
 
 	/**
 	 * Removes the current drawing context.
-	 * @return Indication as to whether removal has succeeded.
+	 * \return Indication as to whether removal has succeeded.
 	 */
 	virtual GHOST_TSuccess removeDrawingContext();
     
 	/**
 	 * Invalidates the contents of this window.
-         * @return Indication of success.
+	 * \return Indication of success.
 	 */
 	virtual GHOST_TSuccess invalidate();
 
@@ -301,7 +310,7 @@ protected:
 	 * native window system calls.
 	 */
 	virtual GHOST_TSuccess setWindowCustomCursorShape(GHOST_TUns8 *bitmap, GHOST_TUns8 *mask,
-					int sizex, int sizey, int hotX, int hotY, int fg_color, int bg_color);
+	                                                  int sizex, int sizey, int hotX, int hotY, int fg_color, int bg_color);
 					
 	virtual GHOST_TSuccess setWindowCustomCursorShape(GHOST_TUns8 bitmap[16][2], GHOST_TUns8 mask[16][2], int hotX, int hotY);
 
@@ -320,7 +329,7 @@ protected:
 	/** The first created OpenGL context (for sharing display lists) */
 	static NSOpenGLContext *s_firstOpenGLcontext;
 	
-	NSCursor*	m_customCursor;
+	NSCursor *m_customCursor;
 
 	GHOST_TabletData m_tablet;
 };
