@@ -31,10 +31,9 @@
 
 
 /**
-
  * Copyright (C) 2001 NaN Technologies B.V.
- * @author	Maarten Gribnau
- * @date	May 11, 2001
+ * \author	Maarten Gribnau
+ * \date	May 11, 2001
  */
 
 #include "GHOST_WindowManager.h"
@@ -57,7 +56,7 @@ GHOST_WindowManager::~GHOST_WindowManager()
 }
 
 
-GHOST_TSuccess GHOST_WindowManager::addWindow(GHOST_IWindow* window)
+GHOST_TSuccess GHOST_WindowManager::addWindow(GHOST_IWindow *window)
 {
 	GHOST_TSuccess success = GHOST_kFailure;
 	if (window) {
@@ -71,7 +70,7 @@ GHOST_TSuccess GHOST_WindowManager::addWindow(GHOST_IWindow* window)
 }
 
 
-GHOST_TSuccess GHOST_WindowManager::removeWindow(const GHOST_IWindow* window)
+GHOST_TSuccess GHOST_WindowManager::removeWindow(const GHOST_IWindow *window)
 {
 	GHOST_TSuccess success = GHOST_kFailure;
 	if (window) {
@@ -79,7 +78,7 @@ GHOST_TSuccess GHOST_WindowManager::removeWindow(const GHOST_IWindow* window)
 			endFullScreen();
 		}
 		else {
-			std::vector<GHOST_IWindow*>::iterator result = find(m_windows.begin(), m_windows.end(), window);
+			std::vector<GHOST_IWindow *>::iterator result = find(m_windows.begin(), m_windows.end(), window);
 			if (result != m_windows.end()) {
 				setWindowInactive(window);
 				m_windows.erase(result);
@@ -91,19 +90,19 @@ GHOST_TSuccess GHOST_WindowManager::removeWindow(const GHOST_IWindow* window)
 }
 
 
-bool GHOST_WindowManager::getWindowFound(const GHOST_IWindow* window) const
+bool GHOST_WindowManager::getWindowFound(const GHOST_IWindow *window) const
 {
 	bool found = false;
 	if (window) {
-        if (getFullScreen() && (window == m_fullScreenWindow)) {
-            found = true;
-        }
-        else {
-            std::vector<GHOST_IWindow*>::const_iterator result = find(m_windows.begin(), m_windows.end(), window);
-            if (result != m_windows.end()) {
-                found = true;
-            }
-        }
+		if (getFullScreen() && (window == m_fullScreenWindow)) {
+			found = true;
+		}
+		else {
+			std::vector<GHOST_IWindow *>::const_iterator result = find(m_windows.begin(), m_windows.end(), window);
+			if (result != m_windows.end()) {
+				found = true;
+			}
+		}
 	}
 	return found;
 }
@@ -115,13 +114,13 @@ bool GHOST_WindowManager::getFullScreen(void) const
 }
 
 
-GHOST_IWindow* GHOST_WindowManager::getFullScreenWindow(void) const
+GHOST_IWindow *GHOST_WindowManager::getFullScreenWindow(void) const
 {
-    return m_fullScreenWindow;
+	return m_fullScreenWindow;
 }
 
 
-GHOST_TSuccess GHOST_WindowManager::beginFullScreen(GHOST_IWindow* window,
+GHOST_TSuccess GHOST_WindowManager::beginFullScreen(GHOST_IWindow *window,
 		bool stereoVisual)
 {
 	GHOST_TSuccess success = GHOST_kFailure;
@@ -151,13 +150,13 @@ GHOST_TSuccess GHOST_WindowManager::endFullScreen(void)
 				setActiveWindow(m_activeWindowBeforeFullScreen);
 			}
 		}
-        success = GHOST_kSuccess;
+		success = GHOST_kSuccess;
 	}
 	return success;
 }
 
 
-GHOST_TSuccess GHOST_WindowManager::setActiveWindow(GHOST_IWindow* window)
+GHOST_TSuccess GHOST_WindowManager::setActiveWindow(GHOST_IWindow *window)
 {
 	GHOST_TSuccess success = GHOST_kSuccess;
 	if (window != m_activeWindow) {
@@ -172,13 +171,13 @@ GHOST_TSuccess GHOST_WindowManager::setActiveWindow(GHOST_IWindow* window)
 }
 	
 
-GHOST_IWindow* GHOST_WindowManager::getActiveWindow(void) const
+GHOST_IWindow *GHOST_WindowManager::getActiveWindow(void) const
 {
 	return m_activeWindow;
 }
 
 
-void GHOST_WindowManager::setWindowInactive(const GHOST_IWindow* window)
+void GHOST_WindowManager::setWindowInactive(const GHOST_IWindow *window)
 {
 	if (window == m_activeWindow) {
 		m_activeWindow = 0;
@@ -192,11 +191,11 @@ std::vector<GHOST_IWindow *> &GHOST_WindowManager::getWindows()
 }
 
 
-GHOST_IWindow* GHOST_WindowManager::getWindowAssociatedWithOSWindow(void* osWindow)
+GHOST_IWindow *GHOST_WindowManager::getWindowAssociatedWithOSWindow(void *osWindow)
 {
-	std::vector<GHOST_IWindow*>::iterator iter;
+	std::vector<GHOST_IWindow *>::iterator iter;
 
-	for (iter = m_windows.begin(); iter != m_windows.end(); iter++) {
+	for (iter = m_windows.begin(); iter != m_windows.end(); ++iter) {
 		if ((*iter)->getOSWindow() == osWindow)
 			return *iter;
 	}
@@ -207,9 +206,9 @@ GHOST_IWindow* GHOST_WindowManager::getWindowAssociatedWithOSWindow(void* osWind
 bool GHOST_WindowManager::getAnyModifiedState()
 {
 	bool isAnyModified = false;
-	std::vector<GHOST_IWindow*>::iterator iter;
+	std::vector<GHOST_IWindow *>::iterator iter;
 	
-	for (iter = m_windows.begin(); iter != m_windows.end(); iter++) {
+	for (iter = m_windows.begin(); iter != m_windows.end(); ++iter) {
 		if ((*iter)->getModifiedState())
 			isAnyModified = true;
 	}
