@@ -51,7 +51,7 @@ static const float persistence = 0.56123f;
 //////////////////////////////////////////////////////////////////////
 // constructor
 //////////////////////////////////////////////////////////////////////
-WTURBULENCE::WTURBULENCE(int xResSm, int yResSm, int zResSm, int amplify, int noisetype)
+WTURBULENCE::WTURBULENCE(int xResSm, int yResSm, int zResSm, int amplify, int noisetype, int init_fire, int init_colors)
 {
 	// if noise magnitude is below this threshold, its contribution
 	// is negilgible, so stop evaluating new octaves
@@ -87,12 +87,26 @@ WTURBULENCE::WTURBULENCE(int xResSm, int yResSm, int zResSm, int amplify, int no
 	// allocate high resolution density field
 	_totalStepsBig = 0;
 	_densityBig = new float[_totalCellsBig];
-	_densityBigOld = new float[_totalCellsBig]; 
+	_densityBigOld = new float[_totalCellsBig];
 	
 	for(int i = 0; i < _totalCellsBig; i++) {
 		_densityBig[i] = 
 		_densityBigOld[i] = 0.;
 	}
+
+	/* fire */
+	_flameBig = _fuelBig = _fuelBigOld = NULL;
+	_reactBig = _reactBigOld = NULL;
+	if (init_fire) {
+		initFire();
+	}
+	/* colors */
+	_color_rBig = _color_rBigOld = NULL;
+	_color_gBig = _color_gBigOld = NULL;
+	_color_bBig = _color_bBigOld = NULL;
+	if (init_colors) {
+		initColors(0.0f, 0.0f, 0.0f);
+	}
 	
 	// allocate & init texture coordinates
 	_tcU = new float[_totalCellsSm];
@@ -128,12 +142,64 @@ WTURBULENCE::WTURBULENCE(int xResSm, int yResSm, int zResSm, int amplify, int no
 	*/
 }
 
+void WTURBULENCE::initFire()
+{
+	if (!_fuelBig) {
+		_flameBig = new float[_totalCellsBig];
+		_fuelBig = new float[_totalCellsBig];
+		_fuelBigOld = new float[_totalCellsBig];
+		_reactBig = new float[_totalCellsBig];
+		_reactBigOld = new float[_totalCellsBig];
+
+		for(int i = 0; i < _totalCellsBig; i++) {
+			_flameBig[i] = 
+			_fuelBig[i] = 
+			_fuelBigOld[i] = 0.;
+			_reactBig[i] = 
+			_reactBigOld[i] = 0.;
+		}
+	}
+}
+
+void WTURBULENCE::initColors(float init_r, float init_g, float init_b)
+{
+	if (!_color_rBig) {
+		_color_rBig = new float[_totalCellsBig];
+		_color_rBigOld = new float[_totalCellsBig];
+		_color_gBig = new float[_totalCellsBig];
+		_color_gBigOld = new float[_totalCellsBig];
+		_color_bBig = new float[_totalCellsBig];
+		_color_bBigOld = new float[_totalCellsBig];
+
+		for(int i = 0; i < _totalCellsBig; i++) {
+			_color_rBig[i] = _densityBig[i] * init_r;
+			_color_rBigOld[i] = 0.0f;
+			_color_gBig[i] = _densityBig[i] * init_g;
+			_color_gBigOld[i] = 0.0f;
+			_color_bBig[i] = _densityBig[i] * init_b;
+			_color_bBigOld[i] = 0.0f;
+		}
+	}
+}
+
 //////////////////////////////////////////////////////////////////////
 // destructor
 //////////////////////////////////////////////////////////////////////
 WTURBULENCE::~WTURBULENCE() {
   delete[] _densityBig;
   delete[] _densityBigOld;
+  if (_flameBig) delete[] _flameBig;
+  if (_fuelBig) delete[] _fuelBig;
+  if (_fuelBigOld) delete[] _fuelBigOld;
+  if (_reactBig) delete[] _reactBig;
+  if (_reactBigOld) delete[] _reactBigOld;
+
+  if (_color_rBig) delete[] _color_rBig;
+  if (_color_rBigOld) delete[] _color_rBigOld;
+  if (_color_gBig) delete[] _color_gBig;
+  if (_color_gBigOld) delete[] _color_gBigOld;
+  if (_color_bBig) delete[] _color_bBig;
+  if (_color_bBigOld) delete[] _color_bBigOld;
 
   delete[] _tcU;
   delete[] _tcV;
@@ -154,21 +220,25 @@ void WTURBULENCE::setNoise(int type)
 {
 	if(type == (1<<1)) // FFT
 	{
+#ifdef WITH_FFTW3
 		// needs fft
-		#ifdef WITH_FFTW3
 		std::string noiseTileFilename = std::string("noise.fft");
 		generatTile_FFT(_noiseTile, noiseTileFilename);
-		#endif
+		return;
+#else
+		fprintf(stderr, "FFTW not enabled, falling back to wavelet noise.\n");
+#endif
 	}
-	else if(type == (1<<2)) // curl
+#if 0
+	if(type == (1<<2)) // curl
 	{
 		// TODO: not supported yet
+		return;
 	}
-	else // standard - wavelet
-	{
-		std::string noiseTileFilename = std::string("noise.wavelets");
-		generateTile_WAVELET(_noiseTile, noiseTileFilename);
-	}
+#endif
+
+	std::string noiseTileFilename = std::string("noise.wavelets");
+	generateTile_WAVELET(_noiseTile, noiseTileFilename);
 }
 
 // init direct access functions from blender
@@ -431,8 +501,11 @@ void WTURBULENCE::decomposeEnergy(float *_energy, float *_highFreqEnergy)
 // compute velocity from energies and march into obstacles
 // for wavelet decomposition
 ////////////////////////////////////////////////////////////////////// 
-void WTURBULENCE::computeEnergy(float *_energy, float* xvel, float* yvel, float* zvel, unsigned char *obstacles) 
+void WTURBULENCE::computeEnergy(float *_energy, float* xvel, float* yvel, float* zvel, unsigned char *origObstacles) 
 {
+  unsigned char *obstacles = new unsigned char[_totalCellsSm];
+  memcpy(obstacles, origObstacles, sizeof(unsigned char) * _totalCellsSm);
+
   // compute everywhere
   for (int x = 0; x < _totalCellsSm; x++) 
     _energy[x] = 0.5f * (xvel[x] * xvel[x] + yvel[x] * yvel[x] + zvel[x] * zvel[x]);
@@ -506,7 +579,9 @@ void WTURBULENCE::computeEnergy(float *_energy, float* xvel, float* yvel, float*
     for (int y = 1; y < _yResSm - 1; y++, index += 2)
       for (int x = 1; x < _xResSm - 1; x++, index++)
         if (obstacles[index])
-          obstacles[index] = 1;
+          obstacles[index] = 1; // DG TODO ? animated obstacle flag?
+
+  delete [] obstacles;
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////
@@ -752,8 +827,10 @@ void WTURBULENCE::stepTurbulenceFull(float dtOrg, float* xvel, float* yvel, floa
 	// enlarge timestep to match grid
 	const float dt = dtOrg * _amplify;
 	const float invAmp = 1.0f / _amplify;
-	float *tempBig1 = (float *)calloc(_totalCellsBig, sizeof(float));
-	float *tempBig2 = (float *)calloc(_totalCellsBig, sizeof(float));
+	float *tempFuelBig = NULL, *tempReactBig = NULL;
+	float *tempColor_rBig = NULL, *tempColor_gBig = NULL, *tempColor_bBig = NULL;
+	float *tempDensityBig = (float *)calloc(_totalCellsBig, sizeof(float));
+	float *tempBig = (float *)calloc(_totalCellsBig, sizeof(float));
 	float *bigUx = (float *)calloc(_totalCellsBig, sizeof(float));
 	float *bigUy = (float *)calloc(_totalCellsBig, sizeof(float));
 	float *bigUz = (float *)calloc(_totalCellsBig, sizeof(float)); 
@@ -762,11 +839,21 @@ void WTURBULENCE::stepTurbulenceFull(float dtOrg, float* xvel, float* yvel, floa
 	float *eigMin  = (float *)calloc(_totalCellsSm, sizeof(float));
 	float *eigMax  = (float *)calloc(_totalCellsSm, sizeof(float));
 
+	if (_fuelBig) {
+		tempFuelBig = (float *)calloc(_totalCellsBig, sizeof(float));
+		tempReactBig = (float *)calloc(_totalCellsBig, sizeof(float));
+	}
+	if (_color_rBig) {
+		tempColor_rBig = (float *)calloc(_totalCellsBig, sizeof(float));
+		tempColor_gBig = (float *)calloc(_totalCellsBig, sizeof(float));
+		tempColor_bBig = (float *)calloc(_totalCellsBig, sizeof(float));
+	}
+
 	memset(_tcTemp, 0, sizeof(float)*_totalCellsSm);
 
 
 	// prepare textures
-	advectTextureCoordinates(dtOrg, xvel,yvel,zvel, tempBig1, tempBig2);
+	advectTextureCoordinates(dtOrg, xvel,yvel,zvel, tempDensityBig, tempBig);
 
 	// do wavelet decomposition of energy
 	computeEnergy(_energy, xvel, yvel, zvel, obstacles);
@@ -967,6 +1054,11 @@ void WTURBULENCE::stepTurbulenceFull(float dtOrg, float* xvel, float* yvel, floa
 
   // prepare density for an advection
   SWAP_POINTERS(_densityBig, _densityBigOld);
+  SWAP_POINTERS(_fuelBig, _fuelBigOld);
+  SWAP_POINTERS(_reactBig, _reactBigOld);
+  SWAP_POINTERS(_color_rBig, _color_rBigOld);
+  SWAP_POINTERS(_color_gBig, _color_gBigOld);
+  SWAP_POINTERS(_color_bBig, _color_bBigOld);
 
   // based on the maximum velocity present, see if we need to substep,
   // but cap the maximum number of substeps to 5
@@ -1012,7 +1104,21 @@ void WTURBULENCE::stepTurbulenceFull(float dtOrg, float* xvel, float* yvel, floa
 		int zEnd = (int)((float)(i+1)*partSize + 0.5f);
 #endif
 		FLUID_3D::advectFieldMacCormack1(dtSubdiv, bigUx, bigUy, bigUz, 
-		    _densityBigOld, tempBig1, _resBig, zBegin, zEnd);
+		    _densityBigOld, tempDensityBig, _resBig, zBegin, zEnd);
+		if (_fuelBig) {
+			FLUID_3D::advectFieldMacCormack1(dtSubdiv, bigUx, bigUy, bigUz, 
+				_fuelBigOld, tempFuelBig, _resBig, zBegin, zEnd);
+			FLUID_3D::advectFieldMacCormack1(dtSubdiv, bigUx, bigUy, bigUz, 
+				_reactBigOld, tempReactBig, _resBig, zBegin, zEnd);
+		}
+		if (_color_rBig) {
+			FLUID_3D::advectFieldMacCormack1(dtSubdiv, bigUx, bigUy, bigUz, 
+				_color_rBigOld, tempColor_rBig, _resBig, zBegin, zEnd);
+			FLUID_3D::advectFieldMacCormack1(dtSubdiv, bigUx, bigUy, bigUz, 
+				_color_gBigOld, tempColor_gBig, _resBig, zBegin, zEnd);
+			FLUID_3D::advectFieldMacCormack1(dtSubdiv, bigUx, bigUy, bigUz, 
+				_color_bBigOld, tempColor_bBig, _resBig, zBegin, zEnd);
+		}
 #if PARALLEL==1
 	}
 
@@ -1025,18 +1131,43 @@ void WTURBULENCE::stepTurbulenceFull(float dtOrg, float* xvel, float* yvel, floa
 		int zEnd = (int)((float)(i+1)*partSize + 0.5f);
 #endif
 		FLUID_3D::advectFieldMacCormack2(dtSubdiv, bigUx, bigUy, bigUz, 
-		    _densityBigOld, _densityBig, tempBig1, tempBig2, _resBig, NULL, zBegin, zEnd);
+		    _densityBigOld, _densityBig, tempDensityBig, tempBig, _resBig, NULL, zBegin, zEnd);
+		if (_fuelBig) {
+			FLUID_3D::advectFieldMacCormack2(dtSubdiv, bigUx, bigUy, bigUz, 
+				_fuelBigOld, _fuelBig, tempFuelBig, tempBig, _resBig, NULL, zBegin, zEnd);
+			FLUID_3D::advectFieldMacCormack2(dtSubdiv, bigUx, bigUy, bigUz, 
+				_reactBigOld, _reactBig, tempReactBig, tempBig, _resBig, NULL, zBegin, zEnd);
+		}
+		if (_color_rBig) {
+			FLUID_3D::advectFieldMacCormack2(dtSubdiv, bigUx, bigUy, bigUz, 
+				_color_rBigOld, _color_rBig, tempColor_rBig, tempBig, _resBig, NULL, zBegin, zEnd);
+			FLUID_3D::advectFieldMacCormack2(dtSubdiv, bigUx, bigUy, bigUz, 
+				_color_gBigOld, _color_gBig, tempColor_gBig, tempBig, _resBig, NULL, zBegin, zEnd);
+			FLUID_3D::advectFieldMacCormack2(dtSubdiv, bigUx, bigUy, bigUz, 
+				_color_bBigOld, _color_bBig, tempColor_bBig, tempBig, _resBig, NULL, zBegin, zEnd);
+		}
 #if PARALLEL==1
 	}
 	}
 #endif
 
-    if (substep < totalSubsteps - 1) 
+	if (substep < totalSubsteps - 1) {
       SWAP_POINTERS(_densityBig, _densityBigOld);
+	  SWAP_POINTERS(_fuelBig, _fuelBigOld);
+	  SWAP_POINTERS(_reactBig, _reactBigOld);
+	  SWAP_POINTERS(_color_rBig, _color_rBigOld);
+	  SWAP_POINTERS(_color_gBig, _color_gBigOld);
+	  SWAP_POINTERS(_color_bBig, _color_bBigOld);
+	}
   } // substep
 
-  free(tempBig1);
-  free(tempBig2);
+  free(tempDensityBig);
+  if (tempFuelBig) free(tempFuelBig);
+  if (tempReactBig) free(tempReactBig);
+  if (tempColor_rBig) free(tempColor_rBig);
+  if (tempColor_gBig) free(tempColor_gBig);
+  if (tempColor_bBig) free(tempColor_bBig);
+  free(tempBig);
   free(bigUx);
   free(bigUy);
   free(bigUz);
@@ -1045,6 +1176,15 @@ void WTURBULENCE::stepTurbulenceFull(float dtOrg, float* xvel, float* yvel, floa
   
   // wipe the density borders
   FLUID_3D::setZeroBorder(_densityBig, _resBig, 0 , _resBig[2]);
+  if (_fuelBig) {
+	FLUID_3D::setZeroBorder(_fuelBig, _resBig, 0 , _resBig[2]);
+	FLUID_3D::setZeroBorder(_reactBig, _resBig, 0 , _resBig[2]);
+  }
+  if (_color_rBig) {
+	  FLUID_3D::setZeroBorder(_color_rBig, _resBig, 0 , _resBig[2]);
+	  FLUID_3D::setZeroBorder(_color_gBig, _resBig, 0 , _resBig[2]);
+	  FLUID_3D::setZeroBorder(_color_bBig, _resBig, 0 , _resBig[2]);
+  }
     
   // reset texture coordinates now in preparation for next timestep
   // Shouldn't do this before generating the noise because then the 
