@@ -43,14 +43,18 @@
 
 #include "DNA_vec_types.h"
 
-
-#include "BLI_blenlib.h"
-#include "BLI_linklist.h"	/* linknode */
+#include "BLI_listbase.h"
 #include "BLI_math.h"
+#include "BLI_rect.h"
+#include "BLI_string.h"
+#include "BLI_string_utf8.h"
+#include "BLI_linklist.h"  /* linknode */
 
 #include "BIF_gl.h"
 #include "BLF_api.h"
 
+#include "IMB_colormanagement.h"
+
 #include "blf_internal_types.h"
 #include "blf_internal.h"
 
@@ -103,7 +107,7 @@ static void blf_font_ensure_ascii_table(FontBLF *font)
 	if (glyph_ascii_table['0'] == NULL) {
 		GlyphBLF *g;
 		unsigned int i;
-		for (i = 0; i<256; i++) {
+		for (i = 0; i < 256; i++) {
 			g = blf_glyph_search(font->glyph_cache, i);
 			if (!g) {
 				FT_UInt glyph_index = FT_Get_Char_Index(font->face, i);
@@ -132,7 +136,7 @@ static void blf_font_ensure_ascii_table(FontBLF *font)
 			_g = blf_glyph_add(_font,                                            \
 			                  FT_Get_Char_Index((_font)->face, _c), _c);         \
 		}                                                                        \
-	}                                                                            \
+	} (void)0
 
 
 #define BLF_KERNING_VARS(_font, _has_kerning, _kern_mode)                        \
@@ -152,12 +156,12 @@ static void blf_font_ensure_ascii_table(FontBLF *font)
 		                   _kern_mode,                                           \
 		                   &(_delta)) == 0)                                      \
 		{                                                                        \
-			_pen_x += delta.x >> 6;                                              \
+			_pen_x += _delta.x >> 6;                                             \
 		}                                                                        \
 	}                                                                            \
-}                                                                                \
+} (void)0
 
-void blf_font_draw(FontBLF *font, const char *str, unsigned int len)
+void blf_font_draw(FontBLF *font, const char *str, size_t len)
 {
 	unsigned int c;
 	GlyphBLF *g, *g_prev = NULL;
@@ -170,7 +174,7 @@ void blf_font_draw(FontBLF *font, const char *str, unsigned int len)
 
 	blf_font_ensure_ascii_table(font);
 
-	while (str[i] && i < len) {
+	while ((i < len) && str[i]) {
 		BLF_UTF8_NEXT_FAST(font, g, str, i, c, glyph_ascii_table);
 
 		if (c == BLI_UTF8_ERR)
@@ -189,7 +193,7 @@ void blf_font_draw(FontBLF *font, const char *str, unsigned int len)
 }
 
 /* faster version of blf_font_draw, ascii only for view dimensions */
-void blf_font_draw_ascii(FontBLF *font, const char *str, unsigned int len)
+void blf_font_draw_ascii(FontBLF *font, const char *str, size_t len)
 {
 	unsigned char c;
 	GlyphBLF *g, *g_prev = NULL;
@@ -225,11 +229,14 @@ void blf_font_buffer(FontBLF *font, const char *str)
 	size_t i = 0;
 	GlyphBLF **glyph_ascii_table = font->glyph_cache->glyph_ascii_table;
 
-	/* buffer specific vars*/
-	const unsigned char b_col_char[4] = {font->b_col[0] * 255,
-	                                     font->b_col[1] * 255,
-	                                     font->b_col[2] * 255,
-	                                     font->b_col[3] * 255};
+	/* buffer specific vars */
+	FontBufInfoBLF *buf_info = &font->buf_info;
+	float b_col_float[4];
+	const unsigned char b_col_char[4] = {buf_info->col[0] * 255,
+	                                     buf_info->col[1] * 255,
+	                                     buf_info->col[2] * 255,
+	                                     buf_info->col[3] * 255};
+
 	unsigned char *cbuf;
 	int chx, chy;
 	int y, x;
@@ -239,6 +246,15 @@ void blf_font_buffer(FontBLF *font, const char *str)
 
 	blf_font_ensure_ascii_table(font);
 
+	/* another buffer specific call for color conversion */
+	if (buf_info->display) {
+		copy_v4_v4(b_col_float, buf_info->col);
+		IMB_colormanagement_display_to_scene_linear_v3(b_col_float, buf_info->display);
+	}
+	else {
+		srgb_to_linearrgb_v4(b_col_float, buf_info->col);
+	}
+
 	while (str[i]) {
 		BLF_UTF8_NEXT_FAST(font, g, str, i, c, glyph_ascii_table);
 
@@ -259,16 +275,16 @@ void blf_font_buffer(FontBLF *font, const char *str)
 			pen_y = (int)font->pos[1] - (g->height - (int)g->pos_y);
 		}
 
-		if ((chx + g->width) >= 0 && chx < font->bw && (pen_y + g->height) >= 0 && pen_y < font->bh) {
+		if ((chx + g->width) >= 0 && chx < buf_info->w && (pen_y + g->height) >= 0 && pen_y < buf_info->h) {
 			/* don't draw beyond the buffer bounds */
 			int width_clip = g->width;
 			int height_clip = g->height;
-			int yb_start = g->pitch < 0 ? 0 : g->height-1;
+			int yb_start = g->pitch < 0 ? 0 : g->height - 1;
 
-			if (width_clip + chx > font->bw)
-				width_clip -= chx + width_clip - font->bw;
-			if (height_clip + pen_y > font->bh)
-				height_clip -= pen_y + height_clip - font->bh;
+			if (width_clip + chx > buf_info->w)
+				width_clip -= chx + width_clip - buf_info->w;
+			if (height_clip + pen_y > buf_info->h)
+				height_clip -= pen_y + height_clip - buf_info->h;
 			
 			/* drawing below the image? */
 			if (pen_y < 0) {
@@ -277,7 +293,7 @@ void blf_font_buffer(FontBLF *font, const char *str)
 				pen_y = 0;
 			}
 
-			if (font->b_fbuf) {
+			if (buf_info->fbuf) {
 				int yb = yb_start;
 				for (y = ((chy >= 0) ? 0 : -chy); y < height_clip; y++) {
 					for (x = ((chx >= 0) ? 0 : -chx); x < width_clip; x++) {
@@ -285,18 +301,18 @@ void blf_font_buffer(FontBLF *font, const char *str)
 
 						if (a > 0.0f) {
 							float alphatest;
-							fbuf = font->b_fbuf + font->bch * ((chx + x) + ((pen_y + y)*font->bw));
+							fbuf = buf_info->fbuf + buf_info->ch * ((chx + x) + ((pen_y + y) * buf_info->w));
 							if (a >= 1.0f) {
-								fbuf[0] = font->b_col[0];
-								fbuf[1] = font->b_col[1];
-								fbuf[2] = font->b_col[2];
-								fbuf[3] = (alphatest = (fbuf[3] + (font->b_col[3]))) < 1.0f ? alphatest : 1.0f;
+								fbuf[0] = b_col_float[0];
+								fbuf[1] = b_col_float[1];
+								fbuf[2] = b_col_float[2];
+								fbuf[3] = (alphatest = (fbuf[3] + (b_col_float[3]))) < 1.0f ? alphatest : 1.0f;
 							}
 							else {
-								fbuf[0] = (font->b_col[0]*a) + (fbuf[0] * (1-a));
-								fbuf[1] = (font->b_col[1]*a) + (fbuf[1] * (1-a));
-								fbuf[2] = (font->b_col[2]*a) + (fbuf[2] * (1-a));
-								fbuf[3] = (alphatest = (fbuf[3] + (font->b_col[3]*a))) < 1.0f ? alphatest : 1.0f;
+								fbuf[0] = (b_col_float[0] * a) + (fbuf[0] * (1.0f - a));
+								fbuf[1] = (b_col_float[1] * a) + (fbuf[1] * (1.0f - a));
+								fbuf[2] = (b_col_float[2] * a) + (fbuf[2] * (1.0f - a));
+								fbuf[3] = (alphatest = (fbuf[3] + (b_col_float[3] * a))) < 1.0f ? alphatest : 1.0f;
 							}
 						}
 					}
@@ -308,7 +324,7 @@ void blf_font_buffer(FontBLF *font, const char *str)
 				}
 			}
 
-			if (font->b_cbuf) {
+			if (buf_info->cbuf) {
 				int yb = yb_start;
 				for (y = 0; y < height_clip; y++) {
 					for (x = 0; x < width_clip; x++) {
@@ -316,7 +332,7 @@ void blf_font_buffer(FontBLF *font, const char *str)
 
 						if (a > 0.0f) {
 							int alphatest;
-							cbuf = font->b_cbuf + font->bch * ((chx + x) + ((pen_y + y)*font->bw));
+							cbuf = buf_info->cbuf + buf_info->ch * ((chx + x) + ((pen_y + y) * buf_info->w));
 							if (a >= 1.0f) {
 								cbuf[0] = b_col_char[0];
 								cbuf[1] = b_col_char[1];
@@ -324,10 +340,10 @@ void blf_font_buffer(FontBLF *font, const char *str)
 								cbuf[3] = (alphatest = ((int)cbuf[3] + (int)b_col_char[3])) < 255 ? alphatest : 255;
 							}
 							else {
-								cbuf[0] = (b_col_char[0]*a) + (cbuf[0] * (1-a));
-								cbuf[1] = (b_col_char[1]*a) + (cbuf[1] * (1-a));
-								cbuf[2] = (b_col_char[2]*a) + (cbuf[2] * (1-a));
-								cbuf[3] = (alphatest = ((int)cbuf[3] + (int)((font->b_col[3]*a)*255.0f))) <
+								cbuf[0] = (b_col_char[0] * a) + (cbuf[0] * (1.0f - a));
+								cbuf[1] = (b_col_char[1] * a) + (cbuf[1] * (1.0f - a));
+								cbuf[2] = (b_col_char[2] * a) + (cbuf[2] * (1.0f - a));
+								cbuf[3] = (alphatest = ((int)cbuf[3] + (int)((b_col_float[3] * a) * 255.0f))) <
 								          255 ? alphatest : 255;
 							}
 						}
@@ -414,8 +430,8 @@ void blf_font_width_and_height(FontBLF *font, const char *str, float *width, flo
 	}
 
 	blf_font_boundbox(font, str, &box);
-	*width = ((box.xmax - box.xmin) * xa);
-	*height = ((box.ymax - box.ymin) * ya);
+	*width  = (BLI_rctf_size_x(&box) * xa);
+	*height = (BLI_rctf_size_y(&box) * ya);
 }
 
 float blf_font_width(FontBLF *font, const char *str)
@@ -429,7 +445,7 @@ float blf_font_width(FontBLF *font, const char *str)
 		xa = 1.0f;
 
 	blf_font_boundbox(font, str, &box);
-	return (box.xmax - box.xmin) * xa;
+	return BLI_rctf_size_x(&box) * xa;
 }
 
 float blf_font_height(FontBLF *font, const char *str)
@@ -443,7 +459,7 @@ float blf_font_height(FontBLF *font, const char *str)
 		ya = 1.0f;
 
 	blf_font_boundbox(font, str, &box);
-	return (box.ymax - box.ymin) * ya;
+	return BLI_rctf_size_y(&box) * ya;
 }
 
 float blf_font_fixed_width(FontBLF *font)
@@ -507,15 +523,17 @@ static void blf_font_fill(FontBLF *font)
 	font->glyph_cache = NULL;
 	font->blur = 0;
 	font->max_tex_size = -1;
-	font->b_fbuf = NULL;
-	font->b_cbuf = NULL;
-	font->bw = 0;
-	font->bh = 0;
-	font->bch = 0;
-	font->b_col[0] = 0;
-	font->b_col[1] = 0;
-	font->b_col[2] = 0;
-	font->b_col[3] = 0;
+
+	font->buf_info.fbuf = NULL;
+	font->buf_info.cbuf = NULL;
+	font->buf_info.w = 0;
+	font->buf_info.h = 0;
+	font->buf_info.ch = 0;
+	font->buf_info.col[0] = 0;
+	font->buf_info.col[1] = 0;
+	font->buf_info.col[2] = 0;
+	font->buf_info.col[3] = 0;
+
 	font->ft_lib = ft_lib;
 }
 
@@ -560,12 +578,12 @@ void blf_font_attach_from_mem(FontBLF *font, const unsigned char *mem, int mem_s
 	FT_Open_Args open;
 
 	open.flags = FT_OPEN_MEMORY;
-	open.memory_base = (FT_Byte *)mem;
+	open.memory_base = (const FT_Byte *)mem;
 	open.memory_size = mem_size;
 	FT_Attach_Stream(font->face, &open);
 }
 
-FontBLF *blf_font_new_from_mem(const char *name, unsigned char *mem, int mem_size)
+FontBLF *blf_font_new_from_mem(const char *name, const unsigned char *mem, int mem_size)
 {
 	FontBLF *font;
 	FT_Error err;
