@@ -46,7 +46,8 @@
 #include "DNA_vec_types.h"
 #include "DNA_userdef_types.h"
 
-#include "BLI_blenlib.h"
+#include "BLI_listbase.h"
+#include "BLI_rect.h"
 
 #include "BIF_gl.h"
 #include "BLF_api.h"
@@ -82,7 +83,7 @@ GlyphCacheBLF *blf_glyph_cache_new(FontBLF *font)
 	memset(gc->glyph_ascii_table, 0, sizeof(gc->glyph_ascii_table));
 	memset(gc->bucket, 0, sizeof(gc->bucket));
 
-	gc->textures = (GLuint *)malloc(sizeof(GLuint)*256);
+	gc->textures = (GLuint *)MEM_mallocN(sizeof(GLuint) * 256, __func__);
 	gc->ntex = 256;
 	gc->cur_tex = -1;
 	gc->x_offs = 0;
@@ -147,9 +148,9 @@ void blf_glyph_cache_free(GlyphCacheBLF *gc)
 		}
 	}
 
-	if (gc->cur_tex+1 > 0)
-		glDeleteTextures(gc->cur_tex+1, gc->textures);
-	free((void *)gc->textures);
+	if (gc->cur_tex + 1 > 0)
+		glDeleteTextures(gc->cur_tex + 1, gc->textures);
+	MEM_freeN((void *)gc->textures);
 	MEM_freeN(gc);
 }
 
@@ -163,7 +164,7 @@ static void blf_glyph_cache_texture(FontBLF *font, GlyphCacheBLF *gc)
 
 	if (gc->cur_tex >= gc->ntex) {
 		gc->ntex *= 2;
-		gc->textures = (GLuint *)realloc((void *)gc->textures, sizeof(GLuint)*gc->ntex);
+		gc->textures = (GLuint *)realloc((void *)gc->textures, sizeof(GLuint) * gc->ntex);
 	}
 
 	gc->p2_width = blf_next_p2((gc->rem_glyphs * gc->max_glyph_width) + (gc->pad * 2));
@@ -177,8 +178,7 @@ static void blf_glyph_cache_texture(FontBLF *font, GlyphCacheBLF *gc)
 		gc->p2_height = font->max_tex_size;
 
 	tot_mem = gc->p2_width * gc->p2_height;
-	buf = (unsigned char *)malloc(tot_mem);
-	memset((void *)buf, 0, tot_mem);
+	buf = (unsigned char *)MEM_callocN(tot_mem, __func__);
 
 	glGenTextures(1, &gc->textures[gc->cur_tex]);
 	glBindTexture(GL_TEXTURE_2D, (font->tex_bind_state = gc->textures[gc->cur_tex]));
@@ -188,7 +188,7 @@ static void blf_glyph_cache_texture(FontBLF *font, GlyphCacheBLF *gc)
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 
 	glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, gc->p2_width, gc->p2_height, 0, GL_ALPHA, GL_UNSIGNED_BYTE, buf);
-	free((void *)buf);
+	MEM_freeN((void *)buf);
 }
 
 GlyphBLF *blf_glyph_search(GlyphCacheBLF *gc, unsigned int c)
@@ -213,6 +213,7 @@ GlyphBLF *blf_glyph_add(FontBLF *font, unsigned int index, unsigned int c)
 	FT_Error err;
 	FT_Bitmap bitmap, tempbitmap;
 	int sharp = (U.text_render & USER_TEXT_DISABLE_AA);
+	int flags = FT_LOAD_TARGET_NORMAL | FT_LOAD_NO_HINTING | FT_LOAD_NO_BITMAP;
 	FT_BBox bbox;
 	unsigned int key;
 
@@ -220,10 +221,13 @@ GlyphBLF *blf_glyph_add(FontBLF *font, unsigned int index, unsigned int c)
 	if (g)
 		return g;
 
+	if (font->flags & BLF_HINTING)
+		flags &= ~FT_LOAD_NO_HINTING;
+	
 	if (sharp)
 		err = FT_Load_Glyph(font->face, (FT_UInt)index, FT_LOAD_TARGET_MONO);
 	else
-		err = FT_Load_Glyph(font->face, (FT_UInt)index, FT_LOAD_TARGET_NORMAL | FT_LOAD_NO_HINTING | FT_LOAD_NO_BITMAP); /* Sure about NO_* flags? */
+		err = FT_Load_Glyph(font->face, (FT_UInt)index, flags);  
 	if (err)
 		return NULL;
 
@@ -314,13 +318,14 @@ static void blf_texture_draw(float uv[2][2], float dx, float y1, float dx1, floa
 
 static void blf_texture5_draw(const float shadow_col[4], float uv[2][2], float x1, float y1, float x2, float y2)
 {
-	float soft[25] = {1/60.0f, 1/60.0f, 2/60.0f, 1/60.0f, 1/60.0f,
-	                  1/60.0f, 3/60.0f, 5/60.0f, 3/60.0f, 1/60.0f,
-	                  2/60.0f, 5/60.0f, 8/60.0f, 5/60.0f, 2/60.0f,
-	                  1/60.0f, 3/60.0f, 5/60.0f, 3/60.0f, 1/60.0f,
-	                  1/60.0f, 1/60.0f, 2/60.0f, 1/60.0f, 1/60.0f};
+	const float soft[25] = {1 / 60.0f, 1 / 60.0f, 2 / 60.0f, 1 / 60.0f, 1 / 60.0f,
+	                        1 / 60.0f, 3 / 60.0f, 5 / 60.0f, 3 / 60.0f, 1 / 60.0f,
+	                        2 / 60.0f, 5 / 60.0f, 8 / 60.0f, 5 / 60.0f, 2 / 60.0f,
+	                        1 / 60.0f, 3 / 60.0f, 5 / 60.0f, 3 / 60.0f, 1 / 60.0f,
+	                        1 / 60.0f, 1 / 60.0f, 2 / 60.0f, 1 / 60.0f, 1 / 60.0f};
 	
-	float color[4], *fp = soft;
+	const float *fp = soft;
+	float color[4];
 	int dx, dy;
 
 	color[0] = shadow_col[0];
@@ -331,7 +336,7 @@ static void blf_texture5_draw(const float shadow_col[4], float uv[2][2], float x
 		for (dy = -2; dy < 3; dy++, fp++) {
 			color[3] = *(fp) * shadow_col[3];
 			glColor4fv(color);
-			blf_texture_draw(uv, x1+dx, y1+dy, x2+dx, y2+dy);
+			blf_texture_draw(uv, x1 + dx, y1 + dy, x2 + dx, y2 + dy);
 		}
 	}
 	
@@ -340,11 +345,12 @@ static void blf_texture5_draw(const float shadow_col[4], float uv[2][2], float x
 
 static void blf_texture3_draw(const float shadow_col[4], float uv[2][2], float x1, float y1, float x2, float y2)
 {
-	float soft[9] = {1/16.0f, 2/16.0f, 1/16.0f,
-	                 2/16.0f,4/16.0f, 2/16.0f,
-	                 1/16.0f, 2/16.0f, 1/16.0f};
+	const float soft[9] = {1 / 16.0f, 2 / 16.0f, 1 / 16.0f,
+	                       2 / 16.0f, 4 / 16.0f, 2 / 16.0f,
+	                       1 / 16.0f, 2 / 16.0f, 1 / 16.0f};
 
-	float color[4], *fp = soft;
+	const float *fp = soft;
+	float color[4];
 	int dx, dy;
 
 	color[0] = shadow_col[0];
@@ -355,7 +361,7 @@ static void blf_texture3_draw(const float shadow_col[4], float uv[2][2], float x
 		for (dy = -1; dy < 2; dy++, fp++) {
 			color[3] = *(fp) * shadow_col[3];
 			glColor4fv(color);
-			blf_texture_draw(uv, x1+dx, y1+dy, x2+dx, y2+dy);
+			blf_texture_draw(uv, x1 + dx, y1 + dy, x2 + dx, y2 + dy);
 		}
 	}
 	
@@ -380,7 +386,7 @@ int blf_glyph_render(FontBLF *font, GlyphBLF *g, float x, float y)
 		if (gc->cur_tex == -1) {
 			blf_glyph_cache_texture(font, gc);
 			gc->x_offs = gc->pad;
-			gc->y_offs = gc->pad;
+			gc->y_offs = 0;
 		}
 
 		if (gc->x_offs > (gc->p2_width - gc->max_glyph_width)) {
@@ -388,7 +394,7 @@ int blf_glyph_render(FontBLF *font, GlyphBLF *g, float x, float y)
 			gc->y_offs += gc->max_glyph_height;
 
 			if (gc->y_offs > (gc->p2_height - gc->max_glyph_height)) {
-				gc->y_offs = gc->pad;
+				gc->y_offs = 0;
 				blf_glyph_cache_texture(font, gc);
 			}
 		}
@@ -397,6 +403,19 @@ int blf_glyph_render(FontBLF *font, GlyphBLF *g, float x, float y)
 		g->xoff = gc->x_offs;
 		g->yoff = gc->y_offs;
 
+		/* prevent glTexSubImage2D from failing if the character
+		 * asks for pixels out of bounds, this tends only to happen
+		 * with very small sizes (5px high or less) */
+		if (UNLIKELY((g->xoff + g->width)  > gc->p2_width)) {
+			g->width  -= (g->xoff + g->width)  - gc->p2_width;
+			BLI_assert(g->width > 0);
+		}
+		if (UNLIKELY((g->yoff + g->height) > gc->p2_height)) {
+			g->height -= (g->yoff + g->height) - gc->p2_height;
+			BLI_assert(g->height > 0);
+		}
+
+
 		glPushClientAttrib(GL_CLIENT_PIXEL_STORE_BIT);
 		glPixelStorei(GL_UNPACK_LSB_FIRST, GL_FALSE);
 		glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
@@ -412,7 +431,7 @@ int blf_glyph_render(FontBLF *font, GlyphBLF *g, float x, float y)
 		g->uv[1][1] = ((float)(g->yoff + g->height)) / ((float)gc->p2_height);
 
 		/* update the x offset for the next glyph. */
-		gc->x_offs += (int)(g->box.xmax - g->box.xmin + gc->pad);
+		gc->x_offs += (int)(BLI_rctf_size_x(&g->box) + gc->pad);
 
 		gc->rem_glyphs--;
 		g->build_tex = 1;
@@ -434,13 +453,13 @@ int blf_glyph_render(FontBLF *font, GlyphBLF *g, float x, float y)
 	y2 = y + g->pos_y - g->height;
 
 	if (font->flags & BLF_CLIPPING) {
-		if (!BLI_in_rctf(&font->clip_rec, dx + font->pos[0], y1 + font->pos[1]))
+		if (!BLI_rctf_isect_pt(&font->clip_rec, dx + font->pos[0], y1 + font->pos[1]))
 			return 0;
-		if (!BLI_in_rctf(&font->clip_rec, dx + font->pos[0], y2 + font->pos[1]))
+		if (!BLI_rctf_isect_pt(&font->clip_rec, dx + font->pos[0], y2 + font->pos[1]))
 			return 0;
-		if (!BLI_in_rctf(&font->clip_rec, dx1 + font->pos[0], y2 + font->pos[1]))
+		if (!BLI_rctf_isect_pt(&font->clip_rec, dx1 + font->pos[0], y2 + font->pos[1]))
 			return 0;
-		if (!BLI_in_rctf(&font->clip_rec, dx1 + font->pos[0], y1 + font->pos[1]))
+		if (!BLI_rctf_isect_pt(&font->clip_rec, dx1 + font->pos[0], y1 + font->pos[1]))
 			return 0;
 	}
 
