@@ -47,12 +47,12 @@ int blf_font_init(void);
 void blf_font_exit(void);
 
 struct FontBLF *blf_font_new(const char *name, const char *filename);
-struct FontBLF *blf_font_new_from_mem(const char *name, unsigned char *mem, int mem_size);
+struct FontBLF *blf_font_new_from_mem(const char *name, const unsigned char *mem, int mem_size);
 void blf_font_attach_from_mem(struct FontBLF *font, const unsigned char *mem, int mem_size);
 
 void blf_font_size(struct FontBLF *font, int size, int dpi);
-void blf_font_draw(struct FontBLF *font, const char *str, unsigned int len);
-void blf_font_draw_ascii(struct FontBLF *font, const char *str, unsigned int len);
+void blf_font_draw(struct FontBLF *font, const char *str, size_t len);
+void blf_font_draw_ascii(struct FontBLF *font, const char *str, size_t len);
 void blf_font_buffer(struct FontBLF *font, const char *str);
 void blf_font_boundbox(struct FontBLF *font, const char *str, struct rctf *box);
 void blf_font_width_and_height(struct FontBLF *font, const char *str, float *width, float *height);
