@@ -55,8 +55,10 @@ int CDDM_Check(struct DerivedMesh *dm);
  * data to not overwrite the original */
 struct DerivedMesh *CDDM_from_mesh(struct Mesh *mesh, struct Object *ob);
 
+struct DerivedMesh *CDDM_from_bmesh(struct BMesh *bm, int use_mdisps);
+
 /* creates a CDDerivedMesh from the given BMEditMesh */
-DerivedMesh *CDDM_from_BMEditMesh(struct BMEditMesh *em, struct Mesh *me, int use_mdisps, int use_tessface);
+DerivedMesh *CDDM_from_editbmesh(struct BMEditMesh *em, int use_mdisps, int use_tessface);
 
 /* merge verts  */
 DerivedMesh *CDDM_merge_verts(DerivedMesh *dm, const int *vtargetmap);
@@ -66,7 +68,7 @@ struct DerivedMesh *CDDM_from_curve(struct Object *ob);
 
 /* creates a CDDerivedMesh from the given curve object and specified dispbase */
 /* useful for OrcoDM creation for curves with constructive modifiers */
-DerivedMesh *CDDM_from_curve_customDB(struct Object *ob, struct ListBase *dispbase);
+DerivedMesh *CDDM_from_curve_displist(struct Object *ob, struct ListBase *dispbase);
 
 /* Copies the given DerivedMesh with verts, faces & edges stored as
  * custom element data.
@@ -79,11 +81,11 @@ struct DerivedMesh *CDDM_copy_from_tessface(struct DerivedMesh *dm);
  * elements are initialized to all zeros
  */
 struct DerivedMesh *CDDM_from_template(struct DerivedMesh *source,
-                                  int numVerts, int numEdges, int numFaces,
-                                  int numLoops, int numPolys);
+                                       int numVerts, int numEdges, int numFaces,
+                                       int numLoops, int numPolys);
 
-/*converts mfaces to mpolys.  note things may break if there are not valid
- *medges surrounding each mface.
+/* converts mfaces to mpolys.  note things may break if there are not valid
+ * medges surrounding each mface.
  */
 void CDDM_tessfaces_to_faces(struct DerivedMesh *dm);
 
