@@ -72,7 +72,7 @@ struct bContextDataResult;
 typedef struct bContextDataResult bContextDataResult;
 
 typedef int (*bContextDataCallback)(const bContext *C,
-	const char *member, bContextDataResult *result);
+                                    const char *member, bContextDataResult *result);
 
 typedef struct bContextStoreEntry {
 	struct bContextStoreEntry *next, *prev;
@@ -88,7 +88,7 @@ typedef struct bContextStore {
 	int used;
 } bContextStore;
 
-/* for the conrtext's rna mode enum
+/* for the context's rna mode enum
  * keep aligned with data_mode_strings in context.c */
 enum {
 	CTX_MODE_EDIT_MESH = 0,
@@ -174,7 +174,7 @@ void CTX_wm_operator_poll_msg_set(struct bContext *C, const char *msg);
  *
  * - listbases consist of CollectionPointerLink items and must be
  *   freed with BLI_freelistN!
- * - the dir listbase consits of LinkData items */
+ * - the dir listbase consists of LinkData items */
 
 /* data type, needed so we can tell between a NULL pointer and an empty list */
 enum {
@@ -185,6 +185,7 @@ enum {
 PointerRNA CTX_data_pointer_get(const bContext *C, const char *member);
 PointerRNA CTX_data_pointer_get_type(const bContext *C, const char *member, StructRNA *type);
 ListBase CTX_data_collection_get(const bContext *C, const char *member);
+ListBase CTX_data_dir_get_ex(const bContext *C, const short use_store, const short use_rna, const short use_all);
 ListBase CTX_data_dir_get(const bContext *C);
 int CTX_data_get(const bContext *C, const char *member, PointerRNA *r_ptr, ListBase *r_lb, short *r_type);
 
@@ -221,9 +222,9 @@ void CTX_data_list_add(bContextDataResult *result, void *data);
 #define CTX_DATA_END                                                          \
 		}                                                                     \
 		BLI_freelistN(&ctx_data_list);                                        \
-	}
+} (void)0
 
-int ctx_data_list_count(const bContext *C, int (*func)(const bContext*, ListBase*));
+int ctx_data_list_count(const bContext *C, int (*func)(const bContext *, ListBase *));
 
 #define CTX_DATA_COUNT(C, member) \
 	ctx_data_list_count(C, CTX_data_##member)
@@ -260,6 +261,7 @@ struct Image *CTX_data_edit_image(const bContext *C);
 
 struct Text *CTX_data_edit_text(const bContext *C);
 struct MovieClip *CTX_data_edit_movieclip(const bContext *C);
+struct Mask *CTX_data_edit_mask(const bContext *C);
 
 int CTX_data_selected_nodes(const bContext *C, ListBase *list);
 
