@@ -40,20 +40,25 @@ struct ListBase;
 struct bDeformGroup;
 struct MDeformVert;
 
-void				 defgroup_copy_list(struct ListBase *lb1, struct ListBase *lb2);
+void                 defgroup_copy_list(struct ListBase *lb1, struct ListBase *lb2);
 struct bDeformGroup *defgroup_duplicate(struct bDeformGroup *ingroup);
 struct bDeformGroup *defgroup_find_name(struct Object *ob, const char *name);
-int					*defgroup_flip_map(struct Object *ob, int *flip_map_len, int use_default);
-int					*defgroup_flip_map_single(struct Object *ob, int *flip_map_len, int use_default, int defgroup);
-int					 defgroup_flip_index(struct Object *ob, int index, int use_default);
-int					 defgroup_name_index(struct Object *ob, const char *name);
-void				 defgroup_unique_name(struct bDeformGroup *dg, struct Object *ob);
+int                 *defgroup_flip_map(struct Object *ob, int *flip_map_len, int use_default);
+int                 *defgroup_flip_map_single(struct Object *ob, int *flip_map_len, int use_default, int defgroup);
+int                  defgroup_flip_index(struct Object *ob, int index, int use_default);
+int                  defgroup_name_index(struct Object *ob, const char *name);
+void                 defgroup_unique_name(struct bDeformGroup *dg, struct Object *ob);
 
-struct MDeformWeight	*defvert_find_index(const struct MDeformVert *dv, const int defgroup);
-struct MDeformWeight	*defvert_verify_index(struct MDeformVert *dv, const int defgroup);
+struct MDeformWeight    *defvert_find_index(const struct MDeformVert *dv, const int defgroup);
+struct MDeformWeight    *defvert_verify_index(struct MDeformVert *dv, const int defgroup);
 void                     defvert_add_index_notest(struct MDeformVert *dv, int defgroup, const float weight);
 void                     defvert_remove_group(struct MDeformVert *dvert, struct MDeformWeight *dw);
 void                     defvert_clear(struct MDeformVert *dvert);
+int                      defvert_find_shared(const struct MDeformVert *dvert_a, const struct MDeformVert *dvert_b);
+
+void BKE_defvert_array_free_elems(struct MDeformVert *dvert, int totvert);
+void BKE_defvert_array_free(struct MDeformVert *dvert, int totvert);
+void BKE_defvert_array_copy(struct MDeformVert *dst, const struct MDeformVert *src, int totvert);
 
 float  defvert_find_weight(const struct MDeformVert *dvert, const int defgroup);
 float  defvert_array_find_weight_safe(const struct MDeformVert *dvert, const int index, const int defgroup);
@@ -63,15 +68,20 @@ void defvert_copy_index(struct MDeformVert *dvert_dst, const struct MDeformVert
 void defvert_sync(struct MDeformVert *dvert_dst, const struct MDeformVert *dvert_src, int use_verify);
 void defvert_sync_mapped(struct MDeformVert *dvert_dst, const struct MDeformVert *dvert_src,
                          const int *flip_map, const int flip_map_len, const int use_verify);
-void defvert_remap (struct MDeformVert *dvert, int *map, const int map_len);
+void defvert_remap(struct MDeformVert *dvert, int *map, const int map_len);
 void defvert_flip(struct MDeformVert *dvert, const int *flip_map, const int flip_map_len);
 void defvert_flip_merged(struct MDeformVert *dvert, const int *flip_map, const int flip_map_len);
 void defvert_normalize(struct MDeformVert *dvert);
-void defvert_normalize_lock(struct MDeformVert *dvert, const int def_nr_lock);
+void defvert_normalize_lock_single(struct MDeformVert *dvert, const int def_nr_lock);
+void defvert_normalize_lock_map(struct MDeformVert *dvert, const char *lock_flags, const int defbase_tot);
 
-/* utility function, note that 32 chars is the maximum string length since its only
+/* utility function, note that MAX_VGROUP_NAME chars is the maximum string length since its only
  * used with defgroups currently */
-void flip_side_name(char name[64], const char from_name[64], int strip_number);
+
+void BKE_deform_split_suffix(const char string[MAX_VGROUP_NAME], char base[MAX_VGROUP_NAME], char ext[MAX_VGROUP_NAME]);
+void BKE_deform_split_prefix(const char string[MAX_VGROUP_NAME], char base[MAX_VGROUP_NAME], char ext[MAX_VGROUP_NAME]);
+
+void flip_side_name(char name[MAX_VGROUP_NAME], const char from_name[MAX_VGROUP_NAME], int strip_number);
 
 #endif
 
