@@ -60,22 +60,33 @@ typedef struct Global {
 
 	/* strings of recent opened files */
 	struct ListBase recent_files;
-        
-	short afbreek, moving, file_loaded;
+
+	/* has escape been pressed or Ctrl+C pressed in background mode, used for render quit */
+	short is_break;
+
+	short moving, file_loaded;
 	char background;
 	char factory_startup;
-	short winpos, displaymode;	/* used to be in Render */
-	short rendering;			/* to indicate render is busy, prevent renderwindow events etc */
+	short winpos, displaymode;  /* used to be in Render */
 
-	short rt;
+	/* to indicate render is busy, prevent renderwindow events etc */
+	short is_rendering;
+
+	/* debug value, can be set from the UI and python, used for testing nonstandard features */
+	short debug_value;
+
+	/* saved to the blend file as FileGlobal.globalf,
+	 * however this is now only used for runtime options */
 	int f;
+
+	/* debug flag, G_DEBUG, G_DEBUG_PYTHON & friends, set python or command line args */
 	int debug;
 
 	/* Used for BMesh transformations */
 	struct BME_Glob *editBMesh;
-    
+
 	/* Frank's variables */
-	int	save_over;
+	int save_over;
 
 	/* Rob's variables (keep here for WM recode) */
 	int have_quicktime;
@@ -88,7 +99,7 @@ typedef struct Global {
 
 	/* this variable is written to / read from FileGlobal->fileflags */
 	int fileflags;
-    
+
 	/* save the allowed windowstate of blender when using -W or -w */
 	int windowstate;
 } Global;
@@ -96,20 +107,20 @@ typedef struct Global {
 /* **************** GLOBAL ********************* */
 
 /* G.f */
-#define G_RENDER_OGL	(1 <<  0)
-#define G_SWAP_EXCHANGE	(1 <<  1)
+#define G_RENDER_OGL    (1 <<  0)
+#define G_SWAP_EXCHANGE (1 <<  1)
 /* also uses G_FILE_AUTOPLAY */
 /* #define G_RENDER_SHADOW	(1 <<  3) */ /* temp flag, removed */
-#define G_BACKBUFSEL	(1 <<  4)
-#define G_PICKSEL		(1 <<  5)
+#define G_BACKBUFSEL    (1 <<  4)
+#define G_PICKSEL       (1 <<  5)
 
-/* #define G_FACESELECT	(1 <<  8) use (mesh->editflag & ME_EDIT_PAINT_MASK) */
+/* #define G_FACESELECT	(1 <<  8) use (mesh->editflag & ME_EDIT_PAINT_FACE_SEL) */
 
 #define G_SCRIPT_AUTOEXEC (1 << 13)
 #define G_SCRIPT_OVERRIDE_PREF (1 << 14) /* when this flag is set ignore the userprefs */
 
 /* #define G_NOFROZEN	(1 << 17) also removed */
-/* #define G_GREASEPENCIL 	(1 << 17)   also removed */
+/* #define G_GREASEPENCIL   (1 << 17)   also removed */
 
 /* #define G_AUTOMATKEYS	(1 << 30)   also removed */
 
@@ -119,10 +130,12 @@ enum {
 	G_DEBUG_FFMPEG =    (1 << 1),
 	G_DEBUG_PYTHON =    (1 << 2), /* extra python info */
 	G_DEBUG_EVENTS =    (1 << 3), /* input/window/screen events */
-	G_DEBUG_WM =        (1 << 4)  /* operator, undo */
+	G_DEBUG_HANDLERS =  (1 << 4), /* events handling */
+	G_DEBUG_WM =        (1 << 5), /* operator, undo */
+	G_DEBUG_JOBS =      (1 << 6)  /* jobs time profiling */
 };
 
-#define G_DEBUG_ALL  (G_DEBUG | G_DEBUG_FFMPEG | G_DEBUG_PYTHON | G_DEBUG_EVENTS | G_DEBUG_WM)
+#define G_DEBUG_ALL  (G_DEBUG | G_DEBUG_FFMPEG | G_DEBUG_PYTHON | G_DEBUG_EVENTS | G_DEBUG_WM | G_DEBUG_JOBS)
 
 
 /* G.fileflags */
@@ -130,39 +143,42 @@ enum {
 #define G_AUTOPACK               (1 << 0)
 #define G_FILE_COMPRESS          (1 << 1)
 #define G_FILE_AUTOPLAY          (1 << 2)
-#define G_FILE_ENABLE_ALL_FRAMES (1 << 3)				/* deprecated */
-#define G_FILE_SHOW_DEBUG_PROPS  (1 << 4)				/* deprecated */
-#define G_FILE_SHOW_FRAMERATE    (1 << 5)				/* deprecated */
-/* #define G_FILE_SHOW_PROFILE   (1 << 6) */			/* deprecated */
+#define G_FILE_ENABLE_ALL_FRAMES (1 << 3)               /* deprecated */
+#define G_FILE_SHOW_DEBUG_PROPS  (1 << 4)               /* deprecated */
+#define G_FILE_SHOW_FRAMERATE    (1 << 5)               /* deprecated */
+/* #define G_FILE_SHOW_PROFILE   (1 << 6) */            /* deprecated */
 #define G_FILE_LOCK              (1 << 7)
 #define G_FILE_SIGN              (1 << 8)
-/* #define G_FILE_PUBLISH	     (1 << 9) */			/* deprecated */
-#define G_FILE_NO_UI			 (1 << 10)
-/* #define G_FILE_GAME_TO_IPO	 (1 << 11) */			/* deprecated */
-#define G_FILE_GAME_MAT			 (1 << 12)				/* deprecated */
-/* #define G_FILE_DISPLAY_LISTS	 (1 << 13) */			/* deprecated */
-#define G_FILE_SHOW_PHYSICS		 (1 << 14)				/* deprecated */
-#define G_FILE_GAME_MAT_GLSL	 (1 << 15)				/* deprecated */
-/* #define G_FILE_GLSL_NO_LIGHTS	 (1 << 16) */		/* deprecated */
-#define G_FILE_GLSL_NO_SHADERS	 (1 << 17)				/* deprecated */
-#define G_FILE_GLSL_NO_SHADOWS	 (1 << 18)				/* deprecated */
-#define G_FILE_GLSL_NO_RAMPS	 (1 << 19)				/* deprecated */
-#define G_FILE_GLSL_NO_NODES	 (1 << 20)				/* deprecated */
-#define G_FILE_GLSL_NO_EXTRA_TEX (1 << 21)				/* deprecated */
-#define G_FILE_IGNORE_DEPRECATION_WARNINGS	(1 << 22)	/* deprecated */
-#define G_FILE_RECOVER			 (1 << 23)
-#define G_FILE_RELATIVE_REMAP	 (1 << 24)
-#define G_FILE_HISTORY			 (1 << 25)
-#define G_FILE_MESH_COMPAT		 (1 << 26)				/* BMesh option to save as older mesh format */
+#define G_FILE_USERPREFS         (1 << 9)
+#define G_FILE_NO_UI             (1 << 10)
+/* #define G_FILE_GAME_TO_IPO    (1 << 11) */           /* deprecated */
+#define G_FILE_GAME_MAT          (1 << 12)              /* deprecated */
+/* #define G_FILE_DISPLAY_LISTS  (1 << 13) */           /* deprecated */
+#define G_FILE_SHOW_PHYSICS      (1 << 14)              /* deprecated */
+#define G_FILE_GAME_MAT_GLSL     (1 << 15)              /* deprecated */
+/* #define G_FILE_GLSL_NO_LIGHTS     (1 << 16) */       /* deprecated */
+#define G_FILE_GLSL_NO_SHADERS   (1 << 17)              /* deprecated */
+#define G_FILE_GLSL_NO_SHADOWS   (1 << 18)              /* deprecated */
+#define G_FILE_GLSL_NO_RAMPS     (1 << 19)              /* deprecated */
+#define G_FILE_GLSL_NO_NODES     (1 << 20)              /* deprecated */
+#define G_FILE_GLSL_NO_EXTRA_TEX (1 << 21)              /* deprecated */
+#define G_FILE_IGNORE_DEPRECATION_WARNINGS  (1 << 22)   /* deprecated */
+#define G_FILE_RECOVER           (1 << 23)
+#define G_FILE_RELATIVE_REMAP    (1 << 24)
+#define G_FILE_HISTORY           (1 << 25)
+#define G_FILE_MESH_COMPAT       (1 << 26)              /* BMesh option to save as older mesh format */
+#define G_FILE_SAVE_COPY         (1 << 27)              /* restore paths after editing them */
+
+#define G_FILE_FLAGS_RUNTIME (G_FILE_NO_UI | G_FILE_RELATIVE_REMAP | G_FILE_MESH_COMPAT | G_FILE_SAVE_COPY)
 
 /* G.windowstate */
-#define G_WINDOWSTATE_USERDEF		0
-#define G_WINDOWSTATE_BORDER		1
-#define G_WINDOWSTATE_FULLSCREEN	2
+#define G_WINDOWSTATE_USERDEF       0
+#define G_WINDOWSTATE_BORDER        1
+#define G_WINDOWSTATE_FULLSCREEN    2
 
 /* ENDIAN_ORDER: indicates what endianness the platform where the file was
  * written had. */
-#if !defined( __BIG_ENDIAN__ ) && !defined( __LITTLE_ENDIAN__ )
+#if !defined(__BIG_ENDIAN__) && !defined(__LITTLE_ENDIAN__)
 #  error Either __BIG_ENDIAN__ or __LITTLE_ENDIAN__ must be defined.
 #endif
 
@@ -171,8 +187,8 @@ enum {
 #  error "Mingw requires GCC 4.6 minimum"
 #endif
 
-#define L_ENDIAN	1
-#define B_ENDIAN	0
+#define L_ENDIAN    1
+#define B_ENDIAN    0
 
 #ifdef __BIG_ENDIAN__
 #  define ENDIAN_ORDER B_ENDIAN
@@ -181,10 +197,10 @@ enum {
 #endif
 
 /* G.moving, signals drawing in (3d) window to denote transform */
-#define G_TRANSFORM_OBJ			1
-#define G_TRANSFORM_EDIT		2
-#define G_TRANSFORM_MANIP		4
-#define G_TRANSFORM_PARTICLE	8
+#define G_TRANSFORM_OBJ         1
+#define G_TRANSFORM_EDIT        2
+#define G_TRANSFORM_MANIP       4
+#define G_TRANSFORM_PARTICLE    8
 
 /* G.special1 */
 
