@@ -32,9 +32,13 @@
  *  \ingroup bke
  */
 
+struct bContext;
+struct BMesh;
+struct BMFace;
 struct Brush;
 struct MDisps;
 struct MeshElemMap;
+struct GridPaintMask;
 struct MFace;
 struct MultireModifierData;
 struct MVert;
@@ -43,17 +47,20 @@ struct Paint;
 struct PBVH;
 struct Scene;
 struct StrokeCache;
+struct ImagePool;
 
 extern const char PAINT_CURSOR_SCULPT[3];
 extern const char PAINT_CURSOR_VERTEX_PAINT[3];
 extern const char PAINT_CURSOR_WEIGHT_PAINT[3];
 extern const char PAINT_CURSOR_TEXTURE_PAINT[3];
 
-void paint_init(struct Paint *p, const char col[3]);
-void free_paint(struct Paint *p);
-void copy_paint(struct Paint *src, struct Paint *tar);
+void BKE_paint_init(struct Paint *p, const char col[3]);
+void BKE_paint_free(struct Paint *p);
+void BKE_paint_copy(struct Paint *src, struct Paint *tar);
 
+/* TODO, give these BKE_ prefix too */
 struct Paint *paint_get_active(struct Scene *sce);
+struct Paint *paint_get_active_from_context(const struct bContext *C);
 struct Brush *paint_brush(struct Paint *paint);
 void paint_brush_set(struct Paint *paint, struct Brush *br);
 
@@ -66,7 +73,12 @@ int paint_vertsel_test(struct Object *ob);
 /* partial visibility */
 int paint_is_face_hidden(const struct MFace *f, const struct MVert *mvert);
 int paint_is_grid_face_hidden(const unsigned int *grid_hidden,
-							  int gridsize, int x, int y);
+                              int gridsize, int x, int y);
+int paint_is_bmesh_face_hidden(struct BMFace *f);
+
+/* paint masks */
+float paint_grid_paint_mask(const struct GridPaintMask *gpm, unsigned level,
+                            unsigned x, unsigned y);
 
 /* Session data (mode-specific) */
 
@@ -79,12 +91,20 @@ typedef struct SculptSession {
 	int totvert, totpoly;
 	float *face_normals;
 	struct KeyBlock *kb;
+	float *vmask;
 	
 	/* Mesh connectivity */
 	const struct MeshElemMap *pmap;
 
+	/* BMesh for dynamic topology sculpting */
+	struct BMesh *bm;
+	int bm_smooth_shading;
+	/* Undo/redo log for dynamic topology sculpting */
+	struct BMLog *bm_log;
+
 	/* PBVH acceleration structure */
 	struct PBVH *pbvh;
+	int show_diffuse_color;
 
 	/* Paiting on deformed mesh */
 	int modifiers_active; /* object is deformed with some modifiers */
@@ -97,15 +117,21 @@ typedef struct SculptSession {
 	
 	/* Used to cache the render of the active texture */
 	unsigned int texcache_side, *texcache, texcache_actual;
+	struct ImagePool *tex_pool;
 
 	/* Layer brush persistence between strokes */
 	float (*layer_co)[3]; /* Copy of the mesh vertices' locations */
 
 	struct SculptStroke *stroke;
 	struct StrokeCache *cache;
+
+	/* last paint/sculpt stroke location */
+	int last_stroke_valid;
+	float last_stroke[3];
 } SculptSession;
 
 void free_sculptsession(struct Object *ob);
 void free_sculptsession_deformMats(struct SculptSession *ss);
+void sculptsession_bm_to_me(struct Object *ob, int reorder);
 
 #endif
