@@ -34,7 +34,10 @@
 extern "C" {
 #endif
 
+#include <stdio.h>
+
 #include "DNA_windowmanager_types.h"
+#include "BLI_utildefines.h"
 
 /* Reporting Information and Errors
  *
@@ -49,14 +52,14 @@ void BKE_reports_clear(ReportList *reports);
 void BKE_report(ReportList *reports, ReportType type, const char *message);
 void BKE_reportf(ReportList *reports, ReportType type, const char *format, ...)
 #ifdef __GNUC__
-__attribute__ ((format (printf, 3, 4)))
+__attribute__ ((format(printf, 3, 4)))
 #endif
 ;
 
 void BKE_reports_prepend(ReportList *reports, const char *prepend);
 void BKE_reports_prependf(ReportList *reports, const char *prepend, ...)
 #ifdef __GNUC__
-__attribute__ ((format (printf, 2, 3)))
+__attribute__ ((format(printf, 2, 3)))
 #endif
 ;
 
@@ -72,7 +75,10 @@ void BKE_reports_print(ReportList *reports, ReportType level);
 Report *BKE_reports_last_displayable(ReportList *reports);
 
 int BKE_reports_contain(ReportList *reports, ReportType level);
-	
+
+bool BKE_report_write_file_fp(FILE *fp, ReportList *reports, const char *header);
+bool BKE_report_write_file(const char *filepath, ReportList *reports, const char *header);
+
 #ifdef __cplusplus
 }
 #endif
