@@ -41,8 +41,7 @@ struct ListBase;
 struct Main;
 struct Sequence;
 
-typedef struct SoundWaveform
-{
+typedef struct SoundWaveform {
 	int length;
 	float *data;
 } SoundWaveform;
@@ -58,29 +57,29 @@ void sound_exit(void);
 void sound_force_device(int device);
 int sound_define_from_str(const char *str);
 
-struct bSound* sound_new_file(struct Main *main, const char *filename);
+struct bSound *sound_new_file(struct Main *main, const char *filename);
 
 // XXX unused currently
 #if 0
-struct bSound* sound_new_buffer(struct Main *bmain, struct bSound *source);
+struct bSound *sound_new_buffer(struct Main *bmain, struct bSound *source);
 
-struct bSound* sound_new_limiter(struct Main *bmain, struct bSound *source, float start, float end);
+struct bSound *sound_new_limiter(struct Main *bmain, struct bSound *source, float start, float end);
 #endif
 
-void sound_delete(struct Main *bmain, struct bSound* sound);
+void sound_delete(struct Main *bmain, struct bSound *sound);
 
-void sound_cache(struct bSound* sound);
+void sound_cache(struct bSound *sound);
 
-void sound_cache_notifying(struct Main* main, struct bSound* sound);
+void sound_cache_notifying(struct Main *main, struct bSound *sound);
 
-void sound_delete_cache(struct bSound* sound);
+void sound_delete_cache(struct bSound *sound);
 
-void sound_load(struct Main *main, struct bSound* sound);
+void sound_load(struct Main *main, struct bSound *sound);
 
-void sound_free(struct bSound* sound);
+void BKE_sound_free(struct bSound *sound);
 
 #ifdef __AUD_C_API_H__
-AUD_Device* sound_mixdown(struct Scene *scene, AUD_DeviceSpecs specs, int start, float volume);
+AUD_Device *sound_mixdown(struct Scene *scene, AUD_DeviceSpecs specs, int start, float volume);
 #endif
 
 void sound_create_scene(struct Scene *scene);
@@ -93,32 +92,32 @@ void sound_update_fps(struct Scene *scene);
 
 void sound_update_scene_listener(struct Scene *scene);
 
-void* sound_scene_add_scene_sound(struct Scene *scene, struct Sequence* sequence, int startframe, int endframe, int frameskip);
-void* sound_scene_add_scene_sound_defaults(struct Scene *scene, struct Sequence* sequence);
+void *sound_scene_add_scene_sound(struct Scene *scene, struct Sequence *sequence, int startframe, int endframe, int frameskip);
+void *sound_scene_add_scene_sound_defaults(struct Scene *scene, struct Sequence *sequence);
 
-void* sound_add_scene_sound(struct Scene *scene, struct Sequence* sequence, int startframe, int endframe, int frameskip);
-void* sound_add_scene_sound_defaults(struct Scene *scene, struct Sequence* sequence);
+void *sound_add_scene_sound(struct Scene *scene, struct Sequence *sequence, int startframe, int endframe, int frameskip);
+void *sound_add_scene_sound_defaults(struct Scene *scene, struct Sequence *sequence);
 
-void sound_remove_scene_sound(struct Scene *scene, void* handle);
+void sound_remove_scene_sound(struct Scene *scene, void *handle);
 
-void sound_mute_scene_sound(void* handle, char mute);
+void sound_mute_scene_sound(void *handle, char mute);
 
-void sound_move_scene_sound(struct Scene *scene, void* handle, int startframe, int endframe, int frameskip);
+void sound_move_scene_sound(struct Scene *scene, void *handle, int startframe, int endframe, int frameskip);
 void sound_move_scene_sound_defaults(struct Scene *scene, struct Sequence *sequence);
 
-void sound_update_scene_sound(void* handle, struct bSound* sound);
+void sound_update_scene_sound(void *handle, struct bSound *sound);
 
 void sound_set_cfra(int cfra);
 
 void sound_set_scene_volume(struct Scene *scene, float volume);
 
-void sound_set_scene_sound_volume(void* handle, float volume, char animated);
+void sound_set_scene_sound_volume(void *handle, float volume, char animated);
 
-void sound_set_scene_sound_pitch(void* handle, float pitch, char animated);
+void sound_set_scene_sound_pitch(void *handle, float pitch, char animated);
 
-void sound_set_scene_sound_pan(void* handle, float pan, char animated);
+void sound_set_scene_sound_pan(void *handle, float pan, char animated);
 
-void sound_update_sequencer(struct Main* main, struct bSound* sound);
+void sound_update_sequencer(struct Main *main, struct bSound *sound);
 
 void sound_play_scene(struct Scene *scene);
 
@@ -130,12 +129,14 @@ float sound_sync_scene(struct Scene *scene);
 
 int sound_scene_playing(struct Scene *scene);
 
-void sound_free_waveform(struct bSound* sound);
+void sound_free_waveform(struct bSound *sound);
 
-void sound_read_waveform(struct bSound* sound);
+void sound_read_waveform(struct bSound *sound);
 
-void sound_update_scene(struct Scene* scene);
+void sound_update_scene(struct Scene *scene);
 
-void* sound_get_factory(void* sound);
+void *sound_get_factory(void *sound);
+
+float sound_get_length(struct bSound *sound);
 
 #endif
