@@ -65,8 +65,8 @@ void printfGlyph(bmGlyph * glyph)
 	printf(" advan: %3d reser: %3d\n", glyph->advance, glyph->reserved);
 }
 
-#define MAX2(x,y)          ( (x)>(y) ? (x) : (y) )
-#define MAX3(x,y,z)                MAX2( MAX2((x),(y)) , (z) )  
+#define MAX2(x, y)          ((x) > (y) ? (x) : (y))
+#define MAX3(x, y, z)       (MAX2(MAX2((x), (y)), (z)))
 
 void calcAlpha(ImBuf * ibuf)
 {
@@ -75,7 +75,7 @@ void calcAlpha(ImBuf * ibuf)
 	
 	if (ibuf) {
 		rect = (char *) ibuf->rect;
-		for (i = ibuf->x * ibuf->y ; i > 0 ; i--) {
+		for (i = ibuf->x * ibuf->y; i > 0; i--) {
 			rect[3] = MAX3(rect[0], rect[1], rect[2]);
 			rect += 4;
 		}
@@ -96,29 +96,29 @@ void readBitmapFontVersion0(ImBuf * ibuf, unsigned char * rect, int step)
 	ysize = (bytes + (ibuf->x - 1)) / ibuf->x;
 	
 	if (ysize < ibuf->y) {
-		// we're first going to copy all data into a liniar buffer.
-		// step can be 4 or 1 bytes, and the data is not sequential because
-		// the bitmap was flipped vertically.
+		/* we're first going to copy all data into a linear buffer.
+		 * step can be 4 or 1 bytes, and the data is not sequential because
+		 * the bitmap was flipped vertically. */
 		
 		buffer = MEM_mallocN(bytes, "readBitmapFontVersion0:buffer");
 		
-		index = 0;	
+		index = 0;
 		for (i = 0; i < bytes; i++) {
 			buffer[i] = rect[index];
 			index += step;
 			if (index >= linelength) {
-				// we've read one line, no skip to the line *before* that
+				/* we've read one line, no skip to the line *before* that */
 				rect -= linelength;
 				index -= linelength;
 			}
 		}
 		
-		// we're now going to endian convert the data
+		/* we're now going to endian convert the data */
 		
 		bmfont = MEM_mallocN(bytes, "readBitmapFontVersion0:bmfont");
 		index = 0;
 		
-		// first read the header
+		/* first read the header */
 		bmfont->magic[0]    = buffer[index++];
 		bmfont->magic[1]    = buffer[index++];
 		bmfont->magic[2]    = buffer[index++];
@@ -151,16 +151,16 @@ void readBitmapFontVersion0(ImBuf * ibuf, unsigned char * rect, int step)
 			printf("bytes = %d\n", bytes);
 		}
 
-		// we've read the data from the image. Now we're going
-		// to crop the image vertically so only the bitmap data
-		// remains visible
-		
+		/* we've read the data from the image. Now we're going
+		 * to crop the image vertically so only the bitmap data
+		 * remains visible */
+
 		ibuf->y -= ysize;
 		ibuf->userdata = bmfont;
 		ibuf->userflags |= IB_BITMAPFONT;
 
 		if (ibuf->planes < 32) {
-			// we're going to fake alpha here:
+			/* we're going to fake alpha here: */
 			calcAlpha(ibuf);
 		}
 	}
@@ -176,32 +176,32 @@ void detectBitmapFont(ImBuf *ibuf)
 	int i;
 	
 	if (ibuf != NULL && ibuf->rect != NULL) {
-			// bitmap must have an x size that is a power of two
+			/* bitmap must have an x size that is a power of two */
 		if (is_power_of_two(ibuf->x)) {
 			rect = (unsigned char *) (ibuf->rect + (ibuf->x * (ibuf->y - 1)));
-			// printf ("starts with: %s %c %c %c %c\n", rect, rect[0], rect[1], rect[2], rect[3]);
+			/* printf ("starts with: %s %c %c %c %c\n", rect, rect[0], rect[1], rect[2], rect[3]); */
 			if (rect[0] == 'B' && rect[1] == 'F' && rect[2] == 'N' && rect[3] == 'T') {
-				// printf("found 8bit font !\n");
-				// round y size down
-				// do the 8 bit font stuff. (not yet)
+				/* printf("found 8bit font !\n");
+				 * round y size down
+				 * do the 8 bit font stuff. (not yet) */
 			}
 			else {
-				// we try all 4 possible combinations
+				/* we try all 4 possible combinations */
 				for (i = 0; i < 4; i++) {
 					if (rect[0] == 'B' && rect[4] == 'F' && rect[8] == 'N' && rect[12] == 'T') {
-						// printf("found 24bit font !\n");
-						// We're going to parse the file:
-						
+						/* printf("found 24bit font !\n");
+						 * We're going to parse the file: */
+
 						version = (rect[16] << 8) | rect[20];
-						
+
 						if (version == 0) {
 							readBitmapFontVersion0(ibuf, rect, 4);
 						}
 						else {
 							printf("detectBitmapFont :Unsupported version %d\n", version);
 						}
-						
-						// on succes ibuf->userdata points to the bitmapfont
+
+						/* on succes ibuf->userdata points to the bitmapfont */
 						if (ibuf->userdata) {
 							break;
 						}
@@ -221,23 +221,23 @@ int locateGlyph(bmFont *bmfont, unsigned short unicode)
 		min = 0;
 		max = bmfont->glyphcount;
 		while (1) {
-			// look halfway for glyph
+			/* look halfway for glyph */
 			current = (min + max) >> 1;
 
 			if (bmfont->glyphs[current].unicode == unicode) {
 				break;
 			}
 			else if (bmfont->glyphs[current].unicode < unicode) {
-				// have to move up
+				/* have to move up */
 				min = current;
 			}
 			else {
-				// have to move down
+				/* have to move down */
 				max = current;
 			}
-			
+
 			if (max - min <= 1) {
-				// unable to locate glyph
+				/* unable to locate glyph */
 				current = 0;
 				break;
 			}
@@ -247,12 +247,13 @@ int locateGlyph(bmFont *bmfont, unsigned short unicode)
 	return(current);
 }
 
-void matrixGlyph(ImBuf * ibuf, unsigned short unicode,
-		float *centerx, float *centery,
-		float *sizex,   float *sizey,
-		float *transx,  float *transy,
-		float *movex,   float *movey,
-		float *advance)
+void matrixGlyph(
+        ImBuf * ibuf, unsigned short unicode,
+        float *centerx, float *centery,
+        float *sizex,   float *sizey,
+        float *transx,  float *transy,
+        float *movex,   float *movey,
+        float *advance)
 {
 	int index;
 	bmFont *bmfont;
@@ -278,7 +279,7 @@ void matrixGlyph(ImBuf * ibuf, unsigned short unicode,
 				*centerx = bmfont->glyphs[0].locx / (float) ibuf->x;
 				*centery = (ibuf->y - bmfont->glyphs[0].locy) / (float) ibuf->y;
 
-				// 2.0 units is the default size of an object
+				/* 2.0 units is the default size of an object */
 				
 				*movey = 1.0f - *sizey + 2.0f * (bmfont->glyphs[index].ofsy - bmfont->glyphs[0].ofsy) / (float) bmfont->glyphs[0].sizey;
 				*movex = *sizex - 1.0f + 2.0f * (bmfont->glyphs[index].ofsx - bmfont->glyphs[0].ofsx) / (float) bmfont->glyphs[0].sizex;
