@@ -101,13 +101,15 @@ static int rule_goal_avoid(BoidRule *rule, BoidBrainData *bbd, BoidValues *val,
 				break;
 			}
 		}
-		else if (rule->type == eBoidRuleType_Goal && eob == bpa->ground)
-			; /* skip current object */
+		else if (rule->type == eBoidRuleType_Goal && eob == bpa->ground) {
+			/* skip current object */
+		}
 		else if (pd->forcefield == PFIELD_BOID && mul * pd->f_strength > 0.0f && get_effector_data(cur, &cur_efd, &epoint, 0)) {
 			float temp = mul * pd->f_strength * effector_falloff(cur, &cur_efd, &epoint, bbd->part->effector_weights);
 
-			if (temp == 0.0f)
-				; /* do nothing */
+			if (temp == 0.0f) {
+				/* do nothing */
+			}
 			else if (temp > priority) {
 				priority = temp;
 				eff = cur;
@@ -230,8 +232,8 @@ static int rule_avoid_collision(BoidRule *rule, BoidBrainData *bbd, BoidValues *
 
 			/* avoid head-on collision */
 			if (dot_v3v3(col.pce.nor, pa->prev_state.ave) < -0.99f) {
-				/* don't know why, but uneven range [0.0,1.0] */
-				/* works much better than even [-1.0,1.0] */
+				/* don't know why, but uneven range [0.0, 1.0] */
+				/* works much better than even [-1.0, 1.0] */
 				bbd->wanted_co[0] = BLI_frand();
 				bbd->wanted_co[1] = BLI_frand();
 				bbd->wanted_co[2] = BLI_frand();
@@ -262,7 +264,7 @@ static int rule_avoid_collision(BoidRule *rule, BoidBrainData *bbd, BoidValues *
 
 			sub_v3_v3v3(vec, vel1, vel2);
 			
-			inp = dot_v3v3(vec,vec);
+			inp = dot_v3v3(vec, vec);
 
 			/* velocities not parallel */
 			if (inp != 0.0f) {
@@ -308,7 +310,7 @@ static int rule_avoid_collision(BoidRule *rule, BoidBrainData *bbd, BoidValues *
 
 				sub_v3_v3v3(vec, vel1, vel2);
 				
-				inp = dot_v3v3(vec,vec);
+				inp = dot_v3v3(vec, vec);
 
 				/* velocities not parallel */
 				if (inp != 0.0f) {
@@ -425,7 +427,7 @@ static int rule_follow_leader(BoidRule *rule, BoidBrainData *bbd, BoidValues *va
 	if (flbr->ob) {
 		float vec2[3], t;
 
-		/* first check we're not blocking the leader*/
+		/* first check we're not blocking the leader */
 		sub_v3_v3v3(vec, flbr->loc, flbr->oloc);
 		mul_v3_fl(vec, 1.0f/bbd->timestep);
 
@@ -583,7 +585,7 @@ static int rule_average_speed(BoidRule *rule, BoidBrainData *bbd, BoidValues *va
 		copy_v3_v3(bbd->wanted_co, pa->prev_state.ave);
 
 		/* may happen at birth */
-		if (dot_v2v2(bbd->wanted_co,bbd->wanted_co)==0.0f) {
+		if (dot_v2v2(bbd->wanted_co, bbd->wanted_co)==0.0f) {
 			bbd->wanted_co[0] = 2.0f*(0.5f - BLI_frand());
 			bbd->wanted_co[1] = 2.0f*(0.5f - BLI_frand());
 			bbd->wanted_co[2] = 2.0f*(0.5f - BLI_frand());
@@ -610,7 +612,7 @@ static int rule_fight(BoidRule *rule, BoidBrainData *bbd, BoidValues *val, Parti
 	ParticleData *enemy_pa = NULL;
 	BoidParticle *bpa;
 	/* friends & enemies */
-	float closest_enemy[3] = {0.0f,0.0f,0.0f};
+	float closest_enemy[3] = {0.0f, 0.0f, 0.0f};
 	float closest_dist = fbr->distance + 1.0f;
 	float f_strength = 0.0f, e_strength = 0.0f;
 	float health = 0.0f;
@@ -748,7 +750,7 @@ static Object *boid_find_ground(BoidBrainData *bbd, ParticleData *pa, float grou
 		SurfaceModifierData *surmd = NULL;
 		float x[3], v[3];
 		
-		surmd = (SurfaceModifierData *)modifiers_findByType ( bpa->ground, eModifierType_Surface );
+		surmd = (SurfaceModifierData *)modifiers_findByType(bpa->ground, eModifierType_Surface );
 
 		/* take surface velocity into account */
 		closest_point_on_surface(surmd, pa->state.co, x, NULL, v);
@@ -920,7 +922,8 @@ static BoidState *get_boid_state(BoidSettings *boids, ParticleData *pa)
 
 	return state;
 }
-//static int boid_condition_is_true(BoidCondition *cond) {
+//static int boid_condition_is_true(BoidCondition *cond)
+//{
 //	/* TODO */
 //	return 0;
 //}
@@ -953,7 +956,8 @@ void boid_brain(BoidBrainData *bbd, int p, ParticleData *pa)
 	//	}
 	//}
 
-	bbd->wanted_co[0]=bbd->wanted_co[1]=bbd->wanted_co[2]=bbd->wanted_speed=0.0f;
+	zero_v3(bbd->wanted_co);
+	bbd->wanted_speed = 0.0f;
 
 	/* create random seed for every particle & frame */
 	rand = (int)(PSYS_FRAND(psys->seed + p) * 1000);
@@ -962,7 +966,7 @@ void boid_brain(BoidBrainData *bbd, int p, ParticleData *pa)
 	set_boid_values(&val, bbd->part->boids, pa);
 
 	/* go through rules */
-	switch(state->ruleset_type) {
+	switch (state->ruleset_type) {
 		case eBoidRulesetType_Fuzzy:
 		{
 			for (rule = state->rules.first; rule; rule = rule->next) {
@@ -987,7 +991,8 @@ void boid_brain(BoidBrainData *bbd, int p, ParticleData *pa)
 					add_v3_v3(wanted_co, bbd->wanted_co);
 					wanted_speed += bbd->wanted_speed;
 					n++;
-					bbd->wanted_co[0]=bbd->wanted_co[1]=bbd->wanted_co[2]=bbd->wanted_speed=0.0f;
+					zero_v3(bbd->wanted_co);
+					bbd->wanted_speed = 0.0f;
 				}
 			}
 
@@ -1096,7 +1101,7 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
 	set_boid_values(&val, boids, pa);
 
 	/* make sure there's something in new velocity, location & rotation */
-	copy_particle_key(&pa->state,&pa->prev_state,0);
+	copy_particle_key(&pa->state, &pa->prev_state, 0);
 
 	if (bbd->part->flag & PART_SIZEMASS)
 		pa_mass*=pa->size;
@@ -1167,16 +1172,16 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
 
 			/* constrain direction with maximum angular velocity */
 			angle = saacos(dot_v3v3(old_dir, wanted_dir));
-			angle = MIN2(angle, val.max_ave);
+			angle = min_ff(angle, val.max_ave);
 
 			cross_v3_v3v3(nor, old_dir, wanted_dir);
-			axis_angle_to_quat( q,nor, angle);
+			axis_angle_to_quat(q, nor, angle);
 			copy_v3_v3(new_dir, old_dir);
 			mul_qt_v3(q, new_dir);
 			normalize_v3(new_dir);
 
 			/* save direction in case resulting velocity too small */
-			axis_angle_to_quat( q,nor, angle*dtime);
+			axis_angle_to_quat(q, nor, angle*dtime);
 			copy_v3_v3(pa->state.ave, old_dir);
 			mul_qt_v3(q, pa->state.ave);
 			normalize_v3(pa->state.ave);
@@ -1196,7 +1201,7 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
 
 		/* maintain minimum flying velocity if not landing */
 		if (level >= landing_level) {
-			float len2 = dot_v2v2(new_vel,new_vel);
+			float len2 = dot_v2v2(new_vel, new_vel);
 			float root;
 
 			len2 = MAX2(len2, val.min_speed*val.min_speed);
@@ -1258,14 +1263,14 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
 	bpa->ground = boid_find_ground(bbd, pa, ground_co, ground_nor);
 
 	/* change modes, constrain movement & keep track of down vector */
-	switch(bpa->data.mode) {
+	switch (bpa->data.mode) {
 		case eBoidMode_InAir:
 		{
 			float grav[3];
 
-			grav[0]= 0.0f;
-			grav[1]= 0.0f;
-			grav[2]= bbd->sim->scene->physics_settings.gravity[2] < 0.0f ? -1.0f : 0.0f;
+			grav[0] = 0.0f;
+			grav[1] = 0.0f;
+			grav[2] = bbd->sim->scene->physics_settings.gravity[2] < 0.0f ? -1.0f : 0.0f;
 
 			/* don't take forward acceleration into account (better banking) */
 			if (dot_v3v3(bpa->data.acc, pa->state.vel) > 0.0f) {
@@ -1306,9 +1311,9 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
 		{
 			float grav[3];
 
-			grav[0]= 0.0f;
-			grav[1]= 0.0f;
-			grav[2]= bbd->sim->scene->physics_settings.gravity[2] < 0.0f ? -1.0f : 0.0f;
+			grav[0] = 0.0f;
+			grav[1] = 0.0f;
+			grav[2] = bbd->sim->scene->physics_settings.gravity[2] < 0.0f ? -1.0f : 0.0f;
 
 
 			/* gather apparent gravity */
@@ -1427,7 +1432,7 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
 	cross_v3_v3v3(mat[1], mat[2], mat[0]);
 	
 	/* apply rotation */
-	mat3_to_quat_is_ok( q,mat);
+	mat3_to_quat_is_ok(q, mat);
 	copy_qt_qt(pa->state.rot, q);
 }
 
@@ -1437,7 +1442,7 @@ BoidRule *boid_new_rule(int type)
 	if (type <= 0)
 		return NULL;
 
-	switch(type) {
+	switch (type) {
 		case eBoidRuleType_Goal:
 		case eBoidRuleType_Avoid:
 			rule = MEM_callocN(sizeof(BoidRuleGoalAvoid), "BoidRuleGoalAvoid");
