@@ -54,7 +54,7 @@
 /* --------- Memory Management ------------ */
 
 /* Free strokes belonging to a gp-frame */
-void free_gpencil_strokes (bGPDframe *gpf)
+void free_gpencil_strokes(bGPDframe *gpf)
 {
 	bGPDstroke *gps, *gpsn;
 	
@@ -62,8 +62,8 @@ void free_gpencil_strokes (bGPDframe *gpf)
 	if (gpf == NULL) return;
 	
 	/* free strokes */
-	for (gps= gpf->strokes.first; gps; gps= gpsn) {
-		gpsn= gps->next;
+	for (gps = gpf->strokes.first; gps; gps = gpsn) {
+		gpsn = gps->next;
 		
 		/* free stroke memory arrays, then stroke itself */
 		if (gps->points) MEM_freeN(gps->points);
@@ -72,7 +72,7 @@ void free_gpencil_strokes (bGPDframe *gpf)
 }
 
 /* Free all of a gp-layer's frames */
-void free_gpencil_frames (bGPDlayer *gpl)
+void free_gpencil_frames(bGPDlayer *gpl)
 {
 	bGPDframe *gpf, *gpfn;
 	
@@ -80,26 +80,27 @@ void free_gpencil_frames (bGPDlayer *gpl)
 	if (gpl == NULL) return;
 	
 	/* free frames */
-	for (gpf= gpl->frames.first; gpf; gpf= gpfn) {
-		gpfn= gpf->next;
+	for (gpf = gpl->frames.first; gpf; gpf = gpfn) {
+		gpfn = gpf->next;
 		
 		/* free strokes and their associated memory */
 		free_gpencil_strokes(gpf);
 		BLI_freelinkN(&gpl->frames, gpf);
 	}
+	gpl->actframe = NULL;
 }
 
 /* Free all of the gp-layers for a viewport (list should be &gpd->layers or so) */
-void free_gpencil_layers (ListBase *list) 
+void free_gpencil_layers(ListBase *list)
 {
 	bGPDlayer *gpl, *gpln;
 	
 	/* error checking */
 	if (list == NULL) return;
 	
-	/* delete layers*/
-	for (gpl= list->first; gpl; gpl= gpln) {
-		gpln= gpl->next;
+	/* delete layers */
+	for (gpl = list->first; gpl; gpl = gpln) {
+		gpln = gpl->next;
 		
 		/* free layers and their data */
 		free_gpencil_frames(gpl);
@@ -108,7 +109,7 @@ void free_gpencil_layers (ListBase *list)
 }
 
 /* Free all of GPencil datablock's related data, but not the block itself */
-void free_gpencil_data (bGPdata *gpd)
+void BKE_gpencil_free(bGPdata *gpd)
 {
 	/* free layers */
 	free_gpencil_layers(&gpd->layers);
@@ -117,32 +118,32 @@ void free_gpencil_data (bGPdata *gpd)
 /* -------- Container Creation ---------- */
 
 /* add a new gp-frame to the given layer */
-bGPDframe *gpencil_frame_addnew (bGPDlayer *gpl, int cframe)
+bGPDframe *gpencil_frame_addnew(bGPDlayer *gpl, int cframe)
 {
 	bGPDframe *gpf, *gf;
-	short state=0;
+	short state = 0;
 	
 	/* error checking */
 	if ((gpl == NULL) || (cframe <= 0))
 		return NULL;
 		
 	/* allocate memory for this frame */
-	gpf= MEM_callocN(sizeof(bGPDframe), "bGPDframe");
-	gpf->framenum= cframe;
+	gpf = MEM_callocN(sizeof(bGPDframe), "bGPDframe");
+	gpf->framenum = cframe;
 	
 	/* find appropriate place to add frame */
 	if (gpl->frames.first) {
-		for (gf= gpl->frames.first; gf; gf= gf->next) {
+		for (gf = gpl->frames.first; gf; gf = gf->next) {
 			/* check if frame matches one that is supposed to be added */
 			if (gf->framenum == cframe) {
-				state= -1;
+				state = -1;
 				break;
 			}
 			
 			/* if current frame has already exceeded the frame to add, add before */
 			if (gf->framenum > cframe) {
 				BLI_insertlinkbefore(&gpl->frames, gf, gpf);
-				state= 1;
+				state = 1;
 				break;
 			}
 		}
@@ -163,7 +164,7 @@ bGPDframe *gpencil_frame_addnew (bGPDlayer *gpl, int cframe)
 }
 
 /* add a new gp-layer and make it the active layer */
-bGPDlayer *gpencil_layer_addnew (bGPdata *gpd)
+bGPDlayer *gpencil_layer_addnew(bGPdata *gpd, const char *name, int setactive)
 {
 	bGPDlayer *gpl;
 	
@@ -172,36 +173,37 @@ bGPDlayer *gpencil_layer_addnew (bGPdata *gpd)
 		return NULL;
 		
 	/* allocate memory for frame and add to end of list */
-	gpl= MEM_callocN(sizeof(bGPDlayer), "bGPDlayer");
+	gpl = MEM_callocN(sizeof(bGPDlayer), "bGPDlayer");
 	
 	/* add to datablock */
 	BLI_addtail(&gpd->layers, gpl);
 	
 	/* set basic settings */
-	gpl->color[3]= 0.9f;
+	gpl->color[3] = 0.9f;
 	gpl->thickness = 3;
 	
 	/* auto-name */
-	strcpy(gpl->info, "GP_Layer");
+	BLI_strncpy(gpl->info, name, sizeof(gpl->info));
 	BLI_uniquename(&gpd->layers, gpl, "GP_Layer", '.', offsetof(bGPDlayer, info), sizeof(gpl->info));
 	
 	/* make this one the active one */
-	gpencil_layer_setactive(gpd, gpl);
+	if (setactive)
+		gpencil_layer_setactive(gpd, gpl);
 	
 	/* return layer */
 	return gpl;
 }
 
 /* add a new gp-datablock */
-bGPdata *gpencil_data_addnew (const char name[])
+bGPdata *gpencil_data_addnew(const char name[])
 {
 	bGPdata *gpd;
 	
 	/* allocate memory for a new block */
-	gpd= alloc_libblock(&G.main->gpencil, ID_GD, name);
+	gpd = BKE_libblock_alloc(&G.main->gpencil, ID_GD, name);
 	
 	/* initial settings */
-	gpd->flag = (GP_DATA_DISPINFO|GP_DATA_EXPAND);
+	gpd->flag = (GP_DATA_DISPINFO | GP_DATA_EXPAND);
 	
 	/* for now, stick to view is also enabled by default
 	 * since this is more useful...
@@ -214,7 +216,7 @@ bGPdata *gpencil_data_addnew (const char name[])
 /* -------- Data Duplication ---------- */
 
 /* make a copy of a given gpencil frame */
-bGPDframe *gpencil_frame_duplicate (bGPDframe *src)
+bGPDframe *gpencil_frame_duplicate(bGPDframe *src)
 {
 	bGPDstroke *gps, *gpsd;
 	bGPDframe *dst;
@@ -224,15 +226,15 @@ bGPDframe *gpencil_frame_duplicate (bGPDframe *src)
 		return NULL;
 		
 	/* make a copy of the source frame */
-	dst= MEM_dupallocN(src);
-	dst->prev= dst->next= NULL;
+	dst = MEM_dupallocN(src);
+	dst->prev = dst->next = NULL;
 	
 	/* copy strokes */
-	dst->strokes.first = dst->strokes.last= NULL;
-	for (gps= src->strokes.first; gps; gps= gps->next) {
+	dst->strokes.first = dst->strokes.last = NULL;
+	for (gps = src->strokes.first; gps; gps = gps->next) {
 		/* make copy of source stroke, then adjust pointer to points too */
-		gpsd= MEM_dupallocN(gps);
-		gpsd->points= MEM_dupallocN(gps->points);
+		gpsd = MEM_dupallocN(gps);
+		gpsd->points = MEM_dupallocN(gps->points);
 		
 		BLI_addtail(&dst->strokes, gpsd);
 	}
@@ -242,7 +244,7 @@ bGPDframe *gpencil_frame_duplicate (bGPDframe *src)
 }
 
 /* make a copy of a given gpencil layer */
-bGPDlayer *gpencil_layer_duplicate (bGPDlayer *src)
+bGPDlayer *gpencil_layer_duplicate(bGPDlayer *src)
 {
 	bGPDframe *gpf, *gpfd;
 	bGPDlayer *dst;
@@ -252,19 +254,19 @@ bGPDlayer *gpencil_layer_duplicate (bGPDlayer *src)
 		return NULL;
 		
 	/* make a copy of source layer */
-	dst= MEM_dupallocN(src);
-	dst->prev= dst->next= NULL;
+	dst = MEM_dupallocN(src);
+	dst->prev = dst->next = NULL;
 	
 	/* copy frames */
-	dst->frames.first= dst->frames.last= NULL;
-	for (gpf= src->frames.first; gpf; gpf= gpf->next) {
+	dst->frames.first = dst->frames.last = NULL;
+	for (gpf = src->frames.first; gpf; gpf = gpf->next) {
 		/* make a copy of source frame */
-		gpfd= gpencil_frame_duplicate(gpf);
+		gpfd = gpencil_frame_duplicate(gpf);
 		BLI_addtail(&dst->frames, gpfd);
 		
 		/* if source frame was the current layer's 'active' frame, reassign that too */
 		if (gpf == dst->actframe)
-			dst->actframe= gpfd;
+			dst->actframe = gpfd;
 	}
 	
 	/* return new layer */
@@ -272,7 +274,7 @@ bGPDlayer *gpencil_layer_duplicate (bGPDlayer *src)
 }
 
 /* make a copy of a given gpencil datablock */
-bGPdata *gpencil_data_duplicate (bGPdata *src)
+bGPdata *gpencil_data_duplicate(bGPdata *src)
 {
 	bGPDlayer *gpl, *gpld;
 	bGPdata *dst;
@@ -282,13 +284,13 @@ bGPdata *gpencil_data_duplicate (bGPdata *src)
 		return NULL;
 	
 	/* make a copy of the base-data */
-	dst= MEM_dupallocN(src);
+	dst = MEM_dupallocN(src);
 	
 	/* copy layers */
-	dst->layers.first= dst->layers.last= NULL;
-	for (gpl= src->layers.first; gpl; gpl= gpl->next) {
+	dst->layers.first = dst->layers.last = NULL;
+	for (gpl = src->layers.first; gpl; gpl = gpl->next) {
 		/* make a copy of source layer and its data */
-		gpld= gpencil_layer_duplicate(gpl);
+		gpld = gpencil_layer_duplicate(gpl);
 		BLI_addtail(&dst->layers, gpld);
 	}
 	
@@ -299,9 +301,9 @@ bGPdata *gpencil_data_duplicate (bGPdata *src)
 /* -------- GP-Frame API ---------- */
 
 /* delete the last stroke of the given frame */
-void gpencil_frame_delete_laststroke (bGPDlayer *gpl, bGPDframe *gpf)
+void gpencil_frame_delete_laststroke(bGPDlayer *gpl, bGPDframe *gpf)
 {
-	bGPDstroke *gps= (gpf) ? gpf->strokes.last : NULL;
+	bGPDstroke *gps = (gpf) ? gpf->strokes.last : NULL;
 	int cfra = (gpf) ? gpf->framenum : 0; /* assume that the current frame was not locked */
 	
 	/* error checking */
@@ -321,11 +323,24 @@ void gpencil_frame_delete_laststroke (bGPDlayer *gpl, bGPDframe *gpf)
 
 /* -------- GP-Layer API ---------- */
 
+bGPDframe *BKE_gpencil_layer_find_frame(bGPDlayer *gpl, int cframe)
+{
+	bGPDframe *gpf;
+
+	for (gpf = gpl->frames.last; gpf; gpf = gpf->prev) {
+		if (gpf->framenum == cframe) {
+			return gpf;
+		}
+	}
+
+	return NULL;
+}
+
 /* get the appropriate gp-frame from a given layer
  *	- this sets the layer's actframe var (if allowed to)
  *	- extension beyond range (if first gp-frame is after all frame in interest and cannot add)
  */
-bGPDframe *gpencil_layer_getframe (bGPDlayer *gpl, int cframe, short addnew)
+bGPDframe *gpencil_layer_getframe(bGPDlayer *gpl, int cframe, short addnew)
 {
 	bGPDframe *gpf = NULL;
 	short found = 0;
@@ -336,12 +351,12 @@ bGPDframe *gpencil_layer_getframe (bGPDlayer *gpl, int cframe, short addnew)
 	
 	/* check if there is already an active frame */
 	if (gpl->actframe) {
-		gpf= gpl->actframe;
+		gpf = gpl->actframe;
 		
 		/* do not allow any changes to layer's active frame if layer is locked from changes
 		 * or if the layer has been set to stay on the current frame
 		 */
-		if (gpl->flag & (GP_LAYER_LOCKED|GP_LAYER_FRAMELOCK))
+		if (gpl->flag & (GP_LAYER_LOCKED | GP_LAYER_FRAMELOCK))
 			return gpf;
 		/* do not allow any changes to actframe if frame has painting tag attached to it */
 		if (gpf->flag & GP_FRAME_PAINT) 
@@ -349,13 +364,13 @@ bGPDframe *gpencil_layer_getframe (bGPDlayer *gpl, int cframe, short addnew)
 		
 		/* try to find matching frame */
 		if (gpf->framenum < cframe) {
-			for (; gpf; gpf= gpf->next) {
+			for (; gpf; gpf = gpf->next) {
 				if (gpf->framenum == cframe) {
-					found= 1;
+					found = 1;
 					break;
 				}
 				else if ((gpf->next) && (gpf->next->framenum > cframe)) {
-					found= 1;
+					found = 1;
 					break;
 				}
 			}
@@ -363,19 +378,19 @@ bGPDframe *gpencil_layer_getframe (bGPDlayer *gpl, int cframe, short addnew)
 			/* set the appropriate frame */
 			if (addnew) {
 				if ((found) && (gpf->framenum == cframe))
-					gpl->actframe= gpf;
+					gpl->actframe = gpf;
 				else
-					gpl->actframe= gpencil_frame_addnew(gpl, cframe);
+					gpl->actframe = gpencil_frame_addnew(gpl, cframe);
 			}
 			else if (found)
-				gpl->actframe= gpf;
+				gpl->actframe = gpf;
 			else
-				gpl->actframe= gpl->frames.last;
+				gpl->actframe = gpl->frames.last;
 		}
 		else {
-			for (; gpf; gpf= gpf->prev) {
+			for (; gpf; gpf = gpf->prev) {
 				if (gpf->framenum <= cframe) {
-					found= 1;
+					found = 1;
 					break;
 				}
 			}
@@ -383,35 +398,35 @@ bGPDframe *gpencil_layer_getframe (bGPDlayer *gpl, int cframe, short addnew)
 			/* set the appropriate frame */
 			if (addnew) {
 				if ((found) && (gpf->framenum == cframe))
-					gpl->actframe= gpf;
+					gpl->actframe = gpf;
 				else
-					gpl->actframe= gpencil_frame_addnew(gpl, cframe);
+					gpl->actframe = gpencil_frame_addnew(gpl, cframe);
 			}
 			else if (found)
-				gpl->actframe= gpf;
+				gpl->actframe = gpf;
 			else
-				gpl->actframe= gpl->frames.first;
+				gpl->actframe = gpl->frames.first;
 		}
 	}
 	else if (gpl->frames.first) {
 		/* check which of the ends to start checking from */
-		const int first= ((bGPDframe *)(gpl->frames.first))->framenum;
-		const int last= ((bGPDframe *)(gpl->frames.last))->framenum;
+		const int first = ((bGPDframe *)(gpl->frames.first))->framenum;
+		const int last = ((bGPDframe *)(gpl->frames.last))->framenum;
 		
-		if (abs(cframe-first) > abs(cframe-last)) {
+		if (abs(cframe - first) > abs(cframe - last)) {
 			/* find gp-frame which is less than or equal to cframe */
-			for (gpf= gpl->frames.last; gpf; gpf= gpf->prev) {
+			for (gpf = gpl->frames.last; gpf; gpf = gpf->prev) {
 				if (gpf->framenum <= cframe) {
-					found= 1;
+					found = 1;
 					break;
 				}
 			}
 		}
 		else {
 			/* find gp-frame which is less than or equal to cframe */
-			for (gpf= gpl->frames.first; gpf; gpf= gpf->next) {
+			for (gpf = gpl->frames.first; gpf; gpf = gpf->next) {
 				if (gpf->framenum <= cframe) {
-					found= 1;
+					found = 1;
 					break;
 				}
 			}
@@ -420,12 +435,12 @@ bGPDframe *gpencil_layer_getframe (bGPDlayer *gpl, int cframe, short addnew)
 		/* set the appropriate frame */
 		if (addnew) {
 			if ((found) && (gpf->framenum == cframe))
-				gpl->actframe= gpf;
+				gpl->actframe = gpf;
 			else
-				gpl->actframe= gpencil_frame_addnew(gpl, cframe);
+				gpl->actframe = gpencil_frame_addnew(gpl, cframe);
 		}
 		else if (found)
-			gpl->actframe= gpf;
+			gpl->actframe = gpf;
 		else {
 			/* unresolved errogenous situation! */
 			printf("Error: cannot find appropriate gp-frame\n");
@@ -435,7 +450,7 @@ bGPDframe *gpencil_layer_getframe (bGPDlayer *gpl, int cframe, short addnew)
 	else {
 		/* currently no frames (add if allowed to) */
 		if (addnew)
-			gpl->actframe= gpencil_frame_addnew(gpl, cframe);
+			gpl->actframe = gpencil_frame_addnew(gpl, cframe);
 		else {
 			/* don't do anything... this may be when no frames yet! */
 			/* gpl->actframe should still be NULL */
@@ -447,7 +462,7 @@ bGPDframe *gpencil_layer_getframe (bGPDlayer *gpl, int cframe, short addnew)
 }
 
 /* delete the given frame from a layer */
-void gpencil_layer_delframe (bGPDlayer *gpl, bGPDframe *gpf)
+void gpencil_layer_delframe(bGPDlayer *gpl, bGPDframe *gpf)
 {
 	/* error checking */
 	if (ELEM(NULL, gpl, gpf))
@@ -460,7 +475,7 @@ void gpencil_layer_delframe (bGPDlayer *gpl, bGPDframe *gpf)
 }
 
 /* get the active gp-layer for editing */
-bGPDlayer *gpencil_layer_getactive (bGPdata *gpd)
+bGPDlayer *gpencil_layer_getactive(bGPdata *gpd)
 {
 	bGPDlayer *gpl;
 	
@@ -469,7 +484,7 @@ bGPDlayer *gpencil_layer_getactive (bGPdata *gpd)
 		return NULL;
 		
 	/* loop over layers until found (assume only one active) */
-	for (gpl=gpd->layers.first; gpl; gpl=gpl->next) {
+	for (gpl = gpd->layers.first; gpl; gpl = gpl->next) {
 		if (gpl->flag & GP_LAYER_ACTIVE)
 			return gpl;
 	}
@@ -479,7 +494,7 @@ bGPDlayer *gpencil_layer_getactive (bGPdata *gpd)
 }
 
 /* set the active gp-layer */
-void gpencil_layer_setactive (bGPdata *gpd, bGPDlayer *active)
+void gpencil_layer_setactive(bGPdata *gpd, bGPDlayer *active)
 {
 	bGPDlayer *gpl;
 	
@@ -488,7 +503,7 @@ void gpencil_layer_setactive (bGPdata *gpd, bGPDlayer *active)
 		return;
 		
 	/* loop over layers deactivating all */
-	for (gpl=gpd->layers.first; gpl; gpl=gpl->next)
+	for (gpl = gpd->layers.first; gpl; gpl = gpl->next)
 		gpl->flag &= ~GP_LAYER_ACTIVE;
 	
 	/* set as active one */
@@ -496,15 +511,13 @@ void gpencil_layer_setactive (bGPdata *gpd, bGPDlayer *active)
 }
 
 /* delete the active gp-layer */
-void gpencil_layer_delactive (bGPdata *gpd)
+void gpencil_layer_delete(bGPdata *gpd, bGPDlayer *gpl)
 {
-	bGPDlayer *gpl= gpencil_layer_getactive(gpd);
-	
 	/* error checking */
 	if (ELEM(NULL, gpd, gpl)) 
 		return;
 	
-	/* free layer */	
+	/* free layer */
 	free_gpencil_frames(gpl);
 	BLI_freelinkN(&gpd->layers, gpl);
 }
