@@ -51,11 +51,11 @@
 
 #include "BLO_sys_types.h" // for intptr_t support
 
-#define GS(a)	(*((short *)(a)))
+#include "GPU_extensions.h"
 
 /* GLOBALS */
 
-static GHash* gIcons = NULL;
+static GHash *gIcons = NULL;
 
 static int gNextIconId = 1;
 
@@ -64,10 +64,10 @@ static int gFirstIconId = 1;
 
 static void icon_free(void *val)
 {
-	Icon* icon = val;
+	Icon *icon = val;
 
 	if (icon) {
-		if (icon->drawinfo_free) {		
+		if (icon->drawinfo_free) {
 			icon->drawinfo_free(icon->drawinfo);
 		}
 		else if (icon->drawinfo) {
@@ -84,15 +84,15 @@ static int get_next_free_id(void)
 	int startId = gFirstIconId;
 
 	/* if we haven't used up the int number range, we just return the next int */
-	if (gNextIconId>=gFirstIconId)
+	if (gNextIconId >= gFirstIconId)
 		return gNextIconId++;
 	
 	/* now we try to find the smallest icon id not stored in the gIcons hash */
-	while (BLI_ghash_lookup(gIcons, SET_INT_IN_POINTER(startId)) && startId>=gFirstIconId) 
+	while (BLI_ghash_lookup(gIcons, SET_INT_IN_POINTER(startId)) && startId >= gFirstIconId)
 		startId++;
 
 	/* if we found a suitable one that isn't used yet, return it */
-	if (startId>=gFirstIconId)
+	if (startId >= gFirstIconId)
 		return startId;
 
 	/* fail */
@@ -105,7 +105,7 @@ void BKE_icons_init(int first_dyn_id)
 	gFirstIconId = first_dyn_id;
 
 	if (!gIcons)
-		gIcons = BLI_ghash_new(BLI_ghashutil_inthash, BLI_ghashutil_intcmp, "icons_init gh");
+		gIcons = BLI_ghash_int_new("icons_init gh");
 }
 
 void BKE_icons_free(void)
@@ -115,14 +115,14 @@ void BKE_icons_free(void)
 	gIcons = NULL;
 }
 
-struct PreviewImage* BKE_previewimg_create(void) 
+PreviewImage *BKE_previewimg_create(void)
 {
-	PreviewImage* prv_img = NULL;
+	PreviewImage *prv_img = NULL;
 	int i;
 
 	prv_img = MEM_callocN(sizeof(PreviewImage), "img_prv");
 
-	for (i=0; i<NUM_ICON_SIZES; ++i) {
+	for (i = 0; i < NUM_ICON_SIZES; ++i) {
 		prv_img->changed[i] = 1;
 		prv_img->changed_timestamp[i] = 0;
 	}
@@ -135,12 +135,15 @@ void BKE_previewimg_freefunc(void *link)
 	if (prv) {
 		int i;
 
-		for (i=0; i<NUM_ICON_SIZES;++i) {
+		for (i = 0; i < NUM_ICON_SIZES; ++i) {
 			if (prv->rect[i]) {
 				MEM_freeN(prv->rect[i]);
 				prv->rect[i] = NULL;
 			}
+			if (prv->gputexture[i])
+				GPU_texture_free(prv->gputexture[i]);
 		}
+		
 		MEM_freeN(prv);
 	}
 }
@@ -153,20 +156,21 @@ void BKE_previewimg_free(PreviewImage **prv)
 	}
 }
 
-struct PreviewImage* BKE_previewimg_copy(PreviewImage *prv) 
+PreviewImage *BKE_previewimg_copy(PreviewImage *prv)
 {
-	PreviewImage* prv_img = NULL;
+	PreviewImage *prv_img = NULL;
 	int i;
 
 	if (prv) {
 		prv_img = MEM_dupallocN(prv);
-		for (i=0; i < NUM_ICON_SIZES; ++i) {
+		for (i = 0; i < NUM_ICON_SIZES; ++i) {
 			if (prv->rect[i]) {
 				prv_img->rect[i] = MEM_dupallocN(prv->rect[i]);
 			}
 			else {
 				prv_img->rect[i] = NULL;
 			}
+			prv_img->gputexture[i] = NULL;
 		}
 	}
 	return prv_img;
@@ -175,62 +179,62 @@ struct PreviewImage* BKE_previewimg_copy(PreviewImage *prv)
 void BKE_previewimg_free_id(ID *id) 
 {
 	if (GS(id->name) == ID_MA) {
-		Material *mat = (Material*)id;	
+		Material *mat = (Material *)id;
 		BKE_previewimg_free(&mat->preview);
 	}
 	else if (GS(id->name) == ID_TE) {
-		Tex *tex = (Tex*)id;
+		Tex *tex = (Tex *)id;
 		BKE_previewimg_free(&tex->preview);
 	}
 	else if (GS(id->name) == ID_WO) {
-		World *wo = (World*)id;
+		World *wo = (World *)id;
 		BKE_previewimg_free(&wo->preview);
 	}
 	else if (GS(id->name) == ID_LA) {
-		Lamp *la  = (Lamp*)id;
+		Lamp *la  = (Lamp *)id;
 		BKE_previewimg_free(&la->preview);
 	}
 	else if (GS(id->name) == ID_IM) {
-		Image *img  = (Image*)id;
+		Image *img  = (Image *)id;
 		BKE_previewimg_free(&img->preview);
 	}
 	else if (GS(id->name) == ID_BR) {
-		Brush *br  = (Brush*)id;
+		Brush *br  = (Brush *)id;
 		BKE_previewimg_free(&br->preview);
 	}
 }
 
-PreviewImage* BKE_previewimg_get(ID *id) 
+PreviewImage *BKE_previewimg_get(ID *id)
 {
-	PreviewImage* prv_img = NULL;
+	PreviewImage *prv_img = NULL;
 
 	if (GS(id->name) == ID_MA) {
-		Material *mat = (Material*)id;	
+		Material *mat = (Material *)id;
 		if (!mat->preview) mat->preview = BKE_previewimg_create();
 		prv_img = mat->preview;
 	}
 	else if (GS(id->name) == ID_TE) {
-		Tex *tex = (Tex*)id;
+		Tex *tex = (Tex *)id;
 		if (!tex->preview) tex->preview = BKE_previewimg_create();
 		prv_img = tex->preview;
 	}
 	else if (GS(id->name) == ID_WO) {
-		World *wo = (World*)id;
+		World *wo = (World *)id;
 		if (!wo->preview) wo->preview = BKE_previewimg_create();
 		prv_img = wo->preview;
 	}
 	else if (GS(id->name) == ID_LA) {
-		Lamp *la  = (Lamp*)id;
+		Lamp *la  = (Lamp *)id;
 		if (!la->preview) la->preview = BKE_previewimg_create();
 		prv_img = la->preview;
 	}
 	else if (GS(id->name) == ID_IM) {
-		Image *img  = (Image*)id;
+		Image *img  = (Image *)id;
 		if (!img->preview) img->preview = BKE_previewimg_create();
 		prv_img = img->preview;
 	}
 	else if (GS(id->name) == ID_BR) {
-		Brush *br  = (Brush*)id;
+		Brush *br  = (Brush *)id;
 		if (!br->preview) br->preview = BKE_previewimg_create();
 		prv_img = br->preview;
 	}
@@ -240,29 +244,29 @@ PreviewImage* BKE_previewimg_get(ID *id)
 
 void BKE_icon_changed(int id)
 {
-	Icon* icon = NULL;
+	Icon *icon = NULL;
 	
 	if (!id || G.background) return;
 
 	icon = BLI_ghash_lookup(gIcons, SET_INT_IN_POINTER(id));
 	
 	if (icon) {
-		PreviewImage *prv = BKE_previewimg_get((ID*)icon->obj);
+		PreviewImage *prv = BKE_previewimg_get((ID *)icon->obj);
 
 		/* all previews changed */
 		if (prv) {
 			int i;
-			for (i=0; i<NUM_ICON_SIZES; ++i) {
+			for (i = 0; i < NUM_ICON_SIZES; ++i) {
 				prv->changed[i] = 1;
 				prv->changed_timestamp[i]++;
 			}
 		}
-	}	
+	}
 }
 
-int BKE_icon_getid(struct ID* id)
+int BKE_icon_getid(struct ID *id)
 {
-	Icon* new_icon = NULL;
+	Icon *new_icon = NULL;
 
 	if (!id || G.background)
 		return 0;
@@ -291,9 +295,9 @@ int BKE_icon_getid(struct ID* id)
 	return id->icon_id;
 }
 
-Icon* BKE_icon_get(int icon_id)
+Icon *BKE_icon_get(int icon_id)
 {
-	Icon* icon = NULL;
+	Icon *icon = NULL;
 
 	icon = BLI_ghash_lookup(gIcons, SET_INT_IN_POINTER(icon_id));
 	
@@ -305,9 +309,9 @@ Icon* BKE_icon_get(int icon_id)
 	return icon;
 }
 
-void BKE_icon_set(int icon_id, struct Icon* icon)
+void BKE_icon_set(int icon_id, struct Icon *icon)
 {
-	Icon* old_icon = NULL;
+	Icon *old_icon = NULL;
 
 	old_icon = BLI_ghash_lookup(gIcons, SET_INT_IN_POINTER(icon_id));
 
@@ -319,10 +323,10 @@ void BKE_icon_set(int icon_id, struct Icon* icon)
 	BLI_ghash_insert(gIcons, SET_INT_IN_POINTER(icon_id), icon);
 }
 
-void BKE_icon_delete(struct ID* id)
+void BKE_icon_delete(struct ID *id)
 {
 
-	if (!id->icon_id) return; /* no icon defined for library object */
+	if (!id->icon_id) return;  /* no icon defined for library object */
 
 	BLI_ghash_remove(gIcons, SET_INT_IN_POINTER(id->icon_id), NULL, icon_free);
 	id->icon_id = 0;
