@@ -27,17 +27,18 @@
  *  \ingroup bke
  */
 
- 
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
 #include <string.h>
 
+#include "BLI_utildefines.h"
+#include "BLI_string.h"
+#include "BLI_listbase.h"
+
 #include "BKE_idprop.h"
 #include "BKE_library.h"
 
-#include "BLI_blenlib.h"
-
 #include "MEM_guardedalloc.h"
 
 /* IDPropertyTemplate is a union in DNA_ID.h */
@@ -47,16 +48,16 @@ static char idp_size_table[] = {
 	1, /*strings*/
 	sizeof(int),
 	sizeof(float),
-	sizeof(float)*3, /*Vector type, deprecated*/
-	sizeof(float)*16, /*Matrix type, deprecated*/
+	sizeof(float) * 3, /*Vector type, deprecated*/
+	sizeof(float) * 16, /*Matrix type, deprecated*/
 	0, /*arrays don't have a fixed size*/
 	sizeof(ListBase), /*Group type*/
-	sizeof(void*),
+	sizeof(void *),
 	sizeof(double)
 };
 
 /* ------------Property Array Type ----------- */
-#define GETPROP(prop, i) (((IDProperty*)(prop)->data.pointer)+(i))
+#define GETPROP(prop, i) (((IDProperty *)(prop)->data.pointer) + (i))
 
 /* --------- property array type -------------*/
 
@@ -78,10 +79,10 @@ IDProperty *IDP_CopyIDPArray(IDProperty *array)
 	IDProperty *narray = MEM_mallocN(sizeof(IDProperty), "IDP_CopyIDPArray"), *tmp;
 	int i;
 
-	*narray= *array;
+	*narray = *array;
 
 	narray->data.pointer = MEM_dupallocN(array->data.pointer);
-	for (i=0; i<narray->len; i++) {
+	for (i = 0; i < narray->len; i++) {
 		/* ok, the copy functions always allocate a new structure,
 		 * which doesn't work here.  instead, simply copy the
 		 * contents of the new structure into the array cell,
@@ -100,7 +101,7 @@ void IDP_FreeIDPArray(IDProperty *prop)
 {
 	int i;
 	
-	for (i=0; i<prop->len; i++)
+	for (i = 0; i < prop->len; i++)
 		IDP_FreeProperty(GETPROP(prop, i));
 
 	if (prop->data.pointer)
@@ -122,24 +123,23 @@ IDProperty *IDP_GetIndexArray(IDProperty *prop, int index)
 	return GETPROP(prop, index);
 }
 
-IDProperty *IDP_AppendArray(IDProperty *prop, IDProperty *item)
+void IDP_AppendArray(IDProperty *prop, IDProperty *item)
 {
-	IDP_ResizeIDPArray(prop, prop->len+1);
-	IDP_SetIndexArray(prop, prop->len-1, item);
-	return item;
+	IDP_ResizeIDPArray(prop, prop->len + 1);
+	IDP_SetIndexArray(prop, prop->len - 1, item);
 }
 
 void IDP_ResizeIDPArray(IDProperty *prop, int newlen)
 {
 	void *newarr;
-	int newsize=newlen;
+	int newsize = newlen;
 
 	/*first check if the array buffer size has room*/
 	/*if newlen is 200 chars less then totallen, reallocate anyway*/
 	if (newlen <= prop->totallen && prop->totallen - newlen < 200) {
 		int i;
 
-		for (i=newlen; i<prop->len; i++)
+		for (i = newlen; i < prop->len; i++)
 			IDP_FreeProperty(GETPROP(prop, i));
 
 		prop->len = newlen;
@@ -156,18 +156,18 @@ void IDP_ResizeIDPArray(IDProperty *prop, int newlen)
 	 */
 	newsize = (newsize >> 3) + (newsize < 9 ? 3 : 6) + newsize;
 
-	newarr = MEM_callocN(sizeof(IDProperty)*newsize, "idproperty array resized");
+	newarr = MEM_callocN(sizeof(IDProperty) * newsize, "idproperty array resized");
 	if (newlen >= prop->len) {
-		/* newlen is bigger*/
-		memcpy(newarr, prop->data.pointer, prop->len*sizeof(IDProperty));
+		/* newlen is bigger */
+		memcpy(newarr, prop->data.pointer, prop->len * sizeof(IDProperty));
 	}
 	else {
 		int i;
-		/* newlen is smaller*/
-		for (i=newlen; i<prop->len; i++) {
+		/* newlen is smaller */
+		for (i = newlen; i < prop->len; i++) {
 			IDP_FreeProperty(GETPROP(prop, i));
 		}
-		memcpy(newarr, prop->data.pointer, newlen*sizeof(IDProperty));
+		memcpy(newarr, prop->data.pointer, newlen * sizeof(IDProperty));
 	}
 
 	if (prop->data.pointer)
@@ -185,21 +185,21 @@ static void idp_resize_group_array(IDProperty *prop, int newlen, void *newarr)
 
 	if (newlen >= prop->len) {
 		/* bigger */
-		IDProperty **array= newarr;
+		IDProperty **array = newarr;
 		IDPropertyTemplate val;
 		int a;
 
-		for (a=prop->len; a<newlen; a++) {
+		for (a = prop->len; a < newlen; a++) {
 			val.i = 0; /* silence MSVC warning about uninitialized var when debugging */
-			array[a]= IDP_New(IDP_GROUP, &val, "IDP_ResizeArray group");
+			array[a] = IDP_New(IDP_GROUP, &val, "IDP_ResizeArray group");
 		}
 	}
 	else {
 		/* smaller */
-		IDProperty **array= prop->data.pointer;
+		IDProperty **array = prop->data.pointer;
 		int a;
 
-		for (a=newlen; a<prop->len; a++) {
+		for (a = newlen; a < prop->len; a++) {
 			IDP_FreeProperty(array[a]);
 			MEM_freeN(array[a]);
 		}
@@ -210,7 +210,7 @@ static void idp_resize_group_array(IDProperty *prop, int newlen, void *newarr)
 void IDP_ResizeArray(IDProperty *prop, int newlen)
 {
 	void *newarr;
-	int newsize=newlen;
+	int newsize = newlen;
 
 	/*first check if the array buffer size has room*/
 	/*if newlen is 200 chars less then totallen, reallocate anyway*/
@@ -230,16 +230,16 @@ void IDP_ResizeArray(IDProperty *prop, int newlen)
 	 */
 	newsize = (newsize >> 3) + (newsize < 9 ? 3 : 6) + newsize;
 
-	newarr = MEM_callocN(idp_size_table[(int)prop->subtype]*newsize, "idproperty array resized");
+	newarr = MEM_callocN(idp_size_table[(int)prop->subtype] * newsize, "idproperty array resized");
 	if (newlen >= prop->len) {
-		/* newlen is bigger*/
-		memcpy(newarr, prop->data.pointer, prop->len*idp_size_table[(int)prop->subtype]);
+		/* newlen is bigger */
+		memcpy(newarr, prop->data.pointer, prop->len * idp_size_table[(int)prop->subtype]);
 		idp_resize_group_array(prop, newlen, newarr);
 	}
 	else {
-		/* newlen is smaller*/
+		/* newlen is smaller */
 		idp_resize_group_array(prop, newlen, newarr);
-		memcpy(newarr, prop->data.pointer, newlen*idp_size_table[(int)prop->subtype]);
+		memcpy(newarr, prop->data.pointer, newlen * idp_size_table[(int)prop->subtype]);
 	}
 
 	MEM_freeN(prop->data.pointer);
@@ -257,8 +257,8 @@ void IDP_FreeArray(IDProperty *prop)
 }
 
 
- static IDProperty *idp_generic_copy(IDProperty *prop)
- {
+static IDProperty *idp_generic_copy(IDProperty *prop)
+{
 	IDProperty *newp = MEM_callocN(sizeof(IDProperty), "IDProperty array dup");
 
 	BLI_strncpy(newp->name, prop->name, MAX_IDPROP_NAME);
@@ -268,7 +268,7 @@ void IDP_FreeArray(IDProperty *prop)
 	newp->data.val2 = prop->data.val2;
 
 	return newp;
- }
+}
 
 static IDProperty *IDP_CopyArray(IDProperty *prop)
 {
@@ -278,11 +278,11 @@ static IDProperty *IDP_CopyArray(IDProperty *prop)
 		newp->data.pointer = MEM_dupallocN(prop->data.pointer);
 
 		if (prop->type == IDP_GROUP) {
-			IDProperty **array= newp->data.pointer;
+			IDProperty **array = newp->data.pointer;
 			int a;
 
-			for (a=0; a<prop->len; a++)
-				array[a]= IDP_CopyProperty(array[a]);
+			for (a = 0; a < prop->len; a++)
+				array[a] = IDP_CopyProperty(array[a]);
 		}
 	}
 	newp->len = prop->len;
@@ -292,17 +292,6 @@ static IDProperty *IDP_CopyArray(IDProperty *prop)
 	return newp;
 }
 
-/*taken from readfile.c*/
-#define SWITCH_LONGINT(a) { \
-	char s_i, *p_i; \
-	p_i= (char *)&(a);  \
-	s_i=p_i[0]; p_i[0]=p_i[7]; p_i[7]=s_i; \
-	s_i=p_i[1]; p_i[1]=p_i[6]; p_i[6]=s_i; \
-	s_i=p_i[2]; p_i[2]=p_i[5]; p_i[5]=s_i; \
-	s_i=p_i[3]; p_i[3]=p_i[4]; p_i[4]=s_i; }
-
-
-
 /* ---------- String Type ------------ */
 IDProperty *IDP_NewString(const char *st, const char *name, int maxlen)
 {
@@ -350,7 +339,7 @@ void IDP_AssignString(IDProperty *prop, const char *st, int maxlen)
 	int stlen = strlen(st);
 
 	if (maxlen > 0 && maxlen < stlen)
-		stlen= maxlen;
+		stlen = maxlen;
 
 	if (prop->subtype == IDP_STRING_SUB_BYTE) {
 		IDP_ResizeArray(prop, stlen);
@@ -396,14 +385,14 @@ void IDP_FreeString(IDProperty *prop)
 
 void IDP_LinkID(IDProperty *prop, ID *id)
 {
-	if (prop->data.pointer) ((ID*)prop->data.pointer)->us--;
+	if (prop->data.pointer) ((ID *)prop->data.pointer)->us--;
 	prop->data.pointer = id;
 	id_us_plus(id);
 }
 
 void IDP_UnlinkID(IDProperty *prop)
 {
-	((ID*)prop->data.pointer)->us--;
+	((ID *)prop->data.pointer)->us--;
 }
 
 /*-------- Group Functions -------*/
@@ -414,7 +403,7 @@ static IDProperty *IDP_CopyGroup(IDProperty *prop)
 	IDProperty *newp = idp_generic_copy(prop), *link;
 	newp->len = prop->len;
 	
-	for (link=prop->data.group.first; link; link=link->next) {
+	for (link = prop->data.group.first; link; link = link->next) {
 		BLI_addtail(&newp->data.group, IDP_CopyProperty(link));
 	}
 
@@ -426,22 +415,22 @@ static IDProperty *IDP_CopyGroup(IDProperty *prop)
 void IDP_SyncGroupValues(IDProperty *dest, IDProperty *src)
 {
 	IDProperty *other, *prop;
-	for (prop=src->data.group.first; prop; prop=prop->next) {
-		other= BLI_findstring(&dest->data.group, prop->name, offsetof(IDProperty, name));
-		if (other && prop->type==other->type) {
+	for (prop = src->data.group.first; prop; prop = prop->next) {
+		other = BLI_findstring(&dest->data.group, prop->name, offsetof(IDProperty, name));
+		if (other && prop->type == other->type) {
 			switch (prop->type) {
 				case IDP_INT:
 				case IDP_FLOAT:
 				case IDP_DOUBLE:
-					other->data= prop->data;
+					other->data = prop->data;
 					break;
 				case IDP_GROUP:
 					IDP_SyncGroupValues(other, prop);
 					break;
 				default:
 				{
-					IDProperty *tmp= other;
-					IDProperty *copy= IDP_CopyProperty(prop);
+					IDProperty *tmp = other;
+					IDProperty *copy = IDP_CopyProperty(prop);
 
 					BLI_insertlinkafter(&dest->data.group, other, copy);
 					BLI_remlink(&dest->data.group, tmp);
@@ -460,8 +449,8 @@ void IDP_SyncGroupValues(IDProperty *dest, IDProperty *src)
 void IDP_ReplaceGroupInGroup(IDProperty *dest, IDProperty *src)
 {
 	IDProperty *loop, *prop;
-	for (prop=src->data.group.first; prop; prop=prop->next) {
-		for (loop=dest->data.group.first; loop; loop=loop->next) {
+	for (prop = src->data.group.first; prop; prop = prop->next) {
+		for (loop = dest->data.group.first; loop; loop = loop->next) {
 			if (strcmp(loop->name, prop->name) == 0) {
 				IDProperty *copy = IDP_CopyProperty(prop);
 
@@ -489,12 +478,12 @@ void IDP_ReplaceGroupInGroup(IDProperty *dest, IDProperty *src)
 void IDP_ReplaceInGroup(IDProperty *group, IDProperty *prop)
 {
 	IDProperty *loop;
-	if ((loop= IDP_GetPropertyFromGroup(group, prop->name))) {
+	if ((loop = IDP_GetPropertyFromGroup(group, prop->name))) {
 		BLI_insertlink(&group->data.group, loop, prop);
 		
 		BLI_remlink(&group->data.group, loop);
 		IDP_FreeProperty(loop);
-		MEM_freeN(loop);			
+		MEM_freeN(loop);
 	}
 	else {
 		group->len++;
@@ -502,6 +491,30 @@ void IDP_ReplaceInGroup(IDProperty *group, IDProperty *prop)
 	}
 }
 
+/*
+ * If a property is missing in \a dest, add it.
+ */
+void IDP_MergeGroup(IDProperty *dest, IDProperty *src, const int do_overwrite)
+{
+	IDProperty *prop;
+
+	if (do_overwrite) {
+		for (prop = src->data.group.first; prop; prop = prop->next) {
+			IDProperty *copy = IDP_CopyProperty(prop);
+			IDP_ReplaceInGroup(dest, copy);
+		}
+	}
+	else {
+		for (prop = src->data.group.first; prop; prop = prop->next) {
+			if (IDP_GetPropertyFromGroup(dest, prop->name) == NULL) {
+				IDProperty *copy = IDP_CopyProperty(prop);
+				dest->len++;
+				BLI_addtail(&dest->data.group, copy);
+			}
+		}
+	}
+}
+
 /* returns 0 if an id property with the same name exists and it failed,
  * or 1 if it succeeded in adding to the group.*/
 int IDP_AddToGroup(IDProperty *group, IDProperty *prop)
@@ -539,7 +552,7 @@ IDProperty *IDP_GetPropertyFromGroup(IDProperty *prop, const char *name)
 
 IDProperty *IDP_GetPropertyTypeFromGroup(IDProperty *prop, const char *name, const char type)
 {
-	IDProperty *idprop= IDP_GetPropertyFromGroup(prop, name);
+	IDProperty *idprop = IDP_GetPropertyFromGroup(prop, name);
 	return (idprop && idprop->type == type) ? idprop : NULL;
 }
 
@@ -553,20 +566,20 @@ void *IDP_GetGroupIterator(IDProperty *prop)
 	IDPIter *iter = MEM_callocN(sizeof(IDPIter), "IDPIter");
 	iter->next = prop->data.group.first;
 	iter->parent = prop;
-	return (void*) iter;
+	return (void *) iter;
 }
 
 IDProperty *IDP_GroupIterNext(void *vself)
 {
-	IDPIter *self = (IDPIter*) vself;
-	Link *next = (Link*) self->next;
+	IDPIter *self = (IDPIter *) vself;
+	Link *next = (Link *) self->next;
 	if (self->next == NULL) {
 		MEM_freeN(self);
 		return NULL;
 	}
 
 	self->next = next->next;
-	return (void*) next;
+	return (void *) next;
 }
 
 void IDP_FreeIterBeforeEnd(void *vself)
@@ -581,8 +594,7 @@ void IDP_FreeIterBeforeEnd(void *vself)
 static void IDP_FreeGroup(IDProperty *prop)
 {
 	IDProperty *loop;
-	for (loop=prop->data.group.first; loop; loop=loop->next)
-	{
+	for (loop = prop->data.group.first; loop; loop = loop->next) {
 		IDP_FreeProperty(loop);
 	}
 	BLI_freelistN(&prop->data.group);
@@ -603,7 +615,9 @@ IDProperty *IDP_CopyProperty(IDProperty *prop)
 
 IDProperty *IDP_GetProperties(ID *id, int create_if_needed)
 {
-	if (id->properties) return id->properties;
+	if (id->properties) {
+		return id->properties;
+	}
 	else {
 		if (create_if_needed) {
 			id->properties = MEM_callocN(sizeof(IDProperty), "IDProperty");
@@ -617,64 +631,81 @@ IDProperty *IDP_GetProperties(ID *id, int create_if_needed)
 	}
 }
 
-int IDP_EqualsProperties(IDProperty *prop1, IDProperty *prop2)
+/**
+ * \param is_strict When FALSE treat missing items as a match */
+int IDP_EqualsProperties_ex(IDProperty *prop1, IDProperty *prop2, const int is_strict)
 {
 	if (prop1 == NULL && prop2 == NULL)
 		return 1;
 	else if (prop1 == NULL || prop2 == NULL)
-		return 0;
+		return is_strict ? 0 : 1;
 	else if (prop1->type != prop2->type)
 		return 0;
 
-	if (prop1->type == IDP_INT)
-		return (IDP_Int(prop1) == IDP_Int(prop2));
-	else if (prop1->type == IDP_FLOAT)
-		return (IDP_Float(prop1) == IDP_Float(prop2));
-	else if (prop1->type == IDP_DOUBLE)
-		return (IDP_Double(prop1) == IDP_Double(prop2));
-	else if (prop1->type == IDP_STRING)
-		return ((prop1->len == prop2->len) && strncmp(IDP_String(prop1), IDP_String(prop2), prop1->len) == 0);
-	else if (prop1->type == IDP_ARRAY) {
-		if (prop1->len == prop2->len && prop1->subtype == prop2->subtype)
-			return memcmp(IDP_Array(prop1), IDP_Array(prop2), idp_size_table[(int)prop1->subtype]*prop1->len);
-		else
-			return 0;
-	}
-	else if (prop1->type == IDP_GROUP) {
-		IDProperty *link1, *link2;
-
-		if (BLI_countlist(&prop1->data.group) != BLI_countlist(&prop2->data.group))
-			return 0;
-
-		for (link1=prop1->data.group.first; link1; link1=link1->next) {
-			link2= IDP_GetPropertyFromGroup(prop2, link1->name);
-
-			if (!IDP_EqualsProperties(link1, link2))
+	switch (prop1->type) {
+		case IDP_INT:
+			return (IDP_Int(prop1) == IDP_Int(prop2));
+		case IDP_FLOAT:
+			return (IDP_Float(prop1) == IDP_Float(prop2));
+		case IDP_DOUBLE:
+			return (IDP_Double(prop1) == IDP_Double(prop2));
+		case IDP_STRING:
+			return ((prop1->len == prop2->len) && strncmp(IDP_String(prop1), IDP_String(prop2), prop1->len) == 0);
+		case IDP_ARRAY:
+			if (prop1->len == prop2->len && prop1->subtype == prop2->subtype) {
+				return memcmp(IDP_Array(prop1), IDP_Array(prop2), idp_size_table[(int)prop1->subtype] * prop1->len);
+			}
+			else {
 				return 0;
+			}
+		case IDP_GROUP:
+		{
+			IDProperty *link1, *link2;
+
+			if (is_strict && prop1->len != prop2->len)
+				return 0;
+
+			for (link1 = prop1->data.group.first; link1; link1 = link1->next) {
+				link2 = IDP_GetPropertyFromGroup(prop2, link1->name);
+
+				if (!IDP_EqualsProperties_ex(link1, link2, is_strict))
+					return 0;
+			}
+
+			return 1;
 		}
+		case IDP_IDPARRAY:
+		{
+			IDProperty *array1 = IDP_IDPArray(prop1);
+			IDProperty *array2 = IDP_IDPArray(prop2);
+			int i;
 
-		return 1;
-	}
-	else if (prop1->type == IDP_IDPARRAY) {
-		IDProperty *array1= IDP_IDPArray(prop1);
-		IDProperty *array2= IDP_IDPArray(prop2);
-		int i;
-
-		if (prop1->len != prop2->len)
-			return 0;
-		
-		for (i=0; i<prop1->len; i++)
-			if (!IDP_EqualsProperties(&array1[i], &array2[i]))
+			if (prop1->len != prop2->len)
 				return 0;
+
+			for (i = 0; i < prop1->len; i++)
+				if (!IDP_EqualsProperties(&array1[i], &array2[i]))
+					return 0;
+			return 1;
+		}
+		default:
+			/* should never get here */
+			BLI_assert(0);
+			break;
 	}
-	
+
 	return 1;
 }
 
+int IDP_EqualsProperties(IDProperty *prop1, IDProperty *prop2)
+{
+	return IDP_EqualsProperties_ex(prop1, prop2, TRUE);
+}
+
 /* 'val' is never NULL, don't check */
 IDProperty *IDP_New(const int type, const IDPropertyTemplate *val, const char *name)
 {
-	IDProperty *prop=NULL;
+	IDProperty *prop = NULL;
 
 	switch (type) {
 		case IDP_INT:
@@ -683,15 +714,15 @@ IDProperty *IDP_New(const int type, const IDPropertyTemplate *val, const char *n
 			break;
 		case IDP_FLOAT:
 			prop = MEM_callocN(sizeof(IDProperty), "IDProperty float");
-			*(float*)&prop->data.val = val->f;
+			*(float *)&prop->data.val = val->f;
 			break;
 		case IDP_DOUBLE:
 			prop = MEM_callocN(sizeof(IDProperty), "IDProperty float");
-			*(double*)&prop->data.val = val->d;
-			break;		
+			*(double *)&prop->data.val = val->d;
+			break;
 		case IDP_ARRAY:
 		{
-			/*for now, we only support float and int and double arrays*/
+			/* for now, we only support float and int and double arrays */
 			if ( (val->array.type == IDP_FLOAT) ||
 			     (val->array.type == IDP_INT) ||
 			     (val->array.type == IDP_DOUBLE) ||
@@ -700,7 +731,7 @@ IDProperty *IDP_New(const int type, const IDPropertyTemplate *val, const char *n
 				prop = MEM_callocN(sizeof(IDProperty), "IDProperty array");
 				prop->subtype = val->array.type;
 				if (val->array.len)
-					prop->data.pointer = MEM_callocN(idp_size_table[val->array.type]*val->array.len, "id property array");
+					prop->data.pointer = MEM_callocN(idp_size_table[val->array.type] * val->array.len, "id property array");
 				prop->len = prop->totallen = val->array.len;
 				break;
 			}
@@ -725,7 +756,7 @@ IDProperty *IDP_New(const int type, const IDPropertyTemplate *val, const char *n
 					prop->len = prop->totallen = val->string.len;
 					memcpy(prop->data.pointer, st, val->string.len);
 				}
-				prop->subtype= IDP_STRING_SUB_BYTE;
+				prop->subtype = IDP_STRING_SUB_BYTE;
 			}
 			else {
 				if (st == NULL) {
@@ -739,7 +770,7 @@ IDProperty *IDP_New(const int type, const IDPropertyTemplate *val, const char *n
 					prop->len = prop->totallen = stlen;
 					memcpy(prop->data.pointer, st, stlen);
 				}
-				prop->subtype= IDP_STRING_SUB_UTF8;
+				prop->subtype = IDP_STRING_SUB_UTF8;
 			}
 			break;
 		}
@@ -783,6 +814,13 @@ void IDP_FreeProperty(IDProperty *prop)
 	}
 }
 
+void IDP_ClearProperty(IDProperty *prop)
+{
+	IDP_FreeProperty(prop);
+	prop->data.pointer = NULL;
+	prop->len = prop->totallen = 0;
+}
+
 /* Unlinks any IDProperty<->ID linkage that might be going on.
  * note: currently unused.*/
 void IDP_UnlinkProperty(IDProperty *prop)
