@@ -33,9 +33,12 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "DNA_anim_types.h"
 #include "DNA_lamp_types.h"
 #include "DNA_material_types.h"
+#include "DNA_node_types.h"
 #include "DNA_object_types.h"
+#include "DNA_scene_types.h"
 #include "DNA_texture_types.h"
 
 #include "BLI_listbase.h"
@@ -51,34 +54,34 @@
 #include "BKE_main.h"
 #include "BKE_node.h"
 
-void *add_lamp(const char *name)
+Lamp *BKE_lamp_add(Main *bmain, const char *name)
 {
 	Lamp *la;
 	
-	la=  alloc_libblock(&G.main->lamp, ID_LA, name);
+	la =  BKE_libblock_alloc(&bmain->lamp, ID_LA, name);
 	
-	la->r= la->g= la->b= la->k= 1.0f;
-	la->haint= la->energy= 1.0f;
-	la->dist= 25.0f;
-	la->spotsize= 45.0f;
-	la->spotblend= 0.15f;
-	la->att2= 1.0f;
-	la->mode= LA_SHAD_BUF;
-	la->bufsize= 512;
-	la->clipsta= 0.5f;
-	la->clipend= 40.0f;
-	la->shadspotsize= 45.0f;
-	la->samp= 3;
-	la->bias= 1.0f;
-	la->soft= 3.0f;
-	la->compressthresh= 0.05f;
-	la->ray_samp= la->ray_sampy= la->ray_sampz= 1; 
-	la->area_size=la->area_sizey=la->area_sizez= 1.0f; 
-	la->buffers= 1;
-	la->buftype= LA_SHADBUF_HALFWAY;
+	la->r = la->g = la->b = la->k = 1.0f;
+	la->haint = la->energy = 1.0f;
+	la->dist = 25.0f;
+	la->spotsize = 45.0f;
+	la->spotblend = 0.15f;
+	la->att2 = 1.0f;
+	la->mode = LA_SHAD_BUF;
+	la->bufsize = 512;
+	la->clipsta = 0.5f;
+	la->clipend = 40.0f;
+	la->shadspotsize = 45.0f;
+	la->samp = 3;
+	la->bias = 1.0f;
+	la->soft = 3.0f;
+	la->compressthresh = 0.05f;
+	la->ray_samp = la->ray_sampy = la->ray_sampz = 1;
+	la->area_size = la->area_sizey = la->area_sizez = 0.1f;
+	la->buffers = 1;
+	la->buftype = LA_SHADBUF_HALFWAY;
 	la->ray_samp_method = LA_SAMP_HALTON;
 	la->adapt_thresh = 0.001f;
-	la->preview=NULL;
+	la->preview = NULL;
 	la->falloff_type = LA_FALLOFF_INVSQUARE;
 	la->curfalloff = curvemapping_add(1, 0.0f, 1.0f, 1.0f, 0.0f);
 	la->sun_effect_type = 0;
@@ -92,25 +95,26 @@ void *add_lamp(const char *name)
 	la->atm_extinction_factor = 1.0f;
 	la->atm_distance_factor = 1.0f;
 	la->sun_intensity = 1.0f;
-	la->skyblendtype= MA_RAMP_ADD;
-	la->skyblendfac= 1.0f;
-	la->sky_colorspace= BLI_XYZ_CIE;
-	la->sky_exposure= 1.0f;
+	la->skyblendtype = MA_RAMP_ADD;
+	la->skyblendfac = 1.0f;
+	la->sky_colorspace = BLI_XYZ_CIE;
+	la->sky_exposure = 1.0f;
+	la->shadow_frustum_size = 10.0f;
 	
 	curvemapping_initialize(la->curfalloff);
 	return la;
 }
 
-Lamp *copy_lamp(Lamp *la)
+Lamp *BKE_lamp_copy(Lamp *la)
 {
 	Lamp *lan;
 	int a;
 	
-	lan= copy_libblock(&la->id);
+	lan = BKE_libblock_copy(&la->id);
 
-	for (a=0; a<MAX_MTEX; a++) {
+	for (a = 0; a < MAX_MTEX; a++) {
 		if (lan->mtex[a]) {
-			lan->mtex[a]= MEM_mallocN(sizeof(MTex), "copylamptex");
+			lan->mtex[a] = MEM_mallocN(sizeof(MTex), "copylamptex");
 			memcpy(lan->mtex[a], la->mtex[a], sizeof(MTex));
 			id_us_plus((ID *)lan->mtex[a]->tex);
 		}
@@ -119,7 +123,7 @@ Lamp *copy_lamp(Lamp *la)
 	lan->curfalloff = curvemapping_copy(la->curfalloff);
 
 	if (la->nodetree)
-		lan->nodetree= ntreeCopyTree(la->nodetree);
+		lan->nodetree = ntreeCopyTree(la->nodetree);
 	
 	if (la->preview)
 		lan->preview = BKE_previewimg_copy(la->preview);
@@ -132,12 +136,12 @@ Lamp *localize_lamp(Lamp *la)
 	Lamp *lan;
 	int a;
 	
-	lan= copy_libblock(&la->id);
+	lan = BKE_libblock_copy(&la->id);
 	BLI_remlink(&G.main->lamp, lan);
 
-	for (a=0; a<MAX_MTEX; a++) {
+	for (a = 0; a < MAX_MTEX; a++) {
 		if (lan->mtex[a]) {
-			lan->mtex[a]= MEM_mallocN(sizeof(MTex), "localize_lamp");
+			lan->mtex[a] = MEM_mallocN(sizeof(MTex), "localize_lamp");
 			memcpy(lan->mtex[a], la->mtex[a], sizeof(MTex));
 			/* free lamp decrements */
 			id_us_plus((ID *)lan->mtex[a]->tex);
@@ -147,71 +151,71 @@ Lamp *localize_lamp(Lamp *la)
 	lan->curfalloff = curvemapping_copy(la->curfalloff);
 
 	if (la->nodetree)
-		lan->nodetree= ntreeLocalize(la->nodetree);
+		lan->nodetree = ntreeLocalize(la->nodetree);
 	
-	lan->preview= NULL;
+	lan->preview = NULL;
 	
 	return lan;
 }
 
-void make_local_lamp(Lamp *la)
+void BKE_lamp_make_local(Lamp *la)
 {
-	Main *bmain= G.main;
+	Main *bmain = G.main;
 	Object *ob;
-	int is_local= FALSE, is_lib= FALSE;
+	int is_local = FALSE, is_lib = FALSE;
 
 	/* - only lib users: do nothing
 	 * - only local users: set flag
 	 * - mixed: make copy
 	 */
 	
-	if (la->id.lib==NULL) return;
-	if (la->id.us==1) {
+	if (la->id.lib == NULL) return;
+	if (la->id.us == 1) {
 		id_clear_lib_data(bmain, &la->id);
 		return;
 	}
 	
-	ob= bmain->object.first;
+	ob = bmain->object.first;
 	while (ob) {
-		if (ob->data==la) {
-			if (ob->id.lib) is_lib= TRUE;
-			else is_local= TRUE;
+		if (ob->data == la) {
+			if (ob->id.lib) is_lib = TRUE;
+			else is_local = TRUE;
 		}
-		ob= ob->id.next;
+		ob = ob->id.next;
 	}
 	
 	if (is_local && is_lib == FALSE) {
 		id_clear_lib_data(bmain, &la->id);
 	}
 	else if (is_local && is_lib) {
-		Lamp *la_new= copy_lamp(la);
-		la_new->id.us= 0;
+		Lamp *la_new = BKE_lamp_copy(la);
+		la_new->id.us = 0;
 
 		/* Remap paths of new ID using old library as base. */
 		BKE_id_lib_local_paths(bmain, la->id.lib, &la_new->id);
 
-		ob= bmain->object.first;
+		ob = bmain->object.first;
 		while (ob) {
-			if (ob->data==la) {
+			if (ob->data == la) {
 				
-				if (ob->id.lib==NULL) {
-					ob->data= la_new;
+				if (ob->id.lib == NULL) {
+					ob->data = la_new;
 					la_new->id.us++;
 					la->id.us--;
 				}
 			}
-			ob= ob->id.next;
+			ob = ob->id.next;
 		}
 	}
 }
 
-void free_lamp(Lamp *la)
+void BKE_lamp_free(Lamp *la)
 {
 	MTex *mtex;
 	int a;
 
-	for (a=0; a<MAX_MTEX; a++) {
-		mtex= la->mtex[a];
+	for (a = 0; a < MAX_MTEX; a++) {
+		mtex = la->mtex[a];
 		if (mtex && mtex->tex) mtex->tex->id.us--;
 		if (mtex) MEM_freeN(mtex);
 	}
@@ -231,3 +235,38 @@ void free_lamp(Lamp *la)
 	la->id.icon_id = 0;
 }
 
+/* Calculate all drivers for lamps, see material_drivers_update for why this is a bad hack */
+
+static void lamp_node_drivers_update(Scene *scene, bNodeTree *ntree, float ctime)
+{
+	bNode *node;
+
+	/* nodetree itself */
+	if (ntree->adt && ntree->adt->drivers.first)
+		BKE_animsys_evaluate_animdata(scene, &ntree->id, ntree->adt, ctime, ADT_RECALC_DRIVERS);
+	
+	/* nodes */
+	for (node = ntree->nodes.first; node; node = node->next)
+		if (node->id && node->type == NODE_GROUP)
+			lamp_node_drivers_update(scene, (bNodeTree *)node->id, ctime);
+}
+
+void lamp_drivers_update(Scene *scene, Lamp *la, float ctime)
+{
+	/* Prevent infinite recursion by checking (and tagging the lamp) as having been visited already
+	 * (see BKE_scene_update_tagged()). This assumes la->id.flag & LIB_DOIT isn't set by anything else
+	 * in the meantime... [#32017] */
+	if (la->id.flag & LIB_DOIT)
+		return;
+	else
+		la->id.flag |= LIB_DOIT;
+	
+	/* lamp itself */
+	if (la->adt && la->adt->drivers.first)
+		BKE_animsys_evaluate_animdata(scene, &la->id, la->adt, ctime, ADT_RECALC_DRIVERS);
+	
+	/* nodes */
+	if (la->nodetree)
+		lamp_node_drivers_update(scene, la->nodetree, ctime);
+}
+
