@@ -52,6 +52,7 @@
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
 #include "DNA_movieclip_types.h"
+#include "DNA_node_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
@@ -66,18 +67,25 @@
 
 #include "BKE_animsys.h"
 #include "BKE_constraint.h"
+#include "BKE_colortools.h"
 #include "BKE_library.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
-#include "BKE_utildefines.h"
 #include "BKE_movieclip.h"
-#include "BKE_image.h"	/* openanim */
+#include "BKE_node.h"
+#include "BKE_image.h"  /* openanim */
 #include "BKE_tracking.h"
+#include "BKE_sequencer.h"
 
+#include "IMB_colormanagement.h"
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
 #include "IMB_moviecache.h"
 
+#ifdef WITH_OPENEXR
+#include "intern/openexr/openexr_multi.h"
+#endif
+
 /*********************** movieclip buffer loaders *************************/
 
 static int sequence_guess_offset(const char *full_name, int head_len, unsigned short numlen)
@@ -154,11 +162,12 @@ static void get_sequence_fname(MovieClip *clip, int framenr, char *name)
 	BLI_stringdec(name, head, tail, &numlen);
 
 	/* movieclips always points to first image from sequence,
-	 * autoguess offset for now. could be something smarter in the future */
-	offset= sequence_guess_offset(clip->name, strlen(head), numlen);
+	 * autoguess offset for now. could be something smarter in the future
+	 */
+	offset = sequence_guess_offset(clip->name, strlen(head), numlen);
 
 	if (numlen)
-		BLI_stringenc(name, head, tail, numlen, offset + framenr - 1);
+		BLI_stringenc(name, head, tail, numlen, offset + framenr - clip->start_frame + clip->frame_offset);
 	else
 		BLI_strncpy(name, clip->name, sizeof(clip->name));
 
@@ -170,6 +179,7 @@ static void get_proxy_fname(MovieClip *clip, int proxy_render_size, int undistor
 {
 	int size = rendersize_to_number(proxy_render_size);
 	char dir[FILE_MAX], clipdir[FILE_MAX], clipfile[FILE_MAX];
+	int proxynr = framenr - clip->start_frame + 1 + clip->frame_offset;
 
 	BLI_split_dirfile(clip->name, clipdir, clipfile, FILE_MAX, FILE_MAX);
 
@@ -181,9 +191,9 @@ static void get_proxy_fname(MovieClip *clip, int proxy_render_size, int undistor
 	}
 
 	if (undistorted)
-		BLI_snprintf(name, FILE_MAX, "%s/%s/proxy_%d_undistorted/%08d", dir, clipfile, size, framenr);
+		BLI_snprintf(name, FILE_MAX, "%s/%s/proxy_%d_undistorted/%08d", dir, clipfile, size, proxynr);
 	else
-		BLI_snprintf(name, FILE_MAX, "%s/%s/proxy_%d/%08d", dir, clipfile, size, framenr);
+		BLI_snprintf(name, FILE_MAX, "%s/%s/proxy_%d/%08d", dir, clipfile, size, proxynr);
 
 	BLI_path_abs(name, G.main->name);
 	BLI_path_frame(name, 1, 0);
@@ -195,20 +205,35 @@ static ImBuf *movieclip_load_sequence_file(MovieClip *clip, MovieClipUser *user,
 {
 	struct ImBuf *ibuf;
 	char name[FILE_MAX];
-	int loadflag, use_proxy= 0;
+	int loadflag, use_proxy = FALSE;
+	char *colorspace;
 
 	use_proxy = (flag & MCLIP_USE_PROXY) && user->render_size != MCLIP_PROXY_RENDER_SIZE_FULL;
 	if (use_proxy) {
 		int undistort = user->render_flag & MCLIP_PROXY_RENDER_UNDISTORT;
 		get_proxy_fname(clip, user->render_size, undistort, framenr, name);
-	}
-	else
-		get_sequence_fname(clip, framenr, name);
 
-	loadflag = IB_rect|IB_multilayer;
+		/* proxies were built using default color space settings */
+		colorspace = NULL;
+	}
+	else {
+		get_sequence_fname(clip, framenr, name);
+		colorspace = clip->colorspace_settings.name;
+	}
+
+	loadflag = IB_rect | IB_multilayer | IB_alphamode_detect;
 
 	/* read ibuf */
-	ibuf = IMB_loadiffname(name, loadflag);
+	ibuf = IMB_loadiffname(name, loadflag, colorspace);
+
+#ifdef WITH_OPENEXR
+	if (ibuf) {
+		if (ibuf->ftype == OPENEXR && ibuf->userdata) {
+			IMB_exr_close(ibuf->userdata);
+			ibuf->userdata = NULL;
+		}
+	}
+#endif
 
 	return ibuf;
 }
@@ -222,7 +247,7 @@ static void movieclip_open_anim_file(MovieClip *clip)
 		BLI_path_abs(str, ID_BLEND_PATH(G.main, &clip->id));
 
 		/* FIXME: make several stream accessible in image editor, too */
-		clip->anim = openanim(str, IB_rect, 0);
+		clip->anim = openanim(str, IB_rect, 0, clip->colorspace_settings.name);
 
 		if (clip->anim) {
 			if (clip->flag & MCLIP_USE_PROXY_CUSTOM_DIR) {
@@ -248,9 +273,9 @@ static ImBuf *movieclip_load_movie_file(MovieClip *clip, MovieClipUser *user, in
 		int fra;
 
 		dur = IMB_anim_get_duration(clip->anim, tc);
-		fra = framenr - 1;
+		fra = framenr - clip->start_frame + clip->frame_offset;
 
-		if (fra<0)
+		if (fra < 0)
 			fra = 0;
 
 		if (fra > (dur - 1))
@@ -287,7 +312,7 @@ static void movieclip_calc_length(MovieClip *clip)
 				get_sequence_fname(clip, framenr, name);
 
 				if (!BLI_exists(name)) {
-					clip->len = framenr + 1;
+					clip->len = framenr;
 					break;
 				}
 
@@ -312,7 +337,7 @@ typedef struct MovieClipCache {
 		/* cache for undistorted shot */
 		float principal[2];
 		float k1, k2, k3;
-		short undistoriton_used;
+		short undistortion_used;
 
 		int proxy;
 		short render_flag;
@@ -320,6 +345,8 @@ typedef struct MovieClipCache {
 
 	/* cache for stable shot */
 	struct {
+		ImBuf *reference_ibuf;
+
 		ImBuf *ibuf;
 		int framenr;
 		int postprocess_flag;
@@ -336,9 +363,13 @@ typedef struct MovieClipImBufCacheKey {
 	short render_flag;
 } MovieClipImBufCacheKey;
 
+typedef struct MovieClipCachePriorityData {
+	int framenr;
+} MovieClipCachePriorityData;
+
 static void moviecache_keydata(void *userkey, int *framenr, int *proxy, int *render_flags)
 {
-	MovieClipImBufCacheKey *key = (MovieClipImBufCacheKey*)userkey;
+	MovieClipImBufCacheKey *key = (MovieClipImBufCacheKey *)userkey;
 
 	*framenr = key->framenr;
 	*proxy = key->proxy;
@@ -347,7 +378,7 @@ static void moviecache_keydata(void *userkey, int *framenr, int *proxy, int *ren
 
 static unsigned int moviecache_hashhash(const void *keyv)
 {
-	MovieClipImBufCacheKey *key = (MovieClipImBufCacheKey*)keyv;
+	MovieClipImBufCacheKey *key = (MovieClipImBufCacheKey *)keyv;
 	int rval = key->framenr;
 
 	return rval;
@@ -355,8 +386,8 @@ static unsigned int moviecache_hashhash(const void *keyv)
 
 static int moviecache_hashcmp(const void *av, const void *bv)
 {
-	const MovieClipImBufCacheKey *a = (MovieClipImBufCacheKey*)av;
-	const MovieClipImBufCacheKey *b = (MovieClipImBufCacheKey*)bv;
+	const MovieClipImBufCacheKey *a = (MovieClipImBufCacheKey *)av;
+	const MovieClipImBufCacheKey *b = (MovieClipImBufCacheKey *)bv;
 
 	if (a->framenr < b->framenr)
 		return -1;
@@ -376,6 +407,32 @@ static int moviecache_hashcmp(const void *av, const void *bv)
 	return 0;
 }
 
+static void *moviecache_getprioritydata(void *key_v)
+{
+	MovieClipImBufCacheKey *key = (MovieClipImBufCacheKey *) key_v;
+	MovieClipCachePriorityData *priority_data;
+
+	priority_data = MEM_callocN(sizeof(priority_data), "movie cache clip priority data");
+	priority_data->framenr = key->framenr;
+
+	return priority_data;
+}
+
+static int moviecache_getitempriority(void *last_userkey_v, void *priority_data_v)
+{
+	MovieClipImBufCacheKey *last_userkey = (MovieClipImBufCacheKey *) last_userkey_v;
+	MovieClipCachePriorityData *priority_data = (MovieClipCachePriorityData *) priority_data_v;
+
+	return -abs(last_userkey->framenr - priority_data->framenr);
+}
+
+static void moviecache_prioritydeleter(void *priority_data_v)
+{
+	MovieClipCachePriorityData *priority_data = (MovieClipCachePriorityData *) priority_data_v;
+
+	MEM_freeN(priority_data);
+}
+
 static ImBuf *get_imbuf_cache(MovieClip *clip, MovieClipUser *user, int flag)
 {
 	if (clip->cache) {
@@ -403,10 +460,20 @@ static void put_imbuf_cache(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf, i
 	MovieClipImBufCacheKey key;
 
 	if (!clip->cache) {
+		struct MovieCache *moviecache;
+
+		// char cache_name[64];
+		// BLI_snprintf(cache_name, sizeof(cache_name), "movie %s", clip->id.name);
+
 		clip->cache = MEM_callocN(sizeof(MovieClipCache), "movieClipCache");
 
-		clip->cache->moviecache = IMB_moviecache_create(sizeof(MovieClipImBufCacheKey), moviecache_hashhash,
-				moviecache_hashcmp, moviecache_keydata);
+		moviecache = IMB_moviecache_create("movieclip", sizeof(MovieClipImBufCacheKey), moviecache_hashhash, moviecache_hashcmp);
+
+		IMB_moviecache_set_getdata_callback(moviecache, moviecache_keydata);
+		IMB_moviecache_set_priority_callback(moviecache, moviecache_getprioritydata, moviecache_getitempriority,
+		                                     moviecache_prioritydeleter);
+
+		clip->cache->moviecache = moviecache;
 	}
 
 	key.framenr = user->framenr;
@@ -426,15 +493,16 @@ static void put_imbuf_cache(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf, i
 /*********************** common functions *************************/
 
 /* only image block itself */
-static MovieClip *movieclip_alloc(const char *name)
+static MovieClip *movieclip_alloc(Main *bmain, const char *name)
 {
 	MovieClip *clip;
 
-	clip = alloc_libblock(&G.main->movieclip, ID_MC, name);
+	clip = BKE_libblock_alloc(&bmain->movieclip, ID_MC, name);
 
-	clip->aspx = clip->aspy= 1.0f;
+	clip->aspx = clip->aspy = 1.0f;
 
-	BKE_tracking_init_settings(&clip->tracking);
+	BKE_tracking_settings_init(&clip->tracking);
+	BKE_color_managed_colorspace_settings_init(&clip->colorspace_settings);
 
 	clip->proxy.build_size_flag = IMB_PROXY_25;
 	clip->proxy.build_tc_flag = IMB_TC_RECORD_RUN |
@@ -443,32 +511,69 @@ static MovieClip *movieclip_alloc(const char *name)
 	                            IMB_TC_RECORD_RUN_NO_GAPS;
 	clip->proxy.quality = 90;
 
+	clip->start_frame = 1;
+	clip->frame_offset = 0;
+
 	return clip;
 }
 
+static void movieclip_load_get_szie(MovieClip *clip)
+{
+	int width, height;
+	MovieClipUser user = {0};
+
+	user.framenr = 1;
+	BKE_movieclip_get_size(clip, &user, &width, &height);
+
+	if (width && height) {
+		clip->tracking.camera.principal[0] = ((float)width) / 2.0f;
+		clip->tracking.camera.principal[1] = ((float)height) / 2.0f;
+	}
+	else {
+		clip->lastsize[0] = clip->lastsize[1] = IMG_SIZE_FALLBACK;
+	}
+}
+
+static void detect_clip_source(MovieClip *clip)
+{
+	ImBuf *ibuf;
+	char name[FILE_MAX];
+
+	BLI_strncpy(name, clip->name, sizeof(name));
+	BLI_path_abs(name, G.main->name);
+
+	ibuf = IMB_testiffname(name, IB_rect | IB_multilayer);
+	if (ibuf) {
+		clip->source = MCLIP_SRC_SEQUENCE;
+		IMB_freeImBuf(ibuf);
+	}
+	else {
+		clip->source = MCLIP_SRC_MOVIE;
+	}
+}
+
 /* checks if image was already loaded, then returns same image
  * otherwise creates new.
  * does not load ibuf itself
  * pass on optional frame for #name images */
-MovieClip *BKE_add_movieclip_file(const char *name)
+MovieClip *BKE_movieclip_file_add(Main *bmain, const char *name)
 {
 	MovieClip *clip;
-	MovieClipUser user;
-	int file, len, width, height;
+	int file, len;
 	const char *libname;
 	char str[FILE_MAX], strtest[FILE_MAX];
 
 	BLI_strncpy(str, name, sizeof(str));
-	BLI_path_abs(str, G.main->name);
+	BLI_path_abs(str, bmain->name);
 
 	/* exists? */
-	file = BLI_open(str, O_BINARY|O_RDONLY,0);
+	file = BLI_open(str, O_BINARY | O_RDONLY, 0);
 	if (file == -1)
 		return NULL;
 	close(file);
 
 	/* ** first search an identical clip ** */
-	for (clip = G.main->movieclip.first; clip; clip = clip->id.next) {
+	for (clip = bmain->movieclip.first; clip; clip = clip->id.next) {
 		BLI_strncpy(strtest, clip->name, sizeof(clip->name));
 		BLI_path_abs(strtest, G.main->name);
 
@@ -489,19 +594,14 @@ MovieClip *BKE_add_movieclip_file(const char *name)
 		len--;
 	libname = name + len;
 
-	clip = movieclip_alloc(libname);
+	clip = movieclip_alloc(bmain, libname);
 	BLI_strncpy(clip->name, name, sizeof(clip->name));
 
-	if (BLI_testextensie_array(name, imb_ext_movie))
-		clip->source = MCLIP_SRC_MOVIE;
-	else
-		clip->source = MCLIP_SRC_SEQUENCE;
+	detect_clip_source(clip);
 
-	user.framenr = 1;
-	BKE_movieclip_get_size(clip, &user, &width, &height);
-	if (width && height) {
-		clip->tracking.camera.principal[0] = ((float)width) / 2.0f;
-		clip->tracking.camera.principal[1] = ((float)height) / 2.0f;
+	movieclip_load_get_szie(clip);
+	if (clip->lastsize[0]) {
+		int width = clip->lastsize[0];
 
 		clip->tracking.camera.focal = 24.0f * width / clip->tracking.camera.sensor_width;
 	}
@@ -517,7 +617,7 @@ static void real_ibuf_size(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf, in
 	*height = ibuf->y;
 
 	if (clip->flag & MCLIP_USE_PROXY) {
-		switch(user->render_size) {
+		switch (user->render_size) {
 			case MCLIP_PROXY_RENDER_SIZE_25:
 				(*width) *= 4;
 				(*height) *= 4;
@@ -529,8 +629,8 @@ static void real_ibuf_size(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf, in
 				break;
 
 			case MCLIP_PROXY_RENDER_SIZE_75:
-				*width = ((float)*width)*4.0f/3.0f;
-				*height = ((float)*height)*4.0f/3.0f;
+				*width = ((float)*width) * 4.0f / 3.0f;
+				*height = ((float)*height) * 4.0f / 3.0f;
 				break;
 		}
 	}
@@ -540,43 +640,32 @@ static ImBuf *get_undistorted_ibuf(MovieClip *clip, struct MovieDistortion *dist
 {
 	ImBuf *undistibuf;
 
-	/* XXX: because of #27997 do not use float buffers to undistort,
-	 *      otherwise, undistorted proxy can be darker than it should */
-	imb_freerectfloatImBuf(ibuf);
-
 	if (distortion)
 		undistibuf = BKE_tracking_distortion_exec(distortion, &clip->tracking, ibuf, ibuf->x, ibuf->y, 0.0f, 1);
 	else
-		undistibuf = BKE_tracking_undistort(&clip->tracking, ibuf, ibuf->x, ibuf->y, 0.0f);
-
-	if (undistibuf->userflags & IB_RECT_INVALID) {
-		ibuf->userflags &= ~IB_RECT_INVALID;
-		IMB_rect_from_float(undistibuf);
-	}
+		undistibuf = BKE_tracking_undistort_frame(&clip->tracking, ibuf, ibuf->x, ibuf->y, 0.0f);
 
 	IMB_scaleImBuf(undistibuf, ibuf->x, ibuf->y);
 
 	return undistibuf;
 }
 
-static int need_undistortion_postprocess(MovieClipUser *user, int flag)
+static int need_undistortion_postprocess(MovieClipUser *user)
 {
 	int result = 0;
 
 	/* only full undistorted render can be used as on-fly undistorting image */
-	if (flag & MCLIP_USE_PROXY) {
-		result |= (user->render_size == MCLIP_PROXY_RENDER_SIZE_FULL) &&
-		          (user->render_flag & MCLIP_PROXY_RENDER_UNDISTORT) != 0;
-	}
+	result |= (user->render_size == MCLIP_PROXY_RENDER_SIZE_FULL) &&
+	          (user->render_flag & MCLIP_PROXY_RENDER_UNDISTORT) != 0;
 
 	return result;
 }
 
-static int need_postprocessed_frame(MovieClipUser *user, int flag, int postprocess_flag)
+static int need_postprocessed_frame(MovieClipUser *user, int postprocess_flag)
 {
 	int result = postprocess_flag;
 
-	result |= need_undistortion_postprocess(user, flag);
+	result |= need_undistortion_postprocess(user);
 
 	return result;
 }
@@ -623,11 +712,11 @@ static ImBuf *get_postprocessed_cached_frame(MovieClip *clip, MovieClipUser *use
 	if (cache->postprocessed.flag != postprocess_flag)
 		return NULL;
 
-	if (need_undistortion_postprocess(user, flag)) {
+	if (need_undistortion_postprocess(user)) {
 		if (!check_undistortion_cache_flags(clip))
 			return NULL;
 	}
-	else if (cache->postprocessed.undistoriton_used)
+	else if (cache->postprocessed.undistortion_used)
 		return NULL;
 
 	IMB_refImBuf(cache->postprocessed.ibuf);
@@ -635,15 +724,13 @@ static ImBuf *get_postprocessed_cached_frame(MovieClip *clip, MovieClipUser *use
 	return cache->postprocessed.ibuf;
 }
 
-static ImBuf *put_postprocessed_frame_to_cache(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf, int flag, int postprocess_flag)
+static ImBuf *put_postprocessed_frame_to_cache(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf,
+                                               int flag, int postprocess_flag)
 {
 	MovieClipCache *cache = clip->cache;
 	MovieTrackingCamera *camera = &clip->tracking.camera;
 	ImBuf *postproc_ibuf = NULL;
 
-	if (cache->postprocessed.ibuf)
-		IMB_freeImBuf(cache->postprocessed.ibuf);
-
 	cache->postprocessed.framenr = user->framenr;
 	cache->postprocessed.flag = postprocess_flag;
 
@@ -656,38 +743,35 @@ static ImBuf *put_postprocessed_frame_to_cache(MovieClip *clip, MovieClipUser *u
 		cache->postprocessed.render_flag = 0;
 	}
 
-	if (need_undistortion_postprocess(user, flag)) {
+	if (need_undistortion_postprocess(user)) {
 		copy_v2_v2(cache->postprocessed.principal, camera->principal);
 		copy_v3_v3(&cache->postprocessed.k1, &camera->k1);
-		cache->postprocessed.undistoriton_used = TRUE;
-		postproc_ibuf= get_undistorted_ibuf(clip, NULL, ibuf);
+		cache->postprocessed.undistortion_used = TRUE;
+		postproc_ibuf = get_undistorted_ibuf(clip, NULL, ibuf);
 	}
 	else {
-		cache->postprocessed.undistoriton_used = FALSE;
+		cache->postprocessed.undistortion_used = FALSE;
 	}
 
 	if (postprocess_flag) {
 		int disable_red   = postprocess_flag & MOVIECLIP_DISABLE_RED,
 		    disable_green = postprocess_flag & MOVIECLIP_DISABLE_GREEN,
-			disable_blue  = postprocess_flag & MOVIECLIP_DISABLE_BLUE,
-			grayscale     = postprocess_flag & MOVIECLIP_PREVIEW_GRAYSCALE;
+		    disable_blue  = postprocess_flag & MOVIECLIP_DISABLE_BLUE,
+		    grayscale     = postprocess_flag & MOVIECLIP_PREVIEW_GRAYSCALE;
 
 		if (!postproc_ibuf)
 			postproc_ibuf = IMB_dupImBuf(ibuf);
 
 		if (disable_red || disable_green || disable_blue || grayscale)
-			BKE_tracking_disable_imbuf_channels(postproc_ibuf, disable_red, disable_green, disable_blue, 1);
+			BKE_tracking_disable_channels(postproc_ibuf, disable_red, disable_green, disable_blue, 1);
 	}
 
 	IMB_refImBuf(postproc_ibuf);
 
-	cache->postprocessed.ibuf = postproc_ibuf;
+	if (cache->postprocessed.ibuf)
+		IMB_freeImBuf(cache->postprocessed.ibuf);
 
-	if (cache->stabilized.ibuf) {
-		/* force stable buffer be re-calculated */
-		IMB_freeImBuf(cache->stabilized.ibuf);
-		cache->stabilized.ibuf = NULL;
-	}
+	cache->postprocessed.ibuf = postproc_ibuf;
 
 	return postproc_ibuf;
 }
@@ -703,15 +787,15 @@ static ImBuf *movieclip_get_postprocessed_ibuf(MovieClip *clip, MovieClipUser *u
 	BLI_lock_thread(LOCK_MOVIECLIP);
 
 	/* try to obtain cached postprocessed frame first */
-	if (need_postprocessed_frame(user, flag, postprocess_flag)) {
-		ibuf= get_postprocessed_cached_frame(clip, user, flag, postprocess_flag);
+	if (need_postprocessed_frame(user, postprocess_flag)) {
+		ibuf = get_postprocessed_cached_frame(clip, user, flag, postprocess_flag);
 
 		if (!ibuf)
 			need_postprocess = TRUE;
 	}
 
 	if (!ibuf)
-		ibuf= get_imbuf_cache(clip, user, flag);
+		ibuf = get_imbuf_cache(clip, user, flag);
 
 	if (!ibuf) {
 		int use_sequence = FALSE;
@@ -763,7 +847,8 @@ ImBuf *BKE_movieclip_get_postprocessed_ibuf(MovieClip *clip, MovieClipUser *user
 	return movieclip_get_postprocessed_ibuf(clip, user, clip->flag, postprocess_flag, 0);
 }
 
-static ImBuf *get_stable_cached_frame(MovieClip *clip, MovieClipUser *user, int framenr, int postprocess_flag)
+static ImBuf *get_stable_cached_frame(MovieClip *clip, MovieClipUser *user, ImBuf *reference_ibuf,
+                                      int framenr, int postprocess_flag)
 {
 	MovieClipCache *cache = clip->cache;
 	MovieTracking *tracking = &clip->tracking;
@@ -771,6 +856,7 @@ static ImBuf *get_stable_cached_frame(MovieClip *clip, MovieClipUser *user, int
 	float tloc[2], tscale, tangle;
 	short proxy = IMB_PROXY_NONE;
 	int render_flag = 0;
+	int clip_framenr = BKE_movieclip_remap_scene_to_clip_frame(clip, framenr);
 
 	if (clip->flag & MCLIP_USE_PROXY) {
 		proxy = rendersize_to_proxy(user, clip->flag);
@@ -781,6 +867,9 @@ static ImBuf *get_stable_cached_frame(MovieClip *clip, MovieClipUser *user, int
 	if (!cache->stabilized.ibuf || cache->stabilized.framenr != framenr)
 		return NULL;
 
+	if (cache->stabilized.reference_ibuf != reference_ibuf)
+		return NULL;
+
 	/* cached ibuf used different proxy settings */
 	if (cache->stabilized.render_flag != render_flag || cache->stabilized.proxy != proxy)
 		return NULL;
@@ -797,12 +886,12 @@ static ImBuf *get_stable_cached_frame(MovieClip *clip, MovieClipUser *user, int
 
 	stableibuf = cache->stabilized.ibuf;
 
-	BKE_tracking_stabilization_data(&clip->tracking, framenr, stableibuf->x, stableibuf->y, tloc, &tscale, &tangle);
+	BKE_tracking_stabilization_data_get(&clip->tracking, clip_framenr, stableibuf->x, stableibuf->y, tloc, &tscale, &tangle);
 
 	/* check for stabilization parameters */
 	if (tscale != cache->stabilized.scale ||
-	   tangle != cache->stabilized.angle ||
-	   !equals_v2v2(tloc, cache->stabilized.loc))
+	    tangle != cache->stabilized.angle ||
+	    !equals_v2v2(tloc, cache->stabilized.loc))
 	{
 		return NULL;
 	}
@@ -819,13 +908,9 @@ static ImBuf *put_stabilized_frame_to_cache(MovieClip *clip, MovieClipUser *user
 	MovieTracking *tracking = &clip->tracking;
 	ImBuf *stableibuf;
 	float tloc[2], tscale, tangle;
+	int clip_framenr = BKE_movieclip_remap_scene_to_clip_frame(clip, framenr);
 
-	if (cache->stabilized.ibuf)
-		IMB_freeImBuf(cache->stabilized.ibuf);
-
-	stableibuf = BKE_tracking_stabilize(&clip->tracking, framenr, ibuf, tloc, &tscale, &tangle);
-
-	cache->stabilized.ibuf= stableibuf;
+	stableibuf = BKE_tracking_stabilize_frame(&clip->tracking, clip_framenr, ibuf, tloc, &tscale, &tangle);
 
 	copy_v2_v2(cache->stabilized.loc, tloc);
 
@@ -846,12 +931,18 @@ static ImBuf *put_stabilized_frame_to_cache(MovieClip *clip, MovieClipUser *user
 
 	cache->stabilized.postprocess_flag = postprocess_flag;
 
+	if (cache->stabilized.ibuf)
+		IMB_freeImBuf(cache->stabilized.ibuf);
+
+	cache->stabilized.ibuf = stableibuf;
+
 	IMB_refImBuf(stableibuf);
 
 	return stableibuf;
 }
 
-ImBuf *BKE_movieclip_get_stable_ibuf(MovieClip *clip, MovieClipUser *user, float loc[2], float *scale, float *angle, int postprocess_flag)
+ImBuf *BKE_movieclip_get_stable_ibuf(MovieClip *clip, MovieClipUser *user, float loc[2], float *scale, float *angle,
+                                     int postprocess_flag)
 {
 	ImBuf *ibuf, *stableibuf = NULL;
 	int framenr = user->framenr;
@@ -862,9 +953,9 @@ ImBuf *BKE_movieclip_get_stable_ibuf(MovieClip *clip, MovieClipUser *user, float
 		return NULL;
 
 	if (clip->tracking.stabilization.flag & TRACKING_2D_STABILIZATION) {
-		MovieClipCache *cache= clip->cache;
+		MovieClipCache *cache = clip->cache;
 
-		stableibuf = get_stable_cached_frame(clip, user, framenr, postprocess_flag);
+		stableibuf = get_stable_cached_frame(clip, user, ibuf, framenr, postprocess_flag);
 
 		if (!stableibuf)
 			stableibuf = put_stabilized_frame_to_cache(clip, user, ibuf, framenr, postprocess_flag);
@@ -873,20 +964,20 @@ ImBuf *BKE_movieclip_get_stable_ibuf(MovieClip *clip, MovieClipUser *user, float
 			copy_v2_v2(loc, cache->stabilized.loc);
 
 		if (scale)
-			*scale= cache->stabilized.scale;
+			*scale = cache->stabilized.scale;
 
 		if (angle)
-			*angle= cache->stabilized.angle;
+			*angle = cache->stabilized.angle;
 	}
 	else {
 		if (loc)
 			zero_v2(loc);
 
 		if (scale)
-			*scale= 1.0f;
+			*scale = 1.0f;
 
 		if (angle)
-			*angle= 0.0f;
+			*angle = 0.0f;
 
 		stableibuf = ibuf;
 	}
@@ -914,7 +1005,17 @@ int BKE_movieclip_has_frame(MovieClip *clip, MovieClipUser *user)
 
 void BKE_movieclip_get_size(MovieClip *clip, MovieClipUser *user, int *width, int *height)
 {
+#if 0
+	/* originally was needed to support image sequences with different image dimensions,
+	 * which might be useful for such things as reconstruction of unordered image sequence,
+	 * or painting/rotoscoping of non-equal-sized images, but this ended up in unneeded
+	 * cache lookups and even unwanted non-proxied files loading when doing mask parenting,
+	 * so let's disable this for now and assume image sequence consists of images with
+	 * equal sizes (sergey)
+	 */
 	if (user->framenr == clip->lastframe) {
+#endif
+	if (clip->lastsize[0] != 0 && clip->lastsize[1] != 0) {
 		*width = clip->lastsize[0];
 		*height = clip->lastsize[1];
 	}
@@ -925,14 +1026,22 @@ void BKE_movieclip_get_size(MovieClip *clip, MovieClipUser *user, int *width, in
 			real_ibuf_size(clip, user, ibuf, width, height);
 		}
 		else {
-			*width = 0;
-			*height = 0;
+			*width = clip->lastsize[0];
+			*height = clip->lastsize[1];
 		}
 
 		if (ibuf)
 			IMB_freeImBuf(ibuf);
 	}
 }
+void BKE_movieclip_get_size_fl(MovieClip *clip, MovieClipUser *user, float size[2])
+{
+	int width, height;
+	BKE_movieclip_get_size(clip, user, &width, &height);
+
+	size[0] = (float)width;
+	size[1] = (float)height;
+}
 
 int BKE_movieclip_get_duration(MovieClip *clip)
 {
@@ -943,9 +1052,9 @@ int BKE_movieclip_get_duration(MovieClip *clip)
 	return clip->len;
 }
 
-void BKE_movieclip_aspect(MovieClip *clip, float *aspx, float *aspy)
+void BKE_movieclip_get_aspect(MovieClip *clip, float *aspx, float *aspy)
 {
-	*aspx = *aspy = 1.0;
+	*aspx = 1.0;
 
 	/* x is always 1 */
 	*aspy = clip->aspy / clip->aspx / clip->tracking.camera.pixel_aspect;
@@ -1002,12 +1111,24 @@ void BKE_movieclip_reload(MovieClip *clip)
 	clip->tracking.stabilization.ok = FALSE;
 
 	/* update clip source */
-	if (BLI_testextensie_array(clip->name, imb_ext_movie))
-		clip->source = MCLIP_SRC_MOVIE;
-	else
-		clip->source = MCLIP_SRC_SEQUENCE;
+	detect_clip_source(clip);
+
+	clip->lastsize[0] = clip->lastsize[1] = 0;
+	movieclip_load_get_szie(clip);
 
 	movieclip_calc_length(clip);
+
+	/* same as for image update -- don't use notifiers because they are not 100% sure to succeeded
+	 * (node trees which are not currently visible wouldn't be refreshed)
+	 */
+	{
+		Scene *scene;
+		for (scene = G.main->scene.first; scene; scene = scene->id.next) {
+			if (scene->nodetree) {
+				nodeUpdateID(scene->nodetree, &clip->id);
+			}
+		}
+	}
 }
 
 void BKE_movieclip_update_scopes(MovieClip *clip, MovieClipUser *user, MovieClipScopes *scopes)
@@ -1020,15 +1141,25 @@ void BKE_movieclip_update_scopes(MovieClip *clip, MovieClipUser *user, MovieClip
 		scopes->track_preview = NULL;
 	}
 
+	if (scopes->track_search) {
+		IMB_freeImBuf(scopes->track_search);
+		scopes->track_search = NULL;
+	}
+
 	scopes->marker = NULL;
 	scopes->track = NULL;
+	scopes->track_locked = TRUE;
 
 	if (clip) {
-		MovieTrackingTrack *act_track = BKE_tracking_active_track(&clip->tracking);
+		MovieTrackingTrack *act_track = BKE_tracking_track_get_active(&clip->tracking);
 
 		if (act_track) {
 			MovieTrackingTrack *track = act_track;
-			MovieTrackingMarker *marker = BKE_tracking_get_marker(track, user->framenr);
+			int framenr = BKE_movieclip_remap_scene_to_clip_frame(clip, user->framenr);
+			MovieTrackingMarker *marker = BKE_tracking_marker_get(track, framenr);
+
+			scopes->marker = marker;
+			scopes->track = track;
 
 			if (marker->flag & MARKER_DISABLED) {
 				scopes->track_disabled = TRUE;
@@ -1039,45 +1170,58 @@ void BKE_movieclip_update_scopes(MovieClip *clip, MovieClipUser *user, MovieClip
 				scopes->track_disabled = FALSE;
 
 				if (ibuf && (ibuf->rect || ibuf->rect_float)) {
-					ImBuf *tmpibuf;
+					ImBuf *search_ibuf;
 					MovieTrackingMarker undist_marker = *marker;
 
 					if (user->render_flag & MCLIP_PROXY_RENDER_UNDISTORT) {
 						int width, height;
-						float aspy= 1.0f / clip->tracking.camera.pixel_aspect;
+						float aspy = 1.0f / clip->tracking.camera.pixel_aspect;
 
 						BKE_movieclip_get_size(clip, user, &width, &height);
 
 						undist_marker.pos[0] *= width;
-						undist_marker.pos[1] *= height*aspy;
+						undist_marker.pos[1] *= height * aspy;
 
-						BKE_tracking_invert_intrinsics(&clip->tracking, undist_marker.pos, undist_marker.pos);
+						BKE_tracking_undistort_v2(&clip->tracking, undist_marker.pos, undist_marker.pos);
 
 						undist_marker.pos[0] /= width;
-						undist_marker.pos[1] /= height*aspy;
+						undist_marker.pos[1] /= height * aspy;
 					}
 
-					/* NOTE: margin should be kept in sync with value from ui_draw_but_TRACKPREVIEW */
-					tmpibuf= BKE_tracking_get_pattern_imbuf(ibuf, track, &undist_marker, 3 /* margin */,
-							1 /* anchor */, scopes->track_pos, NULL);
+					search_ibuf = BKE_tracking_get_search_imbuf(ibuf, track, &undist_marker, TRUE, TRUE);
 
-					if (tmpibuf->rect_float)
-						IMB_rect_from_float(tmpibuf);
+					if (search_ibuf) {
+						if (!search_ibuf->rect_float) {
+							/* sampling happens in float buffer */
+							IMB_float_from_rect(search_ibuf);
+						}
 
-					if (tmpibuf->rect)
-						scopes->track_preview= tmpibuf;
-					else
-						IMB_freeImBuf(tmpibuf);
+						scopes->track_search = search_ibuf;
+					}
+
+					scopes->undist_marker = undist_marker;
+
+					scopes->frame_width = ibuf->x;
+					scopes->frame_height = ibuf->y;
+
+					scopes->use_track_mask = track->flag & TRACK_PREVIEW_ALPHA;
 				}
 
 				IMB_freeImBuf(ibuf);
 			}
 
-			if ((track->flag & TRACK_LOCKED)==0) {
-				scopes->marker = marker;
-				scopes->track = track;
-				scopes->slide_scale[0] = track->pat_max[0]-track->pat_min[0];
-				scopes->slide_scale[1] = track->pat_max[1]-track->pat_min[1];
+			if ((track->flag & TRACK_LOCKED) == 0) {
+				float pat_min[2], pat_max[2];
+
+				scopes->track_locked = FALSE;
+
+				/* XXX: would work fine with non-transformed patterns, but would likely fail
+				 *      with transformed patterns, but that would be easier to debug when
+				 *      we'll have real pattern sampling (at least to test) */
+				BKE_tracking_marker_pattern_minmax(marker, pat_min, pat_max);
+
+				scopes->slide_scale[0] = pat_max[0] - pat_min[0];
+				scopes->slide_scale[1] = pat_max[1] - pat_min[1];
 			}
 		}
 	}
@@ -1149,14 +1293,16 @@ void BKE_movieclip_build_proxy_frame(MovieClip *clip, int clip_flag, struct Movi
 	}
 }
 
-void free_movieclip(MovieClip *clip)
+void BKE_movieclip_free(MovieClip *clip)
 {
+	BKE_sequencer_clear_movieclip_in_clipboard(clip);
+
 	free_buffers(clip);
 
 	BKE_tracking_free(&clip->tracking);
 }
 
-void unlink_movieclip(Main *bmain, MovieClip *clip)
+void BKE_movieclip_unlink(Main *bmain, MovieClip *clip)
 {
 	bScreen *scr;
 	ScrArea *area;
@@ -1195,7 +1341,7 @@ void unlink_movieclip(Main *bmain, MovieClip *clip)
 		bConstraint *con;
 
 		for (con = ob->constraints.first; con; con = con->next) {
-			bConstraintTypeInfo *cti = constraint_get_typeinfo(con);
+			bConstraintTypeInfo *cti = BKE_constraint_get_typeinfo(con);
 
 			if (cti->type == CONSTRAINT_TYPE_FOLLOWTRACK) {
 				bFollowTrackConstraint *data = (bFollowTrackConstraint *) con->data;
@@ -1218,5 +1364,20 @@ void unlink_movieclip(Main *bmain, MovieClip *clip)
 		}
 	}
 
+	{
+		bNodeTreeType *treetype = ntreeGetType(NTREE_COMPOSIT);
+		treetype->foreach_nodetree(bmain, (void *)clip, &BKE_node_tree_unlink_id_cb);
+	}
+
 	clip->id.us = 0;
 }
+
+float BKE_movieclip_remap_scene_to_clip_frame(MovieClip *clip, float framenr)
+{
+	return framenr - (float) clip->start_frame + 1.0f;
+}
+
+float BKE_movieclip_remap_clip_to_scene_frame(MovieClip *clip, float framenr)
+{
+	return framenr + (float) clip->start_frame - 1.0f;
+}
