@@ -44,7 +44,6 @@
 #include "DNA_object_types.h"
 
 #include "BLI_blenlib.h"
-#include "BKE_utildefines.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
 #include "BKE_library.h"
@@ -110,7 +109,7 @@ void init_sensor(bSensor *sens)
 	sens->data= NULL;
 	sens->pulse = 0;
 	
-	switch(sens->type) {
+	switch (sens->type) {
 	case SENS_ALWAYS:
 		sens->pulse = 0;
 		break;
@@ -211,7 +210,7 @@ void unlink_controllers(ListBase *lb)
 	bController *cont;
 	
 	for (cont= lb->first; cont; cont= cont->next)
-		unlink_controller(cont);	
+		unlink_controller(cont);
 }
 
 void free_controller(bController *cont)
@@ -272,7 +271,7 @@ void init_controller(bController *cont)
 	if (cont->data) MEM_freeN(cont->data);
 	cont->data= NULL;
 	
-	switch(cont->type) {
+	switch (cont->type) {
 	case CONT_EXPRESSION:
 		cont->data= MEM_callocN(sizeof(bExpressionCont), "expcont");
 		break;
@@ -400,7 +399,7 @@ void init_actuator(bActuator *act)
 	if (act->data) MEM_freeN(act->data);
 	act->data= NULL;
 	
-	switch(act->type) {
+	switch (act->type) {
 	case ACT_ACTION:
 	case ACT_SHAPEACTION:
 		act->data= MEM_callocN(sizeof(bActionActuator), "actionact");
@@ -537,7 +536,7 @@ void clear_sca_new_poins(void)
 	ob= G.main->object.first;
 	while (ob) {
 		clear_sca_new_poins_ob(ob);
-		ob= ob->id.next;	
+		ob= ob->id.next;
 	}
 }
 
@@ -553,7 +552,7 @@ void set_sca_new_poins_ob(Object *ob)
 		if (sens->flag & SENS_NEW) {
 			for (a=0; a<sens->totlinks; a++) {
 				if (sens->links[a] && sens->links[a]->mynew)
-					sens->links[a]= sens->links[a]->mynew;
+					sens->links[a] = sens->links[a]->mynew;
 			}
 		}
 		sens= sens->next;
@@ -564,7 +563,7 @@ void set_sca_new_poins_ob(Object *ob)
 		if (cont->flag & CONT_NEW) {
 			for (a=0; a<cont->totlinks; a++) {
 				if ( cont->links[a] && cont->links[a]->mynew)
-					cont->links[a]= cont->links[a]->mynew;
+					cont->links[a] = cont->links[a]->mynew;
 			}
 		}
 		cont= cont->next;
@@ -625,7 +624,7 @@ void set_sca_new_poins(void)
 	ob= G.main->object.first;
 	while (ob) {
 		set_sca_new_poins_ob(ob);
-		ob= ob->id.next;	
+		ob= ob->id.next;
 	}
 }
 
@@ -647,7 +646,7 @@ void sca_remove_ob_poin(Object *obt, Object *ob)
 
 	sens= obt->sensors.first;
 	while (sens) {
-		switch(sens->type) {
+		switch (sens->type) {
 		case SENS_MESSAGE:
 			ms= sens->data;
 			if (ms->fromObject==ob) ms->fromObject= NULL;
@@ -657,7 +656,7 @@ void sca_remove_ob_poin(Object *obt, Object *ob)
 
 	act= obt->actuators.first;
 	while (act) {
-		switch(act->type) {
+		switch (act->type) {
 		case ACT_CAMERA:
 			ca= act->data;
 			if (ca->ob==ob) ca->ob= NULL;
@@ -697,7 +696,7 @@ void sca_remove_ob_poin(Object *obt, Object *ob)
 			if (sta->target == ob) sta->target = NULL;
 		}
 		act= act->next;
-	}	
+	}
 }
 
 /* ******************** INTERFACE ******************* */
@@ -717,7 +716,7 @@ void sca_move_sensor(bSensor *sens_to_move, Object *ob, int move_up)
 	if (!sens) return;
 
 	/* move up */
-	if ( val==1 && sens->prev) {
+	if (val == 1 && sens->prev) {
 		for (tmp=sens->prev; tmp; tmp=tmp->prev) {
 			if (tmp->flag & SENS_VISIBLE)
 				break;
@@ -728,7 +727,7 @@ void sca_move_sensor(bSensor *sens_to_move, Object *ob, int move_up)
 		}
 	}
 	/* move down */
-	else if ( val==2 && sens->next) {
+	else if (val == 2 && sens->next) {
 		for (tmp=sens->next; tmp; tmp=tmp->next) {
 			if (tmp->flag & SENS_VISIBLE)
 				break;
@@ -756,7 +755,7 @@ void sca_move_controller(bController *cont_to_move, Object *ob, int move_up)
 	if (!cont) return;
 
 	/* move up */
-	if ( val==1 && cont->prev) {
+	if (val == 1 && cont->prev) {
 		/* locate the controller that has the same state mask but is earlier in the list */
 		tmp = cont->prev;
 		while (tmp) {
@@ -771,7 +770,7 @@ void sca_move_controller(bController *cont_to_move, Object *ob, int move_up)
 	}
 
 	/* move down */
-	else if ( val==2 && cont->next) {
+	else if (val == 2 && cont->next) {
 		tmp = cont->next;
 		while (tmp) {
 			if (tmp->state_mask & cont->state_mask) 
@@ -799,7 +798,7 @@ void sca_move_actuator(bActuator *act_to_move, Object *ob, int move_up)
 	if (!act) return;
 
 	/* move up */
-	if ( val==1 && act->prev) {
+	if (val == 1 && act->prev) {
 		/* locate the first visible actuators before this one */
 		for (tmp = act->prev; tmp; tmp=tmp->prev) {
 			if (tmp->flag & ACT_VISIBLE)
@@ -811,7 +810,7 @@ void sca_move_actuator(bActuator *act_to_move, Object *ob, int move_up)
 		}
 	}
 	/* move down */
-	else if ( val==2 && act->next) {
+	else if (val == 2 && act->next) {
 		/* locate the first visible actuators after this one */
 		for (tmp=act->next; tmp; tmp=tmp->next) {
 			if (tmp->flag & ACT_VISIBLE)
@@ -876,3 +875,20 @@ void unlink_logicbricks(void **poin, void ***ppoin, short *tot)
 		return;
 	}
 }
+
+const char *sca_state_name_get(Object *ob, short bit)
+{
+	bController *cont;
+	unsigned int mask;
+
+	mask = (1<<bit);
+	cont = ob->controllers.first;
+	while (cont) {
+		if (cont->state_mask & mask) {
+			return cont->name;
+		}
+		cont = cont->next;
+	}
+	return NULL;
+}
+
