@@ -35,7 +35,6 @@
 
 #include "BLI_math.h"
 #include "BLI_utildefines.h"
-#include "BLI_bpath.h"
 
 #include "BKE_animsys.h"
 #include "BKE_global.h"
@@ -43,11 +42,11 @@
 #include "BKE_main.h"
 #include "BKE_speaker.h"
 
-void *add_speaker(const char *name)
+void *BKE_speaker_add(Main *bmain, const char *name)
 {
 	Speaker *spk;
 
-	spk=  alloc_libblock(&G.main->speaker, ID_SPK, name);
+	spk =  BKE_libblock_alloc(&bmain->speaker, ID_SPK, name);
 
 	spk->attenuation = 1.0f;
 	spk->cone_angle_inner = 360.0f;
@@ -65,69 +64,69 @@ void *add_speaker(const char *name)
 	return spk;
 }
 
-Speaker *copy_speaker(Speaker *spk)
+Speaker *BKE_speaker_copy(Speaker *spk)
 {
 	Speaker *spkn;
 
-	spkn= copy_libblock(&spk->id);
+	spkn = BKE_libblock_copy(&spk->id);
 	if (spkn->sound)
 		spkn->sound->id.us++;
 
 	return spkn;
 }
 
-void make_local_speaker(Speaker *spk)
+void BKE_speaker_make_local(Speaker *spk)
 {
-	Main *bmain= G.main;
+	Main *bmain = G.main;
 	Object *ob;
-	int is_local= FALSE, is_lib= FALSE;
+	int is_local = FALSE, is_lib = FALSE;
 
 	/* - only lib users: do nothing
 	 * - only local users: set flag
 	 * - mixed: make copy
 	 */
 
-	if (spk->id.lib==NULL) return;
-	if (spk->id.us==1) {
+	if (spk->id.lib == NULL) return;
+	if (spk->id.us == 1) {
 		id_clear_lib_data(bmain, &spk->id);
 		return;
 	}
 
-	ob= bmain->object.first;
+	ob = bmain->object.first;
 	while (ob) {
-		if (ob->data==spk) {
-			if (ob->id.lib) is_lib= TRUE;
-			else is_local= TRUE;
+		if (ob->data == spk) {
+			if (ob->id.lib) is_lib = TRUE;
+			else is_local = TRUE;
 		}
-		ob= ob->id.next;
+		ob = ob->id.next;
 	}
 
 	if (is_local && is_lib == FALSE) {
 		id_clear_lib_data(bmain, &spk->id);
 	}
 	else if (is_local && is_lib) {
-		Speaker *spk_new= copy_speaker(spk);
-		spk_new->id.us= 0;
+		Speaker *spk_new = BKE_speaker_copy(spk);
+		spk_new->id.us = 0;
 
 		/* Remap paths of new ID using old library as base. */
 		BKE_id_lib_local_paths(bmain, spk->id.lib, &spk_new->id);
 
-		ob= bmain->object.first;
+		ob = bmain->object.first;
 		while (ob) {
-			if (ob->data==spk) {
+			if (ob->data == spk) {
 
-				if (ob->id.lib==NULL) {
-					ob->data= spk_new;
+				if (ob->id.lib == NULL) {
+					ob->data = spk_new;
 					spk_new->id.us++;
 					spk->id.us--;
 				}
 			}
-			ob= ob->id.next;
+			ob = ob->id.next;
 		}
 	}
 }
 
-void free_speaker(Speaker *spk)
+void BKE_speaker_free(Speaker *spk)
 {
 	if (spk->sound)
 		spk->sound->id.us--;
