@@ -38,35 +38,35 @@ struct HeapNode;
 typedef struct Heap Heap;
 typedef struct HeapNode HeapNode;
 
-typedef	void	(*HeapFreeFP)(void *ptr);
+typedef void (*HeapFreeFP)(void *ptr);
 
 /* Creates a new heap. BLI_memarena is used for allocating nodes. Removed nodes
  * are recycled, so memory usage will not shrink. */
-Heap*			BLI_heap_new	(void);
-void			BLI_heap_free	(Heap *heap, HeapFreeFP ptrfreefp);
+Heap           *BLI_heap_new_ex(unsigned int tot_reserve);
+Heap           *BLI_heap_new(void);
+void            BLI_heap_free(Heap *heap, HeapFreeFP ptrfreefp);
 
 /* Insert heap node with a value (often a 'cost') and pointer into the heap,
  * duplicate values are allowed. */
-HeapNode*		BLI_heap_insert		(Heap *heap, float value, void *ptr);
+HeapNode       *BLI_heap_insert(Heap *heap, float value, void *ptr);
 
 /* Remove a heap node. */
-void			BLI_heap_remove		(Heap *heap, HeapNode *node);
+void            BLI_heap_remove(Heap *heap, HeapNode *node);
 
 /* Return 0 if the heap is empty, 1 otherwise. */
-int				BLI_heap_empty		(Heap *heap);
+int             BLI_heap_is_empty(Heap *heap);
 
 /* Return the size of the heap. */
-int				BLI_heap_size		(Heap *heap);
+unsigned int    BLI_heap_size(Heap *heap);
 
 /* Return the top node of the heap. This is the node with the lowest value. */
-HeapNode*		BLI_heap_top		(Heap *heap);
+HeapNode       *BLI_heap_top(Heap *heap);
 
 /* Pop the top node off the heap and return it's pointer. */
-void*			BLI_heap_popmin		(Heap *heap);
+void           *BLI_heap_popmin(Heap *heap);
 
 /* Return the value or pointer of a heap node. */
-float			BLI_heap_node_value	(HeapNode *heap);
-void*			BLI_heap_node_ptr	(HeapNode *heap);
-
-#endif
+float           BLI_heap_node_value(HeapNode *heap);
+void           *BLI_heap_node_ptr(HeapNode *heap);
 
+#endif  /* __BLI_HEAP_H__ */
