@@ -46,22 +46,21 @@ typedef struct KDTreeNearest {
 } KDTreeNearest;
 
 /* Creates or free a kdtree */
-KDTree* BLI_kdtree_new(int maxsize);
+KDTree *BLI_kdtree_new(int maxsize);
 void BLI_kdtree_free(KDTree *tree);
 
 /* Construction: first insert points, then call balance. Normal is optional. */
-void BLI_kdtree_insert(KDTree *tree, int index, float *co, float *nor);
+void BLI_kdtree_insert(KDTree *tree, int index, const float co[3], const float nor[3]);
 void BLI_kdtree_balance(KDTree *tree);
 
 /* Find nearest returns index, and -1 if no node is found.
  * Find n nearest returns number of points found, with results in nearest.
  * Normal is optional, but if given will limit results to points in normal direction from co. */
-int	BLI_kdtree_find_nearest(KDTree *tree, float *co, float *nor, KDTreeNearest *nearest);
-int	BLI_kdtree_find_n_nearest(KDTree *tree, int n, float *co, float *nor, KDTreeNearest *nearest);
+int BLI_kdtree_find_nearest(KDTree *tree, const float co[3], const float nor[3], KDTreeNearest *nearest);
+int BLI_kdtree_find_n_nearest(KDTree *tree, int n, const float co[3], const float nor[3], KDTreeNearest *nearest);
 
 /* Range search returns number of points found, with results in nearest */
 /* Normal is optional, but if given will limit results to points in normal direction from co. */
 /* Remember to free nearest after use! */
-int BLI_kdtree_range_search(KDTree *tree, float range, float *co, float *nor, KDTreeNearest **nearest);
+int BLI_kdtree_range_search(KDTree *tree, float range, const float co[3], const float nor[3], KDTreeNearest **nearest);
 #endif
-
