@@ -1,5 +1,4 @@
-/* 
- *
+/*
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -42,25 +41,55 @@
 extern "C" {
 #endif
 
-	/* A reasonable standard buffer size, big
-	 * enough to not cause much internal fragmentation, 
-	 * small enough not to waste resources
-	 */
-#define BLI_MEMARENA_STD_BUFSIZE	(1<<14)
+/* A reasonable standard buffer size, big
+ * enough to not cause much internal fragmentation,
+ * small enough not to waste resources
+ */
+#define BLI_MEMARENA_STD_BUFSIZE    (1 << 14)
+
+/* some GNU attributes are only available from GCC 4.3 */
+#define MEM_GNU_ATTRIBUTES (defined(__GNUC__) && ((__GNUC__ * 100 + __GNUC_MINOR__) >= 403))
 
 struct MemArena;
 typedef struct MemArena MemArena;
 
+struct MemArena    *BLI_memarena_new(const int bufsize, const char *name)
+#if MEM_GNU_ATTRIBUTES
+__attribute__((warn_unused_result))
+__attribute__((nonnull(2)))
+#endif
+;
 
-struct MemArena*	BLI_memarena_new	(int bufsize, const char *name);
-void				BLI_memarena_free	(struct MemArena *ma);
+void                BLI_memarena_free(struct MemArena *ma)
+#if MEM_GNU_ATTRIBUTES
+__attribute__((nonnull(1)))
+#endif
+;
 
-void				BLI_memarena_use_malloc (struct MemArena *ma);
-void				BLI_memarena_use_calloc (struct MemArena *ma);
+void                BLI_memarena_use_malloc(struct MemArena *ma)
+#if MEM_GNU_ATTRIBUTES
+__attribute__((nonnull(1)))
+#endif
+;
+void                BLI_memarena_use_calloc(struct MemArena *ma)
+#if MEM_GNU_ATTRIBUTES
+__attribute__((nonnull(1)))
+#endif
+;
 
-void				BLI_memarena_use_align(struct MemArena *ma, int align);
+void                BLI_memarena_use_align(struct MemArena *ma, const int align)
+#if MEM_GNU_ATTRIBUTES
+__attribute__((nonnull(1)))
+#endif
+;
 
-void*				BLI_memarena_alloc	(struct MemArena *ma, int size);
+void               *BLI_memarena_alloc(struct MemArena *ma, int size)
+#if MEM_GNU_ATTRIBUTES
+__attribute__((warn_unused_result))
+__attribute__((nonnull(1)))
+__attribute__((alloc_size(2)))
+#endif
+;
 
 #ifdef __cplusplus
 }
