@@ -41,8 +41,7 @@ struct ScanFillVert;
 extern "C" {
 #endif
 
-typedef struct ScanFillContext
-{
+typedef struct ScanFillContext {
 	ListBase fillvertbase;
 	ListBase filledgebase;
 	ListBase fillfacebase;
@@ -61,54 +60,68 @@ typedef struct ScanFillContext
 } ScanFillContext;
 
 /* note; changing this also might affect the undo copy in editmesh.c */
-typedef struct ScanFillVert
-{
+typedef struct ScanFillVert {
 	struct ScanFillVert *next, *prev;
 	union {
 		struct ScanFillVert *v;
 		void                *p;
 		intptr_t             l;
+		unsigned int         u;
 	} tmp;
 	float co[3]; /* vertex location */
 	float xy[2]; /* 2D copy of vertex location (using dominant axis) */
-	int keyindex; /* original index #, for restoring  key information */
+	unsigned int keyindex; /* original index #, for restoring  key information */
 	short poly_nr;
 	unsigned char f, h;
 } ScanFillVert;
 
-typedef struct ScanFillEdge
-{
+typedef struct ScanFillEdge {
 	struct ScanFillEdge *next, *prev;
 	struct ScanFillVert *v1, *v2;
 	short poly_nr;
 	unsigned char f;
+	union {
+		unsigned char c;
+	} tmp;
 } ScanFillEdge;
 
-typedef struct ScanFillFace
-{
+typedef struct ScanFillFace {
 	struct ScanFillFace *next, *prev;
 	struct ScanFillVert *v1, *v2, *v3;
 } ScanFillFace;
 
 /* scanfill.c: used in displist only... */
-struct ScanFillVert *BLI_addfillvert(ScanFillContext *sf_ctx, const float vec[3]);
-struct ScanFillEdge *BLI_addfilledge(ScanFillContext *sf_ctx, struct ScanFillVert *v1, struct ScanFillVert *v2);
+struct ScanFillVert *BLI_scanfill_vert_add(ScanFillContext *sf_ctx, const float vec[3]);
+struct ScanFillEdge *BLI_scanfill_edge_add(ScanFillContext *sf_ctx, struct ScanFillVert *v1, struct ScanFillVert *v2);
 
-int BLI_begin_edgefill(ScanFillContext *sf_ctx);
-int BLI_edgefill(ScanFillContext *sf_ctx, const short do_quad_tri_speedup);
-int BLI_edgefill_ex(ScanFillContext *sf_ctx, const short do_quad_tri_speedup,
-                    const float nor_proj[3]);
-void BLI_end_edgefill(ScanFillContext *sf_ctx);
+enum {
+	BLI_SCANFILL_CALC_QUADTRI_FASTPATH = (1 << 0),
+
+	/* note: using BLI_SCANFILL_CALC_REMOVE_DOUBLES
+	 * Assumes ordered edges, otherwise we risk an eternal loop
+	 * removing double verts. - campbell */
+	BLI_SCANFILL_CALC_REMOVE_DOUBLES   = (1 << 1),
+
+	/* note: This flag removes checks for overlapping polygons.
+	 * when this flag is set, we'll never get back more faces then (totvert - 2) */
+	BLI_SCANFILL_CALC_HOLES            = (1 << 2)
+};
+
+int BLI_scanfill_begin(ScanFillContext *sf_ctx);
+int BLI_scanfill_calc(ScanFillContext *sf_ctx, const int flag);
+int BLI_scanfill_calc_ex(ScanFillContext *sf_ctx, const int flag,
+                         const float nor_proj[3]);
+void BLI_scanfill_end(ScanFillContext *sf_ctx);
 
 /* These callbacks are needed to make the lib finction properly */
 
 /**
- * Set a function taking a char* as argument to flag errors. If the
+ * Set a function taking a (char *) as argument to flag errors. If the
  * callback is not set, the error is discarded.
  * \param f The function to use as callback
  * \attention used in creator.c
  */
-void BLI_setErrorCallBack(void (*f)(const char*));
+void BLI_setErrorCallBack(void (*f)(const char *));
 
 /**
  * Set a function to be able to interrupt the execution of processing
