@@ -1,6 +1,4 @@
 /*
- *
- *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -33,11 +31,18 @@
 /** \file BLI_threads.h
  *  \ingroup bli
  */
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 #include <pthread.h>
 
+#ifdef __APPLE__
+#include <libkern/OSAtomic.h>
+#endif
+
 /* for tables, button in UI, etc */
-#define BLENDER_MAX_THREADS		64
+#define BLENDER_MAX_THREADS     64
 
 struct ListBase;
 
@@ -46,15 +51,15 @@ struct ListBase;
 /*this is run once at startup*/
 void BLI_threadapi_init(void);
 
-void	BLI_init_threads	(struct ListBase *threadbase, void *(*do_thread)(void *), int tot);
-int		BLI_available_threads(struct ListBase *threadbase);
-int		BLI_available_thread_index(struct ListBase *threadbase);
-void	BLI_insert_thread	(struct ListBase *threadbase, void *callerdata);
-void	BLI_remove_thread	(struct ListBase *threadbase, void *callerdata);
-void	BLI_remove_thread_index(struct ListBase *threadbase, int index);
-void	BLI_remove_threads(struct ListBase *threadbase);
-void	BLI_end_threads		(struct ListBase *threadbase);
-int BLI_thread_is_main(void);
+void    BLI_init_threads(struct ListBase *threadbase, void *(*do_thread)(void *), int tot);
+int     BLI_available_threads(struct ListBase *threadbase);
+int     BLI_available_thread_index(struct ListBase *threadbase);
+void    BLI_insert_thread(struct ListBase *threadbase, void *callerdata);
+void    BLI_remove_thread(struct ListBase *threadbase, void *callerdata);
+void    BLI_remove_thread_index(struct ListBase *threadbase, int index);
+void    BLI_remove_threads(struct ListBase *threadbase);
+void    BLI_end_threads(struct ListBase *threadbase);
+int     BLI_thread_is_main(void);
 
 
 void BLI_begin_threaded_malloc(void);
@@ -62,38 +67,52 @@ void BLI_end_threaded_malloc(void);
 
 /* System Information */
 
-int		BLI_system_thread_count(void); /* gets the number of threads the system can make use of */
+int     BLI_system_thread_count(void); /* gets the number of threads the system can make use of */
 
 /* Global Mutex Locks
  * 
  * One custom lock available now. can be extended. */
 
-#define LOCK_IMAGE		0
-#define LOCK_PREVIEW	1
-#define LOCK_VIEWER		2
-#define LOCK_CUSTOM1	3
-#define LOCK_RCACHE		4
-#define LOCK_OPENGL		5
-#define LOCK_NODES		6
-#define LOCK_MOVIECLIP	7
+#define LOCK_IMAGE      0
+#define LOCK_DRAW_IMAGE 1
+#define LOCK_VIEWER     2
+#define LOCK_CUSTOM1    3
+#define LOCK_RCACHE     4
+#define LOCK_OPENGL     5
+#define LOCK_NODES      6
+#define LOCK_MOVIECLIP  7
+#define LOCK_COLORMANAGE 8
 
-void	BLI_lock_thread(int type);
-void	BLI_unlock_thread(int type);
+void    BLI_lock_thread(int type);
+void    BLI_unlock_thread(int type);
 
 /* Mutex Lock */
 
 typedef pthread_mutex_t ThreadMutex;
-#define BLI_MUTEX_INITIALIZER	PTHREAD_MUTEX_INITIALIZER
+#define BLI_MUTEX_INITIALIZER   PTHREAD_MUTEX_INITIALIZER
 
 void BLI_mutex_init(ThreadMutex *mutex);
 void BLI_mutex_lock(ThreadMutex *mutex);
 void BLI_mutex_unlock(ThreadMutex *mutex);
 void BLI_mutex_end(ThreadMutex *mutex);
 
+/* Spin Lock */
+
+#ifdef __APPLE__
+typedef OSSpinLock SpinLock;
+#else
+typedef pthread_spinlock_t SpinLock;
+#endif
+
+void BLI_spin_init(SpinLock *spin);
+void BLI_spin_lock(SpinLock *spin);
+void BLI_spin_unlock(SpinLock *spin);
+void BLI_spin_end(SpinLock *spin);
+
 /* Read/Write Mutex Lock */
 
-#define THREAD_LOCK_READ	1
-#define THREAD_LOCK_WRITE	2
+#define THREAD_LOCK_READ    1
+#define THREAD_LOCK_WRITE   2
 
 typedef pthread_rwlock_t ThreadRWMutex;
 
@@ -138,7 +157,12 @@ void *BLI_thread_queue_pop(ThreadQueue *queue);
 void *BLI_thread_queue_pop_timeout(ThreadQueue *queue, int ms);
 int BLI_thread_queue_size(ThreadQueue *queue);
 
+void BLI_thread_queue_wait_finish(ThreadQueue *queue);
 void BLI_thread_queue_nowait(ThreadQueue *queue);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
 
