@@ -1,7 +1,4 @@
 /*
- * Jitter offset table
- *
- *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -30,9 +27,9 @@
 
 /** \file blender/blenlib/intern/jitter.c
  *  \ingroup bli
+ *  \brief Jitter offset table
  */
 
-
 #include <math.h>
 #include <string.h>
 #include "MEM_guardedalloc.h"
@@ -43,44 +40,44 @@
 
 void BLI_jitterate1(float *jit1, float *jit2, int num, float rad1)
 {
-	int i , j , k;
+	int i, j, k;
 	float vecx, vecy, dvecx, dvecy, x, y, len;
 
-	for (i = 2*num-2; i>=0 ; i-=2) {
+	for (i = 2 * num - 2; i >= 0; i -= 2) {
 		dvecx = dvecy = 0.0;
 		x = jit1[i];
-		y = jit1[i+1];
-		for (j = 2*num-2; j>=0 ; j-=2) {
+		y = jit1[i + 1];
+		for (j = 2 * num - 2; j >= 0; j -= 2) {
 			if (i != j) {
 				vecx = jit1[j] - x - 1.0f;
-				vecy = jit1[j+1] - y - 1.0f;
-				for (k = 3; k>0 ; k--) {
-					if ( fabsf(vecx)<rad1 && fabsf(vecy)<rad1) {
-						len=  sqrt(vecx*vecx + vecy*vecy);
-						if (len>0 && len<rad1) {
-							len= len/rad1;
-							dvecx += vecx/len;
-							dvecy += vecy/len;
+				vecy = jit1[j + 1] - y - 1.0f;
+				for (k = 3; k > 0; k--) {
+					if (fabsf(vecx) < rad1 && fabsf(vecy) < rad1) {
+						len =  sqrt(vecx * vecx + vecy * vecy);
+						if (len > 0 && len < rad1) {
+							len = len / rad1;
+							dvecx += vecx / len;
+							dvecy += vecy / len;
 						}
 					}
 					vecx += 1.0f;
 
-					if ( fabsf(vecx)<rad1 && fabsf(vecy)<rad1) {
-						len=  sqrt(vecx*vecx + vecy*vecy);
-						if (len>0 && len<rad1) {
-							len= len/rad1;
-							dvecx += vecx/len;
-							dvecy += vecy/len;
+					if (fabsf(vecx) < rad1 && fabsf(vecy) < rad1) {
+						len =  sqrt(vecx * vecx + vecy * vecy);
+						if (len > 0 && len < rad1) {
+							len = len / rad1;
+							dvecx += vecx / len;
+							dvecy += vecy / len;
 						}
 					}
 					vecx += 1.0f;
 
-					if ( fabsf(vecx)<rad1 && fabsf(vecy)<rad1) {
-						len=  sqrt(vecx*vecx + vecy*vecy);
-						if (len>0 && len<rad1) {
-							len= len/rad1;
-							dvecx += vecx/len;
-							dvecy += vecy/len;
+					if (fabsf(vecx) < rad1 && fabsf(vecy) < rad1) {
+						len =  sqrt(vecx * vecx + vecy * vecy);
+						if (len > 0 && len < rad1) {
+							len = len / rad1;
+							dvecx += vecx / len;
+							dvecy += vecy / len;
 						}
 					}
 					vecx -= 2.0f;
@@ -89,14 +86,14 @@ void BLI_jitterate1(float *jit1, float *jit2, int num, float rad1)
 			}
 		}
 
-		x -= dvecx/18.0f;
-		y -= dvecy/18.0f;
+		x -= dvecx / 18.0f;
+		y -= dvecy / 18.0f;
 		x -= floorf(x);
 		y -= floorf(y);
 		jit2[i] = x;
-		jit2[i+1] = y;
+		jit2[i + 1] = y;
 	}
-	memcpy(jit1,jit2,2 * num * sizeof(float));
+	memcpy(jit1, jit2, 2 * num * sizeof(float));
 }
 
 void BLI_jitterate2(float *jit1, float *jit2, int num, float rad2)
@@ -104,63 +101,63 @@ void BLI_jitterate2(float *jit1, float *jit2, int num, float rad2)
 	int i, j;
 	float vecx, vecy, dvecx, dvecy, x, y;
 
-	for (i=2*num -2; i>= 0 ; i-=2) {
+	for (i = 2 * num - 2; i >= 0; i -= 2) {
 		dvecx = dvecy = 0.0;
 		x = jit1[i];
-		y = jit1[i+1];
-		for (j =2*num -2; j>= 0 ; j-=2) {
+		y = jit1[i + 1];
+		for (j = 2 * num - 2; j >= 0; j -= 2) {
 			if (i != j) {
 				vecx = jit1[j] - x - 1.0f;
-				vecy = jit1[j+1] - y - 1.0f;
+				vecy = jit1[j + 1] - y - 1.0f;
 
-				if ( fabsf(vecx)<rad2) dvecx+= vecx*rad2;
+				if (fabsf(vecx) < rad2) dvecx += vecx * rad2;
 				vecx += 1.0f;
-				if ( fabsf(vecx)<rad2) dvecx+= vecx*rad2;
+				if (fabsf(vecx) < rad2) dvecx += vecx * rad2;
 				vecx += 1.0f;
-				if ( fabsf(vecx)<rad2) dvecx+= vecx*rad2;
+				if (fabsf(vecx) < rad2) dvecx += vecx * rad2;
 
-				if ( fabsf(vecy)<rad2) dvecy+= vecy*rad2;
+				if (fabsf(vecy) < rad2) dvecy += vecy * rad2;
 				vecy += 1.0f;
-				if ( fabsf(vecy)<rad2) dvecy+= vecy*rad2;
+				if (fabsf(vecy) < rad2) dvecy += vecy * rad2;
 				vecy += 1.0f;
-				if ( fabsf(vecy)<rad2) dvecy+= vecy*rad2;
+				if (fabsf(vecy) < rad2) dvecy += vecy * rad2;
 
 			}
 		}
 
-		x -= dvecx/2.0f;
-		y -= dvecy/2.0f;
+		x -= dvecx / 2.0f;
+		y -= dvecy / 2.0f;
 		x -= floorf(x);
 		y -= floorf(y);
 		jit2[i] = x;
-		jit2[i+1] = y;
+		jit2[i + 1] = y;
 	}
-	memcpy(jit1,jit2,2 * num * sizeof(float));
+	memcpy(jit1, jit2, 2 * num * sizeof(float));
 }
 
 
-void BLI_initjit(float *jitarr, int num)
+void BLI_jitter_init(float *jitarr, int num)
 {
 	float *jit2, x, rad1, rad2, rad3;
 	int i;
 
-	if (num==0) return;
+	if (num == 0) return;
 
-	jit2= MEM_mallocN(12 + 2*sizeof(float)*num, "initjit");
-	rad1= 1.0f/sqrtf((float)num);
-	rad2= 1.0f/((float)num);
-	rad3= sqrtf((float)num)/((float)num);
+	jit2 = MEM_mallocN(12 + 2 * sizeof(float) * num, "initjit");
+	rad1 = 1.0f / sqrtf((float)num);
+	rad2 = 1.0f / ((float)num);
+	rad3 = sqrtf((float)num) / ((float)num);
 
 	BLI_srand(31415926 + num);
-	x= 0;
-	for (i=0; i<2*num; i+=2) {
-		jitarr[i]= x+ rad1*(float)(0.5-BLI_drand());
-		jitarr[i+1]= ((float)i/2)/num +rad1*(float)(0.5-BLI_drand());
-		x+= rad3;
+	x = 0;
+	for (i = 0; i < 2 * num; i += 2) {
+		jitarr[i] = x + rad1 * (float)(0.5 - BLI_drand());
+		jitarr[i + 1] = ((float)i / 2) / num + rad1 * (float)(0.5 - BLI_drand());
+		x += rad3;
 		x -= floorf(x);
 	}
 
-	for (i=0 ; i<24 ; i++) {
+	for (i = 0; i < 24; i++) {
 		BLI_jitterate1(jitarr, jit2, num, rad1);
 		BLI_jitterate1(jitarr, jit2, num, rad1);
 		BLI_jitterate2(jitarr, jit2, num, rad2);
@@ -168,10 +165,10 @@ void BLI_initjit(float *jitarr, int num)
 
 	MEM_freeN(jit2);
 	
-	/* finally, move jittertab to be centered around (0,0) */
-	for (i=0; i<2*num; i+=2) {
+	/* finally, move jittertab to be centered around (0, 0) */
+	for (i = 0; i < 2 * num; i += 2) {
 		jitarr[i] -= 0.5f;
-		jitarr[i+1] -= 0.5f;
+		jitarr[i + 1] -= 0.5f;
 	}
 	
 }
