@@ -149,27 +149,6 @@ MINLINE void linearrgb_to_srgb_ushort4(unsigned short srgb[4], const float linea
 	srgb[3] = FTOUSHORT(linear[3]);
 }
 
-MINLINE void linearrgb_to_srgb_ushort4_predivide(unsigned short srgb[4], const float linear[4])
-{
-	float alpha, inv_alpha, t;
-	int i;
-
-	if (linear[3] == 1.0f || linear[3] == 0.0f) {
-		linearrgb_to_srgb_ushort4(srgb, linear);
-		return;
-	}
-
-	alpha = linear[3];
-	inv_alpha = 1.0f / alpha;
-
-	for (i = 0; i < 3; ++i) {
-		t = linear[i] * inv_alpha;
-		srgb[i] = (t < 1.0f) ? (unsigned short) (to_srgb_table_lookup(t) * alpha) : FTOUSHORT(linearrgb_to_srgb(t) * alpha);
-	}
-
-	srgb[3] = FTOUSHORT(linear[3]);
-}
-
 MINLINE void srgb_to_linearrgb_uchar4(float linear[4], const unsigned char srgb[4])
 {
 	linear[0] = BLI_color_from_srgb_table[srgb[0]];
@@ -195,7 +174,8 @@ MINLINE void srgb_to_linearrgb_uchar4_predivide(float linear[4], const unsigned
 }
 
 /* color macros for themes */
-#define rgba_char_args_set_fl(col, r, g, b, a)  rgba_char_args_set(col, r * 255, g * 255, b * 255, a * 255)
+#define rgba_char_args_set_fl(col, r, g, b, a) \
+	rgba_char_args_set(col, (r) * 255, (g) * 255, (b) * 255, (a) * 255)
 
 MINLINE void rgba_char_args_set(char col[4], const char r, const char g, const char b, const char a)
 {
@@ -215,4 +195,146 @@ MINLINE void rgba_char_args_test_set(char col[4], const char r, const char g, co
 	}
 }
 
+MINLINE void cpack_cpy_3ub(unsigned char r_col[3], const unsigned int pack)
+{
+	r_col[0] = ((pack) >>  0) & 0xFF;
+	r_col[1] = ((pack) >>  8) & 0xFF;
+	r_col[2] = ((pack) >> 16) & 0xFF;
+}
+
+/* TODO:
+ *
+ * regarding #rgb_to_bw vs #rgb_to_grayscale,
+ * it seems nobody knows why we have both functions which convert color to grays
+ * but with different influences, this is quite stupid, and should be resolved
+ * by someone who knows this stuff: see this thread
+ * http://lists.blender.org/pipermail/bf-committers/2012-June/037180.html
+ *
+ * Only conclusion is that rgb_to_grayscale is used more for compositing.
+ */
+MINLINE float rgb_to_bw(const float rgb[3])
+{
+	return 0.35f * rgb[0] + 0.45f * rgb[1] + 0.2f * rgb[2];
+}
+
+/* non-linear luma from ITU-R BT.601-2
+ * see: http://www.poynton.com/notes/colour_and_gamma/ColorFAQ.html#RTFToC11
+ * note: the values used for are not exact matches to those documented above,
+ * but they are from the same */
+MINLINE float rgb_to_grayscale(const float rgb[3])
+{
+	return 0.3f * rgb[0] + 0.58f * rgb[1] + 0.12f * rgb[2];
+}
+
+MINLINE unsigned char rgb_to_grayscale_byte(const unsigned char rgb[3])
+{
+	return (76 * (unsigned short) rgb[0] + 148 * (unsigned short) rgb[1] + 31 * (unsigned short) rgb[2]) / 255;
+}
+
+/* luma from defined by 'YCC_JFIF', see #rgb_to_ycc */
+MINLINE float rgb_to_luma(const float rgb[3])
+{
+	return 0.299f * rgb[0] + 0.587f * rgb[1] + 0.114f * rgb[2];
+}
+
+MINLINE unsigned char rgb_to_luma_byte(const unsigned char rgb[3])
+{
+	return (76 * (unsigned short) rgb[0] + 150 * (unsigned short) rgb[1] + 29 * (unsigned short) rgb[2]) / 255;
+}
+
+/* gamma-corrected RGB --> CIE XYZ
+ * for this function we only get the Y component
+ * see: http://software.intel.com/sites/products/documentation/hpc/ipp/ippi/ippi_ch6/ch6_color_models.html
+ *
+ * also known as:
+ * luminance rec. 709 */
+MINLINE float rgb_to_luma_y(const float rgb[3])
+{
+	return 0.212671f * rgb[0] + 0.71516f * rgb[1] + 0.072169f * rgb[2];
+}
+
+MINLINE int compare_rgb_uchar(const unsigned char col_a[3], const unsigned char col_b[3], const int limit)
+{
+	int r = (int)col_a[0] - (int)col_b[0];
+	if (ABS(r) < limit) {
+		int g = (int)col_a[1] - (int)col_b[1];
+		if (ABS(g) < limit) {
+			int b = (int)col_a[2] - (int)col_b[2];
+			if (ABS(b) < limit) {
+				return 1;
+			}
+		}
+	}
+
+	return 0;
+}
+
+/**************** Alpha Transformations *****************/
+
+MINLINE void premul_to_straight_v4_v4(float straight[4], const float premul[4])
+{
+	if (premul[3] == 0.0f || premul[3] == 1.0f) {
+		straight[0] = premul[0];
+		straight[1] = premul[1];
+		straight[2] = premul[2];
+		straight[3] = premul[3];
+	}
+	else {
+		float alpha_inv = 1.0f / premul[3];
+		straight[0] = premul[0] * alpha_inv;
+		straight[1] = premul[1] * alpha_inv;
+		straight[2] = premul[2] * alpha_inv;
+		straight[3] = premul[3];
+	}
+}
+
+MINLINE void premul_to_straight_v4(float color[4])
+{
+	premul_to_straight_v4_v4(color, color);
+}
+
+MINLINE void straight_to_premul_v4_v4(float premul[4], const float straight[4])
+{
+	float alpha = straight[3];
+	premul[0] = straight[0] * alpha;
+	premul[1] = straight[1] * alpha;
+	premul[2] = straight[2] * alpha;
+	premul[3] = straight[3];
+}
+
+MINLINE void straight_to_premul_v4(float color[4])
+{
+	straight_to_premul_v4_v4(color, color);
+}
+
+MINLINE void straight_uchar_to_premul_float(float result[4], const unsigned char color[4])
+{
+	float alpha = color[3] / 255.0f;
+	float fac = alpha / 255.0f;
+
+	result[0] = color[0] * fac;
+	result[1] = color[1] * fac;
+	result[2] = color[2] * fac;
+	result[3] = alpha;
+}
+
+MINLINE void premul_float_to_straight_uchar(unsigned char *result, const float color[4])
+{
+	if (color[3] == 0.0f || color[3] == 1.0f) {
+		result[0] = FTOCHAR(color[0]);
+		result[1] = FTOCHAR(color[1]);
+		result[2] = FTOCHAR(color[2]);
+		result[3] = FTOCHAR(color[3]);
+	}
+	else {
+		float alpha_inv = 1.0f / color[3];
+
+		/* hopefully this would be optimized */
+		result[0] = FTOCHAR(color[0] * alpha_inv);
+		result[1] = FTOCHAR(color[1] * alpha_inv);
+		result[2] = FTOCHAR(color[2] * alpha_inv);
+		result[3] = FTOCHAR(color[3]);
+	}
+}
+
 #endif /* __MATH_COLOR_INLINE_C__ */
