@@ -43,7 +43,7 @@ void zero_m4(float m[4][4])
 	memset(m, 0, 4 * 4 * sizeof(float));
 }
 
-void unit_m3(float m[][3])
+void unit_m3(float m[3][3])
 {
 	m[0][0] = m[1][1] = m[2][2] = 1.0;
 	m[0][1] = m[0][2] = 0.0;
@@ -51,7 +51,7 @@ void unit_m3(float m[][3])
 	m[2][0] = m[2][1] = 0.0;
 }
 
-void unit_m4(float m[][4])
+void unit_m4(float m[4][4])
 {
 	m[0][0] = m[1][1] = m[2][2] = m[3][3] = 1.0;
 	m[0][1] = m[0][2] = m[0][3] = 0.0;
@@ -60,18 +60,18 @@ void unit_m4(float m[][4])
 	m[3][0] = m[3][1] = m[3][2] = 0.0;
 }
 
-void copy_m3_m3(float m1[][3], float m2[][3])
+void copy_m3_m3(float m1[3][3], float m2[3][3])
 {
 	/* destination comes first: */
 	memcpy(&m1[0], &m2[0], 9 * sizeof(float));
 }
 
-void copy_m4_m4(float m1[][4], float m2[][4])
+void copy_m4_m4(float m1[4][4], float m2[4][4])
 {
 	memcpy(m1, m2, 4 * 4 * sizeof(float));
 }
 
-void copy_m3_m4(float m1[][3], float m2[][4])
+void copy_m3_m4(float m1[3][3], float m2[4][4])
 {
 	m1[0][0] = m2[0][0];
 	m1[0][1] = m2[0][1];
@@ -86,7 +86,7 @@ void copy_m3_m4(float m1[][3], float m2[][4])
 	m1[2][2] = m2[2][2];
 }
 
-void copy_m4_m3(float m1[][4], float m2[][3]) /* no clear */
+void copy_m4_m3(float m1[4][4], float m2[3][3]) /* no clear */
 {
 	m1[0][0] = m2[0][0];
 	m1[0][1] = m2[0][1];
@@ -112,7 +112,7 @@ void copy_m4_m3(float m1[][4], float m2[][3]) /* no clear */
 
 }
 
-void swap_m3m3(float m1[][3], float m2[][3])
+void swap_m3m3(float m1[3][3], float m2[3][3])
 {
 	float t;
 	int i, j;
@@ -126,7 +126,7 @@ void swap_m3m3(float m1[][3], float m2[][3])
 	}
 }
 
-void swap_m4m4(float m1[][4], float m2[][4])
+void swap_m4m4(float m1[4][4], float m2[4][4])
 {
 	float t;
 	int i, j;
@@ -142,7 +142,7 @@ void swap_m4m4(float m1[][4], float m2[][4])
 
 /******************************** Arithmetic *********************************/
 
-void mult_m4_m4m4(float m1[][4], float m3_[][4], float m2_[][4])
+void mult_m4_m4m4(float m1[4][4], float m3_[4][4], float m2_[4][4])
 {
 	float m2[4][4], m3[4][4];
 
@@ -173,7 +173,7 @@ void mult_m4_m4m4(float m1[][4], float m3_[][4], float m2_[][4])
 
 }
 
-void mul_m3_m3m3(float m1[][3], float m3_[][3], float m2_[][3])
+void mul_m3_m3m3(float m1[3][3], float m3_[3][3], float m2_[3][3])
 {
 	float m2[3][3], m3[3][3];
 
@@ -195,7 +195,7 @@ void mul_m3_m3m3(float m1[][3], float m3_[][3], float m2_[][3])
 	m1[2][2] = m2[2][0] * m3[0][2] + m2[2][1] * m3[1][2] + m2[2][2] * m3[2][2];
 }
 
-void mul_m4_m4m3(float (*m1)[4], float (*m3_)[4], float (*m2_)[3])
+void mul_m4_m4m3(float m1[4][4], float m3_[4][4], float m2_[3][3])
 {
 	float m2[3][3], m3[4][4];
 
@@ -215,8 +215,14 @@ void mul_m4_m4m3(float (*m1)[4], float (*m3_)[4], float (*m2_)[3])
 }
 
 /* m1 = m2 * m3, ignore the elements on the 4th row/column of m3 */
-void mult_m3_m3m4(float m1[][3], float m3[][4], float m2[][3])
+void mult_m3_m3m4(float m1[3][3], float m3_[4][4], float m2_[3][3])
 {
+	float m2[3][3], m3[4][4];
+
+	/* copy so it works when m1 is the same pointer as m2 or m3 */
+	copy_m3_m3(m2, m2_);
+	copy_m4_m4(m3, m3_);
+
 	/* m1[i][j] = m2[i][k] * m3[k][j] */
 	m1[0][0] = m2[0][0] * m3[0][0] + m2[0][1] * m3[1][0] + m2[0][2] * m3[2][0];
 	m1[0][1] = m2[0][0] * m3[0][1] + m2[0][1] * m3[1][1] + m2[0][2] * m3[2][1];
@@ -231,8 +237,14 @@ void mult_m3_m3m4(float m1[][3], float m3[][4], float m2[][3])
 	m1[2][2] = m2[2][0] * m3[0][2] + m2[2][1] * m3[1][2] + m2[2][2] * m3[2][2];
 }
 
-void mul_m4_m3m4(float (*m1)[4], float (*m3)[3], float (*m2)[4])
+void mul_m4_m3m4(float m1[4][4], float m3_[3][3], float m2_[4][4])
 {
+	float m2[4][4], m3[3][3];
+
+	/* copy so it works when m1 is the same pointer as m2 or m3 */
+	copy_m4_m4(m2, m2_);
+	copy_m3_m3(m3, m3_);
+
 	m1[0][0] = m2[0][0] * m3[0][0] + m2[0][1] * m3[1][0] + m2[0][2] * m3[2][0];
 	m1[0][1] = m2[0][0] * m3[0][1] + m2[0][1] * m3[1][1] + m2[0][2] * m3[2][1];
 	m1[0][2] = m2[0][0] * m3[0][2] + m2[0][1] * m3[1][2] + m2[0][2] * m3[2][2];
@@ -244,10 +256,10 @@ void mul_m4_m3m4(float (*m1)[4], float (*m3)[3], float (*m2)[4])
 	m1[2][2] = m2[2][0] * m3[0][2] + m2[2][1] * m3[1][2] + m2[2][2] * m3[2][2];
 }
 
-void mul_serie_m3(float answ[][3],
-                  float m1[][3], float m2[][3], float m3[][3],
-                  float m4[][3], float m5[][3], float m6[][3],
-                  float m7[][3], float m8[][3])
+void mul_serie_m3(float answ[3][3],
+                  float m1[3][3], float m2[3][3], float m3[3][3],
+                  float m4[3][3], float m5[3][3], float m6[3][3],
+                  float m7[3][3], float m8[3][3])
 {
 	float temp[3][3];
 
@@ -277,10 +289,10 @@ void mul_serie_m3(float answ[][3],
 	}
 }
 
-void mul_serie_m4(float answ[][4], float m1[][4],
-                  float m2[][4], float m3[][4], float m4[][4],
-                  float m5[][4], float m6[][4], float m7[][4],
-                  float m8[][4])
+void mul_serie_m4(float answ[4][4], float m1[4][4],
+                  float m2[4][4], float m3[4][4], float m4[4][4],
+                  float m5[4][4], float m6[4][4], float m7[4][4],
+                  float m8[4][4])
 {
 	float temp[4][4];
 
@@ -310,7 +322,7 @@ void mul_serie_m4(float answ[][4], float m1[][4],
 	}
 }
 
-void mul_m4_v3(float mat[][4], float vec[3])
+void mul_m4_v3(float mat[4][4], float vec[3])
 {
 	float x, y;
 
@@ -321,7 +333,7 @@ void mul_m4_v3(float mat[][4], float vec[3])
 	vec[2] = x * mat[0][2] + y * mat[1][2] + mat[2][2] * vec[2] + mat[3][2];
 }
 
-void mul_v3_m4v3(float in[3], float mat[][4], const float vec[3])
+void mul_v3_m4v3(float in[3], float mat[4][4], const float vec[3])
 {
 	float x, y;
 
@@ -332,8 +344,17 @@ void mul_v3_m4v3(float in[3], float mat[][4], const float vec[3])
 	in[2] = x * mat[0][2] + y * mat[1][2] + mat[2][2] * vec[2] + mat[3][2];
 }
 
+void mul_v2_m2v2(float r[2], float mat[2][2], const float vec[2])
+{
+	float x;
+
+	x = vec[0];
+	r[0] = mat[0][0] * x + mat[1][0] * vec[1];
+	r[1] = mat[0][1] * x + mat[1][1] * vec[1];
+}
+
 /* same as mul_m4_v3() but doesnt apply translation component */
-void mul_mat3_m4_v3(float mat[][4], float vec[3])
+void mul_mat3_m4_v3(float mat[4][4], float vec[3])
 {
 	float x, y;
 
@@ -344,7 +365,7 @@ void mul_mat3_m4_v3(float mat[][4], float vec[3])
 	vec[2] = x * mat[0][2] + y * mat[1][2] + mat[2][2] * vec[2];
 }
 
-void mul_project_m4_v3(float mat[][4], float vec[3])
+void mul_project_m4_v3(float mat[4][4], float vec[3])
 {
 	const float w = vec[0] * mat[0][3] + vec[1] * mat[1][3] + vec[2] * mat[2][3] + mat[3][3];
 	mul_m4_v3(mat, vec);
@@ -354,7 +375,7 @@ void mul_project_m4_v3(float mat[][4], float vec[3])
 	vec[2] /= w;
 }
 
-void mul_v4_m4v4(float r[4], float mat[4][4], float v[4])
+void mul_v4_m4v4(float r[4], float mat[4][4], const float v[4])
 {
 	float x, y, z;
 
@@ -392,13 +413,19 @@ void mul_m4_v4d(float mat[4][4], double r[4])
 	mul_v4d_m4v4d(r, mat, r);
 }
 
-void mul_v3_m3v3(float r[3], float M[3][3], float a[3])
+void mul_v3_m3v3(float r[3], float M[3][3], const float a[3])
 {
 	r[0] = M[0][0] * a[0] + M[1][0] * a[1] + M[2][0] * a[2];
 	r[1] = M[0][1] * a[0] + M[1][1] * a[1] + M[2][1] * a[2];
 	r[2] = M[0][2] * a[0] + M[1][2] * a[1] + M[2][2] * a[2];
 }
 
+void mul_v2_m3v3(float r[2], float M[3][3], const float a[3])
+{
+	r[0] = M[0][0] * a[0] + M[1][0] * a[1] + M[2][0] * a[2];
+	r[1] = M[0][1] * a[0] + M[1][1] * a[1] + M[2][1] * a[2];
+}
+
 void mul_m3_v3(float M[3][3], float r[3])
 {
 	float tmp[3];
@@ -407,7 +434,7 @@ void mul_m3_v3(float M[3][3], float r[3])
 	copy_v3_v3(r, tmp);
 }
 
-void mul_transposed_m3_v3(float mat[][3], float vec[3])
+void mul_transposed_m3_v3(float mat[3][3], float vec[3])
 {
 	float x, y;
 
@@ -445,7 +472,7 @@ void mul_mat3_m4_fl(float m[4][4], float f)
 			m[i][j] *= f;
 }
 
-void mul_m3_v3_double(float mat[][3], double vec[3])
+void mul_m3_v3_double(float mat[3][3], double vec[3])
 {
 	double x, y;
 
@@ -456,7 +483,7 @@ void mul_m3_v3_double(float mat[][3], double vec[3])
 	vec[2] = x * (double)mat[0][2] + y * (double)mat[1][2] + (double)mat[2][2] * vec[2];
 }
 
-void add_m3_m3m3(float m1[][3], float m2[][3], float m3[][3])
+void add_m3_m3m3(float m1[3][3], float m2[3][3], float m3[3][3])
 {
 	int i, j;
 
@@ -465,7 +492,7 @@ void add_m3_m3m3(float m1[][3], float m2[][3], float m3[][3])
 			m1[i][j] = m2[i][j] + m3[i][j];
 }
 
-void add_m4_m4m4(float m1[][4], float m2[][4], float m3[][4])
+void add_m4_m4m4(float m1[4][4], float m2[4][4], float m3[4][4])
 {
 	int i, j;
 
@@ -474,7 +501,7 @@ void add_m4_m4m4(float m1[][4], float m2[][4], float m3[][4])
 			m1[i][j] = m2[i][j] + m3[i][j];
 }
 
-void sub_m3_m3m3(float m1[][3], float m2[][3], float m3[][3])
+void sub_m3_m3m3(float m1[3][3], float m2[3][3], float m3[3][3])
 {
 	int i, j;
 
@@ -483,7 +510,7 @@ void sub_m3_m3m3(float m1[][3], float m2[][3], float m3[][3])
 			m1[i][j] = m2[i][j] - m3[i][j];
 }
 
-void sub_m4_m4m4(float m1[][4], float m2[][4], float m3[][4])
+void sub_m4_m4m4(float m1[4][4], float m2[4][4], float m3[4][4])
 {
 	int i, j;
 
@@ -492,6 +519,50 @@ void sub_m4_m4m4(float m1[][4], float m2[][4], float m3[][4])
 			m1[i][j] = m2[i][j] - m3[i][j];
 }
 
+float determinant_m3_array(float m[3][3])
+{
+	return (m[0][0] * (m[1][1] * m[2][2] - m[1][2] * m[2][1]) -
+	        m[1][0] * (m[0][1] * m[2][2] - m[0][2] * m[2][1]) +
+	        m[2][0] * (m[0][1] * m[1][2] - m[0][2] * m[1][1]));
+}
+
+int invert_m3_ex(float m[3][3], const float epsilon)
+{
+	float tmp[3][3];
+	int success;
+
+	success = invert_m3_m3_ex(tmp, m, epsilon);
+	copy_m3_m3(m, tmp);
+
+	return success;
+}
+
+int invert_m3_m3_ex(float m1[3][3], float m2[3][3], const float epsilon)
+{
+	float det;
+	int a, b, success;
+
+	BLI_assert(epsilon >= 0.0f);
+
+	/* calc adjoint */
+	adjoint_m3_m3(m1, m2);
+
+	/* then determinant old matrix! */
+	det = determinant_m3_array(m2);
+
+	success = (fabsf(det) > epsilon);
+
+	if (LIKELY(det != 0.0f)) {
+		det = 1.0f / det;
+		for (a = 0; a < 3; a++) {
+			for (b = 0; b < 3; b++) {
+				m1[a][b] *= det;
+			}
+		}
+	}
+	return success;
+}
+
 int invert_m3(float m[3][3])
 {
 	float tmp[3][3];
@@ -512,17 +583,16 @@ int invert_m3_m3(float m1[3][3], float m2[3][3])
 	adjoint_m3_m3(m1, m2);
 
 	/* then determinant old matrix! */
-	det = (m2[0][0] * (m2[1][1] * m2[2][2] - m2[1][2] * m2[2][1]) -
-	       m2[1][0] * (m2[0][1] * m2[2][2] - m2[0][2] * m2[2][1]) +
-	       m2[2][0] * (m2[0][1] * m2[1][2] - m2[0][2] * m2[1][1]));
+	det = determinant_m3_array(m2);
 
-	success = (det != 0);
+	success = (det != 0.0f);
 
-	if (det == 0) det = 1;
-	det = 1 / det;
-	for (a = 0; a < 3; a++) {
-		for (b = 0; b < 3; b++) {
-			m1[a][b] *= det;
+	if (LIKELY(det != 0.0f)) {
+		det = 1.0f / det;
+		for (a = 0; a < 3; a++) {
+			for (b = 0; b < 3; b++) {
+				m1[a][b] *= det;
+			}
 		}
 	}
 
@@ -557,6 +627,8 @@ int invert_m4_m4(float inverse[4][4], float mat[4][4])
 	float max;
 	int maxj;
 
+	BLI_assert(inverse != mat);
+
 	/* Set inverse to identity */
 	for (i = 0; i < 4; i++)
 		for (j = 0; j < 4; j++)
@@ -591,15 +663,15 @@ int invert_m4_m4(float inverse[4][4], float mat[4][4])
 		if (temp == 0)
 			return 0;  /* No non-zero pivot */
 		for (k = 0; k < 4; k++) {
-			tempmat[i][k] = (float)(tempmat[i][k] / temp);
-			inverse[i][k] = (float)(inverse[i][k] / temp);
+			tempmat[i][k] = (float)((double)tempmat[i][k] / temp);
+			inverse[i][k] = (float)((double)inverse[i][k] / temp);
 		}
 		for (j = 0; j < 4; j++) {
 			if (j != i) {
 				temp = tempmat[j][i];
 				for (k = 0; k < 4; k++) {
-					tempmat[j][k] -= (float)(tempmat[i][k] * temp);
-					inverse[j][k] -= (float)(inverse[i][k] * temp);
+					tempmat[j][k] -= (float)((double)tempmat[i][k] * temp);
+					inverse[j][k] -= (float)((double)inverse[i][k] * temp);
 				}
 			}
 		}
@@ -609,7 +681,7 @@ int invert_m4_m4(float inverse[4][4], float mat[4][4])
 
 /****************************** Linear Algebra *******************************/
 
-void transpose_m3(float mat[][3])
+void transpose_m3(float mat[3][3])
 {
 	float t;
 
@@ -624,7 +696,7 @@ void transpose_m3(float mat[][3])
 	mat[2][1] = t;
 }
 
-void transpose_m4(float mat[][4])
+void transpose_m4(float mat[4][4])
 {
 	float t;
 
@@ -650,7 +722,7 @@ void transpose_m4(float mat[][4])
 	mat[3][2] = t;
 }
 
-void orthogonalize_m3(float mat[][3], int axis)
+void orthogonalize_m3(float mat[3][3], int axis)
 {
 	float size[3];
 	mat3_to_size(size, mat);
@@ -728,7 +800,7 @@ void orthogonalize_m3(float mat[][3], int axis)
 	mul_v3_fl(mat[2], size[2]);
 }
 
-void orthogonalize_m4(float mat[][4], int axis)
+void orthogonalize_m4(float mat[4][4], int axis)
 {
 	float size[3];
 	mat4_to_size(size, mat);
@@ -807,7 +879,7 @@ void orthogonalize_m4(float mat[][4], int axis)
 	mul_v3_fl(mat[2], size[2]);
 }
 
-int is_orthogonal_m3(float m[][3])
+int is_orthogonal_m3(float m[3][3])
 {
 	int i, j;
 
@@ -821,7 +893,7 @@ int is_orthogonal_m3(float m[][3])
 	return 1;
 }
 
-int is_orthogonal_m4(float m[][4])
+int is_orthogonal_m4(float m[4][4])
 {
 	int i, j;
 
@@ -836,7 +908,7 @@ int is_orthogonal_m4(float m[][4])
 	return 1;
 }
 
-int is_orthonormal_m3(float m[][3])
+int is_orthonormal_m3(float m[3][3])
 {
 	if (is_orthogonal_m3(m)) {
 		int i;
@@ -851,7 +923,7 @@ int is_orthonormal_m3(float m[][3])
 	return 0;
 }
 
-int is_orthonormal_m4(float m[][4])
+int is_orthonormal_m4(float m[4][4])
 {
 	if (is_orthogonal_m4(m)) {
 		int i;
@@ -866,7 +938,7 @@ int is_orthonormal_m4(float m[][4])
 	return 0;
 }
 
-int is_uniform_scaled_m3(float m[][3])
+int is_uniform_scaled_m3(float m[3][3])
 {
 	const float eps = 1e-7;
 	float t[3][3];
@@ -895,21 +967,21 @@ int is_uniform_scaled_m3(float m[][3])
 	return 0;
 }
 
-void normalize_m3(float mat[][3])
+void normalize_m3(float mat[3][3])
 {
 	normalize_v3(mat[0]);
 	normalize_v3(mat[1]);
 	normalize_v3(mat[2]);
 }
 
-void normalize_m3_m3(float rmat[][3], float mat[][3])
+void normalize_m3_m3(float rmat[3][3], float mat[3][3])
 {
 	normalize_v3_v3(rmat[0], mat[0]);
 	normalize_v3_v3(rmat[1], mat[1]);
 	normalize_v3_v3(rmat[2], mat[2]);
 }
 
-void normalize_m4(float mat[][4])
+void normalize_m4(float mat[4][4])
 {
 	float len;
 
@@ -921,20 +993,24 @@ void normalize_m4(float mat[][4])
 	if (len != 0.0f) mat[2][3] /= len;
 }
 
-void normalize_m4_m4(float rmat[][4], float mat[][4])
+void normalize_m4_m4(float rmat[4][4], float mat[4][4])
 {
-	float len;
-
-	len = normalize_v3_v3(rmat[0], mat[0]);
-	if (len != 0.0f) rmat[0][3] = mat[0][3] / len;
-	len = normalize_v3_v3(rmat[1], mat[1]);
-	if (len != 0.0f) rmat[1][3] = mat[1][3] / len;
-	len = normalize_v3_v3(rmat[2], mat[2]);
-	if (len != 0.0f) rmat[2][3] = mat[2][3] / len;
+	copy_m4_m4(rmat, mat);
+	normalize_m4(rmat);
 }
 
-void adjoint_m3_m3(float m1[][3], float m[][3])
+void adjoint_m2_m2(float m1[2][2], float m[2][2])
 {
+	BLI_assert(m1 != m);
+	m1[0][0] =  m[1][1];
+	m1[0][1] = -m[1][0];
+	m1[1][0] = -m[0][1];
+	m1[1][1] =  m[0][0];
+}
+
+void adjoint_m3_m3(float m1[3][3], float m[3][3])
+{
+	BLI_assert(m1 != m);
 	m1[0][0] = m[1][1] * m[2][2] - m[1][2] * m[2][1];
 	m1[0][1] = -m[0][1] * m[2][2] + m[0][2] * m[2][1];
 	m1[0][2] = m[0][1] * m[1][2] - m[0][2] * m[1][1];
@@ -948,7 +1024,7 @@ void adjoint_m3_m3(float m1[][3], float m[][3])
 	m1[2][2] = m[0][0] * m[1][1] - m[0][1] * m[1][0];
 }
 
-void adjoint_m4_m4(float out[][4], float in[][4]) /* out = ADJ(in) */
+void adjoint_m4_m4(float out[4][4], float in[4][4]) /* out = ADJ(in) */
 {
 	float a1, a2, a3, a4, b1, b2, b3, b4;
 	float c1, c2, c3, c4, d1, d2, d3, d4;
@@ -1014,7 +1090,7 @@ float determinant_m3(float a1, float a2, float a3,
 	return ans;
 }
 
-float determinant_m4(float m[][4])
+float determinant_m4(float m[4][4])
 {
 	float ans;
 	float a1, a2, a3, a4, b1, b2, b3, b4, c1, c2, c3, c4, d1, d2, d3, d4;
@@ -1049,7 +1125,7 @@ float determinant_m4(float m[][4])
 
 /****************************** Transformations ******************************/
 
-void size_to_mat3(float mat[][3], const float size[3])
+void size_to_mat3(float mat[3][3], const float size[3])
 {
 	mat[0][0] = size[0];
 	mat[0][1] = 0.0f;
@@ -1062,7 +1138,7 @@ void size_to_mat3(float mat[][3], const float size[3])
 	mat[2][0] = 0.0f;
 }
 
-void size_to_mat4(float mat[][4], const float size[3])
+void size_to_mat4(float mat[4][4], const float size[3])
 {
 	float tmat[3][3];
 
@@ -1071,14 +1147,14 @@ void size_to_mat4(float mat[][4], const float size[3])
 	copy_m4_m3(mat, tmat);
 }
 
-void mat3_to_size(float size[3], float mat[][3])
+void mat3_to_size(float size[3], float mat[3][3])
 {
 	size[0] = len_v3(mat[0]);
 	size[1] = len_v3(mat[1]);
 	size[2] = len_v3(mat[2]);
 }
 
-void mat4_to_size(float size[3], float mat[][4])
+void mat4_to_size(float size[3], float mat[4][4])
 {
 	size[0] = len_v3(mat[0]);
 	size[1] = len_v3(mat[1]);
@@ -1088,7 +1164,7 @@ void mat4_to_size(float size[3], float mat[][4])
 /* this gets the average scale of a matrix, only use when your scaling
  * data that has no idea of scale axis, examples are bone-envelope-radius
  * and curve radius */
-float mat3_to_scale(float mat[][3])
+float mat3_to_scale(float mat[3][3])
 {
 	/* unit length vector */
 	float unit_vec[3] = {0.577350269189626f, 0.577350269189626f, 0.577350269189626f};
@@ -1096,7 +1172,7 @@ float mat3_to_scale(float mat[][3])
 	return len_v3(unit_vec);
 }
 
-float mat4_to_scale(float mat[][4])
+float mat4_to_scale(float mat[4][4])
 {
 	float tmat[3][3];
 	copy_m3_m4(tmat, mat);
@@ -1111,7 +1187,7 @@ void mat3_to_rot_size(float rot[3][3], float size[3], float mat3[3][3])
 	/* rotation & scale are linked, we need to create the mat's
 	 * for these together since they are related. */
 
-	/* so scale doesnt interfear with rotation [#24291] */
+	/* so scale doesn't interfere with rotation [#24291] */
 	/* note: this is a workaround for negative matrix not working for rotation conversion, FIXME */
 	normalize_m3_m3(mat3_n, mat3);
 	if (is_negative_m3(mat3)) {
@@ -1134,7 +1210,7 @@ void mat3_to_rot_size(float rot[3][3], float size[3], float mat3[3][3])
 	size[2] = mat3[2][2];
 }
 
-void mat4_to_loc_rot_size(float loc[3], float rot[3][3], float size[3], float wmat[][4])
+void mat4_to_loc_rot_size(float loc[3], float rot[3][3], float size[3], float wmat[4][4])
 {
 	float mat3[3][3]; /* wmat -> 3x3 */
 
@@ -1145,7 +1221,34 @@ void mat4_to_loc_rot_size(float loc[3], float rot[3][3], float size[3], float wm
 	copy_v3_v3(loc, wmat[3]);
 }
 
-void scale_m3_fl(float m[][3], float scale)
+void mat4_to_loc_quat(float loc[3], float quat[4], float wmat[4][4])
+{
+	float mat3[3][3];
+	float mat3_n[3][3]; /* normalized mat3 */
+
+	copy_m3_m4(mat3, wmat);
+	normalize_m3_m3(mat3_n, mat3);
+
+	/* so scale doesn't interfere with rotation [#24291] */
+	/* note: this is a workaround for negative matrix not working for rotation conversion, FIXME */
+	if (is_negative_m3(mat3)) {
+		negate_v3(mat3_n[0]);
+		negate_v3(mat3_n[1]);
+		negate_v3(mat3_n[2]);
+	}
+
+	mat3_to_quat(quat, mat3_n);
+	copy_v3_v3(loc, wmat[3]);
+}
+
+void mat4_decompose(float loc[3], float quat[4], float size[3], float wmat[4][4])
+{
+	float rot[3][3];
+	mat4_to_loc_rot_size(loc, rot, size, wmat);
+	mat3_to_quat(quat, rot);
+}
+
+void scale_m3_fl(float m[3][3], float scale)
 {
 	m[0][0] = m[1][1] = m[2][2] = scale;
 	m[0][1] = m[0][2] = 0.0;
@@ -1153,7 +1256,7 @@ void scale_m3_fl(float m[][3], float scale)
 	m[2][0] = m[2][1] = 0.0;
 }
 
-void scale_m4_fl(float m[][4], float scale)
+void scale_m4_fl(float m[4][4], float scale)
 {
 	m[0][0] = m[1][1] = m[2][2] = scale;
 	m[3][3] = 1.0;
@@ -1163,14 +1266,14 @@ void scale_m4_fl(float m[][4], float scale)
 	m[3][0] = m[3][1] = m[3][2] = 0.0;
 }
 
-void translate_m4(float mat[][4], float Tx, float Ty, float Tz)
+void translate_m4(float mat[4][4], float Tx, float Ty, float Tz)
 {
 	mat[3][0] += (Tx * mat[0][0] + Ty * mat[1][0] + Tz * mat[2][0]);
 	mat[3][1] += (Tx * mat[0][1] + Ty * mat[1][1] + Tz * mat[2][1]);
 	mat[3][2] += (Tx * mat[0][2] + Ty * mat[1][2] + Tz * mat[2][2]);
 }
 
-void rotate_m4(float mat[][4], const char axis, const float angle)
+void rotate_m4(float mat[4][4], const char axis, const float angle)
 {
 	int col;
 	float temp[4] = {0.0f, 0.0f, 0.0f, 0.0f};
@@ -1178,8 +1281,8 @@ void rotate_m4(float mat[][4], const char axis, const float angle)
 
 	assert(axis >= 'X' && axis <= 'Z');
 
-	cosine = (float)cos(angle);
-	sine = (float)sin(angle);
+	cosine = cosf(angle);
+	sine   = sinf(angle);
 	switch (axis) {
 		case 'X':
 			for (col = 0; col < 4; col++)
@@ -1210,7 +1313,14 @@ void rotate_m4(float mat[][4], const char axis, const float angle)
 	}
 }
 
-void blend_m3_m3m3(float out[][3], float dst[][3], float src[][3], const float srcweight)
+void rotate_m2(float mat[2][2], const float angle)
+{
+	mat[0][0] = mat[1][1] = cosf(angle);
+	mat[0][1] = sinf(angle);
+	mat[1][0] = -mat[0][1];
+}
+
+void blend_m3_m3m3(float out[3][3], float dst[3][3], float src[3][3], const float srcweight)
 {
 	float srot[3][3], drot[3][3];
 	float squat[4], dquat[4], fquat[4];
@@ -1233,7 +1343,7 @@ void blend_m3_m3m3(float out[][3], float dst[][3], float src[][3], const float s
 	mul_m3_m3m3(out, rmat, smat);
 }
 
-void blend_m4_m4m4(float out[][4], float dst[][4], float src[][4], const float srcweight)
+void blend_m4_m4m4(float out[4][4], float dst[4][4], float src[4][4], const float srcweight)
 {
 	float sloc[3], dloc[3], floc[3];
 	float srot[3][3], drot[3][3];
@@ -1255,14 +1365,14 @@ void blend_m4_m4m4(float out[][4], float dst[][4], float src[][4], const float s
 	loc_quat_size_to_mat4(out, floc, fquat, fsize);
 }
 
-int is_negative_m3(float mat[][3])
+int is_negative_m3(float mat[3][3])
 {
 	float vec[3];
 	cross_v3_v3v3(vec, mat[0], mat[1]);
 	return (dot_v3v3(vec, mat[2]) < 0.0f);
 }
 
-int is_negative_m4(float mat[][4])
+int is_negative_m4(float mat[4][4])
 {
 	float vec[3];
 	cross_v3_v3v3(vec, mat[0], mat[1]);
@@ -1271,7 +1381,7 @@ int is_negative_m4(float mat[][4])
 
 /* make a 4x4 matrix out of 3 transform components */
 /* matrices are made in the order: scale * rot * loc */
-// TODO: need to have a version that allows for rotation order...
+/* TODO: need to have a version that allows for rotation order... */
 
 void loc_eul_size_to_mat4(float mat[4][4], const float loc[3], const float eul[3], const float size[3])
 {
@@ -1352,7 +1462,7 @@ void loc_axisangle_size_to_mat4(float mat[4][4], const float loc[3], const float
 
 /*********************************** Other ***********************************/
 
-void print_m3(const char *str, float m[][3])
+void print_m3(const char *str, float m[3][3])
 {
 	printf("%s\n", str);
 	printf("%f %f %f\n", m[0][0], m[1][0], m[2][0]);
@@ -1361,7 +1471,7 @@ void print_m3(const char *str, float m[][3])
 	printf("\n");
 }
 
-void print_m4(const char *str, float m[][4])
+void print_m4(const char *str, float m[4][4])
 {
 	printf("%s\n", str);
 	printf("%f %f %f %f\n", m[0][0], m[1][0], m[2][0], m[3][0]);
@@ -1388,7 +1498,7 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 	int m = 4;
 	int n = 4;
 	int maxiter = 200;
-	int nu = minf(m, n);
+	int nu = min_ff(m, n);
 
 	float *work = work1;
 	float *e = work2;
@@ -1396,22 +1506,22 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 
 	int i = 0, j = 0, k = 0, p, pp, iter;
 
-	// Reduce A to bidiagonal form, storing the diagonal elements
-	// in s and the super-diagonal elements in e.
+	/* Reduce A to bidiagonal form, storing the diagonal elements
+	 * in s and the super-diagonal elements in e. */
 
-	int nct = minf(m - 1, n);
-	int nrt = maxf(0, minf(n - 2, m));
+	int nct = min_ff(m - 1, n);
+	int nrt = max_ff(0, min_ff(n - 2, m));
 
 	copy_m4_m4(A, A_);
 	zero_m4(U);
 	zero_v4(s);
 
-	for (k = 0; k < maxf(nct, nrt); k++) {
+	for (k = 0; k < max_ff(nct, nrt); k++) {
 		if (k < nct) {
 
-			// Compute the transformation for the k-th column and
-			// place the k-th diagonal in s[k].
-			// Compute 2-norm of k-th column without under/overflow.
+			/* Compute the transformation for the k-th column and
+			 * place the k-th diagonal in s[k].
+			 * Compute 2-norm of k-th column without under/overflow. */
 			s[k] = 0;
 			for (i = k; i < m; i++) {
 				s[k] = hypotf(s[k], A[i][k]);
@@ -1432,7 +1542,7 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 		for (j = k + 1; j < n; j++) {
 			if ((k < nct) && (s[k] != 0.0f)) {
 
-				// Apply the transformation.
+				/* Apply the transformation. */
 
 				float t = 0;
 				for (i = k; i < m; i++) {
@@ -1444,24 +1554,24 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 				}
 			}
 
-			// Place the k-th row of A into e for the
-			// subsequent calculation of the row transformation.
+			/* Place the k-th row of A into e for the */
+			/* subsequent calculation of the row transformation. */
 
 			e[j] = A[k][j];
 		}
 		if (k < nct) {
 
-			// Place the transformation in U for subsequent back
-			// multiplication.
+			/* Place the transformation in U for subsequent back
+			 * multiplication. */
 
 			for (i = k; i < m; i++)
 				U[i][k] = A[i][k];
 		}
 		if (k < nrt) {
 
-			// Compute the k-th row transformation and place the
-			// k-th super-diagonal in e[k].
-			// Compute 2-norm without under/overflow.
+			/* Compute the k-th row transformation and place the
+			 * k-th super-diagonal in e[k].
+			 * Compute 2-norm without under/overflow. */
 			e[k] = 0;
 			for (i = k + 1; i < n; i++) {
 				e[k] = hypotf(e[k], e[i]);
@@ -1481,7 +1591,7 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 			if ((k + 1 < m) & (e[k] != 0.0f)) {
 				float invek1;
 
-				// Apply the transformation.
+				/* Apply the transformation. */
 
 				for (i = k + 1; i < m; i++) {
 					work[i] = 0.0f;
@@ -1500,17 +1610,17 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 				}
 			}
 
-			// Place the transformation in V for subsequent
-			// back multiplication.
+			/* Place the transformation in V for subsequent
+			 * back multiplication. */
 
 			for (i = k + 1; i < n; i++)
 				V[i][k] = e[i];
 		}
 	}
 
-	// Set up the final bidiagonal matrix or order p.
+	/* Set up the final bidiagonal matrix or order p. */
 
-	p = minf(n, m + 1);
+	p = min_ff(n, m + 1);
 	if (nct < n) {
 		s[nct] = A[nct][nct];
 	}
@@ -1522,7 +1632,7 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 	}
 	e[p - 1] = 0.0f;
 
-	// If required, generate U.
+	/* If required, generate U. */
 
 	for (j = nct; j < nu; j++) {
 		for (i = 0; i < m; i++) {
@@ -1558,7 +1668,7 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 		}
 	}
 
-	// If required, generate V.
+	/* If required, generate V. */
 
 	for (k = n - 1; k >= 0; k--) {
 		if ((k < nrt) & (e[k] != 0.0f)) {
@@ -1579,7 +1689,7 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 		V[k][k] = 1.0f;
 	}
 
-	// Main iteration loop for the singular values.
+	/* Main iteration loop for the singular values. */
 
 	pp = p - 1;
 	iter = 0;
@@ -1587,20 +1697,20 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 	while (p > 0) {
 		int kase = 0;
 
-		// Test for maximum iterations to avoid infinite loop
+		/* Test for maximum iterations to avoid infinite loop */
 		if (maxiter == 0)
 			break;
 		maxiter--;
 
-		// This section of the program inspects for
-		// negligible elements in the s and e arrays.  On
-		// completion the variables kase and k are set as follows.
-
-		// kase = 1	  if s(p) and e[k - 1] are negligible and k<p
-		// kase = 2	  if s(k) is negligible and k<p
-		// kase = 3	  if e[k - 1] is negligible, k<p, and
-		//               s(k), ..., s(p) are not negligible (qr step).
-		// kase = 4	  if e(p - 1) is negligible (convergence).
+		/* This section of the program inspects for
+		 * negligible elements in the s and e arrays.  On
+		 * completion the variables kase and k are set as follows.
+		 *
+		 * kase = 1	  if s(p) and e[k - 1] are negligible and k<p
+		 * kase = 2	  if s(k) is negligible and k<p
+		 * kase = 3	  if e[k - 1] is negligible, k<p, and
+		 *               s(k), ..., s(p) are not negligible (qr step).
+		 * kase = 4	  if e(p - 1) is negligible (convergence). */
 
 		for (k = p - 2; k >= -1; k--) {
 			if (k == -1) {
@@ -1641,11 +1751,11 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 		}
 		k++;
 
-		// Perform the task indicated by kase.
+		/* Perform the task indicated by kase. */
 
 		switch (kase) {
 
-			// Deflate negligible s(p).
+			/* Deflate negligible s(p). */
 
 			case 1:
 			{
@@ -1671,7 +1781,7 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 				break;
 			}
 
-			// Split at negligible s(k).
+			/* Split at negligible s(k). */
 
 			case 2:
 			{
@@ -1695,16 +1805,16 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 				break;
 			}
 
-			// Perform one qr step.
+			/* Perform one qr step. */
 
 			case 3:
 			{
 
-				// Calculate the shift.
+				/* Calculate the shift. */
 
-				float scale = maxf(maxf(maxf(maxf(
-				                   fabsf(s[p - 1]),fabsf(s[p - 2])),fabsf(e[p - 2])),
-				                   fabsf(s[k])),fabsf(e[k]));
+				float scale = max_ff(max_ff(max_ff(max_ff(
+				                   fabsf(s[p - 1]), fabsf(s[p - 2])), fabsf(e[p - 2])),
+				                   fabsf(s[k])), fabsf(e[k]));
 				float invscale = 1.0f / scale;
 				float sp = s[p - 1] * invscale;
 				float spm1 = s[p - 2] * invscale;
@@ -1725,7 +1835,7 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 				f = (sk + sp) * (sk - sp) + shift;
 				g = sk * ek;
 
-				// Chase zeros.
+				/* Chase zeros. */
 
 				for (j = k; j < p - 1; j++) {
 					float t = hypotf(f, g);
@@ -1767,12 +1877,12 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 				iter = iter + 1;
 				break;
 			}
-			// Convergence.
+			/* Convergence. */
 
 			case 4:
 			{
 
-				// Make the singular values positive.
+				/* Make the singular values positive. */
 
 				if (s[k] <= 0.0f) {
 					s[k] = (s[k] < 0.0f ? -s[k] : 0.0f);
@@ -1781,7 +1891,7 @@ void svd_m4(float U[4][4], float s[4], float V[4][4], float A_[4][4])
 						V[i][k] = -V[i][k];
 				}
 
-				// Order the singular values.
+				/* Order the singular values. */
 
 				while (k < pp) {
 					float t;
@@ -1835,3 +1945,16 @@ void pseudoinverse_m4_m4(float Ainv[4][4], float A[4][4], float epsilon)
 
 	mul_serie_m4(Ainv, U, Wm, V, NULL, NULL, NULL, NULL, NULL);
 }
+
+void pseudoinverse_m3_m3(float Ainv[3][3], float A[3][3], float epsilon)
+{
+	/* try regular inverse when possible, otherwise fall back to slow svd */
+	if (!invert_m3_m3(Ainv, A)) {
+		float tmp[4][4], tmpinv[4][4];
+
+		copy_m4_m3(tmp, A);
+		pseudoinverse_m4_m4(tmpinv, tmp, epsilon);
+		copy_m3_m4(Ainv, tmpinv);
+	}
+}
+
