@@ -34,8 +34,10 @@
 
 /******************************** Quaternions ********************************/
 
-/* used to test is a quat is not normalized */
-#define QUAT_EPSILON 0.0001
+/* used to test is a quat is not normalized (only used for debug prints) */
+#ifdef DEBUG
+#  define QUAT_EPSILON 0.0001
+#endif
 
 /* convenience, avoids setting Y axis everywhere */
 void unit_axis_angle(float axis[3], float *angle)
@@ -78,7 +80,25 @@ void mul_qt_qtqt(float q[4], const float q1[4], const float q2[4])
 	q[2] = t2;
 }
 
-/* Assumes a unit quaternion */
+/**
+ * \note:
+ * Assumes a unit quaternion?
+ *
+ * \note: multiplying by 3x3 matrix is ~25% faster.
+ *
+ * in fact not, but you may want to use a unit quat, read on...
+ *
+ * Shortcut for 'q v q*' when \a v is actually a quaternion.
+ * This removes the need for converting a vector to a quaternion,
+ * calculating q's conjugate and converting back to a vector.
+ * It also happens to be faster (17+,24* vs * 24+,32*).
+ * If \a q is not a unit quaternion, then \a v will be both rotated by
+ * the same amount as if q was a unit quaternion, and scaled by the square of
+ * the length of q.
+ *
+ * For people used to python mathutils, its like:
+ * def mul_qt_v3(q, v): (q * Quaternion((0.0, v[0], v[1], v[2])) * q.conjugated())[1:]
+ */
 void mul_qt_v3(const float q[4], float v[3])
 {
 	float t0, t1, t2;
@@ -97,6 +117,14 @@ void mul_qt_v3(const float q[4], float v[3])
 	v[1] = t2;
 }
 
+void conjugate_qt_qt(float q1[4], const float q2[4])
+{
+	q1[0] =  q2[0];
+	q1[1] = -q2[1];
+	q1[2] = -q2[2];
+	q1[3] = -q2[3];
+}
+
 void conjugate_qt(float q[4])
 {
 	q[1] = -q[1];
@@ -150,17 +178,16 @@ void sub_qt_qtqt(float q[4], const float q1[4], const float q2[4])
 /* angular mult factor */
 void mul_fac_qt_fl(float q[4], const float fac)
 {
-	float angle = fac * saacos(q[0]); /* quat[0] = cos(0.5 * angle), but now the 0.5 and 2.0 rule out */
-
-	float co = (float)cos(angle);
-	float si = (float)sin(angle);
+	const float angle = fac * saacos(q[0]); /* quat[0] = cos(0.5 * angle), but now the 0.5 and 2.0 rule out */
+	const float co = cosf(angle);
+	const float si = sinf(angle);
 	q[0] = co;
 	normalize_v3(q + 1);
 	mul_v3_fl(q + 1, si);
 }
 
 /* skip error check, currently only needed by mat3_to_quat_is_ok */
-static void quat_to_mat3_no_error(float m[][3], const float q[4])
+static void quat_to_mat3_no_error(float m[3][3], const float q[4])
 {
 	double q0, q1, q2, q3, qda, qdb, qdc, qaa, qab, qac, qbb, qbc, qcc;
 
@@ -192,7 +219,7 @@ static void quat_to_mat3_no_error(float m[][3], const float q[4])
 	m[2][2] = (float)(1.0 - qaa - qbb);
 }
 
-void quat_to_mat3(float m[][3], const float q[4])
+void quat_to_mat3(float m[3][3], const float q[4])
 {
 #ifdef DEBUG
 	float f;
@@ -204,12 +231,12 @@ void quat_to_mat3(float m[][3], const float q[4])
 	quat_to_mat3_no_error(m, q);
 }
 
-void quat_to_mat4(float m[][4], const float q[4])
+void quat_to_mat4(float m[4][4], const float q[4])
 {
 	double q0, q1, q2, q3, qda, qdb, qdc, qaa, qab, qac, qbb, qbc, qcc;
 
 #ifdef DEBUG
-	if (!((q0 = dot_qtqt(q, q)) == 0.0f || (fabsf(q0 - 1.0) < QUAT_EPSILON))) {
+	if (!((q0 = dot_qtqt(q, q)) == 0.0 || (fabs(q0 - 1.0) < QUAT_EPSILON))) {
 		fprintf(stderr, "Warning! quat_to_mat4() called with non-normalized: size %.8f *** report a bug ***\n", (float)q0);
 	}
 #endif
@@ -248,7 +275,7 @@ void quat_to_mat4(float m[][4], const float q[4])
 	m[3][3] = 1.0f;
 }
 
-void mat3_to_quat(float q[4], float wmat[][3])
+void mat3_to_quat(float q[4], float wmat[3][3])
 {
 	double tr, s;
 	float mat[3][3];
@@ -263,9 +290,9 @@ void mat3_to_quat(float q[4], float wmat[][3])
 		s = sqrt(tr);
 		q[0] = (float)s;
 		s = 1.0 / (4.0 * s);
-		q[1] = (float)((mat[1][2] - mat[2][1]) * s);
-		q[2] = (float)((mat[2][0] - mat[0][2]) * s);
-		q[3] = (float)((mat[0][1] - mat[1][0]) * s);
+		q[1] = (float)((double)(mat[1][2] - mat[2][1]) * s);
+		q[2] = (float)((double)(mat[2][0] - mat[0][2]) * s);
+		q[3] = (float)((double)(mat[0][1] - mat[1][0]) * s);
 	}
 	else {
 		if (mat[0][0] > mat[1][1] && mat[0][0] > mat[2][2]) {
@@ -300,7 +327,7 @@ void mat3_to_quat(float q[4], float wmat[][3])
 	normalize_qt(q);
 }
 
-void mat4_to_quat(float q[4], float m[][4])
+void mat4_to_quat(float q[4], float m[4][4])
 {
 	float mat[3][3];
 
@@ -326,8 +353,8 @@ void mat3_to_quat_is_ok(float q[4], float wmat[3][3])
 	co = mat[2][2];
 	angle = 0.5f * saacos(co);
 
-	co = (float)cos(angle);
-	si = (float)sin(angle);
+	co = cosf(angle);
+	si = sinf(angle);
 	q1[0] = co;
 	q1[1] = -nor[0] * si; /* negative here, but why? */
 	q1[2] = -nor[1] * si;
@@ -341,8 +368,8 @@ void mat3_to_quat_is_ok(float q[4], float wmat[3][3])
 	/* and align x-axes */
 	angle = (float)(0.5 * atan2(mat[0][1], mat[0][0]));
 
-	co = (float)cos(angle);
-	si = (float)sin(angle);
+	co = cosf(angle);
+	si = sinf(angle);
 	q2[0] = co;
 	q2[1] = 0.0f;
 	q2[2] = 0.0f;
@@ -355,7 +382,7 @@ float normalize_qt(float q[4])
 {
 	float len;
 
-	len = (float)sqrt(dot_qtqt(q, q));
+	len = sqrtf(dot_qtqt(q, q));
 	if (len != 0.0f) {
 		mul_qt_fl(q, 1.0f / len);
 	}
@@ -389,113 +416,110 @@ void rotation_between_vecs_to_quat(float q[4], const float v1[3], const float v2
 void rotation_between_quats_to_quat(float q[4], const float q1[4], const float q2[4])
 {
 	float tquat[4];
-	double dot = 0.0f;
-	int x;
 
-	copy_qt_qt(tquat, q1);
-	conjugate_qt(tquat);
-	dot = 1.0f / dot_qtqt(tquat, tquat);
+	conjugate_qt_qt(tquat, q1);
 
-	for (x = 0; x < 4; x++)
-		tquat[x] *= dot;
+	mul_qt_fl(tquat, 1.0f / dot_qtqt(tquat, tquat));
 
 	mul_qt_qtqt(q, tquat, q2);
 }
 
 void vec_to_quat(float q[4], const float vec[3], short axis, const short upflag)
 {
-	float q2[4], nor[3], *fp, mat[3][3], angle, si, co, x2, y2, z2, len1;
+	float nor[3], tvec[3];
+	float angle, si, co, len;
 
 	assert(axis >= 0 && axis <= 5);
 	assert(upflag >= 0 && upflag <= 2);
 
-	/* first rotate to axis */
+	/* first set the quat to unit */
+	unit_qt(q);
+
+	len = len_v3(vec);
+
+	if (UNLIKELY(len == 0.0f)) {
+		return;
+	}
+
+	/* rotate to axis */
 	if (axis > 2) {
-		x2 = vec[0];
-		y2 = vec[1];
-		z2 = vec[2];
+		copy_v3_v3(tvec, vec);
 		axis -= 3;
 	}
 	else {
-		x2 = -vec[0];
-		y2 = -vec[1];
-		z2 = -vec[2];
+		negate_v3_v3(tvec, vec);
 	}
 
-	q[0] = 1.0;
-	q[1] = q[2] = q[3] = 0.0;
-
-	len1 = (float)sqrt(x2 * x2 + y2 * y2 + z2 * z2);
-	if (len1 == 0.0f) return;
-
 	/* nasty! I need a good routine for this...
 	 * problem is a rotation of an Y axis to the negative Y-axis for example.
 	 */
 
 	if (axis == 0) { /* x-axis */
-		nor[0] = 0.0;
-		nor[1] = -z2;
-		nor[2] = y2;
+		nor[0] =  0.0;
+		nor[1] = -tvec[2];
+		nor[2] =  tvec[1];
 
-		if (fabs(y2) + fabs(z2) < 0.0001)
-			nor[1] = 1.0;
+		if (fabsf(tvec[1]) + fabsf(tvec[2]) < 0.0001f)
+			nor[1] = 1.0f;
 
-		co = x2;
+		co = tvec[0];
 	}
 	else if (axis == 1) { /* y-axis */
-		nor[0] = z2;
-		nor[1] = 0.0;
-		nor[2] = -x2;
+		nor[0] =  tvec[2];
+		nor[1] =  0.0;
+		nor[2] = -tvec[0];
 
-		if (fabs(x2) + fabs(z2) < 0.0001)
-			nor[2] = 1.0;
+		if (fabsf(tvec[0]) + fabsf(tvec[2]) < 0.0001f)
+			nor[2] = 1.0f;
 
-		co = y2;
+		co = tvec[1];
 	}
 	else { /* z-axis */
-		nor[0] = -y2;
-		nor[1] = x2;
-		nor[2] = 0.0;
+		nor[0] = -tvec[1];
+		nor[1] =  tvec[0];
+		nor[2] =  0.0;
 
-		if (fabs(x2) + fabs(y2) < 0.0001)
-			nor[0] = 1.0;
+		if (fabsf(tvec[0]) + fabsf(tvec[1]) < 0.0001f)
+			nor[0] = 1.0f;
 
-		co = z2;
+		co = tvec[2];
 	}
-	co /= len1;
+	co /= len;
 
 	normalize_v3(nor);
 
 	angle = 0.5f * saacos(co);
-	si = (float)sin(angle);
-	q[0] = (float)cos(angle);
+	si   = sinf(angle);
+	q[0] = cosf(angle);
 	q[1] = nor[0] * si;
 	q[2] = nor[1] * si;
 	q[3] = nor[2] * si;
 
 	if (axis != upflag) {
+		float mat[3][3];
+		float q2[4];
+		const float *fp = mat[2];
 		quat_to_mat3(mat, q);
 
-		fp = mat[2];
 		if (axis == 0) {
-			if (upflag == 1) angle = (float)(0.5 * atan2(fp[2], fp[1]));
-			else angle = (float)(-0.5 * atan2(fp[1], fp[2]));
+			if (upflag == 1) angle =  0.5f * atan2f(fp[2], fp[1]);
+			else             angle = -0.5f * atan2f(fp[1], fp[2]);
 		}
 		else if (axis == 1) {
-			if (upflag == 0) angle = (float)(-0.5 * atan2(fp[2], fp[0]));
-			else angle = (float)(0.5 * atan2(fp[0], fp[2]));
+			if (upflag == 0) angle = -0.5f * atan2f(fp[2], fp[0]);
+			else             angle =  0.5f * atan2f(fp[0], fp[2]);
 		}
 		else {
-			if (upflag == 0) angle = (float)(0.5 * atan2(-fp[1], -fp[0]));
-			else angle = (float)(-0.5 * atan2(-fp[0], -fp[1]));
+			if (upflag == 0) angle =  0.5f * atan2f(-fp[1], -fp[0]);
+			else             angle = -0.5f * atan2f(-fp[0], -fp[1]);
 		}
 
 		co = cosf(angle);
-		si = sinf(angle) / len1;
+		si = sinf(angle) / len;
 		q2[0] = co;
-		q2[1] = x2 * si;
-		q2[2] = y2 * si;
-		q2[3] = z2 * si;
+		q2[1] = tvec[0] * si;
+		q2[2] = tvec[1] * si;
+		q2[3] = tvec[2] * si;
 
 		mul_qt_qtqt(q, q2, q);
 	}
@@ -567,10 +591,10 @@ void interp_qt_qtqt(float result[4], const float quat1[4], const float quat2[4],
 	}
 
 	if ((1.0f - cosom) > 0.0001f) {
-		omega = (float)acos(cosom);
-		sinom = (float)sin(omega);
-		sc1 = (float)sin((1 - t) * omega) / sinom;
-		sc2 = (float)sin(t * omega) / sinom;
+		omega = acosf(cosom);
+		sinom = sinf(omega);
+		sc1 = sinf((1.0f - t) * omega) / sinom;
+		sc2 = sinf(t * omega) / sinom;
 	}
 	else {
 		sc1 = 1.0f - t;
@@ -591,24 +615,34 @@ void add_qt_qtqt(float result[4], const float quat1[4], const float quat2[4], co
 	result[3] = quat1[3] + t * quat2[3];
 }
 
-void tri_to_quat(float quat[4], const float v1[3], const float v2[3], const float v3[3])
+/* same as tri_to_quat() but takes pre-computed normal from the triangle
+ * used for ngons when we know their normal */
+void tri_to_quat_ex(float quat[4], const float v1[3], const float v2[3], const float v3[3],
+                    const float no_orig[3])
 {
 	/* imaginary x-axis, y-axis triangle is being rotated */
 	float vec[3], q1[4], q2[4], n[3], si, co, angle, mat[3][3], imat[3][3];
 
 	/* move z-axis to face-normal */
+#if 0
 	normal_tri_v3(vec, v1, v2, v3);
+#else
+	copy_v3_v3(vec, no_orig);
+	(void)v3;
+#endif
 
-	n[0] = vec[1];
+	n[0] =  vec[1];
 	n[1] = -vec[0];
-	n[2] = 0.0f;
+	n[2] =  0.0f;
 	normalize_v3(n);
 
-	if (n[0] == 0.0f && n[1] == 0.0f) n[0] = 1.0f;
+	if (n[0] == 0.0f && n[1] == 0.0f) {
+		n[0] = 1.0f;
+	}
 
 	angle = -0.5f * (float)saacos(vec[2]);
-	co = (float)cos(angle);
-	si = (float)sin(angle);
+	co = cosf(angle);
+	si = sinf(angle);
 	q1[0] = co;
 	q1[1] = n[0] * si;
 	q1[2] = n[1] * si;
@@ -625,8 +659,8 @@ void tri_to_quat(float quat[4], const float v1[3], const float v2[3], const floa
 	normalize_v3(vec);
 
 	angle = (float)(0.5 * atan2(vec[1], vec[0]));
-	co = (float)cos(angle);
-	si = (float)sin(angle);
+	co = cosf(angle);
+	si = sinf(angle);
 	q2[0] = co;
 	q2[1] = 0.0f;
 	q2[2] = 0.0f;
@@ -635,6 +669,13 @@ void tri_to_quat(float quat[4], const float v1[3], const float v2[3], const floa
 	mul_qt_qtqt(quat, q1, q2);
 }
 
+void tri_to_quat(float quat[4], const float v1[3], const float v2[3], const float v3[3])
+{
+	float vec[3];
+	normal_tri_v3(vec, v1, v2, v3);
+	tri_to_quat_ex(quat, v1, v2, v3, vec);
+}
+
 void print_qt(const char *str, const float q[4])
 {
 	printf("%s: %.3f %.3f %.3f %.3f\n", str, q[0], q[1], q[2], q[3]);
@@ -643,22 +684,22 @@ void print_qt(const char *str, const float q[4])
 /******************************** Axis Angle *********************************/
 
 /* Axis angle to Quaternions */
-void axis_angle_to_quat(float q[4], const float axis[3], float angle)
+void axis_angle_to_quat(float q[4], const float axis[3], const float angle)
 {
 	float nor[3];
-	float si;
 
-	if (normalize_v3_v3(nor, axis) == 0.0f) {
+	if (LIKELY(normalize_v3_v3(nor, axis) != 0.0f)) {
+		const float phi = angle / 2.0f;
+		float si;
+		si   = sinf(phi);
+		q[0] = cosf(phi);
+		q[1] = nor[0] * si;
+		q[2] = nor[1] * si;
+		q[3] = nor[2] * si;
+	}
+	else {
 		unit_qt(q);
-		return;
 	}
-
-	angle /= 2;
-	si = (float)sin(angle);
-	q[0] = (float)cos(angle);
-	q[1] = nor[0] * si;
-	q[2] = nor[1] * si;
-	q[3] = nor[2] * si;
 }
 
 /* Quaternions to Axis Angle */
@@ -673,14 +714,14 @@ void quat_to_axis_angle(float axis[3], float *angle, const float q[4])
 #endif
 
 	/* calculate angle/2, and sin(angle/2) */
-	ha = (float)acos(q[0]);
-	si = (float)sin(ha);
+	ha = acosf(q[0]);
+	si = sinf(ha);
 
 	/* from half-angle to angle */
 	*angle = ha * 2;
 
 	/* prevent division by zero for axis conversion */
-	if (fabs(si) < 0.0005)
+	if (fabsf(si) < 0.0005f)
 		si = 1.0f;
 
 	axis[0] = q[1] / si;
@@ -723,8 +764,8 @@ void axis_angle_to_mat3(float mat[3][3], const float axis[3], const float angle)
 	}
 
 	/* now convert this to a 3x3 matrix */
-	co = (float)cos(angle);
-	si = (float)sin(angle);
+	co = cosf(angle);
+	si = sinf(angle);
 
 	ico = (1.0f - co);
 	nsi[0] = nor[0] * si;
@@ -758,7 +799,7 @@ void mat3_to_axis_angle(float axis[3], float *angle, float mat[3][3])
 	float q[4];
 
 	/* use quaternions as intermediate representation */
-	// TODO: it would be nicer to go straight there...
+	/* TODO: it would be nicer to go straight there... */
 	mat3_to_quat(q, mat);
 	quat_to_axis_angle(axis, angle, q);
 }
@@ -769,7 +810,7 @@ void mat4_to_axis_angle(float axis[3], float *angle, float mat[4][4])
 	float q[4];
 
 	/* use quaternions as intermediate representation */
-	// TODO: it would be nicer to go straight there...
+	/* TODO: it would be nicer to go straight there... */
 	mat4_to_quat(q, mat);
 	quat_to_axis_angle(axis, angle, q);
 }
@@ -819,10 +860,10 @@ void single_axis_angle_to_mat3(float mat[3][3], const char axis, const float ang
 }
 
 /****************************** Vector/Rotation ******************************/
-/* TODO: the following calls should probably be depreceated sometime         */
+/* TODO: the following calls should probably be deprecated sometime         */
 
-/* ODO, replace use of this function with axis_angle_to_mat3() */
-void vec_rot_to_mat3(float mat[][3], const float vec[3], const float phi)
+/* TODO, replace use of this function with axis_angle_to_mat3() */
+void vec_rot_to_mat3(float mat[3][3], const float vec[3], const float phi)
 {
 	/* rotation of phi radials around vec */
 	float vx, vx2, vy, vy2, vz, vz2, co, si;
@@ -833,8 +874,8 @@ void vec_rot_to_mat3(float mat[][3], const float vec[3], const float phi)
 	vx2 = vx * vx;
 	vy2 = vy * vy;
 	vz2 = vz * vz;
-	co = (float)cos(phi);
-	si = (float)sin(phi);
+	co = cosf(phi);
+	si = sinf(phi);
 
 	mat[0][0] = vx2 + co * (1.0f - vx2);
 	mat[0][1] = vx * vy * (1.0f - co) + vz * si;
@@ -847,42 +888,10 @@ void vec_rot_to_mat3(float mat[][3], const float vec[3], const float phi)
 	mat[2][2] = vz2 + co * (1.0f - vz2);
 }
 
-/* axis angle to 4x4 matrix */
-void vec_rot_to_mat4(float mat[][4], const float vec[3], const float phi)
-{
-	float tmat[3][3];
-
-	vec_rot_to_mat3(tmat, vec, phi);
-	unit_m4(mat);
-	copy_m4_m3(mat, tmat);
-}
-
-/* axis angle to quaternion */
-void vec_rot_to_quat(float *quat, const float vec[3], const float phi)
-{
-	/* rotation of phi radials around vec */
-	float si;
-
-	quat[1] = vec[0];
-	quat[2] = vec[1];
-	quat[3] = vec[2];
-
-	if (normalize_v3(quat + 1) == 0.0f) {
-		unit_qt(quat);
-	}
-	else {
-		quat[0] = (float)cos((double)phi / 2.0);
-		si = (float)sin((double)phi / 2.0);
-		quat[1] *= si;
-		quat[2] *= si;
-		quat[3] *= si;
-	}
-}
-
 /******************************** XYZ Eulers *********************************/
 
 /* XYZ order */
-void eul_to_mat3(float mat[][3], const float eul[3])
+void eul_to_mat3(float mat[3][3], const float eul[3])
 {
 	double ci, cj, ch, si, sj, sh, cc, cs, sc, ss;
 
@@ -910,7 +919,7 @@ void eul_to_mat3(float mat[][3], const float eul[3])
 }
 
 /* XYZ order */
-void eul_to_mat4(float mat[][4], const float eul[3])
+void eul_to_mat4(float mat[4][4], const float eul[3])
 {
 	double ci, cj, ch, si, sj, sh, cc, cs, sc, ss;
 
@@ -943,7 +952,7 @@ void eul_to_mat4(float mat[][4], const float eul[3])
 /* returns two euler calculation methods, so we can pick the best */
 
 /* XYZ order */
-static void mat3_to_eul2(float tmat[][3], float eul1[3], float eul2[3])
+static void mat3_to_eul2(float tmat[3][3], float eul1[3], float eul2[3])
 {
 	float cy, quat[4], mat[3][3];
 
@@ -975,14 +984,14 @@ static void mat3_to_eul2(float tmat[][3], float eul1[3], float eul2[3])
 }
 
 /* XYZ order */
-void mat3_to_eul(float *eul, float tmat[][3])
+void mat3_to_eul(float *eul, float tmat[3][3])
 {
 	float eul1[3], eul2[3];
 
 	mat3_to_eul2(tmat, eul1, eul2);
 
 	/* return best, which is just the one with lowest values it in */
-	if (fabs(eul1[0]) + fabs(eul1[1]) + fabs(eul1[2]) > fabs(eul2[0]) + fabs(eul2[1]) + fabs(eul2[2])) {
+	if (fabsf(eul1[0]) + fabsf(eul1[1]) + fabsf(eul1[2]) > fabsf(eul2[0]) + fabsf(eul2[1]) + fabsf(eul2[2])) {
 		copy_v3_v3(eul, eul2);
 	}
 	else {
@@ -991,7 +1000,7 @@ void mat3_to_eul(float *eul, float tmat[][3])
 }
 
 /* XYZ order */
-void mat4_to_eul(float *eul, float tmat[][4])
+void mat4_to_eul(float *eul, float tmat[4][4])
 {
 	float tempMat[3][3];
 
@@ -1055,90 +1064,52 @@ void rotate_eul(float *beul, const char axis, const float ang)
 
 }
 
-/* exported to transform.c */
-
 /* order independent! */
 void compatible_eul(float eul[3], const float oldrot[3])
 {
-	float dx, dy, dz;
+	/* we could use M_PI as pi_thresh: which is correct but 5.1 gives better results.
+	 * Checked with baking actions to fcurves - campbell */
+	const float pi_thresh = (5.1f);
+	const float pi_x2     = (2.0f * (float)M_PI);
+
+	float deul[3];
+	unsigned int i;
 
 	/* correct differences of about 360 degrees first */
-	dx = eul[0] - oldrot[0];
-	dy = eul[1] - oldrot[1];
-	dz = eul[2] - oldrot[2];
-
-	while (fabs(dx) > 5.1) {
-		if (dx > 0.0f) eul[0] -= 2.0f * (float)M_PI;
-		else eul[0] += 2.0f * (float)M_PI;
-		dx = eul[0] - oldrot[0];
-	}
-	while (fabs(dy) > 5.1) {
-		if (dy > 0.0f) eul[1] -= 2.0f * (float)M_PI;
-		else eul[1] += 2.0f * (float)M_PI;
-		dy = eul[1] - oldrot[1];
-	}
-	while (fabs(dz) > 5.1) {
-		if (dz > 0.0f) eul[2] -= 2.0f * (float)M_PI;
-		else eul[2] += 2.0f * (float)M_PI;
-		dz = eul[2] - oldrot[2];
+	for (i = 0; i < 3; i++) {
+		deul[i] = eul[i] - oldrot[i];
+		if (deul[i] > pi_thresh) {
+			eul[i] -= floorf(( deul[i] / pi_x2) + 0.5f) * pi_x2;
+			deul[i] = eul[i] - oldrot[i];
+		}
+		else if (deul[i] < -pi_thresh) {
+			eul[i] += floorf((-deul[i] / pi_x2) + 0.5f) * pi_x2;
+			deul[i] = eul[i] - oldrot[i];
+		}
 	}
 
 	/* is 1 of the axis rotations larger than 180 degrees and the other small? NO ELSE IF!! */
-	if (fabs(dx) > 3.2 && fabs(dy) < 1.6 && fabs(dz) < 1.6) {
-		if (dx > 0.0f) eul[0] -= 2.0f * (float)M_PI;
-		else eul[0] += 2.0f * (float)M_PI;
+	if (fabsf(deul[0]) > 3.2f && fabsf(deul[1]) < 1.6f && fabsf(deul[2]) < 1.6f) {
+		if (deul[0] > 0.0f) eul[0] -= pi_x2;
+		else                eul[0] += pi_x2;
 	}
-	if (fabs(dy) > 3.2 && fabs(dz) < 1.6 && fabs(dx) < 1.6) {
-		if (dy > 0.0f) eul[1] -= 2.0f * (float)M_PI;
-		else eul[1] += 2.0f * (float)M_PI;
+	if (fabsf(deul[1]) > 3.2f && fabsf(deul[2]) < 1.6f && fabsf(deul[0]) < 1.6f) {
+		if (deul[1] > 0.0f) eul[1] -= pi_x2;
+		else                eul[1] += pi_x2;
 	}
-	if (fabs(dz) > 3.2 && fabs(dx) < 1.6 && fabs(dy) < 1.6) {
-		if (dz > 0.0f) eul[2] -= 2.0f * (float)M_PI;
-		else eul[2] += 2.0f * (float)M_PI;
+	if (fabsf(deul[2]) > 3.2f && fabsf(deul[0]) < 1.6f && fabsf(deul[1]) < 1.6f) {
+		if (deul[2] > 0.0f) eul[2] -= pi_x2;
+		else                eul[2] += pi_x2;
 	}
 
-	/* the method below was there from ancient days... but why! probably because the code sucks :)
-	 */
-#if 0
-	/* calc again */
-	dx = eul[0] - oldrot[0];
-	dy = eul[1] - oldrot[1];
-	dz = eul[2] - oldrot[2];
-
-	/* special case, tested for x-z  */
-
-	if ((fabs(dx) > 3.1 && fabs(dz) > 1.5) || (fabs(dx) > 1.5 && fabs(dz) > 3.1)) {
-		if (dx > 0.0) eul[0] -= M_PI;
-		else eul[0] += M_PI;
-		if (eul[1] > 0.0) eul[1] = M_PI - eul[1];
-		else eul[1] = -M_PI - eul[1];
-		if (dz > 0.0) eul[2] -= M_PI;
-		else eul[2] += M_PI;
-
-	}
-	else if ((fabs(dx) > 3.1 && fabs(dy) > 1.5) || (fabs(dx) > 1.5 && fabs(dy) > 3.1)) {
-		if (dx > 0.0) eul[0] -= M_PI;
-		else eul[0] += M_PI;
-		if (dy > 0.0) eul[1] -= M_PI;
-		else eul[1] += M_PI;
-		if (eul[2] > 0.0) eul[2] = M_PI - eul[2];
-		else eul[2] = -M_PI - eul[2];
-	}
-	else if ((fabs(dy) > 3.1 && fabs(dz) > 1.5) || (fabs(dy) > 1.5 && fabs(dz) > 3.1)) {
-		if (eul[0] > 0.0) eul[0] = M_PI - eul[0];
-		else eul[0] = -M_PI - eul[0];
-		if (dy > 0.0) eul[1] -= M_PI;
-		else eul[1] += M_PI;
-		if (dz > 0.0) eul[2] -= M_PI;
-		else eul[2] += M_PI;
-	}
-#endif
+#undef PI_THRESH
+#undef PI_2F
 }
 
 /* uses 2 methods to retrieve eulers, and picks the closest */
 
 /* XYZ order */
-void mat3_to_compatible_eul(float eul[3], const float oldrot[3], float mat[][3])
+void mat3_to_compatible_eul(float eul[3], const float oldrot[3], float mat[3][3])
 {
 	float eul1[3], eul2[3];
 	float d1, d2;
@@ -1148,8 +1119,8 @@ void mat3_to_compatible_eul(float eul[3], const float oldrot[3], float mat[][3])
 	compatible_eul(eul1, oldrot);
 	compatible_eul(eul2, oldrot);
 
-	d1 = (float)fabs(eul1[0] - oldrot[0]) + (float)fabs(eul1[1] - oldrot[1]) + (float)fabs(eul1[2] - oldrot[2]);
-	d2 = (float)fabs(eul2[0] - oldrot[0]) + (float)fabs(eul2[1] - oldrot[1]) + (float)fabs(eul2[2] - oldrot[2]);
+	d1 = fabsf(eul1[0] - oldrot[0]) + fabsf(eul1[1] - oldrot[1]) + fabsf(eul1[2] - oldrot[2]);
+	d2 = fabsf(eul2[0] - oldrot[0]) + fabsf(eul2[1] - oldrot[1]) + fabsf(eul2[2] - oldrot[2]);
 
 	/* return best, which is just the one with lowest difference */
 	if (d1 > d2) {
@@ -1181,14 +1152,14 @@ typedef struct RotOrderInfo {
 /* Array of info for Rotation Order calculations
  * WARNING: must be kept in same order as eEulerRotationOrders
  */
-static RotOrderInfo rotOrders[] = {
+static const RotOrderInfo rotOrders[] = {
 	/* i, j, k, n */
-	{{0, 1, 2}, 0}, // XYZ
-	{{0, 2, 1}, 1}, // XZY
-	{{1, 0, 2}, 1}, // YXZ
-	{{1, 2, 0}, 0}, // YZX
-	{{2, 0, 1}, 0}, // ZXY
-	{{2, 1, 0}, 1}  // ZYX
+	{{0, 1, 2}, 0}, /* XYZ */
+	{{0, 2, 1}, 1}, /* XZY */
+	{{1, 0, 2}, 1}, /* YXZ */
+	{{1, 2, 0}, 0}, /* YZX */
+	{{2, 0, 1}, 0}, /* ZXY */
+	{{2, 1, 0}, 1}  /* ZYX */
 };
 
 /* Get relevant pointer to rotation order set from the array
@@ -1200,7 +1171,7 @@ static RotOrderInfo rotOrders[] = {
 /* Construct quaternion from Euler angles (in radians). */
 void eulO_to_quat(float q[4], const float e[3], const short order)
 {
-	RotOrderInfo *R = GET_ROTATIONORDER_INFO(order);
+	const RotOrderInfo *R = GET_ROTATIONORDER_INFO(order);
 	short i = R->axis[0], j = R->axis[1], k = R->axis[2];
 	double ti, tj, th, ci, cj, ch, si, sj, sh, cc, cs, sc, ss;
 	double a[3];
@@ -1245,7 +1216,7 @@ void quat_to_eulO(float e[3], short const order, const float q[4])
 /* Construct 3x3 matrix from Euler angles (in radians). */
 void eulO_to_mat3(float M[3][3], const float e[3], const short order)
 {
-	RotOrderInfo *R = GET_ROTATIONORDER_INFO(order);
+	const RotOrderInfo *R = GET_ROTATIONORDER_INFO(order);
 	short i = R->axis[0], j = R->axis[1], k = R->axis[2];
 	double ti, tj, th, ci, cj, ch, si, sj, sh, cc, cs, sc, ss;
 
@@ -1284,9 +1255,9 @@ void eulO_to_mat3(float M[3][3], const float e[3], const short order)
 }
 
 /* returns two euler calculation methods, so we can pick the best */
-static void mat3_to_eulo2(float M[3][3], float *e1, float *e2, short order)
+static void mat3_to_eulo2(float M[3][3], float *e1, float *e2, const short order)
 {
-	RotOrderInfo *R = GET_ROTATIONORDER_INFO(order);
+	const RotOrderInfo *R = GET_ROTATIONORDER_INFO(order);
 	short i = R->axis[0], j = R->axis[1], k = R->axis[2];
 	float m[3][3];
 	double cy;
@@ -1344,7 +1315,7 @@ void mat3_to_eulO(float eul[3], const short order, float M[3][3])
 	mat3_to_eulo2(M, eul1, eul2, order);
 
 	/* return best, which is just the one with lowest values it in */
-	if (fabs(eul1[0]) + fabs(eul1[1]) + fabs(eul1[2]) > fabs(eul2[0]) + fabs(eul2[1]) + fabs(eul2[2])) {
+	if (fabsf(eul1[0]) + fabsf(eul1[1]) + fabsf(eul1[2]) > fabsf(eul2[0]) + fabsf(eul2[1]) + fabsf(eul2[2])) {
 		copy_v3_v3(eul, eul2);
 	}
 	else {
@@ -1364,7 +1335,7 @@ void mat4_to_eulO(float e[3], const short order, float M[4][4])
 }
 
 /* uses 2 methods to retrieve eulers, and picks the closest */
-void mat3_to_compatible_eulO(float eul[3], float oldrot[3], short order, float mat[3][3])
+void mat3_to_compatible_eulO(float eul[3], float oldrot[3], const short order, float mat[3][3])
 {
 	float eul1[3], eul2[3];
 	float d1, d2;
@@ -1384,7 +1355,7 @@ void mat3_to_compatible_eulO(float eul[3], float oldrot[3], short order, float m
 		copy_v3_v3(eul, eul1);
 }
 
-void mat4_to_compatible_eulO(float eul[3], float oldrot[3], short order, float M[4][4])
+void mat4_to_compatible_eulO(float eul[3], float oldrot[3], const short order, float M[4][4])
 {
 	float m[3][3];
 
@@ -1394,9 +1365,9 @@ void mat4_to_compatible_eulO(float eul[3], float oldrot[3], short order, float M
 	mat3_to_compatible_eulO(eul, oldrot, order, m);
 }
 /* rotate the given euler by the given angle on the specified axis */
-// NOTE: is this safe to do with different axis orders?
+/* NOTE: is this safe to do with different axis orders? */
 
-void rotate_eulO(float beul[3], short order, char axis, float ang)
+void rotate_eulO(float beul[3], const short order, char axis, float ang)
 {
 	float eul[3], mat1[3][3], mat2[3][3], totmat[3][3];
 
@@ -1419,9 +1390,9 @@ void rotate_eulO(float beul[3], short order, char axis, float ang)
 }
 
 /* the matrix is written to as 3 axis vectors */
-void eulO_to_gimbal_axis(float gmat[][3], const float eul[3], const short order)
+void eulO_to_gimbal_axis(float gmat[3][3], const float eul[3], const short order)
 {
-	RotOrderInfo *R = GET_ROTATIONORDER_INFO(order);
+	const RotOrderInfo *R = GET_ROTATIONORDER_INFO(order);
 
 	float mat[3][3];
 	float teul[3];
@@ -1478,7 +1449,7 @@ void eulO_to_gimbal_axis(float gmat[][3], const float eul[3], const short order)
  * - added support for scaling
  */
 
-void mat4_to_dquat(DualQuat *dq, float basemat[][4], float mat[][4])
+void mat4_to_dquat(DualQuat *dq, float basemat[4][4], float mat[4][4])
 {
 	float *t, *q, dscale[3], scale[3], basequat[4];
 	float baseRS[4][4], baseinv[4][4], baseR[4][4], baseRinv[4][4];
@@ -1489,10 +1460,9 @@ void mat4_to_dquat(DualQuat *dq, float basemat[][4], float mat[][4])
 	mult_m4_m4m4(baseRS, mat, basemat);
 	mat4_to_size(scale, baseRS);
 
-	copy_v3_v3(dscale, scale);
-	dscale[0] -= 1.0f;
-	dscale[1] -= 1.0f;
-	dscale[2] -= 1.0f;
+	dscale[0] = scale[0] - 1.0f;
+	dscale[1] = scale[1] - 1.0f;
+	dscale[2] = scale[2] - 1.0f;
 
 	if ((determinant_m4(mat) < 0.0f) || len_v3(dscale) > 1e-4f) {
 		/* extract R and S  */
@@ -1528,13 +1498,13 @@ void mat4_to_dquat(DualQuat *dq, float basemat[][4], float mat[][4])
 	/* dual part */
 	t = R[3];
 	q = dq->quat;
-	dq->trans[0] = -0.5f * (t[0] * q[1] + t[1] * q[2] + t[2] * q[3]);
-	dq->trans[1] = 0.5f * (t[0] * q[0] + t[1] * q[3] - t[2] * q[2]);
-	dq->trans[2] = 0.5f * (-t[0] * q[3] + t[1] * q[0] + t[2] * q[1]);
-	dq->trans[3] = 0.5f * (t[0] * q[2] - t[1] * q[1] + t[2] * q[0]);
+	dq->trans[0] = -0.5f * ( t[0] * q[1] + t[1] * q[2] + t[2] * q[3]);
+	dq->trans[1] =  0.5f * ( t[0] * q[0] + t[1] * q[3] - t[2] * q[2]);
+	dq->trans[2] =  0.5f * (-t[0] * q[3] + t[1] * q[0] + t[2] * q[1]);
+	dq->trans[3] =  0.5f * ( t[0] * q[2] - t[1] * q[1] + t[2] * q[0]);
 }
 
-void dquat_to_mat4(float mat[][4], DualQuat *dq)
+void dquat_to_mat4(float mat[4][4], DualQuat *dq)
 {
 	float len, *t, q0[4];
 
@@ -1542,7 +1512,7 @@ void dquat_to_mat4(float mat[][4], DualQuat *dq)
 	copy_qt_qt(q0, dq->quat);
 
 	/* normalize */
-	len = (float)sqrt(dot_qtqt(q0, q0));
+	len = sqrtf(dot_qtqt(q0, q0));
 	if (len != 0.0f)
 		mul_qt_fl(q0, 1.0f / len);
 
@@ -1615,7 +1585,7 @@ void normalize_dq(DualQuat *dq, float totweight)
 	}
 }
 
-void mul_v3m3_dq(float *co, float mat[][3], DualQuat *dq)
+void mul_v3m3_dq(float co[3], float mat[3][3], DualQuat *dq)
 {
 	float M[3][3], t[3], scalemat[3][3], len2;
 	float w = dq->quat[0], x = dq->quat[1], y = dq->quat[2], z = dq->quat[3];
@@ -1675,12 +1645,13 @@ void quat_apply_track(float quat[4], short axis, short upflag)
 {
 	/* rotations are hard coded to match vec_to_quat */
 	const float quat_track[][4] = {
-	    {0.70710676908493, 0.0, -0.70710676908493, 0.0}, /* pos-y90 */
-	    {0.5, 0.5, 0.5, 0.5}, /* Quaternion((1,0,0), radians(90)) * Quaternion((0,1,0), radians(90)) */
-	    {0.70710676908493, 0.0, 0.0, 0.70710676908493}, /* pos-z90 */
-	    {0.70710676908493, 0.0, 0.70710676908493, 0.0}, /* neg-y90 */
-	    {0.5, -0.5, -0.5, 0.5}, /* Quaternion((1,0,0), radians(-90)) * Quaternion((0,1,0), radians(-90)) */
-	    {-3.0908619663705394e-08, 0.70710676908493, 0.70710676908493, 3.0908619663705394e-08}}; /* no rotation */
+		{M_SQRT1_2, 0.0, -M_SQRT1_2, 0.0}, /* pos-y90 */
+		{0.5, 0.5, 0.5, 0.5}, /* Quaternion((1,0,0), radians(90)) * Quaternion((0,1,0), radians(90)) */
+		{M_SQRT1_2, 0.0, 0.0, M_SQRT1_2}, /* pos-z90 */
+		{M_SQRT1_2, 0.0, M_SQRT1_2, 0.0}, /* neg-y90 */
+		{0.5, -0.5, -0.5, 0.5}, /* Quaternion((1,0,0), radians(-90)) * Quaternion((0,1,0), radians(-90)) */
+		{0.0, M_SQRT1_2, M_SQRT1_2, 0.0} /* no rotation */
+	};
 
 	assert(axis >= 0 && axis <= 5);
 	assert(upflag >= 0 && upflag <= 2);
@@ -1691,11 +1662,11 @@ void quat_apply_track(float quat[4], short axis, short upflag)
 		axis = axis - 3;
 
 	/* there are 2 possible up-axis for each axis used, the 'quat_track' applies so the first
-	 * up axis is used X->Y, Y->X, Z->X, if this first up axis isn used then rotate 90d
+	 * up axis is used X->Y, Y->X, Z->X, if this first up axis isn't used then rotate 90d
 	 * the strange bit shift below just find the low axis {X:Y, Y:X, Z:X} */
 	if (upflag != (2 - axis) >> 1) {
-		float q[4] = {0.70710676908493, 0.0, 0.0, 0.0}; /* assign 90d rotation axis */
-		q[axis + 1] = ((axis == 1)) ? 0.70710676908493 : -0.70710676908493; /* flip non Y axis */
+		float q[4] = {M_SQRT1_2, 0.0, 0.0, 0.0}; /* assign 90d rotation axis */
+		q[axis + 1] = ((axis == 1)) ? M_SQRT1_2 : -M_SQRT1_2; /* flip non Y axis */
 		mul_qt_qtqt(quat, quat, q);
 	}
 }
@@ -1710,34 +1681,34 @@ void vec_apply_track(float vec[3], short axis)
 
 	switch (axis) {
 		case 0: /* pos-x */
-			/* vec[0]=  0.0; */
+			/* vec[0] =  0.0; */
 			vec[1] = tvec[2];
 			vec[2] = -tvec[1];
 			break;
 		case 1: /* pos-y */
-			/* vec[0]= tvec[0]; */
-			/* vec[1]=  0.0; */
-			/* vec[2]= tvec[2]; */
+			/* vec[0] = tvec[0]; */
+			/* vec[1] =  0.0; */
+			/* vec[2] = tvec[2]; */
 			break;
 		case 2: /* pos-z */
-			/* vec[0]= tvec[0]; */
-			/* vec[1]= tvec[1]; */
-			// vec[2]=  0.0; */
+			/* vec[0] = tvec[0]; */
+			/* vec[1] = tvec[1]; */
+			/* vec[2] =  0.0; */
 			break;
 		case 3: /* neg-x */
-			/* vec[0]=  0.0; */
+			/* vec[0] =  0.0; */
 			vec[1] = tvec[2];
 			vec[2] = -tvec[1];
 			break;
 		case 4: /* neg-y */
 			vec[0] = -tvec[2];
-			/* vec[1]=  0.0; */
+			/* vec[1] =  0.0; */
 			vec[2] = tvec[0];
 			break;
 		case 5: /* neg-z */
 			vec[0] = -tvec[0];
 			vec[1] = -tvec[1];
-			/* vec[2]=  0.0; */
+			/* vec[2] =  0.0; */
 			break;
 	}
 }
@@ -1768,3 +1739,158 @@ float angle_wrap_deg(float angle)
 {
 	return mod_inline(angle + 180.0f, 360.0f) - 180.0f;
 }
+
+/* returns an angle compatible with angle_compat */
+float angle_compat_rad(float angle, float angle_compat)
+{
+	return angle + (floorf(((angle_compat - angle) / (float)M_PI) + 0.5f)) * (float)M_PI;
+}
+
+/* axis conversion */
+static float _axis_convert_matrix[23][3][3] = {
+	{{-1.0, 0.0, 0.0}, {0.0, -1.0, 0.0}, {0.0, 0.0, 1.0}},
+	{{-1.0, 0.0, 0.0}, {0.0, 0.0, -1.0}, {0.0, -1.0, 0.0}},
+	{{-1.0, 0.0, 0.0}, {0.0, 0.0, 1.0}, {0.0, 1.0, 0.0}},
+	{{-1.0, 0.0, 0.0}, {0.0, 1.0, 0.0}, {0.0, 0.0, -1.0}},
+	{{0.0, -1.0, 0.0}, {-1.0, 0.0, 0.0}, {0.0, 0.0, -1.0}},
+	{{0.0, 0.0, 1.0}, {-1.0, 0.0, 0.0}, {0.0, -1.0, 0.0}},
+	{{0.0, 0.0, -1.0}, {-1.0, 0.0, 0.0}, {0.0, 1.0, 0.0}},
+	{{0.0, 1.0, 0.0}, {-1.0, 0.0, 0.0}, {0.0, 0.0, 1.0}},
+	{{0.0, -1.0, 0.0}, {0.0, 0.0, 1.0}, {-1.0, 0.0, 0.0}},
+	{{0.0, 0.0, -1.0}, {0.0, -1.0, 0.0}, {-1.0, 0.0, 0.0}},
+	{{0.0, 0.0, 1.0}, {0.0, 1.0, 0.0}, {-1.0, 0.0, 0.0}},
+	{{0.0, 1.0, 0.0}, {0.0, 0.0, -1.0}, {-1.0, 0.0, 0.0}},
+	{{0.0, -1.0, 0.0}, {0.0, 0.0, -1.0}, {1.0, 0.0, 0.0}},
+	{{0.0, 0.0, 1.0}, {0.0, -1.0, 0.0}, {1.0, 0.0, 0.0}},
+	{{0.0, 0.0, -1.0}, {0.0, 1.0, 0.0}, {1.0, 0.0, 0.0}},
+	{{0.0, 1.0, 0.0}, {0.0, 0.0, 1.0}, {1.0, 0.0, 0.0}},
+	{{0.0, -1.0, 0.0}, {1.0, 0.0, 0.0}, {0.0, 0.0, 1.0}},
+	{{0.0, 0.0, -1.0}, {1.0, 0.0, 0.0}, {0.0, -1.0, 0.0}},
+	{{0.0, 0.0, 1.0}, {1.0, 0.0, 0.0}, {0.0, 1.0, 0.0}},
+	{{0.0, 1.0, 0.0}, {1.0, 0.0, 0.0}, {0.0, 0.0, -1.0}},
+	{{1.0, 0.0, 0.0}, {0.0, -1.0, 0.0}, {0.0, 0.0, -1.0}},
+	{{1.0, 0.0, 0.0}, {0.0, 0.0, 1.0}, {0.0, -1.0, 0.0}},
+	{{1.0, 0.0, 0.0}, {0.0, 0.0, -1.0}, {0.0, 1.0, 0.0}},
+};
+
+static int _axis_convert_lut[23][24] = {
+	{0x8C8, 0x4D0, 0x2E0, 0xAE8, 0x701, 0x511, 0x119, 0xB29, 0x682, 0x88A,
+	 0x09A, 0x2A2, 0x80B, 0x413, 0x223, 0xA2B, 0x644, 0x454, 0x05C, 0xA6C,
+	 0x745, 0x94D, 0x15D, 0x365},
+	{0xAC8, 0x8D0, 0x4E0, 0x2E8, 0x741, 0x951, 0x159, 0x369, 0x702, 0xB0A,
+	 0x11A, 0x522, 0xA0B, 0x813, 0x423, 0x22B, 0x684, 0x894, 0x09C, 0x2AC,
+	 0x645, 0xA4D, 0x05D, 0x465},
+	{0x4C8, 0x2D0, 0xAE0, 0x8E8, 0x681, 0x291, 0x099, 0x8A9, 0x642, 0x44A,
+	 0x05A, 0xA62, 0x40B, 0x213, 0xA23, 0x82B, 0x744, 0x354, 0x15C, 0x96C,
+	 0x705, 0x50D, 0x11D, 0xB25},
+	{0x2C8, 0xAD0, 0x8E0, 0x4E8, 0x641, 0xA51, 0x059, 0x469, 0x742, 0x34A,
+	 0x15A, 0x962, 0x20B, 0xA13, 0x823, 0x42B, 0x704, 0xB14, 0x11C, 0x52C,
+	 0x685, 0x28D, 0x09D, 0x8A5},
+	{0x708, 0xB10, 0x120, 0x528, 0x8C1, 0xAD1, 0x2D9, 0x4E9, 0x942, 0x74A,
+	 0x35A, 0x162, 0x64B, 0xA53, 0x063, 0x46B, 0x804, 0xA14, 0x21C, 0x42C,
+	 0x885, 0x68D, 0x29D, 0x0A5},
+	{0xB08, 0x110, 0x520, 0x728, 0x941, 0x151, 0x359, 0x769, 0x802, 0xA0A,
+	 0x21A, 0x422, 0xA4B, 0x053, 0x463, 0x66B, 0x884, 0x094, 0x29C, 0x6AC,
+	 0x8C5, 0xACD, 0x2DD, 0x4E5},
+	{0x508, 0x710, 0xB20, 0x128, 0x881, 0x691, 0x299, 0x0A9, 0x8C2, 0x4CA,
+	 0x2DA, 0xAE2, 0x44B, 0x653, 0xA63, 0x06B, 0x944, 0x754, 0x35C, 0x16C,
+	 0x805, 0x40D, 0x21D, 0xA25},
+	{0x108, 0x510, 0x720, 0xB28, 0x801, 0x411, 0x219, 0xA29, 0x882, 0x08A,
+	 0x29A, 0x6A2, 0x04B, 0x453, 0x663, 0xA6B, 0x8C4, 0x4D4, 0x2DC, 0xAEC,
+	 0x945, 0x14D, 0x35D, 0x765},
+	{0x748, 0x350, 0x160, 0x968, 0xAC1, 0x2D1, 0x4D9, 0x8E9, 0xA42, 0x64A,
+	 0x45A, 0x062, 0x68B, 0x293, 0x0A3, 0x8AB, 0xA04, 0x214, 0x41C, 0x82C,
+	 0xB05, 0x70D, 0x51D, 0x125},
+	{0x948, 0x750, 0x360, 0x168, 0xB01, 0x711, 0x519, 0x129, 0xAC2, 0x8CA,
+	 0x4DA, 0x2E2, 0x88B, 0x693, 0x2A3, 0x0AB, 0xA44, 0x654, 0x45C, 0x06C,
+	 0xA05, 0x80D, 0x41D, 0x225},
+	{0x348, 0x150, 0x960, 0x768, 0xA41, 0x051, 0x459, 0x669, 0xA02, 0x20A,
+	 0x41A, 0x822, 0x28B, 0x093, 0x8A3, 0x6AB, 0xB04, 0x114, 0x51C, 0x72C,
+	 0xAC5, 0x2CD, 0x4DD, 0x8E5},
+	{0x148, 0x950, 0x760, 0x368, 0xA01, 0x811, 0x419, 0x229, 0xB02, 0x10A,
+	 0x51A, 0x722, 0x08B, 0x893, 0x6A3, 0x2AB, 0xAC4, 0x8D4, 0x4DC, 0x2EC,
+	 0xA45, 0x04D, 0x45D, 0x665},
+	{0x688, 0x890, 0x0A0, 0x2A8, 0x4C1, 0x8D1, 0xAD9, 0x2E9, 0x502, 0x70A,
+	 0xB1A, 0x122, 0x74B, 0x953, 0x163, 0x36B, 0x404, 0x814, 0xA1C, 0x22C,
+	 0x445, 0x64D, 0xA5D, 0x065},
+	{0x888, 0x090, 0x2A0, 0x6A8, 0x501, 0x111, 0xB19, 0x729, 0x402, 0x80A,
+	 0xA1A, 0x222, 0x94B, 0x153, 0x363, 0x76B, 0x444, 0x054, 0xA5C, 0x66C,
+	 0x4C5, 0x8CD, 0xADD, 0x2E5},
+	{0x288, 0x690, 0x8A0, 0x0A8, 0x441, 0x651, 0xA59, 0x069, 0x4C2, 0x2CA,
+	 0xADA, 0x8E2, 0x34B, 0x753, 0x963, 0x16B, 0x504, 0x714, 0xB1C, 0x12C,
+	 0x405, 0x20D, 0xA1D, 0x825},
+	{0x088, 0x290, 0x6A0, 0x8A8, 0x401, 0x211, 0xA19, 0x829, 0x442, 0x04A,
+	 0xA5A, 0x662, 0x14B, 0x353, 0x763, 0x96B, 0x4C4, 0x2D4, 0xADC, 0x8EC,
+	 0x505, 0x10D, 0xB1D, 0x725},
+	{0x648, 0x450, 0x060, 0xA68, 0x2C1, 0x4D1, 0x8D9, 0xAE9, 0x282, 0x68A,
+	 0x89A, 0x0A2, 0x70B, 0x513, 0x123, 0xB2B, 0x204, 0x414, 0x81C, 0xA2C,
+	 0x345, 0x74D, 0x95D, 0x165},
+	{0xA48, 0x650, 0x460, 0x068, 0x341, 0x751, 0x959, 0x169, 0x2C2, 0xACA,
+	 0x8DA, 0x4E2, 0xB0B, 0x713, 0x523, 0x12B, 0x284, 0x694, 0x89C, 0x0AC,
+	 0x205, 0xA0D, 0x81D, 0x425},
+	{0x448, 0x050, 0xA60, 0x668, 0x281, 0x091, 0x899, 0x6A9, 0x202, 0x40A,
+	 0x81A, 0xA22, 0x50B, 0x113, 0xB23, 0x72B, 0x344, 0x154, 0x95C, 0x76C,
+	 0x2C5, 0x4CD, 0x8DD, 0xAE5},
+	{0x048, 0xA50, 0x660, 0x468, 0x201, 0xA11, 0x819, 0x429, 0x342, 0x14A,
+	 0x95A, 0x762, 0x10B, 0xB13, 0x723, 0x52B, 0x2C4, 0xAD4, 0x8DC, 0x4EC,
+	 0x285, 0x08D, 0x89D, 0x6A5},
+	{0x808, 0xA10, 0x220, 0x428, 0x101, 0xB11, 0x719, 0x529, 0x142, 0x94A,
+	 0x75A, 0x362, 0x8CB, 0xAD3, 0x2E3, 0x4EB, 0x044, 0xA54, 0x65C, 0x46C,
+	 0x085, 0x88D, 0x69D, 0x2A5},
+	{0xA08, 0x210, 0x420, 0x828, 0x141, 0x351, 0x759, 0x969, 0x042, 0xA4A,
+	 0x65A, 0x462, 0xACB, 0x2D3, 0x4E3, 0x8EB, 0x084, 0x294, 0x69C, 0x8AC,
+	 0x105, 0xB0D, 0x71D, 0x525},
+	{0x408, 0x810, 0xA20, 0x228, 0x081, 0x891, 0x699, 0x2A9, 0x102, 0x50A,
+	 0x71A, 0xB22, 0x4CB, 0x8D3, 0xAE3, 0x2EB, 0x144, 0x954, 0x75C, 0x36C,
+	 0x045, 0x44D, 0x65D, 0xA65},
+	};
+
+// _axis_convert_num = {'X': 0, 'Y': 1, 'Z': 2, '-X': 3, '-Y': 4, '-Z': 5}
+
+MINLINE int _axis_signed(const int axis)
+{
+	return (axis < 3) ? axis : axis - 3;
+}
+
+/*
+ * Each argument us an axis in ['X', 'Y', 'Z', '-X', '-Y', '-Z']
+ * where the first 2 are a source and the second 2 are the target.
+ */
+int mat3_from_axis_conversion(int from_forward, int from_up, int to_forward, int to_up,
+                              float r_mat[3][3])
+{
+	// from functools import reduce
+	int value;
+	int i;
+
+	if (from_forward == to_forward && from_up == to_up) {
+		unit_m3(r_mat);
+		return false;
+	}
+
+	if ((_axis_signed(from_forward) == _axis_signed(from_up)) ||
+	    (_axis_signed(to_forward)   == _axis_signed(to_up)))
+	{
+		/* we could assert here! */
+		unit_m3(r_mat);
+		return false;
+	}
+
+	value = ((from_forward << (0 * 3)) |
+	         (from_up      << (1 * 3)) |
+	         (to_forward   << (2 * 3)) |
+	         (to_up        << (3 * 3)));
+
+	for (i = 0; i < (sizeof(_axis_convert_matrix) / sizeof(*_axis_convert_matrix)); i++) {
+		int j;
+		for (j = 0; j < sizeof(*_axis_convert_lut) / sizeof(*_axis_convert_lut[0]); j++) {
+			if (_axis_convert_lut[i][j] == value) {
+				copy_m3_m3(r_mat, _axis_convert_matrix[i]);
+				return true;
+			}
+		}
+
+	}
+//	BLI_assert(0);
+	return false;
+}
