@@ -115,6 +115,39 @@ void mid_v2_v2v2(float v[2], const float v1[2], const float v2[2])
 	v[1] = 0.5f * (v1[1] + v2[1]);
 }
 
+void mid_v3_v3v3v3(float v[3], const float v1[3], const float v2[3], const float v3[3])
+{
+	v[0] = (v1[0] + v2[0] + v3[0]) / 3.0f;
+	v[1] = (v1[1] + v2[1] + v3[1]) / 3.0f;
+	v[2] = (v1[2] + v2[2] + v3[2]) / 3.0f;
+}
+
+/**
+ * Equivalent to:
+ * interp_v3_v3v3(v, v1, v2, -1.0f);
+ */
+
+void flip_v4_v4v4(float v[4], const float v1[4], const float v2[4])
+{
+	v[0] = v1[0] + (v1[0] - v2[0]);
+	v[1] = v1[1] + (v1[1] - v2[1]);
+	v[2] = v1[2] + (v1[2] - v2[2]);
+	v[3] = v1[3] + (v1[3] - v2[3]);
+}
+
+void flip_v3_v3v3(float v[3], const float v1[3], const float v2[3])
+{
+	v[0] = v1[0] + (v1[0] - v2[0]);
+	v[1] = v1[1] + (v1[1] - v2[1]);
+	v[2] = v1[2] + (v1[2] - v2[2]);
+}
+
+void flip_v2_v2v2(float v[2], const float v1[2], const float v2[2])
+{
+	v[0] = v1[0] + (v1[0] - v2[0]);
+	v[1] = v1[1] + (v1[1] - v2[1]);
+}
+
 /********************************** Angles ***********************************/
 
 /* Return the angle in radians between vecs 1-2 and 2-3 in radians
@@ -136,6 +169,19 @@ float angle_v3v3v3(const float v1[3], const float v2[3], const float v3[3])
 	return angle_normalized_v3v3(vec1, vec2);
 }
 
+/* Quicker than full angle computation */
+float cos_v3v3v3(const float p1[3], const float p2[3], const float p3[3])
+{
+	float vec1[3], vec2[3];
+
+	sub_v3_v3v3(vec1, p2, p1);
+	sub_v3_v3v3(vec2, p2, p3);
+	normalize_v3(vec1);
+	normalize_v3(vec2);
+
+	return dot_v3v3(vec1, vec2);
+}
+
 /* Return the shortest angle in radians between the 2 vectors */
 float angle_v3v3(const float v1[3], const float v2[3])
 {
@@ -188,6 +234,10 @@ float angle_signed_v2v2(const float v1[2], const float v2[2])
 
 float angle_normalized_v3v3(const float v1[3], const float v2[3])
 {
+	/* double check they are normalized */
+	BLI_ASSERT_UNIT_V3(v1);
+	BLI_ASSERT_UNIT_V3(v2);
+
 	/* this is the same as acos(dot_v3v3(v1, v2)), but more accurate */
 	if (dot_v3v3(v1, v2) < 0.0f) {
 		float vec[3];
@@ -204,6 +254,10 @@ float angle_normalized_v3v3(const float v1[3], const float v2[3])
 
 float angle_normalized_v2v2(const float v1[2], const float v2[2])
 {
+	/* double check they are normalized */
+	BLI_ASSERT_UNIT_V2(v1);
+	BLI_ASSERT_UNIT_V2(v2);
+
 	/* this is the same as acos(dot_v3v3(v1, v2)), but more accurate */
 	if (dot_v2v2(v1, v2) < 0.0f) {
 		float vec[2];
@@ -217,6 +271,25 @@ float angle_normalized_v2v2(const float v1[2], const float v2[2])
 		return 2.0f * (float)saasin(len_v2v2(v2, v1) / 2.0f);
 }
 
+/**
+ * angle between 2 vectors defined by 3 coords, about an axis. */
+float angle_on_axis_v3v3v3_v3(const float v1[3], const float v2[3], const float v3[3], const float axis[3])
+{
+	float v1_proj[3], v2_proj[3], tproj[3];
+
+	sub_v3_v3v3(v1_proj, v1, v2);
+	sub_v3_v3v3(v2_proj, v3, v2);
+
+	/* project the vectors onto the axis */
+	project_v3_v3v3(tproj, v1_proj, axis);
+	sub_v3_v3(v1_proj, tproj);
+
+	project_v3_v3v3(tproj, v2_proj, axis);
+	sub_v3_v3(v2_proj, tproj);
+
+	return angle_v3v3(v1_proj, v2_proj);
+}
+
 void angle_tri_v3(float angles[3], const float v1[3], const float v2[3], const float v3[3])
 {
 	float ed1[3], ed2[3], ed3[3];
@@ -369,6 +442,9 @@ void rotate_normalized_v3_v3v3fl(float r[3], const float p[3], const float axis[
 	const float costheta = cos(angle);
 	const float sintheta = sin(angle);
 
+	/* double check they are normalized */
+	BLI_ASSERT_UNIT_V3(axis);
+
 	r[0] = ((costheta + (1 - costheta) * axis[0] * axis[0]) * p[0]) +
 	       (((1 - costheta) * axis[0] * axis[1] - axis[2] * sintheta) * p[1]) +
 	       (((1 - costheta) * axis[0] * axis[2] + axis[1] * sintheta) * p[2]);
@@ -419,6 +495,38 @@ void minmax_v3v3_v3(float min[3], float max[3], const float vec[3])
 	if (max[2] < vec[2]) max[2] = vec[2];
 }
 
+void minmax_v2v2_v2(float min[2], float max[2], const float vec[2])
+{
+	if (min[0] > vec[0]) min[0] = vec[0];
+	if (min[1] > vec[1]) min[1] = vec[1];
+
+	if (max[0] < vec[0]) max[0] = vec[0];
+	if (max[1] < vec[1]) max[1] = vec[1];
+}
+
+/** ensure \a v1 is \a dist from \a v2 */
+void dist_ensure_v3_v3fl(float v1[3], const float v2[3], const float dist)
+{
+	if (!equals_v3v3(v2, v1)) {
+		float nor[3];
+
+		sub_v3_v3v3(nor, v1, v2);
+		normalize_v3(nor);
+		madd_v3_v3v3fl(v1, v2, nor, dist);
+	}
+}
+
+void dist_ensure_v2_v2fl(float v1[2], const float v2[2], const float dist)
+{
+	if (!equals_v2v2(v2, v1)) {
+		float nor[2];
+
+		sub_v2_v2v2(nor, v1, v2);
+		normalize_v2(nor);
+		madd_v2_v2v2fl(v1, v2, nor, dist);
+	}
+}
+
 /***************************** Array Functions *******************************/
 
 double dot_vn_vn(const float *array_src_a, const float *array_src_b, const int size)
@@ -428,7 +536,7 @@ double dot_vn_vn(const float *array_src_a, const float *array_src_b, const int s
 	const float *array_pt_b = array_src_b + (size - 1);
 	int i = size;
 	while (i--) {
-		d += *(array_pt_a--) * *(array_pt_b--);
+		d += (double)(*(array_pt_a--) * *(array_pt_b--));
 	}
 	return d;
 }
@@ -531,6 +639,27 @@ void add_vn_vnvn(float *array_tar, const float *array_src_a, const float *array_
 	}
 }
 
+void madd_vn_vn(float *array_tar, const float *array_src, const float f, const int size)
+{
+	float *tar = array_tar + (size - 1);
+	const float *src = array_src + (size - 1);
+	int i = size;
+	while (i--) {
+		*(tar--) += *(src--) * f;
+	}
+}
+
+void madd_vn_vnvn(float *array_tar, const float *array_src_a, const float *array_src_b, const float f, const int size)
+{
+	float *tar = array_tar + (size - 1);
+	const float *src_a = array_src_a + (size - 1);
+	const float *src_b = array_src_b + (size - 1);
+	int i = size;
+	while (i--) {
+		*(tar--) = *(src_a--) + (*(src_b--) * f);
+	}
+}
+
 void sub_vn_vn(float *array_tar, const float *array_src, const int size)
 {
 	float *tar = array_tar + (size - 1);
@@ -552,6 +681,40 @@ void sub_vn_vnvn(float *array_tar, const float *array_src_a, const float *array_
 	}
 }
 
+void msub_vn_vn(float *array_tar, const float *array_src, const float f, const int size)
+{
+	float *tar = array_tar + (size - 1);
+	const float *src = array_src + (size - 1);
+	int i = size;
+	while (i--) {
+		*(tar--) -= *(src--) * f;
+	}
+}
+
+void msub_vn_vnvn(float *array_tar, const float *array_src_a, const float *array_src_b, const float f, const int size)
+{
+	float *tar = array_tar + (size - 1);
+	const float *src_a = array_src_a + (size - 1);
+	const float *src_b = array_src_b + (size - 1);
+	int i = size;
+	while (i--) {
+		*(tar--) = *(src_a--) - (*(src_b--) * f);
+	}
+}
+
+void interp_vn_vn(float *array_tar, const float *array_src, const float t, const int size)
+{
+	const float s = 1.0f - t;
+	float *tar = array_tar + (size - 1);
+	const float *src = array_src + (size - 1);
+	int i = size;
+	while (i--) {
+		*(tar) = (s * *(tar)) + (t * *(src));
+		tar--;
+		src--;
+	}
+}
+
 void fill_vn_i(int *array_tar, const int size, const int val)
 {
 	int *tar = array_tar + (size - 1);
