@@ -86,8 +86,8 @@ typedef struct PolyFill {
 } PolyFill;
 
 typedef struct ScanFillVertLink {
-	ScanFillVert *v1;
-	ScanFillEdge *first, *last;
+	ScanFillVert *vert;
+	ScanFillEdge *edge_first, *edge_last;
 } ScanFillVertLink;
 
 
@@ -99,7 +99,7 @@ typedef struct ScanFillVertLink {
 #define SF_VERT_ZERO_LEN 255
 
 /* Optionally set ScanFillEdge f to this to mark original boundary edges.
- * Only needed if there are internal diagonal edges passed to BLI_edgefill. */
+ * Only needed if there are internal diagonal edges passed to BLI_scanfill_calc. */
 #define SF_EDGE_BOUNDARY 1
 #define SF_EDGE_UNKNOWN  2    /* TODO, what is this for exactly? - need to document it! */
 
@@ -112,10 +112,10 @@ static int vergscdata(const void *a1, const void *a2)
 {
 	const ScanFillVertLink *x1 = a1, *x2 = a2;
 	
-	if      (x1->v1->xy[1] < x2->v1->xy[1]) return  1;
-	else if (x1->v1->xy[1] > x2->v1->xy[1]) return -1;
-	else if (x1->v1->xy[0] > x2->v1->xy[0]) return  1;
-	else if (x1->v1->xy[0] < x2->v1->xy[0]) return -1;
+	if      (x1->vert->xy[1] < x2->vert->xy[1]) return  1;
+	else if (x1->vert->xy[1] > x2->vert->xy[1]) return -1;
+	else if (x1->vert->xy[0] > x2->vert->xy[0]) return  1;
+	else if (x1->vert->xy[0] < x2->vert->xy[0]) return -1;
 
 	return 0;
 }
@@ -191,9 +191,9 @@ static void mem_element_reset(ScanFillContext *sf_ctx, int keep_first)
 	sf_ctx->melem__offs = 0;
 }
 
-void BLI_end_edgefill(ScanFillContext *sf_ctx)
+void BLI_scanfill_end(ScanFillContext *sf_ctx)
 {
-	 mem_element_reset(sf_ctx, FALSE);
+	mem_element_reset(sf_ctx, FALSE);
 	
 	sf_ctx->fillvertbase.first = sf_ctx->fillvertbase.last = NULL;
 	sf_ctx->filledgebase.first = sf_ctx->filledgebase.last = NULL;
@@ -202,7 +202,7 @@ void BLI_end_edgefill(ScanFillContext *sf_ctx)
 
 /* ****  FILL ROUTINES *************************** */
 
-ScanFillVert *BLI_addfillvert(ScanFillContext *sf_ctx, const float vec[3])
+ScanFillVert *BLI_scanfill_vert_add(ScanFillContext *sf_ctx, const float vec[3])
 {
 	ScanFillVert *eve;
 	
@@ -211,10 +211,10 @@ ScanFillVert *BLI_addfillvert(ScanFillContext *sf_ctx, const float vec[3])
 	
 	copy_v3_v3(eve->co, vec);
 
-	return eve;	
+	return eve;
 }
 
-ScanFillEdge *BLI_addfilledge(ScanFillContext *sf_ctx, ScanFillVert *v1, ScanFillVert *v2)
+ScanFillEdge *BLI_scanfill_edge_add(ScanFillContext *sf_ctx, ScanFillVert *v1, ScanFillVert *v2)
 {
 	ScanFillEdge *newed;
 
@@ -230,14 +230,14 @@ ScanFillEdge *BLI_addfilledge(ScanFillContext *sf_ctx, ScanFillVert *v1, ScanFil
 static void addfillface(ScanFillContext *sf_ctx, ScanFillVert *v1, ScanFillVert *v2, ScanFillVert *v3)
 {
 	/* does not make edges */
-	ScanFillFace *evl;
+	ScanFillFace *sf_tri;
 
-	evl = mem_element_new(sf_ctx, sizeof(ScanFillFace));
-	BLI_addtail(&sf_ctx->fillfacebase, evl);
+	sf_tri = mem_element_new(sf_ctx, sizeof(ScanFillFace));
+	BLI_addtail(&sf_ctx->fillfacebase, sf_tri);
 	
-	evl->v1 = v1;
-	evl->v2 = v2;
-	evl->v3 = v3;
+	sf_tri->v1 = v1;
+	sf_tri->v2 = v2;
+	sf_tri->v3 = v3;
 }
 
 static int boundisect(PolyFill *pf2, PolyFill *pf1)
@@ -288,7 +288,7 @@ static void mergepolysSimp(ScanFillContext *sf_ctx, PolyFill *pf1, PolyFill *pf2
 }
 
 static short testedgeside(const float v1[2], const float v2[2], const float v3[2])
-/* is v3 to the right of v1-v2 ? With exception: v3==v1 || v3==v2 */
+/* is v3 to the right of v1-v2 ? With exception: v3 == v1 || v3 == v2 */
 {
 	float inp;
 
@@ -311,8 +311,8 @@ static short addedgetoscanvert(ScanFillVertLink *sc, ScanFillEdge *eed)
 	ScanFillEdge *ed;
 	float fac, fac1, x, y;
 
-	if (sc->first == NULL) {
-		sc->first = sc->last = eed;
+	if (sc->edge_first == NULL) {
+		sc->edge_first = sc->edge_last = eed;
 		eed->prev = eed->next = NULL;
 		return 1;
 	}
@@ -325,9 +325,11 @@ static short addedgetoscanvert(ScanFillVertLink *sc, ScanFillEdge *eed)
 		fac1 = 1.0e10f * (eed->v2->xy[0] - x);
 
 	}
-	else fac1 = (x - eed->v2->xy[0]) / fac1;
+	else {
+		fac1 = (x - eed->v2->xy[0]) / fac1;
+	}
 
-	for (ed = sc->first; ed; ed = ed->next) {
+	for (ed = sc->edge_first; ed; ed = ed->next) {
 
 		if (ed->v2 == eed->v2) {
 			return 0;
@@ -345,8 +347,8 @@ static short addedgetoscanvert(ScanFillVertLink *sc, ScanFillEdge *eed)
 			break;
 		}
 	}
-	if (ed) BLI_insertlinkbefore((ListBase *)&(sc->first), ed, eed);
-	else BLI_addtail((ListBase *)&(sc->first), eed);
+	if (ed) BLI_insertlinkbefore((ListBase *)&(sc->edge_first), ed, eed);
+	else BLI_addtail((ListBase *)&(sc->edge_first), eed);
 
 	return 1;
 }
@@ -373,7 +375,7 @@ static ScanFillVertLink *addedgetoscanlist(ScanFillContext *sf_ctx, ScanFillEdge
 		eed->v2 = eve;
 	}
 	/* find location in list */
-	scsearch.v1 = eed->v1;
+	scsearch.vert = eed->v1;
 	sc = (ScanFillVertLink *)bsearch(&scsearch, sf_ctx->_scdata, len,
 	                                 sizeof(ScanFillVertLink), vergscdata);
 
@@ -415,7 +417,7 @@ static short boundinsideEV(ScanFillEdge *eed, ScanFillVert *eve)
 
 static void testvertexnearedge(ScanFillContext *sf_ctx)
 {
-	/* only vertices with ->h==1 are being tested for
+	/* only vertices with (->h == 1) are being tested for
 	 * being close to an edge, if true insert */
 
 	ScanFillVert *eve;
@@ -423,12 +425,16 @@ static void testvertexnearedge(ScanFillContext *sf_ctx)
 
 	for (eve = sf_ctx->fillvertbase.first; eve; eve = eve->next) {
 		if (eve->h == 1) {
-			/* find the edge which has vertex eve */
-			ed1 = sf_ctx->filledgebase.first;
-			while (ed1) {
-				if (ed1->v1 == eve || ed1->v2 == eve) break;
-				ed1 = ed1->next;
+			/* find the edge which has vertex eve,
+			 * note: we _know_ this will crash if 'ed1' becomes NULL
+			 * but this will never happen. */
+			for (ed1 = sf_ctx->filledgebase.first;
+			     !(ed1->v1 == eve || ed1->v2 == eve);
+			     ed1 = ed1->next)
+			{
+				/* do nothing */
 			}
+
 			if (ed1->v1 == eve) {
 				ed1->v1 = ed1->v2;
 				ed1->v2 = eve;
@@ -453,9 +459,9 @@ static void testvertexnearedge(ScanFillContext *sf_ctx)
 							const float dist = dist_to_line_v2(eed->v1->xy, eed->v2->xy, eve->xy);
 							if (dist < SF_EPSILON) {
 								/* new edge */
-								ed1 = BLI_addfilledge(sf_ctx, eed->v1, eve);
+								ed1 = BLI_scanfill_edge_add(sf_ctx, eed->v1, eve);
 								
-								/* printf("fill: vertex near edge %x\n",eve); */
+								/* printf("fill: vertex near edge %x\n", eve); */
 								ed1->f = 0;
 								ed1->poly_nr = eed->poly_nr;
 								eed->v1 = eve;
@@ -499,14 +505,13 @@ static void splitlist(ScanFillContext *sf_ctx, ListBase *tempve, ListBase *tempe
 	}
 }
 
-
-static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
+static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf, const int flag)
 {
 	ScanFillVertLink *sc = NULL, *sc1;
 	ScanFillVert *eve, *v1, *v2, *v3;
 	ScanFillEdge *eed, *nexted, *ed1, *ed2, *ed3;
 	int a, b, verts, maxface, totface;
-	short nr, test, twoconnected = 0;
+	short nr, twoconnected = 0;
 
 	nr = pf->nr;
 
@@ -517,7 +522,7 @@ static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
 	while (eve) {
 		printf("vert: %x co: %f %f\n", eve, eve->xy[0], eve->xy[1]);
 		eve = eve->next;
-	}	
+	}
 	eed = sf_ctx->filledgebase.first;
 	while (eed) {
 		printf("edge: %x  verts: %x %x\n", eed, eed->v1, eed->v2);
@@ -526,26 +531,28 @@ static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
 #endif
 
 	/* STEP 0: remove zero sized edges */
-	eed = sf_ctx->filledgebase.first;
-	while (eed) {
-		if (equals_v2v2(eed->v1->xy, eed->v2->xy)) {
-			if (eed->v1->f == SF_VERT_ZERO_LEN && eed->v2->f != SF_VERT_ZERO_LEN) {
-				eed->v2->f = SF_VERT_ZERO_LEN;
-				eed->v2->tmp.v = eed->v1->tmp.v;
-			}
-			else if (eed->v2->f == SF_VERT_ZERO_LEN && eed->v1->f != SF_VERT_ZERO_LEN) {
-				eed->v1->f = SF_VERT_ZERO_LEN;
-				eed->v1->tmp.v = eed->v2->tmp.v;
-			}
-			else if (eed->v2->f == SF_VERT_ZERO_LEN && eed->v1->f == SF_VERT_ZERO_LEN) {
-				eed->v1->tmp.v = eed->v2->tmp.v;
-			}
-			else {
-				eed->v2->f = SF_VERT_ZERO_LEN;
-				eed->v2->tmp.v = eed->v1;
+	if (flag & BLI_SCANFILL_CALC_REMOVE_DOUBLES) {
+		eed = sf_ctx->filledgebase.first;
+		while (eed) {
+			if (equals_v2v2(eed->v1->xy, eed->v2->xy)) {
+				if (eed->v1->f == SF_VERT_ZERO_LEN && eed->v2->f != SF_VERT_ZERO_LEN) {
+					eed->v2->f = SF_VERT_ZERO_LEN;
+					eed->v2->tmp.v = eed->v1->tmp.v;
+				}
+				else if (eed->v2->f == SF_VERT_ZERO_LEN && eed->v1->f != SF_VERT_ZERO_LEN) {
+					eed->v1->f = SF_VERT_ZERO_LEN;
+					eed->v1->tmp.v = eed->v2->tmp.v;
+				}
+				else if (eed->v2->f == SF_VERT_ZERO_LEN && eed->v1->f == SF_VERT_ZERO_LEN) {
+					eed->v1->tmp.v = eed->v2->tmp.v;
+				}
+				else {
+					eed->v2->f = SF_VERT_ZERO_LEN;
+					eed->v2->tmp.v = eed->v1;
+				}
 			}
+			eed = eed->next;
 		}
-		eed = eed->next;
 	}
 
 	/* STEP 1: make using FillVert and FillEdge lists a sorted
@@ -559,7 +566,8 @@ static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
 			if (eve->f != SF_VERT_ZERO_LEN) {
 				verts++;
 				eve->f = 0;  /* flag for connectedges later on */
-				sc->v1 = eve;
+				sc->vert = eve;
+				/* if (even->tmp.v == NULL) eve->tmp.u = verts; */ /* Note, debug print only will work for curve polyfill, union is in use for mesh */
 				sc++;
 			}
 		}
@@ -568,28 +576,42 @@ static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
 
 	qsort(sf_ctx->_scdata, verts, sizeof(ScanFillVertLink), vergscdata);
 
-	eed = sf_ctx->filledgebase.first;
-	while (eed) {
-		nexted = eed->next;
-		BLI_remlink(&sf_ctx->filledgebase, eed);
-		/* This code is for handling zero-length edges that get
-		 * collapsed in step 0. It was removed for some time to
-		 * fix trunk bug #4544, so if that comes back, this code
-		 * may need some work, or there will have to be a better
-		 * fix to #4544. */
-		if (eed->v1->f == SF_VERT_ZERO_LEN) {
-			v1 = eed->v1;
-			while ((eed->v1->f == SF_VERT_ZERO_LEN) && (eed->v1->tmp.v != v1) && (eed->v1 != eed->v1->tmp.v))
-				eed->v1 = eed->v1->tmp.v;
+	if (flag & BLI_SCANFILL_CALC_REMOVE_DOUBLES) {
+		for (eed = sf_ctx->filledgebase.first; eed; eed = nexted) {
+			nexted = eed->next;
+			BLI_remlink(&sf_ctx->filledgebase, eed);
+			/* This code is for handling zero-length edges that get
+			 * collapsed in step 0. It was removed for some time to
+			 * fix trunk bug #4544, so if that comes back, this code
+			 * may need some work, or there will have to be a better
+			 * fix to #4544.
+			 *
+			 * warning, this can hang on un-ordered edges, see: [#33281]
+			 * for now disable 'BLI_SCANFILL_CALC_REMOVE_DOUBLES' for ngons.
+			 */
+			if (eed->v1->f == SF_VERT_ZERO_LEN) {
+				v1 = eed->v1;
+				while ((eed->v1->f == SF_VERT_ZERO_LEN) && (eed->v1->tmp.v != v1) && (eed->v1 != eed->v1->tmp.v))
+					eed->v1 = eed->v1->tmp.v;
+			}
+			if (eed->v2->f == SF_VERT_ZERO_LEN) {
+				v2 = eed->v2;
+				while ((eed->v2->f == SF_VERT_ZERO_LEN) && (eed->v2->tmp.v != v2) && (eed->v2 != eed->v2->tmp.v))
+					eed->v2 = eed->v2->tmp.v;
+			}
+			if (eed->v1 != eed->v2) {
+				addedgetoscanlist(sf_ctx, eed, verts);
+			}
 		}
-		if (eed->v2->f == SF_VERT_ZERO_LEN) {
-			v2 = eed->v2;
-			while ((eed->v2->f == SF_VERT_ZERO_LEN) && (eed->v2->tmp.v != v2) && (eed->v2 != eed->v2->tmp.v))
-				eed->v2 = eed->v2->tmp.v;
+	}
+	else {
+		for (eed = sf_ctx->filledgebase.first; eed; eed = nexted) {
+			nexted = eed->next;
+			BLI_remlink(&sf_ctx->filledgebase, eed);
+			if (eed->v1 != eed->v2) {
+				addedgetoscanlist(sf_ctx, eed, verts);
+			}
 		}
-		if (eed->v1 != eed->v2) addedgetoscanlist(sf_ctx, eed, verts);
-
-		eed = nexted;
 	}
 #if 0
 	sc = scdata;
@@ -611,37 +633,44 @@ static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
 
 	/* (temporal) security: never much more faces than vertices */
 	totface = 0;
-	maxface = 2 * verts;       /* 2*verts: based at a filled circle within a triangle */
+	if (flag & BLI_SCANFILL_CALC_HOLES) {
+		maxface = 2 * verts;       /* 2*verts: based at a filled circle within a triangle */
+	}
+	else {
+		maxface = verts - 2;       /* when we don't calc any holes, we assume face is a non overlapping loop */
+	}
 
 	sc = sf_ctx->_scdata;
 	for (a = 0; a < verts; a++) {
-		/* printf("VERTEX %d %x\n",a,sc->v1); */
-		ed1 = sc->first;
+		/* printf("VERTEX %d index %d\n", a, sc->vert->tmp.u); */
+		ed1 = sc->edge_first;
 		while (ed1) {   /* set connectflags  */
 			nexted = ed1->next;
 			if (ed1->v1->h == 1 || ed1->v2->h == 1) {
-				BLI_remlink((ListBase *)&(sc->first), ed1);
+				BLI_remlink((ListBase *)&(sc->edge_first), ed1);
 				BLI_addtail(&sf_ctx->filledgebase, ed1);
 				if (ed1->v1->h > 1) ed1->v1->h--;
 				if (ed1->v2->h > 1) ed1->v2->h--;
 			}
-			else ed1->v2->f = SF_VERT_UNKNOWN;
+			else {
+				ed1->v2->f = SF_VERT_UNKNOWN;
+			}
 
 			ed1 = nexted;
 		}
-		while (sc->first) { /* for as long there are edges */
-			ed1 = sc->first;
+		while (sc->edge_first) { /* for as long there are edges */
+			ed1 = sc->edge_first;
 			ed2 = ed1->next;
 			
 			/* commented out... the ESC here delivers corrupted memory (and doesnt work during grab) */
 			/* if (callLocalInterruptCallBack()) break; */
-			if (totface > maxface) {
+			if (totface >= maxface) {
 				/* printf("Fill error: endless loop. Escaped at vert %d,  tot: %d.\n", a, verts); */
 				a = verts;
 				break;
 			}
 			if (ed2 == 0) {
-				sc->first = sc->last = NULL;
+				sc->edge_first = sc->edge_last = NULL;
 				/* printf("just 1 edge to vert\n"); */
 				BLI_addtail(&sf_ctx->filledgebase, ed1);
 				ed1->v2->f = 0;
@@ -650,40 +679,69 @@ static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
 			}
 			else {
 				/* test rest of vertices */
+				ScanFillVertLink *best_sc = NULL;
+				float best_angle = 3.14f;
 				float miny;
+				bool firsttime = false;
+				
 				v1 = ed1->v2;
 				v2 = ed1->v1;
 				v3 = ed2->v2;
+				
 				/* this happens with a serial of overlapping edges */
 				if (v1 == v2 || v2 == v3) break;
-				/* printf("test verts %x %x %x\n",v1,v2,v3); */
-				miny = minf(v1->xy[1], v3->xy[1]);
-				/*  miny= MIN2(v1->xy[1],v3->xy[1]); */
+				
+				/* printf("test verts %d %d %d\n", v1->tmp.u, v2->tmp.u, v3->tmp.u); */
+				miny = min_ff(v1->xy[1], v3->xy[1]);
 				sc1 = sc + 1;
-				test = 0;
 
-				for (b = a + 1; b < verts; b++) {
-					if (sc1->v1->f == 0) {
-						if (sc1->v1->xy[1] <= miny) break;
+				for (b = a + 1; b < verts; b++, sc1++) {
+					if (sc1->vert->f == 0) {
+						if (sc1->vert->xy[1] <= miny) break;
+						if (testedgeside(v1->xy, v2->xy, sc1->vert->xy)) {
+							if (testedgeside(v2->xy, v3->xy, sc1->vert->xy)) {
+								if (testedgeside(v3->xy, v1->xy, sc1->vert->xy)) {
+									/* point is in triangle */
+									
+									/* because multiple points can be inside triangle (concave holes) */
+									/* we continue searching and pick the one with sharpest corner */
+									
+									if (best_sc == NULL) {
+										best_sc = sc1;
+										/* only need to continue checking with holes */
+										if ((flag & BLI_SCANFILL_CALC_HOLES) == 0) {
+											break;
+										}
+									}
+									else {
+										float angle;
+										
+										/* prevent angle calc for the simple cases only 1 vertex is found */
+										if (firsttime == false) {
+											best_angle = angle_v2v2v2(v2->co, v1->co, best_sc->vert->co);
+											firsttime = true;
+										}
 
-						if (testedgeside(v1->xy, v2->xy, sc1->v1->xy))
-							if (testedgeside(v2->xy, v3->xy, sc1->v1->xy))
-								if (testedgeside(v3->xy, v1->xy, sc1->v1->xy)) {
-									/* point in triangle */
-								
-									test = 1;
-									break;
+										angle = angle_v2v2v2(v2->co, v1->co, sc1->vert->co);
+										if (angle < best_angle) {
+											best_sc = sc1;
+											best_angle = angle;
+										}
+									}
+										
 								}
+							}
+						}
 					}
-					sc1++;
 				}
-				if (test) {
+					
+				if (best_sc) {
 					/* make new edge, and start over */
-					/* printf("add new edge %x %x and start again\n",v2,sc1->v1); */
+					/* printf("add new edge %d %d and start again\n", v2->tmp.u, best_sc->vert->tmp.u); */
 
-					ed3 = BLI_addfilledge(sf_ctx, v2, sc1->v1);
+					ed3 = BLI_scanfill_edge_add(sf_ctx, v2, best_sc->vert);
 					BLI_remlink(&sf_ctx->filledgebase, ed3);
-					BLI_insertlinkbefore((ListBase *)&(sc->first), ed2, ed3);
+					BLI_insertlinkbefore((ListBase *)&(sc->edge_first), ed2, ed3);
 					ed3->v2->f = SF_VERT_UNKNOWN;
 					ed3->f = SF_EDGE_UNKNOWN;
 					ed3->v1->h++; 
@@ -691,17 +749,17 @@ static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
 				}
 				else {
 					/* new triangle */
-					/* printf("add face %x %x %x\n",v1,v2,v3); */
+					/* printf("add face %d %d %d\n", v1->tmp.u, v2->tmp.u, v3->tmp.u); */
 					addfillface(sf_ctx, v1, v2, v3);
 					totface++;
-					BLI_remlink((ListBase *)&(sc->first), ed1);
+					BLI_remlink((ListBase *)&(sc->edge_first), ed1);
 					BLI_addtail(&sf_ctx->filledgebase, ed1);
 					ed1->v2->f = 0;
 					ed1->v1->h--; 
 					ed1->v2->h--;
 					/* ed2 can be removed when it's a boundary edge */
 					if ((ed2->f == 0 && twoconnected) || (ed2->f == SF_EDGE_BOUNDARY)) {
-						BLI_remlink((ListBase *)&(sc->first), ed2);
+						BLI_remlink((ListBase *)&(sc->edge_first), ed2);
 						BLI_addtail(&sf_ctx->filledgebase, ed2);
 						ed2->v2->f = 0;
 						ed2->v1->h--; 
@@ -709,13 +767,13 @@ static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
 					}
 
 					/* new edge */
-					ed3 = BLI_addfilledge(sf_ctx, v1, v3);
+					ed3 = BLI_scanfill_edge_add(sf_ctx, v1, v3);
 					BLI_remlink(&sf_ctx->filledgebase, ed3);
 					ed3->f = SF_EDGE_UNKNOWN;
 					ed3->v1->h++; 
 					ed3->v2->h++;
 					
-					/* printf("add new edge %x %x\n",v1,v3); */
+					/* printf("add new edge %x %x\n", v1, v3); */
 					sc1 = addedgetoscanlist(sf_ctx, ed3, verts);
 					
 					if (sc1) {  /* ed3 already exists: remove if a boundary */
@@ -723,11 +781,11 @@ static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
 						ed3->v1->h--; 
 						ed3->v2->h--;
 
-						ed3 = sc1->first;
+						ed3 = sc1->edge_first;
 						while (ed3) {
 							if ( (ed3->v1 == v1 && ed3->v2 == v3) || (ed3->v1 == v3 && ed3->v2 == v1) ) {
 								if (twoconnected || ed3->f == SF_EDGE_BOUNDARY) {
-									BLI_remlink((ListBase *)&(sc1->first), ed3);
+									BLI_remlink((ListBase *)&(sc1->edge_first), ed3);
 									BLI_addtail(&sf_ctx->filledgebase, ed3);
 									ed3->v1->h--; 
 									ed3->v2->h--;
@@ -737,15 +795,14 @@ static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
 							ed3 = ed3->next;
 						}
 					}
-
 				}
 			}
 			/* test for loose edges */
-			ed1 = sc->first;
+			ed1 = sc->edge_first;
 			while (ed1) {
 				nexted = ed1->next;
 				if (ed1->v1->h < 2 || ed1->v2->h < 2) {
-					BLI_remlink((ListBase *)&(sc->first), ed1);
+					BLI_remlink((ListBase *)&(sc->edge_first), ed1);
 					BLI_addtail(&sf_ctx->filledgebase, ed1);
 					if (ed1->v1->h > 1) ed1->v1->h--;
 					if (ed1->v2->h > 1) ed1->v2->h--;
@@ -754,29 +811,32 @@ static int scanfill(ScanFillContext *sf_ctx, PolyFill *pf)
 				ed1 = nexted;
 			}
 		}
+
 		sc++;
 	}
 
 	MEM_freeN(sf_ctx->_scdata);
 	sf_ctx->_scdata = NULL;
 
+	BLI_assert(totface <= maxface);
+
 	return totface;
 }
 
 
-int BLI_begin_edgefill(ScanFillContext *sf_ctx)
+int BLI_scanfill_begin(ScanFillContext *sf_ctx)
 {
 	memset(sf_ctx, 0, sizeof(*sf_ctx));
 
 	return 1;
 }
 
-int BLI_edgefill(ScanFillContext *sf_ctx, const short do_quad_tri_speedup)
+int BLI_scanfill_calc(ScanFillContext *sf_ctx, const int flag)
 {
-	return BLI_edgefill_ex(sf_ctx, do_quad_tri_speedup, NULL);
+	return BLI_scanfill_calc_ex(sf_ctx, flag, NULL);
 }
 
-int BLI_edgefill_ex(ScanFillContext *sf_ctx, const short do_quad_tri_speedup, const float nor_proj[3])
+int BLI_scanfill_calc_ex(ScanFillContext *sf_ctx, const int flag, const float nor_proj[3])
 {
 	/*
 	 * - fill works with its own lists, so create that first (no faces!)
@@ -806,30 +866,32 @@ int BLI_edgefill_ex(ScanFillContext *sf_ctx, const short do_quad_tri_speedup, co
 		a += 1;
 	}
 
-	if (do_quad_tri_speedup && (a == 3)) {
-		eve = sf_ctx->fillvertbase.first;
+	if (flag & BLI_SCANFILL_CALC_QUADTRI_FASTPATH) {
+		if (a == 3) {
+			eve = sf_ctx->fillvertbase.first;
 
-		addfillface(sf_ctx, eve, eve->next, eve->next->next);
-		return 1;
-	}
-	else if (do_quad_tri_speedup && (a == 4)) {
-		float vec1[3], vec2[3];
-
-		eve = sf_ctx->fillvertbase.first;
-		/* no need to check 'eve->next->next->next' is valid, already counted */
-		/* use shortest diagonal for quad */
-		sub_v3_v3v3(vec1, eve->co, eve->next->next->co);
-		sub_v3_v3v3(vec2, eve->next->co, eve->next->next->next->co);
-
-		if (dot_v3v3(vec1, vec1) < dot_v3v3(vec2, vec2)) {
 			addfillface(sf_ctx, eve, eve->next, eve->next->next);
-			addfillface(sf_ctx, eve->next->next, eve->next->next->next, eve);
+			return 1;
 		}
-		else {
-			addfillface(sf_ctx, eve->next, eve->next->next, eve->next->next->next);
-			addfillface(sf_ctx, eve->next->next->next, eve, eve->next);
+		else if (a == 4) {
+			float vec1[3], vec2[3];
+
+			eve = sf_ctx->fillvertbase.first;
+			/* no need to check 'eve->next->next->next' is valid, already counted */
+			/* use shortest diagonal for quad */
+			sub_v3_v3v3(vec1, eve->co, eve->next->next->co);
+			sub_v3_v3v3(vec2, eve->next->co, eve->next->next->next->co);
+
+			if (dot_v3v3(vec1, vec1) < dot_v3v3(vec2, vec2)) {
+				addfillface(sf_ctx, eve, eve->next, eve->next->next);
+				addfillface(sf_ctx, eve->next->next, eve->next->next->next, eve);
+			}
+			else {
+				addfillface(sf_ctx, eve->next, eve->next->next, eve->next->next->next);
+				addfillface(sf_ctx, eve->next->next->next, eve, eve->next);
+			}
+			return 2;
 		}
-		return 2;
 	}
 
 	/* first test vertices if they are in edges */
@@ -889,50 +951,68 @@ int BLI_edgefill_ex(ScanFillContext *sf_ctx, const short do_quad_tri_speedup, co
 
 
 	/* STEP 1: COUNT POLYS */
-	eve = sf_ctx->fillvertbase.first;
-	while (eve) {
-		eve->xy[0] = eve->co[co_x];
-		eve->xy[1] = eve->co[co_y];
+	if (flag & BLI_SCANFILL_CALC_HOLES) {
+		eve = sf_ctx->fillvertbase.first;
+		while (eve) {
+			eve->xy[0] = eve->co[co_x];
+			eve->xy[1] = eve->co[co_y];
 
-		/* get first vertex with no poly number */
-		if (eve->poly_nr == 0) {
-			poly++;
-			/* now a sort of select connected */
-			ok = 1;
-			eve->poly_nr = poly;
-			
-			while (ok) {
-				
-				ok = 0;
-				toggle++;
-				if (toggle & 1) eed = sf_ctx->filledgebase.first;
-				else eed = sf_ctx->filledgebase.last;
+			/* get first vertex with no poly number */
+			if (eve->poly_nr == 0) {
+				poly++;
+				/* now a sort of select connected */
+				ok = 1;
+				eve->poly_nr = poly;
 
-				while (eed) {
-					if (eed->v1->poly_nr == 0 && eed->v2->poly_nr == poly) {
-						eed->v1->poly_nr = poly;
-						eed->poly_nr = poly;
-						ok = 1;
-					}
-					else if (eed->v2->poly_nr == 0 && eed->v1->poly_nr == poly) {
-						eed->v2->poly_nr = poly;
-						eed->poly_nr = poly;
-						ok = 1;
-					}
-					else if (eed->poly_nr == 0) {
-						if (eed->v1->poly_nr == poly && eed->v2->poly_nr == poly) {
+				while (ok) {
+
+					ok = 0;
+					toggle++;
+					if (toggle & 1) eed = sf_ctx->filledgebase.first;
+					else eed = sf_ctx->filledgebase.last;
+
+					while (eed) {
+						if (eed->v1->poly_nr == 0 && eed->v2->poly_nr == poly) {
+							eed->v1->poly_nr = poly;
 							eed->poly_nr = poly;
 							ok = 1;
 						}
+						else if (eed->v2->poly_nr == 0 && eed->v1->poly_nr == poly) {
+							eed->v2->poly_nr = poly;
+							eed->poly_nr = poly;
+							ok = 1;
+						}
+						else if (eed->poly_nr == 0) {
+							if (eed->v1->poly_nr == poly && eed->v2->poly_nr == poly) {
+								eed->poly_nr = poly;
+								ok = 1;
+							}
+						}
+						if (toggle & 1) eed = eed->next;
+						else eed = eed->prev;
 					}
-					if (toggle & 1) eed = eed->next;
-					else eed = eed->prev;
 				}
 			}
+			eve = eve->next;
+		}
+		/* printf("amount of poly's: %d\n", poly); */
+	}
+	else {
+		poly = 1;
+
+		eve = sf_ctx->fillvertbase.first;
+		while (eve) {
+			eve->xy[0] = eve->co[co_x];
+			eve->xy[1] = eve->co[co_y];
+			eve->poly_nr = poly;
+			eve = eve->next;
+		}
+		eed = sf_ctx->filledgebase.first;
+		while (eed) {
+			eed->poly_nr = poly;
+			eed = eed->next;
 		}
-		eve = eve->next;
 	}
-	/* printf("amount of poly's: %d\n",poly); */
 
 	/* STEP 2: remove loose edges and strings of edges */
 	eed = sf_ctx->filledgebase.first;
@@ -947,7 +1027,7 @@ int BLI_edgefill_ex(ScanFillContext *sf_ctx, const short do_quad_tri_speedup, co
 		return 0;
 	}
 	
-	/* does it only for vertices with ->h==1 */
+	/* does it only for vertices with (->h == 1) */
 	testvertexnearedge(sf_ctx);
 
 	ok = 1;
@@ -981,12 +1061,12 @@ int BLI_edgefill_ex(ScanFillContext *sf_ctx, const short do_quad_tri_speedup, co
 
 
 	/* CURRENT STATUS:
-	 * - eve->f       :1= availalble in edges
+	 * - eve->f       :1 = available in edges
 	 * - eve->xs      :polynumber
 	 * - eve->h       :amount of edges connected to vertex
 	 * - eve->tmp.v   :store! original vertex number
 	 * 
-	 * - eed->f       :1= boundary edge (optionally set by caller)
+	 * - eed->f       :1 = boundary edge (optionally set by caller)
 	 * - eed->poly_nr :poly number
 	 */
 
@@ -1087,7 +1167,7 @@ int BLI_edgefill_ex(ScanFillContext *sf_ctx, const short do_quad_tri_speedup, co
 	for (a = 0; a < poly; a++) {
 		if (pf->edges > 1) {
 			splitlist(sf_ctx, &tempve, &temped, pf->nr);
-			totfaces += scanfill(sf_ctx, pf);
+			totfaces += scanfill(sf_ctx, pf, flag);
 		}
 		pf++;
 	}
