@@ -1,6 +1,4 @@
 /*
- *
- *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -46,14 +44,14 @@
 
 /* for checking system threads - BLI_system_thread_count */
 #ifdef WIN32
-#include "windows.h"
-#include <sys/timeb.h>
+#  include <windows.h>
+#  include <sys/timeb.h>
 #elif defined(__APPLE__)
-#include <sys/types.h>
-#include <sys/sysctl.h>
+#  include <sys/types.h>
+#  include <sys/sysctl.h>
 #else
-#include <unistd.h> 
-#include <sys/time.h>
+#  include <unistd.h>
+#  include <sys/time.h>
 #endif
 
 #if defined(__APPLE__) && (PARALLEL == 1) && (__GNUC__ == 4) && (__GNUC_MINOR__ == 2)
@@ -73,8 +71,8 @@ static void *thread_tls_data;
  * A sample loop can look like this (pseudo c);
  *
  *     ListBase lb;
- *     int maxthreads= 2;
- *     int cont= 1;
+ *     int maxthreads = 2;
+ *     int cont = 1;
  * 
  *     BLI_init_threads(&lb, do_something_func, maxthreads);
  * 
@@ -87,18 +85,18 @@ static void *thread_tls_data;
  *         else PIL_sleep_ms(50);
  *         
  *         // find if a job is ready, this the do_something_func() should write in job somewhere
- *         cont= 0;
+ *         cont = 0;
  *         for (go over all jobs)
  *             if (job is ready) {
  *                 if (job was not removed) {
  *                     BLI_remove_thread(&lb, job);
  *                 }
  *             }
- *             else cont= 1;
+ *             else cont = 1;
  *         }
  *         // conditions to exit loop 
  *         if (if escape loop event) {
- *             if (BLI_available_threadslots(&lb)==maxthreads)
+ *             if (BLI_available_threadslots(&lb) == maxthreads)
  *                 break;
  *         }
  *     }
@@ -108,15 +106,16 @@ static void *thread_tls_data;
  ************************************************ */
 static pthread_mutex_t _malloc_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t _image_lock = PTHREAD_MUTEX_INITIALIZER;
-static pthread_mutex_t _preview_lock = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t _image_draw_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t _viewer_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t _custom1_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t _rcache_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t _opengl_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t _nodes_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t _movieclip_lock = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t _colormanage_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_t mainid;
-static int thread_levels= 0;	/* threads can be invoked inside threads */
+static int thread_levels = 0;  /* threads can be invoked inside threads */
 
 /* just a max for security reasons */
 #define RE_MAX_THREAD BLENDER_MAX_THREADS
@@ -153,16 +152,16 @@ void BLI_init_threads(ListBase *threadbase, void *(*do_thread)(void *), int tot)
 	int a;
 
 	if (threadbase != NULL && tot > 0) {
-		threadbase->first= threadbase->last= NULL;
+		threadbase->first = threadbase->last = NULL;
 	
-		if (tot>RE_MAX_THREAD) tot= RE_MAX_THREAD;
-		else if (tot<1) tot= 1;
+		if (tot > RE_MAX_THREAD) tot = RE_MAX_THREAD;
+		else if (tot < 1) tot = 1;
 	
-		for (a=0; a<tot; a++) {
-			ThreadSlot *tslot= MEM_callocN(sizeof(ThreadSlot), "threadslot");
+		for (a = 0; a < tot; a++) {
+			ThreadSlot *tslot = MEM_callocN(sizeof(ThreadSlot), "threadslot");
 			BLI_addtail(threadbase, tslot);
-			tslot->do_thread= do_thread;
-			tslot->avail= 1;
+			tslot->do_thread = do_thread;
+			tslot->avail = 1;
 		}
 	}
 	
@@ -184,9 +183,9 @@ void BLI_init_threads(ListBase *threadbase, void *(*do_thread)(void *), int tot)
 int BLI_available_threads(ListBase *threadbase)
 {
 	ThreadSlot *tslot;
-	int counter=0;
+	int counter = 0;
 	
-	for (tslot= threadbase->first; tslot; tslot= tslot->next) {
+	for (tslot = threadbase->first; tslot; tslot = tslot->next) {
 		if (tslot->avail)
 			counter++;
 	}
@@ -197,9 +196,9 @@ int BLI_available_threads(ListBase *threadbase)
 int BLI_available_thread_index(ListBase *threadbase)
 {
 	ThreadSlot *tslot;
-	int counter=0;
+	int counter = 0;
 	
-	for (tslot= threadbase->first; tslot; tslot= tslot->next, counter++) {
+	for (tslot = threadbase->first; tslot; tslot = tslot->next, counter++) {
 		if (tslot->avail)
 			return counter;
 	}
@@ -208,12 +207,12 @@ int BLI_available_thread_index(ListBase *threadbase)
 
 static void *tslot_thread_start(void *tslot_p)
 {
-	ThreadSlot *tslot= (ThreadSlot*)tslot_p;
+	ThreadSlot *tslot = (ThreadSlot *)tslot_p;
 
 #if defined(__APPLE__) && (PARALLEL == 1) && (__GNUC__ == 4) && (__GNUC_MINOR__ == 2)
 	/* workaround for Apple gcc 4.2.1 omp vs background thread bug,
 	 * set gomp thread local storage pointer which was copied beforehand */
-	pthread_setspecific (gomp_tls_key, thread_tls_data);
+	pthread_setspecific(gomp_tls_key, thread_tls_data);
 #endif
 
 	return tslot->do_thread(tslot->callerdata);
@@ -228,10 +227,10 @@ void BLI_insert_thread(ListBase *threadbase, void *callerdata)
 {
 	ThreadSlot *tslot;
 	
-	for (tslot= threadbase->first; tslot; tslot= tslot->next) {
+	for (tslot = threadbase->first; tslot; tslot = tslot->next) {
 		if (tslot->avail) {
-			tslot->avail= 0;
-			tslot->callerdata= callerdata;
+			tslot->avail = 0;
+			tslot->callerdata = callerdata;
 			pthread_create(&tslot->pthread, NULL, tslot_thread_start, tslot);
 			return;
 		}
@@ -243,11 +242,11 @@ void BLI_remove_thread(ListBase *threadbase, void *callerdata)
 {
 	ThreadSlot *tslot;
 	
-	for (tslot= threadbase->first; tslot; tslot= tslot->next) {
-		if (tslot->callerdata==callerdata) {
+	for (tslot = threadbase->first; tslot; tslot = tslot->next) {
+		if (tslot->callerdata == callerdata) {
 			pthread_join(tslot->pthread, NULL);
-			tslot->callerdata= NULL;
-			tslot->avail= 1;
+			tslot->callerdata = NULL;
+			tslot->avail = 1;
 		}
 	}
 }
@@ -255,7 +254,7 @@ void BLI_remove_thread(ListBase *threadbase, void *callerdata)
 void BLI_remove_thread_index(ListBase *threadbase, int index)
 {
 	ThreadSlot *tslot;
-	int counter=0;
+	int counter = 0;
 	
 	for (tslot = threadbase->first; tslot; tslot = tslot->next, counter++) {
 		if (counter == index && tslot->avail == 0) {
@@ -288,8 +287,8 @@ void BLI_end_threads(ListBase *threadbase)
 	 * this way we don't end up decrementing thread_levels on an empty threadbase 
 	 * */
 	if (threadbase && threadbase->first != NULL) {
-		for (tslot= threadbase->first; tslot; tslot= tslot->next) {
-			if (tslot->avail==0) {
+		for (tslot = threadbase->first; tslot; tslot = tslot->next) {
+			if (tslot->avail == 0) {
 				pthread_join(tslot->pthread, NULL);
 			}
 		}
@@ -297,14 +296,14 @@ void BLI_end_threads(ListBase *threadbase)
 	}
 
 	thread_levels--;
-	if (thread_levels==0)
+	if (thread_levels == 0)
 		MEM_set_lock_callback(NULL, NULL);
 }
 
 /* System Information */
 
 /* how many threads are native on this system? */
-int BLI_system_thread_count( void )
+int BLI_system_thread_count(void)
 {
 	int t;
 #ifdef WIN32
@@ -312,7 +311,7 @@ int BLI_system_thread_count( void )
 	GetSystemInfo(&info);
 	t = (int) info.dwNumberOfProcessors;
 #else 
-#	ifdef __APPLE__
+#   ifdef __APPLE__
 	int mib[2];
 	size_t len;
 	
@@ -320,14 +319,14 @@ int BLI_system_thread_count( void )
 	mib[1] = HW_NCPU;
 	len = sizeof(t);
 	sysctl(mib, 2, &t, &len, NULL, 0);
-#	else
+#   else
 	t = (int)sysconf(_SC_NPROCESSORS_ONLN);
-#	endif
+#   endif
 #endif
 	
-	if (t>RE_MAX_THREAD)
+	if (t > RE_MAX_THREAD)
 		return RE_MAX_THREAD;
-	if (t<1)
+	if (t < 1)
 		return 1;
 	
 	return t;
@@ -337,42 +336,46 @@ int BLI_system_thread_count( void )
 
 void BLI_lock_thread(int type)
 {
-	if (type==LOCK_IMAGE)
+	if (type == LOCK_IMAGE)
 		pthread_mutex_lock(&_image_lock);
-	else if (type==LOCK_PREVIEW)
-		pthread_mutex_lock(&_preview_lock);
-	else if (type==LOCK_VIEWER)
+	else if (type == LOCK_DRAW_IMAGE)
+		pthread_mutex_lock(&_image_draw_lock);
+	else if (type == LOCK_VIEWER)
 		pthread_mutex_lock(&_viewer_lock);
-	else if (type==LOCK_CUSTOM1)
+	else if (type == LOCK_CUSTOM1)
 		pthread_mutex_lock(&_custom1_lock);
-	else if (type==LOCK_RCACHE)
+	else if (type == LOCK_RCACHE)
 		pthread_mutex_lock(&_rcache_lock);
-	else if (type==LOCK_OPENGL)
+	else if (type == LOCK_OPENGL)
 		pthread_mutex_lock(&_opengl_lock);
-	else if (type==LOCK_NODES)
+	else if (type == LOCK_NODES)
 		pthread_mutex_lock(&_nodes_lock);
-	else if (type==LOCK_MOVIECLIP)
+	else if (type == LOCK_MOVIECLIP)
 		pthread_mutex_lock(&_movieclip_lock);
+	else if (type == LOCK_COLORMANAGE)
+		pthread_mutex_lock(&_colormanage_lock);
 }
 
 void BLI_unlock_thread(int type)
 {
-	if (type==LOCK_IMAGE)
+	if (type == LOCK_IMAGE)
 		pthread_mutex_unlock(&_image_lock);
-	else if (type==LOCK_PREVIEW)
-		pthread_mutex_unlock(&_preview_lock);
-	else if (type==LOCK_VIEWER)
+	else if (type == LOCK_DRAW_IMAGE)
+		pthread_mutex_unlock(&_image_draw_lock);
+	else if (type == LOCK_VIEWER)
 		pthread_mutex_unlock(&_viewer_lock);
-	else if (type==LOCK_CUSTOM1)
+	else if (type == LOCK_CUSTOM1)
 		pthread_mutex_unlock(&_custom1_lock);
-	else if (type==LOCK_RCACHE)
+	else if (type == LOCK_RCACHE)
 		pthread_mutex_unlock(&_rcache_lock);
-	else if (type==LOCK_OPENGL)
+	else if (type == LOCK_OPENGL)
 		pthread_mutex_unlock(&_opengl_lock);
-	else if (type==LOCK_NODES)
+	else if (type == LOCK_NODES)
 		pthread_mutex_unlock(&_nodes_lock);
-	else if (type==LOCK_MOVIECLIP)
+	else if (type == LOCK_MOVIECLIP)
 		pthread_mutex_unlock(&_movieclip_lock);
+	else if (type == LOCK_COLORMANAGE)
+		pthread_mutex_unlock(&_colormanage_lock);
 }
 
 /* Mutex Locks */
@@ -397,6 +400,42 @@ void BLI_mutex_end(ThreadMutex *mutex)
 	pthread_mutex_destroy(mutex);
 }
 
+/* Spin Locks */
+
+void BLI_spin_init(SpinLock *spin)
+{
+#ifdef __APPLE__
+	*spin = OS_SPINLOCK_INIT;
+#else
+	pthread_spin_init(spin, 0);
+#endif
+}
+
+void BLI_spin_lock(SpinLock *spin)
+{
+#ifdef __APPLE__
+	OSSpinLockLock(spin);
+#else
+	pthread_spin_lock(spin);
+#endif
+}
+
+void BLI_spin_unlock(SpinLock *spin)
+{
+#ifdef __APPLE__
+	OSSpinLockUnlock(spin);
+#else
+	pthread_spin_unlock(spin);
+#endif
+}
+
+void BLI_spin_end(SpinLock *spin)
+{
+#ifndef __APPLE__
+	pthread_spin_destroy(spin);
+#endif
+}
+
 /* Read/Write Mutex Lock */
 
 void BLI_rw_mutex_init(ThreadRWMutex *mutex)
@@ -427,20 +466,20 @@ void BLI_rw_mutex_end(ThreadRWMutex *mutex)
 typedef struct ThreadedWorker {
 	ListBase threadbase;
 	void *(*work_fnct)(void *);
-	char	 busy[RE_MAX_THREAD];
-	int		 total;
-	int		 sleep_time;
+	char busy[RE_MAX_THREAD];
+	int total;
+	int sleep_time;
 } ThreadedWorker;
 
 typedef struct WorkParam {
 	ThreadedWorker *worker;
 	void *param;
-	int	  index;
+	int index;
 } WorkParam;
 
 static void *exec_work_fnct(void *v_param)
 {
-	WorkParam *p = (WorkParam*)v_param;
+	WorkParam *p = (WorkParam *)v_param;
 	void *value;
 	
 	value = p->worker->work_fnct(p->param);
@@ -463,7 +502,7 @@ ThreadedWorker *BLI_create_worker(void *(*do_thread)(void *), int tot, int sleep
 		tot = RE_MAX_THREAD;
 	}
 	else if (tot < 1) {
-		tot= 1;
+		tot = 1;
 	}
 	
 	worker->total = tot;
@@ -522,26 +561,31 @@ void BLI_insert_work(ThreadedWorker *worker, void *param)
 struct ThreadQueue {
 	GSQueue *queue;
 	pthread_mutex_t mutex;
-	pthread_cond_t cond;
-	int nowait;
+	pthread_cond_t push_cond;
+	pthread_cond_t finish_cond;
+	volatile int nowait;
+	volatile int cancelled;
 };
 
 ThreadQueue *BLI_thread_queue_init(void)
 {
 	ThreadQueue *queue;
 
-	queue= MEM_callocN(sizeof(ThreadQueue), "ThreadQueue");
-	queue->queue= BLI_gsqueue_new(sizeof(void*));
+	queue = MEM_callocN(sizeof(ThreadQueue), "ThreadQueue");
+	queue->queue = BLI_gsqueue_new(sizeof(void *));
 
 	pthread_mutex_init(&queue->mutex, NULL);
-	pthread_cond_init(&queue->cond, NULL);
+	pthread_cond_init(&queue->push_cond, NULL);
+	pthread_cond_init(&queue->finish_cond, NULL);
 
 	return queue;
 }
 
 void BLI_thread_queue_free(ThreadQueue *queue)
 {
-	pthread_cond_destroy(&queue->cond);
+	/* destroy everything, assumes no one is using queue anymore */
+	pthread_cond_destroy(&queue->finish_cond);
+	pthread_cond_destroy(&queue->push_cond);
 	pthread_mutex_destroy(&queue->mutex);
 
 	BLI_gsqueue_free(queue->queue);
@@ -556,22 +600,26 @@ void BLI_thread_queue_push(ThreadQueue *queue, void *work)
 	BLI_gsqueue_push(queue->queue, &work);
 
 	/* signal threads waiting to pop */
-	pthread_cond_signal(&queue->cond);
+	pthread_cond_signal(&queue->push_cond);
 	pthread_mutex_unlock(&queue->mutex);
 }
 
 void *BLI_thread_queue_pop(ThreadQueue *queue)
 {
-	void *work= NULL;
+	void *work = NULL;
 
 	/* wait until there is work */
 	pthread_mutex_lock(&queue->mutex);
 	while (BLI_gsqueue_is_empty(queue->queue) && !queue->nowait)
-		pthread_cond_wait(&queue->cond, &queue->mutex);
-
+		pthread_cond_wait(&queue->push_cond, &queue->mutex);
+	
 	/* if we have something, pop it */
-	if (!BLI_gsqueue_is_empty(queue->queue))
+	if (!BLI_gsqueue_is_empty(queue->queue)) {
 		BLI_gsqueue_pop(queue->queue, &work);
+		
+		if (BLI_gsqueue_is_empty(queue->queue))
+			pthread_cond_broadcast(&queue->finish_cond);
+	}
 
 	pthread_mutex_unlock(&queue->mutex);
 
@@ -588,7 +636,7 @@ static void wait_timeout(struct timespec *timeout, int ms)
 		struct _timeb now;
 		_ftime(&now);
 		sec = now.time;
-		usec = now.millitm*1000; /* microsecond precision would be better */
+		usec = now.millitm * 1000; /* microsecond precision would be better */
 	}
 #else
 	{
@@ -603,38 +651,42 @@ static void wait_timeout(struct timespec *timeout, int ms)
 	div_result = ldiv(ms, 1000);
 	timeout->tv_sec = sec + div_result.quot;
 
-	x = usec + (div_result.rem*1000);
+	x = usec + (div_result.rem * 1000);
 
 	if (x >= 1000000) {
 		timeout->tv_sec++;
 		x -= 1000000;
 	}
 
-	timeout->tv_nsec = x*1000;
+	timeout->tv_nsec = x * 1000;
 }
 
 void *BLI_thread_queue_pop_timeout(ThreadQueue *queue, int ms)
 {
 	double t;
-	void *work= NULL;
+	void *work = NULL;
 	struct timespec timeout;
 
-	t= PIL_check_seconds_timer();
+	t = PIL_check_seconds_timer();
 	wait_timeout(&timeout, ms);
 
 	/* wait until there is work */
 	pthread_mutex_lock(&queue->mutex);
 	while (BLI_gsqueue_is_empty(queue->queue) && !queue->nowait) {
-		if (pthread_cond_timedwait(&queue->cond, &queue->mutex, &timeout) == ETIMEDOUT)
+		if (pthread_cond_timedwait(&queue->push_cond, &queue->mutex, &timeout) == ETIMEDOUT)
 			break;
-		else if (PIL_check_seconds_timer() - t >= ms*0.001)
+		else if (PIL_check_seconds_timer() - t >= ms * 0.001)
 			break;
 	}
 
 	/* if we have something, pop it */
-	if (!BLI_gsqueue_is_empty(queue->queue))
+	if (!BLI_gsqueue_is_empty(queue->queue)) {
 		BLI_gsqueue_pop(queue->queue, &work);
-
+		
+		if (BLI_gsqueue_is_empty(queue->queue))
+			pthread_cond_broadcast(&queue->finish_cond);
+	}
+	
 	pthread_mutex_unlock(&queue->mutex);
 
 	return work;
@@ -645,7 +697,7 @@ int BLI_thread_queue_size(ThreadQueue *queue)
 	int size;
 
 	pthread_mutex_lock(&queue->mutex);
-	size= BLI_gsqueue_size(queue->queue);
+	size = BLI_gsqueue_size(queue->queue);
 	pthread_mutex_unlock(&queue->mutex);
 
 	return size;
@@ -655,15 +707,31 @@ void BLI_thread_queue_nowait(ThreadQueue *queue)
 {
 	pthread_mutex_lock(&queue->mutex);
 
-	queue->nowait= 1;
+	queue->nowait = 1;
 
 	/* signal threads waiting to pop */
-	pthread_cond_signal(&queue->cond);
+	pthread_cond_broadcast(&queue->push_cond);
 	pthread_mutex_unlock(&queue->mutex);
 }
 
+void BLI_thread_queue_wait_finish(ThreadQueue *queue)
+{
+	/* wait for finish condition */
+	pthread_mutex_lock(&queue->mutex);
+
+	while (!BLI_gsqueue_is_empty(queue->queue))
+		pthread_cond_wait(&queue->finish_cond, &queue->mutex);
+
+	pthread_mutex_unlock(&queue->mutex);
+}
+
+/* ************************************************ */
+
 void BLI_begin_threaded_malloc(void)
 {
+	/* Used for debug only */
+	/* BLI_assert(thread_levels >= 0); */
+
 	if (thread_levels == 0) {
 		MEM_set_lock_callback(BLI_lock_malloc_thread, BLI_unlock_malloc_thread);
 	}
@@ -672,7 +740,11 @@ void BLI_begin_threaded_malloc(void)
 
 void BLI_end_threaded_malloc(void)
 {
+	/* Used for debug only */
+	/* BLI_assert(thread_levels >= 0); */
+
 	thread_levels--;
-	if (thread_levels==0)
+	if (thread_levels == 0)
 		MEM_set_lock_callback(NULL, NULL);
 }
+
