@@ -45,46 +45,66 @@
 #include "bmesh.h"
 #include "intern/bmesh_private.h"
 
-/**
- * \brief Data, Interp From Verts
- *
- * Interpolates per-vertex data from two sources to a target.
- */
-void BM_data_interp_from_verts(BMesh *bm, BMVert *v1, BMVert *v2, BMVert *v, const float fac)
+/* edge and vertex share, currently theres no need to have different logic */
+static void bm_data_interp_from_elem(CustomData *data_layer, BMElem *ele1, BMElem *ele2, BMElem *ele_dst, const float fac)
 {
-	if (v1->head.data && v2->head.data) {
+	if (ele1->head.data && ele2->head.data) {
 		/* first see if we can avoid interpolation */
 		if (fac <= 0.0f) {
-			if (v1 == v) {
+			if (ele1 == ele_dst) {
 				/* do nothing */
 			}
 			else {
-				CustomData_bmesh_free_block(&bm->vdata, &v->head.data);
-				CustomData_bmesh_copy_data(&bm->vdata, &bm->vdata, v1->head.data, &v->head.data);
+				CustomData_bmesh_free_block(data_layer, &ele_dst->head.data);
+				CustomData_bmesh_copy_data(data_layer, data_layer, ele1->head.data, &ele_dst->head.data);
 			}
 		}
 		else if (fac >= 1.0f) {
-			if (v2 == v) {
+			if (ele2 == ele_dst) {
 				/* do nothing */
 			}
 			else {
-				CustomData_bmesh_free_block(&bm->vdata, &v->head.data);
-				CustomData_bmesh_copy_data(&bm->vdata, &bm->vdata, v2->head.data, &v->head.data);
+				CustomData_bmesh_free_block(data_layer, &ele_dst->head.data);
+				CustomData_bmesh_copy_data(data_layer, data_layer, ele2->head.data, &ele_dst->head.data);
 			}
 		}
 		else {
 			void *src[2];
 			float w[2];
 
-			src[0] = v1->head.data;
-			src[1] = v2->head.data;
-			w[0] = 1.0f-fac;
+			src[0] = ele1->head.data;
+			src[1] = ele2->head.data;
+			w[0] = 1.0f - fac;
 			w[1] = fac;
-			CustomData_bmesh_interp(&bm->vdata, src, w, NULL, 2, v->head.data);
+			CustomData_bmesh_interp(data_layer, src, w, NULL, 2, ele_dst->head.data);
 		}
 	}
 }
 
+/**
+ * \brief Data, Interp From Verts
+ *
+ * Interpolates per-vertex data from two sources to a target.
+ *
+ * \note This is an exact match to #BM_data_interp_from_edges
+ */
+void BM_data_interp_from_verts(BMesh *bm, BMVert *v1, BMVert *v2, BMVert *v, const float fac)
+{
+	bm_data_interp_from_elem(&bm->vdata, (BMElem *)v1, (BMElem *)v2, (BMElem *)v, fac);
+}
+
+/**
+ * \brief Data, Interp From Edges
+ *
+ * Interpolates per-edge data from two sources to a target.
+ *
+ * \note This is an exact match to #BM_data_interp_from_verts
+ */
+void BM_data_interp_from_edges(BMesh *bm, BMEdge *e1, BMEdge *e2, BMEdge *e, const float fac)
+{
+	bm_data_interp_from_elem(&bm->edata, (BMElem *)e1, (BMElem *)e2, (BMElem *)e, fac);
+}
+
 /**
  * \brief Data Vert Average
  *
@@ -152,11 +172,9 @@ void BM_face_interp_from_face(BMesh *bm, BMFace *target, BMFace *source)
 	BMLoop *l_iter;
 	BMLoop *l_first;
 
-	void **blocks = NULL;
-	float (*cos)[3] = NULL, *w = NULL;
-	BLI_array_fixedstack_declare(cos,     BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(w,       BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(blocks,  BM_NGON_STACK_SIZE, source->len, __func__);
+	void **blocks   = BLI_array_alloca(blocks, source->len);
+	float (*cos)[3] = BLI_array_alloca(cos,    source->len);
+	float *w        = BLI_array_alloca(w,      source->len);
 	int i;
 	
 	BM_elem_attrs_copy(bm, bm, source, target);
@@ -176,17 +194,13 @@ void BM_face_interp_from_face(BMesh *bm, BMFace *target, BMFace *source)
 		CustomData_bmesh_interp(&bm->ldata, blocks, w, NULL, source->len, l_iter->head.data);
 		i++;
 	} while ((l_iter = l_iter->next) != l_first);
-
-	BLI_array_fixedstack_free(cos);
-	BLI_array_fixedstack_free(w);
-	BLI_array_fixedstack_free(blocks);
 }
 
 /**
  * \brief Multires Interpolation
  *
  * mdisps is a grid of displacements, ordered thus:
- *
+ * <pre>
  *      v1/center----v4/next -> x
  *          |           |
  *          |           |
@@ -194,28 +208,18 @@ void BM_face_interp_from_face(BMesh *bm, BMFace *target, BMFace *source)
  *          |
  *          V
  *          y
+ * </pre>
  */
 static int compute_mdisp_quad(BMLoop *l, float v1[3], float v2[3], float v3[3], float v4[3],
                               float e1[3], float e2[3])
 {
-	float cent[3] = {0.0f, 0.0f, 0.0f}, n[3], p[3];
-	BMLoop *l_first;
-	BMLoop *l_iter;
-	
+	float cent[3], n[3], p[3];
+
 	/* computer center */
-	l_iter = l_first = BM_FACE_FIRST_LOOP(l->f);
-	do {
-		cent[0] += (float)l_iter->v->co[0];
-		cent[1] += (float)l_iter->v->co[1];
-		cent[2] += (float)l_iter->v->co[2];
-	} while ((l_iter = l_iter->next) != l_first);
-	
-	mul_v3_fl(cent, (1.0 / (float)l->f->len));
-	
-	add_v3_v3v3(p, l->prev->v->co, l->v->co);
-	mul_v3_fl(p, 0.5);
-	add_v3_v3v3(n, l->next->v->co, l->v->co);
-	mul_v3_fl(n, 0.5);
+	BM_face_calc_center_mean(l->f, cent);
+
+	mid_v3_v3v3(p, l->prev->v->co, l->v->co);
+	mid_v3_v3v3(n, l->next->v->co, l->v->co);
 	
 	copy_v3_v3(v1, cent);
 	copy_v3_v3(v2, p);
@@ -240,19 +244,19 @@ static float quad_coord(float aa[3], float bb[3], float cc[3], float dd[3], int
 	if (fabsf(2.0f * (x - y + z)) > FLT_EPSILON * 10.0f) {
 		float f2;
 
-		f1 = (sqrt(y * y - 4.0 * x * z) - y + 2.0 * z) / (2.0 * (x - y + z));
-		f2 = (-sqrt(y * y - 4.0 * x * z) - y + 2.0 * z) / (2.0 * (x - y + z));
+		f1 = ( sqrtf(y * y - 4.0f * x * z) - y + 2.0f * z) / (2.0f * (x - y + z));
+		f2 = (-sqrtf(y * y - 4.0f * x * z) - y + 2.0f * z) / (2.0f * (x - y + z));
 
 		f1 = fabsf(f1);
 		f2 = fabsf(f2);
-		f1 = MIN2(f1, f2);
+		f1 = min_ff(f1, f2);
 		CLAMP(f1, 0.0f, 1.0f + FLT_EPSILON);
 	}
 	else {
 		f1 = -z / (y - 2 * z);
 		CLAMP(f1, 0.0f, 1.0f + FLT_EPSILON);
 		
-		if (isnan(f1) || f1 > 1.0 || f1 < 0.0f) {
+		if (isnan(f1) || f1 > 1.0f || f1 < 0.0f) {
 			int i;
 			
 			for (i = 0; i < 2; i++) {
@@ -333,9 +337,9 @@ static int mdisp_in_mdispquad(BMLoop *l, BMLoop *tl, float p[3], float *x, float
 	float v1[3], v2[3], c[3], v3[3], v4[3], e1[3], e2[3];
 	float eps = FLT_EPSILON * 4000;
 	
-	if (len_v3(l->v->no) == 0.0f)
+	if (is_zero_v3(l->v->no))
 		BM_vert_normal_update_all(l->v);
-	if (len_v3(tl->v->no) == 0.0f)
+	if (is_zero_v3(tl->v->no))
 		BM_vert_normal_update_all(tl->v);
 
 	compute_mdisp_quad(tl, v1, v2, v3, v4, e1, e2);
@@ -345,8 +349,8 @@ static int mdisp_in_mdispquad(BMLoop *l, BMLoop *tl, float p[3], float *x, float
 	
 	sub_v3_v3(v1, c); sub_v3_v3(v2, c);
 	sub_v3_v3(v3, c); sub_v3_v3(v4, c);
-	mul_v3_fl(v1, 1.0 + eps); mul_v3_fl(v2, 1.0 + eps);
-	mul_v3_fl(v3, 1.0 + eps); mul_v3_fl(v4, 1.0 + eps);
+	mul_v3_fl(v1, 1.0f + eps); mul_v3_fl(v2, 1.0f + eps);
+	mul_v3_fl(v3, 1.0f + eps); mul_v3_fl(v4, 1.0f + eps);
 	add_v3_v3(v1, c); add_v3_v3(v2, c);
 	add_v3_v3(v3, c); add_v3_v3(v4, c);
 	
@@ -392,9 +396,9 @@ static void bm_loop_flip_disp(float source_axis_x[3], float source_axis_y[3],
 
 	d = bm_loop_flip_equotion(mat, b, target_axis_x, target_axis_y, coord, 0, 1);
 
-	if (fabsf(d) < 1e-4) {
+	if (fabsf(d) < 1e-4f) {
 		d = bm_loop_flip_equotion(mat, b, target_axis_x, target_axis_y, coord, 0, 2);
-		if (fabsf(d) < 1e-4)
+		if (fabsf(d) < 1e-4f)
 			d = bm_loop_flip_equotion(mat, b, target_axis_x, target_axis_y, coord, 1, 2);
 	}
 
@@ -412,7 +416,7 @@ static void bm_loop_interp_mdisps(BMesh *bm, BMLoop *target, BMFace *source)
 	float axis_x[3], axis_y[3];
 	
 	/* ignore 2-edged faces */
-	if (target->f->len < 3)
+	if (UNLIKELY(target->f->len < 3))
 		return;
 	
 	if (!CustomData_has_layer(&bm->ldata, CD_MDISPS))
@@ -439,7 +443,7 @@ static void bm_loop_interp_mdisps(BMesh *bm, BMLoop *target, BMFace *source)
 	mdisp_axis_from_quad(v1, v2, v3, v4, axis_x, axis_y);
 
 	res = (int)sqrt(mdisps->totdisp);
-	d = 1.0 / (float)(res - 1);
+	d = 1.0f / (float)(res - 1);
 	for (x = 0.0f, ix = 0; ix < res; x += d, ix++) {
 		for (y = 0.0f, iy = 0; iy < res; y += d, iy++) {
 			float co1[3], co2[3], co[3];
@@ -478,7 +482,7 @@ static void bm_loop_interp_mdisps(BMesh *bm, BMLoop *target, BMFace *source)
 }
 
 /**
- * smoothes boundaries between multires grids,
+ * smooths boundaries between multires grids,
  * including some borders in adjacent faces
  */
 void BM_face_multires_bounds_smooth(BMesh *bm, BMFace *f)
@@ -513,8 +517,7 @@ void BM_face_multires_bounds_smooth(BMesh *bm, BMFace *f)
 
 		sides = (int)sqrt(mdp->totdisp);
 		for (y = 0; y < sides; y++) {
-			add_v3_v3v3(co1, mdn->disps[y * sides], mdl->disps[y]);
-			mul_v3_fl(co1, 0.5);
+			mid_v3_v3v3(co1, mdn->disps[y * sides], mdl->disps[y]);
 
 			copy_v3_v3(mdn->disps[y * sides], co1);
 			copy_v3_v3(mdl->disps[y], co1);
@@ -596,18 +599,16 @@ void BM_loop_interp_multires(BMesh *bm, BMLoop *target, BMFace *source)
  * if do_vertex is true, target's vert data will also get interpolated.
  */
 void BM_loop_interp_from_face(BMesh *bm, BMLoop *target, BMFace *source,
-                              int do_vertex, int do_multires)
+                              const bool do_vertex, const bool do_multires)
 {
 	BMLoop *l_iter;
 	BMLoop *l_first;
-	void **blocks = NULL;
-	void **vblocks = NULL;
-	float (*cos)[3] = NULL, co[3], *w = NULL;
-	float cent[3] = {0.0f, 0.0f, 0.0f};
-	BLI_array_fixedstack_declare(cos,      BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(w,        BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(blocks,   BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(vblocks,  BM_NGON_STACK_SIZE, do_vertex ? source->len : 0, __func__);
+	void **vblocks  = do_vertex ? BLI_array_alloca(vblocks, source->len) : NULL;
+	void **blocks   = BLI_array_alloca(blocks,  source->len);
+	float (*cos)[3] = BLI_array_alloca(cos,     source->len);
+	float (*cos_2d)[2] = BLI_array_alloca(cos_2d,     source->len);
+	float *w        = BLI_array_alloca(w,       source->len);
+	float co[2];
 	int i, ax, ay;
 
 	BM_elem_attrs_copy(bm, bm, source, target->f);
@@ -616,7 +617,6 @@ void BM_loop_interp_from_face(BMesh *bm, BMLoop *target, BMFace *source,
 	l_iter = l_first = BM_FACE_FIRST_LOOP(source);
 	do {
 		copy_v3_v3(cos[i], l_iter->v->co);
-		add_v3_v3(cent, cos[i]);
 
 		w[i] = 0.0f;
 		blocks[i] = l_iter->head.data;
@@ -633,38 +633,22 @@ void BM_loop_interp_from_face(BMesh *bm, BMLoop *target, BMFace *source,
 
 	axis_dominant_v3(&ax, &ay, source->no);
 
-	/* scale source face coordinates a bit, so points sitting directly on an
-	 * edge will work. */
-	mul_v3_fl(cent, 1.0f / (float)source->len);
 	for (i = 0; i < source->len; i++) {
-		float vec[3], tmp[3];
-		sub_v3_v3v3(vec, cent, cos[i]);
-		mul_v3_fl(vec, 0.001f);
-		add_v3_v3(cos[i], vec);
-
-		copy_v3_v3(tmp, cos[i]);
-		cos[i][0] = tmp[ax];
-		cos[i][1] = tmp[ay];
-		cos[i][2] = 0.0f;
+		cos_2d[i][0] = cos[i][ax];
+		cos_2d[i][1] = cos[i][ay];
 	}
 
 
 	/* interpolate */
 	co[0] = target->v->co[ax];
 	co[1] = target->v->co[ay];
-	co[2] = 0.0f;
 
-	interp_weights_poly_v3(w, cos, source->len, co);
+	interp_weights_poly_v2(w, cos_2d, source->len, co);
 	CustomData_bmesh_interp(&bm->ldata, blocks, w, NULL, source->len, target->head.data);
 	if (do_vertex) {
 		CustomData_bmesh_interp(&bm->vdata, vblocks, w, NULL, source->len, target->v->head.data);
-		BLI_array_fixedstack_free(vblocks);
 	}
 
-	BLI_array_fixedstack_free(cos);
-	BLI_array_fixedstack_free(w);
-	BLI_array_fixedstack_free(blocks);
-
 	if (do_multires) {
 		if (CustomData_has_layer(&bm->ldata, CD_MDISPS)) {
 			bm_loop_interp_mdisps(bm, target, source);
@@ -677,42 +661,24 @@ void BM_vert_interp_from_face(BMesh *bm, BMVert *v, BMFace *source)
 {
 	BMLoop *l_iter;
 	BMLoop *l_first;
-	void **blocks = NULL;
-	float (*cos)[3] = NULL, *w = NULL;
-	float cent[3] = {0.0f, 0.0f, 0.0f};
-	BLI_array_fixedstack_declare(cos,      BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(w,        BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(blocks,   BM_NGON_STACK_SIZE, source->len, __func__);
+	void **blocks   = BLI_array_alloca(blocks, source->len);
+	float (*cos)[3] = BLI_array_alloca(cos,    source->len);
+	float *w        = BLI_array_alloca(w,      source->len);
 	int i;
 
 	i = 0;
 	l_iter = l_first = BM_FACE_FIRST_LOOP(source);
 	do {
 		copy_v3_v3(cos[i], l_iter->v->co);
-		add_v3_v3(cent, cos[i]);
 
 		w[i] = 0.0f;
 		blocks[i] = l_iter->v->head.data;
 		i++;
 	} while ((l_iter = l_iter->next) != l_first);
 
-	/* scale source face coordinates a bit, so points sitting directly on an
-	 * edge will work. */
-	mul_v3_fl(cent, 1.0f / (float)source->len);
-	for (i = 0; i < source->len; i++) {
-		float vec[3];
-		sub_v3_v3v3(vec, cent, cos[i]);
-		mul_v3_fl(vec, 0.01f);
-		add_v3_v3(cos[i], vec);
-	}
-
 	/* interpolate */
 	interp_weights_poly_v3(w, cos, source->len, v->co);
 	CustomData_bmesh_interp(&bm->vdata, blocks, w, NULL, source->len, v->head.data);
-
-	BLI_array_fixedstack_free(cos);
-	BLI_array_fixedstack_free(w);
-	BLI_array_fixedstack_free(blocks);
 }
 
 static void update_data_blocks(BMesh *bm, CustomData *olddata, CustomData *data)
@@ -861,24 +827,24 @@ void BM_data_layer_copy(BMesh *bm, CustomData *data, int type, int src_n, int ds
 		BMVert *eve;
 
 		BM_ITER_MESH (eve, &iter, bm, BM_VERTS_OF_MESH) {
-			void *ptr = CustomData_bmesh_get_n(data, eve->head.data, type, dst_n);
-			CustomData_bmesh_set_n(data, eve->head.data, type, src_n, ptr);
+			void *ptr = CustomData_bmesh_get_n(data, eve->head.data, type, src_n);
+			CustomData_bmesh_set_n(data, eve->head.data, type, dst_n, ptr);
 		}
 	}
 	else if (&bm->edata == data) {
 		BMEdge *eed;
 
 		BM_ITER_MESH (eed, &iter, bm, BM_EDGES_OF_MESH) {
-			void *ptr = CustomData_bmesh_get_n(data, eed->head.data, type, dst_n);
-			CustomData_bmesh_set_n(data, eed->head.data, type, src_n, ptr);
+			void *ptr = CustomData_bmesh_get_n(data, eed->head.data, type, src_n);
+			CustomData_bmesh_set_n(data, eed->head.data, type, dst_n, ptr);
 		}
 	}
 	else if (&bm->pdata == data) {
 		BMFace *efa;
 
 		BM_ITER_MESH (efa, &iter, bm, BM_FACES_OF_MESH) {
-			void *ptr = CustomData_bmesh_get_n(data, efa->head.data, type, dst_n);
-			CustomData_bmesh_set_n(data, efa->head.data, type, src_n, ptr);
+			void *ptr = CustomData_bmesh_get_n(data, efa->head.data, type, src_n);
+			CustomData_bmesh_set_n(data, efa->head.data, type, dst_n, ptr);
 		}
 	}
 	else if (&bm->ldata == data) {
@@ -888,8 +854,8 @@ void BM_data_layer_copy(BMesh *bm, CustomData *data, int type, int src_n, int ds
 
 		BM_ITER_MESH (efa, &iter, bm, BM_FACES_OF_MESH) {
 			BM_ITER_ELEM (l, &liter, efa, BM_LOOPS_OF_FACE) {
-				void *ptr = CustomData_bmesh_get_n(data, l->head.data, type, dst_n);
-				CustomData_bmesh_set_n(data, l->head.data, type, src_n, ptr);
+				void *ptr = CustomData_bmesh_get_n(data, l->head.data, type, src_n);
+				CustomData_bmesh_set_n(data, l->head.data, type, dst_n, ptr);
 			}
 		}
 	}
