@@ -31,6 +31,8 @@
  * that should be addressed eventually.
  */
 
+#include <stddef.h>
+
 #include "MEM_guardedalloc.h"
 
 #include "DNA_scene_types.h"
@@ -42,8 +44,6 @@
 
 static void recount_totsels(BMesh *bm)
 {
-	BMIter iter;
-	BMElem *ele;
 	const char iter_types[3] = {BM_VERTS_OF_MESH,
 	                            BM_EDGES_OF_MESH,
 	                            BM_FACES_OF_MESH};
@@ -56,11 +56,16 @@ static void recount_totsels(BMesh *bm)
 	tots[1] = &bm->totedgesel;
 	tots[2] = &bm->totfacesel;
 
+#pragma omp parallel for schedule(dynamic)
 	for (i = 0; i < 3; i++) {
-		ele = BM_iter_new(&iter, bm, iter_types[i], NULL);
-		for ( ; ele; ele = BM_iter_step(&iter)) {
-			if (BM_elem_flag_test(ele, BM_ELEM_SELECT)) *tots[i] += 1;
+		BMIter iter;
+		BMElem *ele;
+		int count = 0;
+
+		BM_ITER_MESH (ele, &iter, bm, iter_types[i]) {
+			if (BM_elem_flag_test(ele, BM_ELEM_SELECT)) count += 1;
 		}
+		*tots[i] = count;
 	}
 }
 
@@ -71,7 +76,7 @@ static void recount_totsels(BMesh *bm)
  * (ie: all verts of an edge selects the edge and so on).
  * This should only be called by system and not tool authors.
  */
-void BM_mesh_select_mode_flush(BMesh *bm)
+void BM_mesh_select_mode_flush_ex(BMesh *bm, const short selectmode)
 {
 	BMEdge *e;
 	BMLoop *l_iter;
@@ -81,52 +86,61 @@ void BM_mesh_select_mode_flush(BMesh *bm)
 	BMIter eiter;
 	BMIter fiter;
 
-	int ok;
-
-	if (bm->selectmode & SCE_SELECT_VERTEX) {
-		BM_ITER_MESH (e, &eiter, bm, BM_EDGES_OF_MESH) {
-			if (BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
-			    BM_elem_flag_test(e->v2, BM_ELEM_SELECT) &&
-			    !BM_elem_flag_test(e, BM_ELEM_HIDDEN))
+	if (selectmode & SCE_SELECT_VERTEX) {
+		/* both loops only set edge/face flags and read off verts */
+#pragma omp parallel sections if (bm->totedge + bm->totface >= BM_OMP_LIMIT)
+		{
+#pragma omp section
 			{
-				BM_elem_flag_enable(e, BM_ELEM_SELECT);
-			}
-			else {
-				BM_elem_flag_disable(e, BM_ELEM_SELECT);
-			}
-		}
-		BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
-			ok = TRUE;
-			if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
-				l_iter = l_first = BM_FACE_FIRST_LOOP(f);
-				do {
-					if (!BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)) {
-						ok = FALSE;
-						break;
+				BM_ITER_MESH (e, &eiter, bm, BM_EDGES_OF_MESH) {
+					if (BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
+						BM_elem_flag_test(e->v2, BM_ELEM_SELECT) &&
+						!BM_elem_flag_test(e, BM_ELEM_HIDDEN))
+					{
+						BM_elem_flag_enable(e, BM_ELEM_SELECT);
 					}
-				} while ((l_iter = l_iter->next) != l_first);
-			}
-			else {
-				ok = FALSE;
+					else {
+						BM_elem_flag_disable(e, BM_ELEM_SELECT);
+					}
+				}
 			}
+#pragma omp section
+			{
+				BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
+					bool ok = true;
+					if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
+						l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+						do {
+							if (!BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)) {
+								ok = false;
+								break;
+							}
+						} while ((l_iter = l_iter->next) != l_first);
+					}
+					else {
+						ok = false;
+					}
 
-			BM_elem_flag_set(f, BM_ELEM_SELECT, ok);
+					BM_elem_flag_set(f, BM_ELEM_SELECT, ok);
+				}
+			}
 		}
+		/* end sections */
 	}
-	else if (bm->selectmode & SCE_SELECT_EDGE) {
+	else if (selectmode & SCE_SELECT_EDGE) {
 		BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
-			ok = TRUE;
+			bool ok = true;
 			if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
 				l_iter = l_first = BM_FACE_FIRST_LOOP(f);
 				do {
 					if (!BM_elem_flag_test(l_iter->e, BM_ELEM_SELECT)) {
-						ok = FALSE;
+						ok = false;
 						break;
 					}
 				} while ((l_iter = l_iter->next) != l_first);
 			}
 			else {
-				ok = FALSE;
+				ok = false;
 			}
 
 			BM_elem_flag_set(f, BM_ELEM_SELECT, ok);
@@ -139,6 +153,11 @@ void BM_mesh_select_mode_flush(BMesh *bm)
 	recount_totsels(bm);
 }
 
+void BM_mesh_select_mode_flush(BMesh *bm)
+{
+	BM_mesh_select_mode_flush_ex(bm, bm->selectmode);
+}
+
 /**
  * mode independent flushing up/down
  */
@@ -152,36 +171,47 @@ void BM_mesh_deselect_flush(BMesh *bm)
 	BMIter eiter;
 	BMIter fiter;
 
-	int ok;
+	bool ok;
 
-	BM_ITER_MESH (e, &eiter, bm, BM_EDGES_OF_MESH) {
-		if (!(BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
-		      BM_elem_flag_test(e->v2, BM_ELEM_SELECT) &&
-		      !BM_elem_flag_test(e, BM_ELEM_HIDDEN)))
+	/* we can use 2 sections here because the second loop isnt checking edge selection */
+#pragma omp parallel sections if (bm->totedge + bm->totface >= BM_OMP_LIMIT)
+	{
+#pragma omp section
 		{
-			BM_elem_flag_disable(e, BM_ELEM_SELECT);
-		}
-	}
-
-	BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
-		ok = TRUE;
-		if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
-			l_iter = l_first = BM_FACE_FIRST_LOOP(f);
-			do {
-				if (!BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)) {
-					ok = FALSE;
-					break;
+			BM_ITER_MESH (e, &eiter, bm, BM_EDGES_OF_MESH) {
+				if (!(BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
+				      BM_elem_flag_test(e->v2, BM_ELEM_SELECT) &&
+				      !BM_elem_flag_test(e, BM_ELEM_HIDDEN)))
+				{
+					BM_elem_flag_disable(e, BM_ELEM_SELECT);
 				}
-			} while ((l_iter = l_iter->next) != l_first);
-		}
-		else {
-			ok = FALSE;
+			}
 		}
 
-		if (ok == FALSE) {
-			BM_elem_flag_disable(f, BM_ELEM_SELECT);
+#pragma omp section
+		{
+			BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
+				ok = true;
+				if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
+					l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+					do {
+						if (!BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)) {
+							ok = false;
+							break;
+						}
+					} while ((l_iter = l_iter->next) != l_first);
+				}
+				else {
+					ok = false;
+				}
+
+				if (ok == false) {
+					BM_elem_flag_disable(f, BM_ELEM_SELECT);
+				}
+			}
 		}
 	}
+	/* end sections */
 
 	/* Remove any deselected elements from the BMEditSelection */
 	BM_select_history_validate(bm);
@@ -203,34 +233,44 @@ void BM_mesh_select_flush(BMesh *bm)
 	BMIter eiter;
 	BMIter fiter;
 
-	int ok;
+	bool ok;
 
-	BM_ITER_MESH (e, &eiter, bm, BM_EDGES_OF_MESH) {
-		if (BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
-		    BM_elem_flag_test(e->v2, BM_ELEM_SELECT) &&
-		    !BM_elem_flag_test(e, BM_ELEM_HIDDEN))
+	/* we can use 2 sections here because the second loop isnt checking edge selection */
+#pragma omp parallel sections if (bm->totedge + bm->totface >= BM_OMP_LIMIT)
+	{
+#pragma omp section
 		{
-			BM_elem_flag_enable(e, BM_ELEM_SELECT);
-		}
-	}
-
-	BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
-		ok = TRUE;
-		if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
-			l_iter = l_first = BM_FACE_FIRST_LOOP(f);
-			do {
-				if (!BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)) {
-					ok = FALSE;
-					break;
+			BM_ITER_MESH (e, &eiter, bm, BM_EDGES_OF_MESH) {
+				if (BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
+				    BM_elem_flag_test(e->v2, BM_ELEM_SELECT) &&
+				    !BM_elem_flag_test(e, BM_ELEM_HIDDEN))
+				{
+					BM_elem_flag_enable(e, BM_ELEM_SELECT);
 				}
-			} while ((l_iter = l_iter->next) != l_first);
-		}
-		else {
-			ok = FALSE;
+			}
 		}
 
-		if (ok) {
-			BM_elem_flag_enable(f, BM_ELEM_SELECT);
+#pragma omp section
+		{
+			BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
+				ok = true;
+				if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
+					l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+					do {
+						if (!BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)) {
+							ok = false;
+							break;
+						}
+					} while ((l_iter = l_iter->next) != l_first);
+				}
+				else {
+					ok = false;
+				}
+
+				if (ok) {
+					BM_elem_flag_enable(f, BM_ELEM_SELECT);
+				}
+			}
 		}
 	}
 
@@ -243,7 +283,7 @@ void BM_mesh_select_flush(BMesh *bm)
  * Changes selection state of a single vertex
  * in a mesh
  */
-void BM_vert_select_set(BMesh *bm, BMVert *v, int select)
+void BM_vert_select_set(BMesh *bm, BMVert *v, const bool select)
 {
 	BLI_assert(v->head.htype == BM_VERT);
 
@@ -270,7 +310,7 @@ void BM_vert_select_set(BMesh *bm, BMVert *v, int select)
  *
  * Changes selection state of a single edge in a mesh.
  */
-void BM_edge_select_set(BMesh *bm, BMEdge *e, int select)
+void BM_edge_select_set(BMesh *bm, BMEdge *e, const bool select)
 {
 	BLI_assert(e->head.htype == BM_EDGE);
 
@@ -282,8 +322,8 @@ void BM_edge_select_set(BMesh *bm, BMEdge *e, int select)
 		if (!BM_elem_flag_test(e, BM_ELEM_SELECT)) bm->totedgesel += 1;
 
 		BM_elem_flag_enable(e, BM_ELEM_SELECT);
-		BM_vert_select_set(bm, e->v1, TRUE);
-		BM_vert_select_set(bm, e->v2, TRUE);
+		BM_vert_select_set(bm, e->v1, true);
+		BM_vert_select_set(bm, e->v2, true);
 	}
 	else {
 		if (BM_elem_flag_test(e, BM_ELEM_SELECT)) bm->totedgesel -= 1;
@@ -314,13 +354,13 @@ void BM_edge_select_set(BMesh *bm, BMEdge *e, int select)
 				}
 
 				if (deselect) {
-					BM_vert_select_set(bm, verts[i], FALSE);
+					BM_vert_select_set(bm, verts[i], false);
 				}
 			}
 		}
 		else {
-			BM_vert_select_set(bm, e->v1, FALSE);
-			BM_vert_select_set(bm, e->v2, FALSE);
+			BM_vert_select_set(bm, e->v1, false);
+			BM_vert_select_set(bm, e->v2, false);
 		}
 
 	}
@@ -332,7 +372,7 @@ void BM_edge_select_set(BMesh *bm, BMEdge *e, int select)
  * Changes selection state of a single
  * face in a mesh.
  */
-void BM_face_select_set(BMesh *bm, BMFace *f, int select)
+void BM_face_select_set(BMesh *bm, BMFace *f, const bool select)
 {
 	BMLoop *l_iter;
 	BMLoop *l_first;
@@ -351,8 +391,8 @@ void BM_face_select_set(BMesh *bm, BMFace *f, int select)
 		BM_elem_flag_enable(f, BM_ELEM_SELECT);
 		l_iter = l_first = BM_FACE_FIRST_LOOP(f);
 		do {
-			BM_vert_select_set(bm, l_iter->v, TRUE);
-			BM_edge_select_set(bm, l_iter->e, TRUE);
+			BM_vert_select_set(bm, l_iter->v, true);
+			BM_edge_select_set(bm, l_iter->e, true);
 		} while ((l_iter = l_iter->next) != l_first);
 	}
 	else {
@@ -372,7 +412,7 @@ void BM_face_select_set(BMesh *bm, BMFace *f, int select)
 			}
 
 			if (!f2) {
-				BM_edge_select_set(bm, l->e, FALSE);
+				BM_edge_select_set(bm, l->e, false);
 			}
 		}
 
@@ -386,7 +426,7 @@ void BM_face_select_set(BMesh *bm, BMFace *f, int select)
 			}
 
 			if (!e) {
-				BM_vert_select_set(bm, l->v, FALSE);
+				BM_vert_select_set(bm, l->v, false);
 			}
 		}
 	}
@@ -427,7 +467,7 @@ void BM_mesh_select_mode_set(BMesh *bm, int selectmode)
 
 		BM_ITER_MESH (ele, &iter, bm, BM_EDGES_OF_MESH) {
 			if (BM_elem_flag_test(ele, BM_ELEM_SELECT)) {
-				BM_edge_select_set(bm, (BMEdge *)ele, TRUE);
+				BM_edge_select_set(bm, (BMEdge *)ele, true);
 			}
 		}
 		BM_mesh_select_mode_flush(bm);
@@ -441,7 +481,7 @@ void BM_mesh_select_mode_set(BMesh *bm, int selectmode)
 #endif
 		BM_ITER_MESH (ele, &iter, bm, BM_FACES_OF_MESH) {
 			if (BM_elem_flag_test(ele, BM_ELEM_SELECT)) {
-				BM_face_select_set(bm, (BMFace *)ele, TRUE);
+				BM_face_select_set(bm, (BMFace *)ele, true);
 			}
 		}
 		BM_mesh_select_mode_flush(bm);
@@ -452,14 +492,12 @@ void BM_mesh_select_mode_set(BMesh *bm, int selectmode)
  * counts number of elements with flag enabled/disabled
  */
 static int bm_mesh_flag_count(BMesh *bm, const char htype, const char hflag,
-                              const short respecthide, const short test_for_enabled)
+                              const short respecthide, const bool test_for_enabled)
 {
 	BMElem *ele;
 	BMIter iter;
 	int tot = 0;
 
-	BLI_assert(ELEM(TRUE, FALSE, test_for_enabled));
-
 	if (htype & BM_VERT) {
 		for (ele = BM_iter_new(&iter, bm, BM_VERTS_OF_MESH, NULL); ele; ele = BM_iter_step(&iter)) {
 			if (respecthide && BM_elem_flag_test(ele, BM_ELEM_HIDDEN)) continue;
@@ -482,21 +520,21 @@ static int bm_mesh_flag_count(BMesh *bm, const char htype, const char hflag,
 	return tot;
 }
 
-int BM_mesh_elem_hflag_count_enabled(BMesh *bm, const char htype, const char hflag, int respecthide)
+int BM_mesh_elem_hflag_count_enabled(BMesh *bm, const char htype, const char hflag, const bool respecthide)
 {
-	return bm_mesh_flag_count(bm, htype, hflag, respecthide, TRUE);
+	return bm_mesh_flag_count(bm, htype, hflag, respecthide, true);
 }
 
-int BM_mesh_elem_hflag_count_disabled(BMesh *bm, const char htype, const char hflag, int respecthide)
+int BM_mesh_elem_hflag_count_disabled(BMesh *bm, const char htype, const char hflag, const bool respecthide)
 {
-	return bm_mesh_flag_count(bm, htype, hflag, respecthide, FALSE);
+	return bm_mesh_flag_count(bm, htype, hflag, respecthide, false);
 }
 
 /**
  * \note use BM_elem_flag_test(ele, BM_ELEM_SELECT) to test selection
  * \note by design, this will not touch the editselection history stuff
  */
-void BM_elem_select_set(BMesh *bm, BMElem *ele, int select)
+void BM_elem_select_set(BMesh *bm, BMElem *ele, const bool select)
 {
 	switch (ele->head.htype) {
 		case BM_VERT:
@@ -520,12 +558,12 @@ void BM_active_face_set(BMesh *bm, BMFace *efa)
 	bm->act_face = efa;
 }
 
-BMFace *BM_active_face_get(BMesh *bm, int sloppy)
+BMFace *BM_active_face_get(BMesh *bm, const bool is_sloppy, const bool is_selected)
 {
-	if (bm->act_face) {
+	if (bm->act_face && (!is_selected || BM_elem_flag_test(bm->act_face, BM_ELEM_SELECT))) {
 		return bm->act_face;
 	}
-	else if (sloppy) {
+	else if (is_sloppy) {
 		BMIter iter;
 		BMFace *f = NULL;
 		BMEditSelection *ese;
@@ -539,6 +577,9 @@ BMFace *BM_active_face_get(BMesh *bm, int sloppy)
 				if (BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
 					f = NULL;
 				}
+				else if (is_selected && !BM_elem_flag_test(f, BM_ELEM_SELECT)) {
+					f = NULL;
+				}
 				else {
 					break;
 				}
@@ -562,9 +603,9 @@ BMFace *BM_active_face_get(BMesh *bm, int sloppy)
  * These functions were written to be used by the Modifier widget
  * when in Rotate about active mode, but can be used anywhere.
  *
- * - #EM_editselection_center
- * - #EM_editselection_normal
- * - #EM_editselection_plane
+ * - #BM_editselection_center
+ * - #BM_editselection_normal
+ * - #BM_editselection_plane
  */
 void BM_editselection_center(BMEditSelection *ese, float r_center[3])
 {
@@ -574,8 +615,7 @@ void BM_editselection_center(BMEditSelection *ese, float r_center[3])
 	}
 	else if (ese->htype == BM_EDGE) {
 		BMEdge *eed = (BMEdge *)ese->ele;
-		add_v3_v3v3(r_center, eed->v1->co, eed->v2->co);
-		mul_v3_fl(r_center, 0.5);
+		mid_v3_v3v3(r_center, eed->v1->co, eed->v2->co);
 	}
 	else if (ese->htype == BM_FACE) {
 		BMFace *efa = (BMFace *)ese->ele;
@@ -612,8 +652,6 @@ void BM_editselection_normal(BMEditSelection *ese, float r_normal[3])
 	}
 }
 
-/* ref - editmesh_lib.cL:EM_editselection_plane() */
-
 /* Calculate a plane that is rightangles to the edge/vert/faces normal
  * also make the plane run along an axis that is related to the geometry,
  * because this is used for the manipulators Y axis. */
@@ -664,7 +702,7 @@ void BM_editselection_plane(BMEditSelection *ese, float r_plane[3])
 		 * we cant make a crossvec from a vec thats the same as the vec
 		 * unlikely but possible, so make sure if the normal is (0, 0, 1)
 		 * that vec isn't the same or in the same direction even. */
-		if (efa->len < 3) {
+		if (UNLIKELY(efa->len < 3)) {
 			/* crappy fallback method */
 			if      (efa->no[0] < 0.5f)	vec[0] = 1.0f;
 			else if (efa->no[1] < 0.5f)	vec[1] = 1.0f;
@@ -672,12 +710,13 @@ void BM_editselection_plane(BMEditSelection *ese, float r_plane[3])
 			cross_v3_v3v3(r_plane, efa->no, vec);
 		}
 		else {
-			BMVert *verts[4] = {NULL};
-
-			BM_iter_as_array(NULL, BM_VERTS_OF_FACE, efa, (void **)verts, 4);
-
 			if (efa->len == 4) {
+				BMVert *verts[4] = {NULL};
 				float vecA[3], vecB[3];
+
+				// BM_iter_as_array(NULL, BM_VERTS_OF_FACE, efa, (void **)verts, 4);
+				BM_face_as_array_vert_quad(efa, verts);
+
 				sub_v3_v3v3(vecA, verts[3]->co, verts[2]->co);
 				sub_v3_v3v3(vecB, verts[0]->co, verts[1]->co);
 				add_v3_v3v3(r_plane, vecA, vecB);
@@ -691,21 +730,9 @@ void BM_editselection_plane(BMEditSelection *ese, float r_plane[3])
 				}
 			}
 			else {
-				/* BMESH_TODO (not urgent, use longest ngon edge for alignment) */
+				BMLoop *l_long  = BM_face_find_longest_loop(efa);
 
-				/* start with v1-2 */
-				sub_v3_v3v3(r_plane, verts[0]->co, verts[1]->co);
-
-				/* test the edge between v2-3, use if longer */
-				sub_v3_v3v3(vec, verts[1]->co, verts[2]->co);
-				if (dot_v3v3(r_plane, r_plane) < dot_v3v3(vec, vec))
-					copy_v3_v3(r_plane, vec);
-
-				/* test the edge between v1-3, use if longer */
-				sub_v3_v3v3(vec, verts[2]->co, verts[0]->co);
-				if (dot_v3v3(r_plane, r_plane) < dot_v3v3(vec, vec)) {
-					copy_v3_v3(r_plane, vec);
-				}
+				sub_v3_v3v3(r_plane, l_long->v->co, l_long->next->v->co);
 			}
 
 		}
@@ -715,30 +742,21 @@ void BM_editselection_plane(BMEditSelection *ese, float r_plane[3])
 
 
 /* --- macro wrapped funcs --- */
-int _bm_select_history_check(BMesh *bm, const BMHeader *ele)
+bool _bm_select_history_check(BMesh *bm, const BMHeader *ele)
 {
-	BMEditSelection *ese;
-	
-	for (ese = bm->selected.first; ese; ese = ese->next) {
-		if (ese->ele == (BMElem *)ele) {
-			return TRUE;
-		}
-	}
-	
-	return FALSE;
+	return (BLI_findptr(&bm->selected, ele, offsetof(BMEditSelection, ele)) != NULL);
 }
 
-int _bm_select_history_remove(BMesh *bm, BMHeader *ele)
+bool _bm_select_history_remove(BMesh *bm, BMHeader *ele)
 {
-	BMEditSelection *ese;
-	for (ese = bm->selected.first; ese; ese = ese->next) {
-		if (ese->ele == (BMElem *)ele) {
-			BLI_freelinkN(&(bm->selected), ese);
-			return TRUE;
-		}
+	BMEditSelection *ese = BLI_findptr(&bm->selected, ele, offsetof(BMEditSelection, ele));
+	if (ese) {
+		BLI_freelinkN(&bm->selected, ese);
+		return true;
+	}
+	else {
+		return false;
 	}
-
-	return FALSE;
 }
 
 void _bm_select_history_store_notest(BMesh *bm, BMHeader *ele)
@@ -781,10 +799,10 @@ void BM_select_history_validate(BMesh *bm)
 }
 
 /* utility function */
-int BM_select_history_active_get(BMesh *bm, BMEditSelection *ese)
+bool BM_select_history_active_get(BMesh *bm, BMEditSelection *ese)
 {
 	BMEditSelection *ese_last = bm->selected.last;
-	BMFace *efa = BM_active_face_get(bm, FALSE);
+	BMFace *efa = BM_active_face_get(bm, false, false);
 
 	ese->next = ese->prev = NULL;
 
@@ -809,14 +827,14 @@ int BM_select_history_active_get(BMesh *bm, BMEditSelection *ese)
 	}
 	else {
 		ese->ele = NULL;
-		return FALSE;
+		return false;
 	}
 
-	return TRUE;
+	return true;
 }
 
 void BM_mesh_elem_hflag_disable_test(BMesh *bm, const char htype, const char hflag,
-                                     int respecthide, const char hflag_test)
+                                     const bool respecthide, const char hflag_test)
 {
 	const char iter_types[3] = {BM_VERTS_OF_MESH,
 	                            BM_EDGES_OF_MESH,
@@ -824,8 +842,6 @@ void BM_mesh_elem_hflag_disable_test(BMesh *bm, const char htype, const char hfl
 
 	const char flag_types[3] = {BM_VERT, BM_EDGE, BM_FACE};
 
-	BMIter iter;
-	BMElem *ele;
 	int i;
 
 	if (hflag & BM_ELEM_SELECT) {
@@ -834,21 +850,30 @@ void BM_mesh_elem_hflag_disable_test(BMesh *bm, const char htype, const char hfl
 
 	if ((htype == (BM_VERT | BM_EDGE | BM_FACE)) &&
 	    (hflag == BM_ELEM_SELECT) &&
-	    (respecthide == FALSE) &&
+	    (respecthide == false) &&
 	    (hflag_test == 0))
 	{
 		/* fast path for deselect all, avoid topology loops
 		 * since we know all will be de-selected anyway. */
+
+#pragma omp parallel for schedule(dynamic) if (bm->totvert + bm->totedge + bm->totface >= BM_OMP_LIMIT)
 		for (i = 0; i < 3; i++) {
+			BMIter iter;
+			BMElem *ele;
+
 			ele = BM_iter_new(&iter, bm, iter_types[i], NULL);
 			for ( ; ele; ele = BM_iter_step(&iter)) {
 				BM_elem_flag_disable(ele, BM_ELEM_SELECT);
 			}
 		}
+
 		bm->totvertsel = bm->totedgesel = bm->totfacesel = 0;
 	}
 	else {
 		for (i = 0; i < 3; i++) {
+			BMIter iter;
+			BMElem *ele;
+
 			if (htype & flag_types[i]) {
 				ele = BM_iter_new(&iter, bm, iter_types[i], NULL);
 				for ( ; ele; ele = BM_iter_step(&iter)) {
@@ -861,7 +886,7 @@ void BM_mesh_elem_hflag_disable_test(BMesh *bm, const char htype, const char hfl
 					}
 
 					if (hflag & BM_ELEM_SELECT) {
-						BM_elem_select_set(bm, ele, FALSE);
+						BM_elem_select_set(bm, ele, false);
 					}
 					BM_elem_flag_disable(ele, hflag);
 				}
@@ -871,7 +896,7 @@ void BM_mesh_elem_hflag_disable_test(BMesh *bm, const char htype, const char hfl
 }
 
 void BM_mesh_elem_hflag_enable_test(BMesh *bm, const char htype, const char hflag,
-                                    int respecthide, const char hflag_test)
+                                    const bool respecthide, const char hflag_test)
 {
 	const char iter_types[3] = {BM_VERTS_OF_MESH,
 	                            BM_EDGES_OF_MESH,
@@ -894,7 +919,7 @@ void BM_mesh_elem_hflag_enable_test(BMesh *bm, const char htype, const char hfla
 
 	/* note, better not attempt a fast path for selection as done with de-select
 	 * because hidden geometry and different selection modes can give different results,
-	 * we could of course check for no hiddent faces and then use quicker method but its not worth it. */
+	 * we could of course check for no hidden faces and then use quicker method but its not worth it. */
 
 	for (i = 0; i < 3; i++) {
 		if (htype & flag_types[i]) {
@@ -909,7 +934,7 @@ void BM_mesh_elem_hflag_enable_test(BMesh *bm, const char htype, const char hfla
 				}
 
 				if (hflag & BM_ELEM_SELECT) {
-					BM_elem_select_set(bm, ele, TRUE);
+					BM_elem_select_set(bm, ele, true);
 				}
 				BM_elem_flag_enable(ele, hflag_nosel);
 			}
@@ -918,14 +943,14 @@ void BM_mesh_elem_hflag_enable_test(BMesh *bm, const char htype, const char hfla
 }
 
 void BM_mesh_elem_hflag_disable_all(BMesh *bm, const char htype, const char hflag,
-                                    int respecthide)
+                                    const bool respecthide)
 {
 	/* call with 0 hflag_test */
 	BM_mesh_elem_hflag_disable_test(bm, htype, hflag, respecthide, 0);
 }
 
 void BM_mesh_elem_hflag_enable_all(BMesh *bm, const char htype, const char hflag,
-                                   int respecthide)
+                                   const bool respecthide)
 {
 	/* call with 0 hflag_test */
 	BM_mesh_elem_hflag_enable_test(bm, htype, hflag, respecthide, 0);
@@ -937,7 +962,7 @@ static void vert_flush_hide_set(BMVert *v)
 {
 	BMIter iter;
 	BMEdge *e;
-	int hide = TRUE;
+	bool hide = true;
 
 	BM_ITER_ELEM (e, &iter, v, BM_EDGES_OF_VERT) {
 		hide = hide && BM_elem_flag_test(e, BM_ELEM_HIDDEN);
@@ -950,7 +975,7 @@ static void edge_flush_hide(BMEdge *e)
 {
 	BMIter iter;
 	BMFace *f;
-	int hide = TRUE;
+	bool hide = true;
 
 	BM_ITER_ELEM (f, &iter, e, BM_FACES_OF_EDGE) {
 		hide = hide && BM_elem_flag_test(f, BM_ELEM_HIDDEN);
@@ -959,13 +984,15 @@ static void edge_flush_hide(BMEdge *e)
 	BM_elem_flag_set(e, BM_ELEM_HIDDEN, hide);
 }
 
-void BM_vert_hide_set(BMVert *v, int hide)
+void BM_vert_hide_set(BMVert *v, const bool hide)
 {
 	/* vert hiding: vert + surrounding edges and faces */
 	BMIter iter, fiter;
 	BMEdge *e;
 	BMFace *f;
 
+	BLI_assert(v->head.htype == BM_VERT);
+
 	BM_elem_flag_set(v, BM_ELEM_HIDDEN, hide);
 
 	BM_ITER_ELEM (e, &iter, v, BM_EDGES_OF_VERT) {
@@ -977,12 +1004,14 @@ void BM_vert_hide_set(BMVert *v, int hide)
 	}
 }
 
-void BM_edge_hide_set(BMEdge *e, int hide)
+void BM_edge_hide_set(BMEdge *e, const bool hide)
 {
 	BMIter iter;
 	BMFace *f;
 	/* BMVert *v; */
 
+	BLI_assert(e->head.htype == BM_EDGE);
+
 	/* edge hiding: faces around the edge */
 	BM_ITER_ELEM (f, &iter, e, BM_FACES_OF_EDGE) {
 		BM_elem_flag_set(f, BM_ELEM_HIDDEN, hide);
@@ -995,11 +1024,13 @@ void BM_edge_hide_set(BMEdge *e, int hide)
 	vert_flush_hide_set(e->v2);
 }
 
-void BM_face_hide_set(BMFace *f, int hide)
+void BM_face_hide_set(BMFace *f, const bool hide)
 {
 	BMIter iter;
 	BMLoop *l;
 
+	BLI_assert(f->head.htype == BM_FACE);
+
 	BM_elem_flag_set(f, BM_ELEM_HIDDEN, hide);
 
 	BM_ITER_ELEM (l, &iter, f, BM_LOOPS_OF_FACE) {
@@ -1011,21 +1042,21 @@ void BM_face_hide_set(BMFace *f, int hide)
 	}
 }
 
-void _bm_elem_hide_set(BMesh *bm, BMHeader *head, int hide)
+void _bm_elem_hide_set(BMesh *bm, BMHeader *head, const bool hide)
 {
 	/* Follow convention of always deselecting before
 	 * hiding an element */
 	switch (head->htype) {
 		case BM_VERT:
-			if (hide) BM_vert_select_set(bm, (BMVert *)head, FALSE);
+			if (hide) BM_vert_select_set(bm, (BMVert *)head, false);
 			BM_vert_hide_set((BMVert *)head, hide);
 			break;
 		case BM_EDGE:
-			if (hide) BM_edge_select_set(bm, (BMEdge *)head, FALSE);
+			if (hide) BM_edge_select_set(bm, (BMEdge *)head, false);
 			BM_edge_hide_set((BMEdge *)head, hide);
 			break;
 		case BM_FACE:
-			if (hide) BM_face_select_set(bm, (BMFace *)head, FALSE);
+			if (hide) BM_face_select_set(bm, (BMFace *)head, false);
 			BM_face_hide_set((BMFace *)head, hide);
 			break;
 		default:
