@@ -42,22 +42,84 @@
 #include "intern/bmesh_private.h"
 
 /* used as an extern, defined in bmesh.h */
-BMAllocTemplate bm_mesh_allocsize_default = {512, 1024, 2048, 512};
-BMAllocTemplate bm_mesh_chunksize_default = {512, 1024, 2048, 512};
+const BMAllocTemplate bm_mesh_allocsize_default = {512, 1024, 2048, 512};
+const BMAllocTemplate bm_mesh_chunksize_default = {512, 1024, 2048, 512};
 
 static void bm_mempool_init(BMesh *bm, const BMAllocTemplate *allocsize)
 {
-	bm->vpool = BLI_mempool_create(sizeof(BMVert), allocsize->totvert, bm_mesh_chunksize_default.totvert, BLI_MEMPOOL_ALLOW_ITER);
-	bm->epool = BLI_mempool_create(sizeof(BMEdge), allocsize->totedge, bm_mesh_chunksize_default.totedge, BLI_MEMPOOL_ALLOW_ITER);
-	bm->lpool = BLI_mempool_create(sizeof(BMLoop), allocsize->totloop, bm_mesh_chunksize_default.totloop, 0);
-	bm->fpool = BLI_mempool_create(sizeof(BMFace), allocsize->totface, bm_mesh_chunksize_default.totface, BLI_MEMPOOL_ALLOW_ITER);
+	bm->vpool = BLI_mempool_create(sizeof(BMVert), allocsize->totvert,
+	                               bm_mesh_chunksize_default.totvert, BLI_MEMPOOL_ALLOW_ITER);
+	bm->epool = BLI_mempool_create(sizeof(BMEdge), allocsize->totedge,
+	                               bm_mesh_chunksize_default.totedge, BLI_MEMPOOL_ALLOW_ITER);
+	bm->lpool = BLI_mempool_create(sizeof(BMLoop), allocsize->totloop,
+	                               bm_mesh_chunksize_default.totloop, 0);
+	bm->fpool = BLI_mempool_create(sizeof(BMFace), allocsize->totface,
+	                               bm_mesh_chunksize_default.totface, BLI_MEMPOOL_ALLOW_ITER);
 
 #ifdef USE_BMESH_HOLES
-	bm->looplistpool = BLI_mempool_create(sizeof(BMLoopList), allocsize[3], allocsize[3], FALSE, FALSE);
+	bm->looplistpool = BLI_mempool_create(sizeof(BMLoopList), 512, 512, 0);
 #endif
+}
 
-	/* allocate one flag pool that we don't get rid of. */
-	bm->toolflagpool = BLI_mempool_create(sizeof(BMFlagLayer), 512, 512, 0);
+void BM_mesh_elem_toolflags_ensure(BMesh *bm)
+{
+	if (bm->vtoolflagpool && bm->etoolflagpool && bm->ftoolflagpool) {
+		return;
+	}
+
+	bm->vtoolflagpool = BLI_mempool_create(sizeof(BMFlagLayer), max_ii(512, bm->totvert), 512, 0);
+	bm->etoolflagpool = BLI_mempool_create(sizeof(BMFlagLayer), max_ii(512, bm->totedge), 512, 0);
+	bm->ftoolflagpool = BLI_mempool_create(sizeof(BMFlagLayer), max_ii(512, bm->totface), 512, 0);
+
+#pragma omp parallel sections if (bm->totvert + bm->totedge + bm->totface >= BM_OMP_LIMIT)
+	{
+#pragma omp section
+		{
+			BLI_mempool *toolflagpool = bm->vtoolflagpool;
+			BMIter iter;
+			BMElemF *ele;
+			BM_ITER_MESH (ele, &iter, bm, BM_VERTS_OF_MESH) {
+				ele->oflags = BLI_mempool_calloc(toolflagpool);
+			}
+		}
+#pragma omp section
+		{
+			BLI_mempool *toolflagpool = bm->etoolflagpool;
+			BMIter iter;
+			BMElemF *ele;
+			BM_ITER_MESH (ele, &iter, bm, BM_EDGES_OF_MESH) {
+				ele->oflags = BLI_mempool_calloc(toolflagpool);
+			}
+		}
+#pragma omp section
+		{
+			BLI_mempool *toolflagpool = bm->ftoolflagpool;
+			BMIter iter;
+			BMElemF *ele;
+			BM_ITER_MESH (ele, &iter, bm, BM_FACES_OF_MESH) {
+				ele->oflags = BLI_mempool_calloc(toolflagpool);
+			}
+		}
+	}
+
+
+	bm->totflags = 1;
+}
+
+void BM_mesh_elem_toolflags_clear(BMesh *bm)
+{
+	if (bm->vtoolflagpool) {
+		BLI_mempool_destroy(bm->vtoolflagpool);
+		bm->vtoolflagpool = NULL;
+	}
+	if (bm->etoolflagpool) {
+		BLI_mempool_destroy(bm->etoolflagpool);
+		bm->etoolflagpool = NULL;
+	}
+	if (bm->ftoolflagpool) {
+		BLI_mempool_destroy(bm->ftoolflagpool);
+		bm->ftoolflagpool = NULL;
+	}
 }
 
 /**
@@ -69,7 +131,7 @@ static void bm_mempool_init(BMesh *bm, const BMAllocTemplate *allocsize)
  *
  * \note ob is needed by multires
  */
-BMesh *BM_mesh_create(BMAllocTemplate *allocsize)
+BMesh *BM_mesh_create(const BMAllocTemplate *allocsize)
 {
 	/* allocate the structure */
 	BMesh *bm = MEM_callocN(sizeof(BMesh), __func__);
@@ -79,7 +141,12 @@ BMesh *BM_mesh_create(BMAllocTemplate *allocsize)
 
 	/* allocate one flag pool that we don't get rid of. */
 	bm->stackdepth = 1;
-	bm->totflags = 1;
+	bm->totflags = 0;
+
+	CustomData_reset(&bm->vdata);
+	CustomData_reset(&bm->edata);
+	CustomData_reset(&bm->ldata);
+	CustomData_reset(&bm->pdata);
 
 	return bm;
 }
@@ -134,7 +201,7 @@ void BM_mesh_data_free(BMesh *bm)
 	BLI_mempool_destroy(bm->fpool);
 
 	/* destroy flag pool */
-	BLI_mempool_destroy(bm->toolflagpool);
+	BM_mesh_elem_toolflags_clear(bm);
 
 #ifdef USE_BMESH_HOLES
 	BLI_mempool_destroy(bm->looplistpool);
@@ -161,6 +228,11 @@ void BM_mesh_clear(BMesh *bm)
 
 	bm->stackdepth = 1;
 	bm->totflags = 1;
+
+	CustomData_reset(&bm->vdata);
+	CustomData_reset(&bm->edata);
+	CustomData_reset(&bm->ldata);
+	CustomData_reset(&bm->pdata);
 }
 
 /**
@@ -173,7 +245,7 @@ void BM_mesh_free(BMesh *bm)
 	BM_mesh_data_free(bm);
 
 	if (bm->py_handle) {
-		/* keep this out of 'BM_mesh_data_free' because we wan't python
+		/* keep this out of 'BM_mesh_data_free' because we want python
 		 * to be able to clear the mesh and maintain access. */
 		extern void bpy_bm_generic_invalidate(void *self);
 
@@ -189,7 +261,7 @@ void BM_mesh_free(BMesh *bm)
  *
  * Updates the normals of a mesh.
  */
-void BM_mesh_normals_update(BMesh *bm, const short skip_hidden)
+void BM_mesh_normals_update(BMesh *bm, const bool skip_hidden)
 {
 	BMVert *v;
 	BMFace *f;
@@ -278,7 +350,7 @@ void BM_mesh_normals_update(BMesh *bm, const short skip_hidden)
 		if (skip_hidden && BM_elem_flag_test(v, BM_ELEM_HIDDEN))
 			continue;
 
-		if (normalize_v3(v->no) == 0.0f) {
+		if (UNLIKELY(normalize_v3(v->no) == 0.0f)) {
 			normalize_v3_v3(v->no, v->co);
 		}
 	}
@@ -286,53 +358,12 @@ void BM_mesh_normals_update(BMesh *bm, const short skip_hidden)
 	MEM_freeN(edgevec);
 }
 
-/*
- * This function ensures correct normals for the mesh, but
- * sets the flag BM_ELEM_TAG in flipped faces, to allow restoration
- * of original normals.
- *
- * if undo is 0: calculate right normals
- * if undo is 1: restore original normals
- */
-
-//keep in sycn with utils.c!
-#define FACE_FLIP   8
-static void bm_rationalize_normals(BMesh *bm, int undo)
-{
-	BMOperator bmop;
-	BMFace *f;
-	BMIter iter;
-	
-	if (undo) {
-		BM_ITER_MESH (f, &iter, bm, BM_FACES_OF_MESH) {
-			if (BM_elem_flag_test(f, BM_ELEM_TAG)) {
-				BM_face_normal_flip(bm, f);
-			}
-			BM_elem_flag_disable(f, BM_ELEM_TAG);
-		}
-		
-		return;
-	}
-	
-	BMO_op_initf(bm, &bmop, "righthandfaces faces=%af do_flip=%b", FALSE);
-	
-	BMO_push(bm, &bmop);
-	bmo_righthandfaces_exec(bm, &bmop);
-	
-	BM_ITER_MESH (f, &iter, bm, BM_FACES_OF_MESH) {
-		BM_elem_flag_set(f, BM_ELEM_TAG, BMO_elem_flag_test(bm, f, FACE_FLIP));
-	}
-
-	BMO_pop(bm);
-	BMO_op_finish(bm, &bmop);
-}
-
 static void UNUSED_FUNCTION(bm_mdisps_space_set)(Object *ob, BMesh *bm, int from, int to)
 {
 	/* switch multires data out of tangent space */
 	if (CustomData_has_layer(&bm->ldata, CD_MDISPS)) {
-		BMEditMesh *em = BMEdit_Create(bm, FALSE);
-		DerivedMesh *dm = CDDM_from_BMEditMesh(em, NULL, TRUE, FALSE);
+		BMEditMesh *em = BMEdit_Create(bm, false);
+		DerivedMesh *dm = CDDM_from_editbmesh(em, true, false);
 		MDisps *mdisps;
 		BMFace *f;
 		BMIter iter;
@@ -386,41 +417,32 @@ static void UNUSED_FUNCTION(bm_mdisps_space_set)(Object *ob, BMesh *bm, int from
  * the editing operations are done. These are called by the tools/operator
  * API for each time a tool is executed.
  */
-void bmesh_edit_begin(BMesh *bm, int flag)
+void bmesh_edit_begin(BMesh *UNUSED(bm), int UNUSED(type_flag))
 {
-	bm->opflag = flag;
-	
 	/* Most operators seem to be using BMO_OP_FLAG_UNTAN_MULTIRES to change the MDisps to
 	 * absolute space during mesh edits. With this enabled, changes to the topology
 	 * (loop cuts, edge subdivides, etc) are not reflected in the higher levels of
 	 * the mesh at all, which doesn't seem right. Turning off completely for now,
 	 * until this is shown to be better for certain types of mesh edits. */
-#if BMOP_UNTAN_MULTIRES_ENABLED
+#ifdef BMOP_UNTAN_MULTIRES_ENABLED
 	/* switch multires data out of tangent space */
-	if ((flag & BMO_OP_FLAG_UNTAN_MULTIRES) && CustomData_has_layer(&bm->ldata, CD_MDISPS)) {
+	if ((type_flag & BMO_OP_FLAG_UNTAN_MULTIRES) && CustomData_has_layer(&bm->ldata, CD_MDISPS)) {
 		bmesh_mdisps_space_set(bm, MULTIRES_SPACE_TANGENT, MULTIRES_SPACE_ABSOLUTE);
 
 		/* ensure correct normals, if possible */
 		bmesh_rationalize_normals(bm, 0);
 		BM_mesh_normals_update(bm);
 	}
-	else if (flag & BMO_OP_FLAG_RATIONALIZE_NORMALS) {
-		bmesh_rationalize_normals(bm, 0);
-	}
-#else
-	if (flag & BMO_OP_FLAG_RATIONALIZE_NORMALS) {
-		bm_rationalize_normals(bm, 0);
-	}
 #endif
 }
 
 /**
  * \brief BMesh End Edit
  */
-void bmesh_edit_end(BMesh *bm, int flag)
+void bmesh_edit_end(BMesh *bm, int UNUSED(flag))
 {
 	/* BMO_OP_FLAG_UNTAN_MULTIRES disabled for now, see comment above in bmesh_edit_begin. */
-#if BMOP_UNTAN_MULTIRES_ENABLED
+#ifdef BMOP_UNTAN_MULTIRES_ENABLED
 	/* switch multires data into tangent space */
 	if ((flag & BMO_OP_FLAG_UNTAN_MULTIRES) && CustomData_has_layer(&bm->ldata, CD_MDISPS)) {
 		/* set normals to their previous winding */
@@ -430,72 +452,80 @@ void bmesh_edit_end(BMesh *bm, int flag)
 	else if (flag & BMO_OP_FLAG_RATIONALIZE_NORMALS) {
 		bmesh_rationalize_normals(bm, 1);
 	}
-#else
-	if (flag & BMO_OP_FLAG_RATIONALIZE_NORMALS) {
-		bm_rationalize_normals(bm, 1);
-	}
 #endif
 
-	bm->opflag = 0;
-
 	/* compute normals, clear temp flags and flush selections */
-	BM_mesh_normals_update(bm, TRUE);
+	BM_mesh_normals_update(bm, true);
 	BM_mesh_select_mode_flush(bm);
 }
 
 void BM_mesh_elem_index_ensure(BMesh *bm, const char hflag)
 {
-	BMIter iter;
-	BMElem *ele;
-
 #ifdef DEBUG
 	BM_ELEM_INDEX_VALIDATE(bm, "Should Never Fail!", __func__);
 #endif
 
-	if (hflag & BM_VERT) {
-		if (bm->elem_index_dirty & BM_VERT) {
-			int index = 0;
-			BM_ITER_MESH (ele, &iter, bm, BM_VERTS_OF_MESH) {
-				BM_elem_index_set(ele, index); /* set_ok */
-				index++;
+#pragma omp parallel sections if (bm->totvert + bm->totedge + bm->totface >= BM_OMP_LIMIT)
+	{
+#pragma omp section
+		{
+			if (hflag & BM_VERT) {
+				if (bm->elem_index_dirty & BM_VERT) {
+					BMIter iter;
+					BMElem *ele;
+
+					int index;
+					BM_ITER_MESH_INDEX (ele, &iter, bm, BM_VERTS_OF_MESH, index) {
+						BM_elem_index_set(ele, index); /* set_ok */
+					}
+					BLI_assert(index == bm->totvert);
+				}
+				else {
+					// printf("%s: skipping vert index calc!\n", __func__);
+				}
 			}
-			bm->elem_index_dirty &= ~BM_VERT;
-			BLI_assert(index == bm->totvert);
 		}
-		else {
-			// printf("%s: skipping vert index calc!\n", __func__);
+
+#pragma omp section
+		{
+			if (hflag & BM_EDGE) {
+				if (bm->elem_index_dirty & BM_EDGE) {
+					BMIter iter;
+					BMElem *ele;
+
+					int index;
+					BM_ITER_MESH_INDEX (ele, &iter, bm, BM_EDGES_OF_MESH, index) {
+						BM_elem_index_set(ele, index); /* set_ok */
+					}
+					BLI_assert(index == bm->totedge);
+				}
+				else {
+					// printf("%s: skipping edge index calc!\n", __func__);
+				}
+			}
+		}
+
+#pragma omp section
+		{
+			if (hflag & BM_FACE) {
+				if (bm->elem_index_dirty & BM_FACE) {
+					BMIter iter;
+					BMElem *ele;
+
+					int index;
+					BM_ITER_MESH_INDEX (ele, &iter, bm, BM_FACES_OF_MESH, index) {
+						BM_elem_index_set(ele, index); /* set_ok */
+					}
+					BLI_assert(index == bm->totface);
+				}
+				else {
+					// printf("%s: skipping face index calc!\n", __func__);
+				}
+			}
 		}
 	}
 
-	if (hflag & BM_EDGE) {
-		if (bm->elem_index_dirty & BM_EDGE) {
-			int index = 0;
-			BM_ITER_MESH (ele, &iter, bm, BM_EDGES_OF_MESH) {
-				BM_elem_index_set(ele, index); /* set_ok */
-				index++;
-			}
-			bm->elem_index_dirty &= ~BM_EDGE;
-			BLI_assert(index == bm->totedge);
-		}
-		else {
-			// printf("%s: skipping edge index calc!\n", __func__);
-		}
-	}
-
-	if (hflag & BM_FACE) {
-		if (bm->elem_index_dirty & BM_FACE) {
-			int index = 0;
-			BM_ITER_MESH (ele, &iter, bm, BM_FACES_OF_MESH) {
-				BM_elem_index_set(ele, index); /* set_ok */
-				index++;
-			}
-			bm->elem_index_dirty &= ~BM_FACE;
-			BLI_assert(index == bm->totface);
-		}
-		else {
-			// printf("%s: skipping face index calc!\n", __func__);
-		}
-	}
+	bm->elem_index_dirty &= ~hflag;
 }
 
 
@@ -523,12 +553,12 @@ void BM_mesh_elem_index_validate(BMesh *bm, const char *location, const char *fu
 	BMIter iter;
 	BMElem *ele;
 	int i;
-	int is_any_error = 0;
+	bool is_any_error = 0;
 
 	for (i = 0; i < 3; i++) {
-		const int is_dirty = (flag_types[i] & bm->elem_index_dirty);
+		const bool is_dirty = (flag_types[i] & bm->elem_index_dirty);
 		int index = 0;
-		int is_error = FALSE;
+		bool is_error = false;
 		int err_val = 0;
 		int err_idx = 0;
 
@@ -537,7 +567,7 @@ void BM_mesh_elem_index_validate(BMesh *bm, const char *location, const char *fu
 				if (BM_elem_index_get(ele) != index) {
 					err_val = BM_elem_index_get(ele);
 					err_idx = index;
-					is_error = TRUE;
+					is_error = true;
 				}
 			}
 
@@ -545,13 +575,13 @@ void BM_mesh_elem_index_validate(BMesh *bm, const char *location, const char *fu
 			index++;
 		}
 
-		if ((is_error == TRUE) && (is_dirty == FALSE)) {
-			is_any_error = TRUE;
+		if ((is_error == true) && (is_dirty == false)) {
+			is_any_error = true;
 			fprintf(stderr,
 			        "Invalid Index: at %s, %s, %s[%d] invalid index %d, '%s', '%s'\n",
 			        location, func, type_names[i], err_idx, err_val, msg_a, msg_b);
 		}
-		else if ((is_error == FALSE) && (is_dirty == TRUE)) {
+		else if ((is_error == false) && (is_dirty == true)) {
 
 #if 0       /* mostly annoying */
 
@@ -620,7 +650,7 @@ void BM_mesh_remap(BMesh *bm, int *vert_idx, int *edge_idx, int *face_idx)
 		int *new_idx = NULL;
 
 		/* Init the old-to-new vert pointers mapping */
-		vptr_map = BLI_ghash_new(BLI_ghashutil_ptrhash, BLI_ghashutil_ptrcmp, "BM_mesh_remap vert pointers mapping");
+		vptr_map = BLI_ghash_ptr_new("BM_mesh_remap vert pointers mapping");
 
 		/* Make a copy of all vertices. */
 		verts_pool = MEM_callocN(sizeof(BMVert *) * totvert, "BM_mesh_remap verts pool");
@@ -654,7 +684,7 @@ void BM_mesh_remap(BMesh *bm, int *vert_idx, int *edge_idx, int *face_idx)
 		int *new_idx = NULL;
 
 		/* Init the old-to-new vert pointers mapping */
-		eptr_map = BLI_ghash_new(BLI_ghashutil_ptrhash, BLI_ghashutil_ptrcmp, "BM_mesh_remap edge pointers mapping");
+		eptr_map = BLI_ghash_ptr_new("BM_mesh_remap edge pointers mapping");
 
 		/* Make a copy of all vertices. */
 		edges_pool = MEM_callocN(sizeof(BMEdge *) * totedge, "BM_mesh_remap edges pool");
@@ -672,8 +702,8 @@ void BM_mesh_remap(BMesh *bm, int *vert_idx, int *edge_idx, int *face_idx)
 			BMEdge *new_edp = edges_pool[*new_idx];
 			*new_edp = *ed;
 			BLI_ghash_insert(eptr_map, (void *)*edp, (void *)new_edp);
+/*			printf("mapping edge from %d to %d (%p/%p to %p)\n", i, *new_idx, *edp, edges_pool[i], new_edp);*/
 		}
-
 		bm->elem_index_dirty |= BM_EDGE;
 
 		MEM_freeN(edges_pool);
@@ -687,7 +717,7 @@ void BM_mesh_remap(BMesh *bm, int *vert_idx, int *edge_idx, int *face_idx)
 		int *new_idx = NULL;
 
 		/* Init the old-to-new vert pointers mapping */
-		fptr_map = BLI_ghash_new(BLI_ghashutil_ptrhash, BLI_ghashutil_ptrcmp, "BM_mesh_remap face pointers mapping");
+		fptr_map = BLI_ghash_ptr_new("BM_mesh_remap face pointers mapping");
 
 		/* Make a copy of all vertices. */
 		faces_pool = MEM_callocN(sizeof(BMFace *) * totface, "BM_mesh_remap faces pool");
@@ -717,18 +747,35 @@ void BM_mesh_remap(BMesh *bm, int *vert_idx, int *edge_idx, int *face_idx)
 	/* Verts' pointers, only edge pointers... */
 	if (eptr_map) {
 		BM_ITER_MESH (ve, &iter, bm, BM_VERTS_OF_MESH) {
-/*			printf("Vert e: %p -> %p\n", ve->e, BLI_ghash_lookup(eptr_map, (const void*)ve->e));*/
+/*			printf("Vert e: %p -> %p\n", ve->e, BLI_ghash_lookup(eptr_map, (const void *)ve->e));*/
 			ve->e = BLI_ghash_lookup(eptr_map, (const void *)ve->e);
 		}
 	}
 
-	/* Edges' pointers, only vert pointers (as we don't mess with loops!)... */
-	if (vptr_map) {
+	/* Edges' pointers, only vert pointers (as we don't mess with loops!), and - ack! - edge pointers,
+	 * as we have to handle disklinks... */
+	if (vptr_map || eptr_map) {
 		BM_ITER_MESH (ed, &iter, bm, BM_EDGES_OF_MESH) {
-/*			printf("Edge v1: %p -> %p\n", ed->v1, BLI_ghash_lookup(vptr_map, (const void*)ed->v1));*/
-/*			printf("Edge v2: %p -> %p\n", ed->v2, BLI_ghash_lookup(vptr_map, (const void*)ed->v2));*/
-			ed->v1 = BLI_ghash_lookup(vptr_map, (const void *)ed->v1);
-			ed->v2 = BLI_ghash_lookup(vptr_map, (const void *)ed->v2);
+			if (vptr_map) {
+/*				printf("Edge v1: %p -> %p\n", ed->v1, BLI_ghash_lookup(vptr_map, (const void *)ed->v1));*/
+/*				printf("Edge v2: %p -> %p\n", ed->v2, BLI_ghash_lookup(vptr_map, (const void* )ed->v2));*/
+				ed->v1 = BLI_ghash_lookup(vptr_map, (const void *)ed->v1);
+				ed->v2 = BLI_ghash_lookup(vptr_map, (const void *)ed->v2);
+			}
+			if (eptr_map) {
+/*				printf("Edge v1_disk_link prev: %p -> %p\n", ed->v1_disk_link.prev,*/
+/*				       BLI_ghash_lookup(eptr_map, (const void *)ed->v1_disk_link.prev));*/
+/*				printf("Edge v1_disk_link next: %p -> %p\n", ed->v1_disk_link.next,*/
+/*				       BLI_ghash_lookup(eptr_map, (const void *)ed->v1_disk_link.next));*/
+/*				printf("Edge v2_disk_link prev: %p -> %p\n", ed->v2_disk_link.prev,*/
+/*				       BLI_ghash_lookup(eptr_map, (const void *)ed->v2_disk_link.prev));*/
+/*				printf("Edge v2_disk_link next: %p -> %p\n", ed->v2_disk_link.next,*/
+/*				       BLI_ghash_lookup(eptr_map, (const void *)ed->v2_disk_link.next));*/
+				ed->v1_disk_link.prev = BLI_ghash_lookup(eptr_map, (const void *)ed->v1_disk_link.prev);
+				ed->v1_disk_link.next = BLI_ghash_lookup(eptr_map, (const void *)ed->v1_disk_link.next);
+				ed->v2_disk_link.prev = BLI_ghash_lookup(eptr_map, (const void *)ed->v2_disk_link.prev);
+				ed->v2_disk_link.next = BLI_ghash_lookup(eptr_map, (const void *)ed->v2_disk_link.next);
+			}
 		}
 	}
 
@@ -736,15 +783,15 @@ void BM_mesh_remap(BMesh *bm, int *vert_idx, int *edge_idx, int *face_idx)
 	BM_ITER_MESH (fa, &iter, bm, BM_FACES_OF_MESH) {
 		BM_ITER_ELEM (lo, &iterl, fa, BM_LOOPS_OF_FACE) {
 			if (vptr_map) {
-/*				printf("Loop v: %p -> %p\n", lo->v, BLI_ghash_lookup(vptr_map, (const void*)lo->v));*/
+/*				printf("Loop v: %p -> %p\n", lo->v, BLI_ghash_lookup(vptr_map, (const void *)lo->v));*/
 				lo->v = BLI_ghash_lookup(vptr_map, (const void *)lo->v);
 			}
 			if (eptr_map) {
-/*				printf("Loop e: %p -> %p\n", lo->e, BLI_ghash_lookup(eptr_map, (const void*)lo->e));*/
+/*				printf("Loop e: %p -> %p\n", lo->e, BLI_ghash_lookup(eptr_map, (const void *)lo->e));*/
 				lo->e = BLI_ghash_lookup(eptr_map, (const void *)lo->e);
 			}
 			if (fptr_map) {
-/*				printf("Loop f: %p -> %p\n", lo->f, BLI_ghash_lookup(fptr_map, (const void*)lo->f));*/
+/*				printf("Loop f: %p -> %p\n", lo->f, BLI_ghash_lookup(fptr_map, (const void *)lo->f));*/
 				lo->f = BLI_ghash_lookup(fptr_map, (const void *)lo->f);
 			}
 		}
