@@ -45,54 +45,55 @@
  * Turns the face region surrounding a manifold vertex into a single polygon.
  *
  * \par Example:
- *
+ * <pre>
  *              +---------+             +---------+
  *              |  \   /  |             |         |
  *     Before:  |    v    |      After: |         |
  *              |  /   \  |             |         |
  *              +---------+             +---------+
- *
+ * </pre>
  *
  * This function can also collapse edges too
  * in cases when it cant merge into faces.
  *
  * \par Example:
- *
+ * <pre>
  *     Before:  +----v----+      After: +---------+
+ * </pre>
  *
  * \note dissolves vert, in more situations then BM_disk_dissolve
  * (e.g. if the vert is part of a wire edge, etc).
  */
-int BM_vert_dissolve(BMesh *bm, BMVert *v)
+bool BM_vert_dissolve(BMesh *bm, BMVert *v)
 {
 	const int len = BM_vert_edge_count(v);
 	
 	if (len == 1) {
 		BM_vert_kill(bm, v); /* will kill edges too */
-		return TRUE;
+		return true;
 	}
 	else if (!BM_vert_is_manifold(v)) {
 		if (!v->e) {
 			BM_vert_kill(bm, v);
-			return TRUE;
+			return true;
 		}
 		else if (!v->e->l) {
 			if (len == 2) {
-				return (BM_vert_collapse_edge(bm, v->e, v, TRUE) != NULL);
+				return (BM_vert_collapse_edge(bm, v->e, v, true) != NULL);
 			}
 			else {
 				/* used to kill the vertex here, but it may be connected to faces.
 				 * so better do nothing */
-				return FALSE;
+				return false;
 			}
 		}
 		else {
-			return FALSE;
+			return false;
 		}
 	}
 	else if (len == 2 && BM_vert_face_count(v) == 1) {
 		/* boundary vertex on a face */
-		return (BM_vert_collapse_edge(bm, v->e, v, TRUE) != NULL);
+		return (BM_vert_collapse_edge(bm, v->e, v, true) != NULL);
 	}
 	else {
 		return BM_disk_dissolve(bm, v);
@@ -102,14 +103,14 @@ int BM_vert_dissolve(BMesh *bm, BMVert *v)
 /**
  * dissolves all faces around a vert, and removes it.
  */
-int BM_disk_dissolve(BMesh *bm, BMVert *v)
+bool BM_disk_dissolve(BMesh *bm, BMVert *v)
 {
 	BMFace *f, *f2;
 	BMEdge *e, *keepedge = NULL, *baseedge = NULL;
 	int len = 0;
 
 	if (!BM_vert_is_manifold(v)) {
-		return FALSE;
+		return false;
 	}
 	
 	if (v->e) {
@@ -117,7 +118,7 @@ int BM_disk_dissolve(BMesh *bm, BMVert *v)
 		e = v->e;
 		do {
 			e = bmesh_disk_edge_next(e, v);
-			if (!(BM_edge_share_face_count(e, v->e))) {
+			if (!(BM_edge_share_face_check(e, v->e))) {
 				keepedge = e;
 				baseedge = v->e;
 				break;
@@ -129,84 +130,94 @@ int BM_disk_dissolve(BMesh *bm, BMVert *v)
 	/* this code for handling 2 and 3-valence verts
 	 * may be totally bad */
 	if (keepedge == NULL && len == 3) {
+#if 0
 		/* handle specific case for three-valence.  solve it by
 		 * increasing valence to four.  this may be hackish. .  */
 		BMLoop *loop = e->l;
 		if (loop->v == v) loop = loop->next;
-		if (!BM_face_split(bm, loop->f, v, loop->v, NULL, NULL, FALSE))
-			return FALSE;
+		if (!BM_face_split(bm, loop->f, v, loop->v, NULL, NULL, false))
+			return false;
 
 		if (!BM_disk_dissolve(bm, v)) {
-			return FALSE;
+			return false;
 		}
-		return TRUE;
+#else
+		if (UNLIKELY(!BM_faces_join_pair(bm, e->l->f, e->l->radial_next->f, e, true))) {
+			return false;
+		}
+		else if (UNLIKELY(!BM_vert_collapse_faces(bm, v->e, v, 1.0, false, true))) {
+			return false;
+		}
+#endif
+		return true;
 	}
 	else if (keepedge == NULL && len == 2) {
-		/* collapse the verte */
-		e = BM_vert_collapse_faces(bm, v->e, v, 1.0, TRUE, TRUE);
+		/* collapse the vertex */
+		e = BM_vert_collapse_faces(bm, v->e, v, 1.0, true, true);
 
 		if (!e) {
-			return FALSE;
+			return false;
 		}
 
-		/* handle two-valenc */
+		/* handle two-valence */
 		f = e->l->f;
 		f2 = e->l->radial_next->f;
 
-		if (f != f2 && !BM_faces_join_pair(bm, f, f2, e, TRUE)) {
-			return FALSE;
+		if (f != f2 && !BM_faces_join_pair(bm, f, f2, e, true)) {
+			return false;
 		}
 
-		return TRUE;
+		return true;
 	}
 
 	if (keepedge) {
-		int done = 0;
+		bool done = false;
 
 		while (!done) {
-			done = 1;
+			done = true;
 			e = v->e;
 			do {
 				f = NULL;
 				len = bmesh_radial_length(e->l);
 				if (len == 2 && (e != baseedge) && (e != keepedge)) {
-					f = BM_faces_join_pair(bm, e->l->f, e->l->radial_next->f, e, TRUE);
+					f = BM_faces_join_pair(bm, e->l->f, e->l->radial_next->f, e, true);
 					/* return if couldn't join faces in manifold
 					 * conditions */
-					//!disabled for testing why bad things happen
+					/* !disabled for testing why bad things happen */
 					if (!f) {
-						return FALSE;
+						return false;
 					}
 				}
 
 				if (f) {
-					done = 0;
+					done = false;
 					break;
 				}
 				e = bmesh_disk_edge_next(e, v);
 			} while (e != v->e);
 		}
 
-		/* collapse the verte */
-		e = BM_vert_collapse_faces(bm, baseedge, v, 1.0, TRUE, TRUE);
+		/* collapse the vertex */
+		/* note, the baseedge can be a boundary of manifold, use this as join_faces arg */
+		e = BM_vert_collapse_faces(bm, baseedge, v, 1.0, !BM_edge_is_boundary(baseedge), true);
 
 		if (!e) {
-			return FALSE;
+			return false;
 		}
 		
-		/* get remaining two face */
+		/* get remaining two faces */
 		f = e->l->f;
 		f2 = e->l->radial_next->f;
 
 		if (f != f2) {
-			/* join two remaining face */
-			if (!BM_faces_join_pair(bm, f, f2, e, TRUE)) {
-				return FALSE;
+			/* join two remaining faces */
+			if (!BM_faces_join_pair(bm, f, f2, e, true)) {
+				return false;
 			}
 		}
 	}
 
-	return TRUE;
+	return true;
 }
 
 /**
@@ -224,7 +235,7 @@ int BM_disk_dissolve(BMesh *bm, BMVert *v)
  *
  * \return pointer to the combined face
  */
-BMFace *BM_faces_join_pair(BMesh *bm, BMFace *f1, BMFace *f2, BMEdge *e, const short do_del)
+BMFace *BM_faces_join_pair(BMesh *bm, BMFace *f1, BMFace *f2, BMEdge *e, const bool do_del)
 {
 	BMLoop *l1, *l2;
 	BMEdge *jed = NULL;
@@ -291,7 +302,7 @@ BMEdge *BM_verts_connect(BMesh *bm, BMVert *v1, BMVert *v2, BMFace **r_f)
 			if (v_iter == v2) {
 				BMLoop *nl;
 
-				f_iter = BM_face_split(bm, f_iter, v1, v2, &nl, NULL, FALSE);
+				f_iter = BM_face_split(bm, f_iter, v1, v2, &nl, NULL, false);
 
 				if (r_f) {
 					*r_f = f_iter;
@@ -325,22 +336,22 @@ BMEdge *BM_verts_connect(BMesh *bm, BMVert *v1, BMVert *v2, BMFace **r_f)
  * other side). NULL if the split fails.
  */
 BMFace *BM_face_split(BMesh *bm, BMFace *f, BMVert *v1, BMVert *v2, BMLoop **r_l,
-                      BMEdge *example, const short nodouble)
+                      BMEdge *example, const bool no_double)
 {
-	const int has_mdisp = CustomData_has_layer(&bm->ldata, CD_MDISPS);
+	const bool has_mdisp = CustomData_has_layer(&bm->ldata, CD_MDISPS);
 	BMFace *nf, *of;
 
 	BLI_assert(v1 != v2);
 
-	/* do we have a multires layer */
+	/* do we have a multires layer? */
 	if (has_mdisp) {
-		of = BM_face_copy(bm, f, FALSE, FALSE);
+		of = BM_face_copy(bm, f, false, false);
 	}
 	
 #ifdef USE_BMESH_HOLES
-	nf = bmesh_sfme(bm, f, v1, v2, r_l, NULL, example, nodouble);
+	nf = bmesh_sfme(bm, f, v1, v2, r_l, NULL, example, no_double);
 #else
-	nf = bmesh_sfme(bm, f, v1, v2, r_l, example, nodouble);
+	nf = bmesh_sfme(bm, f, v1, v2, r_l, example, no_double);
 #endif
 	
 	if (nf) {
@@ -354,12 +365,12 @@ BMFace *BM_face_split(BMesh *bm, BMFace *f, BMVert *v1, BMVert *v2, BMLoop **r_l
 
 			l_iter = l_first = BM_FACE_FIRST_LOOP(f);
 			do {
-				BM_loop_interp_from_face(bm, l_iter, of, FALSE, TRUE);
+				BM_loop_interp_multires(bm, l_iter, of);
 			} while ((l_iter = l_iter->next) != l_first);
 
 			l_iter = l_first = BM_FACE_FIRST_LOOP(nf);
 			do {
-				BM_loop_interp_from_face(bm, l_iter, of, FALSE, TRUE);
+				BM_loop_interp_multires(bm, l_iter, of);
 			} while ((l_iter = l_iter->next) != l_first);
 
 			BM_face_kill(bm, of);
@@ -383,7 +394,7 @@ BMFace *BM_face_split(BMesh *bm, BMFace *f, BMVert *v1, BMVert *v2, BMLoop **r_l
  * \param bm The bmesh
  * \param f the original face
  * \param v1, v2 vertices which define the split edge, must be different
- * \param co Array of coordinates for intermediate points
+ * \param cos Array of coordinates for intermediate points
  * \param n Length of \a cos (must be > 0)
  * \param r_l pointer which will receive the BMLoop for the first split edge (from \a v1) in the new face
  * \param example Edge used for attributes of splitting edge, if non-NULL
@@ -403,15 +414,15 @@ BMFace *BM_face_split_n(BMesh *bm, BMFace *f, BMVert *v1, BMVert *v2, float cos[
 
 	BLI_assert(v1 != v2);
 
-	of = BM_face_copy(bm, f, TRUE, TRUE);
+	of = BM_face_copy(bm, f, true, true);
 
 	if (!r_l)
 		r_l = &l_dummy;
 	
 #ifdef USE_BMESH_HOLES
-	nf = bmesh_sfme(bm, f, v1, v2, r_l, NULL, example, FALSE);
+	nf = bmesh_sfme(bm, f, v1, v2, r_l, NULL, example, false);
 #else
-	nf = bmesh_sfme(bm, f, v1, v2, r_l, example, FALSE);
+	nf = bmesh_sfme(bm, f, v1, v2, r_l, example, false);
 #endif
 	/* bmesh_sfme returns in r_l a Loop for nf going from v1 to v2.
 	 * The radial_next is for f and goes from v2 to v1  */
@@ -427,14 +438,14 @@ BMFace *BM_face_split_n(BMesh *bm, BMFace *f, BMVert *v1, BMVert *v2, float cos[
 			/* bmesh_semv returns in newe the edge going from newv to tv */
 			copy_v3_v3(newv->co, cos[i]);
 
-			/* interpolate the loop data for the loops with v==newv, using orig face */
+			/* interpolate the loop data for the loops with (v == newv), using orig face */
 			for (j = 0; j < 2; j++) {
 				BMEdge *e_iter = (j == 0) ? e : newe;
 				BMLoop *l_iter = e_iter->l;
 				do {
 					if (l_iter->v == newv) {
 						/* this interpolates both loop and vertex data */
-						BM_loop_interp_from_face(bm, l_iter, of, TRUE, TRUE);
+						BM_loop_interp_from_face(bm, l_iter, of, true, true);
 					}
 				} while ((l_iter = l_iter->radial_next) != e_iter->l);
 			}
@@ -460,9 +471,6 @@ BMFace *BM_face_split_n(BMesh *bm, BMFace *f, BMVert *v1, BMVert *v2, float cos[
  * both collapse a vertex and return a new edge.
  * Except this takes a factor and merges custom data.
  *
- *  BMESH_TODO:
- *    Insert error checking for KV valance.
- *
  * \param bm The bmesh
  * \param ke The edge to collapse
  * \param kv The vertex  to collapse into the edge
@@ -474,7 +482,7 @@ BMFace *BM_face_split_n(BMesh *bm, BMFace *f, BMVert *v1, BMVert *v2, float cos[
  * \returns The New Edge
  */
 BMEdge *BM_vert_collapse_faces(BMesh *bm, BMEdge *ke, BMVert *kv, float fac,
-                               const short join_faces, const short kill_degenerate_faces)
+                               const bool join_faces, const bool kill_degenerate_faces)
 {
 	BMEdge *ne = NULL;
 	BMVert *tv = bmesh_edge_other_vert_get(ke, kv);
@@ -526,10 +534,10 @@ BMEdge *BM_vert_collapse_faces(BMesh *bm, BMEdge *ke, BMVert *kv, float fac,
 		}
 
 		if (BLI_array_count(faces) >= 2) {
-			BMFace *f2 = BM_faces_join(bm, faces, BLI_array_count(faces), TRUE);
+			BMFace *f2 = BM_faces_join(bm, faces, BLI_array_count(faces), true);
 			if (f2) {
 				BMLoop *nl = NULL;
-				if (BM_face_split(bm, f2, tv, tv2, &nl, NULL, FALSE)) {
+				if (BM_face_split(bm, f2, tv, tv2, &nl, NULL, false)) {
 					ne = nl->e;
 				}
 			}
@@ -541,7 +549,7 @@ BMEdge *BM_vert_collapse_faces(BMesh *bm, BMEdge *ke, BMVert *kv, float fac,
 		/* single face or no faces */
 		/* same as BM_vert_collapse_edge() however we already
 		 * have vars to perform this operation so don't call. */
-		ne = bmesh_jekv(bm, ke, kv, TRUE);
+		ne = bmesh_jekv(bm, ke, kv, true);
 		/* ne = BM_edge_exists(tv, tv2); */ /* same as return above */
 
 		if (ne && kill_degenerate_faces) {
@@ -557,7 +565,7 @@ BMEdge *BM_vert_collapse_faces(BMesh *bm, BMEdge *ke, BMVert *kv, float fac,
 				/* cant kill data we loop on, build a list and remove those */
 				BLI_array_empty(bad_faces);
 				BM_ITER_ELEM (f, &fiter, verts[i], BM_FACES_OF_VERT) {
-					if (f->len < 3) {
+					if (UNLIKELY(f->len < 3)) {
 						BLI_array_append(bad_faces, f);
 					}
 				}
@@ -581,7 +589,7 @@ BMEdge *BM_vert_collapse_faces(BMesh *bm, BMEdge *ke, BMVert *kv, float fac,
  * \return The New Edge
  */
 BMEdge *BM_vert_collapse_edge(BMesh *bm, BMEdge *ke, BMVert *kv,
-                              const short kill_degenerate_faces)
+                              const bool kill_degenerate_faces)
 {
 	/* nice example implementation but we want loops to have their customdata
 	 * accounted for */
@@ -610,7 +618,7 @@ BMEdge *BM_vert_collapse_edge(BMesh *bm, BMEdge *ke, BMVert *kv,
 #else
 	/* with these args faces are never joined, same as above
 	 * but account for loop customdata */
-	return BM_vert_collapse_faces(bm, ke, kv, 1.0f, FALSE, kill_degenerate_faces);
+	return BM_vert_collapse_faces(bm, ke, kv, 1.0f, false, kill_degenerate_faces);
 #endif
 }
 
@@ -633,15 +641,14 @@ BMVert *BM_edge_split(BMesh *bm, BMEdge *e, BMVert *v, BMEdge **r_e, float perce
 	BMFace **oldfaces = NULL;
 	BMEdge *e_dummy;
 	BLI_array_staticdeclare(oldfaces, 32);
-	SmallHash hash;
-	const int do_mdisp = (e->l && CustomData_has_layer(&bm->ldata, CD_MDISPS));
+	const bool do_mdisp = (e->l && CustomData_has_layer(&bm->ldata, CD_MDISPS));
 
 	/* we need this for handling multi-res */
 	if (!r_e) {
 		r_e = &e_dummy;
 	}
 
-	/* do we have a multi-res layer */
+	/* do we have a multi-res layer? */
 	if (do_mdisp) {
 		BMLoop *l;
 		int i;
@@ -652,12 +659,11 @@ BMVert *BM_edge_split(BMesh *bm, BMEdge *e, BMVert *v, BMEdge **r_e, float perce
 			l = l->radial_next;
 		} while (l != e->l);
 		
-		/* create a hash so we can differentiate oldfaces from new face */
-		BLI_smallhash_init(&hash);
-		
+		/* flag existing faces so we can differentiate oldfaces from new faces */
 		for (i = 0; i < BLI_array_count(oldfaces); i++) {
-			oldfaces[i] = BM_face_copy(bm, oldfaces[i], TRUE, TRUE);
-			BLI_smallhash_insert(&hash, (intptr_t)oldfaces[i], NULL);
+			BM_ELEM_API_FLAG_ENABLE(oldfaces[i], _FLAG_OVERLAP);
+			oldfaces[i] = BM_face_copy(bm, oldfaces[i], true, true);
+			BM_ELEM_API_FLAG_DISABLE(oldfaces[i], _FLAG_OVERLAP);
 		}
 	}
 
@@ -681,7 +687,7 @@ BMVert *BM_edge_split(BMesh *bm, BMEdge *e, BMVert *v, BMEdge **r_e, float perce
 	if (do_mdisp) {
 		int i, j;
 
-		/* interpolate new/changed loop data from copied old face */
+		/* interpolate new/changed loop data from copied old faces */
 		for (j = 0; j < 2; j++) {
 			for (i = 0; i < BLI_array_count(oldfaces); i++) {
 				BMEdge *e1 = j ? *r_e : e;
@@ -695,7 +701,8 @@ BMVert *BM_edge_split(BMesh *bm, BMEdge *e, BMVert *v, BMEdge **r_e, float perce
 				}
 				
 				do {
-					if (!BLI_smallhash_haskey(&hash, (intptr_t)l->f)) {
+					/* check this is an old face */
+					if (BM_ELEM_API_FLAG_TEST(l->f, _FLAG_OVERLAP)) {
 						BMLoop *l2_first;
 
 						l2 = l2_first = BM_FACE_FIRST_LOOP(l->f);
@@ -708,7 +715,7 @@ BMVert *BM_edge_split(BMesh *bm, BMEdge *e, BMVert *v, BMEdge **r_e, float perce
 			}
 		}
 		
-		/* destroy the old face */
+		/* destroy the old faces */
 		for (i = 0; i < BLI_array_count(oldfaces); i++) {
 			BM_face_verts_kill(bm, oldfaces[i]);
 		}
@@ -733,7 +740,6 @@ BMVert *BM_edge_split(BMesh *bm, BMEdge *e, BMVert *v, BMEdge **r_e, float perce
 #endif
 		
 		BLI_array_free(oldfaces);
-		BLI_smallhash_release(&hash);
 	}
 
 	return nv;
@@ -755,30 +761,32 @@ BMVert  *BM_edge_split_n(BMesh *bm, BMEdge *e, int numcuts)
 	return nv;
 }
 
+#if 0
 /**
  * Checks if a face is valid in the data structure
  */
-int BM_face_validate(BMFace *face, FILE *err)
+bool BM_face_validate(BMFace *face, FILE *err)
 {
 	BMIter iter;
 	BLI_array_declare(verts);
 	BMVert **verts = NULL;
 	BMLoop *l;
-	int ret = 1, i, j;
+	int i, j;
+	bool ret = true;
 	
 	if (face->len == 2) {
 		fprintf(err, "warning: found two-edged face. face ptr: %p\n", face);
 		fflush(err);
 	}
 
-	BLI_array_growitems(verts, face->len);
+	BLI_array_grow_items(verts, face->len);
 	BM_ITER_ELEM_INDEX (l, &iter, face, BM_LOOPS_OF_FACE, i) {
 		verts[i] = l->v;
 		if (l->e->v1 == l->e->v2) {
 			fprintf(err, "Found bmesh edge with identical verts!\n");
 			fprintf(err, "  edge ptr: %p, vert: %p\n",  l->e, l->e->v1);
 			fflush(err);
-			ret = 0;
+			ret = false;
 		}
 	}
 
@@ -792,7 +800,7 @@ int BM_face_validate(BMFace *face, FILE *err)
 				fprintf(err, "Found duplicate verts in bmesh face!\n");
 				fprintf(err, "  face ptr: %p, vert: %p\n", face, verts[i]);
 				fflush(err);
-				ret = 0;
+				ret = false;
 			}
 		}
 	}
@@ -800,7 +808,7 @@ int BM_face_validate(BMFace *face, FILE *err)
 	BLI_array_free(verts);
 	return ret;
 }
-
+#endif
 
 /**
  * Calculate the 2 loops which _would_ make up the newly rotated Edge
@@ -816,14 +824,14 @@ int BM_face_validate(BMFace *face, FILE *err)
  *
  * \note #BM_edge_rotate_check must have already run.
  */
-void BM_edge_calc_rotate(BMEdge *e, int ccw,
+void BM_edge_calc_rotate(BMEdge *e, const bool ccw,
                          BMLoop **r_l1, BMLoop **r_l2)
 {
 	BMVert *v1, *v2;
 	BMFace *fa, *fb;
 
 	/* this should have already run */
-	BLI_assert(BM_edge_rotate_check(e) == TRUE);
+	BLI_assert(BM_edge_rotate_check(e) == true);
 
 	/* we know this will work */
 	BM_edge_face_pair(e, &fa, &fb);
@@ -849,7 +857,7 @@ void BM_edge_calc_rotate(BMEdge *e, int ccw,
  * Quick check to see if we could rotate the edge,
  * use this to avoid calling exceptions on common cases.
  */
-int BM_edge_rotate_check(BMEdge *e)
+bool BM_edge_rotate_check(BMEdge *e)
 {
 	BMFace *fa, *fb;
 	if (BM_edge_face_pair(e, &fa, &fb)) {
@@ -862,7 +870,7 @@ int BM_edge_rotate_check(BMEdge *e)
 		 * (ie - the next edge doesn't share the same faces).
 		 * since we can't rotate usefully in this case. */
 		if (la->v == lb->v) {
-			return FALSE;
+			return false;
 		}
 
 		/* mirror of the check above but in the opposite direction */
@@ -870,13 +878,13 @@ int BM_edge_rotate_check(BMEdge *e)
 		lb = BM_face_other_vert_loop(fb, e->v1, e->v2);
 
 		if (la->v == lb->v) {
-			return FALSE;
+			return false;
 		}
 
-		return TRUE;
+		return true;
 	}
 	else {
-		return FALSE;
+		return false;
 	}
 }
 
@@ -887,10 +895,11 @@ int BM_edge_rotate_check(BMEdge *e)
  * 1) does the newly forms edge form a flipped face (compare with previous cross product)
  * 2) does the newly formed edge cause a zero area corner (or close enough to be almost zero)
  *
+ * \param e The edge to test rotation.
  * \param l1,l2 are the loops of the proposed verts to rotate too and should
  * be the result of calling #BM_edge_calc_rotate
  */
-int BM_edge_rotate_check_degenerate(BMEdge *e, BMLoop *l1, BMLoop *l2)
+bool BM_edge_rotate_check_degenerate(BMEdge *e, BMLoop *l1, BMLoop *l2)
 {
 	/* note: for these vars 'old' just means initial edge state. */
 
@@ -917,7 +926,7 @@ int BM_edge_rotate_check_degenerate(BMEdge *e, BMLoop *l1, BMLoop *l2)
 	BMVert *v1_alt, *v2_alt;
 
 	/* this should have already run */
-	BLI_assert(BM_edge_rotate_check(e) == TRUE);
+	BLI_assert(BM_edge_rotate_check(e) == true);
 
 	BM_edge_ordered_verts(e, &v1_old, &v2_old);
 
@@ -958,12 +967,12 @@ int BM_edge_rotate_check_degenerate(BMEdge *e, BMLoop *l1, BMLoop *l2)
 	cross_v3_v3v3(cross_old, ed_dir_old, ed_dir_v1_old);
 	cross_v3_v3v3(cross_new, ed_dir_new, ed_dir_v1_new);
 	if (dot_v3v3(cross_old, cross_new) < 0.0f) { /* does this flip? */
-		return FALSE;
+		return false;
 	}
 	cross_v3_v3v3(cross_old, ed_dir_old, ed_dir_v2_old);
 	cross_v3_v3v3(cross_new, ed_dir_new, ed_dir_v2_new);
 	if (dot_v3v3(cross_old, cross_new) < 0.0f) { /* does this flip? */
-		return FALSE;
+		return false;
 	}
 
 	negate_v3_v3(ed_dir_new_flip, ed_dir_new);
@@ -972,14 +981,14 @@ int BM_edge_rotate_check_degenerate(BMEdge *e, BMLoop *l1, BMLoop *l2)
 	if ((dot_v3v3(ed_dir_new,      ed_dir_v1_new) > 0.999f) ||
 	    (dot_v3v3(ed_dir_new_flip, ed_dir_v2_new) > 0.999f))
 	{
-		return FALSE;
+		return false;
 	}
 
-	return TRUE;
+	return true;
 }
 
-int BM_edge_rotate_check_beauty(BMEdge *e,
-                                BMLoop *l1, BMLoop *l2)
+bool BM_edge_rotate_check_beauty(BMEdge *e,
+                                 BMLoop *l1, BMLoop *l2)
 {
 	/* Stupid check for now:
 	 * Could compare angles of surrounding edges
@@ -1002,7 +1011,7 @@ int BM_edge_rotate_check_beauty(BMEdge *e,
  *
  * \see header definition for \a check_flag enum.
  */
-BMEdge *BM_edge_rotate(BMesh *bm, BMEdge *e, const short ccw, const short check_flag)
+BMEdge *BM_edge_rotate(BMesh *bm, BMEdge *e, const bool ccw, const short check_flag)
 {
 	BMVert *v1, *v2;
 	BMLoop *l1, *l2;
@@ -1053,13 +1062,13 @@ BMEdge *BM_edge_rotate(BMesh *bm, BMEdge *e, const short ccw, const short check_
 
 	/* first create the new edge, this is so we can copy the customdata from the old one
 	 * if splice if disabled, always add in a new edge even if theres one there. */
-	e_new = BM_edge_create(bm, v1, v2, e, (check_flag & BM_EDGEROT_CHECK_SPLICE)!=0);
+	e_new = BM_edge_create(bm, v1, v2, e, (check_flag & BM_EDGEROT_CHECK_SPLICE) != 0);
 
 	f_hflag_prev_1 = l1->f->head.hflag;
 	f_hflag_prev_2 = l2->f->head.hflag;
 
 	/* don't delete the edge, manually remove the egde after so we can copy its attributes */
-	f = BM_faces_join_pair(bm, l1->f, l2->f, NULL, TRUE);
+	f = BM_faces_join_pair(bm, l1->f, l2->f, NULL, true);
 
 	if (f == NULL) {
 		return NULL;
@@ -1068,7 +1077,7 @@ BMEdge *BM_edge_rotate(BMesh *bm, BMEdge *e, const short ccw, const short check_
 	/* note, this assumes joining the faces _didnt_ also remove the verts.
 	 * the #BM_edge_rotate_check will ensure this, but its possibly corrupt state or future edits
 	 * break this */
-	if (!BM_face_split(bm, f, v1, v2, NULL, NULL, TRUE)) {
+	if (!BM_face_split(bm, f, v1, v2, NULL, NULL, true)) {
 		return NULL;
 	}
 	else {
