@@ -39,13 +39,11 @@
 #include "bmesh.h"
 #include "intern/bmesh_private.h"
 
-#define BM_OVERLAP (1 << 13)
-
 /**
  * Returns whether or not a given vertex is
  * is part of a given edge.
  */
-int BM_vert_in_edge(BMEdge *e, BMVert *v)
+bool BM_vert_in_edge(BMEdge *e, BMVert *v)
 {
 	return bmesh_vert_in_edge(e, v);
 }
@@ -54,7 +52,7 @@ int BM_vert_in_edge(BMEdge *e, BMVert *v)
  * \brief Other Loop in Face Sharing an Edge
  *
  * Finds the other loop that shares \a v with \a e loop in \a f.
- *
+ * <pre>
  *     +----------+
  *     |          |
  *     |    f     |
@@ -64,7 +62,7 @@ int BM_vert_in_edge(BMEdge *e, BMVert *v)
  *     ^     ^ <------- These vert args define direction
  *                      in the face to check.
  *                      The faces loop direction is ignored.
- *
+ * </pre>
  */
 BMLoop *BM_face_other_edge_loop(BMFace *f, BMEdge *e, BMVert *v)
 {
@@ -92,8 +90,7 @@ BMLoop *BM_face_other_edge_loop(BMFace *f, BMEdge *e, BMVert *v)
  * This function returns a loop in \a f that shares an edge with \a v
  * The direction is defined by \a v_prev, where the return value is
  * the loop of what would be 'v_next'
- *
- *
+ * <pre>
  *     +----------+ <-- return the face loop of this vertex.
  *     |          |
  *     |    f     |
@@ -103,6 +100,7 @@ BMLoop *BM_face_other_edge_loop(BMFace *f, BMEdge *e, BMVert *v)
  *     ^^^^^^     ^ <-- These vert args define direction
  *                      in the face to check.
  *                      The faces loop direction is ignored.
+ * </pre>
  *
  * \note \a v_prev and \a v _implicitly_ define an edge.
  */
@@ -143,7 +141,7 @@ BMLoop *BM_face_other_vert_loop(BMFace *f, BMVert *v_prev, BMVert *v)
  * \brief Other Loop in Face Sharing a Vert
  *
  * Finds the other loop that shares \a v with \a e loop in \a f.
- *
+ * <pre>
  *     +----------+ <-- return the face loop of this vertex.
  *     |          |
  *     |          |
@@ -153,6 +151,7 @@ BMLoop *BM_face_other_vert_loop(BMFace *f, BMVert *v_prev, BMVert *v)
  *           ^ <------- This loop defines both the face to search
  *                      and the edge, in combination with 'v'
  *                      The faces loop direction is ignored.
+ * </pre>
  */
 
 BMLoop *BM_loop_other_vert_loop(BMLoop *l, BMVert *v)
@@ -190,10 +189,28 @@ BMLoop *BM_loop_other_vert_loop(BMLoop *l, BMVert *v)
 }
 
 /**
- * Returns TRUE if the vertex is used in a given face.
+ * Get the first loop of a vert. Uses the same initialization code for the first loop of the
+ * iterator API
  */
+BMLoop *BM_vert_find_first_loop(BMVert *v)
+{
+	BMEdge *e;
 
-int BM_vert_in_face(BMFace *f, BMVert *v)
+	if (!v || !v->e)
+		return NULL;
+
+	e = bmesh_disk_faceedge_find_first(v->e, v);
+
+	if (!e)
+		return NULL;
+
+	return bmesh_radial_faceloop_find_first(e->l, v);
+}
+
+/**
+ * Returns true if the vertex is used in a given face.
+ */
+bool BM_vert_in_face(BMFace *f, BMVert *v)
 {
 	BMLoop *l_iter, *l_first;
 
@@ -209,19 +226,19 @@ int BM_vert_in_face(BMFace *f, BMVert *v)
 #endif
 		do {
 			if (l_iter->v == v) {
-				return TRUE;
+				return true;
 			}
 		} while ((l_iter = l_iter->next) != l_first);
 	}
 
-	return FALSE;
+	return false;
 }
 
 /**
  * Compares the number of vertices in an array
  * that appear in a given face
  */
-int BM_verts_in_face(BMesh *bm, BMFace *f, BMVert **varr, int len)
+int BM_verts_in_face_count(BMFace *f, BMVert **varr, int len)
 {
 	BMLoop *l_iter, *l_first;
 
@@ -232,7 +249,7 @@ int BM_verts_in_face(BMesh *bm, BMFace *f, BMVert **varr, int len)
 	int i, count = 0;
 	
 	for (i = 0; i < len; i++) {
-		BMO_elem_flag_enable(bm, varr[i], BM_OVERLAP);
+		BM_ELEM_API_FLAG_ENABLE(varr[i], _FLAG_OVERLAP);
 	}
 
 #ifdef USE_BMESH_HOLES
@@ -247,22 +264,78 @@ int BM_verts_in_face(BMesh *bm, BMFace *f, BMVert **varr, int len)
 #endif
 
 		do {
-			if (BMO_elem_flag_test(bm, l_iter->v, BM_OVERLAP)) {
+			if (BM_ELEM_API_FLAG_TEST(l_iter->v, _FLAG_OVERLAP)) {
 				count++;
 			}
 
 		} while ((l_iter = l_iter->next) != l_first);
 	}
 
-	for (i = 0; i < len; i++) BMO_elem_flag_disable(bm, varr[i], BM_OVERLAP);
+	for (i = 0; i < len; i++) {
+		BM_ELEM_API_FLAG_DISABLE(varr[i], _FLAG_OVERLAP);
+	}
 
 	return count;
 }
 
+
+/**
+ * Return true if all verts are in the face.
+ */
+bool BM_verts_in_face(BMFace *f, BMVert **varr, int len)
+{
+	BMLoop *l_iter, *l_first;
+
+#ifdef USE_BMESH_HOLES
+	BMLoopList *lst;
+#endif
+
+	int i;
+	bool ok = true;
+
+	/* simple check, we know can't succeed */
+	if (f->len < len) {
+		return false;
+	}
+
+	for (i = 0; i < len; i++) {
+		BM_ELEM_API_FLAG_ENABLE(varr[i], _FLAG_OVERLAP);
+	}
+
+#ifdef USE_BMESH_HOLES
+	for (lst = f->loops.first; lst; lst = lst->next)
+#endif
+	{
+
+#ifdef USE_BMESH_HOLES
+		l_iter = l_first = lst->first;
+#else
+		l_iter = l_first = f->l_first;
+#endif
+
+		do {
+			if (BM_ELEM_API_FLAG_TEST(l_iter->v, _FLAG_OVERLAP)) {
+				/* pass */
+			}
+			else {
+				ok = false;
+				break;
+			}
+
+		} while ((l_iter = l_iter->next) != l_first);
+	}
+
+	for (i = 0; i < len; i++) {
+		BM_ELEM_API_FLAG_DISABLE(varr[i], _FLAG_OVERLAP);
+	}
+
+	return ok;
+}
+
 /**
  * Returns whether or not a given edge is is part of a given face.
  */
-int BM_edge_in_face(BMFace *f, BMEdge *e)
+bool BM_edge_in_face(BMFace *f, BMEdge *e)
 {
 	BMLoop *l_iter;
 	BMLoop *l_first;
@@ -271,18 +344,26 @@ int BM_edge_in_face(BMFace *f, BMEdge *e)
 
 	do {
 		if (l_iter->e == e) {
-			return TRUE;
+			return true;
 		}
 	} while ((l_iter = l_iter->next) != l_first);
 
-	return FALSE;
+	return false;
+}
+
+/**
+ * Returns whether or not a given edge is is part of a given loop.
+ */
+bool BM_edge_in_loop(BMEdge *e, BMLoop *l)
+{
+	return (l->e == e || l->prev->e == e);
 }
 
 /**
  * Returns whether or not two vertices are in
  * a given edge
  */
-int BM_verts_in_edge(BMVert *v1, BMVert *v2, BMEdge *e)
+bool BM_verts_in_edge(BMVert *v1, BMVert *v2, BMEdge *e)
 {
 	return bmesh_verts_in_edge(v1, v2, e);
 }
@@ -296,11 +377,102 @@ BMVert *BM_edge_other_vert(BMEdge *e, BMVert *v)
 	return bmesh_edge_other_vert_get(e, v);
 }
 
+/**
+ * Given a edge and a loop (assumes the edge is manifold). returns
+ * the other faces loop, sharing the same vertex.
+ *
+ * <pre>
+ * +-------------------+
+ * |                   |
+ * |                   |
+ * |l_other <-- return |
+ * +-------------------+ <-- A manifold edge between 2 faces
+ * |l    e  <-- edge   |
+ * |^ <-------- loop   |
+ * |                   |
+ * +-------------------+
+ * </pre>
+ */
+BMLoop *BM_edge_other_loop(BMEdge *e, BMLoop *l)
+{
+	BMLoop *l_other;
+
+	// BLI_assert(BM_edge_is_manifold(e));  // TOO strict, just check if we have another radial face
+	BLI_assert(e->l && e->l->radial_next != e->l);
+	BLI_assert(BM_vert_in_edge(e, l->v));
+
+	l_other = (l->e == e) ? l : l->prev;
+	l_other = l_other->radial_next;
+	BLI_assert(l_other->e == e);
+
+	if (l_other->v == l->v) {
+		/* pass */
+	}
+	else if (l_other->next->v == l->v) {
+		l_other = l_other->next;
+	}
+	else {
+		BLI_assert(0);
+	}
+
+	return l_other;
+}
+
+/**
+ * Utility function to step around a fan of loops,
+ * using an edge to mark the previous side.
+ *
+ * \note all edges must be manifold,
+ * once a non manifold edge is hit, return NULL.
+ *
+ * <pre>
+ *                ,.,-->|
+ *            _,-'      |
+ *          ,'          | (notice how 'e_step'
+ *         /            |  and 'l' define the
+ *        /             |  direction the arrow
+ *       |     return   |  points).
+ *       |     loop --> |
+ * ---------------------+---------------------
+ *         ^      l --> |
+ *         |            |
+ *  assign e_step       |
+ *                      |
+ *   begin e_step ----> |
+ *                      |
+ * </pre>
+ */
+
+BMLoop *BM_vert_step_fan_loop(BMLoop *l, BMEdge **e_step)
+{
+	BMEdge *e_prev = *e_step;
+	BMEdge *e_next;
+	if (l->e == e_prev) {
+		e_next = l->prev->e;
+	}
+	else if (l->prev->e == e_prev) {
+		e_next = l->e;
+	}
+	else {
+		BLI_assert(0);
+		return NULL;
+	}
+
+	if (BM_edge_is_manifold(e_next)) {
+		return BM_edge_other_loop((*e_step = e_next), l);
+	}
+	else {
+		return NULL;
+	}
+}
+
+
+
 /**
  * The function takes a vertex at the center of a fan and returns the opposite edge in the fan.
  * All edges in the fan must be manifold, otherwise return NULL.
  *
- * \note This could (probably) be done more effieiently.
+ * \note This could (probably) be done more efficiently.
  */
 BMEdge *BM_vert_other_disk_edge(BMVert *v, BMEdge *e_first)
 {
@@ -350,35 +522,44 @@ BMEdge *BM_vert_other_disk_edge(BMVert *v, BMEdge *e_first)
 }
 
 /**
- * Returms edge length
+ * Returns edge length
  */
 float BM_edge_calc_length(BMEdge *e)
 {
 	return len_v3v3(e->v1->co, e->v2->co);
 }
 
+/**
+ * Returns edge length squared (for comparisons)
+ */
+float BM_edge_calc_length_squared(BMEdge *e)
+{
+	return len_squared_v3v3(e->v1->co, e->v2->co);
+}
+
 /**
  * Utility function, since enough times we have an edge
  * and want to access 2 connected faces.
  *
- * \return TRUE when only 2 faces are found.
+ * \return true when only 2 faces are found.
  */
-int BM_edge_face_pair(BMEdge *e, BMFace **r_fa, BMFace **r_fb)
+bool BM_edge_face_pair(BMEdge *e, BMFace **r_fa, BMFace **r_fb)
 {
 	BMLoop *la, *lb;
 
 	if ((la = e->l) &&
 	    (lb = la->radial_next) &&
+	    (la != lb) &&
 	    (lb->radial_next == la))
 	{
 		*r_fa = la->f;
 		*r_fb = lb->f;
-		return TRUE;
+		return true;
 	}
 	else {
 		*r_fa = NULL;
 		*r_fb = NULL;
-		return FALSE;
+		return false;
 	}
 }
 
@@ -386,24 +567,25 @@ int BM_edge_face_pair(BMEdge *e, BMFace **r_fa, BMFace **r_fb)
  * Utility function, since enough times we have an edge
  * and want to access 2 connected loops.
  *
- * \return TRUE when only 2 faces are found.
+ * \return true when only 2 faces are found.
  */
-int BM_edge_loop_pair(BMEdge *e, BMLoop **r_la, BMLoop **r_lb)
+bool BM_edge_loop_pair(BMEdge *e, BMLoop **r_la, BMLoop **r_lb)
 {
 	BMLoop *la, *lb;
 
 	if ((la = e->l) &&
 	    (lb = la->radial_next) &&
+	    (la != lb) &&
 	    (lb->radial_next == la))
 	{
 		*r_la = la;
 		*r_lb = lb;
-		return TRUE;
+		return true;
 	}
 	else {
 		*r_la = NULL;
 		*r_lb = NULL;
-		return FALSE;
+		return false;
 	}
 }
 
@@ -449,64 +631,44 @@ int BM_edge_face_count(BMEdge *e)
 }
 
 /**
- *	Returns the number of faces around this vert
+ * Returns the number of faces around this vert
+ * length matches #BM_LOOPS_OF_VERT iterator
  */
 int BM_vert_face_count(BMVert *v)
 {
-	int count = 0;
-	BMLoop *l;
-	BMIter iter;
-
-	BM_ITER_ELEM (l, &iter, v, BM_LOOPS_OF_VERT) {
-		count++;
-	}
-
-	return count;
-#if 0 //this code isn't working
-	BMEdge *curedge = NULL;
-
-	if (v->e) {
-		curedge = v->e;
-		do {
-			if (curedge->l) count += BM_edge_face_count(curedge);
-			curedge = bmesh_disk_edge_next(curedge, v);
-		} while (curedge != v->e);
-	}
-	return count;
-#endif
+	return bmesh_disk_facevert_count(v);
 }
 
 /**
  * Tests whether or not the vertex is part of a wire edge.
  * (ie: has no faces attached to it)
  */
-int BM_vert_is_wire(BMVert *v)
+bool BM_vert_is_wire(BMVert *v)
 {
-	BMEdge *curedge;
+	if (v->e) {
+		BMEdge *e_first, *e_iter;
 
-	if (v->e == NULL) {
-		return FALSE;
+		e_first = e_iter = v->e;
+		do {
+			if (e_iter->l) {
+				return false;
+			}
+		} while ((e_iter = bmesh_disk_edge_next(e_iter, v)) != e_first);
+
+		return true;
+	}
+	else {
+		return false;
 	}
-	
-	curedge = v->e;
-	do {
-		if (curedge->l) {
-			return FALSE;
-		}
-
-		curedge = bmesh_disk_edge_next(curedge, v);
-	} while (curedge != v->e);
-
-	return TRUE;
 }
 
 /**
  * Tests whether or not the edge is part of a wire.
  * (ie: has no faces attached to it)
  */
-int BM_edge_is_wire(BMEdge *e)
+bool BM_edge_is_wire(BMEdge *e)
 {
-	return (e->l) ? FALSE : TRUE;
+	return (e->l == NULL);
 }
 
 /**
@@ -516,7 +678,7 @@ int BM_edge_is_wire(BMEdge *e)
  * 3: Is part of a an edge with more than 2 faces.
  * 4: Is part of a wire edge.
  */
-int BM_vert_is_manifold(BMVert *v)
+bool BM_vert_is_manifold(BMVert *v)
 {
 	BMEdge *e, *oe;
 	BMLoop *l;
@@ -524,7 +686,7 @@ int BM_vert_is_manifold(BMVert *v)
 
 	if (v->e == NULL) {
 		/* loose vert */
-		return FALSE;
+		return false;
 	}
 
 	/* count edges while looking for non-manifold edges */
@@ -535,7 +697,7 @@ int BM_vert_is_manifold(BMVert *v)
 		 * edges with 1 face user are OK, otherwise we could
 		 * use BM_edge_is_manifold() here */
 		if (e->l == NULL || bmesh_radial_length(e->l) > 2) {
-			return FALSE;
+			return false;
 		}
 		len++;
 	} while ((e = bmesh_disk_edge_next(e, v)) != oe);
@@ -569,10 +731,10 @@ int BM_vert_is_manifold(BMVert *v)
 
 	if (count < len) {
 		/* vert shared by multiple regions */
-		return FALSE;
+		return false;
 	}
 
-	return TRUE;
+	return true;
 }
 
 /**
@@ -581,7 +743,7 @@ int BM_vert_is_manifold(BMVert *v)
  */
 
 #if 1 /* fast path for checking manifold */
-int BM_edge_is_manifold(BMEdge *e)
+bool BM_edge_is_manifold(BMEdge *e)
 {
 	const BMLoop *l = e->l;
 	return (l && (l->radial_next != l) &&             /* not 0 or 1 face users */
@@ -592,21 +754,34 @@ int BM_edge_is_manifold(BMEdge *e)
 {
 	int count = BM_edge_face_count(e);
 	if (count == 2) {
-		return TRUE;
+		return true;
 	}
 	else {
-		return FALSE;
+		return false;
 	}
 }
 #endif
 
+/**
+ * Tests that the edge is manifold and
+ * that both its faces point the same way.
+ */
+bool BM_edge_is_contiguous(BMEdge *e)
+{
+	const BMLoop *l = e->l;
+	const BMLoop *l_other = l->radial_next;
+	return (l && (l_other != l) &&               /* not 0 or 1 face users */
+	             (l_other->radial_next == l) &&  /* 2 face users */
+	             (l_other->v != l->v));
+}
+
 /**
  * Tests whether or not an edge is on the boundary
  * of a shell (has one face associated with it)
  */
 
 #if 1 /* fast path for checking boundary */
-int BM_edge_is_boundary(BMEdge *e)
+bool BM_edge_is_boundary(BMEdge *e)
 {
 	const BMLoop *l = e->l;
 	return (l && (l->radial_next == l));
@@ -616,14 +791,55 @@ int BM_edge_is_boundary(BMEdge *e)
 {
 	int count = BM_edge_face_count(e);
 	if (count == 1) {
-		return TRUE;
+		return true;
 	}
 	else {
-		return FALSE;
+		return false;
 	}
 }
 #endif
 
+/**
+ * Returns the number of faces that are adjacent to both f1 and f2,
+ * \note Could be sped up a bit by not using iterators and by tagging
+ * faces on either side, then count the tags rather then searching.
+ */
+int BM_face_share_face_count(BMFace *f1, BMFace *f2)
+{
+	BMIter iter1, iter2;
+	BMEdge *e;
+	BMFace *f;
+	int count = 0;
+
+	BM_ITER_ELEM (e, &iter1, f1, BM_EDGES_OF_FACE) {
+		BM_ITER_ELEM (f, &iter2, e, BM_FACES_OF_EDGE) {
+			if (f != f1 && f != f2 && BM_face_share_edge_check(f, f2))
+				count++;
+		}
+	}
+
+	return count;
+}
+
+/**
+ * same as #BM_face_share_face_count but returns a bool
+ */
+bool BM_face_share_face_check(BMFace *f1, BMFace *f2)
+{
+	BMIter iter1, iter2;
+	BMEdge *e;
+	BMFace *f;
+
+	BM_ITER_ELEM (e, &iter1, f1, BM_EDGES_OF_FACE) {
+		BM_ITER_ELEM (f, &iter2, e, BM_FACES_OF_EDGE) {
+			if (f != f1 && f != f2 && BM_face_share_edge_check(f, f2))
+				return true;
+		}
+	}
+
+	return false;
+}
+
 /**
  *  Counts the number of edges two faces share (if any)
  */
@@ -644,9 +860,27 @@ int BM_face_share_edge_count(BMFace *f1, BMFace *f2)
 }
 
 /**
- *	Test if e1 shares any faces with e2
+ *  Returns true if the faces share an edge
  */
-int BM_edge_share_face_count(BMEdge *e1, BMEdge *e2)
+bool BM_face_share_edge_check(BMFace *f1, BMFace *f2)
+{
+	BMLoop *l_iter;
+	BMLoop *l_first;
+
+	l_iter = l_first = BM_FACE_FIRST_LOOP(f1);
+	do {
+		if (bmesh_radial_face_find(l_iter->e, f2)) {
+			return true;
+		}
+	} while ((l_iter = l_iter->next) != l_first);
+
+	return false;
+}
+
+/**
+ * Test if e1 shares any faces with e2
+ */
+bool BM_edge_share_face_check(BMEdge *e1, BMEdge *e2)
 {
 	BMLoop *l;
 	BMFace *f;
@@ -656,18 +890,41 @@ int BM_edge_share_face_count(BMEdge *e1, BMEdge *e2)
 		do {
 			f = l->f;
 			if (bmesh_radial_face_find(e2, f)) {
-				return TRUE;
+				return true;
 			}
 			l = l->radial_next;
 		} while (l != e1->l);
 	}
-	return FALSE;
+	return false;
+}
+
+/**
+ *	Test if e1 shares any quad faces with e2
+ */
+bool BM_edge_share_quad_check(BMEdge *e1, BMEdge *e2)
+{
+	BMLoop *l;
+	BMFace *f;
+
+	if (e1->l && e2->l) {
+		l = e1->l;
+		do {
+			f = l->f;
+			if (f->len == 4) {
+				if (bmesh_radial_face_find(e2, f)) {
+					return true;
+				}
+			}
+			l = l->radial_next;
+		} while (l != e1->l);
+	}
+	return false;
 }
 
 /**
  *	Tests to see if e1 shares a vertex with e2
  */
-int BM_edge_share_vert_count(BMEdge *e1, BMEdge *e2)
+bool BM_edge_share_vert_check(BMEdge *e1, BMEdge *e2)
 {
 	return (e1->v1 == e2->v1 ||
 	        e1->v1 == e2->v2 ||
@@ -696,7 +953,7 @@ BMVert *BM_edge_share_vert(BMEdge *e1, BMEdge *e2)
  *
  * Finds the loop used which uses \a v in face loop \a l
  *
- * \note currenly this just uses simple loop in future may be speeded up
+ * \note currently this just uses simple loop in future may be sped up
  * using radial vars
  */
 BMLoop *BM_face_vert_share_loop(BMFace *f, BMVert *v)
@@ -719,7 +976,7 @@ BMLoop *BM_face_vert_share_loop(BMFace *f, BMVert *v)
  *
  * Finds the loop used which uses \a e in face loop \a l
  *
- * \note currenly this just uses simple loop in future may be speeded up
+ * \note currently this just uses simple loop in future may be sped up
  * using radial vars
  */
 BMLoop *BM_face_edge_share_loop(BMFace *f, BMEdge *e)
@@ -745,13 +1002,14 @@ BMLoop *BM_face_edge_share_loop(BMFace *f, BMEdge *e)
  * BM_face_create_ngon() on an arbitrary array of verts,
  * though be sure to pick an edge which has a face.
  *
- * \note This is infact quite a simple check, mainly include this function so the intent is more obvious.
+ * \note This is in fact quite a simple check, mainly include this function so the intent is more obvious.
  * We know these 2 verts will _always_ make up the loops edge
  */
 void BM_edge_ordered_verts_ex(BMEdge *edge, BMVert **r_v1, BMVert **r_v2,
                               BMLoop *edge_loop)
 {
 	BLI_assert(edge_loop->e == edge);
+	(void)edge; /* quiet warning in release build */
 	*r_v1 = edge_loop->v;
 	*r_v2 = edge_loop->next->v;
 }
@@ -761,6 +1019,22 @@ void BM_edge_ordered_verts(BMEdge *edge, BMVert **r_v1, BMVert **r_v2)
 	BM_edge_ordered_verts_ex(edge, r_v1, r_v2, edge->l);
 }
 
+/**
+ * Check if the loop is convex or concave
+ * (depends on face normal)
+ */
+bool BM_loop_is_convex(BMLoop *l)
+{
+	float e_dir_prev[3];
+	float e_dir_next[3];
+	float l_no[3];
+
+	sub_v3_v3v3(e_dir_prev, l->prev->v->co, l->v->co);
+	sub_v3_v3v3(e_dir_next, l->next->v->co, l->v->co);
+	cross_v3_v3v3(l_no, e_dir_next, e_dir_prev);
+	return dot_v3v3(l_no, l->f->no) > 0.0f;
+}
+
 /**
  * Calculates the angle between the previous and next loops
  * (angle at this loops face corner).
@@ -777,9 +1051,8 @@ float BM_loop_calc_face_angle(BMLoop *l)
 /**
  * \brief BM_loop_calc_face_normal
  *
- * Calculate the normal at this loop corner or fallback to the face normal on straignt lines.
+ * Calculate the normal at this loop corner or fallback to the face normal on straight lines.
  *
- * \param bm The BMesh
  * \param l The loop to calculate the normal at
  * \param r_normal Resulting normal
  */
@@ -790,20 +1063,42 @@ void BM_loop_calc_face_normal(BMLoop *l, float r_normal[3])
 	                  l->v->co,
 	                  l->next->v->co) != 0.0f)
 	{
-		return;
+		/* pass */
 	}
 	else {
 		copy_v3_v3(r_normal, l->f->no);
 	}
 }
 
+/**
+ * \brief BM_loop_calc_face_direction
+ *
+ * Calculate the direction a loop is pointing.
+ *
+ * \param l The loop to calculate the direction at
+ * \param r_dir Resulting direction
+ */
+void BM_loop_calc_face_direction(BMLoop *l, float r_dir[3])
+{
+	float v_prev[3];
+	float v_next[3];
+
+	sub_v3_v3v3(v_prev, l->v->co, l->prev->v->co);
+	sub_v3_v3v3(v_next, l->next->v->co, l->v->co);
+
+	normalize_v3(v_prev);
+	normalize_v3(v_next);
+
+	add_v3_v3v3(r_dir, v_prev, v_next);
+	normalize_v3(r_dir);
+}
+
 /**
  * \brief BM_loop_calc_face_tangent
  *
- * Calculate the tangent at this loop corner or fallback to the face normal on straignt lines.
+ * Calculate the tangent at this loop corner or fallback to the face normal on straight lines.
  * This vector always points inward into the face.
  *
- * \param bm The BMesh
  * \param l The loop to calculate the tangent at
  * \param r_tangent Resulting tangent
  */
@@ -811,23 +1106,27 @@ void BM_loop_calc_face_tangent(BMLoop *l, float r_tangent[3])
 {
 	float v_prev[3];
 	float v_next[3];
+	float dir[3];
 
 	sub_v3_v3v3(v_prev, l->prev->v->co, l->v->co);
 	sub_v3_v3v3(v_next, l->v->co, l->next->v->co);
 
 	normalize_v3(v_prev);
 	normalize_v3(v_next);
+	add_v3_v3v3(dir, v_prev, v_next);
 
-	if (compare_v3v3(v_prev, v_next, FLT_EPSILON) == FALSE) {
-		float dir[3];
+	if (compare_v3v3(v_prev, v_next, FLT_EPSILON * 10.0f) == false) {
 		float nor[3]; /* for this purpose doesn't need to be normalized */
-		add_v3_v3v3(dir, v_prev, v_next);
 		cross_v3_v3v3(nor, v_prev, v_next);
+		/* concave face check */
+		if (UNLIKELY(dot_v3v3(nor, l->f->no) < 0.0f)) {
+			negate_v3(nor);
+		}
 		cross_v3_v3v3(r_tangent, dir, nor);
 	}
 	else {
 		/* prev/next are the same - compare with face normal since we don't have one */
-		cross_v3_v3v3(r_tangent, v_next, l->f->no);
+		cross_v3_v3v3(r_tangent, dir, l->f->no);
 	}
 
 	normalize_v3(r_tangent);
@@ -856,13 +1155,14 @@ float BM_edge_calc_face_angle(BMEdge *e)
 /**
  * \brief BMESH EDGE/FACE TANGENT
  *
- * Calculate the tangent at this loop corner or fallback to the face normal on straignt lines.
+ * Calculate the tangent at this loop corner or fallback to the face normal on straight lines.
  * This vector always points inward into the face.
  *
  * \brief BM_edge_calc_face_tangent
  * \param e
  * \param e_loop The loop to calculate the tangent at,
  * used to get the face and winding direction.
+ * \param r_tangent The loop corner tangent to set
  */
 
 void BM_edge_calc_face_tangent(BMEdge *e, BMLoop *e_loop, float r_tangent[3])
@@ -900,7 +1200,7 @@ float BM_vert_calc_edge_angle(BMVert *v)
 		BMVert *v1 = BM_edge_other_vert(e1, v);
 		BMVert *v2 = BM_edge_other_vert(e2, v);
 
-		return M_PI - angle_v3v3v3(v1->co, v->co, v2->co);
+		return (float)M_PI - angle_v3v3v3(v1->co, v->co, v2->co);
 	}
 	else {
 		return DEG2RADF(90.0f);
@@ -924,7 +1224,121 @@ float BM_vert_calc_shell_factor(BMVert *v)
 		accum_angle += face_angle;
 	}
 
-	return accum_shell / accum_angle;
+	if (accum_angle != 0.0f) {
+		return accum_shell / accum_angle;
+	}
+	else {
+		return 1.0f;
+	}
+}
+/* alternate version of #BM_vert_calc_shell_factor which only
+ * uses 'hflag' faces, but falls back to all if none found. */
+float BM_vert_calc_shell_factor_ex(BMVert *v, const char hflag)
+{
+	BMIter iter;
+	BMLoop *l;
+	float accum_shell = 0.0f;
+	float accum_angle = 0.0f;
+	int tot_sel = 0, tot = 0;
+
+	BM_ITER_ELEM (l, &iter, v, BM_LOOPS_OF_VERT) {
+		if (BM_elem_flag_test(l->f, hflag)) {  /* <-- main difference to BM_vert_calc_shell_factor! */
+			const float face_angle = BM_loop_calc_face_angle(l);
+			accum_shell += shell_angle_to_dist(angle_normalized_v3v3(v->no, l->f->no)) * face_angle;
+			accum_angle += face_angle;
+			tot_sel++;
+		}
+		tot++;
+	}
+
+	if (accum_angle != 0.0f) {
+		return accum_shell / accum_angle;
+	}
+	else {
+		/* other main difference from BM_vert_calc_shell_factor! */
+		if (tot != 0 && tot_sel == 0) {
+			/* none selected, so use all */
+			return BM_vert_calc_shell_factor(v);
+		}
+		else {
+			return 1.0f;
+		}
+	}
+}
+
+/**
+ * \note quite an obscure function.
+ * used in bmesh operators that have a relative scale options,
+ */
+float BM_vert_calc_mean_tagged_edge_length(BMVert *v)
+{
+	BMIter iter;
+	BMEdge *e;
+	int tot;
+	float length = 0.0f;
+
+	BM_ITER_ELEM_INDEX (e, &iter, v, BM_EDGES_OF_VERT, tot) {
+		BMVert *v_other = BM_edge_other_vert(e, v);
+		if (BM_elem_flag_test(v_other, BM_ELEM_TAG)) {
+			length += BM_edge_calc_length(e);
+		}
+	}
+
+	if (tot) {
+		return length / (float)tot;
+	}
+	else {
+		return 0.0f;
+	}
+}
+
+
+/**
+ * Returns the loop of the shortest edge in f.
+ */
+BMLoop *BM_face_find_shortest_loop(BMFace *f)
+{
+	BMLoop *shortest_loop = NULL;
+	float shortest_len = FLT_MAX;
+
+	BMLoop *l_iter;
+	BMLoop *l_first;
+
+	l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+
+	do {
+		const float len = len_squared_v3v3(l_iter->v->co, l_iter->next->v->co);
+		if (len <= shortest_len) {
+			shortest_loop = l_iter;
+			shortest_len = len;
+		}
+	} while ((l_iter = l_iter->next) != l_first);
+
+	return shortest_loop;
+}
+
+/**
+ * Returns the loop of the longest edge in f.
+ */
+BMLoop *BM_face_find_longest_loop(BMFace *f)
+{
+	BMLoop *longest_loop = NULL;
+	float longest_len = 0.0f;
+
+	BMLoop *l_iter;
+	BMLoop *l_first;
+
+	l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+
+	do {
+		const float len = len_squared_v3v3(l_iter->v->co, l_iter->next->v->co);
+		if (len >= longest_len) {
+			longest_loop = l_iter;
+			longest_len = len;
+		}
+	} while ((l_iter = l_iter->next) != l_first);
+
+	return longest_loop;
 }
 
 /**
@@ -947,59 +1361,50 @@ BMEdge *BM_edge_exists(BMVert *v1, BMVert *v2)
 }
 
 /**
- * Given a set of vertices \a varr, find out if
- * all those vertices overlap an existing face.
- *
- * \note Making a face here is valid but in some cases you wont want to
- * make a face thats part of another.
- *
- * \returns TRUE for overlap
- *
+ * Returns an edge sharing the same vertices as this one.
+ * This isn't an invalid state but tools should clean up these cases before
+ * returning the mesh to the user.
  */
-int BM_face_exists_overlap(BMesh *bm, BMVert **varr, int len, BMFace **r_overlapface)
+BMEdge *BM_edge_find_double(BMEdge *e)
 {
-	BMIter viter;
-	BMFace *f;
-	int i, amount;
+	BMVert *v       = e->v1;
+	BMVert *v_other = e->v2;
 
-	for (i = 0; i < len; i++) {
-		BM_ITER_ELEM (f, &viter, varr[i], BM_FACES_OF_VERT) {
-			amount = BM_verts_in_face(bm, f, varr, len);
-			if (amount >= len) {
-				if (r_overlapface) {
-					*r_overlapface = f;
-				}
-				return TRUE;
-			}
+	BMEdge *e_iter;
+
+	e_iter = e;
+	while ((e_iter = bmesh_disk_edge_next(e_iter, v)) != e) {
+		if (UNLIKELY(BM_vert_in_edge(e_iter, v_other))) {
+			return e_iter;
 		}
 	}
 
-	if (r_overlapface) {
-		*r_overlapface = NULL;
-	}
-
-	return FALSE;
+	return NULL;
 }
 
 /**
  * Given a set of vertices (varr), find out if
  * there is a face with exactly those vertices
  * (and only those vertices).
+ *
+ * \note there used to be a BM_face_exists_overlap function that checked for partial overlap,
+ * however this is no longer used, simple to add back.
  */
-int BM_face_exists(BMesh *bm, BMVert **varr, int len, BMFace **r_existface)
+bool BM_face_exists(BMVert **varr, int len, BMFace **r_existface)
 {
+	BMVert *v_search = varr[0];  /* we can search any of the verts in the array */
 	BMIter viter;
 	BMFace *f;
-	int i, amount;
 
-	for (i = 0; i < len; i++) {
-		BM_ITER_ELEM (f, &viter, varr[i], BM_FACES_OF_VERT) {
-			amount = BM_verts_in_face(bm, f, varr, len);
-			if (amount == len && amount == f->len) {
+
+#if 0
+	BM_ITER_ELEM (f, &viter, v_search, BM_FACES_OF_VERT) {
+		if (f->len == len) {
+			if (BM_verts_in_face(f, varr, len)) {
 				if (r_existface) {
 					*r_existface = f;
 				}
-				return TRUE;
+				return true;
 			}
 		}
 	}
@@ -1007,7 +1412,65 @@ int BM_face_exists(BMesh *bm, BMVert **varr, int len, BMFace **r_existface)
 	if (r_existface) {
 		*r_existface = NULL;
 	}
-	return FALSE;
+	return false;
+
+#else
+
+	/* faster to do the flagging once, and inline */
+	bool is_init = false;
+	bool is_found = false;
+	int i;
+
+
+	BM_ITER_ELEM (f, &viter, v_search, BM_FACES_OF_VERT) {
+		if (f->len == len) {
+			if (is_init == false) {
+				is_init = true;
+				for (i = 0; i < len; i++) {
+					BLI_assert(!BM_ELEM_API_FLAG_TEST(varr[i], _FLAG_OVERLAP));
+					BM_ELEM_API_FLAG_ENABLE(varr[i], _FLAG_OVERLAP);
+				}
+			}
+
+			is_found = true;
+
+			{
+				BMLoop *l_iter;
+				BMLoop *l_first;
+
+				l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+
+				do {
+					if (!BM_ELEM_API_FLAG_TEST(l_iter->v, _FLAG_OVERLAP)) {
+						is_found = false;
+						break;
+					}
+				} while ((l_iter = l_iter->next) != l_first);
+			}
+
+			if (is_found) {
+				if (r_existface) {
+					*r_existface = f;
+				}
+				break;
+			}
+		}
+	}
+
+	if (is_found == false) {
+		if (r_existface) {
+			*r_existface = NULL;
+		}
+	}
+
+	if (is_init == true) {
+		for (i = 0; i < len; i++) {
+			BM_ELEM_API_FLAG_DISABLE(varr[i], _FLAG_OVERLAP);
+		}
+	}
+
+	return is_found;
+#endif
 }
 
 
@@ -1023,12 +1486,12 @@ int BM_face_exists(BMesh *bm, BMVert **varr, int len, BMFace **r_existface)
  *
  * \a earr and \a varr can be in any order, however they _must_ form a closed loop.
  */
-int BM_face_exists_multi(BMVert **varr, BMEdge **earr, int len)
+bool BM_face_exists_multi(BMVert **varr, BMEdge **earr, int len)
 {
 	BMFace *f;
 	BMEdge *e;
 	BMVert *v;
-	int ok;
+	bool ok;
 	int tot_tag;
 
 	BMIter fiter;
@@ -1068,10 +1531,10 @@ int BM_face_exists_multi(BMVert **varr, BMEdge **earr, int len)
 	for (i = 0; i < len; i++) {
 		BM_ITER_ELEM (f, &fiter, earr[i], BM_FACES_OF_EDGE) {
 			if (!BM_elem_flag_test(f, BM_ELEM_INTERNAL_TAG)) {
-				ok = TRUE;
+				ok = true;
 				BM_ITER_ELEM (v, &viter, f, BM_VERTS_OF_FACE) {
 					if (!BM_elem_flag_test(v, BM_ELEM_INTERNAL_TAG)) {
-						ok = FALSE;
+						ok = false;
 						break;
 					}
 				}
@@ -1090,20 +1553,20 @@ int BM_face_exists_multi(BMVert **varr, BMEdge **earr, int len)
 
 	if (tot_tag == 0) {
 		/* no faces use only boundary verts, quit early */
-		return FALSE;
+		return false;
 	}
 
 	/* 2) loop over non-boundary edges that use boundary verts,
 	 *    check each have 2 tagges faces connected (faces that only use 'varr' verts) */
-	ok = TRUE;
+	ok = true;
 	for (i = 0; i < len; i++) {
 		BM_ITER_ELEM (e, &fiter, varr[i], BM_EDGES_OF_VERT) {
 
 			if (/* non-boundary edge */
-			    BM_elem_flag_test(e, BM_ELEM_INTERNAL_TAG) == FALSE &&
+			    BM_elem_flag_test(e, BM_ELEM_INTERNAL_TAG) == false &&
 			    /* ...using boundary verts */
-			    BM_elem_flag_test(e->v1, BM_ELEM_INTERNAL_TAG) == TRUE &&
-			    BM_elem_flag_test(e->v2, BM_ELEM_INTERNAL_TAG) == TRUE)
+			    BM_elem_flag_test(e->v1, BM_ELEM_INTERNAL_TAG) == true &&
+			    BM_elem_flag_test(e->v2, BM_ELEM_INTERNAL_TAG) == true)
 			{
 				int tot_face_tag = 0;
 				BM_ITER_ELEM (f, &fiter, e, BM_FACES_OF_EDGE) {
@@ -1113,14 +1576,14 @@ int BM_face_exists_multi(BMVert **varr, BMEdge **earr, int len)
 				}
 
 				if (tot_face_tag != 2) {
-					ok = FALSE;
+					ok = false;
 					break;
 				}
 
 			}
 		}
 
-		if (ok == FALSE) {
+		if (ok == false) {
 			break;
 		}
 	}
@@ -1129,32 +1592,63 @@ int BM_face_exists_multi(BMVert **varr, BMEdge **earr, int len)
 }
 
 /* same as 'BM_face_exists_multi' but built vert array from edges */
-int BM_face_exists_multi_edge(BMEdge **earr, int len)
+bool BM_face_exists_multi_edge(BMEdge **earr, int len)
 {
-	BMVert **varr;
-	BLI_array_fixedstack_declare(varr, BM_NGON_STACK_SIZE, len, __func__);
+	BMVert **varr = BLI_array_alloca(varr, len);
 
-	int ok;
+	bool ok;
 	int i, i_next;
 
 	/* first check if verts have edges, if not we can bail out early */
-	ok = TRUE;
+	ok = true;
 	for (i = len - 1, i_next = 0; i_next < len; (i = i_next++)) {
 		if (!(varr[i] = BM_edge_share_vert(earr[i], earr[i_next]))) {
-			ok = FALSE;
+			ok = false;
 			break;
 		}
 	}
 
-	if (ok == FALSE) {
+	if (ok == false) {
 		BMESH_ASSERT(0);
-		BLI_array_fixedstack_free(varr);
-		return FALSE;
+		return false;
 	}
 
 	ok = BM_face_exists_multi(varr, earr, len);
 
-	BLI_array_fixedstack_free(varr);
-
 	return ok;
 }
+
+/* convenience functions for checking flags */
+bool BM_edge_is_any_vert_flag_test(BMEdge *e, const char hflag)
+{
+	return (BM_elem_flag_test(e->v1, hflag) ||
+	        BM_elem_flag_test(e->v2, hflag));
+}
+
+bool BM_face_is_any_vert_flag_test(BMFace *f, const char hflag)
+{
+	BMLoop *l_iter;
+	BMLoop *l_first;
+
+	l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+	do {
+		if (BM_elem_flag_test(l_iter->v, hflag)) {
+			return true;
+		}
+	} while ((l_iter = l_iter->next) != l_first);
+	return false;
+}
+
+bool BM_face_is_any_edge_flag_test(BMFace *f, const char hflag)
+{
+	BMLoop *l_iter;
+	BMLoop *l_first;
+
+	l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+	do {
+		if (BM_elem_flag_test(l_iter->e, hflag)) {
+			return true;
+		}
+	} while ((l_iter = l_iter->next) != l_first);
+	return false;
+}
