@@ -83,12 +83,12 @@ static int count_edge_faces(BMesh *bm, BMEdge *e);
 
 BLI_INLINE BMDiskLink *rs_edge_link_get(BMEdge *e, BMVert *v, EdgeData *e_data)
 {
-	return 	v == ((BMEdge *)e)->v1 ? &(((EdgeData *)e_data)->v1_disk_link) :
-	                                 &(((EdgeData *)e_data)->v2_disk_link) ;
+	return v == ((BMEdge *)e)->v1 ? &(((EdgeData *)e_data)->v1_disk_link) :
+	                                &(((EdgeData *)e_data)->v2_disk_link);
 }
 
-static int rotsys_append_edge(BMEdge *e, BMVert *v,
-                              EdgeData *edata, VertData *vdata)
+static bool rotsys_append_edge(BMEdge *e, BMVert *v,
+                               EdgeData *edata, VertData *vdata)
 {
 	EdgeData *ed = &edata[BM_elem_index_get(e)];
 	VertData *vd = &vdata[BM_elem_index_get(v)];
@@ -116,7 +116,7 @@ static int rotsys_append_edge(BMEdge *e, BMVert *v,
 		}
 	}
 
-	return TRUE;
+	return true;
 }
 
 static void UNUSED_FUNCTION(rotsys_remove_edge)(BMEdge *e, BMVert *v,
@@ -168,7 +168,7 @@ static void rotsys_reverse(BMEdge *UNUSED(e), BMVert *v, EdgeData *edata, VertDa
 	BMEdge **edges = NULL;
 	BMEdge *e_first;
 	BMEdge *e;
-	BLI_array_staticdeclare(edges, BM_NGON_STACK_SIZE);
+	BLI_array_staticdeclare(edges, BM_DEFAULT_NGON_STACK_SIZE);
 	int i, totedge;
 	
 	e = e_first = vdata[BM_elem_index_get(v)].e;
@@ -274,9 +274,10 @@ static int UNUSED_FUNCTION(rotsys_fill_faces)(BMesh *bm, EdgeData *edata, VertDa
 			if (!ok || BLI_array_count(edges) < 3)
 				continue;
 			
-			f = BM_face_create_ngon(bm, verts[0], verts[1], edges, BLI_array_count(edges), TRUE);
-			if (!f)
+			f = BM_face_create_ngon(bm, verts[0], verts[1], edges, BLI_array_count(edges), BM_CREATE_NO_DOUBLE);
+			if (UNLIKELY(f == NULL)) {
 				continue;
+			}
 		}
 	}
 	
@@ -358,14 +359,12 @@ static void init_rotsys(BMesh *bm, EdgeData *edata, VertData *vdata)
 	BMIter iter;
 	BMEdge *e;
 	BMEdge **edges = NULL;
-	BLI_array_staticdeclare(edges, BM_NGON_STACK_SIZE);
+	BLI_array_staticdeclare(edges, BM_DEFAULT_NGON_STACK_SIZE);
 	BMVert *v;
 	/* BMVert **verts = NULL; */
-	/* BLI_array_staticdeclare(verts, BM_NGON_STACK_SIZE); */ /* UNUSE */
+	/* BLI_array_staticdeclare(verts, BM_DEFAULT_NGON_STACK_SIZE); */ /* UNUSE */
 	int i;
 	
-#define SIGN(n) ((n)<0.0f)
-	
 	BM_ITER_MESH (v, &iter, bm, BM_VERTS_OF_MESH) {
 		BMIter eiter;
 		float no[3], cent[3];
@@ -579,7 +578,7 @@ static void init_rotsys(BMesh *bm, EdgeData *edata, VertData *vdata)
 	//rotsys_fill_faces(bm, edata, vdata);
 
 #if 0
-	/* create visualizing geometr */
+	/* create visualizing geometry */
 	BMVert *lastv;
 	BM_ITER_MESH (v, &iter, bm, BM_VERTS_OF_MESH) {
 		BMVert *v2;
@@ -612,12 +611,12 @@ static void init_rotsys(BMesh *bm, EdgeData *edata, VertData *vdata)
 			
 			v2 = BM_vert_create(bm, co, NULL);
 			BM_elem_index_set(v2, -1); /* set_dirty! */
-			//BM_edge_create(bm, cv, v2, NULL, FALSE);
+			//BM_edge_create(bm, cv, v2, NULL, 0);
 			
-			BM_vert_select_set(bm, v2, TRUE);
+			BM_vert_select_set(bm, v2, true);
 			if (lastv) {
-				e2 = BM_edge_create(bm, lastv, v2, NULL, FALSE);
-				BM_edge_select_set(bm, e2, TRUE);
+				e2 = BM_edge_create(bm, lastv, v2, NULL, 0);
+				BM_edge_select_set(bm, e2, true);
 			}
 			
 			lastv = v2;
@@ -733,7 +732,7 @@ static EPath *edge_find_shortest_path(BMesh *bm, BMOperator *op, BMEdge *edge, E
                                       VertData *vdata, PathBase *pathbase, int group)
 {
 	BMEdge *e;
-	GHash *gh = BLI_ghash_new(BLI_ghashutil_ptrhash, BLI_ghashutil_ptrcmp, "createops find shortest path");
+	GHash *gh = BLI_ghash_ptr_new("createops find shortest path");
 	BMVert *v1, *v2;
 	BMVert **verts = NULL;
 	BLI_array_staticdeclare(verts, 1024);
@@ -742,7 +741,10 @@ static EPath *edge_find_shortest_path(BMesh *bm, BMOperator *op, BMEdge *edge, E
 	BMVert *startv;
 	BMVert *endv;
 	EPathNode *node;
-	int i, use_restrict = BMO_slot_bool_get(op, "use_restrict");
+	int i;
+	const bool use_restrict = BMO_slot_bool_get(op->slots_in, "use_restrict");
+	BMOpSlot *slot_restrict = BMO_slot_get(op->slots_in, "restrict");
+
 
 	startv = edata[BM_elem_index_get(edge)].ftag ? edge->v2 : edge->v1;
 	endv = edata[BM_elem_index_get(edge)].ftag ? edge->v1 : edge->v2;
@@ -766,11 +768,11 @@ static EPath *edge_find_shortest_path(BMesh *bm, BMOperator *op, BMEdge *edge, E
 			i = 0;
 			BLI_array_empty(verts);
 			for (i = 0, node = path->nodes.first; node; node = node->next, i++) {
-				BLI_array_growone(verts);
+				BLI_array_grow_one(verts);
 				verts[i] = node->v;
 			}
 
-			if (BM_face_exists(bm, verts, i, &f)) {
+			if (BM_face_exists(verts, i, &f)) {
 				if (!BMO_elem_flag_test(bm, f, FACE_IGNORE)) {
 					BLI_ghash_remove(gh, endv, NULL, NULL);
 					continue;
@@ -807,12 +809,13 @@ static EPath *edge_find_shortest_path(BMesh *bm, BMOperator *op, BMEdge *edge, E
 				continue;
 			}
 			
-			if (use_restrict && BMO_slot_map_contains(bm, op, "restrict", e)) {
-				int group = BMO_slot_map_int_get(bm, op, "restrict", e);
-				
-				if (!(group & path->group)) {
-					v2 = NULL;
-					continue;
+			if (use_restrict) {
+				int *group = (int *)BMO_slot_map_data_get(slot_restrict, e);
+				if (group) {
+					if (!(*group & path->group)) {
+						v2 = NULL;
+						continue;
+					}
 				}
 			}
 
@@ -896,12 +899,14 @@ void bmo_edgenet_fill_exec(BMesh *bm, BMOperator *op)
 	BMEdge **edges = NULL;
 	PathBase *pathbase;
 	BLI_array_declare(edges);
-	int use_restrict       = BMO_slot_bool_get(op, "use_restrict");
-	int use_fill_check     = BMO_slot_bool_get(op, "use_fill_check");
-	const short mat_nr     = BMO_slot_int_get(op,  "mat_nr");
-	const short use_smooth = BMO_slot_bool_get(op, "use_smooth");
+	const bool use_restrict   = BMO_slot_bool_get(op->slots_in, "use_restrict");
+	const bool use_fill_check = BMO_slot_bool_get(op->slots_in, "use_fill_check");
+	const short mat_nr        = BMO_slot_int_get(op->slots_in,  "mat_nr");
+	const bool use_smooth     = BMO_slot_bool_get(op->slots_in, "use_smooth");
 	int i, j, group = 0;
 	unsigned int winding[2]; /* accumulte winding directions for each edge which has a face */
+	BMOpSlot *slot_restrict          = BMO_slot_get(op->slots_in, "restrict");
+	BMOpSlot *slot_face_groupmap_out = BMO_slot_get(op->slots_out, "face_groupmap.out");
 
 	if (!bm->totvert || !bm->totedge)
 		return;
@@ -911,8 +916,8 @@ void bmo_edgenet_fill_exec(BMesh *bm, BMOperator *op)
 	edata = MEM_callocN(sizeof(EdgeData) * bm->totedge, "EdgeData");
 	vdata = MEM_callocN(sizeof(VertData) * bm->totvert, "VertData");
 	
-	BMO_slot_buffer_flag_enable(bm, op, "edges", BM_EDGE, EDGE_MARK);
-	BMO_slot_buffer_flag_enable(bm, op, "excludefaces", BM_FACE, FACE_IGNORE);
+	BMO_slot_buffer_flag_enable(bm, op->slots_in, "edges", BM_EDGE, EDGE_MARK);
+	BMO_slot_buffer_flag_enable(bm, op->slots_in, "exclude_faces", BM_FACE, FACE_IGNORE);
 	
 	BM_mesh_elem_index_ensure(bm, BM_VERT);
 
@@ -933,14 +938,14 @@ void bmo_edgenet_fill_exec(BMesh *bm, BMOperator *op)
 	bm->elem_index_dirty &= ~BM_EDGE;
 
 	init_rotsys(bm, edata, vdata);
-	
+
 	while (1) {
 		edge = NULL;
 		group = 0;
 		
-		BMO_ITER (e, &siter, bm, op, "edges", BM_EDGE) {
+		BMO_ITER (e, &siter, op->slots_in, "edges", BM_EDGE) {
 			/* if restrict is on, only start on faces in the restrict map */
-			if (use_restrict && !BMO_slot_map_contains(bm, op, "restrict", e))
+			if (use_restrict && !BMO_slot_map_contains(slot_restrict, e))
 				continue;
 
 			if (edata[BM_elem_index_get(e)].tag < 2) {
@@ -949,7 +954,7 @@ void bmo_edgenet_fill_exec(BMesh *bm, BMOperator *op)
 				if (use_restrict) {
 					int i = 0, j = 0, gi = 0;
 					
-					group = BMO_slot_map_int_get(bm, op, "restrict", e);
+					group = BMO_slot_map_int_get(slot_restrict, e);
 					
 					for (i = 0; i < 30; i++) {
 						if (group & (1 << i)) {
@@ -999,7 +1004,7 @@ void bmo_edgenet_fill_exec(BMesh *bm, BMOperator *op)
 			}
 
 			edata[BM_elem_index_get(e)].ftag++;
-			BLI_array_growone(edges);
+			BLI_array_grow_one(edges);
 			edges[i++] = e;
 
 			BLI_array_append(verts, node->v);
@@ -1009,7 +1014,7 @@ void bmo_edgenet_fill_exec(BMesh *bm, BMOperator *op)
 			vote_on_winding(edge, path->nodes.last, winding);
 		}
 
-		BLI_array_growone(edges);
+		BLI_array_grow_one(edges);
 		edges[i++] = edge;
 		edata[BM_elem_index_get(edge)].ftag++;
 		
@@ -1042,11 +1047,11 @@ void bmo_edgenet_fill_exec(BMesh *bm, BMOperator *op)
 				v2 = verts[0];
 			}
 
-			if ((use_fill_check == FALSE) ||
+			if ((use_fill_check == false) ||
 			    /* fairly expensive check - see if there are already faces filling this area */
-			    (BM_face_exists_multi_edge(edges, i) == FALSE))
+			    (BM_face_exists_multi_edge(edges, i) == false))
 			{
-				f = BM_face_create_ngon(bm, v1, v2, edges, i, TRUE);
+				f = BM_face_create_ngon(bm, v1, v2, edges, i, BM_CREATE_NO_DOUBLE);
 				if (f && !BMO_elem_flag_test(bm, f, ELE_ORIG)) {
 					BMO_elem_flag_enable(bm, f, FACE_NEW);
 					f->mat_nr = mat_nr;
@@ -1056,7 +1061,7 @@ void bmo_edgenet_fill_exec(BMesh *bm, BMOperator *op)
 				}
 
 				if (use_restrict) {
-					BMO_slot_map_int_insert(bm, op, "faceout_groupmap", f, path->group);
+					BMO_slot_map_int_insert(op, slot_face_groupmap_out, f, path->group);
 				}
 			}
 		}
@@ -1064,7 +1069,7 @@ void bmo_edgenet_fill_exec(BMesh *bm, BMOperator *op)
 		edge_free_path(pathbase, path);
 	}
 
-	BMO_slot_buffer_from_enabled_flag(bm, op, "faceout", BM_FACE, FACE_NEW);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "faces.out", BM_FACE, FACE_NEW);
 
 	BLI_array_free(edges);
 	BLI_array_free(verts);
@@ -1093,7 +1098,7 @@ static BMEdge *edge_next(BMesh *bm, BMEdge *e)
 	return NULL;
 }
 
-void bmo_edgenet_prepare(BMesh *bm, BMOperator *op)
+void bmo_edgenet_prepare_exec(BMesh *bm, BMOperator *op)
 {
 	BMOIter siter;
 	BMEdge *e;
@@ -1104,11 +1109,11 @@ void bmo_edgenet_prepare(BMesh *bm, BMOperator *op)
 	int ok = 1;
 	int i, count;
 
-	BMO_slot_buffer_flag_enable(bm, op, "edges", BM_EDGE, EDGE_MARK);
+	BMO_slot_buffer_flag_enable(bm, op->slots_in, "edges", BM_EDGE, EDGE_MARK);
 	
 	/* validate that each edge has at most one other tagged edge in the
 	 * disk cycle around each of it's vertices */
-	BMO_ITER (e, &siter, bm, op, "edges", BM_EDGE) {
+	BMO_ITER (e, &siter, op->slots_in, "edges", BM_EDGE) {
 		for (i = 0; i < 2; i++) {
 			count = BMO_vert_edge_flags_count(bm, i ? e->v2 : e->v1, EDGE_MARK);
 			if (count > 2) {
@@ -1130,7 +1135,7 @@ void bmo_edgenet_prepare(BMesh *bm, BMOperator *op)
 	/* find connected loops within the input edge */
 	count = 0;
 	while (1) {
-		BMO_ITER (e, &siter, bm, op, "edges", BM_EDGE) {
+		BMO_ITER (e, &siter, op->slots_in, "edges", BM_EDGE) {
 			if (!BMO_elem_flag_test(bm, e, EDGE_VIS)) {
 				if (BMO_vert_edge_flags_count(bm, e->v1, EDGE_MARK) == 1 ||
 				    BMO_vert_edge_flags_count(bm, e->v2, EDGE_MARK) == 1)
@@ -1157,7 +1162,7 @@ void bmo_edgenet_prepare(BMesh *bm, BMOperator *op)
 		i = 0;
 		while (e) {
 			BMO_elem_flag_enable(bm, e, EDGE_VIS);
-			BLI_array_growone(edges);
+			BLI_array_grow_one(edges);
 			edges[i] = e;
 
 			e = edge_next(bm, e);
@@ -1166,11 +1171,11 @@ void bmo_edgenet_prepare(BMesh *bm, BMOperator *op)
 
 		if (!count) {
 			edges1 = edges;
-			BLI_array_set_length(edges1, BLI_array_count(edges));
+			BLI_array_length_set(edges1, BLI_array_count(edges));
 		}
 		else {
 			edges2 = edges;
-			BLI_array_set_length(edges2, BLI_array_count(edges));
+			BLI_array_length_set(edges2, BLI_array_count(edges));
 		}
 
 		BLI_array_empty(edges);
@@ -1178,10 +1183,10 @@ void bmo_edgenet_prepare(BMesh *bm, BMOperator *op)
 	}
 
 	if (edges1 && BLI_array_count(edges1) > 2 &&
-	    BM_edge_share_vert_count(edges1[0], edges1[BLI_array_count(edges1) - 1]))
+	    BM_edge_share_vert_check(edges1[0], edges1[BLI_array_count(edges1) - 1]))
 	{
 		if (edges2 && BLI_array_count(edges2) > 2 &&
-		    BM_edge_share_vert_count(edges2[0], edges2[BLI_array_count(edges2) - 1]))
+		    BM_edge_share_vert_check(edges2[0], edges2[BLI_array_count(edges2) - 1]))
 		{
 			BLI_array_free(edges1);
 			BLI_array_free(edges2);
@@ -1194,7 +1199,7 @@ void bmo_edgenet_prepare(BMesh *bm, BMOperator *op)
 	}
 
 	if (edges2 && BLI_array_count(edges2) > 2 &&
-	    BM_edge_share_vert_count(edges2[0], edges2[BLI_array_count(edges2) - 1]))
+	    BM_edge_share_vert_check(edges2[0], edges2[BLI_array_count(edges2) - 1]))
 	{
 		edges2 = NULL;
 	}
@@ -1225,7 +1230,7 @@ void bmo_edgenet_prepare(BMesh *bm, BMOperator *op)
 			v4 = BM_vert_in_edge(edges2[i - 1], edges2[i]->v1) ? edges2[i]->v2 : edges2[i]->v1;
 		}
 
-		/* if there is ever bowtie quads between two edges the problem is here! [#30367] */
+		/* if there is ever bow-tie quads between two edges the problem is here! [#30367] */
 #if 0
 		normal_tri_v3(dvec1, v1->co, v2->co, v4->co);
 		normal_tri_v3(dvec2, v1->co, v4->co, v3->co);
@@ -1244,9 +1249,9 @@ void bmo_edgenet_prepare(BMesh *bm, BMOperator *op)
 			SWAP(BMVert *, v3, v4);
 		}
 
-		e = BM_edge_create(bm, v1, v3, NULL, TRUE);
+		e = BM_edge_create(bm, v1, v3, NULL, BM_CREATE_NO_DOUBLE);
 		BMO_elem_flag_enable(bm, e, ELE_NEW);
-		e = BM_edge_create(bm, v2, v4, NULL, TRUE);
+		e = BM_edge_create(bm, v2, v4, NULL, BM_CREATE_NO_DOUBLE);
 		BMO_elem_flag_enable(bm, e, ELE_NEW);
 	}
 	else if (edges1) {
@@ -1256,12 +1261,12 @@ void bmo_edgenet_prepare(BMesh *bm, BMOperator *op)
 			v1 = BM_vert_in_edge(edges1[1], edges1[0]->v1) ? edges1[0]->v2 : edges1[0]->v1;
 			i  = BLI_array_count(edges1) - 1;
 			v2 = BM_vert_in_edge(edges1[i - 1], edges1[i]->v1) ? edges1[i]->v2 : edges1[i]->v1;
-			e  = BM_edge_create(bm, v1, v2, NULL, TRUE);
+			e  = BM_edge_create(bm, v1, v2, NULL, BM_CREATE_NO_DOUBLE);
 			BMO_elem_flag_enable(bm, e, ELE_NEW);
 		}
 	}
 	
-	BMO_slot_buffer_from_enabled_flag(bm, op, "edgeout", BM_EDGE, ELE_NEW);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "edges.out", BM_EDGE, ELE_NEW);
 
 	BLI_array_free(edges1);
 	BLI_array_free(edges2);
@@ -1281,11 +1286,11 @@ void bmo_contextual_create_exec(BMesh *bm, BMOperator *op)
 	BMEdge *e;
 	BMFace *f;
 	int totv = 0, tote = 0, totf = 0, amount;
-	const short mat_nr = BMO_slot_int_get(op, "mat_nr");
-	const short use_smooth = BMO_slot_bool_get(op, "use_smooth");
+	const short mat_nr     = BMO_slot_int_get(op->slots_in,  "mat_nr");
+	const bool use_smooth  = BMO_slot_bool_get(op->slots_in, "use_smooth");
 
 	/* count number of each element type we were passe */
-	BMO_ITER (h, &oiter, bm, op, "geom", BM_VERT|BM_EDGE|BM_FACE) {
+	BMO_ITER (h, &oiter, op->slots_in, "geom", BM_VERT | BM_EDGE | BM_FACE) {
 		switch (h->htype) {
 			case BM_VERT: totv++; break;
 			case BM_EDGE: tote++; break;
@@ -1312,14 +1317,14 @@ void bmo_contextual_create_exec(BMesh *bm, BMOperator *op)
 	 *
 	 */
 
-	/* Here we check for consistancy and create 2 edges */
+	/* Here we check for consistency and create 2 edges */
 	if (totf == 0 && totv >= 4 && totv == tote + 2) {
 		/* find a free standing vertex and 2 endpoint verts */
 		BMVert *v_free = NULL, *v_a = NULL, *v_b = NULL;
-		int ok = TRUE;
+		bool ok = true;
 
 
-		BMO_ITER (v, &oiter, bm, op, "geom", BM_VERT) {
+		BMO_ITER (v, &oiter, op->slots_in, "geom", BM_VERT) {
 			/* count how many flagged edges this vertex uses */
 			int tot_edges = 0;
 			BM_ITER_ELEM (e, &iter, v, BM_EDGES_OF_VERT) {
@@ -1334,30 +1339,30 @@ void bmo_contextual_create_exec(BMesh *bm, BMOperator *op)
 			if (tot_edges == 0) {
 				/* only accept 1 free vert */
 				if (v_free == NULL)  v_free = v;
-				else                 ok = FALSE;  /* only ever want one of these */
+				else                 ok = false;  /* only ever want one of these */
 			}
 			else if (tot_edges == 1) {
 				if (v_a == NULL)       v_a = v;
 				else if (v_b == NULL)  v_b = v;
-				else                   ok = FALSE;  /* only ever want 2 of these */
+				else                   ok = false;  /* only ever want 2 of these */
 			}
 			else if (tot_edges == 2) {
 				/* do nothing, regular case */
 			}
 			else {
-				ok = FALSE; /* if a vertex has 3+ edge users then cancel - this is only simple cases */
+				ok = false; /* if a vertex has 3+ edge users then cancel - this is only simple cases */
 			}
 
-			if (ok == FALSE) {
+			if (ok == false) {
 				break;
 			}
 		}
 
-		if (ok == TRUE && v_free && v_a && v_b) {
-			e = BM_edge_create(bm, v_free, v_a, NULL, TRUE);
+		if (ok == true && v_free && v_a && v_b) {
+			e = BM_edge_create(bm, v_free, v_a, NULL, BM_CREATE_NO_DOUBLE);
 			BMO_elem_flag_enable(bm, e, ELE_NEW);
 
-			e = BM_edge_create(bm, v_free, v_b, NULL, TRUE);
+			e = BM_edge_create(bm, v_free, v_b, NULL, BM_CREATE_NO_DOUBLE);
 			BMO_elem_flag_enable(bm, e, ELE_NEW);
 		}
 	}
@@ -1365,20 +1370,21 @@ void bmo_contextual_create_exec(BMesh *bm, BMOperator *op)
 
 	/* call edgenet create */
 	/* call edgenet prepare op so additional face creation cases wore */
-	BMO_op_initf(bm, &op2, "edgenet_prepare edges=%fe", ELE_NEW);
+	BMO_op_initf(bm, &op2, op->flag, "edgenet_prepare edges=%fe", ELE_NEW);
 	BMO_op_exec(bm, &op2);
-	BMO_slot_buffer_flag_enable(bm, &op2, "edgeout", BM_EDGE, ELE_NEW);
+	BMO_slot_buffer_flag_enable(bm, op2.slots_out, "edges.out", BM_EDGE, ELE_NEW);
 	BMO_op_finish(bm, &op2);
 
-	BMO_op_initf(bm, &op2,
+	BMO_op_initf(bm, &op2, op->flag,
 	             "edgenet_fill edges=%fe use_fill_check=%b mat_nr=%i use_smooth=%b",
-	             ELE_NEW, TRUE, mat_nr, use_smooth);
+	             ELE_NEW, true, mat_nr, use_smooth);
 
 	BMO_op_exec(bm, &op2);
 
 	/* return if edge net create did something */
-	if (BMO_slot_buffer_count(bm, &op2, "faceout")) {
-		BMO_slot_copy(&op2, op, "faceout", "faceout");
+	if (BMO_slot_buffer_count(op2.slots_out, "faces.out")) {
+		BMO_slot_copy(&op2, slots_out, "faces.out",
+		              op,   slots_out, "faces.out");
 		BMO_op_finish(bm, &op2);
 		return;
 	}
@@ -1386,12 +1392,13 @@ void bmo_contextual_create_exec(BMesh *bm, BMOperator *op)
 	BMO_op_finish(bm, &op2);
 	
 	/* now call dissolve face */
-	BMO_op_initf(bm, &op2, "dissolve_faces faces=%ff", ELE_NEW);
+	BMO_op_initf(bm, &op2, op->flag, "dissolve_faces faces=%ff", ELE_NEW);
 	BMO_op_exec(bm, &op2);
 	
 	/* if we dissolved anything, then return */
-	if (BMO_slot_buffer_count(bm, &op2, "regionout")) {
-		BMO_slot_copy(&op2, op, "regionout", "faceout");
+	if (BMO_slot_buffer_count(op2.slots_out, "region.out")) {
+		BMO_slot_copy(&op2, slots_out, "region.out",
+		              op,   slots_out,  "faces.out");
 		BMO_op_finish(bm, &op2);
 		return;
 	}
@@ -1403,16 +1410,19 @@ void bmo_contextual_create_exec(BMesh *bm, BMOperator *op)
 	BM_ITER_MESH (v, &iter, bm, BM_VERTS_OF_MESH) {
 		if (BMO_elem_flag_test(bm, v, ELE_NEW)) {
 			verts[amount] = v;
+			if (amount == 3) {
+				break;
+			}
 			amount++;
 
-			if (amount > 4) break;
 		}
 	}
 
 	if (amount == 2) {
 		/* create edge */
-		e = BM_edge_create(bm, verts[0], verts[1], NULL, TRUE);
+		e = BM_edge_create(bm, verts[0], verts[1], NULL, BM_CREATE_NO_DOUBLE);
 		BMO_elem_flag_enable(bm, e, ELE_OUT);
+		BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "edges.out", BM_EDGE, ELE_OUT);
 	}
 	else if (0) { /* nice feature but perhaps it should be a different tool? */
 
@@ -1451,13 +1461,14 @@ void bmo_contextual_create_exec(BMesh *bm, BMOperator *op)
 				if (ese->htype == BM_VERT) {
 					v = (BMVert *)ese->ele;
 					if (v_prev) {
-						e = BM_edge_create(bm, v, v_prev, NULL, TRUE);
+						e = BM_edge_create(bm, v, v_prev, NULL, BM_CREATE_NO_DOUBLE);
 						BMO_elem_flag_enable(bm, e, ELE_OUT);
 					}
 					v_prev = v;
 				}
 			}
 		}
+		BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "edges.out", BM_EDGE, ELE_OUT);
 		/* done creating edges */
 	}
 	else if (amount > 2) {
@@ -1467,12 +1478,12 @@ void bmo_contextual_create_exec(BMesh *bm, BMOperator *op)
 		BMVert **vert_arr = MEM_mallocN(sizeof(BMVert **) * totv, __func__);
 		int i = 0;
 
-		BMO_ITER (v, &oiter, bm, op, "geom", BM_VERT) {
+		BMO_ITER (v, &oiter, op->slots_in, "geom", BM_VERT) {
 			vert_arr[i] = v;
 			i++;
 		}
 
-		f = BM_face_create_ngon_vcloud(bm, vert_arr, totv, TRUE);
+		f = BM_face_create_ngon_vcloud(bm, vert_arr, totv, BM_CREATE_NO_DOUBLE);
 
 		if (f) {
 			BMO_elem_flag_enable(bm, f, ELE_OUT);
