@@ -26,9 +26,13 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "DNA_meshdata_types.h"
+
 #include "BLI_math.h"
 #include "BLI_array.h"
 
+#include "BKE_customdata.h"
+
 #include "bmesh.h"
 
 #include "intern/bmesh_operators_private.h" /* own include */
@@ -45,30 +49,37 @@ enum {
 #define VERT_NONMAN 2
 #define EDGE_NONMAN 2
 
-void bmo_extrude_face_indiv_exec(BMesh *bm, BMOperator *op)
+void bmo_extrude_discrete_faces_exec(BMesh *bm, BMOperator *op)
 {
+	BMVert **verts = NULL;
+	BLI_array_declare(verts);
+	BMEdge **edges = NULL;
+	BLI_array_declare(edges);
+
 	BMOIter siter;
 	BMIter liter, liter2;
 	BMFace *f, *f2, *f3;
 	BMLoop *l, *l2, *l3, *l4, *l_tmp;
-	BMEdge **edges = NULL, *e, *laste;
+	BMEdge *e, *laste;
 	BMVert *v, *lastv, *firstv;
-	BLI_array_declare(edges);
 	int i;
 
-	BMO_ITER (f, &siter, bm, op, "faces", BM_FACE) {
+	BMO_ITER (f, &siter, op->slots_in, "faces", BM_FACE) {
+		BLI_array_empty(verts);
 		BLI_array_empty(edges);
-		BLI_array_growitems(edges, f->len);
+		BLI_array_grow_items(verts, f->len);
+		BLI_array_grow_items(edges, f->len);
 
 		i = 0;
 		firstv = lastv = NULL;
 		BM_ITER_ELEM (l, &liter, f, BM_LOOPS_OF_FACE) {
-			v = BM_vert_create(bm, l->v->co, l->v);
-
+			v = BM_vert_create(bm, l->v->co, l->v, 0);
 			/* skip on the first iteration */
 			if (lastv) {
-				e = BM_edge_create(bm, lastv, v, l->e, FALSE);
-				edges[i++] = e;
+				e = BM_edge_create(bm, lastv, v, l->e, 0);
+				edges[i] = e;
+				verts[i] = lastv;
+				i++;
 			}
 
 			lastv = v;
@@ -77,14 +88,16 @@ void bmo_extrude_face_indiv_exec(BMesh *bm, BMOperator *op)
 		}
 
 		/* this fits in the array because we skip one in the loop above */
-		e = BM_edge_create(bm, v, firstv, laste, FALSE);
-		edges[i++] = e;
+		e = BM_edge_create(bm, v, firstv, laste, 0);
+		edges[i] = e;
+		verts[i] = lastv;
+		i++;
 
 		BMO_elem_flag_enable(bm, f, EXT_DEL);
 
-		f2 = BM_face_create_ngon(bm, firstv, BM_edge_other_vert(edges[0], firstv), edges, f->len, FALSE);
-		if (!f2) {
-			BMO_error_raise(bm, op, BMERR_MESH_ERROR, "Extrude failed; could not create face");
+		f2 = BM_face_create(bm, verts, edges, f->len, 0);
+		if (UNLIKELY(f2 == NULL)) {
+			BMO_error_raise(bm, op, BMERR_MESH_ERROR, "Extrude failed: could not create face");
 			BLI_array_free(edges);
 			return;
 		}
@@ -99,7 +112,8 @@ void bmo_extrude_face_indiv_exec(BMesh *bm, BMOperator *op)
 			l3 = l->next;
 			l4 = l2->next;
 
-			f3 = BM_face_create_quad_tri(bm, l3->v, l4->v, l2->v, l->v, f, FALSE);
+			f3 = BM_face_create_quad_tri(bm, l3->v, l4->v, l2->v, l->v, f, false);
+			/* XXX, no error check here, why? - Campbell */
 
 			l_tmp = BM_FACE_FIRST_LOOP(f3);
 
@@ -112,10 +126,13 @@ void bmo_extrude_face_indiv_exec(BMesh *bm, BMOperator *op)
 		}
 	}
 
+	BLI_array_free(verts);
 	BLI_array_free(edges);
 
-	BMO_op_callf(bm, "del geom=%ff context=%i", EXT_DEL, DEL_ONLYFACES);
-	BMO_slot_buffer_from_enabled_flag(bm, op, "faceout", BM_FACE, EXT_KEEP);
+	BMO_op_callf(bm, op->flag,
+	             "delete geom=%ff context=%i",
+	             EXT_DEL, DEL_ONLYFACES);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "faces.out", BM_FACE, EXT_KEEP);
 }
 
 /**
@@ -125,87 +142,94 @@ void bmo_extrude_face_indiv_exec(BMesh *bm, BMOperator *op)
  * This function won't crash if its not but won't work right either.
  * \a e_b is the new edge.
  *
- * \note this function could be exposed as an api call if other areas need it,
- * so far only extrude does.
+ * \note The edge this face comes from needs to be from the first and second verts fo the face.
+ * The caller must ensure this else we will copy from the wrong source.
  */
-static void bm_extrude_copy_face_loop_attributes(BMesh *bm, BMFace *f, BMEdge *e_a, BMEdge *e_b)
+static void bm_extrude_copy_face_loop_attributes(BMesh *bm, BMFace *f)
 {
-	/* 'a' is the starting edge #e, 'b' is the final edge #newedge */
-	BMLoop *l_dst_a = BM_face_edge_share_loop(f, e_a);
-	BMLoop *l_dst_b = BM_face_edge_share_loop(f, e_b);
-	/* we could only have a face on one-or the other edges,
-	 * check if either side of the face has an adjacent face */
-	BMLoop *l_src_1;
-	BMLoop *l_src_2;
+	/* edge we are extruded from */
+	BMLoop *l_first_0 = BM_FACE_FIRST_LOOP(f);
+	BMLoop *l_first_1 = l_first_0->next;
+	BMLoop *l_first_2 = l_first_1->next;
+	BMLoop *l_first_3 = l_first_2->next;
 
-	/* there is no l_src_b */
+	BMLoop *l_other_0;
+	BMLoop *l_other_1;
 
-	/* sanity */
-	BLI_assert(l_dst_a->f == l_dst_b->f);
-
-	if (l_dst_a != l_dst_a->radial_next) {
-		l_src_1 = l_dst_a->radial_next;
-		l_src_2 = l_src_1->next;
-	}
-	else if (l_dst_b != l_dst_b->radial_next) {
-		l_src_2 = l_dst_b->radial_next;
-		l_src_1 = l_src_2->next;
-	}
-	else {
-		/* no new faces on either edge, nothing to copy from */
+	if (UNLIKELY(l_first_0 == l_first_0->radial_next)) {
 		return;
 	}
 
-	BM_elem_attrs_copy(bm, bm, l_src_1->f, l_dst_a->f);
-	BM_elem_flag_disable(f, BM_ELEM_HIDDEN); /* possibly we copy from a hidden face */
+	l_other_0 = BM_edge_other_loop(l_first_0->e, l_first_0);
+	l_other_1 = BM_edge_other_loop(l_first_0->e, l_first_1);
 
 	/* copy data */
-	BM_elem_attrs_copy(bm, bm, l_src_2, l_dst_a);
-	BM_elem_attrs_copy(bm, bm, l_src_2, l_dst_b->next);
+	BM_elem_attrs_copy(bm, bm, l_other_0->f, f);
+	BM_elem_flag_disable(f, BM_ELEM_HIDDEN);  /* possibly we copy from a hidden face */
 
-	BM_elem_attrs_copy(bm, bm, l_src_1, l_dst_a->next);
-	BM_elem_attrs_copy(bm, bm, l_src_1, l_dst_b);
+	BM_elem_attrs_copy(bm, bm, l_other_0, l_first_0);
+	BM_elem_attrs_copy(bm, bm, l_other_0, l_first_3);
+
+	BM_elem_attrs_copy(bm, bm, l_other_1, l_first_1);
+	BM_elem_attrs_copy(bm, bm, l_other_1, l_first_2);
+}
+
+/* Disable the skin root flag on the input vert, assumes that the vert
+ * data includes an CD_MVERT_SKIN layer */
+static void bm_extrude_disable_skin_root(BMesh *bm, BMVert *v)
+{
+	MVertSkin *vs;
+	
+	vs = CustomData_bmesh_get(&bm->vdata, v->head.data, CD_MVERT_SKIN);
+	vs->flag &= ~MVERT_SKIN_ROOT;
 }
 
 void bmo_extrude_edge_only_exec(BMesh *bm, BMOperator *op)
 {
 	BMOIter siter;
 	BMOperator dupeop;
-	BMVert *v1, *v2, *v3, *v4;
-	BMEdge *e, *e2;
 	BMFace *f;
+	BMEdge *e, *e_new;
 	
-	BMO_ITER (e, &siter, bm, op, "edges", BM_EDGE) {
+	BMO_ITER (e, &siter, op->slots_in, "edges", BM_EDGE) {
 		BMO_elem_flag_enable(bm, e, EXT_INPUT);
 		BMO_elem_flag_enable(bm, e->v1, EXT_INPUT);
 		BMO_elem_flag_enable(bm, e->v2, EXT_INPUT);
 	}
 
-	BMO_op_initf(bm, &dupeop, "dupe geom=%fve", EXT_INPUT);
+	BMO_op_initf(bm, &dupeop, op->flag, "duplicate geom=%fve", EXT_INPUT);
 	BMO_op_exec(bm, &dupeop);
 
-	for (e = BMO_iter_new(&siter, bm, &dupeop, "boundarymap", 0); e; e = BMO_iter_step(&siter)) {
-		e2 = BMO_iter_map_value(&siter);
-		e2 = *(BMEdge **)e2;
+	/* disable root flag on all new skin nodes */
+	if (CustomData_has_layer(&bm->vdata, CD_MVERT_SKIN)) {
+		BMVert *v;
+		BMO_ITER(v, &siter, dupeop.slots_out, "geom.out", BM_VERT) {
+			bm_extrude_disable_skin_root(bm, v);
+		}
+	}
+
+	for (e = BMO_iter_new(&siter, dupeop.slots_out, "boundary_map.out", 0); e; e = BMO_iter_step(&siter)) {
+		BMVert *f_verts[4];
+		e_new = *(BMEdge **)BMO_iter_map_value(&siter);
 
 		if (e->l && e->v1 != e->l->v) {
-			v1 = e->v1;
-			v2 = e->v2;
-			v3 = e2->v2;
-			v4 = e2->v1;
+			f_verts[0] = e->v1;
+			f_verts[1] = e->v2;
+			f_verts[2] = e_new->v2;
+			f_verts[3] = e_new->v1;
 		}
 		else {
-			v1 = e2->v1;
-			v2 = e2->v2;
-			v3 = e->v2;
-			v4 = e->v1;
+			f_verts[0] = e->v2;
+			f_verts[1] = e->v1;
+			f_verts[2] = e_new->v1;
+			f_verts[3] = e_new->v2;
 		}
 		/* not sure what to do about example face, pass NULL for now */
-		f = BM_face_create_quad_tri(bm, v1, v2, v3, v4, NULL, FALSE);
-		bm_extrude_copy_face_loop_attributes(bm, f, e, e2);
+		f = BM_face_create_quad_tri_v(bm, f_verts, 4, NULL, false);
+		bm_extrude_copy_face_loop_attributes(bm, f);
 		
 		if (BMO_elem_flag_test(bm, e, EXT_INPUT))
-			e = e2;
+			e = e_new;
 		
 		BMO_elem_flag_enable(bm, f, EXT_KEEP);
 		BMO_elem_flag_enable(bm, e, EXT_KEEP);
@@ -216,7 +240,7 @@ void bmo_extrude_edge_only_exec(BMesh *bm, BMOperator *op)
 
 	BMO_op_finish(bm, &dupeop);
 
-	BMO_slot_buffer_from_enabled_flag(bm, op, "geomout", BM_ALL, EXT_KEEP);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "geom.out", BM_ALL_NOLOOP, EXT_KEEP);
 }
 
 void bmo_extrude_vert_indiv_exec(BMesh *bm, BMOperator *op)
@@ -224,18 +248,21 @@ void bmo_extrude_vert_indiv_exec(BMesh *bm, BMOperator *op)
 	BMOIter siter;
 	BMVert *v, *dupev;
 	BMEdge *e;
+	const bool has_vskin = CustomData_has_layer(&bm->vdata, CD_MVERT_SKIN);
 
-	for (v = BMO_iter_new(&siter, bm, op, "verts", BM_VERT); v; v = BMO_iter_step(&siter)) {
-		dupev = BM_vert_create(bm, v->co, v);
+	for (v = BMO_iter_new(&siter, op->slots_in, "verts", BM_VERT); v; v = BMO_iter_step(&siter)) {
+		dupev = BM_vert_create(bm, v->co, v, 0);
+		if (has_vskin)
+			bm_extrude_disable_skin_root(bm, v);
 
-		e = BM_edge_create(bm, v, dupev, NULL, FALSE);
+		e = BM_edge_create(bm, v, dupev, NULL, 0);
 
 		BMO_elem_flag_enable(bm, e, EXT_KEEP);
 		BMO_elem_flag_enable(bm, dupev, EXT_KEEP);
 	}
 
-	BMO_slot_buffer_from_enabled_flag(bm, op, "vertout", BM_VERT, EXT_KEEP);
-	BMO_slot_buffer_from_enabled_flag(bm, op, "edgeout", BM_EDGE, EXT_KEEP);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "verts.out", BM_VERT, EXT_KEEP);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "edges.out", BM_EDGE, EXT_KEEP);
 }
 
 void bmo_extrude_face_region_exec(BMesh *bm, BMOperator *op)
@@ -243,19 +270,21 @@ void bmo_extrude_face_region_exec(BMesh *bm, BMOperator *op)
 	BMOperator dupeop, delop;
 	BMOIter siter;
 	BMIter iter, fiter, viter;
-	BMEdge *e, *newedge;
-	BMVert *verts[4], *v, *v2;
+	BMEdge *e, *e_new;
+	BMVert *v, *v2;
 	BMFace *f;
-	int found, fwd, delorig = FALSE;
+	bool found, fwd, delorig = false;
+	BMOpSlot *slot_facemap_out;
+	BMOpSlot *slot_edges_exclude;
 
 	/* initialize our sub-operators */
-	BMO_op_init(bm, &dupeop, "dupe");
+	BMO_op_init(bm, &dupeop, op->flag, "duplicate");
 	
-	BMO_slot_buffer_flag_enable(bm, op, "edgefacein", BM_EDGE|BM_FACE, EXT_INPUT);
+	BMO_slot_buffer_flag_enable(bm, op->slots_in, "geom", BM_EDGE | BM_FACE, EXT_INPUT);
 	
 	/* if one flagged face is bordered by an un-flagged face, then we delete
 	 * original geometry unless caller explicitly asked to keep it. */
-	if (!BMO_slot_bool_get(op, "alwayskeeporig")) {
+	if (!BMO_slot_bool_get(op->slots_in, "use_keep_orig")) {
 		BM_ITER_MESH (e, &iter, bm, BM_EDGES_OF_MESH) {
 
 			int edge_face_tot;
@@ -264,20 +293,20 @@ void bmo_extrude_face_region_exec(BMesh *bm, BMOperator *op)
 				continue;
 			}
 
-			found = FALSE; /* found a face that isn't input? */
+			found = false; /* found a face that isn't input? */
 			edge_face_tot = 0; /* edge/face count */
 
 			BM_ITER_ELEM (f, &fiter, e, BM_FACES_OF_EDGE) {
 				if (!BMO_elem_flag_test(bm, f, EXT_INPUT)) {
-					found = TRUE;
-					delorig = TRUE;
+					found = true;
+					delorig = true;
 					break;
 				}
 
 				edge_face_tot++;
 			}
 
-			if ((edge_face_tot > 1) && (found == FALSE)) {
+			if ((edge_face_tot > 1) && (found == false)) {
 				/* edge has a face user, that face isn't extrude input */
 				BMO_elem_flag_enable(bm, e, EXT_DEL);
 			}
@@ -286,27 +315,29 @@ void bmo_extrude_face_region_exec(BMesh *bm, BMOperator *op)
 
 	/* calculate verts to delete */
 	BM_ITER_MESH (v, &iter, bm, BM_VERTS_OF_MESH) {
-		found = FALSE;
+		if (v->e) {  /* only deal with verts attached to geometry [#33651] */
+			found = false;
 
-		BM_ITER_ELEM (e, &viter, v, BM_EDGES_OF_VERT) {
-			if (!BMO_elem_flag_test(bm, e, EXT_INPUT) || !BMO_elem_flag_test(bm, e, EXT_DEL)) {
-				found = TRUE;
-				break;
-			}
-		}
-
-		/* avoid an extra loop */
-		if (found == TRUE) {
-			BM_ITER_ELEM (f, &viter, v, BM_FACES_OF_VERT) {
-				if (!BMO_elem_flag_test(bm, f, EXT_INPUT)) {
-					found = TRUE;
+			BM_ITER_ELEM (e, &viter, v, BM_EDGES_OF_VERT) {
+				if (!BMO_elem_flag_test(bm, e, EXT_INPUT) || !BMO_elem_flag_test(bm, e, EXT_DEL)) {
+					found = true;
 					break;
 				}
 			}
-		}
 
-		if (found == FALSE) {
-			BMO_elem_flag_enable(bm, v, EXT_DEL);
+			/* avoid an extra loop */
+			if (found == true) {
+				BM_ITER_ELEM (f, &viter, v, BM_FACES_OF_VERT) {
+					if (!BMO_elem_flag_test(bm, f, EXT_INPUT)) {
+						found = true;
+						break;
+					}
+				}
+			}
+
+			if (found == false) {
+				BMO_elem_flag_enable(bm, v, EXT_DEL);
+			}
 		}
 	}
 	
@@ -316,16 +347,27 @@ void bmo_extrude_face_region_exec(BMesh *bm, BMOperator *op)
 		}
 	}
 
-	if (delorig == TRUE) {
-		BMO_op_initf(bm, &delop, "del geom=%fvef context=%i",
+	if (delorig == true) {
+		BMO_op_initf(bm, &delop, op->flag,
+		             "delete geom=%fvef context=%i",
 		             EXT_DEL, DEL_ONLYTAGGED);
 	}
 
-	BMO_slot_copy(op, &dupeop, "edgefacein", "geom");
+	BMO_slot_copy(op,      slots_in, "geom",
+	              &dupeop, slots_in, "geom");
 	BMO_op_exec(bm, &dupeop);
 
-	if (bm->act_face && BMO_elem_flag_test(bm, bm->act_face, EXT_INPUT))
-		bm->act_face = BMO_slot_map_ptr_get(bm, &dupeop, "facemap", bm->act_face);
+	/* disable root flag on all new skin nodes */
+	if (CustomData_has_layer(&bm->vdata, CD_MVERT_SKIN)) {
+		BMO_ITER(v, &siter, dupeop.slots_out, "geom.out", BM_VERT) {
+			bm_extrude_disable_skin_root(bm, v);
+		}
+	}
+
+	slot_facemap_out = BMO_slot_get(dupeop.slots_out, "face_map.out");
+	if (bm->act_face && BMO_elem_flag_test(bm, bm->act_face, EXT_INPUT)) {
+		bm->act_face = BMO_slot_map_elem_get(slot_facemap_out, bm->act_face);
+	}
 
 	if (delorig) {
 		BMO_op_exec(bm, &delop);
@@ -340,56 +382,67 @@ void bmo_extrude_face_region_exec(BMesh *bm, BMOperator *op)
 		}
 	}
 	
-	BMO_slot_copy(&dupeop, op, "newout", "geomout");
+	BMO_slot_copy(&dupeop, slots_out, "geom.out",
+	              op,      slots_out, "geom.out");
 
-	for (e = BMO_iter_new(&siter, bm, &dupeop, "boundarymap", 0); e; e = BMO_iter_step(&siter)) {
+	slot_edges_exclude = BMO_slot_get(op->slots_in, "edges_exclude");
+	for (e = BMO_iter_new(&siter, dupeop.slots_out, "boundary_map.out", 0); e; e = BMO_iter_step(&siter)) {
+		BMVert *f_verts[4];
 
 		/* this should always be wire, so this is mainly a speedup to avoid map lookup */
-		if (BM_edge_is_wire(e) && BMO_slot_map_contains(bm, op, "exclude", e)) {
+		if (BM_edge_is_wire(e) && BMO_slot_map_contains(slot_edges_exclude, e)) {
+			BMVert *v1 = e->v1, *v2 = e->v2;
+
 			/* The original edge was excluded,
 			 * this would result in a standalone wire edge - see [#30399] */
 			BM_edge_kill(bm, e);
 
+			/* kill standalone vertices from this edge - see [#32341] */
+			if (!v1->e)
+				BM_vert_kill(bm, v1);
+			if (!v2->e)
+				BM_vert_kill(bm, v2);
+
 			continue;
 		}
 
-		newedge = *(BMEdge **)BMO_iter_map_value(&siter);
+		e_new = *(BMEdge **)BMO_iter_map_value(&siter);
 
-		if (!newedge) {
+		if (!e_new) {
 			continue;
 		}
 
 		/* orient loop to give same normal as a loop of newedge
 		 * if it exists (will be an extruded face),
 		 * else same normal as a loop of e, if it exists */
-		if (!newedge->l)
+		if (!e_new->l)
 			fwd = !e->l || !(e->l->v == e->v1);
 		else
-			fwd = (newedge->l->v == newedge->v1);
+			fwd = (e_new->l->v == e_new->v1);
 
 		
 		if (fwd) {
-			verts[0] = e->v1;
-			verts[1] = e->v2;
-			verts[2] = newedge->v2;
-			verts[3] = newedge->v1;
+			f_verts[0] = e->v1;
+			f_verts[1] = e->v2;
+			f_verts[2] = e_new->v2;
+			f_verts[3] = e_new->v1;
 		}
 		else {
-			verts[3] = e->v1;
-			verts[2] = e->v2;
-			verts[1] = newedge->v2;
-			verts[0] = newedge->v1;
+			f_verts[0] = e->v2;
+			f_verts[1] = e->v1;
+			f_verts[2] = e_new->v1;
+			f_verts[3] = e_new->v2;
 		}
 
 		/* not sure what to do about example face, pass NULL for now */
-		f = BM_face_create_quad_tri_v(bm, verts, 4, NULL, FALSE);
-		bm_extrude_copy_face_loop_attributes(bm, f, e, newedge);
+		f = BM_face_create_quad_tri_v(bm, f_verts, 4, NULL, false);
+		bm_extrude_copy_face_loop_attributes(bm, f);
 	}
 
 	/* link isolated vert */
-	for (v = BMO_iter_new(&siter, bm, &dupeop, "isovertmap", 0); v; v = BMO_iter_step(&siter)) {
+	for (v = BMO_iter_new(&siter, dupeop.slots_out, "isovert_map.out", 0); v; v = BMO_iter_step(&siter)) {
 		v2 = *((void **)BMO_iter_map_value(&siter));
-		BM_edge_create(bm, v, v2, v->e, TRUE);
+		BM_edge_create(bm, v, v2, v->e, BM_CREATE_NO_DOUBLE);
 	}
 
 	/* cleanup */
@@ -559,39 +612,31 @@ static void solidify_add_thickness(BMesh *bm, const float dist)
 	float *vert_accum = vert_angles + bm->totvert;
 	int i, index;
 
-	/* array for passing verts to angle_poly_v3 */
-	float **verts = NULL;
-	BLI_array_staticdeclare(verts, BM_NGON_STACK_SIZE);
-	/* array for receiving angles from angle_poly_v3 */
-	float *face_angles = NULL;
-	BLI_array_staticdeclare(face_angles, BM_NGON_STACK_SIZE);
-
 	BM_mesh_elem_index_ensure(bm, BM_VERT);
 
 	BM_ITER_MESH (f, &iter, bm, BM_FACES_OF_MESH) {
-		if (!BMO_elem_flag_test(bm, f, FACE_MARK)) {
-			continue;
+		if (BMO_elem_flag_test(bm, f, FACE_MARK)) {
+
+			/* array for passing verts to angle_poly_v3 */
+			float  *face_angles = BLI_array_alloca(face_angles, f->len);
+			/* array for receiving angles from angle_poly_v3 */
+			float **verts = BLI_array_alloca(verts, f->len);
+
+			BM_ITER_ELEM_INDEX (l, &loopIter, f, BM_LOOPS_OF_FACE, i) {
+				verts[i] = l->v->co;
+			}
+
+			angle_poly_v3(face_angles, (const float **)verts, f->len);
+
+			i = 0;
+			BM_ITER_ELEM (l, &loopIter, f, BM_LOOPS_OF_FACE) {
+				v = l->v;
+				index = BM_elem_index_get(v);
+				vert_accum[index] += face_angles[i];
+				vert_angles[index] += shell_angle_to_dist(angle_normalized_v3v3(v->no, f->no)) * face_angles[i];
+				i++;
+			}
 		}
-
-		BLI_array_growitems(verts, f->len);
-		BM_ITER_ELEM_INDEX (l, &loopIter, f, BM_LOOPS_OF_FACE, i) {
-			verts[i] = l->v->co;
-		}
-
-		BLI_array_growitems(face_angles, f->len);
-		angle_poly_v3(face_angles, (const float **)verts, f->len);
-
-		i = 0;
-		BM_ITER_ELEM (l, &loopIter, f, BM_LOOPS_OF_FACE) {
-			v = l->v;
-			index = BM_elem_index_get(v);
-			vert_accum[index] += face_angles[i];
-			vert_angles[index] += shell_angle_to_dist(angle_normalized_v3v3(v->no, f->no)) * face_angles[i];
-			i++;
-		}
-
-		BLI_array_empty(verts);
-		BLI_array_empty(face_angles);
 	}
 
 	BM_ITER_MESH (v, &iter, bm, BM_VERTS_OF_MESH) {
@@ -602,9 +647,6 @@ static void solidify_add_thickness(BMesh *bm, const float dist)
 	}
 
 	MEM_freeN(vert_angles);
-
-	BLI_array_free(verts);
-	BLI_array_free(face_angles);
 }
 
 void bmo_solidify_face_region_exec(BMesh *bm, BMOperator *op)
@@ -613,25 +655,28 @@ void bmo_solidify_face_region_exec(BMesh *bm, BMOperator *op)
 	BMOperator reverseop;
 	float thickness;
 
-	thickness = BMO_slot_float_get(op, "thickness");
+	thickness = BMO_slot_float_get(op->slots_in, "thickness");
 
 	/* Flip original faces (so the shell is extruded inward) */
-	BMO_op_init(bm, &reverseop, "reversefaces");
-	BMO_slot_copy(op, &reverseop, "geom", "faces");
+	BMO_op_init(bm, &reverseop, op->flag, "reverse_faces");
+	BMO_slot_copy(op,         slots_in, "geom",
+	              &reverseop, slots_in, "faces");
 	BMO_op_exec(bm, &reverseop);
 	BMO_op_finish(bm, &reverseop);
 
 	/* Extrude the region */
-	BMO_op_initf(bm, &extrudeop, "extrude_face_region alwayskeeporig=%b", TRUE);
-	BMO_slot_copy(op, &extrudeop, "geom", "edgefacein");
+	BMO_op_initf(bm, &extrudeop, op->flag, "extrude_face_region use_keep_orig=%b", true);
+	BMO_slot_copy(op,         slots_in, "geom",
+	              &extrudeop, slots_in, "geom");
 	BMO_op_exec(bm, &extrudeop);
 
 	/* Push the verts of the extruded faces inward to create thickness */
-	BMO_slot_buffer_flag_enable(bm, &extrudeop, "geomout", BM_FACE, FACE_MARK);
+	BMO_slot_buffer_flag_enable(bm, extrudeop.slots_out, "geom.out", BM_FACE, FACE_MARK);
 	calc_solidify_normals(bm);
 	solidify_add_thickness(bm, thickness);
 
-	BMO_slot_copy(&extrudeop, op, "geomout", "geomout");
+	BMO_slot_copy(&extrudeop, slots_out, "geom.out",
+	              op,         slots_out, "geom.out");
 
 	BMO_op_finish(bm, &extrudeop);
 }
