@@ -80,23 +80,6 @@ static BMLoop *bm_edge_is_mixed_face_tag(BMLoop *l)
 	}
 }
 
-float bm_vert_avg_tag_dist(BMVert *v)
-{
-	BMIter iter;
-	BMEdge *e;
-	int tot;
-	float length = 0.0f;
-
-	BM_ITER_ELEM_INDEX (e, &iter, v, BM_EDGES_OF_VERT, tot) {
-		BMVert *v_other = BM_edge_other_vert(e, v);
-		if (BM_elem_flag_test(v_other, BM_ELEM_TAG)) {
-			length += BM_edge_calc_length(e);
-		}
-	}
-
-	return length / (float)tot;
-}
-
 /**
  * implementation is as follows...
  *
@@ -109,13 +92,13 @@ float bm_vert_avg_tag_dist(BMVert *v)
 
 void bmo_inset_exec(BMesh *bm, BMOperator *op)
 {
-	const int use_outset          = BMO_slot_bool_get(op, "use_outset");
-	const int use_boundary        = BMO_slot_bool_get(op, "use_boundary") && (use_outset == FALSE);
-	const int use_even_offset     = BMO_slot_bool_get(op, "use_even_offset");
-	const int use_even_boundry    = use_even_offset; /* could make own option */
-	const int use_relative_offset = BMO_slot_bool_get(op, "use_relative_offset");
-	const float thickness         = BMO_slot_float_get(op, "thickness");
-	const float depth             = BMO_slot_float_get(op, "depth");
+	const bool use_outset          = BMO_slot_bool_get(op->slots_in, "use_outset");
+	const bool use_boundary        = BMO_slot_bool_get(op->slots_in, "use_boundary") && (use_outset == false);
+	const bool use_even_offset     = BMO_slot_bool_get(op->slots_in, "use_even_offset");
+	const bool use_even_boundry    = use_even_offset; /* could make own option */
+	const bool use_relative_offset = BMO_slot_bool_get(op->slots_in, "use_relative_offset");
+	const float thickness          = BMO_slot_float_get(op->slots_in, "thickness");
+	const float depth              = BMO_slot_float_get(op->slots_in, "depth");
 
 	int edge_info_len = 0;
 
@@ -128,13 +111,13 @@ void bmo_inset_exec(BMesh *bm, BMOperator *op)
 	BMFace *f;
 	int i, j, k;
 
-	if (use_outset == FALSE) {
-		BM_mesh_elem_hflag_disable_all(bm, BM_FACE, BM_ELEM_TAG, FALSE);
-		BMO_slot_buffer_hflag_enable(bm, op, "faces", BM_FACE, BM_ELEM_TAG, FALSE);
+	if (use_outset == false) {
+		BM_mesh_elem_hflag_disable_all(bm, BM_FACE, BM_ELEM_TAG, false);
+		BMO_slot_buffer_hflag_enable(bm, op->slots_in, "faces", BM_FACE, BM_ELEM_TAG, false);
 	}
 	else {
-		BM_mesh_elem_hflag_enable_all(bm, BM_FACE, BM_ELEM_TAG, FALSE);
-		BMO_slot_buffer_hflag_disable(bm, op, "faces", BM_FACE, BM_ELEM_TAG, FALSE);
+		BM_mesh_elem_hflag_enable_all(bm, BM_FACE, BM_ELEM_TAG, false);
+		BMO_slot_buffer_hflag_disable(bm, op->slots_in, "faces", BM_FACE, BM_ELEM_TAG, false);
 	}
 
 	/* first count all inset edges we will split */
@@ -198,7 +181,7 @@ void bmo_inset_exec(BMesh *bm, BMOperator *op)
 
 		if (es->e_new == es->e_old) { /* happens on boundary edges */
 			/* take care here, we're creating this double edge which _must_ have its verts replaced later on */
-			es->e_old = BM_edge_create(bm, es->e_new->v1, es->e_new->v2, es->e_new, FALSE);
+			es->e_old = BM_edge_create(bm, es->e_new->v1, es->e_new->v2, es->e_new, 0);
 		}
 
 		/* store index back to original in 'edge_info' */
@@ -222,7 +205,7 @@ void bmo_inset_exec(BMesh *bm, BMOperator *op)
 		v1 = BM_vert_create(bm, tvec, NULL);
 		v2 = BM_vert_create(bm, tvec, NULL);
 		madd_v3_v3fl(v2->co, es->no, 0.1f);
-		BM_edge_create(bm, v1, v2, NULL, FALSE);
+		BM_edge_create(bm, v1, v2, NULL, 0);
 	}
 #endif
 
@@ -297,10 +280,11 @@ void bmo_inset_exec(BMesh *bm, BMOperator *op)
 							BMFace *f_b = e_info_b->l->f;
 
 							/* we use this as either the normal OR to find the right direction for the
-							 * crpss product between both face normals */
+							 * cross product between both face normals */
 							add_v3_v3v3(tvec, e_info_a->no, e_info_b->no);
 
-							if ((f_a == f_b) || compare_v3v3(f_a->no, f_b->no, 0.00001f)) {
+							/* epsilon increased to fix [#32329] */
+							if ((f_a == f_b) || compare_v3v3(f_a->no, f_b->no, 0.001f)) {
 								normalize_v3(tvec);
 							}
 							else {
@@ -330,7 +314,8 @@ void bmo_inset_exec(BMesh *bm, BMOperator *op)
 
 							/* scale by edge angle */
 							if (use_even_offset) {
-								mul_v3_fl(tvec, shell_angle_to_dist(angle_normalized_v3v3(e_info_a->no, e_info_b->no) / 2.0f));
+								mul_v3_fl(tvec, shell_angle_to_dist(angle_normalized_v3v3(e_info_a->no,
+								                                                          e_info_b->no) / 2.0f));
 							}
 
 							/* scale relative to edge lengths */
@@ -426,11 +411,11 @@ void bmo_inset_exec(BMesh *bm, BMOperator *op)
 
 					/* this saves expensive/slow glue check for common cases */
 					if (r_vout_len > 2) {
-						int ok = TRUE;
+						bool ok = true;
 						/* last step, NULL this vertex if has a tagged face */
 						BM_ITER_ELEM (f, &iter, v_split, BM_FACES_OF_VERT) {
 							if (BM_elem_flag_test(f, BM_ELEM_TAG)) {
-								ok = FALSE;
+								ok = false;
 								break;
 							}
 						}
@@ -486,17 +471,54 @@ void bmo_inset_exec(BMesh *bm, BMOperator *op)
 #endif
 		/* no need to check doubles, we KNOW there won't be any */
 		/* yes - reverse face is correct in this case */
-		f = BM_face_create_quad_tri_v(bm, varr, j, es->l->f, FALSE);
+		f = BM_face_create_quad_tri_v(bm, varr, j, es->l->f, false);
 		BMO_elem_flag_enable(bm, f, ELE_NEW);
 
 		/* copy for loop data, otherwise UV's and vcols are no good.
 		 * tiny speedup here we could be more clever and copy from known adjacent data
 		 * also - we could attempt to interpolate the loop data, this would be much slower but more useful too */
+#if 0
+		/* don't use this because face boundaries have no adjacent loops and won't be filled in.
+		 * instead copy from the opposite side with the code below */
 		BM_face_copy_shared(bm, f);
+#else
+		{
+			/* 2 inner loops on the edge between the new face and the original */
+			BMLoop *l_a;
+			BMLoop *l_b;
+			BMLoop *l_a_other;
+			BMLoop *l_b_other;
+
+			l_a = BM_FACE_FIRST_LOOP(f);
+			l_b = l_a->next;
+
+			/* we know this side has a radial_next because of the order of created verts in the quad */
+			l_a_other = BM_edge_other_loop(l_a->e, l_a);
+			l_b_other = BM_edge_other_loop(l_a->e, l_b);
+			BM_elem_attrs_copy(bm, bm, l_a_other, l_a);
+			BM_elem_attrs_copy(bm, bm, l_b_other, l_b);
+
+			/* step around to the opposite side of the quad - warning, this may have no other edges! */
+			l_a = l_a->next->next;
+			l_b = l_a->next;
+			if (!BM_edge_is_boundary(l_a->e)) {
+				/* same as above */
+				l_a_other = BM_edge_other_loop(l_a->e, l_a);
+				l_b_other = BM_edge_other_loop(l_a->e, l_b);
+				BM_elem_attrs_copy(bm, bm, l_a_other, l_a);
+				BM_elem_attrs_copy(bm, bm, l_b_other, l_b);
+			}
+			else {  /* boundary edges have no useful data to copy from, use opposite side of face */
+				/* swap a<->b intentionally */
+				BM_elem_attrs_copy(bm, bm, l_a_other, l_b);
+				BM_elem_attrs_copy(bm, bm, l_b_other, l_a);
+			}
+		}
+#endif
 	}
 
 	/* we could flag new edges/verts too, is it useful? */
-	BMO_slot_buffer_from_enabled_flag(bm, op, "faceout", BM_FACE, ELE_NEW);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "faces.out", BM_FACE, ELE_NEW);
 
 	/* cheap feature to add depth to the inset */
 	if (depth != 0.0f) {
@@ -526,10 +548,10 @@ void bmo_inset_exec(BMesh *bm, BMOperator *op)
 		/* done correcting edge verts normals */
 
 		/* untag verts */
-		BM_mesh_elem_hflag_disable_all(bm, BM_VERT, BM_ELEM_TAG, FALSE);
+		BM_mesh_elem_hflag_disable_all(bm, BM_VERT, BM_ELEM_TAG, false);
 
 		/* tag face verts */
-		BMO_ITER (f, &oiter, bm, op, "faces", BM_FACE) {
+		BMO_ITER (f, &oiter, op->slots_in, "faces", BM_FACE) {
 			BM_ITER_ELEM (v, &iter, f, BM_VERTS_OF_FACE) {
 				BM_elem_flag_enable(v, BM_ELEM_TAG);
 			}
@@ -544,7 +566,7 @@ void bmo_inset_exec(BMesh *bm, BMOperator *op)
 		BM_ITER_MESH_INDEX (v, &iter, bm, BM_VERTS_OF_MESH, i) {
 			if (BM_elem_flag_test(v, BM_ELEM_TAG)) {
 				const float fac = (depth *
-				                   (use_relative_offset ? bm_vert_avg_tag_dist(v) : 1.0f) *
+				                   (use_relative_offset ? BM_vert_calc_mean_tagged_edge_length(v) : 1.0f) *
 				                   (use_even_boundry    ? BM_vert_calc_shell_factor(v) : 1.0f));
 				madd_v3_v3v3fl(varr_co[i], v->co, v->no, fac);
 			}
