@@ -50,7 +50,8 @@ static const char *bc_get_joint_name(T *node)
 }
 
 ArmatureImporter::ArmatureImporter(UnitConverter *conv, MeshImporterBase *mesh, AnimationImporterBase *anim, Scene *sce) :
-	TransformReader(conv), scene(sce), empty(NULL), mesh_importer(mesh), anim_importer(anim) {}
+	TransformReader(conv), scene(sce), empty(NULL), mesh_importer(mesh), anim_importer(anim) {
+}
 
 ArmatureImporter::~ArmatureImporter()
 {
@@ -68,7 +69,7 @@ JointData *ArmatureImporter::get_joint_data(COLLADAFW::Node *node);
 
 	if (joint_id_to_joint_index_map.find(joint_id) == joint_id_to_joint_index_map.end()) {
 		fprintf(stderr, "Cannot find a joint index by joint id for %s.\n",
-				node->getOriginalId().c_str());
+		        node->getOriginalId().c_str());
 		return NULL;
 	}
 
@@ -77,140 +78,66 @@ JointData *ArmatureImporter::get_joint_data(COLLADAFW::Node *node);
 	return &joint_index_to_joint_info_map[joint_index];
 }
 #endif
-void ArmatureImporter::create_unskinned_bone( COLLADAFW::Node *node, EditBone *parent, int totchild,
-				 float parent_mat[][4], Object * ob_arm)
-{
-	std::vector<COLLADAFW::Node*>::iterator it;
-	it = std::find(finished_joints.begin(),finished_joints.end(),node);
-	if ( it != finished_joints.end()) return; 
 
-	float mat[4][4];
-	float obmat[4][4];
-
-	// object-space
-	get_node_mat(obmat, node, NULL, NULL);
-
-	EditBone *bone = ED_armature_edit_bone_add((bArmature*)ob_arm->data, (char*)bc_get_joint_name(node));
-	totbone++;
-	
-	if (parent) bone->parent = parent;
-
-	float angle = 0;
-
-	// get world-space
-	if (parent) {
-		mult_m4_m4m4(mat, parent_mat, obmat);
-
-	}
-	else {
-		copy_m4_m4(mat, obmat);
-
-	}
-	float loc[3], size[3], rot[3][3];
-	mat4_to_loc_rot_size( loc, rot, size, obmat);
-	mat3_to_vec_roll(rot, NULL, &angle );
-	bone->roll=angle;
-	// set head
-	copy_v3_v3(bone->head, mat[3]);
-
-	// set tail, don't set it to head because 0-length bones are not allowed
-	float vec[3] = {0.0f, 0.5f, 0.0f};
-	add_v3_v3v3(bone->tail, bone->head, vec);
-
-	// set parent tail
-	if (parent && totchild == 1) {
-		copy_v3_v3(parent->tail, bone->head);
-		
-		// not setting BONE_CONNECTED because this would lock child bone location with respect to parent
-		// bone->flag |= BONE_CONNECTED;
-	
-		// XXX increase this to prevent "very" small bones?
-		const float epsilon = 0.000001f;
-
-		// derive leaf bone length
-		float length = len_v3v3(parent->head, parent->tail);
-		if ((length < leaf_bone_length || totbone == 0) && length > epsilon) {
-			leaf_bone_length = length;
-		}
-
-		// treat zero-sized bone like a leaf bone
-		if (length <= epsilon) {
-			add_leaf_bone(parent_mat, parent, node);
-		}
-
-	}
-
-	COLLADAFW::NodePointerArray& children = node->getChildNodes();
-	for (unsigned int i = 0; i < children.getCount(); i++) {
-		create_unskinned_bone( children[i], bone, children.getCount(), mat, ob_arm);
-	}
-
-	// in second case it's not a leaf bone, but we handle it the same way
-	if (!children.getCount() || children.getCount() > 1) {
-		add_leaf_bone(mat, bone, node);
-	}
-
-	finished_joints.push_back(node);
-
-}
-
-void ArmatureImporter::create_bone(SkinInfo& skin, COLLADAFW::Node *node, EditBone *parent, int totchild,
-				 float parent_mat[][4], bArmature *arm)
+void ArmatureImporter::create_bone(SkinInfo* skin, COLLADAFW::Node *node, EditBone *parent, int totchild,
+                                   float parent_mat[4][4], bArmature *arm)
 {
 	//Checking if bone is already made.
-	std::vector<COLLADAFW::Node*>::iterator it;
-	it = std::find(finished_joints.begin(),finished_joints.end(),node);
-	if ( it != finished_joints.end()) return; 
+	std::vector<COLLADAFW::Node *>::iterator it;
+	it = std::find(finished_joints.begin(), finished_joints.end(), node);
+	if (it != finished_joints.end()) return;
 
 	float joint_inv_bind_mat[4][4];
 
 	// JointData* jd = get_joint_data(node);
 
 	float mat[4][4];
-
+	float obmat[4][4];
+	
 	// TODO rename from Node "name" attrs later
-	EditBone *bone = ED_armature_edit_bone_add(arm, (char*)bc_get_joint_name(node));
+	EditBone *bone = ED_armature_edit_bone_add(arm, (char *)bc_get_joint_name(node));
 	totbone++;
 
-	if (skin.get_joint_inv_bind_matrix(joint_inv_bind_mat, node)) {
+	if (skin && skin->get_joint_inv_bind_matrix(joint_inv_bind_mat, node)) {
 		// get original world-space matrix
 		invert_m4_m4(mat, joint_inv_bind_mat);
 	}
 	// create a bone even if there's no joint data for it (i.e. it has no influence)
 	else {
-		float obmat[4][4];
-
-		// object-space
+		// bone-space
 		get_node_mat(obmat, node, NULL, NULL);
 
 		// get world-space
-		if (parent)
+		if (parent) {
 			mult_m4_m4m4(mat, parent_mat, obmat);
-		else
+		}
+		else {
 			copy_m4_m4(mat, obmat);
-
-		float loc[3], size[3], rot[3][3] , angle;
-		mat4_to_loc_rot_size( loc, rot, size, obmat);
-		mat3_to_vec_roll(rot, NULL, &angle );
-		bone->roll=angle;
+		}
 	}
 
-	
 	if (parent) bone->parent = parent;
 
+	float loc[3], size[3], rot[3][3]; 
+	float angle;
+	float vec[3] = {0.0f, 0.5f, 0.0f};
+	mat4_to_loc_rot_size(loc, rot, size, mat);
+	//copy_m3_m4(bonemat,mat);
+	mat3_to_vec_roll(rot, vec, &angle);
+
+	bone->roll = angle;
 	// set head
 	copy_v3_v3(bone->head, mat[3]);
 
 	// set tail, don't set it to head because 0-length bones are not allowed
-	float vec[3] = {0.0f, 0.5f, 0.0f};
 	add_v3_v3v3(bone->tail, bone->head, vec);
 
 	// set parent tail
 	if (parent && totchild == 1) {
-		copy_v3_v3(parent->tail, bone->head);
+	   copy_v3_v3(parent->tail, bone->head);
 
 		// not setting BONE_CONNECTED because this would lock child bone location with respect to parent
-		// bone->flag |= BONE_CONNECTED;
+		 bone->flag |= BONE_CONNECTED;
 
 		// XXX increase this to prevent "very" small bones?
 		const float epsilon = 0.000001f;
@@ -226,32 +153,6 @@ void ArmatureImporter::create_bone(SkinInfo& skin, COLLADAFW::Node *node, EditBo
 			add_leaf_bone(parent_mat, parent, node);
 		}
 
-		/*
-#if 0
-		// and which row in mat is bone direction
-		float vec[3];
-		sub_v3_v3v3(vec, parent->tail, parent->head);
-#ifdef COLLADA_DEBUG
-		print_v3("tail - head", vec);
-		print_m4("matrix", parent_mat);
-#endif
-		for (int i = 0; i < 3; i++) {
-#ifdef COLLADA_DEBUG
-			char *axis_names[] = {"X", "Y", "Z"};
-			printf("%s-axis length is %f\n", axis_names[i], len_v3(parent_mat[i]));
-#endif
-			float angle = angle_v2v2(vec, parent_mat[i]);
-			if (angle < min_angle) {
-#ifdef COLLADA_DEBUG
-				print_v3("picking", parent_mat[i]);
-				printf("^ %s axis of %s's matrix\n", axis_names[i], get_dae_name(node));
-#endif
-				bone_direction_row = i;
-				min_angle = angle;
-			}
-		}
-#endif
-		*/
 	}
 
 	COLLADAFW::NodePointerArray& children = node->getChildNodes();
@@ -261,13 +162,15 @@ void ArmatureImporter::create_bone(SkinInfo& skin, COLLADAFW::Node *node, EditBo
 
 	// in second case it's not a leaf bone, but we handle it the same way
 	if (!children.getCount() || children.getCount() > 1) {
-		add_leaf_bone(mat, bone , node);
+		add_leaf_bone(mat, bone, node);
 	}
 
+	bone->length = len_v3v3(bone->head, bone->tail);
+
 	finished_joints.push_back(node);
 }
 
-void ArmatureImporter::add_leaf_bone(float mat[][4], EditBone *bone,  COLLADAFW::Node * node)
+void ArmatureImporter::add_leaf_bone(float mat[4][4], EditBone *bone,  COLLADAFW::Node *node)
 {
 	LeafBone leaf;
 
@@ -278,26 +181,26 @@ void ArmatureImporter::add_leaf_bone(float mat[][4], EditBone *bone,  COLLADAFW:
 	TagsMap::iterator etit;
 	ExtraTags *et = 0;
 	etit = uid_tags_map.find(node->getUniqueId().toAscii());
-	if (etit !=  uid_tags_map.end())
-	{
+	if (etit != uid_tags_map.end()) {
 		et = etit->second;
 		//else return;
 
-		float x,y,z;
-		et->setData("tip_x",&x);
-		et->setData("tip_y",&y);
-		et->setData("tip_z",&z);
-		float vec[3] = {x,y,z};
+		float x, y, z;
+		et->setData("tip_x", &x);
+		et->setData("tip_y", &y);
+		et->setData("tip_z", &z);
+		float vec[3] = {x, y, z};
 		copy_v3_v3(leaf.bone->tail, leaf.bone->head);
 		add_v3_v3v3(leaf.bone->tail, leaf.bone->head, vec);
-	}else
+	}
+	else {
 		leaf_bones.push_back(leaf);
+	}
 }
 
 void ArmatureImporter::fix_leaf_bones( )
 {
 	// just setting tail for leaf bones here
-
 	std::vector<LeafBone>::iterator it;
 	for (it = leaf_bones.begin(); it != leaf_bones.end(); it++) {
 		LeafBone& leaf = *it;
@@ -305,11 +208,10 @@ void ArmatureImporter::fix_leaf_bones( )
 		// pointing up
 		float vec[3] = {0.0f, 0.0f, 0.1f};
 		
-		// if parent: take parent length and direction
-		if (leaf.bone->parent) sub_v3_v3v3(vec, leaf.bone->parent->tail, leaf.bone->parent->head);
+		sub_v3_v3v3(vec, leaf.bone->tail , leaf.bone->head);
+		mul_v3_fl(vec, leaf_bone_length);
+		add_v3_v3v3(leaf.bone->tail, leaf.bone->head , vec);
 
-		copy_v3_v3(leaf.bone->tail, leaf.bone->head);
-		add_v3_v3v3(leaf.bone->tail, leaf.bone->head, vec);
 	}
 }
 
@@ -322,7 +224,7 @@ void ArmatureImporter::set_leaf_bone_shapes(Object *ob_arm)
 	for (it = leaf_bones.begin(); it != leaf_bones.end(); it++) {
 		LeafBone& leaf = *it;
 
-		bPoseChannel *pchan = get_pose_channel(pose, leaf.name);
+		bPoseChannel *pchan = BKE_pose_channel_find_name(pose, leaf.name);
 		if (pchan) {
 			pchan->custom = get_empty_for_leaves();
 		}
@@ -336,7 +238,7 @@ void ArmatureImporter::set_euler_rotmode()
 {
 	// just set rotmode = ROT_MODE_EUL on pose channel for each joint
 
-	std::map<COLLADAFW::UniqueId, COLLADAFW::Node*>::iterator it;
+	std::map<COLLADAFW::UniqueId, COLLADAFW::Node *>::iterator it;
 
 	for (it = joint_by_uid.begin(); it != joint_by_uid.end(); it++) {
 
@@ -377,7 +279,7 @@ Object *ArmatureImporter::get_empty_for_leaves()
 #if 0
 Object *ArmatureImporter::find_armature(COLLADAFW::Node *node)
 {
-	JointData* jd = get_joint_data(node);
+	JointData *jd = get_joint_data(node);
 	if (jd) return jd->ob_arm;
 
 	COLLADAFW::NodePointerArray& children = node->getChildNodes();
@@ -407,47 +309,41 @@ ArmatureJoints& ArmatureImporter::get_armature_joints(Object *ob_arm)
 #endif
 void ArmatureImporter::create_armature_bones( )
 {
-	std::vector<COLLADAFW::Node*>::iterator ri;
+	std::vector<COLLADAFW::Node *>::iterator ri;
+
+	leaf_bone_length = FLT_MAX;
 	//if there is an armature created for root_joint next root_joint
 	for (ri = root_joints.begin(); ri != root_joints.end(); ri++) {
-		if ( get_armature_for_joint(*ri) != NULL ) continue;
+		if (get_armature_for_joint(*ri) != NULL) continue;
 		
-		//add armature object for current joint
-		//Object *ob_arm = bc_add_object(scene, OB_ARMATURE, NULL);
-
 		Object *ob_arm = joint_parent_map[(*ri)->getUniqueId()];
 
 		if (!ob_arm)
 			continue;
 
-		//ob_arm->type = OB_ARMATURE;
 		ED_armature_to_edit(ob_arm);
 
-		// min_angle = 360.0f;		// minimum angle between bone head-tail and a row of bone matrix
-
-		// create unskinned bones
 		/*
-		  TODO:
-		  check if bones have already been created for a given joint
-		*/
-		leaf_bone_length = FLT_MAX;
-		create_unskinned_bone(*ri, NULL, (*ri)->getChildNodes().getCount(), NULL, ob_arm);
+		 * TODO:
+		 * check if bones have already been created for a given joint
+		 */
 
+		create_bone(NULL, *ri , NULL, (*ri)->getChildNodes().getCount(), NULL, (bArmature *)ob_arm->data);
+
+		//leaf bone tails are derived from the matrix, so no need of this.
 		fix_leaf_bones();
 
 		// exit armature edit mode
-	
 		unskinned_armature_map[(*ri)->getUniqueId()] = ob_arm;
 
 		ED_armature_from_edit(ob_arm);
 
-		set_pose(ob_arm , *ri, NULL, NULL ); 
+		//This serves no purpose, as pose is automatically reset later, in BKE_where_is_bone()
+		//set_pose(ob_arm, *ri, NULL, NULL);
 
 		ED_armature_edit_free(ob_arm);
-		DAG_id_tag_update(&ob_arm->id, OB_RECALC_OB|OB_RECALC_DATA);
+		DAG_id_tag_update(&ob_arm->id, OB_RECALC_OB | OB_RECALC_DATA);
 	}
-
-	
 }
 
 void ArmatureImporter::create_armature_bones(SkinInfo& skin)
@@ -495,22 +391,22 @@ void ArmatureImporter::create_armature_bones(SkinInfo& skin)
 
 	SkinInfo *a = &skin;
 	Object *shared = NULL;
-	std::vector<COLLADAFW::Node*> skin_root_joints;
+	std::vector<COLLADAFW::Node *> skin_root_joints;
 
 	std::map<COLLADAFW::UniqueId, SkinInfo>::iterator it;
 	for (it = skin_by_data_uid.begin(); it != skin_by_data_uid.end(); it++) {
 		SkinInfo *b = &it->second;
-		if (b == a || b->get_armature() == NULL)
+		if (b == a || b->BKE_armature_from_object() == NULL)
 			continue;
 
 		skin_root_joints.clear();
 
 		b->find_root_joints(root_joints, joint_by_uid, skin_root_joints);
 
-		std::vector<COLLADAFW::Node*>::iterator ri;
+		std::vector<COLLADAFW::Node *>::iterator ri;
 		for (ri = skin_root_joints.begin(); ri != skin_root_joints.end(); ri++) {
 			if (a->uses_joint_or_descendant(*ri)) {
-				shared = b->get_armature();
+				shared = b->BKE_armature_from_object();
 				break;
 			}
 		}
@@ -519,10 +415,23 @@ void ArmatureImporter::create_armature_bones(SkinInfo& skin)
 			break;
 	}
 
-	if (shared)
+	if (!shared && this->joint_parent_map.size() > 0) {
+		// All armatures have been created while creating the Node tree. 
+		// The Collada exporter currently does not create a 
+		// strict relationship between geometries and armatures
+		// So when we reimport a Blender collada file, then we have
+		// to guess what is meant.
+		// XXX This is not safe when we have more than one armatures
+		// in the import.
+		shared = this->joint_parent_map.begin()->second;
+	}
+
+	if (shared) {
 		ob_arm = skin.set_armature(shared);
-	else
-		ob_arm = skin.create_armature(scene); //once for every armature
+	}
+	else {
+		ob_arm = skin.create_armature(scene);  //once for every armature
+	}
 
 	// enter armature edit mode
 	ED_armature_to_edit(ob_arm);
@@ -531,15 +440,14 @@ void ArmatureImporter::create_armature_bones(SkinInfo& skin)
 	totbone = 0;
 	// bone_direction_row = 1; // TODO: don't default to Y but use asset and based on it decide on default row
 	leaf_bone_length = FLT_MAX;
-	// min_angle = 360.0f;		// minimum angle between bone head-tail and a row of bone matrix
 
 	// create bones
 	/*
 	   TODO:
 	   check if bones have already been created for a given joint
-	*/
+	 */
 
-	std::vector<COLLADAFW::Node*>::iterator ri;
+	std::vector<COLLADAFW::Node *>::iterator ri;
 	for (ri = root_joints.begin(); ri != root_joints.end(); ri++) {
 		// for shared armature check if bone tree is already created
 		if (shared && std::find(skin_root_joints.begin(), skin_root_joints.end(), *ri) != skin_root_joints.end())
@@ -547,7 +455,7 @@ void ArmatureImporter::create_armature_bones(SkinInfo& skin)
 
 		// since root_joints may contain joints for multiple controllers, we need to filter
 		if (skin.uses_joint_or_descendant(*ri)) {
-			create_bone(skin, *ri, NULL, (*ri)->getChildNodes().getCount(), NULL, (bArmature*)ob_arm->data);
+			create_bone(&skin, *ri, NULL, (*ri)->getChildNodes().getCount(), NULL, (bArmature *)ob_arm->data);
 
 			if (joint_parent_map.find((*ri)->getUniqueId()) != joint_parent_map.end() && !skin.get_parent())
 				skin.set_parent(joint_parent_map[(*ri)->getUniqueId()]);
@@ -559,50 +467,43 @@ void ArmatureImporter::create_armature_bones(SkinInfo& skin)
 	// exit armature edit mode
 	ED_armature_from_edit(ob_arm);
 	ED_armature_edit_free(ob_arm);
-	DAG_id_tag_update(&ob_arm->id, OB_RECALC_OB|OB_RECALC_DATA);
+	DAG_id_tag_update(&ob_arm->id, OB_RECALC_OB | OB_RECALC_DATA);
 
-	// set_leaf_bone_shapes(ob_arm);
-	// set_euler_rotmode();
 }
 
-
-// root - if this joint is the top joint in hierarchy, if a joint
-// is a child of a node (not joint), root should be true since
-// this is where we build armature bones from
-
-void ArmatureImporter::set_pose ( Object * ob_arm ,  COLLADAFW::Node * root_node , const char *parentname, float parent_mat[][4])
+void ArmatureImporter::set_pose(Object *ob_arm,  COLLADAFW::Node *root_node, const char *parentname, float parent_mat[4][4])
 { 
-	char * bone_name = (char *) bc_get_joint_name ( root_node);
+	char *bone_name = (char *) bc_get_joint_name(root_node);
 	float mat[4][4];
 	float obmat[4][4];
 
-	float ax[3];
-	float angle = 0.0f;
-	
 	// object-space
 	get_node_mat(obmat, root_node, NULL, NULL);
 
 	//if (*edbone)
-	bPoseChannel * pchan  = get_pose_channel(ob_arm -> pose ,  bone_name); 
+	bPoseChannel *pchan  = BKE_pose_channel_find_name(ob_arm->pose, bone_name);
 	//else fprintf ( "",
 
 	// get world-space
 	if (parentname) {
 		mult_m4_m4m4(mat, parent_mat, obmat);
-		bPoseChannel *parchan = get_pose_channel(ob_arm->pose, parentname);
+		bPoseChannel *parchan = BKE_pose_channel_find_name(ob_arm->pose, parentname);
 
-		mult_m4_m4m4(pchan->pose_mat, parchan->pose_mat, mat );
+		mult_m4_m4m4(pchan->pose_mat, parchan->pose_mat, mat);
 
 	}
 	else {
+		
 		copy_m4_m4(mat, obmat);
 		float invObmat[4][4];
 		invert_m4_m4(invObmat, ob_arm->obmat);
 		mult_m4_m4m4(pchan->pose_mat, invObmat, mat);
+		
 	}
 
-	mat4_to_axis_angle(ax,&angle,mat);
-	pchan->bone->roll = angle;
+	//float angle = 0.0f;
+	///*mat4_to_axis_angle(ax, &angle, mat);
+	//pchan->bone->roll = angle;*/
 
 
 	COLLADAFW::NodePointerArray& children = root_node->getChildNodes();
@@ -612,6 +513,10 @@ void ArmatureImporter::set_pose ( Object * ob_arm ,  COLLADAFW::Node * root_node
 
 }
 
+
+// root - if this joint is the top joint in hierarchy, if a joint
+// is a child of a node (not joint), root should be true since
+// this is where we build armature bones from
 void ArmatureImporter::add_joint(COLLADAFW::Node *node, bool root, Object *parent, Scene *sce)
 {
 	joint_by_uid[node->getUniqueId()] = node;
@@ -630,7 +535,7 @@ void ArmatureImporter::add_root_joint(COLLADAFW::Node *node)
 {
 	// root_joints.push_back(node);
 	Object *ob_arm = find_armature(node);
-	if (ob_arm)	{
+	if (ob_arm) {
 		get_armature_joints(ob_arm).root_joints.push_back(node);
 	}
 #ifdef COLLADA_DEBUG
@@ -652,16 +557,22 @@ void ArmatureImporter::make_armatures(bContext *C)
 		create_armature_bones(skin);
 
 		// link armature with a mesh object
-		Object *ob = mesh_importer->get_object_by_geom_uid(*get_geometry_uid(skin.get_controller_uid()));
-		if (ob)
-			skin.link_armature(C, ob, joint_by_uid, this);
+		const COLLADAFW::UniqueId &uid = skin.get_controller_uid();
+		const COLLADAFW::UniqueId *guid = get_geometry_uid(uid);
+		if (guid != NULL) {
+			Object *ob = mesh_importer->get_object_by_geom_uid(*guid);
+			if (ob)
+				skin.link_armature(C, ob, joint_by_uid, this);
+			else
+				fprintf(stderr, "Cannot find object to link armature with.\n");
+		}
 		else
-			fprintf(stderr, "Cannot find object to link armature with.\n");
+			fprintf(stderr, "Cannot find geometry to link armature with.\n");
 
 		// set armature parent if any
 		Object *par = skin.get_parent();
 		if (par)
-			bc_set_parent(skin.get_armature(), par, C, false);
+			bc_set_parent(skin.BKE_armature_from_object(), par, C, false);
 
 		// free memory stolen from SkinControllerData
 		skin.free();
@@ -693,7 +604,7 @@ void ArmatureImporter::link_armature(Object *ob_arm, const COLLADAFW::UniqueId&
 }
 #endif
 
-bool ArmatureImporter::write_skin_controller_data(const COLLADAFW::SkinControllerData* data)
+bool ArmatureImporter::write_skin_controller_data(const COLLADAFW::SkinControllerData *data)
 {
 	// at this stage we get vertex influence info that should go into me->verts and ob->defbase
 	// there's no info to which object this should be long so we associate it with skin controller data UID
@@ -718,16 +629,15 @@ bool ArmatureImporter::write_skin_controller_data(const COLLADAFW::SkinControlle
 	return true;
 }
 
-bool ArmatureImporter::write_controller(const COLLADAFW::Controller* controller)
+bool ArmatureImporter::write_controller(const COLLADAFW::Controller *controller)
 {
 	// - create and store armature object
-
-	const COLLADAFW::UniqueId& skin_id = controller->getUniqueId();
+	const COLLADAFW::UniqueId& con_id = controller->getUniqueId();
 
 	if (controller->getControllerType() == COLLADAFW::Controller::CONTROLLER_TYPE_SKIN) {
-		COLLADAFW::SkinController *co = (COLLADAFW::SkinController*)controller;
+		COLLADAFW::SkinController *co = (COLLADAFW::SkinController *)controller;
 		// to be able to find geom id by controller id
-		geom_uid_by_controller_uid[skin_id] = co->getSource();
+		geom_uid_by_controller_uid[con_id] = co->getSource();
 
 		const COLLADAFW::UniqueId& data_uid = co->getSkinControllerData();
 		if (skin_by_data_uid.find(data_uid) == skin_by_data_uid.end()) {
@@ -738,14 +648,71 @@ bool ArmatureImporter::write_controller(const COLLADAFW::Controller* controller)
 		skin_by_data_uid[data_uid].set_controller(co);
 	}
 	// morph controller
-	else {
-		// shape keys? :)
-		fprintf(stderr, "Morph controller is not supported yet.\n");
+	else if (controller->getControllerType() == COLLADAFW::Controller::CONTROLLER_TYPE_MORPH) {
+		COLLADAFW::MorphController *co = (COLLADAFW::MorphController *)controller;
+		// to be able to find geom id by controller id
+		geom_uid_by_controller_uid[con_id] = co->getSource();
+		//Shape keys are applied in DocumentImporter->finish()
+		morph_controllers.push_back(co);
 	}
 
 	return true;
 }
 
+void ArmatureImporter::make_shape_keys()
+{
+	std::vector<COLLADAFW::MorphController *>::iterator mc;
+	float weight;
+
+	for (mc = morph_controllers.begin(); mc != morph_controllers.end(); mc++) {
+		//Controller data
+		COLLADAFW::UniqueIdArray& morphTargetIds = (*mc)->getMorphTargets();
+		COLLADAFW::FloatOrDoubleArray& morphWeights = (*mc)->getMorphWeights();
+
+		//Prereq: all the geometries must be imported and mesh objects must be made
+		Object *source_ob = this->mesh_importer->get_object_by_geom_uid((*mc)->getSource());
+		
+		if (source_ob) {
+
+			Mesh *source_me = (Mesh*) source_ob->data;
+			//insert key to source mesh
+			Key *key = source_me->key = BKE_key_add((ID *)source_me);
+			key->type = KEY_RELATIVE;
+			KeyBlock *kb;
+			
+			//insert basis key
+			kb = BKE_keyblock_add_ctime(key, "Basis", FALSE);
+			BKE_key_convert_from_mesh(source_me, kb);
+
+			//insert other shape keys
+			for (int i = 0 ; i < morphTargetIds.getCount() ; i++ ) {
+				//better to have a seperate map of morph objects, 
+				//This'll do for now since only mesh morphing is imported
+
+				Mesh *me = this->mesh_importer->get_mesh_by_geom_uid(morphTargetIds[i]);
+				
+				if (me) {
+					me->key = key;
+					std::string morph_name = *this->mesh_importer->get_geometry_name(me->id.name);
+
+					kb = BKE_keyblock_add_ctime(key, morph_name.c_str(), FALSE);
+					BKE_key_convert_from_mesh(me, kb);
+					
+					//apply weights
+					weight =  morphWeights.getFloatValues()->getData()[i];
+					kb->curval = weight;
+				}
+				else {
+					fprintf(stderr, "Morph target geometry not found.\n");
+				}
+			}
+		}
+		else {
+			fprintf(stderr, "Morph target object not found.\n");
+		}
+	}
+}
+
 
 COLLADAFW::UniqueId *ArmatureImporter::get_geometry_uid(const COLLADAFW::UniqueId& controller_uid)
 {
@@ -762,10 +729,10 @@ Object *ArmatureImporter::get_armature_for_joint(COLLADAFW::Node *node)
 		SkinInfo& skin = it->second;
 
 		if (skin.uses_joint_or_descendant(node))
-			return skin.get_armature();
+			return skin.BKE_armature_from_object();
 	}
 
-	std::map<COLLADAFW::UniqueId, Object*>::iterator arm;
+	std::map<COLLADAFW::UniqueId, Object *>::iterator arm;
 	for (arm = unskinned_armature_map.begin(); arm != unskinned_armature_map.end(); arm++) {
 		if (arm->first == node->getUniqueId() )
 			return arm->second;
@@ -784,7 +751,7 @@ void ArmatureImporter::get_rna_path_for_joint(COLLADAFW::Node *node, char *joint
 }
 
 // gives a world-space mat
-bool ArmatureImporter::get_joint_bind_mat(float m[][4], COLLADAFW::Node *joint)
+bool ArmatureImporter::get_joint_bind_mat(float m[4][4], COLLADAFW::Node *joint)
 {
 	std::map<COLLADAFW::UniqueId, SkinInfo>::iterator it;
 	bool found = false;
@@ -798,6 +765,3 @@ bool ArmatureImporter::get_joint_bind_mat(float m[][4], COLLADAFW::Node *joint)
 
 	return found;
 }
-
-
-
