@@ -39,9 +39,14 @@
 #include "DNA_mesh_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
+#include "DNA_key_types.h"
 
 #include "ExportSettings.h"
 
+#include "BKE_key.h"
+
+extern Object *bc_get_highest_selected_ancestor_or_self(Object *ob);
+
 // TODO: optimize UV sets by making indexed list with duplicates removed
 class GeometryExporter : COLLADASW::LibraryGeometries
 {
@@ -64,11 +69,17 @@ public:
 
 	void operator()(Object *ob);
 
+	void createLooseEdgeList(Object *ob,
+						     Mesh   *me,
+						     std::string& geom_id,
+						     std::vector<Face>& norind);
+
 	// powerful because it handles both cases when there is material and when there's not
 	void createPolylist(short material_index,
 						bool has_uvs,
 						bool has_color,
 						Object *ob,
+						Mesh   *me,
 						std::string& geom_id,
 						std::vector<Face>& norind);
 	
@@ -92,33 +103,30 @@ public:
 	COLLADASW::URI getUrlBySemantics(std::string geom_id, COLLADASW::InputSemantic::Semantics type, std::string other_suffix = "");
 
 	COLLADASW::URI makeUrl(std::string id);
+
+	void export_key_mesh(Object *ob, Mesh *me, KeyBlock *kb);
 	
 	/* int getTriCount(MFace *faces, int totface);*/
 private:
 	std::set<std::string> exportedGeometry;
 	
 	const ExportSettings *export_settings;
+
+	Mesh * get_mesh(Scene *sce, Object *ob, int apply_modifiers);
 };
 
 struct GeometryFunctor {
 	// f should have
 	// void operator()(Object* ob)
 	template<class Functor>
-	void forEachMeshObjectInScene(Scene *sce, Functor &f, bool export_selected)
+	void forEachMeshObjectInExportSet(Scene *sce, Functor &f, LinkNode *export_set)
 	{
-		
-		Base *base= (Base*) sce->base.first;
-		while(base) {
-			Object *ob = base->object;
-			
-			if (ob->type == OB_MESH && ob->data &&
-				!(export_selected && !(ob->flag & SELECT)) &&
-				((sce->lay & ob->lay)!=0))
-			{
+		LinkNode *node;
+		for (node=export_set; node; node = node->next) {
+			Object *ob = (Object *)node->link;
+			if (ob->type == OB_MESH) {
 				f(ob);
 			}
-			base= base->next;
-			
 		}
 	}
 };
