@@ -36,34 +36,33 @@
 #include "collada_internal.h"
 
 template<class Functor>
-void forEachLampObjectInScene(Scene *sce, Functor &f, bool export_selected)
+void forEachLampObjectInExportSet(Scene *sce, Functor &f, LinkNode *export_set)
 {
-	Base *base= (Base*) sce->base.first;
-	while(base) {
-		Object *ob = base->object;
-			
-		if (ob->type == OB_LAMP && ob->data
-			&& !(export_selected && !(ob->flag & SELECT))) {
+	LinkNode *node;
+	for (node = export_set; node; node = node->next) {
+		Object *ob = (Object *)node->link;
+
+		if (ob->type == OB_LAMP && ob->data) {
 			f(ob);
 		}
-		base= base->next;
 	}
 }
 
-LightsExporter::LightsExporter(COLLADASW::StreamWriter *sw, const ExportSettings *export_settings): COLLADASW::LibraryLights(sw), export_settings(export_settings) {}
+LightsExporter::LightsExporter(COLLADASW::StreamWriter *sw, const ExportSettings *export_settings) : COLLADASW::LibraryLights(sw), export_settings(export_settings) {
+}
 
 void LightsExporter::exportLights(Scene *sce)
 {
 	openLibrary();
 	
-	forEachLampObjectInScene(sce, *this, this->export_settings->selected);
+	forEachLampObjectInExportSet(sce, *this, this->export_settings->export_set);
 	
 	closeLibrary();
 }
 
 void LightsExporter::operator()(Object *ob)
 {
-	Lamp *la = (Lamp*)ob->data;
+	Lamp *la = (Lamp *)ob->data;
 	std::string la_id(get_light_id(ob));
 	std::string la_name(id_name(la));
 	COLLADASW::Color col(la->r * la->energy, la->g * la->energy, la->b * la->energy);
@@ -73,7 +72,7 @@ void LightsExporter::operator()(Object *ob)
 	
 	constatt = 1.0f;
 	
-	if (la->falloff_type==LA_FALLOFF_INVLINEAR) {
+	if (la->falloff_type == LA_FALLOFF_INVLINEAR) {
 		linatt = 1.0f / d;
 		quadatt = 0.0f;
 	}
@@ -85,7 +84,7 @@ void LightsExporter::operator()(Object *ob)
 	// sun
 	if (la->type == LA_SUN) {
 		COLLADASW::DirectionalLight cla(mSW, la_id, la_name);
-		cla.setColor(col,false,"color");
+		cla.setColor(col, false, "color");
 		cla.setConstantAttenuation(constatt);
 		exportBlenderProfile(cla, la);
 		addLight(cla);
@@ -93,7 +92,7 @@ void LightsExporter::operator()(Object *ob)
 	// hemi
 	else if (la->type == LA_HEMI) {
 		COLLADASW::AmbientLight cla(mSW, la_id, la_name);
-		cla.setColor(col,false,"color");
+		cla.setColor(col, false, "color");
 		cla.setConstantAttenuation(constatt);
 		exportBlenderProfile(cla, la);
 		addLight(cla);
@@ -101,9 +100,9 @@ void LightsExporter::operator()(Object *ob)
 	// spot
 	else if (la->type == LA_SPOT) {
 		COLLADASW::SpotLight cla(mSW, la_id, la_name);
-		cla.setColor(col,false,"color");
-		cla.setFallOffAngle(la->spotsize,false,"fall_off_angle");
-		cla.setFallOffExponent(la->spotblend,false,"fall_off_exponent");
+		cla.setColor(col, false, "color");
+		cla.setFallOffAngle(la->spotsize, false, "fall_off_angle");
+		cla.setFallOffExponent(la->spotblend, false, "fall_off_exponent");
 		cla.setConstantAttenuation(constatt);
 		cla.setLinearAttenuation(linatt);
 		cla.setQuadraticAttenuation(quadatt);
@@ -113,7 +112,7 @@ void LightsExporter::operator()(Object *ob)
 	// lamp
 	else if (la->type == LA_LOCAL) {
 		COLLADASW::PointLight cla(mSW, la_id, la_name);
-		cla.setColor(col,false,"color");
+		cla.setColor(col, false, "color");
 		cla.setConstantAttenuation(constatt);
 		cla.setLinearAttenuation(linatt);
 		cla.setQuadraticAttenuation(quadatt);
@@ -124,7 +123,7 @@ void LightsExporter::operator()(Object *ob)
 	// it will be exported as a local lamp
 	else {
 		COLLADASW::PointLight cla(mSW, la_id, la_name);
-		cla.setColor(col,false,"color");
+		cla.setColor(col, false, "color");
 		cla.setConstantAttenuation(constatt);
 		cla.setLinearAttenuation(linatt);
 		cla.setQuadraticAttenuation(quadatt);
