@@ -57,14 +57,17 @@ static const char *bc_get_joint_name(T *node)
 // This is used to store data passed in write_controller_data.
 // Arrays from COLLADAFW::SkinControllerData lose ownership, so do this class members
 // so that arrays don't get freed until we free them explicitly.
-SkinInfo::SkinInfo() {}
+SkinInfo::SkinInfo()
+{
+	/* pass */
+}
 
 SkinInfo::SkinInfo(const SkinInfo& skin) : weights(skin.weights),
-								 joint_data(skin.joint_data),
-								 unit_converter(skin.unit_converter),
-								 ob_arm(skin.ob_arm),
-								 controller_uid(skin.controller_uid),
-								 parent(skin.parent)
+	joint_data(skin.joint_data),
+	unit_converter(skin.unit_converter),
+	ob_arm(skin.ob_arm),
+	controller_uid(skin.controller_uid),
+	parent(skin.parent)
 {
 	copy_m4_m4(bind_shape_matrix, (float (*)[4])skin.bind_shape_matrix);
 
@@ -73,7 +76,8 @@ SkinInfo::SkinInfo(const SkinInfo& skin) : weights(skin.weights),
 	transfer_int_array_data_const(skin.joint_indices, joint_indices);
 }
 
-SkinInfo::SkinInfo(UnitConverter *conv) : unit_converter(conv), ob_arm(NULL), parent(NULL) {}
+SkinInfo::SkinInfo(UnitConverter *conv) : unit_converter(conv), ob_arm(NULL), parent(NULL) {
+}
 
 // nobody owns the data after this, so it should be freed manually with releaseMemory
 template <class T>
@@ -87,21 +91,21 @@ void SkinInfo::transfer_array_data(T& src, T& dest)
 // when src is const we cannot src.yieldOwnerShip, this is used by copy constructor
 void SkinInfo::transfer_int_array_data_const(const COLLADAFW::IntValuesArray& src, COLLADAFW::IntValuesArray& dest)
 {
-	dest.setData((int*)src.getData(), src.getCount());
+	dest.setData((int *)src.getData(), src.getCount());
 	dest.yieldOwnerShip();
 }
 
 void SkinInfo::transfer_uint_array_data_const(const COLLADAFW::UIntValuesArray& src, COLLADAFW::UIntValuesArray& dest)
 {
-	dest.setData((unsigned int*)src.getData(), src.getCount());
+	dest.setData((unsigned int *)src.getData(), src.getCount());
 	dest.yieldOwnerShip();
 }
 
-void SkinInfo::borrow_skin_controller_data(const COLLADAFW::SkinControllerData* skin)
+void SkinInfo::borrow_skin_controller_data(const COLLADAFW::SkinControllerData *skin)
 {
-	transfer_array_data((COLLADAFW::UIntValuesArray&)skin->getJointsPerVertex(), joints_per_vertex);
-	transfer_array_data((COLLADAFW::UIntValuesArray&)skin->getWeightIndices(), weight_indices);
-	transfer_array_data((COLLADAFW::IntValuesArray&)skin->getJointIndices(), joint_indices);
+	transfer_array_data((COLLADAFW::UIntValuesArray &)skin->getJointsPerVertex(), joints_per_vertex);
+	transfer_array_data((COLLADAFW::UIntValuesArray &)skin->getWeightIndices(), weight_indices);
+	transfer_array_data((COLLADAFW::IntValuesArray &)skin->getJointIndices(), joint_indices);
 	// transfer_array_data(skin->getWeights(), weights);
 
 	// cannot transfer data for FloatOrDoubleArray, copy values manually
@@ -130,7 +134,7 @@ void SkinInfo::add_joint(const COLLADABU::Math::Matrix4& matrix)
 	joint_data.push_back(jd);
 }
 
-void SkinInfo::set_controller(const COLLADAFW::SkinController* co)
+void SkinInfo::set_controller(const COLLADAFW::SkinController *co)
 {
 	controller_uid = co->getUniqueId();
 
@@ -155,7 +159,7 @@ Object *SkinInfo::create_armature(Scene *scene)
 	return ob_arm;
 }
 
-Object* SkinInfo::set_armature(Object *ob_arm)
+Object *SkinInfo::set_armature(Object *ob_arm)
 {
 	if (this->ob_arm)
 		return this->ob_arm;
@@ -164,7 +168,7 @@ Object* SkinInfo::set_armature(Object *ob_arm)
 	return ob_arm;
 }
 
-bool SkinInfo::get_joint_inv_bind_matrix(float inv_bind_mat[][4], COLLADAFW::Node *node)
+bool SkinInfo::get_joint_inv_bind_matrix(float inv_bind_mat[4][4], COLLADAFW::Node *node)
 {
 	const COLLADAFW::UniqueId& uid = node->getUniqueId();
 	std::vector<JointData>::iterator it;
@@ -178,7 +182,7 @@ bool SkinInfo::get_joint_inv_bind_matrix(float inv_bind_mat[][4], COLLADAFW::Nod
 	return false;
 }
 
-Object *SkinInfo::get_armature()
+Object *SkinInfo::BKE_armature_from_object()
 {
 	return ob_arm;
 }
@@ -211,17 +215,18 @@ bool SkinInfo::uses_joint_or_descendant(COLLADAFW::Node *node)
 	return false;
 }
 
-void SkinInfo::link_armature(bContext *C, Object *ob, std::map<COLLADAFW::UniqueId, COLLADAFW::Node*>& joint_by_uid,
-				   TransformReader *tm)
+void SkinInfo::link_armature(bContext *C, Object *ob, std::map<COLLADAFW::UniqueId, COLLADAFW::Node *>& joint_by_uid,
+                             TransformReader *tm)
 {
 	Main *bmain = CTX_data_main(C);
 	Scene *scene = CTX_data_scene(C);
 
 	ModifierData *md = ED_object_modifier_add(NULL, bmain, scene, ob, NULL, eModifierType_Armature);
-	((ArmatureModifierData *)md)->object = ob_arm;
+	ArmatureModifierData *amd = (ArmatureModifierData *)md;
+	amd->object = ob_arm;
 
 	copy_m4_m4(ob->obmat, bind_shape_matrix);
-	object_apply_mat4(ob, ob->obmat, 0, 0);
+	BKE_object_apply_mat4(ob, ob->obmat, 0, 0);
 #if 1
 	bc_set_parent(ob, ob_arm, C);
 #else
@@ -229,17 +234,17 @@ void SkinInfo::link_armature(bContext *C, Object *ob, std::map<COLLADAFW::Unique
 	ob->parent = ob_arm;
 	ob->partype = PAROBJECT;
 
-	what_does_parent(scene, ob, &workob);
+	BKE_object_workob_calc_parent(scene, ob, &workob);
 	invert_m4_m4(ob->parentinv, workob.obmat);
 
-	ob->recalc |= OB_RECALC_OB|OB_RECALC_DATA;
+	ob->recalc |= OB_RECALC_OB | OB_RECALC_DATA;
 
 	DAG_scene_sort(bmain, scene);
 	DAG_ids_flush_update(bmain, 0);
-	WM_event_add_notifier(C, NC_OBJECT|ND_TRANSFORM, NULL);
+	WM_event_add_notifier(C, NC_OBJECT | ND_TRANSFORM, NULL);
 #endif
 
-	((bArmature*)ob_arm->data)->deformflag = ARM_DEF_VGROUP;
+	amd->deformflag = ARM_DEF_VGROUP;
 
 	// create all vertex groups
 	std::vector<JointData>::iterator it;
@@ -256,7 +261,7 @@ void SkinInfo::link_armature(bContext *C, Object *ob, std::map<COLLADAFW::Unique
 			name = bc_get_joint_name(joint_by_uid[(*it).joint_uid]);
 		}
 
-		ED_vgroup_add_name(ob, (char*)name);
+		ED_vgroup_add_name(ob, (char *)name);
 	}
 
 	// <vcount> - number of joints per vertex - joints_per_vertex
@@ -273,12 +278,12 @@ void SkinInfo::link_armature(bContext *C, Object *ob, std::map<COLLADAFW::Unique
 	for (unsigned int vertex = 0, weight = 0; vertex < joints_per_vertex.getCount(); vertex++) {
 
 		unsigned int limit = weight + joints_per_vertex[vertex];
-		for ( ; weight < limit; weight++) {
+		for (; weight < limit; weight++) {
 			int joint = joint_indices[weight], joint_weight = weight_indices[weight];
 
 			// -1 means "weight towards the bind shape", we just don't assign it to any group
 			if (joint != -1) {
-				bDeformGroup *def = (bDeformGroup*)BLI_findlink(&ob->defbase, joint);
+				bDeformGroup *def = (bDeformGroup *)BLI_findlink(&ob->defbase, joint);
 
 				ED_vgroup_vert_add(ob, def, vertex, weights[joint_weight], WEIGHT_REPLACE);
 			}
@@ -288,7 +293,7 @@ void SkinInfo::link_armature(bContext *C, Object *ob, std::map<COLLADAFW::Unique
 
 bPoseChannel *SkinInfo::get_pose_channel_from_node(COLLADAFW::Node *node)
 {
-	return get_pose_channel(ob_arm->pose, bc_get_joint_name(node));
+	return BKE_pose_channel_find_name(ob_arm->pose, bc_get_joint_name(node));
 }
 
 void SkinInfo::set_parent(Object *_parent)
@@ -296,16 +301,16 @@ void SkinInfo::set_parent(Object *_parent)
 	parent = _parent;
 }
 
-Object* SkinInfo::get_parent()
+Object *SkinInfo::get_parent()
 {
 	return parent;
 }
 
-void SkinInfo::find_root_joints(const std::vector<COLLADAFW::Node*> &root_joints,
-					  std::map<COLLADAFW::UniqueId, COLLADAFW::Node*>& joint_by_uid,
-					  std::vector<COLLADAFW::Node*>& result)
+void SkinInfo::find_root_joints(const std::vector<COLLADAFW::Node *> &root_joints,
+                                std::map<COLLADAFW::UniqueId, COLLADAFW::Node *>& joint_by_uid,
+                                std::vector<COLLADAFW::Node *>& result)
 {
-	std::vector<COLLADAFW::Node*>::const_iterator it;
+	std::vector<COLLADAFW::Node *>::const_iterator it;
 	// for each root_joint
 	for (it = root_joints.begin(); it != root_joints.end(); it++) {
 		COLLADAFW::Node *root = *it;
