@@ -29,9 +29,12 @@
 
 #include "TransformReader.h"
 
-TransformReader::TransformReader(UnitConverter* conv) : unit_converter(conv) {}
+TransformReader::TransformReader(UnitConverter *conv) : unit_converter(conv)
+{
+	/* pass */
+}
 
-void TransformReader::get_node_mat(float mat[][4], COLLADAFW::Node *node, std::map<COLLADAFW::UniqueId, Animation> *animation_map, Object *ob)
+void TransformReader::get_node_mat(float mat[4][4], COLLADAFW::Node *node, std::map<COLLADAFW::UniqueId, Animation> *animation_map, Object *ob)
 {
 	float cur[4][4];
 	float copy[4][4];
@@ -43,28 +46,31 @@ void TransformReader::get_node_mat(float mat[][4], COLLADAFW::Node *node, std::m
 		COLLADAFW::Transformation *tm = node->getTransformations()[i];
 		COLLADAFW::Transformation::TransformationType type = tm->getTransformationType();
 
-			switch(type) {
-			case COLLADAFW::Transformation::TRANSLATE:
-				dae_translate_to_mat4(tm, cur);
-				break;
-			case COLLADAFW::Transformation::ROTATE:
-				dae_rotate_to_mat4(tm, cur);
-				break;
-			case COLLADAFW::Transformation::SCALE:
-				dae_scale_to_mat4(tm, cur);
-				break;
-			case COLLADAFW::Transformation::MATRIX:
-				dae_matrix_to_mat4(tm, cur);
-				break;
-			case COLLADAFW::Transformation::LOOKAT:
-			case COLLADAFW::Transformation::SKEW:
-				fprintf(stderr, "LOOKAT and SKEW transformations are not supported yet.\n");
-				break;
+		if (type == COLLADAFW::Transformation::MATRIX) {
+			// XXX why does this return and discard all following transformations?
+			dae_matrix_to_mat4(tm, mat);
+			return;
+		}
+		else {
+			switch (type) {
+				case COLLADAFW::Transformation::TRANSLATE:
+					dae_translate_to_mat4(tm, cur);
+					break;
+				case COLLADAFW::Transformation::ROTATE:
+					dae_rotate_to_mat4(tm, cur);
+					break;
+				case COLLADAFW::Transformation::SCALE:
+					dae_scale_to_mat4(tm, cur);
+					break;
+				case COLLADAFW::Transformation::LOOKAT:
+				case COLLADAFW::Transformation::SKEW:
+					fprintf(stderr, "LOOKAT and SKEW transformations are not supported yet.\n");
+					break;
 			}
-
-		copy_m4_m4(copy, mat);
-		mult_m4_m4m4(mat, copy, cur);
-
+			copy_m4_m4(copy, mat);
+			mult_m4_m4m4(mat, copy, cur);
+		}
+		
 		if (animation_map) {
 			// AnimationList that drives this Transformation
 			const COLLADAFW::UniqueId& anim_list_id = tm->getAnimationList();
@@ -76,21 +82,21 @@ void TransformReader::get_node_mat(float mat[][4], COLLADAFW::Node *node, std::m
 	}
 }
 
-void TransformReader::dae_rotate_to_mat4(COLLADAFW::Transformation *tm, float m[][4])
+void TransformReader::dae_rotate_to_mat4(COLLADAFW::Transformation *tm, float m[4][4])
 {
-	COLLADAFW::Rotate *ro = (COLLADAFW::Rotate*)tm;
+	COLLADAFW::Rotate *ro = (COLLADAFW::Rotate *)tm;
 	COLLADABU::Math::Vector3& axis = ro->getRotationAxis();
 	const float angle = (float)DEG2RAD(ro->getRotationAngle());
-	const float ax[] = {axis[0], axis[1], axis[2]};
+	const float ax[] = {(float)axis[0], (float)axis[1], (float)axis[2]};
 	// float quat[4];
 	// axis_angle_to_quat(quat, axis, angle);
 	// quat_to_mat4(m, quat);
 	axis_angle_to_mat4(m, ax, angle);
 }
 
-void TransformReader::dae_translate_to_mat4(COLLADAFW::Transformation *tm, float m[][4])
+void TransformReader::dae_translate_to_mat4(COLLADAFW::Transformation *tm, float m[4][4])
 {
-	COLLADAFW::Translate *tra = (COLLADAFW::Translate*)tm;
+	COLLADAFW::Translate *tra = (COLLADAFW::Translate *)tm;
 	COLLADABU::Math::Vector3& t = tra->getTranslation();
 
 	unit_m4(m);
@@ -100,26 +106,26 @@ void TransformReader::dae_translate_to_mat4(COLLADAFW::Transformation *tm, float
 	m[3][2] = (float)t[2];
 }
 
-void TransformReader::dae_scale_to_mat4(COLLADAFW::Transformation *tm, float m[][4])
+void TransformReader::dae_scale_to_mat4(COLLADAFW::Transformation *tm, float m[4][4])
 {
-	COLLADABU::Math::Vector3& s = ((COLLADAFW::Scale*)tm)->getScale();
+	COLLADABU::Math::Vector3& s = ((COLLADAFW::Scale *)tm)->getScale();
 	float size[3] = {(float)s[0], (float)s[1], (float)s[2]};
 	size_to_mat4(m, size);
 }
 
-void TransformReader::dae_matrix_to_mat4(COLLADAFW::Transformation *tm, float m[][4])
+void TransformReader::dae_matrix_to_mat4(COLLADAFW::Transformation *tm, float m[4][4])
 {
-	unit_converter->dae_matrix_to_mat4_(m, ((COLLADAFW::Matrix*)tm)->getMatrix());
+	unit_converter->dae_matrix_to_mat4_(m, ((COLLADAFW::Matrix *)tm)->getMatrix());
 }
 
 void TransformReader::dae_translate_to_v3(COLLADAFW::Transformation *tm, float v[3])
 {
-	dae_vector3_to_v3(((COLLADAFW::Translate*)tm)->getTranslation(), v);
+	dae_vector3_to_v3(((COLLADAFW::Translate *)tm)->getTranslation(), v);
 }
 
 void TransformReader::dae_scale_to_v3(COLLADAFW::Transformation *tm, float v[3])
 {
-	dae_vector3_to_v3(((COLLADAFW::Scale*)tm)->getScale(), v);
+	dae_vector3_to_v3(((COLLADAFW::Scale *)tm)->getScale(), v);
 }
 
 void TransformReader::dae_vector3_to_v3(const COLLADABU::Math::Vector3 &v3, float v[3])
