@@ -39,6 +39,7 @@
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "BLI_math.h"
+#include "BLI_linklist.h"
 
 class UnitConverter
 {
@@ -46,6 +47,10 @@ private:
 	COLLADAFW::FileInfo::Unit unit;
 	COLLADAFW::FileInfo::UpAxisType up_axis;
 
+	float x_up_mat4[4][4];
+	float y_up_mat4[4][4];
+	float z_up_mat4[4][4];
+
 public:
 
 	enum UnitSystem {
@@ -57,7 +62,7 @@ public:
 	// Initialize with Z_UP, since Blender uses right-handed, z-up
 	UnitConverter();
 
-	void read_asset(const COLLADAFW::FileInfo* asset);
+	void read_asset(const COLLADAFW::FileInfo *asset);
 
 	void convertVector3(COLLADABU::Math::Vector3 &vec, float *v);
 	
@@ -67,17 +72,21 @@ public:
 		
 	// TODO need also for angle conversion, time conversion...
 
-	void dae_matrix_to_mat4_(float out[][4], const COLLADABU::Math::Matrix4& in);
+	void dae_matrix_to_mat4_(float out[4][4], const COLLADABU::Math::Matrix4& in);
+
+	void mat4_to_dae(float out[4][4], float in[4][4]);
+
+	void mat4_to_dae_double(double out[4][4], float in[4][4]);
+
+	float(&get_rotation())[4][4];
 
-	void mat4_to_dae(float out[][4], float in[][4]);
 
-	void mat4_to_dae_double(double out[][4], float in[][4]);
 };
 
 class TransformBase
 {
 public:
-	void decompose(float mat[][4], float *loc, float eul[3], float quat[4], float *size);
+	void decompose(float mat[4][4], float *loc, float eul[3], float quat[4], float *size);
 };
 
 extern void clear_global_id_map();
@@ -87,6 +96,7 @@ extern std::string translate_id(const std::string &id);
 extern std::string id_name(void *id);
 
 extern std::string get_geometry_id(Object *ob);
+extern std::string get_geometry_id(Object *ob, bool use_instantiation);
 
 extern std::string get_light_id(Object *ob);
 
@@ -96,6 +106,6 @@ extern std::string get_camera_id(Object *ob);
 
 extern std::string get_material_id(Material *mat);
 
-extern bool has_object_type(Scene* sce, short obtype);
+extern std::string get_morph_id(Object *ob);
 
 #endif /* __COLLADA_INTERNAL_H__ */
