@@ -75,16 +75,16 @@
  */
 
 /* poll callback for adding default KeyingSet */
-static int keyingset_poll_default_add (bContext *C)
+static int keyingset_poll_default_add(bContext *C)
 {
 	/* as long as there's an active Scene, it's fine */
 	return (CTX_data_scene(C) != NULL);
 }
 
 /* poll callback for editing active KeyingSet */
-static int keyingset_poll_active_edit (bContext *C)
+static int keyingset_poll_active_edit(bContext *C)
 {
-	Scene *scene= CTX_data_scene(C);
+	Scene *scene = CTX_data_scene(C);
 	
 	if (scene == NULL)
 		return 0;
@@ -94,9 +94,9 @@ static int keyingset_poll_active_edit (bContext *C)
 }
 
 /* poll callback for editing active KeyingSet Path */
-static int keyingset_poll_activePath_edit (bContext *C)
+static int keyingset_poll_activePath_edit(bContext *C)
 {
-	Scene *scene= CTX_data_scene(C);
+	Scene *scene = CTX_data_scene(C);
 	KeyingSet *ks;
 	
 	if (scene == NULL)
@@ -104,7 +104,7 @@ static int keyingset_poll_activePath_edit (bContext *C)
 	if (scene->active_keyingset <= 0)
 		return 0;
 	else
-		ks= BLI_findlink(&scene->keyingsets, scene->active_keyingset-1);
+		ks = BLI_findlink(&scene->keyingsets, scene->active_keyingset - 1);
 	
 	/* there must be an active KeyingSet and an active path */
 	return ((ks) && (ks->paths.first) && (ks->active_path > 0));
@@ -113,10 +113,10 @@ static int keyingset_poll_activePath_edit (bContext *C)
  
 /* Add a Default (Empty) Keying Set ------------------------- */
 
-static int add_default_keyingset_exec (bContext *C, wmOperator *UNUSED(op))
+static int add_default_keyingset_exec(bContext *C, wmOperator *UNUSED(op))
 {
-	Scene *scene= CTX_data_scene(C);
-	short flag=0, keyingflag=0;
+	Scene *scene = CTX_data_scene(C);
+	short flag = 0, keyingflag = 0;
 	
 	/* validate flags 
 	 *	- absolute KeyingSets should be created by default
@@ -129,15 +129,15 @@ static int add_default_keyingset_exec (bContext *C, wmOperator *UNUSED(op))
 	/* call the API func, and set the active keyingset index */
 	BKE_keyingset_add(&scene->keyingsets, NULL, NULL, flag, keyingflag);
 	
-	scene->active_keyingset= BLI_countlist(&scene->keyingsets);
+	scene->active_keyingset = BLI_countlist(&scene->keyingsets);
 	
 	/* send notifiers */
-	WM_event_add_notifier(C, NC_SCENE|ND_KEYINGSET, NULL);
+	WM_event_add_notifier(C, NC_SCENE | ND_KEYINGSET, NULL);
 	
 	return OPERATOR_FINISHED;
 }
 
-void ANIM_OT_keying_set_add (wmOperatorType *ot)
+void ANIM_OT_keying_set_add(wmOperatorType *ot)
 {
 	/* identifiers */
 	ot->name = "Add Empty Keying Set";
@@ -151,9 +151,9 @@ void ANIM_OT_keying_set_add (wmOperatorType *ot)
 
 /* Remove 'Active' Keying Set ------------------------- */
 
-static int remove_active_keyingset_exec (bContext *C, wmOperator *op)
+static int remove_active_keyingset_exec(bContext *C, wmOperator *op)
 {
-	Scene *scene= CTX_data_scene(C);
+	Scene *scene = CTX_data_scene(C);
 	KeyingSet *ks;
 	
 	/* verify the Keying Set to use:
@@ -161,15 +161,15 @@ static int remove_active_keyingset_exec (bContext *C, wmOperator *op)
 	 *	- return error if it doesn't exist
 	 */
 	if (scene->active_keyingset == 0) {
-		BKE_report(op->reports, RPT_ERROR, "No active Keying Set to remove");
+		BKE_report(op->reports, RPT_ERROR, "No active keying set to remove");
 		return OPERATOR_CANCELLED;
 	}
 	else if (scene->active_keyingset < 0) {
-		BKE_report(op->reports, RPT_ERROR, "Cannot remove built in Keying Set");
+		BKE_report(op->reports, RPT_ERROR, "Cannot remove built in keying set");
 		return OPERATOR_CANCELLED;
 	}
 	else
-		ks= BLI_findlink(&scene->keyingsets, scene->active_keyingset-1);
+		ks = BLI_findlink(&scene->keyingsets, scene->active_keyingset - 1);
 	
 	/* free KeyingSet's data, then remove it from the scene */
 	BKE_keyingset_free(ks);
@@ -179,12 +179,12 @@ static int remove_active_keyingset_exec (bContext *C, wmOperator *op)
 	scene->active_keyingset--;
 	
 	/* send notifiers */
-	WM_event_add_notifier(C, NC_SCENE|ND_KEYINGSET, NULL);
+	WM_event_add_notifier(C, NC_SCENE | ND_KEYINGSET, NULL);
 	
 	return OPERATOR_FINISHED;
 }
 
-void ANIM_OT_keying_set_remove (wmOperatorType *ot)
+void ANIM_OT_keying_set_remove(wmOperatorType *ot)
 {
 	/* identifiers */
 	ot->name = "Remove Active Keying Set";
@@ -198,9 +198,9 @@ void ANIM_OT_keying_set_remove (wmOperatorType *ot)
 
 /* Add Empty Keying Set Path ------------------------- */
 
-static int add_empty_ks_path_exec (bContext *C, wmOperator *op)
+static int add_empty_ks_path_exec(bContext *C, wmOperator *op)
 {
-	Scene *scene= CTX_data_scene(C);
+	Scene *scene = CTX_data_scene(C);
 	KeyingSet *ks;
 	KS_Path *ksp;
 	
@@ -209,25 +209,25 @@ static int add_empty_ks_path_exec (bContext *C, wmOperator *op)
 	 *	- return error if it doesn't exist
 	 */
 	if (scene->active_keyingset == 0) {
-		BKE_report(op->reports, RPT_ERROR, "No active Keying Set to add empty path to");
+		BKE_report(op->reports, RPT_ERROR, "No active keying set to add empty path to");
 		return OPERATOR_CANCELLED;
 	}
 	else
-		ks= BLI_findlink(&scene->keyingsets, scene->active_keyingset-1);
+		ks = BLI_findlink(&scene->keyingsets, scene->active_keyingset - 1);
 	
 	/* don't use the API method for this, since that checks on values... */
-	ksp= MEM_callocN(sizeof(KS_Path), "KeyingSetPath Empty");
+	ksp = MEM_callocN(sizeof(KS_Path), "KeyingSetPath Empty");
 	BLI_addtail(&ks->paths, ksp);
-	ks->active_path= BLI_countlist(&ks->paths);
+	ks->active_path = BLI_countlist(&ks->paths);
 	
-	ksp->groupmode= KSP_GROUP_KSNAME; // XXX?
-	ksp->idtype= ID_OB;
-	ksp->flag= KSP_FLAG_WHOLE_ARRAY;
+	ksp->groupmode = KSP_GROUP_KSNAME; // XXX?
+	ksp->idtype = ID_OB;
+	ksp->flag = KSP_FLAG_WHOLE_ARRAY;
 	
 	return OPERATOR_FINISHED;
 }
 
-void ANIM_OT_keying_set_path_add (wmOperatorType *ot)
+void ANIM_OT_keying_set_path_add(wmOperatorType *ot)
 {
 	/* identifiers */
 	ot->name = "Add Empty Keying Set Path";
@@ -241,14 +241,14 @@ void ANIM_OT_keying_set_path_add (wmOperatorType *ot)
 
 /* Remove Active Keying Set Path ------------------------- */
 
-static int remove_active_ks_path_exec (bContext *C, wmOperator *op)
+static int remove_active_ks_path_exec(bContext *C, wmOperator *op)
 {
-	Scene *scene= CTX_data_scene(C);
-	KeyingSet *ks= BLI_findlink(&scene->keyingsets, scene->active_keyingset-1);
+	Scene *scene = CTX_data_scene(C);
+	KeyingSet *ks = BLI_findlink(&scene->keyingsets, scene->active_keyingset - 1);
 	
 	/* if there is a KeyingSet, find the nominated path to remove */
 	if (ks) {
-		KS_Path *ksp= BLI_findlink(&ks->paths, ks->active_path-1);
+		KS_Path *ksp = BLI_findlink(&ks->paths, ks->active_path - 1);
 		
 		if (ksp) {
 			/* remove the active path from the KeyingSet */
@@ -258,19 +258,19 @@ static int remove_active_ks_path_exec (bContext *C, wmOperator *op)
 			ks->active_path--;
 		}
 		else {
-			BKE_report(op->reports, RPT_ERROR, "No active Keying Set Path to remove");
+			BKE_report(op->reports, RPT_ERROR, "No active keying set path to remove");
 			return OPERATOR_CANCELLED;
 		}
 	}
 	else {
-		BKE_report(op->reports, RPT_ERROR, "No active Keying Set to remove a path from");
+		BKE_report(op->reports, RPT_ERROR, "No active keying set to remove a path from");
 		return OPERATOR_CANCELLED;
 	}
 	
 	return OPERATOR_FINISHED;
 }
 
-void ANIM_OT_keying_set_path_remove (wmOperatorType *ot)
+void ANIM_OT_keying_set_path_remove(wmOperatorType *ot)
 {
 	/* identifiers */
 	ot->name = "Remove Active Keying Set Path";
@@ -287,24 +287,24 @@ void ANIM_OT_keying_set_path_remove (wmOperatorType *ot)
 
 /* Add to KeyingSet Button Operator ------------------------ */
 
-static int add_keyingset_button_exec (bContext *C, wmOperator *op)
+static int add_keyingset_button_exec(bContext *C, wmOperator *op)
 {
-	Main *bmain= CTX_data_main(C);
-	Scene *scene= CTX_data_scene(C);
+	Main *bmain = CTX_data_main(C);
+	Scene *scene = CTX_data_scene(C);
 	KeyingSet *ks = NULL;
-	PropertyRNA *prop= NULL;
-	PointerRNA ptr= {{NULL}};
+	PropertyRNA *prop = NULL;
+	PointerRNA ptr = {{NULL}};
 	char *path = NULL;
-	short success= 0;
-	int index=0, pflag=0;
-	int all= RNA_boolean_get(op->ptr, "all");
+	short success = 0;
+	int index = 0, pflag = 0;
+	int all = RNA_boolean_get(op->ptr, "all");
 	
 	/* verify the Keying Set to use:
 	 *	- use the active one for now (more control over this can be added later)
 	 *	- add a new one if it doesn't exist 
 	 */
 	if (scene->active_keyingset == 0) {
-		short flag=0, keyingflag=0;
+		short flag = 0, keyingflag = 0;
 		
 		/* validate flags 
 		 *	- absolute KeyingSets should be created by default
@@ -317,23 +317,24 @@ static int add_keyingset_button_exec (bContext *C, wmOperator *op)
 			keyingflag |= INSERTKEY_XYZ2RGB;
 			
 		/* call the API func, and set the active keyingset index */
-		ks= BKE_keyingset_add(&scene->keyingsets, "ButtonKeyingSet", "Button Keying Set", flag, keyingflag);
+		ks = BKE_keyingset_add(&scene->keyingsets, "ButtonKeyingSet", "Button Keying Set", flag, keyingflag);
 		
-		scene->active_keyingset= BLI_countlist(&scene->keyingsets);
+		scene->active_keyingset = BLI_countlist(&scene->keyingsets);
 	}
 	else if (scene->active_keyingset < 0) {
-		BKE_report(op->reports, RPT_ERROR, "Cannot add property to built in Keying Set");
+		BKE_report(op->reports, RPT_ERROR, "Cannot add property to built in keying set");
 		return OPERATOR_CANCELLED;
 	}
-	else
-		ks= BLI_findlink(&scene->keyingsets, scene->active_keyingset-1);
+	else {
+		ks = BLI_findlink(&scene->keyingsets, scene->active_keyingset - 1);
+	}
 	
 	/* try to add to keyingset using property retrieved from UI */
 	uiContextActiveProperty(C, &ptr, &prop, &index);
 	
 	/* check if property is able to be added */
 	if (ptr.id.data && ptr.data && prop && RNA_property_animateable(&ptr, prop)) {
-		path= RNA_path_from_ID_to_property(&ptr, prop);
+		path = RNA_path_from_ID_to_property(&ptr, prop);
 		
 		if (path) {
 			/* set flags */
@@ -344,13 +345,13 @@ static int add_keyingset_button_exec (bContext *C, wmOperator *op)
 				 * necessary if we want the entire array for most cases to get included without the user
 				 * having to worry about where they clicked
 				 */
-				index= 0;
+				index = 0;
 			}
 				
 			/* add path to this setting */
 			BKE_keyingset_add_path(ks, ptr.id.data, NULL, path, index, pflag, KSP_GROUP_KSNAME);
-			ks->active_path= BLI_countlist(&ks->paths);
-			success= 1;
+			ks->active_path = BLI_countlist(&ks->paths);
+			success = 1;
 			
 			/* free the temp path created */
 			MEM_freeN(path);
@@ -360,26 +361,28 @@ static int add_keyingset_button_exec (bContext *C, wmOperator *op)
 	if (success) {
 		/* send updates */
 		DAG_ids_flush_update(bmain, 0);
+		WM_event_add_notifier(C, NC_SCENE | ND_KEYINGSET, NULL);
 		
-		/* for now, only send ND_KEYS for KeyingSets */
-		WM_event_add_notifier(C, NC_SCENE|ND_KEYINGSET, NULL);
+		/* show notification/report header, so that users notice that something changed */
+		BKE_reportf(op->reports, RPT_INFO, "Property added to Keying Set: '%s'", ks->name);
 	}
 	
-	return (success)? OPERATOR_FINISHED: OPERATOR_CANCELLED;
+	return (success) ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
 }
 
-void ANIM_OT_keyingset_button_add (wmOperatorType *ot)
+void ANIM_OT_keyingset_button_add(wmOperatorType *ot)
 {
 	/* identifiers */
 	ot->name = "Add to Keying Set";
 	ot->idname = "ANIM_OT_keyingset_button_add";
+	ot->description = "Add current UI-active property to current keying set";
 	
 	/* callbacks */
 	ot->exec = add_keyingset_button_exec; 
-	//op->poll= ???
+	//op->poll = ???
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
 	/* properties */
 	RNA_def_boolean(ot->srna, "all", 1, "All", "Add all elements of the array to a Keying Set");
@@ -387,47 +390,48 @@ void ANIM_OT_keyingset_button_add (wmOperatorType *ot)
 
 /* Remove from KeyingSet Button Operator ------------------------ */
 
-static int remove_keyingset_button_exec (bContext *C, wmOperator *op)
+static int remove_keyingset_button_exec(bContext *C, wmOperator *op)
 {
-	Main *bmain= CTX_data_main(C);
-	Scene *scene= CTX_data_scene(C);
+	Main *bmain = CTX_data_main(C);
+	Scene *scene = CTX_data_scene(C);
 	KeyingSet *ks = NULL;
-	PropertyRNA *prop= NULL;
-	PointerRNA ptr= {{NULL}};
+	PropertyRNA *prop = NULL;
+	PointerRNA ptr = {{NULL}};
 	char *path = NULL;
-	short success= 0;
-	int index=0;
+	short success = 0;
+	int index = 0;
 	
 	/* verify the Keying Set to use:
 	 *	- use the active one for now (more control over this can be added later)
 	 *	- return error if it doesn't exist
 	 */
 	if (scene->active_keyingset == 0) {
-		BKE_report(op->reports, RPT_ERROR, "No active Keying Set to remove property from");
+		BKE_report(op->reports, RPT_ERROR, "No active keying set to remove property from");
 		return OPERATOR_CANCELLED;
 	}
 	else if (scene->active_keyingset < 0) {
-		BKE_report(op->reports, RPT_ERROR, "Cannot remove property from built in Keying Set");
+		BKE_report(op->reports, RPT_ERROR, "Cannot remove property from built in keying set");
 		return OPERATOR_CANCELLED;
 	}
-	else
-		ks= BLI_findlink(&scene->keyingsets, scene->active_keyingset-1);
+	else {
+		ks = BLI_findlink(&scene->keyingsets, scene->active_keyingset - 1);
+	}
 	
 	/* try to add to keyingset using property retrieved from UI */
 	uiContextActiveProperty(C, &ptr, &prop, &index);
 
 	if (ptr.id.data && ptr.data && prop) {
-		path= RNA_path_from_ID_to_property(&ptr, prop);
+		path = RNA_path_from_ID_to_property(&ptr, prop);
 		
 		if (path) {
 			KS_Path *ksp;
 			
 			/* try to find a path matching this description */
-			ksp= BKE_keyingset_find_path(ks, ptr.id.data, ks->name, path, index, KSP_GROUP_KSNAME);
-
+			ksp = BKE_keyingset_find_path(ks, ptr.id.data, ks->name, path, index, KSP_GROUP_KSNAME);
+			
 			if (ksp) {
 				BKE_keyingset_free_path(ks, ksp);
-				success= 1;
+				success = 1;
 			}
 			
 			/* free temp path used */
@@ -439,26 +443,28 @@ static int remove_keyingset_button_exec (bContext *C, wmOperator *op)
 	if (success) {
 		/* send updates */
 		DAG_ids_flush_update(bmain, 0);
+		WM_event_add_notifier(C, NC_SCENE | ND_KEYINGSET, NULL);
 		
-		/* for now, only send ND_KEYS for KeyingSets */
-		WM_event_add_notifier(C, NC_SCENE|ND_KEYINGSET, NULL);
+		/* show warning */
+		BKE_report(op->reports, RPT_INFO, "Property removed from Keying Set");
 	}
 	
-	return (success)? OPERATOR_FINISHED: OPERATOR_CANCELLED;
+	return (success) ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
 }
 
-void ANIM_OT_keyingset_button_remove (wmOperatorType *ot)
+void ANIM_OT_keyingset_button_remove(wmOperatorType *ot)
 {
 	/* identifiers */
 	ot->name = "Remove from Keying Set";
 	ot->idname = "ANIM_OT_keyingset_button_remove";
+	ot->description = "Remove current UI-active property from current keying set";
 	
 	/* callbacks */
 	ot->exec = remove_keyingset_button_exec; 
-	//op->poll= ???
+	//op->poll = ???
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
 
 /* ******************************************* */
@@ -466,7 +472,7 @@ void ANIM_OT_keyingset_button_remove (wmOperatorType *ot)
 /* Change Active KeyingSet Operator ------------------------ */
 /* This operator checks if a menu should be shown for choosing the KeyingSet to make the active one */
 
-static int keyingset_active_menu_invoke (bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int keyingset_active_menu_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
 {
 	uiPopupMenu *pup;
 	uiLayout *layout;
@@ -480,7 +486,7 @@ static int keyingset_active_menu_invoke (bContext *C, wmOperator *op, wmEvent *U
 	return OPERATOR_CANCELLED;
 }
 
-static int keyingset_active_menu_exec (bContext *C, wmOperator *op)
+static int keyingset_active_menu_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
 	int type = RNA_enum_get(op->ptr, "type");
@@ -489,18 +495,19 @@ static int keyingset_active_menu_exec (bContext *C, wmOperator *op)
 	scene->active_keyingset = type;
 	
 	/* send notifiers */
-	WM_event_add_notifier(C, NC_SCENE|ND_KEYINGSET, NULL);
+	WM_event_add_notifier(C, NC_SCENE | ND_KEYINGSET, NULL);
 	
 	return OPERATOR_FINISHED;
 }
  
-void ANIM_OT_keying_set_active_set (wmOperatorType *ot)
+void ANIM_OT_keying_set_active_set(wmOperatorType *ot)
 {
 	PropertyRNA *prop;
 	
 	/* identifiers */
 	ot->name = "Set Active Keying Set";
 	ot->idname = "ANIM_OT_keying_set_active_set";
+	ot->description = "Select a new keying set as the active one";
 	
 	/* callbacks */
 	ot->invoke = keyingset_active_menu_invoke;
@@ -508,10 +515,10 @@ void ANIM_OT_keying_set_active_set (wmOperatorType *ot)
 	ot->poll = ED_operator_areaactive;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 	
 	/* keyingset to use (dynamic enum) */
-	prop= RNA_def_enum(ot->srna, "type", DummyRNA_DEFAULT_items, 0, "Keying Set", "The Keying Set to use");
+	prop = RNA_def_enum(ot->srna, "type", DummyRNA_DEFAULT_items, 0, "Keying Set", "The Keying Set to use");
 	RNA_def_enum_funcs(prop, ANIM_keying_sets_enum_itemf);
 /*	RNA_def_property_flag(prop, PROP_HIDDEN);*/
 }
@@ -528,7 +535,7 @@ ListBase builtin_keyingsets = {NULL, NULL};
 /* --------------- */
 
 /* Find KeyingSet type info given a name */
-KeyingSetInfo *ANIM_keyingset_info_find_named (const char name[])
+KeyingSetInfo *ANIM_keyingset_info_find_name(const char name[])
 {
 	/* sanity checks */
 	if ((name == NULL) || (name[0] == 0))
@@ -539,9 +546,9 @@ KeyingSetInfo *ANIM_keyingset_info_find_named (const char name[])
 }
 
 /* Find builtin KeyingSet by name */
-KeyingSet *ANIM_builtin_keyingset_get_named (KeyingSet *prevKS, const char name[])
+KeyingSet *ANIM_builtin_keyingset_get_named(KeyingSet *prevKS, const char name[])
 {
-	KeyingSet *ks, *first=NULL;
+	KeyingSet *ks, *first = NULL;
 	
 	/* sanity checks  any name to check? */
 	if (name[0] == 0)
@@ -549,12 +556,12 @@ KeyingSet *ANIM_builtin_keyingset_get_named (KeyingSet *prevKS, const char name[
 	
 	/* get first KeyingSet to use */
 	if (prevKS && prevKS->next)
-		first= prevKS->next;
+		first = prevKS->next;
 	else
-		first= builtin_keyingsets.first;
+		first = builtin_keyingsets.first;
 	
 	/* loop over KeyingSets checking names */
-	for (ks= first; ks; ks= ks->next) {
+	for (ks = first; ks; ks = ks->next) {
 		if (strcmp(name, ks->idname) == 0)
 			return ks;
 	}
@@ -571,7 +578,7 @@ KeyingSet *ANIM_builtin_keyingset_get_named (KeyingSet *prevKS, const char name[
 /* --------------- */
 
 /* Add the given KeyingSetInfo to the list of type infos, and create an appropriate builtin set too */
-void ANIM_keyingset_info_register (KeyingSetInfo *ksi)
+void ANIM_keyingset_info_register(KeyingSetInfo *ksi)
 {
 	KeyingSet *ks;
 	
@@ -591,23 +598,23 @@ void ANIM_keyingset_info_register (KeyingSetInfo *ksi)
 }
 
 /* Remove the given KeyingSetInfo from the list of type infos, and also remove the builtin set if appropriate */
-void ANIM_keyingset_info_unregister (Main *bmain, KeyingSetInfo *ksi)
+void ANIM_keyingset_info_unregister(Main *bmain, KeyingSetInfo *ksi)
 {
 	KeyingSet *ks, *ksn;
-	
+
 	/* find relevant builtin KeyingSets which use this, and remove them */
-	// TODO: this isn't done now, since unregister is really only used atm when we
-	// reload the scripts, which kindof defeats the purpose of "builtin"?
-	for (ks= builtin_keyingsets.first; ks; ks= ksn) {
+	/* TODO: this isn't done now, since unregister is really only used atm when we
+	 * reload the scripts, which kindof defeats the purpose of "builtin"? */
+	for (ks = builtin_keyingsets.first; ks; ks = ksn) {
 		ksn = ks->next;
-		
+
 		/* remove if matching typeinfo name */
 		if (strcmp(ks->typeinfo, ksi->idname) == 0) {
 			Scene *scene;
 			BKE_keyingset_free(ks);
 			BLI_remlink(&builtin_keyingsets, ks);
 
-			for (scene= bmain->scene.first; scene; scene= scene->id.next)
+			for (scene = bmain->scene.first; scene; scene = scene->id.next)
 				BLI_remlink_safe(&scene->keyingsets, ks);
 
 			MEM_freeN(ks);
@@ -620,13 +627,13 @@ void ANIM_keyingset_info_unregister (Main *bmain, KeyingSetInfo *ksi)
 
 /* --------------- */
 
-void ANIM_keyingset_infos_exit (void)
+void ANIM_keyingset_infos_exit(void)
 {
 	KeyingSetInfo *ksi, *next;
 	
 	/* free type infos */
-	for (ksi=keyingset_type_infos.first; ksi; ksi=next) {
-		next= ksi->next;
+	for (ksi = keyingset_type_infos.first; ksi; ksi = next) {
+		next = ksi->next;
 		
 		/* free extra RNA data, and remove from list */
 		if (ksi->ext.free)
@@ -644,7 +651,7 @@ void ANIM_keyingset_infos_exit (void)
 /* Getters for Active/Indices ----------------------------- */
 
 /* Get the active Keying Set for the Scene provided */
-KeyingSet *ANIM_scene_get_active_keyingset (Scene *scene)
+KeyingSet *ANIM_scene_get_active_keyingset(Scene *scene)
 {
 	/* if no scene, we've got no hope of finding the Keying Set */
 	if (scene == NULL)
@@ -656,13 +663,13 @@ KeyingSet *ANIM_scene_get_active_keyingset (Scene *scene)
 	 *	- < 0: a builtin keying set
 	 */
 	if (scene->active_keyingset > 0)
-		return BLI_findlink(&scene->keyingsets, scene->active_keyingset-1);
+		return BLI_findlink(&scene->keyingsets, scene->active_keyingset - 1);
 	else
-		return BLI_findlink(&builtin_keyingsets, (-scene->active_keyingset)-1);
+		return BLI_findlink(&builtin_keyingsets, (-scene->active_keyingset) - 1);
 }
 
 /* Get the index of the Keying Set provided, for the given Scene */
-int ANIM_scene_get_keyingset_index (Scene *scene, KeyingSet *ks)
+int ANIM_scene_get_keyingset_index(Scene *scene, KeyingSet *ks)
 {
 	int index;
 	
@@ -696,7 +703,7 @@ KeyingSet *ANIM_get_keyingset_for_autokeying(Scene *scene, const char *tranformK
 {
 	/* get KeyingSet to use 
 	 *	- use the active KeyingSet if defined (and user wants to use it for all autokeying), 
-	 * 	  or otherwise key transforms only
+	 *    or otherwise key transforms only
 	 */
 	if (IS_AUTOKEY_FLAG(scene, ONLYKEYINGSET) && (scene->active_keyingset))
 		return ANIM_scene_get_active_keyingset(scene);
@@ -709,13 +716,13 @@ KeyingSet *ANIM_get_keyingset_for_autokeying(Scene *scene, const char *tranformK
 /* Menu of All Keying Sets ----------------------------- */
 
 /* Dynamically populate an enum of Keying Sets */
-EnumPropertyItem *ANIM_keying_sets_enum_itemf (bContext *C, PointerRNA *UNUSED(ptr), PropertyRNA *UNUSED(prop), int *free)
+EnumPropertyItem *ANIM_keying_sets_enum_itemf(bContext *C, PointerRNA *UNUSED(ptr), PropertyRNA *UNUSED(prop), int *free)
 {
 	Scene *scene = CTX_data_scene(C);
 	KeyingSet *ks;
-	EnumPropertyItem *item= NULL, item_tmp= {0};
-	int totitem= 0;
-	int i= 0;
+	EnumPropertyItem *item = NULL, item_tmp = {0};
+	int totitem = 0;
+	int i = 0;
 
 	if (C == NULL) {
 		return DummyRNA_DEFAULT_items;
@@ -726,9 +733,9 @@ EnumPropertyItem *ANIM_keying_sets_enum_itemf (bContext *C, PointerRNA *UNUSED(p
 	 */
 	if (scene->active_keyingset) {
 		/* active Keying Set */
-		item_tmp.identifier= "__ACTIVE__";
-		item_tmp.name= "Active Keying Set";
-		item_tmp.value= i;
+		item_tmp.identifier = "__ACTIVE__";
+		item_tmp.name = "Active Keying Set";
+		item_tmp.value = i;
 		RNA_enum_item_add(&item, &totitem, &item_tmp);
 		
 		/* separator */
@@ -741,12 +748,12 @@ EnumPropertyItem *ANIM_keying_sets_enum_itemf (bContext *C, PointerRNA *UNUSED(p
 	 *	- these are listed in the order in which they were defined for the active scene
 	 */
 	if (scene->keyingsets.first) {
-		for (ks= scene->keyingsets.first; ks; ks= ks->next, i++) {
+		for (ks = scene->keyingsets.first; ks; ks = ks->next, i++) {
 			if (ANIM_keyingset_context_ok_poll(C, ks)) {
 				item_tmp.identifier = ks->idname;
 				item_tmp.name = ks->name;
 				item_tmp.description = ks->description;
-				item_tmp.value= i;
+				item_tmp.value = i;
 				RNA_enum_item_add(&item, &totitem, &item_tmp);
 			}
 		}
@@ -756,20 +763,20 @@ EnumPropertyItem *ANIM_keying_sets_enum_itemf (bContext *C, PointerRNA *UNUSED(p
 	}
 	
 	/* builtin Keying Sets */
-	i= -1;
-	for (ks= builtin_keyingsets.first; ks; ks= ks->next, i--) {
+	i = -1;
+	for (ks = builtin_keyingsets.first; ks; ks = ks->next, i--) {
 		/* only show KeyingSet if context is suitable */
 		if (ANIM_keyingset_context_ok_poll(C, ks)) {
 			item_tmp.identifier = ks->idname;
 			item_tmp.name = ks->name;
 			item_tmp.description = ks->description;
-			item_tmp.value= i;
+			item_tmp.value = i;
 			RNA_enum_item_add(&item, &totitem, &item_tmp);
 		}
 	}
 
 	RNA_enum_item_end(&item, &totitem);
-	*free= 1;
+	*free = 1;
 
 	return item;
 }
@@ -780,16 +787,16 @@ EnumPropertyItem *ANIM_keying_sets_enum_itemf (bContext *C, PointerRNA *UNUSED(p
 /* Polling API ----------------------------------------------- */
 
 /* Check if KeyingSet can be used in the current context */
-short ANIM_keyingset_context_ok_poll (bContext *C, KeyingSet *ks)
+short ANIM_keyingset_context_ok_poll(bContext *C, KeyingSet *ks)
 {
 	if ((ks->flag & KEYINGSET_ABSOLUTE) == 0) {
-		KeyingSetInfo *ksi = ANIM_keyingset_info_find_named(ks->typeinfo);
+		KeyingSetInfo *ksi = ANIM_keyingset_info_find_name(ks->typeinfo);
 		
 		/* get the associated 'type info' for this KeyingSet */
 		if (ksi == NULL)
 			return 0;
-		// TODO: check for missing callbacks!
-		
+		/* TODO: check for missing callbacks! */
+
 		/* check if it can be used in the current context */
 		return (ksi->poll(ksi, C));
 	}
@@ -800,12 +807,12 @@ short ANIM_keyingset_context_ok_poll (bContext *C, KeyingSet *ks)
 /* Special 'Overrides' Iterator for Relative KeyingSets ------ */
 
 /* 'Data Sources' for relative Keying Set 'overrides' 
- * 	- this is basically a wrapper for PointerRNA's in a linked list
+ *  - this is basically a wrapper for PointerRNA's in a linked list
  *	- do not allow this to be accessed from outside for now
  */
 typedef struct tRKS_DSource {
 	struct tRKS_DSource *next, *prev;
-	PointerRNA ptr;		/* the whole point of this exercise! */
+	PointerRNA ptr;     /* the whole point of this exercise! */
 } tRKS_DSource;
 
 
@@ -813,7 +820,7 @@ typedef struct tRKS_DSource {
  * relative Keying Sets, with the main usage of this being operators 
  * requiring Auto Keyframing. Internal Use Only!
  */
-static void RKS_ITER_overrides_list (KeyingSetInfo *ksi, bContext *C, KeyingSet *ks, ListBase *dsources)
+static void RKS_ITER_overrides_list(KeyingSetInfo *ksi, bContext *C, KeyingSet *ks, ListBase *dsources)
 {
 	tRKS_DSource *ds;
 	
@@ -824,7 +831,7 @@ static void RKS_ITER_overrides_list (KeyingSetInfo *ksi, bContext *C, KeyingSet
 }
 
 /* Add new data source for relative Keying Sets */
-void ANIM_relative_keyingset_add_source (ListBase *dsources, ID *id, StructRNA *srna, void *data)
+void ANIM_relative_keyingset_add_source(ListBase *dsources, ID *id, StructRNA *srna, void *data)
 {
 	tRKS_DSource *ds;
 	
@@ -857,7 +864,7 @@ void ANIM_relative_keyingset_add_source (ListBase *dsources, ID *id, StructRNA *
  *
  * Returns 0 if succeeded, otherwise an error code: eModifyKey_Returns
  */
-short ANIM_validate_keyingset (bContext *C, ListBase *dsources, KeyingSet *ks)
+short ANIM_validate_keyingset(bContext *C, ListBase *dsources, KeyingSet *ks)
 {
 	/* sanity check */
 	if (ks == NULL)
@@ -865,7 +872,7 @@ short ANIM_validate_keyingset (bContext *C, ListBase *dsources, KeyingSet *ks)
 	
 	/* if relative Keying Sets, poll and build up the paths */
 	if ((ks->flag & KEYINGSET_ABSOLUTE) == 0) {
-		KeyingSetInfo *ksi = ANIM_keyingset_info_find_named(ks->typeinfo);
+		KeyingSetInfo *ksi = ANIM_keyingset_info_find_name(ks->typeinfo);
 		
 		/* clear all existing paths 
 		 * NOTE: BKE_keyingset_free() frees all of the paths for the KeyingSet, but not the set itself
@@ -875,8 +882,8 @@ short ANIM_validate_keyingset (bContext *C, ListBase *dsources, KeyingSet *ks)
 		/* get the associated 'type info' for this KeyingSet */
 		if (ksi == NULL)
 			return MODIFYKEY_MISSING_TYPEINFO;
-		// TODO: check for missing callbacks!
-		
+		/* TODO: check for missing callbacks! */
+
 		/* check if it can be used in the current context */
 		if (ksi->poll(ksi, C)) {
 			/* if a list of data sources are provided, run a special iterator over them,
@@ -888,11 +895,13 @@ short ANIM_validate_keyingset (bContext *C, ListBase *dsources, KeyingSet *ks)
 				ksi->iter(ksi, C, ks);
 				
 			/* if we don't have any paths now, then this still qualifies as invalid context */
+			// FIXME: we need some error conditions (to be retrieved from the iterator why this failed!)
 			if (ks->paths.first == NULL)
 				return MODIFYKEY_INVALID_CONTEXT;
 		}
 		else {
 			/* poll callback tells us that KeyingSet is useless in current context */
+			// FIXME: the poll callback needs to give us more info why
 			return MODIFYKEY_INVALID_CONTEXT;
 		}
 	}
@@ -905,13 +914,13 @@ short ANIM_validate_keyingset (bContext *C, ListBase *dsources, KeyingSet *ks)
  * by the KeyingSet. This takes into account many of the different combinations of using KeyingSets.
  * Returns the number of channels that keyframes were added to
  */
-int ANIM_apply_keyingset (bContext *C, ListBase *dsources, bAction *act, KeyingSet *ks, short mode, float cfra)
+int ANIM_apply_keyingset(bContext *C, ListBase *dsources, bAction *act, KeyingSet *ks, short mode, float cfra)
 {
-	Scene *scene= CTX_data_scene(C);
+	Scene *scene = CTX_data_scene(C);
 	ReportList *reports = CTX_wm_reports(C);
 	KS_Path *ksp;
-	int kflag=0, success= 0;
-	char *groupname= NULL;
+	int kflag = 0, success = 0;
+	char *groupname = NULL;
 	
 	/* sanity checks */
 	if (ks == NULL)
@@ -920,13 +929,13 @@ int ANIM_apply_keyingset (bContext *C, ListBase *dsources, bAction *act, KeyingS
 	/* get flags to use */
 	if (mode == MODIFYKEY_MODE_INSERT) {
 		/* use KeyingSet's flags as base */
-		kflag= ks->keyingflag;
+		kflag = ks->keyingflag;
 		
 		/* suppliment with info from the context */
 		kflag |= ANIM_get_keyframing_flags(scene, 1);
 	}
 	else if (mode == MODIFYKEY_MODE_DELETE)
-		kflag= 0;
+		kflag = 0;
 	
 	/* if relative Keying Sets, poll and build up the paths */
 	success = ANIM_validate_keyingset(C, dsources, ks);
@@ -937,15 +946,15 @@ int ANIM_apply_keyingset (bContext *C, ListBase *dsources, bAction *act, KeyingS
 	}
 	
 	/* apply the paths as specified in the KeyingSet now */
-	for (ksp= ks->paths.first; ksp; ksp= ksp->next) { 
+	for (ksp = ks->paths.first; ksp; ksp = ksp->next) {
 		int arraylen, i;
 		short kflag2;
 		
 		/* skip path if no ID pointer is specified */
 		if (ksp->id == NULL) {
 			BKE_reportf(reports, RPT_WARNING,
-				"Skipping path in Keying Set, as it has no ID (KS = '%s', Path = '%s'[%d])",
-				ks->name, ksp->rna_path, ksp->array_index);
+			            "Skipping path in keying set, as it has no ID (KS = '%s', path = '%s[%d]')",
+			            ks->name, ksp->rna_path, ksp->array_index);
 			continue;
 		}
 		
@@ -954,17 +963,17 @@ int ANIM_apply_keyingset (bContext *C, ListBase *dsources, bAction *act, KeyingS
 		
 		/* get pointer to name of group to add channels to */
 		if (ksp->groupmode == KSP_GROUP_NONE)
-			groupname= NULL;
+			groupname = NULL;
 		else if (ksp->groupmode == KSP_GROUP_KSNAME)
-			groupname= ks->name;
+			groupname = ks->name;
 		else
-			groupname= ksp->group;
+			groupname = ksp->group;
 		
 		/* init arraylen and i - arraylen should be greater than i so that
 		 * normal non-array entries get keyframed correctly
 		 */
-		i= ksp->array_index;
-		arraylen= i;
+		i = ksp->array_index;
+		arraylen = i;
 		
 		/* get length of array if whole array option is enabled */
 		if (ksp->flag & KSP_FLAG_WHOLE_ARRAY) {
@@ -973,7 +982,7 @@ int ANIM_apply_keyingset (bContext *C, ListBase *dsources, bAction *act, KeyingS
 			
 			RNA_id_pointer_create(ksp->id, &id_ptr);
 			if (RNA_path_resolve(&id_ptr, ksp->rna_path, &ptr, &prop) && prop)
-				arraylen= RNA_property_array_length(&ptr, prop);
+				arraylen = RNA_property_array_length(&ptr, prop);
 		}
 		
 		/* we should do at least one step */
@@ -995,15 +1004,15 @@ int ANIM_apply_keyingset (bContext *C, ListBase *dsources, bAction *act, KeyingS
 		switch (GS(ksp->id->name)) {
 			case ID_OB: /* Object (or Object-Related) Keyframes */
 			{
-				Object *ob= (Object *)ksp->id;
+				Object *ob = (Object *)ksp->id;
 				
-				ob->recalc |= OB_RECALC_OB|OB_RECALC_DATA|OB_RECALC_TIME; // XXX: only object transforms only?
+				ob->recalc |= OB_RECALC_OB | OB_RECALC_DATA | OB_RECALC_TIME; // XXX: only object transforms only?
 			}
-				break;
+			break;
 		}
 		
 		/* send notifiers for updates (this doesn't require context to work!) */
-		WM_main_add_notifier(NC_ANIMATION|ND_KEYFRAME|NA_EDITED, NULL);
+		WM_main_add_notifier(NC_ANIMATION | ND_KEYFRAME | NA_EDITED, NULL);
 	}
 	
 	/* return the number of channels successfully affected */
