@@ -32,15 +32,16 @@
 #define __ED_MBALL_H__
 
 struct bContext;
+struct MetaBall;
 struct Object;
 struct wmKeyConfig;
 
 void ED_operatortypes_metaball(void);
 void ED_keymap_metaball(struct wmKeyConfig *keyconf);
 
-struct MetaElem *add_metaball_primitive(struct bContext *C, float mat[4][4], int type, int newname);
+struct MetaElem *add_metaball_primitive(struct bContext *C, struct Object *obedit, float mat[4][4], float dia, int type, int newname);
 
-int mouse_mball(struct bContext *C, const int mval[2], int extend);
+int mouse_mball(struct bContext *C, const int mval[2], int extend, int deselect, int toggle);
 
 void free_editMball(struct Object *obedit);
 void make_editMball(struct Object *obedit);
@@ -48,5 +49,6 @@ void load_editMball(struct Object *obedit);
 
 void undo_push_mball(struct bContext *C, const char *name);
 
-#endif
+void ED_mball_transform(struct MetaBall *mb, float *mat);
 
+#endif  /* __ED_MBALL_H__ */
