@@ -41,27 +41,46 @@ struct bNodeTree;
 struct bNode;
 struct bNodeTree;
 struct ScrArea;
+struct Scene;
+struct View2D;
+
+typedef enum {
+	NODE_TOP    = 1,
+	NODE_BOTTOM = 2,
+	NODE_LEFT   = 4,
+	NODE_RIGHT  = 8
+} NodeBorder;
 
 /* drawnode.c */
-void ED_init_node_butfuncs(void);
+void ED_node_init_butfuncs(void);
+void ED_node_sample_set(const float col[4]);
+void ED_node_draw_snap(struct View2D *v2d, const float cent[2], float size, NodeBorder border);
 
 /* node_draw.c */
 void ED_node_tree_update(struct SpaceNode *snode, struct Scene *scene);
 void ED_node_changed_update(struct ID *id, struct bNode *node);
 void ED_node_generic_update(struct Main *bmain, struct bNodeTree *ntree, struct bNode *node);
+void ED_node_sort(struct bNodeTree *ntree);
+
+/* node_relationships.c */
+void ED_node_link_intersect_test(struct ScrArea *sa, int test);
+void ED_node_link_insert(struct ScrArea *sa);
 
 /* node_edit.c */
 void ED_node_shader_default(struct Scene *scene, struct ID *id);
 void ED_node_composit_default(struct Scene *sce);
 void ED_node_texture_default(struct Tex *tex);
-void ED_node_link_intersect_test(struct ScrArea *sa, int test);
-void ED_node_link_insert(struct ScrArea *sa);
-
-void ED_node_update_hierarchy(struct bContext *C, struct bNodeTree *ntree);
-
+int  ED_node_select_check(ListBase *lb);
+void ED_node_post_apply_transform(struct bContext *C, struct bNodeTree *ntree);
 void ED_node_set_active(struct Main *bmain, struct bNodeTree *ntree, struct bNode *node);
-/* node ops.c */
+
+void ED_node_composite_job(const bContext *C, struct bNodeTree *nodetree, struct Scene *scene_owner);
+
+/* node_ops.c */
 void ED_operatormacros_node(void);
 
+/* node_view.c */
+int ED_space_node_color_sample(struct SpaceNode *snode, struct ARegion *ar, int mval[2], float r_col[3]);
+
 #endif /* __ED_NODE_H__ */
 
