@@ -35,27 +35,27 @@
 
 /* for animplayer */
 typedef struct ScreenAnimData {
-	ARegion *ar;		/* do not read from this, only for comparing if region exists */
+	ARegion *ar;            /* do not read from this, only for comparing if region exists */
 	short redraws;
 	short refresh;
-	short flag;			/* flags for playback */
-	int sfra;			/* frame that playback was started from */
-	int nextfra;		/* next frame to go to (when ANIMPLAY_FLAG_USE_NEXT_FRAME is set) */
-	double last_duration;	/* used for frame dropping */
+	short flag;             /* flags for playback */
+	int sfra;               /* frame that playback was started from */
+	int nextfra;            /* next frame to go to (when ANIMPLAY_FLAG_USE_NEXT_FRAME is set) */
+	double last_duration;   /* used for frame dropping */
 } ScreenAnimData;
 
 /* for animplayer */
 enum {
-		/* user-setting - frame range is played backwards */
-	ANIMPLAY_FLAG_REVERSE		= (1<<0),
-		/* temporary - playback just jumped to the start/end */
-	ANIMPLAY_FLAG_JUMPED		= (1<<1),
-		/* drop frames as needed to maintain framerate */
-	ANIMPLAY_FLAG_SYNC			= (1<<2),
-		/* don't drop frames (and ignore SCE_FRAME_DROP flag) */
-	ANIMPLAY_FLAG_NO_SYNC		= (1<<3),
-		/* use nextfra at next timer update */
-	ANIMPLAY_FLAG_USE_NEXT_FRAME = (1<<4)
+	/* user-setting - frame range is played backwards */
+	ANIMPLAY_FLAG_REVERSE       = (1 << 0),
+	/* temporary - playback just jumped to the start/end */
+	ANIMPLAY_FLAG_JUMPED        = (1 << 1),
+	/* drop frames as needed to maintain framerate */
+	ANIMPLAY_FLAG_SYNC          = (1 << 2),
+	/* don't drop frames (and ignore SCE_FRAME_DROP flag) */
+	ANIMPLAY_FLAG_NO_SYNC       = (1 << 3),
+	/* use nextfra at next timer update */
+	ANIMPLAY_FLAG_USE_NEXT_FRAME = (1 << 4)
 };
 
 /* ----------------------------------------------------- */
@@ -76,10 +76,10 @@ typedef struct ScreenFrameRateInfo {
 
 /* Enum for Action Zone Edges. Which edge of area is action zone. */
 typedef enum {
-	AE_RIGHT_TO_TOPLEFT,	/* Region located on the left, _right_ edge is action zone. Region minimised to the top left */
-	AE_LEFT_TO_TOPRIGHT,	/* Region located on the right, _left_ edge is action zone. Region minimised to the top right */
-	AE_TOP_TO_BOTTOMRIGHT,		/* Region located at the bottom, _top_ edge is action zone. Region minimised to the bottom right */
-	AE_BOTTOM_TO_TOPLEFT	/* Region located at the top, _bottom_edge is action zone. Region minimised to the top left */
+	AE_RIGHT_TO_TOPLEFT,    /* Region located on the left, _right_ edge is action zone. Region minimized to the top left */
+	AE_LEFT_TO_TOPRIGHT,    /* Region located on the right, _left_ edge is action zone. Region minimized to the top right */
+	AE_TOP_TO_BOTTOMRIGHT,  /* Region located at the bottom, _top_ edge is action zone. Region minimized to the bottom right */
+	AE_BOTTOM_TO_TOPLEFT    /* Region located at the top, _bottom_edge is action zone. Region minimized to the top left */
 } AZEdge;
 
 /* for editing areas/regions */
@@ -94,11 +94,11 @@ typedef struct AZone {
 	/* for draw */
 	short x1, y1, x2, y2;
 	/* for clip */
-	rcti rect;	
+	rcti rect;
 } AZone;
 
 /* actionzone type */
-#define	AZONE_AREA			1
-#define AZONE_REGION		2
+#define AZONE_AREA      1  /* corner widgets for splitting areas */
+#define AZONE_REGION    2  /* when a region is collapsed, draw a handle to expose */
 
 #endif /* __ED_SCREEN_TYPES_H__ */
