@@ -43,39 +43,39 @@ struct Mesh;
 
 /* ed_util.c */
 
-void	ED_editors_init			(struct bContext *C);
-void	ED_editors_exit			(struct bContext *C);
+void    ED_editors_init(struct bContext *C);
+void    ED_editors_exit(struct bContext *C);
 
 /* ************** Undo ************************ */
 
 /* undo.c */
-void	ED_undo_push			(struct bContext *C, const char *str);
-void	ED_undo_push_op			(struct bContext *C, struct wmOperator *op);
-void	ED_undo_pop_op			(struct bContext *C, struct wmOperator *op);
-void	ED_undo_pop				(struct bContext *C);
-void	ED_undo_redo			(struct bContext *C);
-void	ED_OT_undo				(struct wmOperatorType *ot);
-void	ED_OT_undo_push			(struct wmOperatorType *ot);
-void	ED_OT_redo				(struct wmOperatorType *ot);
-void	ED_OT_undo_history		(struct wmOperatorType *ot);
+void    ED_undo_push(struct bContext *C, const char *str);
+void    ED_undo_push_op(struct bContext *C, struct wmOperator *op);
+void    ED_undo_pop_op(struct bContext *C, struct wmOperator *op);
+void    ED_undo_pop(struct bContext *C);
+void    ED_undo_redo(struct bContext *C);
+void    ED_OT_undo(struct wmOperatorType *ot);
+void    ED_OT_undo_push(struct wmOperatorType *ot);
+void    ED_OT_redo(struct wmOperatorType *ot);
+void    ED_OT_undo_history(struct wmOperatorType *ot);
 
-int		ED_undo_operator_repeat(struct bContext *C, struct wmOperator *op);
-	/* convenience since UI callbacks use this mostly*/
-void	ED_undo_operator_repeat_cb(struct bContext *C, void *arg_op, void *arg_unused);
-void	ED_undo_operator_repeat_cb_evt(struct bContext *C, void *arg_op, int arg_unused);
+int     ED_undo_operator_repeat(struct bContext *C, struct wmOperator *op);
+/* convenience since UI callbacks use this mostly*/
+void    ED_undo_operator_repeat_cb(struct bContext *C, void *arg_op, void *arg_unused);
+void    ED_undo_operator_repeat_cb_evt(struct bContext *C, void *arg_op, int arg_unused);
 
-int		ED_undo_valid			(const struct bContext *C, const char *undoname);
+int     ED_undo_valid(const struct bContext *C, const char *undoname);
 
 /* undo_editmode.c */
 void undo_editmode_push(struct bContext *C, const char *name, 
-						void * (*getdata)(struct bContext *C),
-						void (*freedata)(void *), 
-						void (*to_editmode)(void *, void *, void *),  
-						void *(*from_editmode)(void *, void *),
-						int (*validate_undo)(void *, void *));
+                        void * (*getdata)(struct bContext *C),
+                        void (*freedata)(void *),
+                        void (*to_editmode)(void *, void *, void *),
+                        void *(*from_editmode)(void *, void *),
+                        int (*validate_undo)(void *, void *));
 
-						
-void	undo_editmode_clear			(void);
+
+void    undo_editmode_clear(void);
 
 /* crazyspace.c */
 float *crazyspace_get_mapped_editverts(struct Scene *scene, struct Object *obedit);
@@ -84,6 +84,8 @@ void crazyspace_set_quats_mesh(struct Mesh *me, float *origcos, float *mappedcos
 int sculpt_get_first_deform_matrices(struct Scene *scene, struct Object *ob, float (**deformmats)[3][3], float (**deformcos)[3]);
 void crazyspace_build_sculpt(struct Scene *scene, struct Object *ob, float (**deformmats)[3][3], float (**deformcos)[3]);
 
+/* cut-paste buffer free */
+void ED_clipboard_posebuf_free(void);
 
 /* ************** XXX OLD CRUFT WARNING ************* */
 
