@@ -94,10 +94,10 @@ DEF_ICON(LINK)
 DEF_ICON(INLINK)
 DEF_ICON(PLUGIN)
 	
-	/* various ui */	
+	/* various ui */
 DEF_ICON(HELP)
 DEF_ICON(GHOST_ENABLED)
-DEF_ICON(COLOR)
+DEF_ICON(COLOR)  /* see COLOR_RED/GREEN/BLUE */
 DEF_ICON(LINKED)
 DEF_ICON(UNLINKED)
 DEF_ICON(HAND)
@@ -428,9 +428,11 @@ DEF_ICON(CURVE_PATH)
 	DEF_ICON(BLANK646)
 	DEF_ICON(BLANK647)
 	DEF_ICON(BLANK648)
-	DEF_ICON(BLANK649)
-	DEF_ICON(BLANK650)
-	DEF_ICON(BLANK651)
+#endif
+DEF_ICON(COLOR_RED)
+DEF_ICON(COLOR_GREEN)
+DEF_ICON(COLOR_BLUE)
+#ifndef DEF_ICON_BLANK_SKIP
 	DEF_ICON(BLANK652)
 	DEF_ICON(BLANK653)
 	DEF_ICON(BLANK654)
@@ -450,8 +452,8 @@ DEF_ICON(FORCE_CURVE)
 DEF_ICON(FORCE_BOID)
 DEF_ICON(FORCE_TURBULENCE)
 DEF_ICON(FORCE_DRAG)
+DEF_ICON(FORCE_SMOKEFLOW)
 #ifndef DEF_ICON_BLANK_SKIP
-	DEF_ICON(BLANK672)
 	DEF_ICON(BLANK673)
 	DEF_ICON(BLANK674)
 	DEF_ICON(BLANK675)
@@ -592,8 +594,9 @@ DEF_ICON(MOD_DYNAMICPAINT)
 DEF_ICON(MOD_REMESH)
 DEF_ICON(MOD_OCEAN)
 DEF_ICON(MOD_WARP)
+DEF_ICON(MOD_SKIN)
+DEF_ICON(MOD_TRIANGULATE)
 #ifndef DEF_ICON_BLANK_SKIP
-	DEF_ICON(BLANK165)
 	DEF_ICON(BLANK166)
 	DEF_ICON(BLANK167)
 	DEF_ICON(BLANK168)
@@ -606,7 +609,6 @@ DEF_ICON(MOD_WARP)
 	DEF_ICON(BLANK175)
 	DEF_ICON(BLANK176)
 	DEF_ICON(BLANK177)
-	DEF_ICON(BLANK177b)
 #endif
 	
 	/* ANIMATION */
@@ -697,9 +699,7 @@ DEF_ICON(RNDCURVE)
 DEF_ICON(PROP_OFF)
 DEF_ICON(PROP_ON)
 DEF_ICON(PROP_CON)
-#ifndef DEF_ICON_BLANK_SKIP
-	DEF_ICON(BLANK212)
-#endif
+DEF_ICON(SCULPT_DYNTOPO)
 DEF_ICON(PARTICLE_POINT)
 DEF_ICON(PARTICLE_TIP)
 DEF_ICON(PARTICLE_PATH)
@@ -799,13 +799,12 @@ DEF_ICON(NDOF_TRANS)
 DEF_ICON(LAYER_USED)
 DEF_ICON(LAYER_ACTIVE)
 #ifndef DEF_ICON_BLANK_SKIP
+	/* available */
 	DEF_ICON(BLANK254)
 	DEF_ICON(BLANK255)
 	DEF_ICON(BLANK256)
 	DEF_ICON(BLANK257)
 	DEF_ICON(BLANK257b)
-
-	/* available */
 	DEF_ICON(BLANK258)
 	DEF_ICON(BLANK259)
 	DEF_ICON(BLANK260)
@@ -890,8 +889,8 @@ DEF_ICON(FORWARD)
 	DEF_ICON(BLANK313)
 	DEF_ICON(BLANK314)
 	DEF_ICON(BLANK315)
-	DEF_ICON(BLANK316)
 #endif
+DEF_ICON(FILE_BACKUP)
 DEF_ICON(DISK_DRIVE)
 	
 	/* SHADING / TEXT */
@@ -974,6 +973,7 @@ DEF_ICON(BRUSH_GRAB)
 DEF_ICON(BRUSH_INFLATE)
 DEF_ICON(BRUSH_LAYER)
 DEF_ICON(BRUSH_LIGHTEN)
+DEF_ICON(BRUSH_MASK)
 DEF_ICON(BRUSH_MIX)
 DEF_ICON(BRUSH_MULTIPLY)
 DEF_ICON(BRUSH_NUDGE)
@@ -990,11 +990,37 @@ DEF_ICON(BRUSH_THUMB)
 DEF_ICON(BRUSH_ROTATE)
 DEF_ICON(BRUSH_VERTEXDRAW)
 
+	/* Matcaps */
+DEF_ICON(MATCAP_01)
+DEF_ICON(MATCAP_02)
+DEF_ICON(MATCAP_03)
+DEF_ICON(MATCAP_04)
+DEF_ICON(MATCAP_05)
+DEF_ICON(MATCAP_06)
+DEF_ICON(MATCAP_07)
+DEF_ICON(MATCAP_08)
+DEF_ICON(MATCAP_09)
+DEF_ICON(MATCAP_10)
+DEF_ICON(MATCAP_11)
+DEF_ICON(MATCAP_12)
+DEF_ICON(MATCAP_13)
+DEF_ICON(MATCAP_14)
+DEF_ICON(MATCAP_15)
+DEF_ICON(MATCAP_16)
+DEF_ICON(MATCAP_17)
+DEF_ICON(MATCAP_18)
+DEF_ICON(MATCAP_19)
+DEF_ICON(MATCAP_20)
+DEF_ICON(MATCAP_21)
+DEF_ICON(MATCAP_22)
+DEF_ICON(MATCAP_23)
+DEF_ICON(MATCAP_24)
+
 /* vector icons, VICO_ prefix added */	
 DEF_VICO(VIEW3D_VEC)
 DEF_VICO(EDIT_VEC)
-DEF_VICO(EDITMODE_DEHLT)
-DEF_VICO(EDITMODE_HLT)
+DEF_VICO(EDITMODE_VEC_DEHLT)
+DEF_VICO(EDITMODE_VEC_HLT)
 DEF_VICO(DISCLOSURE_TRI_RIGHT_VEC)
 DEF_VICO(DISCLOSURE_TRI_DOWN_VEC)
 DEF_VICO(MOVE_UP_VEC)
