@@ -94,53 +94,65 @@ typedef enum {
 	UI_WTYPE_BOX,
 	UI_WTYPE_SCROLL,
 	UI_WTYPE_LISTITEM,
-	UI_WTYPE_PROGRESSBAR,
-	
+	UI_WTYPE_PROGRESSBAR
 } uiWidgetTypeEnum;
 
+/* menu scrolling */
+#define UI_MENU_SCROLL_ARROW	12
+#define UI_MENU_SCROLL_MOUSE	(UI_MENU_SCROLL_ARROW + 2)
+#define UI_MENU_SCROLL_PAD		4
+
 /* panel limits */
-#define UI_PANEL_MINX	100
-#define UI_PANEL_MINY	70
+#define UI_PANEL_MINX   100
+#define UI_PANEL_MINY   70
 
 /* uiBut->flag */
-#define UI_SELECT		1	/* use when the button is pressed */
-#define UI_SCROLLED		2	/* temp hidden, scrolled away */
-#define UI_ACTIVE		4
-#define UI_HAS_ICON		8
-#define UI_TEXTINPUT	16
-#define UI_HIDDEN		32
+#define UI_SELECT       1   /* use when the button is pressed */
+#define UI_SCROLLED     2   /* temp hidden, scrolled away */
+#define UI_ACTIVE       4
+#define UI_HAS_ICON     8
+#define UI_TEXTINPUT    16
+#define UI_HIDDEN       32
 /* warn: rest of uiBut->flag in UI_interface.h */
 
 /* internal panel drawing defines */
-#define PNL_GRID	(UI_UNIT_Y / 5)	/* 4 default */
-#define PNL_HEADER  (UI_UNIT_Y + 4)	/* 24 default */
+#define PNL_GRID    (UI_UNIT_Y / 5) /* 4 default */
+#define PNL_HEADER  (UI_UNIT_Y + 4) /* 24 default */
 
 /* panel->flag */
-#define PNL_SELECT	1
-#define PNL_CLOSEDX	2
-#define PNL_CLOSEDY	4
-#define PNL_CLOSED	6
+#define PNL_SELECT  1
+#define PNL_CLOSEDX 2
+#define PNL_CLOSEDY 4
+#define PNL_CLOSED  6
 /*#define PNL_TABBED	8*/ /*UNUSED*/
-#define PNL_OVERLAP	16
+#define PNL_OVERLAP 16
 
 /* Button text selection:
  * extension direction, selextend, inside ui_do_but_TEX */
-#define EXTEND_LEFT		1
-#define EXTEND_RIGHT	2
+#define EXTEND_LEFT     1
+#define EXTEND_RIGHT    2
 
 /* for scope resize zone */
-#define SCOPE_RESIZE_PAD	9
+#define SCOPE_RESIZE_PAD    9
 
-typedef struct uiLinkLine {				/* only for draw/edit */
+/* bit button defines */
+/* Bit operations */
+#define UI_BITBUT_TEST(a, b)    ( ( (a) & 1 << (b) ) != 0)
+#define UI_BITBUT_SET(a, b)     ( (a) | 1 << (b) )
+#define UI_BITBUT_CLR(a, b)     ( (a) & ~(1 << (b)) )
+/* bit-row */
+#define UI_BITBUT_ROW(min, max)  (((max) >= 31 ? 0xFFFFFFFF : (1 << (max + 1)) - 1) - ((min) ? ((1 << (min)) - 1) : 0) )
+
+typedef struct uiLinkLine {  /* only for draw/edit */
 	struct uiLinkLine *next, *prev;
 	struct uiBut *from, *to;
 	short flag, pad;
 } uiLinkLine;
 
 typedef struct {
-	void **poin;		/* pointer to original pointer */
-	void ***ppoin;		/* pointer to original pointer-array */
-	short *totlink;		/* if pointer-array, here is the total */
+	void **poin;        /* pointer to original pointer */
+	void ***ppoin;      /* pointer to original pointer-array */
+	short *totlink;     /* if pointer-array, here is the total */
 	
 	short maxlink, pad;
 	short fromcode, tocode;
@@ -150,19 +162,39 @@ typedef struct {
 
 struct uiBut {
 	struct uiBut *next, *prev;
-	int flag;
-	short type, pointype, bit, bitnr, retval, strwidth, ofs, pos, selsta, selend, alignnr;
-	short pad1;
+	int flag, drawflag;
+	eButType         type;
+	eButPointerType  pointype;
+	short bit, bitnr, retval, strwidth, alignnr;
+	short ofs, pos, selsta, selend;
 
 	char *str;
 	char strdata[UI_MAX_NAME_STR];
 	char drawstr[UI_MAX_DRAW_STR];
 	
-	float x1, y1, x2, y2;
+	rctf rect;
 
 	char *poin;
 	float hardmin, hardmax, softmin, softmax;
-	float a1, a2;
+
+	/* both these values use depends on the button type
+	 * (polymorphic struct or union would be nicer for this stuff) */
+
+	/* (type == COLOR),      Use UI_GRAD_* values.
+	 * (type == NUM),        Use to store RNA 'step' value, for dragging and click-step.
+	 * (type == LABEL),      Use (a1 == 1.0f) to use a2 as a blending factor (wow, this is imaginative!).
+	 * (type == SCROLL)      Use as scroll size.
+	 * (type == SEARCH_MENU) Use as number or rows.
+	 */
+	float a1;
+
+	/* (type == HSVCIRCLE ), Use to store the luminosity.
+	 * (type == NUM),        Use to store RNA 'precision' value, for dragging and click-step.
+	 * (type == LABEL),      If (a1 == 1.0f) use a2 as a blending factor.
+	 * (type == SEARCH_MENU) Use as number or columns.
+	 */
+	float a2;
+
 	float aspect;
 	unsigned char col[4];
 
@@ -176,10 +208,10 @@ struct uiBut {
 
 	struct bContextStore *context;
 
-	/* not ysed yet, was used in 2.4x for ui_draw_pulldown_round & friends */
+	/* not used yet, was used in 2.4x for ui_draw_pulldown_round & friends */
 #if 0
-	void (*embossfunc)(int , int , float, float, float, float, float, int);
-	void (*sliderfunc)(int , float, float, float, float, float, float, int);
+	void (*embossfunc)(int, int, float, float, float, float, float, int);
+	void (*sliderfunc)(int, float, float, float, float, float, float, int);
 #endif
 
 	uiButCompleteFunc autocomplete_func;
@@ -235,17 +267,20 @@ struct uiBut {
 	void *dragpoin;
 	struct ImBuf *imb;
 	float imb_scale;
-
+	
 	/* active button data */
 	struct uiHandleButtonData *active;
 
+	/* Custom button data. */
+	void *custom_data;
+
 	char *editstr;
 	double *editval;
 	float *editvec;
 	void *editcoba;
 	void *editcumap;
 	
-		/* pointer back */
+	/* pointer back */
 	uiBlock *block;
 };
 
@@ -264,11 +299,11 @@ struct uiBlock {
 	char name[UI_MAX_NAME_STR];
 	
 	float winmat[4][4];
-	
-	float minx, miny, maxx, maxy;
+
+	rctf rect;
 	float aspect;
 
-	int puphash;				// popup menu hash for memory
+	int puphash;  /* popup menu hash for memory */
 
 	uiButHandleFunc func;
 	void *func_arg1;
@@ -284,7 +319,7 @@ struct uiBlock {
 	void *handle_func_arg;
 	
 	/* custom extra handling */
-	int (*block_event_func)(const struct bContext *C, struct uiBlock *, struct wmEvent *);
+	int (*block_event_func)(const struct bContext *C, struct uiBlock *, const struct wmEvent *);
 	
 	/* extra draw function for custom blocks */
 	void (*drawextra)(const struct bContext *C, void *idv, void *arg1, void *arg2, rcti *rect);
@@ -303,27 +338,32 @@ struct uiBlock {
 	const char *lockstr;
 
 	char lock;
-	char active;					// to keep blocks while drawing and free them afterwards
-	char tooltipdisabled;			// to avoid tooltip after click
-	char endblock;					// uiEndBlock done?
-	
-	float xofs, yofs;				// offset to parent button
-	int dobounds, mx, my;			// for doing delayed
-	int bounds, minbounds;			// for doing delayed
+	char active;                /* to keep blocks while drawing and free them afterwards */
+	char tooltipdisabled;       /* to avoid tooltip after click */
+	char endblock;              /* uiEndBlock done? */
 
-	rctf safety;				// pulldowns, to detect outside, can differ per case how it is created
-	ListBase saferct;			// uiSafetyRct list
+	eBlockBoundsCalc bounds_type;  /* for doing delayed */
+	int mx, my;
+	int bounds, minbounds;      /* for doing delayed */
 
-	uiPopupBlockHandle *handle;	// handle
+	rctf safety;                /* pulldowns, to detect outside, can differ per case how it is created */
+	ListBase saferct;           /* uiSafetyRct list */
 
-	struct wmOperator *ui_operator;// use so presets can find the operator,
-								// across menus and from nested popups which fail for operator context.
+	uiPopupBlockHandle *handle; /* handle */
 
-	void *evil_C;				// XXX hack for dynamic operator enums
+	struct wmOperator *ui_operator; /* use so presets can find the operator, */
+	                                /* across menus and from nested popups which fail for operator context. */
 
-	struct UnitSettings *unit;	// unit system, used a lot for numeric buttons so include here rather then fetching through the scene every time.
-	float _hsv[3];				// XXX, only access via ui_block_hsv_get()
-	char color_profile;			// color profile for correcting linear colors for display
+	void *evil_C;               /* XXX hack for dynamic operator enums */
+
+	struct UnitSettings *unit;  /* unit system, used a lot for numeric buttons so include here rather then fetching through the scene every time. */
+	float _hsv[3];              /* XXX, only access via ui_block_hsv_get() */
+
+	char color_profile;         /* color profile for correcting linear colors for display */
+
+	const char *display_device; /* display device name used to display this block,
+	                             * used by color widgets to transform colors from/to scene linear
+	                             */
 };
 
 typedef struct uiSafetyRct {
@@ -340,7 +380,7 @@ void ui_fontscale(short *points, float aspect);
 
 extern void ui_block_to_window_fl(const struct ARegion *ar, uiBlock *block, float *x, float *y);
 extern void ui_block_to_window(const struct ARegion *ar, uiBlock *block, int *x, int *y);
-extern void ui_block_to_window_rct(const struct ARegion *ar, uiBlock *block, rctf *graph, rcti *winr);
+extern void ui_block_to_window_rct(const struct ARegion *ar, uiBlock *block, const rctf *graph, rcti *winr);
 extern void ui_window_to_block_fl(const struct ARegion *ar, uiBlock *block, float *x, float *y);
 extern void ui_window_to_block(const struct ARegion *ar, uiBlock *block, int *x, int *y);
 extern void ui_window_to_region(const ARegion *ar, int *x, int *y);
@@ -351,7 +391,8 @@ extern void ui_set_but_hsv(uiBut *but);
 extern void ui_get_but_vectorf(uiBut *but, float vec[3]);
 extern void ui_set_but_vectorf(uiBut *but, const float vec[3]);
 
-extern void ui_hsvcircle_vals_from_pos(float *valrad, float *valdist, rcti *rect, float mx, float my);
+extern void ui_hsvcircle_vals_from_pos(float *val_rad, float *val_dist, const rcti *rect,
+                                       const float mx, const float my);
 
 extern void ui_get_but_string(uiBut *but, char *str, size_t maxlen);
 extern void ui_convert_to_unit_alt_name(uiBut *but, char *str, size_t maxlen);
@@ -373,6 +414,10 @@ extern void ui_bounds_block(uiBlock *block);
 extern void ui_block_translate(uiBlock *block, int x, int y);
 extern void ui_block_do_align(uiBlock *block);
 
+extern struct ColorManagedDisplay *ui_block_display_get(uiBlock *block);
+void ui_block_to_display_space_v3(uiBlock *block, float pixel[3]);
+void ui_block_to_scene_linear_v3(uiBlock *block, float pixel[3]);
+
 /* interface_regions.c */
 
 struct uiPopupBlockHandle {
@@ -384,7 +429,7 @@ struct uiPopupBlockHandle {
 
 	int popup;
 	void (*popup_func)(struct bContext *C, void *arg, int event);
-	void (*cancel_func)(void *arg);
+	void (*cancel_func)(struct bContext *C, void *arg);
 	void *popup_arg;
 	
 	struct wmTimer *scrolltimer;
@@ -400,9 +445,12 @@ struct uiPopupBlockHandle {
 	int menuretval;
 	float retvalue;
 	float retvec[4];
+
+	/* menu direction */
+	int direction;
 };
 
-uiBlock *ui_block_func_COL(struct bContext *C, uiPopupBlockHandle *handle, void *arg_but);
+uiBlock *ui_block_func_COLOR(struct bContext *C, uiPopupBlockHandle *handle, void *arg_but);
 void ui_block_func_ICONROW(struct bContext *C, uiLayout *layout, void *arg_but);
 void ui_block_func_ICONTEXTROW(struct bContext *C, uiLayout *layout, void *arg_but);
 
@@ -420,17 +468,18 @@ ARegion *ui_searchbox_create(struct bContext *C, struct ARegion *butregion, uiBu
 int ui_searchbox_inside(struct ARegion *ar, int x, int y);
 void ui_searchbox_update(struct bContext *C, struct ARegion *ar, uiBut *but, int reset);
 void ui_searchbox_autocomplete(struct bContext *C, struct ARegion *ar, uiBut *but, char *str);
-void ui_searchbox_event(struct bContext *C, struct ARegion *ar, uiBut *but, struct wmEvent *event);
+void ui_searchbox_event(struct bContext *C, struct ARegion *ar, uiBut *but, const struct wmEvent *event);
 void ui_searchbox_apply(uiBut *but, struct ARegion *ar);
 void ui_searchbox_free(struct bContext *C, struct ARegion *ar);
 void ui_but_search_test(uiBut *but);
 
-typedef uiBlock* (*uiBlockHandleCreateFunc)(struct bContext *C, struct uiPopupBlockHandle *handle, void *arg1);
+typedef uiBlock * (*uiBlockHandleCreateFunc)(struct bContext *C, struct uiPopupBlockHandle *handle, void *arg1);
 
 uiPopupBlockHandle *ui_popup_block_create(struct bContext *C, struct ARegion *butregion, uiBut *but,
-	uiBlockCreateFunc create_func, uiBlockHandleCreateFunc handle_create_func, void *arg);
+                                          uiBlockCreateFunc create_func, uiBlockHandleCreateFunc handle_create_func,
+                                          void *arg);
 uiPopupBlockHandle *ui_popup_menu_create(struct bContext *C, struct ARegion *butregion, uiBut *but,
-	uiMenuCreateFunc create_func, void *arg, char *str);
+                                         uiMenuCreateFunc create_func, void *arg, char *str);
 
 void ui_popup_block_free(struct bContext *C, uiPopupBlockHandle *handle);
 
@@ -440,13 +489,13 @@ int ui_step_name_menu(uiBut *but, int step);
 struct AutoComplete;
 
 /* interface_panel.c */
-extern int ui_handler_panel_region(struct bContext *C, struct wmEvent *event);
+extern int ui_handler_panel_region(struct bContext *C, const struct wmEvent *event);
 extern void ui_draw_aligned_panel(struct uiStyle *style, uiBlock *block, rcti *rect);
 
 /* interface_draw.c */
-extern void ui_dropshadow(rctf *rct, float radius, float aspect, int select);
+extern void ui_dropshadow(const rctf *rct, float radius, float aspect, float alpha, int select);
 
-void ui_draw_gradient(rcti *rect, const float hsv[3], int type, float alpha);
+void ui_draw_gradient(const rcti *rect, const float hsv[3], const int type, const float alpha);
 
 void ui_draw_but_HISTOGRAM(ARegion *ar, uiBut *but, struct uiWidgetColors *wcol, rcti *rect);
 void ui_draw_but_WAVEFORM(ARegion *ar, uiBut *but, struct uiWidgetColors *wcol, rcti *rect);
@@ -458,33 +507,38 @@ void ui_draw_but_IMAGE(ARegion *ar, uiBut *but, struct uiWidgetColors *wcol, rct
 void ui_draw_but_TRACKPREVIEW(ARegion *ar, uiBut *but, struct uiWidgetColors *wcol, rcti *rect);
 
 /* interface_handlers.c */
+extern void ui_pan_to_scroll(const struct wmEvent *event, int *type, int *val);
 extern void ui_button_activate_do(struct bContext *C, struct ARegion *ar, uiBut *but);
 extern void ui_button_active_free(const struct bContext *C, uiBut *but);
 extern int ui_button_is_active(struct ARegion *ar);
+extern int ui_button_open_menu_direction(uiBut *but);
+extern void ui_button_text_password_hide(char password_str[UI_MAX_DRAW_STR], uiBut *but, int restore);
 
 /* interface_widgets.c */
 void ui_draw_anti_tria(float x1, float y1, float x2, float y2, float x3, float y3);
 void ui_draw_anti_roundbox(int mode, float minx, float miny, float maxx, float maxy, float rad);
 void ui_draw_menu_back(struct uiStyle *style, uiBlock *block, rcti *rect);
-uiWidgetColors* ui_tooltip_get_theme(void);
-void ui_draw_tooltip_background(uiStyle *UNUSED(style), uiBlock *block, rcti *rect);
+uiWidgetColors *ui_tooltip_get_theme(void);
+void ui_draw_tooltip_background(uiStyle *UNUSED(style), uiBlock * block, rcti * rect);
 void ui_draw_search_back(struct uiStyle *style, uiBlock *block, rcti *rect);
-int ui_link_bezier_points(rcti *rect, float coord_array[][2], int resol);
-void ui_draw_link_bezier(rcti *rect);
+int ui_link_bezier_points(const rcti * rect, float coord_array[][2], int resol);
+void ui_draw_link_bezier(const rcti *rect);
 
 extern void ui_draw_but(const struct bContext *C, ARegion *ar, struct uiStyle *style, uiBut *but, rcti *rect);
-		/* theme color init */
+/* theme color init */
 struct ThemeUI;
 void ui_widget_color_init(struct ThemeUI *tui);
 
 void ui_draw_menu_item(struct uiFontStyle *fstyle, rcti *rect, const char *name, int iconid, int state);
 void ui_draw_preview_item(struct uiFontStyle *fstyle, rcti *rect, const char *name, int iconid, int state);
 
-extern unsigned char checker_stipple_sml[];
+extern unsigned char checker_stipple_sml[32 * 32 / 8];
 /* used for transp checkers */
 #define UI_TRANSP_DARK 100
 #define UI_TRANSP_LIGHT 160
 
+#define UI_TEXT_MARGIN_X 0.4f
+
 /* interface_style.c */
 void uiStyleInit(void);
 
@@ -507,6 +561,7 @@ void ui_but_add_shortcut(uiBut *but, const char *key_str, const short do_strip);
 void ui_but_anim_flag(uiBut *but, float cfra);
 void ui_but_anim_insert_keyframe(struct bContext *C);
 void ui_but_anim_delete_keyframe(struct bContext *C);
+void ui_but_anim_clear_keyframe(struct bContext *C);
 void ui_but_anim_add_driver(struct bContext *C);
 void ui_but_anim_remove_driver(struct bContext *C);
 void ui_but_anim_copy_driver(struct bContext *C);
@@ -519,4 +574,3 @@ int ui_but_anim_expression_create(uiBut *but, const char *str);
 void ui_but_anim_autokey(struct bContext *C, uiBut *but, struct Scene *scene, float cfra);
 
 #endif
-
