@@ -39,7 +39,9 @@
 
 #include "BLI_listbase.h"
 #include "BLI_string.h"
+#include "BLI_rect.h"
 #include "BLI_utildefines.h"
+#include "BLI_math.h"
 
 #include "BLF_translation.h"
 
@@ -63,8 +65,8 @@
 #define RNA_NO_INDEX    -1
 #define RNA_ENUM_VALUE  -2
 
-#define EM_SEPR_X       6
-#define EM_SEPR_Y       6
+
+// #define USE_OP_RESET_BUT  // we may want to make this optional, disable for now.
 
 #define UI_OPERATOR_ERROR_RET(_ot, _opname, return_statement)                 \
 	if (ot == NULL) {                                                         \
@@ -184,13 +186,13 @@ static const char *ui_item_name_add_colon(const char *name, char namestr[UI_MAX_
 
 static int ui_item_fit(int item, int pos, int all, int available, int last, int alignment, int *offset)
 {
+	if (offset)
+		*offset = 0;
+
 	/* available == 0 is unlimited */
 	if (available == 0)
 		return item;
-	
-	if (offset)
-		*offset = 0;
-	
+
 	if (all > available) {
 		/* contents is bigger than available space */
 		if (last)
@@ -223,14 +225,16 @@ static int ui_layout_vary_direction(uiLayout *layout)
 /* estimated size of text + icon */
 static int ui_text_icon_width(uiLayout *layout, const char *name, int icon, int compact)
 {
+	float f5 = 0.25f * UI_UNIT_X;
+	float f10 = 0.5f * UI_UNIT_X;
 	int variable = ui_layout_vary_direction(layout) == UI_ITEM_VARY_X;
 
 	if (icon && !name[0])
 		return UI_UNIT_X;  /* icon only */
 	else if (icon)
-		return (variable) ? UI_GetStringWidth(name) + (compact ? 5 : 10) + UI_UNIT_X : 10 * UI_UNIT_X;  /* icon + text */
+		return (variable) ? UI_GetStringWidth(name) + (compact ? f5 : f10) + UI_UNIT_X : 10 * UI_UNIT_X;  /* icon + text */
 	else
-		return (variable) ? UI_GetStringWidth(name) + (compact ? 5 : 10) + UI_UNIT_X : 10 * UI_UNIT_X;  /* text only */
+		return (variable) ? UI_GetStringWidth(name) + (compact ? f5 : f10) + UI_UNIT_X : 10 * UI_UNIT_X;  /* text only */
 }
 
 static void ui_item_size(uiItem *item, int *r_w, int *r_h)
@@ -238,8 +242,8 @@ static void ui_item_size(uiItem *item, int *r_w, int *r_h)
 	if (item->type == ITEM_BUTTON) {
 		uiButtonItem *bitem = (uiButtonItem *)item;
 
-		if (r_w) *r_w = bitem->but->x2 - bitem->but->x1;
-		if (r_h) *r_h = bitem->but->y2 - bitem->but->y1;
+		if (r_w) *r_w = BLI_rctf_size_x(&bitem->but->rect);
+		if (r_h) *r_h = BLI_rctf_size_y(&bitem->but->rect);
 	}
 	else {
 		uiLayout *litem = (uiLayout *)item;
@@ -254,8 +258,8 @@ static void ui_item_offset(uiItem *item, int *r_x, int *r_y)
 	if (item->type == ITEM_BUTTON) {
 		uiButtonItem *bitem = (uiButtonItem *)item;
 
-		if (r_x) *r_x = bitem->but->x1;
-		if (r_y) *r_y = bitem->but->y1;
+		if (r_x) *r_x = bitem->but->rect.xmin;
+		if (r_y) *r_y = bitem->but->rect.ymin;
 	}
 	else {
 		if (r_x) *r_x = 0;
@@ -268,10 +272,10 @@ static void ui_item_position(uiItem *item, int x, int y, int w, int h)
 	if (item->type == ITEM_BUTTON) {
 		uiButtonItem *bitem = (uiButtonItem *)item;
 
-		bitem->but->x1 = x;
-		bitem->but->y1 = y;
-		bitem->but->x2 = x + w;
-		bitem->but->y2 = y + h;
+		bitem->but->rect.xmin = x;
+		bitem->but->rect.ymin = y;
+		bitem->but->rect.xmax = x + w;
+		bitem->but->rect.ymax = y + h;
 		
 		ui_check_but(bitem->but); /* for strlen */
 	}
@@ -342,7 +346,9 @@ static void ui_layer_but_cb(bContext *C, void *arg_but, void *arg_index)
 }
 
 /* create buttons for an item with an RNA array */
-static void ui_item_array(uiLayout *layout, uiBlock *block, const char *name, int icon, PointerRNA *ptr, PropertyRNA *prop, int len, int x, int y, int w, int UNUSED(h), int expand, int slider, int toggle, int icon_only)
+static void ui_item_array(uiLayout *layout, uiBlock *block, const char *name, int icon,
+                          PointerRNA *ptr, PropertyRNA *prop, int len, int x, int y, int w, int UNUSED(h),
+                          int expand, int slider, int toggle, int icon_only)
 {
 	uiStyle *style = layout->root->style;
 	uiBut *but;
@@ -370,7 +376,7 @@ static void ui_item_array(uiLayout *layout, uiBlock *block, const char *name, in
 		int colbuts = len / (2 * cols);
 		int layer_used = 0;
 
-		uiBlockSetCurLayout(block, uiLayoutAbsolute(layout, 0));
+		uiBlockSetCurLayout(block, uiLayoutAbsolute(layout, FALSE));
 
 		unit = UI_UNIT_X * 0.75;
 		butw = unit;
@@ -409,7 +415,7 @@ static void ui_item_array(uiLayout *layout, uiBlock *block, const char *name, in
 		int totdim, dim_size[3];    /* 3 == RNA_MAX_ARRAY_DIMENSION */
 		int row, col;
 
-		uiBlockSetCurLayout(block, uiLayoutAbsolute(layout, 1));
+		uiBlockSetCurLayout(block, uiLayoutAbsolute(layout, TRUE));
 
 		totdim = RNA_property_array_dimension(ptr, prop, dim_size);
 		if (totdim != 2) return;    /* only 2D matrices supported in UI so far */
@@ -426,7 +432,7 @@ static void ui_item_array(uiLayout *layout, uiBlock *block, const char *name, in
 				but->type = NUMSLI;
 		}
 	}
-	else if (subtype == PROP_DIRECTION) {
+	else if (subtype == PROP_DIRECTION && !expand) {
 		uiDefButR_prop(block, BUT_NORMAL, 0, name, x, y, UI_UNIT_X * 3, UI_UNIT_Y * 3, ptr, prop, 0, 0, 0, -1, -1, NULL);
 	}
 	else {
@@ -485,7 +491,14 @@ static void ui_item_enum_expand(uiLayout *layout, uiBlock *block, PointerRNA *pt
 
 	RNA_property_enum_items_gettexted(block->evil_C, ptr, prop, &item, &totitem, &free);
 
-	uiBlockSetCurLayout(block, ui_item_local_sublayout(layout, layout, 1));
+	/* we dont want nested rows, cols in menus */
+	if (layout->root->type != UI_LAYOUT_MENU) {
+		uiBlockSetCurLayout(block, ui_item_local_sublayout(layout, layout, 1));
+	}
+	else {
+		uiBlockSetCurLayout(block, layout);
+	}
+
 	for (a = 0; a < totitem; a++) {
 		if (!item[a].identifier[0])
 			continue;
@@ -532,7 +545,7 @@ static uiBut *ui_item_with_label(uiLayout *layout, uiBlock *block, const char *n
 	PropertySubType subtype;
 	int labelw;
 
-	sub = uiLayoutRow(layout, 0);
+	sub = uiLayoutRow(layout, FALSE);
 	uiBlockSetCurLayout(block, sub);
 
 	if (name[0]) {
@@ -550,7 +563,7 @@ static uiBut *ui_item_with_label(uiLayout *layout, uiBlock *block, const char *n
 	subtype = RNA_property_subtype(prop);
 
 	if (subtype == PROP_FILEPATH || subtype == PROP_DIRPATH) {
-		uiBlockSetCurLayout(block, uiLayoutRow(sub, 1));
+		uiBlockSetCurLayout(block, uiLayoutRow(sub, TRUE));
 		uiDefAutoButR(block, ptr, prop, index, "", icon, x, y, w - UI_UNIT_X, h);
 
 		/* BUTTONS_OT_file_browse calls uiFileBrowseContextProperty */
@@ -611,6 +624,18 @@ void uiFileBrowseContextProperty(const bContext *C, PointerRNA *ptr, PropertyRNA
 
 /********************* Button Items *************************/
 
+/**
+ * Update a buttons tip with an enum's description if possible.
+ */
+static void ui_but_tip_from_enum_item(uiBut *but, EnumPropertyItem *item)
+{
+	if (but->tip == NULL || but->tip[0] == '\0') {
+		if (item->description && item->description[0]) {
+			but->tip = item->description;
+		}
+	}
+}
+
 /* disabled item */
 static void ui_item_disabled(uiLayout *layout, const char *name)
 {
@@ -627,7 +652,7 @@ static void ui_item_disabled(uiLayout *layout, const char *name)
 
 	but = uiDefBut(block, LABEL, 0, name, 0, 0, w, UI_UNIT_Y, NULL, 0.0, 0.0, 0, 0, "");
 	but->flag |= UI_BUT_DISABLED;
-	but->lock = 1;
+	but->lock = TRUE;
 	but->lockstr = "";
 }
 
@@ -714,8 +739,11 @@ static const char *ui_menu_enumpropname(uiLayout *layout, PointerRNA *ptr, Prope
 	int totitem, free;
 	const char *name;
 
-	RNA_property_enum_items_gettexted(layout->root->block->evil_C, ptr, prop, &item, &totitem, &free);
-	if (RNA_enum_name(item, retval, &name) == 0) {
+	RNA_property_enum_items(layout->root->block->evil_C, ptr, prop, &item, &totitem, &free);
+	if (RNA_enum_name(item, retval, &name)) {
+		name = CTX_IFACE_(RNA_property_translation_context(prop), name);
+	}
+	else {
 		name = "";
 	}
 
@@ -798,8 +826,8 @@ void uiItemsFullEnumO(uiLayout *layout, const char *opname, const char *propname
 	if (prop && RNA_property_type(prop) == PROP_ENUM) {
 		EnumPropertyItem *item;
 		int totitem, i, free;
-		uiLayout *split = uiLayoutSplit(layout, 0, 0);
-		uiLayout *column = uiLayoutColumn(split, 0);
+		uiLayout *split = uiLayoutSplit(layout, 0.0f, FALSE);
+		uiLayout *column = uiLayoutColumn(split, FALSE);
 
 		RNA_property_enum_items_gettexted(block->evil_C, &ptr, prop, &item, &totitem, &free);
 
@@ -821,11 +849,12 @@ void uiItemsFullEnumO(uiLayout *layout, const char *opname, const char *propname
 				else {
 					uiItemEnumO_ptr__internal(column, ot, item[i].name, item[i].icon, prop, item[i].value);
 				}
+				ui_but_tip_from_enum_item(block->buttons.last, &item[i]);
 			}
 			else {
 				if (item[i].name) {
 					if (i != 0) {
-						column = uiLayoutColumn(split, 0);
+						column = uiLayoutColumn(split, FALSE);
 						/* inconsistent, but menus with labels do not look good flipped */
 						block->flag |= UI_BLOCK_NO_FLIP;
 					}
@@ -833,9 +862,12 @@ void uiItemsFullEnumO(uiLayout *layout, const char *opname, const char *propname
 					uiItemL(column, item[i].name, ICON_NONE);
 					bt = block->buttons.last;
 					bt->flag = UI_TEXT_LEFT;
+
+					ui_but_tip_from_enum_item(bt, &item[i]);
 				}
-				else /* XXX bug here, collums draw bottom item badly */
+				else {  /* XXX bug here, colums draw bottom item badly */
 					uiItemS(column);
+				}
 			}
 		}
 
@@ -843,6 +875,14 @@ void uiItemsFullEnumO(uiLayout *layout, const char *opname, const char *propname
 			MEM_freeN(item);
 		}
 	}
+	else if (prop && RNA_property_type(prop) != PROP_ENUM) {
+		RNA_warning("%s.%s, not an enum type", RNA_struct_identifier(ptr.type), propname);
+		return;
+	}
+	else {
+		RNA_warning("%s.%s not found", RNA_struct_identifier(ptr.type), propname);
+		return;
+	}
 }
 
 void uiItemsEnumO(uiLayout *layout, const char *opname, const char *propname)
@@ -891,10 +931,11 @@ void uiItemEnumO_string(uiLayout *layout, const char *name, int icon, const char
 	UI_OPERATOR_ERROR_RET(ot, opname, return );
 
 	WM_operator_properties_create_ptr(&ptr, ot);
-	
+
 	/* enum lookup */
 	if ((prop = RNA_struct_find_property(&ptr, propname))) {
-		RNA_property_enum_items_gettexted(layout->root->block->evil_C, &ptr, prop, &item, NULL, &free);
+		/* no need for translations here */
+		RNA_property_enum_items(layout->root->block->evil_C, &ptr, prop, &item, NULL, &free);
 		if (item == NULL || RNA_enum_value_from_id(item, value_str, &value) == 0) {
 			if (free) {
 				MEM_freeN(item);
@@ -911,9 +952,9 @@ void uiItemEnumO_string(uiLayout *layout, const char *name, int icon, const char
 		RNA_warning("%s.%s not found", RNA_struct_identifier(ptr.type), propname);
 		return;
 	}
-	
+
 	RNA_property_enum_set(&ptr, prop, value);
-	
+
 	/* same as uiItemEnumO */
 	if (!name)
 		name = ui_menu_enumpropname(layout, &ptr, prop, value);
@@ -1159,7 +1200,7 @@ void uiItemEnumR_string(uiLayout *layout, struct PointerRNA *ptr, const char *pr
 		return;
 	}
 
-	RNA_property_enum_items_gettexted(layout->root->block->evil_C, ptr, prop, &item, NULL, &free);
+	RNA_property_enum_items(layout->root->block->evil_C, ptr, prop, &item, NULL, &free);
 
 	if (!RNA_enum_value_from_id(item, value, &ivalue)) {
 		if (free) {
@@ -1172,7 +1213,9 @@ void uiItemEnumR_string(uiLayout *layout, struct PointerRNA *ptr, const char *pr
 
 	for (a = 0; item[a].identifier; a++) {
 		if (item[a].value == ivalue) {
-			uiItemFullR(layout, ptr, prop, RNA_ENUM_VALUE, ivalue, 0, name ? name : item[a].name, icon ? icon : item[a].icon);
+			const char *item_name = CTX_IFACE_(RNA_property_translation_context(prop), item[a].name);
+
+			uiItemFullR(layout, ptr, prop, RNA_ENUM_VALUE, ivalue, 0, item_name ? item_name : name, icon ? icon : item[a].icon);
 			break;
 		}
 	}
@@ -1203,19 +1246,20 @@ void uiItemsEnumR(uiLayout *layout, struct PointerRNA *ptr, const char *propname
 	else {
 		EnumPropertyItem *item;
 		int totitem, i, free;
-		uiLayout *split = uiLayoutSplit(layout, 0, 0);
-		uiLayout *column = uiLayoutColumn(split, 0);
+		uiLayout *split = uiLayoutSplit(layout, 0.0f, FALSE);
+		uiLayout *column = uiLayoutColumn(split, FALSE);
 
 		RNA_property_enum_items_gettexted(block->evil_C, ptr, prop, &item, &totitem, &free);
 
 		for (i = 0; i < totitem; i++) {
 			if (item[i].identifier[0]) {
 				uiItemEnumR(column, item[i].name, ICON_NONE, ptr, propname, item[i].value);
+				ui_but_tip_from_enum_item(block->buttons.last, &item[i]);
 			}
 			else {
 				if (item[i].name) {
 					if (i != 0) {
-						column = uiLayoutColumn(split, 0);
+						column = uiLayoutColumn(split, FALSE);
 						/* inconsistent, but menus with labels do not look good flipped */
 						block->flag |= UI_BLOCK_NO_FLIP;
 					}
@@ -1223,6 +1267,8 @@ void uiItemsEnumR(uiLayout *layout, struct PointerRNA *ptr, const char *propname
 					uiItemL(column, item[i].name, ICON_NONE);
 					bt = block->buttons.last;
 					bt->flag = UI_TEXT_LEFT;
+
+					ui_but_tip_from_enum_item(bt, &item[i]);
 				}
 				else
 					uiItemS(column);
@@ -1265,7 +1311,8 @@ static void rna_search_cb(const struct bContext *C, void *arg_but, const char *s
 	const int skip_filter = !but->changed;
 
 	/* build a temporary list of relevant items first */
-	RNA_PROP_BEGIN(&but->rnasearchpoin, itemptr, but->rnasearchprop) {
+	RNA_PROP_BEGIN (&but->rnasearchpoin, itemptr, but->rnasearchprop)
+	{
 		if (flag & PROP_ID_SELF_CHECK)
 			if (itemptr.data == but->rnapoin.id.data)
 				continue;
@@ -1302,7 +1349,7 @@ static void rna_search_cb(const struct bContext *C, void *arg_but, const char *s
 				BLI_addtail(items_list, cis);
 			}
 			MEM_freeN(name);
-		}			
+		}
 
 		i++;
 	}
@@ -1333,7 +1380,8 @@ static void search_id_collection(StructRNA *ptype, PointerRNA *ptr, PropertyRNA
 
 	*prop = NULL;
 
-	RNA_STRUCT_BEGIN(ptr, iprop) {
+	RNA_STRUCT_BEGIN (ptr, iprop)
+	{
 		/* if it's a collection and has same pointer type, we've got it */
 		if (RNA_property_type(iprop) == PROP_COLLECTION) {
 			srna = RNA_property_pointer_type(ptr, iprop);
@@ -1363,12 +1411,22 @@ void ui_but_add_search(uiBut *but, PointerRNA *ptr, PropertyRNA *prop, PointerRN
 
 	/* turn button into search button */
 	if (searchprop) {
-		but->type = SEARCH_MENU;
-		but->hardmax = MAX2(but->hardmax, 256);
+		if (RNA_property_flag(prop) & PROP_NEVER_UNLINK)
+			but->type = SEARCH_MENU;
+		else
+			but->type = SEARCH_MENU_UNLINK;
+
+		but->hardmax = MAX2(but->hardmax, 256.0f);
 		but->rnasearchpoin = *searchptr;
 		but->rnasearchprop = searchprop;
 		but->flag |= UI_ICON_LEFT | UI_TEXT_LEFT;
 
+		if (RNA_property_type(prop) == PROP_ENUM) {
+			/* XXX, this will have a menu string,
+			 * but in this case we just want the text */
+			but->str[0] = 0;
+		}
+
 		uiButSetSearchFunc(but, rna_search_cb, but, NULL, NULL);
 	}
 }
@@ -1386,13 +1444,14 @@ void uiItemPointerR(uiLayout *layout, struct PointerRNA *ptr, const char *propna
 	prop = RNA_struct_find_property(ptr, propname);
 
 	if (!prop) {
-		RNA_warning("property not found: %s.%s", RNA_struct_identifier(ptr->type), propname);
+		RNA_warning("property not found: %s.%s",
+		            RNA_struct_identifier(ptr->type), propname);
 		return;
 	}
 	
 	type = RNA_property_type(prop);
-	if (!ELEM(type, PROP_POINTER, PROP_STRING)) {
-		RNA_warning("Property %s must be a pointer or string", propname);
+	if (!ELEM3(type, PROP_POINTER, PROP_STRING, PROP_ENUM)) {
+		RNA_warning("Property %s must be a pointer, string or enum", propname);
 		return;
 	}
 
@@ -1400,11 +1459,13 @@ void uiItemPointerR(uiLayout *layout, struct PointerRNA *ptr, const char *propna
 
 
 	if (!searchprop) {
-		RNA_warning("search collection property not found: %s.%s", RNA_struct_identifier(ptr->type), searchpropname);
+		RNA_warning("search collection property not found: %s.%s",
+		            RNA_struct_identifier(searchptr->type), searchpropname);
 		return;
 	}
 	else if (RNA_property_type(searchprop) != PROP_COLLECTION) {
-		RNA_warning("search collection property is not a collection type: %s.%s", RNA_struct_identifier(ptr->type), searchpropname);
+		RNA_warning("search collection property is not a collection type: %s.%s",
+		            RNA_struct_identifier(searchptr->type), searchpropname);
 		return;
 	}
 
@@ -1424,6 +1485,7 @@ void uiItemPointerR(uiLayout *layout, struct PointerRNA *ptr, const char *propna
 	block = uiLayoutGetBlock(layout);
 
 	ui_item_rna_size(layout, name, icon, ptr, prop, 0, 0, &w, &h);
+	w += UI_UNIT_X; /* X icon needs more space */
 	but = ui_item_with_label(layout, block, name, icon, ptr, prop, 0, 0, 0, w, h, 0);
 
 	ui_but_add_search(but, ptr, prop, searchptr, searchprop);
@@ -1437,7 +1499,18 @@ static void ui_item_menutype_func(bContext *C, uiLayout *layout, void *arg_mt)
 
 	menu.type = mt;
 	menu.layout = layout;
+
+	if (G.debug & G_DEBUG_WM) {
+		printf("%s: opening menu \"%s\"\n", __func__, mt->idname);
+	}
+
+	if (layout->context)
+		CTX_store_set(C, layout->context);
+
 	mt->draw(C, &menu);
+
+	if (layout->context)
+		CTX_store_set(C, NULL);
 }
 
 static void ui_item_menu(uiLayout *layout, const char *name, int icon, uiMenuCreateFunc func, void *arg, void *argN, const char *tip)
@@ -1460,7 +1533,7 @@ static void ui_item_menu(uiLayout *layout, const char *name, int icon, uiMenuCre
 	h = UI_UNIT_Y;
 
 	if (layout->root->type == UI_LAYOUT_HEADER) /* ugly .. */
-		w -= 10;
+		w -= UI_UNIT_Y / 2;
 
 	if (name[0] && icon)
 		but = uiDefIconTextMenuBut(block, func, arg, icon, name, 0, 0, w, h, tip);
@@ -1575,7 +1648,7 @@ void uiItemS(uiLayout *layout)
 	uiBlock *block = layout->root->block;
 
 	uiBlockSetCurLayout(block, layout);
-	uiDefBut(block, SEPR, 0, "", 0, 0, EM_SEPR_X, EM_SEPR_Y, NULL, 0.0, 0.0, 0, 0, "");
+	uiDefBut(block, SEPR, 0, "", 0, 0, 0.3f * UI_UNIT_X, 0.3f * UI_UNIT_Y, NULL, 0.0, 0.0, 0, 0, "");
 }
 
 /* level items */
@@ -1696,7 +1769,7 @@ static void ui_litem_layout_row(uiLayout *litem)
 	int x, y, w, tot, totw, neww, itemw, minw, itemh, offset;
 	int fixedw, freew, fixedx, freex, flag = 0, lastw = 0;
 
-	/* x= litem->x; */ /* UNUSED */
+	/* x = litem->x; */ /* UNUSED */
 	y = litem->y;
 	w = litem->w;
 	totw = 0;
@@ -1762,7 +1835,7 @@ static void ui_litem_layout_row(uiLayout *litem)
 
 		if (item->flag) {
 			/* fixed minimum size items */
-			itemw = ui_item_fit(minw, fixedx, fixedw, MIN2(w, fixedw), !item->next, litem->alignment, NULL);
+			itemw = ui_item_fit(minw, fixedx, fixedw, min_ii(w, fixedw), !item->next, litem->alignment, NULL);
 			fixedx += itemw;
 		}
 		else {
@@ -1774,12 +1847,12 @@ static void ui_litem_layout_row(uiLayout *litem)
 		/* align right/center */
 		offset = 0;
 		if (litem->alignment == UI_LAYOUT_ALIGN_RIGHT) {
-			if (freew > 0 && freew < w - fixedw)
-				offset = (w - fixedw) - freew;
+			if (freew + fixedw > 0 && freew + fixedw < w)
+				offset = w - (fixedw + freew);
 		}
 		else if (litem->alignment == UI_LAYOUT_ALIGN_CENTER) {
-			if (freew > 0 && freew < w - fixedw)
-				offset = ((w - fixedw) - freew) / 2;
+			if (freew + fixedw > 0 && freew + fixedw < w)
+				offset = (w - (fixedw + freew)) / 2;
 		}
 
 		/* position item */
@@ -1888,10 +1961,10 @@ static void ui_litem_layout_box(uiLayout *litem)
 
 	/* roundbox around the sublayout */
 	but = box->roundbox;
-	but->x1 = litem->x;
-	but->y1 = litem->y;
-	but->x2 = litem->x + litem->w;
-	but->y2 = litem->y + litem->h;
+	but->rect.xmin = litem->x;
+	but->rect.ymin = litem->y;
+	but->rect.xmax = litem->x + litem->w;
+	but->rect.ymax = litem->y + litem->h;
 }
 
 /* multi-column layout, automatically flowing to the next */
@@ -1920,8 +1993,8 @@ static void ui_litem_estimate_column_flow(uiLayout *litem)
 			return;
 		}
 
-		flow->totcol = MAX2(litem->root->emw / maxw, 1);
-		flow->totcol = MIN2(flow->totcol, totitem);
+		flow->totcol = max_ii(litem->root->emw / maxw, 1);
+		flow->totcol = min_ii(flow->totcol, totitem);
 	}
 	else
 		flow->totcol = flow->number;
@@ -1941,15 +2014,16 @@ static void ui_litem_estimate_column_flow(uiLayout *litem)
 		ui_item_size(item, &itemw, &itemh);
 
 		y -= itemh + style->buttonspacey;
-		miny = MIN2(miny, y);
+		miny = min_ii(miny, y);
 		emy -= itemh;
-		maxw = MAX2(itemw, maxw);
+		maxw = max_ii(itemw, maxw);
 
 		/* decide to go to next one */
 		if (col < flow->totcol - 1 && emy <= -emh) {
 			x += maxw + litem->space;
 			maxw = 0;
 			y = 0;
+			emy = 0; /* need to reset height again for next column */
 			col++;
 		}
 	}
@@ -1994,12 +2068,13 @@ static void ui_litem_layout_column_flow(uiLayout *litem)
 		emy -= itemh;
 		ui_item_position(item, x + offset, y, itemw, itemh);
 		y -= style->buttonspacey;
-		miny = MIN2(miny, y);
+		miny = min_ii(miny, y);
 
 		/* decide to go to next one */
 		if (col < flow->totcol - 1 && emy <= -emh) {
 			x += itemw + style->columnspace;
 			y = litem->y;
+			emy = 0; /* need to reset height again for next column */
 			col++;
 		}
 	}
@@ -2024,8 +2099,8 @@ static void ui_litem_estimate_absolute(uiLayout *litem)
 		ui_item_offset(item, &itemx, &itemy);
 		ui_item_size(item, &itemw, &itemh);
 
-		minx = MIN2(minx, itemx);
-		miny = MIN2(miny, itemy);
+		minx = min_ii(minx, itemx);
+		miny = min_ii(miny, itemy);
 
 		litem->w = MAX2(litem->w, itemx + itemw);
 		litem->h = MAX2(litem->h, itemy + itemh);
@@ -2050,11 +2125,11 @@ static void ui_litem_layout_absolute(uiLayout *litem)
 		ui_item_offset(item, &itemx, &itemy);
 		ui_item_size(item, &itemw, &itemh);
 
-		minx = MIN2(minx, itemx);
-		miny = MIN2(miny, itemy);
+		minx = min_ii(minx, itemx);
+		miny = min_ii(miny, itemy);
 
-		totw = MAX2(totw, itemx + itemw);
-		toth = MAX2(toth, itemy + itemh);
+		totw = max_ii(totw, itemx + itemw);
+		toth = max_ii(toth, itemy + itemh);
 	}
 
 	totw -= minx;
@@ -2263,11 +2338,14 @@ uiLayout *uiLayoutBox(uiLayout *layout)
 	return (uiLayout *)ui_layout_box(layout, ROUNDBOX);
 }
 
-uiLayout *uiLayoutListBox(uiLayout *layout, PointerRNA *ptr, PropertyRNA *prop, PointerRNA *actptr, PropertyRNA *actprop)
+uiLayout *uiLayoutListBox(uiLayout *layout, uiList *ui_list, PointerRNA *ptr, PropertyRNA *prop, PointerRNA *actptr,
+                          PropertyRNA *actprop)
 {
 	uiLayoutItemBx *box = ui_layout_box(layout, LISTBOX);
 	uiBut *but = box->roundbox;
 
+	but->custom_data = ui_list;
+
 	but->rnasearchpoin = *ptr;
 	but->rnasearchprop = prop;
 	but->rnapoin = *actptr;
@@ -2299,7 +2377,7 @@ uiBlock *uiLayoutAbsoluteBlock(uiLayout *layout)
 	uiBlock *block;
 
 	block = uiLayoutGetBlock(layout);
-	uiLayoutAbsolute(layout, 0);
+	uiLayoutAbsolute(layout, FALSE);
 
 	return block;
 }
@@ -2502,8 +2580,12 @@ static void ui_item_align(uiLayout *litem, short nr)
 				if (!bitem->but->alignnr)
 					bitem->but->alignnr = nr;
 		}
-		else if (item->type == ITEM_LAYOUT_ABSOLUTE) ;
-		else if (item->type == ITEM_LAYOUT_OVERLAP) ;
+		else if (item->type == ITEM_LAYOUT_ABSOLUTE) {
+			/* pass */
+		}
+		else if (item->type == ITEM_LAYOUT_OVERLAP) {
+			/* pass */
+		}
 		else if (item->type == ITEM_LAYOUT_BOX) {
 			box = (uiLayoutItemBx *)item;
 			box->roundbox->alignnr = nr;
@@ -2677,7 +2759,7 @@ void ui_layout_add_but(uiLayout *layout, uiBut *but)
 
 	if (layout->context) {
 		but->context = layout->context;
-		but->context->used = 1;
+		but->context->used = TRUE;
 	}
 }
 
@@ -2724,8 +2806,8 @@ void uiLayoutSetContextPointer(uiLayout *layout, const char *name, PointerRNA *p
 
 void uiLayoutContextCopy(uiLayout *layout, bContextStore *context)
 {
-	uiBlock *block= layout->root->block;
-	layout->context= CTX_store_add_all(&block->contexts, context);
+	uiBlock *block = layout->root->block;
+	layout->context = CTX_store_add_all(&block->contexts, context);
 }
 
 
@@ -2737,7 +2819,7 @@ static void ui_intro_button(DynStr *ds, uiButtonItem *bitem)
 	uiBut *but = bitem->but;
 	BLI_dynstr_appendf(ds, "'type':%d, ", but->type); /* see ~ UI_interface.h:200 */
 	BLI_dynstr_appendf(ds, "'draw_string':'''%s''', ", but->drawstr);
-	BLI_dynstr_appendf(ds, "'tip':'''%s''', ", but->tip ? but->tip : ""); // not exactly needed, rna has this
+	BLI_dynstr_appendf(ds, "'tip':'''%s''', ", but->tip ? but->tip : "");  /* not exactly needed, rna has this */
 
 	if (but->optype) {
 		char *opstr = WM_operator_pystring(but->block->evil_C, but->optype, but->opptr, 0);
@@ -2799,7 +2881,7 @@ static void ui_intro_uiLayout(DynStr *ds, uiLayout *layout)
 	ui_intro_items(ds, &layout->items);
 }
 
-static char *str = NULL; // XXX, constant re-freeing, far from ideal.
+static char *str = NULL;  /* XXX, constant re-freeing, far from ideal. */
 const char *uiLayoutIntrospect(uiLayout *layout)
 {
 	DynStr *ds = BLI_dynstr_new();
@@ -2816,13 +2898,17 @@ const char *uiLayoutIntrospect(uiLayout *layout)
 	return str;
 }
 
+#ifdef USE_OP_RESET_BUT
 static void ui_layout_operator_buts__reset_cb(bContext *UNUSED(C), void *op_pt, void *UNUSED(arg_dummy2))
 {
 	WM_operator_properties_reset((wmOperator *)op_pt);
 }
+#endif
 
 /* this function does not initialize the layout, functions can be called on the layout before and after */
-void uiLayoutOperatorButs(const bContext *C, uiLayout *layout, wmOperator *op, int (*check_prop)(struct PointerRNA *, struct PropertyRNA *), const char label_align, const short flag)
+void uiLayoutOperatorButs(const bContext *C, uiLayout *layout, wmOperator *op,
+                          int (*check_prop)(struct PointerRNA *, struct PropertyRNA *),
+                          const char label_align, const short flag)
 {
 	if (!op->properties) {
 		IDPropertyTemplate val = {0};
@@ -2836,8 +2922,10 @@ void uiLayoutOperatorButs(const bContext *C, uiLayout *layout, wmOperator *op, i
 	/* poll() on this operator may still fail, at the moment there is no nice feedback when this happens
 	 * just fails silently */
 	if (!WM_operator_repeat_check(C, op)) {
-		uiBlockSetButLock(uiLayoutGetBlock(layout), TRUE, "Operator cannot redo");
-		uiItemL(layout, IFACE_("* Redo Unsupported *"), ICON_NONE); // XXX, could give some nicer feedback or not show redo panel at all?
+		uiBlockSetButLock(uiLayoutGetBlock(layout), TRUE, "Operator can't' redo");
+
+		/* XXX, could give some nicer feedback or not show redo panel at all? */
+		uiItemL(layout, IFACE_("* Redo Unsupported *"), ICON_NONE);
 	}
 
 	/* menu */
@@ -2853,12 +2941,12 @@ void uiLayoutOperatorButs(const bContext *C, uiLayout *layout, wmOperator *op, i
 
 		WM_operator_properties_create(&op_ptr, "WM_OT_operator_preset_add");
 		RNA_string_set(&op_ptr, "operator", op->type->idname);
-		op_ptr = uiItemFullO(row, "WM_OT_operator_preset_add", "", ICON_ZOOMIN, op_ptr.data, WM_OP_INVOKE_DEFAULT, 0);
+		uiItemFullO(row, "WM_OT_operator_preset_add", "", ICON_ZOOMIN, op_ptr.data, WM_OP_INVOKE_DEFAULT, 0);
 
 		WM_operator_properties_create(&op_ptr, "WM_OT_operator_preset_add");
 		RNA_string_set(&op_ptr, "operator", op->type->idname);
 		RNA_boolean_set(&op_ptr, "remove_active", TRUE);
-		op_ptr = uiItemFullO(row, "WM_OT_operator_preset_add", "", ICON_ZOOMOUT, op_ptr.data, WM_OP_INVOKE_DEFAULT, 0);
+		uiItemFullO(row, "WM_OT_operator_preset_add", "", ICON_ZOOMOUT, op_ptr.data, WM_OP_INVOKE_DEFAULT, 0);
 	}
 
 	if (op->type->ui) {
@@ -2883,6 +2971,7 @@ void uiLayoutOperatorButs(const bContext *C, uiLayout *layout, wmOperator *op, i
 		}
 	}
 
+#ifdef USE_OP_RESET_BUT
 	/* its possible that reset can do nothing if all have PROP_SKIP_SAVE enabled
 	 * but this is not so important if this button is drawn in those cases
 	 * (which isn't all that likely anyway) - campbell */
@@ -2891,12 +2980,13 @@ void uiLayoutOperatorButs(const bContext *C, uiLayout *layout, wmOperator *op, i
 		uiBut *but;
 		uiLayout *col; /* needed to avoid alignment errors with previous buttons */
 
-		col = uiLayoutColumn(layout, 0);
+		col = uiLayoutColumn(layout, FALSE);
 		block = uiLayoutGetBlock(col);
-		but = uiDefIconTextBut(block, BUT, 0, ICON_FILE_REFRESH, IFACE_("Reset"), 0, 0, 18, 20,
+		but = uiDefIconTextBut(block, BUT, 0, ICON_FILE_REFRESH, IFACE_("Reset"), 0, 0, UI_UNIT_X, UI_UNIT_Y,
 		                       NULL, 0.0, 0.0, 0.0, 0.0, TIP_("Reset operator defaults"));
 		uiButSetFunc(but, ui_layout_operator_buts__reset_cb, op, NULL);
 	}
+#endif
 
 	/* set various special settings for buttons */
 	{
