@@ -27,7 +27,6 @@
  *  \ingroup edinterface
  */
 
-
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
@@ -40,8 +39,12 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_math_color.h"
+#include "BLI_math_vector.h"
 #include "BLI_utildefines.h"
 
+#include "BLF_api.h"
+#include "BLF_translation.h"
+
 #include "BKE_context.h"
 #include "BKE_screen.h"
 #include "BKE_global.h"
@@ -55,6 +58,8 @@
 
 #include "UI_interface.h"
 
+#include "IMB_colormanagement.h"
+
 #include "interface_intern.h"
 
 #include "WM_api.h"
@@ -65,26 +70,52 @@
 #include "BKE_main.h"
 #include "BLI_ghash.h"
 
+#include "ED_image.h"  /* for HDR color sampling */
+#include "ED_node.h"   /* for HDR color sampling */
+#include "ED_clip.h"   /* for HDR color sampling */
 
 /* ********************************************************** */
 
 typedef struct Eyedropper {
+	struct ColorManagedDisplay *display;
+
 	PointerRNA ptr;
 	PropertyRNA *prop;
 	int index;
+
+	int   accum_start; /* has mouse been presed */
+	float accum_col[3];
+	int   accum_tot;
 } Eyedropper;
 
 static int eyedropper_init(bContext *C, wmOperator *op)
 {
+	Scene *scene = CTX_data_scene(C);
 	Eyedropper *eye;
 	
 	op->customdata = eye = MEM_callocN(sizeof(Eyedropper), "Eyedropper");
 	
 	uiContextActiveProperty(C, &eye->ptr, &eye->prop, &eye->index);
-	
-	return (eye->ptr.data && eye->prop && RNA_property_editable(&eye->ptr, eye->prop));
+
+	if ((eye->ptr.data == NULL) ||
+	    (eye->prop == NULL) ||
+	    (RNA_property_editable(&eye->ptr, eye->prop) == FALSE) ||
+	    (RNA_property_array_length(&eye->ptr, eye->prop) < 3) ||
+	    (RNA_property_type(eye->prop) != PROP_FLOAT))
+	{
+		return FALSE;
+	}
+
+	if (RNA_property_subtype(eye->prop) == PROP_COLOR) {
+		const char *display_device;
+
+		display_device = scene->display_settings.display_device;
+		eye->display = IMB_colormanagement_display_get_named(display_device);
+	}
+
+	return TRUE;
 }
- 
+
 static void eyedropper_exit(bContext *C, wmOperator *op)
 {
 	WM_cursor_restore(CTX_wm_window(C));
@@ -100,29 +131,111 @@ static int eyedropper_cancel(bContext *C, wmOperator *op)
 	return OPERATOR_CANCELLED;
 }
 
-static void eyedropper_sample(bContext *C, Eyedropper *eye, int mx, int my)
-{
-	if (RNA_property_type(eye->prop) == PROP_FLOAT) {
-		Scene *scene = CTX_data_scene(C);
-		const int color_manage = scene->r.color_mgt_flag & R_COLOR_MANAGEMENT;
-		float col[4];
-	
-		RNA_property_float_get_array(&eye->ptr, eye->prop, col);
-		
-		glReadBuffer(GL_FRONT);
-		glReadPixels(mx, my, 1, 1, GL_RGB, GL_FLOAT, col);
-		glReadBuffer(GL_BACK);
-	
-		if (RNA_property_array_length(&eye->ptr, eye->prop) < 3) return;
+/* *** eyedropper_color_ helper functions *** */
 
-		/* convert from screen (srgb) space to linear rgb space */
-		if (color_manage && RNA_property_subtype(eye->prop) == PROP_COLOR)
-			srgb_to_linearrgb_v3_v3(col, col);
-		
-		RNA_property_float_set_array(&eye->ptr, eye->prop, col);
-		
-		RNA_property_update(C, &eye->ptr, eye->prop);
+/**
+ * \brief get the color from the screen.
+ *
+ * Special check for image or nodes where we MAY have HDR pixels which don't display.
+ */
+static void eyedropper_color_sample_fl(bContext *C, Eyedropper *UNUSED(eye), int mx, int my, float r_col[3])
+{
+
+	/* we could use some clever */
+	wmWindow *win = CTX_wm_window(C);
+	ScrArea *sa;
+	for (sa = win->screen->areabase.first; sa; sa = sa->next) {
+		if (BLI_rcti_isect_pt(&sa->totrct, mx, my)) {
+			if (sa->spacetype == SPACE_IMAGE) {
+				ARegion *ar = BKE_area_find_region_type(sa, RGN_TYPE_WINDOW);
+				if (BLI_rcti_isect_pt(&ar->winrct, mx, my)) {
+					SpaceImage *sima = sa->spacedata.first;
+					int mval[2] = {mx - ar->winrct.xmin,
+					               my - ar->winrct.ymin};
+
+					if (ED_space_image_color_sample(sima, ar, mval, r_col)) {
+						return;
+					}
+				}
+			}
+			else if (sa->spacetype == SPACE_NODE) {
+				ARegion *ar = BKE_area_find_region_type(sa, RGN_TYPE_WINDOW);
+				if (BLI_rcti_isect_pt(&ar->winrct, mx, my)) {
+					SpaceNode *snode = sa->spacedata.first;
+					int mval[2] = {mx - ar->winrct.xmin,
+					               my - ar->winrct.ymin};
+
+					if (ED_space_node_color_sample(snode, ar, mval, r_col)) {
+						return;
+					}
+				}
+			}
+			else if (sa->spacetype == SPACE_CLIP) {
+				ARegion *ar = BKE_area_find_region_type(sa, RGN_TYPE_WINDOW);
+				if (BLI_rcti_isect_pt(&ar->winrct, mx, my)) {
+					SpaceClip *sc = sa->spacedata.first;
+					int mval[2] = {mx - ar->winrct.xmin,
+					               my - ar->winrct.ymin};
+
+					if (ED_space_clip_color_sample(sc, ar, mval, r_col)) {
+						return;
+					}
+				}
+			}
+		}
 	}
+
+	/* fallback to simple opengl picker */
+	glReadBuffer(GL_FRONT);
+	glReadPixels(mx, my, 1, 1, GL_RGB, GL_FLOAT, r_col);
+	glReadBuffer(GL_BACK);
+}
+
+/* sets the sample color RGB, maintaining A */
+static void eyedropper_color_set(bContext *C, Eyedropper *eye, const float col[3])
+{
+	float col_conv[4];
+
+	/* to maintain alpha */
+	RNA_property_float_get_array(&eye->ptr, eye->prop, col_conv);
+
+	/* convert from display space to linear rgb space */
+	if (eye->display) {
+		copy_v3_v3(col_conv, col);
+		IMB_colormanagement_display_to_scene_linear_v3(col_conv, eye->display);
+	}
+	else {
+		copy_v3_v3(col_conv, col);
+	}
+
+	RNA_property_float_set_array(&eye->ptr, eye->prop, col_conv);
+
+	RNA_property_update(C, &eye->ptr, eye->prop);
+}
+
+/* set sample from accumulated values */
+static void eyedropper_color_set_accum(bContext *C, Eyedropper *eye)
+{
+	float col[3];
+	mul_v3_v3fl(col, eye->accum_col, 1.0f / (float)eye->accum_tot);
+	eyedropper_color_set(C, eye, col);
+}
+
+/* single point sample & set */
+static void eyedropper_color_sample(bContext *C, Eyedropper *eye, int mx, int my)
+{
+	float col[3];
+	eyedropper_color_sample_fl(C, eye, mx, my, col);
+	eyedropper_color_set(C, eye, col);
+}
+
+static void eyedropper_color_sample_accum(bContext *C, Eyedropper *eye, int mx, int my)
+{
+	float col[3];
+	eyedropper_color_sample_fl(C, eye, mx, my, col);
+	/* delay linear conversion */
+	add_v3_v3(eye->accum_col, col);
+	eye->accum_tot++;
 }
 
 /* main modal status check */
@@ -136,10 +249,35 @@ static int eyedropper_modal(bContext *C, wmOperator *op, wmEvent *event)
 			return eyedropper_cancel(C, op);
 		case LEFTMOUSE:
 			if (event->val == KM_RELEASE) {
-				eyedropper_sample(C, eye, event->x, event->y);
+				if (eye->accum_tot == 0) {
+					eyedropper_color_sample(C, eye, event->x, event->y);
+				}
+				else {
+					eyedropper_color_set_accum(C, eye);
+				}
 				eyedropper_exit(C, op);
 				return OPERATOR_FINISHED;
 			}
+			else if (event->val == KM_PRESS) {
+				/* enable accum and make first sample */
+				eye->accum_start = TRUE;
+				eyedropper_color_sample_accum(C, eye, event->x, event->y);
+			}
+			break;
+		case MOUSEMOVE:
+			if (eye->accum_start) {
+				/* button is pressed so keep sampling */
+				eyedropper_color_sample_accum(C, eye, event->x, event->y);
+				eyedropper_color_set_accum(C, eye);
+			}
+			break;
+		case SPACEKEY:
+			if (event->val == KM_RELEASE) {
+				eye->accum_tot = 0;
+				zero_v3(eye->accum_col);
+				eyedropper_color_sample_accum(C, eye, event->x, event->y);
+				eyedropper_color_set_accum(C, eye);
+			}
 			break;
 	}
 	
@@ -234,12 +372,32 @@ static void UI_OT_reset_default_theme(wmOperatorType *ot)
 
 /* Copy Data Path Operator ------------------------ */
 
+static int copy_data_path_button_poll(bContext *C)
+{
+	PointerRNA ptr;
+	PropertyRNA *prop;
+	char *path;
+	int index;
+
+	uiContextActiveProperty(C, &ptr, &prop, &index);
+
+	if (ptr.id.data && ptr.data && prop) {
+		path = RNA_path_from_ID_to_property(&ptr, prop);
+		
+		if (path) {
+			MEM_freeN(path);
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
 static int copy_data_path_button_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	PointerRNA ptr;
 	PropertyRNA *prop;
 	char *path;
-	int success = 0;
 	int index;
 
 	/* try to create driver using property retrieved from UI */
@@ -251,11 +409,11 @@ static int copy_data_path_button_exec(bContext *C, wmOperator *UNUSED(op))
 		if (path) {
 			WM_clipboard_text_set(path, FALSE);
 			MEM_freeN(path);
+			return OPERATOR_FINISHED;
 		}
 	}
 
-	/* since we're just copying, we don't really need to do anything else...*/
-	return (success) ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
+	return OPERATOR_CANCELLED;
 }
 
 static void UI_OT_copy_data_path_button(wmOperatorType *ot)
@@ -267,7 +425,7 @@ static void UI_OT_copy_data_path_button(wmOperatorType *ot)
 
 	/* callbacks */
 	ot->exec = copy_data_path_button_exec;
-	//op->poll= ??? // TODO: need to have some valid property before this can be done
+	ot->poll = copy_data_path_button_poll;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER;
@@ -348,7 +506,7 @@ static void UI_OT_reset_default_button(wmOperatorType *ot)
 
 static int copy_to_selected_list(bContext *C, PointerRNA *ptr, ListBase *lb, int *use_path)
 {
-	*use_path = 0;
+	*use_path = FALSE;
 
 	if (RNA_struct_is_a(ptr->type, &RNA_EditBone))
 		*lb = CTX_data_collection_get(C, "selected_editable_bones");
@@ -361,7 +519,7 @@ static int copy_to_selected_list(bContext *C, PointerRNA *ptr, ListBase *lb, int
 
 		if (id && GS(id->name) == ID_OB) {
 			*lb = CTX_data_collection_get(C, "selected_editable_objects");
-			*use_path = 1;
+			*use_path = TRUE;
 		}
 		else
 			return 0;
@@ -503,11 +661,12 @@ static int reports_to_text_poll(bContext *C)
 static int reports_to_text_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	ReportList *reports = CTX_wm_reports(C);
+	Main *bmain = CTX_data_main(C);
 	Text *txt;
 	char *str;
 	
 	/* create new text-block to write to */
-	txt = add_empty_text("Recent Reports");
+	txt = BKE_text_add(bmain, "Recent Reports");
 	
 	/* convert entire list to a display string, and add this to the text-block
 	 *	- if commandline debug option enabled, show debug reports too
@@ -516,7 +675,7 @@ static int reports_to_text_exec(bContext *C, wmOperator *UNUSED(op))
 	str = BKE_reports_string(reports, (G.debug & G_DEBUG) ? RPT_DEBUG : RPT_INFO);
 
 	if (str) {
-		write_text(txt, str);
+		BKE_text_write(txt, str);
 		MEM_freeN(str);
 
 		return OPERATOR_FINISHED;
@@ -569,9 +728,7 @@ static void ui_editsource_active_but_set(uiBut *but)
 	ui_editsource_info = MEM_callocN(sizeof(uiEditSourceStore), __func__);
 	memcpy(&ui_editsource_info->but_orig, but, sizeof(uiBut));
 
-	ui_editsource_info->hash = BLI_ghash_new(BLI_ghashutil_ptrhash,
-	                                         BLI_ghashutil_ptrcmp,
-	                                         __func__);
+	ui_editsource_info->hash = BLI_ghash_ptr_new(__func__);
 }
 
 static void ui_editsource_active_but_clear(void)
@@ -591,10 +748,7 @@ static int ui_editsource_uibut_match(uiBut *but_a, uiBut *but_b)
 	/* this just needs to be a 'good-enough' comparison so we can know beyond
 	 * reasonable doubt that these buttons are the same between redraws.
 	 * if this fails it only means edit-source fails - campbell */
-	if ((but_a->x1 == but_b->x1) &&
-	    (but_a->x2 == but_b->x2) &&
-	    (but_a->y1 == but_b->y1) &&
-	    (but_a->y2 == but_b->y2) &&
+	if (BLI_rctf_compare(&but_a->rect, &but_b->rect, FLT_EPSILON) &&
 	    (but_a->type == but_b->type) &&
 	    (but_a->rnaprop == but_b->rnaprop) &&
 	    (but_a->optype == but_b->optype) &&
@@ -637,8 +791,6 @@ void UI_editsource_active_but_test(uiBut *but)
 	BLI_ghash_insert(ui_editsource_info->hash, but, but_store);
 }
 
-/* editsource operator component */
-
 static int editsource_text_edit(bContext *C, wmOperator *op,
                                 char filepath[FILE_MAX], int line)
 {
@@ -652,12 +804,11 @@ static int editsource_text_edit(bContext *C, wmOperator *op,
 	}
 
 	if (text == NULL) {
-		text = add_text(filepath, bmain->name);
+		text = BKE_text_load(bmain, filepath, bmain->name);
 	}
 
 	if (text == NULL) {
-		BKE_reportf(op->reports, RPT_WARNING,
-		            "file: '%s' can't be opened", filepath);
+		BKE_reportf(op->reports, RPT_WARNING, "File '%s' cannot be opened", filepath);
 		return OPERATOR_CANCELLED;
 	}
 	else {
@@ -669,8 +820,7 @@ static int editsource_text_edit(bContext *C, wmOperator *op,
 			st->text = text;
 		}
 		else {
-			BKE_reportf(op->reports, RPT_INFO,
-			            "See '%s' in the text editor", text->id.name + 2);
+			BKE_reportf(op->reports, RPT_INFO, "See '%s' in the text editor", text->id.name + 2);
 		}
 
 		txt_move_toline(text, line - 1, FALSE);
@@ -706,8 +856,8 @@ static int editsource_exec(bContext *C, wmOperator *op)
 		     !BLI_ghashIterator_isDone(&ghi);
 		     BLI_ghashIterator_step(&ghi))
 		{
-			uiBut *but = BLI_ghashIterator_getKey(&ghi);
-			if (but && ui_editsource_uibut_match(&ui_editsource_info->but_orig, but)) {
+			uiBut *but_key = BLI_ghashIterator_getKey(&ghi);
+			if (but_key && ui_editsource_uibut_match(&ui_editsource_info->but_orig, but_key)) {
 				but_store = BLI_ghashIterator_getValue(&ghi);
 				break;
 			}
@@ -721,14 +871,12 @@ static int editsource_exec(bContext *C, wmOperator *op)
 				                           but_store->py_dbg_ln);
 			}
 			else {
-				BKE_report(op->reports, RPT_ERROR,
-				           "Active button isn't from a script, cant edit source.");
+				BKE_report(op->reports, RPT_ERROR, "Active button is not from a script, cannot edit source");
 				ret = OPERATOR_CANCELLED;
 			}
 		}
 		else {
-			BKE_report(op->reports, RPT_ERROR,
-			           "Active button match can't be found.");
+			BKE_report(op->reports, RPT_ERROR, "Active button match cannot be found");
 			ret = OPERATOR_CANCELLED;
 		}
 
@@ -748,16 +896,181 @@ static int editsource_exec(bContext *C, wmOperator *op)
 static void UI_OT_editsource(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Reports to Text Block";
+	ot->name = "Edit Source";
 	ot->idname = "UI_OT_editsource";
-	ot->description = "Edit source code for a button";
+	ot->description = "Edit UI source code of the active button";
 
 	/* callbacks */
 	ot->exec = editsource_exec;
 }
 
+/* ------------------------------------------------------------------------- */
+/* EditTranslation utility funcs and operator,
+ * Note: this includes utility functions and button matching checks.
+ *       this only works in conjunction with a py operator! */
+
+static void edittranslation_find_po_file(const char *root, const char *uilng, char *path, const size_t maxlen)
+{
+	char tstr[32]; /* Should be more than enough! */
+
+	/* First, full lang code. */
+	BLI_snprintf(tstr, sizeof(tstr), "%s.po", uilng);
+	BLI_join_dirfile(path, maxlen, root, uilng);
+	BLI_join_dirfile(path, maxlen, path, tstr);
+	if (BLI_is_file(path))
+		return;
+
+	/* Now try without the second iso code part (_ES in es_ES). */
+	{
+		char *tc = NULL;
+		size_t szt = 0;
+		tstr[0] = '\0';
+
+		tc = strchr(uilng, '_');
+		if (tc) {
+			szt = tc - uilng;
+			if (szt < sizeof(tstr)) /* Paranoid, should always be true! */
+				BLI_strncpy(tstr, uilng, szt + 1); /* +1 for '\0' char! */
+		}
+		if (tstr[0]) {
+			/* Because of some codes like sr_SR@latin... */
+			tc = strchr(uilng, '@');
+			if (tc)
+				BLI_strncpy(tstr + szt, tc, sizeof(tstr) - szt);
+
+			BLI_join_dirfile(path, maxlen, root, tstr);
+			strcat(tstr, ".po");
+			BLI_join_dirfile(path, maxlen, path, tstr);
+			if (BLI_is_file(path))
+				return;
+		}
+	}
+
+	/* Else no po file! */
+	path[0] = '\0';
+}
+
+static int edittranslation_exec(bContext *C, wmOperator *op)
+{
+	uiBut *but = uiContextActiveButton(C);
+	int ret = OPERATOR_CANCELLED;
+
+	if (but) {
+		PointerRNA ptr;
+		char popath[FILE_MAX];
+		const char *root = U.i18ndir;
+		const char *uilng = BLF_lang_get();
+
+		uiStringInfo but_label = {BUT_GET_LABEL, NULL};
+		uiStringInfo rna_label = {BUT_GET_RNA_LABEL, NULL};
+		uiStringInfo enum_label = {BUT_GET_RNAENUM_LABEL, NULL};
+		uiStringInfo but_tip = {BUT_GET_TIP, NULL};
+		uiStringInfo rna_tip = {BUT_GET_RNA_TIP, NULL};
+		uiStringInfo enum_tip = {BUT_GET_RNAENUM_TIP, NULL};
+		uiStringInfo rna_struct = {BUT_GET_RNASTRUCT_IDENTIFIER, NULL};
+		uiStringInfo rna_prop = {BUT_GET_RNAPROP_IDENTIFIER, NULL};
+		uiStringInfo rna_enum = {BUT_GET_RNAENUM_IDENTIFIER, NULL};
+		uiStringInfo rna_ctxt = {BUT_GET_RNA_LABEL_CONTEXT, NULL};
+
+		if (!BLI_is_dir(root)) {
+			BKE_report(op->reports, RPT_ERROR, "Please set your User Preferences' 'Translation Branches "
+			                                   "Directory' path to a valid directory");
+			return OPERATOR_CANCELLED;
+		}
+		if (!WM_operatortype_find(EDTSRC_I18N_OP_NAME, 0)) {
+			BKE_reportf(op->reports, RPT_ERROR, "Could not find operator '%s'! Please enable ui_translate addon "
+			                                    "in the User Preferences", EDTSRC_I18N_OP_NAME);
+			return OPERATOR_CANCELLED;
+		}
+		/* Try to find a valid po file for current language... */
+		edittranslation_find_po_file(root, uilng, popath, FILE_MAX);
+/*		printf("po path: %s\n", popath);*/
+		if (popath[0] == '\0') {
+			BKE_reportf(op->reports, RPT_ERROR, "No valid po found for language '%s' under %s", uilng, root);
+			return OPERATOR_CANCELLED;
+		}
+
+		uiButGetStrInfo(C, but, &but_label, &rna_label, &enum_label, &but_tip, &rna_tip, &enum_tip,
+		                &rna_struct, &rna_prop, &rna_enum, &rna_ctxt, NULL);
+
+		WM_operator_properties_create(&ptr, EDTSRC_I18N_OP_NAME);
+		RNA_string_set(&ptr, "lang", uilng);
+		RNA_string_set(&ptr, "po_file", popath);
+		RNA_string_set(&ptr, "but_label", but_label.strinfo);
+		RNA_string_set(&ptr, "rna_label", rna_label.strinfo);
+		RNA_string_set(&ptr, "enum_label", enum_label.strinfo);
+		RNA_string_set(&ptr, "but_tip", but_tip.strinfo);
+		RNA_string_set(&ptr, "rna_tip", rna_tip.strinfo);
+		RNA_string_set(&ptr, "enum_tip", enum_tip.strinfo);
+		RNA_string_set(&ptr, "rna_struct", rna_struct.strinfo);
+		RNA_string_set(&ptr, "rna_prop", rna_prop.strinfo);
+		RNA_string_set(&ptr, "rna_enum", rna_enum.strinfo);
+		RNA_string_set(&ptr, "rna_ctxt", rna_ctxt.strinfo);
+		ret = WM_operator_name_call(C, EDTSRC_I18N_OP_NAME, WM_OP_INVOKE_DEFAULT, &ptr);
+
+		/* Clean up */
+		if (but_label.strinfo)
+			MEM_freeN(but_label.strinfo);
+		if (rna_label.strinfo)
+			MEM_freeN(rna_label.strinfo);
+		if (enum_label.strinfo)
+			MEM_freeN(enum_label.strinfo);
+		if (but_tip.strinfo)
+			MEM_freeN(but_tip.strinfo);
+		if (rna_tip.strinfo)
+			MEM_freeN(rna_tip.strinfo);
+		if (enum_tip.strinfo)
+			MEM_freeN(enum_tip.strinfo);
+		if (rna_struct.strinfo)
+			MEM_freeN(rna_struct.strinfo);
+		if (rna_prop.strinfo)
+			MEM_freeN(rna_prop.strinfo);
+		if (rna_enum.strinfo)
+			MEM_freeN(rna_enum.strinfo);
+		if (rna_ctxt.strinfo)
+			MEM_freeN(rna_ctxt.strinfo);
+
+		return ret;
+	}
+	else {
+		BKE_report(op->reports, RPT_ERROR, "Active button not found");
+		return OPERATOR_CANCELLED;
+	}
+}
+
+static void UI_OT_edittranslation_init(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Edit Translation";
+	ot->idname = "UI_OT_edittranslation_init";
+	ot->description = "Edit i18n in current language for the active button";
+
+	/* callbacks */
+	ot->exec = edittranslation_exec;
+}
+
 #endif /* WITH_PYTHON */
 
+static int reloadtranslation_exec(bContext *UNUSED(C), wmOperator *UNUSED(op))
+{
+	BLF_lang_init();
+	BLF_cache_clear();
+	BLF_lang_set(NULL);
+	UI_reinit_font();
+	return OPERATOR_FINISHED;
+}
+
+static void UI_OT_reloadtranslation(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Reload Translation";
+	ot->idname = "UI_OT_reloadtranslation";
+	ot->description = "Force a full reload of UI translation";
+
+	/* callbacks */
+	ot->exec = reloadtranslation_exec;
+}
+
 /* ********************************************************* */
 /* Registration */
 
@@ -768,10 +1081,12 @@ void UI_buttons_operatortypes(void)
 	WM_operatortype_append(UI_OT_copy_data_path_button);
 	WM_operatortype_append(UI_OT_reset_default_button);
 	WM_operatortype_append(UI_OT_copy_to_selected_button);
-	WM_operatortype_append(UI_OT_reports_to_textblock); // XXX: temp?
+	WM_operatortype_append(UI_OT_reports_to_textblock);  /* XXX: temp? */
 
 #ifdef WITH_PYTHON
 	WM_operatortype_append(UI_OT_editsource);
+	WM_operatortype_append(UI_OT_edittranslation_init);
 #endif
+	WM_operatortype_append(UI_OT_reloadtranslation);
 }
 
