@@ -1,5 +1,5 @@
 /*
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -86,12 +86,12 @@ const unsigned char *UI_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colo
 	static char error[4] = {240, 0, 240, 255};
 	static char alert[4] = {240, 60, 60, 255};
 	static char headerdesel[4] = {0, 0, 0, 255};
-	
+	static char setting = 0;
 	const char *cp = error;
 	
 	if (btheme) {
 	
-		// first check for ui buttons theme
+		/* first check for ui buttons theme */
 		if (colorid < TH_THEMEUI) {
 		
 			switch (colorid) {
@@ -170,6 +170,16 @@ const unsigned char *UI_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colo
 					else
 						cp = ts->button;
 					break;
+				case TH_LOW_GRAD:
+					cp = ts->gradients.gradient;
+					break;
+				case TH_HIGH_GRAD:
+					cp = ts->gradients.high_gradient;
+					break;
+				case TH_SHOW_BACK_GRAD:
+					cp = &setting;
+					setting = ts->gradients.show_grad;
+					break;
 				case TH_TEXT:
 					if (theme_regionid == RGN_TYPE_WINDOW)
 						cp = ts->text;
@@ -216,13 +226,19 @@ const unsigned char *UI_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colo
 				case TH_HEADER_TEXT_HI:
 					cp = ts->header_text_hi; break;
 
-				case TH_PANEL:
-					cp = ts->panel; break;
-				case TH_PANEL_TEXT:
-					cp = ts->panel_text; break;
-				case TH_PANEL_TEXT_HI:
-					cp = ts->panel_text_hi; break;
-
+				case TH_PANEL_HEADER:
+					cp = ts->panelcolors.header; break;
+				case TH_PANEL_BACK:
+					cp = ts->panelcolors.back; break;
+				case TH_PANEL_SHOW_HEADER:
+					cp = &setting;
+					setting = ts->panelcolors.show_header;
+					break;
+				case TH_PANEL_SHOW_BACK:
+					cp = &setting;
+					setting = ts->panelcolors.show_back;
+					break;
+					
 				case TH_BUTBACK:
 					cp = ts->button; break;
 				case TH_BUTBACK_TEXT:
@@ -236,7 +252,7 @@ const unsigned char *UI_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colo
 					cp = ts->shade2; break;
 				case TH_HILITE:
 					cp = ts->hilite; break;
-
+				
 				case TH_GRID:
 					cp = ts->grid; break;
 				case TH_WIRE:
@@ -303,6 +319,8 @@ const unsigned char *UI_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colo
 					cp = ts->bone_solid; break;
 				case TH_BONE_POSE:
 					cp = ts->bone_pose; break;
+				case TH_BONE_POSE_ACTIVE:
+					cp = ts->bone_pose_active; break;
 				case TH_STRIP:
 					cp = ts->strip; break;
 				case TH_STRIP_SELECT:
@@ -350,8 +368,14 @@ const unsigned char *UI_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colo
 					cp = ts->syntaxc; break;
 				case TH_SYNTAX_L:
 					cp = ts->syntaxl; break;
+				case TH_SYNTAX_D:
+					cp = ts->syntaxd; break;
+				case TH_SYNTAX_R:
+					cp = ts->syntaxr; break;
 				case TH_SYNTAX_N:
 					cp = ts->syntaxn; break;
+				case TH_SYNTAX_S:
+					cp = ts->syntaxs; break;
 
 				case TH_NODE:
 					cp = ts->syntaxl; break;
@@ -363,6 +387,12 @@ const unsigned char *UI_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colo
 					cp = ts->syntaxv; break;
 				case TH_NODE_GROUP:
 					cp = ts->syntaxc; break;
+				case TH_NODE_FRAME:
+					cp = ts->movie; break;
+				case TH_NODE_MATTE:
+					cp = ts->syntaxs; break;
+				case TH_NODE_DISTORT:
+					cp = ts->syntaxd; break;
 				case TH_NODE_CURVING:
 					cp = &ts->noodle_curving; break;
 
@@ -370,6 +400,8 @@ const unsigned char *UI_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colo
 					cp = ts->movie; break;
 				case TH_SEQ_MOVIECLIP:
 					cp = ts->movieclip; break;
+				case TH_SEQ_MASK:
+					cp = ts->mask; break;
 				case TH_SEQ_IMAGE:
 					cp = ts->image; break;
 				case TH_SEQ_SCENE:
@@ -378,8 +410,6 @@ const unsigned char *UI_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colo
 					cp = ts->audio; break;
 				case TH_SEQ_EFFECT:
 					cp = ts->effect; break;
-				case TH_SEQ_PLUGIN:
-					cp = ts->plugin; break;
 				case TH_SEQ_TRANSITION:
 					cp = ts->transition; break;
 				case TH_SEQ_META:
@@ -397,6 +427,8 @@ const unsigned char *UI_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colo
 					cp = ts->console_error; break;
 				case TH_CONSOLE_CURSOR:
 					cp = ts->console_cursor; break;
+				case TH_CONSOLE_SELECT:
+					cp = ts->console_select; break;
 
 				case TH_HANDLE_VERTEX:
 					cp = ts->handle_vertex;
@@ -469,6 +501,50 @@ const unsigned char *UI_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colo
 				case TH_SELECT_HIGHLIGHT:
 					cp = ts->selected_highlight;
 					break;
+
+				case TH_SKIN_ROOT:
+					cp = ts->skin_root;
+					break;
+					
+				case TH_ANIM_ACTIVE:
+					cp = ts->anim_active;
+					break;
+				case TH_ANIM_INACTIVE:
+					cp = ts->anim_non_active;
+					break;
+				
+				case TH_NLA_TWEAK:
+					cp = ts->nla_tweaking;
+					break;
+				case TH_NLA_TWEAK_DUPLI:
+					cp = ts->nla_tweakdupli;
+					break;
+				
+				case TH_NLA_TRANSITION:
+					cp = ts->nla_transition;
+					break;
+				case TH_NLA_TRANSITION_SEL:
+					cp = ts->nla_transition_sel;
+					break;
+				case TH_NLA_META:
+					cp = ts->nla_meta;
+					break;
+				case TH_NLA_META_SEL:
+					cp = ts->nla_meta_sel;
+					break;
+				case TH_NLA_SOUND:
+					cp = ts->nla_sound;
+					break;
+				case TH_NLA_SOUND_SEL:
+					cp = ts->nla_sound_sel;
+					break;
+					
+				case TH_AXIS_X:
+					cp = btheme->tui.xaxis; break;
+				case TH_AXIS_Y:
+					cp = btheme->tui.yaxis; break;
+				case TH_AXIS_Z:
+					cp = btheme->tui.zaxis; break;
 			}
 		}
 	}
@@ -555,9 +631,9 @@ static void ui_theme_init_new_do(ThemeSpace *ts)
 	rgba_char_args_test_set(ts->header_title,   0, 0, 0, 255);
 	rgba_char_args_test_set(ts->header_text_hi, 255, 255, 255, 255);
 
-	rgba_char_args_test_set(ts->panel_text,     0, 0, 0, 255);
-	rgba_char_args_test_set(ts->panel_title,        0, 0, 0, 255);
-	rgba_char_args_test_set(ts->panel_text_hi,  255, 255, 255, 255);
+//	rgba_char_args_test_set(ts->panel_text,     0, 0, 0, 255);
+//	rgba_char_args_test_set(ts->panel_title,        0, 0, 0, 255);
+//	rgba_char_args_test_set(ts->panel_text_hi,  255, 255, 255, 255);
 
 	rgba_char_args_test_set(ts->button,         145, 145, 145, 245);
 	rgba_char_args_test_set(ts->button_title,   0, 0, 0, 255);
@@ -602,9 +678,7 @@ void ui_theme_init_default(void)
 	bTheme *btheme;
 	
 	/* we search for the theme with name Default */
-	for (btheme = U.themes.first; btheme; btheme = btheme->next) {
-		if (strcmp("Default", btheme->name) == 0) break;
-	}
+	btheme = BLI_findstring(&U.themes, "Default", offsetof(bTheme, name));
 	
 	if (btheme == NULL) {
 		btheme = MEM_callocN(sizeof(bTheme), "theme");
@@ -612,14 +686,23 @@ void ui_theme_init_default(void)
 		strcpy(btheme->name, "Default");
 	}
 	
-	UI_SetTheme(0, 0);  // make sure the global used in this file is set
+	UI_SetTheme(0, 0);  /* make sure the global used in this file is set */
 
 	/* UI buttons */
 	ui_widget_color_init(&btheme->tui);
+	
 	btheme->tui.iconfile[0] = 0;
+	btheme->tui.panel.show_back = FALSE;
 	btheme->tui.panel.show_header = FALSE;
 	rgba_char_args_set(btheme->tui.panel.header, 0, 0, 0, 25);
+	
+	rgba_char_args_set(btheme->tui.xaxis, 220,   0,   0, 255);
+	rgba_char_args_set(btheme->tui.yaxis,   0, 220,   0, 255);
+	rgba_char_args_set(btheme->tui.zaxis,   0,   0, 220, 255);
 
+	btheme->tui.menu_shadow_fac = 0.5f;
+	btheme->tui.menu_shadow_width = 12;
+	
 	/* Bone Color Sets */
 	ui_theme_init_boneColorSets(btheme);
 	
@@ -627,13 +710,17 @@ void ui_theme_init_default(void)
 	ui_theme_init_new(btheme);
 	
 	/* space view3d */
+	btheme->tv3d.panelcolors.show_back = FALSE;
+	btheme->tv3d.panelcolors.show_header = FALSE;
+	rgba_char_args_set_fl(btheme->tv3d.panelcolors.back, 0.45, 0.45, 0.45, 0.5);
+	rgba_char_args_set_fl(btheme->tv3d.panelcolors.header, 0, 0, 0, 0.01);
 	rgba_char_args_set_fl(btheme->tv3d.back,       0.225, 0.225, 0.225, 1.0);
 	rgba_char_args_set(btheme->tv3d.text,       0, 0, 0, 255);
 	rgba_char_args_set(btheme->tv3d.text_hi, 255, 255, 255, 255);
 	
 	rgba_char_args_set_fl(btheme->tv3d.header,  0.45, 0.45, 0.45, 1.0);
-	rgba_char_args_set_fl(btheme->tv3d.button,  0.45, 0.45, 0.45, 1.0);
-	rgba_char_args_set(btheme->tv3d.panel,      165, 165, 165, 127);
+	rgba_char_args_set_fl(btheme->tv3d.button,  0.45, 0.45, 0.45, 0.5);
+//	rgba_char_args_set(btheme->tv3d.panel,      165, 165, 165, 127);
 	
 	rgba_char_args_set(btheme->tv3d.shade1,  160, 160, 160, 100);
 	rgba_char_args_set(btheme->tv3d.shade2,  0x7f, 0x70, 0x70, 100);
@@ -698,24 +785,31 @@ void ui_theme_init_default(void)
 	rgba_char_args_set(btheme->tv3d.lastsel_point,  0xff, 0xff, 0xff, 255);
 
 	rgba_char_args_set(btheme->tv3d.bone_solid, 200, 200, 200, 255);
-	rgba_char_args_set(btheme->tv3d.bone_pose, 80, 200, 255, 80);               // alpha 80 is not meant editable, used for wire+action draw
+	/* alpha 80 is not meant editable, used for wire+action draw */
+	rgba_char_args_set(btheme->tv3d.bone_pose, 80, 200, 255, 80);
+	rgba_char_args_set(btheme->tv3d.bone_pose_active, 140, 255, 255, 80);
 
 	rgba_char_args_set(btheme->tv3d.bundle_solid, 200, 200, 200, 255);
 	rgba_char_args_set(btheme->tv3d.camera_path, 0x00, 0x00, 0x00, 255);
-	
+
+	rgba_char_args_set(btheme->tv3d.skin_root, 180, 77, 77, 255);
+	rgba_char_args_set(btheme->tv3d.gradients.gradient, 0, 0, 0, 0);
+	rgba_char_args_set(btheme->tv3d.gradients.high_gradient, 58, 58, 58, 255);
+	btheme->tv3d.gradients.show_grad = FALSE;
+
 	/* space buttons */
 	/* to have something initialized */
 	btheme->tbuts = btheme->tv3d;
 
 	rgba_char_args_set_fl(btheme->tbuts.back,   0.45, 0.45, 0.45, 1.0);
-	rgba_char_args_set(btheme->tbuts.panel, 0x82, 0x82, 0x82, 255);
+//	rgba_char_args_set(btheme->tbuts.panel, 0x82, 0x82, 0x82, 255);
 
 	/* graph editor */
 	btheme->tipo = btheme->tv3d;
 	rgba_char_args_set_fl(btheme->tipo.back,    0.42, 0.42, 0.42, 1.0);
 	rgba_char_args_set_fl(btheme->tipo.list,    0.4, 0.4, 0.4, 1.0);
 	rgba_char_args_set(btheme->tipo.grid,   94, 94, 94, 255);
-	rgba_char_args_set(btheme->tipo.panel,  255, 255, 255, 150);
+//	rgba_char_args_set(btheme->tipo.panel,  255, 255, 255, 150);
 	rgba_char_args_set(btheme->tipo.shade1,     150, 150, 150, 100);    /* scrollbars */
 	rgba_char_args_set(btheme->tipo.shade2,     0x70, 0x70, 0x70, 100);
 	rgba_char_args_set(btheme->tipo.vertex,     0, 0, 0, 255);
@@ -729,30 +823,45 @@ void ui_theme_init_default(void)
 	rgba_char_args_set(btheme->tipo.handle_sel_auto_clamped, 0xf0, 0xaf, 0x90, 255);
 	btheme->tipo.handle_vertex_size = 4;
 	
-	rgba_char_args_set(btheme->tipo.ds_channel,     82, 96, 110, 255);
+	rgba_char_args_set(btheme->tipo.ds_channel,      82, 96, 110, 255);
 	rgba_char_args_set(btheme->tipo.ds_subchannel,  124, 137, 150, 255);
-	rgba_char_args_set(btheme->tipo.group, 79, 101, 73, 255);
-	rgba_char_args_set(btheme->tipo.group_active, 135, 177, 125, 255);
+	rgba_char_args_set(btheme->tipo.group,           79, 101, 73, 255);
+	rgba_char_args_set(btheme->tipo.group_active,   135, 177, 125, 255);
 
 	/* dopesheet */
 	btheme->tact = btheme->tipo;
 	rgba_char_args_set(btheme->tact.strip,          12, 10, 10, 128);
 	rgba_char_args_set(btheme->tact.strip_select,   255, 140, 0, 255);
 	
+	rgba_char_args_set(btheme->tact.anim_active,    204, 112, 26, 102);
+	
 	/* space nla */
 	btheme->tnla = btheme->tact;
 	
+	rgba_char_args_set(btheme->tnla.anim_active,     204, 112, 26, 102); /* same as for dopesheet; duplicate here for easier reference */
+	rgba_char_args_set(btheme->tnla.anim_non_active, 153, 135, 97, 77);
+	
+	rgba_char_args_set(btheme->tnla.nla_tweaking,   77, 243, 26, 77);
+	rgba_char_args_set(btheme->tnla.nla_tweakdupli, 217, 0, 0, 255);
+	
+	rgba_char_args_set(btheme->tnla.nla_transition,     28, 38, 48, 255);
+	rgba_char_args_set(btheme->tnla.nla_transition_sel, 46, 117, 219, 255);
+	rgba_char_args_set(btheme->tnla.nla_meta,           51, 38, 66, 255);
+	rgba_char_args_set(btheme->tnla.nla_meta_sel,       105, 33, 150, 255);
+	rgba_char_args_set(btheme->tnla.nla_sound,          43, 61, 61, 255);
+	rgba_char_args_set(btheme->tnla.nla_sound_sel,      31, 122, 122, 255);
+	
 	/* space file */
 	/* to have something initialized */
 	btheme->tfile = btheme->tv3d;
 	rgba_char_args_set_fl(btheme->tfile.back, 0.3, 0.3, 0.3, 1);
-	rgba_char_args_set_fl(btheme->tfile.panel, 0.3, 0.3, 0.3, 1);
+//	rgba_char_args_set_fl(btheme->tfile.panel, 0.3, 0.3, 0.3, 1);
 	rgba_char_args_set_fl(btheme->tfile.list, 0.4, 0.4, 0.4, 1);
 	rgba_char_args_set(btheme->tfile.text,  250, 250, 250, 255);
 	rgba_char_args_set(btheme->tfile.text_hi, 15, 15, 15, 255);
-	rgba_char_args_set(btheme->tfile.panel, 145, 145, 145, 255);    // bookmark/ui regions
-	rgba_char_args_set(btheme->tfile.active, 130, 130, 130, 255); // selected files
-	rgba_char_args_set(btheme->tfile.hilite, 255, 140, 25, 255); // selected files
+//	rgba_char_args_set(btheme->tfile.panel, 145, 145, 145, 255);  /* bookmark/ui regions */
+	rgba_char_args_set(btheme->tfile.active, 130, 130, 130, 255); /* selected files */
+	rgba_char_args_set(btheme->tfile.hilite, 255, 140, 25, 255);  /* selected files */
 	
 	rgba_char_args_set(btheme->tfile.grid,  250, 250, 250, 255);
 	rgba_char_args_set(btheme->tfile.image, 250, 250, 250, 255);
@@ -765,11 +874,11 @@ void ui_theme_init_default(void)
 	rgba_char_args_set(btheme->tseq.back,   116, 116, 116, 255);
 	rgba_char_args_set(btheme->tseq.movie,  81, 105, 135, 255);
 	rgba_char_args_set(btheme->tseq.movieclip,  32, 32, 143, 255);
+	rgba_char_args_set(btheme->tseq.mask,   152, 78, 62, 255);
 	rgba_char_args_set(btheme->tseq.image,  109, 88, 129, 255);
 	rgba_char_args_set(btheme->tseq.scene,  78, 152, 62, 255);
 	rgba_char_args_set(btheme->tseq.audio,  46, 143, 143, 255);
 	rgba_char_args_set(btheme->tseq.effect,     169, 84, 124, 255);
-	rgba_char_args_set(btheme->tseq.plugin,     126, 126, 80, 255);
 	rgba_char_args_set(btheme->tseq.transition, 162, 95, 111, 255);
 	rgba_char_args_set(btheme->tseq.meta,   109, 145, 131, 255);
 	rgba_char_args_set(btheme->tseq.preview_back,   0, 0, 0, 255);
@@ -790,6 +899,7 @@ void ui_theme_init_default(void)
 	rgba_char_args_set_fl(btheme->tima.preview_stitch_vert, 0.0, 0.0, 1.0, 0.2);
 	rgba_char_args_set_fl(btheme->tima.preview_stitch_stitchable, 0.0, 1.0, 0.0, 1.0);
 	rgba_char_args_set_fl(btheme->tima.preview_stitch_unstitchable, 1.0, 0.0, 0.0, 1.0);
+	rgba_char_args_set_fl(btheme->tima.preview_stitch_active, 0.886, 0.824, 0.765, 0.140);
 
 	/* space text */
 	btheme->text = btheme->tv3d;
@@ -800,10 +910,13 @@ void ui_theme_init_default(void)
 	
 	/* syntax highlighting */
 	rgba_char_args_set(btheme->text.syntaxn,    0, 0, 200, 255);    /* Numbers  Blue*/
-	rgba_char_args_set(btheme->text.syntaxl,    100, 0, 0, 255);    /* Strings  red */
-	rgba_char_args_set(btheme->text.syntaxc,    0, 100, 50, 255);   /* Comments greenish */
-	rgba_char_args_set(btheme->text.syntaxv,    95, 95, 0, 255);    /* Special */
-	rgba_char_args_set(btheme->text.syntaxb,    128, 0, 80, 255);   /* Builtin, red-purple */
+	rgba_char_args_set(btheme->text.syntaxl,    100, 0, 0, 255);    /* Strings  Red */
+	rgba_char_args_set(btheme->text.syntaxc,    0, 100, 50, 255);   /* Comments  Greenish */
+	rgba_char_args_set(btheme->text.syntaxv,    95, 95, 0, 255);    /* Special  Yellow*/
+	rgba_char_args_set(btheme->text.syntaxd,    50, 0, 140, 255);   /* Decorator/Preprocessor Dir.  Blue-purple */
+	rgba_char_args_set(btheme->text.syntaxr,    140, 60, 0, 255);   /* Reserved  Orange*/
+	rgba_char_args_set(btheme->text.syntaxb,    128, 0, 80, 255);   /* Builtin  Red-purple */
+	rgba_char_args_set(btheme->text.syntaxs,    76, 76, 76, 255);   /* Grey (mix between fg/bg) */
 	
 	/* space oops */
 	btheme->toops = btheme->tv3d;
@@ -828,21 +941,23 @@ void ui_theme_init_default(void)
 	rgba_char_args_set(btheme->tconsole.console_info, 0, 170, 0, 255);
 	rgba_char_args_set(btheme->tconsole.console_error, 220, 96, 96, 255);
 	rgba_char_args_set(btheme->tconsole.console_cursor, 220, 96, 96, 255);
+	rgba_char_args_set(btheme->tconsole.console_select, 255, 255, 255, 48);
 	
 	/* space time */
 	btheme->ttime = btheme->tv3d;
 	rgba_char_args_set_fl(btheme->ttime.back,   0.45, 0.45, 0.45, 1.0);
 	rgba_char_args_set_fl(btheme->ttime.grid,   0.36, 0.36, 0.36, 1.0);
-	rgba_char_args_set(btheme->ttime.shade1,  173, 173, 173, 255);      // sliders
+	rgba_char_args_set(btheme->ttime.shade1,  173, 173, 173, 255);      /* sliders */
 	
 	/* space node, re-uses syntax color storage */
 	btheme->tnode = btheme->tv3d;
-	rgba_char_args_set(btheme->tnode.edge_select, 255, 255, 255, 255);
+	rgba_char_args_set(btheme->tnode.edge_select, 255, 255, 255, 255);	/* wire selected */
 	rgba_char_args_set(btheme->tnode.syntaxl, 155, 155, 155, 160);  /* TH_NODE, backdrop */
 	rgba_char_args_set(btheme->tnode.syntaxn, 100, 100, 100, 255);  /* in/output */
 	rgba_char_args_set(btheme->tnode.syntaxb, 108, 105, 111, 255);  /* operator */
 	rgba_char_args_set(btheme->tnode.syntaxv, 104, 106, 117, 255);  /* generator */
 	rgba_char_args_set(btheme->tnode.syntaxc, 105, 117, 110, 255);  /* group */
+	rgba_char_args_set(btheme->tnode.movie, 155, 155, 155, 160);  /* frame */
 	btheme->tnode.noodle_curving = 5;
 
 	/* space logic */
@@ -864,31 +979,34 @@ void ui_theme_init_default(void)
 	rgba_char_args_set(btheme->tclip.cframe, 0x60, 0xc0, 0x40, 255);
 	rgba_char_args_set(btheme->tclip.handle_vertex, 0x00, 0x00, 0x00, 0xff);
 	rgba_char_args_set(btheme->tclip.handle_vertex_select, 0xff, 0xff, 0, 0xff);
+	rgba_char_args_set(btheme->tclip.list, 0x66, 0x66, 0x66, 0xff);
+	rgba_char_args_set(btheme->tclip.strip, 0x0c, 0x0a, 0x0a, 0x80);
+	rgba_char_args_set(btheme->tclip.strip_select, 0xff, 0x8c, 0x00, 0xff);
 	btheme->tclip.handle_vertex_size = 4;
 }
 
 
 void UI_SetTheme(int spacetype, int regionid)
 {
-	if (spacetype == 0) { // called for safety, when delete themes
+	if (spacetype == 0) {  /* called for safety, when delete themes */
 		theme_active = U.themes.first;
 		theme_spacetype = SPACE_VIEW3D;
 		theme_regionid = RGN_TYPE_WINDOW;
 	}
 	else {
-		// later on, a local theme can be found too
+		/* later on, a local theme can be found too */
 		theme_active = U.themes.first;
 		theme_spacetype = spacetype;
 		theme_regionid = regionid;
 	}
 }
 
-bTheme *UI_GetTheme()
+bTheme *UI_GetTheme(void)
 {
 	return U.themes.first;
 }
 
-// for space windows only
+/* for space windows only */
 void UI_ThemeColor(int colorid)
 {
 	const unsigned char *cp;
@@ -898,7 +1016,7 @@ void UI_ThemeColor(int colorid)
 
 }
 
-// plus alpha
+/* plus alpha */
 void UI_ThemeColor4(int colorid)
 {
 	const unsigned char *cp;
@@ -908,7 +1026,7 @@ void UI_ThemeColor4(int colorid)
 
 }
 
-// set the color with offset for shades
+/* set the color with offset for shades */
 void UI_ThemeColorShade(int colorid, int offset)
 {
 	int r, g, b;
@@ -941,7 +1059,7 @@ void UI_ThemeColorShadeAlpha(int colorid, int coloffset, int alphaoffset)
 	glColor4ub(r, g, b, a);
 }
 
-// blend between to theme colors, and set it
+/* blend between to theme colors, and set it */
 void UI_ThemeColorBlend(int colorid1, int colorid2, float fac)
 {
 	int r, g, b;
@@ -958,7 +1076,7 @@ void UI_ThemeColorBlend(int colorid1, int colorid2, float fac)
 	glColor3ub(r, g, b);
 }
 
-// blend between to theme colors, shade it, and set it
+/* blend between to theme colors, shade it, and set it */
 void UI_ThemeColorBlendShade(int colorid1, int colorid2, float fac, int offset)
 {
 	int r, g, b;
@@ -979,7 +1097,7 @@ void UI_ThemeColorBlendShade(int colorid1, int colorid2, float fac, int offset)
 	glColor3ub(r, g, b);
 }
 
-// blend between to theme colors, shade it, and set it
+/* blend between to theme colors, shade it, and set it */
 void UI_ThemeColorBlendShadeAlpha(int colorid1, int colorid2, float fac, int offset, int alphaoffset)
 {
 	int r, g, b, a;
@@ -1003,29 +1121,27 @@ void UI_ThemeColorBlendShadeAlpha(int colorid1, int colorid2, float fac, int off
 }
 
 
-// get individual values, not scaled
+/* get individual values, not scaled */
 float UI_GetThemeValuef(int colorid)
 {
 	const unsigned char *cp;
 	
 	cp = UI_ThemeGetColorPtr(theme_active, theme_spacetype, colorid);
 	return ((float)cp[0]);
-
 }
 
-// get individual values, not scaled
+/* get individual values, not scaled */
 int UI_GetThemeValue(int colorid)
 {
 	const unsigned char *cp;
 	
 	cp = UI_ThemeGetColorPtr(theme_active, theme_spacetype, colorid);
 	return ((int) cp[0]);
-
 }
 
 
-// get the color, range 0.0-1.0
-void UI_GetThemeColor3fv(int colorid, float *col)
+/* get the color, range 0.0-1.0 */
+void UI_GetThemeColor3fv(int colorid, float col[3])
 {
 	const unsigned char *cp;
 	
@@ -1035,8 +1151,19 @@ void UI_GetThemeColor3fv(int colorid, float *col)
 	col[2] = ((float)cp[2]) / 255.0f;
 }
 
-// get the color, range 0.0-1.0, complete with shading offset
-void UI_GetThemeColorShade3fv(int colorid, int offset, float *col)
+void UI_GetThemeColor4fv(int colorid, float col[4])
+{
+	const unsigned char *cp;
+	
+	cp = UI_ThemeGetColorPtr(theme_active, theme_spacetype, colorid);
+	col[0] = ((float)cp[0]) / 255.0f;
+	col[1] = ((float)cp[1]) / 255.0f;
+	col[2] = ((float)cp[2]) / 255.0f;
+	col[3] = ((float)cp[3]) / 255.0f;
+}
+
+/* get the color, range 0.0-1.0, complete with shading offset */
+void UI_GetThemeColorShade3fv(int colorid, int offset, float col[3])
 {
 	int r, g, b;
 	const unsigned char *cp;
@@ -1055,7 +1182,26 @@ void UI_GetThemeColorShade3fv(int colorid, int offset, float *col)
 	col[2] = ((float)b) / 255.0f;
 }
 
-// get the color, in char pointer
+void UI_GetThemeColorShade3ubv(int colorid, int offset, unsigned char col[3])
+{
+	int r, g, b;
+	const unsigned char *cp;
+
+	cp = UI_ThemeGetColorPtr(theme_active, theme_spacetype, colorid);
+
+	r = offset + (int) cp[0];
+	CLAMP(r, 0, 255);
+	g = offset + (int) cp[1];
+	CLAMP(g, 0, 255);
+	b = offset + (int) cp[2];
+	CLAMP(b, 0, 255);
+
+	col[0] = r;
+	col[1] = g;
+	col[2] = b;
+}
+
+/* get the color, in char pointer */
 void UI_GetThemeColor3ubv(int colorid, unsigned char col[3])
 {
 	const unsigned char *cp;
@@ -1066,7 +1212,7 @@ void UI_GetThemeColor3ubv(int colorid, unsigned char col[3])
 	col[2] = cp[2];
 }
 
-// get the color, in char pointer
+/* get the color, in char pointer */
 void UI_GetThemeColor4ubv(int colorid, unsigned char col[4])
 {
 	const unsigned char *cp;
@@ -1089,7 +1235,7 @@ void UI_GetThemeColorType4ubv(int colorid, int spacetype, char col[4])
 	col[3] = cp[3];
 }
 
-// blends and shades between two char color pointers
+/* blends and shades between two char color pointers */
 void UI_ColorPtrBlendShade3ubv(const unsigned char cp1[3], const unsigned char cp2[3], float fac, int offset)
 {
 	int r, g, b;
@@ -1122,8 +1268,9 @@ void UI_GetColorPtrShade3ubv(const unsigned char cp[3], unsigned char col[3], in
 	col[2] = b;
 }
 
-// get a 3 byte color, blended and shaded between two other char color pointers
-void UI_GetColorPtrBlendShade3ubv(const unsigned char cp1[3], const unsigned char cp2[3], unsigned char col[3], float fac, int offset)
+/* get a 3 byte color, blended and shaded between two other char color pointers */
+void UI_GetColorPtrBlendShade3ubv(const unsigned char cp1[3], const unsigned char cp2[3], unsigned char col[3],
+                                  float fac, int offset)
 {
 	int r, g, b;
 
@@ -1149,24 +1296,28 @@ void UI_ThemeClearColor(int colorid)
 	glClearColor(col[0], col[1], col[2], 0.0);
 }
 
+int UI_ThemeMenuShadowWidth(void)
+{
+	bTheme *btheme = UI_GetTheme();
+	return (int)(btheme->tui.menu_shadow_width * UI_DPI_FAC);
+}
+
 void UI_make_axis_color(const unsigned char src_col[3], unsigned char dst_col[3], const char axis)
 {
-	switch (axis)
-	{
+	unsigned char col[3];
+	
+	switch (axis) {
 		case 'X':
-			dst_col[0] = src_col[0] > 219 ? 255 : src_col[0] + 36;
-			dst_col[1] = src_col[1] < 26 ? 0 : src_col[1] - 26;
-			dst_col[2] = src_col[2] < 26 ? 0 : src_col[2] - 26;
+			UI_GetThemeColor3ubv(TH_AXIS_X, col);
+			UI_GetColorPtrBlendShade3ubv(src_col, col, dst_col, 0.5f, -10);
 			break;
 		case 'Y':
-			dst_col[0] = src_col[0] < 46 ? 0 : src_col[0] - 36;
-			dst_col[1] = src_col[1] > 189 ? 255 : src_col[1] + 66;
-			dst_col[2] = src_col[2] < 46 ? 0 : src_col[2] - 36;
+			UI_GetThemeColor3ubv(TH_AXIS_Y, col);
+			UI_GetColorPtrBlendShade3ubv(src_col, col, dst_col, 0.5f, -10);
 			break;
 		case 'Z':
-			dst_col[0] = src_col[0] < 26 ? 0 : src_col[0] - 26;
-			dst_col[1] = src_col[1] < 26 ? 0 : src_col[1] - 26;
-			dst_col[2] = src_col[2] > 209 ? 255 : src_col[2] + 46;
+			UI_GetThemeColor3ubv(TH_AXIS_Z, col);
+			UI_GetColorPtrBlendShade3ubv(src_col, col, dst_col, 0.5f, -10);
 			break;
 		default:
 			BLI_assert(!"invalid axis arg");
@@ -1179,7 +1330,6 @@ void UI_make_axis_color(const unsigned char src_col[3], unsigned char dst_col[3]
 void init_userdef_do_versions(void)
 {
 	Main *bmain = G.main;
-//	countall();
 	
 	/* the UserDef struct is not corrected with do_versions() .... ugh! */
 	if (U.wheellinescroll == 0) U.wheellinescroll = 3;
@@ -1206,8 +1356,8 @@ void init_userdef_do_versions(void)
 	/* transform widget settings */
 	if (U.tw_hotspot == 0) {
 		U.tw_hotspot = 14;
-		U.tw_size = 20;          // percentage of window size
-		U.tw_handlesize = 16;    // percentage of widget radius
+		U.tw_size = 20;          /* percentage of window size */
+		U.tw_handlesize = 16;    /* percentage of widget radius */
 	}
 	if (U.pad_rot_angle == 0)
 		U.pad_rot_angle = 15;
@@ -1221,7 +1371,6 @@ void init_userdef_do_versions(void)
 	vDM_ColorBand_store((U.flag & USER_CUSTOM_RANGE) ? (&U.coba_weight) : NULL);
 
 	if (bmain->versionfile <= 191) {
-		BLI_strncpy(U.plugtexdir, U.textudir, sizeof(U.plugtexdir));
 		strcpy(U.sounddir, "/");
 	}
 	
@@ -1238,7 +1387,7 @@ void init_userdef_do_versions(void)
 		if (U.undosteps == 0) U.undosteps = 32;
 		
 		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
-			/* check for alpha==0 is safe, then color was never set */
+			/* check for (alpha == 0) is safe, then color was never set */
 			if (btheme->tv3d.edge_seam[3] == 0) {
 				rgba_char_args_set(btheme->tv3d.edge_seam, 230, 150, 50, 255);
 			}
@@ -1263,13 +1412,13 @@ void init_userdef_do_versions(void)
 		bTheme *btheme;
 		/* new space type */
 		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
-			/* check for alpha==0 is safe, then color was never set */
+			/* check for (alpha == 0) is safe, then color was never set */
 			if (btheme->ttime.back[3] == 0) {
-				// copied from ui_theme_init_default
+				/* copied from ui_theme_init_default */
 				btheme->ttime = btheme->tv3d;
 				rgba_char_args_set_fl(btheme->ttime.back,   0.45, 0.45, 0.45, 1.0);
 				rgba_char_args_set_fl(btheme->ttime.grid,   0.36, 0.36, 0.36, 1.0);
-				rgba_char_args_set(btheme->ttime.shade1,  173, 173, 173, 255);      // sliders
+				rgba_char_args_set(btheme->ttime.shade1,  173, 173, 173, 255);  /* sliders */
 			}
 			if (btheme->text.syntaxn[3] == 0) {
 				rgba_char_args_set(btheme->text.syntaxn,    0, 0, 200, 255);    /* Numbers  Blue*/
@@ -1342,7 +1491,6 @@ void init_userdef_do_versions(void)
 				rgba_char_args_set(btheme->tseq.scene,  78, 152, 62, 255);
 				rgba_char_args_set(btheme->tseq.audio,  46, 143, 143, 255);
 				rgba_char_args_set(btheme->tseq.effect,     169, 84, 124, 255);
-				rgba_char_args_set(btheme->tseq.plugin,     126, 126, 80, 255);
 				rgba_char_args_set(btheme->tseq.transition, 162, 95, 111, 255);
 				rgba_char_args_set(btheme->tseq.meta,   109, 145, 131, 255);
 			}
@@ -1653,7 +1801,8 @@ void init_userdef_do_versions(void)
 	}
 
 	if (bmain->versionfile < 257) {
-		/* clear "AUTOKEY_FLAG_ONLYKEYINGSET" flag from userprefs, so that it doesn't linger around from old configs like a ghost */
+		/* clear "AUTOKEY_FLAG_ONLYKEYINGSET" flag from userprefs,
+		 * so that it doesn't linger around from old configs like a ghost */
 		U.autokey_flag &= ~AUTOKEY_FLAG_ONLYKEYINGSET;
 	}
 
@@ -1744,7 +1893,7 @@ void init_userdef_do_versions(void)
 			rgba_char_args_set_fl(btheme->toops.selected_highlight, 0.51, 0.53, 0.55, 0.3);
 		}
 		
-		U.use_16bit_textures = 1;
+		U.use_16bit_textures = TRUE;
 	}
 
 	if (bmain->versionfile < 262 || (bmain->versionfile == 262 && bmain->subversionfile < 2)) {
@@ -1776,6 +1925,96 @@ void init_userdef_do_versions(void)
 		}
 	}
 
+	if (bmain->versionfile < 263 || (bmain->versionfile == 263 && bmain->subversionfile < 2)) {
+		bTheme *btheme;
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			if (btheme->tclip.strip[0] == 0) {
+				rgba_char_args_set(btheme->tclip.list, 0x66, 0x66, 0x66, 0xff);
+				rgba_char_args_set(btheme->tclip.strip, 0x0c, 0x0a, 0x0a, 0x80);
+				rgba_char_args_set(btheme->tclip.strip_select, 0xff, 0x8c, 0x00, 0xff);
+			}
+		}
+	}
+
+	if (bmain->versionfile < 263 || (bmain->versionfile == 263 && bmain->subversionfile < 6)) {
+		bTheme *btheme;
+		for (btheme = U.themes.first; btheme; btheme = btheme->next)
+			rgba_char_args_set(btheme->tv3d.skin_root, 180, 77, 77, 255);
+	}
+	
+	if (bmain->versionfile < 263 || (bmain->versionfile == 263 && bmain->subversionfile < 7)) {
+		bTheme *btheme;
+		
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			/* DopeSheet Summary */
+			rgba_char_args_set(btheme->tact.anim_active,    204, 112, 26, 102); 
+			
+			/* NLA Colors */
+			rgba_char_args_set(btheme->tnla.anim_active,     204, 112, 26, 102); /* same as dopesheet above */
+			rgba_char_args_set(btheme->tnla.anim_non_active, 153, 135, 97, 77);
+			
+			rgba_char_args_set(btheme->tnla.nla_tweaking,   77, 243, 26, 77);
+			rgba_char_args_set(btheme->tnla.nla_tweakdupli, 217, 0, 0, 255);
+			
+			rgba_char_args_set(btheme->tnla.nla_transition,     28, 38, 48, 255);
+			rgba_char_args_set(btheme->tnla.nla_transition_sel, 46, 117, 219, 255);
+			rgba_char_args_set(btheme->tnla.nla_meta,           51, 38, 66, 255);
+			rgba_char_args_set(btheme->tnla.nla_meta_sel,       105, 33, 150, 255);
+			rgba_char_args_set(btheme->tnla.nla_sound,          43, 61, 61, 255);
+			rgba_char_args_set(btheme->tnla.nla_sound_sel,      31, 122, 122, 255);
+		}
+	}
+
+	if (bmain->versionfile < 263 || (bmain->versionfile == 263 && bmain->subversionfile < 11)) {
+		bTheme *btheme;
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			if (btheme->tseq.movieclip[0] == 0) {
+				rgba_char_args_set(btheme->tseq.mask,  152, 78, 62, 255);
+			}
+		}
+	}
+
+	if (bmain->versionfile < 263 || (bmain->versionfile == 263 && bmain->subversionfile < 15)) {
+		bTheme *btheme;
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			rgba_char_args_set(btheme->tv3d.bone_pose_active, 140, 255, 255, 80);
+		}
+	}
+
+	if (bmain->versionfile < 263 || (bmain->versionfile == 263 && bmain->subversionfile < 16)) {
+		bTheme *btheme;
+
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			if (btheme->tact.anim_active[3] == 0)
+				rgba_char_args_set(btheme->tact.anim_active, 204, 112, 26, 102);
+
+			if (btheme->tnla.anim_active[3] == 0)
+				rgba_char_args_set(btheme->tnla.anim_active, 204, 112, 26, 102);
+		}
+	}
+
+	if (bmain->versionfile < 263 || (bmain->versionfile == 263 && bmain->subversionfile < 22)) {
+		bTheme *btheme;
+
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			if (btheme->tipo.lastsel_point[3] == 0)
+				rgba_char_args_set(btheme->tipo.lastsel_point, 0xff, 0xff, 0xff, 255);
+
+			if (btheme->tv3d.skin_root[3] == 0)
+				rgba_char_args_set(btheme->tv3d.skin_root, 180, 77, 77, 255);
+		}
+	}
+	
+	if (bmain->versionfile < 264 || (bmain->versionfile == 264 && bmain->subversionfile < 9)) {
+		bTheme *btheme;
+		
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			rgba_char_args_set(btheme->tui.xaxis, 220,   0,   0, 255);
+			rgba_char_args_set(btheme->tui.yaxis,   0, 220,   0, 255);
+			rgba_char_args_set(btheme->tui.zaxis,   0,   0, 220, 255);
+		}
+	}
+
 	/* GL Texture Garbage Collection (variable abused above!) */
 	if (U.textimeout == 0) {
 		U.texcollectrate = 60;
@@ -1790,9 +2029,6 @@ void init_userdef_do_versions(void)
 	if (U.dbl_click_time == 0) {
 		U.dbl_click_time = 350;
 	}
-	if (U.anim_player_preset == 0) {
-		U.anim_player_preset = 1;
-	}
 	if (U.scrcastfps == 0) {
 		U.scrcastfps = 10;
 		U.scrcastwait = 50;
@@ -1803,7 +2039,7 @@ void init_userdef_do_versions(void)
 	if (U.dragthreshold == 0)
 		U.dragthreshold = 5;
 	if (U.widget_unit == 0)
-		U.widget_unit = (U.dpi * 20 + 36) / 72;
+		U.widget_unit = 20;
 	if (U.anisotropic_filter <= 0)
 		U.anisotropic_filter = 1;
 
@@ -1812,9 +2048,111 @@ void init_userdef_do_versions(void)
 		U.ndof_flag = NDOF_LOCK_HORIZON |
 		              NDOF_SHOULD_PAN | NDOF_SHOULD_ZOOM | NDOF_SHOULD_ROTATE;
 	}
+	
+	if (U.ndof_orbit_sensitivity == 0.0f) {
+		U.ndof_orbit_sensitivity = U.ndof_sensitivity;
+
+		if (!(U.flag & USER_TRACKBALL))
+			U.ndof_flag |= NDOF_TURNTABLE;
+	}
 	if (U.tweak_threshold == 0)
 		U.tweak_threshold = 10;
 
+	if (bmain->versionfile < 265 || (bmain->versionfile == 265 && bmain->subversionfile < 1)) {
+		bTheme *btheme;
+		
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			/* note: the toggle operator for transparent backdrops limits to these spacetypes */
+			if (btheme->tnode.button[3] == 255) {
+				btheme->tv3d.button[3] = 128;
+				btheme->tnode.button[3] = 128;
+				btheme->tima.button[3] = 128;
+				btheme->tseq.button[3] = 128;
+				btheme->tclip.button[3] = 128;
+			}
+		}
+	}
+	
+	/* panel header/backdrop supported locally per editor now */
+	if (bmain->versionfile < 265 || (bmain->versionfile == 265 && bmain->subversionfile < 2)) {
+		bTheme *btheme;
+		
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			
+			/* new color, panel backdrop. Not used anywhere yet, until you enable it */
+			copy_v3_v3_char(btheme->tui.panel.back, btheme->tbuts.button);
+			btheme->tui.panel.back[3] = 128;
+			
+			btheme->tbuts.panelcolors = btheme->tui.panel;
+			btheme->tv3d.panelcolors = btheme->tui.panel;
+			btheme->tfile.panelcolors = btheme->tui.panel;
+			btheme->tipo.panelcolors = btheme->tui.panel;
+			btheme->tinfo.panelcolors = btheme->tui.panel;
+			btheme->tact.panelcolors = btheme->tui.panel;
+			btheme->tnla.panelcolors = btheme->tui.panel;
+			btheme->tseq.panelcolors = btheme->tui.panel;
+			btheme->tima.panelcolors = btheme->tui.panel;
+			btheme->text.panelcolors = btheme->tui.panel;
+			btheme->toops.panelcolors = btheme->tui.panel;
+			btheme->ttime.panelcolors = btheme->tui.panel;
+			btheme->tnode.panelcolors = btheme->tui.panel;
+			btheme->tlogic.panelcolors = btheme->tui.panel;
+			btheme->tuserpref.panelcolors = btheme->tui.panel;
+			btheme->tconsole.panelcolors = btheme->tui.panel;
+			btheme->tclip.panelcolors = btheme->tui.panel;
+		}
+	}
+
+	/* NOTE!! from now on use U.versionfile and U.subversionfile */
+	if (U.versionfile < 266) {
+		bTheme *btheme;
+		
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			/* rna definition limits fac to 0.01 */
+			if (btheme->tui.menu_shadow_fac == 0.0f) {
+				btheme->tui.menu_shadow_fac = 0.5f;
+				btheme->tui.menu_shadow_width = 12;
+			}
+		}
+	}
+
+	if (U.versionfile < 265 || (U.versionfile == 265 && U.subversionfile < 4)) {
+		bTheme *btheme;
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			rgba_char_args_set(btheme->text.syntaxd,    50, 0, 140, 255);   /* Decorator/Preprocessor Dir.  Blue-purple */
+			rgba_char_args_set(btheme->text.syntaxr,    140, 60, 0, 255);   /* Reserved  Orange */
+			rgba_char_args_set(btheme->text.syntaxs,    76, 76, 76, 255);   /* Grey (mix between fg/bg) */
+		}
+	}
+
+	if (U.versionfile < 265 || (U.versionfile == 265 && U.subversionfile < 6)) {
+		bTheme *btheme;
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			copy_v4_v4_char(btheme->tv3d.gradients.high_gradient, btheme->tv3d.back);
+		}
+	}
+
+	if (U.versionfile < 265 || (U.versionfile == 265 && U.subversionfile < 9)) {
+		bTheme *btheme;
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			rgba_char_args_test_set(btheme->tnode.syntaxs, 151, 116, 116, 255);  /* matte nodes */
+			rgba_char_args_test_set(btheme->tnode.syntaxd, 116, 151, 151, 255);  /* distort nodes */
+		}
+	}
+
+	if (U.versionfile < 265 || (U.versionfile == 265 && U.subversionfile < 11)) {
+		bTheme *btheme;
+		for (btheme = U.themes.first; btheme; btheme = btheme->next) {
+			rgba_char_args_test_set(btheme->tconsole.console_select, 255, 255, 255, 48);
+		}
+	}
+
+	/* NOTE!! from now on use U.versionfile and U.subversionfile */
+	
+	
+	if (U.pixelsize == 0.0f)
+		U.pixelsize = 1.0f;
+	
 	/* funny name, but it is GE stuff, moves userdef stuff to engine */
 // XXX	space_set_commmandline_options();
 	/* this timer uses U */
