@@ -27,23 +27,20 @@
  *  \ingroup edmesh
  */
 
-#include <math.h>
-#include <string.h>
 
 #include "MEM_guardedalloc.h"
 
-#include "BLI_utildefines.h"
 #include "BLI_blenlib.h"
 #include "BLI_math.h"
 #include "BLI_edgehash.h"
 
+#include "BLF_translation.h"
+
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
 
 #include "DNA_mesh_types.h"
-#include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
-#include "DNA_scene_types.h"
 
 #include "BKE_DerivedMesh.h"
 #include "BKE_global.h"
@@ -61,13 +58,12 @@
 #include "WM_types.h"
 
 /* own include */
-#include "mesh_intern.h"
 
 /* copy the face flags, most importantly selection from the mesh to the final derived mesh,
  * use in object mode when selecting faces (while painting) */
 void paintface_flush_flags(Object *ob)
 {
-	Mesh *me = get_mesh(ob);
+	Mesh *me = BKE_mesh_from_object(ob);
 	DerivedMesh *dm = ob->derivedFinal;
 	MPoly *polys, *mp_orig;
 	MFace *faces;
@@ -75,7 +71,14 @@ void paintface_flush_flags(Object *ob)
 	int totface, totpoly;
 	int i;
 	
-	if (me == NULL || dm == NULL)
+	if (me == NULL)
+		return;
+
+	/* we could call this directly in all areas that change selection,
+	 * since this could become slow for realtime updates (circle-select for eg) */
+	BKE_mesh_flush_select_from_polys(me);
+
+	if (dm == NULL)
 		return;
 
 	/*
@@ -85,7 +88,7 @@ void paintface_flush_flags(Object *ob)
 	 *  - Final derived polys => Final derived tessfaces
 	 */
 
-	if ((index_array = CustomData_get_layer(&me->fdata, CD_POLYINDEX))) {
+	if ((index_array = CustomData_get_layer(&me->fdata, CD_ORIGINDEX))) {
 		faces = me->mface;
 		totface = me->totface;
 		
@@ -109,7 +112,7 @@ void paintface_flush_flags(Object *ob)
 		}
 	}
 
-	if ((index_array = CustomData_get_layer(&dm->faceData, CD_POLYINDEX))) {
+	if ((index_array = CustomData_get_layer(&dm->faceData, CD_ORIGINDEX))) {
 		polys = dm->getPolyArray(dm);
 		faces = dm->getTessFaceArray(dm);
 		totface = dm->getNumTessFaces(dm);
@@ -123,50 +126,13 @@ void paintface_flush_flags(Object *ob)
 	}
 }
 
-/* returns 0 if not found, otherwise 1 */
-static int facesel_face_pick(struct bContext *C, Mesh *me, Object *ob, const int mval[2], unsigned int *index, short rect)
-{
-	Scene *scene = CTX_data_scene(C);
-	ViewContext vc;
-	view3d_set_viewcontext(C, &vc);
-
-	if (!me || me->totpoly == 0)
-		return 0;
-
-	makeDerivedMesh(scene, ob, NULL, CD_MASK_BAREMESH, 0);
-
-	// XXX  if (v3d->flag & V3D_INVALID_BACKBUF) {
-// XXX drawview.c!		check_backbuf();
-// XXX		persp(PERSP_VIEW);
-// XXX  }
-
-	if (rect) {
-		/* sample rect to increase changes of selecting, so that when clicking
-		 * on an edge in the backbuf, we can still select a face */
-
-		int dist;
-		*index = view3d_sample_backbuf_rect(&vc, mval, 3, 1, me->totpoly + 1, &dist, 0, NULL, NULL);
-	}
-	else {
-		/* sample only on the exact position */
-		*index = view3d_sample_backbuf(&vc, mval[0], mval[1]);
-	}
-
-	if ((*index) <= 0 || (*index) > (unsigned int)me->totpoly)
-		return 0;
-
-	(*index)--;
-	
-	return 1;
-}
-
 void paintface_hide(Object *ob, const int unselected)
 {
 	Mesh *me;
 	MPoly *mpoly;
 	int a;
 	
-	me = get_mesh(ob);
+	me = BKE_mesh_from_object(ob);
 	if (me == NULL || me->totpoly == 0) return;
 
 	mpoly = me->mpoly;
@@ -174,10 +140,10 @@ void paintface_hide(Object *ob, const int unselected)
 	while (a--) {
 		if ((mpoly->flag & ME_HIDE) == 0) {
 			if (unselected) {
-				if ( (mpoly->flag & ME_FACE_SEL) == 0) mpoly->flag |= ME_HIDE;
+				if ((mpoly->flag & ME_FACE_SEL) == 0) mpoly->flag |= ME_HIDE;
 			}
 			else {
-				if ( (mpoly->flag & ME_FACE_SEL)) mpoly->flag |= ME_HIDE;
+				if ((mpoly->flag & ME_FACE_SEL)) mpoly->flag |= ME_HIDE;
 			}
 		}
 		if (mpoly->flag & ME_HIDE) mpoly->flag &= ~ME_FACE_SEL;
@@ -195,7 +161,7 @@ void paintface_reveal(Object *ob)
 	MPoly *mpoly;
 	int a;
 
-	me = get_mesh(ob);
+	me = BKE_mesh_from_object(ob);
 	if (me == NULL || me->totpoly == 0) return;
 
 	mpoly = me->mpoly;
@@ -215,11 +181,17 @@ void paintface_reveal(Object *ob)
 
 static void hash_add_face(EdgeHash *ehash, MPoly *mp, MLoop *mloop)
 {
-	MLoop *ml;
-	int i;
+	MLoop *ml, *ml_next;
+	int i = mp->totloop;
 
-	for (i = 0, ml = mloop; i < mp->totloop; i++, ml++) {
-		BLI_edgehash_insert(ehash, ml->v, ME_POLY_LOOP_NEXT(mloop, mp, i)->v, NULL);
+	ml_next = mloop;       /* first loop */
+	ml = &ml_next[i - 1];  /* last loop */
+
+	while (i-- != 0) {
+		BLI_edgehash_insert(ehash, ml->v, ml_next->v, NULL);
+
+		ml = ml_next;
+		ml_next++;
 	}
 }
 
@@ -231,7 +203,7 @@ static void select_linked_tfaces_with_seams(int mode, Mesh *me, unsigned int ind
 	MLoop *ml;
 	MEdge *med;
 	char *linkflag;
-	int a, b, doit = 1, mark = 0;
+	int a, b, do_it = TRUE, mark = 0;
 
 	ehash = BLI_edgehash_new();
 	seamhash = BLI_edgehash_new();
@@ -251,7 +223,9 @@ static void select_linked_tfaces_with_seams(int mode, Mesh *me, unsigned int ind
 		/* fill array by selection */
 		mp = me->mpoly;
 		for (a = 0; a < me->totpoly; a++, mp++) {
-			if (mp->flag & ME_HIDE) ;
+			if (mp->flag & ME_HIDE) {
+				/* pass */
+			}
 			else if (mp->flag & ME_FACE_SEL) {
 				hash_add_face(ehash, mp, me->mloop + mp->loopstart);
 				linkflag[a] = 1;
@@ -259,8 +233,8 @@ static void select_linked_tfaces_with_seams(int mode, Mesh *me, unsigned int ind
 		}
 	}
 
-	while (doit) {
-		doit = 0;
+	while (do_it) {
+		do_it = FALSE;
 
 		/* expand selection */
 		mp = me->mpoly;
@@ -283,7 +257,7 @@ static void select_linked_tfaces_with_seams(int mode, Mesh *me, unsigned int ind
 				if (mark) {
 					linkflag[a] = 1;
 					hash_add_face(ehash, mp, me->mloop + mp->loopstart);
-					doit = 1;
+					do_it = TRUE;
 				}
 			}
 		}
@@ -325,13 +299,13 @@ void paintface_select_linked(bContext *UNUSED(C), Object *ob, int UNUSED(mval[2]
 	Mesh *me;
 	unsigned int index = 0;
 
-	me = get_mesh(ob);
+	me = BKE_mesh_from_object(ob);
 	if (me == NULL || me->totpoly == 0) return;
 
 	if (mode == 0 || mode == 1) {
-		// XXX - Causes glitches, not sure why
+		/* XXX - Causes glitches, not sure why */
 #if 0
-		if (!facesel_face_pick(C, me, mval, &index, 1))
+		if (!ED_mesh_pick_face(C, me, mval, &index, ED_MESH_PICK_DEFAULT_FACE_SIZE))
 			return;
 #endif
 	}
@@ -347,7 +321,7 @@ void paintface_deselect_all_visible(Object *ob, int action, short flush_flags)
 	MPoly *mpoly;
 	int a;
 
-	me = get_mesh(ob);
+	me = BKE_mesh_from_object(ob);
 	if (me == NULL) return;
 	
 	if (action == SEL_INVERT) {
@@ -400,17 +374,17 @@ void paintface_deselect_all_visible(Object *ob, int action, short flush_flags)
 	}
 }
 
-int paintface_minmax(Object *ob, float *min, float *max)
+int paintface_minmax(Object *ob, float r_min[3], float r_max[3])
 {
 	Mesh *me;
 	MPoly *mp;
 	MTexPoly *tf;
 	MLoop *ml;
 	MVert *mvert;
-	int a, b, ok = 0;
+	int a, b, ok = FALSE;
 	float vec[3], bmat[3][3];
 
-	me = get_mesh(ob);
+	me = BKE_mesh_from_object(ob);
 	if (!me || !me->mtpoly) return ok;
 	
 	copy_m3_m4(bmat, ob->obmat);
@@ -427,10 +401,10 @@ int paintface_minmax(Object *ob, float *min, float *max)
 			copy_v3_v3(vec, (mvert[ml->v].co));
 			mul_m3_v3(bmat, vec);
 			add_v3_v3v3(vec, vec, ob->obmat[3]);
-			DO_MINMAX(vec, min, max);		
+			minmax_v3v3_v3(r_min, r_max, vec);
 		}
 
-		ok = 1;
+		ok = TRUE;
 	}
 
 	return ok;
@@ -459,11 +433,11 @@ void seam_mark_clear_tface(Scene *scene, short mode)
 	MEdge *med;
 	int a, b;
 	
-	me = get_mesh(OBACT);
+	me = BKE_mesh_from_object(OBACT);
 	if (me == 0 ||  me->totpoly == 0) return;
 
 	if (mode == 0)
-		mode = pupmenu("Seams%t|Mark Border Seam %x1|Clear Seam %x2");
+		mode = pupmenu(IFACE_("Seams %t|Mark Border Seam %x1|Clear Seam %x2"));
 
 	if (mode != 1 && mode != 2)
 		return;
@@ -502,26 +476,26 @@ void seam_mark_clear_tface(Scene *scene, short mode)
 		BLI_edgehash_free(ehash2, NULL);
 	}
 
-// XXX	if (G.rt == 8)
+// XXX	if (G.debug_value == 8)
 //		unwrap_lscm(1);
 
 	me->drawflag |= ME_DRAWSEAMS;
 }
 #endif
 
-int paintface_mouse_select(struct bContext *C, Object *ob, const int mval[2], int extend)
+int paintface_mouse_select(struct bContext *C, Object *ob, const int mval[2], int extend, int deselect, int toggle)
 {
 	Mesh *me;
 	MPoly *mpoly, *mpoly_sel;
 	unsigned int a, index;
 	
 	/* Get the face under the cursor */
-	me = get_mesh(ob);
+	me = BKE_mesh_from_object(ob);
 
-	if (!facesel_face_pick(C, me, ob, mval, &index, 1))
+	if (!ED_mesh_pick_face(C, ob, mval, &index, ED_MESH_PICK_DEFAULT_FACE_SIZE))
 		return 0;
 	
-	if (index >= me->totpoly || index < 0)
+	if (index >= me->totpoly)
 		return 0;
 
 	mpoly_sel = me->mpoly + index;
@@ -530,7 +504,7 @@ int paintface_mouse_select(struct bContext *C, Object *ob, const int mval[2], in
 	/* clear flags */
 	mpoly = me->mpoly;
 	a = me->totpoly;
-	if (!extend) {
+	if (!extend && !deselect && !toggle) {
 		while (a--) {
 			mpoly->flag &= ~ME_FACE_SEL;
 			mpoly++;
@@ -540,6 +514,12 @@ int paintface_mouse_select(struct bContext *C, Object *ob, const int mval[2], in
 	me->act_face = (int)index;
 
 	if (extend) {
+		mpoly_sel->flag |= ME_FACE_SEL;
+	}
+	else if (deselect) {
+		mpoly_sel->flag &= ~ME_FACE_SEL;
+	}
+	else if (toggle) {
 		if (mpoly_sel->flag & ME_FACE_SEL)
 			mpoly_sel->flag &= ~ME_FACE_SEL;
 		else
@@ -564,10 +544,10 @@ int do_paintface_box_select(ViewContext *vc, rcti *rect, int select, int extend)
 	unsigned int *rt;
 	char *selar;
 	int a, index;
-	int sx = rect->xmax - rect->xmin + 1;
-	int sy = rect->ymax - rect->ymin + 1;
+	int sx = BLI_rcti_size_x(rect) + 1;
+	int sy = BLI_rcti_size_y(rect) + 1;
 	
-	me = get_mesh(ob);
+	me = BKE_mesh_from_object(ob);
 
 	if (me == NULL || me->totpoly == 0 || sx * sy <= 0)
 		return OPERATOR_CANCELLED;
@@ -603,7 +583,9 @@ int do_paintface_box_select(ViewContext *vc, rcti *rect, int select, int extend)
 	mpoly = me->mpoly;
 	for (a = 1; a <= me->totpoly; a++, mpoly++) {
 		if (selar[a]) {
-			if (mpoly->flag & ME_HIDE) ;
+			if (mpoly->flag & ME_HIDE) {
+				/* pass */
+			}
 			else {
 				if (select) mpoly->flag |= ME_FACE_SEL;
 				else mpoly->flag &= ~ME_FACE_SEL;
@@ -629,14 +611,21 @@ int do_paintface_box_select(ViewContext *vc, rcti *rect, int select, int extend)
  * use in object mode when selecting vertices (while painting) */
 void paintvert_flush_flags(Object *ob)
 {
-	Mesh *me = get_mesh(ob);
+	Mesh *me = BKE_mesh_from_object(ob);
 	DerivedMesh *dm = ob->derivedFinal;
 	MVert *dm_mvert, *dm_mv;
 	int *index_array = NULL;
 	int totvert;
 	int i;
 
-	if (me == NULL || dm == NULL)
+	if (me == NULL)
+		return;
+
+	/* we could call this directly in all areas that change selection,
+	 * since this could become slow for realtime updates (circle-select for eg) */
+	BKE_mesh_flush_select_from_verts(me);
+
+	if (dm == NULL)
 		return;
 
 	index_array = dm->getVertDataArray(dm, CD_ORIGINDEX);
@@ -668,7 +657,7 @@ void paintvert_deselect_all_visible(Object *ob, int action, short flush_flags)
 	MVert *mvert;
 	int a;
 
-	me = get_mesh(ob);
+	me = BKE_mesh_from_object(ob);
 	if (me == NULL) return;
 	
 	if (action == SEL_INVERT) {
@@ -860,7 +849,7 @@ void ED_mesh_mirrtopo_init(Mesh *me, const int ob_mode, MirrTopoStore_t *mesh_to
 		}
 
 		if (tot_unique <= tot_unique_prev) {
-			/* Finish searching for unique valus when 1 loop dosnt give a
+			/* Finish searching for unique values when 1 loop dosnt give a
 			 * higher number of unique values compared to the previous loop */
 			break;
 		}
@@ -879,7 +868,7 @@ void ED_mesh_mirrtopo_init(Mesh *me, const int ob_mode, MirrTopoStore_t *mesh_to
 
 	if (em) {
 		if (skip_em_vert_array_init == FALSE) {
-			EDBM_index_arrays_init(em, 1, 0, 0);
+			EDBM_index_arrays_ensure(em, BM_VERT);
 		}
 	}
 
@@ -915,11 +904,6 @@ void ED_mesh_mirrtopo_init(Mesh *me, const int ob_mode, MirrTopoStore_t *mesh_to
 			last = a;
 		}
 	}
-	if (em) {
-		if (skip_em_vert_array_init == FALSE) {
-			EDBM_index_arrays_free(em);
-		}
-	}
 
 	MEM_freeN(topo_pairs);
 	topo_pairs = NULL;
