@@ -29,7 +29,6 @@
  *  \ingroup edmesh
  */
 
-#include "DNA_mesh_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 
@@ -39,7 +38,6 @@
 #include "BLI_math.h"
 
 #include "BKE_context.h"
-#include "BKE_depsgraph.h"
 #include "BKE_library.h"
 #include "BKE_tessmesh.h"
 
@@ -53,46 +51,20 @@
 
 #include "mesh_intern.h"
 
-/* uses context to figure out transform for primitive */
-/* returns standard diameter */
-static float new_primitive_matrix(bContext *C, float *loc, float *rot, float primmat[][4])
-{
-	Object *obedit = CTX_data_edit_object(C);
-	View3D *v3d = CTX_wm_view3d(C);
-	float mat[3][3], rmat[3][3], cmat[3][3], imat[3][3];
-	
-	unit_m4(primmat);
-
-	eul_to_mat3(rmat, rot);
-	invert_m3(rmat);
-	
-	/* inverse transform for initial rotation and object */
-	copy_m3_m4(mat, obedit->obmat);
-	mul_m3_m3m3(cmat, rmat, mat);
-	invert_m3_m3(imat, cmat);
-	copy_m4_m3(primmat, imat);
-
-	/* center */
-	copy_v3_v3(primmat[3], loc);
-	sub_v3_v3(primmat[3], obedit->obmat[3]);
-	invert_m3_m3(imat, mat);
-	mul_m3_v3(imat, primmat[3]);
-
-	return v3d ? v3d->grid : 1.0f;
-}
-
 /* ********* add primitive operators ************* */
 
-static void make_prim_init(bContext *C, const char *idname,
-                           float *dia, float mat[][4],
-                           int *state, float *loc, float *rot, unsigned int layer)
+/* BMESH_TODO: 'state' is not a good name, should be flipped and called 'was_editmode',
+ * or at least something more descriptive */
+static Object *make_prim_init(bContext *C, const char *idname,
+                              float *dia, float mat[4][4],
+                              int *state, const float loc[3], const float rot[3], const unsigned int layer)
 {
 	Object *obedit = CTX_data_edit_object(C);
 
 	*state = 0;
 	if (obedit == NULL || obedit->type != OB_MESH) {
 		obedit = ED_object_add_type(C, OB_MESH, loc, rot, FALSE, layer);
-		
+
 		rename_id((ID *)obedit, idname);
 		rename_id((ID *)obedit->data, idname);
 
@@ -101,22 +73,25 @@ static void make_prim_init(bContext *C, const char *idname,
 		*state = 1;
 	}
 
-	*dia = new_primitive_matrix(C, loc, rot, mat);
+	*dia = ED_object_new_primitive_matrix(C, obedit, loc, rot, mat, FALSE);
+
+	return obedit;
 }
 
-static void make_prim_finish(bContext *C, int *state, int enter_editmode)
+static void make_prim_finish(bContext *C, Object *obedit, int *state, int enter_editmode)
 {
-	Object *obedit = CTX_data_edit_object(C);
 	BMEditMesh *em = BMEdit_FromObject(obedit);
+	const int exit_editmode = (*state && !enter_editmode);
 
 	/* Primitive has all verts selected, use vert select flush
 	 * to push this up to edges & faces. */
 	EDBM_selectmode_flush_ex(em, SCE_SELECT_VERTEX);
 
-	EDBM_update_generic(C, em, TRUE);
+	/* only recalc editmode tessface if we are staying in editmode */
+	EDBM_update_generic(em, !exit_editmode, TRUE);
 
 	/* userdef */
-	if (*state && !enter_editmode) {
+	if (exit_editmode) {
 		ED_object_exit_editmode(C, EM_FREEDATA); /* adding EM_DO_UNDO messes up operator redo */
 	}
 	WM_event_add_notifier(C, NC_OBJECT | ND_DRAW, obedit);
@@ -125,27 +100,23 @@ static void make_prim_finish(bContext *C, int *state, int enter_editmode)
 static int add_primitive_plane_exec(bContext *C, wmOperator *op)
 {
 	Object *obedit;
-	Mesh *me;
 	BMEditMesh *em;
 	float loc[3], rot[3], mat[4][4], dia;
 	int enter_editmode;
 	int state;
 	unsigned int layer;
-	
+
 	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, NULL);
-	make_prim_init(C, "Plane", &dia, mat, &state, loc, rot, layer);
+	obedit = make_prim_init(C, "Plane", &dia, mat, &state, loc, rot, layer);
+	em = BMEdit_FromObject(obedit);
 
-	obedit = CTX_data_edit_object(C);
-	me = obedit->data;
-	em = me->edit_btmesh;
-
-	if (!EDBM_op_call_and_selectf(em, op, "vertout",
-	                              "create_grid xsegments=%i ysegments=%i size=%f mat=%m4", 1, 1, dia, mat))
+	if (!EDBM_op_call_and_selectf(em, op, "verts.out",
+	                              "create_grid x_segments=%i y_segments=%i size=%f matrix=%m4", 1, 1, dia, mat))
 	{
 		return OPERATOR_CANCELLED;
 	}
 
-	make_prim_finish(C, &state, enter_editmode);
+	make_prim_finish(C, obedit, &state, enter_editmode);
 
 	return OPERATOR_FINISHED;
 }
@@ -156,12 +127,11 @@ void MESH_OT_primitive_plane_add(wmOperatorType *ot)
 	ot->name = "Add Plane";
 	ot->description = "Construct a filled planar mesh with 4 vertices";
 	ot->idname = "MESH_OT_primitive_plane_add";
-	
+
 	/* api callbacks */
-	ot->invoke = ED_object_add_generic_invoke;
 	ot->exec = add_primitive_plane_exec;
 	ot->poll = ED_operator_scene_editable;
-	
+
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
@@ -171,26 +141,22 @@ void MESH_OT_primitive_plane_add(wmOperatorType *ot)
 static int add_primitive_cube_exec(bContext *C, wmOperator *op)
 {
 	Object *obedit;
-	Mesh *me;
 	BMEditMesh *em;
 	float loc[3], rot[3], mat[4][4], dia;
 	int enter_editmode;
 	int state;
 	unsigned int layer;
-	
+
 	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, NULL);
-	make_prim_init(C, "Cube", &dia, mat, &state, loc, rot, layer);
+	obedit = make_prim_init(C, "Cube", &dia, mat, &state, loc, rot, layer);
+	em = BMEdit_FromObject(obedit);
 
-	obedit = CTX_data_edit_object(C);
-	me = obedit->data;
-	em = me->edit_btmesh;
-
-	if (!EDBM_op_call_and_selectf(em, op, "vertout", "create_cube mat=%m4 size=%f", mat, dia * 2.0f)) {
+	if (!EDBM_op_call_and_selectf(em, op, "verts.out", "create_cube matrix=%m4 size=%f", mat, dia * 2.0f)) {
 		return OPERATOR_CANCELLED;
 	}
-	
+
 	/* BMESH_TODO make plane side this: M_SQRT2 - plane (diameter of 1.41 makes it unit size) */
-	make_prim_finish(C, &state, enter_editmode);
+	make_prim_finish(C, obedit, &state, enter_editmode);
 
 	return OPERATOR_FINISHED;
 }
@@ -201,12 +167,11 @@ void MESH_OT_primitive_cube_add(wmOperatorType *ot)
 	ot->name = "Add Cube";
 	ot->description = "Construct a cube mesh";
 	ot->idname = "MESH_OT_primitive_cube_add";
-	
+
 	/* api callbacks */
-	ot->invoke = ED_object_add_generic_invoke;
 	ot->exec = add_primitive_cube_exec;
 	ot->poll = ED_operator_scene_editable;
-	
+
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
@@ -222,33 +187,29 @@ static const EnumPropertyItem fill_type_items[] = {
 static int add_primitive_circle_exec(bContext *C, wmOperator *op)
 {
 	Object *obedit;
-	Mesh *me;
 	BMEditMesh *em;
 	float loc[3], rot[3], mat[4][4], dia;
 	int enter_editmode;
 	int state, cap_end, cap_tri;
 	unsigned int layer;
-	
+
 	cap_end = RNA_enum_get(op->ptr, "fill_type");
 	cap_tri = (cap_end == 2);
-	
+
 	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, NULL);
-	make_prim_init(C, "Circle", &dia, mat, &state, loc, rot, layer);
+	obedit = make_prim_init(C, "Circle", &dia, mat, &state, loc, rot, layer);
+	em = BMEdit_FromObject(obedit);
 
-	obedit = CTX_data_edit_object(C);
-	me = obedit->data;
-	em = me->edit_btmesh;
-
-	if (!EDBM_op_call_and_selectf(em, op, "vertout",
-	                              "create_circle segments=%i diameter=%f cap_ends=%b cap_tris=%b mat=%m4",
+	if (!EDBM_op_call_and_selectf(em, op, "verts.out",
+	                              "create_circle segments=%i diameter=%f cap_ends=%b cap_tris=%b matrix=%m4",
 	                              RNA_int_get(op->ptr, "vertices"), RNA_float_get(op->ptr, "radius"),
 	                              cap_end, cap_tri, mat))
 	{
 		return OPERATOR_CANCELLED;
 	}
-	
-	make_prim_finish(C, &state, enter_editmode);
-	
+
+	make_prim_finish(C, obedit, &state, enter_editmode);
+
 	return OPERATOR_FINISHED;
 }
 
@@ -260,15 +221,15 @@ void MESH_OT_primitive_circle_add(wmOperatorType *ot)
 	ot->name = "Add Circle";
 	ot->description = "Construct a circle mesh";
 	ot->idname = "MESH_OT_primitive_circle_add";
-	
+
 	/* api callbacks */
-	ot->invoke = ED_object_add_generic_invoke;
+	ot->invoke = WM_operator_view3d_distance_invoke;
 	ot->exec = add_primitive_circle_exec;
 	ot->poll = ED_operator_scene_editable;
-	
+
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
-	
+
 	/* props */
 	RNA_def_int(ot->srna, "vertices", 32, 3, INT_MAX, "Vertices", "", 3, 500);
 	prop = RNA_def_float(ot->srna, "radius", 1.0f, 0.0, FLT_MAX, "Radius", "", 0.001, 100.00);
@@ -281,26 +242,22 @@ void MESH_OT_primitive_circle_add(wmOperatorType *ot)
 static int add_primitive_cylinder_exec(bContext *C, wmOperator *op)
 {
 	Object *obedit;
-	Mesh *me;
 	BMEditMesh *em;
 	float loc[3], rot[3], mat[4][4], dia;
 	int enter_editmode;
 	int state, cap_end, cap_tri;
 	unsigned int layer;
-	
+
 	cap_end = RNA_enum_get(op->ptr, "end_fill_type");
 	cap_tri = (cap_end == 2);
-	
-	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, NULL);
-	make_prim_init(C, "Cylinder", &dia, mat, &state, loc, rot, layer);
 
-	obedit = CTX_data_edit_object(C);
-	me = obedit->data;
-	em = me->edit_btmesh;
+	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, NULL);
+	obedit = make_prim_init(C, "Cylinder", &dia, mat, &state, loc, rot, layer);
+	em = BMEdit_FromObject(obedit);
 
 	if (!EDBM_op_call_and_selectf(
-	        em, op, "vertout",
-	        "create_cone segments=%i diameter1=%f diameter2=%f cap_ends=%b cap_tris=%b depth=%f mat=%m4",
+	        em, op, "verts.out",
+	        "create_cone segments=%i diameter1=%f diameter2=%f cap_ends=%b cap_tris=%b depth=%f matrix=%m4",
 	        RNA_int_get(op->ptr, "vertices"),
 	        RNA_float_get(op->ptr, "radius"),
 	        RNA_float_get(op->ptr, "radius"),
@@ -309,9 +266,9 @@ static int add_primitive_cylinder_exec(bContext *C, wmOperator *op)
 	{
 		return OPERATOR_CANCELLED;
 	}
-	
-	make_prim_finish(C, &state, enter_editmode);
-	
+
+	make_prim_finish(C, obedit, &state, enter_editmode);
+
 	return OPERATOR_FINISHED;
 }
 
@@ -323,15 +280,15 @@ void MESH_OT_primitive_cylinder_add(wmOperatorType *ot)
 	ot->name = "Add Cylinder";
 	ot->description = "Construct a cylinder mesh";
 	ot->idname = "MESH_OT_primitive_cylinder_add";
-	
+
 	/* api callbacks */
-	ot->invoke = ED_object_add_generic_invoke;
+	ot->invoke = WM_operator_view3d_distance_invoke;
 	ot->exec = add_primitive_cylinder_exec;
 	ot->poll = ED_operator_scene_editable;
-	
+
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
-	
+
 	/* props */
 	RNA_def_int(ot->srna, "vertices", 32, 3, INT_MAX, "Vertices", "", 3, 500);
 	prop = RNA_def_float(ot->srna, "radius", 1.0f, 0.0, FLT_MAX, "Radius", "", 0.001, 100.00);
@@ -346,33 +303,29 @@ void MESH_OT_primitive_cylinder_add(wmOperatorType *ot)
 static int add_primitive_cone_exec(bContext *C, wmOperator *op)
 {
 	Object *obedit;
-	Mesh *me;
 	BMEditMesh *em;
 	float loc[3], rot[3], mat[4][4], dia;
 	int enter_editmode;
 	int state, cap_end, cap_tri;
 	unsigned int layer;
-	
+
 	cap_end = RNA_enum_get(op->ptr, "end_fill_type");
 	cap_tri = (cap_end == 2);
-	
-	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, NULL);
-	make_prim_init(C, "Cone", &dia, mat, &state, loc, rot, layer);
 
-	obedit = CTX_data_edit_object(C);
-	me = obedit->data;
-	em = me->edit_btmesh;
+	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, NULL);
+	obedit = make_prim_init(C, "Cone", &dia, mat, &state, loc, rot, layer);
+	em = BMEdit_FromObject(obedit);
 
 	if (!EDBM_op_call_and_selectf(
-	        em, op, "vertout",
-	        "create_cone segments=%i diameter1=%f diameter2=%f cap_ends=%b cap_tris=%b depth=%f mat=%m4",
+	        em, op, "verts.out",
+	        "create_cone segments=%i diameter1=%f diameter2=%f cap_ends=%b cap_tris=%b depth=%f matrix=%m4",
 	        RNA_int_get(op->ptr, "vertices"), RNA_float_get(op->ptr, "radius1"),
 	        RNA_float_get(op->ptr, "radius2"), cap_end, cap_tri, RNA_float_get(op->ptr, "depth"), mat))
 	{
 		return OPERATOR_CANCELLED;
 	}
-	
-	make_prim_finish(C, &state, enter_editmode);
+
+	make_prim_finish(C, obedit, &state, enter_editmode);
 
 	return OPERATOR_FINISHED;
 }
@@ -385,15 +338,15 @@ void MESH_OT_primitive_cone_add(wmOperatorType *ot)
 	ot->name = "Add Cone";
 	ot->description = "Construct a conic mesh";
 	ot->idname = "MESH_OT_primitive_cone_add";
-	
+
 	/* api callbacks */
-	ot->invoke = ED_object_add_generic_invoke;
+	ot->invoke = WM_operator_view3d_distance_invoke;
 	ot->exec = add_primitive_cone_exec;
 	ot->poll = ED_operator_scene_editable;
-	
+
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
-	
+
 	/* props */
 	RNA_def_int(ot->srna, "vertices", 32, 3, INT_MAX, "Vertices", "", 3, 500);
 	prop = RNA_def_float(ot->srna, "radius1", 1.0f, 0.0, FLT_MAX, "Radius 1", "", 0.001, 100.00);
@@ -410,30 +363,27 @@ void MESH_OT_primitive_cone_add(wmOperatorType *ot)
 static int add_primitive_grid_exec(bContext *C, wmOperator *op)
 {
 	Object *obedit;
-	Mesh *me;
 	BMEditMesh *em;
 	float loc[3], rot[3], mat[4][4], dia;
 	int enter_editmode;
 	int state;
 	unsigned int layer;
-	
+
 	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, NULL);
-	make_prim_init(C, "Grid", &dia, mat, &state, loc, rot, layer);
+	obedit = make_prim_init(C, "Grid", &dia, mat, &state, loc, rot, layer);
+	em = BMEdit_FromObject(obedit);
 
-	obedit = CTX_data_edit_object(C);
-	me = obedit->data;
-	em = me->edit_btmesh;
-
-	if (!EDBM_op_call_and_selectf(em, op, "vertout",
-	                              "create_grid xsegments=%i ysegments=%i size=%f mat=%m4",
+	if (!EDBM_op_call_and_selectf(em, op, "verts.out",
+	                              "create_grid x_segments=%i y_segments=%i size=%f matrix=%m4",
 	                              RNA_int_get(op->ptr, "x_subdivisions"),
 	                              RNA_int_get(op->ptr, "y_subdivisions"),
-	                              RNA_float_get(op->ptr, "size") * dia, mat))
+	                              RNA_float_get(op->ptr, "size"), mat))
 	{
 		return OPERATOR_CANCELLED;
 	}
-	
-	make_prim_finish(C, &state, enter_editmode);
+
+	make_prim_finish(C, obedit, &state, enter_editmode);
+
 	return OPERATOR_FINISHED;
 }
 
@@ -445,15 +395,15 @@ void MESH_OT_primitive_grid_add(wmOperatorType *ot)
 	ot->name = "Add Grid";
 	ot->description = "Construct a grid mesh";
 	ot->idname = "MESH_OT_primitive_grid_add";
-	
+
 	/* api callbacks */
-	ot->invoke = ED_object_add_generic_invoke;
+	ot->invoke = WM_operator_view3d_distance_invoke;
 	ot->exec = add_primitive_grid_exec;
 	ot->poll = ED_operator_scene_editable;
-	
+
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
-	
+
 	/* props */
 	RNA_def_int(ot->srna, "x_subdivisions", 10, 3, INT_MAX, "X Subdivisions", "", 3, 1000);
 	RNA_def_int(ot->srna, "y_subdivisions", 10, 3, INT_MAX, "Y Subdivisions", "", 3, 1000);
@@ -466,28 +416,29 @@ void MESH_OT_primitive_grid_add(wmOperatorType *ot)
 static int add_primitive_monkey_exec(bContext *C, wmOperator *op)
 {
 	Object *obedit;
-	Mesh *me;
 	BMEditMesh *em;
 	float loc[3], rot[3], mat[4][4], dia;
 	int enter_editmode;
 	int state, view_aligned;
 	unsigned int layer;
-	
+
 	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, &view_aligned);
 	if (!view_aligned)
-		rot[0] += M_PI / 2.0f;
-	
-	make_prim_init(C, "Monkey", &dia, mat, &state, loc, rot, layer);
+		rot[0] += (float)M_PI / 2.0f;
 
-	obedit = CTX_data_edit_object(C);
-	me = obedit->data;
-	em = me->edit_btmesh;
+	obedit = make_prim_init(C, "Suzanne", &dia, mat, &state, loc, rot, layer);
+	mat[0][0] *= dia;
+	mat[1][1] *= dia;
+	mat[2][2] *= dia;
 
-	if (!EDBM_op_call_and_selectf(em, op, "vertout", "create_monkey mat=%m4", mat)) {
+	em = BMEdit_FromObject(obedit);
+
+	if (!EDBM_op_call_and_selectf(em, op, "verts.out", "create_monkey matrix=%m4", mat)) {
 		return OPERATOR_CANCELLED;
 	}
-	
-	make_prim_finish(C, &state, enter_editmode);
+
+	make_prim_finish(C, obedit, &state, enter_editmode);
+
 	return OPERATOR_FINISHED;
 }
 
@@ -497,12 +448,11 @@ void MESH_OT_primitive_monkey_add(wmOperatorType *ot)
 	ot->name = "Add Monkey";
 	ot->description = "Construct a Suzanne mesh";
 	ot->idname = "MESH_OT_primitive_monkey_add";
-	
+
 	/* api callbacks */
-	ot->invoke = ED_object_add_generic_invoke;
 	ot->exec = add_primitive_monkey_exec;
 	ot->poll = ED_operator_scene_editable;
-	
+
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
@@ -512,31 +462,27 @@ void MESH_OT_primitive_monkey_add(wmOperatorType *ot)
 static int add_primitive_uvsphere_exec(bContext *C, wmOperator *op)
 {
 	Object *obedit;
-	Mesh *me;
 	BMEditMesh *em;
 	float loc[3], rot[3], mat[4][4], dia;
 	int enter_editmode;
 	int state;
 	unsigned int layer;
-	
+
 	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, NULL);
-	make_prim_init(C, "Sphere", &dia, mat, &state, loc, rot, layer);
+	obedit = make_prim_init(C, "Sphere", &dia, mat, &state, loc, rot, layer);
+	em = BMEdit_FromObject(obedit);
 
-	obedit = CTX_data_edit_object(C);
-	me = obedit->data;
-	em = me->edit_btmesh;
-
-	if (!EDBM_op_call_and_selectf(em, op, "vertout",
-	                              "create_uvsphere segments=%i revolutions=%i diameter=%f mat=%m4",
+	if (!EDBM_op_call_and_selectf(em, op, "verts.out",
+	                              "create_uvsphere u_segments=%i v_segments=%i diameter=%f matrix=%m4",
 	                              RNA_int_get(op->ptr, "segments"), RNA_int_get(op->ptr, "ring_count"),
 	                              RNA_float_get(op->ptr, "size"), mat))
 	{
 		return OPERATOR_CANCELLED;
 	}
-	
-	make_prim_finish(C, &state, enter_editmode);
 
-	return OPERATOR_FINISHED;	
+	make_prim_finish(C, obedit, &state, enter_editmode);
+
+	return OPERATOR_FINISHED;
 }
 
 void MESH_OT_primitive_uv_sphere_add(wmOperatorType *ot)
@@ -547,15 +493,15 @@ void MESH_OT_primitive_uv_sphere_add(wmOperatorType *ot)
 	ot->name = "Add UV Sphere";
 	ot->description = "Construct a UV sphere mesh";
 	ot->idname = "MESH_OT_primitive_uv_sphere_add";
-	
+
 	/* api callbacks */
-	ot->invoke = ED_object_add_generic_invoke;
+	ot->invoke = WM_operator_view3d_distance_invoke;
 	ot->exec = add_primitive_uvsphere_exec;
 	ot->poll = ED_operator_scene_editable;
-	
+
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
-	
+
 	/* props */
 	RNA_def_int(ot->srna, "segments", 32, 3, INT_MAX, "Segments", "", 3, 500);
 	RNA_def_int(ot->srna, "ring_count", 16, 3, INT_MAX, "Rings", "", 3, 500);
@@ -568,32 +514,28 @@ void MESH_OT_primitive_uv_sphere_add(wmOperatorType *ot)
 static int add_primitive_icosphere_exec(bContext *C, wmOperator *op)
 {
 	Object *obedit;
-	Mesh *me;
 	BMEditMesh *em;
 	float loc[3], rot[3], mat[4][4], dia;
 	int enter_editmode;
 	int state;
 	unsigned int layer;
-	
-	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, NULL);
-	make_prim_init(C, "Icosphere", &dia, mat, &state, loc, rot, layer);
 
-	obedit = CTX_data_edit_object(C);
-	me = obedit->data;
-	em = me->edit_btmesh;
+	ED_object_add_generic_get_opts(C, op, loc, rot, &enter_editmode, &layer, NULL);
+	obedit = make_prim_init(C, "Icosphere", &dia, mat, &state, loc, rot, layer);
+	em = BMEdit_FromObject(obedit);
 
 	if (!EDBM_op_call_and_selectf(
-	        em, op, "vertout",
-	        "create_icosphere subdivisions=%i diameter=%f mat=%m4",
+	        em, op, "verts.out",
+	        "create_icosphere subdivisions=%i diameter=%f matrix=%m4",
 	        RNA_int_get(op->ptr, "subdivisions"),
 	        RNA_float_get(op->ptr, "size"), mat))
 	{
 		return OPERATOR_CANCELLED;
 	}
-	
-	make_prim_finish(C, &state, enter_editmode);
 
-	return OPERATOR_FINISHED;	
+	make_prim_finish(C, obedit, &state, enter_editmode);
+
+	return OPERATOR_FINISHED;
 }
 
 void MESH_OT_primitive_ico_sphere_add(wmOperatorType *ot)
@@ -604,15 +546,15 @@ void MESH_OT_primitive_ico_sphere_add(wmOperatorType *ot)
 	ot->name = "Add Ico Sphere";
 	ot->description = "Construct an Icosphere mesh";
 	ot->idname = "MESH_OT_primitive_ico_sphere_add";
-	
+
 	/* api callbacks */
-	ot->invoke = ED_object_add_generic_invoke;
+	ot->invoke = WM_operator_view3d_distance_invoke;
 	ot->exec = add_primitive_icosphere_exec;
 	ot->poll = ED_operator_scene_editable;
-	
+
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
-	
+
 	/* props */
 	RNA_def_int(ot->srna, "subdivisions", 2, 1, INT_MAX, "Subdivisions", "", 1, 8);
 	prop = RNA_def_float(ot->srna, "size", 1.0f, 0.0f, FLT_MAX, "Size", "", 0.001f, 100.00);
