@@ -28,33 +28,23 @@
  *  \ingroup edmesh
  */
 
-
-#include <math.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include "MEM_guardedalloc.h"
 
-#include "DNA_material_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
 
-#include "BLI_utildefines.h"
+#include "BLI_path_util.h"
 #include "BLI_array.h"
 #include "BLI_math.h"
-#include "BLI_edgehash.h"
-#include "BLI_utildefines.h"
 
 #include "BKE_context.h"
 #include "BKE_depsgraph.h"
-#include "BKE_displist.h"
 #include "BKE_image.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
-#include "BKE_material.h"
 #include "BKE_mesh.h"
 #include "BKE_report.h"
 #include "BKE_tessmesh.h"
@@ -70,10 +60,67 @@
 #include "ED_uvedit.h"
 #include "ED_view3d.h"
 
-#include "RE_render_ext.h"
-
 #include "mesh_intern.h"
 
+
+static CustomData *mesh_customdata_get_type(Mesh *me, const char htype, int *r_tot)
+{
+	CustomData *data;
+	BMesh *bm = (me->edit_btmesh) ? me->edit_btmesh->bm : NULL;
+	int tot;
+
+	/* this  */
+	switch (htype) {
+		case BM_VERT:
+			if (bm) {
+				data = &bm->vdata;
+				tot  = bm->totvert;
+			}
+			else {
+				data = &me->vdata;
+				tot  = me->totvert;
+			}
+			break;
+		case BM_EDGE:
+			if (bm) {
+				data = &bm->edata;
+				tot  = bm->totedge;
+			}
+			else {
+				data = &me->edata;
+				tot  = me->totedge;
+			}
+			break;
+		case BM_LOOP:
+			if (bm) {
+				data = &bm->ldata;
+				tot  = bm->totloop;
+			}
+			else {
+				data = &me->ldata;
+				tot  = me->totloop;
+			}
+			break;
+		case BM_FACE:
+			if (bm) {
+				data = &bm->pdata;
+				tot  = bm->totface;
+			}
+			else {
+				data = &me->pdata;
+				tot  = me->totpoly;
+			}
+			break;
+		default:
+			BLI_assert(0);
+			tot = 0;
+			data = NULL;
+	}
+
+	*r_tot = tot;
+	return data;
+}
+
 #define GET_CD_DATA(me, data) (me->edit_btmesh ? &me->edit_btmesh->bm->data : &me->data)
 static void delete_customdata_layer(bContext *C, Object *ob, CustomDataLayer *layer)
 {
@@ -85,24 +132,10 @@ static void delete_customdata_layer(bContext *C, Object *ob, CustomDataLayer *la
 	int i, actindex, rndindex, cloneindex, stencilindex, tot;
 
 	if (layer->type == CD_MLOOPCOL || layer->type == CD_MLOOPUV) {
-		if (me->edit_btmesh) {
-			data = &me->edit_btmesh->bm->ldata;
-			tot = me->edit_btmesh->bm->totloop;
-		}
-		else {
-			data = &me->ldata;
-			tot = me->totloop;
-		}
+		data = mesh_customdata_get_type(me, BM_LOOP, &tot);
 	}
 	else {
-		if (me->edit_btmesh) {
-			data = &me->edit_btmesh->bm->pdata;
-			tot = me->edit_btmesh->bm->totface;
-		}
-		else {
-			data = &me->pdata;
-			tot = me->totpoly;
-		}
+		data = mesh_customdata_get_type(me, BM_FACE, &tot);
 	}
 	
 	index = CustomData_get_layer_index(data, type);
@@ -189,7 +222,8 @@ static void delete_customdata_layer(bContext *C, Object *ob, CustomDataLayer *la
 	}
 }
 
-int ED_mesh_uv_loop_reset_ex(struct bContext *C, struct Mesh *me, const int layernum)
+/* without bContext, called in uvedit */
+int ED_mesh_uv_loop_reset_ex(struct Mesh *me, const int layernum)
 {
 	BMEditMesh *em = me->edit_btmesh;
 	MLoopUV *luv;
@@ -275,11 +309,11 @@ int ED_mesh_uv_loop_reset_ex(struct bContext *C, struct Mesh *me, const int laye
 		else if (len > 2) {
 			float fac = 0.0f, dfac = 1.0f / (float)len;
 
-			dfac *= M_PI * 2;
+			dfac *= (float)M_PI * 2.0f;
 
 			for (i = 0; i < len; i++) {
-				fuvs[i][0] = 0.5f * sin(fac) + 0.5f;
-				fuvs[i][1] = 0.5f * cos(fac) + 0.5f;
+				fuvs[i][0] = 0.5f * sinf(fac) + 0.5f;
+				fuvs[i][1] = 0.5f * cosf(fac) + 0.5f;
 
 				fac += dfac;
 			}
@@ -292,7 +326,6 @@ int ED_mesh_uv_loop_reset_ex(struct bContext *C, struct Mesh *me, const int laye
 	BLI_array_free(polylengths);
 
 	DAG_id_tag_update(&me->id, 0);
-	WM_event_add_notifier(C, NC_GEOM | ND_DATA, me);
 
 	return 1;
 }
@@ -302,51 +335,55 @@ int ED_mesh_uv_loop_reset(struct bContext *C, struct Mesh *me)
 	/* could be ldata or pdata */
 	CustomData *pdata = GET_CD_DATA(me, pdata);
 	const int layernum = CustomData_get_active_layer_index(pdata, CD_MTEXPOLY);
-	return ED_mesh_uv_loop_reset_ex(C, me, layernum);
+	int retval = ED_mesh_uv_loop_reset_ex(me, layernum);
+	
+	WM_event_add_notifier(C, NC_GEOM | ND_DATA, me);
+	
+	return retval;
 }
 
 /* note: keep in sync with ED_mesh_color_add */
 int ED_mesh_uv_texture_add(bContext *C, Mesh *me, const char *name, int active_set)
 {
 	BMEditMesh *em;
-	int layernum;
+	int layernum_dst;
 
 	short is_init = FALSE;
 
 	if (me->edit_btmesh) {
 		em = me->edit_btmesh;
 
-		layernum = CustomData_number_of_layers(&em->bm->pdata, CD_MTEXPOLY);
-		if (layernum >= MAX_MTFACE)
+		layernum_dst = CustomData_number_of_layers(&em->bm->pdata, CD_MTEXPOLY);
+		if (layernum_dst >= MAX_MTFACE)
 			return -1;
 
 		/* CD_MTEXPOLY */
 		BM_data_layer_add_named(em->bm, &em->bm->pdata, CD_MTEXPOLY, name);
 		/* copy data from active UV */
-		if (layernum) {
-			const int layernum_dst = CustomData_get_active_layer(&em->bm->pdata, CD_MTEXPOLY);
-			BM_data_layer_copy(em->bm, &em->bm->pdata, CD_MTEXPOLY, layernum, layernum_dst);
+		if (layernum_dst) {
+			const int layernum_src = CustomData_get_active_layer(&em->bm->pdata, CD_MTEXPOLY);
+			BM_data_layer_copy(em->bm, &em->bm->pdata, CD_MTEXPOLY, layernum_src, layernum_dst);
 		}
-		if (active_set || layernum == 0) {
-			CustomData_set_layer_active(&em->bm->pdata, CD_MTEXPOLY, layernum);
+		if (active_set || layernum_dst == 0) {
+			CustomData_set_layer_active(&em->bm->pdata, CD_MTEXPOLY, layernum_dst);
 		}
 
 		/* CD_MLOOPUV */
 		BM_data_layer_add_named(em->bm, &em->bm->ldata, CD_MLOOPUV, name);
 		/* copy data from active UV */
-		if (layernum) {
-			const int layernum_dst = CustomData_get_active_layer(&em->bm->ldata, CD_MLOOPUV);
-			BM_data_layer_copy(em->bm, &em->bm->ldata, CD_MLOOPUV, layernum, layernum_dst);
+		if (layernum_dst) {
+			const int layernum_src = CustomData_get_active_layer(&em->bm->ldata, CD_MLOOPUV);
+			BM_data_layer_copy(em->bm, &em->bm->ldata, CD_MLOOPUV, layernum_src, layernum_dst);
 
 			is_init = TRUE;
 		}
-		if (active_set || layernum == 0) {
-			CustomData_set_layer_active(&em->bm->ldata, CD_MLOOPUV, layernum);
+		if (active_set || layernum_dst == 0) {
+			CustomData_set_layer_active(&em->bm->ldata, CD_MLOOPUV, layernum_dst);
 		}
 	}
 	else {
-		layernum = CustomData_number_of_layers(&me->pdata, CD_MTEXPOLY);
-		if (layernum >= MAX_MTFACE)
+		layernum_dst = CustomData_number_of_layers(&me->pdata, CD_MTEXPOLY);
+		if (layernum_dst >= MAX_MTFACE)
 			return -1;
 
 		if (me->mtpoly) {
@@ -361,11 +398,11 @@ int ED_mesh_uv_texture_add(bContext *C, Mesh *me, const char *name, int active_s
 			CustomData_add_layer_named(&me->fdata, CD_MTFACE, CD_DEFAULT, NULL, me->totface, name);
 		}
 		
-		if (active_set || layernum == 0) {
-			CustomData_set_layer_active(&me->pdata, CD_MTEXPOLY, layernum);
-			CustomData_set_layer_active(&me->ldata, CD_MLOOPUV, layernum);
+		if (active_set || layernum_dst == 0) {
+			CustomData_set_layer_active(&me->pdata, CD_MTEXPOLY, layernum_dst);
+			CustomData_set_layer_active(&me->ldata, CD_MLOOPUV, layernum_dst);
 
-			CustomData_set_layer_active(&me->fdata, CD_MTFACE, layernum);
+			CustomData_set_layer_active(&me->fdata, CD_MTFACE, layernum_dst);
 		}
 
 		mesh_update_customdata_pointers(me, TRUE);
@@ -373,13 +410,13 @@ int ED_mesh_uv_texture_add(bContext *C, Mesh *me, const char *name, int active_s
 
 	/* don't overwrite our copied coords */
 	if (is_init == FALSE) {
-		ED_mesh_uv_loop_reset_ex(C, me, layernum);
+		ED_mesh_uv_loop_reset_ex(me, layernum_dst);
 	}
 
 	DAG_id_tag_update(&me->id, 0);
 	WM_event_add_notifier(C, NC_GEOM | ND_DATA, me);
 
-	return layernum;
+	return layernum_dst;
 }
 
 int ED_mesh_uv_texture_remove(bContext *C, Object *ob, Mesh *me)
@@ -425,7 +462,7 @@ int ED_mesh_color_add(bContext *C, Scene *UNUSED(scene), Object *UNUSED(ob), Mes
 		/* copy data from active vertex color layer */
 		if (layernum) {
 			const int layernum_dst = CustomData_get_active_layer(&em->bm->ldata, CD_MLOOPCOL);
-			BM_data_layer_copy(em->bm, &em->bm->ldata, CD_MLOOPUV, layernum, layernum_dst);
+			BM_data_layer_copy(em->bm, &em->bm->ldata, CD_MLOOPCOL, layernum, layernum_dst);
 		}
 		if (active_set || layernum == 0) {
 			CustomData_set_layer_active(&em->bm->ldata, CD_MLOOPCOL, layernum);
@@ -538,16 +575,22 @@ static int drop_named_image_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	Main *bmain = CTX_data_main(C);
 	Scene *scene = CTX_data_scene(C);
 	View3D *v3d = CTX_wm_view3d(C);
-	Base *base = ED_view3d_give_base_under_cursor(C, event->mval);
+	Base *base;
 	Image *ima = NULL;
 	Mesh *me;
 	Object *obedit;
 	int exitmode = 0;
-	char name[MAX_ID_NAME - 2];
 	
+	if (v3d == NULL) {
+		BKE_report(op->reports, RPT_ERROR, "No 3D View Available");
+		return OPERATOR_CANCELLED;
+	}
+
+	base = ED_view3d_give_base_under_cursor(C, event->mval);
+
 	/* Check context */
 	if (base == NULL || base->object->type != OB_MESH) {
-		BKE_report(op->reports, RPT_ERROR, "Not an Object or Mesh");
+		BKE_report(op->reports, RPT_ERROR, "Not an object or mesh");
 		return OPERATOR_CANCELLED;
 	}
 	
@@ -556,15 +599,16 @@ static int drop_named_image_invoke(bContext *C, wmOperator *op, wmEvent *event)
 		char path[FILE_MAX];
 		
 		RNA_string_get(op->ptr, "filepath", path);
-		ima = BKE_add_image_file(path);
+		ima = BKE_image_load_exists(path);
 	}
 	else {
+		char name[MAX_ID_NAME - 2];
 		RNA_string_get(op->ptr, "name", name);
-		ima = (Image *)find_id("IM", name);
+		ima = (Image *)BKE_libblock_find_name(ID_IM, name);
 	}
 	
 	if (!ima) {
-		BKE_report(op->reports, RPT_ERROR, "Not an Image");
+		BKE_report(op->reports, RPT_ERROR, "Not an image");
 		return OPERATOR_CANCELLED;
 	}
 	
@@ -604,7 +648,7 @@ static int drop_named_image_invoke(bContext *C, wmOperator *op, wmEvent *event)
 void MESH_OT_drop_named_image(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Assign Image to UV Map";
+	ot->name = "Drop Image to Mesh UV Map";
 	ot->description = "Assign Image to active UV Map, or create an UV Map";
 	ot->idname = "MESH_OT_drop_named_image";
 	
@@ -613,7 +657,7 @@ void MESH_OT_drop_named_image(wmOperatorType *ot)
 	ot->invoke = drop_named_image_invoke;
 	
 	/* flags */
-	ot->flag = OPTYPE_UNDO;
+	ot->flag = OPTYPE_UNDO | OPTYPE_INTERNAL;
 	
 	/* properties */
 	RNA_def_string(ot->srna, "name", "Image", MAX_ID_NAME - 2, "Name", "Image name to assign");
@@ -701,71 +745,124 @@ void MESH_OT_vertex_color_remove(wmOperatorType *ot)
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
 
-/*********************** sticky operators ************************/
+/* *** CustomData clear functions, we need an operator for each *** */
 
-static int mesh_sticky_add_exec(bContext *C, wmOperator *UNUSED(op))
+static int mesh_customdata_clear_exec__internal(bContext *C,
+                                                char htype, int type)
 {
-	Scene *scene = CTX_data_scene(C);
-	View3D *v3d = CTX_wm_view3d(C);
-	Object *ob = ED_object_context(C);
-	Mesh *me = ob->data;
+	Object *obedit = ED_object_context(C);
+	Mesh       *me = obedit->data;
 
-	/* why is this commented out? */
-#if 0
-	if (me->msticky)
+	int tot;
+	CustomData *data = mesh_customdata_get_type(me, htype, &tot);
+
+	BLI_assert(CustomData_layertype_is_singleton(type) == TRUE);
+
+	if (CustomData_has_layer(data, type)) {
+		if (me->edit_btmesh) {
+			BM_data_layer_free(me->edit_btmesh->bm, data, type);
+		}
+		else {
+			CustomData_free_layers(data, type, tot);
+		}
+
+		DAG_id_tag_update(&me->id, 0);
+		WM_event_add_notifier(C, NC_GEOM | ND_DATA, me);
+
+		return OPERATOR_FINISHED;
+	}
+	else {
 		return OPERATOR_CANCELLED;
-#endif
-
-	RE_make_sticky(scene, v3d);
-
-	DAG_id_tag_update(&me->id, 0);
-	WM_event_add_notifier(C, NC_GEOM | ND_DATA, me);
-
-	return OPERATOR_FINISHED;
+	}
 }
 
-void MESH_OT_sticky_add(wmOperatorType *ot)
+/* Clear Mask */
+static int mesh_customdata_clear_mask_poll(bContext *C)
 {
+	Object *ob = ED_object_context(C);
+	if (ob && ob->type == OB_MESH) {
+		Mesh *me = ob->data;
+
+		/* special case - can't run this if we're in sculpt mode */
+		if (ob->mode & OB_MODE_SCULPT) {
+			return FALSE;
+		}
+
+		if (me->id.lib == NULL) {
+			CustomData *data = GET_CD_DATA(me, vdata);
+			if (CustomData_has_layer(data, CD_PAINT_MASK)) {
+				return TRUE;
+			}
+			data = GET_CD_DATA(me, ldata);
+			if (CustomData_has_layer(data, CD_GRID_PAINT_MASK)) {
+				return TRUE;
+			}
+		}
+	}
+	return FALSE;
+}
+static int mesh_customdata_clear_mask_exec(bContext *C, wmOperator *UNUSED(op))
+{
+	int ret_a = mesh_customdata_clear_exec__internal(C, BM_VERT, CD_PAINT_MASK);
+	int ret_b = mesh_customdata_clear_exec__internal(C, BM_LOOP, CD_GRID_PAINT_MASK);
+
+	if (ret_a == OPERATOR_FINISHED ||
+		ret_b == OPERATOR_FINISHED)
+	{
+		return OPERATOR_FINISHED;
+	}
+	else {
+		return OPERATOR_CANCELLED;
+	}
+}
+
+void MESH_OT_customdata_clear_mask(wmOperatorType *ot)
+{
+
 	/* identifiers */
-	ot->name = "Add Sticky";
-	ot->description = "Add sticky UV texture layer";
-	ot->idname = "MESH_OT_sticky_add";
-	
+	ot->name = "Clear Sculpt-Mask Data";
+	ot->idname = "MESH_OT_customdata_clear_mask";
+	ot->description = "Clear vertex sculpt masking data from the mesh";
+
 	/* api callbacks */
-	ot->poll = layers_poll;
-	ot->exec = mesh_sticky_add_exec;
+	ot->exec = mesh_customdata_clear_mask_exec;
+	ot->poll = mesh_customdata_clear_mask_poll;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
 
-static int mesh_sticky_remove_exec(bContext *C, wmOperator *UNUSED(op))
+/* Clear Skin */
+static int mesh_customdata_clear_skin_poll(bContext *C)
 {
 	Object *ob = ED_object_context(C);
-	Mesh *me = ob->data;
 
-	if (!me->msticky)
-		return OPERATOR_CANCELLED;
-
-	CustomData_free_layer_active(&me->vdata, CD_MSTICKY, me->totvert);
-	me->msticky = NULL;
-
-	DAG_id_tag_update(&me->id, 0);
-	WM_event_add_notifier(C, NC_GEOM | ND_DATA, me);
-
-	return OPERATOR_FINISHED;
+	if (ob && ob->type == OB_MESH) {
+		Mesh *me = ob->data;
+		if (me->id.lib == NULL) {
+			CustomData *data = GET_CD_DATA(me, vdata);
+			if (CustomData_has_layer(data, CD_MVERT_SKIN)) {
+				return TRUE;
+			}
+		}
+	}
+	return FALSE;
+}
+static int mesh_customdata_clear_skin_exec(bContext *C, wmOperator *UNUSED(op))
+{
+	return mesh_customdata_clear_exec__internal(C, BM_VERT, CD_MVERT_SKIN);
 }
 
-void MESH_OT_sticky_remove(wmOperatorType *ot)
+void MESH_OT_customdata_clear_skin(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Remove Sticky";
-	ot->description = "Remove sticky UV texture layer";
-	ot->idname = "MESH_OT_sticky_remove";
-	
+	ot->name = "Clear Skin Data";
+	ot->idname = "MESH_OT_customdata_clear_skin";
+	ot->description = "Clear vertex skin layer";
+
 	/* api callbacks */
-	ot->poll = layers_poll;
-	ot->exec = mesh_sticky_remove_exec;
+	ot->exec = mesh_customdata_clear_skin_exec;
+	ot->poll = mesh_customdata_clear_skin_poll;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -803,21 +900,21 @@ void ED_mesh_update(Mesh *mesh, bContext *C, int calc_edges, int calc_tessface)
 	 * so rather then add poly-index layer and calculate normals for it
 	 * calculate normals only for the mvert's. - campbell */
 #ifdef USE_BMESH_MPOLY_NORMALS
-	polyindex = CustomData_get_layer(&mesh->fdata, CD_POLYINDEX);
+	polyindex = CustomData_get_layer(&mesh->fdata, CD_ORIGINDEX);
 	/* add a normals layer for tessellated faces, a tessface normal will
 	 * contain the normal of the poly the face was tessellated from. */
 	face_nors = CustomData_add_layer(&mesh->fdata, CD_NORMAL, CD_CALLOC, NULL, mesh->totface);
 
-	mesh_calc_normals_mapping_ex(mesh->mvert, mesh->totvert,
-	                             mesh->mloop, mesh->mpoly,
-	                             mesh->totloop, mesh->totpoly,
-	                             NULL /* polyNors_r */,
-	                             mesh->mface, mesh->totface,
-	                             polyindex, face_nors, FALSE);
+	BKE_mesh_calc_normals_mapping_ex(mesh->mvert, mesh->totvert,
+	                                 mesh->mloop, mesh->mpoly,
+	                                 mesh->totloop, mesh->totpoly,
+	                                 NULL /* polyNors_r */,
+	                                 mesh->mface, mesh->totface,
+	                                 polyindex, face_nors, FALSE);
 #else
-	mesh_calc_normals(mesh->mvert, mesh->totvert,
-	                  mesh->mloop, mesh->mpoly, mesh->totloop, mesh->totpoly,
-	                  NULL);
+	BKE_mesh_calc_normals(mesh->mvert, mesh->totvert,
+	                      mesh->mloop, mesh->mpoly, mesh->totloop, mesh->totpoly,
+	                      NULL);
 	(void)polyindex;
 	(void)face_nors;
 #endif
@@ -848,6 +945,7 @@ static void mesh_add_verts(Mesh *mesh, int len)
 
 	/* scan the input list and insert the new vertices */
 
+	/* set default flags */
 	mvert = &mesh->mvert[mesh->totvert];
 	for (i = 0; i < len; i++, mvert++)
 		mvert->flag |= SELECT;
@@ -1025,7 +1123,7 @@ static void mesh_remove_faces(Mesh *mesh, int len)
 void ED_mesh_geometry_add(Mesh *mesh, ReportList *reports, int verts, int edges, int faces)
 {
 	if (mesh->edit_btmesh) {
-		BKE_report(reports, RPT_ERROR, "Can't add geometry in edit mode");
+		BKE_report(reports, RPT_ERROR, "Cannot add geometry in edit mode");
 		return;
 	}
 
@@ -1041,12 +1139,12 @@ void ED_mesh_geometry_add(Mesh *mesh, ReportList *reports, int verts, int edges,
 void ED_mesh_tessfaces_add(Mesh *mesh, ReportList *reports, int count)
 {
 	if (mesh->edit_btmesh) {
-		BKE_report(reports, RPT_ERROR, "Can't add tessfaces in edit mode");
+		BKE_report(reports, RPT_ERROR, "Cannot add tessfaces in edit mode");
 		return;
 	}
 
 	if (mesh->mpoly) {
-		BKE_report(reports, RPT_ERROR, "Can't add tessfaces to a mesh that already has polygons");
+		BKE_report(reports, RPT_ERROR, "Cannot add tessfaces to a mesh that already has polygons");
 		return;
 	}
 
@@ -1056,7 +1154,7 @@ void ED_mesh_tessfaces_add(Mesh *mesh, ReportList *reports, int count)
 void ED_mesh_edges_add(Mesh *mesh, ReportList *reports, int count)
 {
 	if (mesh->edit_btmesh) {
-		BKE_report(reports, RPT_ERROR, "Can't add edges in edit mode");
+		BKE_report(reports, RPT_ERROR, "Cannot add edges in edit mode");
 		return;
 	}
 
@@ -1066,7 +1164,7 @@ void ED_mesh_edges_add(Mesh *mesh, ReportList *reports, int count)
 void ED_mesh_vertices_add(Mesh *mesh, ReportList *reports, int count)
 {
 	if (mesh->edit_btmesh) {
-		BKE_report(reports, RPT_ERROR, "Can't add vertices in edit mode");
+		BKE_report(reports, RPT_ERROR, "Cannot add vertices in edit mode");
 		return;
 	}
 
@@ -1076,11 +1174,11 @@ void ED_mesh_vertices_add(Mesh *mesh, ReportList *reports, int count)
 void ED_mesh_faces_remove(Mesh *mesh, ReportList *reports, int count)
 {
 	if (mesh->edit_btmesh) {
-		BKE_report(reports, RPT_ERROR, "Can't remove faces in edit mode");
+		BKE_report(reports, RPT_ERROR, "Cannot remove faces in edit mode");
 		return;
 	}
 	else if (count > mesh->totface) {
-		BKE_report(reports, RPT_ERROR, "Can't remove more faces than the mesh contains");
+		BKE_report(reports, RPT_ERROR, "Cannot remove more faces than the mesh contains");
 		return;
 	}
 
@@ -1090,11 +1188,11 @@ void ED_mesh_faces_remove(Mesh *mesh, ReportList *reports, int count)
 void ED_mesh_edges_remove(Mesh *mesh, ReportList *reports, int count)
 {
 	if (mesh->edit_btmesh) {
-		BKE_report(reports, RPT_ERROR, "Can't remove edges in edit mode");
+		BKE_report(reports, RPT_ERROR, "Cannot remove edges in edit mode");
 		return;
 	}
 	else if (count > mesh->totedge) {
-		BKE_report(reports, RPT_ERROR, "Can't remove more edges than the mesh contains");
+		BKE_report(reports, RPT_ERROR, "Cannot remove more edges than the mesh contains");
 		return;
 	}
 
@@ -1104,11 +1202,11 @@ void ED_mesh_edges_remove(Mesh *mesh, ReportList *reports, int count)
 void ED_mesh_vertices_remove(Mesh *mesh, ReportList *reports, int count)
 {
 	if (mesh->edit_btmesh) {
-		BKE_report(reports, RPT_ERROR, "Can't remove vertices in edit mode");
+		BKE_report(reports, RPT_ERROR, "Cannot remove vertices in edit mode");
 		return;
 	}
 	else if (count > mesh->totvert) {
-		BKE_report(reports, RPT_ERROR, "Can't remove more vertices than the mesh contains");
+		BKE_report(reports, RPT_ERROR, "Cannot remove more vertices than the mesh contains");
 		return;
 	}
 
@@ -1118,7 +1216,7 @@ void ED_mesh_vertices_remove(Mesh *mesh, ReportList *reports, int count)
 void ED_mesh_loops_add(Mesh *mesh, ReportList *reports, int count)
 {
 	if (mesh->edit_btmesh) {
-		BKE_report(reports, RPT_ERROR, "Can't add loops in edit mode.");
+		BKE_report(reports, RPT_ERROR, "Cannot add loops in edit mode");
 		return;
 	}
 
@@ -1128,7 +1226,7 @@ void ED_mesh_loops_add(Mesh *mesh, ReportList *reports, int count)
 void ED_mesh_polys_add(Mesh *mesh, ReportList *reports, int count)
 {
 	if (mesh->edit_btmesh) {
-		BKE_report(reports, RPT_ERROR, "Can't add polys in edit mode.");
+		BKE_report(reports, RPT_ERROR, "Cannot add polygons in edit mode");
 		return;
 	}
 
@@ -1138,13 +1236,13 @@ void ED_mesh_polys_add(Mesh *mesh, ReportList *reports, int count)
 void ED_mesh_calc_normals(Mesh *mesh)
 {
 #ifdef USE_BMESH_MPOLY_NORMALS
-	mesh_calc_normals_mapping_ex(mesh->mvert, mesh->totvert,
-	                             mesh->mloop, mesh->mpoly, mesh->totloop, mesh->totpoly,
-	                             NULL, NULL, 0, NULL, NULL, FALSE);
+	BKE_mesh_calc_normals_mapping_ex(mesh->mvert, mesh->totvert,
+	                                 mesh->mloop, mesh->mpoly, mesh->totloop, mesh->totpoly,
+	                                 NULL, NULL, 0, NULL, NULL, FALSE);
 #else
-	mesh_calc_normals(mesh->mvert, mesh->totvert,
-	                  mesh->mloop, mesh->mpoly, mesh->totloop, mesh->totpoly,
-	                  NULL);
+	BKE_mesh_calc_normals(mesh->mvert, mesh->totvert,
+	                      mesh->mloop, mesh->mpoly, mesh->totloop, mesh->totpoly,
+	                      NULL);
 #endif
 }
 
