@@ -42,6 +42,7 @@
 #include "DNA_property_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_armature_types.h"
+#include "DNA_lamp_types.h"
 
 #include "BLI_math.h"
 #include "BLI_listbase.h"
@@ -88,34 +89,34 @@
 void ED_base_object_select(Base *base, short mode)
 {
 	if (base) {
-		if (mode==BA_SELECT) {
+		if (mode == BA_SELECT) {
 			if (!(base->object->restrictflag & OB_RESTRICT_SELECT))
 				base->flag |= SELECT;
 		}
-		else if (mode==BA_DESELECT) {
+		else if (mode == BA_DESELECT) {
 			base->flag &= ~SELECT;
 		}
-		base->object->flag= base->flag;
+		base->object->flag = base->flag;
 	}
 }
 
 /* also to set active NULL */
 void ED_base_object_activate(bContext *C, Base *base)
 {
-	Scene *scene= CTX_data_scene(C);
+	Scene *scene = CTX_data_scene(C);
 	
 	/* sets scene->basact */
-	BASACT= base;
+	BASACT = base;
 	
 	if (base) {
 		
 		/* XXX old signals, remember to handle notifiers now! */
 		//		select_actionchannel_by_name(base->object->action, "Object", 1);
 		
-		WM_event_add_notifier(C, NC_SCENE|ND_OB_ACTIVE, scene);
+		WM_event_add_notifier(C, NC_SCENE | ND_OB_ACTIVE, scene);
 	}
 	else
-		WM_event_add_notifier(C, NC_SCENE|ND_OB_ACTIVE, NULL);
+		WM_event_add_notifier(C, NC_SCENE | ND_OB_ACTIVE, NULL);
 }
 
 /********************** Selection Operators **********************/
@@ -124,7 +125,7 @@ static int objects_selectable_poll(bContext *C)
 {
 	/* we don't check for linked scenes here, selection is
 	 * still allowed then for inspection of scene */
-	Object *obact= CTX_data_active_object(C);
+	Object *obact = CTX_data_active_object(C);
 
 	if (CTX_data_edit_object(C))
 		return 0;
@@ -141,23 +142,25 @@ static int object_select_by_type_exec(bContext *C, wmOperator *op)
 	short obtype, extend;
 	
 	obtype = RNA_enum_get(op->ptr, "type");
-	extend= RNA_boolean_get(op->ptr, "extend");
+	extend = RNA_boolean_get(op->ptr, "extend");
 		
 	if (extend == 0) {
-		CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
+		CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+		{
 			ED_base_object_select(base, BA_DESELECT);
 		}
 		CTX_DATA_END;
 	}
 	
-	CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
-		if (base->object->type==obtype) {
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
+		if (base->object->type == obtype) {
 			ED_base_object_select(base, BA_SELECT);
 		}
 	}
 	CTX_DATA_END;
 	
-	WM_event_add_notifier(C, NC_SCENE|ND_OB_SELECT, CTX_data_scene(C));
+	WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, CTX_data_scene(C));
 	
 	return OPERATOR_FINISHED;
 }
@@ -175,7 +178,7 @@ void OBJECT_OT_select_by_type(wmOperatorType *ot)
 	ot->poll = objects_selectable_poll;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 	
 	/* properties */
 	RNA_def_boolean(ot->srna, "extend", FALSE, "Extend", "Extend selection instead of deselecting everything first");
@@ -184,160 +187,304 @@ void OBJECT_OT_select_by_type(wmOperatorType *ot)
 
 /*********************** Selection by Links *********************/
 
+enum {
+	OBJECT_SELECT_LINKED_IPO = 1,
+	OBJECT_SELECT_LINKED_OBDATA,
+	OBJECT_SELECT_LINKED_MATERIAL,
+	OBJECT_SELECT_LINKED_TEXTURE,
+	OBJECT_SELECT_LINKED_DUPGROUP,
+	OBJECT_SELECT_LINKED_PARTICLE,
+	OBJECT_SELECT_LINKED_LIBRARY,
+	OBJECT_SELECT_LINKED_LIBRARY_OBDATA
+};
+
 static EnumPropertyItem prop_select_linked_types[] = {
-	//{1, "IPO", 0, "Object IPO", ""}, // XXX depreceated animation system stuff...
-	{2, "OBDATA", 0, "Object Data", ""},
-	{3, "MATERIAL", 0, "Material", ""},
-	{4, "TEXTURE", 0, "Texture", ""},
-	{5, "DUPGROUP", 0, "Dupligroup", ""},
-	{6, "PARTICLE", 0, "Particle System", ""},
-	{7, "LIBRARY", 0, "Library", ""},
-	{8, "LIBRARY_OBDATA", 0, "Library (Object Data)", ""},
+	//{OBJECT_SELECT_LINKED_IPO, "IPO", 0, "Object IPO", ""}, // XXX deprecated animation system stuff...
+	{OBJECT_SELECT_LINKED_OBDATA, "OBDATA", 0, "Object Data", ""},
+	{OBJECT_SELECT_LINKED_MATERIAL, "MATERIAL", 0, "Material", ""},
+	{OBJECT_SELECT_LINKED_TEXTURE, "TEXTURE", 0, "Texture", ""},
+	{OBJECT_SELECT_LINKED_DUPGROUP, "DUPGROUP", 0, "Dupligroup", ""},
+	{OBJECT_SELECT_LINKED_PARTICLE, "PARTICLE", 0, "Particle System", ""},
+	{OBJECT_SELECT_LINKED_LIBRARY, "LIBRARY", 0, "Library", ""},
+	{OBJECT_SELECT_LINKED_LIBRARY_OBDATA, "LIBRARY_OBDATA", 0, "Library (Object Data)", ""},
 	{0, NULL, 0, NULL, NULL}
 };
 
-static int object_select_linked_exec(bContext *C, wmOperator *op)
+// XXX old animation system
+#if 0
+static int object_select_all_by_ipo(bContext *C, Ipo *ipo)
 {
-	Scene *scene= CTX_data_scene(C);
-	Object *ob;
-	void *obdata = NULL;
-	Material *mat = NULL, *mat1;
-	Tex *tex= NULL;
-	int a, b;
-	int nr = RNA_enum_get(op->ptr, "type");
-	short changed = 0, extend;
-	/* events (nr):
-	 * Object Ipo: 1
-	 * ObData: 2
-	 * Current Material: 3
-	 * Current Texture: 4
-	 * DupliGroup: 5
-	 * PSys: 6
-	 */
+	int changed = FALSE;
 
-	extend= RNA_boolean_get(op->ptr, "extend");
-	
-	if (extend == 0) {
-		CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
-			ED_base_object_select(base, BA_DESELECT);
-		}
-		CTX_DATA_END;
-	}
-	
-	ob= OBACT;
-	if (ob==NULL) { 
-		BKE_report(op->reports, RPT_ERROR, "No Active Object");
-		return OPERATOR_CANCELLED;
-	}
-	
-	if (nr==1) {	
-			// XXX old animation system
-		//ipo= ob->ipo;
-		//if (ipo==0) return OPERATOR_CANCELLED;
-		return OPERATOR_CANCELLED;
-	}
-	else if (nr==2) {
-		if (ob->data==NULL) return OPERATOR_CANCELLED;
-		obdata= ob->data;
-	}
-	else if (nr==3 || nr==4) {
-		mat= give_current_material(ob, ob->actcol);
-		if (mat==NULL) return OPERATOR_CANCELLED;
-		if (nr==4) {
-			if (mat->mtex[ (int)mat->texact ]) tex= mat->mtex[ (int)mat->texact ]->tex;
-			if (tex==NULL) return OPERATOR_CANCELLED;
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
+		if (base->object->ipo == ipo) {
+			base->flag |= SELECT;
+			base->object->flag = base->flag;
+
+			changed = TRUE;
 		}
 	}
-	else if (nr==5) {
-		if (ob->dup_group==NULL) return OPERATOR_CANCELLED;
-	}
-	else if (nr==6) {
-		if (ob->particlesystem.first==NULL) return OPERATOR_CANCELLED;
-	}
-	else if (nr==7) {
-		/* do nothing */
-	}
-	else if (nr==8) {
-		if (ob->data==NULL) return OPERATOR_CANCELLED;
-	}
-	else
-		return OPERATOR_CANCELLED;
-	
-	CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
-		if (nr==1) {
-				// XXX old animation system
-			//if (base->object->ipo==ipo) base->flag |= SELECT;
-			//changed = 1;
+	CTX_DATA_END;
+
+	return changed;
+}
+#endif
+
+static int object_select_all_by_obdata(bContext *C, void *obdata)
+{
+	int changed = FALSE;
+
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
+		if ((base->flag & SELECT) == 0) {
+			if (base->object->data == obdata) {
+				base->flag |= SELECT;
+				base->object->flag = base->flag;
+
+				changed = TRUE;
+			}
 		}
-		else if (nr==2) {
-			if (base->object->data==obdata) base->flag |= SELECT;
-			changed = 1;
-		}
-		else if (nr==3 || nr==4) {
-			ob= base->object;
-			
-			for (a=1; a<=ob->totcol; a++) {
-				mat1= give_current_material(ob, a);
-				if (nr==3) {
-					if (mat1==mat) base->flag |= SELECT;
-					changed = 1;
+	}
+	CTX_DATA_END;
+
+	return changed;
+}
+
+static int object_select_all_by_material_texture(bContext *C, int use_texture, Material *mat, Tex *tex)
+{
+	int changed = FALSE;
+
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
+		if ((base->flag & SELECT) == 0) {
+			Object *ob = base->object;
+			Material *mat1;
+			int a, b;
+
+			for (a = 1; a <= ob->totcol; a++) {
+				mat1 = give_current_material(ob, a);
+
+				if (!use_texture) {
+					if (mat1 == mat) {
+						base->flag |= SELECT;
+						changed = TRUE;
+					}
 				}
-				else if (mat1 && nr==4) {
-					for (b=0; b<MAX_MTEX; b++) {
+				else if (mat1 && use_texture) {
+					for (b = 0; b < MAX_MTEX; b++) {
 						if (mat1->mtex[b]) {
-							if (tex==mat1->mtex[b]->tex) {
+							if (tex == mat1->mtex[b]->tex) {
 								base->flag |= SELECT;
-								changed = 1;
+								changed = TRUE;
 								break;
 							}
 						}
 					}
 				}
 			}
+
+			base->object->flag = base->flag;
 		}
-		else if (nr==5) {
-			if (base->object->dup_group==ob->dup_group) {
+	}
+	CTX_DATA_END;
+
+	return changed;
+}
+
+static int object_select_all_by_dup_group(bContext *C, Object *ob)
+{
+	int changed = FALSE;
+	Group *dup_group = (ob->transflag & OB_DUPLIGROUP) ? ob->dup_group : NULL;
+
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
+		if ((base->flag & SELECT) == 0) {
+			Group *dup_group_other = (base->object->transflag & OB_DUPLIGROUP) ? base->object->dup_group : NULL;
+			if (dup_group == dup_group_other) {
 				base->flag |= SELECT;
-				changed = 1;
+				base->object->flag = base->flag;
+
+				changed = TRUE;
 			}
 		}
-		else if (nr==6) {
+	}
+	CTX_DATA_END;
+
+	return changed;
+}
+
+static int object_select_all_by_particle(bContext *C, Object *ob)
+{
+	int changed = FALSE;
+
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
+		if ((base->flag & SELECT) == 0) {
 			/* loop through other, then actives particles*/
 			ParticleSystem *psys;
 			ParticleSystem *psys_act;
-			
-			for (psys=base->object->particlesystem.first; psys; psys=psys->next) {
-				for (psys_act=ob->particlesystem.first; psys_act; psys_act=psys_act->next) {
+
+			for (psys = base->object->particlesystem.first; psys; psys = psys->next) {
+				for (psys_act = ob->particlesystem.first; psys_act; psys_act = psys_act->next) {
 					if (psys->part == psys_act->part) {
 						base->flag |= SELECT;
-						changed = 1;
+						changed = TRUE;
 						break;
 					}
 				}
-				
+
 				if (base->flag & SELECT) {
 					break;
 				}
 			}
+
+			base->object->flag = base->flag;
 		}
-		else if (nr==7) {
-			if (ob->id.lib == base->object->id.lib) {
-				base->flag |= SELECT;
-				changed= 1;
-			}
-		}
-		else if (nr==8) {
-			if (base->object->data && ((ID *)ob->data)->lib == ((ID *)base->object->data)->lib) {
-				base->flag |= SELECT;
-				changed= 1;
-			}
-		}
-		base->object->flag= base->flag;
 	}
 	CTX_DATA_END;
-	
+
+	return changed;
+}
+
+static int object_select_all_by_library(bContext *C, Library *lib)
+{
+	int changed = FALSE;
+
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
+		if ((base->flag & SELECT) == 0) {
+			if (lib == base->object->id.lib) {
+				base->flag |= SELECT;
+				base->object->flag = base->flag;
+
+				changed = TRUE;
+			}
+		}
+	}
+	CTX_DATA_END;
+
+	return changed;
+}
+
+static int object_select_all_by_library_obdata(bContext *C, Library *lib)
+{
+	int changed = FALSE;
+
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
+		if ((base->flag & SELECT) == 0) {
+			if (base->object->data && lib == ((ID *)base->object->data)->lib) {
+				base->flag |= SELECT;
+				base->object->flag = base->flag;
+
+				changed = TRUE;
+			}
+		}
+	}
+	CTX_DATA_END;
+
+	return changed;
+}
+
+void ED_object_select_linked_by_id(bContext *C, ID *id)
+{
+	int idtype = GS(id->name);
+	int changed = FALSE;
+
+	if (OB_DATA_SUPPORT_ID(idtype)) {
+		changed = object_select_all_by_obdata(C, id);
+	}
+	else if (idtype == ID_MA) {
+		changed = object_select_all_by_material_texture(C, FALSE, (Material *)id, NULL);
+	}
+	else if (idtype == ID_LI) {
+		changed = object_select_all_by_library(C, (Library *) id);
+	}
+
 	if (changed) {
-		WM_event_add_notifier(C, NC_SCENE|ND_OB_SELECT, CTX_data_scene(C));
+		WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, CTX_data_scene(C));
+	}
+}
+
+static int object_select_linked_exec(bContext *C, wmOperator *op)
+{
+	Scene *scene = CTX_data_scene(C);
+	Object *ob;
+	int nr = RNA_enum_get(op->ptr, "type");
+	short changed = FALSE, extend;
+
+	extend = RNA_boolean_get(op->ptr, "extend");
+	
+	if (extend == 0) {
+		CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+		{
+			ED_base_object_select(base, BA_DESELECT);
+		}
+		CTX_DATA_END;
+	}
+	
+	ob = OBACT;
+	if (ob == NULL) {
+		BKE_report(op->reports, RPT_ERROR, "No active object");
+		return OPERATOR_CANCELLED;
+	}
+	
+	if (nr == OBJECT_SELECT_LINKED_IPO) {
+		// XXX old animation system
+		//if (ob->ipo == 0) return OPERATOR_CANCELLED;
+		//object_select_all_by_ipo(C, ob->ipo)
+		return OPERATOR_CANCELLED;
+	}
+	else if (nr == OBJECT_SELECT_LINKED_OBDATA) {
+		if (ob->data == 0)
+			return OPERATOR_CANCELLED;
+
+		changed = object_select_all_by_obdata(C, ob->data);
+	}
+	else if (nr == OBJECT_SELECT_LINKED_MATERIAL || nr == OBJECT_SELECT_LINKED_TEXTURE) {
+		Material *mat = NULL;
+		Tex *tex = NULL;
+		int use_texture = FALSE;
+
+		mat = give_current_material(ob, ob->actcol);
+		if (mat == NULL) return OPERATOR_CANCELLED;
+		if (nr == OBJECT_SELECT_LINKED_TEXTURE) {
+			use_texture = TRUE;
+
+			if (mat->mtex[(int)mat->texact]) tex = mat->mtex[(int)mat->texact]->tex;
+			if (tex == NULL) return OPERATOR_CANCELLED;
+		}
+
+		changed = object_select_all_by_material_texture(C, use_texture, mat, tex);
+	}
+	else if (nr == OBJECT_SELECT_LINKED_DUPGROUP) {
+		if (ob->dup_group == NULL)
+			return OPERATOR_CANCELLED;
+
+		changed = object_select_all_by_dup_group(C, ob);
+	}
+	else if (nr == OBJECT_SELECT_LINKED_PARTICLE) {
+		if (ob->particlesystem.first == NULL)
+			return OPERATOR_CANCELLED;
+
+		changed = object_select_all_by_particle(C, ob);
+	}
+	else if (nr == OBJECT_SELECT_LINKED_LIBRARY) {
+		/* do nothing */
+		changed = object_select_all_by_library(C, ob->id.lib);
+	}
+	else if (nr == OBJECT_SELECT_LINKED_LIBRARY_OBDATA) {
+		if (ob->data == NULL)
+			return OPERATOR_CANCELLED;
+
+		changed = object_select_all_by_library_obdata(C, ((ID *) ob->data)->lib);
+	}
+	else
+		return OPERATOR_CANCELLED;
+
+	if (changed) {
+		WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, CTX_data_scene(C));
 		return OPERATOR_FINISHED;
 	}
 	
@@ -357,7 +504,7 @@ void OBJECT_OT_select_linked(wmOperatorType *ot)
 	ot->poll = objects_selectable_poll;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 	
 	/* properties */
 	RNA_def_boolean(ot->srna, "extend", FALSE, "Extend", "Extend selection instead of deselecting everything first");
@@ -379,6 +526,7 @@ static EnumPropertyItem prop_select_grouped_types[] = {
 	{10, "COLOR", 0, "Color", "Object Color"},
 	{11, "PROPERTIES", 0, "Properties", "Game Properties"},
 	{12, "KEYINGSET", 0, "Keying Set", "Objects included in active Keying Set"},
+	{13, "LAMP_TYPE", 0, "Lamp Type", "Matching lamp types"},
 	{0, NULL, 0, NULL, NULL}
 };
 
@@ -386,7 +534,8 @@ static short select_grouped_children(bContext *C, Object *ob, int recursive)
 {
 	short changed = 0;
 
-	CTX_DATA_BEGIN(C, Base*, base, selectable_bases) {
+	CTX_DATA_BEGIN (C, Base *, base, selectable_bases)
+	{
 		if (ob == base->object->parent) {
 			if (!(base->flag & SELECT)) {
 				ED_base_object_select(base, BA_SELECT);
@@ -401,17 +550,17 @@ static short select_grouped_children(bContext *C, Object *ob, int recursive)
 	return changed;
 }
 
-static short select_grouped_parent(bContext *C)	/* Makes parent active and de-selected OBACT */
+static short select_grouped_parent(bContext *C) /* Makes parent active and de-selected OBACT */
 {
-	Scene *scene= CTX_data_scene(C);
-	View3D *v3d= CTX_wm_view3d(C);
+	Scene *scene = CTX_data_scene(C);
+	View3D *v3d = CTX_wm_view3d(C);
 
 	short changed = 0;
-	Base *baspar, *basact= CTX_data_active_base(C);
+	Base *baspar, *basact = CTX_data_active_base(C);
 
-	if (!basact || !(basact->object->parent)) return 0; /* we know OBACT is valid */
+	if (!basact || !(basact->object->parent)) return 0;  /* we know OBACT is valid */
 
-	baspar= object_in_scene(basact->object->parent, scene);
+	baspar = BKE_scene_base_find(scene, basact->object->parent);
 
 	/* can be NULL if parent in other scene */
 	if (baspar && BASE_SELECTABLE(v3d, baspar)) {
@@ -424,17 +573,17 @@ static short select_grouped_parent(bContext *C)	/* Makes parent active and de-se
 }
 
 
-#define GROUP_MENU_MAX	24
-static short select_grouped_group(bContext *C, Object *ob)	/* Select objects in the same group as the active */
+#define GROUP_MENU_MAX  24
+static short select_grouped_group(bContext *C, Object *ob)  /* Select objects in the same group as the active */
 {
 	short changed = 0;
 	Group *group, *ob_groups[GROUP_MENU_MAX];
-	int group_count=0, i;
+	int group_count = 0, i;
 	uiPopupMenu *pup;
 	uiLayout *layout;
 
-	for (group=CTX_data_main(C)->group.first; group && group_count < GROUP_MENU_MAX; group=group->id.next) {
-		if (object_in_group (ob, group)) {
+	for (group = CTX_data_main(C)->group.first; group && group_count < GROUP_MENU_MAX; group = group->id.next) {
+		if (object_in_group(ob, group)) {
 			ob_groups[group_count] = group;
 			group_count++;
 		}
@@ -444,7 +593,8 @@ static short select_grouped_group(bContext *C, Object *ob)	/* Select objects in
 		return 0;
 	else if (group_count == 1) {
 		group = ob_groups[0];
-		CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
+		CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+		{
 			if (!(base->flag & SELECT) && object_in_group(base->object, group)) {
 				ED_base_object_select(base, BA_SELECT);
 				changed = 1;
@@ -455,33 +605,33 @@ static short select_grouped_group(bContext *C, Object *ob)	/* Select objects in
 	}
 
 	/* build the menu. */
-	pup= uiPupMenuBegin(C, "Select Group", ICON_NONE);
-	layout= uiPupMenuLayout(pup);
+	pup = uiPupMenuBegin(C, "Select Group", ICON_NONE);
+	layout = uiPupMenuLayout(pup);
 
-	for (i=0; i<group_count; i++) {
+	for (i = 0; i < group_count; i++) {
 		group = ob_groups[i];
-		uiItemStringO(layout, group->id.name+2, 0, "OBJECT_OT_select_same_group", "group", group->id.name);
+		uiItemStringO(layout, group->id.name + 2, 0, "OBJECT_OT_select_same_group", "group", group->id.name + 2);
 	}
 
 	uiPupMenuEnd(C, pup);
-	return changed; // The operator already handle this!
+	return changed;  /* The operator already handle this! */
 }
 
 static short select_grouped_object_hooks(bContext *C, Object *ob)
 {
-	Scene *scene= CTX_data_scene(C);
-	View3D *v3d= CTX_wm_view3d(C);
+	Scene *scene = CTX_data_scene(C);
+	View3D *v3d = CTX_wm_view3d(C);
 
 	short changed = 0;
 	Base *base;
 	ModifierData *md;
 	HookModifierData *hmd;
 
-	for (md = ob->modifiers.first; md; md=md->next) {
-		if (md->type==eModifierType_Hook) {
-			hmd= (HookModifierData*) md;
+	for (md = ob->modifiers.first; md; md = md->next) {
+		if (md->type == eModifierType_Hook) {
+			hmd = (HookModifierData *) md;
 			if (hmd->object && !(hmd->object->flag & SELECT)) {
-				base= object_in_scene(hmd->object, scene);
+				base = BKE_scene_base_find(scene, hmd->object);
 				if (base && (BASE_SELECTABLE(v3d, base))) {
 					ED_base_object_select(base, BA_SELECT);
 					changed = 1;
@@ -498,8 +648,9 @@ static short select_grouped_siblings(bContext *C, Object *ob)
 {
 	short changed = 0;
 
-	CTX_DATA_BEGIN(C, Base*, base, selectable_bases) {
-		if ((base->object->parent==ob->parent)  && !(base->flag & SELECT)) {
+	CTX_DATA_BEGIN (C, Base *, base, selectable_bases)
+	{
+		if ((base->object->parent == ob->parent) && !(base->flag & SELECT)) {
 			ED_base_object_select(base, BA_SELECT);
 			changed = 1;
 		}
@@ -507,12 +658,31 @@ static short select_grouped_siblings(bContext *C, Object *ob)
 	CTX_DATA_END;
 	return changed;
 }
+static short select_similar_lamps(bContext *C, Object *ob)
+{
+	Lamp *la = ob->data;
 
+	short changed = 0;
+
+	CTX_DATA_BEGIN (C, Base *, base, selectable_bases)
+	{
+		if (base->object->type == OB_LAMP) {
+			Lamp *la_test = base->object->data;
+			if ((la->type == la_test->type) && !(base->flag & SELECT)) {
+				ED_base_object_select(base, BA_SELECT);
+				changed = 1;
+			}
+		}
+	}
+	CTX_DATA_END;
+	return changed;
+}
 static short select_grouped_type(bContext *C, Object *ob)
 {
 	short changed = 0;
 
-	CTX_DATA_BEGIN(C, Base*, base, selectable_bases) {
+	CTX_DATA_BEGIN (C, Base *, base, selectable_bases)
+	{
 		if ((base->object->type == ob->type) && !(base->flag & SELECT)) {
 			ED_base_object_select(base, BA_SELECT);
 			changed = 1;
@@ -526,7 +696,8 @@ static short select_grouped_layer(bContext *C, Object *ob)
 {
 	char changed = 0;
 
-	CTX_DATA_BEGIN(C, Base*, base, selectable_bases) {
+	CTX_DATA_BEGIN (C, Base *, base, selectable_bases)
+	{
 		if ((base->lay & ob->lay) && !(base->flag & SELECT)) {
 			ED_base_object_select(base, BA_SELECT);
 			changed = 1;
@@ -540,7 +711,8 @@ static short select_grouped_index_object(bContext *C, Object *ob)
 {
 	char changed = 0;
 
-	CTX_DATA_BEGIN(C, Base*, base, selectable_bases) {
+	CTX_DATA_BEGIN (C, Base *, base, selectable_bases)
+	{
 		if ((base->object->index == ob->index) && !(base->flag & SELECT)) {
 			ED_base_object_select(base, BA_SELECT);
 			changed = 1;
@@ -554,7 +726,8 @@ static short select_grouped_color(bContext *C, Object *ob)
 {
 	char changed = 0;
 
-	CTX_DATA_BEGIN(C, Base*, base, selectable_bases) {
+	CTX_DATA_BEGIN (C, Base *, base, selectable_bases)
+	{
 		if (!(base->flag & SELECT) && (compare_v3v3(base->object->col, ob->col, 0.005f))) {
 			ED_base_object_select(base, BA_SELECT);
 			changed = 1;
@@ -569,8 +742,8 @@ static short objects_share_gameprop(Object *a, Object *b)
 	bProperty *prop;
 	/*make a copy of all its properties*/
 
-	for ( prop= a->prop.first; prop; prop = prop->next ) {
-		if ( get_ob_property(b, prop->name) )
+	for (prop = a->prop.first; prop; prop = prop->next) {
+		if (BKE_bproperty_object_get(b, prop->name) )
 			return 1;
 	}
 	return 0;
@@ -580,7 +753,8 @@ static short select_grouped_gameprops(bContext *C, Object *ob)
 {
 	char changed = 0;
 
-	CTX_DATA_BEGIN(C, Base*, base, selectable_bases) {
+	CTX_DATA_BEGIN (C, Base *, base, selectable_bases)
+	{
 		if (!(base->flag & SELECT) && (objects_share_gameprop(base->object, ob))) {
 			ED_base_object_select(base, BA_SELECT);
 			changed = 1;
@@ -600,9 +774,9 @@ static short select_grouped_keyingset(bContext *C, Object *UNUSED(ob))
 		return 0;
 	
 	/* select each object that Keying Set refers to */
-	// TODO: perhaps to be more in line with the rest of these, we should only take objects 
-	// if the passed in object is included in this too
-	CTX_DATA_BEGIN(C, Base*, base, selectable_bases) 
+	/* TODO: perhaps to be more in line with the rest of these, we should only take objects
+	 * if the passed in object is included in this too */
+	CTX_DATA_BEGIN (C, Base *, base, selectable_bases)
 	{
 		/* only check for this object if it isn't selected already, to limit time wasted */
 		if ((base->flag & SELECT) == 0) {
@@ -628,42 +802,49 @@ static short select_grouped_keyingset(bContext *C, Object *UNUSED(ob))
 
 static int object_select_grouped_exec(bContext *C, wmOperator *op)
 {
-	Scene *scene= CTX_data_scene(C);
+	Scene *scene = CTX_data_scene(C);
 	Object *ob;
 	int nr = RNA_enum_get(op->ptr, "type");
 	short changed = 0, extend;
 
-	extend= RNA_boolean_get(op->ptr, "extend");
+	extend = RNA_boolean_get(op->ptr, "extend");
 	
 	if (extend == 0) {
-		CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
+		CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+		{
 			ED_base_object_select(base, BA_DESELECT);
 			changed = 1;
 		}
 		CTX_DATA_END;
 	}
 	
-	ob= OBACT;
-	if (ob==NULL) { 
-		BKE_report(op->reports, RPT_ERROR, "No Active Object");
+	ob = OBACT;
+	if (ob == NULL) {
+		BKE_report(op->reports, RPT_ERROR, "No active object");
 		return OPERATOR_CANCELLED;
 	}
-	
-	if (nr==1)		changed |= select_grouped_children(C, ob, 1);
-	else if (nr==2)	changed |= select_grouped_children(C, ob, 0);
-	else if (nr==3)	changed |= select_grouped_parent(C);
-	else if (nr==4)	changed |= select_grouped_siblings(C, ob);
-	else if (nr==5)	changed |= select_grouped_type(C, ob);
-	else if (nr==6)	changed |= select_grouped_layer(C, ob);
-	else if (nr==7)	changed |= select_grouped_group(C, ob);
-	else if (nr==8)	changed |= select_grouped_object_hooks(C, ob);
-	else if (nr==9)	changed |= select_grouped_index_object(C, ob);
-	else if (nr==10)	changed |= select_grouped_color(C, ob);
-	else if (nr==11)	changed |= select_grouped_gameprops(C, ob);
-	else if (nr==12) changed |= select_grouped_keyingset(C, ob);
+
+	if (nr == 13 && ob->type != OB_LAMP) {
+		BKE_report(op->reports, RPT_ERROR, "Active object must be a lamp");
+		return OPERATOR_CANCELLED;
+	}
+
+	if      (nr == 1) changed |= select_grouped_children(C, ob, 1);
+	else if (nr == 2) changed |= select_grouped_children(C, ob, 0);
+	else if (nr == 3) changed |= select_grouped_parent(C);
+	else if (nr == 4) changed |= select_grouped_siblings(C, ob);
+	else if (nr == 5) changed |= select_grouped_type(C, ob);
+	else if (nr == 6) changed |= select_grouped_layer(C, ob);
+	else if (nr == 7) changed |= select_grouped_group(C, ob);
+	else if (nr == 8) changed |= select_grouped_object_hooks(C, ob);
+	else if (nr == 9) changed |= select_grouped_index_object(C, ob);
+	else if (nr == 10) changed |= select_grouped_color(C, ob);
+	else if (nr == 11) changed |= select_grouped_gameprops(C, ob);
+	else if (nr == 12) changed |= select_grouped_keyingset(C, ob);
+	else if (nr == 13) changed |= select_similar_lamps(C, ob);
 	
 	if (changed) {
-		WM_event_add_notifier(C, NC_SCENE|ND_OB_SELECT, CTX_data_scene(C));
+		WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, CTX_data_scene(C));
 		return OPERATOR_FINISHED;
 	}
 	
@@ -683,7 +864,7 @@ void OBJECT_OT_select_grouped(wmOperatorType *ot)
 	ot->poll = objects_selectable_poll;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 	
 	/* properties */
 	RNA_def_boolean(ot->srna, "extend", FALSE, "Extend", "Extend selection instead of deselecting everything first");
@@ -695,32 +876,48 @@ void OBJECT_OT_select_grouped(wmOperatorType *ot)
 static int object_select_by_layer_exec(bContext *C, wmOperator *op)
 {
 	unsigned int layernum;
-	short extend;
+	short extend, match;
 	
-	extend= RNA_boolean_get(op->ptr, "extend");
+	extend = RNA_boolean_get(op->ptr, "extend");
 	layernum = RNA_int_get(op->ptr, "layers");
+	match = RNA_enum_get(op->ptr, "match");
 	
 	if (extend == 0) {
-		CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
+		CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+		{
 			ED_base_object_select(base, BA_DESELECT);
 		}
 		CTX_DATA_END;
 	}
 		
-	CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
-		if (base->lay == (1<< (layernum -1)))
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
+		int ok = 0;
+
+		if (match == 1) /* exact */
+			ok = (base->lay == (1 << (layernum - 1)));
+		else /* shared layers */
+			ok = (base->lay & (1 << (layernum - 1)));
+
+		if (ok)
 			ED_base_object_select(base, BA_SELECT);
 	}
 	CTX_DATA_END;
 	
 	/* undo? */
-	WM_event_add_notifier(C, NC_SCENE|ND_OB_SELECT, CTX_data_scene(C));
+	WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, CTX_data_scene(C));
 	
 	return OPERATOR_FINISHED;
 }
 
 void OBJECT_OT_select_by_layer(wmOperatorType *ot)
 {
+	static EnumPropertyItem match_items[] = {
+		{1, "EXACT", 0, "Exact Match", ""},
+		{2, "SHARED", 0, "Shared Layers", ""},
+		{0, NULL, 0, NULL, NULL}
+	};
+
 	/* identifiers */
 	ot->name = "Select by Layer";
 	ot->description = "Select all visible objects on a layer";
@@ -732,9 +929,10 @@ void OBJECT_OT_select_by_layer(wmOperatorType *ot)
 	ot->poll = objects_selectable_poll;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 	
 	/* properties */
+	RNA_def_enum(ot->srna, "match", match_items, 0, "Match", "");
 	RNA_def_boolean(ot->srna, "extend", FALSE, "Extend", "Extend selection instead of deselecting everything first");
 	RNA_def_int(ot->srna, "layers", 1, 1, 20, "Layer", "", 1, 20);
 }
@@ -750,7 +948,8 @@ static int object_select_all_exec(bContext *C, wmOperator *op)
 
 	if (action == SEL_TOGGLE) {
 		action = SEL_SELECT;
-		CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
+		CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+		{
 			if (base->flag & SELECT) {
 				action = SEL_DESELECT;
 				break;
@@ -759,27 +958,28 @@ static int object_select_all_exec(bContext *C, wmOperator *op)
 		CTX_DATA_END;
 	}
 
-	CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
 		switch (action) {
-		case SEL_SELECT:
-			ED_base_object_select(base, BA_SELECT);
-			break;
-		case SEL_DESELECT:
-			ED_base_object_select(base, BA_DESELECT);
-			break;
-		case SEL_INVERT:
-			if (base->flag & SELECT) {
-				ED_base_object_select(base, BA_DESELECT);
-			}
-			else {
+			case SEL_SELECT:
 				ED_base_object_select(base, BA_SELECT);
-			}
-			break;
+				break;
+			case SEL_DESELECT:
+				ED_base_object_select(base, BA_DESELECT);
+				break;
+			case SEL_INVERT:
+				if (base->flag & SELECT) {
+					ED_base_object_select(base, BA_DESELECT);
+				}
+				else {
+					ED_base_object_select(base, BA_SELECT);
+				}
+				break;
 		}
 	}
 	CTX_DATA_END;
 	
-	WM_event_add_notifier(C, NC_SCENE|ND_OB_SELECT, CTX_data_scene(C));
+	WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, CTX_data_scene(C));
 	
 	return OPERATOR_FINISHED;
 }
@@ -797,7 +997,7 @@ void OBJECT_OT_select_all(wmOperatorType *ot)
 	ot->poll = objects_selectable_poll;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
 	WM_operator_properties_select_all(ot);
 }
@@ -814,21 +1014,20 @@ static int object_select_same_group_exec(bContext *C, wmOperator *op)
 
 	RNA_string_get(op->ptr, "group", group_name);
 
-	for (group=CTX_data_main(C)->group.first;	group; group=group->id.next) {
-		if (!strcmp(group->id.name, group_name))
-			break;
+	group = (Group *)BKE_libblock_find_name(ID_GR, group_name);
+
+	if (!group) {
+		return OPERATOR_PASS_THROUGH;
 	}
 
-	if (!group)
-		return OPERATOR_PASS_THROUGH;
-
-	CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
 		if (!(base->flag & SELECT) && object_in_group(base->object, group))
 			ED_base_object_select(base, BA_SELECT);
 	}
 	CTX_DATA_END;
 
-	WM_event_add_notifier(C, NC_SCENE|ND_OB_SELECT, CTX_data_scene(C));
+	WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, CTX_data_scene(C));
 	
 	return OPERATOR_FINISHED;
 }
@@ -846,7 +1045,7 @@ void OBJECT_OT_select_same_group(wmOperatorType *ot)
 	ot->poll = objects_selectable_poll;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
 	RNA_def_string(ot->srna, "group", "", MAX_ID_NAME, "Group", "Name of the group to select");
 }
@@ -854,20 +1053,21 @@ void OBJECT_OT_select_same_group(wmOperatorType *ot)
 /**************************** Select Mirror ****************************/
 static int object_select_mirror_exec(bContext *C, wmOperator *op)
 {
-	Scene *scene= CTX_data_scene(C);
+	Scene *scene = CTX_data_scene(C);
 	short extend;
 	
-	extend= RNA_boolean_get(op->ptr, "extend");
+	extend = RNA_boolean_get(op->ptr, "extend");
 	
-	CTX_DATA_BEGIN(C, Base*, primbase, selected_bases) {
+	CTX_DATA_BEGIN (C, Base *, primbase, selected_bases)
+	{
 		char tmpname[MAXBONENAME];
 
-		flip_side_name(tmpname, primbase->object->id.name+2, TRUE);
+		flip_side_name(tmpname, primbase->object->id.name + 2, TRUE);
 		
-		if (strcmp(tmpname, primbase->object->id.name+2)!=0) { /* names differ */
-			Object *ob= (Object *)find_id("OB", tmpname);
+		if (strcmp(tmpname, primbase->object->id.name + 2) != 0) { /* names differ */
+			Object *ob = (Object *)BKE_libblock_find_name(ID_OB, tmpname);
 			if (ob) {
-				Base *secbase= object_in_scene(ob, scene);
+				Base *secbase = BKE_scene_base_find(scene, ob);
 
 				if (secbase) {
 					ED_base_object_select(secbase, BA_SELECT);
@@ -881,7 +1081,7 @@ static int object_select_mirror_exec(bContext *C, wmOperator *op)
 	CTX_DATA_END;
 	
 	/* undo? */
-	WM_event_add_notifier(C, NC_SCENE|ND_OB_SELECT, CTX_data_scene(C));
+	WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, CTX_data_scene(C));
 	
 	return OPERATOR_FINISHED;
 }
@@ -899,7 +1099,7 @@ void OBJECT_OT_select_mirror(wmOperatorType *ot)
 	ot->poll = objects_selectable_poll;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 	
 	RNA_def_boolean(ot->srna, "extend", 0, "Extend", "Extend selection instead of deselecting everything first");
 }
@@ -912,24 +1112,26 @@ static int object_select_random_exec(bContext *C, wmOperator *op)
 	float percent;
 	short extend;
 	
-	extend= RNA_boolean_get(op->ptr, "extend");
+	extend = RNA_boolean_get(op->ptr, "extend");
 	
 	if (extend == 0) {
-		CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
+		CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+		{
 			ED_base_object_select(base, BA_DESELECT);
 		}
 		CTX_DATA_END;
 	}
-	percent = RNA_float_get(op->ptr, "percent")/100.0f;
+	percent = RNA_float_get(op->ptr, "percent") / 100.0f;
 		
-	CTX_DATA_BEGIN(C, Base*, base, visible_bases) {
+	CTX_DATA_BEGIN (C, Base *, base, visible_bases)
+	{
 		if (BLI_frand() < percent) {
 			ED_base_object_select(base, BA_SELECT);
 		}
 	}
 	CTX_DATA_END;
 	
-	WM_event_add_notifier(C, NC_SCENE|ND_OB_SELECT, CTX_data_scene(C));
+	WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, CTX_data_scene(C));
 	
 	return OPERATOR_FINISHED;
 }
@@ -947,11 +1149,9 @@ void OBJECT_OT_select_random(wmOperatorType *ot)
 	ot->poll = objects_selectable_poll;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 	
 	/* properties */
 	RNA_def_float_percentage(ot->srna, "percent", 50.f, 0.0f, 100.0f, "Percent", "Percentage of objects to select randomly", 0.f, 100.0f);
 	RNA_def_boolean(ot->srna, "extend", FALSE, "Extend Selection", "Extend selection instead of deselecting everything first");
 }
-
-
