@@ -56,23 +56,18 @@
 /************************ add/del boid rule operators *********************/
 static int rule_add_exec(bContext *C, wmOperator *op)
 {
-	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_system", &RNA_ParticleSystem);
-	ParticleSystem *psys= ptr.data;
-	Object *ob= ptr.id.data;
-	ParticleSettings *part;
+	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_settings", &RNA_ParticleSettings);
+	ParticleSettings *part = ptr.data;
 	int type= RNA_enum_get(op->ptr, "type");
 
 	BoidRule *rule;
 	BoidState *state;
 
-	if (!psys || !psys->part || psys->part->phystype != PART_PHYS_BOIDS)
+	if (!part || part->phystype != PART_PHYS_BOIDS)
 		return OPERATOR_CANCELLED;
 
-	part = psys->part;
-
 	state = boid_get_current_state(part->boids);
 
-
 	for (rule=state->rules.first; rule; rule=rule->next)
 		rule->flag &= ~BOIDRULE_CURRENT;
 
@@ -82,7 +77,6 @@ static int rule_add_exec(bContext *C, wmOperator *op)
 	BLI_addtail(&state->rules, rule);
 
 	DAG_id_tag_update(&part->id, OB_RECALC_DATA|PSYS_RECALC_RESET);
-	WM_event_add_notifier(C, NC_OBJECT|ND_DRAW, ob);
 	
 	return OPERATOR_FINISHED;
 }
@@ -107,25 +101,22 @@ static int rule_del_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	Main *bmain = CTX_data_main(C);
 	Scene *scene = CTX_data_scene(C);
-	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_system", &RNA_ParticleSystem);
-	ParticleSystem *psys= ptr.data;
-	Object *ob = ptr.id.data;
+	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_settings", &RNA_ParticleSettings);
+	ParticleSettings *part = ptr.data;
 	BoidRule *rule;
 	BoidState *state;
 
-	if (!psys || !psys->part || psys->part->phystype != PART_PHYS_BOIDS)
+	if (!part || part->phystype != PART_PHYS_BOIDS)
 		return OPERATOR_CANCELLED;
 
-	state = boid_get_current_state(psys->part->boids);
+	state = boid_get_current_state(part->boids);
 
-	
 	for (rule=state->rules.first; rule; rule=rule->next) {
 		if (rule->flag & BOIDRULE_CURRENT) {
 			BLI_remlink(&state->rules, rule);
 			MEM_freeN(rule);
 			break;
 		}
-
 	}
 	rule = state->rules.first;
 
@@ -133,10 +124,8 @@ static int rule_del_exec(bContext *C, wmOperator *UNUSED(op))
 		rule->flag |= BOIDRULE_CURRENT;
 
 	DAG_scene_sort(bmain, scene);
-	DAG_id_tag_update(&psys->part->id, OB_RECALC_DATA|PSYS_RECALC_RESET);
+	DAG_id_tag_update(&part->id, OB_RECALC_DATA|PSYS_RECALC_RESET);
 
-	WM_event_add_notifier(C, NC_OBJECT|ND_DRAW, ob);
-	
 	return OPERATOR_FINISHED;
 }
 
@@ -145,6 +134,7 @@ void BOID_OT_rule_del(wmOperatorType *ot)
 	/* identifiers */
 	ot->name = "Remove Boid Rule";
 	ot->idname = "BOID_OT_rule_del";
+	ot->description = "Delete current boid rule";
 	
 	/* api callbacks */
 	ot->exec = rule_del_exec;
@@ -156,23 +146,21 @@ void BOID_OT_rule_del(wmOperatorType *ot)
 /************************ move up/down boid rule operators *********************/
 static int rule_move_up_exec(bContext *C, wmOperator *UNUSED(op))
 {
-	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_system", &RNA_ParticleSystem);
-	ParticleSystem *psys= ptr.data;
-	Object *ob = ptr.id.data;
+	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_settings", &RNA_ParticleSettings);
+	ParticleSettings *part = ptr.data;
 	BoidRule *rule;
 	BoidState *state;
 
-	if (!psys || !psys->part || psys->part->phystype != PART_PHYS_BOIDS)
+	if (!part || part->phystype != PART_PHYS_BOIDS)
 		return OPERATOR_CANCELLED;
 	
-	state = boid_get_current_state(psys->part->boids);
+	state = boid_get_current_state(part->boids);
 	for (rule = state->rules.first; rule; rule=rule->next) {
 		if (rule->flag & BOIDRULE_CURRENT && rule->prev) {
 			BLI_remlink(&state->rules, rule);
 			BLI_insertlink(&state->rules, rule->prev->prev, rule);
 
-			DAG_id_tag_update(&psys->part->id, OB_RECALC_DATA|PSYS_RECALC_RESET);
-			WM_event_add_notifier(C, NC_OBJECT|ND_DRAW, ob);
+			DAG_id_tag_update(&part->id, OB_RECALC_DATA|PSYS_RECALC_RESET);
 			break;
 		}
 	}
@@ -194,23 +182,21 @@ void BOID_OT_rule_move_up(wmOperatorType *ot)
 
 static int rule_move_down_exec(bContext *C, wmOperator *UNUSED(op))
 {
-	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_system", &RNA_ParticleSystem);
-	ParticleSystem *psys= ptr.data;
-	Object *ob = ptr.id.data;
+	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_settings", &RNA_ParticleSettings);
+	ParticleSettings *part = ptr.data;
 	BoidRule *rule;
 	BoidState *state;
 
-	if (!psys || !psys->part || psys->part->phystype != PART_PHYS_BOIDS)
+	if (!part || part->phystype != PART_PHYS_BOIDS)
 		return OPERATOR_CANCELLED;
 	
-	state = boid_get_current_state(psys->part->boids);
+	state = boid_get_current_state(part->boids);
 	for (rule = state->rules.first; rule; rule=rule->next) {
 		if (rule->flag & BOIDRULE_CURRENT && rule->next) {
 			BLI_remlink(&state->rules, rule);
 			BLI_insertlink(&state->rules, rule->next, rule);
 
-			DAG_id_tag_update(&psys->part->id, OB_RECALC_DATA|PSYS_RECALC_RESET);
-			WM_event_add_notifier(C, NC_OBJECT|ND_DRAW, ob);
+			DAG_id_tag_update(&part->id, OB_RECALC_DATA|PSYS_RECALC_RESET);
 			break;
 		}
 	}
@@ -234,17 +220,13 @@ void BOID_OT_rule_move_down(wmOperatorType *ot)
 /************************ add/del boid state operators *********************/
 static int state_add_exec(bContext *C, wmOperator *UNUSED(op))
 {
-	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_system", &RNA_ParticleSystem);
-	ParticleSystem *psys= ptr.data;
-	Object *ob= ptr.id.data;
-	ParticleSettings *part;
+	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_settings", &RNA_ParticleSettings);
+	ParticleSettings *part = ptr.data;
 	BoidState *state;
 
-	if (!psys || !psys->part || psys->part->phystype != PART_PHYS_BOIDS)
+	if (!part || part->phystype != PART_PHYS_BOIDS)
 		return OPERATOR_CANCELLED;
 
-	part = psys->part;
-
 	for (state=part->boids->states.first; state; state=state->next)
 		state->flag &= ~BOIDSTATE_CURRENT;
 
@@ -253,8 +235,6 @@ static int state_add_exec(bContext *C, wmOperator *UNUSED(op))
 
 	BLI_addtail(&part->boids->states, state);
 
-	WM_event_add_notifier(C, NC_OBJECT|ND_DRAW, ob);
-	
 	return OPERATOR_FINISHED;
 }
 
@@ -275,30 +255,25 @@ static int state_del_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	Main *bmain = CTX_data_main(C);
 	Scene *scene = CTX_data_scene(C);
-	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_system", &RNA_ParticleSystem);
-	ParticleSystem *psys= ptr.data;
-	Object *ob = ptr.id.data;
-	ParticleSettings *part;
+	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_settings", &RNA_ParticleSettings);
+	ParticleSettings *part = ptr.data;
 	BoidState *state;
 
-	if (!psys || !psys->part || psys->part->phystype != PART_PHYS_BOIDS)
+	if (!part || part->phystype != PART_PHYS_BOIDS)
 		return OPERATOR_CANCELLED;
 
-	part = psys->part;
-	
 	for (state=part->boids->states.first; state; state=state->next) {
 		if (state->flag & BOIDSTATE_CURRENT) {
 			BLI_remlink(&part->boids->states, state);
 			MEM_freeN(state);
 			break;
 		}
-
 	}
 
 	/* there must be at least one state */
 	if (!part->boids->states.first) {
 		state = boid_new_state(part->boids);
-		BLI_addtail(&part->boids->states, state);	
+		BLI_addtail(&part->boids->states, state);
 	}
 	else
 		state = part->boids->states.first;
@@ -306,9 +281,7 @@ static int state_del_exec(bContext *C, wmOperator *UNUSED(op))
 	state->flag |= BOIDSTATE_CURRENT;
 
 	DAG_scene_sort(bmain, scene);
-	DAG_id_tag_update(&psys->part->id, OB_RECALC_DATA|PSYS_RECALC_RESET);
-
-	WM_event_add_notifier(C, NC_OBJECT|ND_DRAW, ob);
+	DAG_id_tag_update(&part->id, OB_RECALC_DATA|PSYS_RECALC_RESET);
 	
 	return OPERATOR_FINISHED;
 }
@@ -318,6 +291,7 @@ void BOID_OT_state_del(wmOperatorType *ot)
 	/* identifiers */
 	ot->name = "Remove Boid State";
 	ot->idname = "BOID_OT_state_del";
+	ot->description = "Delete current boid state";
 	
 	/* api callbacks */
 	ot->exec = state_del_exec;
@@ -329,22 +303,20 @@ void BOID_OT_state_del(wmOperatorType *ot)
 /************************ move up/down boid state operators *********************/
 static int state_move_up_exec(bContext *C, wmOperator *UNUSED(op))
 {
-	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_system", &RNA_ParticleSystem);
-	ParticleSystem *psys= ptr.data;
-	Object *ob = ptr.id.data;
+	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_settings", &RNA_ParticleSettings);
+	ParticleSettings *part = ptr.data;
 	BoidSettings *boids;
 	BoidState *state;
 
-	if (!psys || !psys->part || psys->part->phystype != PART_PHYS_BOIDS)
+	if (!part || part->phystype != PART_PHYS_BOIDS)
 		return OPERATOR_CANCELLED;
 
-	boids = psys->part->boids;
+	boids = part->boids;
 	
 	for (state = boids->states.first; state; state=state->next) {
 		if (state->flag & BOIDSTATE_CURRENT && state->prev) {
 			BLI_remlink(&boids->states, state);
 			BLI_insertlink(&boids->states, state->prev->prev, state);
-			WM_event_add_notifier(C, NC_OBJECT|ND_DRAW, ob);
 			break;
 		}
 	}
@@ -366,21 +338,21 @@ void BOID_OT_state_move_up(wmOperatorType *ot)
 
 static int state_move_down_exec(bContext *C, wmOperator *UNUSED(op))
 {
-	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_system", &RNA_ParticleSystem);
-	ParticleSystem *psys= ptr.data;
+	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_settings", &RNA_ParticleSettings);
+	ParticleSettings *part = ptr.data;
 	BoidSettings *boids;
 	BoidState *state;
 
-	if (!psys || !psys->part || psys->part->phystype != PART_PHYS_BOIDS)
+	if (!part || part->phystype != PART_PHYS_BOIDS)
 		return OPERATOR_CANCELLED;
 
-	boids = psys->part->boids;
+	boids = part->boids;
 	
 	for (state = boids->states.first; state; state=state->next) {
 		if (state->flag & BOIDSTATE_CURRENT && state->next) {
 			BLI_remlink(&boids->states, state);
 			BLI_insertlink(&boids->states, state->next, state);
-			DAG_id_tag_update(&psys->part->id, OB_RECALC_DATA|PSYS_RECALC_RESET);
+			DAG_id_tag_update(&part->id, OB_RECALC_DATA|PSYS_RECALC_RESET);
 			break;
 		}
 	}
