@@ -49,6 +49,7 @@
 #include "BKE_main.h"
 #include "BKE_particle.h"
 #include "BKE_pointcache.h"
+#include "BKE_report.h"
 
 
 #include "RNA_access.h"
@@ -98,21 +99,26 @@ void OBJECT_OT_particle_system_add(wmOperatorType *ot)
 
 static int particle_system_remove_exec(bContext *C, wmOperator *UNUSED(op))
 {
-	Object *ob= ED_object_context(C);
+	Object *ob = ED_object_context(C);
 	Scene *scene = CTX_data_scene(C);
-	int mode_orig = ob->mode;
+	int mode_orig;
+
 	if (!scene || !ob)
 		return OPERATOR_CANCELLED;
 
+	mode_orig = ob->mode;
 	object_remove_particle_system(scene, ob);
 
 	/* possible this isn't the active object
 	 * object_remove_particle_system() clears the mode on the last psys
-	 * */
-	if (mode_orig & OB_MODE_PARTICLE_EDIT)
-		if ((ob->mode & OB_MODE_PARTICLE_EDIT)==0)
-			if (scene->basact && scene->basact->object==ob)
+	 */
+	if (mode_orig & OB_MODE_PARTICLE_EDIT) {
+		if ((ob->mode & OB_MODE_PARTICLE_EDIT) == 0) {
+			if (scene->basact && scene->basact->object == ob) {
 				WM_event_add_notifier(C, NC_SCENE|ND_MODE|NS_MODE_OBJECT, NULL);
+			}
+		}
+	}
 
 	WM_event_add_notifier(C, NC_OBJECT|ND_PARTICLE, ob);
 	WM_event_add_notifier(C, NC_OBJECT|ND_POINTCACHE, ob);
@@ -158,7 +164,7 @@ static int new_particle_settings_exec(bContext *C, wmOperator *UNUSED(op))
 
 	/* add or copy particle setting */
 	if (psys->part)
-		part= psys_copy_settings(psys->part);
+		part= BKE_particlesettings_copy(psys->part);
 	else
 		part= psys_new_settings("ParticleSettings", bmain);
 
@@ -532,7 +538,7 @@ void PARTICLE_OT_dupliob_move_down(wmOperatorType *ot)
 
 static void disconnect_hair(Scene *scene, Object *ob, ParticleSystem *psys)
 {
-	ParticleSystemModifierData *psmd = psys_get_modifier(ob,psys);
+	ParticleSystemModifierData *psmd = psys_get_modifier(ob, psys);
 	ParticleEditSettings *pset= PE_settings(scene);
 	ParticleData *pa;
 	PTCacheEdit *edit;
@@ -551,7 +557,7 @@ static void disconnect_hair(Scene *scene, Object *ob, ParticleSystem *psys)
 	edit = psys->edit;
 	point= edit ? edit->points : NULL;
 
-	for (i=0, pa=psys->particles; i<psys->totpart; i++,pa++) {
+	for (i=0, pa=psys->particles; i<psys->totpart; i++, pa++) {
 		if (point) {
 			ekey = point->keys;
 			point++;
@@ -559,8 +565,8 @@ static void disconnect_hair(Scene *scene, Object *ob, ParticleSystem *psys)
 
 		psys_mat_hair_to_global(ob, psmd->dm, psys->part->from, pa, hairmat);
 
-		for (k=0,key=pa->hair; k<pa->totkey; k++,key++) {
-			mul_m4_v3(hairmat,key->co);
+		for (k=0, key=pa->hair; k<pa->totkey; k++, key++) {
+			mul_m4_v3(hairmat, key->co);
 			
 			if (ekey) {
 				ekey->flag &= ~PEK_USE_WCO;
@@ -620,9 +626,9 @@ void PARTICLE_OT_disconnect_hair(wmOperatorType *ot)
 	RNA_def_boolean(ot->srna, "all", 0, "All hair", "Disconnect all hair systems from the emitter mesh");
 }
 
-static void connect_hair(Scene *scene, Object *ob, ParticleSystem *psys)
+static int connect_hair(Scene *scene, Object *ob, ParticleSystem *psys)
 {
-	ParticleSystemModifierData *psmd = psys_get_modifier(ob,psys);
+	ParticleSystemModifierData *psmd = psys_get_modifier(ob, psys);
 	ParticleData *pa;
 	PTCacheEdit *edit;
 	PTCacheEditPoint *point;
@@ -637,8 +643,8 @@ static void connect_hair(Scene *scene, Object *ob, ParticleSystem *psys)
 	float hairmat[4][4], imat[4][4];
 	float v[4][3], vec[3];
 
-	if (!psys || !psys->part || psys->part->type != PART_HAIR)
-		return;
+	if (!psys || !psys->part || psys->part->type != PART_HAIR || !psmd->dm)
+		return FALSE;
 	
 	edit= psys->edit;
 	point=  edit ? edit->points : NULL;
@@ -661,7 +667,7 @@ static void connect_hair(Scene *scene, Object *ob, ParticleSystem *psys)
 
 	bvhtree_from_mesh_faces(&bvhtree, dm, 0.0, 2, 6);
 
-	for (i=0, pa= psys->particles; i<psys->totpart; i++,pa++) {
+	for (i=0, pa= psys->particles; i<psys->totpart; i++, pa++) {
 		key = pa->hair;
 
 		nearest.index = -1;
@@ -675,23 +681,23 @@ static void connect_hair(Scene *scene, Object *ob, ParticleSystem *psys)
 			continue;
 		}
 
-		mface = CDDM_get_tessface(dm,nearest.index);
+		mface = CDDM_get_tessface(dm, nearest.index);
 
-		copy_v3_v3(v[0], CDDM_get_vert(dm,mface->v1)->co);
-		copy_v3_v3(v[1], CDDM_get_vert(dm,mface->v2)->co);
-		copy_v3_v3(v[2], CDDM_get_vert(dm,mface->v3)->co);
+		copy_v3_v3(v[0], CDDM_get_vert(dm, mface->v1)->co);
+		copy_v3_v3(v[1], CDDM_get_vert(dm, mface->v2)->co);
+		copy_v3_v3(v[2], CDDM_get_vert(dm, mface->v3)->co);
 		if (mface->v4) {
-			copy_v3_v3(v[3], CDDM_get_vert(dm,mface->v4)->co);
-			interp_weights_poly_v3( pa->fuv,v, 4, nearest.co);
+			copy_v3_v3(v[3], CDDM_get_vert(dm, mface->v4)->co);
+			interp_weights_poly_v3(pa->fuv, v, 4, nearest.co);
 		}
 		else
-			interp_weights_poly_v3( pa->fuv,v, 3, nearest.co);
+			interp_weights_poly_v3(pa->fuv, v, 3, nearest.co);
 
 		pa->num = nearest.index;
-		pa->num_dmcache = psys_particle_dm_face_lookup(ob,psmd->dm,pa->num,pa->fuv,NULL);
+		pa->num_dmcache = psys_particle_dm_face_lookup(ob, psmd->dm, pa->num, pa->fuv, NULL);
 		
 		psys_mat_hair_to_global(ob, psmd->dm, psys->part->from, pa, hairmat);
-		invert_m4_m4(imat,hairmat);
+		invert_m4_m4(imat, hairmat);
 
 		sub_v3_v3v3(vec, nearest.co, key->co);
 
@@ -700,9 +706,9 @@ static void connect_hair(Scene *scene, Object *ob, ParticleSystem *psys)
 			point++;
 		}
 
-		for (k=0,key=pa->hair; k<pa->totkey; k++,key++) {
+		for (k=0, key=pa->hair; k<pa->totkey; k++, key++) {
 			add_v3_v3(key->co, vec);
-			mul_m4_v3(imat,key->co);
+			mul_m4_v3(imat, key->co);
 
 			if (ekey) {
 				ekey->flag |= PEK_USE_WCO;
@@ -719,6 +725,8 @@ static void connect_hair(Scene *scene, Object *ob, ParticleSystem *psys)
 	psys->flag &= ~PSYS_GLOBAL_HAIR;
 
 	PE_update_object(scene, ob, 0);
+
+	return TRUE;
 }
 
 static int connect_hair_exec(bContext *C, wmOperator *op)
@@ -728,18 +736,24 @@ static int connect_hair_exec(bContext *C, wmOperator *op)
 	PointerRNA ptr = CTX_data_pointer_get_type(C, "particle_system", &RNA_ParticleSystem);
 	ParticleSystem *psys= NULL;
 	int all = RNA_boolean_get(op->ptr, "all");
+	int any_connected = FALSE;
 
 	if (!ob)
 		return OPERATOR_CANCELLED;
 
 	if (all) {
 		for (psys=ob->particlesystem.first; psys; psys=psys->next) {
-			connect_hair(scene, ob, psys);
+			any_connected |= connect_hair(scene, ob, psys);
 		}
 	}
 	else {
 		psys = ptr.data;
-		connect_hair(scene, ob, psys);
+		any_connected |= connect_hair(scene, ob, psys);
+	}
+
+	if (!any_connected) {
+		BKE_report(op->reports, RPT_ERROR, "Can't disconnect hair if particle system modifier is disabled");
+		return OPERATOR_CANCELLED;
 	}
 
 	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
