@@ -1,7 +1,4 @@
 /*
- * fluidsim.c
- * 
- *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -32,9 +29,6 @@
  *  \ingroup edphys
  */
 
-
-
-
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
@@ -132,8 +126,8 @@ static float get_fluid_size_m(Scene *scene, Object *domainob, FluidsimSettings *
 		float dim[3];
 		float longest_axis;
 		
-		object_get_dimensions(domainob, dim);
-		longest_axis = MAX3(dim[0], dim[1], dim[2]);
+		BKE_object_dimensions_get(domainob, dim);
+		longest_axis = max_fff(dim[0], dim[1], dim[2]);
 		
 		return longest_axis * scene->unit.scale_length;
 	}
@@ -148,10 +142,10 @@ static int fluid_is_animated_mesh(FluidsimSettings *fss)
 
 #if 0
 /* helper function */
-void fluidsimGetGeometryObjFilename(Object *ob, char *dst) { //, char *srcname)
+void fluidsimGetGeometryObjFilename(Object *ob, char *dst)  //, char *srcname)
 {
-	//BLI_snprintf(dst,FILE_MAXFILE, "%s_cfgdata_%s.bobj.gz", srcname, ob->id.name);
-	BLI_snprintf(dst,FILE_MAXFILE, "fluidcfgdata_%s.bobj.gz", ob->id.name);
+	//BLI_snprintf(dst, FILE_MAXFILE, "%s_cfgdata_%s.bobj.gz", srcname, ob->id.name);
+	BLI_snprintf(dst, FILE_MAXFILE, "fluidcfgdata_%s.bobj.gz", ob->id.name);
 }
 #endif
 
@@ -199,30 +193,30 @@ typedef struct FluidObject {
 #if 0
 static void fluidsimPrintChannel(FILE *file, float *channel, int paramsize, char *str, int entries) 
 { 
-	int i,j; 
+	int i, j;
 	int channelSize = paramsize; 
 
-	if (entries==3) {
-		elbeemSimplifyChannelVec3( channel, &channelSize); 
+	if (entries == 3) {
+		elbeemSimplifyChannelVec3(channel, &channelSize);
 	}
-	else if (entries==1) {
-		elbeemSimplifyChannelFloat( channel, &channelSize); 
+	else if (entries == 1) {
+		elbeemSimplifyChannelFloat(channel, &channelSize);
 	}
 	else {
-		// invalid, cant happen?
+		/* invalid, cant happen? */
 	}
 
 	fprintf(file, "      CHANNEL %s =\n", str);
-	for (i=0; i<channelSize;i++) { 
-		fprintf(file,"        ");  
-		for (j=0;j<=entries;j++) {  // also print time value
-			fprintf(file," %f ", channel[i*(entries+1)+j] ); 
-			if (j==entries-1) { fprintf(file,"  "); }
-		} 
-		fprintf(file,"\n");
-	} 
+	for (i=0; i < channelSize; i++) {
+		fprintf(file, "        ");
+		for (j=0;j <= entries;j++) {  // also print time value
+			fprintf(file, " %f ", channel[i*(entries + 1) + j]);
+			if (j == entries-1) { fprintf(file, "  "); }
+		}
+		fprintf(file, "\n");
+	}
 
-	fprintf(file,  "      ;\n" );
+	fprintf(file,  "      ;\n");
 }
 #endif
 
@@ -243,12 +237,12 @@ static void init_time(FluidsimSettings *domainSettings, FluidAnimChannels *chann
 {
 	int i;
 	
-	channels->timeAtFrame = MEM_callocN( (channels->length+1)*sizeof(float), "timeAtFrame channel");
+	channels->timeAtFrame = MEM_callocN((channels->length + 1) * sizeof(float), "timeAtFrame channel");
 	
 	channels->timeAtFrame[0] = channels->timeAtFrame[1] = domainSettings->animStart; // start at index 1
 	
 	for (i=2; i <= channels->length; i++) {
-		channels->timeAtFrame[i] = channels->timeAtFrame[i-1] + channels->aniFrameTime;
+		channels->timeAtFrame[i] = channels->timeAtFrame[i - 1] + (float)channels->aniFrameTime;
 	}
 }
 
@@ -273,7 +267,7 @@ static void set_vertex_channel(float *channel, float time, struct Scene *scene,
 	FluidsimModifierData *fluidmd = (FluidsimModifierData *)modifiers_findByType(ob, eModifierType_Fluidsim);
 	float *verts;
 	int *tris=NULL, numVerts=0, numTris=0;
-	int modifierIndex = modifiers_indexInObject(ob, (ModifierData *)fluidmd);
+	int modifierIndex = BLI_findindex(&ob->modifiers, fluidmd);
 	int framesize = (3*fobj->numVerts) + 1;
 	int j;
 	
@@ -361,9 +355,9 @@ static void fluid_init_all_channels(bContext *C, Object *UNUSED(fsDomain), Fluid
 	init_time(domainSettings, channels);
 	
 	/* allocate domain animation channels */
-	channels->DomainGravity = MEM_callocN( length * (CHANNEL_VEC+1) * sizeof(float), "channel DomainGravity");
-	channels->DomainViscosity = MEM_callocN( length * (CHANNEL_FLOAT+1) * sizeof(float), "channel DomainViscosity");
-	channels->DomainTime = MEM_callocN( length * (CHANNEL_FLOAT+1) * sizeof(float), "channel DomainTime");
+	channels->DomainGravity = MEM_callocN(length * (CHANNEL_VEC+1) * sizeof(float), "channel DomainGravity");
+	channels->DomainViscosity = MEM_callocN(length * (CHANNEL_FLOAT+1) * sizeof(float), "channel DomainViscosity");
+	channels->DomainTime = MEM_callocN(length * (CHANNEL_FLOAT+1) * sizeof(float), "channel DomainTime");
 	
 	/* allocate fluid objects */
 	for (base=scene->base.first; base; base= base->next) {
@@ -379,25 +373,25 @@ static void fluid_init_all_channels(bContext *C, Object *UNUSED(fsDomain), Fluid
 				continue;
 			}
 			
-			fobj->Translation = MEM_callocN( length * (CHANNEL_VEC+1) * sizeof(float), "fluidobject Translation");
-			fobj->Rotation = MEM_callocN( length * (CHANNEL_VEC+1) * sizeof(float), "fluidobject Rotation");
-			fobj->Scale = MEM_callocN( length * (CHANNEL_VEC+1) * sizeof(float), "fluidobject Scale");
-			fobj->Active = MEM_callocN( length * (CHANNEL_FLOAT+1) * sizeof(float), "fluidobject Active");
-			fobj->InitialVelocity = MEM_callocN( length * (CHANNEL_VEC+1) * sizeof(float), "fluidobject InitialVelocity");
+			fobj->Translation = MEM_callocN(length * (CHANNEL_VEC+1) * sizeof(float), "fluidobject Translation");
+			fobj->Rotation = MEM_callocN(length * (CHANNEL_VEC+1) * sizeof(float), "fluidobject Rotation");
+			fobj->Scale = MEM_callocN(length * (CHANNEL_VEC+1) * sizeof(float), "fluidobject Scale");
+			fobj->Active = MEM_callocN(length * (CHANNEL_FLOAT+1) * sizeof(float), "fluidobject Active");
+			fobj->InitialVelocity = MEM_callocN(length * (CHANNEL_VEC+1) * sizeof(float), "fluidobject InitialVelocity");
 			
 			if (fluidmd->fss->type == OB_FLUIDSIM_CONTROL) {
-				fobj->AttractforceStrength = MEM_callocN( length * (CHANNEL_FLOAT+1) * sizeof(float), "fluidobject AttractforceStrength");
-				fobj->AttractforceRadius = MEM_callocN( length * (CHANNEL_FLOAT+1) * sizeof(float), "fluidobject AttractforceRadius");
-				fobj->VelocityforceStrength = MEM_callocN( length * (CHANNEL_FLOAT+1) * sizeof(float), "fluidobject VelocityforceStrength");
-				fobj->VelocityforceRadius = MEM_callocN( length * (CHANNEL_FLOAT+1) * sizeof(float), "fluidobject VelocityforceRadius");
+				fobj->AttractforceStrength = MEM_callocN(length * (CHANNEL_FLOAT+1) * sizeof(float), "fluidobject AttractforceStrength");
+				fobj->AttractforceRadius = MEM_callocN(length * (CHANNEL_FLOAT+1) * sizeof(float), "fluidobject AttractforceRadius");
+				fobj->VelocityforceStrength = MEM_callocN(length * (CHANNEL_FLOAT+1) * sizeof(float), "fluidobject VelocityforceStrength");
+				fobj->VelocityforceRadius = MEM_callocN(length * (CHANNEL_FLOAT+1) * sizeof(float), "fluidobject VelocityforceRadius");
 			}
 			
 			if (fluid_is_animated_mesh(fluidmd->fss)) {
 				float *verts=NULL;
-				int *tris=NULL, modifierIndex = modifiers_indexInObject(ob, (ModifierData *)fluidmd);
+				int *tris=NULL, modifierIndex = BLI_findindex(&ob->modifiers, (ModifierData *)fluidmd);
 
 				initElbeemMesh(scene, ob, &fobj->numVerts, &verts, &fobj->numTris, &tris, 0, modifierIndex);
-				fobj->VertexCache = MEM_callocN( length *((fobj->numVerts*CHANNEL_VEC)+1) * sizeof(float), "fluidobject VertexCache");
+				fobj->VertexCache = MEM_callocN(length *((fobj->numVerts*CHANNEL_VEC)+1) * sizeof(float), "fluidobject VertexCache");
 				
 				MEM_freeN(verts);
 				MEM_freeN(tris);
@@ -408,7 +402,7 @@ static void fluid_init_all_channels(bContext *C, Object *UNUSED(fsDomain), Fluid
 	}
 	
 	/* now we loop over the frames and fill the allocated channels with data */
-	for (i=0; i<channels->length; i++) {
+	for (i=0; i < channels->length; i++) {
 		FluidObject *fobj;
 		float viscosity, gravity[3];
 		float timeAtFrame, time;
@@ -425,14 +419,14 @@ static void fluid_init_all_channels(bContext *C, Object *UNUSED(fsDomain), Fluid
 		/* Modifying the global scene isn't nice, but we can do it in 
 		 * this part of the process before a threaded job is created */
 		scene->r.cfra = (int)eval_time;
-		ED_update_for_newframe(CTX_data_main(C), scene, CTX_wm_screen(C), 1);
+		ED_update_for_newframe(CTX_data_main(C), scene, 1);
 		
 		/* now scene data should be current according to animation system, so we fill the channels */
 		
 		/* Domain time */
 		// TODO: have option for not running sim, time mangling, in which case second case comes in handy
 		if (channels->DomainTime) {
-			time = get_fluid_rate(domainSettings) * channels->aniFrameTime;
+			time = get_fluid_rate(domainSettings) * (float)channels->aniFrameTime;
 			timeAtFrame = channels->timeAtFrame[i] + time;
 			
 			channels->timeAtFrame[i+1] = timeAtFrame;
@@ -452,7 +446,7 @@ static void fluid_init_all_channels(bContext *C, Object *UNUSED(fsDomain), Fluid
 		for (fobj=fobjects->first; fobj; fobj=fobj->next) {
 			Object *ob = fobj->object;
 			FluidsimModifierData *fluidmd = (FluidsimModifierData *)modifiers_findByType(ob, eModifierType_Fluidsim);
-			float active= (float)(fluidmd->fss->flag & OB_FLUIDSIM_ACTIVE);
+			float active= (float) ((fluidmd->fss->flag & OB_FLUIDSIM_ACTIVE) > 0 ? 1 : 0);
 			float rot_d[3] = {0.f, 0.f, 0.f}, old_rot[3] = {0.f, 0.f, 0.f};
 			
 			if (ELEM(fluidmd->fss->type, OB_FLUIDSIM_DOMAIN, OB_FLUIDSIM_PARTICLE))
@@ -462,17 +456,19 @@ static void fluid_init_all_channels(bContext *C, Object *UNUSED(fsDomain), Fluid
 			/* get the rotation from ob->obmat rather than ob->rot to account for parent animations */
 			if (i) {
 				copy_v3_v3(old_rot, fobj->Rotation + 4*(i-1));
-				mul_v3_fl(old_rot, -M_PI/180.f);
+				mul_v3_fl(old_rot, (float)-M_PI / 180.f);
 			}
 
 			mat4_to_compatible_eulO(rot_d, old_rot, 0, ob->obmat);
-			mul_v3_fl(rot_d, -180.f/M_PI);
+			mul_v3_fl(rot_d, -180.0f / (float)M_PI);
 			
 			set_channel(fobj->Translation, timeAtFrame, ob->loc, i, CHANNEL_VEC);
 			set_channel(fobj->Rotation, timeAtFrame, rot_d, i, CHANNEL_VEC);
 			set_channel(fobj->Scale, timeAtFrame, ob->size, i, CHANNEL_VEC);
 			set_channel(fobj->Active, timeAtFrame, &active, i, CHANNEL_FLOAT);
 			set_channel(fobj->InitialVelocity, timeAtFrame, &fluidmd->fss->iniVelx, i, CHANNEL_VEC);
+
+			// printf("Active: %f, Frame: %f\n", active, timeAtFrame);
 			
 			if (fluidmd->fss->type == OB_FLUIDSIM_CONTROL) {
 				set_channel(fobj->AttractforceStrength, timeAtFrame, &fluidmd->fss->attractforceStrength, i, CHANNEL_FLOAT);
@@ -495,7 +491,7 @@ static void export_fluid_objects(ListBase *fobjects, Scene *scene, int length)
 	for (fobj=fobjects->first; fobj; fobj=fobj->next) {
 		Object *ob = fobj->object;
 		FluidsimModifierData *fluidmd = (FluidsimModifierData *)modifiers_findByType(ob, eModifierType_Fluidsim);
-		int modifierIndex = modifiers_indexInObject(ob, (ModifierData *)fluidmd);
+		int modifierIndex = BLI_findindex(&ob->modifiers, fluidmd);
 		
 		float *verts=NULL;
 		int *tris=NULL;
@@ -507,7 +503,7 @@ static void export_fluid_objects(ListBase *fobjects, Scene *scene, int length)
 		if (ELEM(fluidmd->fss->type, OB_FLUIDSIM_DOMAIN, OB_FLUIDSIM_PARTICLE))
 			continue;
 		
-		elbeemResetMesh( &fsmesh );
+		elbeemResetMesh(&fsmesh);
 		
 		fsmesh.type = fluidmd->fss->type;
 		fsmesh.name = ob->id.name;
@@ -533,7 +529,7 @@ static void export_fluid_objects(ListBase *fobjects, Scene *scene, int length)
 		if ( ELEM(fsmesh.type, OB_FLUIDSIM_FLUID, OB_FLUIDSIM_INFLOW)) {
 			fsmesh.channelInitialVel = fobj->InitialVelocity;
 			fsmesh.localInivelCoords = ((fluidmd->fss->typeFlags & OB_FSINFLOW_LOCALCOORD)?1:0);
-		} 
+		}
 		
 		if (fluidmd->fss->typeFlags & OB_FSBND_NOSLIP)
 			fsmesh.obstacleType = FLUIDSIM_OBSTACLE_NOSLIP;
@@ -546,7 +542,7 @@ static void export_fluid_objects(ListBase *fobjects, Scene *scene, int length)
 		fsmesh.volumeInitType = fluidmd->fss->volumeInitType;
 		fsmesh.obstacleImpactFactor = fluidmd->fss->surfaceSmoothing; // misused value
 		
-		if (fsmesh.type == OB_FLUIDSIM_CONTROL)	{
+		if (fsmesh.type == OB_FLUIDSIM_CONTROL) {
 			fsmesh.cpsTimeStart = fluidmd->fss->cpsTimeStart;
 			fsmesh.cpsTimeEnd = fluidmd->fss->cpsTimeEnd;
 			fsmesh.cpsQuality = fluidmd->fss->cpsQuality;
@@ -573,11 +569,15 @@ static void export_fluid_objects(ListBase *fobjects, Scene *scene, int length)
 		if (deform) {
 			fsmesh.channelSizeVertices = length;
 			fsmesh.channelVertices = fobj->VertexCache;
-				
-			// remove channels
+			
+			/* remove channels */
 			fsmesh.channelTranslation      = 
 			fsmesh.channelRotation         = 
-			fsmesh.channelScale            = NULL; 
+			fsmesh.channelScale            = NULL;
+			
+			/* Override user settings, only noslip is supported here! */
+			if (fsmesh.type != OB_FLUIDSIM_CONTROL)
+				fsmesh.obstacleType = FLUIDSIM_OBSTACLE_NOSLIP;
 		}
 		
 		elbeemAddMesh(&fsmesh);
@@ -594,10 +594,9 @@ static int fluid_validate_scene(ReportList *reports, Scene *scene, Object *fsDom
 	int channelObjCount = 0;
 	int fluidInputCount = 0;
 
-	for (base=scene->base.first; base; base= base->next)
-	{
+	for (base=scene->base.first; base; base= base->next) {
 		Object *ob = base->object;
-		FluidsimModifierData *fluidmdtmp = (FluidsimModifierData *)modifiers_findByType(ob, eModifierType_Fluidsim);			
+		FluidsimModifierData *fluidmdtmp = (FluidsimModifierData *)modifiers_findByType(ob, eModifierType_Fluidsim);
 
 		/* only find objects with fluid modifiers */
 		if (!fluidmdtmp || ob->type != OB_MESH) continue;
@@ -631,7 +630,7 @@ static int fluid_validate_scene(ReportList *reports, Scene *scene, Object *fsDom
 		return 0;
 	}
 	
-	if (channelObjCount>=255) {
+	if (channelObjCount >= 255) {
 		BKE_report(reports, RPT_ERROR, "Cannot bake with more then 256 objects");
 		return 0;
 	}
@@ -652,19 +651,19 @@ static int fluid_validate_scene(ReportList *reports, Scene *scene, Object *fsDom
 static int fluid_init_filepaths(Object *fsDomain, char *targetDir, char *targetFile, char *debugStrBuffer)
 {
 	FluidsimModifierData *fluidmd = (FluidsimModifierData *)modifiers_findByType(fsDomain, eModifierType_Fluidsim);
-	FluidsimSettings *domainSettings= fluidmd->fss;	
+	FluidsimSettings *domainSettings= fluidmd->fss;
 	FILE *fileCfg;
 	int dirExist = 0;
-	char newSurfdataPath[FILE_MAX]; // modified output settings
+	char newSurfdataPath[FILE_MAX]; /* modified output settings */
 	const char *suffixConfigTmp = FLUID_SUFFIX_CONFIG_TMP;
 	int outStringsChanged = 0;
 
-	// prepare names...
+	/* prepare names... */
 	const char *relbase= modifier_path_relbase(fsDomain);
 
 	BLI_strncpy(targetDir, domainSettings->surfdataPath, FILE_MAXDIR);
 	BLI_strncpy(newSurfdataPath, domainSettings->surfdataPath, FILE_MAXDIR); /* if 0'd out below, this value is never used! */
-	BLI_path_abs(targetDir, relbase); // fixed #frame-no
+	BLI_path_abs(targetDir, relbase); /* fixed #frame-no */
 
 	/* .tmp: don't overwrite/delete original file */
 	BLI_join_dirfile(targetFile, FILE_MAX, targetDir, suffixConfigTmp);
@@ -677,10 +676,10 @@ static int fluid_init_filepaths(Object *fsDomain, char *targetDir, char *targetF
 	// check selected directory
 	// simply try to open cfg file for writing to test validity of settings
 	fileCfg = BLI_fopen(targetFile, "w");
-	if (fileCfg) { 
+	if (fileCfg) {
 		dirExist = 1; fclose(fileCfg); 
 		// remove cfg dummy from  directory test
-		BLI_delete(targetFile, 0,0);
+		BLI_delete(targetFile, 0, 0);
 	}
 	
 	if (targetDir[0] == '\0' || (!dirExist)) {
@@ -692,30 +691,30 @@ static int fluid_init_filepaths(Object *fsDomain, char *targetDir, char *targetF
 		BLI_splitdirstring(blendDir, blendFile);
 		BLI_replace_extension(blendFile, FILE_MAX, ""); /* strip .blend */
 
-		BLI_snprintf(newSurfdataPath, FILE_MAX ,"//fluidsimdata/%s_%s_", blendFile, fsDomain->id.name);
+		BLI_snprintf(newSurfdataPath, FILE_MAX, "//fluidsimdata/%s_%s_", blendFile, fsDomain->id.name);
 		
 		BLI_snprintf(debugStrBuffer, 256, "fluidsimBake::error - warning resetting output dir to '%s'\n", newSurfdataPath);
 		elbeemDebugOut(debugStrBuffer);
 		outStringsChanged=1;
 	}
 	
-	// check if modified output dir is ok
+	/* check if modified output dir is ok */
 #if 0
 	if (outStringsChanged) {
 		char dispmsg[FILE_MAX+256];
 		int  selection=0;
-		BLI_strncpy(dispmsg,"Output settings set to: '", sizeof(dispmsg));
+		BLI_strncpy(dispmsg, "Output settings set to: '", sizeof(dispmsg));
 		strcat(dispmsg, newSurfdataPath);
-		strcat(dispmsg, "'%t|Continue with changed settings%x1|Discard and abort%x0");
+		strcat(dispmsg, "'%t|Continue with changed settings %x1|Discard and abort %x0");
 		
-		// ask user if thats what he/she wants...
+		/* ask user if thats what he/she wants... */
 		selection = pupmenu(dispmsg);
-		if (selection<1) return 0; // 0 from menu, or -1 aborted
+		if (selection < 1) return 0; /* 0 from menu, or -1 aborted */
 		BLI_strncpy(targetDir, newSurfdataPath, sizeof(targetDir));
 		strncpy(domainSettings->surfdataPath, newSurfdataPath, FILE_MAXDIR);
-		BLI_path_abs(targetDir, G.main->name); // fixed #frame-no 
+		BLI_path_abs(targetDir, G.main->name); /* fixed #frame-no */
 	}
-#endif	
+#endif
 	return outStringsChanged;
 }
 
@@ -749,7 +748,7 @@ static int fluidbake_breakjob(void *customdata)
 	/* this is not nice yet, need to make the jobs list template better 
 	 * for identifying/acting upon various different jobs */
 	/* but for now we'll reuse the render break... */
-	return (G.afbreek);
+	return (G.is_break);
 }
 
 /* called by fluidbake, wmJob sends notifier */
@@ -757,8 +756,8 @@ static void fluidbake_updatejob(void *customdata, float progress)
 {
 	FluidBakeJob *fb= (FluidBakeJob *)customdata;
 	
-	*(fb->do_update)= 1;
-	*(fb->progress)= progress;
+	*(fb->do_update) = TRUE;
+	*(fb->progress) = progress;
 }
 
 static void fluidbake_startjob(void *customdata, short *stop, short *do_update, float *progress)
@@ -769,10 +768,10 @@ static void fluidbake_startjob(void *customdata, short *stop, short *do_update,
 	fb->do_update = do_update;
 	fb->progress = progress;
 	
-	G.afbreek= 0;	/* XXX shared with render - replace with job 'stop' switch */
+	G.is_break = FALSE;  /* XXX shared with render - replace with job 'stop' switch */
 	
 	elbeemSimulate();
-	*do_update= 1;
+	*do_update = TRUE;
 	*stop = 0;
 }
 
@@ -786,14 +785,14 @@ static void fluidbake_endjob(void *customdata)
 	}
 }
 
-int runSimulationCallback(void *data, int status, int frame)
+static int runSimulationCallback(void *data, int status, int frame)
 {
 	FluidBakeJob *fb = (FluidBakeJob *)data;
 	elbeemSimulationSettings *settings = fb->settings;
 	
 	if (status == FLUIDSIM_CBSTATUS_NEWFRAME) {
 		fluidbake_updatejob(fb, frame / (float)settings->noOfFrames);
-		//printf("elbeem blender cb s%d, f%d, domainid:%d noOfFrames: %d\n", status,frame, settings->domainId, settings->noOfFrames ); // DEBUG
+		//printf("elbeem blender cb s%d, f%d, domainid:%d noOfFrames: %d\n", status, frame, settings->domainId, settings->noOfFrames ); // DEBUG
 	}
 	
 	if (fluidbake_breakjob(fb)) {
@@ -899,14 +898,14 @@ static int fluidsimBake(bContext *C, ReportList *reports, Object *fsDomain, shor
 	if (getenv(strEnvName)) {
 		int dlevel = atoi(getenv(strEnvName));
 		elbeemSetDebugLevel(dlevel);
-		BLI_snprintf(debugStrBuffer, sizeof(debugStrBuffer),"fluidsimBake::msg: Debug messages activated due to envvar '%s'\n",strEnvName);
+		BLI_snprintf(debugStrBuffer, sizeof(debugStrBuffer), "fluidsimBake::msg: Debug messages activated due to envvar '%s'\n", strEnvName);
 		elbeemDebugOut(debugStrBuffer);
 	}
 	
 	/* make sure it corresponds to startFrame setting (old: noFrames = scene->r.efra - scene->r.sfra +1) */;
 	noFrames = scene->r.efra - 0;
 	if (noFrames<=0) {
-		BKE_report(reports, RPT_ERROR, "No frames to export - check your animation range settings");
+		BKE_report(reports, RPT_ERROR, "No frames to export (check your animation range settings)");
 		fluidbake_free_data(channels, fobjects, fsset, fb);
 		return 0;
 	}
@@ -917,7 +916,7 @@ static int fluidsimBake(bContext *C, ReportList *reports, Object *fsDomain, shor
 		return 0;
 	}
 	
-	/* these both have to be valid, otherwise we wouldnt be here */
+	/* these both have to be valid, otherwise we wouldn't be here */
 	fluidmd = (FluidsimModifierData *)modifiers_findByType(fsDomain, eModifierType_Fluidsim);
 	domainSettings = fluidmd->fss;
 	mesh = fsDomain->data;
@@ -936,7 +935,7 @@ static int fluidsimBake(bContext *C, ReportList *reports, Object *fsDomain, shor
 	
 	/* rough check of settings... */
 	if (domainSettings->previewresxyz > domainSettings->resolutionxyz) {
-		BLI_snprintf(debugStrBuffer,sizeof(debugStrBuffer),"fluidsimBake::warning - Preview (%d) >= Resolution (%d)... setting equal.\n", domainSettings->previewresxyz ,  domainSettings->resolutionxyz);
+		BLI_snprintf(debugStrBuffer, sizeof(debugStrBuffer), "fluidsimBake::warning - Preview (%d) >= Resolution (%d)... setting equal.\n", domainSettings->previewresxyz,  domainSettings->resolutionxyz);
 		elbeemDebugOut(debugStrBuffer);
 		domainSettings->previewresxyz = domainSettings->resolutionxyz;
 	}
@@ -948,8 +947,7 @@ static int fluidsimBake(bContext *C, ReportList *reports, Object *fsDomain, shor
 		if (domainSettings->resolutionxyz>128) {
 			gridlevels = 2;
 		}
-		else
-		if (domainSettings->resolutionxyz>64) {
+		else if (domainSettings->resolutionxyz > 64) {
 			gridlevels = 1;
 		}
 		else {
@@ -959,27 +957,27 @@ static int fluidsimBake(bContext *C, ReportList *reports, Object *fsDomain, shor
 	else {
 		gridlevels = domainSettings->maxRefine;
 	}
-	BLI_snprintf(debugStrBuffer,sizeof(debugStrBuffer),"fluidsimBake::msg: Baking %s, refine: %d\n", fsDomain->id.name , gridlevels );
+	BLI_snprintf(debugStrBuffer, sizeof(debugStrBuffer), "fluidsimBake::msg: Baking %s, refine: %d\n", fsDomain->id.name, gridlevels);
 	elbeemDebugOut(debugStrBuffer);
 	
 	
 	
 	/* ******** prepare output file paths ******** */
 	outStringsChanged = fluid_init_filepaths(fsDomain, targetDir, targetFile, debugStrBuffer);
-	channels->length = scene->r.efra;
-	channels->aniFrameTime = (domainSettings->animEnd - domainSettings->animStart)/(double)noFrames;
+	channels->length = scene->r.efra; // DG TODO: why using endframe and not "noFrames" here? .. because "noFrames" is buggy too? (not using sfra)
+	channels->aniFrameTime = (double)((double)domainSettings->animEnd - (double)domainSettings->animStart) / (double)noFrames;
 	
 	/* ******** initialize and allocate animation channels ******** */
 	fluid_init_all_channels(C, fsDomain, domainSettings, channels, fobjects);
 
 	/* reset to original current frame */
 	scene->r.cfra = origFrame;
-	ED_update_for_newframe(CTX_data_main(C), scene, CTX_wm_screen(C), 1);
+	ED_update_for_newframe(CTX_data_main(C), scene, 1);
 		
 	/* ******** init domain object's matrix ******** */
 	copy_m4_m4(domainMat, fsDomain->obmat);
 	if (!invert_m4_m4(invDomMat, domainMat)) {
-		BLI_snprintf(debugStrBuffer,sizeof(debugStrBuffer),"fluidsimBake::error - Invalid obj matrix?\n");
+		BLI_snprintf(debugStrBuffer, sizeof(debugStrBuffer), "fluidsimBake::error - Invalid obj matrix?\n");
 		elbeemDebugOut(debugStrBuffer);
 		BKE_report(reports, RPT_ERROR, "Invalid object matrix"); 
 
@@ -1054,7 +1052,7 @@ static int fluidsimBake(bContext *C, ReportList *reports, Object *fsDomain, shor
 	fsset->generateVertexVectors = (domainSettings->domainNovecgen==0);
 
 	// init blender domain transform matrix
-	{ int j; 
+	{ int j;
 	for (i=0; i<4; i++) {
 		for (j=0; j<4; j++) {
 			fsset->surfaceTrafo[i*4+j] = invDomMat[j][i];
@@ -1072,14 +1070,15 @@ static int fluidsimBake(bContext *C, ReportList *reports, Object *fsDomain, shor
 	fb->settings = fsset;
 	
 	if (do_job) {
-		wmJob *steve= WM_jobs_get(CTX_wm_manager(C), CTX_wm_window(C), scene, "Fluid Simulation", WM_JOB_PROGRESS);
+		wmJob *wm_job = WM_jobs_get(CTX_wm_manager(C), CTX_wm_window(C), scene, "Fluid Simulation",
+		                            WM_JOB_PROGRESS, WM_JOB_TYPE_OBJECT_SIM_FLUID);
 
 		/* setup job */
-		WM_jobs_customdata(steve, fb, fluidbake_free);
-		WM_jobs_timer(steve, 0.1, NC_SCENE|ND_FRAME, NC_SCENE|ND_FRAME);
-		WM_jobs_callbacks(steve, fluidbake_startjob, NULL, NULL, fluidbake_endjob);
+		WM_jobs_customdata_set(wm_job, fb, fluidbake_free);
+		WM_jobs_timer(wm_job, 0.1, NC_SCENE|ND_FRAME, NC_SCENE|ND_FRAME);
+		WM_jobs_callbacks(wm_job, fluidbake_startjob, NULL, NULL, fluidbake_endjob);
 
-		WM_jobs_start(CTX_wm_manager(C), steve);
+		WM_jobs_start(CTX_wm_manager(C), wm_job);
 	}
 	else {
 		short dummy_stop, dummy_do_update;
@@ -1098,7 +1097,7 @@ static int fluidsimBake(bContext *C, ReportList *reports, Object *fsDomain, shor
 	return 1;
 }
 
-void fluidsimFreeBake(Object *UNUSED(ob))
+static void UNUSED_FUNCTION(fluidsimFreeBake)(Object *UNUSED(ob))
 {
 	/* not implemented yet */
 }
@@ -1116,7 +1115,7 @@ void fluidsimSettingsFree(FluidsimSettings *UNUSED(fss))
 {
 }
 
-FluidsimSettings* fluidsimSettingsCopy(FluidsimSettings *UNUSED(fss))
+FluidsimSettings *fluidsimSettingsCopy(FluidsimSettings *UNUSED(fss))
 {
 	return NULL;
 }
@@ -1134,7 +1133,7 @@ static int fluidsimBake(bContext *UNUSED(C), ReportList *UNUSED(reports), Object
 static int fluid_bake_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
 {
 	/* only one bake job at a time */
-	if (WM_jobs_test(CTX_wm_manager(C), CTX_data_scene(C)))
+	if (WM_jobs_test(CTX_wm_manager(C), CTX_data_scene(C), WM_JOB_TYPE_OBJECT_SIM_FLUID))
 		return OPERATOR_CANCELLED;
 
 	if (!fluidsimBake(C, op->reports, CTX_data_active_object(C), TRUE))
