@@ -61,6 +61,7 @@
 #include "ED_object.h"
 
 #include "RE_pipeline.h"
+#include "IMB_colormanagement.h"
 #include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
 
@@ -72,18 +73,18 @@
 #include "render_intern.h"
 
 /* Render Callbacks */
+static int render_break(void *rjv);
 
 /* called inside thread! */
 void image_buffer_rect_update(Scene *scene, RenderResult *rr, ImBuf *ibuf, volatile rcti *renrect)
 {
 	float *rectf = NULL;
 	int ymin, ymax, xmin, xmax;
-	int rymin, rxmin, predivide, profile_from;
-	unsigned char *rectc;
+	int rymin, rxmin;
 
 	/* if renrect argument, we only refresh scanlines */
 	if (renrect) {
-		/* if ymax==recty, rendering of layer is ready, we should not draw, other things happen... */
+		/* if (ymax == recty), rendering of layer is ready, we should not draw, other things happen... */
 		if (rr->renlay == NULL || renrect->ymax >= rr->recty)
 			return;
 
@@ -123,8 +124,14 @@ void image_buffer_rect_update(Scene *scene, RenderResult *rr, ImBuf *ibuf, volat
 	if (rr->rectf)
 		rectf = rr->rectf;
 	else {
-		if (rr->rect32)
+		if (rr->rect32) {
+			/* special case, currently only happens with sequencer rendering,
+			 * which updates the whole frame, so we can only mark display buffer
+			 * as invalid here (sergey)
+			 */
+			ibuf->userflags |= IB_DISPLAY_BUFFER_INVALID;
 			return;
+		}
 		else {
 			if (rr->renlay == NULL || rr->renlay->rectf == NULL) return;
 			rectf = rr->renlay->rectf;
@@ -136,20 +143,10 @@ void image_buffer_rect_update(Scene *scene, RenderResult *rr, ImBuf *ibuf, volat
 		imb_addrectImBuf(ibuf);
 	
 	rectf += 4 * (rr->rectx * ymin + xmin);
-	rectc = (unsigned char *)(ibuf->rect + ibuf->x * rymin + rxmin);
 
-	if (scene && (scene->r.color_mgt_flag & R_COLOR_MANAGEMENT)) {
-		profile_from = IB_PROFILE_LINEAR_RGB;
-		predivide = (scene->r.color_mgt_flag & R_COLOR_MANAGEMENT_PREDIVIDE);
-	}
-	else {
-		profile_from = IB_PROFILE_SRGB;
-		predivide = 0;
-	}
-
-	IMB_buffer_byte_from_float(rectc, rectf,
-	                           4, ibuf->dither, IB_PROFILE_SRGB, profile_from, predivide,
-	                           xmax, ymax, ibuf->x, rr->rectx);
+	IMB_partial_display_buffer_update(ibuf, rectf, NULL, rr->rectx, rxmin, rymin,
+	                                  &scene->view_settings, &scene->display_settings,
+	                                  rxmin, rymin, rxmin + xmax, rymin + ymax, TRUE);
 }
 
 /* ****************************** render invoking ***************** */
@@ -170,7 +167,7 @@ static void screen_render_scene_layer_set(wmOperator *op, Main *mainp, Scene **s
 		if (scn) {
 			/* camera switch wont have updated */
 			scn->r.cfra = (*scene)->r.cfra;
-			scene_camera_switch_update(scn);
+			BKE_scene_camera_switch_update(scn);
 
 			*scene = scn;
 		}
@@ -206,15 +203,15 @@ static int screen_render_exec(bContext *C, wmOperator *op)
 	screen_render_scene_layer_set(op, mainp, &scene, &srl);
 
 	if (!is_animation && is_write_still && BKE_imtype_is_movie(scene->r.im_format.imtype)) {
-		BKE_report(op->reports, RPT_ERROR, "Can't write a single file with an animation format selected");
+		BKE_report(op->reports, RPT_ERROR, "Cannot write a single file with an animation format selected");
 		return OPERATOR_CANCELLED;
 	}
 
 	re = RE_NewRender(scene->id.name);
 	lay = (v3d) ? v3d->lay : scene->lay;
 
-	G.afbreek = 0;
-	RE_test_break_cb(re, NULL, (int (*)(void *))blender_test_break);
+	G.is_break = FALSE;
+	RE_test_break_cb(re, NULL, render_break);
 
 	ima = BKE_image_verify_viewer(IMA_TYPE_R_RESULT, "Render Result");
 	BKE_image_signal(ima, NULL, IMA_SIGNAL_FREE);
@@ -224,7 +221,7 @@ static int screen_render_exec(bContext *C, wmOperator *op)
 	 * otherwise, invalidated cache entries can make their way into
 	 * the output rendering. We can't put that into RE_BlenderFrame,
 	 * since sequence rendering can call that recursively... (peter) */
-	seq_stripelem_cache_cleanup();
+	BKE_sequencer_cache_cleanup();
 
 	RE_SetReports(re, op->reports);
 
@@ -236,7 +233,7 @@ static int screen_render_exec(bContext *C, wmOperator *op)
 	RE_SetReports(re, NULL);
 
 	// no redraw needed, we leave state as we entered it
-	ED_update_for_newframe(mainp, scene, CTX_wm_screen(C), 1);
+	ED_update_for_newframe(mainp, scene, 1);
 
 	WM_event_add_notifier(C, NC_SCENE | ND_RENDER_RESULT, scene);
 
@@ -288,17 +285,22 @@ static void make_renderinfo_string(RenderStats *rs, Scene *scene, char *str)
 	else if (scene->r.scemode & R_SINGLE_LAYER)
 		spos += sprintf(spos, "Single Layer | ");
 
+	spos += sprintf(spos, "Frame:%d ", (scene->r.cfra));
+
 	if (rs->statstr) {
-		spos += sprintf(spos, "%s ", rs->statstr);
+		spos += sprintf(spos, "| %s ", rs->statstr);
 	}
 	else {
-		spos += sprintf(spos, "Fra:%d  ", (scene->r.cfra));
 		if (rs->totvert) spos += sprintf(spos, "Ve:%d ", rs->totvert);
 		if (rs->totface) spos += sprintf(spos, "Fa:%d ", rs->totface);
 		if (rs->tothalo) spos += sprintf(spos, "Ha:%d ", rs->tothalo);
 		if (rs->totstrand) spos += sprintf(spos, "St:%d ", rs->totstrand);
 		if (rs->totlamp) spos += sprintf(spos, "La:%d ", rs->totlamp);
-		spos += sprintf(spos, "Mem:%.2fM (%.2fM, peak %.2fM) ", megs_used_memory, mmap_used_memory, megs_peak_memory);
+
+		if (rs->mem_peak == 0.0f)
+			spos += sprintf(spos, "Mem:%.2fM (%.2fM, peak %.2fM) ", megs_used_memory, mmap_used_memory, megs_peak_memory);
+		else
+			spos += sprintf(spos, "Mem:%.2fM, Peak: %.2fM ", rs->mem_used, rs->mem_peak);
 
 		if (rs->curfield)
 			spos += sprintf(spos, "Field %d ", rs->curfield);
@@ -340,7 +342,7 @@ static void image_renderinfo_cb(void *rjv, RenderStats *rs)
 	RE_ReleaseResult(rj->re);
 
 	/* make jobs timer to send notifier */
-	*(rj->do_update) = 1;
+	*(rj->do_update) = TRUE;
 
 }
 
@@ -352,7 +354,7 @@ static void render_progress_update(void *rjv, float progress)
 		*rj->progress = progress;
 
 		/* make jobs timer to send notifier */
-		*(rj->do_update) = 1;
+		*(rj->do_update) = TRUE;
 	}
 }
 
@@ -372,9 +374,9 @@ static void image_rect_update(void *rjv, RenderResult *rr, volatile rcti *renrec
 		image_buffer_rect_update(rj->scene, rr, ibuf, renrect);
 
 		/* make jobs timer to send notifier */
-		*(rj->do_update) = 1;
+		*(rj->do_update) = TRUE;
 	}
-	BKE_image_release_ibuf(ima, lock);
+	BKE_image_release_ibuf(ima, ibuf, lock);
 }
 
 static void render_startjob(void *rjv, short *stop, short *do_update, float *progress)
@@ -407,8 +409,12 @@ static void render_endjob(void *rjv)
 		free_main(rj->main);
 
 	/* else the frame will not update for the original value */
-	if (!(rj->scene->r.scemode & R_NO_FRAME_UPDATE))
-		ED_update_for_newframe(G.main, rj->scene, rj->win->screen, 1);
+	if (!(rj->scene->r.scemode & R_NO_FRAME_UPDATE)) {
+		/* possible this fails of loading new file while rendering */
+		if (G.main->wm.first) {
+			ED_update_for_newframe(G.main, rj->scene, 1);
+		}
+	}
 	
 	/* XXX above function sets all tags in nodes */
 	ntreeCompositClearTags(rj->scene->nodetree);
@@ -422,7 +428,7 @@ static void render_endjob(void *rjv)
 	}
 	
 	/* XXX render stability hack */
-	G.rendering = 0;
+	G.is_rendering = FALSE;
 	WM_main_add_notifier(NC_WINDOW, NULL);
 }
 
@@ -431,13 +437,22 @@ static int render_breakjob(void *rjv)
 {
 	RenderJob *rj = rjv;
 
-	if (G.afbreek)
+	if (G.is_break)
 		return 1;
 	if (rj->stop && *(rj->stop))
 		return 1;
 	return 0;
 }
 
+/* for exec() when there is no render job
+ * note: this wont check for the escape key being pressed, but doing so isnt threadsafe */
+static int render_break(void *UNUSED(rjv))
+{
+	if (G.is_break)
+		return 1;
+	return 0;
+}
+
 /* runs in thread, no cursor setting here works. careful with notifiers too (malloc conflicts) */
 /* maybe need a way to get job send notifer? */
 static void render_drawlock(void *UNUSED(rjv), int lock)
@@ -447,10 +462,12 @@ static void render_drawlock(void *UNUSED(rjv), int lock)
 }
 
 /* catch esc */
-static int screen_render_modal(bContext *C, wmOperator *UNUSED(op), wmEvent *event)
+static int screen_render_modal(bContext *C, wmOperator *op, wmEvent *event)
 {
+	Scene *scene = (Scene *) op->customdata;
+
 	/* no running blender, remove handler and pass through */
-	if (0 == WM_jobs_test(CTX_wm_manager(C), CTX_data_scene(C))) {
+	if (0 == WM_jobs_test(CTX_wm_manager(C), scene, WM_JOB_TYPE_RENDER)) {
 		return OPERATOR_FINISHED | OPERATOR_PASS_THROUGH;
 	}
 
@@ -470,10 +487,9 @@ static int screen_render_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	Main *mainp;
 	Scene *scene = CTX_data_scene(C);
 	SceneRenderLayer *srl = NULL;
-	bScreen *screen = CTX_wm_screen(C);
 	View3D *v3d = CTX_wm_view3d(C);
 	Render *re;
-	wmJob *steve;
+	wmJob *wm_job;
 	RenderJob *rj;
 	Image *ima;
 	int jobflag;
@@ -483,7 +499,7 @@ static int screen_render_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	const char *name;
 	
 	/* only one render job at a time */
-	if (WM_jobs_test(CTX_wm_manager(C), scene))
+	if (WM_jobs_test(CTX_wm_manager(C), scene, WM_JOB_TYPE_RENDER))
 		return OPERATOR_CANCELLED;
 
 	if (!RE_is_rendering_allowed(scene, camera_override, op->reports)) {
@@ -491,15 +507,15 @@ static int screen_render_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	}
 
 	if (!is_animation && is_write_still && BKE_imtype_is_movie(scene->r.im_format.imtype)) {
-		BKE_report(op->reports, RPT_ERROR, "Can't write a single file with an animation format selected");
+		BKE_report(op->reports, RPT_ERROR, "Cannot write a single file with an animation format selected");
 		return OPERATOR_CANCELLED;
-	}	
+	}
 	
-	/* stop all running jobs, currently previews frustrate Render */
-	WM_jobs_stop_all(CTX_wm_manager(C));
+	/* stop all running jobs, except screen one. currently previews frustrate Render */
+	WM_jobs_kill_all_except(CTX_wm_manager(C), CTX_wm_screen(C));
 
 	/* get main */
-	if (G.rt == 101) {
+	if (G.debug_value == 101) {
 		/* thread-safety experiment, copy main from the undo buffer */
 		mainp = BKE_undo_get_main(&scene);
 	}
@@ -507,7 +523,7 @@ static int screen_render_invoke(bContext *C, wmOperator *op, wmEvent *event)
 		mainp = CTX_data_main(C);
 
 	/* cancel animation playback */
-	if (screen->animtimer)
+	if (ED_screen_animation_playing(CTX_wm_manager(C)))
 		ED_screen_animation_play(C, 0, 0);
 	
 	/* handle UI stuff */
@@ -520,7 +536,7 @@ static int screen_render_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	 * otherwise, invalidated cache entries can make their way into
 	 * the output rendering. We can't put that into RE_BlenderFrame,
 	 * since sequence rendering can call that recursively... (peter) */
-	seq_stripelem_cache_cleanup();
+	BKE_sequencer_cache_cleanup();
 
 	/* get editmode results */
 	ED_object_exit_editmode(C, 0);  /* 0 = does not exit editmode */
@@ -547,21 +563,28 @@ static int screen_render_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	rj->win = CTX_wm_window(C);
 	rj->srl = srl;
 	rj->camera_override = camera_override;
-	rj->lay = (v3d) ? v3d->lay : scene->lay;
+	rj->lay = scene->lay;
 	rj->anim = is_animation;
 	rj->write_still = is_write_still && !is_animation;
 	rj->iuser.scene = scene;
 	rj->iuser.ok = 1;
 	rj->reports = op->reports;
 
+	if (v3d) {
+		rj->lay = v3d->lay;
+
+		if (v3d->localvd)
+			rj->lay |= v3d->localvd->lay;
+	}
+
 	/* setup job */
 	if (RE_seq_render_active(scene, &scene->r)) name = "Sequence Render";
 	else name = "Render";
 
-	steve = WM_jobs_get(CTX_wm_manager(C), CTX_wm_window(C), scene, name, jobflag);
-	WM_jobs_customdata(steve, rj, render_freejob);
-	WM_jobs_timer(steve, 0.2, NC_SCENE | ND_RENDER_RESULT, 0);
-	WM_jobs_callbacks(steve, render_startjob, NULL, NULL, render_endjob);
+	wm_job = WM_jobs_get(CTX_wm_manager(C), CTX_wm_window(C), scene, name, jobflag, WM_JOB_TYPE_RENDER);
+	WM_jobs_customdata_set(wm_job, rj, render_freejob);
+	WM_jobs_timer(wm_job, 0.2, NC_SCENE | ND_RENDER_RESULT, 0);
+	WM_jobs_callbacks(wm_job, render_startjob, NULL, NULL, render_endjob);
 
 	/* get a render result image, and make sure it is empty */
 	ima = BKE_image_verify_viewer(IMA_TYPE_R_RESULT, "Render Result");
@@ -578,17 +601,23 @@ static int screen_render_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	RE_progress_cb(re, rj, render_progress_update);
 
 	rj->re = re;
-	G.afbreek = 0;
+	G.is_break = FALSE;
 
-	WM_jobs_start(CTX_wm_manager(C), steve);
+	/* store actual owner of job, so modal operator could check for it,
+	 * the reason of this is that active scene could change when rendering
+	 * several layers from compositor [#31800]
+	 */
+	op->customdata = scene;
+
+	WM_jobs_start(CTX_wm_manager(C), wm_job);
 
 	WM_cursor_wait(0);
 	WM_event_add_notifier(C, NC_SCENE | ND_RENDER_RESULT, scene);
 
-	/* we set G.rendering here already instead of only in the job, this ensure
+	/* we set G.is_rendering here already instead of only in the job, this ensure
 	 * main loop or other scene updates are disabled in time, since they may
 	 * have started before the job thread */
-	G.rendering = 1;
+	G.is_rendering = TRUE;
 
 	/* add modal handler for ESC */
 	WM_event_add_modal_handler(C, op);
@@ -599,6 +628,8 @@ static int screen_render_invoke(bContext *C, wmOperator *op, wmEvent *event)
 /* contextual render, using current scene, view3d? */
 void RENDER_OT_render(wmOperatorType *ot)
 {
+	PropertyRNA *prop;
+
 	/* identifiers */
 	ot->name = "Render";
 	ot->description = "Render active scene";
@@ -613,7 +644,9 @@ void RENDER_OT_render(wmOperatorType *ot)
 
 	RNA_def_boolean(ot->srna, "animation", 0, "Animation", "Render files from the animation range of this scene");
 	RNA_def_boolean(ot->srna, "write_still", 0, "Write Image", "Save rendered the image to the output path (used only when animation is disabled)");
-	RNA_def_string(ot->srna, "layer", "", RE_MAXNAME, "Render Layer", "Single render layer to re-render (used only when animation is disabled)");
-	RNA_def_string(ot->srna, "scene", "", MAX_ID_NAME - 2, "Scene", "Scene to render, current scene if not specified");
+	prop = RNA_def_string(ot->srna, "layer", "", RE_MAXNAME, "Render Layer", "Single render layer to re-render (used only when animation is disabled)");
+	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
+	prop = RNA_def_string(ot->srna, "scene", "", MAX_ID_NAME - 2, "Scene", "Scene to render, current scene if not specified");
+	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
 }
 
