@@ -43,6 +43,7 @@
 
 #include "DNA_scene_types.h"
 #include "DNA_object_types.h"
+#include "DNA_world_types.h"
 
 #include "BKE_context.h"
 #include "BKE_global.h"
@@ -63,6 +64,7 @@
 #include "RE_pipeline.h"
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
+#include "IMB_colormanagement.h"
 
 #include "RNA_access.h"
 #include "RNA_define.h"
@@ -138,35 +140,45 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
 		SeqRenderData context;
 		int chanshown = oglrender->sseq ? oglrender->sseq->chanshown : 0;
 
-		context = seq_new_render_data(oglrender->bmain, scene, oglrender->sizex, oglrender->sizey, 100.0f);
+		context = BKE_sequencer_new_render_data(oglrender->bmain, scene, oglrender->sizex, oglrender->sizey, 100.0f);
 
-		ibuf = give_ibuf_seq(context, CFRA, chanshown);
+		ibuf = BKE_sequencer_give_ibuf(context, CFRA, chanshown);
 
 		if (ibuf) {
+			ImBuf *linear_ibuf;
+
 			BLI_assert((oglrender->sizex == ibuf->x) && (oglrender->sizey == ibuf->y));
 
-			if (ibuf->rect_float == NULL) {
-				/* internally sequencer working in sRGB space and stores both bytes and float
-				 * buffers in sRGB space, but if byte->float onversion doesn't happen in sequencer
-				 * (e.g. when adding image sequence/movie into sequencer) there'll be only
-				 * byte buffer and profile will still indicate sRGB->linear space conversion is needed
-				 * here we're ensure there'll be no conversion happen and float buffer would store
-				 * linear frame (sergey) */
-				ibuf->profile = IB_PROFILE_NONE;
-				IMB_float_from_rect(ibuf);
+			linear_ibuf = IMB_dupImBuf(ibuf);
+			IMB_freeImBuf(ibuf);
+
+			if (linear_ibuf->rect_float == NULL) {
+				/* internally sequencer working in display space and stores both bytes and float buffers in that space.
+				 * It is possible that byte->float onversion didn't happen in sequencer (e.g. when adding image sequence/movie
+				 * into sequencer) there'll be only byte buffer. Create float buffer from existing byte buffer, making it linear
+				 */
+
+				IMB_float_from_rect(linear_ibuf);
+			}
+			else {
+				/* ensure float buffer is in linear space, not in display space */
+				BKE_sequencer_imbuf_from_sequencer_space(scene, linear_ibuf);
 			}
 
-			memcpy(rr->rectf, ibuf->rect_float, sizeof(float) * 4 * oglrender->sizex * oglrender->sizey);
 
-			IMB_freeImBuf(ibuf);
+			memcpy(rr->rectf, linear_ibuf->rect_float, sizeof(float) * 4 * oglrender->sizex * oglrender->sizey);
+
+			IMB_freeImBuf(linear_ibuf);
 		}
 	}
 	else if (view_context) {
+		ED_view3d_draw_offscreen_init(scene, v3d);
+
 		GPU_offscreen_bind(oglrender->ofs); /* bind */
 
 		/* render 3d view */
 		if (rv3d->persp == RV3D_CAMOB && v3d->camera) {
-			/*int is_ortho= scene->r.mode & R_ORTHO;*/
+			/*int is_ortho = scene->r.mode & R_ORTHO;*/
 			camera = v3d->camera;
 			RE_GetCameraWindow(oglrender->re, camera, scene->r.cfra, winmat);
 			
@@ -180,7 +192,7 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
 			else perspective_m4(winmat, viewplane.xmin, viewplane.xmax, viewplane.ymin, viewplane.ymax, clipsta, clipend);
 		}
 
-		if ((scene->r.mode & R_OSA) == 0) { 
+		if ((scene->r.mode & R_OSA) == 0) {
 			ED_view3d_draw_offscreen(scene, v3d, ar, sizex, sizey, NULL, winmat, TRUE);
 			GPU_offscreen_read_pixels(oglrender->ofs, GL_FLOAT, rr->rectf);
 		}
@@ -192,7 +204,7 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
 			float *accum_tmp = MEM_mallocN(sizex * sizey * sizeof(float) * 4, "accum2");
 			int j;
 
-			BLI_initjit(jit_ofs[0], scene->r.osa);
+			BLI_jitter_init(jit_ofs[0], scene->r.osa);
 
 			/* first sample buffer, also initializes 'rv3d->persmat' */
 			ED_view3d_draw_offscreen(scene, v3d, ar, sizex, sizey, NULL, winmat, TRUE);
@@ -221,7 +233,8 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
 	else {
 		/* shouldnt suddenly give errors mid-render but possible */
 		char err_out[256] = "unknown";
-		ImBuf *ibuf_view = ED_view3d_draw_offscreen_imbuf_simple(scene, scene->camera, oglrender->sizex, oglrender->sizey, IB_rectfloat, OB_SOLID, TRUE, err_out);
+		ImBuf *ibuf_view = ED_view3d_draw_offscreen_imbuf_simple(scene, scene->camera, oglrender->sizex, oglrender->sizey,
+		                                                         IB_rectfloat, OB_SOLID, FALSE, TRUE, R_ALPHAPREMUL, err_out);
 		camera = scene->camera;
 
 		if (ibuf_view) {
@@ -232,28 +245,34 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
 			fprintf(stderr, "%s: failed to get buffer, %s\n", __func__, err_out);
 		}
 	}
-	
-	/* rr->rectf is now filled with image data */
 
-	if ((scene->r.stamp & R_STAMP_ALL) && (scene->r.stamp & R_STAMP_DRAW))
-		BKE_stamp_buf(scene, camera, NULL, rr->rectf, rr->rectx, rr->recty, 4);
+	if (scene->r.alphamode == R_ADDSKY) {
+		float sky_color[3];
+		ED_view3d_offscreen_sky_color_get(scene, sky_color);
+		IMB_alpha_under_color_float(rr->rectf, sizex, sizey, sky_color);
+	}
 
 	/* note on color management:
 	 *
 	 * OpenGL renders into sRGB colors, but render buffers are expected to be
-	 * linear if color management is enabled. So we convert to linear here, so
-	 * the conversion back to bytes using the color management flag can make it
-	 * sRGB again, and so that e.g. openexr saving also saves the correct linear
-	 * float buffer. */
+	 * linear So we convert to linear here, so the conversion back to bytes can make it
+	 * sRGB (or other display space) again, and so that e.g. openexr saving also saves the
+	 * correct linear float buffer.
+	 */
 
-	if (oglrender->scene->r.color_mgt_flag & R_COLOR_MANAGEMENT) {
-		int predivide = 0; /* no alpha */
+	if (!oglrender->is_sequencer) {
+		/* sequencer has got tricker ocnversion happened above */
 
 		IMB_buffer_float_from_float(rr->rectf, rr->rectf,
-		                            4, IB_PROFILE_LINEAR_RGB, IB_PROFILE_SRGB, predivide,
+		                            4, IB_PROFILE_LINEAR_RGB, IB_PROFILE_SRGB, TRUE,
 		                            oglrender->sizex, oglrender->sizey, oglrender->sizex, oglrender->sizex);
 	}
 
+	/* rr->rectf is now filled with image data */
+
+	if ((scene->r.stamp & R_STAMP_ALL) && (scene->r.stamp & R_STAMP_DRAW))
+		BKE_stamp_buf(scene, camera, NULL, rr->rectf, rr->rectx, rr->recty, 4);
+
 	RE_ReleaseResult(oglrender->re);
 
 	/* update byte from float buffer */
@@ -270,14 +289,14 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
 				IMB_color_to_bw(ibuf);
 			}
 
-			BKE_makepicstring(name, scene->r.pic, oglrender->bmain->name, scene->r.cfra, scene->r.im_format.imtype, scene->r.scemode & R_EXTENSION, FALSE);
-			ok = BKE_write_ibuf_as(ibuf, name, &scene->r.im_format, TRUE); /* no need to stamp here */
+			BKE_makepicstring(name, scene->r.pic, oglrender->bmain->name, scene->r.cfra, &scene->r.im_format, scene->r.scemode & R_EXTENSION, FALSE);
+			ok = BKE_imbuf_write_as(ibuf, name, &scene->r.im_format, TRUE); /* no need to stamp here */
 			if (ok) printf("OpenGL Render written to '%s'\n", name);
 			else printf("OpenGL Render failed to write '%s'\n", name);
 		}
 	}
 	
-	BKE_image_release_ibuf(oglrender->ima, lock);
+	BKE_image_release_ibuf(oglrender->ima, ibuf, lock);
 }
 
 static int screen_opengl_render_init(bContext *C, wmOperator *op)
@@ -297,7 +316,7 @@ static int screen_opengl_render_init(bContext *C, wmOperator *op)
 	char err_out[256] = "unknown";
 
 	if (G.background) {
-		BKE_report(op->reports, RPT_ERROR, "Can't use OpenGL render in background mode (no opengl context)");
+		BKE_report(op->reports, RPT_ERROR, "Cannot use OpenGL render in background mode (no opengl context)");
 		return 0;
 	}
 
@@ -309,7 +328,7 @@ static int screen_opengl_render_init(bContext *C, wmOperator *op)
 	}
 
 	/* only one render job at a time */
-	if (WM_jobs_test(CTX_wm_manager(C), scene))
+	if (WM_jobs_test(CTX_wm_manager(C), scene, WM_JOB_TYPE_RENDER))
 		return 0;
 	
 	if (!is_view_context && scene->camera == NULL) {
@@ -318,12 +337,12 @@ static int screen_opengl_render_init(bContext *C, wmOperator *op)
 	}
 
 	if (!is_animation && is_write_still && BKE_imtype_is_movie(scene->r.im_format.imtype)) {
-		BKE_report(op->reports, RPT_ERROR, "Can't write a single file with an animation format selected");
+		BKE_report(op->reports, RPT_ERROR, "Cannot write a single file with an animation format selected");
 		return 0;
 	}
 
-	/* stop all running jobs, currently previews frustrate Render */
-	WM_jobs_stop_all(CTX_wm_manager(C));
+	/* stop all running jobs, except screen one. currently previews frustrate Render */
+	WM_jobs_kill_all_except(CTX_wm_manager(C), CTX_wm_screen(C));
 
 	/* create offscreen buffer */
 	sizex = (scene->r.size * scene->r.xsch) / 100;
@@ -333,7 +352,7 @@ static int screen_opengl_render_init(bContext *C, wmOperator *op)
 	ofs = GPU_offscreen_create(sizex, sizey, err_out);
 
 	if (!ofs) {
-		BKE_reportf(op->reports, RPT_ERROR, "Failed to create OpenGL offscreen buffer, %s", err_out);
+		BKE_reportf(op->reports, RPT_ERROR, "Failed to create OpenGL off-screen buffer, %s", err_out);
 		return 0;
 	}
 
@@ -355,7 +374,7 @@ static int screen_opengl_render_init(bContext *C, wmOperator *op)
 
 	oglrender->is_sequencer = is_sequencer;
 	if (is_sequencer) {
-		oglrender->sseq = CTX_wm_space_seq(C);;
+		oglrender->sseq = CTX_wm_space_seq(C);
 	}
 
 
@@ -371,7 +390,7 @@ static int screen_opengl_render_init(bContext *C, wmOperator *op)
 
 		/* MUST be cleared on exit */
 		oglrender->scene->customdata_mask_modal = (ED_view3d_datamask(oglrender->scene, oglrender->v3d) |
-		                                           ED_view3d_object_datamask(oglrender->scene) );
+		                                           ED_view3d_object_datamask(oglrender->scene));
 
 		/* apply immediately in case we're rendering from a script,
 		 * running notifiers again will overwrite */
@@ -413,7 +432,7 @@ static void screen_opengl_render_end(bContext *C, OGLRender *oglrender)
 
 	if (oglrender->timer) { /* exec will not have a timer */
 		scene->r.cfra = oglrender->cfrao;
-		scene_update_for_newframe(bmain, scene, screen_opengl_layers(oglrender));
+		BKE_scene_update_for_newframe(bmain, scene, screen_opengl_layers(oglrender));
 
 		WM_event_remove_timer(CTX_wm_manager(C), CTX_wm_window(C), oglrender->timer);
 	}
@@ -451,7 +470,7 @@ static int screen_opengl_render_anim_initialize(bContext *C, wmOperator *op)
 	scene = oglrender->scene;
 
 	oglrender->reports = op->reports;
-	oglrender->mh = BKE_get_movie_handle(scene->r.im_format.imtype);
+	oglrender->mh = BKE_movie_handle_get(scene->r.im_format.imtype);
 	if (BKE_imtype_is_movie(scene->r.im_format.imtype)) {
 		if (!oglrender->mh->start_movie(scene, &scene->r, oglrender->sizex, oglrender->sizey, oglrender->reports)) {
 			screen_opengl_render_end(C, oglrender);
@@ -470,35 +489,53 @@ static int screen_opengl_render_anim_step(bContext *C, wmOperator *op)
 	Main *bmain = CTX_data_main(C);
 	OGLRender *oglrender = op->customdata;
 	Scene *scene = oglrender->scene;
-	ImBuf *ibuf;
+	ImBuf *ibuf, *ibuf_save = NULL;
 	void *lock;
 	char name[FILE_MAX];
 	int ok = 0;
 	const short view_context = (oglrender->v3d != NULL);
 	Object *camera = NULL;
-
-	/* update animated image textures for gpu, etc,
-	 * call before scene_update_for_newframe so modifiers with textures don't lag 1 frame */
-	ED_image_update_frame(bmain, scene->r.cfra);
+	int is_movie;
 
 	/* go to next frame */
+	if (CFRA < oglrender->nfra)
+		CFRA++;
 	while (CFRA < oglrender->nfra) {
 		unsigned int lay = screen_opengl_layers(oglrender);
 
 		if (lay & 0xFF000000)
 			lay &= 0xFF000000;
 
-		scene_update_for_newframe(bmain, scene, lay);
+		BKE_scene_update_for_newframe(bmain, scene, lay);
 		CFRA++;
 	}
 
-	scene_update_for_newframe(bmain, scene, screen_opengl_layers(oglrender));
+	is_movie = BKE_imtype_is_movie(scene->r.im_format.imtype);
+
+	if (!is_movie) {
+		BKE_makepicstring(name, scene->r.pic, oglrender->bmain->name, scene->r.cfra, &scene->r.im_format, scene->r.scemode & R_EXTENSION, TRUE);
+
+		if ((scene->r.mode & R_NO_OVERWRITE) && BLI_exists(name)) {
+			printf("skipping existing frame \"%s\"\n", name);
+
+			/* go to next frame */
+			oglrender->nfra += scene->r.frame_step;
+
+			return 1;
+		}
+	}
+
+	/* update animated image textures for gpu, etc,
+	 * call before BKE_scene_update_for_newframe so modifiers with textures don't lag 1 frame */
+	ED_image_update_frame(bmain, CFRA);
+
+	BKE_scene_update_for_newframe(bmain, scene, screen_opengl_layers(oglrender));
 
 	if (view_context) {
 		if (oglrender->rv3d->persp == RV3D_CAMOB && oglrender->v3d->camera && oglrender->v3d->scenelock) {
-			/* since scene_update_for_newframe() is used rather
+			/* since BKE_scene_update_for_newframe() is used rather
 			 * then ED_update_for_newframe() the camera needs to be set */
-			if (scene_camera_switch_update(scene)) {
+			if (BKE_scene_camera_switch_update(scene)) {
 				oglrender->v3d->camera = scene->camera;
 			}
 
@@ -506,7 +543,7 @@ static int screen_opengl_render_anim_step(bContext *C, wmOperator *op)
 		}
 	}
 	else {
-		scene_camera_switch_update(scene);
+		BKE_scene_camera_switch_update(scene);
 
 		camera = scene->camera;
 	}
@@ -518,26 +555,48 @@ static int screen_opengl_render_anim_step(bContext *C, wmOperator *op)
 	ibuf = BKE_image_acquire_ibuf(oglrender->ima, &oglrender->iuser, &lock);
 
 	if (ibuf) {
-		/* color -> greyscale */
+		int needs_free = FALSE;
+
+		ibuf_save = ibuf;
+
+		if (is_movie || !BKE_imtype_requires_linear_float(scene->r.im_format.imtype)) {
+			ibuf_save = IMB_colormanagement_imbuf_for_write(ibuf, TRUE, TRUE, &scene->view_settings,
+			                                                &scene->display_settings, &scene->r.im_format);
+
+			needs_free = TRUE;
+		}
+
+		/* color -> grayscale */
 		/* editing directly would alter the render view */
 		if (scene->r.im_format.planes == R_IMF_PLANES_BW) {
-			ImBuf *ibuf_bw = IMB_dupImBuf(ibuf);
+			ImBuf *ibuf_bw = IMB_dupImBuf(ibuf_save);
 			IMB_color_to_bw(ibuf_bw);
-			// IMB_freeImBuf(ibuf); /* owned by the image */
-			ibuf = ibuf_bw;
+
+			if (needs_free)
+				IMB_freeImBuf(ibuf_save);
+
+			ibuf_save = ibuf_bw;
 		}
 		else {
 			/* this is lightweight & doesnt re-alloc the buffers, only do this
 			 * to save the correct bit depth since the image is always RGBA */
-			ImBuf *ibuf_cpy = IMB_allocImBuf(ibuf->x, ibuf->y, scene->r.im_format.planes, 0);
-			ibuf_cpy->rect = ibuf->rect;
-			ibuf_cpy->rect_float = ibuf->rect_float;
-			ibuf_cpy->zbuf_float = ibuf->zbuf_float;
-			ibuf = ibuf_cpy;
+			ImBuf *ibuf_cpy = IMB_allocImBuf(ibuf_save->x, ibuf_save->y, scene->r.im_format.planes, 0);
+
+			ibuf_cpy->rect = ibuf_save->rect;
+			ibuf_cpy->rect_float = ibuf_save->rect_float;
+			ibuf_cpy->zbuf_float = ibuf_save->zbuf_float;
+
+			if (needs_free) {
+				ibuf_cpy->mall = ibuf_save->mall;
+				ibuf_save->mall = 0;
+				IMB_freeImBuf(ibuf_save);
+			}
+
+			ibuf_save = ibuf_cpy;
 		}
 
-		if (BKE_imtype_is_movie(scene->r.im_format.imtype)) {
-			ok = oglrender->mh->append_movie(&scene->r, SFRA, CFRA, (int *)ibuf->rect,
+		if (is_movie) {
+			ok = oglrender->mh->append_movie(&scene->r, SFRA, CFRA, (int *)ibuf_save->rect,
 			                                 oglrender->sizex, oglrender->sizey, oglrender->reports);
 			if (ok) {
 				printf("Append frame %d", scene->r.cfra);
@@ -545,8 +604,7 @@ static int screen_opengl_render_anim_step(bContext *C, wmOperator *op)
 			}
 		}
 		else {
-			BKE_makepicstring(name, scene->r.pic, oglrender->bmain->name, scene->r.cfra, scene->r.im_format.imtype, scene->r.scemode & R_EXTENSION, TRUE);
-			ok = BKE_write_ibuf_stamp(scene, camera, ibuf, name, &scene->r.im_format);
+			ok = BKE_imbuf_write_stamp(scene, camera, ibuf_save, name, &scene->r.im_format);
 
 			if (ok == 0) {
 				printf("Write error: cannot save %s\n", name);
@@ -558,21 +616,20 @@ static int screen_opengl_render_anim_step(bContext *C, wmOperator *op)
 			}
 		}
 
-		/* imbuf knows which rects are not part of ibuf */
-		IMB_freeImBuf(ibuf);
+		if (needs_free)
+			IMB_freeImBuf(ibuf_save);
 	}
 
-	BKE_image_release_ibuf(oglrender->ima, lock);
+	BKE_image_release_ibuf(oglrender->ima, ibuf, lock);
 
 	/* movie stats prints have no line break */
 	printf("\n");
 
 	/* go to next frame */
 	oglrender->nfra += scene->r.frame_step;
-	scene->r.cfra++;
 
 	/* stop at the end or on error */
-	if (scene->r.cfra > PEFRA || !ok) {
+	if (CFRA >= PEFRA || !ok) {
 		screen_opengl_render_end(C, op->customdata);
 		return 0;
 	}
@@ -668,7 +725,7 @@ static int screen_opengl_render_exec(bContext *C, wmOperator *op)
 		}
 	}
 
-	// no redraw needed, we leave state as we entered it
+	/* no redraw needed, we leave state as we entered it */
 //	ED_update_for_newframe(C, 1);
 	WM_event_add_notifier(C, NC_SCENE | ND_RENDER_RESULT, CTX_data_scene(C));
 
