@@ -33,6 +33,7 @@
 
 #include "DNA_lamp_types.h"
 #include "DNA_material_types.h"
+#include "DNA_meshdata_types.h"
 #include "DNA_node_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
@@ -46,6 +47,7 @@
 
 #include "BKE_context.h"
 #include "BKE_depsgraph.h"
+#include "BKE_DerivedMesh.h"
 #include "BKE_icons.h"
 #include "BKE_image.h"
 #include "BKE_main.h"
@@ -56,14 +58,18 @@
 #include "BKE_world.h"
 
 #include "GPU_material.h"
+#include "GPU_buffers.h"
 
 #include "RE_engine.h"
+#include "RE_pipeline.h"
 
 #include "ED_node.h"
 #include "ED_render.h"
 
 #include "render_intern.h"  // own include
 
+extern Material defmaterial;
+
 /***************************** Render Engines ********************************/
 
 void ED_render_scene_update(Main *bmain, Scene *scene, int updated)
@@ -74,12 +80,19 @@ void ED_render_scene_update(Main *bmain, Scene *scene, int updated)
 	bScreen *sc;
 	ScrArea *sa;
 	ARegion *ar;
+	static int recursive_check = FALSE;
 
 	/* don't do this render engine update if we're updating the scene from
 	 * other threads doing e.g. rendering or baking jobs */
 	if (!BLI_thread_is_main())
 		return;
 
+	/* don't call this recursively for frame updates */
+	if (recursive_check)
+		return;
+	
+	recursive_check = TRUE;
+
 	C = CTX_create();
 	CTX_data_main_set(C, bmain);
 	CTX_data_scene_set(C, scene);
@@ -114,6 +127,8 @@ void ED_render_scene_update(Main *bmain, Scene *scene, int updated)
 	}
 
 	CTX_free(C);
+
+	recursive_check = FALSE;
 }
 
 void ED_render_engine_area_exit(ScrArea *sa)
@@ -127,7 +142,7 @@ void ED_render_engine_area_exit(ScrArea *sa)
 	for (ar = sa->regionbase.first; ar; ar = ar->next) {
 		RegionView3D *rv3d;
 
-		if (ar->regiontype != RGN_TYPE_WINDOW)
+		if (ar->regiontype != RGN_TYPE_WINDOW || !(ar->regiondata))
 			continue;
 		
 		rv3d = ar->regiondata;
@@ -144,10 +159,16 @@ void ED_render_engine_changed(Main *bmain)
 	/* on changing the render engine type, clear all running render engines */
 	bScreen *sc;
 	ScrArea *sa;
+	Scene *scene;
 
 	for (sc = bmain->screen.first; sc; sc = sc->id.next)
 		for (sa = sc->areabase.first; sa; sa = sa->next)
 			ED_render_engine_area_exit(sa);
+
+	RE_FreePersistentData();
+
+	for (scene = bmain->scene.first; scene; scene = scene->id.next)
+		ED_render_id_flush_update(bmain, &scene->id);
 }
 
 /***************************** Updates ***********************************
@@ -214,6 +235,9 @@ static int nodes_use_material(bNodeTree *ntree, Material *ma)
 static void material_changed(Main *bmain, Material *ma)
 {
 	Material *parent;
+	Object *ob;
+	Scene *scene;
+	int texture_draw = FALSE;
 
 	/* icons */
 	BKE_icon_changed(BKE_icon_getid(&ma->id));
@@ -224,66 +248,45 @@ static void material_changed(Main *bmain, Material *ma)
 
 	/* find node materials using this */
 	for (parent = bmain->mat.first; parent; parent = parent->id.next) {
-		if (parent->use_nodes && parent->nodetree && nodes_use_material(parent->nodetree, ma)) ;
-		else continue;
+		if (parent->use_nodes && parent->nodetree && nodes_use_material(parent->nodetree, ma)) {
+			/* pass */
+		}
+		else {
+			continue;
+		}
 
 		BKE_icon_changed(BKE_icon_getid(&parent->id));
 
 		if (parent->gpumaterial.first)
 			GPU_material_free(parent);
 	}
-}
 
-static void texture_changed(Main *bmain, Tex *tex)
-{
-	Material *ma;
-	Lamp *la;
-	World *wo;
-	Scene *scene;
-	bNode *node;
-
-	/* icons */
-	BKE_icon_changed(BKE_icon_getid(&tex->id));
-
-	/* find materials */
-	for (ma = bmain->mat.first; ma; ma = ma->id.next) {
-		if (mtex_use_tex(ma->mtex, MAX_MTEX, tex)) ;
-		else if (ma->use_nodes && ma->nodetree && nodes_use_tex(ma->nodetree, tex)) ;
-		else continue;
-
-		BKE_icon_changed(BKE_icon_getid(&ma->id));
-
-		if (ma->gpumaterial.first)
-			GPU_material_free(ma);
-	}
-
-	/* find lamps */
-	for (la = bmain->lamp.first; la; la = la->id.next) {
-		if (mtex_use_tex(la->mtex, MAX_MTEX, tex)) ;
-		else if (la->nodetree && nodes_use_tex(la->nodetree, tex)) ;
-		else continue;
-
-		BKE_icon_changed(BKE_icon_getid(&la->id));
-	}
-
-	/* find worlds */
-	for (wo = bmain->world.first; wo; wo = wo->id.next) {
-		if (mtex_use_tex(wo->mtex, MAX_MTEX, tex)) ;
-		else if (wo->nodetree && nodes_use_tex(wo->nodetree, tex)) ;
-		else continue;
-
-		BKE_icon_changed(BKE_icon_getid(&wo->id));
-	}
-
-	/* find compositing nodes */
+	/* find if we have a scene with textured display */
 	for (scene = bmain->scene.first; scene; scene = scene->id.next) {
-		if (scene->use_nodes && scene->nodetree) {
-			for (node = scene->nodetree->nodes.first; node; node = node->next) {
-				if (node->id == &tex->id)
-					ED_node_changed_update(&scene->id, node);
+		if (scene->customdata_mask & CD_MASK_MTFACE) {
+			texture_draw = TRUE;
+			break;
+		}
+	}
+
+	/* find textured objects */
+	if (texture_draw && !(U.gameflags & USER_DISABLE_VBO)) {
+		for (ob = bmain->object.first; ob; ob = ob->id.next) {
+			DerivedMesh *dm = ob->derivedFinal;
+			Material ***material = give_matarar(ob);
+			short a, *totmaterial = give_totcolp(ob);
+
+			if (dm && totmaterial && material) {
+				for (a = 0; a < *totmaterial; a++) {
+					if ((*material)[a] == ma) {
+						GPU_drawobject_free(dm);
+						break;
+					}
+				}
 			}
 		}
 	}
+
 }
 
 static void lamp_changed(Main *bmain, Lamp *la)
@@ -302,6 +305,110 @@ static void lamp_changed(Main *bmain, Lamp *la)
 	for (ma = bmain->mat.first; ma; ma = ma->id.next)
 		if (ma->gpumaterial.first)
 			GPU_material_free(ma);
+
+	if (defmaterial.gpumaterial.first)
+		GPU_material_free(&defmaterial);
+}
+
+static int material_uses_texture(Material *ma, Tex *tex)
+{
+	if (mtex_use_tex(ma->mtex, MAX_MTEX, tex))
+		return TRUE;
+	else if (ma->use_nodes && ma->nodetree && nodes_use_tex(ma->nodetree, tex))
+		return TRUE;
+	
+	return FALSE;
+}
+
+static void texture_changed(Main *bmain, Tex *tex)
+{
+	Material *ma;
+	Lamp *la;
+	World *wo;
+	Scene *scene;
+	Object *ob;
+	bNode *node;
+	int texture_draw = FALSE;
+
+	/* icons */
+	BKE_icon_changed(BKE_icon_getid(&tex->id));
+
+	/* find materials */
+	for (ma = bmain->mat.first; ma; ma = ma->id.next) {
+		if (!material_uses_texture(ma, tex))
+			continue;
+
+		BKE_icon_changed(BKE_icon_getid(&ma->id));
+
+		if (ma->gpumaterial.first)
+			GPU_material_free(ma);
+	}
+
+	/* find lamps */
+	for (la = bmain->lamp.first; la; la = la->id.next) {
+		if (mtex_use_tex(la->mtex, MAX_MTEX, tex)) {
+			lamp_changed(bmain, la);
+		}
+		else if (la->nodetree && nodes_use_tex(la->nodetree, tex)) {
+			lamp_changed(bmain, la);
+		}
+		else {
+			continue;
+		}
+	}
+
+	/* find worlds */
+	for (wo = bmain->world.first; wo; wo = wo->id.next) {
+		if (mtex_use_tex(wo->mtex, MAX_MTEX, tex)) {
+			/* pass */
+		}
+		else if (wo->nodetree && nodes_use_tex(wo->nodetree, tex)) {
+			/* pass */
+		}
+		else {
+			continue;
+		}
+
+		BKE_icon_changed(BKE_icon_getid(&wo->id));
+	}
+
+	/* find compositing nodes */
+	for (scene = bmain->scene.first; scene; scene = scene->id.next) {
+		if (scene->use_nodes && scene->nodetree) {
+			for (node = scene->nodetree->nodes.first; node; node = node->next) {
+				if (node->id == &tex->id)
+					ED_node_changed_update(&scene->id, node);
+			}
+		}
+
+		if (scene->customdata_mask & CD_MASK_MTFACE)
+			texture_draw = TRUE;
+	}
+
+	/* find textured objects */
+	if (texture_draw && !(U.gameflags & USER_DISABLE_VBO)) {
+		for (ob = bmain->object.first; ob; ob = ob->id.next) {
+			DerivedMesh *dm = ob->derivedFinal;
+			Material ***material = give_matarar(ob);
+			short a, *totmaterial = give_totcolp(ob);
+
+			if (dm && totmaterial && material) {
+				for (a = 0; a < *totmaterial; a++) {
+					Material *ma;
+
+					if (ob->matbits && ob->matbits[a])
+						ma = ob->mat[a];
+					else
+						ma = (*material)[a];
+
+					if (ma && material_uses_texture(ma, tex)) {
+						GPU_drawobject_free(dm);
+						break;
+					}
+				}
+			}
+		}
+	}
 }
 
 static void world_changed(Main *bmain, World *wo)
@@ -315,6 +422,9 @@ static void world_changed(Main *bmain, World *wo)
 	for (ma = bmain->mat.first; ma; ma = ma->id.next)
 		if (ma->gpumaterial.first)
 			GPU_material_free(ma);
+
+	if (defmaterial.gpumaterial.first)
+		GPU_material_free(&defmaterial);
 }
 
 static void image_changed(Main *bmain, Image *ima)
@@ -343,10 +453,19 @@ static void scene_changed(Main *bmain, Scene *UNUSED(scene))
 	for (ma = bmain->mat.first; ma; ma = ma->id.next)
 		if (ma->gpumaterial.first)
 			GPU_material_free(ma);
+
+	if (defmaterial.gpumaterial.first)
+		GPU_material_free(&defmaterial);
 }
 
 void ED_render_id_flush_update(Main *bmain, ID *id)
 {
+	/* this can be called from render or baking thread when a python script makes
+	 * changes, in that case we don't want to do any editor updates, and making
+	 * GPU changes is not possible because OpenGL only works in the main thread */
+	if (!BLI_thread_is_main())
+		return;
+
 	switch (GS(id->name)) {
 		case ID_MA:
 			material_changed(bmain, (Material *)id);
