@@ -31,6 +31,7 @@
 #include "MEM_guardedalloc.h"
 
 #include "BLI_math.h"
+#include "BLI_rect.h"
 #include "BLI_utildefines.h"
 
 #include "DNA_brush_types.h"
@@ -42,6 +43,7 @@
 
 #include "BKE_brush.h"
 #include "BKE_context.h"
+#include "BKE_image.h"
 #include "BKE_paint.h"
 
 #include "WM_api.h"
@@ -69,7 +71,7 @@ typedef struct Snapshot {
 	float size[3];
 	float ofs[3];
 	float rot;
-	int brush_size;
+	int BKE_brush_size_get;
 	int winx;
 	int winy;
 	int brush_map_mode;
@@ -86,9 +88,9 @@ static int same_snap(Snapshot *snap, Brush *brush, ViewContext *vc)
 	         mtex->rot == snap->rot) &&
 
 	        /* make brush smaller shouldn't cause a resample */
-	        ((mtex->brush_map_mode == MTEX_MAP_MODE_FIXED &&
-	          (brush_size(vc->scene, brush) <= snap->brush_size)) ||
-	         (brush_size(vc->scene, brush) == snap->brush_size)) &&
+	        ((mtex->brush_map_mode == MTEX_MAP_MODE_VIEW &&
+	          (BKE_brush_size_get(vc->scene, brush) <= snap->BKE_brush_size_get)) ||
+	         (BKE_brush_size_get(vc->scene, brush) == snap->BKE_brush_size_get)) &&
 
 	        (mtex->brush_map_mode == snap->brush_map_mode) &&
 	        (vc->ar->winx == snap->winx) &&
@@ -110,12 +112,12 @@ static void make_snap(Snapshot *snap, Brush *brush, ViewContext *vc)
 		snap->rot = -1;
 	}
 
-	snap->brush_size = brush_size(vc->scene, brush);
+	snap->BKE_brush_size_get = BKE_brush_size_get(vc->scene, brush);
 	snap->winx = vc->ar->winx;
 	snap->winy = vc->ar->winy;
 }
 
-static int load_tex(Sculpt *sd, Brush *br, ViewContext *vc)
+static int load_tex(Brush *br, ViewContext *vc)
 {
 	static GLuint overlay_texture = 0;
 	static int init = 0;
@@ -129,10 +131,6 @@ static int load_tex(Sculpt *sd, Brush *br, ViewContext *vc)
 	int size;
 	int j;
 	int refresh;
-
-#ifndef _OPENMP
-	(void)sd; /* quied unused warning */
-#endif
 	
 	if (br->mtex.brush_map_mode == MTEX_MAP_MODE_TILED && !br->mtex.tex) return 0;
 	
@@ -146,6 +144,8 @@ static int load_tex(Sculpt *sd, Brush *br, ViewContext *vc)
 	    !same_snap(&snap, br, vc);
 
 	if (refresh) {
+		struct ImagePool *pool = NULL;
+
 		if (br->mtex.tex && br->mtex.tex->preview)
 			tex_changed_timestamp = br->mtex.tex->preview->changed_timestamp[0];
 
@@ -154,8 +154,8 @@ static int load_tex(Sculpt *sd, Brush *br, ViewContext *vc)
 
 		make_snap(&snap, br, vc);
 
-		if (br->mtex.brush_map_mode == MTEX_MAP_MODE_FIXED) {
-			int s = brush_size(vc->scene, br);
+		if (br->mtex.brush_map_mode == MTEX_MAP_MODE_VIEW) {
+			int s = BKE_brush_size_get(vc->scene, br);
 			int r = 1;
 
 			for (s >>= 1; s > 0; s >>= 1)
@@ -185,7 +185,10 @@ static int load_tex(Sculpt *sd, Brush *br, ViewContext *vc)
 
 		buffer = MEM_mallocN(sizeof(GLubyte) * size * size, "load_tex");
 
-		#pragma omp parallel for schedule(static) if (sd->flags & SCULPT_USE_OPENMP)
+		if (br->mtex.tex)
+			pool = BKE_image_pool_new();
+
+		#pragma omp parallel for schedule(static)
 		for (j = 0; j < size; j++) {
 			int i;
 			float y;
@@ -196,7 +199,7 @@ static int load_tex(Sculpt *sd, Brush *br, ViewContext *vc)
 				// largely duplicated from tex_strength
 
 				const float rotation = -br->mtex.rot;
-				float radius = brush_size(vc->scene, br);
+				float radius = BKE_brush_size_get(vc->scene, br);
 				int index = j * size + i;
 				float x;
 				float avg;
@@ -235,12 +238,12 @@ static int load_tex(Sculpt *sd, Brush *br, ViewContext *vc)
 					x += br->mtex.ofs[0];
 					y += br->mtex.ofs[1];
 
-					avg = br->mtex.tex ? paint_get_tex_pixel(br, x, y) : 1;
+					avg = br->mtex.tex ? paint_get_tex_pixel(br, x, y, pool) : 1;
 
 					avg += br->texture_sample_bias;
 
-					if (br->mtex.brush_map_mode == MTEX_MAP_MODE_FIXED)
-						avg *= brush_curve_strength(br, len, 1);  /* Falloff curve */
+					if (br->mtex.brush_map_mode == MTEX_MAP_MODE_VIEW)
+						avg *= BKE_brush_curve_strength(br, len, 1);  /* Falloff curve */
 
 					buffer[index] = 255 - (GLubyte)(255 * avg);
 				}
@@ -250,6 +253,9 @@ static int load_tex(Sculpt *sd, Brush *br, ViewContext *vc)
 			}
 		}
 
+		if (pool)
+			BKE_image_pool_free(pool);
+
 		if (!overlay_texture)
 			glGenTextures(1, &overlay_texture);
 	}
@@ -278,7 +284,7 @@ static int load_tex(Sculpt *sd, Brush *br, ViewContext *vc)
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 
-	if (br->mtex.brush_map_mode == MTEX_MAP_MODE_FIXED) {
+	if (br->mtex.brush_map_mode == MTEX_MAP_MODE_VIEW) {
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
 	}
@@ -294,7 +300,7 @@ static int project_brush_radius(ViewContext *vc,
 
 	ED_view3d_global_to_vector(vc->rv3d, location, view);
 
-	// create a vector that is not orthogonal to view
+	/* create a vector that is not orthogonal to view */
 
 	if (fabsf(view[0]) < 0.1f) {
 		nonortho[0] = view[0] + 1.0f;
@@ -312,20 +318,25 @@ static int project_brush_radius(ViewContext *vc,
 		nonortho[2] = view[2] + 1.0f;
 	}
 
-	// get a vector in the plane of the view
+	/* get a vector in the plane of the view */
 	cross_v3_v3v3(ortho, nonortho, view);
 	normalize_v3(ortho);
 
-	// make a point on the surface of the brush tagent to the view
+	/* make a point on the surface of the brush tagent to the view */
 	mul_v3_fl(ortho, radius);
 	add_v3_v3v3(offset, location, ortho);
 
-	// project the center of the brush, and the tangent point to the view onto the screen
-	project_float(vc->ar, location, p1);
-	project_float(vc->ar, offset, p2);
-
-	// the distance between these points is the size of the projected brush in pixels
-	return len_v2v2(p1, p2);
+	/* project the center of the brush, and the tangent point to the view onto the screen */
+	if ((ED_view3d_project_float_global(vc->ar, location, p1, V3D_PROJ_TEST_NOP) == V3D_PROJ_RET_OK) &&
+	    (ED_view3d_project_float_global(vc->ar, offset,   p2, V3D_PROJ_TEST_NOP) == V3D_PROJ_RET_OK))
+	{
+		/* the distance between these points is the size of the projected brush in pixels */
+		return len_v2v2(p1, p2);
+	}
+	else {
+		BLI_assert(0);  /* assert because the code that sets up the vectors should disallow this */
+		return 0;
+	}
 }
 
 static int sculpt_get_brush_geometry(bContext *C, ViewContext *vc,
@@ -333,8 +344,7 @@ static int sculpt_get_brush_geometry(bContext *C, ViewContext *vc,
                                      float location[3])
 {
 	Scene *scene = CTX_data_scene(C);
-	Paint *paint = paint_get_active(scene);
-	Brush *brush = paint_brush(paint);
+	Paint *paint = paint_get_active_from_context(C);
 	float window[2];
 	int hit;
 
@@ -342,14 +352,16 @@ static int sculpt_get_brush_geometry(bContext *C, ViewContext *vc,
 	window[1] = y + vc->ar->winrct.ymin;
 
 	if (vc->obact->sculpt && vc->obact->sculpt->pbvh &&
-	    sculpt_stroke_get_location(C, location, window)) {
+	    sculpt_stroke_get_location(C, location, window))
+	{
+		Brush *brush = paint_brush(paint);
 		*pixel_radius =
 		    project_brush_radius(vc,
-		                         brush_unprojected_radius(scene, brush),
+		                         BKE_brush_unprojected_radius_get(scene, brush),
 		                         location);
 
 		if (*pixel_radius == 0)
-			*pixel_radius = brush_size(scene, brush);
+			*pixel_radius = BKE_brush_size_get(scene, brush);
 
 		mul_m4_v3(vc->obact->obmat, location);
 
@@ -359,7 +371,7 @@ static int sculpt_get_brush_geometry(bContext *C, ViewContext *vc,
 		Sculpt *sd    = CTX_data_tool_settings(C)->sculpt;
 		Brush *brush = paint_brush(&sd->paint);
 
-		*pixel_radius = brush_size(scene, brush);
+		*pixel_radius = BKE_brush_size_get(scene, brush);
 		hit = 0;
 	}
 
@@ -369,15 +381,17 @@ static int sculpt_get_brush_geometry(bContext *C, ViewContext *vc,
 /* Draw an overlay that shows what effect the brush's texture will
  * have on brush strength */
 /* TODO: sculpt only for now */
-static void paint_draw_alpha_overlay(Sculpt *sd, Brush *brush,
+static void paint_draw_alpha_overlay(UnifiedPaintSettings *ups, Brush *brush,
                                      ViewContext *vc, int x, int y)
 {
 	rctf quad;
 
 	/* check for overlay mode */
 	if (!(brush->flag & BRUSH_TEXTURE_OVERLAY) ||
-	    !(ELEM(brush->mtex.brush_map_mode, MTEX_MAP_MODE_FIXED, MTEX_MAP_MODE_TILED)))
+	    !(ELEM(brush->mtex.brush_map_mode, MTEX_MAP_MODE_VIEW, MTEX_MAP_MODE_TILED)))
+	{
 		return;
+	}
 
 	/* save lots of GL state
 	 * TODO: check on whether all of these are needed? */
@@ -392,7 +406,7 @@ static void paint_draw_alpha_overlay(Sculpt *sd, Brush *brush,
 	             GL_VIEWPORT_BIT |
 	             GL_TEXTURE_BIT);
 
-	if (load_tex(sd, brush, vc)) {
+	if (load_tex(brush, vc)) {
 		glEnable(GL_BLEND);
 
 		glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
@@ -403,31 +417,31 @@ static void paint_draw_alpha_overlay(Sculpt *sd, Brush *brush,
 		glPushMatrix();
 		glLoadIdentity();
 
-		if (brush->mtex.brush_map_mode == MTEX_MAP_MODE_FIXED) {
+		if (brush->mtex.brush_map_mode == MTEX_MAP_MODE_VIEW) {
 			/* brush rotation */
 			glTranslatef(0.5, 0.5, 0);
 			glRotatef((double)RAD2DEGF((brush->flag & BRUSH_RAKE) ?
-			                           sd->last_angle : sd->special_rotation),
+			                           ups->last_angle : ups->special_rotation),
 			          0.0, 0.0, 1.0);
 			glTranslatef(-0.5f, -0.5f, 0);
 
 			/* scale based on tablet pressure */
-			if (sd->draw_pressure && brush_use_size_pressure(vc->scene, brush)) {
+			if (ups->draw_pressure && BKE_brush_use_size_pressure(vc->scene, brush)) {
 				glTranslatef(0.5f, 0.5f, 0);
-				glScalef(1.0f / sd->pressure_value, 1.0f / sd->pressure_value, 1);
+				glScalef(1.0f / ups->pressure_value, 1.0f / ups->pressure_value, 1);
 				glTranslatef(-0.5f, -0.5f, 0);
 			}
 
-			if (sd->draw_anchored) {
-				const float *aim = sd->anchored_initial_mouse;
+			if (ups->draw_anchored) {
+				const float *aim = ups->anchored_initial_mouse;
 				const rcti *win = &vc->ar->winrct;
-				quad.xmin = aim[0] - sd->anchored_size - win->xmin;
-				quad.ymin = aim[1] - sd->anchored_size - win->ymin;
-				quad.xmax = aim[0] + sd->anchored_size - win->xmin;
-				quad.ymax = aim[1] + sd->anchored_size - win->ymin;
+				quad.xmin = aim[0] - ups->anchored_size - win->xmin;
+				quad.ymin = aim[1] - ups->anchored_size - win->ymin;
+				quad.xmax = aim[0] + ups->anchored_size - win->xmin;
+				quad.ymax = aim[1] + ups->anchored_size - win->ymin;
 			}
 			else {
-				const int radius = brush_size(vc->scene, brush);
+				const int radius = BKE_brush_size_get(vc->scene, brush);
 				quad.xmin = x - radius;
 				quad.ymin = y - radius;
 				quad.xmax = x + radius;
@@ -437,8 +451,8 @@ static void paint_draw_alpha_overlay(Sculpt *sd, Brush *brush,
 		else {
 			quad.xmin = 0;
 			quad.ymin = 0;
-			quad.xmax = vc->ar->winrct.xmax - vc->ar->winrct.xmin;
-			quad.ymax = vc->ar->winrct.ymax - vc->ar->winrct.ymin;
+			quad.xmax = BLI_rcti_size_x(&vc->ar->winrct);
+			quad.ymax = BLI_rcti_size_y(&vc->ar->winrct);
 		}
 
 		/* set quad color */
@@ -467,21 +481,21 @@ static void paint_draw_alpha_overlay(Sculpt *sd, Brush *brush,
 
 /* Special actions taken when paint cursor goes over mesh */
 /* TODO: sculpt only for now */
-static void paint_cursor_on_hit(Sculpt *sd, Brush *brush, ViewContext *vc,
+static void paint_cursor_on_hit(UnifiedPaintSettings *ups, Brush *brush, ViewContext *vc,
                                 const float location[3])
 {
 	float unprojected_radius, projected_radius;
 
 	/* update the brush's cached 3D radius */
-	if (!brush_use_locked_size(vc->scene, brush)) {
+	if (!BKE_brush_use_locked_size(vc->scene, brush)) {
 		/* get 2D brush radius */
-		if (sd->draw_anchored)
-			projected_radius = sd->anchored_size;
+		if (ups->draw_anchored)
+			projected_radius = ups->anchored_size;
 		else {
 			if (brush->flag & BRUSH_ANCHORED)
 				projected_radius = 8;
 			else
-				projected_radius = brush_size(vc->scene, brush);
+				projected_radius = BKE_brush_size_get(vc->scene, brush);
 		}
 	
 		/* convert brush radius from 2D to 3D */
@@ -489,18 +503,19 @@ static void paint_cursor_on_hit(Sculpt *sd, Brush *brush, ViewContext *vc,
 		                                                    projected_radius);
 
 		/* scale 3D brush radius by pressure */
-		if (sd->draw_pressure && brush_use_size_pressure(vc->scene, brush))
-			unprojected_radius *= sd->pressure_value;
+		if (ups->draw_pressure && BKE_brush_use_size_pressure(vc->scene, brush))
+			unprojected_radius *= ups->pressure_value;
 
 		/* set cached value in either Brush or UnifiedPaintSettings */
-		brush_set_unprojected_radius(vc->scene, brush, unprojected_radius);
+		BKE_brush_unprojected_radius_set(vc->scene, brush, unprojected_radius);
 	}
 }
 
 static void paint_draw_cursor(bContext *C, int x, int y, void *UNUSED(unused))
 {
 	Scene *scene = CTX_data_scene(C);
-	Paint *paint = paint_get_active(scene);
+	UnifiedPaintSettings *ups = &scene->toolsettings->unified_paint_settings;
+	Paint *paint = paint_get_active_from_context(C);
 	Brush *brush = paint_brush(paint);
 	ViewContext vc;
 	float final_radius;
@@ -512,7 +527,7 @@ static void paint_draw_cursor(bContext *C, int x, int y, void *UNUSED(unused))
 	translation[1] = y;
 	outline_alpha = 0.5;
 	outline_col = brush->add_col;
-	final_radius = brush_size(scene, brush);
+	final_radius = BKE_brush_size_get(scene, brush);
 
 	/* check that brush drawing is enabled */
 	if (!(paint->flags & PAINT_SHOW_BRUSH))
@@ -525,7 +540,6 @@ static void paint_draw_cursor(bContext *C, int x, int y, void *UNUSED(unused))
 	/* TODO: as sculpt and other paint modes are unified, this
 	 * special mode of drawing will go away */
 	if (vc.obact->sculpt) {
-		Sculpt *sd = CTX_data_tool_settings(C)->sculpt;
 		float location[3];
 		int pixel_radius, hit;
 
@@ -538,14 +552,14 @@ static void paint_draw_cursor(bContext *C, int x, int y, void *UNUSED(unused))
 			const float v = 1 - u;
 			const float r = 20;
 
-			const float dx = sd->last_x - x;
-			const float dy = sd->last_y - y;
+			const float dx = ups->last_x - x;
+			const float dy = ups->last_y - y;
 
 			if (dx * dx + dy * dy >= r * r) {
-				sd->last_angle = atan2(dx, dy);
+				ups->last_angle = atan2(dx, dy);
 
-				sd->last_x = u * sd->last_x + v * x;
-				sd->last_y = u * sd->last_y + v * y;
+				ups->last_x = u * ups->last_x + v * x;
+				ups->last_y = u * ups->last_y + v * y;
 			}
 		}
 
@@ -553,10 +567,10 @@ static void paint_draw_cursor(bContext *C, int x, int y, void *UNUSED(unused))
 		hit = sculpt_get_brush_geometry(C, &vc, x, y, &pixel_radius, location);
 
 		/* draw overlay */
-		paint_draw_alpha_overlay(sd, brush, &vc, x, y);
+		paint_draw_alpha_overlay(ups, brush, &vc, x, y);
 
-		if (brush_use_locked_size(scene, brush))
-			brush_set_size(scene, brush, pixel_radius);
+		if (BKE_brush_use_locked_size(scene, brush))
+			BKE_brush_size_set(scene, brush, pixel_radius);
 
 		/* check if brush is subtracting, use different color then */
 		/* TODO: no way currently to know state of pen flip or
@@ -566,16 +580,18 @@ static void paint_draw_cursor(bContext *C, int x, int y, void *UNUSED(unused))
 		    ELEM5(brush->sculpt_tool, SCULPT_TOOL_DRAW,
 		          SCULPT_TOOL_INFLATE, SCULPT_TOOL_CLAY,
 		          SCULPT_TOOL_PINCH, SCULPT_TOOL_CREASE))
+		{
 			outline_col = brush->sub_col;
+		}
 
 		/* only do if brush is over the mesh */
 		if (hit)
-			paint_cursor_on_hit(sd, brush, &vc, location);
+			paint_cursor_on_hit(ups, brush, &vc, location);
 
-		if (sd->draw_anchored) {
-			final_radius = sd->anchored_size;
-			translation[0] = sd->anchored_initial_mouse[0] - vc.ar->winrct.xmin;
-			translation[1] = sd->anchored_initial_mouse[1] - vc.ar->winrct.ymin;
+		if (ups->draw_anchored) {
+			final_radius = ups->anchored_size;
+			translation[0] = ups->anchored_initial_mouse[0] - vc.ar->winrct.xmin;
+			translation[1] = ups->anchored_initial_mouse[1] - vc.ar->winrct.ymin;
 		}
 	}
 
@@ -588,6 +604,14 @@ static void paint_draw_cursor(bContext *C, int x, int y, void *UNUSED(unused))
 
 	/* draw brush outline */
 	glTranslatef(translation[0], translation[1], 0);
+
+	/* draw an inner brush */
+	if (ups->draw_pressure && BKE_brush_use_size_pressure(scene, brush)) {
+		/* inner at full alpha */
+		glutil_draw_lined_arc(0.0, M_PI * 2.0, final_radius * ups->pressure_value, 40);
+		/* outer at half alpha */
+		glColor4f(outline_col[0], outline_col[1], outline_col[2], outline_alpha * 0.5f);
+	}
 	glutil_draw_lined_arc(0.0, M_PI * 2.0, final_radius, 40);
 	glTranslatef(-translation[0], -translation[1], 0);
 
@@ -600,7 +624,7 @@ static void paint_draw_cursor(bContext *C, int x, int y, void *UNUSED(unused))
 
 void paint_cursor_start(bContext *C, int (*poll)(bContext *C))
 {
-	Paint *p = paint_get_active(CTX_data_scene(C));
+	Paint *p = paint_get_active_from_context(C);
 
 	if (p && !p->paint_cursor)
 		p->paint_cursor = WM_paint_cursor_activate(CTX_wm_manager(C), poll, paint_draw_cursor, NULL);
