@@ -25,6 +25,7 @@
 #include "MEM_guardedalloc.h"
 
 #include <stdlib.h>
+#include "BLI_listbase.h"
 #include "BLI_string.h"
 #include "BLI_utildefines.h"
 
@@ -59,15 +60,16 @@
 static int brush_add_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	/*int type = RNA_enum_get(op->ptr, "type");*/
-	Paint *paint = paint_get_active(CTX_data_scene(C));
+	Paint *paint = paint_get_active_from_context(C);
 	struct Brush *br = paint_brush(paint);
+	Main *bmain = CTX_data_main(C);
 
 	if (br)
-		br = copy_brush(br);
+		br = BKE_brush_copy(br);
 	else
-		br = add_brush("Brush");
+		br = BKE_brush_add(bmain, "Brush");
 
-	paint_brush_set(paint_get_active(CTX_data_scene(C)), br);
+	paint_brush_set(paint, br);
 
 	return OPERATOR_FINISHED;
 }
@@ -90,15 +92,15 @@ static void BRUSH_OT_add(wmOperatorType *ot)
 static int brush_scale_size_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
-	Paint  *paint =  paint_get_active(scene);
+	Paint  *paint =  paint_get_active_from_context(C);
 	struct Brush  *brush =  paint_brush(paint);
-	// Object *ob=     CTX_data_active_object(C);
+	// Object *ob = CTX_data_active_object(C);
 	float scalar = RNA_float_get(op->ptr, "scalar");
 
 	if (brush) {
 		// pixel radius
 		{
-			const int old_size = brush_size(scene, brush);
+			const int old_size = BKE_brush_size_get(scene, brush);
 			int size = (int)(scalar * old_size);
 
 			if (old_size == size) {
@@ -111,17 +113,17 @@ static int brush_scale_size_exec(bContext *C, wmOperator *op)
 			}
 			CLAMP(size, 1, 2000); // XXX magic number
 
-			brush_set_size(scene, brush, size);
+			BKE_brush_size_set(scene, brush, size);
 		}
 
 		// unprojected radius
 		{
-			float unprojected_radius = scalar * brush_unprojected_radius(scene, brush);
+			float unprojected_radius = scalar * BKE_brush_unprojected_radius_get(scene, brush);
 
 			if (unprojected_radius < 0.001f) // XXX magic number
 				unprojected_radius = 0.001f;
 
-			brush_set_unprojected_radius(scene, brush, unprojected_radius);
+			BKE_brush_unprojected_radius_set(scene, brush, unprojected_radius);
 		}
 	}
 
@@ -160,6 +162,7 @@ static void PAINT_OT_vertex_color_set(wmOperatorType *ot)
 	/* identifiers */
 	ot->name = "Set Vertex Colors";
 	ot->idname = "PAINT_OT_vertex_color_set";
+	ot->description = "Fill the active vertex color layer with the current paint color";
 	
 	/* api callbacks */
 	ot->exec = vertex_color_set_exec;
@@ -171,14 +174,14 @@ static void PAINT_OT_vertex_color_set(wmOperatorType *ot)
 
 static int brush_reset_exec(bContext *C, wmOperator *UNUSED(op))
 {
-	Paint *paint = paint_get_active(CTX_data_scene(C));
+	Paint *paint = paint_get_active_from_context(C);
 	struct Brush *brush = paint_brush(paint);
 	Object *ob = CTX_data_active_object(C);
 
 	if (!ob) return OPERATOR_CANCELLED;
 
 	if (ob->mode & OB_MODE_SCULPT)
-		brush_reset_sculpt(brush);
+		BKE_brush_sculpt_reset(brush);
 	/* TODO: other modes */
 
 	return OPERATOR_FINISHED;
@@ -203,6 +206,11 @@ static int brush_tool(const Brush *brush, size_t tool_offset)
 	return *(((char *)brush) + tool_offset);
 }
 
+static void brush_tool_set(const Brush *brush, size_t tool_offset, int tool)
+{
+	*(((char *)brush) + tool_offset) = tool;
+}
+
 /* generic functions for setting the active brush based on the tool */
 static Brush *brush_tool_cycle(Main *bmain, Brush *brush_orig, const int tool, const size_t tool_offset, const int ob_mode)
 {
@@ -218,7 +226,8 @@ static Brush *brush_tool_cycle(Main *bmain, Brush *brush_orig, const int tool, c
 	     brush = brush->id.next ? brush->id.next : bmain->brush.first)
 	{
 		if ((brush->ob_mode & ob_mode) &&
-		    (brush_tool(brush, tool_offset) == tool)) {
+		    (brush_tool(brush, tool_offset) == tool))
+		{
 			return brush;
 		}
 	}
@@ -226,11 +235,49 @@ static Brush *brush_tool_cycle(Main *bmain, Brush *brush_orig, const int tool, c
 	return NULL;
 }
 
-static int brush_generic_tool_set(Main *bmain, Paint *paint, const int tool, const size_t tool_offset, const int ob_mode)
+static Brush *brush_tool_toggle(Main *bmain, Brush *brush_orig, const int tool, const size_t tool_offset, const int ob_mode)
+{
+	if (!brush_orig || brush_tool(brush_orig, tool_offset) != tool) {
+		Brush *br;
+		/* if the current brush is not using the desired tool, look
+		 * for one that is */
+		br = brush_tool_cycle(bmain, brush_orig, tool, tool_offset, ob_mode);
+		/* store the previously-selected brush */
+		if (br)
+			br->toggle_brush = brush_orig;
+		
+		return br;
+	}
+	else if (brush_orig->toggle_brush &&
+	         BLI_findindex(bmain->brush.first, brush_orig->toggle_brush) != -1)
+	{
+		/* if current brush is using the desired tool, try to toggle
+		 * back to the previously selected brush (if it was set, and
+		 * if it still exists) */
+		return brush_orig->toggle_brush;
+	}
+	else
+		return NULL;
+}
+
+static int brush_generic_tool_set(Main *bmain, Paint *paint, const int tool,
+                                  const size_t tool_offset, const int ob_mode,
+                                  const char *tool_name, int create_missing,
+                                  int toggle)
 {
 	struct Brush *brush, *brush_orig = paint_brush(paint);
 
-	brush = brush_tool_cycle(bmain, brush_orig, tool, tool_offset, ob_mode);
+	if (toggle)
+		brush = brush_tool_toggle(bmain, brush_orig, tool, tool_offset, ob_mode);
+	else
+		brush = brush_tool_cycle(bmain, brush_orig, tool, tool_offset, ob_mode);
+
+	if (!brush && brush_tool(brush_orig, tool_offset) != tool && create_missing) {
+		brush = BKE_brush_add(bmain, tool_name);
+		brush_tool_set(brush, tool_offset, tool);
+		brush->ob_mode = ob_mode;
+		brush->toggle_brush = brush_orig;
+	}
 
 	if (brush) {
 		paint_brush_set(paint, brush);
@@ -251,6 +298,9 @@ static int brush_select_exec(bContext *C, wmOperator *op)
 	ToolSettings *toolsettings = CTX_data_tool_settings(C);
 	Paint *paint = NULL;
 	int tool, paint_mode = RNA_enum_get(op->ptr, "paint_mode");
+	int create_missing = RNA_boolean_get(op->ptr, "create_missing");
+	int toggle = RNA_boolean_get(op->ptr, "toggle");
+	const char *tool_name = "Brush";
 	size_t tool_offset;
 
 	if (paint_mode == OB_MODE_ACTIVE) {
@@ -273,29 +323,35 @@ static int brush_select_exec(bContext *C, wmOperator *op)
 			paint = &toolsettings->sculpt->paint;
 			tool_offset = offsetof(Brush, sculpt_tool);
 			tool = RNA_enum_get(op->ptr, "sculpt_tool");
+			RNA_enum_name_from_value(brush_sculpt_tool_items, tool, &tool_name);
 			break;
 		case OB_MODE_VERTEX_PAINT:
 			paint = &toolsettings->vpaint->paint;
 			tool_offset = offsetof(Brush, vertexpaint_tool);
 			tool = RNA_enum_get(op->ptr, "vertex_paint_tool");
+			RNA_enum_name_from_value(brush_vertex_tool_items, tool, &tool_name);
 			break;
 		case OB_MODE_WEIGHT_PAINT:
 			paint = &toolsettings->wpaint->paint;
 			/* vertexpaint_tool is used for weight paint mode */
 			tool_offset = offsetof(Brush, vertexpaint_tool);
 			tool = RNA_enum_get(op->ptr, "weight_paint_tool");
+			RNA_enum_name_from_value(brush_vertex_tool_items, tool, &tool_name);
 			break;
 		case OB_MODE_TEXTURE_PAINT:
 			paint = &toolsettings->imapaint.paint;
 			tool_offset = offsetof(Brush, imagepaint_tool);
 			tool = RNA_enum_get(op->ptr, "texture_paint_tool");
+			RNA_enum_name_from_value(brush_image_tool_items, tool, &tool_name);
 			break;
 		default:
 			/* invalid paint mode */
 			return OPERATOR_CANCELLED;
 	}
 
-	return brush_generic_tool_set(bmain, paint, tool, tool_offset, paint_mode);
+	return brush_generic_tool_set(bmain, paint, tool, tool_offset,
+	                              paint_mode, tool_name, create_missing,
+	                              toggle);
 }
 
 static void PAINT_OT_brush_select(wmOperatorType *ot)
@@ -326,6 +382,9 @@ static void PAINT_OT_brush_select(wmOperatorType *ot)
 	RNA_def_enum(ot->srna, "vertex_paint_tool", brush_vertex_tool_items, 0, "Vertex Paint Tool", "");
 	RNA_def_enum(ot->srna, "weight_paint_tool", brush_vertex_tool_items, 0, "Weight Paint Tool", "");
 	RNA_def_enum(ot->srna, "texture_paint_tool", brush_image_tool_items, 0, "Texture Paint Tool", "");
+
+	RNA_def_boolean(ot->srna, "toggle", 0, "Toggle", "Toggle between two brushes rather than cycling");
+	RNA_def_boolean(ot->srna, "create_missing", 0, "Create Missing", "If the requested brush type does not exist, create a new brush");
 }
 
 static wmKeyMapItem *keymap_brush_select(wmKeyMap *keymap, int paint_mode,
@@ -417,6 +476,7 @@ void ED_operatortypes_paint(void)
 	WM_operatortype_append(PAINT_OT_weight_paint);
 	WM_operatortype_append(PAINT_OT_weight_set);
 	WM_operatortype_append(PAINT_OT_weight_from_bones);
+	WM_operatortype_append(PAINT_OT_weight_gradient);
 	WM_operatortype_append(PAINT_OT_weight_sample);
 	WM_operatortype_append(PAINT_OT_weight_sample_group);
 
@@ -425,7 +485,6 @@ void ED_operatortypes_paint(void)
 
 	/* vertex selection */
 	WM_operatortype_append(PAINT_OT_vert_select_all);
-	WM_operatortype_append(PAINT_OT_vert_select_inverse);
 
 	/* vertex */
 	WM_operatortype_append(PAINT_OT_vertex_paint_toggle);
@@ -436,12 +495,14 @@ void ED_operatortypes_paint(void)
 	WM_operatortype_append(PAINT_OT_face_select_linked);
 	WM_operatortype_append(PAINT_OT_face_select_linked_pick);
 	WM_operatortype_append(PAINT_OT_face_select_all);
-	WM_operatortype_append(PAINT_OT_face_select_inverse);
 	WM_operatortype_append(PAINT_OT_face_select_hide);
 	WM_operatortype_append(PAINT_OT_face_select_reveal);
 
 	/* partial visibility */
 	WM_operatortype_append(PAINT_OT_hide_show);
+
+	/* paint masking */
+	WM_operatortype_append(PAINT_OT_mask_flood_fill);
 }
 
 
@@ -523,24 +584,29 @@ static void ed_keymap_paint_brush_radial_control(wmKeyMap *keymap, const char *p
                                                  RCFlags flags)
 {
 	wmKeyMapItem *kmi;
+	/* only size needs to follow zoom, strength shows fixed size circle */
+	int flags_nozoom = flags & (~RC_ZOOM);
 
 	kmi = WM_keymap_add_item(keymap, "WM_OT_radial_control", FKEY, KM_PRESS, 0, 0);
 	set_brush_rc_props(kmi->ptr, paint, "size", "use_unified_size", flags);
 
 	kmi = WM_keymap_add_item(keymap, "WM_OT_radial_control", FKEY, KM_PRESS, KM_SHIFT, 0);
-	set_brush_rc_props(kmi->ptr, paint, "strength", "use_unified_strength", flags);
+	set_brush_rc_props(kmi->ptr, paint, "strength", "use_unified_strength", flags_nozoom);
+
+	kmi = WM_keymap_add_item(keymap, "WM_OT_radial_control", WKEY, KM_PRESS, 0, 0);
+	set_brush_rc_props(kmi->ptr, paint, "weight", "use_unified_weight", flags_nozoom);
 
 	if (flags & RC_ROTATION) {
 		kmi = WM_keymap_add_item(keymap, "WM_OT_radial_control", FKEY, KM_PRESS, KM_CTRL, 0);
-		set_brush_rc_props(kmi->ptr, paint, "texture_slot.angle", NULL, flags);
+		set_brush_rc_props(kmi->ptr, paint, "texture_slot.angle", NULL, flags_nozoom);
 	}
 }
 
-void paint_partial_visibility_keys(wmKeyMap *keymap)
+static void paint_partial_visibility_keys(wmKeyMap *keymap)
 {
 	wmKeyMapItem *kmi;
 	
-	/* Partial visiblity */
+	/* Partial visibility */
 	kmi = WM_keymap_add_item(keymap, "PAINT_OT_hide_show", HKEY, KM_PRESS, KM_SHIFT, 0);
 	RNA_enum_set(kmi->ptr, "action", PARTIALVIS_SHOW);
 	RNA_enum_set(kmi->ptr, "area", PARTIALVIS_INSIDE);
@@ -560,7 +626,7 @@ void ED_keymap_paint(wmKeyConfig *keyconf)
 	
 	/* Sculpt mode */
 	keymap = WM_keymap_find(keyconf, "Sculpt", 0, 0);
-	keymap->poll = sculpt_poll;
+	keymap->poll = sculpt_mode_poll;
 
 	RNA_enum_set(WM_keymap_add_item(keymap, "SCULPT_OT_brush_stroke", LEFTMOUSE, KM_PRESS, 0,        0)->ptr, "mode", BRUSH_STROKE_NORMAL);
 	RNA_enum_set(WM_keymap_add_item(keymap, "SCULPT_OT_brush_stroke", LEFTMOUSE, KM_PRESS, KM_CTRL,  0)->ptr, "mode", BRUSH_STROKE_INVERT);
@@ -572,6 +638,26 @@ void ED_keymap_paint(wmKeyConfig *keyconf)
 	for (i = 0; i <= 5; i++)
 		RNA_int_set(WM_keymap_add_item(keymap, "OBJECT_OT_subdivision_set", ZEROKEY + i, KM_PRESS, KM_CTRL, 0)->ptr, "level", i);
 
+	/* Clear mask */
+	kmi = WM_keymap_add_item(keymap, "PAINT_OT_mask_flood_fill", MKEY, KM_PRESS, KM_ALT, 0);
+	RNA_enum_set(kmi->ptr, "mode", PAINT_MASK_FLOOD_VALUE);
+	RNA_float_set(kmi->ptr, "value", 0);
+
+	/* Invert mask */
+	kmi = WM_keymap_add_item(keymap, "PAINT_OT_mask_flood_fill", IKEY, KM_PRESS, KM_CTRL, 0);
+	RNA_enum_set(kmi->ptr, "mode", PAINT_MASK_INVERT);
+
+	/* Toggle dynamic topology */
+	WM_keymap_add_item(keymap, "SCULPT_OT_dynamic_topology_toggle", DKEY, KM_PRESS, KM_CTRL, 0);
+
+	/* Dynamic-topology detail size
+	 * 
+	 * This should be improved further, perhaps by showing a triangle
+	 * grid rather than brush alpha */
+	kmi = WM_keymap_add_item(keymap, "WM_OT_radial_control", DKEY, KM_PRESS, KM_SHIFT, 0);
+	set_brush_rc_props(kmi->ptr, "sculpt", "detail_size", NULL, 0);
+	RNA_string_set(kmi->ptr, "data_path_primary", "tool_settings.sculpt.detail_size");
+
 	/* multires switch */
 	kmi = WM_keymap_add_item(keymap, "OBJECT_OT_subdivision_set", PAGEUPKEY, KM_PRESS, 0, 0);
 	RNA_int_set(kmi->ptr, "level", 1);
@@ -594,6 +680,10 @@ void ED_keymap_paint(wmKeyConfig *keyconf)
 	keymap_brush_select(keymap, OB_MODE_SCULPT, SCULPT_TOOL_FLATTEN, TKEY, KM_SHIFT);
 	keymap_brush_select(keymap, OB_MODE_SCULPT, SCULPT_TOOL_CLAY, CKEY, 0);
 	keymap_brush_select(keymap, OB_MODE_SCULPT, SCULPT_TOOL_CREASE, CKEY, KM_SHIFT);
+	keymap_brush_select(keymap, OB_MODE_SCULPT, SCULPT_TOOL_SNAKE_HOOK, KKEY, 0);
+	kmi = keymap_brush_select(keymap, OB_MODE_SCULPT, SCULPT_TOOL_MASK, MKEY, 0);
+	RNA_boolean_set(kmi->ptr, "toggle", 1);
+	RNA_boolean_set(kmi->ptr, "create_missing", 1);
 
 	/* */
 	kmi = WM_keymap_add_item(keymap, "WM_OT_context_menu_enum", AKEY, KM_PRESS, 0, 0);
@@ -629,8 +719,11 @@ void ED_keymap_paint(wmKeyConfig *keyconf)
 	WM_keymap_verify_item(keymap, "PAINT_OT_weight_paint", LEFTMOUSE, KM_PRESS, 0, 0);
 
 	/* these keys are from 2.4x but could be changed */
-	WM_keymap_verify_item(keymap, "PAINT_OT_weight_sample", LEFTMOUSE, KM_PRESS, KM_CTRL, 0);
-	WM_keymap_verify_item(keymap, "PAINT_OT_weight_sample_group", LEFTMOUSE, KM_PRESS, KM_SHIFT, 0);
+	WM_keymap_verify_item(keymap, "PAINT_OT_weight_sample", ACTIONMOUSE, KM_PRESS, KM_CTRL, 0);
+	WM_keymap_verify_item(keymap, "PAINT_OT_weight_sample_group", ACTIONMOUSE, KM_PRESS, KM_SHIFT, 0);
+
+	RNA_enum_set(WM_keymap_add_item(keymap, "PAINT_OT_weight_gradient", LEFTMOUSE, KM_PRESS, KM_ALT, 0)->ptr,           "type", WPAINT_GRADIENT_TYPE_LINEAR);
+	RNA_enum_set(WM_keymap_add_item(keymap, "PAINT_OT_weight_gradient", LEFTMOUSE, KM_PRESS, KM_ALT | KM_CTRL, 0)->ptr, "type", WPAINT_GRADIENT_TYPE_RADIAL);
 
 	WM_keymap_add_item(keymap,
 	                   "PAINT_OT_weight_set", KKEY, KM_PRESS, KM_SHIFT, 0);
@@ -653,7 +746,8 @@ void ED_keymap_paint(wmKeyConfig *keyconf)
 	keymap = WM_keymap_find(keyconf, "Weight Paint Vertex Selection", 0, 0);
 	keymap->poll = vert_paint_poll;
 	WM_keymap_add_item(keymap, "PAINT_OT_vert_select_all", AKEY, KM_PRESS, 0, 0);
-	WM_keymap_add_item(keymap, "PAINT_OT_vert_select_inverse", IKEY, KM_PRESS, KM_CTRL, 0);
+	kmi = WM_keymap_add_item(keymap, "PAINT_OT_vert_select_all", IKEY, KM_PRESS, KM_CTRL, 0);
+	RNA_enum_set(kmi->ptr, "action", SEL_INVERT);
 	WM_keymap_add_item(keymap, "VIEW3D_OT_select_border", BKEY, KM_PRESS, 0, 0);
 	kmi = WM_keymap_add_item(keymap, "VIEW3D_OT_select_lasso", EVT_TWEAK_A, KM_ANY, KM_CTRL, 0);
 	RNA_boolean_set(kmi->ptr, "deselect", FALSE);
@@ -682,7 +776,8 @@ void ED_keymap_paint(wmKeyConfig *keyconf)
 	keymap->poll = facemask_paint_poll;
 
 	WM_keymap_add_item(keymap, "PAINT_OT_face_select_all", AKEY, KM_PRESS, 0, 0);
-	WM_keymap_add_item(keymap, "PAINT_OT_face_select_inverse", IKEY, KM_PRESS, KM_CTRL, 0);
+	kmi = WM_keymap_add_item(keymap, "PAINT_OT_face_select_all", IKEY, KM_PRESS, KM_CTRL, 0);
+	RNA_enum_set(kmi->ptr, "action", SEL_INVERT);
 	kmi = WM_keymap_add_item(keymap, "PAINT_OT_face_select_hide", HKEY, KM_PRESS, 0, 0);
 	RNA_boolean_set(kmi->ptr, "unselected", FALSE);
 	kmi = WM_keymap_add_item(keymap, "PAINT_OT_face_select_hide", HKEY, KM_PRESS, KM_SHIFT, 0);
@@ -709,4 +804,7 @@ void ED_keymap_paint(wmKeyConfig *keyconf)
 	RNA_enum_set(WM_keymap_add_item(keymap, "BRUSH_OT_uv_sculpt_tool_set", PKEY, KM_PRESS, 0, 0)->ptr, "tool", UV_SCULPT_TOOL_PINCH);
 	RNA_enum_set(WM_keymap_add_item(keymap, "BRUSH_OT_uv_sculpt_tool_set", GKEY, KM_PRESS, 0, 0)->ptr, "tool", UV_SCULPT_TOOL_GRAB);
 
+	/* paint stroke */
+	keymap = paint_stroke_modal_keymap(keyconf);
+	WM_modalkeymap_assign(keymap, "SCULPT_OT_brush_stroke");
 }
