@@ -1,7 +1,4 @@
 /*
- *
- * Undo system for painting and sculpting.
- * 
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -23,9 +20,9 @@
 
 /** \file blender/editors/sculpt_paint/paint_undo.c
  *  \ingroup edsculpt
+ *  \brief Undo system for painting and sculpting.
  */
 
-
 #include <stdlib.h>
 #include <string.h>
 
@@ -37,7 +34,7 @@
 
 #include "DNA_userdef_types.h"
 
-
+#include "BKE_blender.h"
 #include "BKE_context.h"
 #include "BKE_global.h"
 
@@ -45,11 +42,9 @@
 
 #include "paint_intern.h"
 
-#define MAXUNDONAME 64
-
 typedef struct UndoElem {
 	struct UndoElem *next, *prev;
-	char name[MAXUNDONAME];
+	char name[BKE_UNDO_STR_MAX];
 	uintptr_t undosize;
 
 	ListBase elems;
@@ -157,7 +152,9 @@ static int undo_stack_step(bContext *C, UndoStack *stack, int step, const char *
 	UndoElem *undo;
 
 	if (step == 1) {
-		if (stack->current == NULL) ;
+		if (stack->current == NULL) {
+			/* pass */
+		}
 		else {
 			if (!name || strcmp(stack->current->name, name) == 0) {
 				if (G.debug & G_DEBUG_WM) {
@@ -170,7 +167,9 @@ static int undo_stack_step(bContext *C, UndoStack *stack, int step, const char *
 		}
 	}
 	else if (step == -1) {
-		if ((stack->current != NULL && stack->current->next == NULL) || stack->elems.first == NULL) ;
+		if ((stack->current != NULL && stack->current->next == NULL) || stack->elems.first == NULL) {
+			/* pass */
+		}
 		else {
 			if (!name || strcmp(stack->current->name, name) == 0) {
 				undo = (stack->current && stack->current->next) ? stack->current->next : stack->elems.first;
@@ -259,7 +258,9 @@ int ED_undo_paint_valid(int type, const char *name)
 	else 
 		return 0;
 	
-	if (stack->current == NULL) ;
+	if (stack->current == NULL) {
+		/* pass */
+	}
 	else {
 		if (name && strcmp(stack->current->name, name) == 0)
 			return 1;
@@ -274,4 +275,3 @@ void ED_undo_paint_free(void)
 	undo_stack_free(&ImageUndoStack);
 	undo_stack_free(&MeshUndoStack);
 }
-
