@@ -32,6 +32,7 @@
  *  \ingroup edsculpt
  */
 
+#include <stddef.h>
 
 #include "MEM_guardedalloc.h"
 
@@ -47,6 +48,7 @@
 #include "DNA_scene_types.h"
 #include "DNA_mesh_types.h"
 
+#include "BKE_ccg.h"
 #include "BKE_cdderivedmesh.h"
 #include "BKE_context.h"
 #include "BKE_depsgraph.h"
@@ -63,6 +65,7 @@
 #include "GPU_buffers.h"
 
 #include "ED_sculpt.h"
+#include "bmesh.h"
 #include "paint_intern.h"
 #include "sculpt_intern.h"
 
@@ -70,13 +73,16 @@
 
 static void update_cb(PBVHNode *node, void *rebuild)
 {
-	BLI_pbvh_node_mark_update(node);
+	BKE_pbvh_node_mark_update(node);
 	if (*((int *)rebuild))
-		BLI_pbvh_node_mark_rebuild_draw(node);
-	BLI_pbvh_node_fully_hidden_set(node, 0);
+		BKE_pbvh_node_mark_rebuild_draw(node);
+	BKE_pbvh_node_fully_hidden_set(node, 0);
 }
 
-static void sculpt_undo_restore_deformed(SculptSession *ss, SculptUndoNode *unode, int uindex, int oindex, float coord[3])
+static void sculpt_undo_restore_deformed(const SculptSession *ss,
+                                         SculptUndoNode *unode,
+                                         int uindex, int oindex,
+                                         float coord[3])
 {
 	if (unode->orig_co) {
 		swap_v3_v3(coord, unode->orig_co[uindex]);
@@ -94,7 +100,7 @@ static int sculpt_undo_restore_coords(bContext *C, DerivedMesh *dm, SculptUndoNo
 	Object *ob = CTX_data_active_object(C);
 	SculptSession *ss = ob->sculpt;
 	MVert *mvert;
-	int *index, i, j;	
+	int *index, i, j;
 	
 	if (unode->maxvert) {
 		/* regular mesh restore */
@@ -102,13 +108,13 @@ static int sculpt_undo_restore_coords(bContext *C, DerivedMesh *dm, SculptUndoNo
 		if (ss->kb && strcmp(ss->kb->name, unode->shapeName)) {
 			/* shape key has been changed before calling undo operator */
 
-			Key *key = ob_get_key(ob);
-			KeyBlock *kb = key_get_named_keyblock(key, unode->shapeName);
+			Key *key = BKE_key_from_object(ob);
+			KeyBlock *kb = key ? BKE_keyblock_find_name(key, unode->shapeName) : NULL;
 
 			if (kb) {
 				ob->shapenr = BLI_findindex(&key->block, kb) + 1;
 
-				sculpt_update_mesh_elements(scene, sd, ob, 0);
+				sculpt_update_mesh_elements(scene, sd, ob, 0, FALSE);
 				WM_event_add_notifier(C, NC_OBJECT | ND_DATA, ob);
 			}
 			else {
@@ -122,7 +128,7 @@ static int sculpt_undo_restore_coords(bContext *C, DerivedMesh *dm, SculptUndoNo
 
 		if (ss->kb) {
 			float (*vertCos)[3];
-			vertCos = key_to_vertcos(ob, ss->kb);
+			vertCos = BKE_key_convert_to_vertcos(ob, ss->kb);
 
 			for (i = 0; i < unode->totvert; i++) {
 				if (ss->modifiers_active) sculpt_undo_restore_deformed(ss, unode, i, index[i], vertCos[index[i]]);
@@ -137,7 +143,7 @@ static int sculpt_undo_restore_coords(bContext *C, DerivedMesh *dm, SculptUndoNo
 
 			/* pbvh uses it's own mvert array, so coords should be */
 			/* propagated to pbvh here */
-			BLI_pbvh_apply_vertCos(ss->pbvh, vertCos);
+			BKE_pbvh_apply_vertCos(ss->pbvh, vertCos);
 
 			MEM_freeN(vertCos);
 		}
@@ -154,19 +160,21 @@ static int sculpt_undo_restore_coords(bContext *C, DerivedMesh *dm, SculptUndoNo
 	}
 	else if (unode->maxgrid && dm->getGridData) {
 		/* multires restore */
-		DMGridData **grids, *grid;
+		CCGElem **grids, *grid;
+		CCGKey key;
 		float (*co)[3];
 		int gridsize;
 
 		grids = dm->getGridData(dm);
 		gridsize = dm->getGridSize(dm);
+		dm->getGridKey(dm, &key);
 
 		co = unode->co;
 		for (j = 0; j < unode->totgrid; j++) {
 			grid = grids[unode->grids[j]];
 
 			for (i = 0; i < gridsize * gridsize; i++, co++)
-				swap_v3_v3(grid[i].co, co[0]);
+				swap_v3_v3(CCG_elem_offset_co(&key, grid, i), co[0]);
 		}
 	}
 
@@ -211,18 +219,184 @@ static int sculpt_undo_restore_hidden(bContext *C, DerivedMesh *dm,
 	return 1;
 }
 
+static int sculpt_undo_restore_mask(bContext *C, DerivedMesh *dm, SculptUndoNode *unode)
+{
+	Object *ob = CTX_data_active_object(C);
+	SculptSession *ss = ob->sculpt;
+	MVert *mvert;
+	float *vmask;
+	int *index, i, j;
+	
+	if (unode->maxvert) {
+		/* regular mesh restore */
+
+		index = unode->index;
+		mvert = ss->mvert;
+		vmask = ss->vmask;
+
+		for (i = 0; i < unode->totvert; i++) {
+			SWAP(float, vmask[index[i]], unode->mask[i]);
+			mvert[index[i]].flag |= ME_VERT_PBVH_UPDATE;
+		}
+	}
+	else if (unode->maxgrid && dm->getGridData) {
+		/* multires restore */
+		CCGElem **grids, *grid;
+		CCGKey key;
+		float *mask;
+		int gridsize;
+
+		grids = dm->getGridData(dm);
+		gridsize = dm->getGridSize(dm);
+		dm->getGridKey(dm, &key);
+
+		mask = unode->mask;
+		for (j = 0; j < unode->totgrid; j++) {
+			grid = grids[unode->grids[j]];
+
+			for (i = 0; i < gridsize * gridsize; i++, mask++)
+				SWAP(float, *CCG_elem_offset_mask(&key, grid, i), *mask);
+		}
+	}
+
+	return 1;
+}
+
+static void sculpt_undo_bmesh_restore_generic(SculptUndoNode *unode,
+											  Object *ob,
+											  SculptSession *ss)
+{
+	if (unode->applied) {
+		BM_log_undo(ss->bm, ss->bm_log);
+		unode->applied = FALSE;
+	}
+	else {
+		BM_log_redo(ss->bm, ss->bm_log);
+		unode->applied = TRUE;
+	}
+
+	/* A bit lame, but for now just recreate the PBVH. The alternative
+	 * is to store changes to the PBVH in the undo stack. */
+	sculpt_pbvh_clear(ob);
+}
+
+/* Create empty sculpt BMesh and enable logging */
+static void sculpt_undo_bmesh_enable(Object *ob,
+									 SculptUndoNode *unode)
+{
+	SculptSession *ss = ob->sculpt;
+	Mesh *me = ob->data;
+
+	sculpt_pbvh_clear(ob);
+
+	/* Create empty BMesh and enable logging */
+	ss->bm = BM_mesh_create(&bm_mesh_allocsize_default);
+	BM_data_layer_add(ss->bm, &ss->bm->vdata, CD_PAINT_MASK);
+	me->flag |= ME_SCULPT_DYNAMIC_TOPOLOGY;
+
+	/* Restore the BMLog using saved entries */
+	ss->bm_log = BM_log_from_existing_entries_create(ss->bm,
+													 unode->bm_entry);
+}
+
+static void sculpt_undo_bmesh_restore_begin(bContext *C,
+											SculptUndoNode *unode,
+											Object *ob,
+											SculptSession *ss)
+{
+	if (unode->applied) {
+		sculpt_dynamic_topology_disable(C, unode);
+		unode->applied = FALSE;
+	}
+	else {
+		sculpt_undo_bmesh_enable(ob, unode);
+
+		/* Restore the mesh from the first log entry */
+		BM_log_redo(ss->bm, ss->bm_log);
+
+		unode->applied = TRUE;
+	}
+}
+
+static void sculpt_undo_bmesh_restore_end(bContext *C,
+										  SculptUndoNode *unode,
+										  Object *ob,
+										  SculptSession *ss)
+{
+	if (unode->applied) {
+		sculpt_undo_bmesh_enable(ob, unode);
+
+		/* Restore the mesh from the last log entry */
+		BM_log_undo(ss->bm, ss->bm_log);
+
+		unode->applied = FALSE;
+	}
+	else {
+		/* Disable dynamic topology sculpting */
+		sculpt_dynamic_topology_disable(C, NULL);
+		unode->applied = TRUE;
+	}
+}
+
+/* Handle all dynamic-topology updates
+ *
+ * Returns TRUE if this was a dynamic-topology undo step, otherwise
+ * returns FALSE to indicate the non-dyntopo code should run. */
+static int sculpt_undo_bmesh_restore(bContext *C,
+									 SculptUndoNode *unode,
+									 Object *ob,
+									 SculptSession *ss)
+{
+	switch (unode->type) {
+		case SCULPT_UNDO_DYNTOPO_BEGIN:
+			sculpt_undo_bmesh_restore_begin(C, unode, ob, ss);
+			return TRUE;
+
+		case SCULPT_UNDO_DYNTOPO_END:
+			sculpt_undo_bmesh_restore_end(C, unode, ob, ss);
+			return TRUE;
+
+		default:
+			if (ss->bm_log) {
+				sculpt_undo_bmesh_restore_generic(unode, ob, ss);
+				return TRUE;
+			}
+			break;
+	}
+
+	return FALSE;
+}
+
 static void sculpt_undo_restore(bContext *C, ListBase *lb)
 {
 	Scene *scene = CTX_data_scene(C);
 	Sculpt *sd = CTX_data_tool_settings(C)->sculpt;
 	Object *ob = CTX_data_active_object(C);
-	DerivedMesh *dm = mesh_get_derived_final(scene, ob, 0);
+	DerivedMesh *dm;
 	SculptSession *ss = ob->sculpt;
 	SculptUndoNode *unode;
 	MultiresModifierData *mmd;
-	int update = 0, rebuild = 1;
+	int update = FALSE, rebuild = FALSE;
+	int need_mask = FALSE;
 
-	sculpt_update_mesh_elements(scene, sd, ob, 0);
+	for (unode = lb->first; unode; unode = unode->next) {
+		if (strcmp(unode->idname, ob->id.name) == 0) {
+			if (unode->type == SCULPT_UNDO_MASK) {
+				/* is possible that we can't do the mask undo (below)
+				 * because of the vertex count */
+				need_mask = TRUE;
+				break;
+			}
+		}
+	}
+
+	sculpt_update_mesh_elements(scene, sd, ob, 0, need_mask);
+
+	/* call _after_ sculpt_update_mesh_elements() which may update 'ob->derivedFinal' */
+	dm = mesh_get_derived_final(scene, ob, 0);
+
+	if (lb->first && sculpt_undo_bmesh_restore(C, lb->first, ob, ss))
+		return;
 
 	for (unode = lb->first; unode; unode = unode->next) {
 		if (!(strcmp(unode->idname, ob->id.name) == 0))
@@ -237,20 +411,29 @@ static void sculpt_undo_restore(bContext *C, ListBase *lb)
 		else if (unode->maxgrid && dm->getGridData) {
 			if ((dm->getNumGrids(dm) != unode->maxgrid) ||
 			    (dm->getGridSize(dm) != unode->gridsize))
+			{
 				continue;
-		}
-		else {
-			continue;
+			}
 		}
 
 		switch (unode->type) {
 			case SCULPT_UNDO_COORDS:
 				if (sculpt_undo_restore_coords(C, dm, unode))
-					update = 1;
+					update = TRUE;
 				break;
 			case SCULPT_UNDO_HIDDEN:
 				if (sculpt_undo_restore_hidden(C, dm, unode))
-					rebuild = 1;
+					rebuild = TRUE;
+				break;
+			case SCULPT_UNDO_MASK:
+				if (sculpt_undo_restore_mask(C, dm, unode))
+					update = TRUE;
+				break;
+
+			case SCULPT_UNDO_DYNTOPO_BEGIN:
+			case SCULPT_UNDO_DYNTOPO_END:
+			case SCULPT_UNDO_DYNTOPO_SYMMETRIZE:
+				BLI_assert(!"Dynamic topology should've already been handled");
 				break;
 		}
 	}
@@ -260,8 +443,8 @@ static void sculpt_undo_restore(bContext *C, ListBase *lb)
 		/* we update all nodes still, should be more clever, but also
 		 * needs to work correct when exiting/entering sculpt mode and
 		 * the nodes get recreated, though in that case it could do all */
-		BLI_pbvh_search_callback(ss->pbvh, NULL, NULL, update_cb, &rebuild);
-		BLI_pbvh_update(ss->pbvh, PBVH_UpdateBB | PBVH_UpdateOriginalBB | PBVH_UpdateRedraw, NULL);
+		BKE_pbvh_search_callback(ss->pbvh, NULL, NULL, update_cb, &rebuild);
+		BKE_pbvh_update(ss->pbvh, PBVH_UpdateBB | PBVH_UpdateOriginalBB | PBVH_UpdateRedraw, NULL);
 
 		if ((mmd = sculpt_multires_active(scene, ob))) {
 			if (rebuild)
@@ -274,8 +457,8 @@ static void sculpt_undo_restore(bContext *C, ListBase *lb)
 
 		if (ss->modifiers_active) {
 			Mesh *mesh = ob->data;
-			mesh_calc_normals_tessface(mesh->mvert, mesh->totvert,
-									   mesh->mface, mesh->totface, NULL);
+			BKE_mesh_calc_normals_tessface(mesh->mvert, mesh->totvert,
+			                               mesh->mface, mesh->totface, NULL);
 
 			free_sculptsession_deformMats(ss);
 			tag_update |= 1;
@@ -303,8 +486,6 @@ static void sculpt_undo_free(ListBase *lb)
 			MEM_freeN(unode->index);
 		if (unode->grids)
 			MEM_freeN(unode->grids);
-		if (unode->layer_disp)
-			MEM_freeN(unode->layer_disp);
 		if (unode->orig_co)
 			MEM_freeN(unode->orig_co);
 		if (unode->vert_hidden)
@@ -316,22 +497,31 @@ static void sculpt_undo_free(ListBase *lb)
 			}
 			MEM_freeN(unode->grid_hidden);
 		}
+		if (unode->mask)
+			MEM_freeN(unode->mask);
+		if (unode->bm_entry) {
+			BM_log_entry_drop(unode->bm_entry);
+		}
+		if (unode->bm_enter_totvert)
+			CustomData_free(&unode->bm_enter_vdata, unode->bm_enter_totvert);
+		if (unode->bm_enter_totedge)
+			CustomData_free(&unode->bm_enter_edata, unode->bm_enter_totedge);
+		if (unode->bm_enter_totloop)
+			CustomData_free(&unode->bm_enter_ldata, unode->bm_enter_totloop);
+		if (unode->bm_enter_totpoly)
+			CustomData_free(&unode->bm_enter_pdata, unode->bm_enter_totpoly);
 	}
 }
 
 SculptUndoNode *sculpt_undo_get_node(PBVHNode *node)
 {
 	ListBase *lb = undo_paint_push_get_list(UNDO_PAINT_MESH);
-	SculptUndoNode *unode;
 
-	if (!lb)
+	if (!lb) {
 		return NULL;
+	}
 
-	for (unode = lb->first; unode; unode = unode->next)
-		if (unode->node == node)
-			return unode;
-
-	return NULL;
+	return BLI_findptr(lb, node, offsetof(SculptUndoNode, node));
 }
 
 static void sculpt_undo_alloc_and_store_hidden(PBVH *pbvh,
@@ -341,9 +531,9 @@ static void sculpt_undo_alloc_and_store_hidden(PBVH *pbvh,
 	BLI_bitmap *grid_hidden;
 	int i, *grid_indices, totgrid;
 
-	grid_hidden = BLI_pbvh_grid_hidden(pbvh);
+	grid_hidden = BKE_pbvh_grid_hidden(pbvh);
 
-	BLI_pbvh_node_get_grids(pbvh, node, &grid_indices, &totgrid,
+	BKE_pbvh_node_get_grids(pbvh, node, &grid_indices, &totgrid,
 	                        NULL, NULL, NULL, NULL);
 			
 	unode->grid_hidden = MEM_mapallocN(sizeof(BLI_bitmap) * totgrid,
@@ -370,11 +560,13 @@ static SculptUndoNode *sculpt_undo_alloc_node(Object *ob, PBVHNode *node,
 	unode->type = type;
 	unode->node = node;
 
-	BLI_pbvh_node_num_verts(ss->pbvh, node, &totvert, &allvert);
-	BLI_pbvh_node_get_grids(ss->pbvh, node, &grids, &totgrid,
-	                        &maxgrid, &gridsize, NULL, NULL);
+	if (node) {
+		BKE_pbvh_node_num_verts(ss->pbvh, node, &totvert, &allvert);
+		BKE_pbvh_node_get_grids(ss->pbvh, node, &grids, &totgrid,
+								&maxgrid, &gridsize, NULL, NULL);
 
-	unode->totvert = totvert;
+		unode->totvert = totvert;
+	}
 	
 	/* we will use this while sculpting, is mapalloc slow to access then? */
 
@@ -383,7 +575,10 @@ static SculptUndoNode *sculpt_undo_alloc_node(Object *ob, PBVHNode *node,
 		case SCULPT_UNDO_COORDS:
 			unode->co = MEM_mapallocN(sizeof(float) * 3 * allvert, "SculptUndoNode.co");
 			unode->no = MEM_mapallocN(sizeof(short) * 3 * allvert, "SculptUndoNode.no");
-			undo_paint_push_count_alloc(UNDO_PAINT_MESH, (sizeof(float) * 3 + sizeof(short) * 3 + sizeof(int)) * allvert);
+			undo_paint_push_count_alloc(UNDO_PAINT_MESH,
+			                            (sizeof(float) * 3 +
+			                             sizeof(short) * 3 +
+			                             sizeof(int)) * allvert);
 			break;
 		case SCULPT_UNDO_HIDDEN:
 			if (maxgrid)
@@ -392,6 +587,15 @@ static SculptUndoNode *sculpt_undo_alloc_node(Object *ob, PBVHNode *node,
 				unode->vert_hidden = BLI_BITMAP_NEW(allvert, "SculptUndoNode.vert_hidden");
 		
 			break;
+		case SCULPT_UNDO_MASK:
+			unode->mask = MEM_mapallocN(sizeof(float) * allvert, "SculptUndoNode.mask");
+			undo_paint_push_count_alloc(UNDO_PAINT_MESH, (sizeof(float) * sizeof(int)) * allvert);
+			break;
+		case SCULPT_UNDO_DYNTOPO_BEGIN:
+		case SCULPT_UNDO_DYNTOPO_END:
+		case SCULPT_UNDO_DYNTOPO_SYMMETRIZE:
+			BLI_assert(!"Dynamic topology should've already been handled");
+			break;
 	}
 	
 	BLI_addtail(lb, unode);
@@ -420,7 +624,8 @@ static void sculpt_undo_store_coords(Object *ob, SculptUndoNode *unode)
 	SculptSession *ss = ob->sculpt;
 	PBVHVertexIter vd;
 
-	BLI_pbvh_vertex_iter_begin(ss->pbvh, unode->node, vd, PBVH_ITER_ALL) {
+	BKE_pbvh_vertex_iter_begin(ss->pbvh, unode->node, vd, PBVH_ITER_ALL)
+	{
 		copy_v3_v3(unode->co[vd.i], vd.co);
 		if (vd.no) copy_v3_v3_short(unode->no[vd.i], vd.no);
 		else normal_float_to_short_v3(unode->no[vd.i], vd.fno);
@@ -428,7 +633,7 @@ static void sculpt_undo_store_coords(Object *ob, SculptUndoNode *unode)
 		if (ss->modifiers_active)
 			copy_v3_v3(unode->orig_co[vd.i], ss->orig_cos[unode->index[vd.i]]);
 	}
-	BLI_pbvh_vertex_iter_end;
+	BKE_pbvh_vertex_iter_end;
 }
 
 static void sculpt_undo_store_hidden(Object *ob, SculptUndoNode *unode)
@@ -444,8 +649,8 @@ static void sculpt_undo_store_hidden(Object *ob, SculptUndoNode *unode)
 		int *vert_indices, allvert;
 		int i;
 		
-		BLI_pbvh_node_num_verts(pbvh, node, NULL, &allvert);
-		BLI_pbvh_node_get_verts(pbvh, node, &vert_indices, &mvert);
+		BKE_pbvh_node_num_verts(pbvh, node, NULL, &allvert);
+		BKE_pbvh_node_get_verts(pbvh, node, &vert_indices, &mvert);
 		for (i = 0; i < allvert; i++) {
 			BLI_BITMAP_MODIFY(unode->vert_hidden, i,
 			                  mvert[vert_indices[i]].flag & ME_HIDE);
@@ -453,6 +658,92 @@ static void sculpt_undo_store_hidden(Object *ob, SculptUndoNode *unode)
 	}
 }
 
+static void sculpt_undo_store_mask(Object *ob, SculptUndoNode *unode)
+{
+	SculptSession *ss = ob->sculpt;
+	PBVHVertexIter vd;
+
+	BKE_pbvh_vertex_iter_begin(ss->pbvh, unode->node, vd, PBVH_ITER_ALL)
+	{
+		unode->mask[vd.i] = *vd.mask;
+	}
+	BKE_pbvh_vertex_iter_end;
+}
+
+static SculptUndoNode *sculpt_undo_bmesh_push(Object *ob,
+											  PBVHNode *node,
+											  SculptUndoType type)
+{
+	ListBase *lb = undo_paint_push_get_list(UNDO_PAINT_MESH);
+	SculptUndoNode *unode = lb->first;
+	SculptSession *ss = ob->sculpt;
+	PBVHVertexIter vd;
+
+	if (!lb->first) {
+		unode = MEM_callocN(sizeof(*unode), AT);
+
+		BLI_strncpy(unode->idname, ob->id.name, sizeof(unode->idname));
+		unode->type = type;
+		unode->applied = TRUE;
+
+		if (type == SCULPT_UNDO_DYNTOPO_END) {
+			unode->bm_entry = BM_log_entry_add(ss->bm_log);
+			BM_log_before_all_removed(ss->bm, ss->bm_log);
+		}
+		else if (type == SCULPT_UNDO_DYNTOPO_BEGIN) {
+			Mesh *me = ob->data;
+
+			/* Store a copy of the mesh's current vertices, loops, and
+			 * polys. A full copy like this is needed because entering
+			 * dynamic-topology immediately does topological edits
+			 * (converting polys to triangles) that the BMLog can't
+			 * fully restore from */
+			CustomData_copy(&me->vdata, &unode->bm_enter_vdata, CD_MASK_MESH,
+							CD_DUPLICATE, me->totvert);
+			CustomData_copy(&me->edata, &unode->bm_enter_edata, CD_MASK_MESH,
+							CD_DUPLICATE, me->totedge);
+			CustomData_copy(&me->ldata, &unode->bm_enter_ldata, CD_MASK_MESH,
+							CD_DUPLICATE, me->totloop);
+			CustomData_copy(&me->pdata, &unode->bm_enter_pdata, CD_MASK_MESH,
+							CD_DUPLICATE, me->totpoly);
+			unode->bm_enter_totvert = me->totvert;
+			unode->bm_enter_totedge = me->totedge;
+			unode->bm_enter_totloop = me->totloop;
+			unode->bm_enter_totpoly = me->totpoly;
+
+			unode->bm_entry = BM_log_entry_add(ss->bm_log);
+			BM_log_all_added(ss->bm, ss->bm_log);
+		}
+		else {
+			unode->bm_entry = BM_log_entry_add(ss->bm_log);
+		}
+
+		BLI_addtail(lb, unode);
+	}
+
+	if (node) {
+		switch (type) {
+			case SCULPT_UNDO_COORDS:
+			case SCULPT_UNDO_HIDDEN:
+			case SCULPT_UNDO_MASK:
+				/* Before any vertex values get modified, ensure their
+				 * original positions are logged */
+				BKE_pbvh_vertex_iter_begin(ss->pbvh, node, vd, PBVH_ITER_ALL) {
+					BM_log_vert_before_modified(ss->bm, ss->bm_log, vd.bm_vert);
+				}
+				BKE_pbvh_vertex_iter_end;
+				break;
+
+			case SCULPT_UNDO_DYNTOPO_BEGIN:
+			case SCULPT_UNDO_DYNTOPO_END:
+			case SCULPT_UNDO_DYNTOPO_SYMMETRIZE:
+				break;
+		}
+	}
+
+	return unode;
+}
+
 SculptUndoNode *sculpt_undo_push_node(Object *ob, PBVHNode *node,
                                       SculptUndoType type)
 {
@@ -462,7 +753,18 @@ SculptUndoNode *sculpt_undo_push_node(Object *ob, PBVHNode *node,
 	/* list is manipulated by multiple threads, so we lock */
 	BLI_lock_thread(LOCK_CUSTOM1);
 
-	if ((unode = sculpt_undo_get_node(node))) {
+	if (ss->bm ||
+		ELEM(type,
+			 SCULPT_UNDO_DYNTOPO_BEGIN,
+			 SCULPT_UNDO_DYNTOPO_END))
+	{
+		/* Dynamic topology stores only one undo node per stroke,
+		 * regardless of the number of PBVH nodes modified */
+		unode = sculpt_undo_bmesh_push(ob, node, type);
+		BLI_unlock_thread(LOCK_CUSTOM1);
+		return unode;
+	}
+	else if ((unode = sculpt_undo_get_node(node))) {
 		BLI_unlock_thread(LOCK_CUSTOM1);
 		return unode;
 	}
@@ -475,14 +777,14 @@ SculptUndoNode *sculpt_undo_push_node(Object *ob, PBVHNode *node,
 
 	if (unode->grids) {
 		int totgrid, *grids;
-		BLI_pbvh_node_get_grids(ss->pbvh, node, &grids, &totgrid,
+		BKE_pbvh_node_get_grids(ss->pbvh, node, &grids, &totgrid,
 		                        NULL, NULL, NULL, NULL);
 		memcpy(unode->grids, grids, sizeof(int) * totgrid);
 	}
 	else {
 		int *vert_indices, allvert;
-		BLI_pbvh_node_num_verts(ss->pbvh, node, NULL, &allvert);
-		BLI_pbvh_node_get_verts(ss->pbvh, node, &vert_indices, NULL);
+		BKE_pbvh_node_num_verts(ss->pbvh, node, NULL, &allvert);
+		BKE_pbvh_node_get_verts(ss->pbvh, node, &vert_indices, NULL);
 		memcpy(unode->index, vert_indices, sizeof(int) * unode->totvert);
 	}
 
@@ -493,6 +795,14 @@ SculptUndoNode *sculpt_undo_push_node(Object *ob, PBVHNode *node,
 		case SCULPT_UNDO_HIDDEN:
 			sculpt_undo_store_hidden(ob, unode);
 			break;
+		case SCULPT_UNDO_MASK:
+			sculpt_undo_store_mask(ob, unode);
+			break;
+		case SCULPT_UNDO_DYNTOPO_BEGIN:
+		case SCULPT_UNDO_DYNTOPO_END:
+		case SCULPT_UNDO_DYNTOPO_SYMMETRIZE:
+			BLI_assert(!"Dynamic topology should've already been handled");
+			break;
 	}
 
 	/* store active shape key */
@@ -520,10 +830,8 @@ void sculpt_undo_push_end(void)
 			unode->no = NULL;
 		}
 
-		if (unode->layer_disp) {
-			MEM_freeN(unode->layer_disp);
-			unode->layer_disp = NULL;
-		}
+		if (unode->node)
+			BKE_pbvh_node_layer_disp_free(unode->node);
 	}
 
 	undo_paint_push_end(UNDO_PAINT_MESH);
