@@ -80,7 +80,7 @@
 static int sound_open_cancel(bContext *UNUSED(C), wmOperator *op)
 {
 	MEM_freeN(op->customdata);
-	op->customdata= NULL;
+	op->customdata = NULL;
 	return OPERATOR_CANCELLED;
 }
 
@@ -88,7 +88,7 @@ static void sound_open_init(bContext *C, wmOperator *op)
 {
 	PropertyPointerRNA *pprop;
 
-	op->customdata= pprop= MEM_callocN(sizeof(PropertyPointerRNA), "OpenPropertyPointerRNA");
+	op->customdata = pprop = MEM_callocN(sizeof(PropertyPointerRNA), "OpenPropertyPointerRNA");
 	uiIDContextProperty(C, &pprop->ptr, &pprop->prop);
 }
 
@@ -108,7 +108,7 @@ static int sound_open_exec(bContext *C, wmOperator *op)
 	if (!op->customdata)
 		sound_open_init(C, op);
 
-	if (sound==NULL || sound->playback_handle == NULL) {
+	if (sound == NULL || sound->playback_handle == NULL) {
 		if (op->customdata) MEM_freeN(op->customdata);
 		BKE_report(op->reports, RPT_ERROR, "Unsupported audio format");
 		return OPERATOR_CANCELLED;
@@ -133,7 +133,7 @@ static int sound_open_exec(bContext *C, wmOperator *op)
 	}
 
 	/* hook into UI */
-	pprop= op->customdata;
+	pprop = op->customdata;
 
 	if (pprop->prop) {
 		/* when creating new ID blocks, use is already 1, but RNA
@@ -183,10 +183,11 @@ static void SOUND_OT_open(wmOperatorType *ot)
 	ot->cancel = sound_open_cancel;
 
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
 	/* properties */
-	WM_operator_properties_filesel(ot, FOLDERFILE|SOUNDFILE|MOVIEFILE, FILE_SPECIAL, FILE_OPENFILE, WM_FILESEL_FILEPATH | WM_FILESEL_RELPATH, FILE_DEFAULTDISPLAY);
+	WM_operator_properties_filesel(ot, FOLDERFILE | SOUNDFILE | MOVIEFILE, FILE_SPECIAL, FILE_OPENFILE,
+	                               WM_FILESEL_FILEPATH | WM_FILESEL_RELPATH, FILE_DEFAULTDISPLAY);
 	RNA_def_boolean(ot->srna, "cache", FALSE, "Cache", "Cache the sound in memory");
 	RNA_def_boolean(ot->srna, "mono", FALSE, "Mono", "Mixdown the sound to mono");
 }
@@ -204,10 +205,11 @@ static void SOUND_OT_open_mono(wmOperatorType *ot)
 	ot->cancel = sound_open_cancel;
 
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
 	/* properties */
-	WM_operator_properties_filesel(ot, FOLDERFILE|SOUNDFILE|MOVIEFILE, FILE_SPECIAL, FILE_OPENFILE, WM_FILESEL_FILEPATH | WM_FILESEL_RELPATH, FILE_DEFAULTDISPLAY);
+	WM_operator_properties_filesel(ot, FOLDERFILE | SOUNDFILE | MOVIEFILE, FILE_SPECIAL, FILE_OPENFILE,
+	                               WM_FILESEL_FILEPATH | WM_FILESEL_RELPATH, FILE_DEFAULTDISPLAY);
 	RNA_def_boolean(ot->srna, "cache", FALSE, "Cache", "Cache the sound in memory");
 	RNA_def_boolean(ot->srna, "mono", TRUE, "Mono", "Mixdown the sound to mono");
 }
@@ -216,12 +218,13 @@ static void SOUND_OT_open_mono(wmOperatorType *ot)
 
 static int sound_update_animation_flags_exec(bContext *C, wmOperator *UNUSED(op))
 {
-	Sequence* seq;
-	Scene* scene = CTX_data_scene(C);
-	struct FCurve* fcu;
+	Sequence *seq;
+	Scene *scene = CTX_data_scene(C);
+	struct FCurve *fcu;
 	char driven;
 
-	SEQ_BEGIN(scene->ed, seq) {
+	SEQ_BEGIN(scene->ed, seq)
+	{
 		fcu = id_data_find_fcurve(&scene->id, seq, &RNA_Sequence, "volume", 0, &driven);
 		if (fcu || driven)
 			seq->flag |= SEQ_AUDIO_VOLUME_ANIMATED;
@@ -242,7 +245,7 @@ static int sound_update_animation_flags_exec(bContext *C, wmOperator *UNUSED(op)
 	}
 	SEQ_END
 
-	fcu = id_data_find_fcurve(&scene->id, scene, &RNA_Scene, "audio_volume", 0, &driven);
+	    fcu = id_data_find_fcurve(&scene->id, scene, &RNA_Scene, "audio_volume", 0, &driven);
 	if (fcu || driven)
 		scene->audio.flag |= AUDIO_VOLUME_ANIMATED;
 	else
@@ -261,7 +264,7 @@ static void SOUND_OT_update_animation_flags(wmOperatorType *ot)
 	 */
 
 	/* identifiers */
-	ot->name = "Update animation";
+	ot->name = "Update Animation";
 	ot->description = "Update animation flags";
 	ot->idname = "SOUND_OT_update_animation_flags";
 
@@ -276,8 +279,8 @@ static void SOUND_OT_update_animation_flags(wmOperatorType *ot)
 
 static int sound_bake_animation_exec(bContext *C, wmOperator *UNUSED(op))
 {
-	Main* bmain = CTX_data_main(C);
-	Scene* scene = CTX_data_scene(C);
+	Main *bmain = CTX_data_main(C);
+	Scene *scene = CTX_data_scene(C);
 	int oldfra = scene->r.cfra;
 	int cfra;
 
@@ -285,11 +288,11 @@ static int sound_bake_animation_exec(bContext *C, wmOperator *UNUSED(op))
 
 	for (cfra = (scene->r.sfra > 0) ? (scene->r.sfra - 1) : 0; cfra <= scene->r.efra + 1; cfra++) {
 		scene->r.cfra = cfra;
-		scene_update_for_newframe(bmain, scene, scene->lay);
+		BKE_scene_update_for_newframe(bmain, scene, scene->lay);
 	}
 
 	scene->r.cfra = oldfra;
-	scene_update_for_newframe(bmain, scene, scene->lay);
+	BKE_scene_update_for_newframe(bmain, scene, scene->lay);
 
 	return OPERATOR_FINISHED;
 }
@@ -297,7 +300,7 @@ static int sound_bake_animation_exec(bContext *C, wmOperator *UNUSED(op))
 static void SOUND_OT_bake_animation(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Update animation cache";
+	ot->name = "Update Animation Cache";
 	ot->description = "Update the audio animation cache";
 	ot->idname = "SOUND_OT_bake_animation";
 
@@ -324,7 +327,7 @@ static int sound_mixdown_exec(bContext *C, wmOperator *op)
 	AUD_DeviceSpecs specs;
 	AUD_Container container;
 	AUD_Codec codec;
-	const char* result;
+	const char *result;
 
 	sound_bake_animation_exec(C, op);
 
@@ -343,12 +346,12 @@ static int sound_mixdown_exec(bContext *C, wmOperator *op)
 	BLI_strncpy(filename, path, sizeof(filename));
 	BLI_path_abs(filename, bmain->name);
 
-	if(split)
+	if (split)
 		result = AUD_mixdown_per_channel(scene->sound_scene, SFRA * specs.rate / FPS, (EFRA - SFRA) * specs.rate / FPS,
-										 accuracy, filename, specs, container, codec, bitrate);
+		                                 accuracy, filename, specs, container, codec, bitrate);
 	else
 		result = AUD_mixdown(scene->sound_scene, SFRA * specs.rate / FPS, (EFRA - SFRA) * specs.rate / FPS,
-							 accuracy, filename, specs, container, codec, bitrate);
+		                     accuracy, filename, specs, container, codec, bitrate);
 
 	if (result) {
 		BKE_report(op->reports, RPT_ERROR, result);
@@ -361,6 +364,81 @@ static int sound_mixdown_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
+#ifdef WITH_AUDASPACE
+static EnumPropertyItem container_items[] = {
+#ifdef WITH_FFMPEG
+	{AUD_CONTAINER_AC3, "AC3", 0, "ac3", "Dolby Digital ATRAC 3"},
+#endif
+	{AUD_CONTAINER_FLAC, "FLAC", 0, "flac", "Free Lossless Audio Codec"},
+#ifdef WITH_FFMPEG
+	{AUD_CONTAINER_MATROSKA, "MATROSKA", 0, "mkv", "Matroska"},
+	{AUD_CONTAINER_MP2, "MP2", 0, "mp2", "MPEG-1 Audio Layer II"},
+	{AUD_CONTAINER_MP3, "MP3", 0, "mp3", "MPEG-2 Audio Layer III"},
+#endif
+	{AUD_CONTAINER_OGG, "OGG", 0, "ogg", "Xiph.Org Ogg Container"},
+	{AUD_CONTAINER_WAV, "WAV", 0, "wav", "Waveform Audio File Format"},
+	{0, NULL, 0, NULL, NULL}
+};
+
+static const char *snd_ext_sound[] = {
+	".ac3",
+	".flac",
+	".mkv",
+	".mp2",
+	".mp3",
+	".ogg",
+	".wav",
+	NULL
+};
+
+static int sound_mixdown_check(bContext *UNUSED(C), wmOperator *op)
+{
+	AUD_Container container = RNA_enum_get(op->ptr, "container");
+
+	const char *extension = NULL;
+
+	EnumPropertyItem *item = container_items;
+	while (item->identifier != NULL) {
+		if (item->value == container) {
+			const char **ext = snd_ext_sound;
+			while (*ext != NULL) {
+				if (!strcmp(*ext + 1, item->name)) {
+					extension = *ext;
+					break;
+				}
+
+				ext++;
+			}
+		}
+		item++;
+	}
+
+	if (extension) {
+		PropertyRNA *prop;
+		char filepath[FILE_MAX];
+
+		int check;
+
+		prop = RNA_struct_find_property(op->ptr, "filepath");
+		RNA_property_string_get(op->ptr, prop, filepath);
+
+		if (BLI_testextensie_array(filepath, snd_ext_sound))
+			check = BLI_replace_extension(filepath, FILE_MAX, extension);
+		else
+			check = BLI_ensure_extension(filepath, FILE_MAX, extension);
+
+		if (!check)
+			return check;
+
+		RNA_property_string_set(op->ptr, prop, filepath);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+#endif // WITH_AUDASPACE
+
 static int sound_mixdown_invoke(bContext *C, wmOperator *op, wmEvent *event)
 {
 	if (RNA_struct_property_is_set(op->ptr, "filepath"))
@@ -373,11 +451,10 @@ static int sound_mixdown_invoke(bContext *C, wmOperator *op, wmEvent *event)
 
 static int sound_mixdown_draw_check_prop(PointerRNA *UNUSED(ptr), PropertyRNA *prop)
 {
-	const char *prop_id= RNA_property_identifier(prop);
-	return !(	strcmp(prop_id, "filepath") == 0 ||
-				strcmp(prop_id, "directory") == 0 ||
-				strcmp(prop_id, "filename") == 0
-	);
+	const char *prop_id = RNA_property_identifier(prop);
+	return !(strcmp(prop_id, "filepath") == 0 ||
+	         strcmp(prop_id, "directory") == 0 ||
+	         strcmp(prop_id, "filename") == 0);
 }
 
 static void sound_mixdown_draw(bContext *C, wmOperator *op)
@@ -391,23 +468,27 @@ static void sound_mixdown_draw(bContext *C, wmOperator *op)
 		{AUD_FORMAT_S32, "S32", 0, "S32", "32 bit signed"},
 		{AUD_FORMAT_FLOAT32, "F32", 0, "F32", "32 bit floating point"},
 		{AUD_FORMAT_FLOAT64, "F64", 0, "F64", "64 bit floating point"},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 
 	static EnumPropertyItem mp3_format_items[] = {
 		{AUD_FORMAT_S16, "S16", 0, "S16", "16 bit signed"},
 		{AUD_FORMAT_S32, "S32", 0, "S32", "32 bit signed"},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 
 	static EnumPropertyItem ac3_format_items[] = {
 		{AUD_FORMAT_S16, "S16", 0, "S16", "16 bit signed"},
 		{AUD_FORMAT_FLOAT32, "F32", 0, "F32", "32 bit floating point"},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 
 #ifdef WITH_SNDFILE
 	static EnumPropertyItem flac_format_items[] = {
 		{AUD_FORMAT_S16, "S16", 0, "S16", "16 bit signed"},
 		{AUD_FORMAT_S24, "S24", 0, "S24", "24 bit signed"},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 #endif
 
 	static EnumPropertyItem all_codec_items[] = {
@@ -418,15 +499,17 @@ static void sound_mixdown_draw(bContext *C, wmOperator *op)
 		{AUD_CODEC_MP3, "MP3", 0, "MP3", "MPEG-2 Audio Layer III"},
 		{AUD_CODEC_PCM, "PCM", 0, "PCM", "Pulse Code Modulation (RAW)"},
 		{AUD_CODEC_VORBIS, "VORBIS", 0, "Vorbis", "Xiph.Org Vorbis Codec"},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 
 	static EnumPropertyItem ogg_codec_items[] = {
 		{AUD_CODEC_FLAC, "FLAC", 0, "FLAC", "Free Lossless Audio Codec"},
 		{AUD_CODEC_VORBIS, "VORBIS", 0, "Vorbis", "Xiph.Org Vorbis Codec"},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 
 	uiLayout *layout = op->layout;
-	wmWindowManager *wm= CTX_wm_manager(C);
+	wmWindowManager *wm = CTX_wm_manager(C);
 	PointerRNA ptr;
 	PropertyRNA *prop_format;
 	PropertyRNA *prop_codec;
@@ -443,87 +526,85 @@ static void sound_mixdown_draw(bContext *C, wmOperator *op)
 	RNA_def_property_flag(prop_codec, PROP_HIDDEN);
 	RNA_def_property_flag(prop_format, PROP_HIDDEN);
 
-	switch(container)
-	{
-	case AUD_CONTAINER_AC3:
-		RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
-		RNA_def_property_enum_items(prop_format, ac3_format_items);
-		RNA_def_property_enum_items(prop_codec, all_codec_items);
-		RNA_enum_set(op->ptr, "codec", AUD_CODEC_AC3);
-		break;
-	case AUD_CONTAINER_FLAC:
-		RNA_def_property_flag(prop_bitrate, PROP_HIDDEN);
-		RNA_def_property_enum_items(prop_codec, all_codec_items);
-		RNA_enum_set(op->ptr, "codec", AUD_CODEC_FLAC);
-#ifdef WITH_SNDFILE
-		RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
-		RNA_def_property_enum_items(prop_format, flac_format_items);
-#else
-		RNA_enum_set(op->ptr, "format", AUD_FORMAT_S16);
-#endif
-		break;
-	case AUD_CONTAINER_MATROSKA:
-		RNA_def_property_clear_flag(prop_codec, PROP_HIDDEN);
-		RNA_def_property_enum_items(prop_codec, all_codec_items);
-
-		switch(codec)
-		{
-		case AUD_CODEC_AAC:
-			RNA_enum_set(op->ptr, "format", AUD_FORMAT_S16);
-			break;
-		case AUD_CODEC_AC3:
+	switch (container) {
+		case AUD_CONTAINER_AC3:
+			RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
 			RNA_def_property_enum_items(prop_format, ac3_format_items);
-			RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
+			RNA_def_property_enum_items(prop_codec, all_codec_items);
+			RNA_enum_set(op->ptr, "codec", AUD_CODEC_AC3);
 			break;
-		case AUD_CODEC_FLAC:
+		case AUD_CONTAINER_FLAC:
 			RNA_def_property_flag(prop_bitrate, PROP_HIDDEN);
+			RNA_def_property_enum_items(prop_codec, all_codec_items);
+			RNA_enum_set(op->ptr, "codec", AUD_CODEC_FLAC);
+#ifdef WITH_SNDFILE
+			RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
+			RNA_def_property_enum_items(prop_format, flac_format_items);
+#else
 			RNA_enum_set(op->ptr, "format", AUD_FORMAT_S16);
+#endif
 			break;
-		case AUD_CODEC_MP2:
+		case AUD_CONTAINER_MATROSKA:
+			RNA_def_property_clear_flag(prop_codec, PROP_HIDDEN);
+			RNA_def_property_enum_items(prop_codec, all_codec_items);
+
+			switch (codec) {
+				case AUD_CODEC_AAC:
+					RNA_enum_set(op->ptr, "format", AUD_FORMAT_S16);
+					break;
+				case AUD_CODEC_AC3:
+					RNA_def_property_enum_items(prop_format, ac3_format_items);
+					RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
+					break;
+				case AUD_CODEC_FLAC:
+					RNA_def_property_flag(prop_bitrate, PROP_HIDDEN);
+					RNA_enum_set(op->ptr, "format", AUD_FORMAT_S16);
+					break;
+				case AUD_CODEC_MP2:
+					RNA_enum_set(op->ptr, "format", AUD_FORMAT_S16);
+					break;
+				case AUD_CODEC_MP3:
+					RNA_def_property_enum_items(prop_format, mp3_format_items);
+					RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
+					break;
+				case AUD_CODEC_PCM:
+					RNA_def_property_flag(prop_bitrate, PROP_HIDDEN);
+					RNA_def_property_enum_items(prop_format, pcm_format_items);
+					RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
+					break;
+				case AUD_CODEC_VORBIS:
+					RNA_enum_set(op->ptr, "format", AUD_FORMAT_S16);
+					break;
+				default:
+					break;
+			}
+
+			break;
+		case AUD_CONTAINER_MP2:
 			RNA_enum_set(op->ptr, "format", AUD_FORMAT_S16);
+			RNA_enum_set(op->ptr, "codec", AUD_CODEC_MP2);
+			RNA_def_property_enum_items(prop_codec, all_codec_items);
 			break;
-		case AUD_CODEC_MP3:
+		case AUD_CONTAINER_MP3:
+			RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
 			RNA_def_property_enum_items(prop_format, mp3_format_items);
-			RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
+			RNA_def_property_enum_items(prop_codec, all_codec_items);
+			RNA_enum_set(op->ptr, "codec", AUD_CODEC_MP3);
 			break;
-		case AUD_CODEC_PCM:
-			RNA_def_property_flag(prop_bitrate, PROP_HIDDEN);
-			RNA_def_property_enum_items(prop_format, pcm_format_items);
-			RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
-			break;
-		case AUD_CODEC_VORBIS:
+		case AUD_CONTAINER_OGG:
+			RNA_def_property_clear_flag(prop_codec, PROP_HIDDEN);
+			RNA_def_property_enum_items(prop_codec, ogg_codec_items);
 			RNA_enum_set(op->ptr, "format", AUD_FORMAT_S16);
 			break;
+		case AUD_CONTAINER_WAV:
+			RNA_def_property_flag(prop_bitrate, PROP_HIDDEN);
+			RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
+			RNA_def_property_enum_items(prop_format, pcm_format_items);
+			RNA_def_property_enum_items(prop_codec, all_codec_items);
+			RNA_enum_set(op->ptr, "codec", AUD_CODEC_PCM);
+			break;
 		default:
 			break;
-		}
-
-		break;
-	case AUD_CONTAINER_MP2:
-		RNA_enum_set(op->ptr, "format", AUD_FORMAT_S16);
-		RNA_enum_set(op->ptr, "codec", AUD_CODEC_MP2);
-		RNA_def_property_enum_items(prop_codec, all_codec_items);
-		break;
-	case AUD_CONTAINER_MP3:
-		RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
-		RNA_def_property_enum_items(prop_format, mp3_format_items);
-		RNA_def_property_enum_items(prop_codec, all_codec_items);
-		RNA_enum_set(op->ptr, "codec", AUD_CODEC_MP3);
-		break;
-	case AUD_CONTAINER_OGG:
-		RNA_def_property_clear_flag(prop_codec, PROP_HIDDEN);
-		RNA_def_property_enum_items(prop_codec, ogg_codec_items);
-		RNA_enum_set(op->ptr, "format", AUD_FORMAT_S16);
-		break;
-	case AUD_CONTAINER_WAV:
-		RNA_def_property_flag(prop_bitrate, PROP_HIDDEN);
-		RNA_def_property_clear_flag(prop_format, PROP_HIDDEN);
-		RNA_def_property_enum_items(prop_format, pcm_format_items);
-		RNA_def_property_enum_items(prop_codec, all_codec_items);
-		RNA_enum_set(op->ptr, "codec", AUD_CODEC_PCM);
-		break;
-	default:
-		break;
 	}
 
 	RNA_pointer_create(&wm->id, op->type->srna, op->properties, &ptr);
@@ -543,21 +624,8 @@ static void SOUND_OT_mixdown(wmOperatorType *ot)
 		{AUD_FORMAT_S32, "S32", 0, "S32", "32 bit signed"},
 		{AUD_FORMAT_FLOAT32, "F32", 0, "F32", "32 bit floating point"},
 		{AUD_FORMAT_FLOAT64, "F64", 0, "F64", "64 bit floating point"},
-		{0, NULL, 0, NULL, NULL}};
-
-	static EnumPropertyItem container_items[] = {
-#ifdef WITH_FFMPEG
-		{AUD_CONTAINER_AC3, "AC3", 0, "ac3", "Dolby Digital ATRAC 3"},
-#endif
-		{AUD_CONTAINER_FLAC, "FLAC", 0, "flac", "Free Lossless Audio Codec"},
-#ifdef WITH_FFMPEG
-		{AUD_CONTAINER_MATROSKA, "MATROSKA", 0, "mkv", "Matroska"},
-		{AUD_CONTAINER_MP2, "MP2", 0, "mp2", "MPEG-1 Audio Layer II"},
-		{AUD_CONTAINER_MP3, "MP3", 0, "mp3", "MPEG-2 Audio Layer III"},
-#endif
-		{AUD_CONTAINER_OGG, "OGG", 0, "ogg", "Xiph.Org Ogg Container"},
-		{AUD_CONTAINER_WAV, "WAV", 0, "wav", "Waveform Audio File Format"},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 
 	static EnumPropertyItem codec_items[] = {
 #ifdef WITH_FFMPEG
@@ -571,7 +639,8 @@ static void SOUND_OT_mixdown(wmOperatorType *ot)
 #endif
 		{AUD_CODEC_PCM, "PCM", 0, "PCM", "Pulse Code Modulation (RAW)"},
 		{AUD_CODEC_VORBIS, "VORBIS", 0, "Vorbis", "Xiph.Org Vorbis Codec"},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 
 #endif // WITH_AUDASPACE
 
@@ -585,13 +654,15 @@ static void SOUND_OT_mixdown(wmOperatorType *ot)
 	ot->invoke = sound_mixdown_invoke;
 
 #ifdef WITH_AUDASPACE
+	ot->check = sound_mixdown_check;
 	ot->ui = sound_mixdown_draw;
 #endif
 	/* flags */
 	ot->flag = OPTYPE_REGISTER;
 
 	/* properties */
-	WM_operator_properties_filesel(ot, FOLDERFILE|SOUNDFILE, FILE_SPECIAL, FILE_SAVE, WM_FILESEL_FILEPATH, FILE_DEFAULTDISPLAY);
+	WM_operator_properties_filesel(ot, FOLDERFILE | SOUNDFILE, FILE_SPECIAL, FILE_SAVE,
+	                               WM_FILESEL_FILEPATH | WM_FILESEL_RELPATH, FILE_DEFAULTDISPLAY);
 #ifdef WITH_AUDASPACE
 	RNA_def_int(ot->srna, "accuracy", 1024, 1, 16777216, "Accuracy", "Sample accuracy, important for animation data (the lower the value, the more accurate)", 1, 16777216);
 	RNA_def_enum(ot->srna, "container", container_items, AUD_CONTAINER_FLAC, "Container", "File format");
@@ -606,9 +677,9 @@ static void SOUND_OT_mixdown(wmOperatorType *ot)
 
 static int sound_poll(bContext *C)
 {
-	Editing* ed = CTX_data_scene(C)->ed;
+	Editing *ed = CTX_data_scene(C)->ed;
 
-	if (!ed || !ed->act_seq || ed->act_seq->type != SEQ_SOUND)
+	if (!ed || !ed->act_seq || ed->act_seq->type != SEQ_TYPE_SOUND_RAM)
 		return 0;
 
 	return 1;
@@ -617,11 +688,11 @@ static int sound_poll(bContext *C)
 
 static int sound_pack_exec(bContext *C, wmOperator *op)
 {
-	Main *bmain= CTX_data_main(C);
-	Editing* ed = CTX_data_scene(C)->ed;
-	bSound* sound;
+	Main *bmain = CTX_data_main(C);
+	Editing *ed = CTX_data_scene(C)->ed;
+	bSound *sound;
 
-	if (!ed || !ed->act_seq || ed->act_seq->type != SEQ_SOUND)
+	if (!ed || !ed->act_seq || ed->act_seq->type != SEQ_TYPE_SOUND_RAM)
 		return OPERATOR_CANCELLED;
 
 	sound = ed->act_seq->sound;
@@ -629,8 +700,8 @@ static int sound_pack_exec(bContext *C, wmOperator *op)
 	if (!sound || sound->packedfile)
 		return OPERATOR_CANCELLED;
 
-	sound->packedfile= newPackedFile(op->reports, sound->name, ID_BLEND_PATH(bmain, &sound->id));
-	sound_load(CTX_data_main(C), sound);
+	sound->packedfile = newPackedFile(op->reports, sound->name, ID_BLEND_PATH(bmain, &sound->id));
+	sound_load(bmain, sound);
 
 	return OPERATOR_FINISHED;
 }
@@ -647,21 +718,22 @@ static void SOUND_OT_pack(wmOperatorType *ot)
 	ot->poll = sound_poll;
 
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
 
 /********************* unpack operator *********************/
 
 static int sound_unpack_exec(bContext *C, wmOperator *op)
 {
-	int method= RNA_enum_get(op->ptr, "method");
-	bSound* sound= NULL;
+	Main *bmain = CTX_data_main(C);
+	int method = RNA_enum_get(op->ptr, "method");
+	bSound *sound = NULL;
 
 	/* find the suppplied image by name */
 	if (RNA_struct_property_is_set(op->ptr, "id")) {
-		char sndname[MAX_ID_NAME-2];
+		char sndname[MAX_ID_NAME - 2];
 		RNA_string_get(op->ptr, "id", sndname);
-		sound = BLI_findstring(&CTX_data_main(C)->sound, sndname, offsetof(ID, name) + 2);
+		sound = BLI_findstring(&bmain->sound, sndname, offsetof(ID, name) + 2);
 	}
 
 	if (!sound || !sound->packedfile)
@@ -670,20 +742,20 @@ static int sound_unpack_exec(bContext *C, wmOperator *op)
 	if (G.fileflags & G_AUTOPACK)
 		BKE_report(op->reports, RPT_WARNING, "AutoPack is enabled, so image will be packed again on file save");
 
-	unpackSound(CTX_data_main(C), op->reports, sound, method);
+	unpackSound(bmain, op->reports, sound, method);
 
 	return OPERATOR_FINISHED;
 }
 
 static int sound_unpack_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
 {
-	Editing* ed = CTX_data_scene(C)->ed;
-	bSound* sound;
+	Editing *ed = CTX_data_scene(C)->ed;
+	bSound *sound;
 
 	if (RNA_struct_property_is_set(op->ptr, "id"))
 		return sound_unpack_exec(C, op);
 
-	if (!ed || !ed->act_seq || ed->act_seq->type != SEQ_SOUND)
+	if (!ed || !ed->act_seq || ed->act_seq->type != SEQ_TYPE_SOUND_RAM)
 		return OPERATOR_CANCELLED;
 
 	sound = ed->act_seq->sound;
@@ -694,7 +766,7 @@ static int sound_unpack_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(even
 	if (G.fileflags & G_AUTOPACK)
 		BKE_report(op->reports, RPT_WARNING, "AutoPack is enabled, so image will be packed again on file save");
 
-	unpack_menu(C, "SOUND_OT_unpack", sound->id.name+2, sound->name, "sounds", sound->packedfile);
+	unpack_menu(C, "SOUND_OT_unpack", sound->id.name + 2, sound->name, "sounds", sound->packedfile);
 
 	return OPERATOR_FINISHED;
 }
@@ -712,11 +784,11 @@ static void SOUND_OT_unpack(wmOperatorType *ot)
 	ot->poll = sound_poll;
 
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
 	/* properties */
 	RNA_def_enum(ot->srna, "method", unpack_method_items, PF_USE_LOCAL, "Method", "How to unpack");
-	RNA_def_string(ot->srna, "id", "", MAX_ID_NAME-2, "Sound Name", "Sound datablock name to unpack"); /* XXX, weark!, will fail with library, name collisions */
+	RNA_def_string(ot->srna, "id", "", MAX_ID_NAME - 2, "Sound Name", "Sound datablock name to unpack"); /* XXX, weark!, will fail with library, name collisions */
 }
 
 /* ******************************************************* */
