@@ -36,6 +36,8 @@
 #include "BLI_listbase.h"
 #include "BLI_utildefines.h"
 
+#include "BLF_translation.h"
+
 #include "DNA_armature_types.h"
 #include "DNA_lamp_types.h"
 #include "DNA_material_types.h"
@@ -63,15 +65,15 @@
 #include "UI_interface.h"
 #include "UI_resources.h"
 
-#include "buttons_intern.h"	// own include
+#include "buttons_intern.h" // own include
 
 static int set_pointer_type(ButsContextPath *path, bContextDataResult *result, StructRNA *type)
 {
 	PointerRNA *ptr;
 	int a;
 
-	for (a=0; a<path->len; a++) {
-		ptr= &path->ptr[a];
+	for (a = 0; a < path->len; a++) {
+		ptr = &path->ptr[a];
 
 		if (RNA_struct_is_a(ptr->type, type)) {
 			CTX_data_pointer_set(result, ptr->id.data, ptr->type, ptr->data);
@@ -87,8 +89,8 @@ static PointerRNA *get_pointer_type(ButsContextPath *path, StructRNA *type)
 	PointerRNA *ptr;
 	int a;
 
-	for (a=0; a<path->len; a++) {
-		ptr= &path->ptr[a];
+	for (a = 0; a < path->len; a++) {
+		ptr = &path->ptr[a];
 
 		if (RNA_struct_is_a(ptr->type, type))
 			return ptr;
@@ -101,7 +103,7 @@ static PointerRNA *get_pointer_type(ButsContextPath *path, StructRNA *type)
 
 static int buttons_context_path_scene(ButsContextPath *path)
 {
-	PointerRNA *ptr= &path->ptr[path->len-1];
+	PointerRNA *ptr = &path->ptr[path->len - 1];
 
 	/* this one just verifies */
 	return RNA_struct_is_a(ptr->type, &RNA_Scene);
@@ -113,7 +115,7 @@ static int buttons_context_path_world(ButsContextPath *path)
 {
 	Scene *scene;
 	World *world;
-	PointerRNA *ptr= &path->ptr[path->len-1];
+	PointerRNA *ptr = &path->ptr[path->len - 1];
 
 	/* if we already have a (pinned) world, we're done */
 	if (RNA_struct_is_a(ptr->type, &RNA_World)) {
@@ -121,8 +123,8 @@ static int buttons_context_path_world(ButsContextPath *path)
 	}
 	/* if we have a scene, use the scene's world */
 	else if (buttons_context_path_scene(path)) {
-		scene= path->ptr[path->len-1].data;
-		world= scene->world;
+		scene = path->ptr[path->len - 1].data;
+		world = scene->world;
 		
 		if (world) {
 			RNA_id_pointer_create(&scene->world->id, &path->ptr[path->len]);
@@ -143,7 +145,7 @@ static int buttons_context_path_object(ButsContextPath *path)
 {
 	Scene *scene;
 	Object *ob;
-	PointerRNA *ptr= &path->ptr[path->len-1];
+	PointerRNA *ptr = &path->ptr[path->len - 1];
 
 	/* if we already have a (pinned) object, we're done */
 	if (RNA_struct_is_a(ptr->type, &RNA_Object)) {
@@ -151,8 +153,8 @@ static int buttons_context_path_object(ButsContextPath *path)
 	}
 	/* if we have a scene, use the scene's active object */
 	else if (buttons_context_path_scene(path)) {
-		scene= path->ptr[path->len-1].data;
-		ob= (scene->basact)? scene->basact->object: NULL;
+		scene = path->ptr[path->len - 1].data;
+		ob = (scene->basact) ? scene->basact->object : NULL;
 
 		if (ob) {
 			RNA_id_pointer_create(&ob->id, &path->ptr[path->len]);
@@ -169,7 +171,7 @@ static int buttons_context_path_object(ButsContextPath *path)
 static int buttons_context_path_data(ButsContextPath *path, int type)
 {
 	Object *ob;
-	PointerRNA *ptr= &path->ptr[path->len-1];
+	PointerRNA *ptr = &path->ptr[path->len - 1];
 
 	/* if we already have a data, we're done */
 	if (RNA_struct_is_a(ptr->type, &RNA_Mesh) && (type == -1 || type == OB_MESH)) return 1;
@@ -182,7 +184,7 @@ static int buttons_context_path_data(ButsContextPath *path, int type)
 	else if (RNA_struct_is_a(ptr->type, &RNA_Speaker) && (type == -1 || type == OB_SPEAKER)) return 1;
 	/* try to get an object in the path, no pinning supported here */
 	else if (buttons_context_path_object(path)) {
-		ob= path->ptr[path->len-1].data;
+		ob = path->ptr[path->len - 1].data;
 
 		if (ob && (type == -1 || type == ob->type)) {
 			RNA_id_pointer_create(ob->data, &path->ptr[path->len]);
@@ -201,7 +203,7 @@ static int buttons_context_path_modifier(ButsContextPath *path)
 	Object *ob;
 
 	if (buttons_context_path_object(path)) {
-		ob= path->ptr[path->len-1].data;
+		ob = path->ptr[path->len - 1].data;
 
 		if (ob && ELEM5(ob->type, OB_MESH, OB_CURVE, OB_FONT, OB_SURF, OB_LATTICE))
 			return 1;
@@ -213,7 +215,7 @@ static int buttons_context_path_modifier(ButsContextPath *path)
 static int buttons_context_path_material(ButsContextPath *path, int for_texture)
 {
 	Object *ob;
-	PointerRNA *ptr= &path->ptr[path->len-1];
+	PointerRNA *ptr = &path->ptr[path->len - 1];
 	Material *ma;
 
 	/* if we already have a (pinned) material, we're done */
@@ -222,21 +224,21 @@ static int buttons_context_path_material(ButsContextPath *path, int for_texture)
 	}
 	/* if we have an object, use the object material slot */
 	else if (buttons_context_path_object(path)) {
-		ob= path->ptr[path->len-1].data;
+		ob = path->ptr[path->len - 1].data;
 
 		if (ob && OB_TYPE_SUPPORT_MATERIAL(ob->type)) {
-			ma= give_current_material(ob, ob->actcol);
+			ma = give_current_material(ob, ob->actcol);
 			RNA_id_pointer_create(&ma->id, &path->ptr[path->len]);
 			path->len++;
 
 			if (for_texture && give_current_material_texture_node(ma))
 				return 1;
 			
-			ma= give_node_material(ma);
+			ma = give_node_material(ma);
 			if (ma) {
 				RNA_id_pointer_create(&ma->id, &path->ptr[path->len]);
 				path->len++;
-			}			
+			}
 			return 1;
 		}
 	}
@@ -252,11 +254,11 @@ static int buttons_context_path_bone(ButsContextPath *path)
 
 	/* if we have an armature, get the active bone */
 	if (buttons_context_path_data(path, OB_ARMATURE)) {
-		arm= path->ptr[path->len-1].data;
+		arm = path->ptr[path->len - 1].data;
 
 		if (arm->edbo) {
 			if (arm->act_edbone) {
-				edbo= arm->act_edbone;
+				edbo = arm->act_edbone;
 				RNA_pointer_create(&arm->id, &RNA_EditBone, edbo, &path->ptr[path->len]);
 				path->len++;
 				return 1;
@@ -277,7 +279,7 @@ static int buttons_context_path_bone(ButsContextPath *path)
 
 static int buttons_context_path_pose_bone(ButsContextPath *path)
 {
-	PointerRNA *ptr= &path->ptr[path->len-1];
+	PointerRNA *ptr = &path->ptr[path->len - 1];
 
 	/* if we already have a (pinned) PoseBone, we're done */
 	if (RNA_struct_is_a(ptr->type, &RNA_PoseBone)) {
@@ -286,15 +288,15 @@ static int buttons_context_path_pose_bone(ButsContextPath *path)
 
 	/* if we have an armature, get the active bone */
 	if (buttons_context_path_object(path)) {
-		Object *ob= path->ptr[path->len-1].data;
-		bArmature *arm= ob->data; /* path->ptr[path->len-1].data - works too */
+		Object *ob = path->ptr[path->len - 1].data;
+		bArmature *arm = ob->data; /* path->ptr[path->len-1].data - works too */
 
 		if (ob->type != OB_ARMATURE || arm->edbo) {
 			return 0;
 		}
 		else {
 			if (arm->act_bone) {
-				bPoseChannel *pchan= get_pose_channel(ob->pose, arm->act_bone->name);
+				bPoseChannel *pchan = BKE_pose_channel_find_name(ob->pose, arm->act_bone->name);
 				if (pchan) {
 					RNA_pointer_create(&ob->id, &RNA_PoseBone, pchan, &path->ptr[path->len]);
 					path->len++;
@@ -313,7 +315,7 @@ static int buttons_context_path_particle(ButsContextPath *path)
 {
 	Object *ob;
 	ParticleSystem *psys;
-	PointerRNA *ptr= &path->ptr[path->len-1];
+	PointerRNA *ptr = &path->ptr[path->len - 1];
 
 	/* if we already have (pinned) particle settings, we're done */
 	if (RNA_struct_is_a(ptr->type, &RNA_ParticleSettings)) {
@@ -321,10 +323,10 @@ static int buttons_context_path_particle(ButsContextPath *path)
 	}
 	/* if we have an object, get the active particle system */
 	if (buttons_context_path_object(path)) {
-		ob= path->ptr[path->len-1].data;
+		ob = path->ptr[path->len - 1].data;
 
 		if (ob && ob->type == OB_MESH) {
-			psys= psys_get_current(ob);
+			psys = psys_get_current(ob);
 
 			RNA_pointer_create(&ob->id, &RNA_ParticleSystem, psys, &path->ptr[path->len]);
 			path->len++;
@@ -339,8 +341,8 @@ static int buttons_context_path_particle(ButsContextPath *path)
 static int buttons_context_path_brush(ButsContextPath *path)
 {
 	Scene *scene;
-	Brush *br= NULL;
-	PointerRNA *ptr= &path->ptr[path->len-1];
+	Brush *br = NULL;
+	PointerRNA *ptr = &path->ptr[path->len - 1];
 
 	/* if we already have a (pinned) brush, we're done */
 	if (RNA_struct_is_a(ptr->type, &RNA_Brush)) {
@@ -348,10 +350,10 @@ static int buttons_context_path_brush(ButsContextPath *path)
 	}
 	/* if we have a scene, use the toolsettings brushes */
 	else if (buttons_context_path_scene(path)) {
-		scene= path->ptr[path->len-1].data;
+		scene = path->ptr[path->len - 1].data;
 
 		if (scene)
-			br= paint_brush(paint_get_active(scene));
+			br = paint_brush(paint_get_active(scene));
 
 		if (br) {
 			RNA_id_pointer_create((ID *)br, &path->ptr[path->len]);
@@ -369,7 +371,7 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 {
 	if (ct) {
 		/* new shading system */
-		PointerRNA *ptr= &path->ptr[path->len-1];
+		PointerRNA *ptr = &path->ptr[path->len - 1];
 		ID *id;
 
 		/* if we already have a (pinned) texture, we're done */
@@ -379,7 +381,7 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 		if (!ct->user)
 			return 0;
 		
-		id= ct->user->id;
+		id = ct->user->id;
 
 		if (id) {
 			if (GS(id->name) == ID_BR)
@@ -411,7 +413,7 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 		World *wo;
 		ParticleSystem *psys;
 		Tex *tex;
-		PointerRNA *ptr= &path->ptr[path->len-1];
+		PointerRNA *ptr = &path->ptr[path->len - 1];
 		int orig_len = path->len;
 
 		/* if we already have a (pinned) texture, we're done */
@@ -420,10 +422,10 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 		}
 		/* try brush */
 		if ((path->tex_ctx == SB_TEXC_BRUSH) && buttons_context_path_brush(path)) {
-			br= path->ptr[path->len-1].data;
+			br = path->ptr[path->len - 1].data;
 			
 			if (br) {
-				tex= give_current_brush_texture(br);
+				tex = give_current_brush_texture(br);
 
 				RNA_id_pointer_create(&tex->id, &path->ptr[path->len]);
 				path->len++;
@@ -432,10 +434,10 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 		}
 		/* try world */
 		if ((path->tex_ctx == SB_TEXC_WORLD) && buttons_context_path_world(path)) {
-			wo= path->ptr[path->len-1].data;
+			wo = path->ptr[path->len - 1].data;
 
-			if (wo && GS(wo->id.name)==ID_WO) {
-				tex= give_current_world_texture(wo);
+			if (wo && GS(wo->id.name) == ID_WO) {
+				tex = give_current_world_texture(wo);
 
 				RNA_id_pointer_create(&tex->id, &path->ptr[path->len]);
 				path->len++;
@@ -444,19 +446,19 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 		}
 		/* try particles */
 		if ((path->tex_ctx == SB_TEXC_PARTICLES) && buttons_context_path_particle(path)) {
-			if (path->ptr[path->len-1].type == &RNA_ParticleSettings) {
-				ParticleSettings *part = path->ptr[path->len-1].data;
+			if (path->ptr[path->len - 1].type == &RNA_ParticleSettings) {
+				ParticleSettings *part = path->ptr[path->len - 1].data;
 
-				tex= give_current_particle_texture(part);
+				tex = give_current_particle_texture(part);
 				RNA_id_pointer_create(&tex->id, &path->ptr[path->len]);
 				path->len++;
 				return 1;
 			}
 			else {
-				psys= path->ptr[path->len-1].data;
+				psys = path->ptr[path->len - 1].data;
 
-				if (psys && psys->part && GS(psys->part->id.name)==ID_PA) {
-					tex= give_current_particle_texture(psys->part);
+				if (psys && psys->part && GS(psys->part->id.name) == ID_PA) {
+					tex = give_current_particle_texture(psys->part);
 
 					RNA_id_pointer_create(&tex->id, &path->ptr[path->len]);
 					path->len++;
@@ -466,10 +468,10 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 		}
 		/* try material */
 		if (buttons_context_path_material(path, 1)) {
-			ma= path->ptr[path->len-1].data;
+			ma = path->ptr[path->len - 1].data;
 
 			if (ma) {
-				tex= give_current_material_texture(ma);
+				tex = give_current_material_texture(ma);
 
 				RNA_id_pointer_create(&tex->id, &path->ptr[path->len]);
 				path->len++;
@@ -478,10 +480,10 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 		}
 		/* try lamp */
 		if (buttons_context_path_data(path, OB_LAMP)) {
-			la= path->ptr[path->len-1].data;
+			la = path->ptr[path->len - 1].data;
 
 			if (la) {
-				tex= give_current_lamp_texture(la);
+				tex = give_current_lamp_texture(la);
 
 				RNA_id_pointer_create(&tex->id, &path->ptr[path->len]);
 				path->len++;
@@ -491,10 +493,10 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 		/* try brushes again in case of no material, lamp, etc */
 		path->len = orig_len;
 		if (buttons_context_path_brush(path)) {
-			br= path->ptr[path->len-1].data;
+			br = path->ptr[path->len - 1].data;
 			
 			if (br) {
-				tex= give_current_brush_texture(br);
+				tex = give_current_brush_texture(br);
 				
 				RNA_id_pointer_create(&tex->id, &path->ptr[path->len]);
 				path->len++;
@@ -510,17 +512,17 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 
 static int buttons_context_path(const bContext *C, ButsContextPath *path, int mainb, int flag)
 {
-	SpaceButs *sbuts= CTX_wm_space_buts(C);
+	SpaceButs *sbuts = CTX_wm_space_buts(C);
 	ID *id;
 	int found;
 
 	memset(path, 0, sizeof(*path));
-	path->flag= flag;
+	path->flag = flag;
 	path->tex_ctx = sbuts->texture_context;
 
 	/* if some ID datablock is pinned, set the root pointer */
 	if (sbuts->pinid) {
-		id= sbuts->pinid;
+		id = sbuts->pinid;
 
 		RNA_id_pointer_create(id, &path->ptr[0]);
 		path->len++;
@@ -528,51 +530,51 @@ static int buttons_context_path(const bContext *C, ButsContextPath *path, int ma
 
 	/* no pinned root, use scene as root */
 	if (path->len == 0) {
-		id= (ID*)CTX_data_scene(C);
+		id = (ID *)CTX_data_scene(C);
 		RNA_id_pointer_create(id, &path->ptr[0]);
 		path->len++;
 	}
 
 	/* now for each buttons context type, we try to construct a path,
 	 * tracing back recursively */
-	switch(mainb) {
+	switch (mainb) {
 		case BCONTEXT_SCENE:
 		case BCONTEXT_RENDER:
-			found= buttons_context_path_scene(path);
+			found = buttons_context_path_scene(path);
 			break;
 		case BCONTEXT_WORLD:
-			found= buttons_context_path_world(path);
+			found = buttons_context_path_world(path);
 			break;
 		case BCONTEXT_OBJECT:
 		case BCONTEXT_PHYSICS:
 		case BCONTEXT_CONSTRAINT:
-			found= buttons_context_path_object(path);
+			found = buttons_context_path_object(path);
 			break;
 		case BCONTEXT_MODIFIER:
-			found= buttons_context_path_modifier(path);
+			found = buttons_context_path_modifier(path);
 			break;
 		case BCONTEXT_DATA:
-			found= buttons_context_path_data(path, -1);
+			found = buttons_context_path_data(path, -1);
 			break;
 		case BCONTEXT_PARTICLE:
-			found= buttons_context_path_particle(path);
+			found = buttons_context_path_particle(path);
 			break;
 		case BCONTEXT_MATERIAL:
-			found= buttons_context_path_material(path, 0);
+			found = buttons_context_path_material(path, 0);
 			break;
 		case BCONTEXT_TEXTURE:
-			found= buttons_context_path_texture(path, sbuts->texuser);
+			found = buttons_context_path_texture(path, sbuts->texuser);
 			break;
 		case BCONTEXT_BONE:
-			found= buttons_context_path_bone(path);
+			found = buttons_context_path_bone(path);
 			if (!found)
-				found= buttons_context_path_data(path, OB_ARMATURE);
+				found = buttons_context_path_data(path, OB_ARMATURE);
 			break;
 		case BCONTEXT_BONE_CONSTRAINT:
-			found= buttons_context_path_pose_bone(path);
+			found = buttons_context_path_pose_bone(path);
 			break;
 		default:
-			found= 0;
+			found = 0;
 			break;
 	}
 
@@ -581,7 +583,7 @@ static int buttons_context_path(const bContext *C, ButsContextPath *path, int ma
 
 static int buttons_shading_context(const bContext *C, int mainb)
 {
-	Object *ob= CTX_data_active_object(C);
+	Object *ob = CTX_data_active_object(C);
 
 	if (ELEM3(mainb, BCONTEXT_MATERIAL, BCONTEXT_WORLD, BCONTEXT_TEXTURE))
 		return 1;
@@ -593,7 +595,7 @@ static int buttons_shading_context(const bContext *C, int mainb)
 
 static int buttons_shading_new_context(const bContext *C, int flag)
 {
-	Object *ob= CTX_data_active_object(C);
+	Object *ob = CTX_data_active_object(C);
 
 	if (flag & (1 << BCONTEXT_MATERIAL))
 		return BCONTEXT_MATERIAL;
@@ -609,29 +611,29 @@ void buttons_context_compute(const bContext *C, SpaceButs *sbuts)
 {
 	ButsContextPath *path;
 	PointerRNA *ptr;
-	int a, pflag= 0, flag= 0;
+	int a, pflag = 0, flag = 0;
 
 	buttons_texture_context_compute(C, sbuts);
 
 	if (!sbuts->path)
-		sbuts->path= MEM_callocN(sizeof(ButsContextPath), "ButsContextPath");
+		sbuts->path = MEM_callocN(sizeof(ButsContextPath), "ButsContextPath");
 	
-	path= sbuts->path;
+	path = sbuts->path;
 	
 	/* for each context, see if we can compute a valid path to it, if
 	 * this is the case, we know we have to display the button */
-	for (a=0; a<BCONTEXT_TOT; a++) {
+	for (a = 0; a < BCONTEXT_TOT; a++) {
 		if (buttons_context_path(C, path, a, pflag)) {
-			flag |= (1<<a);
+			flag |= (1 << a);
 
 			/* setting icon for data context */
 			if (a == BCONTEXT_DATA) {
-				ptr= &path->ptr[path->len-1];
+				ptr = &path->ptr[path->len - 1];
 
 				if (ptr->type)
-					sbuts->dataicon= RNA_struct_ui_icon(ptr->type);
+					sbuts->dataicon = RNA_struct_ui_icon(ptr->type);
 				else
-					sbuts->dataicon= ICON_EMPTY_DATA;
+					sbuts->dataicon = ICON_EMPTY_DATA;
 			}
 		}
 	}
@@ -639,21 +641,21 @@ void buttons_context_compute(const bContext *C, SpaceButs *sbuts)
 	/* always try to use the tab that was explicitly
 	 * set to the user, so that once that context comes
 	 * back, the tab is activated again */
-	sbuts->mainb= sbuts->mainbuser;
+	sbuts->mainb = sbuts->mainbuser;
 
 	/* in case something becomes invalid, change */
 	if ((flag & (1 << sbuts->mainb)) == 0) {
 		if (sbuts->flag & SB_SHADING_CONTEXT) {
 			/* try to keep showing shading related buttons */
-			sbuts->mainb= buttons_shading_new_context(C, flag);
+			sbuts->mainb = buttons_shading_new_context(C, flag);
 		}
 		else if (flag & BCONTEXT_OBJECT) {
-			sbuts->mainb= BCONTEXT_OBJECT;
+			sbuts->mainb = BCONTEXT_OBJECT;
 		}
 		else {
-			for (a=0; a<BCONTEXT_TOT; a++) {
+			for (a = 0; a < BCONTEXT_TOT; a++) {
 				if (flag & (1 << a)) {
-					sbuts->mainb= a;
+					sbuts->mainb = a;
 					break;
 				}
 			}
@@ -664,9 +666,9 @@ void buttons_context_compute(const bContext *C, SpaceButs *sbuts)
 
 	if (!(flag & (1 << sbuts->mainb))) {
 		if (flag & (1 << BCONTEXT_OBJECT))
-			sbuts->mainb= BCONTEXT_OBJECT;
+			sbuts->mainb = BCONTEXT_OBJECT;
 		else
-			sbuts->mainb= BCONTEXT_SCENE;
+			sbuts->mainb = BCONTEXT_SCENE;
 	}
 
 	if (buttons_shading_context(C, sbuts->mainb))
@@ -674,29 +676,35 @@ void buttons_context_compute(const bContext *C, SpaceButs *sbuts)
 	else
 		sbuts->flag &= ~SB_SHADING_CONTEXT;
 
-	sbuts->pathflag= flag;
+	sbuts->pathflag = flag;
 }
 
 /************************* Context Callback ************************/
 
 const char *buttons_context_dir[] = {
-	"world", "object", "mesh", "armature", "lattice", "curve",
+	"texture_slot", "world", "object", "mesh", "armature", "lattice", "curve",
 	"meta_ball", "lamp", "speaker", "camera", "material", "material_slot",
-	"texture", "texture_slot", "texture_user", "bone", "edit_bone",
-	"pose_bone", "particle_system", "particle_system_editable",
-	"cloth", "soft_body", "fluid", "smoke", "collision", "brush", "dynamic_paint", NULL};
+	"texture", "texture_user", "bone", "edit_bone",
+	"pose_bone", "particle_system", "particle_system_editable", "particle_settings",
+	"cloth", "soft_body", "fluid", "smoke", "collision", "brush", "dynamic_paint", NULL
+};
 
 int buttons_context(const bContext *C, const char *member, bContextDataResult *result)
 {
-	SpaceButs *sbuts= CTX_wm_space_buts(C);
-	ButsContextPath *path= sbuts?sbuts->path:NULL;
+	SpaceButs *sbuts = CTX_wm_space_buts(C);
+	ButsContextPath *path = sbuts ? sbuts->path : NULL;
 
 	if (!path)
 		return 0;
 
 	/* here we handle context, getting data from precomputed path */
 	if (CTX_data_dir(member)) {
-		CTX_data_dir_set(result, buttons_context_dir);
+		/* in case of new shading system we skip texture_slot, complex python
+		 * UI script logic depends on checking if this is available */
+		if (sbuts->texuser)
+			CTX_data_dir_set(result, buttons_context_dir + 1);
+		else
+			CTX_data_dir_set(result, buttons_context_dir);
 		return 1;
 	}
 	else if (CTX_data_equals(member, "world")) {
@@ -744,7 +752,7 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 		return 1;
 	}
 	else if (CTX_data_equals(member, "texture")) {
-		ButsContextTexture *ct= sbuts->texuser;
+		ButsContextTexture *ct = sbuts->texuser;
 
 		if (ct) {
 			/* new shading system */
@@ -758,15 +766,15 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 		return 1;
 	}
 	else if (CTX_data_equals(member, "material_slot")) {
-		PointerRNA *ptr= get_pointer_type(path, &RNA_Object);
+		PointerRNA *ptr = get_pointer_type(path, &RNA_Object);
 
 		if (ptr) {
-			Object *ob= ptr->data;
+			Object *ob = ptr->data;
 
 			if (ob && OB_TYPE_SUPPORT_MATERIAL(ob->type) && ob->totcol) {
 				/* a valid actcol isn't ensured [#27526] */
-				int matnr= ob->actcol-1;
-				if (matnr < 0) matnr= 0;
+				int matnr = ob->actcol - 1;
+				if (matnr < 0) matnr = 0;
 				CTX_data_pointer_set(result, &ob->id, &RNA_MaterialSlot, &ob->mat[matnr]);
 			}
 		}
@@ -774,20 +782,20 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 		return 1;
 	}
 	else if (CTX_data_equals(member, "texture_user")) {
-		ButsContextTexture *ct= sbuts->texuser;
+		ButsContextTexture *ct = sbuts->texuser;
 
 		if (!ct)
-			return -1; /* old shading system (found but not available) */
+			return -1;  /* old shading system (found but not available) */
 
 		if (ct->user && ct->user->ptr.data) {
-			ButsTextureUser *user= ct->user; 
+			ButsTextureUser *user = ct->user;
 			CTX_data_pointer_set(result, user->ptr.id.data, user->ptr.type, user->ptr.data);
 		}
 
 		return 1;
 	}
 	else if (CTX_data_equals(member, "texture_node")) {
-		ButsContextTexture *ct= sbuts->texuser;
+		ButsContextTexture *ct = sbuts->texuser;
 
 		if (ct) {
 			/* new shading system */
@@ -800,11 +808,11 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 			/* old shading system */
 			PointerRNA *ptr;
 
-			if ((ptr=get_pointer_type(path, &RNA_Material))) {
-				Material *ma= ptr->data;
+			if ((ptr = get_pointer_type(path, &RNA_Material))) {
+				Material *ma = ptr->data;
 
 				if (ma) {
-					bNode *node= give_current_material_texture_node(ma);
+					bNode *node = give_current_material_texture_node(ma);
 					CTX_data_pointer_set(result, &ma->nodetree->id, &RNA_Node, node);
 				}
 			}
@@ -813,14 +821,14 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 		}
 	}
 	else if (CTX_data_equals(member, "texture_slot")) {
-		ButsContextTexture *ct= sbuts->texuser;
+		ButsContextTexture *ct = sbuts->texuser;
 		PointerRNA *ptr;
 
-		if (ct)
-			return 0; /* new shading system */
+		if ((ptr = get_pointer_type(path, &RNA_Material))) {
+			Material *ma = ptr->data;
 
-		if ((ptr=get_pointer_type(path, &RNA_Material))) {
-			Material *ma= ptr->data;
+			if (ct)
+				return 0;  /* new shading system */
 
 			/* if we have a node material, get slot from material in material node */
 			if (ma && ma->use_nodes && ma->nodetree) {
@@ -829,7 +837,7 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 				if (give_current_material_texture_node(ma))
 					return 0;
 				
-				ma= give_node_material(ma);
+				ma = give_node_material(ma);
 				if (ma)
 					CTX_data_pointer_set(result, &ma->id, &RNA_MaterialTextureSlot, ma->mtex[(int)ma->texact]);
 				else
@@ -839,26 +847,32 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 				CTX_data_pointer_set(result, &ma->id, &RNA_MaterialTextureSlot, ma->mtex[(int)ma->texact]);
 			}
 		}
-		else if ((ptr=get_pointer_type(path, &RNA_Lamp))) {
-			Lamp *la= ptr->data;
+		else if ((ptr = get_pointer_type(path, &RNA_Lamp))) {
+			Lamp *la = ptr->data;
+
+			if (ct)
+				return 0;  /* new shading system */
 
 			if (la)
 				CTX_data_pointer_set(result, &la->id, &RNA_LampTextureSlot, la->mtex[(int)la->texact]);
 		}
-		else if ((ptr=get_pointer_type(path, &RNA_World))) {
-			World *wo= ptr->data;
+		else if ((ptr = get_pointer_type(path, &RNA_World))) {
+			World *wo = ptr->data;
+
+			if (ct)
+				return 0;  /* new shading system */
 
 			if (wo)
 				CTX_data_pointer_set(result, &wo->id, &RNA_WorldTextureSlot, wo->mtex[(int)wo->texact]);
 		}
-		else if ((ptr=get_pointer_type(path, &RNA_Brush))) { /* how to get this into context? */
-			Brush *br= ptr->data;
+		else if ((ptr = get_pointer_type(path, &RNA_Brush))) { /* how to get this into context? */
+			Brush *br = ptr->data;
 
 			if (br)
 				CTX_data_pointer_set(result, &br->id, &RNA_BrushTextureSlot, &br->mtex);
 		}
-		else if ((ptr=get_pointer_type(path, &RNA_ParticleSystem))) {
-			ParticleSettings *part= ((ParticleSystem *)ptr->data)->part;
+		else if ((ptr = get_pointer_type(path, &RNA_ParticleSystem))) {
+			ParticleSettings *part = ((ParticleSystem *)ptr->data)->part;
 
 			if (part)
 				CTX_data_pointer_set(result, &part->id, &RNA_ParticleSettingsTextureSlot, part->mtex[(int)part->texact]);
@@ -883,59 +897,80 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 		return 1;
 	}
 	else if (CTX_data_equals(member, "particle_system_editable")) {
-		if (PE_poll((bContext*)C))
+		if (PE_poll((bContext *)C))
 			set_pointer_type(path, result, &RNA_ParticleSystem);
 		else
 			CTX_data_pointer_set(result, NULL, &RNA_ParticleSystem, NULL);
 		return 1;
-	}	
+	}
+	else if (CTX_data_equals(member, "particle_settings")) {
+		/* only available when pinned */
+		PointerRNA *ptr = get_pointer_type(path, &RNA_ParticleSettings);
+		
+		if (ptr && ptr->data) {
+			CTX_data_pointer_set(result, ptr->id.data, &RNA_ParticleSettings, ptr->data);
+			return 1;
+		}
+		else {
+			/* get settings from active particle system instead */
+			ptr = get_pointer_type(path, &RNA_ParticleSystem);
+			
+			if (ptr && ptr->data) {
+				ParticleSettings *part = ((ParticleSystem *)ptr->data)->part;
+				CTX_data_pointer_set(result, ptr->id.data, &RNA_ParticleSettings, part);
+				return 1;
+			}
+		}
+		set_pointer_type(path, result, &RNA_ParticleSettings);
+		return 1;
+	}
 	else if (CTX_data_equals(member, "cloth")) {
-		PointerRNA *ptr= get_pointer_type(path, &RNA_Object);
+		PointerRNA *ptr = get_pointer_type(path, &RNA_Object);
 
 		if (ptr && ptr->data) {
-			Object *ob= ptr->data;
-			ModifierData *md= modifiers_findByType(ob, eModifierType_Cloth);
+			Object *ob = ptr->data;
+			ModifierData *md = modifiers_findByType(ob, eModifierType_Cloth);
 			CTX_data_pointer_set(result, &ob->id, &RNA_ClothModifier, md);
 			return 1;
 		}
 	}
 	else if (CTX_data_equals(member, "soft_body")) {
-		PointerRNA *ptr= get_pointer_type(path, &RNA_Object);
+		PointerRNA *ptr = get_pointer_type(path, &RNA_Object);
 
 		if (ptr && ptr->data) {
-			Object *ob= ptr->data;
-			ModifierData *md= modifiers_findByType(ob, eModifierType_Softbody);
+			Object *ob = ptr->data;
+			ModifierData *md = modifiers_findByType(ob, eModifierType_Softbody);
 			CTX_data_pointer_set(result, &ob->id, &RNA_SoftBodyModifier, md);
 			return 1;
 		}
 	}
 	else if (CTX_data_equals(member, "fluid")) {
-		PointerRNA *ptr= get_pointer_type(path, &RNA_Object);
+		PointerRNA *ptr = get_pointer_type(path, &RNA_Object);
 
 		if (ptr && ptr->data) {
-			Object *ob= ptr->data;
-			ModifierData *md= modifiers_findByType(ob, eModifierType_Fluidsim);
+			Object *ob = ptr->data;
+			ModifierData *md = modifiers_findByType(ob, eModifierType_Fluidsim);
 			CTX_data_pointer_set(result, &ob->id, &RNA_FluidSimulationModifier, md);
 			return 1;
 		}
 	}
 	
 	else if (CTX_data_equals(member, "smoke")) {
-		PointerRNA *ptr= get_pointer_type(path, &RNA_Object);
+		PointerRNA *ptr = get_pointer_type(path, &RNA_Object);
 
 		if (ptr && ptr->data) {
-			Object *ob= ptr->data;
-			ModifierData *md= modifiers_findByType(ob, eModifierType_Smoke);
+			Object *ob = ptr->data;
+			ModifierData *md = modifiers_findByType(ob, eModifierType_Smoke);
 			CTX_data_pointer_set(result, &ob->id, &RNA_SmokeModifier, md);
 			return 1;
 		}
 	}
 	else if (CTX_data_equals(member, "collision")) {
-		PointerRNA *ptr= get_pointer_type(path, &RNA_Object);
+		PointerRNA *ptr = get_pointer_type(path, &RNA_Object);
 
 		if (ptr && ptr->data) {
-			Object *ob= ptr->data;
-			ModifierData *md= modifiers_findByType(ob, eModifierType_Collision);
+			Object *ob = ptr->data;
+			ModifierData *md = modifiers_findByType(ob, eModifierType_Collision);
 			CTX_data_pointer_set(result, &ob->id, &RNA_CollisionModifier, md);
 			return 1;
 		}
@@ -945,11 +980,11 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 		return 1;
 	}
 	else if (CTX_data_equals(member, "dynamic_paint")) {
-		PointerRNA *ptr= get_pointer_type(path, &RNA_Object);
+		PointerRNA *ptr = get_pointer_type(path, &RNA_Object);
 
 		if (ptr && ptr->data) {
-			Object *ob= ptr->data;
-			ModifierData *md= modifiers_findByType(ob, eModifierType_DynamicPaint);
+			Object *ob = ptr->data;
+			ModifierData *md = modifiers_findByType(ob, eModifierType_DynamicPaint);
 			CTX_data_pointer_set(result, &ob->id, &RNA_DynamicPaintModifier, md);
 			return 1;
 		}
@@ -965,21 +1000,21 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 
 static void pin_cb(bContext *C, void *UNUSED(arg1), void *UNUSED(arg2))
 {
-	SpaceButs *sbuts= CTX_wm_space_buts(C);
+	SpaceButs *sbuts = CTX_wm_space_buts(C);
 
 	if (sbuts->flag & SB_PIN_CONTEXT) {
-		sbuts->pinid= buttons_context_id_path(C);
+		sbuts->pinid = buttons_context_id_path(C);
 	}
 	else
-		sbuts->pinid= NULL;
+		sbuts->pinid = NULL;
 	
 	ED_area_tag_redraw(CTX_wm_area(C));
 }
 
 void buttons_context_draw(const bContext *C, uiLayout *layout)
 {
-	SpaceButs *sbuts= CTX_wm_space_buts(C);
-	ButsContextPath *path= sbuts->path;
+	SpaceButs *sbuts = CTX_wm_space_buts(C);
+	ButsContextPath *path = sbuts->path;
 	uiLayout *row;
 	uiBlock *block;
 	uiBut *but;
@@ -990,28 +1025,29 @@ void buttons_context_draw(const bContext *C, uiLayout *layout)
 	if (!path)
 		return;
 
-	row= uiLayoutRow(layout, 1);
+	row = uiLayoutRow(layout, TRUE);
 	uiLayoutSetAlignment(row, UI_LAYOUT_ALIGN_LEFT);
 
-	block= uiLayoutGetBlock(row);
+	block = uiLayoutGetBlock(row);
 	uiBlockSetEmboss(block, UI_EMBOSSN);
-	but = uiDefIconButBitC(block, ICONTOG, SB_PIN_CONTEXT, 0, ICON_UNPINNED, 0, 0, UI_UNIT_X, UI_UNIT_Y, &sbuts->flag, 0, 0, 0, 0, "Follow context or keep fixed datablock displayed");
+	but = uiDefIconButBitC(block, ICONTOG, SB_PIN_CONTEXT, 0, ICON_UNPINNED, 0, 0, UI_UNIT_X, UI_UNIT_Y, &sbuts->flag,
+	                       0, 0, 0, 0, IFACE_("Follow context or keep fixed datablock displayed"));
 	uiButClearFlag(but, UI_BUT_UNDO); /* skip undo on screen buttons */
 	uiButSetFunc(but, pin_cb, NULL, NULL);
 
-	for (a=0; a<path->len; a++) {
-		ptr= &path->ptr[a];
+	for (a = 0; a < path->len; a++) {
+		ptr = &path->ptr[a];
 
 		if (a != 0)
 			uiItemL(row, "", VICO_SMALL_TRI_RIGHT_VEC);
 
 		if (ptr->data) {
-			icon= RNA_struct_ui_icon(ptr->type);
-			name= RNA_struct_name_get_alloc(ptr, namebuf, sizeof(namebuf), NULL);
+			icon = RNA_struct_ui_icon(ptr->type);
+			name = RNA_struct_name_get_alloc(ptr, namebuf, sizeof(namebuf), NULL);
 
 			if (name) {
 				if (!ELEM(sbuts->mainb, BCONTEXT_RENDER, BCONTEXT_SCENE) && ptr->type == &RNA_Scene)
-					uiItemLDrag(row, ptr, "", icon); /* save some space */
+					uiItemLDrag(row, ptr, "", icon);  /* save some space */
 				else
 					uiItemLDrag(row, ptr, name, icon);
 								 
@@ -1033,24 +1069,24 @@ void buttons_context_register(ARegionType *art)
 {
 	PanelType *pt;
 
-	pt= MEM_callocN(sizeof(PanelType), "spacetype buttons panel context");
+	pt = MEM_callocN(sizeof(PanelType), "spacetype buttons panel context");
 	strcpy(pt->idname, "BUTTONS_PT_context");
-	strcpy(pt->label, "Context");
-	pt->draw= buttons_panel_context;
-	pt->flag= PNL_NO_HEADER;
+	strcpy(pt->label, N_("Context"));  /* XXX C panels are not available through RNA (bpy.types)! */
+	pt->draw = buttons_panel_context;
+	pt->flag = PNL_NO_HEADER;
 	BLI_addtail(&art->paneltypes, pt);
 }
 
 ID *buttons_context_id_path(const bContext *C)
 {
-	SpaceButs *sbuts= CTX_wm_space_buts(C);
-	ButsContextPath *path= sbuts->path;
+	SpaceButs *sbuts = CTX_wm_space_buts(C);
+	ButsContextPath *path = sbuts->path;
 	PointerRNA *ptr;
 	int a;
 
 	if (path->len) {
-		for (a=path->len-1; a>=0; a--) {
-			ptr= &path->ptr[a];
+		for (a = path->len - 1; a >= 0; a--) {
+			ptr = &path->ptr[a];
 
 			/* pin particle settings instead of system, since only settings are an idblock*/
 			if (sbuts->mainb == BCONTEXT_PARTICLE && sbuts->flag & SB_PIN_CONTEXT) {
