@@ -33,11 +33,15 @@
 
 #include "DNA_windowmanager_types.h"
 
+#include "RNA_access.h"
+
 #include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
 
+#include "BLF_translation.h"
+
 #include "BKE_context.h"
 #include "BKE_screen.h"
 
@@ -56,7 +60,7 @@
 
 /************************** properties ******************************/
 
-static ARegion *clip_has_properties_region(ScrArea *sa)
+ARegion *ED_clip_has_properties_region(ScrArea *sa)
 {
 	ARegion *ar, *arnew;
 
@@ -90,9 +94,9 @@ static int properties_poll(bContext *C)
 static int properties_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	ScrArea *sa = CTX_wm_area(C);
-	ARegion *ar = clip_has_properties_region(sa);
+	ARegion *ar = ED_clip_has_properties_region(sa);
 
-	if (ar)
+	if (ar && ar->alignment != RGN_ALIGN_NONE)
 		ED_region_toggle_hidden(C, ar);
 
 	return OPERATOR_FINISHED;
@@ -151,7 +155,7 @@ static ARegion *clip_has_tools_region(ScrArea *sa)
 
 		BLI_insertlinkafter(&sa->regionbase, artool, arprops);
 		arprops->regiontype = RGN_TYPE_TOOL_PROPS;
-		arprops->alignment = RGN_ALIGN_BOTTOM|RGN_SPLIT_PREV;
+		arprops->alignment = RGN_ALIGN_BOTTOM | RGN_SPLIT_PREV;
 	}
 
 	return artool;
@@ -167,7 +171,7 @@ static int tools_exec(bContext *C, wmOperator *UNUSED(op))
 	ScrArea *sa = CTX_wm_area(C);
 	ARegion *ar = clip_has_tools_region(sa);
 
-	if (ar)
+	if (ar && ar->alignment != RGN_ALIGN_NONE)
 		ED_region_toggle_hidden(C, ar);
 
 	return OPERATOR_FINISHED;
@@ -197,16 +201,16 @@ static void clip_panel_operator_redo_header(const bContext *C, Panel *pa)
 	wmOperator *op = WM_operator_last_redo(C);
 
 	if (op)
-		BLI_strncpy(pa->drawname, op->type->name, sizeof(pa->drawname));
+		BLI_strncpy(pa->drawname, RNA_struct_ui_name(op->type->srna), sizeof(pa->drawname));
 	else
-		BLI_strncpy(pa->drawname, "Operator", sizeof(pa->drawname));
+		BLI_strncpy(pa->drawname, IFACE_("Operator"), sizeof(pa->drawname));
 }
 
 static void clip_panel_operator_redo_operator(const bContext *C, Panel *pa, wmOperator *op)
 {
 	if (op->type->flag & OPTYPE_MACRO) {
 		for (op = op->macro.first; op; op = op->next) {
-			uiItemL(pa->layout, op->type->name, ICON_NONE);
+			uiItemL(pa->layout, RNA_struct_ui_name(op->type->srna), ICON_NONE);
 			clip_panel_operator_redo_operator(C, pa, op);
 		}
 	}
@@ -219,23 +223,32 @@ static void clip_panel_operator_redo_operator(const bContext *C, Panel *pa, wmOp
 static void clip_panel_operator_redo(const bContext *C, Panel *pa)
 {
 	wmOperator *op = WM_operator_last_redo(C);
-	uiBlock *block;
+	ARegion *ar;
+	ARegion *ar1;
 
 	if (op == NULL)
 		return;
 
-	if (WM_operator_poll((bContext*)C, op->type) == 0)
-		return;
+	/* keep in sync with logic in ED_undo_operator_repeat() */
+	ar = CTX_wm_region(C);
+	ar1 = BKE_area_find_region_type(CTX_wm_area(C), RGN_TYPE_WINDOW);
+	if (ar1)
+		CTX_wm_region_set((bContext *)C, ar1);
 
-	block = uiLayoutGetBlock(pa->layout);
+	if (WM_operator_poll((bContext *)C, op->type)) {
+		uiBlock *block = uiLayoutGetBlock(pa->layout);
 
-	if (!WM_operator_check_ui_enabled(C, op->type->name))
-		uiLayoutSetEnabled(pa->layout, 0);
+		if (!WM_operator_check_ui_enabled(C, op->type->name))
+			uiLayoutSetEnabled(pa->layout, FALSE);
 
-	/* note, blockfunc is a default but->func, use Handle func to allow button callbacks too */
-	uiBlockSetHandleFunc(block, ED_undo_operator_repeat_cb_evt, op);
+		/* note, blockfunc is a default but->func, use Handle func to allow button callbacks too */
+		uiBlockSetHandleFunc(block, ED_undo_operator_repeat_cb_evt, op);
 
-	clip_panel_operator_redo_operator(C, pa, op);
+		clip_panel_operator_redo_operator(C, pa, op);
+	}
+
+	/* set region back */
+	CTX_wm_region_set((bContext *)C, ar);
 }
 
 void ED_clip_tool_props_register(ARegionType *art)
