@@ -34,11 +34,13 @@
 
 #include "DNA_userdef_types.h"
 
+#include "BLI_utildefines.h"
 #include "BLI_listbase.h"
 #include "BLI_string_cursor_utf8.h"
+#include "BLI_string_utf8.h"
 #include "BLI_string.h"
 #include "BLI_dynstr.h"
-#include "BLI_utildefines.h"
+#include "BLI_math.h"
 
 #include "BKE_context.h"
 #include "BKE_text.h" /* only for character utility funcs */
@@ -156,10 +158,9 @@ static ConsoleLine *console_lb_add__internal(ListBase *lb, ConsoleLine *from)
 	ConsoleLine *ci = MEM_callocN(sizeof(ConsoleLine), "ConsoleLine Add");
 	
 	if (from) {
-		ci->line = BLI_strdup(from->line);
-		ci->len = strlen(ci->line);
-		ci->len_alloc = ci->len;
-		
+		BLI_assert(strlen(from->line) == from->len);
+		ci->line = BLI_strdupn(from->line, from->len);
+		ci->len = ci->len_alloc = from->len;
 		ci->cursor = from->cursor;
 		ci->type = from->type;
 	}
@@ -173,10 +174,8 @@ static ConsoleLine *console_lb_add__internal(ListBase *lb, ConsoleLine *from)
 	return ci;
 }
 
-static ConsoleLine *console_history_add(const bContext *C, ConsoleLine *from)
+static ConsoleLine *console_history_add(SpaceConsole *sc, ConsoleLine *from)
 {
-	SpaceConsole *sc = CTX_wm_space_console(C);
-	
 	return console_lb_add__internal(&sc->history, from);
 }
 
@@ -216,7 +215,7 @@ ConsoleLine *console_history_verify(const bContext *C)
 	SpaceConsole *sc = CTX_wm_space_console(C);
 	ConsoleLine *ci = sc->history.last;
 	if (ci == NULL)
-		ci = console_history_add(C, NULL);
+		ci = console_history_add(sc, NULL);
 	
 	return ci;
 }
@@ -277,7 +276,7 @@ static int console_move_exec(bContext *C, wmOperator *op)
 	ConsoleLine *ci = console_history_verify(C);
 	
 	int type = RNA_enum_get(op->ptr, "type");
-	int done = 0;
+	int done = FALSE;
 	int pos;
 	
 	switch (type) {
@@ -285,28 +284,28 @@ static int console_move_exec(bContext *C, wmOperator *op)
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_PREV,
-			                         STRCUR_JUMP_ALL);
+			                         STRCUR_JUMP_ALL, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 		case LINE_END:
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_NEXT,
-			                         STRCUR_JUMP_ALL);
+			                         STRCUR_JUMP_ALL, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 		case PREV_CHAR:
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_PREV,
-			                         STRCUR_JUMP_NONE);
+			                         STRCUR_JUMP_NONE, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 		case NEXT_CHAR:
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_NEXT,
-			                         STRCUR_JUMP_NONE);
+			                         STRCUR_JUMP_NONE, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 
@@ -316,14 +315,14 @@ static int console_move_exec(bContext *C, wmOperator *op)
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_PREV,
-			                         STRCUR_JUMP_DELIM);
+			                         STRCUR_JUMP_DELIM, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 		case NEXT_WORD:
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_NEXT,
-			                         STRCUR_JUMP_DELIM);
+			                         STRCUR_JUMP_DELIM, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 	}
@@ -364,9 +363,8 @@ static int console_insert_exec(bContext *C, wmOperator *op)
 	char *str = RNA_string_get_alloc(op->ptr, "text", NULL, 0);
 	int len;
 
-	// XXX, alligned tab key hack
 	if (str[0] == '\t' && str[1] == '\0') {
-		len = TAB_LENGTH - (ci->cursor % TAB_LENGTH);
+		len = TAB_LENGTH;
 		MEM_freeN(str);
 		str = MEM_mallocN(len + 1, "insert_exec");
 		memset(str, ' ', len);
@@ -396,15 +394,26 @@ static int console_insert_invoke(bContext *C, wmOperator *op, wmEvent *event)
 {
 	// if (!RNA_struct_property_is_set(op->ptr, "text")) { /* always set from keymap XXX */
 	if (!RNA_string_length(op->ptr, "text")) {
-		/* if alt/ctrl/super are pressed pass through */
-		if (event->ctrl || event->oskey) {
+		/* if alt/ctrl/super are pressed pass through except for utf8 character event
+		 * (when input method are used for utf8 inputs, the user may assign key event
+		 * including alt/ctrl/super like ctrl+m to commit utf8 string.  in such case,
+		 * the modifiers in the utf8 character event make no sense.) */
+		if ((event->ctrl || event->oskey) && !event->utf8_buf[0]) {
 			return OPERATOR_PASS_THROUGH;
 		}
 		else {
-			char str[2];
-			str[0] = event->ascii;
-			str[1] = '\0';
-
+			char str[BLI_UTF8_MAX + 1];
+			size_t len;
+			
+			if (event->utf8_buf[0]) {
+				len = BLI_str_utf8_size_safe(event->utf8_buf);
+				memcpy(str, event->utf8_buf, len);
+			}
+			else {
+				/* in theory, ghost can set value to extended ascii here */
+				len = BLI_str_utf8_from_unicode(event->ascii, str);
+			}
+			str[len] = '\0';
 			RNA_string_set(op->ptr, "text", str);
 		}
 	}
@@ -430,6 +439,98 @@ void CONSOLE_OT_insert(wmOperatorType *ot)
 	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
 }
 
+static int console_indent_exec(bContext *C, wmOperator *UNUSED(op))
+{
+	SpaceConsole *sc = CTX_wm_space_console(C);
+	ARegion *ar = CTX_wm_region(C);
+	ConsoleLine *ci = console_history_verify(C);
+	int spaces;
+	int len;
+
+	for (spaces = 0; spaces < ci->len; spaces++) {
+		if (ci->line[spaces] != ' ')
+			break;
+	}
+
+	len = TAB_LENGTH - spaces % TAB_LENGTH;
+
+	console_line_verify_length(ci, ci->len + len);
+
+	memmove(ci->line + len, ci->line, ci->len + 1);
+	memset(ci->line, ' ', len);
+	ci->len += len;
+	BLI_assert(ci->len >= 0);
+	console_line_cursor_set(ci, ci->cursor + len);
+	console_select_offset(sc, len);
+
+	console_textview_update_rect(sc, ar);
+	ED_area_tag_redraw(CTX_wm_area(C));
+
+	console_scroll_bottom(ar);
+
+	return OPERATOR_FINISHED;
+}
+
+void CONSOLE_OT_indent(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Indent";
+	ot->description = "Add 4 spaces at line beginning";
+	ot->idname = "CONSOLE_OT_indent";
+
+	/* api callbacks */
+	ot->exec = console_indent_exec;
+	ot->poll = ED_operator_console_active;
+}
+
+static int console_unindent_exec(bContext *C, wmOperator *UNUSED(op))
+{
+	SpaceConsole *sc = CTX_wm_space_console(C);
+	ARegion *ar = CTX_wm_region(C);
+	ConsoleLine *ci = console_history_verify(C);
+	int spaces;
+	int len;
+
+	for (spaces = 0; spaces < ci->len; spaces++) {
+		if (ci->line[spaces] != ' ')
+			break;
+	}
+
+	if (spaces == 0)
+		return OPERATOR_CANCELLED;
+
+	len = spaces % TAB_LENGTH;
+	if (len == 0)
+		len = TAB_LENGTH;
+
+	console_line_verify_length(ci, ci->len - len);
+
+	memmove(ci->line, ci->line + len, (ci->len - len) + 1);
+	ci->len -= len;
+	BLI_assert(ci->len >= 0);
+
+	console_line_cursor_set(ci, ci->cursor - len);
+	console_select_offset(sc, -len);
+
+	console_textview_update_rect(sc, ar);
+	ED_area_tag_redraw(CTX_wm_area(C));
+
+	console_scroll_bottom(ar);
+
+	return OPERATOR_FINISHED;
+}
+
+void CONSOLE_OT_unindent(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Unindent";
+	ot->description = "Delete 4 spaces from line beginning";
+	ot->idname = "CONSOLE_OT_unindent";
+
+	/* api callbacks */
+	ot->exec = console_unindent_exec;
+	ot->poll = ED_operator_console_active;
+}
 
 static EnumPropertyItem console_delete_type_items[] = {
 	{DEL_NEXT_CHAR, "NEXT_CHARACTER", 0, "Next Character", ""},
@@ -448,7 +549,7 @@ static int console_delete_exec(bContext *C, wmOperator *op)
 	int stride;
 
 	const short type = RNA_enum_get(op->ptr, "type");
-	int done = 0;
+	int done = FALSE;
 	
 	if (ci->len == 0) {
 		return OPERATOR_CANCELLED;
@@ -461,12 +562,13 @@ static int console_delete_exec(bContext *C, wmOperator *op)
 				pos = ci->cursor;
 				BLI_str_cursor_step_utf8(ci->line, ci->len,
 				                         &pos, STRCUR_DIR_NEXT,
-				                         (type == DEL_NEXT_CHAR) ? STRCUR_JUMP_NONE : STRCUR_JUMP_DELIM);
+				                         (type == DEL_NEXT_CHAR) ? STRCUR_JUMP_NONE : STRCUR_JUMP_DELIM, true);
 				stride = pos - ci->cursor;
 				if (stride) {
 					memmove(ci->line + ci->cursor, ci->line + ci->cursor + stride, (ci->len - ci->cursor) + 1);
 					ci->len -= stride;
-					done = 1;
+					BLI_assert(ci->len >= 0);
+					done = TRUE;
 				}
 			}
 			break;
@@ -476,13 +578,14 @@ static int console_delete_exec(bContext *C, wmOperator *op)
 				pos = ci->cursor;
 				BLI_str_cursor_step_utf8(ci->line, ci->len,
 				                         &pos, STRCUR_DIR_PREV,
-				                         (type == DEL_PREV_CHAR) ? STRCUR_JUMP_NONE : STRCUR_JUMP_DELIM);
+				                         (type == DEL_PREV_CHAR) ? STRCUR_JUMP_NONE : STRCUR_JUMP_DELIM, true);
 				stride = ci->cursor - pos;
 				if (stride) {
 					ci->cursor -= stride; /* same as above */
 					memmove(ci->line + ci->cursor, ci->line + ci->cursor + stride, (ci->len - ci->cursor) + 1);
 					ci->len -= stride;
-					done = 1;
+					BLI_assert(ci->len >= 0);
+					done = TRUE;
 				}
 			}
 			break;
@@ -492,7 +595,7 @@ static int console_delete_exec(bContext *C, wmOperator *op)
 		return OPERATOR_CANCELLED;
 	}
 	else {
-		console_select_offset(sc, -1);
+		console_select_offset(sc, -stride);
 	}
 
 	console_textview_update_rect(sc, ar);
@@ -519,6 +622,40 @@ void CONSOLE_OT_delete(wmOperatorType *ot)
 	RNA_def_enum(ot->srna, "type", console_delete_type_items, DEL_NEXT_CHAR, "Type", "Which part of the text to delete");
 }
 
+static int console_clear_line_exec(bContext *C, wmOperator *UNUSED(op))
+{
+	SpaceConsole *sc = CTX_wm_space_console(C);
+	ARegion *ar = CTX_wm_region(C);
+	ConsoleLine *ci = console_history_verify(C);
+
+	if (ci->len == 0) {
+		return OPERATOR_CANCELLED;
+	}
+
+	console_history_add(sc, ci);
+	console_history_add(sc, NULL);
+	console_select_offset(sc, -ci->len);
+
+	console_textview_update_rect(sc, ar);
+
+	ED_area_tag_redraw(CTX_wm_area(C));
+
+	console_scroll_bottom(ar);
+
+	return OPERATOR_FINISHED;
+}
+
+void CONSOLE_OT_clear_line(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Clear Line";
+	ot->description = "Clear the line and store in history";
+	ot->idname = "CONSOLE_OT_clear_line";
+
+	/* api callbacks */
+	ot->exec = console_clear_line_exec;
+	ot->poll = ED_operator_console_active;
+}
 
 /* the python exec operator uses this */
 static int console_clear_exec(bContext *C, wmOperator *op)
@@ -529,7 +666,7 @@ static int console_clear_exec(bContext *C, wmOperator *op)
 	short scrollback = RNA_boolean_get(op->ptr, "scrollback");
 	short history = RNA_boolean_get(op->ptr, "history");
 	
-	/*ConsoleLine *ci= */ console_history_verify(C);
+	/*ConsoleLine *ci = */ console_history_verify(C);
 	
 	if (scrollback) { /* last item in mistory */
 		while (sc->scrollback.first)
@@ -571,7 +708,7 @@ static int console_history_cycle_exec(bContext *C, wmOperator *op)
 	SpaceConsole *sc = CTX_wm_space_console(C);
 	ARegion *ar = CTX_wm_region(C);
 
-	ConsoleLine *ci = console_history_verify(C); /* TODO - stupid, just prevernts crashes when no command line */
+	ConsoleLine *ci = console_history_verify(C); /* TODO - stupid, just prevents crashes when no command line */
 	short reverse = RNA_boolean_get(op->ptr, "reverse"); /* assumes down, reverse is up */
 	int prev_len = ci->len;
 
@@ -584,7 +721,7 @@ static int console_history_cycle_exec(bContext *C, wmOperator *op)
 			console_history_free(sc, ci_prev);
 	}
 
-	if (reverse) { /* last item in mistory */
+	if (reverse) { /* last item in history */
 		ci = sc->history.last;
 		BLI_remlink(&sc->history, ci);
 		BLI_addhead(&sc->history, ci);
@@ -600,7 +737,7 @@ static int console_history_cycle_exec(bContext *C, wmOperator *op)
 		while ((cl = console_history_find(sc, ci->line, ci)))
 			console_history_free(sc, cl);
 
-		console_history_add(C, (ConsoleLine *)sc->history.last);
+		console_history_add(sc, (ConsoleLine *)sc->history.last);
 	}
 	
 	ci = sc->history.last;
@@ -682,7 +819,7 @@ void CONSOLE_OT_history_append(wmOperatorType *ot)
 	ot->poll = ED_operator_console_active;
 	
 	/* properties */
-	RNA_def_string(ot->srna, "text", "", 0, "Text", "Text to insert at the cursor position");	
+	RNA_def_string(ot->srna, "text", "", 0, "Text", "Text to insert at the cursor position");
 	RNA_def_int(ot->srna, "current_character", 0, 0, INT_MAX, "Cursor", "The index of the cursor", 0, 10000);
 	RNA_def_boolean(ot->srna, "remove_duplicates", 0, "Remove Duplicates", "Remove duplicate items in the history");
 }
@@ -724,7 +861,8 @@ void CONSOLE_OT_scrollback_append(wmOperatorType *ot)
 		{CONSOLE_LINE_INPUT,    "INPUT", 0, "Input", ""},
 		{CONSOLE_LINE_INFO,     "INFO", 0, "Information", ""},
 		{CONSOLE_LINE_ERROR,    "ERROR", 0, "Error", ""},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 
 	/* identifiers */
 	ot->name = "Scrollback Append";
@@ -782,8 +920,8 @@ static int console_copy_exec(bContext *C, wmOperator *UNUSED(op))
 
 	for (cl = sc->scrollback.first; cl; cl = cl->next) {
 		if (sel[0] <= cl->len && sel[1] >= 0) {
-			int sta = MAX2(sel[0], 0);
-			int end = MIN2(sel[1], cl->len);
+			int sta = max_ii(sel[0], 0);
+			int end = min_ii(sel[1], cl->len);
 
 			if (BLI_dynstr_get_len(buf_dyn))
 				BLI_dynstr_append(buf_dyn, "\n");
@@ -928,7 +1066,7 @@ static void console_modal_select_apply(bContext *C, wmOperator *op, wmEvent *eve
 
 static void console_cursor_set_exit(bContext *UNUSED(C), wmOperator *op)
 {
-//	SpaceConsole *sc= CTX_wm_space_console(C);
+//	SpaceConsole *sc = CTX_wm_space_console(C);
 	SetConsoleCursor *scu = op->customdata;
 
 #if 0
@@ -945,7 +1083,7 @@ static void console_cursor_set_exit(bContext *UNUSED(C), wmOperator *op)
 static int console_modal_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 {
 	SpaceConsole *sc = CTX_wm_space_console(C);
-//	ARegion *ar= CTX_wm_region(C);
+//	ARegion *ar = CTX_wm_region(C);
 	SetConsoleCursor *scu;
 
 	op->customdata = MEM_callocN(sizeof(SetConsoleCursor), "SetConsoleCursor");
