@@ -71,7 +71,7 @@
 #include "fsmenu.h"
 #include "filelist.h"
 
-#include "file_intern.h"	// own include
+#include "file_intern.h"    // own include
 
 /* button events */
 enum {
@@ -82,7 +82,7 @@ enum {
 
 static void do_file_buttons(bContext *C, void *UNUSED(arg), int event)
 {
-	switch(event) {
+	switch (event) {
 		case B_FS_FILENAME:
 			file_filename_exec(C, NULL);
 			break;
@@ -109,8 +109,8 @@ void file_draw_buttons(const bContext *C, ARegion *ar)
 {
 	/* Button layout. */
 	const int max_x      = ar->winx - 10;
-	const int line1_y    = ar->winy - (IMASEL_BUTTONS_HEIGHT/2 + IMASEL_BUTTONS_MARGIN);
-	const int line2_y    = line1_y - (IMASEL_BUTTONS_HEIGHT/2 + IMASEL_BUTTONS_MARGIN);
+	const int line1_y    = ar->winy - (IMASEL_BUTTONS_HEIGHT / 2 + IMASEL_BUTTONS_MARGIN);
+	const int line2_y    = line1_y - (IMASEL_BUTTONS_HEIGHT / 2 + IMASEL_BUTTONS_MARGIN);
 	const int input_minw = 20;
 	const int btn_h      = UI_UNIT_Y;
 	const int btn_fn_w   = UI_UNIT_X;
@@ -123,16 +123,16 @@ void file_draw_buttons(const bContext *C, ARegion *ar)
 	int loadbutton;
 	int fnumbuttons;
 	int min_x       = 10;
-	int chan_offs	= 0;
+	int chan_offs   = 0;
 	int available_w = max_x - min_x;
 	int line1_w     = available_w;
 	int line2_w     = available_w;
 	
-	uiBut*            but;
-	uiBlock*          block;
-	SpaceFile*        sfile  = CTX_wm_space_file(C);
-	FileSelectParams* params = ED_fileselect_get_params(sfile);
-	ARegion*		  artmp;
+	uiBut *but;
+	uiBlock *block;
+	SpaceFile *sfile  = CTX_wm_space_file(C);
+	FileSelectParams *params = ED_fileselect_get_params(sfile);
+	ARegion *artmp;
 	
 	/* Initialize UI block. */
 	BLI_snprintf(uiblockstr, sizeof(uiblockstr), "win %p", (void *)ar);
@@ -140,7 +140,7 @@ void file_draw_buttons(const bContext *C, ARegion *ar)
 	uiBlockSetHandleFunc(block, do_file_buttons, NULL);
 
 	/* exception to make space for collapsed region icon */
-	for (artmp=CTX_wm_area(C)->regionbase.first; artmp; artmp=artmp->next) {
+	for (artmp = CTX_wm_area(C)->regionbase.first; artmp; artmp = artmp->next) {
 		if (artmp->regiontype == RGN_TYPE_CHANNELS && artmp->flag & RGN_FLAG_HIDDEN) {
 			chan_offs = 16;
 			min_x += chan_offs;
@@ -152,7 +152,7 @@ void file_draw_buttons(const bContext *C, ARegion *ar)
 	loadbutton = UI_GetStringWidth(sfile->params->title) + btn_margin;
 	if (loadbutton < btn_minw) {
 		loadbutton = MAX2(btn_minw, 
-						  btn_margin + UI_GetStringWidth(params->title));
+		                  btn_margin + UI_GetStringWidth(params->title));
 	}
 	
 	if (available_w <= loadbutton + separator + input_minw || params->title[0] == 0) {
@@ -174,12 +174,12 @@ void file_draw_buttons(const bContext *C, ARegion *ar)
 	
 	/* Text input fields for directory and file. */
 	if (available_w > 0) {
-		int overwrite_alert= file_draw_check_exists(sfile);
+		int overwrite_alert = file_draw_check_exists(sfile);
 		/* callbacks for operator check functions */
 		uiBlockSetFunc(block, file_draw_check_cb, NULL, NULL);
 
 		but = uiDefButTextO(block, TEX, "FILE_OT_directory", 0, "",
-		                    min_x, line1_y, line1_w-chan_offs, btn_h,
+		                    min_x, line1_y, line1_w - chan_offs, btn_h,
 		                    params->dir, 0.0, (float)FILE_MAX, 0, 0,
 		                    TIP_("File path"));
 		uiButSetCompleteFunc(but, autocomplete_directory, NULL);
@@ -187,9 +187,9 @@ void file_draw_buttons(const bContext *C, ARegion *ar)
 
 		if ((params->flag & FILE_DIRSEL_ONLY) == 0) {
 			but = uiDefBut(block, TEX, B_FS_FILENAME, "",
-			               min_x, line2_y, line2_w-chan_offs, btn_h,
+			               min_x, line2_y, line2_w - chan_offs, btn_h,
 			               params->file, 0.0, (float)FILE_MAXFILE, 0, 0,
-			               TIP_(overwrite_alert ?N_("File name, overwrite existing") : N_("File name")));
+			               TIP_(overwrite_alert ? N_("File name, overwrite existing") : N_("File name")));
 			uiButSetCompleteFunc(but, autocomplete_file, NULL);
 			uiButSetFlag(but, UI_BUT_NO_UTF8);
 			uiButClearFlag(but, UI_BUT_UNDO); /* operator button above does this automatic */
@@ -246,8 +246,8 @@ static void draw_tile(int sx, int sy, int width, int height, int colorid, int sh
 static int get_file_icon(struct direntry *file)
 {
 	if (file->type & S_IFDIR) {
-		if ( strcmp(file->relname, "..") == 0) {
-				return  ICON_FILE_PARENT;
+		if (strcmp(file->relname, "..") == 0) {
+			return ICON_FILE_PARENT;
 		}
 		if (file->flags & BLENDERFILE) {
 			return ICON_FILE_BLEND;
@@ -257,21 +257,23 @@ static int get_file_icon(struct direntry *file)
 	else if (file->flags & BLENDERFILE)
 		return ICON_FILE_BLEND;
 	else if (file->flags & BLENDERFILE_BACKUP)
-		return ICON_FILE_BLEND;
+		return ICON_FILE_BACKUP;
 	else if (file->flags & IMAGEFILE)
 		return ICON_FILE_IMAGE;
 	else if (file->flags & MOVIEFILE)
 		return ICON_FILE_MOVIE;
 	else if (file->flags & PYSCRIPTFILE)
 		return ICON_FILE_SCRIPT;
-	else if (file->flags & SOUNDFILE) 
+	else if (file->flags & SOUNDFILE)
 		return ICON_FILE_SOUND;
-	else if (file->flags & FTFONTFILE) 
+	else if (file->flags & FTFONTFILE)
 		return ICON_FILE_FONT;
-	else if (file->flags & BTXFILE) 
+	else if (file->flags & BTXFILE)
 		return ICON_FILE_BLANK;
-	else if (file->flags & COLLADAFILE) 
+	else if (file->flags & COLLADAFILE)
 		return ICON_FILE_BLANK;
+	else if (file->flags & TEXTFILE)
+		return ICON_FILE_TEXT;
 	else
 		return ICON_FILE_BLANK;
 }
@@ -279,11 +281,11 @@ static int get_file_icon(struct direntry *file)
 static void file_draw_icon(uiBlock *block, char *path, int sx, int sy, int icon, int width, int height)
 {
 	uiBut *but;
-	int x,y;
-	/*float alpha=1.0f;*/
+	int x, y;
+	// float alpha = 1.0f;
 	
 	x = sx;
-	y = sy-height;
+	y = sy - height;
 	
 	/*if (icon == ICON_FILE_BLANK) alpha = 0.375f;*/
 
@@ -292,21 +294,21 @@ static void file_draw_icon(uiBlock *block, char *path, int sx, int sy, int icon,
 }
 
 
-static void file_draw_string(int sx, int sy, const char* string, float width, int height, short align)
+static void file_draw_string(int sx, int sy, const char *string, float width, int height, short align)
 {
-	uiStyle *style= UI_GetStyle();
+	uiStyle *style = UI_GetStyle();
 	uiFontStyle fs = style->widgetlabel;
 	rcti rect;
 	char fname[FILE_MAXFILE];
 
 	fs.align = align;
 
-	BLI_strncpy(fname,string, FILE_MAXFILE);
+	BLI_strncpy(fname, string, FILE_MAXFILE);
 	file_shorten_string(fname, width + 1.0f, 0);
 
 	/* no text clipping needed, uiStyleFontDraw does it but is a bit too strict (for buttons it works) */
 	rect.xmin = sx;
-	rect.xmax = (int)(sx + ceil(width+4.0f));
+	rect.xmax = (int)(sx + ceil(width + 4.0f));
 	rect.ymin = sy - height;
 	rect.ymax = sy;
 	
@@ -315,12 +317,11 @@ static void file_draw_string(int sx, int sy, const char* string, float width, in
 
 void file_calc_previews(const bContext *C, ARegion *ar)
 {
-	SpaceFile *sfile= CTX_wm_space_file(C);
-	View2D *v2d= &ar->v2d;
+	SpaceFile *sfile = CTX_wm_space_file(C);
+	View2D *v2d = &ar->v2d;
 	
 	ED_fileselect_init_layout(sfile, ar);
-	/* +SCROLL_HEIGHT is bad hack to work around issue in UI_view2d_totRect_set */
-	UI_view2d_totRect_set(v2d, sfile->layout->width, sfile->layout->height+V2D_SCROLL_HEIGHT);
+	UI_view2d_totRect_set(v2d, sfile->layout->width, sfile->layout->height);
 }
 
 static void file_draw_preview(uiBlock *block, struct direntry *file, int sx, int sy, ImBuf *imb, FileLayout *layout, short dropshadow)
@@ -334,27 +335,30 @@ static void file_draw_preview(uiBlock *block, struct direntry *file, int sx, int
 		float scale;
 		int ex, ey;
 		
-		if ( (imb->x > layout->prv_w) || (imb->y > layout->prv_h) ) {
+		if ((imb->x * UI_DPI_FAC > layout->prv_w) ||
+		    (imb->y * UI_DPI_FAC > layout->prv_h))
+		{
 			if (imb->x > imb->y) {
 				scaledx = (float)layout->prv_w;
-				scaledy =  ( (float)imb->y/(float)imb->x )*layout->prv_w;
-				scale = scaledx/imb->x;
+				scaledy =  ( (float)imb->y / (float)imb->x) * layout->prv_w;
+				scale = scaledx / imb->x;
 			}
 			else {
 				scaledy = (float)layout->prv_h;
-				scaledx =  ( (float)imb->x/(float)imb->y )*layout->prv_h;
-				scale = scaledy/imb->y;
+				scaledx =  ( (float)imb->x / (float)imb->y) * layout->prv_h;
+				scale = scaledy / imb->y;
 			}
 		}
 		else {
-			scaledx = (float)imb->x;
-			scaledy = (float)imb->y;
-			scale = 1.0;
+			scaledx = (float)imb->x * UI_DPI_FAC;
+			scaledy = (float)imb->y * UI_DPI_FAC;
+			scale = UI_DPI_FAC;
 		}
+
 		ex = (int)scaledx;
 		ey = (int)scaledy;
-		fx = ((float)layout->prv_w - (float)ex)/2.0f;
-		fy = ((float)layout->prv_h - (float)ey)/2.0f;
+		fx = ((float)layout->prv_w - (float)ex) / 2.0f;
+		fy = ((float)layout->prv_h - (float)ey) / 2.0f;
 		dx = (fx + 0.5f + layout->prv_border_x);
 		dy = (fy + 0.5f - layout->prv_border_y);
 		xco = sx + (int)dx;
@@ -389,17 +393,17 @@ static void file_draw_preview(uiBlock *block, struct direntry *file, int sx, int
 
 static void renamebutton_cb(bContext *C, void *UNUSED(arg1), char *oldname)
 {
-	char newname[FILE_MAX+12];
-	char orgname[FILE_MAX+12];
-	char filename[FILE_MAX+12];
-	SpaceFile *sfile= (SpaceFile*)CTX_wm_space_data(C);
-	ARegion* ar = CTX_wm_region(C);
+	char newname[FILE_MAX + 12];
+	char orgname[FILE_MAX + 12];
+	char filename[FILE_MAX + 12];
+	SpaceFile *sfile = (SpaceFile *)CTX_wm_space_data(C);
+	ARegion *ar = CTX_wm_region(C);
 
 	BLI_make_file_string(G.main->name, orgname, sfile->params->dir, oldname);
 	BLI_strncpy(filename, sfile->params->renameedit, sizeof(filename));
 	BLI_make_file_string(G.main->name, newname, sfile->params->dir, filename);
 
-	if ( strcmp(orgname, newname) != 0 ) {
+	if (strcmp(orgname, newname) != 0) {
 		if (!BLI_exists(newname)) {
 			BLI_rename(orgname, newname);
 			/* to make sure we show what is on disk */
@@ -417,12 +421,11 @@ static void draw_background(FileLayout *layout, View2D *v2d)
 	int sy;
 
 	/* alternating flat shade background */
-	for (i=0; (i <= layout->rows); i+=2)
-	{
-		sy = (int)v2d->cur.ymax - i*(layout->tile_h+2*layout->tile_border_y) - layout->tile_border_y;
+	for (i = 0; (i <= layout->rows); i += 2) {
+		sy = (int)v2d->cur.ymax - i * (layout->tile_h + 2 * layout->tile_border_y) - layout->tile_border_y;
 
 		UI_ThemeColorShade(TH_BACK, -7);
-		glRectf(v2d->cur.xmin, (float)sy, v2d->cur.xmax, (float)(sy+layout->tile_h+2*layout->tile_border_y));
+		glRectf(v2d->cur.xmin, (float)sy, v2d->cur.xmax, (float)(sy + layout->tile_h + 2 * layout->tile_border_y));
 		
 	}
 }
@@ -434,10 +437,10 @@ static void draw_dividers(FileLayout *layout, View2D *v2d)
 	/* vertical column dividers */
 	sx = (int)v2d->tot.xmin;
 	while (sx < v2d->cur.xmax) {
-		sx += (layout->tile_w+2*layout->tile_border_x);
+		sx += (layout->tile_w + 2 * layout->tile_border_x);
 		
 		UI_ThemeColorShade(TH_BACK, 30);
-		sdrawline(sx+1, (short)(v2d->cur.ymax - layout->tile_border_y),  sx+1,  (short)v2d->cur.ymin); 
+		sdrawline(sx + 1, (short)(v2d->cur.ymax - layout->tile_border_y),  sx + 1,  (short)v2d->cur.ymin);
 		UI_ThemeColorShade(TH_BACK, -30);
 		sdrawline(sx, (short)(v2d->cur.ymax - layout->tile_border_y),  sx,  (short)v2d->cur.ymin); 
 	}
@@ -445,11 +448,11 @@ static void draw_dividers(FileLayout *layout, View2D *v2d)
 
 void file_draw_list(const bContext *C, ARegion *ar)
 {
-	SpaceFile *sfile= CTX_wm_space_file(C);
-	FileSelectParams* params = ED_fileselect_get_params(sfile);
-	FileLayout* layout= ED_fileselect_get_layout(sfile, ar);
-	View2D *v2d= &ar->v2d;
-	struct FileList* files = sfile->files;
+	SpaceFile *sfile = CTX_wm_space_file(C);
+	FileSelectParams *params = ED_fileselect_get_params(sfile);
+	FileLayout *layout = ED_fileselect_get_layout(sfile, ar);
+	View2D *v2d = &ar->v2d;
+	struct FileList *files = sfile->files;
 	struct direntry *file;
 	ImBuf *imb;
 	uiBlock *block = uiBeginBlock(C, ar, __func__, UI_EMBOSS);
@@ -461,7 +464,7 @@ void file_draw_list(const bContext *C, ARegion *ar)
 	int i;
 	short is_icon;
 	short align;
-
+	int column_space = 0.6f * UI_UNIT_X;
 
 	numfiles = filelist_numfiles(files);
 	
@@ -473,7 +476,7 @@ void file_draw_list(const bContext *C, ARegion *ar)
 	}
 
 	offset = ED_fileselect_layout_offset(layout, (int)ar->v2d.cur.xmin, (int)-ar->v2d.cur.ymax);
-	if (offset<0) offset=0;
+	if (offset < 0) offset = 0;
 
 	numfiles_layout = ED_fileselect_layout_numfiles(layout, ar);
 
@@ -485,18 +488,17 @@ void file_draw_list(const bContext *C, ARegion *ar)
 		numfiles_layout += layout->columns;
 	}
 
-	textwidth =( FILE_IMGDISPLAY == params->display) ? layout->tile_w : (int)layout->column_widths[COLUMN_NAME];
-	textheight = (int)(layout->textheight*3.0/2.0 + 0.5);
+	textwidth = (FILE_IMGDISPLAY == params->display) ? layout->tile_w : (int)layout->column_widths[COLUMN_NAME];
+	textheight = (int)(layout->textheight * 3.0 / 2.0 + 0.5);
 
-	align = ( FILE_IMGDISPLAY == params->display) ? UI_STYLE_TEXT_CENTER : UI_STYLE_TEXT_LEFT;
+	align = (FILE_IMGDISPLAY == params->display) ? UI_STYLE_TEXT_CENTER : UI_STYLE_TEXT_LEFT;
 
-	for (i=offset; (i < numfiles) && (i<offset+numfiles_layout); ++i)
-	{
+	for (i = offset; (i < numfiles) && (i < offset + numfiles_layout); i++) {
 		ED_fileselect_layout_tilepos(layout, i, &sx, &sy);
-		sx += (int)(v2d->tot.xmin+2.0f);
+		sx += (int)(v2d->tot.xmin + 0.1f * UI_UNIT_X);
 		sy = (int)(v2d->tot.ymax - sy);
 
-		file = filelist_file(files, i);	
+		file = filelist_file(files, i);
 		
 		UI_ThemeColor4(TH_TEXT);
 
@@ -505,78 +507,78 @@ void file_draw_list(const bContext *C, ARegion *ar)
 			if ((params->active_file == i) || (file->selflag & HILITED_FILE) || (file->selflag & SELECTED_FILE)) {
 				int colorid = (file->selflag & SELECTED_FILE) ? TH_HILITE : TH_BACK;
 				int shade = (params->active_file == i) || (file->selflag & HILITED_FILE) ? 20 : 0;
-				draw_tile(sx, sy-1, layout->tile_w+4, sfile->layout->tile_h+layout->tile_border_y, colorid, shade);
+				draw_tile(sx, sy - 1, layout->tile_w + 4, sfile->layout->tile_h + layout->tile_border_y, colorid, shade);
 			}
 		}
 		uiSetRoundBox(UI_CNR_NONE);
 
-		if ( FILE_IMGDISPLAY == params->display ) {
+		if (FILE_IMGDISPLAY == params->display) {
 			is_icon = 0;
 			imb = filelist_getimage(files, i);
 			if (!imb) {
-				imb = filelist_geticon(files,i);
+				imb = filelist_geticon(files, i);
 				is_icon = 1;
 			}
 			
 			file_draw_preview(block, file, sx, sy, imb, layout, !is_icon && (file->flags & IMAGEFILE));
 		}
 		else {
-			file_draw_icon(block, file->path, sx, sy-(UI_UNIT_Y / 6), get_file_icon(file), ICON_DEFAULT_WIDTH_SCALE, ICON_DEFAULT_HEIGHT_SCALE);
-			sx += ICON_DEFAULT_WIDTH_SCALE + 4;
+			file_draw_icon(block, file->path, sx, sy - (UI_UNIT_Y / 6), get_file_icon(file), ICON_DEFAULT_WIDTH_SCALE, ICON_DEFAULT_HEIGHT_SCALE);
+			sx += ICON_DEFAULT_WIDTH_SCALE + 0.2f * UI_UNIT_X;
 		}
 
 		UI_ThemeColor4(TH_TEXT);
 
 		if (file->selflag & EDITING_FILE) {
-			uiBut *but = uiDefBut(block, TEX, 1, "", sx , sy-layout->tile_h-3, 
-				textwidth, textheight, sfile->params->renameedit, 1.0f, (float)sizeof(sfile->params->renameedit),0,0,"");
+			uiBut *but = uiDefBut(block, TEX, 1, "", sx, sy - layout->tile_h - 0.15f * UI_UNIT_X,
+			                      textwidth, textheight, sfile->params->renameedit, 1.0f, (float)sizeof(sfile->params->renameedit), 0, 0, "");
 			uiButSetRenameFunc(but, renamebutton_cb, file);
 			uiButSetFlag(but, UI_BUT_NO_UTF8); /* allow non utf8 names */
 			uiButClearFlag(but, UI_BUT_UNDO);
-			if ( 0 == uiButActiveOnly(C, block, but)) {
+			if (0 == uiButActiveOnly(C, block, but)) {
 				file->selflag &= ~EDITING_FILE;
 			}
 		}
 
 		if (!(file->selflag & EDITING_FILE)) {
 			int tpos = (FILE_IMGDISPLAY == params->display) ? sy - layout->tile_h + layout->textheight : sy;
-			file_draw_string(sx+1, tpos, file->relname, (float)textwidth, textheight, align);
+			file_draw_string(sx + 1, tpos, file->relname, (float)textwidth, textheight, align);
 		}
 
 		if (params->display == FILE_SHORTDISPLAY) {
-			sx += (int)layout->column_widths[COLUMN_NAME] + 12;
+			sx += (int)layout->column_widths[COLUMN_NAME] + column_space;
 			if (!(file->type & S_IFDIR)) {
-				file_draw_string(sx, sy, file->size, layout->column_widths[COLUMN_SIZE], layout->tile_h, align);	
-				sx += (int)layout->column_widths[COLUMN_SIZE] + 12;
+				file_draw_string(sx, sy, file->size, layout->column_widths[COLUMN_SIZE], layout->tile_h, align);
+				sx += (int)layout->column_widths[COLUMN_SIZE] + column_space;
 			}
 		}
 		else if (params->display == FILE_LONGDISPLAY) {
-			sx += (int)layout->column_widths[COLUMN_NAME] + 12;
+			sx += (int)layout->column_widths[COLUMN_NAME] + column_space;
 
 #ifndef WIN32
 			/* rwx rwx rwx */
 			file_draw_string(sx, sy, file->mode1, layout->column_widths[COLUMN_MODE1], layout->tile_h, align); 
-			sx += layout->column_widths[COLUMN_MODE1] + 12;
+			sx += layout->column_widths[COLUMN_MODE1] + column_space;
 
 			file_draw_string(sx, sy, file->mode2, layout->column_widths[COLUMN_MODE2], layout->tile_h, align);
-			sx += layout->column_widths[COLUMN_MODE2] + 12;
+			sx += layout->column_widths[COLUMN_MODE2] + column_space;
 
 			file_draw_string(sx, sy, file->mode3, layout->column_widths[COLUMN_MODE3], layout->tile_h, align);
-			sx += layout->column_widths[COLUMN_MODE3] + 12;
+			sx += layout->column_widths[COLUMN_MODE3] + column_space;
 
-			file_draw_string(sx, sy, file->owner, layout->column_widths[COLUMN_OWNER] , layout->tile_h, align);
-			sx += layout->column_widths[COLUMN_OWNER] + 12;
+			file_draw_string(sx, sy, file->owner, layout->column_widths[COLUMN_OWNER], layout->tile_h, align);
+			sx += layout->column_widths[COLUMN_OWNER] + column_space;
 #endif
 
 			file_draw_string(sx, sy, file->date, layout->column_widths[COLUMN_DATE], layout->tile_h, align);
-			sx += (int)layout->column_widths[COLUMN_DATE] + 12;
+			sx += (int)layout->column_widths[COLUMN_DATE] + column_space;
 
-			file_draw_string(sx, sy, file->time, layout->column_widths[COLUMN_TIME] , layout->tile_h, align); 
-			sx += (int)layout->column_widths[COLUMN_TIME] + 12;
+			file_draw_string(sx, sy, file->time, layout->column_widths[COLUMN_TIME], layout->tile_h, align);
+			sx += (int)layout->column_widths[COLUMN_TIME] + column_space;
 
 			if (!(file->type & S_IFDIR)) {
 				file_draw_string(sx, sy, file->size, layout->column_widths[COLUMN_SIZE], layout->tile_h, align);
-				sx += (int)layout->column_widths[COLUMN_SIZE] + 12;
+				sx += (int)layout->column_widths[COLUMN_SIZE] + column_space;
 			}
 		}
 	}
