@@ -48,6 +48,7 @@
 
 #include "BKE_context.h"
 #include "BKE_screen.h"
+#include "BKE_global.h"
 
 #include "ED_space_api.h"
 #include "ED_screen.h"
@@ -63,7 +64,7 @@
 #include "UI_view2d.h"
 
 
-#include "file_intern.h"	// own include
+#include "file_intern.h"    // own include
 #include "fsmenu.h"
 #include "filelist.h"
 
@@ -74,36 +75,36 @@ static SpaceLink *file_new(const bContext *UNUSED(C))
 	ARegion *ar;
 	SpaceFile *sfile;
 	
-	sfile= MEM_callocN(sizeof(SpaceFile), "initfile");
-	sfile->spacetype= SPACE_FILE;
+	sfile = MEM_callocN(sizeof(SpaceFile), "initfile");
+	sfile->spacetype = SPACE_FILE;
 
 	/* header */
-	ar= MEM_callocN(sizeof(ARegion), "header for file");
+	ar = MEM_callocN(sizeof(ARegion), "header for file");
 	BLI_addtail(&sfile->regionbase, ar);
-	ar->regiontype= RGN_TYPE_HEADER;
-	ar->alignment= RGN_ALIGN_TOP;
+	ar->regiontype = RGN_TYPE_HEADER;
+	ar->alignment = RGN_ALIGN_TOP;
 
 	/* channel list region */
-	ar= MEM_callocN(sizeof(ARegion), "channel area for file");
+	ar = MEM_callocN(sizeof(ARegion), "channel area for file");
 	BLI_addtail(&sfile->regionbase, ar);
-	ar->regiontype= RGN_TYPE_CHANNELS;
-	ar->alignment= RGN_ALIGN_LEFT;	
+	ar->regiontype = RGN_TYPE_CHANNELS;
+	ar->alignment = RGN_ALIGN_LEFT;
 
 	/* ui list region */
-	ar= MEM_callocN(sizeof(ARegion), "ui area for file");
+	ar = MEM_callocN(sizeof(ARegion), "ui area for file");
 	BLI_addtail(&sfile->regionbase, ar);
-	ar->regiontype= RGN_TYPE_UI;
-	ar->alignment= RGN_ALIGN_TOP;
+	ar->regiontype = RGN_TYPE_UI;
+	ar->alignment = RGN_ALIGN_TOP;
 
 	/* main area */
-	ar= MEM_callocN(sizeof(ARegion), "main area for file");
+	ar = MEM_callocN(sizeof(ARegion), "main area for file");
 	BLI_addtail(&sfile->regionbase, ar);
-	ar->regiontype= RGN_TYPE_WINDOW;
+	ar->regiontype = RGN_TYPE_WINDOW;
 	ar->v2d.scroll = (V2D_SCROLL_RIGHT | V2D_SCROLL_BOTTOM);
-	ar->v2d.align = (V2D_ALIGN_NO_NEG_X|V2D_ALIGN_NO_POS_Y);
-	ar->v2d.keepzoom = (V2D_LOCKZOOM_X|V2D_LOCKZOOM_Y|V2D_LIMITZOOM|V2D_KEEPASPECT);
-	ar->v2d.keeptot= V2D_KEEPTOT_STRICT;
-	ar->v2d.minzoom= ar->v2d.maxzoom= 1.0f;
+	ar->v2d.align = (V2D_ALIGN_NO_NEG_X | V2D_ALIGN_NO_POS_Y);
+	ar->v2d.keepzoom = (V2D_LOCKZOOM_X | V2D_LOCKZOOM_Y | V2D_LIMITZOOM | V2D_KEEPASPECT);
+	ar->v2d.keeptot = V2D_KEEPTOT_STRICT;
+	ar->v2d.minzoom = ar->v2d.maxzoom = 1.0f;
 
 	return (SpaceLink *)sfile;
 }
@@ -111,31 +112,31 @@ static SpaceLink *file_new(const bContext *UNUSED(C))
 /* not spacelink itself */
 static void file_free(SpaceLink *sl)
 {	
-	SpaceFile *sfile= (SpaceFile *) sl;
+	SpaceFile *sfile = (SpaceFile *) sl;
 	
 	if (sfile->files) {
 		// XXXXX would need to do thumbnails_stop here, but no context available
 		filelist_freelib(sfile->files);
 		filelist_free(sfile->files);
 		MEM_freeN(sfile->files);
-		sfile->files= NULL;
+		sfile->files = NULL;
 	}
 
 	if (sfile->folders_prev) {
 		folderlist_free(sfile->folders_prev);
 		MEM_freeN(sfile->folders_prev);
-		sfile->folders_prev= NULL;
+		sfile->folders_prev = NULL;
 	}
 
 	if (sfile->folders_next) {
 		folderlist_free(sfile->folders_next);
 		MEM_freeN(sfile->folders_next);
-		sfile->folders_next= NULL;
+		sfile->folders_next = NULL;
 	}
 
 	if (sfile->params) {
 		MEM_freeN(sfile->params);
-		sfile->params= NULL;
+		sfile->params = NULL;
 	}
 
 	if (sfile->layout) {
@@ -148,24 +149,27 @@ static void file_free(SpaceLink *sl)
 /* spacetype; init callback, area size changes, screen set, etc */
 static void file_init(struct wmWindowManager *UNUSED(wm), ScrArea *sa)
 {
-	SpaceFile *sfile= (SpaceFile*)sa->spacedata.first;
+	SpaceFile *sfile = (SpaceFile *)sa->spacedata.first;
 	//printf("file_init\n");
 
-	if (sfile->layout) sfile->layout->dirty= TRUE;
+	/* refresh system directory list */
+	fsmenu_refresh_system_category(fsmenu_get());
+
+	if (sfile->layout) sfile->layout->dirty = TRUE;
 }
 
 
 static SpaceLink *file_duplicate(SpaceLink *sl)
 {
-	SpaceFile *sfileo= (SpaceFile*)sl;
-	SpaceFile *sfilen= MEM_dupallocN(sl);
+	SpaceFile *sfileo = (SpaceFile *)sl;
+	SpaceFile *sfilen = MEM_dupallocN(sl);
 	
 	/* clear or remove stuff from old */
 	sfilen->op = NULL; /* file window doesn't own operators */
 
 	if (sfileo->params) {
 		sfilen->files = filelist_new(sfileo->params->type);
-		sfilen->params= MEM_dupallocN(sfileo->params);
+		sfilen->params = MEM_dupallocN(sfileo->params);
 		filelist_setdir(sfilen->files, sfilen->params->dir);
 	}
 
@@ -176,14 +180,14 @@ static SpaceLink *file_duplicate(SpaceLink *sl)
 		sfilen->folders_next = folderlist_duplicate(sfileo->folders_next);
 	
 	if (sfileo->layout) {
-		sfilen->layout= MEM_dupallocN(sfileo->layout);
+		sfilen->layout = MEM_dupallocN(sfileo->layout);
 	}
 	return (SpaceLink *)sfilen;
 }
 
 static void file_refresh(const bContext *C, ScrArea *UNUSED(sa))
 {
-	SpaceFile *sfile= CTX_wm_space_file(C);
+	SpaceFile *sfile = CTX_wm_space_file(C);
 	FileSelectParams *params = ED_fileselect_get_params(sfile);
 
 	if (!sfile->folders_prev)
@@ -194,13 +198,13 @@ static void file_refresh(const bContext *C, ScrArea *UNUSED(sa))
 		params->active_file = -1; // added this so it opens nicer (ton)
 	}
 	filelist_hidedot(sfile->files, params->flag & FILE_HIDE_DOT);
-	filelist_setfilter(sfile->files, params->flag & FILE_FILTER ? params->filter : 0);	
+	filelist_setfilter(sfile->files, params->flag & FILE_FILTER ? params->filter : 0);
 	filelist_setfilter_types(sfile->files, params->filter_glob);
 
 	if (filelist_empty(sfile->files)) {
 		thumbnails_stop(sfile->files, C);
 		filelist_readdir(sfile->files);
-		if (params->sort!=FILE_SORT_NONE) {
+		if (params->sort != FILE_SORT_NONE) {
 			filelist_sort(sfile->files, params->sort);
 		}
 		BLI_strncpy(params->dir, filelist_dir(sfile->files), FILE_MAX);
@@ -209,7 +213,7 @@ static void file_refresh(const bContext *C, ScrArea *UNUSED(sa))
 		}
 	}
 	else {
-		if (params->sort!=FILE_SORT_NONE) {
+		if (params->sort != FILE_SORT_NONE) {
 			thumbnails_stop(sfile->files, C);
 			filelist_sort(sfile->files, params->sort);
 			if (params->display == FILE_IMGDISPLAY) {
@@ -218,7 +222,7 @@ static void file_refresh(const bContext *C, ScrArea *UNUSED(sa))
 		}
 		else {
 			if (params->display == FILE_IMGDISPLAY) {
-				if (!thumbnails_running(sfile->files,C)) {
+				if (!thumbnails_running(sfile->files, C)) {
 					thumbnails_start(sfile->files, C);
 				}
 			}
@@ -234,7 +238,7 @@ static void file_refresh(const bContext *C, ScrArea *UNUSED(sa))
 	if (params->renamefile[0] != '\0') {
 		int idx = filelist_find(sfile->files, params->renamefile);
 		if (idx >= 0) {
-			struct direntry *file= filelist_file(sfile->files, idx);
+			struct direntry *file = filelist_file(sfile->files, idx);
 			if (file) {
 				file->selflag |= EDITING_FILE;
 			}
@@ -242,16 +246,16 @@ static void file_refresh(const bContext *C, ScrArea *UNUSED(sa))
 		BLI_strncpy(sfile->params->renameedit, sfile->params->renamefile, sizeof(sfile->params->renameedit));
 		params->renamefile[0] = '\0';
 	}
-	if (sfile->layout) sfile->layout->dirty= TRUE;
+	if (sfile->layout) sfile->layout->dirty = TRUE;
 
 }
 
 static void file_listener(ScrArea *sa, wmNotifier *wmn)
 {
-	/* SpaceFile* sfile = (SpaceFile*)sa->spacedata.first; */
+	/* SpaceFile *sfile = (SpaceFile *)sa->spacedata.first; */
 
 	/* context changes */
-	switch(wmn->category) {
+	switch (wmn->category) {
 		case NC_SPACE:
 			switch (wmn->data) {
 				case ND_SPACE_FILE_LIST:
@@ -280,14 +284,12 @@ static void file_main_area_init(wmWindowManager *wm, ARegion *ar)
 
 	keymap = WM_keymap_find(wm->defaultconf, "File Browser Main", SPACE_FILE, 0);
 	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
-							   
-
 }
 
 static void file_main_area_listener(ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
-	switch(wmn->category) {
+	switch (wmn->category) {
 		case NC_SPACE:
 			switch (wmn->data) {
 				case ND_SPACE_FILE_LIST:
@@ -304,11 +306,11 @@ static void file_main_area_listener(ARegion *ar, wmNotifier *wmn)
 static void file_main_area_draw(const bContext *C, ARegion *ar)
 {
 	/* draw entirely, view changes should be handled here */
-	SpaceFile *sfile= CTX_wm_space_file(C);
+	SpaceFile *sfile = CTX_wm_space_file(C);
 	FileSelectParams *params = ED_fileselect_get_params(sfile);
-	FileLayout *layout=NULL;
+	FileLayout *layout = NULL;
 
-	View2D *v2d= &ar->v2d;
+	View2D *v2d = &ar->v2d;
 	View2DScrollers *scrollers;
 	float col[3];
 
@@ -352,8 +354,8 @@ static void file_main_area_draw(const bContext *C, ARegion *ar)
 	
 	/* on first read, find active file */
 	if (params->active_file == -1) {
-		wmEvent *event= CTX_wm_window(C)->eventstate;
-		file_hilight_set(sfile, ar, event->x, event->y);
+		wmEvent *event = CTX_wm_window(C)->eventstate;
+		file_highlight_set(sfile, ar, event->x, event->y);
 	}
 	
 	file_draw_list(C, ar);
@@ -362,7 +364,7 @@ static void file_main_area_draw(const bContext *C, ARegion *ar)
 	UI_view2d_view_restore(C);
 	
 	/* scrollers */
-	scrollers= UI_view2d_scrollers_calc(C, v2d, V2D_ARG_DUMMY, V2D_ARG_DUMMY, V2D_ARG_DUMMY, V2D_ARG_DUMMY);
+	scrollers = UI_view2d_scrollers_calc(C, v2d, V2D_ARG_DUMMY, V2D_ARG_DUMMY, V2D_ARG_DUMMY, V2D_ARG_DUMMY);
 	UI_view2d_scrollers_draw(C, v2d, scrollers);
 	UI_view2d_scrollers_free(scrollers);
 
@@ -384,6 +386,7 @@ static void file_operatortypes(void)
 	WM_operatortype_append(FILE_OT_bookmark_toggle);
 	WM_operatortype_append(FILE_OT_bookmark_add);
 	WM_operatortype_append(FILE_OT_delete_bookmark);
+	WM_operatortype_append(FILE_OT_reset_recent);
 	WM_operatortype_append(FILE_OT_hidedot);
 	WM_operatortype_append(FILE_OT_filenum);
 	WM_operatortype_append(FILE_OT_directory_new);
@@ -414,12 +417,26 @@ static void file_keymap(struct wmKeyConfig *keyconf)
 	keymap = WM_keymap_find(keyconf, "File Browser Main", SPACE_FILE, 0);
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_execute", LEFTMOUSE, KM_DBL_CLICK, 0, 0);
 	RNA_boolean_set(kmi->ptr, "need_active", TRUE);
+
+	/* left mouse selects and opens */
 	WM_keymap_add_item(keymap, "FILE_OT_select", LEFTMOUSE, KM_CLICK, 0, 0);
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_select", LEFTMOUSE, KM_CLICK, KM_SHIFT, 0);
 	RNA_boolean_set(kmi->ptr, "extend", TRUE);
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_select", LEFTMOUSE, KM_CLICK, KM_ALT, 0);
 	RNA_boolean_set(kmi->ptr, "extend", TRUE);
 	RNA_boolean_set(kmi->ptr, "fill", TRUE);
+
+	/* right mouse selects without opening */
+	kmi = WM_keymap_add_item(keymap, "FILE_OT_select", RIGHTMOUSE, KM_CLICK, 0, 0);
+	RNA_boolean_set(kmi->ptr, "open", FALSE);
+	kmi = WM_keymap_add_item(keymap, "FILE_OT_select", RIGHTMOUSE, KM_CLICK, KM_SHIFT, 0);
+	RNA_boolean_set(kmi->ptr, "extend", TRUE);
+	RNA_boolean_set(kmi->ptr, "open", FALSE);
+	kmi = WM_keymap_add_item(keymap, "FILE_OT_select", RIGHTMOUSE, KM_CLICK, KM_ALT, 0);
+	RNA_boolean_set(kmi->ptr, "extend", TRUE);
+	RNA_boolean_set(kmi->ptr, "fill", TRUE);
+	RNA_boolean_set(kmi->ptr, "open", FALSE);
+
 	WM_keymap_add_item(keymap, "FILE_OT_select_all_toggle", AKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "FILE_OT_refresh", PADPERIOD, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "FILE_OT_select_border", BKEY, KM_PRESS, 0, 0);
@@ -432,12 +449,12 @@ static void file_keymap(struct wmKeyConfig *keyconf)
 	RNA_int_set(kmi->ptr, "increment", 10);
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_filenum", PADPLUSKEY, KM_PRESS, KM_CTRL, 0);
 	RNA_int_set(kmi->ptr, "increment", 100);
-	kmi = WM_keymap_add_item(keymap, "FILE_OT_filenum", PADMINUS, KM_PRESS, 0,0);
+	kmi = WM_keymap_add_item(keymap, "FILE_OT_filenum", PADMINUS, KM_PRESS, 0, 0);
 	RNA_int_set(kmi->ptr, "increment", -1);
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_filenum", PADMINUS, KM_PRESS, KM_SHIFT, 0);
 	RNA_int_set(kmi->ptr, "increment", -10);
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_filenum", PADMINUS, KM_PRESS, KM_CTRL, 0);
-	RNA_int_set(kmi->ptr, "increment",-100);
+	RNA_int_set(kmi->ptr, "increment", -100);
 	
 	
 	/* keys for button area (top) */
@@ -450,10 +467,10 @@ static void file_keymap(struct wmKeyConfig *keyconf)
 	RNA_int_set(kmi->ptr, "increment", 100);
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_filenum", PADMINUS, KM_PRESS, 0, 0);
 	RNA_int_set(kmi->ptr, "increment", -1);
-	kmi = WM_keymap_add_item(keymap, "FILE_OT_filenum", PADMINUS, KM_PRESS, KM_SHIFT,0);
+	kmi = WM_keymap_add_item(keymap, "FILE_OT_filenum", PADMINUS, KM_PRESS, KM_SHIFT, 0);
 	RNA_int_set(kmi->ptr, "increment", -10);
-	kmi = WM_keymap_add_item(keymap, "FILE_OT_filenum", PADMINUS, KM_PRESS, KM_CTRL,0);
-	RNA_int_set(kmi->ptr, "increment",-100);
+	kmi = WM_keymap_add_item(keymap, "FILE_OT_filenum", PADMINUS, KM_PRESS, KM_CTRL, 0);
+	RNA_int_set(kmi->ptr, "increment", -100);
 }
 
 
@@ -461,10 +478,11 @@ static void file_channel_area_init(wmWindowManager *wm, ARegion *ar)
 {
 	wmKeyMap *keymap;
 
+	ar->v2d.scroll = V2D_SCROLL_RIGHT | V2D_SCROLL_VERTICAL_HIDE;
 	ED_region_panels_init(wm, ar);
 
 	/* own keymaps */
-	keymap = WM_keymap_find(wm->defaultconf, "File Browser", SPACE_FILE, 0);	
+	keymap = WM_keymap_find(wm->defaultconf, "File Browser", SPACE_FILE, 0);
 	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
 }
 
@@ -476,7 +494,7 @@ static void file_channel_area_draw(const bContext *C, ARegion *ar)
 static void file_channel_area_listener(ARegion *UNUSED(ar), wmNotifier *wmn)
 {
 	/* context changes */
-	switch(wmn->category) {
+	switch (wmn->category) {
 		
 	}
 }
@@ -488,7 +506,7 @@ static void file_header_area_init(wmWindowManager *wm, ARegion *ar)
 	
 	ED_region_header_init(ar);
 	
-	keymap = WM_keymap_find(wm->defaultconf, "File Browser", SPACE_FILE, 0);	
+	keymap = WM_keymap_find(wm->defaultconf, "File Browser", SPACE_FILE, 0);
 	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
 }
 
@@ -516,12 +534,12 @@ static void file_ui_area_draw(const bContext *C, ARegion *ar)
 {
 	float col[3];
 	/* clear */
-	UI_GetThemeColor3fv(TH_PANEL, col);
+	UI_GetThemeColor3fv(TH_BACK, col);
 	glClearColor(col[0], col[1], col[2], 0.0);
 	glClear(GL_COLOR_BUFFER_BIT);
 
 	/* scrolling here is just annoying, disable it */
-	ar->v2d.cur.ymax = ar->v2d.cur.ymax - ar->v2d.cur.ymin;
+	ar->v2d.cur.ymax = BLI_rctf_size_y(&ar->v2d.cur);
 	ar->v2d.cur.ymin = 0;
 
 	/* set view2d view matrix for scrolling (without scrollers) */
@@ -536,7 +554,7 @@ static void file_ui_area_draw(const bContext *C, ARegion *ar)
 static void file_ui_area_listener(ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
-	switch(wmn->category) {
+	switch (wmn->category) {
 		case NC_SPACE:
 			switch (wmn->data) {
 				case ND_SPACE_FILE_LIST:
@@ -550,58 +568,58 @@ static void file_ui_area_listener(ARegion *ar, wmNotifier *wmn)
 /* only called once, from space/spacetypes.c */
 void ED_spacetype_file(void)
 {
-	SpaceType *st= MEM_callocN(sizeof(SpaceType), "spacetype file");
+	SpaceType *st = MEM_callocN(sizeof(SpaceType), "spacetype file");
 	ARegionType *art;
 	
-	st->spaceid= SPACE_FILE;
+	st->spaceid = SPACE_FILE;
 	strncpy(st->name, "File", BKE_ST_MAXNAME);
 	
-	st->new= file_new;
-	st->free= file_free;
-	st->init= file_init;
-	st->duplicate= file_duplicate;
-	st->refresh= file_refresh;
-	st->listener= file_listener;
-	st->operatortypes= file_operatortypes;
-	st->keymap= file_keymap;
+	st->new = file_new;
+	st->free = file_free;
+	st->init = file_init;
+	st->duplicate = file_duplicate;
+	st->refresh = file_refresh;
+	st->listener = file_listener;
+	st->operatortypes = file_operatortypes;
+	st->keymap = file_keymap;
 	
 	/* regions: main window */
-	art= MEM_callocN(sizeof(ARegionType), "spacetype file region");
+	art = MEM_callocN(sizeof(ARegionType), "spacetype file region");
 	art->regionid = RGN_TYPE_WINDOW;
-	art->init= file_main_area_init;
-	art->draw= file_main_area_draw;
-	art->listener= file_main_area_listener;
-	art->keymapflag= ED_KEYMAP_UI|ED_KEYMAP_VIEW2D;
+	art->init = file_main_area_init;
+	art->draw = file_main_area_draw;
+	art->listener = file_main_area_listener;
+	art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_VIEW2D;
 	BLI_addhead(&st->regiontypes, art);
 	
 	/* regions: header */
-	art= MEM_callocN(sizeof(ARegionType), "spacetype file region");
+	art = MEM_callocN(sizeof(ARegionType), "spacetype file region");
 	art->regionid = RGN_TYPE_HEADER;
-	art->prefsizey= HEADERY;
-	art->keymapflag= ED_KEYMAP_UI|ED_KEYMAP_VIEW2D|ED_KEYMAP_HEADER;
-	art->init= file_header_area_init;
-	art->draw= file_header_area_draw;
-	// art->listener= file_header_area_listener;
+	art->prefsizey = HEADERY;
+	art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_VIEW2D | ED_KEYMAP_HEADER;
+	art->init = file_header_area_init;
+	art->draw = file_header_area_draw;
+	// art->listener = file_header_area_listener;
 	BLI_addhead(&st->regiontypes, art);
 	
 	/* regions: ui */
-	art= MEM_callocN(sizeof(ARegionType), "spacetype file region");
+	art = MEM_callocN(sizeof(ARegionType), "spacetype file region");
 	art->regionid = RGN_TYPE_UI;
-	art->prefsizey= 60;
-	art->keymapflag= ED_KEYMAP_UI;
-	art->listener= file_ui_area_listener;
-	art->init= file_ui_area_init;
-	art->draw= file_ui_area_draw;
+	art->prefsizey = 60;
+	art->keymapflag = ED_KEYMAP_UI;
+	art->listener = file_ui_area_listener;
+	art->init = file_ui_area_init;
+	art->draw = file_ui_area_draw;
 	BLI_addhead(&st->regiontypes, art);
 
 	/* regions: channels (directories) */
-	art= MEM_callocN(sizeof(ARegionType), "spacetype file region");
+	art = MEM_callocN(sizeof(ARegionType), "spacetype file region");
 	art->regionid = RGN_TYPE_CHANNELS;
-	art->prefsizex= 240;
-	art->keymapflag= ED_KEYMAP_UI;
-	art->listener= file_channel_area_listener;
-	art->init= file_channel_area_init;
-	art->draw= file_channel_area_draw;
+	art->prefsizex = 240;
+	art->keymapflag = ED_KEYMAP_UI;
+	art->listener = file_channel_area_listener;
+	art->init = file_channel_area_init;
+	art->draw = file_channel_area_draw;
 	BLI_addhead(&st->regiontypes, art);
 	file_panels_register(art);
 
@@ -613,7 +631,7 @@ void ED_file_init(void)
 {
 	char *cfgdir = BLI_get_folder(BLENDER_USER_CONFIG, NULL);
 	
-	fsmenu_read_system(fsmenu_get());
+	fsmenu_read_system(fsmenu_get(), TRUE);
 
 	if (cfgdir) {
 		char name[FILE_MAX];
@@ -621,12 +639,18 @@ void ED_file_init(void)
 		fsmenu_read_bookmarks(fsmenu_get(), name);
 	}
 	
-	filelist_init_icons();
+	if (G.background == FALSE) {
+		filelist_init_icons();
+	}
+
 	IMB_thumb_makedirs();
 }
 
 void ED_file_exit(void)
 {
 	fsmenu_free(fsmenu_get());
-	filelist_free_icons();
+
+	if (G.background == FALSE) {
+		filelist_free_icons();
+	}
 }
