@@ -28,11 +28,8 @@
  *  \ingroup spimage
  */
 
-
-#include <string.h>
-#include <stdio.h>
-
 #include "DNA_mesh_types.h"
+#include "DNA_mask_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
@@ -41,23 +38,24 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_math.h"
-#include "BLI_rand.h"
-#include "BLI_utildefines.h"
+#include "BLI_threads.h"
 
 #include "BKE_colortools.h"
 #include "BKE_context.h"
 #include "BKE_image.h"
 #include "BKE_global.h"
-#include "BKE_main.h"
-#include "BKE_mesh.h"
 #include "BKE_scene.h"
 #include "BKE_screen.h"
 #include "BKE_tessmesh.h"
+#include "BKE_sequencer.h"
+#include "BKE_node.h"
 
 #include "IMB_imbuf_types.h"
 
 #include "ED_image.h"
+#include "ED_mask.h"
 #include "ED_mesh.h"
+#include "ED_node.h"
 #include "ED_space_api.h"
 #include "ED_screen.h"
 #include "ED_uvedit.h"
@@ -76,236 +74,6 @@
 
 /**************************** common state *****************************/
 
-/* note; image_panel_properties() uses pointer to sima->image directly */
-Image *ED_space_image(SpaceImage *sima)
-{
-	return sima->image;
-}
-
-/* called to assign images to UV faces */
-void ED_space_image_set(SpaceImage *sima, Scene *scene, Object *obedit, Image *ima)
-{
-	/* context may be NULL, so use global */
-	ED_uvedit_assign_image(G.main, scene, obedit, ima, sima->image);
-	
-	/* change the space ima after because uvedit_face_visible_test uses the space ima
-	 * to check if the face is displayed in UV-localview */
-	sima->image = ima;
-	
-	if (ima == NULL || ima->type == IMA_TYPE_R_RESULT || ima->type == IMA_TYPE_COMPOSITE)
-		sima->flag &= ~SI_DRAWTOOL;
-	
-	if (sima->image)
-		BKE_image_signal(sima->image, &sima->iuser, IMA_SIGNAL_USER_NEW_IMAGE);
-	
-	if (sima->image && sima->image->id.us == 0)
-		sima->image->id.us = 1;
-	
-	if (obedit)
-		WM_main_add_notifier(NC_GEOM | ND_DATA, obedit->data);
-
-	WM_main_add_notifier(NC_SPACE | ND_SPACE_IMAGE, NULL);
-}
-
-ImBuf *ED_space_image_acquire_buffer(SpaceImage *sima, void **lock_r)
-{
-	ImBuf *ibuf;
-	
-	if (sima && sima->image) {
-#if 0
-		if (sima->image->type == IMA_TYPE_R_RESULT && BIF_show_render_spare())
-			return BIF_render_spare_imbuf();
-		else
-#endif
-		ibuf = BKE_image_acquire_ibuf(sima->image, &sima->iuser, lock_r);
-		
-		if (ibuf && (ibuf->rect || ibuf->rect_float))
-			return ibuf;
-	}
-	
-	return NULL;
-}
-
-void ED_space_image_release_buffer(SpaceImage *sima, void *lock)
-{
-	if (sima && sima->image)
-		BKE_image_release_ibuf(sima->image, lock);
-}
-
-int ED_space_image_has_buffer(SpaceImage *sima)
-{
-	ImBuf *ibuf;
-	void *lock;
-	int has_buffer;
-	
-	ibuf = ED_space_image_acquire_buffer(sima, &lock);
-	has_buffer = (ibuf != NULL);
-	ED_space_image_release_buffer(sima, lock);
-	
-	return has_buffer;
-}
-
-void ED_image_size(Image *ima, int *width, int *height)
-{
-	ImBuf *ibuf = NULL;
-	void *lock;
-	
-	if (ima)
-		ibuf = BKE_image_acquire_ibuf(ima, NULL, &lock);
-	
-	if (ibuf && ibuf->x > 0 && ibuf->y > 0) {
-		*width = ibuf->x;
-		*height = ibuf->y;
-	}
-	else {
-		*width = 256;
-		*height = 256;
-	}
-	
-	if (ima)
-		BKE_image_release_ibuf(ima, lock);
-}
-
-void ED_space_image_size(SpaceImage *sima, int *width, int *height)
-{
-	Scene *scene = sima->iuser.scene;
-	ImBuf *ibuf;
-	void *lock;
-	
-	ibuf = ED_space_image_acquire_buffer(sima, &lock);
-	
-	if (ibuf && ibuf->x > 0 && ibuf->y > 0) {
-		*width = ibuf->x;
-		*height = ibuf->y;
-	}
-	else if (sima->image && sima->image->type == IMA_TYPE_R_RESULT && scene) {
-		/* not very important, just nice */
-		*width = (scene->r.xsch * scene->r.size) / 100;
-		*height = (scene->r.ysch * scene->r.size) / 100;
-
-		if ((scene->r.mode & R_BORDER) && (scene->r.mode & R_CROP)) {
-			*width *= (scene->r.border.xmax - scene->r.border.xmin);
-			*height *= (scene->r.border.ymax - scene->r.border.ymin);
-		}
-
-	}
-	/* I know a bit weak... but preview uses not actual image size */
-	// XXX else if (image_preview_active(sima, width, height));
-	else {
-		*width = 256;
-		*height = 256;
-	}
-	
-	ED_space_image_release_buffer(sima, lock);
-}
-
-void ED_image_aspect(Image *ima, float *aspx, float *aspy)
-{
-	*aspx = *aspy = 1.0;
-	
-	if ((ima == NULL) || (ima->type == IMA_TYPE_R_RESULT) || (ima->type == IMA_TYPE_COMPOSITE) ||
-	    (ima->aspx == 0.0f || ima->aspy == 0.0f))
-		return;
-	
-	/* x is always 1 */
-	*aspy = ima->aspy / ima->aspx;
-}
-
-void ED_space_image_aspect(SpaceImage *sima, float *aspx, float *aspy)
-{
-	ED_image_aspect(ED_space_image(sima), aspx, aspy);
-}
-
-void ED_space_image_zoom(SpaceImage *sima, ARegion *ar, float *zoomx, float *zoomy)
-{
-	int width, height;
-	
-	ED_space_image_size(sima, &width, &height);
-	
-	*zoomx = (float)(ar->winrct.xmax - ar->winrct.xmin + 1) / (float)((ar->v2d.cur.xmax - ar->v2d.cur.xmin) * width);
-	*zoomy = (float)(ar->winrct.ymax - ar->winrct.ymin + 1) / (float)((ar->v2d.cur.ymax - ar->v2d.cur.ymin) * height);
-}
-
-void ED_space_image_uv_aspect(SpaceImage *sima, float *aspx, float *aspy)
-{
-	int w, h;
-	
-	ED_space_image_aspect(sima, aspx, aspy);
-	ED_space_image_size(sima, &w, &h);
-
-	*aspx *= (float)w;
-	*aspy *= (float)h;
-	
-	if (*aspx < *aspy) {
-		*aspy = *aspy / *aspx;
-		*aspx = 1.0f;
-	}
-	else {
-		*aspx = *aspx / *aspy;
-		*aspy = 1.0f;
-	}
-}
-
-void ED_image_uv_aspect(Image *ima, float *aspx, float *aspy)
-{
-	int w, h;
-	
-	ED_image_aspect(ima, aspx, aspy);
-	ED_image_size(ima, &w, &h);
-	
-	*aspx *= (float)w;
-	*aspy *= (float)h;
-}
-
-int ED_space_image_show_render(SpaceImage *sima)
-{
-	return (sima->image && ELEM(sima->image->type, IMA_TYPE_R_RESULT, IMA_TYPE_COMPOSITE));
-}
-
-int ED_space_image_show_paint(SpaceImage *sima)
-{
-	if (ED_space_image_show_render(sima))
-		return 0;
-	
-	return (sima->flag & SI_DRAWTOOL);
-}
-
-int ED_space_image_show_uvedit(SpaceImage *sima, Object *obedit)
-{
-	if (sima && (ED_space_image_show_render(sima) || ED_space_image_show_paint(sima)))
-		return 0;
-
-	if (obedit && obedit->type == OB_MESH) {
-		struct BMEditMesh *em = BMEdit_FromObject(obedit);
-		int ret;
-		
-		ret = EDBM_mtexpoly_check(em);
-		
-		return ret;
-	}
-	
-	return 0;
-}
-
-int ED_space_image_show_uvshadow(SpaceImage *sima, Object *obedit)
-{
-	if (ED_space_image_show_render(sima))
-		return 0;
-	
-	if (ED_space_image_show_paint(sima))
-		if (obedit && obedit->type == OB_MESH) {
-			struct BMEditMesh *em = BMEdit_FromObject(obedit);
-			int ret;
-			
-			ret = EDBM_mtexpoly_check(em);
-			
-			return ret;
-		}
-	
-	return 0;
-}
-
-
 static void image_scopes_tag_refresh(ScrArea *sa)
 {
 	SpaceImage *sima = (SpaceImage *)sa->spacedata.first;
@@ -382,10 +150,11 @@ static SpaceLink *image_new(const bContext *UNUSED(C))
 	
 	simage = MEM_callocN(sizeof(SpaceImage), "initimage");
 	simage->spacetype = SPACE_IMAGE;
-	simage->zoom = 1;
-	simage->lock = 1;
+	simage->zoom = 1.0f;
+	simage->lock = TRUE;
+	simage->flag = SI_SHOW_GPENCIL | SI_USE_ALPHA;
 
-	simage->iuser.ok = 1;
+	simage->iuser.ok = TRUE;
 	simage->iuser.fie_ima = 2;
 	simage->iuser.frames = 100;
 	
@@ -428,9 +197,7 @@ static SpaceLink *image_new(const bContext *UNUSED(C))
 static void image_free(SpaceLink *sl)
 {	
 	SpaceImage *simage = (SpaceImage *) sl;
-	
-	if (simage->cumap)
-		curvemapping_free(simage->cumap);
+
 	scopes_free(&simage->scopes);
 }
 
@@ -450,8 +217,6 @@ static SpaceLink *image_duplicate(SpaceLink *sl)
 	SpaceImage *simagen = MEM_dupallocN(sl);
 	
 	/* clear or remove stuff from old */
-	if (simagen->cumap)
-		simagen->cumap = curvemapping_copy(simagen->cumap);
 
 	scopes_new(&simagen->scopes);
 
@@ -471,6 +236,7 @@ static void image_operatortypes(void)
 
 	WM_operatortype_append(IMAGE_OT_new);
 	WM_operatortype_append(IMAGE_OT_open);
+	WM_operatortype_append(IMAGE_OT_match_movie_length);
 	WM_operatortype_append(IMAGE_OT_replace);
 	WM_operatortype_append(IMAGE_OT_reload);
 	WM_operatortype_append(IMAGE_OT_save);
@@ -487,8 +253,6 @@ static void image_operatortypes(void)
 	WM_operatortype_append(IMAGE_OT_sample_line);
 	WM_operatortype_append(IMAGE_OT_curves_point_set);
 
-	WM_operatortype_append(IMAGE_OT_record_composite);
-
 	WM_operatortype_append(IMAGE_OT_properties);
 	WM_operatortype_append(IMAGE_OT_scopes);
 }
@@ -527,10 +291,16 @@ static void image_keymap(struct wmKeyConfig *keyconf)
 	WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom_out", PADMINUS, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom", MIDDLEMOUSE, KM_PRESS, KM_CTRL, 0);
 	WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom", MOUSEZOOM, 0, 0, 0);
+	WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom", MOUSEPAN, 0, KM_CTRL, 0);
 
+	/* ctrl now works as well, shift + numpad works as arrow keys on Windows */
+	RNA_float_set(WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom_ratio", PAD8, KM_PRESS, KM_CTRL, 0)->ptr, "ratio", 8.0f);
+	RNA_float_set(WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom_ratio", PAD4, KM_PRESS, KM_CTRL, 0)->ptr, "ratio", 4.0f);
+	RNA_float_set(WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom_ratio", PAD2, KM_PRESS, KM_CTRL, 0)->ptr, "ratio", 2.0f);
 	RNA_float_set(WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom_ratio", PAD8, KM_PRESS, KM_SHIFT, 0)->ptr, "ratio", 8.0f);
 	RNA_float_set(WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom_ratio", PAD4, KM_PRESS, KM_SHIFT, 0)->ptr, "ratio", 4.0f);
 	RNA_float_set(WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom_ratio", PAD2, KM_PRESS, KM_SHIFT, 0)->ptr, "ratio", 2.0f);
+
 	RNA_float_set(WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom_ratio", PAD1, KM_PRESS, 0, 0)->ptr, "ratio", 1.0f);
 	RNA_float_set(WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom_ratio", PAD2, KM_PRESS, 0, 0)->ptr, "ratio", 0.5f);
 	RNA_float_set(WM_keymap_add_item(keymap, "IMAGE_OT_view_zoom_ratio", PAD4, KM_PRESS, 0, 0)->ptr, "ratio", 0.25f);
@@ -547,10 +317,23 @@ static void image_keymap(struct wmKeyConfig *keyconf)
 
 	/* fast switch to render slots */
 	for (i = 0; i < MAX2(IMA_MAX_RENDER_SLOT, 9); i++) {
-		kmi = WM_keymap_add_item(keymap, "WM_OT_context_set_int", ONEKEY+i, KM_PRESS, 0, 0);
+		kmi = WM_keymap_add_item(keymap, "WM_OT_context_set_int", ONEKEY + i, KM_PRESS, 0, 0);
 		RNA_string_set(kmi->ptr, "data_path", "space_data.image.render_slot");
 		RNA_int_set(kmi->ptr, "value", i);
 	}
+
+	/* pivot */
+	kmi = WM_keymap_add_item(keymap, "WM_OT_context_set_enum", COMMAKEY, KM_PRESS, 0, 0);
+	RNA_string_set(kmi->ptr, "data_path", "space_data.pivot_point");
+	RNA_string_set(kmi->ptr, "value", "CENTER");
+
+	kmi = WM_keymap_add_item(keymap, "WM_OT_context_set_enum", COMMAKEY, KM_PRESS, KM_CTRL, 0);
+	RNA_string_set(kmi->ptr, "data_path", "space_data.pivot_point");
+	RNA_string_set(kmi->ptr, "value", "MEDIAN");
+
+	kmi = WM_keymap_add_item(keymap, "WM_OT_context_set_enum", PERIODKEY, KM_PRESS, 0, 0);
+	RNA_string_set(kmi->ptr, "data_path", "space_data.pivot_point");
+	RNA_string_set(kmi->ptr, "value", "CURSOR");
 }
 
 /* dropboxes */
@@ -576,30 +359,47 @@ static void image_dropboxes(void)
 	WM_dropbox_add(lb, "IMAGE_OT_open", image_drop_poll, image_drop_copy);
 }
 
-
-
-static void image_refresh(const bContext *C, ScrArea *UNUSED(sa))
+/**
+ * \note take care not to get into feedback loop here,
+ *       calling composite job causes viewer to refresh.
+ */
+static void image_refresh(const bContext *C, ScrArea *sa)
 {
 	Scene *scene = CTX_data_scene(C);
-	SpaceImage *sima = CTX_wm_space_image(C);
+	SpaceImage *sima = sa->spacedata.first;
 	Object *obedit = CTX_data_edit_object(C);
 	Image *ima;
 
 	ima = ED_space_image(sima);
 
-	if (sima->iuser.flag & IMA_ANIM_ALWAYS)
-		BKE_image_user_calc_frame(&sima->iuser, scene->r.cfra, 0);
+	BKE_image_user_check_frame_calc(&sima->iuser, scene->r.cfra, 0);
 	
 	/* check if we have to set the image from the editmesh */
-	if (ima && (ima->source == IMA_SRC_VIEWER || sima->pin)) ;
+	if (ima && (ima->source == IMA_SRC_VIEWER && sima->mode == SI_MODE_MASK)) {
+		if (sima->lock == FALSE && G.moving) {
+			/* pass */
+		}
+		else {
+			if (scene->nodetree) {
+				Mask *mask = ED_space_image_get_mask(sima);
+				if (mask) {
+					ED_node_composite_job(C, scene->nodetree, scene);
+				}
+			}
+		}
+	}
+	else if (ima && (ima->source == IMA_SRC_VIEWER || sima->pin)) {
+		/* pass */
+	}
 	else if (obedit && obedit->type == OB_MESH) {
 		Mesh *me = (Mesh *)obedit->data;
 		struct BMEditMesh *em = me->edit_btmesh;
-		int sloppy = 1; /* partially selected face is ok */
+		int sloppy = TRUE; /* partially selected face is ok */
+		int selected = !(scene->toolsettings->uv_flag & UV_SYNC_SELECTION); /* only selected active face? */
 
-		if (scene_use_new_shading_nodes(scene)) {
+		if (BKE_scene_use_new_shading_nodes(scene)) {
 			/* new shading system, get image from material */
-			BMFace *efa = BM_active_face_get(em->bm, sloppy);
+			BMFace *efa = BM_active_face_get(em->bm, sloppy, selected);
 
 			if (efa) {
 				Image *node_ima;
@@ -614,16 +414,15 @@ static void image_refresh(const bContext *C, ScrArea *UNUSED(sa))
 			MTexPoly *tf;
 			
 			if (em && EDBM_mtexpoly_check(em)) {
-				sima->image = NULL;
-				
-				tf = EDBM_mtexpoly_active_get(em, NULL, TRUE); /* partially selected face is ok */
-				
+				tf = EDBM_mtexpoly_active_get(em, NULL, sloppy, selected);
+
 				if (tf) {
 					/* don't need to check for pin here, see above */
 					sima->image = tf->tpage;
 					
-					if (sima->flag & SI_EDITTILE) ;
-					else sima->curtile = tf->tile;
+					if ((sima->flag & SI_EDITTILE) == 0) {
+						sima->curtile = tf->tile;
+					}
 				}
 			}
 		}
@@ -641,15 +440,19 @@ static void image_listener(ScrArea *sa, wmNotifier *wmn)
 				case ND_FRAME:
 					image_scopes_tag_refresh(sa);
 					ED_area_tag_refresh(sa);
-					ED_area_tag_redraw(sa);					
+					ED_area_tag_redraw(sa);
 					break;
 				case ND_MODE:
+					if (wmn->subtype == NS_EDITMODE_MESH)
+						ED_area_tag_refresh(sa);
+					ED_area_tag_redraw(sa);
+					break;
 				case ND_RENDER_RESULT:
+				case ND_RENDER_OPTIONS:
 				case ND_COMPO_RESULT:
 					if (ED_space_image_show_render(sima))
 						image_scopes_tag_refresh(sa);
-					ED_area_tag_refresh(sa);
-					ED_area_tag_redraw(sa);					
+					ED_area_tag_redraw(sa);
 					break;
 			}
 			break;
@@ -660,12 +463,41 @@ static void image_listener(ScrArea *sa, wmNotifier *wmn)
 				ED_area_tag_redraw(sa);
 			}
 			break;
-		case NC_SPACE:	
+		case NC_SPACE:
 			if (wmn->data == ND_SPACE_IMAGE) {
 				image_scopes_tag_refresh(sa);
 				ED_area_tag_redraw(sa);
 			}
 			break;
+		case NC_MASK:
+		{
+			// Scene *scene = wmn->window->screen->scene;
+			/* ideally would check for: ED_space_image_check_show_maskedit(scene, sima) but we cant get the scene */
+			if (sima->mode == SI_MODE_MASK) {
+				switch (wmn->data) {
+					case ND_SELECT:
+						ED_area_tag_redraw(sa);
+						break;
+					case ND_DATA:
+					case ND_DRAW:
+						/* causes node-recalc */
+						ED_area_tag_redraw(sa);
+						ED_area_tag_refresh(sa);
+						break;
+				}
+				switch (wmn->action) {
+					case NA_SELECTED:
+						ED_area_tag_redraw(sa);
+						break;
+					case NA_EDITED:
+						/* causes node-recalc */
+						ED_area_tag_redraw(sa);
+						ED_area_tag_refresh(sa);
+						break;
+				}
+			}
+			break;
+		}
 		case NC_GEOM:
 			switch (wmn->data) {
 				case ND_DATA:
@@ -691,7 +523,7 @@ static void image_listener(ScrArea *sa, wmNotifier *wmn)
 	}
 }
 
-const char *image_context_dir[] = {"edit_image", NULL};
+const char *image_context_dir[] = {"edit_image", "edit_mask", NULL};
 
 static int image_context(const bContext *C, const char *member, bContextDataResult *result)
 {
@@ -704,7 +536,13 @@ static int image_context(const bContext *C, const char *member, bContextDataResu
 		CTX_data_id_pointer_set(result, (ID *)ED_space_image(sima));
 		return 1;
 	}
-
+	else if (CTX_data_equals(member, "edit_mask")) {
+		Mask *mask = ED_space_image_get_mask(sima);
+		if (mask) {
+			CTX_data_id_pointer_set(result, &mask->id);
+		}
+		return TRUE;
+	}
 	return 0;
 }
 
@@ -718,10 +556,10 @@ static void image_main_area_set_view2d(SpaceImage *sima, ARegion *ar)
 	int width, height, winx, winy;
 	
 #if 0
-	if (image_preview_active(curarea, &width, &height)) ;
+	if (image_preview_active(curarea, &width, &height)) {}
 	else
 #endif
-	ED_space_image_size(sima, &width, &height);
+	ED_space_image_get_size(sima, &width, &height);
 
 	w = width;
 	h = height;
@@ -729,8 +567,8 @@ static void image_main_area_set_view2d(SpaceImage *sima, ARegion *ar)
 	if (ima)
 		h *= ima->aspy / ima->aspx;
 
-	winx = ar->winrct.xmax - ar->winrct.xmin + 1;
-	winy = ar->winrct.ymax - ar->winrct.ymin + 1;
+	winx = BLI_rcti_size_x(&ar->winrct) + 1;
+	winy = BLI_rcti_size_y(&ar->winrct) + 1;
 		
 	ar->v2d.tot.xmin = 0;
 	ar->v2d.tot.ymin = 0;
@@ -771,6 +609,10 @@ static void image_main_area_init(wmWindowManager *wm, ARegion *ar)
 	// image space manages own v2d
 	// UI_view2d_region_reinit(&ar->v2d, V2D_COMMONVIEW_STANDARD, ar->winx, ar->winy);
 
+	/* mask polls mode */
+	keymap = WM_keymap_find(wm->defaultconf, "Mask Editing", 0, 0);
+	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
+
 	/* image paint polls for mode */
 	keymap = WM_keymap_find(wm->defaultconf, "Image Paint", 0, 0);
 	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
@@ -795,6 +637,7 @@ static void image_main_area_draw(const bContext *C, ARegion *ar)
 	SpaceImage *sima = CTX_wm_space_image(C);
 	Object *obact = CTX_data_active_object(C);
 	Object *obedit = CTX_data_edit_object(C);
+	Mask *mask = NULL;
 	Scene *scene = CTX_data_scene(C);
 	View2D *v2d = &ar->v2d;
 	//View2DScrollers *scrollers;
@@ -813,24 +656,72 @@ static void image_main_area_draw(const bContext *C, ARegion *ar)
 
 	/* we set view2d from own zoom and offset each time */
 	image_main_area_set_view2d(sima, ar);
-	
+
 	/* we draw image in pixelspace */
-	draw_image_main(sima, ar, scene);
+	draw_image_main(C, ar);
 
 	/* and uvs in 0.0-1.0 space */
 	UI_view2d_view_ortho(v2d);
 	draw_uvedit_main(sima, ar, scene, obedit, obact);
 
+	/* check for mask (delay draw) */
+	if (ED_space_image_show_uvedit(sima, obedit)) {
+		/* pass */
+	}
+	else if (sima->mode == SI_MODE_MASK) {
+		mask = ED_space_image_get_mask(sima);
+		draw_image_cursor(sima, ar);
+	}
+
 	ED_region_draw_cb_draw(C, ar, REGION_DRAW_POST_VIEW);
-		
-	/* Grease Pencil too (in addition to UV's) */
-	draw_image_grease_pencil((bContext *)C, 1); 
+
+	if (sima->flag & SI_SHOW_GPENCIL) {
+		/* Grease Pencil too (in addition to UV's) */
+		draw_image_grease_pencil((bContext *)C, TRUE);
+	}
+
+	/* sample line */
+	draw_image_sample_line(sima);
 
 	UI_view2d_view_restore(C);
 
-	/* draw Grease Pencil - screen space only */
-	draw_image_grease_pencil((bContext *)C, 0);
-	
+	if (sima->flag & SI_SHOW_GPENCIL) {
+		/* draw Grease Pencil - screen space only */
+		draw_image_grease_pencil((bContext *)C, FALSE);
+	}
+
+	if (mask) {
+		Image *image = ED_space_image(sima);
+		int width, height, show_viewer;
+		float aspx, aspy;
+
+		show_viewer = (image && image->source == IMA_SRC_VIEWER);
+
+		if (show_viewer) {
+			/* ED_space_image_get* will acquire image buffer which requires
+			 * lock here by the same reason why lock is needed in draw_image_main
+			 */
+			BLI_lock_thread(LOCK_DRAW_IMAGE);
+		}
+
+		ED_space_image_get_size(sima, &width, &height);
+		ED_space_image_get_aspect(sima, &aspx, &aspy);
+
+		if (show_viewer)
+			BLI_unlock_thread(LOCK_DRAW_IMAGE);
+
+		ED_mask_draw_region(mask, ar,
+		                    sima->mask_info.draw_flag, sima->mask_info.draw_type,
+		                    width, height,
+		                    aspx, aspy,
+		                    TRUE, FALSE,
+		                    NULL, C);
+
+		ED_mask_draw_frames(mask, ar, CFRA, mask->sfra, mask->efra);
+
+		draw_image_cursor(sima, ar);
+	}
+
 	/* scrollers? */
 #if 0
 	scrollers = UI_view2d_scrollers_calc(C, v2d, V2D_UNIT_VALUES, V2D_GRID_CLAMP, V2D_ARG_DUMMY, V2D_ARG_DUMMY);
@@ -843,8 +734,8 @@ static void image_main_area_listener(ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
 	switch (wmn->category) {
-		case NC_SCREEN:
-			if (wmn->data == ND_GPENCIL)
+		case NC_GPENCIL:
+			if (wmn->action == NA_EDITED)
 				ED_region_tag_redraw(ar);
 			break;
 	}
@@ -857,6 +748,7 @@ static void image_buttons_area_init(wmWindowManager *wm, ARegion *ar)
 {
 	wmKeyMap *keymap;
 
+	ar->v2d.scroll = V2D_SCROLL_RIGHT | V2D_SCROLL_VERTICAL_HIDE;
 	ED_region_panels_init(wm, ar);
 	
 	keymap = WM_keymap_find(wm->defaultconf, "Image Generic", SPACE_IMAGE, 0);
@@ -872,8 +764,8 @@ static void image_buttons_area_listener(ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
 	switch (wmn->category) {
-		case NC_SCREEN:
-			if (wmn->data == ND_GPENCIL)
+		case NC_GPENCIL:
+			if (wmn->data == ND_DATA)
 				ED_region_tag_redraw(ar);
 			break;
 		case NC_BRUSH:
@@ -896,6 +788,7 @@ static void image_scope_area_init(wmWindowManager *wm, ARegion *ar)
 {
 	wmKeyMap *keymap;
 	
+	ar->v2d.scroll = V2D_SCROLL_RIGHT | V2D_SCROLL_VERTICAL_HIDE;
 	ED_region_panels_init(wm, ar);
 	
 	keymap = WM_keymap_find(wm->defaultconf, "Image Generic", SPACE_IMAGE, 0);
@@ -909,9 +802,12 @@ static void image_scope_area_draw(const bContext *C, ARegion *ar)
 	void *lock;
 	ImBuf *ibuf = ED_space_image_acquire_buffer(sima, &lock);
 	if (ibuf) {
-		scopes_update(&sima->scopes, ibuf, scene->r.color_mgt_flag & R_COLOR_MANAGEMENT);
+		if (!sima->scopes.ok) {
+			BKE_histogram_update_sample_line(&sima->sample_line_hist, ibuf, &scene->view_settings, &scene->display_settings);
+		}
+		scopes_update(&sima->scopes, ibuf, &scene->view_settings, &scene->display_settings);
 	}
-	ED_space_image_release_buffer(sima, lock);
+	ED_space_image_release_buffer(sima, ibuf, lock);
 	
 	ED_region_panels(C, ar, 1, NULL, -1);
 }
