@@ -62,22 +62,25 @@ int info_report_mask(SpaceInfo *UNUSED(sinfo))
 	return report_mask;
 #endif
 
-	return RPT_DEBUG_ALL | RPT_INFO_ALL | RPT_OPERATOR_ALL | RPT_WARNING_ALL | RPT_ERROR_ALL;
+	return RPT_DEBUG_ALL | RPT_INFO_ALL | RPT_OPERATOR_ALL | RPT_PROPERTY_ALL | RPT_WARNING_ALL | RPT_ERROR_ALL;
 }
 
 // TODO, get this working again!
 static int report_replay_exec(bContext *C, wmOperator *UNUSED(op))
 {
-//	SpaceInfo *sc= CTX_wm_space_info(C);
-//	ReportList *reports= CTX_wm_reports(C);
-//	int report_mask= info_report_mask(sc);
+//	SpaceInfo *sc = CTX_wm_space_info(C);
+//	ReportList *reports = CTX_wm_reports(C);
+//	int report_mask = info_report_mask(sc);
 //	Report *report;
 
 #if 0
 	sc->type = CONSOLE_TYPE_PYTHON;
 
 	for (report = reports->list.last; report; report = report->prev) {
-		if ((report->type & report_mask) && (report->type & RPT_OPERATOR_ALL) && (report->flag & SELECT)) {
+		if ((report->type & report_mask) &&
+		    (report->type & RPT_OPERATOR_ALL | RPT_PROPERTY_ALL) &&
+		    (report->flag & SELECT))
+		{
 			console_history_add_str(sc, report->message, 0);
 			WM_operator_name_call(C, "CONSOLE_OT_execute", WM_OP_EXEC_DEFAULT, NULL);
 
@@ -145,7 +148,7 @@ static int select_report_pick_invoke(bContext *C, wmOperator *op, wmEvent *event
 void INFO_OT_select_pick(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Select report";
+	ot->name = "Select Report";
 	ot->description = "Select reports by index";
 	ot->idname = "INFO_OT_select_pick";
 
@@ -223,7 +226,7 @@ static int borderselect_exec(bContext *C, wmOperator *op)
 	int extend = RNA_boolean_get(op->ptr, "extend");
 	Report *report_min, *report_max, *report;
 
-	//View2D *v2d= UI_view2d_fromcontext(C);
+	//View2D *v2d = UI_view2d_fromcontext(C);
 
 
 	rcti rect;
@@ -231,10 +234,7 @@ static int borderselect_exec(bContext *C, wmOperator *op)
 	short selecting = (RNA_int_get(op->ptr, "gesture_mode") == GESTURE_MODAL_SELECT);
 	//int mval[2];
 
-	rect.xmin = RNA_int_get(op->ptr, "xmin");
-	rect.ymin = RNA_int_get(op->ptr, "ymin");
-	rect.xmax = RNA_int_get(op->ptr, "xmax");
-	rect.ymax = RNA_int_get(op->ptr, "ymax");
+	WM_operator_properties_border_to_rcti(op, &rect);
 
 #if 0
 	mval[0] = rect.xmin;
