@@ -39,6 +39,9 @@
 #include "DNA_scene_types.h"
 
 #include "BLI_utildefines.h"
+#include "BLI_math.h"
+
+#include "BLF_translation.h"
 
 #include "BKE_anim.h"
 #include "BKE_blender.h"
@@ -47,6 +50,7 @@
 #include "BKE_DerivedMesh.h"
 #include "BKE_key.h"
 #include "BKE_mesh.h"
+#include "BKE_paint.h"
 #include "BKE_particle.h"
 #include "BKE_tessmesh.h"
 
@@ -60,8 +64,9 @@ typedef struct SceneStats {
 	int totedge, totedgesel;
 	int totface, totfacesel;
 	int totbone, totbonesel;
-	int totobj, totobjsel;
-	int totmesh, totlamp, totcurve;
+	int totobj,  totobjsel;
+	int totlamp, totlampsel; 
+	int tottri, totmesh;
 
 	char infostr[512];
 } SceneStats;
@@ -69,10 +74,11 @@ typedef struct SceneStats {
 static void stats_object(Object *ob, int sel, int totob, SceneStats *stats)
 {
 	switch (ob->type) {
-		case OB_MESH: {
+		case OB_MESH:
+		{
 			/* we assume derivedmesh is already built, this strictly does stats now. */
 			DerivedMesh *dm = ob->derivedFinal;
-			int totvert, totedge, totface;
+			int totvert, totedge, totface, totloop;
 
 			stats->totmesh += totob;
 
@@ -80,10 +86,12 @@ static void stats_object(Object *ob, int sel, int totob, SceneStats *stats)
 				totvert = dm->getNumVerts(dm);
 				totedge = dm->getNumEdges(dm);
 				totface = dm->getNumPolys(dm);
+				totloop = dm->getNumLoops(dm);
 
 				stats->totvert += totvert * totob;
 				stats->totedge += totedge * totob;
 				stats->totface += totface * totob;
+				stats->tottri  += poly_to_tri_count(totface, totloop) * totob;
 
 				if (sel) {
 					stats->totvertsel += totvert;
@@ -94,42 +102,30 @@ static void stats_object(Object *ob, int sel, int totob, SceneStats *stats)
 		}
 		case OB_LAMP:
 			stats->totlamp += totob;
+			if (sel) {
+				stats->totlampsel += totob;
+			}
 			break;
 		case OB_SURF:
 		case OB_CURVE:
-		case OB_FONT: {
-			int tot = 0, totf = 0;
-
-			stats->totcurve += totob;
+		case OB_FONT:
+		case OB_MBALL:
+		{
+			int totv = 0, totf = 0, tottri = 0;
 
 			if (ob->disp.first)
-				count_displist(&ob->disp, &tot, &totf);
+				BKE_displist_count(&ob->disp, &totv, &totf, &tottri);
 
-			tot *= totob;
-			totf *= totob;
+			totv   *= totob;
+			totf   *= totob;
+			tottri *= totob;
 
-			stats->totvert += tot;
+			stats->totvert += totv;
 			stats->totface += totf;
+			stats->tottri  += tottri;
 
 			if (sel) {
-				stats->totvertsel += tot;
-				stats->totfacesel += totf;
-			}
-			break;
-		}
-		case OB_MBALL: {
-			int tot = 0, totf = 0;
-
-			count_displist(&ob->disp, &tot, &totf);
-
-			tot *= totob;
-			totf *= totob;
-
-			stats->totvert += tot;
-			stats->totface += totf;
-
-			if (sel) {
-				stats->totvertsel += tot;
+				stats->totvertsel += totv;
 				stats->totfacesel += totf;
 			}
 			break;
@@ -150,6 +146,8 @@ static void stats_object_edit(Object *obedit, SceneStats *stats)
 		
 		stats->totface = em->bm->totface;
 		stats->totfacesel = em->bm->totfacesel;
+
+		stats->tottri = em->tottri;
 	}
 	else if (obedit->type == OB_ARMATURE) {
 		/* Armature Edit */
@@ -170,8 +168,11 @@ static void stats_object_edit(Object *obedit, SceneStats *stats)
 			if (ebo->flag & BONE_SELECTED) stats->totbonesel++;
 
 			/* if this is a connected child and it's parent is being moved, remove our root */
-			if ((ebo->flag & BONE_CONNECTED) && (ebo->flag & BONE_ROOTSEL) && ebo->parent && (ebo->parent->flag & BONE_TIPSEL))
+			if ((ebo->flag & BONE_CONNECTED) && (ebo->flag & BONE_ROOTSEL) &&
+			    ebo->parent && (ebo->parent->flag & BONE_TIPSEL))
+			{
 				stats->totvertsel--;
+			}
 
 			stats->totvert += 2;
 		}
@@ -183,7 +184,7 @@ static void stats_object_edit(Object *obedit, SceneStats *stats)
 		BezTriple *bezt;
 		BPoint *bp;
 		int a;
-		ListBase *nurbs = curve_editnurbs(cu);
+		ListBase *nurbs = BKE_curve_editNurbs_get(cu);
 
 		for (nu = nurbs->first; nu; nu = nu->next) {
 			if (nu->type == CU_BEZIER) {
@@ -251,6 +252,12 @@ static void stats_object_pose(Object *ob, SceneStats *stats)
 	}
 }
 
+static void stats_object_sculpt_dynamic_topology(Object *ob, SceneStats *stats)
+{
+	stats->totvert = ob->sculpt->bm->totvert;
+	stats->tottri = ob->sculpt->bm->totface;
+}
+
 static void stats_dupli_object(Base *base, Object *ob, SceneStats *stats)
 {
 	if (base->flag & SELECT) stats->totobjsel++;
@@ -310,6 +317,12 @@ static void stats_dupli_object(Base *base, Object *ob, SceneStats *stats)
 	}
 }
 
+static int stats_is_object_dynamic_topology_sculpt(Object *ob)
+{
+	return (ob && (ob->mode & OB_MODE_SCULPT) &&
+	        ob->sculpt && ob->sculpt->bm);
+}
+
 /* Statistics displayed in info header. Called regularly on scene changes. */
 static void stats_update(Scene *scene)
 {
@@ -325,6 +338,10 @@ static void stats_update(Scene *scene)
 		/* Pose Mode */
 		stats_object_pose(ob, &stats);
 	}
+	else if (stats_is_object_dynamic_topology_sculpt(ob)) {
+		/* Dynamic-topology sculpt mode */
+		stats_object_sculpt_dynamic_topology(ob, &stats);
+	}
 	else {
 		/* Objects */
 		for (base = scene->base.first; base; base = base->next)
@@ -359,35 +376,35 @@ static void stats_string(Scene *scene)
 	s += sprintf(s, "%s | ", versionstr);
 
 	if (scene->obedit) {
-		if (ob_get_keyblock(scene->obedit))
+		if (BKE_keyblock_from_object(scene->obedit))
 			s += sprintf(s, "(Key) ");
 
 		if (scene->obedit->type == OB_MESH) {
-			if (scene->toolsettings->selectmode & SCE_SELECT_VERTEX)
-				s += sprintf(s, "Ve:%d-%d | Ed:%d-%d | Fa:%d-%d",
-				             stats->totvertsel, stats->totvert, stats->totedgesel, stats->totedge, stats->totfacesel, stats->totface);
-			else if (scene->toolsettings->selectmode & SCE_SELECT_EDGE)
-				s += sprintf(s, "Ed:%d-%d | Fa:%d-%d",
-				             stats->totedgesel, stats->totedge, stats->totfacesel, stats->totface);
-			else
-				s += sprintf(s, "Fa:%d-%d", stats->totfacesel, stats->totface);
+			s += sprintf(s, "Verts:%d/%d | Edges:%d/%d | Faces:%d/%d | Tris:%d",
+		                 stats->totvertsel, stats->totvert, stats->totedgesel, stats->totedge, stats->totfacesel,
+		                 stats->totface, stats->tottri);
 		}
 		else if (scene->obedit->type == OB_ARMATURE) {
-			s += sprintf(s, "Ve:%d-%d | Bo:%d-%d", stats->totvertsel, stats->totvert, stats->totbonesel, stats->totbone);
+			s += sprintf(s, "Verts:%d/%d | Bones:%d/%d", stats->totvertsel, stats->totvert, stats->totbonesel,
+			             stats->totbone);
 		}
 		else {
-			s += sprintf(s, "Ve:%d-%d", stats->totvertsel, stats->totvert);
+			s += sprintf(s, "Verts:%d/%d", stats->totvertsel, stats->totvert);
 		}
 
 		strcat(s, memstr);
 	}
 	else if (ob && (ob->mode & OB_MODE_POSE)) {
-		s += sprintf(s, "Bo:%d-%d %s",
+		s += sprintf(s, "Bones:%d/%d %s",
 		             stats->totbonesel, stats->totbone, memstr);
 	}
+	else if (stats_is_object_dynamic_topology_sculpt(ob)) {
+		s += sprintf(s, "Verts:%d | Tris:%d", stats->totvert, stats->tottri);
+	}
 	else {
-		s += sprintf(s, "Ve:%d | Fa:%d | Ob:%d-%d | La:%d%s",
-		             stats->totvert, stats->totface, stats->totobjsel, stats->totobj, stats->totlamp, memstr);
+		s += sprintf(s, "Verts:%d | Faces:%d | Tris:%d | Objects:%d/%d | Lamps:%d/%d%s",
+		             stats->totvert, stats->totface, stats->tottri, stats->totobjsel, stats->totobj, stats->totlampsel,
+		             stats->totlamp, memstr);
 	}
 
 	if (ob)
@@ -410,4 +427,3 @@ const char *ED_info_stats_string(Scene *scene)
 
 	return scene->stats->infostr;
 }
-
