@@ -36,8 +36,6 @@
 #include "BLI_math.h"
 #include "BLI_utildefines.h"
 
-
-
 #include "BIF_gl.h"
 #include "BIF_glutil.h"
 
@@ -47,7 +45,8 @@
 
 static void console_font_begin(TextViewContext *sc)
 {
-	BLF_size(blf_mono_font, sc->lheight - 2, 72);
+	/* 0.875 is based on: 16 pixels lines get 14 pixel text */
+	BLF_size(blf_mono_font, 0.875 * sc->lheight, 72);
 }
 
 typedef struct ConsoleDrawContext {
@@ -63,32 +62,35 @@ typedef struct ConsoleDrawContext {
 	int draw;
 } ConsoleDrawContext;
 
-static void console_draw_sel(int sel[2], int xy[2], int str_len_draw, int cwidth, int lheight)
+BLI_INLINE void console_step_sel(ConsoleDrawContext *cdc, const int step)
+{
+	cdc->sel[0] += step;
+	cdc->sel[1] += step;
+}
+
+static void console_draw_sel(const int sel[2], const int xy[2], const int str_len_draw, int cwidth, int lheight,
+                             const unsigned char bg_sel[4])
 {
 	if (sel[0] <= str_len_draw && sel[1] >= 0) {
-		int sta = MAX2(sel[0], 0);
-		int end = MIN2(sel[1], str_len_draw);
+		const int sta = max_ii(sel[0], 0);
+		const int end = min_ii(sel[1], str_len_draw);
 
-		glEnable(GL_POLYGON_STIPPLE);
-		glPolygonStipple(stipple_halftone);
 		glEnable(GL_BLEND);
 		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-		glColor4ub(255, 255, 255, 96);
+		glColor4ubv(bg_sel);
 
 		glRecti(xy[0] + (cwidth * sta), xy[1] - 2 + lheight, xy[0] + (cwidth * end), xy[1] - 2);
 
-		glDisable(GL_POLYGON_STIPPLE);
 		glDisable(GL_BLEND);
 	}
 }
 
-
 /* return 0 if the last line is off the screen
  * should be able to use this for any string type */
 
-static int console_draw_string(ConsoleDrawContext *cdc, const char *str, int str_len, unsigned char *fg, unsigned char *bg)
+static int console_draw_string(ConsoleDrawContext *cdc, const char *str, const int str_len,
+                               const unsigned char fg[3], const unsigned char bg[3], const unsigned char bg_sel[4])
 {
-#define STEP_SEL(value) cdc->sel[0] += (value); cdc->sel[1] += (value)
 	int rct_ofs = cdc->lheight / 4;
 	int tot_lines = (str_len / cdc->console_width) + 1; /* total number of lines for wrapping */
 	int y_next = (str_len > cdc->console_width) ? cdc->xy[1] + cdc->lheight * tot_lines : cdc->xy[1] + cdc->lheight;
@@ -103,7 +105,8 @@ static int console_draw_string(ConsoleDrawContext *cdc, const char *str, int str
 
 					/* wrap */
 					if (str_len > cdc->console_width)
-						ofs += (cdc->console_width * ((int)((((float)(y_next - cdc->mval[1]) / (float)(y_next - cdc->xy[1])) * tot_lines))));
+						ofs += cdc->console_width * ((int)((((float)(y_next - cdc->mval[1]) /
+						                                     (float)(y_next - cdc->xy[1])) * tot_lines)));
 	
 					CLAMP(ofs, 0, str_len);
 					*cdc->pos_pick += str_len - ofs;
@@ -122,13 +125,13 @@ static int console_draw_string(ConsoleDrawContext *cdc, const char *str, int str
 
 		/* adjust selection even if not drawing */
 		if (cdc->sel[0] != cdc->sel[1]) {
-			STEP_SEL(-(str_len + 1));
+			console_step_sel(cdc, -(str_len + 1));
 		}
 
 		return 1;
 	}
 
-	if (str_len > cdc->console_width) { /* wrap? */
+	if (tot_lines > 1) { /* wrap? */
 		const int initial_offset = ((tot_lines - 1) * cdc->console_width);
 		const char *line_stride = str + initial_offset;  /* advance to the last line and draw it first */
 		
@@ -151,10 +154,10 @@ static int console_draw_string(ConsoleDrawContext *cdc, const char *str, int str
 		BLF_draw(mono, line_stride, str_len - initial_offset);
 
 		if (cdc->sel[0] != cdc->sel[1]) {
-			STEP_SEL(-initial_offset);
+			console_step_sel(cdc, -initial_offset);
 			// glColor4ub(255, 0, 0, 96); // debug
-			console_draw_sel(cdc->sel, cdc->xy, str_len % cdc->console_width, cdc->cwidth, cdc->lheight);
-			STEP_SEL(cdc->console_width);
+			console_draw_sel(cdc->sel, cdc->xy, str_len % cdc->console_width, cdc->cwidth, cdc->lheight, bg_sel);
+			console_step_sel(cdc, cdc->console_width);
 			glColor3ubv(fg);
 		}
 
@@ -168,8 +171,8 @@ static int console_draw_string(ConsoleDrawContext *cdc, const char *str, int str
 			
 			if (cdc->sel[0] != cdc->sel[1]) {
 				// glColor4ub(0, 255, 0, 96); // debug
-				console_draw_sel(cdc->sel, cdc->xy, cdc->console_width, cdc->cwidth, cdc->lheight);
-				STEP_SEL(cdc->console_width);
+				console_draw_sel(cdc->sel, cdc->xy, cdc->console_width, cdc->cwidth, cdc->lheight, bg_sel);
+				console_step_sel(cdc, cdc->console_width);
 				glColor3ubv(fg);
 			}
 
@@ -181,7 +184,7 @@ static int console_draw_string(ConsoleDrawContext *cdc, const char *str, int str
 		}
 
 		copy_v2_v2_int(cdc->sel, sel_orig);
-		STEP_SEL(-(str_len + 1));
+		console_step_sel(cdc, -(str_len + 1));
 	}
 	else { /* simple, no wrap */
 
@@ -202,8 +205,8 @@ static int console_draw_string(ConsoleDrawContext *cdc, const char *str, int str
 			isel[1] = str_len - cdc->sel[0];
 
 			// glColor4ub(255, 255, 0, 96); // debug
-			console_draw_sel(isel, cdc->xy, str_len, cdc->cwidth, cdc->lheight);
-			STEP_SEL(-(str_len + 1));
+			console_draw_sel(isel, cdc->xy, str_len, cdc->cwidth, cdc->lheight, bg_sel);
+			console_step_sel(cdc, -(str_len + 1));
 		}
 
 		cdc->xy[1] += cdc->lheight;
@@ -213,13 +216,11 @@ static int console_draw_string(ConsoleDrawContext *cdc, const char *str, int str
 	}
 
 	return 1;
-#undef STEP_SEL
 }
 
 #define CONSOLE_DRAW_MARGIN 4
-#define CONSOLE_DRAW_SCROLL 16
 
-int textview_draw(TextViewContext *tvc, int draw, int mval[2], void **mouse_pick, int *pos_pick)
+int textview_draw(TextViewContext *tvc, const int draw, int mval[2], void **mouse_pick, int *pos_pick)
 {
 	ConsoleDrawContext cdc = {0};
 
@@ -243,9 +244,10 @@ int textview_draw(TextViewContext *tvc, int draw, int mval[2], void **mouse_pick
 	cdc.cwidth = (int)BLF_fixed_width(mono);
 	assert(cdc.cwidth > 0);
 	cdc.lheight = tvc->lheight;
-	cdc.console_width = (tvc->winx - (CONSOLE_DRAW_SCROLL + CONSOLE_DRAW_MARGIN * 2) ) / cdc.cwidth;
+	/* note, scroll bar must be already subtracted () */
+	cdc.console_width = (tvc->winx - (CONSOLE_DRAW_MARGIN * 2) ) / cdc.cwidth;
 	CLAMP(cdc.console_width, 1, INT_MAX); /* avoid divide by zero on small windows */
-	cdc.winx = tvc->winx - (CONSOLE_DRAW_MARGIN + CONSOLE_DRAW_SCROLL);
+	cdc.winx = tvc->winx - CONSOLE_DRAW_MARGIN;
 	cdc.ymin = tvc->ymin;
 	cdc.ymax = tvc->ymax;
 	cdc.xy = xy;
@@ -265,6 +267,11 @@ int textview_draw(TextViewContext *tvc, int draw, int mval[2], void **mouse_pick
 	}
 
 	if (tvc->begin(tvc)) {
+		unsigned char bg_sel[4] = {0};
+
+		if (draw && tvc->const_colors) {
+			tvc->const_colors(tvc, bg_sel);
+		}
 
 		do {
 			const char *ext_line;
@@ -278,7 +285,11 @@ int textview_draw(TextViewContext *tvc, int draw, int mval[2], void **mouse_pick
 
 			tvc->line_get(tvc, &ext_line, &ext_len);
 
-			if (!console_draw_string(&cdc, ext_line, ext_len, (color_flag & TVC_LINE_FG) ? fg : NULL, (color_flag & TVC_LINE_BG) ? bg : NULL)) {
+			if (!console_draw_string(&cdc, ext_line, ext_len,
+			                         (color_flag & TVC_LINE_FG) ? fg : NULL,
+			                         (color_flag & TVC_LINE_BG) ? bg : NULL,
+			                         bg_sel))
+			{
 				/* when drawing, if we pass v2d->cur.ymax, then quit */
 				if (draw) {
 					break; /* past the y limits */
