@@ -52,8 +52,8 @@
 
 static int logic_properties(bContext *C, wmOperator *UNUSED(op))
 {
-	ScrArea *sa= CTX_wm_area(C);
-	ARegion *ar= logic_has_buttons_region(sa);
+	ScrArea *sa = CTX_wm_area(C);
+	ARegion *ar = logic_has_buttons_region(sa);
 	
 	if (ar)
 		ED_region_toggle_hidden(C, ar);
@@ -84,10 +84,10 @@ static int cut_links_intersect(uiLinkLine *line, float mcoords[][2], int tot)
 	int i, b;
 	rcti rectlink;
 
-	rectlink.xmin = (int) (line->from->x1 + line->from->x2) / 2;
-	rectlink.ymin = (int) (line->from->y1 + line->from->y2) / 2;
-	rectlink.xmax = (int) (line->to->x1 + line->to->x2) / 2;
-	rectlink.ymax = (int) (line->to->y1 + line->to->y2) / 2;
+	rectlink.xmin = (int)BLI_rctf_cent_x(&line->from->rect);
+	rectlink.ymin = (int)BLI_rctf_cent_y(&line->from->rect);
+	rectlink.xmax = (int)BLI_rctf_cent_x(&line->to->rect);
+	rectlink.ymax = (int)BLI_rctf_cent_y(&line->to->rect);
 
 	if (ui_link_bezier_points(&rectlink, coord_array, LINK_RESOL)) {
 		for (i=0; i<tot-1; i++)
@@ -100,18 +100,20 @@ static int cut_links_intersect(uiLinkLine *line, float mcoords[][2], int tot)
 
 static int cut_links_exec(bContext *C, wmOperator *op)
 {
-	ARegion *ar= CTX_wm_region(C);
+	ARegion *ar = CTX_wm_region(C);
 	float mcoords[256][2];
-	int i= 0;
+	int i = 0;
 	
-	RNA_BEGIN(op->ptr, itemptr, "path") {
+	RNA_BEGIN (op->ptr, itemptr, "path")
+	{
 		float loc[2];
 		
 		RNA_float_get_array(&itemptr, "loc", loc);
-		UI_view2d_region_to_view(&ar->v2d, (short)loc[0], (short)loc[1], 
-								 &mcoords[i][0], &mcoords[i][1]);
+		UI_view2d_region_to_view(&ar->v2d,
+		                         (int)loc[0], (int)loc[1],
+		                         &mcoords[i][0], &mcoords[i][1]);
 		i++;
-		if (i>= 256) break;
+		if (i >= 256) break;
 	}
 	RNA_END;
 
@@ -119,24 +121,23 @@ static int cut_links_exec(bContext *C, wmOperator *op)
 		uiBlock *block;
 		uiLinkLine *line, *nline;
 		uiBut *but;
-		for (block= ar->uiblocks.first; block; block= block->next)
-		{
-			but= block->buttons.first;
+		for (block = ar->uiblocks.first; block; block = block->next) {
+			but = block->buttons.first;
 			while (but) {
 				if (but->type==LINK && but->link) {
-					for (line= but->link->lines.first; line; line= nline) {
-						nline= line->next;
+					for (line = but->link->lines.first; line; line = nline) {
+						nline = line->next;
 
 						if (cut_links_intersect(line, mcoords, i)) {
 							ui_delete_linkline(line, but);
 						}
 					}
 				}
-				but= but->next;
+				but = but->next;
 			}
 		}
 		return OPERATOR_FINISHED;
-	}	
+	}
 	return OPERATOR_CANCELLED|OPERATOR_PASS_THROUGH;
 }
 
@@ -144,7 +145,7 @@ void LOGIC_OT_links_cut(wmOperatorType *ot)
 {
 	PropertyRNA *prop;
 	
-	ot->name = "Cut links";
+	ot->name = "Cut Links";
 	ot->idname = "LOGIC_OT_links_cut";
 	ot->description = "Remove logic brick connections";
 	
@@ -158,7 +159,7 @@ void LOGIC_OT_links_cut(wmOperatorType *ot)
 	/* flags */
 	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
 	
-	prop= RNA_def_property(ot->srna, "path", PROP_COLLECTION, PROP_NONE);
+	prop = RNA_def_property(ot->srna, "path", PROP_COLLECTION, PROP_NONE);
 	RNA_def_property_struct_runtime(prop, &RNA_OperatorMousePath);
 	/* internal */
 	RNA_def_int(ot->srna, "cursor", BC_KNIFECURSOR, 0, INT_MAX, "Cursor", "", 0, INT_MAX);
