@@ -62,20 +62,20 @@ ARegion *logic_has_buttons_region(ScrArea *sa)
 {
 	ARegion *ar, *arnew;
 
-	ar= BKE_area_find_region_type(sa, RGN_TYPE_UI);
+	ar = BKE_area_find_region_type(sa, RGN_TYPE_UI);
 	if (ar) return ar;
 	
 	/* add subdiv level; after header */
-	ar= BKE_area_find_region_type(sa, RGN_TYPE_HEADER);
+	ar = BKE_area_find_region_type(sa, RGN_TYPE_HEADER);
 
 	/* is error! */
-	if (ar==NULL) return NULL;
+	if (ar == NULL) return NULL;
 	
 	arnew= MEM_callocN(sizeof(ARegion), "buttons for image");
 	
 	BLI_insertlinkafter(&sa->regionbase, ar, arnew);
-	arnew->regiontype= RGN_TYPE_UI;
-	arnew->alignment= RGN_ALIGN_LEFT;
+	arnew->regiontype = RGN_TYPE_UI;
+	arnew->alignment = RGN_ALIGN_LEFT;
 	
 	arnew->flag = RGN_FLAG_HIDDEN;
 	
@@ -84,8 +84,9 @@ ARegion *logic_has_buttons_region(ScrArea *sa)
 
 /* ******************** default callbacks for image space ***************** */
 
-static SpaceLink *logic_new(const bContext *UNUSED(C))
+static SpaceLink *logic_new(const bContext *C)
 {
+	ScrArea *sa= CTX_wm_area(C);
 	ARegion *ar;
 	SpaceLogic *slogic;
 	
@@ -93,10 +94,10 @@ static SpaceLink *logic_new(const bContext *UNUSED(C))
 	slogic->spacetype= SPACE_LOGIC;
 	
 	/* default options */
-	slogic->scaflag =	(BUTS_SENS_SEL|BUTS_SENS_ACT|BUTS_SENS_LINK) 	|
-						(BUTS_CONT_SEL|BUTS_CONT_ACT|BUTS_CONT_LINK) 	|
-						(BUTS_ACT_SEL|BUTS_ACT_ACT|BUTS_ACT_LINK)		|
-						(BUTS_SENS_STATE|BUTS_ACT_STATE);
+	slogic->scaflag = ((BUTS_SENS_SEL|BUTS_SENS_ACT|BUTS_SENS_LINK) |
+	                   (BUTS_CONT_SEL|BUTS_CONT_ACT|BUTS_CONT_LINK) |
+	                   (BUTS_ACT_SEL|BUTS_ACT_ACT|BUTS_ACT_LINK)    |
+	                   (BUTS_SENS_STATE|BUTS_ACT_STATE));
 	
 	
 	/* header */
@@ -120,28 +121,26 @@ static SpaceLink *logic_new(const bContext *UNUSED(C))
 	ar->regiontype= RGN_TYPE_WINDOW;
 
 	ar->v2d.tot.xmin =  0.0f;
-	ar->v2d.tot.ymin =  0.0f;
-	ar->v2d.tot.xmax = 1280;
-	ar->v2d.tot.ymax = 240.0f;
+	ar->v2d.tot.ymax =  0.0f;
+	ar->v2d.tot.xmax = 1150.0f;
+	ar->v2d.tot.ymin = ( 1150.0f/(float)sa->winx ) * (float)-sa->winy;
 	
-	ar->v2d.cur.xmin =  0.0f;
-	ar->v2d.cur.ymin =  0.0f;
-	ar->v2d.cur.xmax = 1280.0f;
-	ar->v2d.cur.ymax = 240.0f;
+	ar->v2d.cur = ar->v2d.tot;
 	
-	ar->v2d.min[0]= 1.0f;
-	ar->v2d.min[1]= 1.0f;
+	ar->v2d.min[0] = 1.0f;
+	ar->v2d.min[1] = 1.0f;
 	
-	ar->v2d.max[0]= 32000.0f;
-	ar->v2d.max[1]= 32000.0f;
+	ar->v2d.max[0] = 32000.0f;
+	ar->v2d.max[1] = 32000.0f;
 	
-	ar->v2d.minzoom= 0.5f;
-	ar->v2d.maxzoom= 1.21f;
-	
-	ar->v2d.scroll= (V2D_SCROLL_RIGHT|V2D_SCROLL_BOTTOM);
-	ar->v2d.keepzoom= V2D_LIMITZOOM|V2D_KEEPASPECT;
-	ar->v2d.keeptot= 0;
+	ar->v2d.minzoom = 0.5f;
+	ar->v2d.maxzoom = 1.5f;
 	
+	ar->v2d.scroll = (V2D_SCROLL_RIGHT | V2D_SCROLL_BOTTOM);
+	ar->v2d.keepzoom = V2D_KEEPZOOM | V2D_LIMITZOOM | V2D_KEEPASPECT;
+	ar->v2d.keeptot = V2D_KEEPTOT_BOUNDS;
+	ar->v2d.align = V2D_ALIGN_NO_POS_Y | V2D_ALIGN_NO_NEG_X;
+	ar->v2d.keepofs = V2D_KEEPOFS_Y;
 	
 	return (SpaceLink *)slogic;
 }
@@ -149,10 +148,10 @@ static SpaceLink *logic_new(const bContext *UNUSED(C))
 /* not spacelink itself */
 static void logic_free(SpaceLink *UNUSED(sl))
 {	
-//	Spacelogic *slogic= (SpaceLogic*) sl;
+//	Spacelogic *slogic= (SpaceLogic *) sl;
 	
 //	if (slogic->gpd)
-// XXX		free_gpencil_data(slogic->gpd);
+// XXX		BKE_gpencil_free(slogic->gpd);
 	
 }
 
@@ -183,6 +182,9 @@ static void logic_keymap(struct wmKeyConfig *keyconf)
 	WM_keymap_add_item(keymap, "LOGIC_OT_properties", NKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "LOGIC_OT_links_cut", LEFTMOUSE, KM_PRESS, KM_CTRL, 0);
 	WM_keymap_add_menu(keymap, "LOGIC_MT_logicbricks_add", AKEY, KM_PRESS, KM_SHIFT, 0);
+	
+	WM_keymap_add_item(keymap, "LOGIC_OT_view_all", HOMEKEY, KM_PRESS, 0, 0);
+
 }
 
 static void logic_refresh(const bContext *UNUSED(C), ScrArea *UNUSED(sa))
@@ -195,12 +197,12 @@ static void logic_refresh(const bContext *UNUSED(C), ScrArea *UNUSED(sa))
 static void logic_listener(ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
-	switch(wmn->category) {
+	switch (wmn->category) {
 		case NC_LOGIC:
 			ED_region_tag_redraw(ar);
 			break;
 		case NC_SCENE:
-			switch(wmn->data) {
+			switch (wmn->data) {
 				case ND_FRAME:
 					ED_region_tag_redraw(ar);
 					break;
@@ -234,7 +236,7 @@ static void logic_main_area_init(wmWindowManager *wm, ARegion *ar)
 	wmKeyMap *keymap;
 	
 	UI_view2d_region_reinit(&ar->v2d, V2D_COMMONVIEW_CUSTOM, ar->winx, ar->winy);
-
+	
 	/* own keymaps */
 	keymap = WM_keymap_find(wm->defaultconf, "Logic Editor", SPACE_LOGIC, 0);
 	WM_event_add_keymap_handler(&ar->handlers, keymap);
@@ -259,7 +261,7 @@ static void logic_main_area_draw(const bContext *C, ARegion *ar)
 	UI_view2d_view_restore(C);
 	
 	/* scrollers */
-	scrollers= UI_view2d_scrollers_calc(C, v2d, 10, V2D_GRID_CLAMP, V2D_ARG_DUMMY, V2D_ARG_DUMMY);
+	scrollers= UI_view2d_scrollers_calc(C, v2d, V2D_ARG_DUMMY, V2D_ARG_DUMMY, V2D_ARG_DUMMY, V2D_ARG_DUMMY);
 	UI_view2d_scrollers_draw(C, v2d, scrollers);
 	UI_view2d_scrollers_free(scrollers);
 	
@@ -302,48 +304,48 @@ static void logic_header_area_draw(const bContext *C, ARegion *ar)
 /* only called once, from space/spacetypes.c */
 void ED_spacetype_logic(void)
 {
-	SpaceType *st= MEM_callocN(sizeof(SpaceType), "spacetype logic");
+	SpaceType *st = MEM_callocN(sizeof(SpaceType), "spacetype logic");
 	ARegionType *art;
 	
-	st->spaceid= SPACE_LOGIC;
+	st->spaceid = SPACE_LOGIC;
 	strncpy(st->name, "Logic", BKE_ST_MAXNAME);
 	
-	st->new= logic_new;
-	st->free= logic_free;
-	st->init= logic_init;
-	st->duplicate= logic_duplicate;
-	st->operatortypes= logic_operatortypes;
-	st->keymap= logic_keymap;
-	st->refresh= logic_refresh;
-	st->context= logic_context;
+	st->new = logic_new;
+	st->free = logic_free;
+	st->init = logic_init;
+	st->duplicate = logic_duplicate;
+	st->operatortypes = logic_operatortypes;
+	st->keymap = logic_keymap;
+	st->refresh = logic_refresh;
+	st->context = logic_context;
 	
 	/* regions: main window */
-	art= MEM_callocN(sizeof(ARegionType), "spacetype logic region");
+	art = MEM_callocN(sizeof(ARegionType), "spacetype logic region");
 	art->regionid = RGN_TYPE_WINDOW;
-	art->keymapflag= ED_KEYMAP_UI|ED_KEYMAP_FRAMES|ED_KEYMAP_VIEW2D;
-	art->init= logic_main_area_init;
-	art->draw= logic_main_area_draw;
-	art->listener= logic_listener;
+	art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_FRAMES | ED_KEYMAP_VIEW2D;
+	art->init = logic_main_area_init;
+	art->draw = logic_main_area_draw;
+	art->listener = logic_listener;
 
 	BLI_addhead(&st->regiontypes, art);
 	
 	/* regions: listview/buttons */
-	art= MEM_callocN(sizeof(ARegionType), "spacetype logic region");
+	art = MEM_callocN(sizeof(ARegionType), "spacetype logic region");
 	art->regionid = RGN_TYPE_UI;
 	art->prefsizex= 220; // XXX
-	art->keymapflag= ED_KEYMAP_UI|ED_KEYMAP_FRAMES;
-	art->listener= logic_listener;
-	art->init= logic_buttons_area_init;
-	art->draw= logic_buttons_area_draw;
+	art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_FRAMES;
+	art->listener = logic_listener;
+	art->init = logic_buttons_area_init;
+	art->draw = logic_buttons_area_draw;
 	BLI_addhead(&st->regiontypes, art);
 
 	/* regions: header */
 	art= MEM_callocN(sizeof(ARegionType), "spacetype logic region");
 	art->regionid = RGN_TYPE_HEADER;
-	art->prefsizey= HEADERY;
-	art->keymapflag= ED_KEYMAP_UI|ED_KEYMAP_VIEW2D|ED_KEYMAP_FRAMES|ED_KEYMAP_HEADER;
-	art->init= logic_header_area_init;
-	art->draw= logic_header_area_draw;
+	art->prefsizey = HEADERY;
+	art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_VIEW2D | ED_KEYMAP_FRAMES | ED_KEYMAP_HEADER;
+	art->init = logic_header_area_init;
+	art->draw = logic_header_area_draw;
 	
 	BLI_addhead(&st->regiontypes, art);
 	
