@@ -28,21 +28,17 @@
  *  \ingroup spnode
  */
 
-
-#include <stdio.h>
-
-#include "BLI_listbase.h"
-
 #include "DNA_node_types.h"
-#include "DNA_scene_types.h"
+
+#include "BLI_rect.h"
+#include "BLI_lasso.h"
+#include "BLI_utildefines.h"
 
 #include "BKE_context.h"
 #include "BKE_main.h"
+#include "BKE_node.h"
 
-#include "BLI_rect.h"
-#include "BLI_utildefines.h"
-
-#include "ED_node.h"
+#include "ED_node.h"  /* own include */
 #include "ED_screen.h"
 #include "ED_types.h"
 
@@ -53,124 +49,37 @@
 #include "WM_types.h"
 
 #include "UI_view2d.h"
- 
-#include "node_intern.h"
+
+#include "MEM_guardedalloc.h"
+
+#include "node_intern.h"  /* own include */
 
 /* ****** helpers ****** */
 
-static bNode *node_under_mouse(bNodeTree *ntree, int mx, int my)
+static bNode *node_under_mouse_select(bNodeTree *ntree, int mx, int my)
 {
 	bNode *node;
 	
-	for (node=ntree->nodes.last; node; node=node->prev) {
-		/* node body (header and scale are in other operators) */
-		if (BLI_in_rctf(&node->totr, mx, my))
-			return node;
+	for (node = ntree->nodes.last; node; node = node->prev) {
+		if (node->typeinfo->select_area_func) {
+			if (node->typeinfo->select_area_func(node, mx, my))
+				return node;
+		}
 	}
 	return NULL;
 }
 
-static int compare_nodes(bNode *a, bNode *b)
+static bNode *node_under_mouse_tweak(bNodeTree *ntree, int mx, int my)
 {
-	bNode *parent;
-	/* These tell if either the node or any of the parent nodes is selected.
-	 * A selected parent means an unselected node is also in foreground!
-	 */
-	int a_select=(a->flag & NODE_SELECT), b_select=(b->flag & NODE_SELECT);
-	int a_active=(a->flag & NODE_ACTIVE), b_active=(b->flag & NODE_ACTIVE);
+	bNode *node;
 	
-	/* if one is an ancestor of the other */
-	/* XXX there might be a better sorting algorithm for stable topological sort, this is O(n^2) worst case */
-	for (parent = a->parent; parent; parent=parent->parent) {
-		/* if b is an ancestor, it is always behind a */
-		if (parent==b)
-			return 1;
-		/* any selected ancestor moves the node forward */
-		if (parent->flag & NODE_ACTIVE)
-			a_active = 1;
-		if (parent->flag & NODE_SELECT)
-			a_select = 1;
-	}
-	for (parent = b->parent; parent; parent=parent->parent) {
-		/* if a is an ancestor, it is always behind b */
-		if (parent==a)
-			return 0;
-		/* any selected ancestor moves the node forward */
-		if (parent->flag & NODE_ACTIVE)
-			b_active = 1;
-		if (parent->flag & NODE_SELECT)
-			b_select = 1;
-	}
-
-	/* if one of the nodes is in the background and the other not */
-	if ((a->flag & NODE_BACKGROUND) && !(b->flag & NODE_BACKGROUND))
-		return 0;
-	else if (!(a->flag & NODE_BACKGROUND) && (b->flag & NODE_BACKGROUND))
-		return 1;
-	
-	/* if one has a higher selection state (active > selected > nothing) */
-	if (!b_active && a_active)
-		return 1;
-	else if (!b_select && (a_active || a_select))
-		return 1;
-	
-	return 0;
-}
-
-/* Sorts nodes by selection: unselected nodes first, then selected,
- * then the active node at the very end. Relative order is kept intact!
- */
-static void node_sort(bNodeTree *ntree)
-{
-	/* merge sort is the algorithm of choice here */
-	bNode *first_a, *first_b, *node_a, *node_b, *tmp;
-	int totnodes= BLI_countlist(&ntree->nodes);
-	int k, a, b;
-	
-	k = 1;
-	while (k < totnodes) {
-		first_a = first_b = ntree->nodes.first;
-		
-		do {
-			/* setup first_b pointer */
-			for (b=0; b < k && first_b; ++b) {
-				first_b = first_b->next;
-			}
-			/* all batches merged? */
-			if (first_b==NULL)
-				break;
-			
-			/* merge batches */
-			node_a = first_a;
-			node_b = first_b;
-			a = b = 0;
-			while (a < k && b < k && node_b) {
-				if (compare_nodes(node_a, node_b)==0) {
-					node_a = node_a->next;
-					++a;
-				}
-				else {
-					tmp = node_b;
-					node_b = node_b->next;
-					++b;
-					BLI_remlink(&ntree->nodes, tmp);
-					BLI_insertlinkbefore(&ntree->nodes, node_a, tmp);
-				}
-			}
-
-			/* setup first pointers for next batch */
-			first_b = node_b;
-			for (; b < k; ++b) {
-				/* all nodes sorted? */
-				if (first_b==NULL)
-					break;
-				first_b = first_b->next;
-			}
-			first_a = first_b;
-		} while (first_b);
-		
-		k = k << 1;
+	for (node = ntree->nodes.last; node; node = node->prev) {
+		if (node->typeinfo->tweak_area_func) {
+			if (node->typeinfo->tweak_area_func(node, mx, my))
+				return node;
+		}
 	}
+	return NULL;
 }
 
 void node_select(bNode *node)
@@ -185,9 +94,9 @@ void node_deselect(bNode *node)
 	node->flag &= ~SELECT;
 	
 	/* deselect sockets too */
-	for (sock=node->inputs.first; sock; sock=sock->next)
+	for (sock = node->inputs.first; sock; sock = sock->next)
 		sock->flag &= ~SELECT;
-	for (sock=node->outputs.first; sock; sock=sock->next)
+	for (sock = node->outputs.first; sock; sock = sock->next)
 		sock->flag &= ~SELECT;
 }
 
@@ -213,16 +122,16 @@ void node_socket_deselect(bNode *node, bNodeSocket *sock, int deselect_node)
 	sock->flag &= ~SELECT;
 	
 	if (node && deselect_node) {
-		int sel=0;
+		int sel = 0;
 		
 		/* if no selected sockets remain, also deselect the node */
-		for (sock=node->inputs.first; sock; sock=sock->next) {
+		for (sock = node->inputs.first; sock; sock = sock->next) {
 			if (sock->flag & SELECT) {
 				sel = 1;
 				break;
 			}
 		}
-		for (sock=node->outputs.first; sock; sock=sock->next) {
+		for (sock = node->outputs.first; sock; sock = sock->next) {
 			if (sock->flag & SELECT) {
 				sel = 1;
 				break;
@@ -247,7 +156,7 @@ void node_deselect_all(SpaceNode *snode)
 {
 	bNode *node;
 	
-	for (node= snode->edittree->nodes.first; node; node= node->next)
+	for (node = snode->edittree->nodes.first; node; node = node->next)
 		node_deselect(node);
 }
 
@@ -261,15 +170,15 @@ void node_deselect_all_input_sockets(SpaceNode *snode, int deselect_nodes)
 	 * We can do that more efficiently here.
 	 */
 	
-	for (node= snode->edittree->nodes.first; node; node= node->next) {
-		int sel=0;
+	for (node = snode->edittree->nodes.first; node; node = node->next) {
+		int sel = 0;
 		
-		for (sock= node->inputs.first; sock; sock=sock->next)
+		for (sock = node->inputs.first; sock; sock = sock->next)
 			sock->flag &= ~SELECT;
 		
 		/* if no selected sockets remain, also deselect the node */
 		if (deselect_nodes) {
-			for (sock= node->outputs.first; sock; sock=sock->next) {
+			for (sock = node->outputs.first; sock; sock = sock->next) {
 				if (sock->flag & SELECT) {
 					sel = 1;
 					break;
@@ -281,7 +190,7 @@ void node_deselect_all_input_sockets(SpaceNode *snode, int deselect_nodes)
 		}
 	}
 	
-	for (sock= snode->edittree->outputs.first; sock; sock=sock->next)
+	for (sock = snode->edittree->outputs.first; sock; sock = sock->next)
 		sock->flag &= ~SELECT;
 }
 
@@ -295,15 +204,15 @@ void node_deselect_all_output_sockets(SpaceNode *snode, int deselect_nodes)
 	 * We can do that more efficiently here.
 	 */
 	
-	for (node= snode->edittree->nodes.first; node; node= node->next) {
-		int sel=0;
+	for (node = snode->edittree->nodes.first; node; node = node->next) {
+		int sel = 0;
 		
-		for (sock= node->outputs.first; sock; sock=sock->next)
+		for (sock = node->outputs.first; sock; sock = sock->next)
 			sock->flag &= ~SELECT;
 		
 		/* if no selected sockets remain, also deselect the node */
 		if (deselect_nodes) {
-			for (sock= node->inputs.first; sock; sock=sock->next) {
+			for (sock = node->inputs.first; sock; sock = sock->next) {
 				if (sock->flag & SELECT) {
 					sel = 1;
 					break;
@@ -315,7 +224,7 @@ void node_deselect_all_output_sockets(SpaceNode *snode, int deselect_nodes)
 		}
 	}
 	
-	for (sock= snode->edittree->inputs.first; sock; sock=sock->next)
+	for (sock = snode->edittree->inputs.first; sock; sock = sock->next)
 		sock->flag &= ~SELECT;
 }
 
@@ -326,7 +235,7 @@ int node_select_same_type(SpaceNode *snode)
 	int redraw;
 
 	/* search for the active node. */
-	for (nac= snode->edittree->nodes.first; nac; nac= nac->next) {
+	for (nac = snode->edittree->nodes.first; nac; nac = nac->next) {
 		if (nac->flag & SELECT)
 			break;
 	}
@@ -335,16 +244,16 @@ int node_select_same_type(SpaceNode *snode)
 	if (!nac)
 		return(0);
 
-	redraw= 0;
-	for (p= snode->edittree->nodes.first; p; p= p->next) {
+	redraw = 0;
+	for (p = snode->edittree->nodes.first; p; p = p->next) {
 		if (p->type != nac->type && p->flag & SELECT) {
 			/* if it's selected but different type, unselect */
-			redraw= 1;
+			redraw = 1;
 			node_deselect(p);
 		}
 		else if (p->type == nac->type && (!(p->flag & SELECT))) {
 			/* if it's the same type and is not selected, select! */
-			redraw= 1;
+			redraw = 1;
 			node_select(p);
 		}
 	}
@@ -359,7 +268,7 @@ int node_select_same_type_np(SpaceNode *snode, int dir)
 	bNode *nac, *p, *tnode;
 
 	/* search the active one. */
-	for (nac= snode->edittree->nodes.first; nac; nac= nac->next) {
+	for (nac = snode->edittree->nodes.first; nac; nac = nac->next) {
 		if (nac->flag & SELECT)
 			break;
 	}
@@ -369,9 +278,9 @@ int node_select_same_type_np(SpaceNode *snode, int dir)
 		return(0);
 
 	if (dir == 0)
-		p= nac->next;
+		p = nac->next;
 	else
-		p= nac->prev;
+		p = nac->prev;
 
 	while (p) {
 		/* Now search the next with the same type. */
@@ -379,14 +288,14 @@ int node_select_same_type_np(SpaceNode *snode, int dir)
 			break;
 
 		if (dir == 0)
-			p= p->next;
+			p = p->next;
 		else
-			p= p->prev;
+			p = p->prev;
 	}
 
 	if (p) {
-		for (tnode=snode->edittree->nodes.first; tnode; tnode=tnode->next)
-			if (tnode!=p)
+		for (tnode = snode->edittree->nodes.first; tnode; tnode = tnode->next)
+			if (tnode != p)
 				node_deselect(tnode);
 		node_select(p);
 		return(1);
@@ -396,20 +305,20 @@ int node_select_same_type_np(SpaceNode *snode, int dir)
 
 void node_select_single(bContext *C, bNode *node)
 {
-	Main *bmain= CTX_data_main(C);
-	SpaceNode *snode= CTX_wm_space_node(C);
+	Main *bmain = CTX_data_main(C);
+	SpaceNode *snode = CTX_wm_space_node(C);
 	bNode *tnode;
 	
-	for (tnode=snode->edittree->nodes.first; tnode; tnode=tnode->next)
-		if (tnode!=node)
+	for (tnode = snode->edittree->nodes.first; tnode; tnode = tnode->next)
+		if (tnode != node)
 			node_deselect(tnode);
 	node_select(node);
 	
 	ED_node_set_active(bmain, snode->edittree, node);
 	
-	node_sort(snode->edittree);
+	ED_node_sort(snode->edittree);
 	
-	WM_event_add_notifier(C, NC_NODE|NA_SELECTED, NULL);
+	WM_event_add_notifier(C, NC_NODE | NA_SELECTED, NULL);
 }
 
 /* ****** Click Select ****** */
@@ -424,8 +333,8 @@ static int node_mouse_select(Main *bmain, SpaceNode *snode, ARegion *ar, const i
 	/* get mouse coordinates in view2d space */
 	UI_view2d_region_to_view(&ar->v2d, mval[0], mval[1], &mx, &my);
 	/* node_find_indicated_socket uses snode->mx/my */
-	snode->mx = mx;
-	snode->my = my;
+	snode->cursor[0] = mx;
+	snode->cursor[1] = my;
 	
 	if (extend) {
 		/* first do socket selection, these generally overlap with nodes.
@@ -444,7 +353,7 @@ static int node_mouse_select(Main *bmain, SpaceNode *snode, ARegion *ar, const i
 				 * allows selecting outputs from different nodes though.
 				 */
 				if (node) {
-					for (tsock=node->outputs.first; tsock; tsock=tsock->next)
+					for (tsock = node->outputs.first; tsock; tsock = tsock->next)
 						node_socket_deselect(node, tsock, 1);
 				}
 				node_socket_select(node, sock);
@@ -453,23 +362,31 @@ static int node_mouse_select(Main *bmain, SpaceNode *snode, ARegion *ar, const i
 		}
 		else {
 			/* find the closest visible node */
-			node = node_under_mouse(snode->edittree, mx, my);
+			node = node_under_mouse_select(snode->edittree, mx, my);
 			
 			if (node) {
-				node_toggle(node);
-				
-				ED_node_set_active(bmain, snode->edittree, node);
+				if ((node->flag & SELECT) && (node->flag & NODE_ACTIVE) == 0) {
+					/* if node is selected but not active make it active
+					 * before it'll be desleected
+					 */
+					ED_node_set_active(bmain, snode->edittree, node);
+				}
+				else {
+					node_toggle(node);
+					ED_node_set_active(bmain, snode->edittree, node);
+				}
+
 				selected = 1;
 			}
 		}
 	}
-	else {	/* extend==0 */
+	else {  /* extend == 0 */
 		
 		/* find the closest visible node */
-		node = node_under_mouse(snode->edittree, mx, my);
+		node = node_under_mouse_select(snode->edittree, mx, my);
 		
 		if (node) {
-			for (tnode=snode->edittree->nodes.first; tnode; tnode=tnode->next)
+			for (tnode = snode->edittree->nodes.first; tnode; tnode = tnode->next)
 				node_deselect(tnode);
 			node_select(node);
 			ED_node_set_active(bmain, snode->edittree, node);
@@ -479,16 +396,16 @@ static int node_mouse_select(Main *bmain, SpaceNode *snode, ARegion *ar, const i
 	
 	/* update node order */
 	if (selected)
-		node_sort(snode->edittree);
+		ED_node_sort(snode->edittree);
 	
 	return selected;
 }
 
 static int node_select_exec(bContext *C, wmOperator *op)
 {
-	Main *bmain= CTX_data_main(C);
-	SpaceNode *snode= CTX_wm_space_node(C);
-	ARegion *ar= CTX_wm_region(C);
+	Main *bmain = CTX_data_main(C);
+	SpaceNode *snode = CTX_wm_space_node(C);
+	ARegion *ar = CTX_wm_region(C);
 	int mval[2];
 	short extend;
 	
@@ -501,14 +418,14 @@ static int node_select_exec(bContext *C, wmOperator *op)
 	/* perform the select */
 	if (node_mouse_select(bmain, snode, ar, mval, extend)) {
 		/* send notifiers */
-		WM_event_add_notifier(C, NC_NODE|NA_SELECTED, NULL);
+		WM_event_add_notifier(C, NC_NODE | NA_SELECTED, NULL);
 		
 		/* allow tweak event to work too */
-		return OPERATOR_FINISHED|OPERATOR_PASS_THROUGH;
+		return OPERATOR_FINISHED | OPERATOR_PASS_THROUGH;
 	}
 	else {
 		/* allow tweak event to work too */
-		return OPERATOR_CANCELLED|OPERATOR_PASS_THROUGH;
+		return OPERATOR_CANCELLED | OPERATOR_PASS_THROUGH;
 	}
 }
 
@@ -517,7 +434,7 @@ static int node_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	RNA_int_set(op->ptr, "mouse_x", event->mval[0]);
 	RNA_int_set(op->ptr, "mouse_y", event->mval[1]);
 
-	return node_select_exec(C,op);
+	return node_select_exec(C, op);
 }
 
 
@@ -533,7 +450,7 @@ void NODE_OT_select(wmOperatorType *ot)
 	ot->poll = ED_operator_node_active;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 	
 	/* properties */
 	RNA_def_int(ot->srna, "mouse_x", 0, INT_MIN, INT_MAX, "Mouse X", "", INT_MIN, INT_MAX);
@@ -545,25 +462,22 @@ void NODE_OT_select(wmOperatorType *ot)
 
 static int node_borderselect_exec(bContext *C, wmOperator *op)
 {
-	SpaceNode *snode= CTX_wm_space_node(C);
-	ARegion *ar= CTX_wm_region(C);
+	SpaceNode *snode = CTX_wm_space_node(C);
+	ARegion *ar = CTX_wm_region(C);
 	bNode *node;
 	rcti rect;
 	rctf rectf;
-	int gesture_mode= RNA_int_get(op->ptr, "gesture_mode");
-	int extend= RNA_boolean_get(op->ptr, "extend");
+	int gesture_mode = RNA_int_get(op->ptr, "gesture_mode");
+	int extend = RNA_boolean_get(op->ptr, "extend");
 	
-	rect.xmin = RNA_int_get(op->ptr, "xmin");
-	rect.ymin = RNA_int_get(op->ptr, "ymin");
+	WM_operator_properties_border_to_rcti(op, &rect);
+
 	UI_view2d_region_to_view(&ar->v2d, rect.xmin, rect.ymin, &rectf.xmin, &rectf.ymin);
-	
-	rect.xmax = RNA_int_get(op->ptr, "xmax");
-	rect.ymax = RNA_int_get(op->ptr, "ymax");
 	UI_view2d_region_to_view(&ar->v2d, rect.xmax, rect.ymax, &rectf.xmax, &rectf.ymax);
 	
-	for (node= snode->edittree->nodes.first; node; node= node->next) {
-		if (BLI_isect_rctf(&rectf, &node->totr, NULL)) {
-			if (gesture_mode==GESTURE_MODAL_SELECT)
+	for (node = snode->edittree->nodes.first; node; node = node->next) {
+		if (BLI_rctf_isect(&rectf, &node->totr, NULL)) {
+			if (gesture_mode == GESTURE_MODAL_SELECT)
 				node_select(node);
 			else
 				node_deselect(node);
@@ -573,9 +487,9 @@ static int node_borderselect_exec(bContext *C, wmOperator *op)
 		}
 	}
 	
-	node_sort(snode->edittree);
+	ED_node_sort(snode->edittree);
 	
-	WM_event_add_notifier(C, NC_NODE|NA_SELECTED, NULL);
+	WM_event_add_notifier(C, NC_NODE | NA_SELECTED, NULL);
 
 	return OPERATOR_FINISHED;
 }
@@ -587,14 +501,14 @@ static int node_border_select_invoke(bContext *C, wmOperator *op, wmEvent *event
 	if (tweak) {
 		/* prevent initiating the border select if the mouse is over a node */
 		/* this allows border select on empty space, but drag-translate on nodes */
-		SpaceNode *snode= CTX_wm_space_node(C);
-		ARegion *ar= CTX_wm_region(C);
+		SpaceNode *snode = CTX_wm_space_node(C);
+		ARegion *ar = CTX_wm_region(C);
 		float mx, my;
 
 		UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], &mx, &my);
 		
-		if (node_under_mouse(snode->edittree, mx, my))
-			return OPERATOR_CANCELLED|OPERATOR_PASS_THROUGH;
+		if (node_under_mouse_tweak(snode->edittree, mx, my))
+			return OPERATOR_CANCELLED | OPERATOR_PASS_THROUGH;
 	}
 	
 	return WM_border_select_invoke(C, op, event);
@@ -616,38 +530,132 @@ void NODE_OT_select_border(wmOperatorType *ot)
 	ot->poll = ED_operator_node_active;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 	
 	/* rna */
 	WM_operator_properties_gesture_border(ot, TRUE);
 	RNA_def_boolean(ot->srna, "tweak", 0, "Tweak", "Only activate when mouse is not over a node - useful for tweak gesture");
 }
 
-/* ****** Select/Deselect All ****** */
+/* ****** Lasso Select ****** */
 
-static int node_select_all_exec(bContext *C, wmOperator *UNUSED(op))
+static int do_lasso_select_node(bContext *C, const int mcords[][2], short moves, short select)
 {
 	SpaceNode *snode = CTX_wm_space_node(C);
-	bNode *first = snode->edittree->nodes.first;
 	bNode *node;
-	int count= 0;
 
-	for (node=first; node; node=node->next)
-		if (node->flag & NODE_SELECT)
-			count++;
+	ARegion *ar = CTX_wm_region(C);
 
-	if (count) {
-		for (node=first; node; node=node->next)
-			node_deselect(node);
+	rcti rect;
+	int change = FALSE;
+
+	/* get rectangle from operator */
+	BLI_lasso_boundbox(&rect, mcords, moves);
+
+	/* do actual selection */
+	for (node = snode->edittree->nodes.first; node; node = node->next) {
+		int screen_co[2];
+		const float cent[2] = {BLI_rctf_cent_x(&node->totr),
+		                       BLI_rctf_cent_y(&node->totr)};
+
+		/* marker in screen coords */
+		UI_view2d_view_to_region(&ar->v2d,
+		                         cent[0], cent[1],
+		                         &screen_co[0], &screen_co[1]);
+
+		if (BLI_rcti_isect_pt(&rect, screen_co[0], screen_co[1]) &&
+			BLI_lasso_is_point_inside(mcords, moves, screen_co[0], screen_co[1], INT_MAX))
+		{
+			if (select)
+				node_select(node);
+			else
+				node_deselect(node);
+
+			change = TRUE;
+		}
 	}
-	else {
-		for (node=first; node; node=node->next)
-			node_select(node);
+
+	if (change) {
+		WM_event_add_notifier(C, NC_NODE | NA_SELECTED, NULL);
 	}
+
+	return change;
+}
+
+static int node_lasso_select_exec(bContext *C, wmOperator *op)
+{
+	int mcords_tot;
+	const int (*mcords)[2] = WM_gesture_lasso_path_to_array(C, op, &mcords_tot);
+
+	if (mcords) {
+		short select;
+
+		select = !RNA_boolean_get(op->ptr, "deselect");
+		do_lasso_select_node(C, mcords, mcords_tot, select);
+
+		MEM_freeN((void *)mcords);
+
+		return OPERATOR_FINISHED;
+	}
+	return OPERATOR_PASS_THROUGH;
+}
+
+void NODE_OT_select_lasso(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Lasso Select";
+	ot->description = "Select nodes using lasso selection";
+	ot->idname = "NODE_OT_select_lasso";
+
+	/* api callbacks */
+	ot->invoke = WM_gesture_lasso_invoke;
+	ot->modal = WM_gesture_lasso_modal;
+	ot->exec = node_lasso_select_exec;
+	ot->poll = ED_operator_node_active;
+	ot->cancel = WM_gesture_lasso_cancel;
+
+	/* flags */
+	ot->flag = OPTYPE_UNDO;
+
+	/* properties */
+	RNA_def_collection_runtime(ot->srna, "path", &RNA_OperatorMousePath, "Path", "");
+	RNA_def_boolean(ot->srna, "deselect", 0, "Deselect", "Deselect rather than select items");
+	RNA_def_boolean(ot->srna, "extend", 1, "Extend", "Extend selection instead of deselecting everything first");
+}
+
+/* ****** Select/Deselect All ****** */
+
+static int node_select_all_exec(bContext *C, wmOperator *op)
+{
+	SpaceNode *snode = CTX_wm_space_node(C);
+	ListBase *node_lb = &snode->edittree->nodes;
+	bNode *node;
+	int action = RNA_enum_get(op->ptr, "action");
+
+	if (action == SEL_TOGGLE) {
+		if (ED_node_select_check(node_lb))
+			action = SEL_DESELECT;
+		else
+			action = SEL_SELECT;
+	}
+
+	for (node = node_lb->first; node; node = node->next) {
+		switch (action) {
+			case SEL_SELECT:
+				node_select(node);
+				break;
+			case SEL_DESELECT:
+				node_deselect(node);
+				break;
+			case SEL_INVERT:
+				((node->flag & SELECT) ? node_deselect : node_select)(node);
+				break;
+		}
+	}
+
+	ED_node_sort(snode->edittree);
 	
-	node_sort(snode->edittree);
-	
-	WM_event_add_notifier(C, NC_NODE|NA_SELECTED, NULL);
+	WM_event_add_notifier(C, NC_NODE | NA_SELECTED, NULL);
 	return OPERATOR_FINISHED;
 }
 
@@ -663,7 +671,9 @@ void NODE_OT_select_all(wmOperatorType *ot)
 	ot->poll = ED_operator_node_active;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+
+	WM_operator_properties_select_all(ot);
 }
 
 /* ****** Select Linked To ****** */
@@ -674,22 +684,22 @@ static int node_select_linked_to_exec(bContext *C, wmOperator *UNUSED(op))
 	bNodeLink *link;
 	bNode *node;
 	
-	for (node=snode->edittree->nodes.first; node; node=node->next)
+	for (node = snode->edittree->nodes.first; node; node = node->next)
 		node->flag &= ~NODE_TEST;
 
-	for (link=snode->edittree->links.first; link; link=link->next) {
+	for (link = snode->edittree->links.first; link; link = link->next) {
 		if (link->fromnode && link->tonode && (link->fromnode->flag & NODE_SELECT))
 			link->tonode->flag |= NODE_TEST;
 	}
 	
-	for (node=snode->edittree->nodes.first; node; node=node->next) {
+	for (node = snode->edittree->nodes.first; node; node = node->next) {
 		if (node->flag & NODE_TEST)
 			node_select(node);
 	}
 	
-	node_sort(snode->edittree);
+	ED_node_sort(snode->edittree);
 	
-	WM_event_add_notifier(C, NC_NODE|NA_SELECTED, NULL);
+	WM_event_add_notifier(C, NC_NODE | NA_SELECTED, NULL);
 	return OPERATOR_FINISHED;
 }
 
@@ -705,7 +715,7 @@ void NODE_OT_select_linked_to(wmOperatorType *ot)
 	ot->poll = ED_operator_node_active;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
 
 /* ****** Select Linked From ****** */
@@ -716,22 +726,22 @@ static int node_select_linked_from_exec(bContext *C, wmOperator *UNUSED(op))
 	bNodeLink *link;
 	bNode *node;
 	
-	for (node=snode->edittree->nodes.first; node; node=node->next)
+	for (node = snode->edittree->nodes.first; node; node = node->next)
 		node->flag &= ~NODE_TEST;
 
-	for (link=snode->edittree->links.first; link; link=link->next) {
+	for (link = snode->edittree->links.first; link; link = link->next) {
 		if (link->fromnode && link->tonode && (link->tonode->flag & NODE_SELECT))
 			link->fromnode->flag |= NODE_TEST;
 	}
 	
-	for (node=snode->edittree->nodes.first; node; node=node->next) {
+	for (node = snode->edittree->nodes.first; node; node = node->next) {
 		if (node->flag & NODE_TEST)
 			node_select(node);
 	}
 	
-	node_sort(snode->edittree);
+	ED_node_sort(snode->edittree);
 	
-	WM_event_add_notifier(C, NC_NODE|NA_SELECTED, NULL);
+	WM_event_add_notifier(C, NC_NODE | NA_SELECTED, NULL);
 	return OPERATOR_FINISHED;
 }
 
@@ -747,7 +757,7 @@ void NODE_OT_select_linked_from(wmOperatorType *ot)
 	ot->poll = ED_operator_node_active;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
 
 /* ****** Select Same Type ****** */
@@ -758,9 +768,9 @@ static int node_select_same_type_exec(bContext *C, wmOperator *UNUSED(op))
 
 	node_select_same_type(snode);
 
-	node_sort(snode->edittree);
+	ED_node_sort(snode->edittree);
 
-	WM_event_add_notifier(C, NC_NODE|NA_SELECTED, NULL);
+	WM_event_add_notifier(C, NC_NODE | NA_SELECTED, NULL);
 	return OPERATOR_FINISHED;
 }
 
@@ -776,7 +786,7 @@ void NODE_OT_select_same_type(wmOperatorType *ot)
 	ot->poll = ED_operator_node_active;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
 
 /* ****** Select The Next/Prev Node Of The Same Type ****** */
@@ -787,9 +797,9 @@ static int node_select_same_type_next_exec(bContext *C, wmOperator *UNUSED(op))
 
 	node_select_same_type_np(snode, 0);
 
-	node_sort(snode->edittree);
+	ED_node_sort(snode->edittree);
 
-	WM_event_add_notifier(C, NC_NODE|NA_SELECTED, NULL);
+	WM_event_add_notifier(C, NC_NODE | NA_SELECTED, NULL);
 
 	return OPERATOR_FINISHED;
 }
@@ -806,7 +816,7 @@ void NODE_OT_select_same_type_next(wmOperatorType *ot)
 	ot->poll = ED_operator_node_active;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
 
 static int node_select_same_type_prev_exec(bContext *C, wmOperator *UNUSED(op))
@@ -815,9 +825,9 @@ static int node_select_same_type_prev_exec(bContext *C, wmOperator *UNUSED(op))
 
 	node_select_same_type_np(snode, 1);
 
-	node_sort(snode->edittree);
+	ED_node_sort(snode->edittree);
 
-	WM_event_add_notifier(C, NC_NODE|NA_SELECTED, NULL);
+	WM_event_add_notifier(C, NC_NODE | NA_SELECTED, NULL);
 	return OPERATOR_FINISHED;
 }
 
@@ -833,5 +843,5 @@ void NODE_OT_select_same_type_prev(wmOperatorType *ot)
 	ot->poll = ED_operator_node_active;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER|OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
