@@ -24,43 +24,37 @@
  *  \ingroup edinterface
  */
 
-#include <stdlib.h>
-#include <stddef.h>
 #include <string.h>
 
 #include "MEM_guardedalloc.h"
 
 #include "DNA_node_types.h"
-#include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
+#include "DNA_space_types.h"
 
 #include "BLI_listbase.h"
 #include "BLI_string.h"
-#include "BLI_utildefines.h"
 
 #include "BLF_translation.h"
 
 #include "BKE_context.h"
-#include "BKE_depsgraph.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
-#include "BKE_node.h"
 #include "BKE_scene.h"
 
 #include "RNA_access.h"
 
 #include "NOD_socket.h"
 
-#include "WM_api.h"
-#include "WM_types.h"
-
 #include "UI_interface.h"
-#include "UI_resources.h"
-#include "../interface/interface_intern.h"
+#include "../interface/interface_intern.h"  /* XXX bad level */
+
+#include "ED_node.h"  /* own include */
 
-#include "ED_node.h"
 #include "ED_util.h"
 
+#include "node_intern.h"
+
 /************************* Node Socket Manipulation **************************/
 
 static void node_tag_recursive(bNode *node)
@@ -68,11 +62,11 @@ static void node_tag_recursive(bNode *node)
 	bNodeSocket *input;
 
 	if (!node || (node->flag & NODE_TEST))
-		return; /* in case of cycles */
-	
+		return;  /* in case of cycles */
+
 	node->flag |= NODE_TEST;
 
-	for (input=node->inputs.first; input; input=input->next)
+	for (input = node->inputs.first; input; input = input->next)
 		if (input->link)
 			node_tag_recursive(input->link->fromnode);
 }
@@ -82,11 +76,11 @@ static void node_clear_recursive(bNode *node)
 	bNodeSocket *input;
 
 	if (!node || !(node->flag & NODE_TEST))
-		return; /* in case of cycles */
-	
+		return;  /* in case of cycles */
+
 	node->flag &= ~NODE_TEST;
 
-	for (input=node->inputs.first; input; input=input->next)
+	for (input = node->inputs.first; input; input = input->next)
 		if (input->link)
 			node_clear_recursive(input->link->fromnode);
 }
@@ -100,20 +94,20 @@ static void node_remove_linked(bNodeTree *ntree, bNode *rem_node)
 		return;
 
 	/* tag linked nodes to be removed */
-	for (node=ntree->nodes.first; node; node=node->next)
+	for (node = ntree->nodes.first; node; node = node->next)
 		node->flag &= ~NODE_TEST;
-	
+
 	node_tag_recursive(rem_node);
 
 	/* clear tags on nodes that are still used by other nodes */
-	for (node=ntree->nodes.first; node; node=node->next)
+	for (node = ntree->nodes.first; node; node = node->next)
 		if (!(node->flag & NODE_TEST))
-			for (sock=node->inputs.first; sock; sock=sock->next)
+			for (sock = node->inputs.first; sock; sock = sock->next)
 				if (sock->link && sock->link->fromnode != rem_node)
 					node_clear_recursive(sock->link->fromnode);
 
 	/* remove nodes */
-	for (node=ntree->nodes.first; node; node=next) {
+	for (node = ntree->nodes.first; node; node = next) {
 		next = node->next;
 
 		if (node->flag & NODE_TEST) {
@@ -131,6 +125,7 @@ static void node_socket_disconnect(Main *bmain, bNodeTree *ntree, bNode *node_to
 		return;
 
 	nodeRemLink(ntree, sock_to->link);
+	sock_to->flag |= SOCK_COLLAPSED;
 
 	nodeUpdate(ntree, node_to);
 	ntreeUpdateTree(ntree);
@@ -145,6 +140,7 @@ static void node_socket_remove(Main *bmain, bNodeTree *ntree, bNode *node_to, bN
 		return;
 
 	node_remove_linked(ntree, sock_to->link->fromnode);
+	sock_to->flag |= SOCK_COLLAPSED;
 
 	nodeUpdate(ntree, node_to);
 	ntreeUpdateTree(ntree);
@@ -156,7 +152,7 @@ static void node_socket_remove(Main *bmain, bNodeTree *ntree, bNode *node_to, bN
 static void node_socket_add_replace(Main *bmain, bNodeTree *ntree, bNode *node_to, bNodeSocket *sock_to, bNodeTemplate *ntemp, int sock_num)
 {
 	bNode *node_from;
-	bNodeSocket *sock_from;
+	bNodeSocket *sock_from_tmp;
 	bNode *node_prev = NULL;
 
 	/* unlink existing node */
@@ -166,7 +162,7 @@ static void node_socket_add_replace(Main *bmain, bNodeTree *ntree, bNode *node_t
 	}
 
 	/* find existing node that we can use */
-	for (node_from=ntree->nodes.first; node_from; node_from=node_from->next)
+	for (node_from = ntree->nodes.first; node_from; node_from = node_from->next)
 		if (node_from->type == ntemp->type)
 			break;
 
@@ -175,12 +171,13 @@ static void node_socket_add_replace(Main *bmain, bNodeTree *ntree, bNode *node_t
 			node_from = NULL;
 
 	if (node_prev && node_prev->type == ntemp->type &&
-		(ntemp->type != NODE_GROUP || node_prev->id == &ntemp->ngroup->id)) {
+	    (ntemp->type != NODE_GROUP || node_prev->id == &ntemp->ngroup->id))
+	{
 		/* keep the previous node if it's the same type */
 		node_from = node_prev;
 	}
 	else if (!node_from) {
-		node_from= nodeAddNode(ntree, ntemp);
+		node_from = nodeAddNode(ntree, ntemp);
 		node_from->locx = node_to->locx - (node_from->typeinfo->width + 50);
 		node_from->locy = node_to->locy;
 
@@ -191,18 +188,19 @@ static void node_socket_add_replace(Main *bmain, bNodeTree *ntree, bNode *node_t
 	nodeSetActive(ntree, node_from);
 
 	/* add link */
-	sock_from = BLI_findlink(&node_from->outputs, sock_num);
-	nodeAddLink(ntree, node_from, sock_from, node_to, sock_to);
+	sock_from_tmp = BLI_findlink(&node_from->outputs, sock_num);
+	nodeAddLink(ntree, node_from, sock_from_tmp, node_to, sock_to);
+	sock_to->flag &= ~SOCK_COLLAPSED;
 
 	/* copy input sockets from previous node */
 	if (node_prev && node_from != node_prev) {
 		bNodeSocket *sock_prev, *sock_from;
 
-		for (sock_prev=node_prev->inputs.first; sock_prev; sock_prev=sock_prev->next) {
-			for (sock_from=node_from->inputs.first; sock_from; sock_from=sock_from->next) {
+		for (sock_prev = node_prev->inputs.first; sock_prev; sock_prev = sock_prev->next) {
+			for (sock_from = node_from->inputs.first; sock_from; sock_from = sock_from->next) {
 				if (nodeCountSocketLinks(ntree, sock_from) >= sock_from->limit)
 					continue;
-				
+
 				if (strcmp(sock_prev->name, sock_from->name) == 0 && sock_prev->type == sock_from->type) {
 					bNodeLink *link = sock_prev->link;
 
@@ -220,8 +218,10 @@ static void node_socket_add_replace(Main *bmain, bNodeTree *ntree, bNode *node_t
 
 		/* also preserve mapping for texture nodes */
 		if (node_from->typeinfo->nclass == NODE_CLASS_TEXTURE &&
-		   node_prev->typeinfo->nclass == NODE_CLASS_TEXTURE)
+		    node_prev->typeinfo->nclass == NODE_CLASS_TEXTURE)
+		{
 			memcpy(node_from->storage, node_prev->storage, sizeof(NodeTexBase));
+		}
 
 		/* remove node */
 		node_remove_linked(ntree, node_prev);
@@ -236,9 +236,9 @@ static void node_socket_add_replace(Main *bmain, bNodeTree *ntree, bNode *node_t
 
 /****************************** Node Link Menu *******************************/
 
-#define UI_NODE_LINK_ADD		0
-#define UI_NODE_LINK_DISCONNECT	-1
-#define UI_NODE_LINK_REMOVE		-2
+// #define UI_NODE_LINK_ADD        0
+#define UI_NODE_LINK_DISCONNECT -1
+#define UI_NODE_LINK_REMOVE     -2
 
 typedef struct NodeLinkArg {
 	Main *bmain;
@@ -256,7 +256,7 @@ typedef struct NodeLinkArg {
 
 static void ui_node_link(bContext *C, void *arg_p, void *event_p)
 {
-	NodeLinkArg *arg = (NodeLinkArg*)arg_p;
+	NodeLinkArg *arg = (NodeLinkArg *)arg_p;
 	Main *bmain = arg->bmain;
 	bNode *node_to = arg->node;
 	bNodeSocket *sock_to = arg->sock;
@@ -275,7 +275,7 @@ static void ui_node_link(bContext *C, void *arg_p, void *event_p)
 		node_socket_remove(bmain, ntree, node_to, sock_to);
 	else
 		node_socket_add_replace(bmain, ntree, node_to, sock_to, &ntemp, arg->output);
-	
+
 	ED_undo_push(C, "Node input modify");
 }
 
@@ -287,24 +287,26 @@ static void ui_node_sock_name(bNodeSocket *sock, char name[UI_MAX_NAME_STR])
 
 		if (node->type == NODE_GROUP) {
 			if (node->id)
-				BLI_strncpy(node_name, node->id->name+2, UI_MAX_NAME_STR);
+				BLI_strncpy(node_name, node->id->name + 2, UI_MAX_NAME_STR);
 			else
-				BLI_strncpy(node_name, "Group", UI_MAX_NAME_STR);
+				BLI_strncpy(node_name, N_("Group"), UI_MAX_NAME_STR);
 		}
 		else
 			BLI_strncpy(node_name, node->typeinfo->name, UI_MAX_NAME_STR);
 
 		if (node->inputs.first == NULL &&
-		   node->outputs.first != node->outputs.last &&
-		   !(node->typeinfo->flag & NODE_OPTIONS))
-			BLI_snprintf(name, UI_MAX_NAME_STR, "%s | %s", node_name, sock->link->fromsock->name);
-		else
-			BLI_strncpy(name, node_name, UI_MAX_NAME_STR);
+		    node->outputs.first != node->outputs.last)
+		{
+			BLI_snprintf(name, UI_MAX_NAME_STR, "%s | %s", IFACE_(node_name), IFACE_(sock->link->fromsock->name));
+		}
+		else {
+			BLI_strncpy(name, IFACE_(node_name), UI_MAX_NAME_STR);
+		}
 	}
 	else if (sock->type == SOCK_SHADER)
-		BLI_strncpy(name, "None", UI_MAX_NAME_STR);
+		BLI_strncpy(name, IFACE_("None"), UI_MAX_NAME_STR);
 	else
-		BLI_strncpy(name, "Default", UI_MAX_NAME_STR);
+		BLI_strncpy(name, IFACE_("Default"), UI_MAX_NAME_STR);
 }
 
 static int ui_compatible_sockets(int typeA, int typeB)
@@ -325,17 +327,17 @@ static void ui_node_menu_column(NodeLinkArg *arg, int nclass, const char *cname)
 	bNodeTree *ngroup;
 	NodeLinkArg *argN;
 	int first = 1;
-	int compatibility= 0;
+	int compatibility = 0;
 
 	if (ntree->type == NTREE_SHADER) {
-		if (scene_use_new_shading_nodes(arg->scene))
-			compatibility= NODE_NEW_SHADING;
+		if (BKE_scene_use_new_shading_nodes(arg->scene))
+			compatibility = NODE_NEW_SHADING;
 		else
-			compatibility= NODE_OLD_SHADING;
+			compatibility = NODE_OLD_SHADING;
 	}
 
 	if (nclass == NODE_CLASS_GROUP) {
-		for (ngroup=bmain->nodetree.first; ngroup; ngroup=ngroup->id.next) {
+		for (ngroup = bmain->nodetree.first; ngroup; ngroup = ngroup->id.next) {
 			bNodeSocket *gsock;
 			char name[UI_MAX_NAME_STR];
 			int i, j, num = 0;
@@ -343,40 +345,40 @@ static void ui_node_menu_column(NodeLinkArg *arg, int nclass, const char *cname)
 			if (ngroup->type != ntree->type)
 				continue;
 
-			for (gsock=ngroup->inputs.first; gsock; gsock=gsock->next)
+			for (gsock = ngroup->inputs.first; gsock; gsock = gsock->next)
 				if (ui_compatible_sockets(gsock->type, sock->type))
 					num++;
 
-			for (i=0, j=0, gsock=ngroup->outputs.first; gsock; gsock=gsock->next, i++) {
+			for (i = 0, j = 0, gsock = ngroup->outputs.first; gsock; gsock = gsock->next, i++) {
 				if (!ui_compatible_sockets(gsock->type, sock->type))
 					continue;
 
 				if (first) {
-					column= uiLayoutColumn(layout, 0);
+					column = uiLayoutColumn(layout, FALSE);
 					uiBlockSetCurLayout(block, column);
 
-					uiItemL(column, cname, ICON_NODE);
-					but= block->buttons.last;
-					but->flag= UI_TEXT_LEFT;
+					uiItemL(column, IFACE_(cname), ICON_NODE);
+					but = block->buttons.last;
+					but->flag = UI_TEXT_LEFT;
 
 					first = 0;
 				}
 
 				if (num > 1) {
 					if (j == 0) {
-						uiItemL(column, ngroup->id.name+2, ICON_NODE);
-						but= block->buttons.last;
-						but->flag= UI_TEXT_LEFT;
+						uiItemL(column, ngroup->id.name + 2, ICON_NODE);
+						but = block->buttons.last;
+						but->flag = UI_TEXT_LEFT;
 					}
 
 					BLI_snprintf(name, UI_MAX_NAME_STR, "  %s", gsock->name);
 					j++;
 				}
 				else
-					BLI_strncpy(name, ngroup->id.name+2, UI_MAX_NAME_STR);
+					BLI_strncpy(name, ngroup->id.name + 2, UI_MAX_NAME_STR);
 
-				but = uiDefBut(block, BUT, 0, ngroup->id.name+2, 0, 0, UI_UNIT_X*4, UI_UNIT_Y,
-					NULL, 0.0, 0.0, 0.0, 0.0, "Add node to input");
+				but = uiDefBut(block, BUT, 0, ngroup->id.name + 2, 0, 0, UI_UNIT_X * 4, UI_UNIT_Y,
+				               NULL, 0.0, 0.0, 0.0, 0.0, TIP_("Add node to input"));
 
 				argN = MEM_dupallocN(arg);
 				argN->type = NODE_GROUP;
@@ -387,9 +389,9 @@ static void ui_node_menu_column(NodeLinkArg *arg, int nclass, const char *cname)
 		}
 	}
 	else {
-		bNodeTreeType *ttype= ntreeGetType(ntree->type);
+		bNodeTreeType *ttype = ntreeGetType(ntree->type);
 
-		for (ntype=ttype->node_types.first; ntype; ntype=ntype->next) {
+		for (ntype = ttype->node_types.first; ntype; ntype = ntype->next) {
 			bNodeSocketTemplate *stemp;
 			char name[UI_MAX_NAME_STR];
 			int i, j, num = 0;
@@ -400,40 +402,40 @@ static void ui_node_menu_column(NodeLinkArg *arg, int nclass, const char *cname)
 			if (ntype->nclass != nclass)
 				continue;
 
-			for (i=0, stemp=ntype->outputs; stemp && stemp->type != -1; stemp++, i++)
+			for (i = 0, stemp = ntype->outputs; stemp && stemp->type != -1; stemp++, i++)
 				if (ui_compatible_sockets(stemp->type, sock->type))
 					num++;
 
-			for (i=0, j=0, stemp=ntype->outputs; stemp && stemp->type != -1; stemp++, i++) {
+			for (i = 0, j = 0, stemp = ntype->outputs; stemp && stemp->type != -1; stemp++, i++) {
 				if (!ui_compatible_sockets(stemp->type, sock->type))
 					continue;
 
 				if (first) {
-					column= uiLayoutColumn(layout, 0);
+					column = uiLayoutColumn(layout, FALSE);
 					uiBlockSetCurLayout(block, column);
 
-					uiItemL(column, cname, ICON_NODE);
-					but= block->buttons.last;
-					but->flag= UI_TEXT_LEFT;
+					uiItemL(column, IFACE_(cname), ICON_NODE);
+					but = block->buttons.last;
+					but->flag = UI_TEXT_LEFT;
 
 					first = 0;
 				}
 
 				if (num > 1) {
 					if (j == 0) {
-						uiItemL(column, ntype->name, ICON_NODE);
-						but= block->buttons.last;
-						but->flag= UI_TEXT_LEFT;
+						uiItemL(column, IFACE_(ntype->name), ICON_NODE);
+						but = block->buttons.last;
+						but->flag = UI_TEXT_LEFT;
 					}
 
-					BLI_snprintf(name, UI_MAX_NAME_STR, "  %s", stemp->name);
+					BLI_snprintf(name, UI_MAX_NAME_STR, "  %s", IFACE_(stemp->name));
 					j++;
 				}
 				else
-					BLI_strncpy(name, ntype->name, UI_MAX_NAME_STR);
+					BLI_strncpy(name, IFACE_(ntype->name), UI_MAX_NAME_STR);
 
-				but = uiDefBut(block, BUT, 0, name, 0, 0, UI_UNIT_X*4, UI_UNIT_Y,
-					NULL, 0.0, 0.0, 0.0, 0.0, "Add node to input");
+				but = uiDefBut(block, BUT, 0, name, 0, 0, UI_UNIT_X * 4, UI_UNIT_Y,
+				               NULL, 0.0, 0.0, 0.0, 0.0, TIP_("Add node to input"));
 
 				argN = MEM_dupallocN(arg);
 				argN->type = ntype->type;
@@ -446,7 +448,7 @@ static void ui_node_menu_column(NodeLinkArg *arg, int nclass, const char *cname)
 
 static void node_menu_column_foreach_cb(void *calldata, int nclass, const char *name)
 {
-	NodeLinkArg *arg = (NodeLinkArg*)calldata;
+	NodeLinkArg *arg = (NodeLinkArg *)calldata;
 
 	if (!ELEM(nclass, NODE_CLASS_GROUP, NODE_CLASS_LAYOUT))
 		ui_node_menu_column(arg, nclass, name);
@@ -454,43 +456,43 @@ static void node_menu_column_foreach_cb(void *calldata, int nclass, const char *
 
 static void ui_template_node_link_menu(bContext *C, uiLayout *layout, void *but_p)
 {
-	Main *bmain= CTX_data_main(C);
-	Scene *scene= CTX_data_scene(C);
+	Main *bmain = CTX_data_main(C);
+	Scene *scene = CTX_data_scene(C);
 	uiBlock *block = uiLayoutGetBlock(layout);
-	uiBut *but = (uiBut*)but_p;
+	uiBut *but = (uiBut *)but_p;
 	uiLayout *split, *column;
-	NodeLinkArg *arg = (NodeLinkArg*)but->func_argN;
+	NodeLinkArg *arg = (NodeLinkArg *)but->func_argN;
 	bNodeSocket *sock = arg->sock;
-	bNodeTreeType *ntreetype= ntreeGetType(arg->ntree->type);
+	bNodeTreeType *ntreetype = ntreeGetType(arg->ntree->type);
 
 	uiBlockSetCurLayout(block, layout);
-	split= uiLayoutSplit(layout, 0, 0);
+	split = uiLayoutSplit(layout, 0.0f, FALSE);
+
+	arg->bmain = bmain;
+	arg->scene = scene;
+	arg->layout = split;
 
-	arg->bmain= bmain;
-	arg->scene= scene;
-	arg->layout= split;
-	
 	if (ntreetype && ntreetype->foreach_nodeclass)
 		ntreetype->foreach_nodeclass(scene, arg, node_menu_column_foreach_cb);
 
-	column= uiLayoutColumn(split, 0);
+	column = uiLayoutColumn(split, FALSE);
 	uiBlockSetCurLayout(block, column);
 
 	if (sock->link) {
-		uiItemL(column, "Link", ICON_NONE);
-		but= block->buttons.last;
-		but->flag= UI_TEXT_LEFT;
+		uiItemL(column, IFACE_("Link"), ICON_NONE);
+		but = block->buttons.last;
+		but->flag = UI_TEXT_LEFT;
 
-		but = uiDefBut(block, BUT, 0, "Remove", 0, 0, UI_UNIT_X*4, UI_UNIT_Y,
-			NULL, 0.0, 0.0, 0.0, 0.0, "Remove nodes connected to the input");
+		but = uiDefBut(block, BUT, 0, IFACE_("Remove"), 0, 0, UI_UNIT_X * 4, UI_UNIT_Y,
+		               NULL, 0.0, 0.0, 0.0, 0.0, TIP_("Remove nodes connected to the input"));
 		uiButSetNFunc(but, ui_node_link, MEM_dupallocN(arg), SET_INT_IN_POINTER(UI_NODE_LINK_REMOVE));
 
-		but = uiDefBut(block, BUT, 0, "Disconnect", 0, 0, UI_UNIT_X*4, UI_UNIT_Y,
-			NULL, 0.0, 0.0, 0.0, 0.0, "Disconnect nodes connected to the input");
+		but = uiDefBut(block, BUT, 0, IFACE_("Disconnect"), 0, 0, UI_UNIT_X * 4, UI_UNIT_Y,
+		               NULL, 0.0, 0.0, 0.0, 0.0, TIP_("Disconnect nodes connected to the input"));
 		uiButSetNFunc(but, ui_node_link, MEM_dupallocN(arg), SET_INT_IN_POINTER(UI_NODE_LINK_DISCONNECT));
 	}
 
-	ui_node_menu_column(arg, NODE_CLASS_GROUP, IFACE_("Group"));
+	ui_node_menu_column(arg, NODE_CLASS_GROUP, N_("Group"));
 }
 
 void uiTemplateNodeLink(uiLayout *layout, bNodeTree *ntree, bNode *node, bNodeSocket *sock)
@@ -511,14 +513,14 @@ void uiTemplateNodeLink(uiLayout *layout, bNodeTree *ntree, bNode *node, bNodeSo
 	if (sock->link || sock->type == SOCK_SHADER || (sock->flag & SOCK_HIDE_VALUE)) {
 		char name[UI_MAX_NAME_STR];
 		ui_node_sock_name(sock, name);
-		but = uiDefMenuBut(block, ui_template_node_link_menu, NULL, name, 0, 0, UI_UNIT_X*4, UI_UNIT_Y, "");
+		but = uiDefMenuBut(block, ui_template_node_link_menu, NULL, name, 0, 0, UI_UNIT_X * 4, UI_UNIT_Y, "");
 	}
 	else
 		but = uiDefIconMenuBut(block, ui_template_node_link_menu, NULL, ICON_NONE, 0, 0, UI_UNIT_X, UI_UNIT_Y, "");
 
-	but->type= MENU;
-	but->flag |= UI_TEXT_LEFT|UI_BUT_NODE_LINK;
-	but->poin= (char*)but;
+	but->type = MENU;
+	but->flag |= UI_TEXT_LEFT | UI_BUT_NODE_LINK;
+	but->poin = (char *)but;
 	but->func_argN = arg;
 
 	if (sock->link && sock->link->fromnode)
@@ -529,7 +531,7 @@ void uiTemplateNodeLink(uiLayout *layout, bNodeTree *ntree, bNode *node, bNodeSo
 /**************************** Node Tree Layout *******************************/
 
 static void ui_node_draw_input(uiLayout *layout, bContext *C,
-	bNodeTree *ntree, bNode *node, bNodeSocket *input, int depth);
+                               bNodeTree *ntree, bNode *node, bNodeSocket *input, int depth);
 
 static void ui_node_draw_node(uiLayout *layout, bContext *C, bNodeTree *ntree, bNode *node, int depth)
 {
@@ -541,16 +543,16 @@ static void ui_node_draw_node(uiLayout *layout, bContext *C, bNodeTree *ntree, b
 
 	if (node->typeinfo->uifunc) {
 		if (node->type != NODE_GROUP) {
-			split = uiLayoutSplit(layout, 0.35f, 0);
-			col = uiLayoutColumn(split, 0);
-			col = uiLayoutColumn(split, 0);
+			split = uiLayoutSplit(layout, 0.35f, FALSE);
+			col = uiLayoutColumn(split, FALSE);
+			col = uiLayoutColumn(split, FALSE);
 
 			node->typeinfo->uifunc(col, C, &nodeptr);
 		}
 	}
 
-	for (input=node->inputs.first; input; input=input->next)
-		ui_node_draw_input(layout, C, ntree, node, input, depth+1);
+	for (input = node->inputs.first; input; input = input->next)
+		ui_node_draw_input(layout, C, ntree, node, input, depth + 1);
 }
 
 static void ui_node_draw_input(uiLayout *layout, bContext *C, bNodeTree *ntree, bNode *node, bNodeSocket *input, int depth)
@@ -561,7 +563,7 @@ static void ui_node_draw_input(uiLayout *layout, bContext *C, bNodeTree *ntree,
 	uiLayout *split, *row, *col;
 	bNode *lnode;
 	char label[UI_MAX_NAME_STR];
-	int indent = (depth > 1)? 2*(depth - 1): 0;
+	int indent = (depth > 1) ? 2 * (depth - 1) : 0;
 	int dependency_loop;
 
 	if (input->flag & SOCK_UNAVAIL)
@@ -569,7 +571,7 @@ static void ui_node_draw_input(uiLayout *layout, bContext *C, bNodeTree *ntree,
 
 	/* to avoid eternal loops on cyclic dependencies */
 	node->flag |= NODE_TEST;
-	lnode = (input->link)? input->link->fromnode: NULL;
+	lnode = (input->link) ? input->link->fromnode : NULL;
 
 	dependency_loop = (lnode && (lnode->flag & NODE_TEST));
 	if (dependency_loop)
@@ -581,42 +583,42 @@ static void ui_node_draw_input(uiLayout *layout, bContext *C, bNodeTree *ntree,
 	/* indented label */
 	memset(label, ' ', indent);
 	label[indent] = '\0';
-	BLI_snprintf(label, UI_MAX_NAME_STR, "%s%s:", label, input->name);
+	BLI_snprintf(label, UI_MAX_NAME_STR, "%s%s:", label, IFACE_(input->name));
 
 	/* split in label and value */
-	split = uiLayoutSplit(layout, 0.35f, 0);
+	split = uiLayoutSplit(layout, 0.35f, FALSE);
 
-	row = uiLayoutRow(split, 1);
+	row = uiLayoutRow(split, TRUE);
 
 	if (depth > 0) {
 		uiBlockSetEmboss(block, UI_EMBOSSN);
 
 		if (lnode && (lnode->inputs.first || (lnode->typeinfo->uifunc && lnode->type != NODE_GROUP))) {
-			int icon = (input->flag & SOCK_COLLAPSED)? ICON_DISCLOSURE_TRI_RIGHT: ICON_DISCLOSURE_TRI_DOWN;
+			int icon = (input->flag & SOCK_COLLAPSED) ? ICON_DISCLOSURE_TRI_RIGHT : ICON_DISCLOSURE_TRI_DOWN;
 			uiItemR(row, &inputptr, "show_expanded", UI_ITEM_R_ICON_ONLY, "", icon);
 		}
 		else
 			uiItemL(row, "", ICON_BLANK1);
 
 		bt = block->buttons.last;
-		bt->x2 = UI_UNIT_X/2;
+		bt->rect.xmax = UI_UNIT_X / 2;
 
 		uiBlockSetEmboss(block, UI_EMBOSS);
 	}
 
 	uiItemL(row, label, ICON_NONE);
-	bt= block->buttons.last;
-	bt->flag= UI_TEXT_LEFT;
+	bt = block->buttons.last;
+	bt->flag = UI_TEXT_LEFT;
 
 	if (dependency_loop) {
-		row = uiLayoutRow(split, 0);
-		uiItemL(row, "Dependency Loop", ICON_ERROR);
+		row = uiLayoutRow(split, FALSE);
+		uiItemL(row, IFACE_("Dependency Loop"), ICON_ERROR);
 	}
 	else if (lnode) {
 		/* input linked to a node */
 		uiTemplateNodeLink(split, ntree, node, input);
 
-		if (!(input->flag & SOCK_COLLAPSED)) {
+		if (depth == 0 || !(input->flag & SOCK_COLLAPSED)) {
 			if (depth == 0)
 				uiItemS(layout);
 
@@ -627,18 +629,18 @@ static void ui_node_draw_input(uiLayout *layout, bContext *C, bNodeTree *ntree,
 		/* input not linked, show value */
 		if (input->type != SOCK_SHADER && !(input->flag & SOCK_HIDE_VALUE)) {
 			if (input->type == SOCK_VECTOR) {
-				row = uiLayoutRow(split, 0);
-				col = uiLayoutColumn(row, 0);
+				row = uiLayoutRow(split, FALSE);
+				col = uiLayoutColumn(row, FALSE);
 
-				uiItemR(col, &inputptr, "default_value", 0, "", 0);
+				uiItemR(col, &inputptr, "default_value", 0, "", ICON_NONE);
 			}
 			else {
-				row = uiLayoutRow(split, 1);
-				uiItemR(row, &inputptr, "default_value", 0, "", 0);
+				row = uiLayoutRow(split, TRUE);
+				uiItemR(row, &inputptr, "default_value", 0, "", ICON_NONE);
 			}
 		}
 		else
-			row = uiLayoutRow(split, 0);
+			row = uiLayoutRow(split, FALSE);
 
 		uiTemplateNodeLink(row, ntree, node, input);
 	}
@@ -655,7 +657,7 @@ void uiTemplateNodeView(uiLayout *layout, bContext *C, bNodeTree *ntree, bNode *
 		return;
 
 	/* clear for cycle check */
-	for (tnode=ntree->nodes.first; tnode; tnode=tnode->next)
+	for (tnode = ntree->nodes.first; tnode; tnode = tnode->next)
 		tnode->flag &= ~NODE_TEST;
 
 	if (input)
