@@ -25,8 +25,6 @@
  *  \ingroup spseq
  */
 
-
-
 #include <string.h>
 #include <stdio.h>
 
@@ -36,10 +34,14 @@
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
 
+#include "BLF_translation.h"
+
 #include "BKE_context.h"
 #include "BKE_screen.h"
 
 #include "ED_screen.h"
+#include "ED_gpencil.h"
+#include "ED_sequencer.h"
 
 #include "WM_api.h"
 #include "WM_types.h"
@@ -48,6 +50,29 @@
 
 #include "sequencer_intern.h"
 
+/* **************************** buttons ********************************* */
+
+static int sequencer_grease_pencil_panel_poll(const bContext *C, PanelType *UNUSED(pt))
+{
+	SpaceSeq *sseq = CTX_wm_space_seq(C);
+
+	/* don't show the gpencil if we are not showing the image */
+	return ED_space_sequencer_check_show_imbuf(sseq);
+}
+
+void sequencer_buttons_register(ARegionType *art)
+{
+	PanelType *pt;
+	
+	pt = MEM_callocN(sizeof(PanelType), "spacetype sequencer panel gpencil");
+	strcpy(pt->idname, "SEQUENCER_PT_gpencil");
+	strcpy(pt->label, N_("Grease Pencil"));
+	pt->draw_header = gpencil_panel_standard_header;
+	pt->draw = gpencil_panel_standard;
+	pt->poll = sequencer_grease_pencil_panel_poll;
+	BLI_addtail(&art->paneltypes, pt);
+}
+
 /* **************** operator to open/close properties view ************* */
 
 static int sequencer_properties(bContext *C, wmOperator *UNUSED(op))
