@@ -58,6 +58,7 @@
 
 #include "ED_types.h"
 #include "ED_screen.h"
+#include "ED_sequencer.h"
 
 #include "UI_view2d.h"
 
@@ -161,20 +162,20 @@ void select_surround_from_last(Scene *scene)
 }
 #endif
 
-
-static void UNUSED_FUNCTION(select_single_seq) (Scene * scene, Sequence * seq, int deselect_all) /* BRING BACK */
+void ED_sequencer_select_sequence_single(Scene * scene, Sequence * seq, int deselect_all)
 {
-	Editing *ed = seq_give_editing(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
 	
 	if (deselect_all)
-		deselect_all_seq(scene);
-	seq_active_set(scene, seq);
+		ED_sequencer_deselect_all(scene);
 
-	if ((seq->type == SEQ_IMAGE) || (seq->type == SEQ_MOVIE)) {
+	BKE_sequencer_active_set(scene, seq);
+
+	if ((seq->type == SEQ_TYPE_IMAGE) || (seq->type == SEQ_TYPE_MOVIE)) {
 		if (seq->strip)
 			BLI_strncpy(ed->act_imagedir, seq->strip->dir, FILE_MAXDIR);
 	}
-	else if (seq->type == SEQ_SOUND) {
+	else if (seq->type == SEQ_TYPE_SOUND_RAM) {
 		if (seq->strip)
 			BLI_strncpy(ed->act_sounddir, seq->strip->dir, FILE_MAXDIR);
 	}
@@ -185,7 +186,7 @@ static void UNUSED_FUNCTION(select_single_seq) (Scene * scene, Sequence * seq, i
 #if 0
 static void select_neighbor_from_last(Scene *scene, int lr)
 {
-	Sequence *seq = seq_active_get(scene);
+	Sequence *seq = BKE_sequencer_active_get(scene);
 	Sequence *neighbor;
 	int change = 0;
 	if (seq) {
@@ -220,7 +221,7 @@ static int sequencer_de_select_all_exec(bContext *C, wmOperator *op)
 	int action = RNA_enum_get(op->ptr, "action");
 
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = seq_give_editing(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
 	Sequence *seq;
 
 	if (action == SEL_TOGGLE) {
@@ -281,7 +282,7 @@ void SEQUENCER_OT_select_all(struct wmOperatorType *ot)
 static int sequencer_select_inverse_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = seq_give_editing(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
 	Sequence *seq;
 
 	for (seq = ed->seqbasep->first; seq; seq = seq->next) {
@@ -318,7 +319,7 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 {
 	View2D *v2d = UI_view2d_fromcontext(C);
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = seq_give_editing(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
 	short extend = RNA_boolean_get(op->ptr, "extend");
 	short linked_handle = RNA_boolean_get(op->ptr, "linked_handle");
 	short left_right = RNA_boolean_get(op->ptr, "left_right");
@@ -352,17 +353,17 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 		}
 		else {
 			/* deselect_markers(0, 0); */ /* XXX, in 2.4x, seq selection used to deselect all, need to re-thnik this for 2.5 */
-			marker->flag |= SELECT;				
+			marker->flag |= SELECT;
 		}
 		
 	}
 	else if (left_right) {
 		/* use different logic for this */
 		float x;
-		deselect_all_seq(scene);
+		ED_sequencer_deselect_all(scene);
 		UI_view2d_region_to_view(v2d, event->mval[0], event->mval[1], &x, NULL);
 
-		SEQP_BEGIN(ed, seq)
+		SEQP_BEGIN (ed, seq)
 		{
 			if (x < CFRA) {
 				if (seq->enddisp < CFRA) {
@@ -398,23 +399,22 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 		}
 	}
 	else {
-		// seq= find_nearest_seq(scene, v2d, &hand, mval);
+		// seq = find_nearest_seq(scene, v2d, &hand, mval);
 
 		act_orig = ed->act_seq;
 
 		if (extend == 0 && linked_handle == 0)
-			deselect_all_seq(scene);
+			ED_sequencer_deselect_all(scene);
 	
 		if (seq) {
-			seq_active_set(scene, seq);
+			BKE_sequencer_active_set(scene, seq);
 	
-			if ((seq->type == SEQ_IMAGE) || (seq->type == SEQ_MOVIE)) {
+			if ((seq->type == SEQ_TYPE_IMAGE) || (seq->type == SEQ_TYPE_MOVIE)) {
 				if (seq->strip) {
 					BLI_strncpy(ed->act_imagedir, seq->strip->dir, FILE_MAXDIR);
 				}
 			}
-			else
-			if (seq->type == SEQ_SOUND) {
+			else if (seq->type == SEQ_TYPE_SOUND_RAM) {
 				if (seq->strip) {
 					BLI_strncpy(ed->act_sounddir, seq->strip->dir, FILE_MAXDIR);
 				}
@@ -442,7 +442,7 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 			
 			/* On Alt selection, select the strip and bordering handles */
 			if (linked_handle && !ELEM(hand, SEQ_SIDE_LEFT, SEQ_SIDE_RIGHT)) {
-				if (extend == 0) deselect_all_seq(scene);
+				if (extend == 0) ED_sequencer_deselect_all(scene);
 				seq->flag |= SELECT;
 				select_surrounding_handles(scene, seq);
 			}
@@ -458,13 +458,13 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 					switch (sel_side) {
 						case SEQ_SIDE_LEFT:
 							if ((seq->flag & SEQ_LEFTSEL) && (neighbor->flag & SEQ_RIGHTSEL)) {
-								if (extend == 0) deselect_all_seq(scene);
+								if (extend == 0) ED_sequencer_deselect_all(scene);
 								seq->flag |= SELECT;
 
 								select_active_side(ed->seqbasep, SEQ_SIDE_LEFT, seq->machine, seq->startdisp);
 							}
 							else {
-								if (extend == 0) deselect_all_seq(scene);
+								if (extend == 0) ED_sequencer_deselect_all(scene);
 								seq->flag |= SELECT;
 
 								neighbor->flag |= SELECT;
@@ -475,13 +475,13 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 							break;
 						case SEQ_SIDE_RIGHT:
 							if ((seq->flag & SEQ_RIGHTSEL) && (neighbor->flag & SEQ_LEFTSEL)) {
-								if (extend == 0) deselect_all_seq(scene);
+								if (extend == 0) ED_sequencer_deselect_all(scene);
 								seq->flag |= SELECT;
 
 								select_active_side(ed->seqbasep, SEQ_SIDE_RIGHT, seq->machine, seq->startdisp);
 							}
 							else {
-								if (extend == 0) deselect_all_seq(scene);
+								if (extend == 0) ED_sequencer_deselect_all(scene);
 								seq->flag |= SELECT;
 
 								neighbor->flag |= SELECT;
@@ -493,7 +493,7 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 					}
 				}
 				else {
-					if (extend == 0) deselect_all_seq(scene);
+					if (extend == 0) ED_sequencer_deselect_all(scene);
 					select_active_side(ed->seqbasep, sel_side, seq->machine, seq->startdisp);
 				}
 			}
@@ -557,7 +557,7 @@ void SEQUENCER_OT_select(wmOperatorType *ot)
 /* run recursively to select linked */
 static int select_more_less_seq__internal(Scene *scene, int sel, int linked)
 {
-	Editing *ed = seq_give_editing(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
 	Sequence *seq, *neighbor;
 	int change = 0;
 	int isel;
@@ -586,14 +586,14 @@ static int select_more_less_seq__internal(Scene *scene, int sel, int linked)
 				/* only get unselected nabours */
 				neighbor = find_neighboring_sequence(scene, seq, SEQ_SIDE_LEFT, isel);
 				if (neighbor) {
-					if (sel) {neighbor->flag |= SELECT; recurs_sel_seq(neighbor); }
+					if (sel) { neighbor->flag |= SELECT; recurs_sel_seq(neighbor); }
 					else neighbor->flag &= ~SELECT;
 					if (linked == 0) neighbor->tmp = (Sequence *)1;
 					change = 1;
 				}
 				neighbor = find_neighboring_sequence(scene, seq, SEQ_SIDE_RIGHT, isel);
 				if (neighbor) {
-					if (sel) {neighbor->flag |= SELECT; recurs_sel_seq(neighbor); }
+					if (sel) { neighbor->flag |= SELECT; recurs_sel_seq(neighbor); }
 					else neighbor->flag &= ~SELECT;
 					if (linked == 0) neighbor->tmp = (void *)1;
 					change = 1;
@@ -686,7 +686,7 @@ static int sequencer_select_linked_pick_invoke(bContext *C, wmOperator *op, wmEv
 		return OPERATOR_FINISHED;  /* user error as with mesh?? */
 	
 	if (extend == 0)
-		deselect_all_seq(scene);
+		ED_sequencer_deselect_all(scene);
 	
 	mouse_seq->flag |= SELECT;
 	recurs_sel_seq(mouse_seq);
@@ -704,7 +704,7 @@ static int sequencer_select_linked_pick_invoke(bContext *C, wmOperator *op, wmEv
 void SEQUENCER_OT_select_linked_pick(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Select pick linked";
+	ot->name = "Select Pick Linked";
 	ot->idname = "SEQUENCER_OT_select_linked_pick";
 	ot->description = "Select a chain of linked strips nearest to the mouse pointer";
 	
@@ -739,7 +739,7 @@ static int sequencer_select_linked_exec(bContext *C, wmOperator *UNUSED(op))
 void SEQUENCER_OT_select_linked(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Select linked";
+	ot->name = "Select Linked";
 	ot->idname = "SEQUENCER_OT_select_linked";
 	ot->description = "Select all strips adjacent to the current selection";
 	
@@ -758,7 +758,7 @@ void SEQUENCER_OT_select_linked(wmOperatorType *ot)
 static int sequencer_select_handles_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = seq_give_editing(scene, 0);
+	Editing *ed = BKE_sequencer_editing_get(scene, 0);
 	Sequence *seq;
 	int sel_side = RNA_enum_get(op->ptr, "side");
 
@@ -808,8 +808,8 @@ void SEQUENCER_OT_select_handles(wmOperatorType *ot)
 static int sequencer_select_active_side_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = seq_give_editing(scene, 0);
-	Sequence *seq_act = seq_active_get(scene);
+	Editing *ed = BKE_sequencer_editing_get(scene, 0);
+	Sequence *seq_act = BKE_sequencer_active_get(scene);
 
 	if (ed == NULL || seq_act == NULL)
 		return OPERATOR_CANCELLED;
@@ -846,7 +846,7 @@ void SEQUENCER_OT_select_active_side(wmOperatorType *ot)
 static int sequencer_borderselect_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = seq_give_editing(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
 	View2D *v2d = UI_view2d_fromcontext(C);
 	
 	Sequence *seq;
@@ -859,10 +859,7 @@ static int sequencer_borderselect_exec(bContext *C, wmOperator *op)
 	if (ed == NULL)
 		return OPERATOR_CANCELLED;
 
-	rect.xmin = RNA_int_get(op->ptr, "xmin");
-	rect.ymin = RNA_int_get(op->ptr, "ymin");
-	rect.xmax = RNA_int_get(op->ptr, "xmax");
-	rect.ymax = RNA_int_get(op->ptr, "ymax");
+	WM_operator_properties_border_to_rcti(op, &rect);
 	
 	mval[0] = rect.xmin;
 	mval[1] = rect.ymin;
@@ -874,7 +871,7 @@ static int sequencer_borderselect_exec(bContext *C, wmOperator *op)
 	for (seq = ed->seqbasep->first; seq; seq = seq->next) {
 		seq_rectf(seq, &rq);
 		
-		if (BLI_isect_rctf(&rq, &rectf, NULL)) {
+		if (BLI_rctf_isect(&rq, &rectf, NULL)) {
 			if (selecting) seq->flag |= SELECT;
 			else seq->flag &= ~SEQ_ALLSEL;
 			recurs_sel_seq(seq);
@@ -929,18 +926,19 @@ static EnumPropertyItem sequencer_prop_select_grouped_types[] = {
 	{0, NULL, 0, NULL, NULL}
 };
 
-#define SEQ_IS_SOUND(_seq) ((_seq->type & SEQ_SOUND) && !(_seq->type & SEQ_EFFECT))
+#define SEQ_IS_SOUND(_seq) ((_seq->type & SEQ_TYPE_SOUND_RAM) && !(_seq->type & SEQ_TYPE_EFFECT))
 
-#define SEQ_IS_EFFECT(_seq) (_seq->type & SEQ_EFFECT)
+#define SEQ_IS_EFFECT(_seq) (_seq->type & SEQ_TYPE_EFFECT)
 
-#define SEQ_USE_DATA(_seq) (ELEM(_seq->type, SEQ_SCENE, SEQ_MOVIECLIP) || SEQ_HAS_PATH(_seq))
+#define SEQ_USE_DATA(_seq) (ELEM3(_seq->type, SEQ_TYPE_SCENE, SEQ_TYPE_MOVIECLIP, SEQ_TYPE_MASK) || SEQ_HAS_PATH(_seq))
 
 static short select_grouped_type(Editing *ed, Sequence *actseq)
 {
 	Sequence *seq;
 	short changed = FALSE;
 
-	SEQP_BEGIN(ed, seq) {
+	SEQP_BEGIN (ed, seq)
+	{
 		if (seq->type == actseq->type) {
 			seq->flag |= SELECT;
 			changed = TRUE;
@@ -957,7 +955,8 @@ static short select_grouped_type_basic(Editing *ed, Sequence *actseq)
 	short changed = FALSE;
 	short is_sound = SEQ_IS_SOUND(actseq);
 
-	SEQP_BEGIN(ed, seq) {
+	SEQP_BEGIN (ed, seq)
+	{
 		if (is_sound ? SEQ_IS_SOUND(seq) : !SEQ_IS_SOUND(seq)) {
 			seq->flag |= SELECT;
 			changed = TRUE;
@@ -974,7 +973,8 @@ static short select_grouped_type_effect(Editing *ed, Sequence *actseq)
 	short changed = FALSE;
 	short is_effect = SEQ_IS_EFFECT(actseq);
 
-	SEQP_BEGIN(ed, seq) {
+	SEQP_BEGIN (ed, seq)
+	{
 		if (is_effect ? SEQ_IS_EFFECT(seq) : !SEQ_IS_EFFECT(seq)) {
 			seq->flag |= SELECT;
 			changed = TRUE;
@@ -995,7 +995,8 @@ static short select_grouped_data(Editing *ed, Sequence *actseq)
 		return changed;
 
 	if (SEQ_HAS_PATH(actseq) && dir) {
-		SEQP_BEGIN(ed, seq) {
+		SEQP_BEGIN (ed, seq)
+		{
 			if (SEQ_HAS_PATH(seq) && seq->strip && strcmp(seq->strip->dir, dir) == 0) {
 				seq->flag |= SELECT;
 				changed = TRUE;
@@ -1003,20 +1004,33 @@ static short select_grouped_data(Editing *ed, Sequence *actseq)
 		}
 		SEQ_END;
 	}
-	else if (actseq->type == SEQ_SCENE) {
+	else if (actseq->type == SEQ_TYPE_SCENE) {
 		Scene *sce = actseq->scene;
-		SEQP_BEGIN(ed, seq) {
-			if (seq->type == SEQ_SCENE && seq->scene == sce) {
+		SEQP_BEGIN (ed, seq)
+		{
+			if (seq->type == SEQ_TYPE_SCENE && seq->scene == sce) {
 				seq->flag |= SELECT;
 				changed = TRUE;
 			}
 		}
 		SEQ_END;
 	}
-	else if (actseq->type == SEQ_MOVIECLIP) {
+	else if (actseq->type == SEQ_TYPE_MOVIECLIP) {
 		MovieClip *clip = actseq->clip;
-		SEQP_BEGIN(ed, seq) {
-			if (seq->type == SEQ_MOVIECLIP && seq->clip == clip) {
+		SEQP_BEGIN (ed, seq)
+		{
+			if (seq->type == SEQ_TYPE_MOVIECLIP && seq->clip == clip) {
+				seq->flag |= SELECT;
+				changed = TRUE;
+			}
+		}
+		SEQ_END;
+	}
+	else if (actseq->type == SEQ_TYPE_MASK) {
+		struct Mask *mask = actseq->mask;
+		SEQP_BEGIN (ed, seq)
+		{
+			if (seq->type == SEQ_TYPE_MASK && seq->mask == mask) {
 				seq->flag |= SELECT;
 				changed = TRUE;
 			}
@@ -1031,20 +1045,22 @@ static short select_grouped_effect(Editing *ed, Sequence *actseq)
 {
 	Sequence *seq;
 	short changed = FALSE;
-	short effects[SEQ_EFFECT_MAX + 1];
+	short effects[SEQ_TYPE_EFFECT_MAX + 1];
 	int i;
 
-	for (i = 0; i <= SEQ_EFFECT_MAX; i++)
+	for (i = 0; i <= SEQ_TYPE_EFFECT_MAX; i++)
 		effects[i] = FALSE;
 
-	SEQP_BEGIN(ed, seq) {
+	SEQP_BEGIN (ed, seq)
+	{
 		if (ELEM3(actseq, seq->seq1, seq->seq2, seq->seq3)) {
 			effects[seq->type] = TRUE;
 		}
 	}
 	SEQ_END;
 
-	SEQP_BEGIN(ed, seq) {
+	SEQP_BEGIN (ed, seq)
+	{
 		if (effects[seq->type]) {
 			if (seq->seq1) seq->seq1->flag |= SELECT;
 			if (seq->seq2) seq->seq2->flag |= SELECT;
@@ -1062,7 +1078,8 @@ static short select_grouped_time_overlap(Editing *ed, Sequence *actseq)
 	Sequence *seq;
 	short changed = FALSE;
 
-	SEQP_BEGIN(ed, seq) {
+	SEQP_BEGIN (ed, seq)
+	{
 		if (!((seq->startdisp >= actseq->enddisp) || (seq->enddisp < actseq->startdisp))) {
 			seq->flag |= SELECT;
 			changed = TRUE;
@@ -1077,20 +1094,21 @@ static short select_grouped_effect_link(Editing *ed, Sequence *actseq)
 {
 	Sequence *seq = NULL;
 	short changed = FALSE;
-	short is_audio = ((actseq->type == SEQ_META) || SEQ_IS_SOUND(actseq));
+	short is_audio = ((actseq->type == SEQ_TYPE_META) || SEQ_IS_SOUND(actseq));
 	int startdisp = actseq->startdisp;
 	int enddisp   = actseq->enddisp;
 	int machine   = actseq->machine;
 	SeqIterator iter;
 
-	SEQP_BEGIN(ed, seq) {
-		seq->tmp= NULL;
+	SEQP_BEGIN (ed, seq)
+	{
+		seq->tmp = NULL;
 	}
 	SEQ_END;
 
 	actseq->tmp = SET_INT_IN_POINTER(TRUE);
 
-	for (seq_begin(ed, &iter, 1); iter.valid; seq_next(&iter)) {
+	for (BKE_sequence_iterator_begin(ed, &iter, TRUE); iter.valid; BKE_sequence_iterator_next(&iter)) {
 		seq = iter.seq;
 
 		/* Ignore all seqs already selected! */
@@ -1098,7 +1116,7 @@ static short select_grouped_effect_link(Editing *ed, Sequence *actseq)
 		/* Ignore all seqs of incompatible types (audio vs video). */
 		if ((seq->flag & SELECT) || (seq->startdisp >= enddisp) || (seq->enddisp < startdisp) ||
 		    (!is_audio && SEQ_IS_SOUND(seq)) ||
-		    (is_audio && !((seq->type == SEQ_META) || SEQ_IS_SOUND(seq))))
+		    (is_audio && !((seq->type == SEQ_TYPE_META) || SEQ_IS_SOUND(seq))))
 		{
 			continue;
 		}
@@ -1118,8 +1136,8 @@ static short select_grouped_effect_link(Editing *ed, Sequence *actseq)
 			changed = TRUE;
 
 			/* Unfortunately, we must restart checks from the beginning. */
-			seq_end(&iter);
-			seq_begin(ed, &iter, 1);
+			BKE_sequence_iterator_end(&iter);
+			BKE_sequence_iterator_begin(ed, &iter, TRUE);
 		}
 
 		/* Video strips bellow active one, or any strip for audio (order do no matters here!). */
@@ -1128,7 +1146,7 @@ static short select_grouped_effect_link(Editing *ed, Sequence *actseq)
 			changed = TRUE;
 		}
 	}
-	seq_end(&iter);
+	BKE_sequence_iterator_end(&iter);
 
 	return changed;
 }
@@ -1140,20 +1158,21 @@ static short select_grouped_effect_link(Editing *ed, Sequence *actseq)
 static int sequencer_select_grouped_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene  = CTX_data_scene(C);
-	Editing *ed   = seq_give_editing(scene, 0);
-	Sequence *seq, *actseq = seq_active_get(scene);
+	Editing *ed   = BKE_sequencer_editing_get(scene, 0);
+	Sequence *seq, *actseq = BKE_sequencer_active_get(scene);
 	int type = RNA_enum_get(op->ptr, "type");
 	short changed = 0, extend;
 
 	extend = RNA_boolean_get(op->ptr, "extend");
 
 	if (actseq == NULL) {
-		BKE_report(op->reports, RPT_ERROR, "No Active Sequence!");
+		BKE_report(op->reports, RPT_ERROR, "No active sequence!");
 		return OPERATOR_CANCELLED;
 	}
 
 	if (extend == 0) {
-		SEQP_BEGIN(ed, seq) {
+		SEQP_BEGIN (ed, seq)
+		{
 			seq->flag &= ~SELECT;
 			changed = TRUE;
 		}
