@@ -33,6 +33,7 @@
 #include <stdio.h>
 
 #include "DNA_scene_types.h"
+#include "DNA_mask_types.h"
 
 #include "MEM_guardedalloc.h"
 
@@ -56,8 +57,19 @@
 #include "UI_resources.h"
 #include "UI_view2d.h"
 
+#include "IMB_imbuf.h"
+
 #include "sequencer_intern.h"   // own include
 
+/**************************** common state *****************************/
+
+static void sequencer_scopes_tag_refresh(ScrArea *sa)
+{
+	SpaceSeq *sseq = (SpaceSeq *)sa->spacedata.first;
+
+	sseq->scopes.reference_ibuf = NULL;
+}
+
 /* ******************** manage regions ********************* */
 
 ARegion *sequencer_has_buttons_region(ScrArea *sa)
@@ -108,6 +120,8 @@ static SpaceLink *sequencer_new(const bContext *C)
 	sseq->chanshown = 0;
 	sseq->view = SEQ_VIEW_SEQUENCE;
 	sseq->mainb = SEQ_DRAW_IMG_IMBUF;
+	sseq->flag = SEQ_SHOW_GPENCIL | SEQ_USE_ALPHA;
+
 	/* header */
 	ar = MEM_callocN(sizeof(ARegion), "header for sequencer");
 	
@@ -182,12 +196,27 @@ static SpaceLink *sequencer_new(const bContext *C)
 }
 
 /* not spacelink itself */
-static void sequencer_free(SpaceLink *UNUSED(sl))
+static void sequencer_free(SpaceLink *sl)
 {	
-//	SpaceSeq *sseq= (SpaceSequencer*) sl;
-	
-// XXX	if (sseq->gpd) free_gpencil_data(sseq->gpd);
+	SpaceSeq *sseq = (SpaceSeq *) sl;
+	SequencerScopes *scopes = &sseq->scopes;
 
+// XXX	if (sseq->gpd) BKE_gpencil_free(sseq->gpd);
+
+	if (scopes->zebra_ibuf)
+		IMB_freeImBuf(scopes->zebra_ibuf);
+
+	if (scopes->waveform_ibuf)
+		IMB_freeImBuf(scopes->waveform_ibuf);
+
+	if (scopes->sep_waveform_ibuf)
+		IMB_freeImBuf(scopes->sep_waveform_ibuf);
+
+	if (scopes->vector_ibuf)
+		IMB_freeImBuf(scopes->vector_ibuf);
+
+	if (scopes->histogram_ibuf)
+		IMB_freeImBuf(scopes->histogram_ibuf);
 }
 
 
@@ -284,12 +313,29 @@ static SpaceLink *sequencer_duplicate(SpaceLink *sl)
 	SpaceSeq *sseqn = MEM_dupallocN(sl);
 	
 	/* clear or remove stuff from old */
-// XXX	sseq->gpd= gpencil_data_duplicate(sseq->gpd);
+// XXX	sseq->gpd = gpencil_data_duplicate(sseq->gpd);
 
 	return (SpaceLink *)sseqn;
 }
 
-
+static void sequencer_listener(ScrArea *sa, wmNotifier *wmn)
+{
+	/* context changes */
+	switch (wmn->category) {
+		case NC_SCENE:
+			switch (wmn->data) {
+				case ND_FRAME:
+				case ND_SEQUENCER:
+					sequencer_scopes_tag_refresh(sa);
+					break;
+			}
+			break;
+		case NC_SPACE:
+			if (wmn->data == ND_SPACE_SEQUENCER)
+				sequencer_scopes_tag_refresh(sa);
+			break;
+	}
+}
 
 /* *********************** sequencer (main) region ************************ */
 /* add handlers, stuff you only do once or on area/region changes */
@@ -299,7 +345,10 @@ static void sequencer_main_area_init(wmWindowManager *wm, ARegion *ar)
 	ListBase *lb;
 	
 	UI_view2d_region_reinit(&ar->v2d, V2D_COMMONVIEW_CUSTOM, ar->winx, ar->winy);
-	
+
+//	keymap = WM_keymap_find(wm->defaultconf, "Mask Editing", 0, 0);
+//	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
+
 	keymap = WM_keymap_find(wm->defaultconf, "SequencerCommon", SPACE_SEQ, 0);
 	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
 	
@@ -316,7 +365,7 @@ static void sequencer_main_area_init(wmWindowManager *wm, ARegion *ar)
 
 static void sequencer_main_area_draw(const bContext *C, ARegion *ar)
 {
-//	ScrArea *sa= CTX_wm_area(C);
+//	ScrArea *sa = CTX_wm_area(C);
 	
 	/* NLE - strip editing timeline interface */
 	draw_timeline_seq(C, ar);
@@ -324,27 +373,43 @@ static void sequencer_main_area_draw(const bContext *C, ARegion *ar)
 
 /* ************* dropboxes ************* */
 
-static int image_drop_poll(bContext *UNUSED(C), wmDrag *drag, wmEvent *UNUSED(event))
+static int image_drop_poll(bContext *C, wmDrag *drag, wmEvent *event)
 {
+	ARegion *ar = CTX_wm_region(C);
+	Scene *scene = CTX_data_scene(C);
+	int hand;
+
 	if (drag->type == WM_DRAG_PATH)
 		if (ELEM(drag->icon, ICON_FILE_IMAGE, ICON_FILE_BLANK)) /* rule might not work? */
-			return 1;
+			if (find_nearest_seq(scene, &ar->v2d, &hand, event->mval) == NULL)
+				return 1;
+
 	return 0;
 }
 
-static int movie_drop_poll(bContext *UNUSED(C), wmDrag *drag, wmEvent *UNUSED(event))
+static int movie_drop_poll(bContext *C, wmDrag *drag, wmEvent *event)
 {
+	ARegion *ar = CTX_wm_region(C);
+	Scene *scene = CTX_data_scene(C);
+	int hand;
+
 	if (drag->type == WM_DRAG_PATH)
 		if (ELEM3(drag->icon, 0, ICON_FILE_MOVIE, ICON_FILE_BLANK)) /* rule might not work? */
-			return 1;
+			if (find_nearest_seq(scene, &ar->v2d, &hand, event->mval) == NULL)
+				return 1;
 	return 0;
 }
 
-static int sound_drop_poll(bContext *UNUSED(C), wmDrag *drag, wmEvent *UNUSED(event))
+static int sound_drop_poll(bContext *C, wmDrag *drag, wmEvent *event)
 {
+	ARegion *ar = CTX_wm_region(C);
+	Scene *scene = CTX_data_scene(C);
+	int hand;
+
 	if (drag->type == WM_DRAG_PATH)
 		if (ELEM(drag->icon, ICON_FILE_SOUND, ICON_FILE_BLANK)) /* rule might not work? */
-			return 1;
+			if (find_nearest_seq(scene, &ar->v2d, &hand, event->mval) == NULL)
+				return 1;
 	return 0;
 }
 
@@ -380,6 +445,29 @@ static void sequencer_dropboxes(void)
 
 /* ************* end drop *********** */
 
+const char *sequencer_context_dir[] = {"edit_mask", NULL};
+
+static int sequencer_context(const bContext *C, const char *member, bContextDataResult *result)
+{
+	Scene *scene = CTX_data_scene(C);
+
+	if (CTX_data_dir(member)) {
+		CTX_data_dir_set(result, sequencer_context_dir);
+
+		return TRUE;
+	}
+	else if (CTX_data_equals(member, "edit_mask")) {
+		Mask *mask = BKE_sequencer_mask_get(scene);
+		if (mask) {
+			CTX_data_id_pointer_set(result, &mask->id);
+		}
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+
 /* add handlers, stuff you only do once or on area/region changes */
 static void sequencer_header_area_init(wmWindowManager *UNUSED(wm), ARegion *ar)
 {
@@ -414,6 +502,10 @@ static void sequencer_main_area_listener(ARegion *ar, wmNotifier *wmn)
 			if (wmn->action == NA_RENAME)
 				ED_region_tag_redraw(ar);
 			break;
+		case NC_SCREEN:
+			if (ELEM(wmn->data, ND_SCREENCAST, ND_ANIMPLAY))
+				ED_region_tag_redraw(ar);
+			break;
 	}
 }
 
@@ -424,6 +516,9 @@ static void sequencer_preview_area_init(wmWindowManager *wm, ARegion *ar)
 
 	UI_view2d_region_reinit(&ar->v2d, V2D_COMMONVIEW_CUSTOM, ar->winx, ar->winy);
 	
+//	keymap = WM_keymap_find(wm->defaultconf, "Mask Editing", 0, 0);
+//	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
+
 	keymap = WM_keymap_find(wm->defaultconf, "SequencerCommon", SPACE_SEQ, 0);
 	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
 
@@ -437,13 +532,15 @@ static void sequencer_preview_area_draw(const bContext *C, ARegion *ar)
 	ScrArea *sa = CTX_wm_area(C);
 	SpaceSeq *sseq = sa->spacedata.first;
 	Scene *scene = CTX_data_scene(C);
+	int show_split = scene->ed && scene->ed->over_flag & SEQ_EDIT_OVERLAY_SHOW && sseq->mainb == SEQ_DRAW_IMG_IMBUF;
 	
 	/* XXX temp fix for wrong setting in sseq->mainb */
 	if (sseq->mainb == SEQ_DRAW_SEQUENCE) sseq->mainb = SEQ_DRAW_IMG_IMBUF;
 
-	draw_image_seq(C, scene, ar, sseq, scene->r.cfra, 0);
+	if (!show_split || sseq->overlay_type != SEQ_DRAW_OVERLAY_REFERENCE)
+		draw_image_seq(C, scene, ar, sseq, scene->r.cfra, 0, FALSE);
 
-	if (scene->ed && scene->ed->over_flag & SEQ_EDIT_OVERLAY_SHOW && sseq->mainb == SEQ_DRAW_IMG_IMBUF) {
+	if (show_split && sseq->overlay_type != SEQ_DRAW_OVERLAY_CURRENT) {
 		int over_cfra;
 
 		if (scene->ed->over_flag & SEQ_EDIT_OVERLAY_ABS)
@@ -451,8 +548,8 @@ static void sequencer_preview_area_draw(const bContext *C, ARegion *ar)
 		else
 			over_cfra = scene->r.cfra + scene->ed->over_ofs;
 
-		if (over_cfra != scene->r.cfra)
-			draw_image_seq(C, scene, ar, sseq, scene->r.cfra, over_cfra - scene->r.cfra);
+		if (over_cfra != scene->r.cfra || sseq->overlay_type != SEQ_DRAW_OVERLAY_RECT)
+			draw_image_seq(C, scene, ar, sseq, scene->r.cfra, over_cfra - scene->r.cfra, TRUE);
 	}
 
 }
@@ -461,6 +558,11 @@ static void sequencer_preview_area_listener(ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
 	switch (wmn->category) {
+		case NC_GPENCIL:
+			if (wmn->action == NA_EDITED) {
+				ED_region_tag_redraw(ar);
+			}
+			break;
 		case NC_SCENE:
 			switch (wmn->data) {
 				case ND_FRAME:
@@ -482,6 +584,12 @@ static void sequencer_preview_area_listener(ARegion *ar, wmNotifier *wmn)
 					break;
 			}
 			break;
+
+		case NC_MASK:
+			if (wmn->action == NA_EDITED) {
+				ED_region_tag_redraw(ar);
+			}
+			break;
 	}
 }
 
@@ -504,6 +612,11 @@ static void sequencer_buttons_area_listener(ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
 	switch (wmn->category) {
+		case NC_GPENCIL:
+			if (wmn->data == ND_DATA) {
+				ED_region_tag_redraw(ar);
+			}
+			break;
 		case NC_SCENE:
 			switch (wmn->data) {
 				case ND_FRAME:
@@ -539,8 +652,10 @@ void ED_spacetype_sequencer(void)
 	st->duplicate = sequencer_duplicate;
 	st->operatortypes = sequencer_operatortypes;
 	st->keymap = sequencer_keymap;
+	st->context = sequencer_context;
 	st->dropboxes = sequencer_dropboxes;
 	st->refresh = sequencer_refresh;
+	st->listener = sequencer_listener;
 
 	/* regions: main window */
 	art = MEM_callocN(sizeof(ARegionType), "spacetype sequencer region");
@@ -559,7 +674,7 @@ void ED_spacetype_sequencer(void)
 	art->init = sequencer_preview_area_init;
 	art->draw = sequencer_preview_area_draw;
 	art->listener = sequencer_preview_area_listener;
-	art->keymapflag = ED_KEYMAP_VIEW2D | ED_KEYMAP_FRAMES | ED_KEYMAP_ANIMATION;
+	art->keymapflag = ED_KEYMAP_VIEW2D | ED_KEYMAP_FRAMES | ED_KEYMAP_GPENCIL;
 	BLI_addhead(&st->regiontypes, art);
 	
 	/* regions: listview/buttons */
@@ -571,6 +686,8 @@ void ED_spacetype_sequencer(void)
 	art->init = sequencer_buttons_area_init;
 	art->draw = sequencer_buttons_area_draw;
 	BLI_addhead(&st->regiontypes, art);
+	
+	sequencer_buttons_register(art);
 
 	/* regions: header */
 	art = MEM_callocN(sizeof(ARegionType), "spacetype sequencer region");
@@ -591,4 +708,3 @@ void ED_spacetype_sequencer(void)
 		sequencer_view3d_cb = ED_view3d_draw_offscreen_imbuf_simple;
 	}
 }
-
