@@ -29,35 +29,30 @@
  */
 
 
-#include <math.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/stat.h>
 
 #include "MEM_guardedalloc.h"
 
 #include "BLF_api.h"
 
 #include "BLI_blenlib.h"
-#include "BLI_utildefines.h"
+#include "BLI_math.h"
 
 #include "DNA_text_types.h"
 #include "DNA_space_types.h"
 #include "DNA_screen_types.h"
-#include "DNA_userdef_types.h"
 
 #include "BKE_context.h"
 #include "BKE_suggestions.h"
 #include "BKE_text.h"
 
-
 #include "BIF_gl.h"
 
-#include "ED_datafiles.h"
 #include "UI_interface.h"
 #include "UI_resources.h"
+#include "UI_view2d.h"
 
 #include "text_intern.h"
+#include "text_format.h"
 
 /******************** text font drawing ******************/
 // XXX, fixme
@@ -65,7 +60,7 @@
 
 static void text_font_begin(SpaceText *st)
 {
-	BLF_size(mono, st->lheight, 72);
+	BLF_size(mono, st->lheight_dpi, 72);
 }
 
 static void text_font_end(SpaceText *UNUSED(st))
@@ -82,374 +77,20 @@ static int text_font_draw(SpaceText *UNUSED(st), int x, int y, const char *str)
 
 static int text_font_draw_character(SpaceText *st, int x, int y, char c)
 {
-	char str[2];
-	str[0] = c;
-	str[1] = '\0';
-
 	BLF_position(mono, x, y, 0);
-	BLF_draw(mono, str, 1);
+	BLF_draw(mono, &c, 1);
 
 	return st->cwidth;
 }
 
 static int text_font_draw_character_utf8(SpaceText *st, int x, int y, const char *c)
 {
-	char str[BLI_UTF8_MAX + 1];
-	size_t len = BLI_str_utf8_size(c);
-	memcpy(str, c, len);
-	str[len] = '\0';
-
+	const size_t len = BLI_str_utf8_size_safe(c);
 	BLF_position(mono, x, y, 0);
-	BLF_draw(mono, str, len);
-
+	BLF_draw(mono, c, len);
 	return st->cwidth;
 }
 
-/****************** flatten string **********************/
-
-static void flatten_string_append(FlattenString *fs, const char *c, int accum, int len) 
-{
-	int i;
-	
-	if (fs->pos + len > fs->len) {
-		char *nbuf; int *naccum;
-		fs->len *= 2;
-
-		nbuf = MEM_callocN(sizeof(*fs->buf) * fs->len, "fs->buf");
-		naccum = MEM_callocN(sizeof(*fs->accum) * fs->len, "fs->accum");
-
-		memcpy(nbuf, fs->buf, fs->pos * sizeof(*fs->buf));
-		memcpy(naccum, fs->accum, fs->pos * sizeof(*fs->accum));
-		
-		if (fs->buf != fs->fixedbuf) {
-			MEM_freeN(fs->buf);
-			MEM_freeN(fs->accum);
-		}
-		
-		fs->buf = nbuf;
-		fs->accum = naccum;
-	}
-	
-	for (i = 0; i < len; i++)
-	{
-		fs->buf[fs->pos + i] = c[i];
-		fs->accum[fs->pos + i] = accum;
-	}
-
-	fs->pos += len;
-}
-
-int flatten_string(SpaceText *st, FlattenString *fs, const char *in)
-{
-	int r, i, total = 0;
-
-	memset(fs, 0, sizeof(FlattenString));
-	fs->buf = fs->fixedbuf;
-	fs->accum = fs->fixedaccum;
-	fs->len = sizeof(fs->fixedbuf);
-
-	for (r = 0, i = 0; *in; r++) {
-		if (*in == '\t') {
-			i = st->tabnumber - (total % st->tabnumber);
-			total += i;
-			
-			while (i--)
-				flatten_string_append(fs, " ", r, 1);
-			
-			in++;
-		}
-		else {
-			size_t len = BLI_str_utf8_size(in);
-			flatten_string_append(fs, in, r, len);
-			in += len;
-			total++;
-		}
-	}
-	
-	flatten_string_append(fs, "\0", r, 1);
-
-	return total;
-}
-
-void flatten_string_free(FlattenString *fs)
-{
-	if (fs->buf != fs->fixedbuf)
-		MEM_freeN(fs->buf);
-	if (fs->accum != fs->fixedaccum)
-		MEM_freeN(fs->accum);
-}
-
-/* Checks the specified source string for a Python built-in function name. This
- * name must start at the beginning of the source string and must be followed by
- * a non-identifier (see text_check_identifier(char)) or null character.
- * 
- * If a built-in function is found, the length of the matching name is returned.
- * Otherwise, -1 is returned.
- *
- * See:
- * http://docs.python.org/py3k/reference/lexical_analysis.html#keywords
- */
-
-static int find_builtinfunc(char *string)
-{
-	int a, i;
-	const char *builtinfuncs[] = {
-		/* "False", "None", "True", */ /* see find_bool() */
-		"and", "as", "assert", "break",
-		"class", "continue", "def", "del", "elif", "else", "except",
-		"finally", "for", "from", "global", "if", "import", "in",
-		"is", "lambda", "nonlocal", "not", "or", "pass", "raise",
-		"return", "try", "while", "with", "yield",
-	};
-
-	for (a = 0; a < sizeof(builtinfuncs) / sizeof(builtinfuncs[0]); a++) {
-		i = 0;
-		while (1) {
-			/* If we hit the end of a keyword... (eg. "def") */
-			if (builtinfuncs[a][i] == '\0') {
-				/* If we still have identifier chars in the source (eg. "definate") */
-				if (text_check_identifier(string[i]))
-					i = -1;  /* No match */
-				break; /* Next keyword if no match, otherwise we're done */
-				
-				/* If chars mismatch, move on to next keyword */
-			}
-			else if (string[i] != builtinfuncs[a][i]) {
-				i = -1;
-				break; /* Break inner loop, start next keyword */
-			}
-			i++;
-		}
-		if (i > 0) break;  /* If we have a match, we're done */
-	}
-	return i;
-}
-
-/* Checks the specified source string for a Python special name. This name must
- * start at the beginning of the source string and must be followed by a non-
- * identifier (see text_check_identifier(char)) or null character.
- * 
- * If a special name is found, the length of the matching name is returned.
- * Otherwise, -1 is returned. */
-
-static int find_specialvar(char *string) 
-{
-	int i = 0;
-	/* Check for "def" */
-	if (string[0] == 'd' && string[1] == 'e' && string[2] == 'f')
-		i = 3;
-	/* Check for "class" */
-	else if (string[0] == 'c' && string[1] == 'l' && string[2] == 'a' && string[3] == 's' && string[4] == 's')
-		i = 5;
-	/* If next source char is an identifier (eg. 'i' in "definate") no match */
-	if (i == 0 || text_check_identifier(string[i]))
-		return -1;
-	return i;
-}
-
-static int find_decorator(char *string) 
-{
-	if (string[0] == '@') {
-		int i = 1;
-		while (text_check_identifier(string[i])) {
-			i++;
-		}
-		return i;
-	}
-	return -1;
-}
-
-static int find_bool(char *string) 
-{
-	int i = 0;
-	/* Check for "False" */
-	if (string[0] == 'F' && string[1] == 'a' && string[2] == 'l' && string[3] == 's' && string[4] == 'e')
-		i = 5;
-	/* Check for "True" */
-	else if (string[0] == 'T' && string[1] == 'r' && string[2] == 'u' && string[3] == 'e')
-		i = 4;
-	/* Check for "None" */
-	else if (string[0] == 'N' && string[1] == 'o' && string[2] == 'n' && string[3] == 'e')
-		i = 4;
-	/* If next source char is an identifier (eg. 'i' in "definate") no match */
-	if (i == 0 || text_check_identifier(string[i]))
-		return -1;
-	return i;
-}
-
-/* Ensures the format string for the given line is long enough, reallocating
- * as needed. Allocation is done here, alone, to ensure consistency. */
-static int text_check_format_len(TextLine *line, unsigned int len)
-{
-	if (line->format) {
-		if (strlen(line->format) < len) {
-			MEM_freeN(line->format);
-			line->format = MEM_mallocN(len + 2, "SyntaxFormat");
-			if (!line->format) return 0;
-		}
-	}
-	else {
-		line->format = MEM_mallocN(len + 2, "SyntaxFormat");
-		if (!line->format) return 0;
-	}
-
-	return 1;
-}
-
-/* Formats the specified line. If do_next is set, the process will move on to
- * the succeeding line if it is affected (eg. multiline strings). Format strings
- * may contain any of the following characters:
- *  '_'  Whitespace
- *  '#'  Comment text
- *  '!'  Punctuation and other symbols
- *  'n'  Numerals
- *  'l'  String letters
- *  'v'  Special variables (class, def)
- *  'b'  Built-in names (print, for, etc.)
- *  'q'  Other text (identifiers, etc.)
- * It is terminated with a null-terminator '\0' followed by a continuation
- * flag indicating whether the line is part of a multi-line string. */
-
-static void txt_format_line(SpaceText *st, TextLine *line, int do_next)
-{
-	FlattenString fs;
-	char *str, *fmt, orig, cont, find, prev = ' ';
-	int len, i;
-
-	/* Get continuation from previous line */
-	if (line->prev && line->prev->format != NULL) {
-		fmt = line->prev->format;
-		cont = fmt[strlen(fmt) + 1]; /* Just after the null-terminator */
-	}
-	else cont = 0;
-
-	/* Get original continuation from this line */
-	if (line->format != NULL) {
-		fmt = line->format;
-		orig = fmt[strlen(fmt) + 1]; /* Just after the null-terminator */
-	}
-	else orig = 0xFF;
-
-	len = flatten_string(st, &fs, line->line);
-	str = fs.buf;
-	if (!text_check_format_len(line, len)) {
-		flatten_string_free(&fs);
-		return;
-	}
-	fmt = line->format;
-
-	while (*str) {
-		/* Handle escape sequences by skipping both \ and next char */
-		if (*str == '\\') {
-			*fmt = prev; fmt++; str++;
-			if (*str == '\0') break;
-			*fmt = prev; fmt++; str += BLI_str_utf8_size(str);
-			continue;
-		}
-		/* Handle continuations */
-		else if (cont) {
-			/* Triple strings ("""...""" or '''...''') */
-			if (cont & TXT_TRISTR) {
-				find = (cont & TXT_DBLQUOTSTR) ? '"' : '\'';
-				if (*str == find && *(str + 1) == find && *(str + 2) == find) {
-					*fmt = 'l'; fmt++; str++;
-					*fmt = 'l'; fmt++; str++;
-					cont = 0;
-				}
-				/* Handle other strings */
-			}
-			else {
-				find = (cont & TXT_DBLQUOTSTR) ? '"' : '\'';
-				if (*str == find) cont = 0;
-			}
-
-			*fmt = 'l';
-			str += BLI_str_utf8_size(str) - 1;
-		}
-		/* Not in a string... */
-		else {
-			/* Deal with comments first */
-			if (prev == '#' || *str == '#') {
-				*fmt = '#';
-				str += BLI_str_utf8_size(str) - 1;
-			}
-			else if (*str == '"' || *str == '\'') {
-				/* Strings */
-				find = *str;
-				cont = (*str == '"') ? TXT_DBLQUOTSTR : TXT_SNGQUOTSTR;
-				if (*(str + 1) == find && *(str + 2) == find) {
-					*fmt = 'l'; fmt++; str++;
-					*fmt = 'l'; fmt++; str++;
-					cont |= TXT_TRISTR;
-				}
-				*fmt = 'l';
-			}
-			/* Whitespace (all ws. has been converted to spaces) */
-			else if (*str == ' ')
-				*fmt = '_';
-			/* Numbers (digits not part of an identifier and periods followed by digits) */
-			else if ((prev != 'q' && text_check_digit(*str)) || (*str == '.' && text_check_digit(*(str + 1))))
-				*fmt = 'n';
-			/* Booleans */
-			else if (prev != 'q' && (i = find_bool(str)) != -1)
-				if (i > 0) {
-					while (i > 1) {
-						*fmt = 'n'; fmt++; str++;
-						i--;
-					}
-					*fmt = 'n';
-				}
-				else {
-					str += BLI_str_utf8_size(str) - 1;
-					*fmt = 'q';
-				}
-			/* Punctuation */
-			else if (text_check_delim(*str))
-				*fmt = '!';
-			/* Identifiers and other text (no previous ws. or delims. so text continues) */
-			else if (prev == 'q') {
-				str += BLI_str_utf8_size(str) - 1;
-				*fmt = 'q';
-			}
-			/* Not ws, a digit, punct, or continuing text. Must be new, check for special words */
-			else {
-				/* Special vars(v) or built-in keywords(b) */
-				if ((i = find_specialvar(str)) != -1)
-					prev = 'v';
-				else if ((i = find_builtinfunc(str)) != -1)
-					prev = 'b';
-				else if ((i = find_decorator(str)) != -1)
-					prev = 'v';  /* could have a new color for this */
-				if (i > 0) {
-					while (i > 1) {
-						*fmt = prev; fmt++; str++;
-						i--;
-					}
-					*fmt = prev;
-				}
-				else {
-					str += BLI_str_utf8_size(str) - 1;
-					*fmt = 'q';
-				}
-			}
-		}
-		prev = *fmt;
-		fmt++;
-		str++;
-	}
-
-	/* Terminate and add continuation char */
-	*fmt = '\0'; fmt++;
-	*fmt = cont;
-
-	/* If continuation has changed and we're allowed, process the next line */
-	if (cont != orig && do_next && line->next) {
-		txt_format_line(st, line->next, do_next);
-	}
-	
-	flatten_string_free(&fs);
-}
-
 #if 0
 /* Formats every line of the current text */
 static void txt_format_text(SpaceText *st) 
@@ -467,27 +108,33 @@ static void txt_format_text(SpaceText *st)
 static void format_draw_color(char formatchar)
 {
 	switch (formatchar) {
-		case '_': /* Whitespace */
+		case FMT_TYPE_WHITESPACE:
 			break;
-		case '!': /* Symbols */
-			UI_ThemeColorBlend(TH_TEXT, TH_BACK, 0.5f);
+		case FMT_TYPE_SYMBOL:
+			UI_ThemeColor(TH_SYNTAX_S);
 			break;
-		case '#': /* Comments */
+		case FMT_TYPE_COMMENT:
 			UI_ThemeColor(TH_SYNTAX_C);
 			break;
-		case 'n': /* Numerals */
+		case FMT_TYPE_NUMERAL:
 			UI_ThemeColor(TH_SYNTAX_N);
 			break;
-		case 'l': /* Strings */
+		case FMT_TYPE_STRING:
 			UI_ThemeColor(TH_SYNTAX_L);
 			break;
-		case 'v': /* Specials: class, def */
+		case FMT_TYPE_DIRECTIVE:
+			UI_ThemeColor(TH_SYNTAX_D);
+			break;
+		case FMT_TYPE_SPECIAL:
 			UI_ThemeColor(TH_SYNTAX_V);
 			break;
-		case 'b': /* Keywords: for, print, etc. */
+		case FMT_TYPE_RESERVED:
+			UI_ThemeColor(TH_SYNTAX_R);
+			break;
+		case FMT_TYPE_KEYWORD:
 			UI_ThemeColor(TH_SYNTAX_B);
 			break;
-		case 'q': /* Other text (identifiers) */
+		case FMT_TYPE_DEFAULT:
 		default:
 			UI_ThemeColor(TH_TEXT);
 			break;
@@ -576,7 +223,7 @@ void wrap_offset(SpaceText *st, ARegion *ar, TextLine *linein, int cursin, int *
 		end = max;
 		chop = 1;
 		*offc = 0;
-		for (i = 0, j = 0; linep->line[j]; j += BLI_str_utf8_size(linep->line + j)) {
+		for (i = 0, j = 0; linep->line[j]; j += BLI_str_utf8_size_safe(linep->line + j)) {
 			int chars;
 
 			/* Mimic replacement of tabs */
@@ -641,7 +288,7 @@ void wrap_offset_in_line(SpaceText *st, ARegion *ar, TextLine *linein, int cursi
 	*offc = 0;
 	cursin = txt_utf8_offset_to_index(linein->line, cursin);
 
-	for (i = 0, j = 0; linein->line[j]; j += BLI_str_utf8_size(linein->line + j)) {
+	for (i = 0, j = 0; linein->line[j]; j += BLI_str_utf8_size_safe(linein->line + j)) {
 
 		/* Mimic replacement of tabs */
 		ch = linein->line[j];
@@ -686,7 +333,7 @@ int text_get_char_pos(SpaceText *st, const char *line, int cur)
 {
 	int a = 0, i;
 	
-	for (i = 0; i < cur && line[i]; i += BLI_str_utf8_size(line + i)) {
+	for (i = 0; i < cur && line[i]; i += BLI_str_utf8_size_safe(line + i)) {
 		if (line[i] == '\t')
 			a += st->tabnumber - a % st->tabnumber;
 		else
@@ -699,7 +346,7 @@ static const char *txt_utf8_get_nth(const char *str, int n)
 {
 	int pos = 0;
 	while (str[pos] && n--) {
-		pos += BLI_str_utf8_size(str + pos);
+		pos += BLI_str_utf8_size_safe(str + pos);
 	}
 	return str + pos;
 }
@@ -709,6 +356,7 @@ static int text_draw_wrapped(SpaceText *st, const char *str, int x, int y, int w
 	FlattenString fs;
 	int basex, i, a, start, end, max, lines; /* view */
 	int mi, ma, mstart, mend;                /* mem */
+	char fmt_prev = 0xff;
 	
 	flatten_string(st, &fs, str);
 	str = fs.buf;
@@ -720,7 +368,7 @@ static int text_draw_wrapped(SpaceText *st, const char *str, int x, int y, int w
 	start = 0; mstart = 0;
 	end = max; mend = txt_utf8_get_nth(str, max) - str;
 	
-	for (i = 0, mi = 0; str[mi]; i++, mi += BLI_str_utf8_size(str + mi)) {
+	for (i = 0, mi = 0; str[mi]; i++, mi += BLI_str_utf8_size_safe(str + mi)) {
 		if (i - start >= max) {
 			/* skip hidden part of line */
 			if (skip) {
@@ -731,11 +379,13 @@ static int text_draw_wrapped(SpaceText *st, const char *str, int x, int y, int w
 			}
 
 			/* Draw the visible portion of text on the overshot line */
-			for (a = start, ma = mstart; a < end; a++, ma += BLI_str_utf8_size(str + ma)) {
-				if (st->showsyntax && format) format_draw_color(format[a]);
+			for (a = start, ma = mstart; a < end; a++, ma += BLI_str_utf8_size_safe(str + ma)) {
+				if (st->showsyntax && format) {
+					if (fmt_prev != format[a]) format_draw_color(fmt_prev = format[a]);
+				}
 				x += text_font_draw_character_utf8(st, x, y, str + ma);
 			}
-			y -= st->lheight;
+			y -= st->lheight_dpi + TXT_LINE_SPACING;
 			x = basex;
 			lines++;
 			start = end; mstart = mend;
@@ -749,9 +399,10 @@ static int text_draw_wrapped(SpaceText *st, const char *str, int x, int y, int w
 	}
 
 	/* Draw the remaining text */
-	for (a = start, ma = mstart; str[ma] && y > 0; a++, ma += BLI_str_utf8_size(str + ma)) {
-		if (st->showsyntax && format)
-			format_draw_color(format[a]);
+	for (a = start, ma = mstart; str[ma] && y > 0; a++, ma += BLI_str_utf8_size_safe(str + ma)) {
+		if (st->showsyntax && format) {
+			if (fmt_prev != format[a]) format_draw_color(fmt_prev = format[a]);
+		}
 
 		x += text_font_draw_character_utf8(st, x, y, str + ma);
 	}
@@ -782,15 +433,18 @@ static int text_draw(SpaceText *st, char *str, int cshift, int maxwidth, int dra
 		
 		if (st->showsyntax && format) {
 			int a, str_shift = 0;
+			char fmt_prev = 0xff;
 			format = format + cshift;
 
 			for (a = 0; a < amount; a++) {
-				format_draw_color(format[a]);
+				if (format[a] != fmt_prev) format_draw_color(fmt_prev = format[a]);
 				x += text_font_draw_character_utf8(st, x, y, in + str_shift);
-				str_shift += BLI_str_utf8_size(in + str_shift);
+				str_shift += BLI_str_utf8_size_safe(in + str_shift);
 			}
 		}
-		else text_font_draw(st, x, y, in);
+		else {
+			text_font_draw(st, x, y, in);
+		}
 	}
 	else {
 		while (w-- && *acc++ < maxwidth)
@@ -826,7 +480,7 @@ typedef struct DrawCache {
 
 static void text_drawcache_init(SpaceText *st)
 {
-	DrawCache *drawcache = MEM_callocN(sizeof (DrawCache), "text draw cache");
+	DrawCache *drawcache = MEM_callocN(sizeof(DrawCache), "text draw cache");
 
 	drawcache->winx = -1;
 	drawcache->nlines = BLI_countlist(&st->text->lines);
@@ -853,7 +507,7 @@ static void text_update_drawcache(SpaceText *st, ARegion *ar)
 	full_update |= drawcache->wordwrap != st->wordwrap;       /* word-wrapping option was toggled */
 	full_update |= drawcache->showlinenrs != st->showlinenrs; /* word-wrapping option was toggled */
 	full_update |= drawcache->tabnumber != st->tabnumber;     /* word-wrapping option was toggled */
-	full_update |= drawcache->lheight != st->lheight;         /* word-wrapping option was toggled */
+	full_update |= drawcache->lheight != st->lheight_dpi;         /* word-wrapping option was toggled */
 	full_update |= drawcache->cwidth != st->cwidth;           /* word-wrapping option was toggled */
 	full_update |= strncmp(drawcache->text_id, txt->id.name, MAX_ID_NAME); /* text datablock was changed */
 
@@ -929,7 +583,7 @@ static void text_update_drawcache(SpaceText *st, ARegion *ar)
 	/* store settings */
 	drawcache->winx        = ar->winx;
 	drawcache->wordwrap    = st->wordwrap;
-	drawcache->lheight     = st->lheight;
+	drawcache->lheight     = st->lheight_dpi;
 	drawcache->cwidth      = st->cwidth;
 	drawcache->showlinenrs = st->showlinenrs;
 	drawcache->tabnumber   = st->tabnumber;
@@ -1017,7 +671,7 @@ int text_get_visible_lines(SpaceText *st, ARegion *ar, const char *str)
 	lines = 1;
 	start = 0;
 	end = max;
-	for (i = 0, j = 0; str[j]; j += BLI_str_utf8_size(str + j)) {
+	for (i = 0, j = 0; str[j]; j += BLI_str_utf8_size_safe(str + j)) {
 		/* Mimic replacement of tabs */
 		ch = str[j];
 		if (ch == '\t') {
@@ -1071,40 +725,6 @@ int text_get_total_lines(SpaceText *st, ARegion *ar)
 	return drawcache->total_lines;
 }
 
-/* Move pointer to first visible line (top) */
-static TextLine *first_visible_line(SpaceText *st, ARegion *ar, int *wrap_top)
-{
-	Text *text = st->text;
-	TextLine *pline = text->lines.first;
-	int i = st->top, lineno = 0;
-
-	text_update_drawcache(st, ar);
-
-	if (wrap_top) *wrap_top = 0;
-
-	if (st->wordwrap) {
-		while (i > 0 && pline) {
-			int lines = text_get_visible_lines_no(st, lineno);
-
-			if (i - lines < 0) {
-				if (wrap_top) *wrap_top = i;
-				break;
-			}
-			else {
-				pline = pline->next;
-				i -= lines;
-				lineno++;
-			}
-		}
-	}
-	else {
-		for (i = st->top; pline->next && i > 0; i--)
-			pline = pline->next;
-	}
-
-	return pline;
-}
-
 /************************ draw scrollbar *****************************/
 
 static void calc_text_rcts(SpaceText *st, ARegion *ar, rcti *scroll, rcti *back)
@@ -1120,12 +740,12 @@ static void calc_text_rcts(SpaceText *st, ARegion *ar, rcti *scroll, rcti *back)
 	blank_lines = st->viewlines / 2;
 	
 	/* nicer code: use scroll rect for entire bar */
-	back->xmin = ar->winx - 18;
+	back->xmin = ar->winx - (V2D_SCROLL_WIDTH + 1);
 	back->xmax = ar->winx;
 	back->ymin = 0;
 	back->ymax = ar->winy;
 	
-	scroll->xmin = ar->winx - 17;
+	scroll->xmin = ar->winx - V2D_SCROLL_WIDTH;
 	scroll->xmax = ar->winx - 5;
 	scroll->ymin = 4;
 	scroll->ymax = 4 + pix_available;
@@ -1140,7 +760,7 @@ static void calc_text_rcts(SpaceText *st, ARegion *ar, rcti *scroll, rcti *back)
 	barheight = (ltexth > 0) ? (st->viewlines * pix_available) / ltexth : 0;
 	pix_bardiff = 0;
 	if (barheight < 20) {
-		pix_bardiff = 20 - barheight; /* take into account the now non-linear sizing of the bar */	
+		pix_bardiff = 20 - barheight; /* take into account the now non-linear sizing of the bar */
 		barheight = 20;
 	}
 	barstart = (ltexth > 0) ? ((pix_available - pix_bardiff) * st->top) / ltexth : 0;
@@ -1167,48 +787,48 @@ static void calc_text_rcts(SpaceText *st, ARegion *ar, rcti *scroll, rcti *back)
 		/* the scrollbar is non-linear sized */
 		if (pix_bardiff > 0) {
 			/* the start of the highlight is in the current viewport */
-			if (ltexth && st->viewlines && lhlstart >= st->top && lhlstart <= st->top + st->viewlines) { 
+			if (ltexth && st->viewlines && lhlstart >= st->top && lhlstart <= st->top + st->viewlines) {
 				/* speed the progresion of the start of the highlight through the scrollbar */
-				hlstart = ( ( (pix_available - pix_bardiff) * lhlstart) / ltexth) + (pix_bardiff * (lhlstart - st->top) / st->viewlines); 	
+				hlstart = ( ( (pix_available - pix_bardiff) * lhlstart) / ltexth) + (pix_bardiff * (lhlstart - st->top) / st->viewlines);
 			}
 			else if (lhlstart > st->top + st->viewlines && hlstart < barstart + barheight && hlstart > barstart) {
 				/* push hl start down */
 				hlstart = barstart + barheight;
 			}
-			else if (lhlend > st->top  && lhlstart < st->top && hlstart > barstart) {
+			else if (lhlend > st->top && lhlstart < st->top && hlstart > barstart) {
 				/*fill out start */
 				hlstart = barstart;
 			}
 
-			if (hlend <= hlstart) { 
+			if (hlend <= hlstart) {
 				hlend = hlstart + 2;
 			}
 
 			/* the end of the highlight is in the current viewport */
-			if (ltexth && st->viewlines && lhlend >= st->top && lhlend <= st->top + st->viewlines) { 
+			if (ltexth && st->viewlines && lhlend >= st->top && lhlend <= st->top + st->viewlines) {
 				/* speed the progresion of the end of the highlight through the scrollbar */
 				hlend = (((pix_available - pix_bardiff) * lhlend) / ltexth) + (pix_bardiff * (lhlend - st->top) / st->viewlines);
 			}
 			else if (lhlend < st->top && hlend >= barstart - 2 && hlend < barstart + barheight) {
 				/* push hl end up */
 				hlend = barstart;
-			}					
+			}
 			else if (lhlend > st->top + st->viewlines && lhlstart < st->top + st->viewlines && hlend < barstart + barheight) {
 				/* fill out end */
 				hlend = barstart + barheight;
 			}
 
-			if (hlend <= hlstart) { 
+			if (hlend <= hlstart) {
 				hlstart = hlend - 2;
-			}	
-		}	
+			}
+		}
 	}
 	else {
 		hlstart = 0;
 		hlend = 0;
 	}
 
-	if (hlend - hlstart < 2) { 
+	if (hlend - hlstart < 2) {
 		hlend = hlstart + 2;
 	}
 	
@@ -1233,7 +853,7 @@ static void draw_textscroll(SpaceText *st, rcti *scroll, rcti *back)
 	uiWidgetScrollDraw(&wcol, scroll, &st->txtbar, (st->flags & ST_SCROLL_SELECT) ? UI_SCROLL_PRESSED : 0);
 
 	uiSetRoundBox(UI_CNR_ALL);
-	rad = 0.4f * MIN2(st->txtscroll.xmax - st->txtscroll.xmin, st->txtscroll.ymax - st->txtscroll.ymin);
+	rad = 0.4f * min_ii(BLI_rcti_size_x(&st->txtscroll), BLI_rcti_size_y(&st->txtscroll));
 	UI_GetThemeColor3ubv(TH_HILITE, col);
 	col[3] = 48;
 	glColor4ubv(col);
@@ -1242,90 +862,6 @@ static void draw_textscroll(SpaceText *st, rcti *scroll, rcti *back)
 	glDisable(GL_BLEND);
 }
 
-/************************** draw markers **************************/
-
-static void draw_markers(SpaceText *st, ARegion *ar)
-{
-	Text *text = st->text;
-	TextMarker *marker, *next;
-	TextLine *top, *line;
-	int offl, offc, i, x1, x2, y1, y2, x, y;
-	int topi, topy;
-
-	/* Move pointer to first visible line (top) */
-	top = first_visible_line(st, ar, NULL);
-	topi = BLI_findindex(&text->lines, top);
-
-	topy = txt_get_span(text->lines.first, top);
-
-	for (marker = text->markers.first; marker; marker = next) {
-		next = marker->next;
-
-		/* invisible line (before top) */
-		if (marker->lineno < topi) continue;
-
-		line = BLI_findlink(&text->lines, marker->lineno);
-
-		/* Remove broken markers */
-		if (marker->end > line->len || marker->start > marker->end) {
-			BLI_freelinkN(&text->markers, marker);
-			continue;
-		}
-
-		wrap_offset(st, ar, line, marker->start, &offl, &offc);
-		y1 = txt_get_span(top, line) - st->top + offl + topy;
-		x1 = text_get_char_pos(st, line->line, marker->start) - st->left + offc;
-
-		wrap_offset(st, ar, line, marker->end, &offl, &offc);
-		y2 = txt_get_span(top, line) - st->top + offl + topy;
-		x2 = text_get_char_pos(st, line->line, marker->end) - st->left + offc;
-
-		/* invisible part of line (before top, after last visible line) */
-		if (y2 < 0 || y1 > st->top + st->viewlines) continue;
-
-		glColor3ubv(marker->color);
-		x = st->showlinenrs ? TXT_OFFSET + TEXTXLOC : TXT_OFFSET;
-		y = ar->winy - 3;
-
-		if (y1 == y2) {
-			y -= y1 * st->lheight;
-			glBegin(GL_LINE_LOOP);
-			glVertex2i(x + x2 * st->cwidth + 1, y);
-			glVertex2i(x + x1 * st->cwidth - 2, y);
-			glVertex2i(x + x1 * st->cwidth - 2, y - st->lheight);
-			glVertex2i(x + x2 * st->cwidth + 1, y - st->lheight);
-			glEnd();
-		}
-		else {
-			y -= y1 * st->lheight;
-			glBegin(GL_LINE_STRIP);
-			glVertex2i(ar->winx, y);
-			glVertex2i(x + x1 * st->cwidth - 2, y);
-			glVertex2i(x + x1 * st->cwidth - 2, y - st->lheight);
-			glVertex2i(ar->winx, y - st->lheight);
-			glEnd();
-			y -= st->lheight;
-
-			for (i = y1 + 1; i < y2; i++) {
-				glBegin(GL_LINES);
-				glVertex2i(x, y);
-				glVertex2i(ar->winx, y);
-				glVertex2i(x, y - st->lheight);
-				glVertex2i(ar->winx, y - st->lheight);
-				glEnd();
-				y -= st->lheight;
-			}
-
-			glBegin(GL_LINE_STRIP);
-			glVertex2i(x, y);
-			glVertex2i(x + x2 * st->cwidth + 1, y);
-			glVertex2i(x + x2 * st->cwidth + 1, y - st->lheight);
-			glVertex2i(x, y - st->lheight);
-			glEnd();
-		}
-	}
-}
-
 /*********************** draw documentation *******************************/
 
 static void draw_documentation(SpaceText *st, ARegion *ar)
@@ -1356,9 +892,9 @@ static void draw_documentation(SpaceText *st, ARegion *ar)
 		x += SUGG_LIST_WIDTH * st->cwidth + 50;
 	}
 
-	/* top= */ /* UNUSED */ y = ar->winy - st->lheight * l - 2;
+	/* top = */ /* UNUSED */ y = ar->winy - st->lheight_dpi * l - 2;
 	boxw = DOC_WIDTH * st->cwidth + 20;
-	boxh = (DOC_HEIGHT + 1) * st->lheight;
+	boxh = (DOC_HEIGHT + 1) * (st->lheight_dpi + TXT_LINE_SPACING);
 
 	/* Draw panel */
 	UI_ThemeColor(TH_BACK);
@@ -1390,7 +926,7 @@ static void draw_documentation(SpaceText *st, ARegion *ar)
 		else if (*p == '\n') {
 			buf[i] = '\0';
 			if (lines >= 0) {
-				y -= st->lheight;
+				y -= st->lheight_dpi;
 				text_draw(st, buf, 0, 0, 1, x + 4, y - 3, NULL);
 			}
 			i = 0; br = DOC_WIDTH; lines++;
@@ -1399,7 +935,7 @@ static void draw_documentation(SpaceText *st, ARegion *ar)
 		if (i == DOC_WIDTH) { /* Reached the width, go to last break and wrap there */
 			buf[br] = '\0';
 			if (lines >= 0) {
-				y -= st->lheight;
+				y -= st->lheight_dpi;
 				text_draw(st, buf, 0, 0, 1, x + 4, y - 3, NULL);
 			}
 			p -= i - br - 1; /* Rewind pointer to last break */
@@ -1421,9 +957,11 @@ static void draw_suggestion_list(SpaceText *st, ARegion *ar)
 	SuggItem *item, *first, *last, *sel;
 	TextLine *tmp;
 	char str[SUGG_LIST_WIDTH + 1];
-	int w, boxw = 0, boxh, i, l, x, y, b, *top;
+	int w, boxw = 0, boxh, i, l, x, y, *top;
+	const int lheight = st->lheight_dpi + TXT_LINE_SPACING;
+	const int margin_x = 2;
 	
-	if (!st || !st->text) return;
+	if (!st->text) return;
 	if (!texttool_text_is_active(st->text)) return;
 
 	first = texttool_suggest_first();
@@ -1445,14 +983,20 @@ static void draw_suggestion_list(SpaceText *st, ARegion *ar)
 	else {
 		x = st->cwidth * (st->text->curc - st->left) + TXT_OFFSET - 4;
 	}
-	y = ar->winy - st->lheight * l - 2;
+	/* offset back so the start of the text lines up with the suggestions,
+	 * not essential but makes suggestions easier to follow */
+	x -= st->cwidth * (st->text->curc - text_find_identifier_start(st->text->curl->line, st->text->curc));
+	y = ar->winy - lheight * l - 2;
 
 	boxw = SUGG_LIST_WIDTH * st->cwidth + 20;
-	boxh = SUGG_LIST_SIZE * st->lheight + 8;
+	boxh = SUGG_LIST_SIZE * lheight + 8;
 	
+	/* not needed but stands out nicer */
+	uiDrawBoxShadow(220, x, y - boxh, x + boxw, y);
+
 	UI_ThemeColor(TH_SHADE1);
 	glRecti(x - 1, y + 1, x + boxw + 1, y - boxh - 1);
-	UI_ThemeColor(TH_BACK);
+	UI_ThemeColorShade(TH_BACK, 16);
 	glRecti(x, y, x + boxw, y - boxh);
 
 	/* Set the top 'item' of the visible list */
@@ -1460,7 +1004,7 @@ static void draw_suggestion_list(SpaceText *st, ARegion *ar)
 
 	for (i = 0; i < SUGG_LIST_SIZE && item; i++, item = item->next) {
 
-		y -= st->lheight;
+		y -= lheight;
 
 		BLI_strncpy(str, item->name, SUGG_LIST_WIDTH);
 
@@ -1468,21 +1012,11 @@ static void draw_suggestion_list(SpaceText *st, ARegion *ar)
 		
 		if (item == sel) {
 			UI_ThemeColor(TH_SHADE2);
-			glRecti(x + 16, y - 3, x + 16 + w, y + st->lheight - 3);
+			glRecti(x + margin_x, y - 3, x + margin_x + w, y + lheight - 3);
 		}
-		b = 1; /* b=1 color block, text is default. b=0 no block, color text */
-		switch (item->type) {
-			case 'k': UI_ThemeColor(TH_SYNTAX_B); b = 0; break;
-			case 'm': UI_ThemeColor(TH_TEXT); break;
-			case 'f': UI_ThemeColor(TH_SYNTAX_L); break;
-			case 'v': UI_ThemeColor(TH_SYNTAX_N); break;
-			case '?': UI_ThemeColor(TH_TEXT); b = 0; break;
-		}
-		if (b) {
-			glRecti(x + 8, y + 2, x + 11, y + 5);
-			UI_ThemeColor(TH_TEXT);
-		}
-		text_draw(st, str, 0, 0, 1, x + 16, y - 1, NULL);
+
+		format_draw_color(item->type);
+		text_draw(st, str, 0, 0, 1, x + margin_x, y - 1, NULL);
 
 		if (item == last) break;
 	}
@@ -1495,6 +1029,7 @@ static void draw_cursor(SpaceText *st, ARegion *ar)
 	Text *text = st->text;
 	int vcurl, vcurc, vsell, vselc, hidden = 0;
 	int x, y, w, i;
+	const int lheight = st->lheight_dpi + TXT_LINE_SPACING;
 
 	/* Draw the selection */
 	if (text->curl != text->sell || text->curc != text->selc) {
@@ -1512,14 +1047,14 @@ static void draw_cursor(SpaceText *st, ARegion *ar)
 		
 		UI_ThemeColor(TH_SHADE2);
 		x = st->showlinenrs ? TXT_OFFSET + TEXTXLOC : TXT_OFFSET;
-		y = ar->winy - 2;
+		y = ar->winy;
 
 		if (vcurl == vsell) {
-			y -= vcurl * st->lheight;
+			y -= vcurl * lheight;
 			if (vcurc < vselc)
-				glRecti(x + vcurc * st->cwidth - 1, y, x + vselc * st->cwidth, y - st->lheight);
+				glRecti(x + vcurc * st->cwidth - 1, y, x + vselc * st->cwidth, y - lheight);
 			else
-				glRecti(x + vselc * st->cwidth - 1, y, x + vcurc * st->cwidth, y - st->lheight);
+				glRecti(x + vselc * st->cwidth - 1, y, x + vcurc * st->cwidth, y - lheight);
 		}
 		else {
 			int froml, fromc, tol, toc;
@@ -1533,12 +1068,12 @@ static void draw_cursor(SpaceText *st, ARegion *ar)
 				fromc = vselc; toc = vcurc;
 			}
 
-			y -= froml * st->lheight;
-			glRecti(x + fromc * st->cwidth - 1, y, ar->winx, y - st->lheight); y -= st->lheight;
+			y -= froml * lheight;
+			glRecti(x + fromc * st->cwidth - 1, y, ar->winx, y - lheight); y -= lheight;
 			for (i = froml + 1; i < tol; i++)
-				glRecti(x - 4, y, ar->winx, y - st->lheight),  y -= st->lheight;
+				glRecti(x - 4, y, ar->winx, y - lheight),  y -= lheight;
 
-			glRecti(x - 4, y, x + toc * st->cwidth, y - st->lheight);  y -= st->lheight;
+			glRecti(x - 4, y, x + toc * st->cwidth, y - lheight);  y -= lheight;
 		}
 	}
 	else {
@@ -1562,12 +1097,12 @@ static void draw_cursor(SpaceText *st, ARegion *ar)
 
 			wrap_offset_in_line(st, ar, text->sell, text->selc, &offl, &offc);
 
-			y1 = ar->winy - 2 - (vsell - offl) * st->lheight;
-			y2 = y1 - st->lheight * visible_lines + 1;
+			y1 = ar->winy - 2 - (vsell - offl) * lheight;
+			y2 = y1 - (lheight * visible_lines + TXT_LINE_SPACING);
 		}
 		else {
-			y1 = ar->winy - 2 - vsell * st->lheight;
-			y2 = y1 - st->lheight + 1;
+			y1 = ar->winy - 2 - vsell * lheight;
+			y2 = y1 - (lheight + TXT_LINE_SPACING);
 		}
 
 		if (!(y1 < 0 || y2 > ar->winy)) { /* check we need to draw */
@@ -1578,7 +1113,7 @@ static void draw_cursor(SpaceText *st, ARegion *ar)
 			
 			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
-			glRecti(x1 - 4, y1, x2, y2);
+			glRecti(x1 - 4, y1, x2, y2 + TXT_LINE_SPACING);
 			glDisable(GL_BLEND);
 		}
 	}
@@ -1587,20 +1122,21 @@ static void draw_cursor(SpaceText *st, ARegion *ar)
 		/* Draw the cursor itself (we draw the sel. cursor as this is the leading edge) */
 		x = st->showlinenrs ? TXT_OFFSET + TEXTXLOC : TXT_OFFSET;
 		x += vselc * st->cwidth;
-		y = ar->winy - 2 - vsell * st->lheight;
+		y = ar->winy - vsell * lheight;
 		
 		if (st->overwrite) {
 			char ch = text->sell->line[text->selc];
 			
+			y += TXT_LINE_SPACING;
 			w = st->cwidth;
 			if (ch == '\t') w *= st->tabnumber - (vselc + st->left) % st->tabnumber;
 			
 			UI_ThemeColor(TH_HILITE);
-			glRecti(x, y - st->lheight - 1, x + w, y - st->lheight + 1);
+			glRecti(x, y - lheight - 1, x + w, y - lheight + 1);
 		}
 		else {
 			UI_ThemeColor(TH_HILITE);
-			glRecti(x - 1, y, x + 1, y - st->lheight);
+			glRecti(x - 1, y, x + 1, y - lheight);
 		}
 	}
 }
@@ -1634,16 +1170,16 @@ static void draw_brackets(SpaceText *st, ARegion *ar)
 	stack = 0;
 	
 	/* Don't highlight backets if syntax HL is off or bracket in string or comment. */
-	if (!linep->format || linep->format[fc] == 'l' || linep->format[fc] == '#')
+	if (!linep->format || linep->format[fc] == FMT_TYPE_STRING || linep->format[fc] == FMT_TYPE_COMMENT)
 		return;
 
 	if (b > 0) {
 		/* opening bracket, search forward for close */
 		fc++;
-		c += BLI_str_utf8_size(linep->line + c);
+		c += BLI_str_utf8_size_safe(linep->line + c);
 		while (linep) {
 			while (c < linep->len) {
-				if (linep->format && linep->format[fc] != 'l' && linep->format[fc] != '#') {
+				if (linep->format && linep->format[fc] != FMT_TYPE_STRING && linep->format[fc] != FMT_TYPE_COMMENT) {
 					b = text_check_bracket(linep->line[c]);
 					if (b == find) {
 						if (stack == 0) {
@@ -1658,7 +1194,7 @@ static void draw_brackets(SpaceText *st, ARegion *ar)
 					}
 				}
 				fc++;
-				c += BLI_str_utf8_size(linep->line + c);
+				c += BLI_str_utf8_size_safe(linep->line + c);
 			}
 			if (endl) break;
 			linep = linep->next;
@@ -1672,7 +1208,7 @@ static void draw_brackets(SpaceText *st, ARegion *ar)
 		if (c > 0) c -= linep->line + c - BLI_str_prev_char_utf8(linep->line + c);
 		while (linep) {
 			while (fc >= 0) {
-				if (linep->format && linep->format[fc] != 'l' && linep->format[fc] != '#') {
+				if (linep->format && linep->format[fc] != FMT_TYPE_STRING && linep->format[fc] != FMT_TYPE_COMMENT) {
 					b = text_check_bracket(linep->line[c]);
 					if (b == find) {
 						if (stack == 0) {
@@ -1703,9 +1239,9 @@ static void draw_brackets(SpaceText *st, ARegion *ar)
 	if (!endl || endc == -1)
 		return;
 
-	UI_ThemeColor(TH_HILITE);	
+	UI_ThemeColor(TH_HILITE);
 	x = st->showlinenrs ? TXT_OFFSET + TEXTXLOC : TXT_OFFSET;
-	y = ar->winy - st->lheight;
+	y = ar->winy - st->lheight_dpi;
 
 	/* draw opening bracket */
 	ch = startl->line[startc];
@@ -1715,8 +1251,8 @@ static void draw_brackets(SpaceText *st, ARegion *ar)
 	if (viewc >= 0) {
 		viewl = txt_get_span(text->lines.first, startl) - st->top + offl;
 
-		text_font_draw_character(st, x + viewc * st->cwidth, y - viewl * st->lheight, ch);
-		text_font_draw_character(st, x + viewc * st->cwidth + 1, y - viewl * st->lheight, ch);
+		text_font_draw_character(st, x + viewc * st->cwidth, y - viewl * (st->lheight_dpi + TXT_LINE_SPACING), ch);
+		text_font_draw_character(st, x + viewc * st->cwidth + 1, y - viewl * (st->lheight_dpi + TXT_LINE_SPACING), ch);
 	}
 
 	/* draw closing bracket */
@@ -1727,8 +1263,8 @@ static void draw_brackets(SpaceText *st, ARegion *ar)
 	if (viewc >= 0) {
 		viewl = txt_get_span(text->lines.first, endl) - st->top + offl;
 
-		text_font_draw_character(st, x + viewc * st->cwidth, y - viewl * st->lheight, ch);
-		text_font_draw_character(st, x + viewc * st->cwidth + 1, y - viewl * st->lheight, ch);
+		text_font_draw_character(st, x + viewc * st->cwidth, y - viewl * (st->lheight_dpi + TXT_LINE_SPACING), ch);
+		text_font_draw_character(st, x + viewc * st->cwidth + 1, y - viewl * (st->lheight_dpi + TXT_LINE_SPACING), ch);
 	}
 }
 
@@ -1737,13 +1273,18 @@ static void draw_brackets(SpaceText *st, ARegion *ar)
 void draw_text_main(SpaceText *st, ARegion *ar)
 {
 	Text *text = st->text;
+	TextFormatType *tft;
 	TextLine *tmp;
 	rcti scroll, back;
 	char linenr[12];
 	int i, x, y, winx, linecount = 0, lineno = 0;
 	int wraplinecount = 0, wrap_skip = 0;
+	int margin_column_x;
 
-	if (st->lheight) st->viewlines = (int)ar->winy / st->lheight;
+	/* dpi controlled line height and font size */
+	st->lheight_dpi = (U.widget_unit * st->lheight) / 20;
+	
+	if (st->lheight_dpi) st->viewlines = (int)ar->winy / (st->lheight_dpi + TXT_LINE_SPACING);
 	else st->viewlines = 0;
 
 	/* if no text, nothing to do */
@@ -1760,11 +1301,12 @@ void draw_text_main(SpaceText *st, ARegion *ar)
 	calc_text_rcts(st, ar, &scroll, &back); /* scroll will hold the entire bar size */
 
 	/* update syntax formatting if needed */
+	tft = ED_text_format_get(text);
 	tmp = text->lines.first;
 	lineno = 0;
 	for (i = 0; i < st->top && tmp; i++) {
 		if (st->showsyntax && !tmp->format)
-			txt_format_line(st, tmp, 0);
+			tft->format_line(st, tmp, 0);
 
 		if (st->wordwrap) {
 			int lines = text_get_visible_lines_no(st, lineno);
@@ -1789,7 +1331,7 @@ void draw_text_main(SpaceText *st, ARegion *ar)
 
 	text_font_begin(st);
 	st->cwidth = BLF_fixed_width(mono);
-	st->cwidth = MAX2(st->cwidth, 1);
+	st->cwidth = MAX2(st->cwidth, (char)1);
 
 	/* draw line numbers background */
 	if (st->showlinenrs) {
@@ -1802,7 +1344,7 @@ void draw_text_main(SpaceText *st, ARegion *ar)
 		st->linenrs_tot = 0; /* not used */
 		x = TXT_OFFSET;
 	}
-	y = ar->winy - st->lheight;
+	y = ar->winy - st->lheight_dpi;
 	winx = ar->winx - TXT_SCROLL_WIDTH;
 	
 	/* draw cursor */
@@ -1813,7 +1355,7 @@ void draw_text_main(SpaceText *st, ARegion *ar)
 
 	for (i = 0; y > 0 && i < st->viewlines && tmp; i++, tmp = tmp->next) {
 		if (st->showsyntax && !tmp->format)
-			txt_format_line(st, tmp, 0);
+			tft->format_line(st, tmp, 0);
 
 		if (st->showlinenrs && !wrap_skip) {
 			/* draw line number */
@@ -1832,30 +1374,33 @@ void draw_text_main(SpaceText *st, ARegion *ar)
 		if (st->wordwrap) {
 			/* draw word wrapped text */
 			int lines = text_draw_wrapped(st, tmp->line, x, y, winx - x, tmp->format, wrap_skip);
-			y -= lines * st->lheight;
+			y -= lines * (st->lheight_dpi + TXT_LINE_SPACING);
 		}
 		else {
 			/* draw unwrapped text */
 			text_draw(st, tmp->line, st->left, ar->winx / st->cwidth, 1, x, y, tmp->format);
-			y -= st->lheight;
+			y -= st->lheight_dpi + TXT_LINE_SPACING;
 		}
-
+		
 		wrap_skip = 0;
 	}
 	
 	if (st->flags & ST_SHOW_MARGIN) {
 		UI_ThemeColor(TH_HILITE);
 
-		glBegin(GL_LINES);
-		glVertex2i(x + st->cwidth * st->margin_column, 0);
-		glVertex2i(x + st->cwidth * st->margin_column, ar->winy - 2);
-		glEnd();
+		margin_column_x = x + st->cwidth * (st->margin_column - st->left);
+		
+		if (margin_column_x >= x) {
+			glBegin(GL_LINES);
+			glVertex2i(margin_column_x, 0);
+			glVertex2i(margin_column_x, ar->winy - 2);
+			glEnd();
+		}
 	}
 
 	/* draw other stuff */
 	draw_brackets(st, ar);
-	draw_markers(st, ar);
-	glTranslatef(0.375f, 0.375f, 0.0f); /* XXX scroll requires exact pixel space */
+	glTranslatef(GLA_PIXEL_OFS, GLA_PIXEL_OFS, 0.0f); /* XXX scroll requires exact pixel space */
 	draw_textscroll(st, &scroll, &back);
 	draw_documentation(st, ar);
 	draw_suggestion_list(st, ar);
@@ -1869,7 +1414,7 @@ void text_update_character_width(SpaceText *st)
 {
 	text_font_begin(st);
 	st->cwidth = BLF_fixed_width(mono);
-	st->cwidth = MAX2(st->cwidth, 1);
+	st->cwidth = MAX2(st->cwidth, (char)1);
 	text_font_end(st);
 }
 
