@@ -73,15 +73,15 @@ static void time_draw_sfra_efra(Scene *scene, View2D *v2d)
 	 */
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	glEnable(GL_BLEND);
-		glColor4f(0.0f, 0.0f, 0.0f, 0.4f);
+	glColor4f(0.0f, 0.0f, 0.0f, 0.4f);
 		
-		if (PSFRA < PEFRA) {
-			glRectf(v2d->cur.xmin, v2d->cur.ymin, (float)PSFRA, v2d->cur.ymax);
-			glRectf((float)PEFRA, v2d->cur.ymin, v2d->cur.xmax, v2d->cur.ymax);
-		}
-		else {
-			glRectf(v2d->cur.xmin, v2d->cur.ymin, v2d->cur.xmax, v2d->cur.ymax);
-		}
+	if (PSFRA < PEFRA) {
+		glRectf(v2d->cur.xmin, v2d->cur.ymin, (float)PSFRA, v2d->cur.ymax);
+		glRectf((float)PEFRA, v2d->cur.ymin, v2d->cur.xmax, v2d->cur.ymax);
+	}
+	else {
+		glRectf(v2d->cur.xmin, v2d->cur.ymin, v2d->cur.xmax, v2d->cur.ymax);
+	}
 	glDisable(GL_BLEND);
 
 	UI_ThemeColorShade(TH_BACK, -60);
@@ -90,43 +90,46 @@ static void time_draw_sfra_efra(Scene *scene, View2D *v2d)
 	fdrawline((float)PEFRA, v2d->cur.ymin, (float)PEFRA, v2d->cur.ymax);
 }
 
-#define CACHE_DRAW_HEIGHT	3.0f
+#define CACHE_DRAW_HEIGHT   3.0f
 
-static void time_draw_cache(SpaceTime *stime, Object *ob)
+static void time_draw_cache(SpaceTime *stime, Object *ob, Scene *scene)
 {
 	PTCacheID *pid;
 	ListBase pidlist;
 	SpaceTimeCache *stc = stime->caches.first;
-	float yoffs=0.f;
+	float yoffs = 0.f;
 	
 	if (!(stime->cache_display & TIME_CACHE_DISPLAY) || (!ob))
 		return;
 
-	BKE_ptcache_ids_from_object(&pidlist, ob, NULL, 0);
+	BKE_ptcache_ids_from_object(&pidlist, ob, scene, 0);
 
 	/* iterate over pointcaches on the active object, 
 	 * add spacetimecache and vertex array for each */
-	for (pid=pidlist.first; pid; pid=pid->next) {
+	for (pid = pidlist.first; pid; pid = pid->next) {
 		float col[4], *fp;
 		int i, sta = pid->cache->startframe, end = pid->cache->endframe;
-		int len = (end - sta + 1)*4;
+		int len = (end - sta + 1) * 4;
 
-		switch(pid->type) {
+		switch (pid->type) {
 			case PTCACHE_TYPE_SOFTBODY:
 				if (!(stime->cache_display & TIME_CACHE_SOFTBODY)) continue;
 				break;
 			case PTCACHE_TYPE_PARTICLES:
-				if (!(stime->cache_display & TIME_CACHE_PARTICLES))	continue;
+				if (!(stime->cache_display & TIME_CACHE_PARTICLES)) continue;
 				break;
 			case PTCACHE_TYPE_CLOTH:
-				if (!(stime->cache_display & TIME_CACHE_CLOTH))	continue;
+				if (!(stime->cache_display & TIME_CACHE_CLOTH)) continue;
 				break;
 			case PTCACHE_TYPE_SMOKE_DOMAIN:
 			case PTCACHE_TYPE_SMOKE_HIGHRES:
-				if (!(stime->cache_display & TIME_CACHE_SMOKE))	continue;
+				if (!(stime->cache_display & TIME_CACHE_SMOKE)) continue;
 				break;
 			case PTCACHE_TYPE_DYNAMICPAINT:
-				if (!(stime->cache_display & TIME_CACHE_DYNAMICPAINT))	continue;
+				if (!(stime->cache_display & TIME_CACHE_DYNAMICPAINT)) continue;
+				break;
+			case PTCACHE_TYPE_RIGIDBODY:
+				if (!(stime->cache_display & TIME_CACHE_RIGIDBODY)) continue;
 				break;
 		}
 
@@ -134,7 +137,7 @@ static void time_draw_cache(SpaceTime *stime, Object *ob)
 			continue;
 
 		/* make sure we have stc with correct array length */
-		if (stc == NULL || MEM_allocN_len(stc->array) != len*2*sizeof(float)) {
+		if (stc == NULL || MEM_allocN_len(stc->array) != len * 2 * sizeof(float)) {
 			if (stc) {
 				MEM_freeN(stc->array);
 			}
@@ -143,59 +146,63 @@ static void time_draw_cache(SpaceTime *stime, Object *ob)
 				BLI_addtail(&stime->caches, stc);
 			}
 
-			stc->array = MEM_callocN(len*2*sizeof(float), "SpaceTimeCache array");
+			stc->array = MEM_callocN(len * 2 * sizeof(float), "SpaceTimeCache array");
 		}
 
 		/* fill the vertex array with a quad for each cached frame */
-		for (i=sta, fp=stc->array; i<=end; i++) {
-			if (pid->cache->cached_frames[i-sta]) {
-				fp[0] = (float)i-0.5f;
+		for (i = sta, fp = stc->array; i <= end; i++) {
+			if (pid->cache->cached_frames[i - sta]) {
+				fp[0] = (float)i - 0.5f;
 				fp[1] = 0.0;
-				fp+=2;
+				fp += 2;
 				
-				fp[0] = (float)i-0.5f;
+				fp[0] = (float)i - 0.5f;
 				fp[1] = 1.0;
-				fp+=2;
+				fp += 2;
 				
-				fp[0] = (float)i+0.5f;
+				fp[0] = (float)i + 0.5f;
 				fp[1] = 1.0;
-				fp+=2;
+				fp += 2;
 				
-				fp[0] = (float)i+0.5f;
+				fp[0] = (float)i + 0.5f;
 				fp[1] = 0.0;
-				fp+=2;
+				fp += 2;
 			}
 		}
 		
 		glPushMatrix();
-		glTranslatef(0.0, (float)V2D_SCROLL_HEIGHT+yoffs, 0.0);
+		glTranslatef(0.0, (float)V2D_SCROLL_HEIGHT + yoffs, 0.0);
 		glScalef(1.0, CACHE_DRAW_HEIGHT, 0.0);
 		
-		switch(pid->type) {
+		switch (pid->type) {
 			case PTCACHE_TYPE_SOFTBODY:
-				col[0] = 1.0;	col[1] = 0.4;	col[2] = 0.02;
+				col[0] = 1.0;   col[1] = 0.4;   col[2] = 0.02;
 				col[3] = 0.1;
 				break;
 			case PTCACHE_TYPE_PARTICLES:
-				col[0] = 1.0;	col[1] = 0.1;	col[2] = 0.02;
+				col[0] = 1.0;   col[1] = 0.1;   col[2] = 0.02;
 				col[3] = 0.1;
 				break;
 			case PTCACHE_TYPE_CLOTH:
-				col[0] = 0.1;	col[1] = 0.1;	col[2] = 0.75;
+				col[0] = 0.1;   col[1] = 0.1;   col[2] = 0.75;
 				col[3] = 0.1;
 				break;
 			case PTCACHE_TYPE_SMOKE_DOMAIN:
 			case PTCACHE_TYPE_SMOKE_HIGHRES:
-				col[0] = 0.2;	col[1] = 0.2;	col[2] = 0.2;
+				col[0] = 0.2;   col[1] = 0.2;   col[2] = 0.2;
 				col[3] = 0.1;
 				break;
 			case PTCACHE_TYPE_DYNAMICPAINT:
-				col[0] = 1.0;	col[1] = 0.1;	col[2] = 0.75;
+				col[0] = 1.0;   col[1] = 0.1;   col[2] = 0.75;
+				col[3] = 0.1;
+				break;
+			case PTCACHE_TYPE_RIGIDBODY:
+				col[0] = 1.0;   col[1] = 0.6;   col[2] = 0.0;
 				col[3] = 0.1;
 				break;
 			default:
 				BLI_assert(0);
-				col[0] = 1.0;	col[1] = 0.0;	col[2] = 1.0;
+				col[0] = 1.0;   col[1] = 0.0;   col[2] = 1.0;
 				col[3] = 0.1;
 		}
 		glColor4fv(col);
@@ -206,13 +213,16 @@ static void time_draw_cache(SpaceTime *stime, Object *ob)
 		
 		col[3] = 0.4f;
 		if (pid->cache->flag & PTCACHE_BAKED) {
-			col[0] -= 0.4f;	col[1] -= 0.4f;	col[2] -= 0.4f;
+			col[0] -= 0.4f; col[1] -= 0.4f; col[2] -= 0.4f;
+		}
+		else if (pid->cache->flag & PTCACHE_OUTDATED) {
+			col[0] += 0.4f; col[1] += 0.4f; col[2] += 0.4f;
 		}
 		glColor4fv(col);
 		
 		glEnableClientState(GL_VERTEX_ARRAY);
 		glVertexPointer(2, GL_FLOAT, 0, stc->array);
-		glDrawArrays(GL_QUADS, 0, (fp-stc->array)/2);
+		glDrawArrays(GL_QUADS, 0, (fp - stc->array) / 2);
 		glDisableClientState(GL_VERTEX_ARRAY);
 		
 		glDisable(GL_BLEND);
@@ -240,7 +250,7 @@ static void time_cache_free(SpaceTime *stime)
 {
 	SpaceTimeCache *stc;
 	
-	for (stc= stime->caches.first; stc; stc=stc->next) {
+	for (stc = stime->caches.first; stc; stc = stc->next) {
 		if (stc->array) {
 			MEM_freeN(stc->array);
 			stc->array = NULL;
@@ -257,9 +267,9 @@ static void time_cache_refresh(SpaceTime *stime)
 }
 
 /* helper function - find actkeycolumn that occurs on cframe, or the nearest one if not found */
-static ActKeyColumn *time_cfra_find_ak (ActKeyColumn *ak, float cframe)
+static ActKeyColumn *time_cfra_find_ak(ActKeyColumn *ak, float cframe)
 {
-	ActKeyColumn *akn= NULL;
+	ActKeyColumn *akn = NULL;
 	
 	/* sanity checks */
 	if (ak == NULL)
@@ -267,9 +277,9 @@ static ActKeyColumn *time_cfra_find_ak (ActKeyColumn *ak, float cframe)
 	
 	/* check if this is a match, or whether it is in some subtree */
 	if (cframe < ak->cfra)
-		akn= time_cfra_find_ak(ak->left, cframe);
+		akn = time_cfra_find_ak(ak->left, cframe);
 	else if (cframe > ak->cfra)
-		akn= time_cfra_find_ak(ak->right, cframe);
+		akn = time_cfra_find_ak(ak->right, cframe);
 		
 	/* if no match found (or found match), just use the current one */
 	if (akn == NULL)
@@ -281,7 +291,7 @@ static ActKeyColumn *time_cfra_find_ak (ActKeyColumn *ak, float cframe)
 /* helper for time_draw_keyframes() */
 static void time_draw_idblock_keyframes(View2D *v2d, ID *id, short onlysel)
 {
-	bDopeSheet ads= {NULL};
+	bDopeSheet ads = {NULL};
 	DLRBT_Tree keys;
 	ActKeyColumn *ak;
 	
@@ -311,13 +321,13 @@ static void time_draw_idblock_keyframes(View2D *v2d, ID *id, short onlysel)
 	 *	- draw within a single GL block to be faster
 	 */
 	glBegin(GL_LINES);
-		for ( ak=time_cfra_find_ak(keys.root, v2d->cur.xmin); 
-			 (ak) && (ak->cfra <= v2d->cur.xmax); 
-			  ak=ak->next ) 
-		{
-			glVertex2f(ak->cfra, v2d->tot.ymin);
-			glVertex2f(ak->cfra, v2d->tot.ymax);
-		}
+	for (ak = time_cfra_find_ak(keys.root, v2d->cur.xmin);
+	     (ak) && (ak->cfra <= v2d->cur.xmax);
+	     ak = ak->next)
+	{
+		glVertex2f(ak->cfra, v2d->tot.ymin);
+		glVertex2f(ak->cfra, v2d->tot.ymax);
+	}
 	glEnd(); // GL_LINES
 		
 	/* free temp stuff */
@@ -327,25 +337,25 @@ static void time_draw_idblock_keyframes(View2D *v2d, ID *id, short onlysel)
 /* draw keyframe lines for timeline */
 static void time_draw_keyframes(const bContext *C, SpaceTime *stime, ARegion *ar)
 {
-	Scene *scene= CTX_data_scene(C);
-	Object *ob= CTX_data_active_object(C);
-	View2D *v2d= &ar->v2d;
-	short onlysel= (stime->flag & TIME_ONLYACTSEL);
+	Scene *scene = CTX_data_scene(C);
+	Object *ob = CTX_data_active_object(C);
+	View2D *v2d = &ar->v2d;
+	short onlysel = (stime->flag & TIME_ONLYACTSEL);
 	
 	/* draw scene keyframes first 
 	 *	- don't try to do this when only drawing active/selected data keyframes,
 	 *	  since this can become quite slow
 	 */
-	if (scene && onlysel==0) {
+	if (scene && onlysel == 0) {
 		/* set draw color */
 		glColor3ub(0xDD, 0xA7, 0x00);
 		time_draw_idblock_keyframes(v2d, (ID *)scene, onlysel);
 	}
 	
 	/* draw keyframes from selected objects 
-	 *	- only do the active object if in posemode (i.e. showing only keyframes for the bones)
-	 *	  OR the onlysel flag was set, which means that only active object's keyframes should
-	 *	  be considered
+	 *  - only do the active object if in posemode (i.e. showing only keyframes for the bones)
+	 *    OR the onlysel flag was set, which means that only active object's keyframes should
+	 *    be considered
 	 */
 	glColor3ub(0xDD, 0xD7, 0x00);
 	
@@ -354,17 +364,17 @@ static void time_draw_keyframes(const bContext *C, SpaceTime *stime, ARegion *ar
 		time_draw_idblock_keyframes(v2d, (ID *)ob, onlysel);
 	}
 	else {
-		short active_done = 0;
+		short active_done = FALSE;
 		
 		/* draw keyframes from all selected objects */
-		CTX_DATA_BEGIN(C, Object*, obsel, selected_objects) 
+		CTX_DATA_BEGIN (C, Object *, obsel, selected_objects)
 		{
 			/* last arg is 0, since onlysel doesn't apply here... */
 			time_draw_idblock_keyframes(v2d, (ID *)obsel, 0);
 			
 			/* if this object is the active one, set flag so that we don't draw again */
 			if (obsel == ob)
-				active_done= 1;
+				active_done = TRUE;
 		}
 		CTX_DATA_END;
 		
@@ -379,7 +389,7 @@ static void time_draw_keyframes(const bContext *C, SpaceTime *stime, ARegion *ar
 static void time_refresh(const bContext *UNUSED(C), ScrArea *sa)
 {
 	/* find the main timeline region and refresh cache display*/
-	ARegion *ar= BKE_area_find_region_type(sa, RGN_TYPE_WINDOW);
+	ARegion *ar = BKE_area_find_region_type(sa, RGN_TYPE_WINDOW);
 	if (ar) {
 		SpaceTime *stime = (SpaceTime *)sa->spacedata.first;
 		time_cache_refresh(stime);
@@ -398,6 +408,7 @@ static void time_listener(ScrArea *sa, wmNotifier *wmn)
 				case ND_POINTCACHE:
 				case ND_MODIFIER:
 				case ND_PARTICLE:
+				case ND_KEYS:
 					ED_area_tag_refresh(sa);
 					ED_area_tag_redraw(sa);
 					break;
@@ -410,19 +421,19 @@ static void time_listener(ScrArea *sa, wmNotifier *wmn)
 					ED_area_tag_refresh(sa);
 					break;
 				case ND_FRAME_RANGE:
-					{
-						ARegion *ar;
-						Scene *scene = wmn->reference;
+				{
+					ARegion *ar;
+					Scene *scene = wmn->reference;
 
-						for (ar= sa->regionbase.first; ar; ar= ar->next) {
-							if (ar->regiontype==RGN_TYPE_WINDOW) {
-								ar->v2d.tot.xmin = (float)(SFRA - 4);
-								ar->v2d.tot.xmax = (float)(EFRA + 4);
-								break;
-							}
+					for (ar = sa->regionbase.first; ar; ar = ar->next) {
+						if (ar->regiontype == RGN_TYPE_WINDOW) {
+							ar->v2d.tot.xmin = (float)(SFRA - 4);
+							ar->v2d.tot.xmax = (float)(EFRA + 4);
+							break;
 						}
 					}
-					break;
+				}
+				break;
 			}
 		case NC_SPACE:
 			switch (wmn->data) {
@@ -456,13 +467,13 @@ static void time_main_area_init(wmWindowManager *wm, ARegion *ar)
 static void time_main_area_draw(const bContext *C, ARegion *ar)
 {
 	/* draw entirely, view changes should be handled here */
-	Scene *scene= CTX_data_scene(C);
-	SpaceTime *stime= CTX_wm_space_time(C);
+	Scene *scene = CTX_data_scene(C);
+	SpaceTime *stime = CTX_wm_space_time(C);
 	Object *obact = CTX_data_active_object(C);
-	View2D *v2d= &ar->v2d;
+	View2D *v2d = &ar->v2d;
 	View2DGrid *grid;
 	View2DScrollers *scrollers;
-	int unit, flag=0;
+	int unit, flag = 0;
 	
 	/* clear and setup matrix */
 	UI_ThemeClearColor(TH_BACK);
@@ -471,9 +482,9 @@ static void time_main_area_draw(const bContext *C, ARegion *ar)
 	UI_view2d_view_ortho(v2d);
 	
 	/* grid */
-	unit= (stime->flag & TIME_DRAWFRAMES)? V2D_UNIT_FRAMES: V2D_UNIT_SECONDS;
-	grid= UI_view2d_grid_calc(scene, v2d, unit, V2D_GRID_CLAMP, V2D_ARG_DUMMY, V2D_ARG_DUMMY, ar->winx, ar->winy);
-	UI_view2d_grid_draw(v2d, grid, (V2D_VERTICAL_LINES|V2D_VERTICAL_AXIS));
+	unit = (stime->flag & TIME_DRAWFRAMES) ? V2D_UNIT_FRAMES : V2D_UNIT_SECONDS;
+	grid = UI_view2d_grid_calc(scene, v2d, unit, V2D_GRID_CLAMP, V2D_ARG_DUMMY, V2D_ARG_DUMMY, ar->winx, ar->winy);
+	UI_view2d_grid_draw(v2d, grid, (V2D_VERTICAL_LINES | V2D_VERTICAL_AXIS));
 	UI_view2d_grid_free(grid);
 	
 	/* start and end frame */
@@ -481,8 +492,8 @@ static void time_main_area_draw(const bContext *C, ARegion *ar)
 	
 	/* current frame */
 	flag = DRAWCFRA_WIDE; /* this is only really needed on frames where there's a keyframe, but this will do... */
-	if ((stime->flag & TIME_DRAWFRAMES)==0) 	flag |= DRAWCFRA_UNIT_SECONDS;
-	if (stime->flag & TIME_CFRA_NUM) 			flag |= DRAWCFRA_SHOW_NUMBOX;
+	if ((stime->flag & TIME_DRAWFRAMES) == 0)  flag |= DRAWCFRA_UNIT_SECONDS;
+	if (stime->flag & TIME_CFRA_NUM)           flag |= DRAWCFRA_SHOW_NUMBOX;
 	ANIM_draw_cfra(C, v2d, flag);
 	
 	UI_view2d_view_ortho(v2d);
@@ -495,13 +506,13 @@ static void time_main_area_draw(const bContext *C, ARegion *ar)
 	draw_markers_time(C, 0);
 	
 	/* caches */
-	time_draw_cache(stime, obact);
+	time_draw_cache(stime, obact, scene);
 	
 	/* reset view matrix */
 	UI_view2d_view_restore(C);
 	
 	/* scrollers */
-	scrollers= UI_view2d_scrollers_calc(C, v2d, unit, V2D_GRID_CLAMP, V2D_ARG_DUMMY, V2D_ARG_DUMMY);
+	scrollers = UI_view2d_scrollers_calc(C, v2d, unit, V2D_GRID_CLAMP, V2D_ARG_DUMMY, V2D_ARG_DUMMY);
 	UI_view2d_scrollers_draw(C, v2d, scrollers);
 	UI_view2d_scrollers_free(scrollers);
 }
@@ -509,7 +520,7 @@ static void time_main_area_draw(const bContext *C, ARegion *ar)
 static void time_main_area_listener(ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
-	switch(wmn->category) {
+	switch (wmn->category) {
 		case NC_SPACE:
 			if (wmn->data == ND_SPACE_TIME)
 				ED_region_tag_redraw(ar);
@@ -528,7 +539,7 @@ static void time_main_area_listener(ARegion *ar, wmNotifier *wmn)
 				case ND_KEYINGSET:
 				case ND_RENDER_OPTIONS:
 					ED_region_tag_redraw(ar);
-				break;
+					break;
 			}
 	}
 }
@@ -549,9 +560,9 @@ static void time_header_area_draw(const bContext *C, ARegion *ar)
 static void time_header_area_listener(ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
-	switch(wmn->category) {
+	switch (wmn->category) {
 		case NC_SCREEN:
-			if (wmn->data==ND_ANIMPLAY)
+			if (wmn->data == ND_ANIMPLAY)
 				ED_region_tag_redraw(ar);
 			break;
 
@@ -563,7 +574,7 @@ static void time_header_area_listener(ARegion *ar, wmNotifier *wmn)
 				case ND_KEYINGSET:
 				case ND_RENDER_OPTIONS:
 					ED_region_tag_redraw(ar);
-				break;
+					break;
 			}
 
 		case NC_SPACE:
@@ -577,57 +588,57 @@ static void time_header_area_listener(ARegion *ar, wmNotifier *wmn)
 
 static SpaceLink *time_new(const bContext *C)
 {
-	Scene *scene= CTX_data_scene(C);
+	Scene *scene = CTX_data_scene(C);
 	ARegion *ar;
 	SpaceTime *stime;
 
-	stime= MEM_callocN(sizeof(SpaceTime), "inittime");
+	stime = MEM_callocN(sizeof(SpaceTime), "inittime");
 
-	stime->spacetype= SPACE_TIME;
+	stime->spacetype = SPACE_TIME;
 	stime->flag |= TIME_DRAWFRAMES;
 
 	/* header */
-	ar= MEM_callocN(sizeof(ARegion), "header for time");
+	ar = MEM_callocN(sizeof(ARegion), "header for time");
 	
 	BLI_addtail(&stime->regionbase, ar);
-	ar->regiontype= RGN_TYPE_HEADER;
-	ar->alignment= RGN_ALIGN_BOTTOM;
+	ar->regiontype = RGN_TYPE_HEADER;
+	ar->alignment = RGN_ALIGN_BOTTOM;
 	
 	/* main area */
-	ar= MEM_callocN(sizeof(ARegion), "main area for time");
+	ar = MEM_callocN(sizeof(ARegion), "main area for time");
 	
 	BLI_addtail(&stime->regionbase, ar);
-	ar->regiontype= RGN_TYPE_WINDOW;
+	ar->regiontype = RGN_TYPE_WINDOW;
 	
 	ar->v2d.tot.xmin = (float)(SFRA - 4);
 	ar->v2d.tot.ymin = 0.0f;
 	ar->v2d.tot.xmax = (float)(EFRA + 4);
 	ar->v2d.tot.ymax = 50.0f;
 	
-	ar->v2d.cur= ar->v2d.tot;
+	ar->v2d.cur = ar->v2d.tot;
 
-	ar->v2d.min[0]= 1.0f;
-	ar->v2d.min[1]= 50.0f;
+	ar->v2d.min[0] = 1.0f;
+	ar->v2d.min[1] = 50.0f;
 
-	ar->v2d.max[0]= MAXFRAMEF;
-	ar->v2d.max[1]= 50.0; 
+	ar->v2d.max[0] = MAXFRAMEF;
+	ar->v2d.max[1] = 50.0;
 
-	ar->v2d.minzoom= 0.1f;
-	ar->v2d.maxzoom= 10.0;
+	ar->v2d.minzoom = 0.1f;
+	ar->v2d.maxzoom = 10.0;
 
-	ar->v2d.scroll |= (V2D_SCROLL_BOTTOM|V2D_SCROLL_SCALE_HORIZONTAL);
+	ar->v2d.scroll |= (V2D_SCROLL_BOTTOM | V2D_SCROLL_SCALE_HORIZONTAL);
 	ar->v2d.align |= V2D_ALIGN_NO_NEG_Y;
 	ar->v2d.keepofs |= V2D_LOCKOFS_Y;
 	ar->v2d.keepzoom |= V2D_LOCKZOOM_Y;
 
 
-	return (SpaceLink*)stime;
+	return (SpaceLink *)stime;
 }
 
 /* not spacelink itself */
 static void time_free(SpaceLink *sl)
 {
-	SpaceTime *stime= (SpaceTime *)sl;
+	SpaceTime *stime = (SpaceTime *)sl;
 	
 	time_cache_free(stime);
 }
@@ -636,20 +647,21 @@ static void time_free(SpaceLink *sl)
 /* validate spacedata, add own area level handlers */
 static void time_init(wmWindowManager *UNUSED(wm), ScrArea *sa)
 {
-	SpaceTime *stime= (SpaceTime *)sa->spacedata.first;
+	SpaceTime *stime = (SpaceTime *)sa->spacedata.first;
 	
 	time_cache_free(stime);
 	
 	/* enable all cache display */
 	stime->cache_display |= TIME_CACHE_DISPLAY;
-	stime->cache_display |= (TIME_CACHE_SOFTBODY|TIME_CACHE_PARTICLES);
-	stime->cache_display |= (TIME_CACHE_CLOTH|TIME_CACHE_SMOKE|TIME_CACHE_DYNAMICPAINT);
+	stime->cache_display |= (TIME_CACHE_SOFTBODY | TIME_CACHE_PARTICLES);
+	stime->cache_display |= (TIME_CACHE_CLOTH | TIME_CACHE_SMOKE | TIME_CACHE_DYNAMICPAINT);
+	stime->cache_display |= TIME_CACHE_RIGIDBODY;
 }
 
 static SpaceLink *time_duplicate(SpaceLink *sl)
 {
-	SpaceTime *stime= (SpaceTime *)sl;
-	SpaceTime *stimen= MEM_dupallocN(stime);
+	SpaceTime *stime = (SpaceTime *)sl;
+	SpaceTime *stimen = MEM_dupallocN(stime);
 	
 	stimen->caches.first = stimen->caches.last = NULL;
 	
@@ -660,43 +672,42 @@ static SpaceLink *time_duplicate(SpaceLink *sl)
 /* it defines all callbacks to maintain spaces */
 void ED_spacetype_time(void)
 {
-	SpaceType *st= MEM_callocN(sizeof(SpaceType), "spacetype time");
+	SpaceType *st = MEM_callocN(sizeof(SpaceType), "spacetype time");
 	ARegionType *art;
 	
-	st->spaceid= SPACE_TIME;
+	st->spaceid = SPACE_TIME;
 	strncpy(st->name, "Timeline", BKE_ST_MAXNAME);
 	
-	st->new= time_new;
-	st->free= time_free;
-	st->init= time_init;
-	st->duplicate= time_duplicate;
-	st->operatortypes= time_operatortypes;
-	st->keymap= NULL;
-	st->listener= time_listener;
-	st->refresh= time_refresh;
+	st->new = time_new;
+	st->free = time_free;
+	st->init = time_init;
+	st->duplicate = time_duplicate;
+	st->operatortypes = time_operatortypes;
+	st->keymap = NULL;
+	st->listener = time_listener;
+	st->refresh = time_refresh;
 	
 	/* regions: main window */
-	art= MEM_callocN(sizeof(ARegionType), "spacetype time region");
+	art = MEM_callocN(sizeof(ARegionType), "spacetype time region");
 	art->regionid = RGN_TYPE_WINDOW;
-	art->keymapflag= ED_KEYMAP_VIEW2D|ED_KEYMAP_MARKERS|ED_KEYMAP_ANIMATION|ED_KEYMAP_FRAMES;
+	art->keymapflag = ED_KEYMAP_VIEW2D | ED_KEYMAP_MARKERS | ED_KEYMAP_ANIMATION | ED_KEYMAP_FRAMES;
 	
-	art->init= time_main_area_init;
-	art->draw= time_main_area_draw;
-	art->listener= time_main_area_listener;
-	art->keymap= time_keymap;
+	art->init = time_main_area_init;
+	art->draw = time_main_area_draw;
+	art->listener = time_main_area_listener;
+	art->keymap = time_keymap;
 	BLI_addhead(&st->regiontypes, art);
 	
 	/* regions: header */
-	art= MEM_callocN(sizeof(ARegionType), "spacetype time region");
+	art = MEM_callocN(sizeof(ARegionType), "spacetype time region");
 	art->regionid = RGN_TYPE_HEADER;
-	art->prefsizey= HEADERY;
-	art->keymapflag= ED_KEYMAP_UI|ED_KEYMAP_VIEW2D|ED_KEYMAP_FRAMES|ED_KEYMAP_HEADER;
+	art->prefsizey = HEADERY;
+	art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_VIEW2D | ED_KEYMAP_FRAMES | ED_KEYMAP_HEADER;
 	
-	art->init= time_header_area_init;
-	art->draw= time_header_area_draw;
-	art->listener= time_header_area_listener;
+	art->init = time_header_area_init;
+	art->draw = time_header_area_draw;
+	art->listener = time_header_area_listener;
 	BLI_addhead(&st->regiontypes, art);
 		
 	BKE_spacetype_register(st);
 }
-
