@@ -51,6 +51,7 @@
 
 #include "BKE_DerivedMesh.h"
 #include "BKE_effect.h"
+#include "BKE_global.h"
 #include "BKE_image.h"
 #include "BKE_material.h"
 #include "BKE_paint.h"
@@ -71,7 +72,7 @@
 #include "ED_mesh.h"
 #include "ED_uvedit.h"
 
-#include "view3d_intern.h"  // own include
+#include "view3d_intern.h"  /* own include */
 
 /* user data structures for derived mesh callbacks */
 typedef struct drawMeshFaceSelect_userData {
@@ -179,7 +180,7 @@ static DMDrawOption draw_mesh_face_select__drawFaceOptsInv(void *userData, int i
 		return DM_DRAW_OPTION_SKIP;
 }
 
-void draw_mesh_face_select(RegionView3D *rv3d, Mesh *me, DerivedMesh *dm)
+static void draw_mesh_face_select(RegionView3D *rv3d, Mesh *me, DerivedMesh *dm)
 {
 	drawMeshFaceSelect_userData data;
 
@@ -214,7 +215,7 @@ void draw_mesh_face_select(RegionView3D *rv3d, Mesh *me, DerivedMesh *dm)
 	dm->drawMappedEdges(dm, draw_mesh_face_select__setSelectOpts, &data);
 	setlinestyle(0);
 
-	bglPolygonOffset(rv3d->dist, 0.0);  // resets correctly now, even after calling accumulated offsets
+	bglPolygonOffset(rv3d->dist, 0.0);  /* resets correctly now, even after calling accumulated offsets */
 
 	BLI_edgehash_free(data.eh, NULL);
 }
@@ -223,7 +224,7 @@ void draw_mesh_face_select(RegionView3D *rv3d, Mesh *me, DerivedMesh *dm)
 
 static Material *give_current_material_or_def(Object *ob, int matnr)
 {
-	extern Material defmaterial;    // render module abuse...
+	extern Material defmaterial;  /* render module abuse... */
 	Material *ma = give_current_material(ob, matnr);
 
 	return ma ? ma : &defmaterial;
@@ -233,10 +234,11 @@ static Material *give_current_material_or_def(Object *ob, int matnr)
 
 static struct TextureDrawState {
 	Object *ob;
-	int islit, istex;
+	int is_lit, is_tex;
 	int color_profile;
+	bool use_backface_culling;
 	unsigned char obcol[4];
-} Gtexdraw = {NULL, 0, 0, 0, {0, 0, 0, 0}};
+} Gtexdraw = {NULL, 0, 0, 0, false, {0, 0, 0, 0}};
 
 static int set_draw_settings_cached(int clearcache, MTFace *texface, Material *ma, struct TextureDrawState gtexdraw)
 {
@@ -248,8 +250,8 @@ static int set_draw_settings_cached(int clearcache, MTFace *texface, Material *m
 	static int c_lit;
 	static int c_has_texface;
 
-	Object *litob = NULL; //to get mode to turn off mipmap in painting mode
-	int backculled = GEMAT_BACKCULL;
+	Object *litob = NULL;  /* to get mode to turn off mipmap in painting mode */
+	int backculled = GEMAT_BACKCULL || gtexdraw.use_backface_culling;
 	int alphablend = 0;
 	int textured = 0;
 	int lit = 0;
@@ -263,17 +265,17 @@ static int set_draw_settings_cached(int clearcache, MTFace *texface, Material *m
 		c_has_texface = -1;
 	}
 	else {
-		textured = gtexdraw.istex;
+		textured = gtexdraw.is_tex;
 		litob = gtexdraw.ob;
 	}
 
 	/* convert number of lights into boolean */
-	if (gtexdraw.islit) lit = 1;
+	if (gtexdraw.is_lit) lit = 1;
 
 	if (ma) {
 		alphablend = ma->game.alpha_blend;
 		if (ma->mode & MA_SHLESS) lit = 0;
-		backculled = ma->game.flag & GEMAT_BACKCULL;
+		backculled = (ma->game.flag & GEMAT_BACKCULL) || gtexdraw.use_backface_culling;
 	}
 
 	if (texface) {
@@ -317,7 +319,7 @@ static int set_draw_settings_cached(int clearcache, MTFace *texface, Material *m
 	if (lit != c_lit || ma != c_ma) {
 		if (lit) {
 			float spec[4];
-			if (!ma) ma = give_current_material_or_def(NULL, 0);  //default material
+			if (!ma) ma = give_current_material_or_def(NULL, 0);  /* default material */
 
 			spec[0] = ma->spec * ma->specr;
 			spec[1] = ma->spec * ma->specg;
@@ -343,39 +345,44 @@ static int set_draw_settings_cached(int clearcache, MTFace *texface, Material *m
 static void draw_textured_begin(Scene *scene, View3D *v3d, RegionView3D *rv3d, Object *ob)
 {
 	unsigned char obcol[4];
-	int istex, solidtex;
+	int is_tex, solidtex;
+	Mesh *me = ob->data;
 
-	// XXX scene->obedit warning
+	/* XXX scene->obedit warning */
 
 	/* texture draw is abused for mask selection mode, do this so wire draw
 	 * with face selection in weight paint is not lit. */
 	if ((v3d->drawtype <= OB_WIRE) && (ob->mode & (OB_MODE_VERTEX_PAINT | OB_MODE_WEIGHT_PAINT))) {
 		solidtex = FALSE;
-		Gtexdraw.islit = 0;
+		Gtexdraw.is_lit = 0;
 	}
 	else if (v3d->drawtype == OB_SOLID || ((ob->mode & OB_MODE_EDIT) && v3d->drawtype != OB_TEXTURE)) {
 		/* draw with default lights in solid draw mode and edit mode */
 		solidtex = TRUE;
-		Gtexdraw.islit = -1;
+		Gtexdraw.is_lit = -1;
 	}
 	else {
 		/* draw with lights in the scene otherwise */
 		solidtex = FALSE;
-		Gtexdraw.islit = GPU_scene_object_lights(scene, ob, v3d->lay, rv3d->viewmat, !rv3d->is_persp);
+		Gtexdraw.is_lit = GPU_scene_object_lights(scene, ob, v3d->lay, rv3d->viewmat, !rv3d->is_persp);
 	}
 	
 	rgba_float_to_uchar(obcol, ob->col);
 
-	glCullFace(GL_BACK); glEnable(GL_CULL_FACE);
-	if (solidtex || v3d->drawtype == OB_TEXTURE) istex = 1;
-	else istex = 0;
+	if (solidtex || v3d->drawtype == OB_TEXTURE) is_tex = 1;
+	else is_tex = 0;
 
 	Gtexdraw.ob = ob;
-	Gtexdraw.istex = istex;
-	Gtexdraw.color_profile = scene->r.color_mgt_flag & R_COLOR_MANAGEMENT;
+	Gtexdraw.is_tex = is_tex;
+
+	Gtexdraw.color_profile = BKE_scene_check_color_management_enabled(scene);
+	Gtexdraw.use_backface_culling = (v3d->flag2 & V3D_BACKFACE_CULLING) != 0;
+
 	memcpy(Gtexdraw.obcol, obcol, sizeof(obcol));
 	set_draw_settings_cached(1, NULL, NULL, Gtexdraw);
 	glShadeModel(GL_SMOOTH);
+	glCullFace(GL_BACK);
+	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, (me->flag & ME_TWOSIDED) ? GL_TRUE : GL_FALSE);
 }
 
 static void draw_textured_end(void)
@@ -385,9 +392,10 @@ static void draw_textured_end(void)
 
 	glShadeModel(GL_FLAT);
 	glDisable(GL_CULL_FACE);
+	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
 
 	/* XXX, bad patch - GPU_default_lights() calls
-	 * glLightfv(GL_LIGHT_POSITION, ...) which
+	 * glLightfv(GL_POSITION, ...) which
 	 * is transformed by the current matrix... we
 	 * need to make sure that matrix is identity.
 	 * 
@@ -396,7 +404,7 @@ static void draw_textured_end(void)
 	 *  - zr
 	 */
 	glPushMatrix();
-	glLoadIdentity();	
+	glLoadIdentity();
 	GPU_default_lights();
 	glPopMatrix();
 }
@@ -404,14 +412,14 @@ static void draw_textured_end(void)
 static DMDrawOption draw_tface__set_draw_legacy(MTFace *tface, int has_mcol, int matnr)
 {
 	Material *ma = give_current_material(Gtexdraw.ob, matnr + 1);
-	int validtexture = 0;
+	int invalidtexture = 0;
 
 	if (ma && (ma->game.flag & GEMAT_INVISIBLE))
 		return DM_DRAW_OPTION_SKIP;
 
-	validtexture = set_draw_settings_cached(0, tface, ma, Gtexdraw);
+	invalidtexture = set_draw_settings_cached(0, tface, ma, Gtexdraw);
 
-	if (tface && validtexture) {
+	if (tface && invalidtexture) {
 		glColor3ub(0xFF, 0x00, 0xFF);
 		return DM_DRAW_OPTION_NO_MCOL; /* Don't set color */
 	}
@@ -446,27 +454,20 @@ static DMDrawOption draw_mcol__set_draw_legacy(MTFace *UNUSED(tface), int has_mc
 		return DM_DRAW_OPTION_NO_MCOL;
 }
 
-static DMDrawOption draw_tface__set_draw(MTFace *tface, int has_mcol, int matnr)
+static DMDrawOption draw_tface__set_draw(MTFace *tface, int UNUSED(has_mcol), int matnr)
 {
 	Material *ma = give_current_material(Gtexdraw.ob, matnr + 1);
 
 	if (ma && (ma->game.flag & GEMAT_INVISIBLE)) return 0;
 
-	if (tface && set_draw_settings_cached(0, tface, ma, Gtexdraw)) {
-		return DM_DRAW_OPTION_NO_MCOL; /* Don't set color */
-	}
-	else if (tface && (tface->mode & TF_OBCOL)) {
-		return DM_DRAW_OPTION_NO_MCOL; /* Don't set color */
-	}
-	else if (!has_mcol) {
-		/* XXX: this return value looks wrong (and doesn't match comment) */
-		return DM_DRAW_OPTION_NORMAL; /* Don't set color */
-	}
-	else {
-		return DM_DRAW_OPTION_NORMAL; /* Set color from mcol */
-	}
+	if (tface)
+		set_draw_settings_cached(0, tface, ma, Gtexdraw);
+
+	/* always use color from mcol, as set in update_tface_color_layer */
+	return DM_DRAW_OPTION_NORMAL;
 }
-static void add_tface_color_layer(DerivedMesh *dm)
+
+static void update_tface_color_layer(DerivedMesh *dm)
 {
 	MTFace *tface = DM_get_tessface_data_layer(dm, CD_MTFACE);
 	MFace *mface = dm->getTessFaceArray(dm);
@@ -476,7 +477,15 @@ static void add_tface_color_layer(DerivedMesh *dm)
 	if (!mcol)
 		mcol = dm->getTessFaceDataArray(dm, CD_MCOL);
 
-	finalCol = MEM_mallocN(sizeof(MCol) * 4 * dm->getNumTessFaces(dm), "add_tface_color_layer");
+	if (CustomData_has_layer(&dm->faceData, CD_TEXTURE_MCOL)) {
+		finalCol = CustomData_get_layer(&dm->faceData, CD_TEXTURE_MCOL);
+	}
+	else {
+		finalCol = MEM_mallocN(sizeof(MCol) * 4 * dm->getNumTessFaces(dm), "add_tface_color_layer");
+
+		CustomData_add_layer(&dm->faceData, CD_TEXTURE_MCOL, CD_ASSIGN, finalCol, dm->numTessFaceData);
+	}
+
 	for (i = 0; i < dm->getNumTessFaces(dm); i++) {
 		Material *ma = give_current_material(Gtexdraw.ob, mface[i].mat_nr + 1);
 
@@ -490,18 +499,18 @@ static void add_tface_color_layer(DerivedMesh *dm)
 					finalCol[i * 4 + j].r = 255;
 				}
 		}
-		else if (tface && mface && set_draw_settings_cached(0, tface, ma, Gtexdraw)) {
+		else if (tface && set_draw_settings_cached(0, tface, ma, Gtexdraw)) {
 			for (j = 0; j < 4; j++) {
 				finalCol[i * 4 + j].b = 255;
 				finalCol[i * 4 + j].g = 0;
 				finalCol[i * 4 + j].r = 255;
 			}
 		}
-		else if (tface && (tface->mode & TF_OBCOL)) {
+		else if (ma && (ma->shade_flag & MA_OBCOLOR)) {
 			for (j = 0; j < 4; j++) {
-				finalCol[i * 4 + j].b = FTOCHAR(Gtexdraw.obcol[0]);
-				finalCol[i * 4 + j].g = FTOCHAR(Gtexdraw.obcol[1]);
-				finalCol[i * 4 + j].r = FTOCHAR(Gtexdraw.obcol[2]);
+				finalCol[i * 4 + j].b = Gtexdraw.obcol[0];
+				finalCol[i * 4 + j].g = Gtexdraw.obcol[1];
+				finalCol[i * 4 + j].r = Gtexdraw.obcol[2];
 			}
 		}
 		else if (!mcol) {
@@ -514,8 +523,7 @@ static void add_tface_color_layer(DerivedMesh *dm)
 			}
 			else {
 				float col[3];
-				Material *ma = give_current_material(Gtexdraw.ob, mface[i].mat_nr + 1);
-				
+
 				if (ma) {
 					if (Gtexdraw.color_profile) linearrgb_to_srgb_v3_v3(col, &ma->r);
 					else copy_v3_v3(col, &ma->r);
@@ -542,7 +550,6 @@ static void add_tface_color_layer(DerivedMesh *dm)
 			}
 		}
 	}
-	CustomData_add_layer(&dm->faceData, CD_TEXTURE_MCOL, CD_ASSIGN, finalCol, dm->numTessFaceData);
 }
 
 static DMDrawOption draw_tface_mapped__set_draw(void *userData, int index)
@@ -614,12 +621,12 @@ static void draw_mesh_text(Scene *scene, Object *ob, int glsl)
 	MLoopCol *mloopcol = me->mloopcol;  /* why does mcol exist? */
 	MLoopCol *lcol;
 
-	bProperty *prop = get_ob_property(ob, "Text");
+	bProperty *prop = BKE_bproperty_object_get(ob, "Text");
 	GPUVertexAttribs gattribs;
 	int a, totpoly = me->totpoly;
 
 	/* fake values to pass to GPU_render_text() */
-	MCol tmp_mcol[4]  = {{0}};
+	MCol  tmp_mcol[4]  = {{0}};
 	MCol *tmp_mcol_pt  = mloopcol ? tmp_mcol : NULL;
 	MTFace tmp_tf      = {{{0}}};
 
@@ -640,7 +647,7 @@ static void draw_mesh_text(Scene *scene, Object *ob, int glsl)
 		short matnr = mp->mat_nr;
 		int mf_smooth = mp->flag & ME_SMOOTH;
 		Material *mat = me->mat[matnr];
-		int mode = mat->game.flag;
+		int mode = mat ? mat->game.flag : GEMAT_INVISIBLE;
 
 		if (!(mode & GEMAT_INVISIBLE) && (mode & GEMAT_TEXT) && mp->totloop >= 3) {
 			/* get the polygon as a tri/quad */
@@ -686,11 +693,11 @@ static void draw_mesh_text(Scene *scene, Object *ob, int glsl)
 
 			/* COLOR */
 			if (mloopcol) {
-				unsigned int totloop_clamp = MIN2(4, mp->totloop);
+				unsigned int totloop_clamp = min_ii(4, mp->totloop);
 				unsigned int j;
 				lcol = &mloopcol[mp->loopstart];
 
-				for (j = 0; j <= totloop_clamp; j++, lcol++) {
+				for (j = 0; j < totloop_clamp; j++, lcol++) {
 					MESH_MLOOPCOL_TO_MCOL(lcol, &tmp_mcol[j]);
 				}
 			}
@@ -705,13 +712,13 @@ static void draw_mesh_text(Scene *scene, Object *ob, int glsl)
 
 
 
-			// The BM_FONT handling is in the gpu module, shared with the
-			// game engine, was duplicated previously
+			/* The BM_FONT handling is in the gpu module, shared with the
+			 * game engine, was duplicated previously */
 
-			set_property_valstr(prop, string);
+			BKE_bproperty_set_valstr(prop, string);
 			characters = strlen(string);
 			
-			if (!BKE_image_get_ibuf(mtpoly->tpage, NULL))
+			if (!BKE_image_has_ibuf(mtpoly->tpage, NULL))
 				characters = 0;
 
 			if (!mf_smooth) {
@@ -756,7 +763,8 @@ static int compareDrawOptionsEm(void *userData, int cur_index, int next_index)
 	return 1;
 }
 
-void draw_mesh_textured_old(Scene *scene, View3D *v3d, RegionView3D *rv3d, Object *ob, DerivedMesh *dm, int draw_flags)
+static void draw_mesh_textured_old(Scene *scene, View3D *v3d, RegionView3D *rv3d,
+                                   Object *ob, DerivedMesh *dm, const int draw_flags)
 {
 	Mesh *me = ob->data;
 	
@@ -797,8 +805,7 @@ void draw_mesh_textured_old(Scene *scene, View3D *v3d, RegionView3D *rv3d, Objec
 		else {
 			drawTFace_userData userData;
 
-			if (!CustomData_has_layer(&dm->faceData, CD_TEXTURE_MCOL))
-				add_tface_color_layer(dm);
+			update_tface_color_layer(dm);
 
 			userData.mf = DM_get_tessface_data_layer(dm, CD_MFACE);
 			userData.tf = DM_get_tessface_data_layer(dm, CD_MTFACE);
@@ -808,7 +815,7 @@ void draw_mesh_textured_old(Scene *scene, View3D *v3d, RegionView3D *rv3d, Objec
 	}
 
 	/* draw game engine text hack */
-	if (get_ob_property(ob, "Text"))
+	if (BKE_bproperty_object_get(ob, "Text"))
 		draw_mesh_text(scene, ob, 0);
 
 	draw_textured_end();
@@ -856,7 +863,7 @@ static void tex_mat_set_texture_cb(void *userData, int mat_nr, void *attribs)
 	if (ED_object_get_active_image(data->ob, mat_nr, &ima, &iuser, &node)) {
 		/* get openl texture */
 		int mipmap = 1;
-		int bindcode = (ima) ? GPU_verify_image(ima, iuser, 0, 0, mipmap) : 0;
+		int bindcode = (ima) ? GPU_verify_image(ima, iuser, 0, 0, mipmap, FALSE) : 0;
 		float zero[4] = {0.0f, 0.0f, 0.0f, 0.0f};
 
 		if (bindcode) {
@@ -926,14 +933,16 @@ static int tex_mat_set_face_editmesh_cb(void *userData, int index)
 	return !BM_elem_flag_test(efa, BM_ELEM_HIDDEN);
 }
 
-void draw_mesh_textured(Scene *scene, View3D *v3d, RegionView3D *rv3d, Object *ob, DerivedMesh *dm, int draw_flags)
+void draw_mesh_textured(Scene *scene, View3D *v3d, RegionView3D *rv3d,
+                        Object *ob, DerivedMesh *dm, const int draw_flags)
 {
-	if ((!scene_use_new_shading_nodes(scene)) || (draw_flags & DRAW_MODIFIERS_PREVIEW)) {
+	/* if not cycles, or preview-modifiers, or drawing matcaps */
+	if ((!BKE_scene_use_new_shading_nodes(scene)) || (draw_flags & DRAW_MODIFIERS_PREVIEW) || (v3d->flag2 & V3D_SHOW_SOLID_MATCAP)) {
 		draw_mesh_textured_old(scene, v3d, rv3d, ob, dm, draw_flags);
 		return;
 	}
 	else if (ob->mode & (OB_MODE_VERTEX_PAINT | OB_MODE_WEIGHT_PAINT)) {
-		draw_mesh_paint(rv3d, ob, dm, draw_flags);
+		draw_mesh_paint(v3d, rv3d, ob, dm, draw_flags);
 		return;
 	}
 
@@ -947,7 +956,7 @@ void draw_mesh_textured(Scene *scene, View3D *v3d, RegionView3D *rv3d, Object *o
 		Mesh *me = ob->data;
 		TexMatCallback data = {scene, ob, me, dm};
 		int (*set_face_cb)(void *, int);
-		int glsl;
+		int glsl, picking = (G.f & G_PICKSEL);
 		
 		/* face hiding callback depending on mode */
 		if (ob == scene->obedit)
@@ -958,11 +967,11 @@ void draw_mesh_textured(Scene *scene, View3D *v3d, RegionView3D *rv3d, Object *o
 			set_face_cb = NULL;
 
 		/* test if we can use glsl */
-		glsl = (v3d->drawtype == OB_MATERIAL) && GPU_glsl_support();
+		glsl = (v3d->drawtype == OB_MATERIAL) && GPU_glsl_support() && !picking;
 
 		GPU_begin_object_materials(v3d, rv3d, scene, ob, glsl, NULL);
 
-		if (glsl) {
+		if (glsl || picking) {
 			/* draw glsl */
 			dm->drawMappedFacesMat(dm,
 			                       tex_mat_set_material_cb,
@@ -1002,51 +1011,92 @@ void draw_mesh_textured(Scene *scene, View3D *v3d, RegionView3D *rv3d, Object *o
 
 /* Vertex Paint and Weight Paint */
 
-void draw_mesh_paint(RegionView3D *rv3d, Object *ob, DerivedMesh *dm, int draw_flags)
+void draw_mesh_paint(View3D *v3d, RegionView3D *rv3d,
+                     Object *ob, DerivedMesh *dm, const int draw_flags)
 {
 	DMSetDrawOptions facemask = NULL;
 	Mesh *me = ob->data;
+	const short do_light = (v3d->drawtype >= OB_SOLID);
 
 	/* hide faces in face select mode */
-	if (draw_flags & DRAW_FACE_SELECT)
+	if (me->editflag & (ME_EDIT_PAINT_VERT_SEL | ME_EDIT_PAINT_FACE_SEL))
 		facemask = wpaint__setSolidDrawOptions_facemask;
 
-	if (ob && ob->mode & OB_MODE_WEIGHT_PAINT) {
-		/* enforce default material settings */
-		GPU_enable_material(0, NULL);
-		
-		/* but set default spec */
-		glColorMaterial(GL_FRONT_AND_BACK, GL_SPECULAR);
-		glEnable(GL_COLOR_MATERIAL);    /* according manpages needed */
-		glColor3ub(120, 120, 120);
-		glDisable(GL_COLOR_MATERIAL);
+	if (ob->mode & OB_MODE_WEIGHT_PAINT) {
 
-		/* diffuse */
-		glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
-		glEnable(GL_LIGHTING);
-		glEnable(GL_COLOR_MATERIAL);
+		if (do_light) {
+			const float spec[4] = {0.47f, 0.47f, 0.47f, 0.47f};
+
+			/* enforce default material settings */
+			GPU_enable_material(0, NULL);
+		
+			/* but set default spec */
+			glColorMaterial(GL_FRONT_AND_BACK, GL_SPECULAR);
+			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, spec);
+
+			/* diffuse */
+			glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
+			glEnable(GL_LIGHTING);
+			glEnable(GL_COLOR_MATERIAL);
+		}
 
 		dm->drawMappedFaces(dm, facemask, GPU_enable_material, NULL, me,
-							DM_DRAW_USE_COLORS | DM_DRAW_ALWAYS_SMOOTH);
+		                    DM_DRAW_USE_COLORS | DM_DRAW_ALWAYS_SMOOTH);
 
-		glDisable(GL_COLOR_MATERIAL);
-		glDisable(GL_LIGHTING);
+		if (do_light) {
+			glDisable(GL_COLOR_MATERIAL);
+			glDisable(GL_LIGHTING);
 
-		GPU_disable_material();
+			GPU_disable_material();
+		}
 	}
 	else if (ob->mode & OB_MODE_VERTEX_PAINT) {
-		if (me->mloopcol)
+		if (me->mloopcol) {
 			dm->drawMappedFaces(dm, facemask, GPU_enable_material, NULL, me,
-								DM_DRAW_USE_COLORS | DM_DRAW_ALWAYS_SMOOTH);
+			                    DM_DRAW_USE_COLORS | DM_DRAW_ALWAYS_SMOOTH);
+		}
 		else {
 			glColor3f(1.0f, 1.0f, 1.0f);
 			dm->drawMappedFaces(dm, facemask, GPU_enable_material, NULL, me,
-								DM_DRAW_ALWAYS_SMOOTH);
+			                    DM_DRAW_ALWAYS_SMOOTH);
 		}
 	}
 
 	/* draw face selection on top */
-	if (draw_flags & DRAW_FACE_SELECT)
+	if (draw_flags & DRAW_FACE_SELECT) {
 		draw_mesh_face_select(rv3d, me, dm);
+	}
+	else if ((do_light == FALSE) || (ob->dtx & OB_DRAWWIRE)) {
+		const int use_depth = (v3d->flag & V3D_ZBUF_SELECT) || !(ob->mode & OB_MODE_WEIGHT_PAINT);
+
+		/* weight paint in solid mode, special case. focus on making the weights clear
+		 * rather than the shading, this is also forced in wire view */
+
+		if (use_depth) {
+			bglPolygonOffset(rv3d->dist, 1.0);
+			glDepthMask(0);  /* disable write in zbuffer, selected edge wires show better */
+		}
+		else {
+			glDisable(GL_DEPTH_TEST);
+		}
+
+		glEnable(GL_BLEND);
+		glColor4ub(255, 255, 255, 96);
+		glEnable(GL_LINE_STIPPLE);
+		glLineStipple(1, 0xAAAA);
+
+		dm->drawEdges(dm, 1, 1);
+
+		if (use_depth) {
+			bglPolygonOffset(rv3d->dist, 0.0);
+			glDepthMask(1);
+		}
+		else {
+			glEnable(GL_DEPTH_TEST);
+		}
+
+		glDisable(GL_LINE_STIPPLE);
+		glDisable(GL_BLEND);
+	}
 }
 
