@@ -27,7 +27,7 @@
 /* defines VIEW3D_OT_fly modal operator */
 
 //#define NDOF_FLY_DEBUG
-//#define NDOF_FLY_DRAW_TOOMUCH // is this needed for ndof? - commented so redraw doesnt thrash - campbell
+//#define NDOF_FLY_DRAW_TOOMUCH  /* is this needed for ndof? - commented so redraw doesnt thrash - campbell */
 #include "DNA_anim_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_object_types.h"
@@ -76,7 +76,8 @@ enum {
 	FLY_MODAL_PRECISION_ENABLE,
 	FLY_MODAL_PRECISION_DISABLE,
 	FLY_MODAL_FREELOOK_ENABLE,
-	FLY_MODAL_FREELOOK_DISABLE
+	FLY_MODAL_FREELOOK_DISABLE,
+	FLY_MODAL_SPEED,	/* mousepan typically */
 
 };
 
@@ -132,6 +133,8 @@ void fly_modal_keymap(wmKeyConfig *keyconf)
 	WM_modalkeymap_add_item(keymap, WHEELUPMOUSE, KM_PRESS, KM_ANY, 0, FLY_MODAL_ACCELERATE);
 	WM_modalkeymap_add_item(keymap, WHEELDOWNMOUSE, KM_PRESS, KM_ANY, 0, FLY_MODAL_DECELERATE);
 
+	WM_modalkeymap_add_item(keymap, MOUSEPAN, 0, 0, 0, FLY_MODAL_SPEED);
+	
 	WM_modalkeymap_add_item(keymap, MIDDLEMOUSE, KM_PRESS, KM_ANY, 0, FLY_MODAL_PAN_ENABLE);
 	/* XXX - Bug in the event system, middle mouse release doesnt work */
 	WM_modalkeymap_add_item(keymap, MIDDLEMOUSE, KM_RELEASE, KM_ANY, 0, FLY_MODAL_PAN_DISABLE);
@@ -174,7 +177,7 @@ typedef struct FlyInfo {
 	unsigned char use_freelook;
 
 	int mval[2]; /* latest 2D mouse values */
-	wmNDOFMotionData*ndof;  /* latest 3D mouse values */
+	wmNDOFMotionData *ndof;  /* latest 3D mouse values */
 
 	/* fly state state */
 	float speed; /* the speed the view is moving per redraw */
@@ -219,7 +222,7 @@ typedef struct FlyInfo {
 
 } FlyInfo;
 
-static void drawFlyPixel(const struct bContext *UNUSED(C), struct ARegion *UNUSED(ar), void *arg)
+static void drawFlyPixel(const struct bContext *UNUSED(C), ARegion *UNUSED(ar), void *arg)
 {
 	FlyInfo *fly = arg;
 
@@ -284,6 +287,11 @@ static int initFlyInfo(bContext *C, FlyInfo *fly, wmOperator *op, wmEvent *event
 	puts("\n-- fly begin --");
 #endif
 
+	/* sanity check: for rare but possible case (if lib-linking the camera fails) */
+	if ((fly->rv3d->persp == RV3D_CAMOB) && (fly->v3d->camera == NULL)) {
+		fly->rv3d->persp = RV3D_PERSP;
+	}
+
 	if (fly->rv3d->persp == RV3D_CAMOB && fly->v3d->camera->id.lib) {
 		BKE_report(op->reports, RPT_ERROR, "Cannot fly a camera from an external library");
 		return FALSE;
@@ -333,8 +341,9 @@ static int initFlyInfo(bContext *C, FlyInfo *fly, wmOperator *op, wmEvent *event
 	upvec[2] = 0.0f;
 	copy_m3_m4(mat, fly->rv3d->viewinv);
 	mul_m3_v3(mat, upvec);
-	if (fabs(upvec[2]) < 0.1)
+	if (fabsf(upvec[2]) < 0.1f) {
 		fly->zlock = 1;
+	}
 	upvec[0] = 0;
 	upvec[1] = 0;
 	upvec[2] = 0;
@@ -345,7 +354,6 @@ static int initFlyInfo(bContext *C, FlyInfo *fly, wmOperator *op, wmEvent *event
 	/* check for flying ortho camera - which we cant support well
 	 * we _could_ also check for an ortho camera but this is easier */
 	if ((fly->rv3d->persp == RV3D_CAMOB) &&
-	    (fly->v3d->camera != NULL) &&
 	    (fly->rv3d->is_persp == FALSE))
 	{
 		((Camera *)fly->v3d->camera->data)->type = CAM_PERSP;
@@ -364,11 +372,9 @@ static int initFlyInfo(bContext *C, FlyInfo *fly, wmOperator *op, wmEvent *event
 		}
 
 		/* store the original camera loc and rot */
-		/* TODO. axis angle etc */
+		fly->obtfm = BKE_object_tfm_backup(ob_back);
 
-		fly->obtfm = object_tfm_backup(ob_back);
-
-		where_is_object(fly->scene, fly->v3d->camera);
+		BKE_object_where_is_calc(fly->scene, fly->v3d->camera);
 		negate_v3_v3(fly->rv3d->ofs, fly->v3d->camera->obmat[3]);
 
 		fly->rv3d->dist = 0.0;
@@ -421,7 +427,6 @@ static int flyEnd(bContext *C, FlyInfo *fly)
 	ED_region_draw_cb_exit(fly->ar->type, fly->draw_handle_pixel);
 
 	rv3d->dist = fly->dist_backup;
-
 	if (fly->state == FLY_CANCEL) {
 		/* Revert to original view? */
 		if (fly->persp_backup == RV3D_CAMOB) { /* a camera view */
@@ -429,19 +434,23 @@ static int flyEnd(bContext *C, FlyInfo *fly)
 			ob_back = (fly->root_parent) ? fly->root_parent : fly->v3d->camera;
 
 			/* store the original camera loc and rot */
-			object_tfm_restore(ob_back, fly->obtfm);
+			BKE_object_tfm_restore(ob_back, fly->obtfm);
 
 			DAG_id_tag_update(&ob_back->id, OB_RECALC_OB);
 		}
 		else {
 			/* Non Camera we need to reset the view back to the original location bacause the user canceled*/
 			copy_qt_qt(rv3d->viewquat, fly->rot_backup);
-			copy_v3_v3(rv3d->ofs, fly->ofs_backup);
 			rv3d->persp = fly->persp_backup;
 		}
+		/* always, is set to zero otherwise */
+		copy_v3_v3(rv3d->ofs, fly->ofs_backup);
 	}
 	else if (fly->persp_backup == RV3D_CAMOB) { /* camera */
 		DAG_id_tag_update(fly->root_parent ? &fly->root_parent->id : &v3d->camera->id, OB_RECALC_OB);
+		
+		/* always, is set to zero otherwise */
+		copy_v3_v3(rv3d->ofs, fly->ofs_backup);
 	}
 	else { /* not camera */
 
@@ -488,7 +497,7 @@ static void flyEvent(FlyInfo *fly, wmEvent *event)
 	else if (event->type == NDOF_MOTION) {
 		/* do these automagically get delivered? yes. */
 		// puts("ndof motion detected in fly mode!");
-		// static const char* tag_name = "3D mouse position";
+		// static const char *tag_name = "3D mouse position";
 
 		wmNDOFMotionData *incoming_ndof = (wmNDOFMotionData *)event->customdata;
 		switch (incoming_ndof->progress) {
@@ -513,7 +522,7 @@ static void flyEvent(FlyInfo *fly, wmEvent *event)
 				}
 				break;
 			case P_FINISHING:
-				// stop keeping track of 3D mouse position
+				/* stop keeping track of 3D mouse position */
 #ifdef NDOF_FLY_DEBUG
 				puts("stop keeping track of 3D mouse position");
 #endif
@@ -538,7 +547,22 @@ static void flyEvent(FlyInfo *fly, wmEvent *event)
 			case FLY_MODAL_CONFIRM:
 				fly->state = FLY_CONFIRM;
 				break;
-
+				
+			/* speed adjusting with mousepan (trackpad) */
+			case FLY_MODAL_SPEED:
+			{
+				float fac = 0.02f * (event->prevy - event->y);
+				
+				/* allowing to brake immediate */
+				if (fac > 0.0f && fly->speed < 0.0f)
+					fly->speed = 0.0f;
+				else if (fac < 0.0f && fly->speed > 0.0f)
+					fly->speed = 0.0f;
+				else
+					fly->speed += fly->grid * fac;
+				
+				break;
+			}
 			case FLY_MODAL_ACCELERATE:
 			{
 				double time_currwheel;
@@ -547,8 +571,8 @@ static void flyEvent(FlyInfo *fly, wmEvent *event)
 				time_currwheel = PIL_check_seconds_timer();
 				time_wheel = (float)(time_currwheel - fly->time_lastwheel);
 				fly->time_lastwheel = time_currwheel;
-				/* Mouse wheel delays range from 0.5==slow to 0.01==fast */
-				time_wheel = 1.0f + (10.0f - (20.0f * MIN2(time_wheel, 0.5f))); /* 0-0.5 -> 0-5.0 */
+				/* Mouse wheel delays range from (0.5 == slow) to (0.01 == fast) */
+				time_wheel = 1.0f + (10.0f - (20.0f * min_ff(time_wheel, 0.5f))); /* 0-0.5 -> 0-5.0 */
 
 				if (fly->speed < 0.0f) {
 					fly->speed = 0.0f;
@@ -566,7 +590,7 @@ static void flyEvent(FlyInfo *fly, wmEvent *event)
 				time_currwheel = PIL_check_seconds_timer();
 				time_wheel = (float)(time_currwheel - fly->time_lastwheel);
 				fly->time_lastwheel = time_currwheel;
-				time_wheel = 1.0f + (10.0f - (20.0f * MIN2(time_wheel, 0.5f))); /* 0-0.5 -> 0-5.0 */
+				time_wheel = 1.0f + (10.0f - (20.0f * min_ff(time_wheel, 0.5f))); /* 0-0.5 -> 0-5.0 */
 
 				if (fly->speed > 0.0f) {
 					fly->speed = 0;
@@ -580,7 +604,7 @@ static void flyEvent(FlyInfo *fly, wmEvent *event)
 				fly->pan_view = TRUE;
 				break;
 			case FLY_MODAL_PAN_DISABLE:
-//XXX2.5		warp_pointer(cent_orig[0], cent_orig[1]);
+//XXX2.5				WM_cursor_warp(CTX_wm_window(C), cent_orig[0], cent_orig[1]);
 				fly->pan_view = FALSE;
 				break;
 
@@ -690,7 +714,7 @@ static void move_camera(bContext *C, RegionView3D *rv3d, FlyInfo *fly, int orien
 {
 	/* we are in camera view so apply the view ofs and quat to the view matrix and set the camera to the view */
 
-	View3D*v3d = fly->v3d;
+	View3D *v3d = fly->v3d;
 	Scene *scene = fly->scene;
 	ID *id_key;
 
@@ -709,9 +733,9 @@ static void move_camera(bContext *C, RegionView3D *rv3d, FlyInfo *fly, int orien
 		ED_view3d_to_m4(view_mat, rv3d->ofs, rv3d->viewquat, rv3d->dist);
 		mult_m4_m4m4(diff_mat, view_mat, prev_view_imat);
 		mult_m4_m4m4(parent_mat, diff_mat, fly->root_parent->obmat);
-		object_apply_mat4(fly->root_parent, parent_mat, TRUE, FALSE);
+		BKE_object_apply_mat4(fly->root_parent, parent_mat, TRUE, FALSE);
 
-		// where_is_object(scene, fly->root_parent);
+		// BKE_object_where_is_calc(scene, fly->root_parent);
 
 		ob_update = v3d->camera->parent;
 		while (ob_update) {
@@ -724,7 +748,7 @@ static void move_camera(bContext *C, RegionView3D *rv3d, FlyInfo *fly, int orien
 	else {
 		float view_mat[4][4];
 		ED_view3d_to_m4(view_mat, rv3d->ofs, rv3d->viewquat, rv3d->dist);
-		object_apply_mat4(v3d->camera, view_mat, TRUE, FALSE);
+		BKE_object_apply_mat4(v3d->camera, view_mat, TRUE, FALSE);
 		id_key = &v3d->camera->id;
 	}
 
@@ -790,8 +814,8 @@ static int flyApply(bContext *C, FlyInfo *fly)
 	ymargin = ar->winy / 20.0f;
 
 	// UNUSED
-	// cent_orig[0]= ar->winrct.xmin + ar->winx/2;
-	// cent_orig[1]= ar->winrct.ymin + ar->winy/2;
+	// cent_orig[0] = ar->winrct.xmin + ar->winx / 2;
+	// cent_orig[1] = ar->winrct.ymin + ar->winy / 2;
 
 	{
 
@@ -810,7 +834,7 @@ static int flyApply(bContext *C, FlyInfo *fly)
 
 
 		/* scale the mouse movement by this value - scales mouse movement to the view size
-		 * moffset[0]/(ar->winx-xmargin*2) - window size minus margin (same for y)
+		 * moffset[0] / (ar->winx-xmargin * 2) - window size minus margin (same for y)
 		 *
 		 * the mouse moves isn't linear */
 
@@ -842,7 +866,7 @@ static int flyApply(bContext *C, FlyInfo *fly)
 #endif
 			time_current = PIL_check_seconds_timer();
 			time_redraw = (float)(time_current - fly->time_lastdraw);
-			time_redraw_clamped = MIN2(0.05f, time_redraw); /* clamp redraw time to avoid jitter in roll correction */
+			time_redraw_clamped = min_ff(0.05f, time_redraw); /* clamp redraw time to avoid jitter in roll correction */
 			fly->time_lastdraw = time_current;
 
 			/* Scale the time to use shift to scale the speed down- just like
@@ -1031,12 +1055,12 @@ static int flyApply_ndof(bContext *C, FlyInfo *fly)
 	/* shorthand for oft-used variables */
 	wmNDOFMotionData *ndof = fly->ndof;
 	const float dt = ndof->dt;
-	RegionView3D*rv3d = fly->rv3d;
+	RegionView3D *rv3d = fly->rv3d;
 	const int flag = U.ndof_flag;
 
 #if 0
-	int shouldRotate = (flag & NDOF_SHOULD_ROTATE) && (fly->pan_view == FALSE),
-	    shouldTranslate = (flag & (NDOF_SHOULD_PAN | NDOF_SHOULD_ZOOM));
+	int shouldRotate = (flag & NDOF_SHOULD_ROTATE) && (fly->pan_view == FALSE);
+	int shouldTranslate = (flag & (NDOF_SHOULD_PAN | NDOF_SHOULD_ZOOM));
 #endif
 
 	int shouldRotate = (fly->pan_view == FALSE);
