@@ -53,7 +53,6 @@
 
 #include "BKE_context.h"
 
-
 #include "ED_image.h"
 #include "ED_view3d.h"
 
@@ -61,13 +60,10 @@
 #include "BLI_utildefines.h"
 #include "BLI_string.h"
 
-//#include "blendef.h"
-//
-//#include "mydevice.h"
+#include "BLF_translation.h"
 
 #include "UI_resources.h"
 
-
 #include "transform.h"
 
 static void drawObjectConstraint(TransInfo *t);
@@ -76,9 +72,8 @@ static void drawObjectConstraint(TransInfo *t);
 static void constraintAutoValues(TransInfo *t, float vec[3])
 {
 	int mode = t->con.mode;
-	if (mode & CON_APPLY)
-	{
-		float nval = (t->flag & T_NULL_ONE)?1.0f:0.0f;
+	if (mode & CON_APPLY) {
+		float nval = (t->flag & T_NULL_ONE) ? 1.0f : 0.0f;
 
 		if ((mode & CON_AXIS0) == 0) {
 			vec[0] = nval;
@@ -96,21 +91,21 @@ void constraintNumInput(TransInfo *t, float vec[3])
 {
 	int mode = t->con.mode;
 	if (mode & CON_APPLY) {
-		float nval = (t->flag & T_NULL_ONE)?1.0f:0.0f;
+		float nval = (t->flag & T_NULL_ONE) ? 1.0f : 0.0f;
 
 		if (getConstraintSpaceDimension(t) == 2) {
-			int axis = mode & (CON_AXIS0|CON_AXIS1|CON_AXIS2);
-			if (axis == (CON_AXIS0|CON_AXIS1)) {
+			int axis = mode & (CON_AXIS0 | CON_AXIS1 | CON_AXIS2);
+			if (axis == (CON_AXIS0 | CON_AXIS1)) {
 				/* vec[0] = vec[0]; */ /* same */
 				/* vec[1] = vec[1]; */ /* same */
 				vec[2] = nval;
 			}
-			else if (axis == (CON_AXIS1|CON_AXIS2)) {
+			else if (axis == (CON_AXIS1 | CON_AXIS2)) {
 				vec[2] = vec[1];
 				vec[1] = vec[0];
 				vec[0] = nval;
 			}
-			else if (axis == (CON_AXIS0|CON_AXIS2)) {
+			else if (axis == (CON_AXIS0 | CON_AXIS2)) {
 				/* vec[0] = vec[0]; */  /* same */
 				vec[2] = vec[1];
 				vec[1] = nval;
@@ -157,8 +152,8 @@ static void postConstraintChecks(TransInfo *t, float vec[3], float pvec[3])
 
 	if (hasNumInput(&t->num)) {
 		applyNumInput(&t->num, vec);
-		removeAspectRatio(t, vec);
 		constraintNumInput(t, vec);
+		removeAspectRatio(t, vec);
 	}
 
 	/* autovalues is operator param, use that directly but not if snapping is forced */
@@ -185,7 +180,7 @@ static void viewAxisCorrectCenter(TransInfo *t, float t_con_center[3])
 {
 	if (t->spacetype == SPACE_VIEW3D) {
 		// View3D *v3d = t->sa->spacedata.first;
-		const float min_dist= 1.0f; // v3d->near;
+		const float min_dist = 1.0f;  /* v3d->near; */
 		float dir[3];
 		float l;
 
@@ -195,7 +190,7 @@ static void viewAxisCorrectCenter(TransInfo *t, float t_con_center[3])
 		}
 		project_v3_v3v3(dir, dir, t->viewinv[2]);
 
-		l= len_v3(dir);
+		l = len_v3(dir);
 
 		if (l < min_dist) {
 			float diff[3];
@@ -207,13 +202,14 @@ static void viewAxisCorrectCenter(TransInfo *t, float t_con_center[3])
 	}
 }
 
-static void axisProjection(TransInfo *t, float axis[3], float in[3], float out[3])
+static void axisProjection(TransInfo *t, const float axis[3], const float in[3], float out[3])
 {
 	float norm[3], vec[3], factor, angle;
 	float t_con_center[3];
 
-	if (in[0]==0.0f && in[1]==0.0f && in[2]==0.0f)
+	if (is_zero_v3(in)) {
 		return;
+	}
 
 	copy_v3_v3(t_con_center, t->con.center);
 
@@ -233,12 +229,12 @@ static void axisProjection(TransInfo *t, float axis[3], float in[3], float out[3
 		project_v3_v3v3(vec, in, t->viewinv[1]);
 		factor = dot_v3v3(t->viewinv[1], vec) * 2.0f;
 		/* since camera distance is quite relative, use quadratic relationship. holding shift can compensate */
-		if (factor<0.0f) factor*= -factor;
-		else factor*= factor;
+		if (factor < 0.0f) factor *= -factor;
+		else factor *= factor;
 
 		copy_v3_v3(out, axis);
 		normalize_v3(out);
-		mul_v3_fl(out, -factor);	/* -factor makes move down going backwards */
+		mul_v3_fl(out, -factor);  /* -factor makes move down going backwards */
 	}
 	else {
 		float v[3], i1[3], i2[3];
@@ -278,14 +274,14 @@ static void axisProjection(TransInfo *t, float axis[3], float in[3], float out[3
 
 			/* possible some values become nan when
 			 * viewpoint and object are both zero */
-			if (!finite(out[0])) out[0]= 0.0f;
-			if (!finite(out[1])) out[1]= 0.0f;
-			if (!finite(out[2])) out[2]= 0.0f;
+			if (!finite(out[0])) out[0] = 0.0f;
+			if (!finite(out[1])) out[1] = 0.0f;
+			if (!finite(out[2])) out[2] = 0.0f;
 		}
 	}
 }
 
-static void planeProjection(TransInfo *t, float in[3], float out[3])
+static void planeProjection(TransInfo *t, const float in[3], float out[3])
 {
 	float vec[3], factor, norm[3];
 
@@ -295,7 +291,7 @@ static void planeProjection(TransInfo *t, float in[3], float out[3])
 	sub_v3_v3v3(vec, out, in);
 
 	factor = dot_v3v3(vec, norm);
-	if (fabs(factor) <= 0.001) {
+	if (fabsf(factor) <= 0.001f) {
 		return; /* prevent divide by zero */
 	}
 	factor = dot_v3v3(vec, vec) / factor;
@@ -307,7 +303,7 @@ static void planeProjection(TransInfo *t, float in[3], float out[3])
 }
 
 /*
- * Generic callback for constant spacial constraints applied to linear motion
+ * Generic callback for constant spatial constraints applied to linear motion
  *
  * The IN vector in projected into the constrained space and then further
  * projected along the view vector.
@@ -315,7 +311,7 @@ static void planeProjection(TransInfo *t, float in[3], float out[3])
  *
  */
 
-static void applyAxisConstraintVec(TransInfo *t, TransData *td, float in[3], float out[3], float pvec[3])
+static void applyAxisConstraintVec(TransInfo *t, TransData *td, const float in[3], float out[3], float pvec[3])
 {
 	copy_v3_v3(out, in);
 	if (!td && t->con.mode & CON_APPLY) {
@@ -358,7 +354,7 @@ static void applyAxisConstraintVec(TransInfo *t, TransData *td, float in[3], flo
  * Further down, that vector is mapped to each data's space.
  */
 
-static void applyObjectConstraintVec(TransInfo *t, TransData *td, float in[3], float out[3], float pvec[3])
+static void applyObjectConstraintVec(TransInfo *t, TransData *td, const float in[3], float out[3], float pvec[3])
 {
 	copy_v3_v3(out, in);
 	if (t->con.mode & CON_APPLY) {
@@ -387,7 +383,7 @@ static void applyObjectConstraintVec(TransInfo *t, TransData *td, float in[3], f
 			copy_v3_v3(out, pvec);
 		}
 		else {
-			int i=0;
+			int i = 0;
 
 			out[0] = out[1] = out[2] = 0.0f;
 			if (t->con.mode & CON_AXIS0) {
@@ -405,9 +401,7 @@ static void applyObjectConstraintVec(TransInfo *t, TransData *td, float in[3], f
 }
 
 /*
- * Generic callback for constant spacial constraints applied to resize motion
- *
- *
+ * Generic callback for constant spatial constraints applied to resize motion
  */
 
 static void applyAxisConstraintSize(TransInfo *t, TransData *td, float smat[3][3])
@@ -431,9 +425,7 @@ static void applyAxisConstraintSize(TransInfo *t, TransData *td, float smat[3][3
 }
 
 /*
- * Callback for object based spacial constraints applied to resize motion
- *
- *
+ * Callback for object based spatial constraints applied to resize motion
  */
 
 static void applyObjectConstraintSize(TransInfo *t, TransData *td, float smat[3][3])
@@ -460,7 +452,7 @@ static void applyObjectConstraintSize(TransInfo *t, TransData *td, float smat[3]
 }
 
 /*
- * Generic callback for constant spacial constraints applied to rotations
+ * Generic callback for constant spatial constraints applied to rotations
  *
  * The rotation axis is copied into VEC.
  *
@@ -476,21 +468,21 @@ static void applyObjectConstraintSize(TransInfo *t, TransData *td, float smat[3]
 static void applyAxisConstraintRot(TransInfo *t, TransData *td, float vec[3], float *angle)
 {
 	if (!td && t->con.mode & CON_APPLY) {
-		int mode = t->con.mode & (CON_AXIS0|CON_AXIS1|CON_AXIS2);
+		int mode = t->con.mode & (CON_AXIS0 | CON_AXIS1 | CON_AXIS2);
 
-		switch(mode) {
-		case CON_AXIS0:
-		case (CON_AXIS1|CON_AXIS2):
-			copy_v3_v3(vec, t->con.mtx[0]);
-			break;
-		case CON_AXIS1:
-		case (CON_AXIS0|CON_AXIS2):
-			copy_v3_v3(vec, t->con.mtx[1]);
-			break;
-		case CON_AXIS2:
-		case (CON_AXIS0|CON_AXIS1):
-			copy_v3_v3(vec, t->con.mtx[2]);
-			break;
+		switch (mode) {
+			case CON_AXIS0:
+			case (CON_AXIS1 | CON_AXIS2):
+				copy_v3_v3(vec, t->con.mtx[0]);
+				break;
+			case CON_AXIS1:
+			case (CON_AXIS0 | CON_AXIS2):
+				copy_v3_v3(vec, t->con.mtx[1]);
+				break;
+			case CON_AXIS2:
+			case (CON_AXIS0 | CON_AXIS1):
+				copy_v3_v3(vec, t->con.mtx[2]);
+				break;
 		}
 		/* don't flip axis if asked to or if num input */
 		if (angle && (mode & CON_NOFLIP) == 0 && hasNumInput(&t->num) == 0) {
@@ -502,7 +494,7 @@ static void applyAxisConstraintRot(TransInfo *t, TransData *td, float vec[3], fl
 }
 
 /*
- * Callback for object based spacial constraints applied to rotations
+ * Callback for object based spatial constraints applied to rotations
  *
  * The rotation axis is copied into VEC.
  *
@@ -518,26 +510,26 @@ static void applyAxisConstraintRot(TransInfo *t, TransData *td, float vec[3], fl
 static void applyObjectConstraintRot(TransInfo *t, TransData *td, float vec[3], float *angle)
 {
 	if (t->con.mode & CON_APPLY) {
-		int mode = t->con.mode & (CON_AXIS0|CON_AXIS1|CON_AXIS2);
+		int mode = t->con.mode & (CON_AXIS0 | CON_AXIS1 | CON_AXIS2);
 
 		/* on setup call, use first object */
 		if (td == NULL) {
-			td= t->data;
+			td = t->data;
 		}
 
-		switch(mode) {
-		case CON_AXIS0:
-		case (CON_AXIS1|CON_AXIS2):
-			copy_v3_v3(vec, td->axismtx[0]);
-			break;
-		case CON_AXIS1:
-		case (CON_AXIS0|CON_AXIS2):
-			copy_v3_v3(vec, td->axismtx[1]);
-			break;
-		case CON_AXIS2:
-		case (CON_AXIS0|CON_AXIS1):
-			copy_v3_v3(vec, td->axismtx[2]);
-			break;
+		switch (mode) {
+			case CON_AXIS0:
+			case (CON_AXIS1 | CON_AXIS2):
+				copy_v3_v3(vec, td->axismtx[0]);
+				break;
+			case CON_AXIS1:
+			case (CON_AXIS0 | CON_AXIS2):
+				copy_v3_v3(vec, td->axismtx[1]);
+				break;
+			case CON_AXIS2:
+			case (CON_AXIS0 | CON_AXIS1):
+				copy_v3_v3(vec, td->axismtx[2]);
+				break;
 		}
 		if (angle && (mode & CON_NOFLIP) == 0 && hasNumInput(&t->num) == 0) {
 			if (dot_v3v3(vec, t->viewinv[2]) > 0.0f) {
@@ -604,34 +596,34 @@ void setUserConstraint(TransInfo *t, short orientation, int mode, const char fte
 {
 	char text[40];
 
-	switch(orientation) {
-	case V3D_MANIP_GLOBAL:
+	switch (orientation) {
+		case V3D_MANIP_GLOBAL:
 		{
-			float mtx[3][3]= MAT3_UNITY;
-			BLI_snprintf(text, sizeof(text), ftext, "global");
+			float mtx[3][3] = MAT3_UNITY;
+			BLI_snprintf(text, sizeof(text), ftext, IFACE_("global"));
 			setConstraint(t, mtx, mode, text);
 		}
 		break;
-	case V3D_MANIP_LOCAL:
-		BLI_snprintf(text, sizeof(text), ftext, "local");
-		setLocalConstraint(t, mode, text);
-		break;
-	case V3D_MANIP_NORMAL:
-		BLI_snprintf(text, sizeof(text), ftext, "normal");
-		setConstraint(t, t->spacemtx, mode, text);
-		break;
-	case V3D_MANIP_VIEW:
-		BLI_snprintf(text, sizeof(text), ftext, "view");
-		setConstraint(t, t->spacemtx, mode, text);
-		break;
-	case V3D_MANIP_GIMBAL:
-		BLI_snprintf(text, sizeof(text), ftext, "gimbal");
-		setConstraint(t, t->spacemtx, mode, text);
-		break;
-	default: /* V3D_MANIP_CUSTOM */
-		BLI_snprintf(text, sizeof(text), ftext, t->spacename);
-		setConstraint(t, t->spacemtx, mode, text);
-		break;
+		case V3D_MANIP_LOCAL:
+			BLI_snprintf(text, sizeof(text), ftext, IFACE_("local"));
+			setLocalConstraint(t, mode, text);
+			break;
+		case V3D_MANIP_NORMAL:
+			BLI_snprintf(text, sizeof(text), ftext, IFACE_("normal"));
+			setConstraint(t, t->spacemtx, mode, text);
+			break;
+		case V3D_MANIP_VIEW:
+			BLI_snprintf(text, sizeof(text), ftext, IFACE_("view"));
+			setConstraint(t, t->spacemtx, mode, text);
+			break;
+		case V3D_MANIP_GIMBAL:
+			BLI_snprintf(text, sizeof(text), ftext, IFACE_("gimbal"));
+			setConstraint(t, t->spacemtx, mode, text);
+			break;
+		default: /* V3D_MANIP_CUSTOM */
+			BLI_snprintf(text, sizeof(text), ftext, t->spacename);
+			setConstraint(t, t->spacemtx, mode, text);
+			break;
 	}
 
 	t->con.orientation = orientation;
@@ -645,7 +637,7 @@ void drawConstraint(TransInfo *t)
 {
 	TransCon *tc = &(t->con);
 
-	if (!ELEM(t->spacetype, SPACE_VIEW3D, SPACE_IMAGE))
+	if (!ELEM3(t->spacetype, SPACE_VIEW3D, SPACE_IMAGE, SPACE_NODE))
 		return;
 	if (!(tc->mode & CON_APPLY))
 		return;
@@ -665,7 +657,7 @@ void drawConstraint(TransInfo *t)
 	else {
 		if (tc->mode & CON_SELECT) {
 			float vec[3];
-			char col2[3] = {255,255,255};
+			char col2[3] = {255, 255, 255};
 			int depth_test_enabled;
 
 			convertViewVec(t, vec, (t->mval[0] - t->con.imval[0]), (t->mval[1] - t->con.imval[1]));
@@ -683,8 +675,8 @@ void drawConstraint(TransInfo *t)
 
 			setlinestyle(1);
 			glBegin(GL_LINE_STRIP);
-				glVertex3fv(tc->center);
-				glVertex3fv(vec);
+			glVertex3fv(tc->center);
+			glVertex3fv(vec);
 			glEnd();
 			setlinestyle(0);
 
@@ -733,8 +725,14 @@ void drawPropCircle(const struct bContext *C, TransInfo *t)
 		else if (t->spacetype == SPACE_IMAGE) {
 			float aspx, aspy;
 
-			ED_space_image_uv_aspect(t->sa->spacedata.first, &aspx, &aspy);
-			glScalef(1.0f/aspx, 1.0f/aspy, 1.0);
+			if (t->options & CTX_MASK) {
+				/* untested - mask aspect is TODO */
+				ED_space_image_get_aspect(t->sa->spacedata.first, &aspx, &aspy);
+			}
+			else {
+				ED_space_image_get_uv_aspect(t->sa->spacedata.first, &aspx, &aspy);
+			}
+			glScalef(1.0f / aspx, 1.0f / aspy, 1.0);
 		}
 
 		set_inverted_drawing(1);
@@ -748,7 +746,7 @@ void drawPropCircle(const struct bContext *C, TransInfo *t)
 static void drawObjectConstraint(TransInfo *t)
 {
 	int i;
-	TransData * td = t->data;
+	TransData *td = t->data;
 
 	/* Draw the first one lighter because that's the one who controls the others.
 	 * Meaning the transformation is projected on that one and just copied on the others
@@ -768,7 +766,7 @@ static void drawObjectConstraint(TransInfo *t)
 
 	td++;
 
-	for (i=1;i<t->total;i++,td++) {
+	for (i = 1; i < t->total; i++, td++) {
 		if (t->con.mode & CON_AXIS0) {
 			drawLine(t, td->ob->obmat[3], td->axismtx[0], 'X', 0);
 		}
@@ -787,12 +785,12 @@ void startConstraint(TransInfo *t)
 {
 	t->con.mode |= CON_APPLY;
 	*t->con.text = ' ';
-	t->num.idx_max = MIN2(getConstraintSpaceDimension(t) - 1, t->idx_max);
+	t->num.idx_max = min_ii(getConstraintSpaceDimension(t) - 1, t->idx_max);
 }
 
 void stopConstraint(TransInfo *t)
 {
-	t->con.mode &= ~(CON_APPLY|CON_SELECT);
+	t->con.mode &= ~(CON_APPLY | CON_SELECT);
 	*t->con.text = '\0';
 	t->num.idx_max = t->idx_max;
 }
@@ -804,21 +802,21 @@ void getConstraintMatrix(TransInfo *t)
 	unit_m3(t->con.pmtx);
 
 	if (!(t->con.mode & CON_AXIS0)) {
-		t->con.pmtx[0][0]		=
-			t->con.pmtx[0][1]	=
-			t->con.pmtx[0][2]	= 0.0f;
+		t->con.pmtx[0][0]       =
+		    t->con.pmtx[0][1]   =
+		    t->con.pmtx[0][2]   = 0.0f;
 	}
 
 	if (!(t->con.mode & CON_AXIS1)) {
-		t->con.pmtx[1][0]		=
-			t->con.pmtx[1][1]	=
-			t->con.pmtx[1][2]	= 0.0f;
+		t->con.pmtx[1][0]       =
+		    t->con.pmtx[1][1]   =
+		    t->con.pmtx[1][2]   = 0.0f;
 	}
 
 	if (!(t->con.mode & CON_AXIS2)) {
-		t->con.pmtx[2][0]		=
-			t->con.pmtx[2][1]	=
-			t->con.pmtx[2][2]	= 0.0f;
+		t->con.pmtx[2][0]       =
+		    t->con.pmtx[2][1]   =
+		    t->con.pmtx[2][2]   = 0.0f;
 	}
 
 	mul_m3_m3m3(mat, t->con.pmtx, t->con.imtx);
@@ -867,13 +865,13 @@ void postSelectConstraint(TransInfo *t)
 static void setNearestAxis2d(TransInfo *t)
 {
 	/* no correction needed... just use whichever one is lower */
-	if ( abs(t->mval[0]-t->con.imval[0]) < abs(t->mval[1]-t->con.imval[1]) ) {
+	if (abs(t->mval[0] - t->con.imval[0]) < abs(t->mval[1] - t->con.imval[1]) ) {
 		t->con.mode |= CON_AXIS1;
-		BLI_snprintf(t->con.text, sizeof(t->con.text), " along Y axis");
+		BLI_snprintf(t->con.text, sizeof(t->con.text), IFACE_(" along Y axis"));
 	}
 	else {
 		t->con.mode |= CON_AXIS0;
-		BLI_snprintf(t->con.text, sizeof(t->con.text), " along X axis");
+		BLI_snprintf(t->con.text, sizeof(t->con.text), IFACE_(" along X axis"));
 	}
 }
 
@@ -896,10 +894,10 @@ static void setNearestAxis3d(TransInfo *t)
 	 * of two 2D points 30 pixels apart (that's the last factor in the formula) after
 	 * projecting them with window_to_3d_delta and then get the length of that vector.
 	 */
-	zfac= t->persmat[0][3]*t->center[0]+ t->persmat[1][3]*t->center[1]+ t->persmat[2][3]*t->center[2]+ t->persmat[3][3];
-	zfac = len_v3(t->persinv[0]) * 2.0f/t->ar->winx * zfac * 30.0f;
+	zfac = t->persmat[0][3] * t->center[0] + t->persmat[1][3] * t->center[1] + t->persmat[2][3] * t->center[2] + t->persmat[3][3];
+	zfac = len_v3(t->persinv[0]) * 2.0f / t->ar->winx * zfac * 30.0f;
 
-	for (i = 0; i<3; i++) {
+	for (i = 0; i < 3; i++) {
 		copy_v3_v3(axis, t->con.mtx[i]);
 
 		mul_v3_fl(axis, zfac);
@@ -923,32 +921,32 @@ static void setNearestAxis3d(TransInfo *t)
 
 	if (len[0] <= len[1] && len[0] <= len[2]) {
 		if (t->modifiers & MOD_CONSTRAINT_PLANE) {
-			t->con.mode |= (CON_AXIS1|CON_AXIS2);
-			BLI_snprintf(t->con.text, sizeof(t->con.text), " locking %s X axis", t->spacename);
+			t->con.mode |= (CON_AXIS1 | CON_AXIS2);
+			BLI_snprintf(t->con.text, sizeof(t->con.text), IFACE_(" locking %s X axis"), t->spacename);
 		}
 		else {
 			t->con.mode |= CON_AXIS0;
-			BLI_snprintf(t->con.text, sizeof(t->con.text), " along %s X axis", t->spacename);
+			BLI_snprintf(t->con.text, sizeof(t->con.text), IFACE_(" along %s X axis"), t->spacename);
 		}
 	}
 	else if (len[1] <= len[0] && len[1] <= len[2]) {
 		if (t->modifiers & MOD_CONSTRAINT_PLANE) {
-			t->con.mode |= (CON_AXIS0|CON_AXIS2);
-			BLI_snprintf(t->con.text, sizeof(t->con.text), " locking %s Y axis", t->spacename);
+			t->con.mode |= (CON_AXIS0 | CON_AXIS2);
+			BLI_snprintf(t->con.text, sizeof(t->con.text), IFACE_(" locking %s Y axis"), t->spacename);
 		}
 		else {
 			t->con.mode |= CON_AXIS1;
-			BLI_snprintf(t->con.text, sizeof(t->con.text), " along %s Y axis", t->spacename);
+			BLI_snprintf(t->con.text, sizeof(t->con.text), IFACE_(" along %s Y axis"), t->spacename);
 		}
 	}
 	else if (len[2] <= len[1] && len[2] <= len[0]) {
 		if (t->modifiers & MOD_CONSTRAINT_PLANE) {
-			t->con.mode |= (CON_AXIS0|CON_AXIS1);
-			BLI_snprintf(t->con.text, sizeof(t->con.text), " locking %s Z axis", t->spacename);
+			t->con.mode |= (CON_AXIS0 | CON_AXIS1);
+			BLI_snprintf(t->con.text, sizeof(t->con.text), IFACE_(" locking %s Z axis"), t->spacename);
 		}
 		else {
 			t->con.mode |= CON_AXIS2;
-			BLI_snprintf(t->con.text, sizeof(t->con.text), " along %s Z axis", t->spacename);
+			BLI_snprintf(t->con.text, sizeof(t->con.text), IFACE_(" along %s Z axis"), t->spacename);
 		}
 	}
 }
@@ -977,21 +975,21 @@ void setNearestAxis(TransInfo *t)
 
 char constraintModeToChar(TransInfo *t)
 {
-	if ((t->con.mode & CON_APPLY)==0) {
+	if ((t->con.mode & CON_APPLY) == 0) {
 		return '\0';
 	}
-	switch (t->con.mode & (CON_AXIS0|CON_AXIS1|CON_AXIS2)) {
-	case (CON_AXIS0):
-	case (CON_AXIS1|CON_AXIS2):
-		return 'X';
-	case (CON_AXIS1):
-	case (CON_AXIS0|CON_AXIS2):
-		return 'Y';
-	case (CON_AXIS2):
-	case (CON_AXIS0|CON_AXIS1):
-		return 'Z';
-	default:
-		return '\0';
+	switch (t->con.mode & (CON_AXIS0 | CON_AXIS1 | CON_AXIS2)) {
+		case (CON_AXIS0):
+		case (CON_AXIS1 | CON_AXIS2):
+			return 'X';
+		case (CON_AXIS1):
+		case (CON_AXIS0 | CON_AXIS2):
+			return 'Y';
+		case (CON_AXIS2):
+		case (CON_AXIS0 | CON_AXIS1):
+			return 'Z';
+		default:
+			return '\0';
 	}
 }
 
@@ -1000,13 +998,13 @@ int isLockConstraint(TransInfo *t)
 {
 	int mode = t->con.mode;
 
-	if ( (mode & (CON_AXIS0|CON_AXIS1)) == (CON_AXIS0|CON_AXIS1))
+	if ((mode & (CON_AXIS0 | CON_AXIS1)) == (CON_AXIS0 | CON_AXIS1))
 		return 1;
 
-	if ( (mode & (CON_AXIS1|CON_AXIS2)) == (CON_AXIS1|CON_AXIS2))
+	if ((mode & (CON_AXIS1 | CON_AXIS2)) == (CON_AXIS1 | CON_AXIS2))
 		return 1;
 
-	if ( (mode & (CON_AXIS0|CON_AXIS2)) == (CON_AXIS0|CON_AXIS2))
+	if ((mode & (CON_AXIS0 | CON_AXIS2)) == (CON_AXIS0 | CON_AXIS2))
 		return 1;
 
 	return 0;
