@@ -64,18 +64,18 @@ static void InputSpring(TransInfo *UNUSED(t), MouseInput *mi, const int mval[2],
 		/* calculate ratio for shiftkey pos, and for total, and blend these for precision */
 		dx = (float)(mi->center[0] - mi->precision_mval[0]);
 		dy = (float)(mi->center[1] - mi->precision_mval[1]);
-		ratio = (float)sqrt( dx*dx + dy*dy);
+		ratio = sqrtf(dx * dx + dy * dy);
 
-		dx= (float)(mi->center[0] - mval[0]);
-		dy= (float)(mi->center[1] - mval[1]);
-		precise_ratio = (float)sqrt( dx*dx + dy*dy);
+		dx = (float)(mi->center[0] - mval[0]);
+		dy = (float)(mi->center[1] - mval[1]);
+		precise_ratio = (float)sqrt(dx * dx + dy * dy);
 
 		ratio = (ratio + (precise_ratio - ratio) / 10.0f) / mi->factor;
 	}
 	else {
 		dx = (float)(mi->center[0] - mval[0]);
 		dy = (float)(mi->center[1] - mval[1]);
-		ratio = (float)sqrt( dx*dx + dy*dy) / mi->factor;
+		ratio = sqrtf(dx * dx + dy * dy) / mi->factor;
 	}
 
 	output[0] = ratio;
@@ -87,23 +87,23 @@ static void InputSpringFlip(TransInfo *t, MouseInput *mi, const int mval[2], flo
 
 	/* flip scale */
 	/* values can become really big when zoomed in so use longs [#26598] */
-	if	((long long int)(mi->center[0] - mval[0]) * (long long int)(mi->center[0] - mi->imval[0]) +
-		 (long long int)(mi->center[1] - mval[1]) * (long long int)(mi->center[1] - mi->imval[1]) < 0)
-	 {
+	if ((long long int)(mi->center[0] - mval[0]) * (long long int)(mi->center[0] - mi->imval[0]) +
+	    (long long int)(mi->center[1] - mval[1]) * (long long int)(mi->center[1] - mi->imval[1]) < 0)
+	{
 		output[0] *= -1.0f;
-	 }
+	}
 }
 
 static void InputTrackBall(TransInfo *UNUSED(t), MouseInput *mi, const int mval[2], float output[3])
 {
 
 	if (mi->precision) {
-		output[0] = ( mi->imval[1] - mi->precision_mval[1] ) + ( mi->precision_mval[1] - mval[1] ) * 0.1f;
-		output[1] = ( mi->precision_mval[0] - mi->imval[0] ) + ( mval[0] - mi->precision_mval[0] ) * 0.1f;
+		output[0] = (mi->imval[1] - mi->precision_mval[1]) + (mi->precision_mval[1] - mval[1]) * 0.1f;
+		output[1] = (mi->precision_mval[0] - mi->imval[0]) + (mval[0] - mi->precision_mval[0]) * 0.1f;
 	}
 	else {
-		output[0] = (float)( mi->imval[1] - mval[1] );
-		output[1] = (float)( mval[0] - mi->imval[0] );
+		output[0] = (float)(mi->imval[1] - mval[1]);
+		output[1] = (float)(mval[0] - mi->imval[0]);
 	}
 
 	output[0] *= mi->factor;
@@ -164,7 +164,7 @@ static void InputVerticalAbsolute(TransInfo *t, MouseInput *mi, const int mval[2
 	output[0] = dot_v3v3(t->viewinv[1], vec) * 2.0f;
 }
 
-void setCustomPoints(TransInfo *UNUSED(t), MouseInput *mi, int start[2], int end[2])
+void setCustomPoints(TransInfo *UNUSED(t), MouseInput *mi, const int start[2], const int end[2])
 {
 	int *data;
 
@@ -180,7 +180,7 @@ void setCustomPoints(TransInfo *UNUSED(t), MouseInput *mi, int start[2], int end
 	data[3] = end[1];
 }
 
-static void InputCustomRatio(TransInfo *UNUSED(t), MouseInput *mi, const int mval[2], float output[3])
+static void InputCustomRatioFlip(TransInfo *UNUSED(t), MouseInput *mi, const int mval[2], float output[3])
 {
 	double length;
 	double distance;
@@ -191,7 +191,7 @@ static void InputCustomRatio(TransInfo *UNUSED(t), MouseInput *mi, const int mva
 		dx = data[2] - data[0];
 		dy = data[3] - data[1];
 		
-		length = sqrt(dx*dx + dy*dy);
+		length = sqrt(dx * dx + dy * dy);
 		
 		if (mi->precision) {
 			/* deal with Shift key by adding motion / 10 to motion before shift press */
@@ -199,29 +199,35 @@ static void InputCustomRatio(TransInfo *UNUSED(t), MouseInput *mi, const int mva
 			mdx = (mi->precision_mval[0] + (float)(mval[0] - mi->precision_mval[0]) / 10.0f) - data[2];
 			mdy = (mi->precision_mval[1] + (float)(mval[1] - mi->precision_mval[1]) / 10.0f) - data[3];
 
-			distance = (length != 0.0f)? (mdx*dx + mdy*dy) / length: 0.0f;
+			distance = (length != 0.0) ? (mdx * dx + mdy * dy) / length : 0.0;
 		}
 		else {
 			int mdx, mdy;
 			mdx = mval[0] - data[2];
 			mdy = mval[1] - data[3];
 
-			distance = (length != 0.0f)? (mdx*dx + mdy*dy) / length: 0.0f;
+			distance = (length != 0.0) ? (mdx * dx + mdy * dy) / length : 0.0;
 		}
 
-		output[0] = (float)((length != 0.0f)? distance / length: 0.0f);
+		output[0] = (length != 0.0) ? (double)(distance / length) : 0.0;
 	}
 }
 
+static void InputCustomRatio(TransInfo *t, MouseInput *mi, const int mval[2], float output[3])
+{
+	InputCustomRatioFlip(t, mi, mval, output);
+	output[0] = -output[0];
+}
+
 static void InputAngle(TransInfo *UNUSED(t), MouseInput *mi, const int mval[2], float output[3])
 {
 	double dx2 = mval[0] - mi->center[0];
 	double dy2 = mval[1] - mi->center[1];
-	double B = sqrt(dx2*dx2+dy2*dy2);
+	double B = sqrt(dx2 * dx2 + dy2 * dy2);
 
 	double dx1 = mi->imval[0] - mi->center[0];
 	double dy1 = mi->imval[1] - mi->center[1];
-	double A = sqrt(dx1*dx1+dy1*dy1);
+	double A = sqrt(dx1 * dx1 + dy1 * dy1);
 
 	double dx3 = mval[0] - mi->imval[0];
 	double dy3 = mval[1] - mi->imval[1];
@@ -229,14 +235,15 @@ static void InputAngle(TransInfo *UNUSED(t), MouseInput *mi, const int mval[2],
 	double *angle = mi->data;
 
 	/* use doubles here, to make sure a "1.0" (no rotation) doesnt become 9.999999e-01, which gives 0.02 for acos */
-	double deler = ((dx1*dx1+dy1*dy1)+(dx2*dx2+dy2*dy2)-(dx3*dx3+dy3*dy3))
-		/ (2.0 * ((A*B)?(A*B):1.0));
-	/* ((A*B)?(A*B):1.0) this takes care of potential divide by zero errors */
+	double deler = (((dx1 * dx1 + dy1 * dy1) +
+	                 (dx2 * dx2 + dy2 * dy2) -
+	                 (dx3 * dx3 + dy3 * dy3)) / (2.0 * ((A * B) ? (A * B) : 1.0)));
+	/* ((A * B) ? (A * B) : 1.0) this takes care of potential divide by zero errors */
 
 	float dphi;
 
 	dphi = saacos((float)deler);
-	if ( (dx1*dy2-dx2*dy1)>0.0 ) dphi= -dphi;
+	if ((dx1 * dy2 - dx2 * dy1) > 0.0) dphi = -dphi;
 
 	/* If the angle is zero, because of lack of precision close to the 1.0 value in acos
 	 * approximate the angle with the opposite side of the normalized triangle
@@ -255,12 +262,12 @@ static void InputAngle(TransInfo *UNUSED(t), MouseInput *mi, const int mval[2],
 		dx = dx1 - dx2;
 		dy = dy1 - dy2;
 
-		dphi = sqrt(dx*dx + dy*dy);
-		if ( (dx1*dy2-dx2*dy1)>0.0 ) dphi= -dphi;
+		dphi = sqrt(dx * dx + dy * dy);
+		if ((dx1 * dy2 - dx2 * dy1) > 0.0) dphi = -dphi;
 	}
 
 	if (mi->precision) {
-		dphi = dphi/30.0f;
+		dphi = dphi / 30.0f;
 	}
 
 	/* if no delta angle, don't update initial position */
@@ -274,7 +281,7 @@ static void InputAngle(TransInfo *UNUSED(t), MouseInput *mi, const int mval[2],
 	output[0] = *angle;
 }
 
-void initMouseInput(TransInfo *UNUSED(t), MouseInput *mi, int center[2], int mval[2])
+void initMouseInput(TransInfo *UNUSED(t), MouseInput *mi, const int center[2], const int mval[2])
 {
 	mi->factor = 0;
 	mi->precision = 0;
@@ -290,88 +297,87 @@ void initMouseInput(TransInfo *UNUSED(t), MouseInput *mi, int center[2], int mva
 
 static void calcSpringFactor(MouseInput *mi)
 {
-	mi->factor = (float)sqrt(
-		(
-			((float)(mi->center[1] - mi->imval[1]))*((float)(mi->center[1] - mi->imval[1]))
-		+
-			((float)(mi->center[0] - mi->imval[0]))*((float)(mi->center[0] - mi->imval[0]))
-		) );
+	mi->factor = sqrtf(((float)(mi->center[1] - mi->imval[1])) * ((float)(mi->center[1] - mi->imval[1])) +
+	                   ((float)(mi->center[0] - mi->imval[0])) * ((float)(mi->center[0] - mi->imval[0])));
 
-	if (mi->factor==0.0f)
-		mi->factor= 1.0f; /* prevent Inf */
+	if (mi->factor == 0.0f) {
+		mi->factor = 1.0f; /* prevent Inf */
+	}
 }
 
 void initMouseInputMode(TransInfo *t, MouseInput *mi, MouseInputMode mode)
 {
-	/* may have been allocated previously */
-	/* TODO, holding R-key can cause mem leak, but this causes [#28903]
-	 * disable for now. */
-#if 0
-	if (mi->data) {
-		MEM_freeN(mi->data);
-		mi->data= NULL;
-	}
-#endif
+	/* incase we allocate a new value */
+	void *mi_data_prev = mi->data;
 
-	switch(mode)
-	{
-	case INPUT_VECTOR:
-		mi->apply = InputVector;
-		t->helpline = HLP_NONE;
-		break;
-	case INPUT_SPRING:
-		calcSpringFactor(mi);
-		mi->apply = InputSpring;
-		t->helpline = HLP_SPRING;
-		break;
-	case INPUT_SPRING_FLIP:
-		calcSpringFactor(mi);
-		mi->apply = InputSpringFlip;
-		t->helpline = HLP_SPRING;
-		break;
-	case INPUT_ANGLE:
-		mi->data = MEM_callocN(sizeof(double), "angle accumulator");
-		mi->apply = InputAngle;
-		t->helpline = HLP_ANGLE;
-		break;
-	case INPUT_TRACKBALL:
-		/* factor has to become setting or so */
-		mi->factor = 0.01f;
-		mi->apply = InputTrackBall;
-		t->helpline = HLP_TRACKBALL;
-		break;
-	case INPUT_HORIZONTAL_RATIO:
-		mi->factor = (float)(mi->center[0] - mi->imval[0]);
-		mi->apply = InputHorizontalRatio;
-		t->helpline = HLP_HARROW;
-		break;
-	case INPUT_HORIZONTAL_ABSOLUTE:
-		mi->apply = InputHorizontalAbsolute;
-		t->helpline = HLP_HARROW;
-		break;
-	case INPUT_VERTICAL_RATIO:
-		mi->apply = InputVerticalRatio;
-		t->helpline = HLP_VARROW;
-		break;
-	case INPUT_VERTICAL_ABSOLUTE:
-		mi->apply = InputVerticalAbsolute;
-		t->helpline = HLP_VARROW;
-		break;
-	case INPUT_CUSTOM_RATIO:
-		mi->apply = InputCustomRatio;
-		t->helpline = HLP_NONE;
-		break;
-	case INPUT_NONE:
-	default:
-		mi->apply = NULL;
-		break;
+	switch (mode) {
+		case INPUT_VECTOR:
+			mi->apply = InputVector;
+			t->helpline = HLP_NONE;
+			break;
+		case INPUT_SPRING:
+			calcSpringFactor(mi);
+			mi->apply = InputSpring;
+			t->helpline = HLP_SPRING;
+			break;
+		case INPUT_SPRING_FLIP:
+			calcSpringFactor(mi);
+			mi->apply = InputSpringFlip;
+			t->helpline = HLP_SPRING;
+			break;
+		case INPUT_ANGLE:
+			mi->data = MEM_callocN(sizeof(double), "angle accumulator");
+			mi->apply = InputAngle;
+			t->helpline = HLP_ANGLE;
+			break;
+		case INPUT_TRACKBALL:
+			/* factor has to become setting or so */
+			mi->factor = 0.01f;
+			mi->apply = InputTrackBall;
+			t->helpline = HLP_TRACKBALL;
+			break;
+		case INPUT_HORIZONTAL_RATIO:
+			mi->factor = (float)(mi->center[0] - mi->imval[0]);
+			mi->apply = InputHorizontalRatio;
+			t->helpline = HLP_HARROW;
+			break;
+		case INPUT_HORIZONTAL_ABSOLUTE:
+			mi->apply = InputHorizontalAbsolute;
+			t->helpline = HLP_HARROW;
+			break;
+		case INPUT_VERTICAL_RATIO:
+			mi->apply = InputVerticalRatio;
+			t->helpline = HLP_VARROW;
+			break;
+		case INPUT_VERTICAL_ABSOLUTE:
+			mi->apply = InputVerticalAbsolute;
+			t->helpline = HLP_VARROW;
+			break;
+		case INPUT_CUSTOM_RATIO:
+			mi->apply = InputCustomRatio;
+			t->helpline = HLP_NONE;
+			break;
+		case INPUT_CUSTOM_RATIO_FLIP:
+			mi->apply = InputCustomRatioFlip;
+			t->helpline = HLP_NONE;
+			break;
+		case INPUT_NONE:
+		default:
+			mi->apply = NULL;
+			break;
+	}
+
+	/* if we've allocated new data, free the old data
+	 * less hassle then checking before every alloc above */
+	if (mi_data_prev && (mi_data_prev != mi->data)) {
+		MEM_freeN(mi_data_prev);
 	}
 
 	/* bootstrap mouse input with initial values */
 	applyMouseInput(t, mi, mi->imval, t->values);
 }
 
-void setInputPostFct(MouseInput *mi, void	(*post)(struct TransInfo *, float [3]))
+void setInputPostFct(MouseInput *mi, void (*post)(struct TransInfo *t, float values[3]))
 {
 	mi->post = post;
 }
@@ -391,23 +397,23 @@ int handleMouseInput(TransInfo *t, MouseInput *mi, wmEvent *event)
 {
 	int redraw = TREDRAW_NOTHING;
 
-	switch (event->type)
-	{
-	case LEFTSHIFTKEY:
-	case RIGHTSHIFTKEY:
-		if (event->val == KM_PRESS) {
-			t->modifiers |= MOD_PRECISION;
-			/* shift is modifier for higher precision transform
-			 * store the mouse position where the normal movement ended */
-			copy_v2_v2_int(mi->precision_mval, event->mval);
-			mi->precision = 1;
-		}
-		else {
-			t->modifiers &= ~MOD_PRECISION;
-			mi->precision = 0;
-		}
-		redraw = TREDRAW_HARD;
-		break;
+	switch (event->type) {
+		case LEFTSHIFTKEY:
+		case RIGHTSHIFTKEY:
+			if (event->val == KM_PRESS) {
+				t->modifiers |= MOD_PRECISION;
+				/* shift is modifier for higher precision transform
+				 * store the mouse position where the normal movement ended */
+				copy_v2_v2_int(mi->precision_mval, event->mval);
+				mi->precision = 1;
+				redraw = TREDRAW_HARD;
+			}
+			else if (event->val == KM_RELEASE) {
+				t->modifiers &= ~MOD_PRECISION;
+				mi->precision = 0;
+				redraw = TREDRAW_HARD;
+			}
+			break;
 	}
 
 	return redraw;
