@@ -33,6 +33,7 @@
 #include "DNA_armature_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_mesh_types.h"
+#include "DNA_meta_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
@@ -42,6 +43,7 @@
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
 
+#include "BKE_action.h"
 #include "BKE_armature.h"
 #include "BKE_curve.h"
 #include "BKE_context.h"
@@ -50,11 +52,6 @@
 
 #include "BLF_translation.h"
 
-//#include "BIF_editmesh.h"
-//#include "BIF_interface.h"
-//#include "BIF_space.h"
-//#include "BIF_toolbox.h"
-
 #include "ED_armature.h"
 #include "ED_mesh.h"
 
@@ -75,16 +72,16 @@ void BIF_clearTransformOrientation(bContext *C)
 	
 	// Need to loop over all view3d
 	if (v3d && v3d->twmode >= V3D_MANIP_CUSTOM) {
-		v3d->twmode = V3D_MANIP_GLOBAL;	/* fallback to global	*/
+		v3d->twmode = V3D_MANIP_GLOBAL; /* fallback to global	*/
 	}
 }
 
-static TransformOrientation* findOrientationName(ListBase *lb, const char *name)
+static TransformOrientation *findOrientationName(ListBase *lb, const char *name)
 {
-	TransformOrientation *ts= NULL;
+	TransformOrientation *ts = NULL;
 
-	for (ts= lb->first; ts; ts = ts->next) {
-		if (strncmp(ts->name, name, sizeof(ts->name)-1) == 0) {
+	for (ts = lb->first; ts; ts = ts->next) {
+		if (strncmp(ts->name, name, sizeof(ts->name) - 1) == 0) {
 			return ts;
 		}
 	}
@@ -115,7 +112,7 @@ void BIF_createTransformOrientation(bContext *C, ReportList *reports, char *name
 			ts = createBoneSpace(C, reports, name, overwrite);
 	}
 	else if (ob && (ob->mode & OB_MODE_POSE)) {
-			ts = createBoneSpace(C, reports, name, overwrite);
+		ts = createBoneSpace(C, reports, name, overwrite);
 	}
 	else {
 		ts = createObjectSpace(C, reports, name, overwrite);
@@ -143,10 +140,10 @@ TransformOrientation *createObjectSpace(bContext *C, ReportList *UNUSED(reports)
 
 	/* use object name if no name is given */
 	if (name[0] == 0) {
-		strncpy(name, ob->id.name+2, MAX_ID_NAME-2);
+		strncpy(name, ob->id.name + 2, MAX_ID_NAME - 2);
 	}
 
-	return addMatrixSpace(C, mat, name, overwrite);	
+	return addMatrixSpace(C, mat, name, overwrite);
 }
 
 TransformOrientation *createBoneSpace(bContext *C, ReportList *reports, char *name, int overwrite)
@@ -176,8 +173,7 @@ TransformOrientation *createMeshSpace(bContext *C, ReportList *reports, char *na
 
 	type = getTransformOrientation(C, normal, plane, 0);
 	
-	switch (type)
-	{
+	switch (type) {
 		case ORIENTATION_VERT:
 			if (createSpaceNormal(mat, normal) == 0) {
 				BKE_reports_prepend(reports, "Cannot use vertex with zero-length normal");
@@ -263,7 +259,7 @@ int createSpaceNormalTangent(float mat[3][3], float normal[3], float tangent[3])
 	return 1;
 }
 
-TransformOrientation* addMatrixSpace(bContext *C, float mat[3][3], char name[], int overwrite)
+TransformOrientation *addMatrixSpace(bContext *C, float mat[3][3], char name[], int overwrite)
 {
 	ListBase *transform_spaces = &CTX_data_scene(C)->transform_spaces;
 	TransformOrientation *ts = NULL;
@@ -302,7 +298,7 @@ void BIF_removeTransformOrientation(bContext *C, TransformOrientation *target)
 				
 				// Transform_fix_me NEED TO DO THIS FOR ALL VIEW3D
 				if (selected_index == i) {
-					v3d->twmode = V3D_MANIP_GLOBAL;	/* fallback to global	*/
+					v3d->twmode = V3D_MANIP_GLOBAL; /* fallback to global	*/
 				}
 				else if (selected_index > i) {
 					v3d->twmode--;
@@ -319,7 +315,7 @@ void BIF_removeTransformOrientation(bContext *C, TransformOrientation *target)
 void BIF_removeTransformOrientationIndex(bContext *C, int index)
 {
 	ListBase *transform_spaces = &CTX_data_scene(C)->transform_spaces;
-	TransformOrientation *ts= BLI_findlink(transform_spaces, index);
+	TransformOrientation *ts = BLI_findlink(transform_spaces, index);
 
 	if (ts) {
 		View3D *v3d = CTX_wm_view3d(C);
@@ -328,7 +324,7 @@ void BIF_removeTransformOrientationIndex(bContext *C, int index)
 			
 			// Transform_fix_me NEED TO DO THIS FOR ALL VIEW3D
 			if (selected_index == index) {
-				v3d->twmode = V3D_MANIP_GLOBAL;	/* fallback to global	*/
+				v3d->twmode = V3D_MANIP_GLOBAL; /* fallback to global	*/
 			}
 			else if (selected_index > index) {
 				v3d->twmode--;
@@ -366,15 +362,15 @@ EnumPropertyItem *BIF_enumTransformOrientation(bContext *C)
 {
 	Scene *scene;
 	ListBase *transform_spaces;
-	TransformOrientation *ts= NULL;
+	TransformOrientation *ts = NULL;
 
-	EnumPropertyItem global	= {V3D_MANIP_GLOBAL, "GLOBAL", 0, "Global", ""};
+	EnumPropertyItem global = {V3D_MANIP_GLOBAL, "GLOBAL", 0, "Global", ""};
 	EnumPropertyItem normal = {V3D_MANIP_NORMAL, "NORMAL", 0, "Normal", ""};
 	EnumPropertyItem local = {V3D_MANIP_LOCAL, "LOCAL", 0, "Local", ""};
 	EnumPropertyItem view = {V3D_MANIP_VIEW, "VIEW", 0, "View", ""};
 	EnumPropertyItem tmp = {0, "", 0, "", ""};
-	EnumPropertyItem *item= NULL;
-	int i = V3D_MANIP_CUSTOM, totitem= 0;
+	EnumPropertyItem *item = NULL;
+	int i = V3D_MANIP_CUSTOM, totitem = 0;
 
 	RNA_enum_item_add(&item, &totitem, &global);
 	RNA_enum_item_add(&item, &totitem, &normal);
@@ -382,7 +378,7 @@ EnumPropertyItem *BIF_enumTransformOrientation(bContext *C)
 	RNA_enum_item_add(&item, &totitem, &view);
 
 	if (C) {
-		scene= CTX_data_scene(C);
+		scene = CTX_data_scene(C);
 
 		if (scene) {
 			transform_spaces = &scene->transform_spaces;
@@ -395,7 +391,7 @@ EnumPropertyItem *BIF_enumTransformOrientation(bContext *C)
 
 	for (; ts; ts = ts->next) {
 		tmp.identifier = "CUSTOM";
-		tmp.name= ts->name;
+		tmp.name = ts->name;
 		tmp.value = i++;
 		RNA_enum_item_add(&item, &totitem, &tmp);
 	}
@@ -405,9 +401,9 @@ EnumPropertyItem *BIF_enumTransformOrientation(bContext *C)
 	return item;
 }
 
-const char * BIF_menustringTransformOrientation(const bContext *C, const char *title)
+const char *BIF_menustringTransformOrientation(const bContext *C, const char *title)
 {
-	const char* menu = IFACE_("%t|Global%x0|Local%x1|Gimbal%x4|Normal%x2|View%x3");
+	const char *menu = IFACE_("%t|Global %x0|Local %x1|Gimbal %x4|Normal %x2|View %x3");
 	ListBase *transform_spaces = &CTX_data_scene(C)->transform_spaces;
 	TransformOrientation *ts;
 	int i = V3D_MANIP_CUSTOM;
@@ -416,14 +412,14 @@ const char * BIF_menustringTransformOrientation(const bContext *C, const char *t
 
 	title = IFACE_(title);
 
-	str_menu = MEM_callocN(strlen(menu) + strlen(title) + 1 + elem_size * BIF_countTransformOrientation(C), TIP_("UserTransSpace from matrix"));
+	str_menu = MEM_callocN(strlen(menu) + strlen(title) + 1 + elem_size * BIF_countTransformOrientation(C), "UserTransSpace from matrix");
 	p = str_menu;
 	
 	p += sprintf(str_menu, "%s", title);
 	p += sprintf(p, "%s", menu);
 	
 	for (ts = transform_spaces->first; ts; ts = ts->next) {
-		p += sprintf(p, "|%s%%x%d", ts->name, i++);
+		p += sprintf(p, "|%s %%x%d", ts->name, i++);
 	}
 	
 	return str_menu;
@@ -442,7 +438,7 @@ int BIF_countTransformOrientation(const bContext *C)
 	return count;
 }
 
-void applyTransformOrientation(const bContext *C, float mat[3][3], char *name)
+void applyTransformOrientation(const bContext *C, float mat[3][3], char name[MAX_NAME])
 {
 	TransformOrientation *ts;
 	View3D *v3d = CTX_wm_view3d(C);
@@ -453,14 +449,15 @@ void applyTransformOrientation(const bContext *C, float mat[3][3], char *name)
 		for (i = 0, ts = CTX_data_scene(C)->transform_spaces.first; ts; ts = ts->next, i++) {
 			if (selected_index == i) {
 				
-				if (name)
-					strcpy(name, ts->name);
+				if (name) {
+					BLI_strncpy(name, ts->name, MAX_NAME);
+				}
 				
 				copy_m3_m3(mat, ts->mat);
 				break;
 			}
 		}
-	  }
+	}
 }
 
 static int count_bone_select(bArmature *arm, ListBase *lb, int do_it) 
@@ -469,7 +466,7 @@ static int count_bone_select(bArmature *arm, ListBase *lb, int do_it)
 	int do_next;
 	int total = 0;
 	
-	for (bone= lb->first; bone; bone= bone->next) {
+	for (bone = lb->first; bone; bone = bone->next) {
 		bone->flag &= ~BONE_TRANSFORM;
 		do_next = do_it;
 		if (do_it) {
@@ -477,7 +474,7 @@ static int count_bone_select(bArmature *arm, ListBase *lb, int do_it)
 				if (bone->flag & BONE_SELECTED) {
 					bone->flag |= BONE_TRANSFORM;
 					total++;
-					do_next= 0;	// no transform on children if one parent bone is selected
+					do_next = FALSE;    // no transform on children if one parent bone is selected
 				}
 			}
 		}
@@ -493,56 +490,56 @@ void initTransformOrientation(bContext *C, TransInfo *t)
 	Object *ob = CTX_data_active_object(C);
 	Object *obedit = CTX_data_active_object(C);
 
-	switch(t->current_orientation) {
-	case V3D_MANIP_GLOBAL:
-		unit_m3(t->spacemtx);
-		strcpy(t->spacename, "global");
-		break;
-
-	case V3D_MANIP_GIMBAL:
-		unit_m3(t->spacemtx);
-		if (gimbal_axis(ob, t->spacemtx)) {
-			strcpy(t->spacename, "gimbal");
+	switch (t->current_orientation) {
+		case V3D_MANIP_GLOBAL:
+			unit_m3(t->spacemtx);
+			strcpy(t->spacename, IFACE_("global"));
 			break;
-		}
+
+		case V3D_MANIP_GIMBAL:
+			unit_m3(t->spacemtx);
+			if (gimbal_axis(ob, t->spacemtx)) {
+				strcpy(t->spacename, IFACE_("gimbal"));
+				break;
+			}
 		/* no gimbal fallthrough to normal */
-	case V3D_MANIP_NORMAL:
-		if (obedit || (ob && ob->mode & OB_MODE_POSE)) {
-			strcpy(t->spacename, "normal");
-			ED_getTransformOrientationMatrix(C, t->spacemtx, (v3d->around == V3D_ACTIVE));
-			break;
-		}
+		case V3D_MANIP_NORMAL:
+			if (obedit || (ob && ob->mode & OB_MODE_POSE)) {
+				strcpy(t->spacename, IFACE_("normal"));
+				ED_getTransformOrientationMatrix(C, t->spacemtx, (v3d->around == V3D_ACTIVE));
+				break;
+			}
 		/* no break we define 'normal' as 'local' in Object mode */
-	case V3D_MANIP_LOCAL:
-		strcpy(t->spacename, "local");
+		case V3D_MANIP_LOCAL:
+			strcpy(t->spacename, IFACE_("local"));
 		
-		if (ob) {
-			copy_m3_m4(t->spacemtx, ob->obmat);
-			normalize_m3(t->spacemtx);
-		}
-		else {
-			unit_m3(t->spacemtx);
-		}
+			if (ob) {
+				copy_m3_m4(t->spacemtx, ob->obmat);
+				normalize_m3(t->spacemtx);
+			}
+			else {
+				unit_m3(t->spacemtx);
+			}
 		
-		break;
+			break;
 		
-	case V3D_MANIP_VIEW:
-		if (t->ar->regiontype == RGN_TYPE_WINDOW) {
-			RegionView3D *rv3d = t->ar->regiondata;
-			float mat[3][3];
+		case V3D_MANIP_VIEW:
+			if (t->ar->regiontype == RGN_TYPE_WINDOW) {
+				RegionView3D *rv3d = t->ar->regiondata;
+				float mat[3][3];
 
-			strcpy(t->spacename, "view");
-			copy_m3_m4(mat, rv3d->viewinv);
-			normalize_m3(mat);
-			copy_m3_m3(t->spacemtx, mat);
-		}
-		else {
-			unit_m3(t->spacemtx);
-		}
-		break;
-	default: /* V3D_MANIP_CUSTOM */
-		applyTransformOrientation(C, t->spacemtx, t->spacename);
-		break;
+				strcpy(t->spacename, IFACE_("view"));
+				copy_m3_m4(mat, rv3d->viewinv);
+				normalize_m3(mat);
+				copy_m3_m3(t->spacemtx, mat);
+			}
+			else {
+				unit_m3(t->spacemtx);
+			}
+			break;
+		default: /* V3D_MANIP_CUSTOM */
+			applyTransformOrientation(C, t->spacemtx, t->spacename);
+			break;
 	}
 }
 
@@ -550,13 +547,13 @@ int getTransformOrientation(const bContext *C, float normal[3], float plane[3],
 {
 	Scene *scene = CTX_data_scene(C);
 	View3D *v3d = CTX_wm_view3d(C);
-	Object *obedit= CTX_data_edit_object(C);
+	Object *obedit = CTX_data_edit_object(C);
 	Base *base;
 	Object *ob = OBACT;
 	int result = ORIENTATION_NONE;
 
-	normal[0] = normal[1] = normal[2] = 0;
-	plane[0] = plane[1] = plane[2] = 0;
+	zero_v3(normal);
+	zero_v3(plane);
 
 	if (obedit) {
 		float imat[3][3], mat[3][3];
@@ -567,22 +564,20 @@ int getTransformOrientation(const bContext *C, float normal[3], float plane[3],
 		invert_m3_m3(mat, imat);
 		transpose_m3(mat);
 
-		ob= obedit;
+		ob = obedit;
 
-		if (ob->type==OB_MESH) {
-			Mesh *me= ob->data;
-			BMEditMesh *em = me->edit_btmesh;
+		if (ob->type == OB_MESH) {
+			BMEditMesh *em = BMEdit_FromObject(ob);
 			BMVert *eve;
 			BMEditSelection ese;
-			float vec[3]= {0,0,0};
+			float vec[3] = {0, 0, 0};
 			
 			/* USE LAST SELECTED WITH ACTIVE */
 			if (activeOnly && BM_select_history_active_get(em->bm, &ese)) {
 				BM_editselection_normal(&ese, normal);
 				BM_editselection_plane(&ese, plane);
 				
-				switch (ese.htype)
-				{
+				switch (ese.htype) {
 					case BM_VERT:
 						result = ORIENTATION_VERT;
 						break;
@@ -638,7 +633,6 @@ int getTransformOrientation(const bContext *C, float normal[3], float plane[3],
 					/* if there's an edge available, use that for the tangent */
 					if (em->bm->totedgesel >= 1) {
 						BMEdge *eed = NULL;
-						BMIter iter;
 						
 						BM_ITER_MESH (eed, &iter, em->bm, BM_EDGES_OF_MESH) {
 							if (BM_elem_flag_test(eed, BM_ELEM_SELECT)) {
@@ -713,19 +707,18 @@ int getTransformOrientation(const bContext *C, float normal[3], float plane[3],
 			}
 		} /* end editmesh */
 		else if (ELEM(obedit->type, OB_CURVE, OB_SURF)) {
-			Curve *cu= obedit->data;
+			Curve *cu = obedit->data;
 			Nurb *nu;
 			BezTriple *bezt;
 			int a;
-			ListBase *nurbs= curve_editnurbs(cu);
+			ListBase *nurbs = BKE_curve_editNurbs_get(cu);
 
 			for (nu = nurbs->first; nu; nu = nu->next) {
 				/* only bezier has a normal */
 				if (nu->type == CU_BEZIER) {
-					bezt= nu->bezt;
-					a= nu->pntsu;
-					while (a--)
-					{
+					bezt = nu->bezt;
+					a = nu->pntsu;
+					while (a--) {
 						/* exception */
 						if ((bezt->f1 & SELECT) + (bezt->f2 & SELECT) + (bezt->f3 & SELECT) > SELECT) {
 							sub_v3_v3v3(normal, bezt->vec[0], bezt->vec[2]);
@@ -746,69 +739,68 @@ int getTransformOrientation(const bContext *C, float normal[3], float plane[3],
 				}
 			}
 			
-			if (normal[0] != 0 || normal[1] != 0 || normal[2] != 0) {
+			if (!is_zero_v3(normal)) {
 				result = ORIENTATION_NORMAL;
 			}
 		}
-		else if (obedit->type==OB_MBALL) {
-#if 0 // XXX
-			/* editmball.c */
-			MetaElem *ml, *ml_sel = NULL;
-	
-			/* loop and check that only one element is selected */	
-			for (ml = editelems.first; ml; ml = ml->next) {
-				if (ml->flag & SELECT) {
-					if (ml_sel == NULL) {
-						ml_sel = ml;
-					}
-					else {
-						ml_sel = NULL;
-						break;
-					}
-				}
-			}
+		else if (obedit->type == OB_MBALL) {
+			MetaBall *mb = obedit->data;
 			
-			if (ml_sel) {	
-				float mat[4][4];
+			if (mb->lastelem) {
+				float qmat[3][3];
 
 				/* Rotation of MetaElem is stored in quat */
-				quat_to_mat4( mat,ml_sel->quat);
+				quat_to_mat3(qmat, mb->lastelem->quat);
 
-				copy_v3_v3(normal, mat[2]);
+				copy_v3_v3(normal, qmat[2]);
 
-				negate_v3_v3(plane, mat[1]);
+				negate_v3_v3(plane, qmat[1]);
 				
-				result = ORIENTATION_NORMAL;
+				result = ORIENTATION_FACE;
 			}
-#endif
-			
 		}
 		else if (obedit->type == OB_ARMATURE) {
 			bArmature *arm = obedit->data;
 			EditBone *ebone;
+			int ok = FALSE;
 			
-			for (ebone = arm->edbo->first; ebone; ebone=ebone->next) {
-				if (arm->layer & ebone->layer) {
-					if (ebone->flag & BONE_SELECTED) {
-						float tmat[3][3];
-						float vec[3];
-						sub_v3_v3v3(vec, ebone->tail, ebone->head);
-						normalize_v3(vec);
-						add_v3_v3(normal, vec);
-						
-						vec_roll_to_mat3(vec, ebone->roll, tmat);
-						add_v3_v3(plane, tmat[2]);
+			/* grr. but better then duplicate code */
+#define EBONE_CALC_NORMAL_PLANE  { \
+			float tmat[3][3]; \
+			float vec[3]; \
+			sub_v3_v3v3(vec, ebone->tail, ebone->head); \
+			normalize_v3(vec); \
+			add_v3_v3(normal, vec); \
+			\
+			vec_roll_to_mat3(vec, ebone->roll, tmat); \
+			add_v3_v3(plane, tmat[2]); \
+		} (void)0
+
+
+			if (activeOnly && (ebone = arm->act_edbone)) {
+				EBONE_CALC_NORMAL_PLANE;
+				ok = TRUE;
+			}
+			else {
+				for (ebone = arm->edbo->first; ebone; ebone = ebone->next) {
+					if (arm->layer & ebone->layer) {
+						if (ebone->flag & BONE_SELECTED) {
+							EBONE_CALC_NORMAL_PLANE;
+							ok = TRUE;
+						}
 					}
 				}
 			}
 			
-			normalize_v3(normal);
-			normalize_v3(plane);
+			if (ok) {
+				normalize_v3(normal);
+				normalize_v3(plane);
 
-			if (plane[0] != 0 || plane[1] != 0 || plane[2] != 0) {
-				result = ORIENTATION_EDGE;
+				if (!is_zero_v3(plane)) {
+					result = ORIENTATION_EDGE;
+				}
 			}
-
+#undef EBONE_CALC_NORMAL_PLANE
 		}
 
 		/* Vectors from edges don't need the special transpose inverse multiplication */
@@ -822,21 +814,34 @@ int getTransformOrientation(const bContext *C, float normal[3], float plane[3],
 		}
 	}
 	else if (ob && (ob->mode & OB_MODE_POSE)) {
-		bArmature *arm= ob->data;
+		bArmature *arm = ob->data;
 		bPoseChannel *pchan;
-		int totsel;
-		
-		totsel = count_bone_select(arm, &arm->bonebase, 1);
-		if (totsel) {
-			float imat[3][3], mat[3][3];
+		float imat[3][3], mat[3][3];
+		int ok = FALSE;
 
-			/* use channels to get stats */
-			for (pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
-				if (pchan->bone && pchan->bone->flag & BONE_TRANSFORM) {
-					add_v3_v3(normal, pchan->pose_mat[2]);
-					add_v3_v3(plane, pchan->pose_mat[1]);
+		if (activeOnly && (pchan = BKE_pose_channel_active(ob))) {
+			add_v3_v3(normal, pchan->pose_mat[2]);
+			add_v3_v3(plane, pchan->pose_mat[1]);
+			ok = TRUE;
+		}
+		else {
+			int totsel;
+
+			totsel = count_bone_select(arm, &arm->bonebase, 1);
+			if (totsel) {
+				/* use channels to get stats */
+				for (pchan = ob->pose->chanbase.first; pchan; pchan = pchan->next) {
+					if (pchan->bone && pchan->bone->flag & BONE_TRANSFORM) {
+						add_v3_v3(normal, pchan->pose_mat[2]);
+						add_v3_v3(plane, pchan->pose_mat[1]);
+					}
 				}
+				ok = TRUE;
 			}
+		}
+
+		/* use for both active & all */
+		if (ok) {
 			negate_v3(plane);
 			
 			/* we need the transpose of the inverse for a normal... */
@@ -850,7 +855,7 @@ int getTransformOrientation(const bContext *C, float normal[3], float plane[3],
 			result = ORIENTATION_EDGE;
 		}
 	}
-	else if (ob && (ob->mode & (OB_MODE_ALL_PAINT|OB_MODE_PARTICLE_EDIT))) {
+	else if (ob && (ob->mode & (OB_MODE_ALL_PAINT | OB_MODE_PARTICLE_EDIT))) {
 		/* pass */
 	}
 	else {
@@ -858,10 +863,10 @@ int getTransformOrientation(const bContext *C, float normal[3], float plane[3],
 		ob = OBACT;
 		if (ob && !(ob->flag & SELECT)) ob = NULL;
 		
-		for (base= scene->base.first; base; base= base->next) {
+		for (base = scene->base.first; base; base = base->next) {
 			if (TESTBASELIB(v3d, base)) {
 				if (ob == NULL) {
-					ob= base->object;
+					ob = base->object;
 					break;
 				}
 			}
@@ -877,17 +882,16 @@ int getTransformOrientation(const bContext *C, float normal[3], float plane[3],
 	return result;
 }
 
-void ED_getTransformOrientationMatrix(const bContext *C, float orientation_mat[][3], int activeOnly)
+void ED_getTransformOrientationMatrix(const bContext *C, float orientation_mat[3][3], int activeOnly)
 {
-	float normal[3]={0.0, 0.0, 0.0};
-	float plane[3]={0.0, 0.0, 0.0};
+	float normal[3] = {0.0, 0.0, 0.0};
+	float plane[3] = {0.0, 0.0, 0.0};
 
 	int type;
 
 	type = getTransformOrientation(C, normal, plane, activeOnly);
 
-	switch (type)
-	{
+	switch (type) {
 		case ORIENTATION_NORMAL:
 			if (createSpaceNormalTangent(orientation_mat, normal, plane) == 0) {
 				type = ORIENTATION_NONE;
