@@ -187,75 +187,6 @@ void crazyspace_set_quats_editmesh(BMEditMesh *em, float *origcos, float *mapped
 	em->bm->elem_index_dirty |= BM_VERT;
 
 	MEM_freeN(vert_table);
-#if 0
-	BMEditVert *eve, *prev;
-	BMEditFace *efa;
-	BMIter iter;
-	float *v1, *v2, *v3, *v4, *co1, *co2, *co3, *co4;
-	intptr_t index = 0;
-
-	/* two abused locations in vertices */
-	for (eve = em->verts.first; eve; eve = eve->next, index++) {
-		eve->tmp.p = NULL;
-		eve->prev = (EditVert *)index;
-	}
-
-	/* first store two sets of tangent vectors in vertices, we derive it just from the face-edges */
-	for (efa = em->faces.first; efa; efa = efa->next) {
-
-		/* retrieve mapped coordinates */
-		v1 = mappedcos + 3 * (intptr_t)(efa->v1->prev);
-		v2 = mappedcos + 3 * (intptr_t)(efa->v2->prev);
-		v3 = mappedcos + 3 * (intptr_t)(efa->v3->prev);
-
-		co1 = (origcos) ? origcos + 3 * (intptr_t)(efa->v1->prev) : efa->v1->co;
-		co2 = (origcos) ? origcos + 3 * (intptr_t)(efa->v2->prev) : efa->v2->co;
-		co3 = (origcos) ? origcos + 3 * (intptr_t)(efa->v3->prev) : efa->v3->co;
-
-		if (efa->v2->tmp.p == NULL && efa->v2->f1) {
-			set_crazy_vertex_quat(quats, co2, co3, co1, v2, v3, v1);
-			efa->v2->tmp.p = (void *)quats;
-			quats += 4;
-		}
-
-		if (efa->v4) {
-			v4 = mappedcos + 3 * (intptr_t)(efa->v4->prev);
-			co4 = (origcos) ? origcos + 3 * (intptr_t)(efa->v4->prev) : efa->v4->co;
-
-			if (efa->v1->tmp.p == NULL && efa->v1->f1) {
-				set_crazy_vertex_quat(quats, co1, co2, co4, v1, v2, v4);
-				efa->v1->tmp.p = (void *)quats;
-				quats += 4;
-			}
-			if (efa->v3->tmp.p == NULL && efa->v3->f1) {
-				set_crazy_vertex_quat(quats, co3, co4, co2, v3, v4, v2);
-				efa->v3->tmp.p = (void *)quats;
-				quats += 4;
-			}
-			if (efa->v4->tmp.p == NULL && efa->v4->f1) {
-				set_crazy_vertex_quat(quats, co4, co1, co3, v4, v1, v3);
-				efa->v4->tmp.p = (void *)quats;
-				quats += 4;
-			}
-		}
-		else {
-			if (efa->v1->tmp.p == NULL && efa->v1->f1) {
-				set_crazy_vertex_quat(quats, co1, co2, co3, v1, v2, v3);
-				efa->v1->tmp.p = (void *)quats;
-				quats += 4;
-			}
-			if (efa->v3->tmp.p == NULL && efa->v3->f1) {
-				set_crazy_vertex_quat(quats, co3, co1, co2, v3, v1, v2);
-				efa->v3->tmp.p = (void *)quats;
-				quats += 4;
-			}
-		}
-	}
-
-	/* restore abused prev pointer */
-	for (prev = NULL, eve = em->verts.first; eve; prev = eve, eve = eve->next)
-		eve->prev = prev;
-#endif
 }
 
 /* BMESH_TODO - use MPolys over MFace's */
@@ -457,7 +388,7 @@ void crazyspace_build_sculpt(Scene *scene, Object *ob, float (**deformmats)[3][3
 				if (mti->deformMatrices && !deformed)
 					continue;
 
-				mti->deformVerts(md, ob, NULL, deformedVerts, me->totvert, 0, 0);
+				mti->deformVerts(md, ob, NULL, deformedVerts, me->totvert, 0);
 				deformed = 1;
 			}
 		}
@@ -478,7 +409,7 @@ void crazyspace_build_sculpt(Scene *scene, Object *ob, float (**deformmats)[3][3
 		MEM_freeN(quats);
 	}
 
-	if (!*deformmats) {
+	if (*deformmats == NULL) {
 		int a, numVerts;
 		Mesh *me = (Mesh *)ob->data;
 
