@@ -37,12 +37,12 @@ struct ImBuf;
 
 typedef struct ImMetaData {
 	struct ImMetaData *next, *prev;
-	char* key;
-	char* value;
+	char *key;
+	char *value;
 	int len;
 } ImMetaData;
 
-/** The metadata is a list of key/value pairs (both char*) that can me
+/** The metadata is a list of key/value pairs (both char *) that can me
  * saved in the header of several image formats.
  * Apart from some common keys like
  * 'Software' and 'Description' (png standard) we'll use keys within the
@@ -52,7 +52,7 @@ typedef struct ImMetaData {
 
 
 /* free blender ImMetaData struct */
-void IMB_metadata_free(struct ImBuf* img);
+void IMB_metadata_free(struct ImBuf *img);
 
 /** read the field from the image info into the field 
  *  \param img - the ImBuf that contains the image data
@@ -62,7 +62,7 @@ void IMB_metadata_free(struct ImBuf* img);
  *  \param len - length of value buffer allocated by user.
  *  \return    - 1 (true) if ImageInfo present and value for the key found, 0 (false) otherwise
  */
-int IMB_metadata_get_field(struct ImBuf* img, const char* key, char* value, int len);
+int IMB_metadata_get_field(struct ImBuf *img, const char *key, char *value, const size_t len);
 
 /** set user data in the ImMetaData struct, which has to be allocated with IMB_metadata_create
  *  before calling this function.
@@ -71,7 +71,7 @@ int IMB_metadata_get_field(struct ImBuf* img, const char* key, char* value, int
  *  \param value - the data to be written to the field. zero terminated string
  *  \return    - 1 (true) if ImageInfo present, 0 (false) otherwise
  */
-int IMB_metadata_add_field(struct ImBuf* img, const char* key, const char* value);
+int IMB_metadata_add_field(struct ImBuf *img, const char *key, const char *value);
 
 /** delete the key/field par in the ImMetaData struct.
  * \param img - the ImBuf that contains the image data
