@@ -66,17 +66,14 @@
 		// Convert component \a c having \a inbits to the returned value having \a outbits.
 		inline uint convert(uint c, uint inbits, uint outbits)
 		{
-			if (inbits == 0)
-			{
+			if (inbits == 0) {
 				return 0;
 			}
-			else if (inbits >= outbits)
-			{
+			else if (inbits >= outbits) {
 				// truncate
 				return c >> (inbits - outbits);
 			}
-			else
-			{
+			else {
 				// bitexpand
 				return (c << (outbits - inbits)) | convert(c, inbits, outbits - inbits);
 			}
@@ -85,58 +82,57 @@
 		// Get pixel component shift and size given its mask.
 		inline void maskShiftAndSize(uint mask, uint * shift, uint * size)
 		{
-			if (!mask)
-			{
+			if (!mask) {
 				*shift = 0;
 				*size = 0;
 				return;
 			}
 
 			*shift = 0;
-			while((mask & 1) == 0) {
+			while ((mask & 1) == 0) {
 				++(*shift);
 				mask >>= 1;
 			}
 			
 			*size = 0;
-			while((mask & 1) == 1) {
+			while ((mask & 1) == 1) {
 				++(*size);
 				mask >>= 1;
 			}
 		}
 
-        inline float quantizeCeil(float f, int inbits, int outbits)
-        {
-            //uint i = f * (float(1 << inbits) - 1);
-            //i = convert(i, inbits, outbits);
-            //float result = float(i) / (float(1 << outbits) - 1);
-            //nvCheck(result >= f);
-            float result;
-            int offset = 0;
-            do {
-                uint i = offset + uint(f * (float(1 << inbits) - 1));
-                i = convert(i, inbits, outbits);
-                result = float(i) / (float(1 << outbits) - 1);
-                offset++;
-            } while (result < f);
+		inline float quantizeCeil(float f, int inbits, int outbits)
+		{
+			//uint i = f * (float(1 << inbits) - 1);
+			//i = convert(i, inbits, outbits);
+			//float result = float(i) / (float(1 << outbits) - 1);
+			//nvCheck(result >= f);
+			float result;
+			int offset = 0;
+			do {
+				uint i = offset + uint(f * (float(1 << inbits) - 1));
+				i = convert(i, inbits, outbits);
+				result = float(i) / (float(1 << outbits) - 1);
+				offset++;
+			} while (result < f);
 
-            return result;
-        }
+			return result;
+		}
 
-        /*
-        inline float quantizeRound(float f, int bits)
-        {
-            float scale = float(1 << bits);
-            return fround(f * scale) / scale;
-        }
+#if 0
+		inline float quantizeRound(float f, int bits)
+		{
+			float scale = float(1 << bits);
+			return fround(f * scale) / scale;
+		}
 
-        inline float quantizeFloor(float f, int bits)
-        {
-            float scale = float(1 << bits);
-            return floor(f * scale) / scale;
-        }
-        */
+		inline float quantizeFloor(float f, int bits)
+		{
+			float scale = float(1 << bits);
+			return floor(f * scale) / scale;
+		}
+#endif
 
 	} // PixelFormat namespace
 
-#endif // _DDS_IMAGE_PIXELFORMAT_H
+#endif  /* __PIXELFORMAT_H__ */
