@@ -29,9 +29,14 @@
 #include <dds_api.h>
 #include <Stream.h>
 #include <DirectDrawSurface.h>
+#include <FlipDXT.h>
 #include <stdio.h> // printf
 #include <fstream>
 
+#if defined (WIN32) && !defined(FREE_WINDOWS)
+#include "utfconv.h"
+#endif
+
 extern "C" {
 
 #include "imbuf.h"
@@ -39,6 +44,8 @@ extern "C" {
 #include "IMB_imbuf.h"
 #include "IMB_allocimbuf.h"
 
+#include "IMB_colormanagement.h"
+#include "IMB_colormanagement_intern.h"
 
 int imb_save_dds(struct ImBuf * ibuf, const char *name, int flags)
 {
@@ -49,7 +56,15 @@ int imb_save_dds(struct ImBuf * ibuf, const char *name, int flags)
 	if (ibuf->rect == 0) return (0);
 
 	/* open file for writing */
-	std::ofstream fildes(name);
+	std::ofstream fildes;
+
+#if defined (WIN32) && !defined(FREE_WINDOWS)
+	wchar_t *wname = alloc_utf16_from_8(name, 0);
+	fildes.open(wname);
+	free(wname);
+#else
+	fildes.open(name);
+#endif
 
 	/* write header */
 	fildes << "DDS ";
@@ -68,7 +83,7 @@ int imb_is_a_dds(unsigned char *mem) // note: use at most first 32 bytes
 	return(1);
 }
 
-struct ImBuf *imb_load_dds(unsigned char *mem, size_t size, int flags)
+struct ImBuf *imb_load_dds(unsigned char *mem, size_t size, int flags, char colorspace[IM_MAX_SPACE])
 {
 	struct ImBuf * ibuf = 0;
 	DirectDrawSurface dds(mem, size); /* reads header */
@@ -81,6 +96,12 @@ struct ImBuf *imb_load_dds(unsigned char *mem, size_t size, int flags)
 	Color32 pixel;
 	Color32 *pixels = 0;
 
+	/* OCIO_TODO: never was able to save DDS, so can't test loading
+	 *            but profile used to be set to sRGB and can't see rect_float here, so
+	 *            default byte space should work fine
+	 */
+	colorspace_set_default_role(colorspace, IM_MAX_SPACE, COLOR_ROLE_DEFAULT_BYTE);
+
 	if (!imb_is_a_dds(mem))
 		return (0);
 
@@ -115,14 +136,15 @@ struct ImBuf *imb_load_dds(unsigned char *mem, size_t size, int flags)
 			if (pixel.a != 255) {
 				bits_per_pixel = 32;
 				break;
-			};
-		};
-	};
+			}
+		}
+	}
 	ibuf = IMB_allocImBuf(dds.width(), dds.height(), bits_per_pixel, 0); 
 	if (ibuf == 0) return(0); /* memory allocation failed */
 
 	ibuf->ftype = DDS;
-	ibuf->profile = IB_PROFILE_SRGB;
+	ibuf->dds_data.fourcc = dds.fourCC();
+	ibuf->dds_data.nummipmaps = dds.mipmapCount();
 
 	if ((flags & IB_test) == 0) {
 		if (!imb_addrectImBuf(ibuf)) return(ibuf);
@@ -136,10 +158,23 @@ struct ImBuf *imb_load_dds(unsigned char *mem, size_t size, int flags)
 			cp[0] = pixel.r; /* set R component of col */
 			cp[1] = pixel.g; /* set G component of col */
 			cp[2] = pixel.b; /* set B component of col */
-			if (bits_per_pixel == 32)
+			if (dds.hasAlpha())
 				cp[3] = pixel.a; /* set A component of col */
 			rect[i] = col;
 		}
+
+		if (ibuf->dds_data.fourcc != FOURCC_DDS) {
+			ibuf->dds_data.data = (unsigned char *)dds.readData(ibuf->dds_data.size);
+
+			/* flip compressed texture */
+			FlipDXTCImage(dds.width(), dds.height(), dds.mipmapCount(), dds.fourCC(), ibuf->dds_data.data);
+		}
+		else {
+			ibuf->dds_data.data = NULL;
+			ibuf->dds_data.size = 0;
+		}
+
+		/* flip uncompressed texture */
 		IMB_flipy(ibuf);
 	}
 
