@@ -22,55 +22,60 @@
  * ***** END GPL LICENSE BLOCK *****
  */
 
-#include "IMB_indexer.h"
-#include "IMB_anim.h"
-#include "AVI_avi.h"
-#include "imbuf.h"
+#include <stdlib.h>
+
 #include "MEM_guardedalloc.h"
 
 #include "BLI_utildefines.h"
-#include "BLI_blenlib.h"
+#include "BLI_endian_switch.h"
+#include "BLI_path_util.h"
+#include "BLI_string.h"
+#include "BLI_path_util.h"
+#include "BLI_fileops.h"
 #include "BLI_math_base.h"
 
+#include "IMB_indexer.h"
+#include "IMB_anim.h"
+#include "imbuf.h"
+
 #include "MEM_guardedalloc.h"
 #include "DNA_userdef_types.h"
 #include "BKE_global.h"
-#include <stdlib.h>
+
+#ifdef WITH_AVI
+#  include "AVI_avi.h"
+#endif
 
 #ifdef WITH_FFMPEG
-
-#include "ffmpeg_compat.h"
-
-#endif //WITH_FFMPEG
+#  include "ffmpeg_compat.h"
+#endif
 
 
 static char magic[] = "BlenMIdx";
-static char temp_ext [] = "_part";
+static char temp_ext[] = "_part";
 
 static int proxy_sizes[] = { IMB_PROXY_25, IMB_PROXY_50, IMB_PROXY_75,
-                             IMB_PROXY_100 };
+	                         IMB_PROXY_100 };
 static float proxy_fac[] = { 0.25, 0.50, 0.75, 1.00 };
 
 #ifdef WITH_FFMPEG
-static int tc_types[] = { IMB_TC_RECORD_RUN,
-                          IMB_TC_FREE_RUN,
-                          IMB_TC_INTERPOLATED_REC_DATE_FREE_RUN,
-                          IMB_TC_RECORD_RUN_NO_GAPS,
-                        };
+static int tc_types[] = {IMB_TC_RECORD_RUN,
+                         IMB_TC_FREE_RUN,
+                         IMB_TC_INTERPOLATED_REC_DATE_FREE_RUN,
+                         IMB_TC_RECORD_RUN_NO_GAPS};
 #endif
 
 #define INDEX_FILE_VERSION 1
 
 /* ----------------------------------------------------------------------
-   - time code index functions
-   ---------------------------------------------------------------------- */
+ * - time code index functions
+ * ---------------------------------------------------------------------- */
 
-anim_index_builder * IMB_index_builder_create(const char * name)
+anim_index_builder *IMB_index_builder_create(const char *name)
 {
 
-	anim_index_builder * rv
-	        = MEM_callocN( sizeof(struct anim_index_builder),
-	                       "index builder");
+	anim_index_builder *rv = MEM_callocN(sizeof(struct anim_index_builder),
+	                                     "index builder");
 
 	fprintf(stderr, "Starting work on index: %s\n", name);
 
@@ -85,21 +90,21 @@ anim_index_builder * IMB_index_builder_create(const char * name)
 
 	if (!rv->fp) {
 		fprintf(stderr, "Couldn't open index target: %s! "
-			"Index build broken!\n", rv->temp_name);
+		        "Index build broken!\n", rv->temp_name);
 		MEM_freeN(rv);
 		return NULL;
 	}
 
-	fprintf(rv->fp, "%s%c%.3d", magic, (ENDIAN_ORDER==B_ENDIAN)?'V':'v',
-		INDEX_FILE_VERSION);
+	fprintf(rv->fp, "%s%c%.3d", magic, (ENDIAN_ORDER == B_ENDIAN) ? 'V' : 'v',
+	        INDEX_FILE_VERSION);
 
 	return rv;
 }
 
-void IMB_index_builder_add_entry(anim_index_builder * fp, 
-				 int frameno, unsigned long long seek_pos,
-				 unsigned long long seek_pos_dts,
-				 unsigned long long pts)
+void IMB_index_builder_add_entry(anim_index_builder *fp,
+                                 int frameno, unsigned long long seek_pos,
+                                 unsigned long long seek_pos_dts,
+                                 unsigned long long pts)
 {
 	fwrite(&frameno, sizeof(int), 1, fp->fp);
 	fwrite(&seek_pos, sizeof(unsigned long long), 1, fp->fp);
@@ -107,12 +112,12 @@ void IMB_index_builder_add_entry(anim_index_builder * fp,
 	fwrite(&pts, sizeof(unsigned long long), 1, fp->fp);
 }
 
-void IMB_index_builder_proc_frame(anim_index_builder * fp, 
-				  unsigned char * buffer,
-				  int data_size,
-				  int frameno, unsigned long long seek_pos,
-				  unsigned long long seek_pos_dts,
-				  unsigned long long pts)
+void IMB_index_builder_proc_frame(anim_index_builder *fp,
+                                  unsigned char *buffer,
+                                  int data_size,
+                                  int frameno, unsigned long long seek_pos,
+                                  unsigned long long seek_pos_dts,
+                                  unsigned long long pts)
 {
 	if (fp->proc_frame) {
 		anim_index_entry e;
@@ -125,11 +130,11 @@ void IMB_index_builder_proc_frame(anim_index_builder * fp,
 	}
 	else {
 		IMB_index_builder_add_entry(fp, frameno, seek_pos,
-					    seek_pos_dts, pts);
+		                            seek_pos_dts, pts);
 	}
 }
 
-void IMB_index_builder_finish(anim_index_builder * fp, int rollback)
+void IMB_index_builder_finish(anim_index_builder *fp, int rollback)
 {
 	if (fp->delete_priv_data) {
 		fp->delete_priv_data(fp);
@@ -148,11 +153,11 @@ void IMB_index_builder_finish(anim_index_builder * fp, int rollback)
 	MEM_freeN(fp);
 }
 
-struct anim_index * IMB_indexer_open(const char * name)
+struct anim_index *IMB_indexer_open(const char *name)
 {
 	char header[13];
-	struct anim_index * idx;
-	FILE * fp = BLI_fopen(name, "rb");
+	struct anim_index *idx;
+	FILE *fp = BLI_fopen(name, "rb");
 	int i;
 
 	if (!fp) {
@@ -171,28 +176,28 @@ struct anim_index * IMB_indexer_open(const char * name)
 		return NULL;
 	}
 
-	if (atoi(header+9) != INDEX_FILE_VERSION) {
+	if (atoi(header + 9) != INDEX_FILE_VERSION) {
 		fclose(fp);
 		return NULL;
 	}
 
-	idx = MEM_callocN( sizeof(struct anim_index), "anim_index");
+	idx = MEM_callocN(sizeof(struct anim_index), "anim_index");
 
 	BLI_strncpy(idx->name, name, sizeof(idx->name));
 	
 	fseek(fp, 0, SEEK_END);
 
-	idx->num_entries = (ftell(fp) - 12)
-		/ (sizeof(int) // framepos
-		   + sizeof(unsigned long long) // seek_pos
-		   + sizeof(unsigned long long) // seek_pos_dts
-		   + sizeof(unsigned long long) // pts
-			);
-	
+	idx->num_entries = (ftell(fp) - 12) /
+	                   (sizeof(int) +                /* framepos */
+	                    sizeof(unsigned long long) + /* seek_pos */
+	                    sizeof(unsigned long long) + /* seek_pos_dts */
+	                    sizeof(unsigned long long)   /* pts */
+	                   );
+
 	fseek(fp, 12, SEEK_SET);
 
-	idx->entries = MEM_callocN( sizeof(struct anim_index_entry) 
-				    * idx->num_entries, "anim_index_entries");
+	idx->entries = MEM_callocN(sizeof(struct anim_index_entry) *
+	                           idx->num_entries, "anim_index_entries");
 
 	for (i = 0; i < idx->num_entries; i++) {
 		fread(&idx->entries[i].frameno,
@@ -207,10 +212,10 @@ struct anim_index * IMB_indexer_open(const char * name)
 
 	if (((ENDIAN_ORDER == B_ENDIAN) != (header[8] == 'V'))) {
 		for (i = 0; i < idx->num_entries; i++) {
-			SWITCH_INT(idx->entries[i].frameno);
-			SWITCH_INT64(idx->entries[i].seek_pos);
-			SWITCH_INT64(idx->entries[i].seek_pos_dts);
-			SWITCH_INT64(idx->entries[i].pts);
+			BLI_endian_switch_int32(&idx->entries[i].frameno);
+			BLI_endian_switch_int64((int64_t *)&idx->entries[i].seek_pos);
+			BLI_endian_switch_int64((int64_t *)&idx->entries[i].seek_pos_dts);
+			BLI_endian_switch_int64((int64_t *)&idx->entries[i].pts);
 		}
 	}
 
@@ -220,7 +225,7 @@ struct anim_index * IMB_indexer_open(const char * name)
 }
 
 unsigned long long IMB_indexer_get_seek_pos(
-	struct anim_index * idx, int frame_index)
+        struct anim_index *idx, int frame_index)
 {
 	if (frame_index < 0) {
 		frame_index = 0;
@@ -232,7 +237,7 @@ unsigned long long IMB_indexer_get_seek_pos(
 }
 
 unsigned long long IMB_indexer_get_seek_pos_dts(
-	struct anim_index * idx, int frame_index)
+        struct anim_index *idx, int frame_index)
 {
 	if (frame_index < 0) {
 		frame_index = 0;
@@ -243,7 +248,7 @@ unsigned long long IMB_indexer_get_seek_pos_dts(
 	return idx->entries[frame_index].seek_pos_dts;
 }
 
-int IMB_indexer_get_frame_index(struct anim_index * idx, int frameno)
+int IMB_indexer_get_frame_index(struct anim_index *idx, int frameno)
 {
 	int len = idx->num_entries;
 	int half;
@@ -260,7 +265,7 @@ int IMB_indexer_get_frame_index(struct anim_index * idx, int frameno)
 
 		if (idx->entries[middle].frameno < frameno) {
 			first = middle;
-			++first;
+			first++;
 			len = len - half - 1;
 		}
 		else {
@@ -276,8 +281,8 @@ int IMB_indexer_get_frame_index(struct anim_index * idx, int frameno)
 	}
 }
 
-unsigned long long IMB_indexer_get_pts(struct anim_index * idx, 
-				       int frame_index)
+unsigned long long IMB_indexer_get_pts(struct anim_index *idx,
+                                       int frame_index)
 {
 	if (frame_index < 0) {
 		frame_index = 0;
@@ -288,25 +293,24 @@ unsigned long long IMB_indexer_get_pts(struct anim_index * idx,
 	return idx->entries[frame_index].pts;
 }
 
-int IMB_indexer_get_duration(struct anim_index * idx)
+int IMB_indexer_get_duration(struct anim_index *idx)
 {
 	if (idx->num_entries == 0) {
 		return 0;
 	}
-	return idx->entries[idx->num_entries-1].frameno + 1;
+	return idx->entries[idx->num_entries - 1].frameno + 1;
 }
 
-int IMB_indexer_can_scan(struct anim_index * idx, 
-			 int old_frame_index, int new_frame_index)
+int IMB_indexer_can_scan(struct anim_index *idx,
+                         int old_frame_index, int new_frame_index)
 {
 	/* makes only sense, if it is the same I-Frame and we are not
 	 * trying to run backwards in time... */
-	return (IMB_indexer_get_seek_pos(idx, old_frame_index)
-		== IMB_indexer_get_seek_pos(idx, new_frame_index) && 
-		old_frame_index < new_frame_index);
+	return (IMB_indexer_get_seek_pos(idx, old_frame_index) == IMB_indexer_get_seek_pos(idx, new_frame_index) &&
+	        old_frame_index < new_frame_index);
 }
 
-void IMB_indexer_close(struct anim_index * idx)
+void IMB_indexer_close(struct anim_index *idx)
 {
 	MEM_freeN(idx->entries);
 	MEM_freeN(idx);
@@ -315,49 +319,49 @@ void IMB_indexer_close(struct anim_index * idx)
 int IMB_proxy_size_to_array_index(IMB_Proxy_Size pr_size)
 {
 	switch (pr_size) {
-	case IMB_PROXY_NONE: /* if we got here, something is broken anyways,
-				so sane defaults... */
-		return 0;
-	case IMB_PROXY_25:
-		return 0;
-	case IMB_PROXY_50:
-		return 1;
-	case IMB_PROXY_75:
-		return 2;
-	case IMB_PROXY_100:
-		return 3;
-	default:
-		return 0;
-	};
+		case IMB_PROXY_NONE: /* if we got here, something is broken anyways,
+		                      * so sane defaults... */
+			return 0;
+		case IMB_PROXY_25:
+			return 0;
+		case IMB_PROXY_50:
+			return 1;
+		case IMB_PROXY_75:
+			return 2;
+		case IMB_PROXY_100:
+			return 3;
+		default:
+			return 0;
+	}
 	return 0;
 }
 
 int IMB_timecode_to_array_index(IMB_Timecode_Type tc)
 {
 	switch (tc) {
-	case IMB_TC_NONE: /* if we got here, something is broken anyways,
-				so sane defaults... */
-		return 0;
-	case IMB_TC_RECORD_RUN:
-		return 0;
-	case IMB_TC_FREE_RUN:
-		return 1;
-	case IMB_TC_INTERPOLATED_REC_DATE_FREE_RUN:
-		return 2;
-	case IMB_TC_RECORD_RUN_NO_GAPS:
-		return 3;
-	default:
-		return 0;
-	};
+		case IMB_TC_NONE: /* if we got here, something is broken anyways,
+		                   * so sane defaults... */
+			return 0;
+		case IMB_TC_RECORD_RUN:
+			return 0;
+		case IMB_TC_FREE_RUN:
+			return 1;
+		case IMB_TC_INTERPOLATED_REC_DATE_FREE_RUN:
+			return 2;
+		case IMB_TC_RECORD_RUN_NO_GAPS:
+			return 3;
+		default:
+			return 0;
+	}
 	return 0;
 }
 
 
 /* ----------------------------------------------------------------------
-   - rebuild helper functions
-   ---------------------------------------------------------------------- */
+ * - rebuild helper functions
+ * ---------------------------------------------------------------------- */
 
-static void get_index_dir(struct anim * anim, char * index_dir)
+static void get_index_dir(struct anim *anim, char *index_dir)
 {
 	if (!anim->index_dir[0]) {
 		char fname[FILE_MAXFILE];
@@ -371,8 +375,8 @@ static void get_index_dir(struct anim * anim, char * index_dir)
 	}
 }
 
-static void get_proxy_filename(struct anim * anim, IMB_Proxy_Size preview_size,
-			       char * fname, int temp)
+static void get_proxy_filename(struct anim *anim, IMB_Proxy_Size preview_size,
+                               char *fname, int temp)
 {
 	char index_dir[FILE_MAXDIR];
 	int i = IMB_proxy_size_to_array_index(preview_size);
@@ -388,9 +392,9 @@ static void get_proxy_filename(struct anim * anim, IMB_Proxy_Size preview_size,
 	}
 
 	BLI_snprintf(proxy_name, sizeof(proxy_name), "proxy_%d%s.avi",
-		     (int) (proxy_fac[i] * 100), stream_suffix);
+	             (int) (proxy_fac[i] * 100), stream_suffix);
 	BLI_snprintf(proxy_temp_name, sizeof(proxy_temp_name), "proxy_%d%s_part.avi",
-		     (int) (proxy_fac[i] * 100), stream_suffix);
+	             (int) (proxy_fac[i] * 100), stream_suffix);
 
 	get_index_dir(anim, index_dir);
 
@@ -398,16 +402,17 @@ static void get_proxy_filename(struct anim * anim, IMB_Proxy_Size preview_size,
 	                 temp ? proxy_temp_name : proxy_name);
 }
 
-static void get_tc_filename(struct anim * anim, IMB_Timecode_Type tc,
-			    char * fname)
+static void get_tc_filename(struct anim *anim, IMB_Timecode_Type tc,
+                            char *fname)
 {
 	char index_dir[FILE_MAXDIR];
 	int i = IMB_timecode_to_array_index(tc);
-	const char * index_names[] = {
+	const char *index_names[] = {
 		"record_run%s.blen_tc",
 		"free_run%s.blen_tc",
 		"interp_free_run%s.blen_tc",
-		"record_run_no_gaps%s.blen_tc"};
+		"record_run_no_gaps%s.blen_tc"
+	};
 
 	char stream_suffix[20];
 	char index_name[256];
@@ -427,8 +432,8 @@ static void get_tc_filename(struct anim * anim, IMB_Timecode_Type tc,
 }
 
 /* ----------------------------------------------------------------------
-   - common rebuilder structures
-   ---------------------------------------------------------------------- */
+ * - common rebuilder structures
+ * ---------------------------------------------------------------------- */
 
 typedef struct IndexBuildContext {
 	int anim_type;
@@ -436,24 +441,24 @@ typedef struct IndexBuildContext {
 
 
 /* ----------------------------------------------------------------------
-   - ffmpeg rebuilder
-   ---------------------------------------------------------------------- */
+ * - ffmpeg rebuilder
+ * ---------------------------------------------------------------------- */
 
 #ifdef WITH_FFMPEG
 
 struct proxy_output_ctx {
-	AVFormatContext* of;
-	AVStream* st;
-	AVCodecContext* c;
-	AVCodec* codec;
-	struct SwsContext * sws_ctx;
-	AVFrame* frame;
-	uint8_t* video_buffer;
+	AVFormatContext *of;
+	AVStream *st;
+	AVCodecContext *c;
+	AVCodec *codec;
+	struct SwsContext *sws_ctx;
+	AVFrame *frame;
+	uint8_t *video_buffer;
 	int video_buffersize;
 	int cfra;
 	int proxy_size;
 	int orig_height;
-	struct anim * anim;
+	struct anim *anim;
 };
 
 // work around stupid swscaler 16 bytes alignment bug...
@@ -463,18 +468,18 @@ static int round_up(int x, int mod)
 	return x + ((mod - (x % mod)) % mod);
 }
 
-static struct proxy_output_ctx * alloc_proxy_output_ffmpeg(
-	struct anim * anim,
-	AVStream * st, int proxy_size, int width, int height,
-	int quality)
+static struct proxy_output_ctx *alloc_proxy_output_ffmpeg(
+        struct anim *anim,
+        AVStream *st, int proxy_size, int width, int height,
+        int quality)
 {
-	struct proxy_output_ctx * rv = MEM_callocN(
-		sizeof(struct proxy_output_ctx), "alloc_proxy_output");
+	struct proxy_output_ctx *rv = MEM_callocN(
+	        sizeof(struct proxy_output_ctx), "alloc_proxy_output");
 	
 	char fname[FILE_MAX];
 	int ffmpeg_quality;
 
-	// JPEG requires this
+	/* JPEG requires this */
 	width = round_up(width, 8);
 	height = round_up(height, 8);
 
@@ -491,7 +496,9 @@ static struct proxy_output_ctx * alloc_proxy_output_ffmpeg(
 
 	fprintf(stderr, "Starting work on proxy: %s\n", rv->of->filename);
 
-	rv->st = av_new_stream(rv->of, 0);
+	rv->st = avformat_new_stream(rv->of, NULL);
+	rv->st->id = 0;
+
 	rv->c = rv->st->codec;
 	rv->c->codec_type = AVMEDIA_TYPE_VIDEO;
 	rv->c->codec_id = CODEC_ID_MJPEG;
@@ -503,7 +510,7 @@ static struct proxy_output_ctx * alloc_proxy_output_ffmpeg(
 
 	if (!rv->codec) {
 		fprintf(stderr, "No ffmpeg MJPEG encoder available? "
-			"Proxy not built!\n");
+		        "Proxy not built!\n");
 		av_free(rv->of);
 		return NULL;
 	}
@@ -515,9 +522,9 @@ static struct proxy_output_ctx * alloc_proxy_output_ffmpeg(
 		rv->c->pix_fmt = PIX_FMT_YUVJ420P;
 	}
 
-	rv->c->sample_aspect_ratio 
-		= rv->st->sample_aspect_ratio 
-		= st->codec->sample_aspect_ratio;
+	rv->c->sample_aspect_ratio =
+	    rv->st->sample_aspect_ratio =
+	        st->codec->sample_aspect_ratio;
 
 	rv->c->time_base.den = 25;
 	rv->c->time_base.num = 1;
@@ -526,40 +533,33 @@ static struct proxy_output_ctx * alloc_proxy_output_ffmpeg(
 	/* there's no  way to set JPEG quality in the same way as in AVI JPEG and image sequence,
 	 * but this seems to be giving expected quality result */
 	ffmpeg_quality = (int)(1.0f + 30.0f * (1.0f - (float)quality / 100.0f) + 0.5f);
-	av_set_int(rv->c, "qmin", ffmpeg_quality);
-	av_set_int(rv->c, "qmax", ffmpeg_quality);
+	av_opt_set_int(rv->c, "qmin", ffmpeg_quality, 0);
+	av_opt_set_int(rv->c, "qmax", ffmpeg_quality, 0);
 
 	if (rv->of->flags & AVFMT_GLOBALHEADER) {
 		rv->c->flags |= CODEC_FLAG_GLOBAL_HEADER;
 	}
 
-	if (av_set_parameters(rv->of, NULL) < 0) {
-		fprintf(stderr, "Couldn't set output parameters? "
-			"Proxy not built!\n");
-		av_free(rv->of);
-		return 0;
-	}
-
 	if (avio_open(&rv->of->pb, fname, AVIO_FLAG_WRITE) < 0) {
 		fprintf(stderr, "Couldn't open outputfile! "
-			"Proxy not built!\n");
+		        "Proxy not built!\n");
 		av_free(rv->of);
 		return 0;
 	}
 
-	avcodec_open(rv->c, rv->codec);
+	avcodec_open2(rv->c, rv->codec, NULL);
 
 	rv->video_buffersize = 2000000;
-	rv->video_buffer = (uint8_t*)MEM_mallocN(
-		rv->video_buffersize, "FFMPEG video buffer");
+	rv->video_buffer = (uint8_t *)MEM_mallocN(
+	        rv->video_buffersize, "FFMPEG video buffer");
 
 	rv->orig_height = st->codec->height;
 
 	if (st->codec->width != width || st->codec->height != height ||
-	        st->codec->pix_fmt != rv->c->pix_fmt)
+	    st->codec->pix_fmt != rv->c->pix_fmt)
 	{
 		rv->frame = avcodec_alloc_frame();
-		avpicture_fill((AVPicture*) rv->frame,
+		avpicture_fill((AVPicture *) rv->frame,
 		               MEM_mallocN(avpicture_get_size(
 		                               rv->c->pix_fmt,
 		                               round_up(width, 16), height),
@@ -567,22 +567,27 @@ static struct proxy_output_ctx * alloc_proxy_output_ffmpeg(
 		               rv->c->pix_fmt, round_up(width, 16), height);
 
 		rv->sws_ctx = sws_getContext(
-			st->codec->width,
-			st->codec->height,
-			st->codec->pix_fmt,
-			width, height,
-			rv->c->pix_fmt,
-			SWS_FAST_BILINEAR | SWS_PRINT_INFO,
-			NULL, NULL, NULL);
+		        st->codec->width,
+		        st->codec->height,
+		        st->codec->pix_fmt,
+		        width, height,
+		        rv->c->pix_fmt,
+		        SWS_FAST_BILINEAR | SWS_PRINT_INFO,
+		        NULL, NULL, NULL);
 	}
 
-	av_write_header(rv->of);
+	if (avformat_write_header(rv->of, NULL) < 0) {
+		fprintf(stderr, "Couldn't set output parameters? "
+		        "Proxy not built!\n");
+		av_free(rv->of);
+		return 0;
+	}
 
 	return rv;
 }
 
 static int add_to_proxy_output_ffmpeg(
-	struct proxy_output_ctx * ctx, AVFrame * frame)
+        struct proxy_output_ctx *ctx, AVFrame *frame)
 {
 	int outsize = 0;
 
@@ -590,11 +595,11 @@ static int add_to_proxy_output_ffmpeg(
 		return 0;
 	}
 
-	if (    ctx->sws_ctx && frame &&
-	        (frame->data[0] || frame->data[1] ||
-	         frame->data[2] || frame->data[3]))
+	if (ctx->sws_ctx && frame &&
+	    (frame->data[0] || frame->data[1] ||
+	     frame->data[2] || frame->data[3]))
 	{
-		sws_scale(ctx->sws_ctx, (const uint8_t * const*) frame->data,
+		sws_scale(ctx->sws_ctx, (const uint8_t *const *) frame->data,
 		          frame->linesize, 0, ctx->orig_height,
 		          ctx->frame->data, ctx->frame->linesize);
 	}
@@ -606,12 +611,12 @@ static int add_to_proxy_output_ffmpeg(
 	}
 
 	outsize = avcodec_encode_video(
-		ctx->c, ctx->video_buffer, ctx->video_buffersize, 
-		frame);
+	        ctx->c, ctx->video_buffer, ctx->video_buffersize,
+	        frame);
 
 	if (outsize < 0) {
 		fprintf(stderr, "Error encoding proxy frame %d for '%s'\n", 
-			ctx->cfra - 1, ctx->of->filename);
+		        ctx->cfra - 1, ctx->of->filename);
 		return 0;
 	}
 
@@ -621,8 +626,8 @@ static int add_to_proxy_output_ffmpeg(
 
 		if (ctx->c->coded_frame->pts != AV_NOPTS_VALUE) {
 			packet.pts = av_rescale_q(ctx->c->coded_frame->pts,
-						  ctx->c->time_base,
-						  ctx->st->time_base);
+			                          ctx->c->time_base,
+			                          ctx->st->time_base);
 		}
 		if (ctx->c->coded_frame->key_frame)
 			packet.flags |= AV_PKT_FLAG_KEY;
@@ -633,8 +638,8 @@ static int add_to_proxy_output_ffmpeg(
 
 		if (av_interleaved_write_frame(ctx->of, &packet) != 0) {
 			fprintf(stderr, "Error writing proxy frame %d "
-				"into '%s'\n", ctx->cfra - 1, 
-				ctx->of->filename);
+			        "into '%s'\n", ctx->cfra - 1,
+			        ctx->of->filename);
 			return 0;
 		}
 
@@ -645,8 +650,8 @@ static int add_to_proxy_output_ffmpeg(
 	}
 }
 
-static void free_proxy_output_ffmpeg(struct proxy_output_ctx * ctx,
-				     int rollback)
+static void free_proxy_output_ffmpeg(struct proxy_output_ctx *ctx,
+                                     int rollback)
 {
 	int i;
 	char fname[FILE_MAX];
@@ -716,11 +721,22 @@ typedef struct FFmpegIndexBuilderContext {
 	int num_proxy_sizes;
 	int num_indexers;
 
-	struct proxy_output_ctx * proxy_ctx[IMB_PROXY_MAX_SLOT];
-	anim_index_builder * indexer [IMB_TC_MAX_SLOT];
+	struct proxy_output_ctx *proxy_ctx[IMB_PROXY_MAX_SLOT];
+	anim_index_builder *indexer[IMB_TC_MAX_SLOT];
 
 	IMB_Timecode_Type tcs_in_use;
 	IMB_Proxy_Size proxy_sizes_in_use;
+
+	unsigned long long seek_pos;
+	unsigned long long last_seek_pos;
+	unsigned long long seek_pos_dts;
+	unsigned long long seek_pos_pts;
+	unsigned long long last_seek_pos_dts;
+	unsigned long long start_pts;
+	double frame_rate;
+	double pts_time_base;
+	int frameno, frameno_gapless;
+	int start_pts_set;
 } FFmpegIndexBuilderContext;
 
 static IndexBuildContext *index_ffmpeg_create_context(struct anim *anim, IMB_Timecode_Type tcs_in_use,
@@ -739,12 +755,12 @@ static IndexBuildContext *index_ffmpeg_create_context(struct anim *anim, IMB_Tim
 	memset(context->proxy_ctx, 0, sizeof(context->proxy_ctx));
 	memset(context->indexer, 0, sizeof(context->indexer));
 
-	if (av_open_input_file(&context->iFormatCtx, anim->name, NULL, 0, NULL) != 0) {
+	if (avformat_open_input(&context->iFormatCtx, anim->name, NULL, NULL) != 0) {
 		MEM_freeN(context);
 		return NULL;
 	}
 
-	if (av_find_stream_info(context->iFormatCtx) < 0) {
+	if (avformat_find_stream_info(context->iFormatCtx, NULL) < 0) {
 		av_close_input_file(context->iFormatCtx);
 		MEM_freeN(context);
 		return NULL;
@@ -755,8 +771,7 @@ static IndexBuildContext *index_ffmpeg_create_context(struct anim *anim, IMB_Tim
 	/* Find the video stream */
 	context->videoStream = -1;
 	for (i = 0; i < context->iFormatCtx->nb_streams; i++)
-		if (context->iFormatCtx->streams[i]->codec->codec_type
-		   == AVMEDIA_TYPE_VIDEO) {
+		if (context->iFormatCtx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO) {
 			if (streamcount > 0) {
 				streamcount--;
 				continue;
@@ -784,7 +799,7 @@ static IndexBuildContext *index_ffmpeg_create_context(struct anim *anim, IMB_Tim
 
 	context->iCodecCtx->workaround_bugs = 1;
 
-	if (avcodec_open(context->iCodecCtx, context->iCodec) < 0) {
+	if (avcodec_open2(context->iCodecCtx, context->iCodec, NULL) < 0) {
 		av_close_input_file(context->iFormatCtx);
 		MEM_freeN(context);
 		return NULL;
@@ -793,10 +808,10 @@ static IndexBuildContext *index_ffmpeg_create_context(struct anim *anim, IMB_Tim
 	for (i = 0; i < num_proxy_sizes; i++) {
 		if (proxy_sizes_in_use & proxy_sizes[i]) {
 			context->proxy_ctx[i] = alloc_proxy_output_ffmpeg(
-				anim, context->iStream, proxy_sizes[i],
-				context->iCodecCtx->width * proxy_fac[i],
-				context->iCodecCtx->height * proxy_fac[i],
-				quality);
+			        anim, context->iStream, proxy_sizes[i],
+			        context->iCodecCtx->width * proxy_fac[i],
+			        context->iCodecCtx->height * proxy_fac[i],
+			        quality);
 			if (!context->proxy_ctx[i]) {
 				proxy_sizes_in_use &= ~proxy_sizes[i];
 			}
@@ -838,39 +853,84 @@ static void index_rebuild_ffmpeg_finish(FFmpegIndexBuilderContext *context, int
 	MEM_freeN(context);
 }
 
+static void index_rebuild_ffmpeg_proc_decoded_frame(
+	FFmpegIndexBuilderContext *context, 
+	AVPacket * curr_packet,
+	AVFrame *in_frame)
+{
+	int i;
+	unsigned long long s_pos = context->seek_pos;
+	unsigned long long s_dts = context->seek_pos_dts;
+	unsigned long long pts = av_get_pts_from_frame(context->iFormatCtx, in_frame);
+
+	for (i = 0; i < context->num_proxy_sizes; i++) {
+		add_to_proxy_output_ffmpeg(context->proxy_ctx[i], in_frame);
+	}
+
+	if (!context->start_pts_set) {
+		context->start_pts = pts;
+		context->start_pts_set = TRUE;
+	}
+
+	context->frameno = floor((pts - context->start_pts) *
+	                         context->pts_time_base  *
+	                         context->frame_rate + 0.5);
+
+	/* decoding starts *always* on I-Frames,
+	 * so: P-Frames won't work, even if all the
+	 * information is in place, when we seek
+	 * to the I-Frame presented *after* the P-Frame,
+	 * but located before the P-Frame within
+	 * the stream */
+
+	if (pts < context->seek_pos_pts) {
+		s_pos = context->last_seek_pos;
+		s_dts = context->last_seek_pos_dts;
+	}
+
+	for (i = 0; i < context->num_indexers; i++) {
+		if (context->tcs_in_use & tc_types[i]) {
+			int tc_frameno = context->frameno;
+
+			if (tc_types[i] == IMB_TC_RECORD_RUN_NO_GAPS)
+				tc_frameno = context->frameno_gapless;
+			
+			IMB_index_builder_proc_frame(
+				context->indexer[i],
+				curr_packet->data,
+				curr_packet->size,
+				tc_frameno,
+				s_pos, s_dts, pts);
+		}
+	}
+	
+	context->frameno_gapless++;
+}
+
 static int index_rebuild_ffmpeg(FFmpegIndexBuilderContext *context,
                                 short *stop, short *do_update, float *progress)
 {
-	int i;
-	unsigned long long seek_pos = 0;
-	unsigned long long last_seek_pos = 0;
-	unsigned long long seek_pos_dts = 0;
-	unsigned long long seek_pos_pts = 0;
-	unsigned long long last_seek_pos_dts = 0;
-	unsigned long long start_pts = 0;
-	double frame_rate;
-	double pts_time_base;
-	int frameno = 0, frameno_gapless = 0;
-	int start_pts_set = FALSE;
-	AVFrame* in_frame = 0;
+	AVFrame *in_frame = 0;
 	AVPacket next_packet;
 	uint64_t stream_size;
 
+	memset(&next_packet, 0, sizeof(AVPacket));
+
 	in_frame = avcodec_alloc_frame();
 
 	stream_size = avio_size(context->iFormatCtx->pb);
 
-	frame_rate = av_q2d(context->iStream->r_frame_rate);
-	pts_time_base = av_q2d(context->iStream->time_base);
+	context->frame_rate = av_q2d(context->iStream->r_frame_rate);
+	context->pts_time_base = av_q2d(context->iStream->time_base);
 
 	while (av_read_frame(context->iFormatCtx, &next_packet) >= 0) {
 		int frame_finished = 0;
 		float next_progress =  (float)((int)floor(((double) next_packet.pos) * 100 /
-		                                   ((double) stream_size)+0.5)) / 100;
+		                                          ((double) stream_size) + 0.5)) / 100;
 
 		if (*progress != next_progress) {
 			*progress = next_progress;
-			*do_update = 1;
+			*do_update = TRUE;
 		}
 
 		if (*stop) {
@@ -880,68 +940,48 @@ static int index_rebuild_ffmpeg(FFmpegIndexBuilderContext *context,
 
 		if (next_packet.stream_index == context->videoStream) {
 			if (next_packet.flags & AV_PKT_FLAG_KEY) {
-				last_seek_pos = seek_pos;
-				last_seek_pos_dts = seek_pos_dts;
-				seek_pos = next_packet.pos;
-				seek_pos_dts = next_packet.dts;
-				seek_pos_pts = next_packet.pts;
+				context->last_seek_pos = context->seek_pos;
+				context->last_seek_pos_dts = context->seek_pos_dts;
+				context->seek_pos = next_packet.pos;
+				context->seek_pos_dts = next_packet.dts;
+				context->seek_pos_pts = next_packet.pts;
 			}
 
+			avcodec_decode_video2(
+			        context->iCodecCtx, in_frame, &frame_finished,
+			        &next_packet);
+		}
+
+		if (frame_finished) {
+			index_rebuild_ffmpeg_proc_decoded_frame(
+				context, &next_packet, in_frame);
+		}
+		av_free_packet(&next_packet);
+	}
+
+	/* process pictures still stuck in decoder engine after EOF
+	 * according to ffmpeg docs using 0-size packets.
+	 *
+	 * At least, if we haven't already stopped... */
+
+	/* this creates the 0-size packet and prevents a memory leak. */
+	av_free_packet(&next_packet);
+
+	if (!*stop) {
+		int frame_finished;
+
+		do {
+			frame_finished = 0;
+
 			avcodec_decode_video2(
 				context->iCodecCtx, in_frame, &frame_finished,
 				&next_packet);
-		}
 
-		if (frame_finished) {
-			unsigned long long s_pos = seek_pos;
-			unsigned long long s_dts = seek_pos_dts;
-			unsigned long long pts 
-				= av_get_pts_from_frame(context->iFormatCtx, in_frame);
-
-			for (i = 0; i < context->num_proxy_sizes; i++) {
-				add_to_proxy_output_ffmpeg(
-					context->proxy_ctx[i], in_frame);
+			if (frame_finished) {
+				index_rebuild_ffmpeg_proc_decoded_frame(
+					context, &next_packet, in_frame);
 			}
-
-			if (!start_pts_set) {
-				start_pts = pts;
-				start_pts_set = TRUE;
-			}
-
-			frameno = floor((pts - start_pts)
-				* pts_time_base * frame_rate + 0.5f);
-
-			/* decoding starts *always* on I-Frames,
-			   so: P-Frames won't work, even if all the
-			   information is in place, when we seek
-			   to the I-Frame presented *after* the P-Frame,
-			   but located before the P-Frame within
-			   the stream */
-
-			if (pts < seek_pos_pts) {
-				s_pos = last_seek_pos;
-				s_dts = last_seek_pos_dts;
-			}
-
-			for (i = 0; i < context->num_indexers; i++) {
-				if (context->tcs_in_use & tc_types[i]) {
-					int tc_frameno = frameno;
-
-					if (tc_types[i] == IMB_TC_RECORD_RUN_NO_GAPS)
-						tc_frameno = frameno_gapless;
-
-					IMB_index_builder_proc_frame(
-						context->indexer[i],
-						next_packet.data, 
-						next_packet.size,
-						tc_frameno,
-						s_pos, s_dts, pts);
-				}
-			}
-
-			frameno_gapless++;
-		}
-		av_free_packet(&next_packet);
+		} while (frame_finished);
 	}
 
 	av_free(in_frame);
@@ -952,9 +992,10 @@ static int index_rebuild_ffmpeg(FFmpegIndexBuilderContext *context,
 #endif
 
 /* ----------------------------------------------------------------------
-   - internal AVI (fallback) rebuilder
-   ---------------------------------------------------------------------- */
+ * - internal AVI (fallback) rebuilder
+ * ---------------------------------------------------------------------- */
 
+#ifdef WITH_AVI
 typedef struct FallbackIndexBuilderContext {
 	int anim_type;
 
@@ -963,16 +1004,16 @@ typedef struct FallbackIndexBuilderContext {
 	IMB_Proxy_Size proxy_sizes_in_use;
 } FallbackIndexBuilderContext;
 
-static AviMovie * alloc_proxy_output_avi(
-	struct anim * anim, char * filename, int width, int height,
-	int quality)
+static AviMovie *alloc_proxy_output_avi(
+        struct anim *anim, char *filename, int width, int height,
+        int quality)
 {
 	int x, y;
 	AviFormat format;
 	double framerate;
-	AviMovie * avi;
+	AviMovie *avi;
 	short frs_sec = 25;      /* it doesn't really matter for proxies,
-				    but sane defaults help anyways...*/
+	                          * but sane defaults help anyways...*/
 	float frs_sec_base = 1.0;
 
 	IMB_anim_get_fps(anim, &frs_sec, &frs_sec_base);
@@ -980,24 +1021,24 @@ static AviMovie * alloc_proxy_output_avi(
 	x = width;
 	y = height;
 
-	framerate= (double) frs_sec / (double) frs_sec_base;
+	framerate = (double) frs_sec / (double) frs_sec_base;
 	
-	avi = MEM_mallocN (sizeof(AviMovie), "avimovie");
+	avi = MEM_mallocN(sizeof(AviMovie), "avimovie");
 
 	format = AVI_FORMAT_MJPEG;
 
-	if (AVI_open_compress (filename, avi, 1, format) != AVI_ERROR_NONE) {
+	if (AVI_open_compress(filename, avi, 1, format) != AVI_ERROR_NONE) {
 		MEM_freeN(avi);
 		return NULL;
 	}
 			
-	AVI_set_compress_option (avi, AVI_OPTION_TYPE_MAIN, 0, AVI_OPTION_WIDTH, &x);
-	AVI_set_compress_option (avi, AVI_OPTION_TYPE_MAIN, 0, AVI_OPTION_HEIGHT, &y);
-	AVI_set_compress_option (avi, AVI_OPTION_TYPE_MAIN, 0, AVI_OPTION_QUALITY, &quality);		
-	AVI_set_compress_option (avi, AVI_OPTION_TYPE_MAIN, 0, AVI_OPTION_FRAMERATE, &framerate);
+	AVI_set_compress_option(avi, AVI_OPTION_TYPE_MAIN, 0, AVI_OPTION_WIDTH, &x);
+	AVI_set_compress_option(avi, AVI_OPTION_TYPE_MAIN, 0, AVI_OPTION_HEIGHT, &y);
+	AVI_set_compress_option(avi, AVI_OPTION_TYPE_MAIN, 0, AVI_OPTION_QUALITY, &quality);
+	AVI_set_compress_option(avi, AVI_OPTION_TYPE_MAIN, 0, AVI_OPTION_FRAMERATE, &framerate);
 
-	avi->interlace= 0;
-	avi->odd_fields= 0;
+	avi->interlace = 0;
+	avi->odd_fields = 0;
 
 	return avi;
 }
@@ -1009,10 +1050,10 @@ static IndexBuildContext *index_fallback_create_context(struct anim *anim, IMB_T
 	int i;
 
 	/* since timecode indices only work with ffmpeg right now,
-	   don't know a sensible fallback here...
-
-	   so no proxies, no game to play...
-	*/
+	 * don't know a sensible fallback here...
+	 *
+	 * so no proxies, no game to play...
+	 */
 	if (proxy_sizes_in_use == IMB_PROXY_NONE) {
 		return NULL;
 	}
@@ -1032,7 +1073,7 @@ static IndexBuildContext *index_fallback_create_context(struct anim *anim, IMB_T
 			BLI_make_existing_file(fname);
 
 			context->proxy_ctx[i] = alloc_proxy_output_avi(anim, fname,
-				anim->x * proxy_fac[i], anim->y * proxy_fac[i], quality);
+			                                               anim->x * proxy_fac[i], anim->y * proxy_fac[i], quality);
 		}
 	}
 
@@ -1079,7 +1120,7 @@ static void index_rebuild_fallback(FallbackIndexBuilderContext *context,
 
 		if (*progress != next_progress) {
 			*progress = next_progress;
-			*do_update = 1;
+			*do_update = TRUE;
 		}
 		
 		if (*stop) {
@@ -1093,15 +1134,15 @@ static void index_rebuild_fallback(FallbackIndexBuilderContext *context,
 				int x = anim->x * proxy_fac[i];
 				int y = anim->y * proxy_fac[i];
 
-				struct ImBuf * s_ibuf = IMB_dupImBuf(tmp_ibuf);
+				struct ImBuf *s_ibuf = IMB_dupImBuf(tmp_ibuf);
 
 				IMB_scalefastImBuf(s_ibuf, x, y);
 
 				IMB_convert_rgba_to_abgr(s_ibuf);
 	
-				AVI_write_frame (context->proxy_ctx[i], pos, 
-						 AVI_FORMAT_RGB32, 
-						 s_ibuf->rect, x * y * 4);
+				AVI_write_frame(context->proxy_ctx[i], pos,
+				                AVI_FORMAT_RGB32,
+				                s_ibuf->rect, x * y * 4);
 
 				/* note that libavi free's the buffer... */
 				s_ibuf->rect = NULL;
@@ -1115,30 +1156,36 @@ static void index_rebuild_fallback(FallbackIndexBuilderContext *context,
 	}
 }
 
+#endif  /* WITH_AVI */
+
 /* ----------------------------------------------------------------------
-   - public API
-   ---------------------------------------------------------------------- */
+ * - public API
+ * ---------------------------------------------------------------------- */
 
 IndexBuildContext *IMB_anim_index_rebuild_context(struct anim *anim, IMB_Timecode_Type tcs_in_use,
-                                                         IMB_Proxy_Size proxy_sizes_in_use, int quality)
+                                                  IMB_Proxy_Size proxy_sizes_in_use, int quality)
 {
 	IndexBuildContext *context = NULL;
 
 	switch (anim->curtype) {
 #ifdef WITH_FFMPEG
-	case ANIM_FFMPEG:
-		context = index_ffmpeg_create_context(anim, tcs_in_use, proxy_sizes_in_use, quality);
-		break;
+		case ANIM_FFMPEG:
+			context = index_ffmpeg_create_context(anim, tcs_in_use, proxy_sizes_in_use, quality);
+			break;
+#endif
+#ifdef WITH_AVI
+		default:
+			context = index_fallback_create_context(anim, tcs_in_use, proxy_sizes_in_use, quality);
+			break;
 #endif
-	default:
-		context = index_fallback_create_context(anim, tcs_in_use, proxy_sizes_in_use, quality);
-		break;
 	}
 
 	if (context)
 		context->anim_type = anim->curtype;
 
 	return context;
+
+	(void)tcs_in_use, (void)proxy_sizes_in_use, (void)quality;
 }
 
 void IMB_anim_index_rebuild(struct IndexBuildContext *context,
@@ -1146,32 +1193,41 @@ void IMB_anim_index_rebuild(struct IndexBuildContext *context,
 {
 	switch (context->anim_type) {
 #ifdef WITH_FFMPEG
-	case ANIM_FFMPEG:
-		index_rebuild_ffmpeg((FFmpegIndexBuilderContext*)context, stop, do_update, progress);
-		break;
+		case ANIM_FFMPEG:
+			index_rebuild_ffmpeg((FFmpegIndexBuilderContext *)context, stop, do_update, progress);
+			break;
+#endif
+#ifdef WITH_AVI
+		default:
+			index_rebuild_fallback((FallbackIndexBuilderContext *)context, stop, do_update, progress);
+			break;
 #endif
-	default:
-		index_rebuild_fallback((FallbackIndexBuilderContext*)context, stop, do_update, progress);
-		break;
 	}
+
+	(void)stop, (void)do_update, (void)progress;
 }
 
 void IMB_anim_index_rebuild_finish(IndexBuildContext *context, short stop)
 {
 	switch (context->anim_type) {
 #ifdef WITH_FFMPEG
-	case ANIM_FFMPEG:
-		index_rebuild_ffmpeg_finish((FFmpegIndexBuilderContext*)context, stop);
-		break;
+		case ANIM_FFMPEG:
+			index_rebuild_ffmpeg_finish((FFmpegIndexBuilderContext *)context, stop);
+			break;
+#endif
+#ifdef WITH_AVI
+		default:
+			index_rebuild_fallback_finish((FallbackIndexBuilderContext *)context, stop);
+			break;
 #endif
-	default:
-		index_rebuild_fallback_finish((FallbackIndexBuilderContext*)context, stop);
-		break;
 	}
+
+	(void)stop;
+	(void)proxy_sizes;  /* static defined at top of the file */
 }
 
 
-void IMB_free_indices(struct anim * anim)
+void IMB_free_indices(struct anim *anim)
 {
 	int i;
 
@@ -1194,7 +1250,7 @@ void IMB_free_indices(struct anim * anim)
 	anim->indices_tried = 0;
 }
 
-void IMB_anim_set_index_dir(struct anim * anim, const char * dir)
+void IMB_anim_set_index_dir(struct anim *anim, const char *dir)
 {
 	if (strcmp(anim->index_dir, dir) == 0) {
 		return;
@@ -1204,8 +1260,8 @@ void IMB_anim_set_index_dir(struct anim * anim, const char * dir)
 	IMB_free_indices(anim);
 }
 
-struct anim * IMB_anim_open_proxy(
-	struct anim * anim, IMB_Proxy_Size preview_size)
+struct anim *IMB_anim_open_proxy(
+        struct anim *anim, IMB_Proxy_Size preview_size)
 {
 	char fname[FILE_MAX];
 	int i = IMB_proxy_size_to_array_index(preview_size);
@@ -1220,15 +1276,16 @@ struct anim * IMB_anim_open_proxy(
 
 	get_proxy_filename(anim, preview_size, fname, FALSE);
 
-	anim->proxy_anim[i] = IMB_open_anim(fname, 0, 0);
+	/* proxies are generated in default color space */
+	anim->proxy_anim[i] = IMB_open_anim(fname, 0, 0, NULL);
 	
 	anim->proxies_tried |= preview_size;
 
 	return anim->proxy_anim[i];
 }
 
-struct anim_index * IMB_anim_open_index(
-	struct anim * anim, IMB_Timecode_Type tc)
+struct anim_index *IMB_anim_open_index(
+        struct anim *anim, IMB_Timecode_Type tc)
 {
 	char fname[FILE_MAX];
 	int i = IMB_timecode_to_array_index(tc);
@@ -1250,10 +1307,10 @@ struct anim_index * IMB_anim_open_index(
 	return anim->curr_idx[i];
 }
 
-int IMB_anim_index_get_frame_index(struct anim * anim, IMB_Timecode_Type tc,
-				   int position)
+int IMB_anim_index_get_frame_index(struct anim *anim, IMB_Timecode_Type tc,
+                                   int position)
 {
-	struct anim_index * idx = IMB_anim_open_index(anim, tc);
+	struct anim_index *idx = IMB_anim_open_index(anim, tc);
 
 	if (!idx) {
 		return position;
