@@ -33,7 +33,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "BLI_blenlib.h"
+#include "BLI_string.h"
+
 #include "MEM_guardedalloc.h"
 
 #include "IMB_imbuf_types.h"
@@ -43,7 +44,7 @@
 
 
 
-void IMB_metadata_free(struct ImBuf* img)
+void IMB_metadata_free(struct ImBuf *img)
 {
 	ImMetaData *info;
 
@@ -54,7 +55,7 @@ void IMB_metadata_free(struct ImBuf* img)
 	}
 	info = img->metadata;
 	while (info) {
-		ImMetaData* next = info->next;
+		ImMetaData *next = info->next;
 		MEM_freeN(info->key);
 		MEM_freeN(info->value);
 		MEM_freeN(info);
@@ -62,7 +63,7 @@ void IMB_metadata_free(struct ImBuf* img)
 	}
 }
 
-int IMB_metadata_get_field(struct ImBuf* img, const char* key, char* field, int len)
+int IMB_metadata_get_field(struct ImBuf *img, const char *key, char *field, const size_t len)
 {
 	ImMetaData *info;
 	int retval = 0;
@@ -84,7 +85,7 @@ int IMB_metadata_get_field(struct ImBuf* img, const char* key, char* field, int
 	return retval;
 }
 
-int IMB_metadata_add_field(struct ImBuf* img, const char* key, const char* value)
+int IMB_metadata_add_field(struct ImBuf *img, const char *key, const char *value)
 {
 	ImMetaData *info;
 	ImMetaData *last;
@@ -121,7 +122,7 @@ int IMB_metadata_del_field(struct ImBuf *img, const char *key)
 	p = img->metadata;
 	p1 = NULL;
 	while (p) {
-		if (!strcmp (key, p->key)) {
+		if (!strcmp(key, p->key)) {
 			if (p1)
 				p1->next = p->next;
 			else
@@ -146,18 +147,18 @@ int IMB_metadata_change_field(struct ImBuf *img, const char *key, const char *fi
 		return (0);
 
 	if (!img->metadata)
-		return (IMB_metadata_add_field (img, key, field));
+		return (IMB_metadata_add_field(img, key, field));
 
 	p = img->metadata;
 	while (p) {
-		if (!strcmp (key, p->key)) {
-			MEM_freeN (p->value);
-			p->value = BLI_strdup (field);
+		if (!strcmp(key, p->key)) {
+			MEM_freeN(p->value);
+			p->value = BLI_strdup(field);
 			return (1);
 		}
 		p = p->next;
 	}
 
-	return (IMB_metadata_add_field (img, key, field));
+	return (IMB_metadata_add_field(img, key, field));
 }
 
