@@ -42,7 +42,7 @@ struct Text;
 struct Ipo;
 
 /* channels reside in Object or Action (ListBase) constraintChannels */
-// XXX depreceated... old AnimSys
+// XXX deprecated... old AnimSys
 typedef struct bConstraintChannel {
 	struct bConstraintChannel *next, *prev;
 	struct Ipo			*ipo;
@@ -104,9 +104,9 @@ typedef enum B_CONSTRAINT_TARGET_FLAG {
 /* bConstraintTarget/bConstraintOb -> type */
 typedef enum B_CONSTRAINT_OB_TYPE {
 	CONSTRAINT_OBTYPE_OBJECT = 1,	/*	string is ""				*/
-	CONSTRAINT_OBTYPE_BONE,			/*	string is bone-name		*/
-	CONSTRAINT_OBTYPE_VERT,			/*	string is vertex-group name 	*/
-	CONSTRAINT_OBTYPE_CV			/* 	string is vertex-group name - is not available until curves get vgroups */
+	CONSTRAINT_OBTYPE_BONE   = 2,	/*	string is bone-name		*/
+	CONSTRAINT_OBTYPE_VERT   = 3,	/*	string is vertex-group name 	*/
+	CONSTRAINT_OBTYPE_CV     = 4	/*	string is vertex-group name - is not available until curves get vgroups */
 } B_CONSTRAINT_OB_TYPE;
 
 
@@ -152,7 +152,7 @@ typedef struct bKinematicConstraint {
 
 typedef enum B_CONSTRAINT_IK_TYPE {
 	CONSTRAINT_IK_COPYPOSE = 0,		/* 'standard' IK constraint: match position and/or orientation of target */
-	CONSTRAINT_IK_DISTANCE			/* maintain distance with target */
+	CONSTRAINT_IK_DISTANCE = 1		/* maintain distance with target */
 } B_CONSTRAINT_IK_TYPE;
 
 
@@ -170,7 +170,7 @@ typedef struct bSplineIKConstraint {
 	short 		numpoints;	/* number of points to bound in points array */
 	short		chainlen;	/* number of bones ('n') that are in the chain */
 	
-		/* settings */	
+		/* settings */
 	short flag;				/* general settings for constraint */
 	short xzScaleMode;		/* method used for determining the x & z scaling of the bones */
 } bSplineIKConstraint;
@@ -182,7 +182,7 @@ typedef struct bSplineIKConstraint {
 typedef struct bTrackToConstraint {
 	struct Object		*tar;
 	int			reserved1; /* I'll be using reserved1 and reserved2 as Track and Up flags,
-	                        * not sure if that's what they were intented for anyway.
+	                        * not sure if that's what they were intended for anyway.
 	                        * Not sure either if it would create backward incompatibility if I were to rename them.
 	                        * - theeth*/
 	int			reserved2;
@@ -247,7 +247,7 @@ typedef struct bActionConstraint {
 	int			end;
 	float		min;
 	float		max;
-	int         pad;
+	int			flag;
 	struct bAction	*act;
 	char		subtarget[64];	/* MAX_ID_NAME-2 */
 } bActionConstraint;
@@ -332,7 +332,7 @@ typedef struct bTransformConstraint {
 	struct Object 		*tar;			/* target (i.e. 'driver' object/bone) */
 	char 		subtarget[64];		/* MAX_ID_NAME-2 */
 	
-	short		from, to;		/* can be loc(0) , rot(1),  or size(2) */
+	short		from, to;		/* can be loc(0), rot(1) or size(2) */
 	char		map[3];			/* defines which target-axis deform is copied by each owner-axis */
 	char		expo;			/* extrapolate motion? if 0, confine to ranges */
 	
@@ -340,7 +340,7 @@ typedef struct bTransformConstraint {
 	float		from_max[3];	/* 	to map on to to_min/max range. 			*/
 	
 	float		to_min[3];		/* range of motion on owner caused by target  */
-	float		to_max[3];	
+	float		to_max[3];
 } bTransformConstraint;
 
 /* Pivot Constraint */
@@ -380,7 +380,7 @@ typedef struct bRotLimitConstraint {
 	short		flag2;
 } bRotLimitConstraint;
 
-/* Limit Scaling Constraint */
+/* Limit Scale Constraint */
 typedef struct bSizeLimitConstraint {
 	float 		xmin, xmax;
 	float		ymin, ymax;
@@ -415,7 +415,8 @@ typedef struct bShrinkwrapConstraint {
 typedef struct bFollowTrackConstraint {
 	struct MovieClip	*clip;
 	char	track[64];	/* MAX_NAME */
-	int		flag, pad;
+	int		flag;
+	int		frame_method;
 	char		object[64];	/* MAX_NAME */
 	struct Object *camera;
 	struct Object *depth_ob;
@@ -444,34 +445,34 @@ typedef struct bObjectSolverConstraint {
  */
 typedef enum eBConstraint_Types {
 	CONSTRAINT_TYPE_NULL = 0,			/* Invalid/legacy constraint */
-	CONSTRAINT_TYPE_CHILDOF,			/* Unimplemented non longer :) - during constraints recode, Aligorith */
-	CONSTRAINT_TYPE_TRACKTO,
-	CONSTRAINT_TYPE_KINEMATIC,
-	CONSTRAINT_TYPE_FOLLOWPATH,
-	CONSTRAINT_TYPE_ROTLIMIT,			/* Unimplemented no longer :) - Aligorith */
-	CONSTRAINT_TYPE_LOCLIMIT,			/* Unimplemented no longer :) - Aligorith */
-	CONSTRAINT_TYPE_SIZELIMIT,			/* Unimplemented no longer :) - Aligorith */
-	CONSTRAINT_TYPE_ROTLIKE,	
-	CONSTRAINT_TYPE_LOCLIKE,	
-	CONSTRAINT_TYPE_SIZELIKE,
-	CONSTRAINT_TYPE_PYTHON,				/* Unimplemented no longer :) - Aligorith. Scripts */
-	CONSTRAINT_TYPE_ACTION,
-	CONSTRAINT_TYPE_LOCKTRACK,			/* New Tracking constraint that locks an axis in place - theeth */
-	CONSTRAINT_TYPE_DISTLIMIT,			/* limit distance */
-	CONSTRAINT_TYPE_STRETCHTO, 			/* claiming this to be mine :) is in tuhopuu bjornmose */ 
-	CONSTRAINT_TYPE_MINMAX,  			/* floor constraint */
-	CONSTRAINT_TYPE_RIGIDBODYJOINT,		/* rigidbody constraint */
-	CONSTRAINT_TYPE_CLAMPTO, 			/* clampto constraint */	
-	CONSTRAINT_TYPE_TRANSFORM,			/* transformation (loc/rot/size -> loc/rot/size) constraint */	
-	CONSTRAINT_TYPE_SHRINKWRAP,			/* shrinkwrap (loc/rot) constraint */
-	CONSTRAINT_TYPE_DAMPTRACK,			/* New Tracking constraint that minimises twisting */
-	CONSTRAINT_TYPE_SPLINEIK,			/* Spline-IK - Align 'n' bones to a curve */
-	CONSTRAINT_TYPE_TRANSLIKE,			/* Copy transform matrix */
-	CONSTRAINT_TYPE_SAMEVOL,			/* Maintain volume during scaling */
-	CONSTRAINT_TYPE_PIVOT,				/* Pivot Constraint */
-	CONSTRAINT_TYPE_FOLLOWTRACK,		/* Follow Track Constraint */
-	CONSTRAINT_TYPE_CAMERASOLVER,		/* Camera Solver Constraint */
-	CONSTRAINT_TYPE_OBJECTSOLVER,		/* Object Solver Constraint */
+	CONSTRAINT_TYPE_CHILDOF = 1,			/* Unimplemented non longer :) - during constraints recode, Aligorith */
+	CONSTRAINT_TYPE_TRACKTO = 2,
+	CONSTRAINT_TYPE_KINEMATIC = 3,
+	CONSTRAINT_TYPE_FOLLOWPATH = 4,
+	CONSTRAINT_TYPE_ROTLIMIT = 5,			/* Unimplemented no longer :) - Aligorith */
+	CONSTRAINT_TYPE_LOCLIMIT = 6,			/* Unimplemented no longer :) - Aligorith */
+	CONSTRAINT_TYPE_SIZELIMIT = 7,			/* Unimplemented no longer :) - Aligorith */
+	CONSTRAINT_TYPE_ROTLIKE = 8,
+	CONSTRAINT_TYPE_LOCLIKE = 9,
+	CONSTRAINT_TYPE_SIZELIKE = 10,
+	CONSTRAINT_TYPE_PYTHON = 11,				/* Unimplemented no longer :) - Aligorith. Scripts */
+	CONSTRAINT_TYPE_ACTION = 12,
+	CONSTRAINT_TYPE_LOCKTRACK = 13,			/* New Tracking constraint that locks an axis in place - theeth */
+	CONSTRAINT_TYPE_DISTLIMIT = 14,			/* limit distance */
+	CONSTRAINT_TYPE_STRETCHTO = 15,			/* claiming this to be mine :) is in tuhopuu bjornmose */ 
+	CONSTRAINT_TYPE_MINMAX = 16,  			/* floor constraint */
+	CONSTRAINT_TYPE_RIGIDBODYJOINT = 17,		/* rigidbody constraint */
+	CONSTRAINT_TYPE_CLAMPTO = 18, 			/* clampto constraint */
+	CONSTRAINT_TYPE_TRANSFORM = 19,			/* transformation (loc/rot/size -> loc/rot/size) constraint */
+	CONSTRAINT_TYPE_SHRINKWRAP = 20,		/* shrinkwrap (loc/rot) constraint */
+	CONSTRAINT_TYPE_DAMPTRACK = 21,			/* New Tracking constraint that minimizes twisting */
+	CONSTRAINT_TYPE_SPLINEIK = 22,			/* Spline-IK - Align 'n' bones to a curve */
+	CONSTRAINT_TYPE_TRANSLIKE = 23,			/* Copy transform matrix */
+	CONSTRAINT_TYPE_SAMEVOL = 24,			/* Maintain volume during scaling */
+	CONSTRAINT_TYPE_PIVOT = 25,			/* Pivot Constraint */
+	CONSTRAINT_TYPE_FOLLOWTRACK = 26,		/* Follow Track Constraint */
+	CONSTRAINT_TYPE_CAMERASOLVER = 27,		/* Camera Solver Constraint */
+	CONSTRAINT_TYPE_OBJECTSOLVER = 28,		/* Object Solver Constraint */
 	
 	/* NOTE: no constraints are allowed to be added after this */
 	NUM_CONSTRAINT_TYPES
@@ -485,7 +486,7 @@ typedef enum eBConstraint_Flags {
 	CONSTRAINT_EXPAND =		(1<<0), 
 		/* pre-check for illegal object name or bone name */
 	CONSTRAINT_DISABLE = 	(1<<2), 
-		/* to indicate which Ipo should be shown, maybe for 3d access later too */	
+		/* to indicate which Ipo should be shown, maybe for 3d access later too */
 	CONSTRAINT_ACTIVE = 	(1<<4), 
 		/* to indicate that the owner's space should only be changed into ownspace, but not out of it */
 	CONSTRAINT_SPACEONCE = 	(1<<6),
@@ -504,17 +505,17 @@ typedef enum eBConstraint_SpaceTypes {
 		/* for objects (relative to parent/without parent influence), 
 		 * for bones (along normals of bone, without parent/restpositions) 
 		 */
-	CONSTRAINT_SPACE_LOCAL, /* = 1 */
+	CONSTRAINT_SPACE_LOCAL = 1,
 		/* for posechannels - pose space  */
-	CONSTRAINT_SPACE_POSE, /* = 2 */
+	CONSTRAINT_SPACE_POSE = 2,
 		 /* for posechannels - local with parent  */
-	CONSTRAINT_SPACE_PARLOCAL, /* = 3 */
+	CONSTRAINT_SPACE_PARLOCAL = 3,
 		/* for files from between 2.43-2.46 (should have been parlocal) */
-	CONSTRAINT_SPACE_INVALID /* = 4. do not exchange for anything! */
+	CONSTRAINT_SPACE_INVALID = 4 /* do not exchange for anything! */
 } eBConstraint_SpaceTypes;
 
 /* bConstraintChannel.flag */
-// XXX depreceated... old AnimSys
+// XXX deprecated... old AnimSys
 typedef enum eConstraintChannel_Flags {
 	CONSTRAINT_CHANNEL_SELECT =		(1<<0),
 	CONSTRAINT_CHANNEL_PROTECTED =	(1<<1)
@@ -538,7 +539,7 @@ typedef enum eCopyLocation_Flags {
 	LOCLIKE_X			= (1<<0),
 	LOCLIKE_Y			= (1<<1),
 	LOCLIKE_Z			= (1<<2),
-		/* LOCLIKE_TIP is a depreceated option... use headtail=1.0f instead */
+		/* LOCLIKE_TIP is a deprecated option... use headtail=1.0f instead */
 	LOCLIKE_TIP			= (1<<3),
 	LOCLIKE_X_INVERT	= (1<<4),
 	LOCLIKE_Y_INVERT	= (1<<5),
@@ -557,32 +558,38 @@ typedef enum eCopyScale_Flags {
 /* bSameVolumeConstraint.flag */
 typedef enum eSameVolume_Modes {
 	SAMEVOL_X		= 0,
-	SAMEVOL_Y,
-	SAMEVOL_Z
+	SAMEVOL_Y		= 1,
+	SAMEVOL_Z		= 2
 } eSameVolume_Modes;
 
+/* bActionConstraint.flag */
+typedef enum eActionConstraint_Flags {
+	/* Bones use "object" part of target action, instead of "same bone name" part */
+	ACTCON_BONE_USE_OBJECT_ACTION = (1 << 0), 
+} eActionConstraint_Flags;
+
 /* Locked-Axis Values (Locked Track) */
 typedef enum eLockAxis_Modes {
-	LOCK_X	= 0,
-	LOCK_Y,
-	LOCK_Z
+	LOCK_X = 0,
+	LOCK_Y = 1,
+	LOCK_Z = 2
 } eLockAxis_Modes;
 
 /* Up-Axis Values (TrackTo and Locked Track) */
 typedef enum eUpAxis_Modes {
-	UP_X	= 0,
-	UP_Y,
-	UP_Z
+	UP_X = 0,
+	UP_Y = 1,
+	UP_Z = 2
 } eUpAxis_Modes;
 
 /* Tracking axis (TrackTo, Locked Track, Damped Track) and minmax (floor) constraint */
 typedef enum eTrackToAxis_Modes {
 	TRACK_X		= 0,
-	TRACK_Y,
-	TRACK_Z,
-	TRACK_nX,
-	TRACK_nY,
-	TRACK_nZ
+	TRACK_Y		= 1,
+	TRACK_Z		= 2,
+	TRACK_nX	= 3,
+	TRACK_nY	= 4,
+	TRACK_nZ	= 5
 } eTrackToAxis_Modes;
 
 /* FollowPath flags */
@@ -600,24 +607,24 @@ typedef enum eTrackTo_Flags {
 /* Strech To Constraint -> volmode */
 typedef enum eStretchTo_VolMode {
 	VOLUME_XZ	= 0,
-	VOLUME_X,
-	VOLUME_Z,
-	NO_VOLUME
+	VOLUME_X	= 1,
+	VOLUME_Z	= 2,
+	NO_VOLUME	= 3
 } eStretchTo_VolMode;
 
 /* Stretch To Constraint -> plane mode */
 typedef enum eStretchTo_PlaneMode {
 	PLANE_X		= 0,
-	PLANE_Y,
-	PLANE_Z
+	PLANE_Y		= 1,
+	PLANE_Z		= 2
 } eStretchTo_PlaneMode;
 
 /* Clamp-To Constraint ->flag */
 typedef enum eClampTo_Modes {
 	CLAMPTO_AUTO = 0,
-	CLAMPTO_X,
-	CLAMPTO_Y,
-	CLAMPTO_Z
+	CLAMPTO_X    = 1,
+	CLAMPTO_Y    = 2,
+	CLAMPTO_Z    = 3
 } eClampTo_Modes;
 
 /* ClampTo Constraint ->flag2 */
@@ -657,7 +664,7 @@ typedef enum eSplineIK_Flags {
 	/* bones in the chain should not scale to fit the curve */
 	CONSTRAINT_SPLINEIK_SCALE_LIMITED	= (1<<2),
 	/* evenly distribute the bones along the path regardless of length */
-	CONSTRAINT_SPLINEIK_EVENSPLITS		= (1<<3),	
+	CONSTRAINT_SPLINEIK_EVENSPLITS		= (1<<3),
 	/* don't adjust the x and z scaling of the bones by the curve radius */
 	CONSTRAINT_SPLINEIK_NO_CURVERAD	= (1<<4)
 } eSplineIK_Flags;
@@ -667,9 +674,9 @@ typedef enum eSplineIK_XZScaleModes {
 	/* no x/z scaling */
 	CONSTRAINT_SPLINEIK_XZS_NONE			= 0,
 	/* bones in the chain should take their x/z scales from the original scaling */
-	CONSTRAINT_SPLINEIK_XZS_ORIGINAL,
+	CONSTRAINT_SPLINEIK_XZS_ORIGINAL		= 1,
 	/* x/z scales are the inverse of the y-scale */
-	CONSTRAINT_SPLINEIK_XZS_VOLUMETRIC
+	CONSTRAINT_SPLINEIK_XZS_VOLUMETRIC		= 2
 } eSplineIK_XZScaleModes;
 
 /* MinMax (floor) flags */
@@ -715,9 +722,9 @@ typedef enum eDistLimit_Flag {
 
 	/* bDistLimitConstraint->mode */
 typedef enum eDistLimit_Modes {
-	LIMITDIST_INSIDE = 0,
-	LIMITDIST_OUTSIDE,
-	LIMITDIST_ONSURFACE
+	LIMITDIST_INSIDE    = 0,
+	LIMITDIST_OUTSIDE   = 1,
+	LIMITDIST_ONSURFACE = 2
 } eDistLimit_Modes;
 	
 /* python constraint -> flag */
@@ -744,21 +751,21 @@ typedef enum eChildOf_Flags {
 	/* Restrictions for Pivot Constraint axis to consider for enabling constraint */
 typedef enum ePivotConstraint_Axis {
 	/* do not consider this activity-clamping */
-	PIVOTCON_AXIS_NONE = -1,	
+	PIVOTCON_AXIS_NONE = -1,
 	
 	/* consider -ve x-axis rotations */
-	PIVOTCON_AXIS_X_NEG,
+	PIVOTCON_AXIS_X_NEG = 0,
 	/* consider -ve y-axis rotations */
-	PIVOTCON_AXIS_Y_NEG,
+	PIVOTCON_AXIS_Y_NEG = 1,
 	/* consider -ve z-axis rotations */
-	PIVOTCON_AXIS_Z_NEG,
+	PIVOTCON_AXIS_Z_NEG = 2,
 	
 	/* consider +ve x-axis rotations */
-	PIVOTCON_AXIS_X,
+	PIVOTCON_AXIS_X = 3,
 	/* consider +ve y-axis rotations */
-	PIVOTCON_AXIS_Y,
+	PIVOTCON_AXIS_Y = 4,
 	/* consider +ve z-axis rotations */
-	PIVOTCON_AXIS_Z
+	PIVOTCON_AXIS_Z = 5
 } ePivotConstraint_Axis;
 
 	/* settings for Pivot Constraint in general */
@@ -774,6 +781,12 @@ typedef enum eFollowTrack_Flags {
 	FOLLOWTRACK_USE_3D_POSITION	= (1<<1)
 } eFollowTrack_Flags;
 
+typedef enum eFollowTrack_FrameMethod {
+	FOLLOWTRACK_FRAME_STRETCH = 0,
+	FOLLOWTRACK_FRAME_FIT = 1,
+	FOLLOWTRACK_FRAME_CROP = 2
+} eFollowTrack_FrameMethod;
+
 /* CameraSolver Constraint -> flag */
 typedef enum eCameraSolver_Flags {
 	CAMERASOLVER_ACTIVECLIP	= (1<<0)
