@@ -37,8 +37,6 @@
 #include "DNA_ID.h"
 #include "DNA_customdata_types.h"
 
-#include "DNA_defs.h" /* USE_BMESH_FORWARD_COMPAT */
-
 struct AnimData;
 struct DerivedMesh;
 struct Ipo;
@@ -89,10 +87,10 @@ typedef struct Mesh {
 
 	/* array of colors for the tessellated faces, must be number of tessellated
 	 * faces * 4 in length */
-	struct MCol *mcol;		
-	struct MSticky *msticky;
+	struct MCol *mcol;
 	struct Mesh *texcomesh;
-	
+
+	/* When the object is available, the preferred access method is: BMEdit_FromObject(ob) */
 	struct BMEditMesh *edit_btmesh;	/* not saved in file! */
 
 	struct CustomData vdata, edata, fdata;
@@ -121,7 +119,10 @@ typedef struct Mesh {
 	short texflag, drawflag;
 	short smoothresh, flag;
 
-	short subdiv  DNA_DEPRECATED, subdivr  DNA_DEPRECATED;
+	/* customdata flag, for bevel-weight and crease, which are now optional */
+	char cd_flag, pad;
+
+	char subdiv  DNA_DEPRECATED, subdivr  DNA_DEPRECATED;
 	char subsurftype  DNA_DEPRECATED; /* only kept for backwards compat, not used anymore */
 	char editflag;
 
@@ -149,15 +150,15 @@ typedef struct TFace {
 #define ME_EDIT_MIRROR_Y (1 << 1) // unused so far
 #define ME_EDIT_MIRROR_Z (1 << 2) // unused so far
 
-#define ME_EDIT_PAINT_MASK (1 << 3)
+#define ME_EDIT_PAINT_FACE_SEL (1 << 3)
 #define ME_EDIT_MIRROR_TOPO (1 << 4)
-#define ME_EDIT_VERT_SEL (1 << 5)
+#define ME_EDIT_PAINT_VERT_SEL (1 << 5)
 
 /* we cant have both flags enabled at once,
  * flags defined in DNA_scene_types.h */
 #define ME_EDIT_PAINT_SEL_MODE(_me)  (                                        \
-	(_me->editflag & ME_EDIT_PAINT_MASK) ? SCE_SELECT_FACE :                  \
-		(_me->editflag & ME_EDIT_VERT_SEL) ? SCE_SELECT_VERTEX :              \
+	(_me->editflag & ME_EDIT_PAINT_FACE_SEL) ? SCE_SELECT_FACE :              \
+		(_me->editflag & ME_EDIT_PAINT_VERT_SEL) ? SCE_SELECT_VERTEX :        \
 			0                                                                 \
 	)
 
@@ -172,6 +173,12 @@ typedef struct TFace {
 #define ME_SUBSURF		128
 #define ME_OPT_EDGES	256
 #define ME_DS_EXPAND	512
+#define ME_SCULPT_DYNAMIC_TOPOLOGY 1024
+
+/* me->cd_flag */
+#define ME_CDFLAG_VERT_BWEIGHT (1 << 0)
+#define ME_CDFLAG_EDGE_BWEIGHT (1 << 1)
+#define ME_CDFLAG_EDGE_CREASE  (1 << 2)
 
 /* me->drawflag, short */
 #define ME_DRAWEDGES	(1 << 0)
@@ -179,7 +186,7 @@ typedef struct TFace {
 #define ME_DRAWNORMALS	(1 << 2)
 #define ME_DRAW_VNORMALS (1 << 3)
 
-#define ME_ALLEDGES		(1 << 4)
+// #define ME_ALLEDGES		(1 << 4)
 #define ME_HIDDENEDGES  (1 << 5)
 
 #define ME_DRAWCREASES	(1 << 6)
