@@ -28,55 +28,57 @@
 #include "DNA_defs.h"
 #include "DNA_listBase.h"
 
-
-#define MODSTACK_DEBUG 1
-
 /* WARNING ALERT! TYPEDEF VALUES ARE WRITTEN IN FILES! SO DO NOT CHANGE!
  * (ONLY ADD NEW ITEMS AT THE END) */
 
 typedef enum ModifierType {
-	eModifierType_None = 0,
-	eModifierType_Subsurf,
-	eModifierType_Lattice,
-	eModifierType_Curve,
-	eModifierType_Build,
-	eModifierType_Mirror,
-	eModifierType_Decimate,
-	eModifierType_Wave,
-	eModifierType_Armature,
-	eModifierType_Hook,
-	eModifierType_Softbody,
-	eModifierType_Boolean,
-	eModifierType_Array,
-	eModifierType_EdgeSplit,
-	eModifierType_Displace,
-	eModifierType_UVProject,
-	eModifierType_Smooth,
-	eModifierType_Cast,
-	eModifierType_MeshDeform,
-	eModifierType_ParticleSystem,
-	eModifierType_ParticleInstance,
-	eModifierType_Explode,
-	eModifierType_Cloth,
-	eModifierType_Collision,
-	eModifierType_Bevel,
-	eModifierType_Shrinkwrap,
-	eModifierType_Fluidsim,
-	eModifierType_Mask,
-	eModifierType_SimpleDeform,
-	eModifierType_Multires,
-	eModifierType_Surface,
-	eModifierType_Smoke,
-	eModifierType_ShapeKey,
-	eModifierType_Solidify,
-	eModifierType_Screw,
-	eModifierType_Warp,
-	eModifierType_WeightVGEdit,
-	eModifierType_WeightVGMix,
-	eModifierType_WeightVGProximity,
-	eModifierType_Ocean,
-	eModifierType_DynamicPaint,
-	eModifierType_Remesh,
+	eModifierType_None              = 0,
+	eModifierType_Subsurf           = 1,
+	eModifierType_Lattice           = 2,
+	eModifierType_Curve             = 3,
+	eModifierType_Build             = 4,
+	eModifierType_Mirror            = 5,
+	eModifierType_Decimate          = 6,
+	eModifierType_Wave              = 7,
+	eModifierType_Armature          = 8,
+	eModifierType_Hook              = 9,
+	eModifierType_Softbody          = 10,
+	eModifierType_Boolean           = 11,
+	eModifierType_Array             = 12,
+	eModifierType_EdgeSplit         = 13,
+	eModifierType_Displace          = 14,
+	eModifierType_UVProject         = 15,
+	eModifierType_Smooth            = 16,
+	eModifierType_Cast              = 17,
+	eModifierType_MeshDeform        = 18,
+	eModifierType_ParticleSystem    = 19,
+	eModifierType_ParticleInstance  = 20,
+	eModifierType_Explode           = 21,
+	eModifierType_Cloth             = 22,
+	eModifierType_Collision         = 23,
+	eModifierType_Bevel             = 24,
+	eModifierType_Shrinkwrap        = 25,
+	eModifierType_Fluidsim          = 26,
+	eModifierType_Mask              = 27,
+	eModifierType_SimpleDeform      = 28,
+	eModifierType_Multires          = 29,
+	eModifierType_Surface           = 30,
+	eModifierType_Smoke             = 31,
+	eModifierType_ShapeKey          = 32,
+	eModifierType_Solidify          = 33,
+	eModifierType_Screw             = 34,
+	eModifierType_Warp              = 35,
+	eModifierType_WeightVGEdit      = 36,
+	eModifierType_WeightVGMix       = 37,
+	eModifierType_WeightVGProximity = 38,
+	eModifierType_Ocean             = 39,
+	eModifierType_DynamicPaint      = 40,
+	eModifierType_Remesh            = 41,
+	eModifierType_Skin              = 42,
+	eModifierType_LaplacianSmooth   = 43,
+	eModifierType_Triangulate       = 44,
+	eModifierType_UVWarp            = 45,
+	eModifierType_MeshCache         = 46,
 	NUM_MODIFIER_TYPES
 } ModifierType;
 
@@ -326,19 +328,19 @@ typedef struct DisplaceModifierData {
 
 /* DisplaceModifierData->direction */
 enum {
-	MOD_DISP_DIR_X,
-	MOD_DISP_DIR_Y,
-	MOD_DISP_DIR_Z,
-	MOD_DISP_DIR_NOR,
-	MOD_DISP_DIR_RGB_XYZ,
+	MOD_DISP_DIR_X = 0,
+	MOD_DISP_DIR_Y = 1,
+	MOD_DISP_DIR_Z = 2,
+	MOD_DISP_DIR_NOR = 3,
+	MOD_DISP_DIR_RGB_XYZ = 4,
 };
 
 /* DisplaceModifierData->texmapping */
 enum {
-	MOD_DISP_MAP_LOCAL,
-	MOD_DISP_MAP_GLOBAL,
-	MOD_DISP_MAP_OBJECT,
-	MOD_DISP_MAP_UV
+	MOD_DISP_MAP_LOCAL = 0,
+	MOD_DISP_MAP_GLOBAL = 1,
+	MOD_DISP_MAP_OBJECT = 2,
+	MOD_DISP_MAP_UV = 3
 };
 
 typedef struct UVProjectModifierData {
@@ -350,7 +352,7 @@ typedef struct UVProjectModifierData {
 	int flags;
 	int num_projectors;
 	float aspectx, aspecty;
-	float scalex, scaley;												
+	float scalex, scaley;
 	char uvlayer_name[64];	/* MAX_CUSTOMDATA_LAYER_NAME */
 	int uvlayer_tmp, pad;
 } UVProjectModifierData;
@@ -363,10 +365,30 @@ typedef struct UVProjectModifierData {
 typedef struct DecimateModifierData {
 	ModifierData modifier;
 
-	float percent;
-	int faceCount;
+	float percent;  /* (mode == MOD_DECIM_MODE_COLLAPSE) */
+	short   iter;   /* (mode == MOD_DECIM_MODE_UNSUBDIV) */
+	short   pad;
+	float   angle;  /* (mode == MOD_DECIM_MODE_DISSOLVE) */
+
+	char defgrp_name[64];	/* MAX_VGROUP_NAME */
+	short flag, mode;
+
+	/* runtime only */
+	int face_count, pad2;
 } DecimateModifierData;
 
+enum {
+	MOD_DECIM_FLAG_INVERT_VGROUP       = (1 << 0),
+	MOD_DECIM_FLAG_TRIANGULATE         = (1 << 1),  /* for collapse only. dont convert tri pairs back to quads */
+	MOD_DECIM_FLAG_ALL_BOUNDARY_VERTS  = (1 << 2)   /* for dissolve only. collapse all verts between 2 faces */
+};
+
+enum {
+	MOD_DECIM_MODE_COLLAPSE,
+	MOD_DECIM_MODE_UNSUBDIV,
+	MOD_DECIM_MODE_DISSOLVE   /* called planar in the UI */
+};
+
 /* Smooth modifier flags */
 #define MOD_SMOOTH_X (1<<1)
 #define MOD_SMOOTH_Y (1<<2)
@@ -508,9 +530,9 @@ typedef struct SurfaceModifierData {
 } SurfaceModifierData;
 
 typedef enum {
-	eBooleanModifierOp_Intersect,
-	eBooleanModifierOp_Union,
-	eBooleanModifierOp_Difference,
+	eBooleanModifierOp_Intersect = 0,
+	eBooleanModifierOp_Union = 1,
+	eBooleanModifierOp_Difference = 2,
 } BooleanModifierOp;
 typedef struct BooleanModifierData {
 	ModifierData modifier;
@@ -566,7 +588,7 @@ typedef struct MeshDeformModifierData {
 	/* runtime */
 	void (*bindfunc)(struct Scene *scene,
 		struct MeshDeformModifierData *mmd,
-		float *vertexcos, int totvert, float cagemat[][4]);
+		float *vertexcos, int totvert, float cagemat[4][4]);
 } MeshDeformModifierData;
 
 typedef enum {
@@ -646,7 +668,8 @@ typedef struct ShrinkwrapModifierData {
 	float keepDist;			/* distance offset to keep from mesh/projection point */
 	short shrinkType;		/* shrink type projection */
 	short shrinkOpts;		/* shrink options */
-	char projAxis;			/* axis to project over */
+	float projLimit;		/* limit the projection ray cast */
+	char  projAxis;			/* axis to project over */
 
 	/*
 	 * if using projection over vertex normal this controls the
@@ -655,7 +678,7 @@ typedef struct ShrinkwrapModifierData {
 	 */
 	char subsurfLevels;
 
-	char pad[6];
+	char pad[2];
 
 } ShrinkwrapModifierData;
 
@@ -685,7 +708,7 @@ typedef struct SimpleDeformModifierData {
 	struct Object *origin;	/* object to control the origin of modifier space coordinates */
 	char vgroup_name[64];	/* optional vertexgroup name, MAX_VGROUP_NAME */
 	float factor;			/* factors to control simple deforms */
-	float limit[2];			/* lower and upper limit */		
+	float limit[2];			/* lower and upper limit */
 
 	char mode;				/* deform function */
 	char axis;				/* lock axis (for taper and strech) */
@@ -704,6 +727,8 @@ typedef struct SimpleDeformModifierData {
 
 /* indicates whether simple deform should use the local
  * coordinates or global coordinates of origin */
+/* XXX, this should have never been an option, all other modifiers work relatively
+ * (so moving both objects makes no change!) - Campbell */
 #define MOD_SIMPLEDEFORM_ORIGIN_LOCAL			(1<<0)
 
 #define MOD_UVPROJECT_MAX				10
@@ -727,11 +752,12 @@ typedef struct SolidifyModifierData {
 	short mat_ofs_rim;
 } SolidifyModifierData;
 
-#define MOD_SOLIDIFY_RIM			(1<<0)
-#define MOD_SOLIDIFY_EVEN			(1<<1)
-#define MOD_SOLIDIFY_NORMAL_CALC	(1<<2)
-#define MOD_SOLIDIFY_VGROUP_INV		(1<<3)
-#define MOD_SOLIDIFY_RIM_MATERIAL	(1<<4) /* deprecated, used in do_versions */
+#define MOD_SOLIDIFY_RIM            (1 << 0)
+#define MOD_SOLIDIFY_EVEN           (1 << 1)
+#define MOD_SOLIDIFY_NORMAL_CALC    (1 << 2)
+#define MOD_SOLIDIFY_VGROUP_INV     (1 << 3)
+#define MOD_SOLIDIFY_RIM_MATERIAL   (1 << 4) /* deprecated, used in do_versions */
+#define MOD_SOLIDIFY_FLIP           (1 << 5)
 
 typedef struct ScrewModifierData {
 	ModifierData modifier;
@@ -745,10 +771,11 @@ typedef struct ScrewModifierData {
 	short	flag;
 } ScrewModifierData;
 
-#define MOD_SCREW_NORMAL_FLIP	(1<<0)
-#define MOD_SCREW_NORMAL_CALC	(1<<1)
-#define MOD_SCREW_OBJECT_OFFSET	(1<<2)
-// #define MOD_SCREW_OBJECT_ANGLE	(1<<4)
+#define MOD_SCREW_NORMAL_FLIP    (1 << 0)
+#define MOD_SCREW_NORMAL_CALC    (1 << 1)
+#define MOD_SCREW_OBJECT_OFFSET  (1 << 2)
+// #define MOD_SCREW_OBJECT_ANGLE	(1 << 4)
+#define MOD_SCREW_SMOOTH_SHADING (1 << 5)
 
 typedef struct OceanModifierData {
 	ModifierData modifier;
@@ -1032,6 +1059,7 @@ typedef struct DynamicPaintModifierData {
 
 typedef enum RemeshModifierFlags {
 	MOD_REMESH_FLOOD_FILL = 1,
+	MOD_REMESH_SMOOTH_SHADING = 2,
 } RemeshModifierFlags;
 
 typedef enum RemeshModifierMode {
@@ -1063,4 +1091,132 @@ typedef struct RemeshModifierData {
 	char pad;
 } RemeshModifierData;
 
-#endif
+/* Skin modifier */
+
+typedef struct SkinModifierData {
+	ModifierData modifier;
+
+	float branch_smoothing;
+
+	char flag;
+	
+	char symmetry_axes;
+
+	char pad[2];
+} SkinModifierData;
+
+/* SkinModifierData.symmetry_axes */
+enum {
+	MOD_SKIN_SYMM_X = 1,
+	MOD_SKIN_SYMM_Y = 2,
+	MOD_SKIN_SYMM_Z = 4,
+};
+
+/* SkinModifierData.flag */
+enum {
+	MOD_SKIN_SMOOTH_SHADING = 1
+};
+
+/* Triangulate modifier */
+
+typedef struct TriangulateModifierData {
+	ModifierData modifier;
+	int flag;
+	int pad;
+} TriangulateModifierData;
+
+enum {
+	MOD_TRIANGULATE_BEAUTY = (1 << 0),
+};
+
+/* Smooth modifier flags */
+#define MOD_LAPLACIANSMOOTH_X (1<<1)
+#define MOD_LAPLACIANSMOOTH_Y (1<<2)
+#define MOD_LAPLACIANSMOOTH_Z (1<<3)
+#define MOD_LAPLACIANSMOOTH_PRESERVE_VOLUME (1 << 4)
+#define MOD_LAPLACIANSMOOTH_NORMALIZED (1 << 5)
+
+typedef struct LaplacianSmoothModifierData {
+	ModifierData modifier;
+	float lambda, lambda_border, pad1;
+	char defgrp_name[64]; /* MAX_VGROUP_NAME */
+	short flag, repeat;
+} LaplacianSmoothModifierData;
+
+typedef struct UVWarpModifierData {
+	ModifierData modifier;
+
+	char axis_u, axis_v;
+	char pad[6];
+	float center[2];       /* used for rotate/scale */
+
+	struct Object *object_src;  /* source */
+	char bone_src[64];     /* optional name of bone target, MAX_ID_NAME-2 */
+	struct Object *object_dst;  /* target */
+	char bone_dst[64];     /* optional name of bone target, MAX_ID_NAME-2 */
+
+	char vgroup_name[64];   /* optional vertexgroup name, MAX_VGROUP_NAME */
+	char uvlayer_name[64];  /* MAX_CUSTOMDATA_LAYER_NAME */
+} UVWarpModifierData;
+
+/* cache modifier */
+typedef struct MeshCacheModifierData {
+	ModifierData modifier;
+	char flag;
+	char type;  /* file format */
+	char time_mode;
+	char play_mode;
+
+	/* axis conversion */
+	char forward_axis;
+	char up_axis;
+	char flip_axis;
+
+	char interp;
+
+	float factor;
+	char deform_mode;
+	char pad[7];
+
+	/* play_mode == MOD_MESHCACHE_PLAY_CFEA */
+	float frame_start;
+	float frame_scale;
+
+	/* play_mode == MOD_MESHCACHE_PLAY_EVAL */
+	/* we could use one float for all these but their purpose is very different */
+	float eval_frame;
+	float eval_time;
+	float eval_factor;
+
+	char filepath[1024];	// FILE_MAX
+} MeshCacheModifierData;
+
+enum {
+	MOD_MESHCACHE_TYPE_MDD  = 1,
+	MOD_MESHCACHE_TYPE_PC2  = 2
+};
+
+enum {
+	MOD_MESHCACHE_DEFORM_OVERWRITE  = 0,
+	MOD_MESHCACHE_DEFORM_INTEGRATE  = 1
+};
+
+enum {
+	MOD_MESHCACHE_INTERP_NONE  = 0,
+	MOD_MESHCACHE_INTERP_LINEAR = 1,
+	// MOD_MESHCACHE_INTERP_CARDINAL  = 2
+};
+
+enum {
+	MOD_MESHCACHE_TIME_FRAME = 0,
+	MOD_MESHCACHE_TIME_SECONDS = 1,
+	MOD_MESHCACHE_TIME_FACTOR = 2,
+};
+
+enum {
+	MOD_MESHCACHE_PLAY_CFEA = 0,
+	MOD_MESHCACHE_PLAY_EVAL = 1,
+};
+
+
+#endif  /* __DNA_MODIFIER_TYPES_H__ */
