@@ -44,15 +44,6 @@ typedef struct TextLine {
 	int len, blen; /* blen unused */
 } TextLine;
 
-typedef struct TextMarker {
-	struct TextMarker *next, *prev;
-
-	int lineno, start, end, pad1; /* line number and start/end character indices */
-	
-	int group, flags; /* see BKE_text.h for flag defines */
-	unsigned char color[4], pad[4]; /* draw color of the marker */
-} TextMarker;
-
 typedef struct Text {
 	ID id;
 	
@@ -63,7 +54,6 @@ typedef struct Text {
 	ListBase lines;
 	TextLine *curl, *sell;
 	int curc, selc;
-	ListBase markers;
 	
 	char *undo_buf;
 	int undo_pos, undo_len;
@@ -78,7 +68,6 @@ typedef struct Text {
 
 /* text flags */
 #define TXT_ISDIRTY             0x0001
-#define TXT_DEPRECATED          0x0004 /* deprecated ISTMP flag */
 #define TXT_ISMEM               0x0004
 #define TXT_ISEXT               0x0008
 #define TXT_ISSCRIPT            0x0010 /* used by space handler scriptlinks */
@@ -86,12 +75,4 @@ typedef struct Text {
 #define TXT_FOLLOW              0x0200 /* always follow cursor (console) */
 #define TXT_TABSTOSPACES        0x0400 /* use space instead of tabs */
 
-/* format continuation flags */
-#define TXT_NOCONT				0x00 /* no continuation */
-#define TXT_SNGQUOTSTR			0x01 /* single quotes */
-#define TXT_DBLQUOTSTR			0x02 /* double quotes */
-#define TXT_TRISTR				0x04 /* triplets of quotes: """ or ''' */
-#define TXT_SNGTRISTR			0x05 /*(TXT_TRISTR | TXT_SNGQUOTSTR)*/
-#define TXT_DBLTRISTR			0x06 /*(TXT_TRISTR | TXT_DBLQUOTSTR)*/
-
 #endif
