@@ -29,8 +29,8 @@
  *  \ingroup DNA
  */
 
-#ifndef DNA_WORLD_TYPES_H
-#define DNA_WORLD_TYPES_H
+#ifndef __DNA_WORLD_TYPES_H__
+#define __DNA_WORLD_TYPES_H__
 
 #include "DNA_defs.h"
 #include "DNA_ID.h"
@@ -61,14 +61,14 @@ typedef struct World {
 	float ambr, ambg, ambb;
 	float pad2;
 
-	unsigned int fastcol;	
+	unsigned int fastcol;
 	
 	/**
 	 * Exposure= mult factor. unused now, but maybe back later. Kept in to be upward compat.
 	 * New is exp/range control. linfac & logfac are constants... don't belong in
-	 * file, but allocating 8 bytes for temp mem isnt useful either.
+	 * file, but allocating 8 bytes for temp mem isn't useful either.
 	 */
-	float exposure, exp, range;	
+	float exposure, exp, range;
 	float linfac, logfac;
 
 	/**
@@ -129,7 +129,7 @@ typedef struct World {
 	struct PreviewImage *preview;
 
 	/* nodes */
-	struct bNodeTree *nodetree;	
+	struct bNodeTree *nodetree;
 
 } World;
 
