@@ -24,7 +24,6 @@
  *  \ingroup RNA
  */
 
-
 #include <stdlib.h>
 
 #include "RNA_access.h"
@@ -37,6 +36,8 @@
 #include "DNA_action_types.h"
 #include "DNA_scene_types.h"
 
+#include "BLI_utildefines.h"
+
 #include "MEM_guardedalloc.h"
 
 #include "ED_keyframing.h"
@@ -48,10 +49,27 @@ EnumPropertyItem keyingset_path_grouping_items[] = {
 	{KSP_GROUP_NAMED, "NAMED", 0, "Named Group", ""},
 	{KSP_GROUP_NONE, "NONE", 0, "None", ""},
 	{KSP_GROUP_KSNAME, "KEYINGSET", 0, "Keying Set Name", ""},
-	{0, NULL, 0, NULL, NULL}};
+	{0, NULL, 0, NULL, NULL}
+};
+
+/* It would be cool to get rid of this 'INSERTKEY_' prefix in 'py strings' values, but it would break existing
+ * exported keyingset... :/
+ */
+EnumPropertyItem keying_flag_items[] = {
+	{INSERTKEY_NEEDED, "INSERTKEY_NEEDED", 0, "Only Needed",
+	                   "Only insert keyframes where they're needed in the relevant F-Curves"},
+	{INSERTKEY_MATRIX, "INSERTKEY_VISUAL", 0, "Visual Keying",
+	                   "Insert keyframes based on 'visual transforms'"},
+	{INSERTKEY_XYZ2RGB, "INSERTKEY_XYZ_TO_RGB", 0, "XYZ=RGB Colors",
+	                    "Color for newly added transformation F-Curves (Location, Rotation, Scale) "
+	                    "and also Color is based on the transform axis"},
+	{0, NULL, 0, NULL, NULL}
+};
 
 #ifdef RNA_RUNTIME
 
+#include "BLI_math_base.h"
+
 #include "BKE_animsys.h"
 #include "BKE_fcurve.h"
 #include "BKE_nla.h"
@@ -92,6 +110,7 @@ static int RKS_POLL_rna_internal(KeyingSetInfo *ksi, bContext *C)
 	func = &rna_KeyingSetInfo_poll_func; /* RNA_struct_find_function(&ptr, "poll"); */
 
 	RNA_parameter_list_create(&list, &ptr, func);
+	{
 		/* hook up arguments */
 		RNA_parameter_set_lookup(&list, "ksi", &ksi);
 		RNA_parameter_set_lookup(&list, "context", &C);
@@ -101,7 +120,8 @@ static int RKS_POLL_rna_internal(KeyingSetInfo *ksi, bContext *C)
 		
 		/* read the result */
 		RNA_parameter_get_lookup(&list, "ok", &ret);
-		ok = *(int*)ret;
+		ok = *(int *)ret;
+	}
 	RNA_parameter_list_free(&list);
 	
 	return ok;
@@ -120,6 +140,7 @@ static void RKS_ITER_rna_internal(KeyingSetInfo *ksi, bContext *C, KeyingSet *ks
 	func = &rna_KeyingSetInfo_iterator_func; /* RNA_struct_find_function(&ptr, "poll"); */
 
 	RNA_parameter_list_create(&list, &ptr, func);
+	{
 		/* hook up arguments */
 		RNA_parameter_set_lookup(&list, "ksi", &ksi);
 		RNA_parameter_set_lookup(&list, "context", &C);
@@ -127,6 +148,7 @@ static void RKS_ITER_rna_internal(KeyingSetInfo *ksi, bContext *C, KeyingSet *ks
 		
 		/* execute the function */
 		ksi->ext.call(C, &ptr, func, &list);
+	}
 	RNA_parameter_list_free(&list);
 }
 
@@ -143,6 +165,7 @@ static void RKS_GEN_rna_internal(KeyingSetInfo *ksi, bContext *C, KeyingSet *ks,
 	func = &rna_KeyingSetInfo_generate_func; /* RNA_struct_find_generate(&ptr, "poll"); */
 
 	RNA_parameter_list_create(&list, &ptr, func);
+	{
 		/* hook up arguments */
 		RNA_parameter_set_lookup(&list, "ksi", &ksi);
 		RNA_parameter_set_lookup(&list, "context", &C);
@@ -151,6 +174,7 @@ static void RKS_GEN_rna_internal(KeyingSetInfo *ksi, bContext *C, KeyingSet *ks,
 		
 		/* execute the function */
 		ksi->ext.call(C, &ptr, func, &list);
+	}
 	RNA_parameter_list_free(&list);
 }
 
@@ -160,7 +184,7 @@ static void RKS_GEN_rna_internal(KeyingSetInfo *ksi, bContext *C, KeyingSet *ks,
 static StructRNA *rna_KeyingSetInfo_refine(PointerRNA *ptr)
 {
 	KeyingSetInfo *ksi = (KeyingSetInfo *)ptr->data;
-	return (ksi->ext.srna)? ksi->ext.srna: &RNA_KeyingSetInfo;
+	return (ksi->ext.srna) ? ksi->ext.srna : &RNA_KeyingSetInfo;
 }
 
 static void rna_KeyingSetInfo_unregister(Main *bmain, StructRNA *type)
@@ -174,6 +198,8 @@ static void rna_KeyingSetInfo_unregister(Main *bmain, StructRNA *type)
 	RNA_struct_free_extension(type, &ksi->ext);
 	RNA_struct_free(&BLENDER_RNA, type);
 	
+	WM_main_add_notifier(NC_WINDOW, NULL);
+
 	/* unlink Blender-side data */
 	ANIM_keyingset_info_unregister(bmain, ksi);
 }
@@ -201,7 +227,7 @@ static StructRNA *rna_KeyingSetInfo_register(Main *bmain, ReportList *reports, v
 	}
 	
 	/* check if we have registered this info before, and remove it */
-	ksi = ANIM_keyingset_info_find_named(dummyksi.idname);
+	ksi = ANIM_keyingset_info_find_name(dummyksi.idname);
 	if (ksi && ksi->ext.srna)
 		rna_KeyingSetInfo_unregister(bmain, ksi->ext.srna);
 	
@@ -210,7 +236,7 @@ static StructRNA *rna_KeyingSetInfo_register(Main *bmain, ReportList *reports, v
 	memcpy(ksi, &dummyksi, sizeof(KeyingSetInfo));
 	
 	/* set RNA-extensions info */
-	ksi->ext.srna = RNA_def_struct(&BLENDER_RNA, ksi->idname, "KeyingSetInfo");
+	ksi->ext.srna = RNA_def_struct_ptr(&BLENDER_RNA, ksi->idname, &RNA_KeyingSetInfo);
 	ksi->ext.data = data;
 	ksi->ext.call = call;
 	ksi->ext.free = free;
@@ -218,13 +244,15 @@ static StructRNA *rna_KeyingSetInfo_register(Main *bmain, ReportList *reports, v
 	
 	/* set callbacks */
 	/* NOTE: we really should have all of these...  */
-	ksi->poll = (have_function[0])? RKS_POLL_rna_internal: NULL;
-	ksi->iter = (have_function[1])? RKS_ITER_rna_internal: NULL;
-	ksi->generate = (have_function[2])? RKS_GEN_rna_internal: NULL;
+	ksi->poll = (have_function[0]) ? RKS_POLL_rna_internal : NULL;
+	ksi->iter = (have_function[1]) ? RKS_ITER_rna_internal : NULL;
+	ksi->generate = (have_function[2]) ? RKS_GEN_rna_internal : NULL;
 	
 	/* add and register with other info as needed */
 	ANIM_keyingset_info_register(ksi);
 	
+	WM_main_add_notifier(NC_WINDOW, NULL);
+
 	/* return the struct-rna added */
 	return ksi->ext.srna;
 }
@@ -233,19 +261,19 @@ static StructRNA *rna_KeyingSetInfo_register(Main *bmain, ReportList *reports, v
 
 static StructRNA *rna_ksPath_id_typef(PointerRNA *ptr)
 {
-	KS_Path *ksp = (KS_Path*)ptr->data;
+	KS_Path *ksp = (KS_Path *)ptr->data;
 	return ID_code_to_RNA_type(ksp->idtype);
 }
 
 static int rna_ksPath_id_editable(PointerRNA *ptr)
 {
-	KS_Path *ksp = (KS_Path*)ptr->data;
-	return (ksp->idtype)? PROP_EDITABLE : 0;
+	KS_Path *ksp = (KS_Path *)ptr->data;
+	return (ksp->idtype) ? PROP_EDITABLE : 0;
 }
 
 static void rna_ksPath_id_type_set(PointerRNA *ptr, int value)
 {
-	KS_Path *data = (KS_Path*)(ptr->data);
+	KS_Path *data = (KS_Path *)(ptr->data);
 	
 	/* set the driver type, then clear the id-block if the type is invalid */
 	data->idtype = value;
@@ -288,6 +316,43 @@ static void rna_ksPath_RnaPath_set(PointerRNA *ptr, const char *value)
 
 /* ****************************** */
 
+static void rna_KeyingSet_name_set(PointerRNA *ptr, const char *value)
+{
+	KeyingSet *ks = (KeyingSet *)ptr->data;
+	
+	/* update names of corresponding groups if name changes */
+	if (strcmp(ks->name, value)) {
+		KS_Path *ksp;
+		
+		for (ksp = ks->paths.first; ksp; ksp = ksp->next) {
+			if ((ksp->groupmode == KSP_GROUP_KSNAME) && (ksp->id)) {
+				AnimData *adt = BKE_animdata_from_id(ksp->id);
+				
+				/* TODO: NLA strips? */
+				if (adt && adt->action) {
+					bActionGroup *agrp;
+					
+					/* lazy check - should really find the F-Curve for the affected path and check its group 
+					 * but this way should be faster and work well for most cases, as long as there are no
+					 * conflicts
+					 */
+					for (agrp = adt->action->groups.first; agrp; agrp = agrp->next) {
+						if (strcmp(ks->name, agrp->name) == 0) {
+							/* there should only be one of these in the action, so can stop... */
+							BLI_strncpy(agrp->name, value, sizeof(agrp->name));
+							break;
+						}
+					}
+				}
+			}
+		}
+	}
+	
+	/* finally, update name to new value */
+	BLI_strncpy(ks->name, value, sizeof(ks->name));
+}
+
+
 static int rna_KeyingSet_active_ksPath_editable(PointerRNA *ptr)
 {
 	KeyingSet *ks = (KeyingSet *)ptr->data;
@@ -299,26 +364,26 @@ static int rna_KeyingSet_active_ksPath_editable(PointerRNA *ptr)
 static PointerRNA rna_KeyingSet_active_ksPath_get(PointerRNA *ptr)
 {
 	KeyingSet *ks = (KeyingSet *)ptr->data;
-	return rna_pointer_inherit_refine(ptr, &RNA_KeyingSetPath, BLI_findlink(&ks->paths, ks->active_path-1));
+	return rna_pointer_inherit_refine(ptr, &RNA_KeyingSetPath, BLI_findlink(&ks->paths, ks->active_path - 1));
 }
 
 static void rna_KeyingSet_active_ksPath_set(PointerRNA *ptr, PointerRNA value)
 {
 	KeyingSet *ks = (KeyingSet *)ptr->data;
-	KS_Path *ksp = (KS_Path*)value.data;
+	KS_Path *ksp = (KS_Path *)value.data;
 	ks->active_path = BLI_findindex(&ks->paths, ksp) + 1;
 }
 
 static int rna_KeyingSet_active_ksPath_index_get(PointerRNA *ptr)
 {
 	KeyingSet *ks = (KeyingSet *)ptr->data;
-	return MAX2(ks->active_path-1, 0);
+	return MAX2(ks->active_path - 1, 0);
 }
 
 static void rna_KeyingSet_active_ksPath_index_set(PointerRNA *ptr, int value)
 {
 	KeyingSet *ks = (KeyingSet *)ptr->data;
-	ks->active_path = value+1;
+	ks->active_path = value + 1;
 }
 
 static void rna_KeyingSet_active_ksPath_index_range(PointerRNA *ptr, int *min, int *max, int *softmin, int *softmax)
@@ -326,8 +391,7 @@ static void rna_KeyingSet_active_ksPath_index_range(PointerRNA *ptr, int *min, i
 	KeyingSet *ks = (KeyingSet *)ptr->data;
 
 	*min = 0;
-	*max = BLI_countlist(&ks->paths)-1;
-	*max = MAX2(0, *max);
+	*max = max_ii(0, BLI_countlist(&ks->paths) - 1);
 }
 
 static PointerRNA rna_KeyingSet_typeinfo_get(PointerRNA *ptr)
@@ -337,14 +401,14 @@ static PointerRNA rna_KeyingSet_typeinfo_get(PointerRNA *ptr)
 	
 	/* keying set info is only for builtin Keying Sets */
 	if ((ks->flag & KEYINGSET_ABSOLUTE) == 0)
-		ksi = ANIM_keyingset_info_find_named(ks->typeinfo);
+		ksi = ANIM_keyingset_info_find_name(ks->typeinfo);
 	return rna_pointer_inherit_refine(ptr, &RNA_KeyingSetInfo, ksi);
 }
 
 
 
 static KS_Path *rna_KeyingSet_paths_add(KeyingSet *keyingset, ReportList *reports,
-		ID *id, const char rna_path[], int index, int group_method, const char group_name[])
+                                        ID *id, const char rna_path[], int index, int group_method, const char group_name[])
 {
 	KS_Path *ksp = NULL;
 	short flag = 0;
@@ -361,27 +425,30 @@ static KS_Path *rna_KeyingSet_paths_add(KeyingSet *keyingset, ReportList *report
 		keyingset->active_path = BLI_countlist(&keyingset->paths);
 	}
 	else {
-		BKE_report(reports, RPT_ERROR, "Keying Set Path could not be added");
+		BKE_report(reports, RPT_ERROR, "Keying set path could not be added");
 	}
 	
 	/* return added path */
 	return ksp;
 }
 
-static void rna_KeyingSet_paths_remove(KeyingSet *keyingset, ReportList *reports, KS_Path *ksp)
+static void rna_KeyingSet_paths_remove(KeyingSet *keyingset, ReportList *reports, PointerRNA *ksp_ptr)
 {
+	KS_Path *ksp = ksp_ptr->data;
+
 	/* if data is valid, call the API function for this */
-	if (keyingset && ksp) {
-		/* remove the active path from the KeyingSet */
-		BKE_keyingset_free_path(keyingset, ksp);
-			
-		/* the active path number will most likely have changed */
-		/* TODO: we should get more fancy and actually check if it was removed, but this will do for now */
-		keyingset->active_path = 0;
-	}
-	else {
-		BKE_report(reports, RPT_ERROR, "Keying Set Path could not be removed");
+	if ((keyingset && ksp) == FALSE) {
+		BKE_report(reports, RPT_ERROR, "Keying set path could not be removed");
+		return;
 	}
+
+	/* remove the active path from the KeyingSet */
+	BKE_keyingset_free_path(keyingset, ksp);
+	RNA_POINTER_INVALIDATE(ksp_ptr);
+
+	/* the active path number will most likely have changed */
+	/* TODO: we should get more fancy and actually check if it was removed, but this will do for now */
+	keyingset->active_path = 0;
 }
 
 static void rna_KeyingSet_paths_clear(KeyingSet *keyingset, ReportList *reports)
@@ -400,7 +467,7 @@ static void rna_KeyingSet_paths_clear(KeyingSet *keyingset, ReportList *reports)
 		keyingset->active_path = 0;
 	}
 	else {
-		BKE_report(reports, RPT_ERROR, "Keying Set Paths could not be removed");
+		BKE_report(reports, RPT_ERROR, "Keying set paths could not be removed");
 	}
 }
 
@@ -409,29 +476,37 @@ static NlaTrack *rna_NlaTrack_new(AnimData *adt, bContext *C, NlaTrack *track)
 {
 	NlaTrack *new_track = add_nlatrack(adt, track);
 
-	WM_event_add_notifier(C, NC_ANIMATION|ND_NLA|NA_ADDED, NULL);
+	WM_event_add_notifier(C, NC_ANIMATION | ND_NLA | NA_ADDED, NULL);
 
 	return new_track;
 }
 
-static void rna_NlaTrack_remove(AnimData *adt, bContext *C, NlaTrack *track)
+static void rna_NlaTrack_remove(AnimData *adt, bContext *C, ReportList *reports, PointerRNA *track_ptr)
 {
-	free_nlatrack(&adt->nla_tracks, track);
+	NlaTrack *track = track_ptr->data;
 
-	WM_event_add_notifier(C, NC_ANIMATION|ND_NLA|NA_REMOVED, NULL);
+	if (BLI_findindex(&adt->nla_tracks, track) == -1) {
+		BKE_reportf(reports, RPT_ERROR, "NlaTrack '%s' cannot be removed", track->name);
+		return;
+	}
+
+	free_nlatrack(&adt->nla_tracks, track);
+	RNA_POINTER_INVALIDATE(track_ptr);
+
+	WM_event_add_notifier(C, NC_ANIMATION | ND_NLA | NA_REMOVED, NULL);
 }
 
 static PointerRNA rna_NlaTrack_active_get(PointerRNA *ptr)
 {
-	AnimData *adt = (AnimData*)ptr->data;
+	AnimData *adt = (AnimData *)ptr->data;
 	NlaTrack *track = BKE_nlatrack_find_active(&adt->nla_tracks);
 	return rna_pointer_inherit_refine(ptr, &RNA_NlaTrack, track);
 }
 
 static void rna_NlaTrack_active_set(PointerRNA *ptr, PointerRNA value)
 {
-	AnimData *adt = (AnimData*)ptr->data;
-	NlaTrack *track = (NlaTrack*)value.data;
+	AnimData *adt = (AnimData *)ptr->data;
+	NlaTrack *track = (NlaTrack *)value.data;
 	BKE_nlatrack_set_active(&adt->nla_tracks, track);
 }
 
@@ -440,7 +515,7 @@ static FCurve *rna_Driver_from_existing(AnimData *adt, bContext *C, FCurve *src_
 {
 	/* verify that we've got a driver to duplicate */
 	if (ELEM(NULL, src_driver, src_driver->driver)) {
-		BKE_reportf(CTX_wm_reports(C), RPT_ERROR, "No valid driver data to create copy of");
+		BKE_report(CTX_wm_reports(C), RPT_ERROR, "No valid driver data to create copy of");
 		return NULL;
 	}
 	else {
@@ -456,34 +531,26 @@ static FCurve *rna_Driver_from_existing(AnimData *adt, bContext *C, FCurve *src_
 #else
 
 /* helper function for Keying Set -> keying settings */
-static void rna_def_common_keying_flags(StructRNA *srna, short reg)
+/* TODO: use reg option! */
+static void rna_def_common_keying_flags(StructRNA *srna, short UNUSED(reg))
 {
 	PropertyRNA *prop;
 
-	static EnumPropertyItem keying_flag_items[] = {
-			{INSERTKEY_NEEDED, "INSERTKEY_NEEDED", 0, "Insert Keyframes - Only Needed",
-			                   "Only insert keyframes where they're needed in the relevant F-Curves"},
-			{INSERTKEY_MATRIX, "INSERTKEY_VISUAL", 0, "Insert Keyframes - Visual",
-			                   "Insert keyframes based on 'visual transforms'"},
-			{INSERTKEY_XYZ2RGB, "INSERTKEY_XYZ_TO_RGB", 0, "F-Curve Colors - XYZ to RGB",
-			                    "Color for newly added transformation F-Curves (Location, Rotation, Scale) "
-			                    "and also Color is based on the transform axis"},
-			{0, NULL, 0, NULL, NULL}};
-
 	prop = RNA_def_property(srna, "bl_options", PROP_ENUM, PROP_NONE);
 	RNA_def_property_enum_sdna(prop, NULL, "keyingflag");
 	RNA_def_property_enum_items(prop, keying_flag_items);
-	RNA_def_property_flag(prop, PROP_REGISTER_OPTIONAL|PROP_ENUM_FLAG);
+	RNA_def_property_flag(prop, PROP_REGISTER_OPTIONAL | PROP_ENUM_FLAG);
 	RNA_def_property_ui_text(prop, "Options",  "Keying set options");
 }
 
 /* --- */
 
 /* To avoid repeating it twice! */
-#define KEYINGSET_IDNAME_DOC "If this is set, the Keying Set gets a custom ID, otherwise it takes " \
-                             "the name of the class used to define the Keying Set (for example, "   \
-                             "if the class name is \"BUILTIN_KSI_location\", and bl_idname is not " \
-                             "set by the script, then bl_idname = \"BUILTIN_KSI_location\")"
+#define KEYINGSET_IDNAME_DOC \
+    "If this is set, the Keying Set gets a custom ID, otherwise it takes " \
+    "the name of the class used to define the Keying Set (for example, "   \
+    "if the class name is \"BUILTIN_KSI_location\", and bl_idname is not " \
+    "set by the script, then bl_idname = \"BUILTIN_KSI_location\")"
 
 
 static void rna_def_keyingset_info(BlenderRNA *brna)
@@ -505,7 +572,7 @@ static void rna_def_keyingset_info(BlenderRNA *brna)
 	
 	prop = RNA_def_property(srna, "bl_idname", PROP_STRING, PROP_NONE);
 	RNA_def_property_string_sdna(prop, NULL, "idname");
-	RNA_def_property_flag(prop, PROP_REGISTER|PROP_NEVER_CLAMP);
+	RNA_def_property_flag(prop, PROP_REGISTER | PROP_NEVER_CLAMP);
 	RNA_def_property_ui_text(prop, "ID Name", KEYINGSET_IDNAME_DOC);
 	
 	prop = RNA_def_property(srna, "bl_label", PROP_STRING, PROP_NONE);
@@ -514,7 +581,7 @@ static void rna_def_keyingset_info(BlenderRNA *brna)
 	RNA_def_struct_name_property(srna, prop);
 	RNA_def_property_flag(prop, PROP_REGISTER);
 	
-	prop = RNA_def_property(srna, "bl_description", PROP_STRING, PROP_TRANSLATE);
+	prop = RNA_def_property(srna, "bl_description", PROP_STRING, PROP_NONE);
 	RNA_def_property_string_sdna(prop, NULL, "description");
 	RNA_def_property_string_maxlength(prop, RNA_DYN_DESCR_MAX); /* else it uses the pointer size! */
 	RNA_def_property_flag(prop, PROP_REGISTER_OPTIONAL);
@@ -525,7 +592,7 @@ static void rna_def_keyingset_info(BlenderRNA *brna)
 	RNA_define_verify_sdna(1);
 	
 	/* Function Callbacks ------------- */
-		/* poll */
+	/* poll */
 	func = RNA_def_function(srna, "poll", NULL);
 	RNA_def_function_ui_description(func, "Test if Keying Set can be used or not");
 	RNA_def_function_flag(func, FUNC_REGISTER);
@@ -533,7 +600,7 @@ static void rna_def_keyingset_info(BlenderRNA *brna)
 	parm = RNA_def_pointer(func, "context", "Context", "", "");
 	RNA_def_property_flag(parm, PROP_REQUIRED);
 	
-		/* iterator */
+	/* iterator */
 	func = RNA_def_function(srna, "iterator", NULL);
 	RNA_def_function_ui_description(func, "Call generate() on the structs which have properties to be keyframed");
 	RNA_def_function_flag(func, FUNC_REGISTER);
@@ -542,7 +609,7 @@ static void rna_def_keyingset_info(BlenderRNA *brna)
 	parm = RNA_def_pointer(func, "ks", "KeyingSet", "", "");
 	RNA_def_property_flag(parm, PROP_REQUIRED);
 	
-		/* generate */
+	/* generate */
 	func = RNA_def_function(srna, "generate", NULL);
 	RNA_def_function_ui_description(func, "Add Paths to the Keying Set to keyframe the properties of the given data");
 	RNA_def_function_flag(func, FUNC_REGISTER);
@@ -551,7 +618,7 @@ static void rna_def_keyingset_info(BlenderRNA *brna)
 	parm = RNA_def_pointer(func, "ks", "KeyingSet", "", "");
 	RNA_def_property_flag(parm, PROP_REQUIRED);
 	parm = RNA_def_pointer(func, "data", "AnyType", "", "");
-	RNA_def_property_flag(parm, PROP_REQUIRED|PROP_RNAPTR|PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_RNAPTR | PROP_NEVER_NULL);
 }
 
 static void rna_def_keyingset_path(BlenderRNA *brna)
@@ -572,7 +639,7 @@ static void rna_def_keyingset_path(BlenderRNA *brna)
 	RNA_def_property_ui_text(prop, "ID-Block",
 	                         "ID-Block that keyframes for Keying Set should be added to "
 	                         "(for Absolute Keying Sets only)");
-	RNA_def_property_update(prop, NC_SCENE|ND_KEYINGSET|NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
+	RNA_def_property_update(prop, NC_SCENE | ND_KEYINGSET | NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
 	
 	prop = RNA_def_property(srna, "id_type", PROP_ENUM, PROP_NONE);
 	RNA_def_property_enum_sdna(prop, NULL, "idtype");
@@ -580,19 +647,19 @@ static void rna_def_keyingset_path(BlenderRNA *brna)
 	RNA_def_property_enum_default(prop, ID_OB);
 	RNA_def_property_enum_funcs(prop, NULL, "rna_ksPath_id_type_set", NULL);
 	RNA_def_property_ui_text(prop, "ID Type", "Type of ID-block that can be used");
-	RNA_def_property_update(prop, NC_SCENE|ND_KEYINGSET|NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
+	RNA_def_property_update(prop, NC_SCENE | ND_KEYINGSET | NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
 	
 	/* Group */
 	prop = RNA_def_property(srna, "group", PROP_STRING, PROP_NONE);
 	RNA_def_property_ui_text(prop, "Group Name", "Name of Action Group to assign setting(s) for this path to");
-	RNA_def_property_update(prop, NC_SCENE|ND_KEYINGSET|NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
+	RNA_def_property_update(prop, NC_SCENE | ND_KEYINGSET | NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
 	
 	/* Grouping */
 	prop = RNA_def_property(srna, "group_method", PROP_ENUM, PROP_NONE);
 	RNA_def_property_enum_sdna(prop, NULL, "groupmode");
 	RNA_def_property_enum_items(prop, keyingset_path_grouping_items);
 	RNA_def_property_ui_text(prop, "Grouping Method", "Method used to define which Group-name to use");
-	RNA_def_property_update(prop, NC_SCENE|ND_KEYINGSET|NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
+	RNA_def_property_update(prop, NC_SCENE | ND_KEYINGSET | NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
 	
 	/* Path + Array Index */
 	prop = RNA_def_property(srna, "data_path", PROP_STRING, PROP_NONE);
@@ -600,12 +667,12 @@ static void rna_def_keyingset_path(BlenderRNA *brna)
 	                              "rna_ksPath_RnaPath_set");
 	RNA_def_property_ui_text(prop, "Data Path", "Path to property setting");
 	RNA_def_struct_name_property(srna, prop); /* XXX this is the best indicator for now... */
-	RNA_def_property_update(prop, NC_SCENE|ND_KEYINGSET|NA_EDITED, NULL);
+	RNA_def_property_update(prop, NC_SCENE | ND_KEYINGSET | NA_EDITED, NULL);
 
 	/* called 'index' when given as function arg */
 	prop = RNA_def_property(srna, "array_index", PROP_INT, PROP_NONE);
 	RNA_def_property_ui_text(prop, "RNA Array Index", "Index to the specific setting if applicable");
-	RNA_def_property_update(prop, NC_SCENE|ND_KEYINGSET|NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
+	RNA_def_property_update(prop, NC_SCENE | ND_KEYINGSET | NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
 	
 	/* Flags */
 	prop = RNA_def_property(srna, "use_entire_array", PROP_BOOLEAN, PROP_NONE);
@@ -613,7 +680,7 @@ static void rna_def_keyingset_path(BlenderRNA *brna)
 	RNA_def_property_ui_text(prop, "Entire Array",
 	                         "When an 'array/vector' type is chosen (Location, Rotation, Color, etc.), "
 	                         "entire array is to be used");
-	RNA_def_property_update(prop, NC_SCENE|ND_KEYINGSET|NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
+	RNA_def_property_update(prop, NC_SCENE | ND_KEYINGSET | NA_EDITED, NULL); /* XXX: maybe a bit too noisy */
 	
 	/* Keyframing Settings */
 	rna_def_common_keying_flags(srna, 0);
@@ -640,21 +707,21 @@ static void rna_def_keyingset_paths(BlenderRNA *brna, PropertyRNA *cprop)
 	func = RNA_def_function(srna, "add", "rna_KeyingSet_paths_add");
 	RNA_def_function_ui_description(func, "Add a new path for the Keying Set");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
-		/* return arg */
+	/* return arg */
 	parm = RNA_def_pointer(func, "ksp", "KeyingSetPath", "New Path", "Path created and added to the Keying Set");
-		RNA_def_function_return(func, parm);
-		/* ID-block for target */
+	RNA_def_function_return(func, parm);
+	/* ID-block for target */
 	parm = RNA_def_pointer(func, "target_id", "ID", "Target ID", "ID-Datablock for the destination");
-		RNA_def_property_flag(parm, PROP_REQUIRED);
-		/* rna-path */
-		/* XXX hopefully this is long enough */
+	RNA_def_property_flag(parm, PROP_REQUIRED);
+	/* rna-path */
+	/* XXX hopefully this is long enough */
 	parm = RNA_def_string(func, "data_path", "", 256, "Data-Path", "RNA-Path to destination property");
-		RNA_def_property_flag(parm, PROP_REQUIRED);
-		/* index (defaults to -1 for entire array) */
+	RNA_def_property_flag(parm, PROP_REQUIRED);
+	/* index (defaults to -1 for entire array) */
 	RNA_def_int(func, "index", -1, -1, INT_MAX, "Index",
 	            "The index of the destination property (i.e. axis of Location/Rotation/etc.), "
 	            "or -1 for the entire array", 0, INT_MAX);
-		/* grouping */
+	/* grouping */
 	RNA_def_enum(func, "group_method", keyingset_path_grouping_items, KSP_GROUP_KSNAME,
 	             "Grouping Method", "Method used to define which Group-name to use");
 	RNA_def_string(func, "group_name", "", 64, "Group Name",
@@ -665,9 +732,10 @@ static void rna_def_keyingset_paths(BlenderRNA *brna, PropertyRNA *cprop)
 	func = RNA_def_function(srna, "remove", "rna_KeyingSet_paths_remove");
 	RNA_def_function_ui_description(func, "Remove the given path from the Keying Set");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
-		/* path to remove */
+	/* path to remove */
 	parm = RNA_def_pointer(func, "path", "KeyingSetPath", "Path", "");
-		RNA_def_property_flag(parm, PROP_REQUIRED|PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL | PROP_RNAPTR);
+	RNA_def_property_clear_flag(parm, PROP_THICK_WRAP);
 
 
 	/* Remove All Paths */
@@ -698,21 +766,22 @@ static void rna_def_keyingset(BlenderRNA *brna)
 	srna = RNA_def_struct(brna, "KeyingSet", NULL);
 	RNA_def_struct_ui_text(srna, "Keying Set", "Settings that should be keyframed together");
 	
-	/* Id/Label. */
+	/* Id/Label */
 	prop = RNA_def_property(srna, "bl_idname", PROP_STRING, PROP_NONE);
 	RNA_def_property_string_sdna(prop, NULL, "idname");
-	RNA_def_property_flag(prop, PROP_REGISTER|PROP_NEVER_CLAMP);
+	RNA_def_property_flag(prop, PROP_REGISTER | PROP_NEVER_CLAMP);
 	RNA_def_property_ui_text(prop, "ID Name", KEYINGSET_IDNAME_DOC);
-	RNA_def_property_update(prop, NC_SCENE|ND_KEYINGSET|NA_RENAME, NULL);
+/*	RNA_def_property_update(prop, NC_SCENE | ND_KEYINGSET | NA_RENAME, NULL); */ /* NOTE: disabled, as ID name shouldn't be editable */
 	
 	prop = RNA_def_property(srna, "bl_label", PROP_STRING, PROP_NONE);
 	RNA_def_property_string_sdna(prop, NULL, "name");
+	RNA_def_property_string_funcs(prop, NULL, NULL, "rna_KeyingSet_name_set");
 	RNA_def_property_ui_text(prop, "UI Name", "");
 	RNA_def_struct_ui_icon(srna, ICON_KEYINGSET);
 	RNA_def_struct_name_property(srna, prop);
-/*	RNA_def_property_update(prop, NC_SCENE|ND_KEYINGSET|NA_RENAME, NULL);*/
+	RNA_def_property_update(prop, NC_SCENE | ND_KEYINGSET | NA_RENAME, NULL);
 	
-	prop = RNA_def_property(srna, "bl_description", PROP_STRING, PROP_TRANSLATE);
+	prop = RNA_def_property(srna, "bl_description", PROP_STRING, PROP_NONE);
 	RNA_def_property_string_sdna(prop, NULL, "description");
 	RNA_def_property_string_maxlength(prop, RNA_DYN_DESCR_MAX); /* else it uses the pointer size! */
 	RNA_def_property_flag(prop, PROP_REGISTER_OPTIONAL);
@@ -772,10 +841,11 @@ static void rna_api_animdata_nla_tracks(BlenderRNA *brna, PropertyRNA *cprop)
 	RNA_def_function_return(func, parm);
 	
 	func = RNA_def_function(srna, "remove", "rna_NlaTrack_remove");
-	RNA_def_function_flag(func, FUNC_USE_CONTEXT);
+	RNA_def_function_flag(func, FUNC_USE_REPORTS | FUNC_USE_CONTEXT);
 	RNA_def_function_ui_description(func, "Remove a NLA Track");
 	parm = RNA_def_pointer(func, "track", "NlaTrack", "", "NLA Track to remove");
-	RNA_def_property_flag(parm, PROP_REQUIRED|PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL | PROP_RNAPTR);
+	RNA_def_property_clear_flag(parm, PROP_THICK_WRAP);
 
 	prop = RNA_def_property(srna, "active", PROP_POINTER, PROP_NONE);
 	RNA_def_property_struct_type(prop, "NlaTrack");
@@ -783,7 +853,7 @@ static void rna_api_animdata_nla_tracks(BlenderRNA *brna, PropertyRNA *cprop)
 	RNA_def_property_flag(prop, PROP_EDITABLE);
 	RNA_def_property_ui_text(prop, "Active Constraint", "Active Object constraint");
 	/* XXX: should (but doesn't) update the active track in the NLA window */
-	RNA_def_property_update(prop, NC_ANIMATION|ND_NLA|NA_SELECTED, NULL);
+	RNA_def_property_update(prop, NC_ANIMATION | ND_NLA | NA_SELECTED, NULL);
 }
 
 static void rna_api_animdata_drivers(BlenderRNA *brna, PropertyRNA *cprop)
@@ -818,7 +888,7 @@ void rna_def_animdata_common(StructRNA *srna)
 	RNA_def_property_ui_text(prop, "Animation Data", "Animation data for this datablock");
 }
 
-void rna_def_animdata(BlenderRNA *brna)
+static void rna_def_animdata(BlenderRNA *brna)
 {
 	StructRNA *srna;
 	PropertyRNA *prop;
@@ -836,8 +906,8 @@ void rna_def_animdata(BlenderRNA *brna)
 	
 	/* Active Action */
 	prop = RNA_def_property(srna, "action", PROP_POINTER, PROP_NONE);
-		/* this flag as well as the dynamic test must be defined for this to be editable... */
-	RNA_def_property_flag(prop, PROP_EDITABLE|PROP_ID_REFCOUNT);
+	/* this flag as well as the dynamic test must be defined for this to be editable... */
+	RNA_def_property_flag(prop, PROP_EDITABLE | PROP_ID_REFCOUNT);
 	RNA_def_property_pointer_funcs(prop, NULL, "rna_AnimData_action_set", NULL, "rna_Action_id_poll");
 	RNA_def_property_editable_func(prop, "rna_AnimData_action_editable");
 	RNA_def_property_ui_text(prop, "Action", "Active Action for this datablock");
@@ -849,14 +919,14 @@ void rna_def_animdata(BlenderRNA *brna)
 	RNA_def_property_enum_items(prop, nla_mode_extend_items);
 	RNA_def_property_ui_text(prop, "Action Extrapolation",
 	                         "Action to take for gaps past the Active Action's range (when evaluating with NLA)");
-	RNA_def_property_update(prop, NC_ANIMATION|ND_NLA, NULL); /* this will do? */
+	RNA_def_property_update(prop, NC_ANIMATION | ND_NLA, NULL); /* this will do? */
 	
 	prop = RNA_def_property(srna, "action_blend_type", PROP_ENUM, PROP_NONE);
 	RNA_def_property_enum_sdna(prop, NULL, "act_blendmode");
 	RNA_def_property_enum_items(prop, nla_mode_blend_items);
 	RNA_def_property_ui_text(prop, "Action Blending",
 	                         "Method used for combining Active Action's result with result of NLA stack");
-	RNA_def_property_update(prop, NC_ANIMATION|ND_NLA, NULL); /* this will do? */
+	RNA_def_property_update(prop, NC_ANIMATION | ND_NLA, NULL); /* this will do? */
 	
 	prop = RNA_def_property(srna, "action_influence", PROP_FLOAT, PROP_NONE);
 	RNA_def_property_float_sdna(prop, NULL, "act_influence");
@@ -864,7 +934,7 @@ void rna_def_animdata(BlenderRNA *brna)
 	RNA_def_property_range(prop, 0.0f, 1.0f);
 	RNA_def_property_ui_text(prop, "Action Influence",
 	                         "Amount the Active Action contributes to the result of the NLA stack");
-	RNA_def_property_update(prop, NC_ANIMATION|ND_NLA, NULL); /* this will do? */
+	RNA_def_property_update(prop, NC_ANIMATION | ND_NLA, NULL); /* this will do? */
 	
 	/* Drivers */
 	prop = RNA_def_property(srna, "drivers", PROP_COLLECTION, PROP_NONE);
@@ -878,7 +948,7 @@ void rna_def_animdata(BlenderRNA *brna)
 	prop = RNA_def_property(srna, "use_nla", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "flag", ADT_NLA_EVAL_OFF);
 	RNA_def_property_ui_text(prop, "NLA Evaluation Enabled", "NLA stack is evaluated when evaluating this block");
-	RNA_def_property_update(prop, NC_ANIMATION|ND_NLA, NULL); /* this will do? */
+	RNA_def_property_update(prop, NC_ANIMATION | ND_NLA, NULL); /* this will do? */
 }
 
 /* --- */
