@@ -50,7 +50,7 @@ EnumPropertyItem boidrule_type_items[] = {
 	{eBoidRuleType_Goal, "GOAL", 0, "Goal", "Go to assigned object or loudest assigned signal source"},
 	{eBoidRuleType_Avoid, "AVOID", 0, "Avoid", "Get away from assigned object or loudest assigned signal source"},
 	{eBoidRuleType_AvoidCollision, "AVOID_COLLISION", 0, "Avoid Collision",
-	                               "Manoeuvre to avoid collisions with other boids and deflector objects in "
+	                               "Maneuver to avoid collisions with other boids and deflector objects in "
 	                               "near future"},
 	{eBoidRuleType_Separate, "SEPARATE", 0, "Separate", "Keep from going through other boids"},
 	{eBoidRuleType_Flock, "FLOCK", 0, "Flock", "Move to center of neighbors and match their velocity"},
@@ -65,7 +65,8 @@ EnumPropertyItem boidrule_type_items[] = {
 	{eBoidRuleType_FollowWall, "FOLLOW_WALL", 0, "Follow Wall",
 	                           "Move next to a deflector object's in direction of it's tangent"},
 #endif
-	{0, NULL, 0, NULL, NULL}};
+	{0, NULL, 0, NULL, NULL}
+};
 
 EnumPropertyItem boidruleset_type_items[] = {
 	{eBoidRulesetType_Fuzzy, "FUZZY", 0, "Fuzzy",
@@ -73,11 +74,14 @@ EnumPropertyItem boidruleset_type_items[] = {
 	                         "fuzziness threshold is evaluated)"},
 	{eBoidRulesetType_Random, "RANDOM", 0, "Random", "A random rule is selected for each boid"},
 	{eBoidRulesetType_Average, "AVERAGE", 0, "Average", "All rules are averaged"},
-	{0, NULL, 0, NULL, NULL}};
+	{0, NULL, 0, NULL, NULL}
+};
 
 
 #ifdef RNA_RUNTIME
 
+#include "BLI_math_base.h"
+
 #include "BKE_context.h"
 #include "BKE_depsgraph.h"
 #include "BKE_particle.h"
@@ -85,37 +89,37 @@ EnumPropertyItem boidruleset_type_items[] = {
 static void rna_Boids_reset(Main *UNUSED(bmain), Scene *UNUSED(scene), PointerRNA *ptr)
 {
 	if (ptr->type == &RNA_ParticleSystem) {
-		ParticleSystem *psys = (ParticleSystem*)ptr->data;
+		ParticleSystem *psys = (ParticleSystem *)ptr->data;
 		
 		psys->recalc = PSYS_RECALC_RESET;
 
 		DAG_id_tag_update(ptr->id.data, OB_RECALC_DATA);
 	}
 	else
-		DAG_id_tag_update(ptr->id.data, OB_RECALC_DATA|PSYS_RECALC_RESET);
+		DAG_id_tag_update(ptr->id.data, OB_RECALC_DATA | PSYS_RECALC_RESET);
 
-	WM_main_add_notifier(NC_OBJECT|ND_PARTICLE|NA_EDITED, NULL);
+	WM_main_add_notifier(NC_OBJECT | ND_PARTICLE | NA_EDITED, NULL);
 }
 static void rna_Boids_reset_deps(Main *bmain, Scene *scene, PointerRNA *ptr)
 {
 	if (ptr->type == &RNA_ParticleSystem) {
-		ParticleSystem *psys = (ParticleSystem*)ptr->data;
+		ParticleSystem *psys = (ParticleSystem *)ptr->data;
 		
 		psys->recalc = PSYS_RECALC_RESET;
 
 		DAG_id_tag_update(ptr->id.data, OB_RECALC_DATA);
 	}
 	else
-		DAG_id_tag_update(ptr->id.data, OB_RECALC_DATA|PSYS_RECALC_RESET);
+		DAG_id_tag_update(ptr->id.data, OB_RECALC_DATA | PSYS_RECALC_RESET);
 
 	DAG_scene_sort(bmain, scene);
 
-	WM_main_add_notifier(NC_OBJECT|ND_PARTICLE|NA_EDITED, NULL);
+	WM_main_add_notifier(NC_OBJECT | ND_PARTICLE | NA_EDITED, NULL);
 }
 
-static StructRNA* rna_BoidRule_refine(struct PointerRNA *ptr)
+static StructRNA *rna_BoidRule_refine(struct PointerRNA *ptr)
 {
-	BoidRule *rule = (BoidRule*)ptr->data;
+	BoidRule *rule = (BoidRule *)ptr->data;
 
 	switch (rule->type) {
 		case eBoidRuleType_Goal:
@@ -137,13 +141,13 @@ static StructRNA* rna_BoidRule_refine(struct PointerRNA *ptr)
 
 static char *rna_BoidRule_path(PointerRNA *ptr)
 {
-	return BLI_sprintfN("rules[\"%s\"]", ((BoidRule*)ptr->data)->name);  /* XXX not unique */
+	return BLI_sprintfN("rules[\"%s\"]", ((BoidRule *)ptr->data)->name);  /* XXX not unique */
 }
 
 static PointerRNA rna_BoidState_active_boid_rule_get(PointerRNA *ptr)
 {
-	BoidState *state = (BoidState*)ptr->data;
-	BoidRule *rule = (BoidRule*)state->rules.first;
+	BoidState *state = (BoidState *)ptr->data;
+	BoidRule *rule = (BoidRule *)state->rules.first;
 
 	for (; rule; rule = rule->next) {
 		if (rule->flag & BOIDRULE_CURRENT)
@@ -153,16 +157,15 @@ static PointerRNA rna_BoidState_active_boid_rule_get(PointerRNA *ptr)
 }
 static void rna_BoidState_active_boid_rule_index_range(PointerRNA *ptr, int *min, int *max, int *softmin, int *softmax)
 {
-	BoidState *state = (BoidState*)ptr->data;
+	BoidState *state = (BoidState *)ptr->data;
 	*min = 0;
-	*max = BLI_countlist(&state->rules)-1;
-	*max = MAX2(0, *max);
+	*max = max_ii(0, BLI_countlist(&state->rules) - 1);
 }
 
 static int rna_BoidState_active_boid_rule_index_get(PointerRNA *ptr)
 {
-	BoidState *state = (BoidState*)ptr->data;
-	BoidRule *rule = (BoidRule*)state->rules.first;
+	BoidState *state = (BoidState *)ptr->data;
+	BoidRule *rule = (BoidRule *)state->rules.first;
 	int i = 0;
 
 	for (; rule; rule = rule->next, i++) {
@@ -174,8 +177,8 @@ static int rna_BoidState_active_boid_rule_index_get(PointerRNA *ptr)
 
 static void rna_BoidState_active_boid_rule_index_set(struct PointerRNA *ptr, int value)
 {
-	BoidState *state = (BoidState*)ptr->data;
-	BoidRule *rule = (BoidRule*)state->rules.first;
+	BoidState *state = (BoidState *)ptr->data;
+	BoidRule *rule = (BoidRule *)state->rules.first;
 	int i = 0;
 
 	for (; rule; rule = rule->next, i++) {
@@ -198,7 +201,7 @@ static char *rna_BoidSettings_path(PointerRNA *ptr)
 	BoidSettings *boids = (BoidSettings *)ptr->data;
 	
 	if (particle_id_check(ptr)) {
-		ParticleSettings *part = (ParticleSettings*)ptr->id.data;
+		ParticleSettings *part = (ParticleSettings *)ptr->id.data;
 		
 		if (part->boids == boids)
 			return BLI_sprintfN("boids");
@@ -208,8 +211,8 @@ static char *rna_BoidSettings_path(PointerRNA *ptr)
 
 static PointerRNA rna_BoidSettings_active_boid_state_get(PointerRNA *ptr)
 {
-	BoidSettings *boids = (BoidSettings*)ptr->data;
-	BoidState *state = (BoidState*)boids->states.first;
+	BoidSettings *boids = (BoidSettings *)ptr->data;
+	BoidState *state = (BoidState *)boids->states.first;
 
 	for (; state; state = state->next) {
 		if (state->flag & BOIDSTATE_CURRENT)
@@ -220,16 +223,15 @@ static PointerRNA rna_BoidSettings_active_boid_state_get(PointerRNA *ptr)
 static void rna_BoidSettings_active_boid_state_index_range(PointerRNA *ptr, int *min, int *max,
                                                            int *softmin, int *softmax)
 {
-	BoidSettings *boids = (BoidSettings*)ptr->data;
+	BoidSettings *boids = (BoidSettings *)ptr->data;
 	*min = 0;
-	*max = BLI_countlist(&boids->states)-1;
-	*max = MAX2(0, *max);
+	*max = max_ii(0, BLI_countlist(&boids->states) - 1);
 }
 
 static int rna_BoidSettings_active_boid_state_index_get(PointerRNA *ptr)
 {
-	BoidSettings *boids = (BoidSettings*)ptr->data;
-	BoidState *state = (BoidState*)boids->states.first;
+	BoidSettings *boids = (BoidSettings *)ptr->data;
+	BoidState *state = (BoidState *)boids->states.first;
 	int i = 0;
 
 	for (; state; state = state->next, i++) {
@@ -241,8 +243,8 @@ static int rna_BoidSettings_active_boid_state_index_get(PointerRNA *ptr)
 
 static void rna_BoidSettings_active_boid_state_index_set(struct PointerRNA *ptr, int value)
 {
-	BoidSettings *boids = (BoidSettings*)ptr->data;
-	BoidState *state = (BoidState*)boids->states.first;
+	BoidSettings *boids = (BoidSettings *)ptr->data;
+	BoidState *state = (BoidState *)boids->states.first;
 	int i = 0;
 
 	for (; state; state = state->next, i++) {
@@ -407,7 +409,7 @@ static void rna_def_boidrule(BlenderRNA *brna)
 	
 	/* data */
 	srna = RNA_def_struct(brna, "BoidRule", NULL);
-	RNA_def_struct_ui_text(srna , "Boid Rule", "");
+	RNA_def_struct_ui_text(srna, "Boid Rule", "");
 	RNA_def_struct_refine_func(srna, "rna_BoidRule_refine");
 	RNA_def_struct_path_func(srna, "rna_BoidRule_path");
 	
@@ -434,7 +436,7 @@ static void rna_def_boidrule(BlenderRNA *brna)
 	RNA_def_property_ui_text(prop, "On Land", "Use rule when boid is on land");
 	RNA_def_property_update(prop, 0, "rna_Boids_reset");
 	
-	/*prop= RNA_def_property(srna, "show_expanded", PROP_BOOLEAN, PROP_NONE); */
+	/*prop = RNA_def_property(srna, "show_expanded", PROP_BOOLEAN, PROP_NONE); */
 	/*RNA_def_property_boolean_sdna(prop, NULL, "mode", eModifierMode_Expanded); */
 	/*RNA_def_property_ui_text(prop, "Expanded", "Set modifier expanded in the user interface"); */
 
