@@ -41,54 +41,54 @@
 
 static PointerRNA rna_Context_manager_get(PointerRNA *ptr)
 {
-	bContext *C = (bContext*)ptr->data;
+	bContext *C = (bContext *)ptr->data;
 	return rna_pointer_inherit_refine(ptr, &RNA_WindowManager, CTX_wm_manager(C));
 }
 
 static PointerRNA rna_Context_window_get(PointerRNA *ptr)
 {
-	bContext *C = (bContext*)ptr->data;
+	bContext *C = (bContext *)ptr->data;
 	return rna_pointer_inherit_refine(ptr, &RNA_Window, CTX_wm_window(C));
 }
 
 static PointerRNA rna_Context_screen_get(PointerRNA *ptr)
 {
-	bContext *C = (bContext*)ptr->data;
+	bContext *C = (bContext *)ptr->data;
 	return rna_pointer_inherit_refine(ptr, &RNA_Screen, CTX_wm_screen(C));
 }
 
 static PointerRNA rna_Context_area_get(PointerRNA *ptr)
 {
-	bContext *C = (bContext*)ptr->data;
+	bContext *C = (bContext *)ptr->data;
 	PointerRNA newptr;
-	RNA_pointer_create((ID*)CTX_wm_screen(C), &RNA_Area, CTX_wm_area(C), &newptr);
+	RNA_pointer_create((ID *)CTX_wm_screen(C), &RNA_Area, CTX_wm_area(C), &newptr);
 	return newptr;
 }
 
 static PointerRNA rna_Context_space_data_get(PointerRNA *ptr)
 {
-	bContext *C = (bContext*)ptr->data;
+	bContext *C = (bContext *)ptr->data;
 	PointerRNA newptr;
-	RNA_pointer_create((ID*)CTX_wm_screen(C), &RNA_Space, CTX_wm_space_data(C), &newptr);
+	RNA_pointer_create((ID *)CTX_wm_screen(C), &RNA_Space, CTX_wm_space_data(C), &newptr);
 	return newptr;
 }
 
 static PointerRNA rna_Context_region_get(PointerRNA *ptr)
 {
-	bContext *C = (bContext*)ptr->data;
+	bContext *C = (bContext *)ptr->data;
 	PointerRNA newptr;
-	RNA_pointer_create((ID*)CTX_wm_screen(C), &RNA_Region, CTX_wm_region(C), &newptr);
+	RNA_pointer_create((ID *)CTX_wm_screen(C), &RNA_Region, CTX_wm_region(C), &newptr);
 	return newptr;
 }
 
 static PointerRNA rna_Context_region_data_get(PointerRNA *ptr)
 {
-	bContext *C = (bContext*)ptr->data;
+	bContext *C = (bContext *)ptr->data;
 
 	/* only exists for one space still, no generic system yet */
 	if (CTX_wm_view3d(C)) {
 		PointerRNA newptr;
-		RNA_pointer_create((ID*)CTX_wm_screen(C), &RNA_RegionView3D, CTX_wm_region_data(C), &newptr);
+		RNA_pointer_create((ID *)CTX_wm_screen(C), &RNA_RegionView3D, CTX_wm_region_data(C), &newptr);
 		return newptr;
 	}
 
@@ -97,19 +97,19 @@ static PointerRNA rna_Context_region_data_get(PointerRNA *ptr)
 
 static PointerRNA rna_Context_main_get(PointerRNA *ptr)
 {
-	bContext *C = (bContext*)ptr->data;
+	bContext *C = (bContext *)ptr->data;
 	return rna_pointer_inherit_refine(ptr, &RNA_BlendData, CTX_data_main(C));
 }
 
 static PointerRNA rna_Context_scene_get(PointerRNA *ptr)
 {
-	bContext *C = (bContext*)ptr->data;
+	bContext *C = (bContext *)ptr->data;
 	return rna_pointer_inherit_refine(ptr, &RNA_Scene, CTX_data_scene(C));
 }
 
 static PointerRNA rna_Context_tool_settings_get(PointerRNA *ptr)
 {
-	bContext *C = (bContext*)ptr->data;
+	bContext *C = (bContext *)ptr->data;
 	ptr->id.data = CTX_data_scene(C);
 	return rna_pointer_inherit_refine(ptr, &RNA_ToolSettings, CTX_data_tool_settings(C));
 }
@@ -123,7 +123,7 @@ static PointerRNA rna_Context_user_preferences_get(PointerRNA *UNUSED(ptr))
 
 static int rna_Context_mode_get(PointerRNA *ptr)
 {
-	bContext *C = (bContext*)ptr->data;
+	bContext *C = (bContext *)ptr->data;
 	return CTX_data_mode_enum(C);
 }
 
@@ -149,7 +149,8 @@ void RNA_def_context(BlenderRNA *brna)
 		{CTX_MODE_PAINT_TEXTURE, "PAINT_TEXTURE", 0, "Texture Paint", ""},
 		{CTX_MODE_PARTICLE, "PARTICLE", 0, "Particle", ""},
 		{CTX_MODE_OBJECT, "OBJECT", 0, "Object", ""},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 
 	srna = RNA_def_struct(brna, "Context", NULL);
 	RNA_def_struct_ui_text(srna, "Context", "Current windowmanager and data context");
