@@ -1,5 +1,4 @@
 /*
- *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -42,6 +41,8 @@
 
 #include "BIF_gl.h"
 
+#include "rna_internal.h"  /* own include */
+
 #ifdef RNA_RUNTIME
 
 #include "BKE_image.h"
@@ -51,6 +52,7 @@
 #include "BKE_global.h" /* grr: G.main->name */
 
 #include "IMB_imbuf.h"
+#include "IMB_colormanagement.h"
 
 #include "BIF_gl.h"
 #include "GPU_draw.h"
@@ -78,31 +80,35 @@ static void rna_Image_save_render(Image *image, bContext *C, ReportList *reports
 		ibuf = BKE_image_acquire_ibuf(image, &iuser, &lock);
 
 		if (ibuf == NULL) {
-			BKE_reportf(reports, RPT_ERROR, "Couldn't acquire buffer from image");
+			BKE_report(reports, RPT_ERROR, "Could not acquire buffer from image");
 		}
 		else {
-			/* temp swap out the color */
-			const unsigned char imb_planes_back = ibuf->planes;
-			const float dither_back = ibuf->dither;
-			ibuf->planes = scene->r.im_format.planes;
-			ibuf->dither = scene->r.dither_intensity;
-			if (!BKE_write_ibuf(ibuf, path, &scene->r.im_format)) {
-				BKE_reportf(reports, RPT_ERROR, "Couldn't write image: %s", path);
+			ImBuf *write_ibuf;
+
+			write_ibuf = IMB_colormanagement_imbuf_for_write(ibuf, TRUE, TRUE, &scene->view_settings,
+			                                                 &scene->display_settings, &scene->r.im_format);
+
+			write_ibuf->planes = scene->r.im_format.planes;
+			write_ibuf->dither = scene->r.dither_intensity;
+
+			if (!BKE_imbuf_write(write_ibuf, path, &scene->r.im_format)) {
+				BKE_reportf(reports, RPT_ERROR, "Could not write image '%s'", path);
 			}
-			ibuf->planes = imb_planes_back;
-			ibuf->dither = dither_back;
+
+			if (write_ibuf != ibuf)
+				IMB_freeImBuf(write_ibuf);
 		}
 
-		BKE_image_release_ibuf(image, lock);
+		BKE_image_release_ibuf(image, ibuf, lock);
 	}
 	else {
-		BKE_reportf(reports, RPT_ERROR, "Scene not in context, couldn't get save parameters");
+		BKE_report(reports, RPT_ERROR, "Scene not in context, could not get save parameters");
 	}
 }
 
 static void rna_Image_save(Image *image, ReportList *reports)
 {
-	ImBuf *ibuf = BKE_image_get_ibuf(image, NULL);
+	ImBuf *ibuf = BKE_image_acquire_ibuf(image, NULL, NULL);
 	if (ibuf) {
 		char filename[FILE_MAX];
 		BLI_strncpy(filename, image->name, sizeof(filename));
@@ -110,7 +116,7 @@ static void rna_Image_save(Image *image, ReportList *reports)
 
 		if (image->packedfile) {
 			if (writePackedFile(reports, image->name, image->packedfile, 0) != RET_OK) {
-				BKE_reportf(reports, RPT_ERROR, "Image \"%s\" could saved packed file to \"%s\"", image->id.name+2, image->name);
+				BKE_reportf(reports, RPT_ERROR, "Image '%s' could not save packed file to '%s'", image->id.name + 2, image->name);
 			}
 		}
 		else if (IMB_saveiff(ibuf, filename, ibuf->flags)) {
@@ -119,23 +125,27 @@ static void rna_Image_save(Image *image, ReportList *reports)
 			if (image->source == IMA_SRC_GENERATED)
 				image->source = IMA_SRC_FILE;
 
+			IMB_colormanagment_colorspace_from_ibuf_ftype(&image->colorspace_settings, ibuf);
+
 			ibuf->userflags &= ~IB_BITMAPDIRTY;
 		}
 		else {
-			BKE_reportf(reports, RPT_ERROR, "Image \"%s\" could not be saved to \"%s\"", image->id.name+2, image->name);
+			BKE_reportf(reports, RPT_ERROR, "Image '%s' could not be saved to '%s'", image->id.name + 2, image->name);
 		}
 	}
 	else {
-		BKE_reportf(reports, RPT_ERROR, "Image \"%s\" does not have any image data", image->id.name+2);
+		BKE_reportf(reports, RPT_ERROR, "Image '%s' does not have any image data", image->id.name + 2);
 	}
+
+	BKE_image_release_ibuf(image, ibuf, NULL);
 }
 
 static void rna_Image_pack(Image *image, ReportList *reports, int as_png)
 {
-	ImBuf *ibuf = BKE_image_get_ibuf(image, NULL);
+	ImBuf *ibuf = BKE_image_acquire_ibuf(image, NULL, NULL);
 
 	if (!as_png && (ibuf && (ibuf->userflags & IB_BITMAPDIRTY))) {
-		BKE_reportf(reports, RPT_ERROR, "Can't pack edited image from disk, only as internal PNG");
+		BKE_report(reports, RPT_ERROR, "Cannot pack edited image from disk, only as internal PNG");
 	}
 	else {
 		if (as_png) {
@@ -145,6 +155,8 @@ static void rna_Image_pack(Image *image, ReportList *reports, int as_png)
 			image->packedfile = newPackedFile(reports, image->name, ID_BLEND_PATH(G.main, &image->id));
 		}
 	}
+
+	BKE_image_release_ibuf(image, ibuf, NULL);
 }
 
 static void rna_Image_unpack(Image *image, ReportList *reports, int method)
@@ -158,7 +170,7 @@ static void rna_Image_unpack(Image *image, ReportList *reports, int method)
 	}
 	else {
 		/* reports its own error on failure */
-		unpackImage (reports, image, method);
+		unpackImage(reports, image, method);
 	}
 }
 
@@ -169,14 +181,26 @@ static void rna_Image_reload(Image *image)
 
 static void rna_Image_update(Image *image, ReportList *reports)
 {
-	ImBuf *ibuf = BKE_image_get_ibuf(image, NULL);
+	ImBuf *ibuf = BKE_image_acquire_ibuf(image, NULL, NULL);
 
 	if (ibuf == NULL) {
-		BKE_reportf(reports, RPT_ERROR, "Image \"%s\" does not have any image data", image->id.name+2);
+		BKE_reportf(reports, RPT_ERROR, "Image '%s' does not have any image data", image->id.name + 2);
 		return;
 	}
 
-	IMB_rect_from_float(ibuf);
+	if (ibuf->rect)
+		IMB_rect_from_float(ibuf);
+
+	ibuf->userflags |= IB_DISPLAY_BUFFER_INVALID;
+
+	BKE_image_release_ibuf(image, ibuf, NULL);
+}
+
+static void rna_Image_scale(Image *image, ReportList *reports, int width, int height)
+{
+	if (!BKE_image_scale(image, width, height)) {
+		BKE_reportf(reports, RPT_ERROR, "Image '%s' does not have any image data", image->id.name + 2);
+	}
 }
 
 static int rna_Image_gl_load(Image *image, ReportList *reports, int filter, int mag)
@@ -188,21 +212,25 @@ static int rna_Image_gl_load(Image *image, ReportList *reports, int filter, int
 	if (*bind)
 		return error;
 
-	ibuf = BKE_image_get_ibuf(image, NULL);
+	ibuf = BKE_image_acquire_ibuf(image, NULL, NULL);
 
-	if (ibuf == NULL || ibuf->rect == NULL ) {
-		BKE_reportf(reports, RPT_ERROR, "Image \"%s\" does not have any image data", image->id.name+2);
+	if (ibuf == NULL || ibuf->rect == NULL) {
+		BKE_reportf(reports, RPT_ERROR, "Image '%s' does not have any image data", image->id.name + 2);
+		BKE_image_release_ibuf(image, ibuf, NULL);
 		return (int)GL_INVALID_OPERATION;
 	}
 
 	/* could be made into a function? */
-	glGenTextures(1, (GLuint*)bind);
+	glGenTextures(1, (GLuint *)bind);
 	glBindTexture(GL_TEXTURE_2D, *bind);
 
 	if (filter != GL_NEAREST && filter != GL_LINEAR)
 		error = (int)gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, ibuf->x, ibuf->y, GL_RGBA, GL_UNSIGNED_BYTE, ibuf->rect);
 
 	if (!error) {
+		/* clean glError buffer */
+		while (glGetError() != GL_NO_ERROR) {}
+
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, image->tpageflag & IMA_CLAMP_U ? GL_CLAMP : GL_REPEAT);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, image->tpageflag & IMA_CLAMP_V ? GL_CLAMP : GL_REPEAT);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, (GLint)filter);
@@ -213,10 +241,25 @@ static int rna_Image_gl_load(Image *image, ReportList *reports, int filter, int
 	}
 
 	if (error) {
-		glDeleteTextures(1, (GLuint*)bind);
+		glDeleteTextures(1, (GLuint *)bind);
 		image->bindcode = 0;
 	}
 
+	BKE_image_release_ibuf(image, ibuf, NULL);
+
+	return error;
+}
+
+static int rna_Image_gl_touch(Image *image, ReportList *reports, int filter, int mag)
+{
+	unsigned int *bind = &image->bindcode;
+	int error = GL_NO_ERROR;
+
+	BKE_image_tag_time(image);
+
+	if (*bind == 0)
+		error = rna_Image_gl_load(image, reports, filter, mag);
+
 	return error;
 }
 
@@ -237,7 +280,7 @@ void RNA_api_image(StructRNA *srna)
 
 	func = RNA_def_function(srna, "save_render", "rna_Image_save_render");
 	RNA_def_function_ui_description(func, "Save image to a specific path using a scenes render settings");
-	RNA_def_function_flag(func, FUNC_USE_CONTEXT|FUNC_USE_REPORTS);
+	RNA_def_function_flag(func, FUNC_USE_CONTEXT | FUNC_USE_REPORTS);
 	parm = RNA_def_string_file_path(func, "filepath", "", 0, "", "Save path");
 	RNA_def_property_flag(parm, PROP_REQUIRED);
 	RNA_def_pointer(func, "scene", "Scene", "", "Scene to take image parameters from");
@@ -263,6 +306,25 @@ void RNA_api_image(StructRNA *srna)
 	RNA_def_function_ui_description(func, "Update the display image from the floating point buffer");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
 
+	func = RNA_def_function(srna, "scale", "rna_Image_scale");
+	RNA_def_function_ui_description(func, "Scale the image in pixels");
+	RNA_def_function_flag(func, FUNC_USE_REPORTS);
+	parm = RNA_def_int(func, "width", 1, 1, 10000, "", "Width", 1, 10000);
+	RNA_def_property_flag(parm, PROP_REQUIRED);
+	parm = RNA_def_int(func, "height", 1, 1, 10000, "", "Height", 1, 10000);
+	RNA_def_property_flag(parm, PROP_REQUIRED);
+
+	func = RNA_def_function(srna, "gl_touch", "rna_Image_gl_touch");
+	RNA_def_function_ui_description(func, "Delay the image from being cleaned from the cache due inactivity");
+	RNA_def_function_flag(func, FUNC_USE_REPORTS);
+	RNA_def_int(func, "filter", GL_LINEAR_MIPMAP_NEAREST, -INT_MAX, INT_MAX, "Filter",
+	            "The texture minifying function to use if the image wasn't loaded", -INT_MAX, INT_MAX);
+	RNA_def_int(func, "mag", GL_LINEAR, -INT_MAX, INT_MAX, "Magnification",
+	            "The texture magnification function to use if the image wasn't loaded", -INT_MAX, INT_MAX);
+	/* return value */
+	parm = RNA_def_int(func, "error", 0, -INT_MAX, INT_MAX, "Error", "OpenGL error value", -INT_MAX, INT_MAX);
+	RNA_def_function_return(func, parm);
+
 	func = RNA_def_function(srna, "gl_load", "rna_Image_gl_load");
 	RNA_def_function_ui_description(func, "Load the image into OpenGL graphics memory");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
