@@ -27,6 +27,7 @@
 
 #include <stdlib.h>
 
+#include "RNA_access.h"
 #include "RNA_define.h"
 #include "RNA_enum_types.h"
 
@@ -53,13 +54,13 @@
 
 static int rna_Meta_texspace_editable(PointerRNA *ptr)
 {
-	MetaBall *mb = (MetaBall*)ptr->data;
-	return (mb->texflag & MB_AUTOSPACE)? 0: PROP_EDITABLE;
+	MetaBall *mb = (MetaBall *)ptr->data;
+	return (mb->texflag & MB_AUTOSPACE) ? 0 : PROP_EDITABLE;
 }
 
 static void rna_Meta_texspace_loc_get(PointerRNA *ptr, float *values)
 {
-	MetaBall *mb = (MetaBall*)ptr->data;
+	MetaBall *mb = (MetaBall *)ptr->data;
 	
 	/* tex_space_mball() needs object.. ugh */
 	
@@ -68,14 +69,14 @@ static void rna_Meta_texspace_loc_get(PointerRNA *ptr, float *values)
 
 static void rna_Meta_texspace_loc_set(PointerRNA *ptr, const float *values)
 {
-	MetaBall *mb = (MetaBall*)ptr->data;
+	MetaBall *mb = (MetaBall *)ptr->data;
 	
 	copy_v3_v3(mb->loc, values);
 }
 
 static void rna_Meta_texspace_size_get(PointerRNA *ptr, float *values)
 {
-	MetaBall *mb = (MetaBall*)ptr->data;
+	MetaBall *mb = (MetaBall *)ptr->data;
 	
 	/* tex_space_mball() needs object.. ugh */
 	
@@ -84,7 +85,7 @@ static void rna_Meta_texspace_size_get(PointerRNA *ptr, float *values)
 
 static void rna_Meta_texspace_size_set(PointerRNA *ptr, const float *values)
 {
-	MetaBall *mb = (MetaBall*)ptr->data;
+	MetaBall *mb = (MetaBall *)ptr->data;
 	
 	copy_v3_v3(mb->size, values);
 }
@@ -99,10 +100,10 @@ static void rna_MetaBall_update_data(Main *bmain, Scene *scene, PointerRNA *ptr)
 	if (mb->id.us > 0) {
 		for (ob = bmain->object.first; ob; ob = ob->id.next)
 			if (ob->data == mb)
-				copy_mball_properties(scene, ob);
+				BKE_mball_properties_copy(scene, ob);
 	
 		DAG_id_tag_update(&mb->id, 0);
-		WM_main_add_notifier(NC_GEOM|ND_DATA, mb);
+		WM_main_add_notifier(NC_GEOM | ND_DATA, mb);
 	}
 }
 
@@ -115,35 +116,33 @@ static void rna_MetaBall_update_rotation(Main *bmain, Scene *scene, PointerRNA *
 
 static MetaElem *rna_MetaBall_elements_new(MetaBall *mb, int type)
 {
-	MetaElem *ml = add_metaball_element(mb, type);
+	MetaElem *ml = BKE_mball_element_add(mb, type);
 
 	/* cheating way for importers to avoid slow updates */
 	if (mb->id.us > 0) {
 		DAG_id_tag_update(&mb->id, 0);
-		WM_main_add_notifier(NC_GEOM|ND_DATA, &mb->id);
+		WM_main_add_notifier(NC_GEOM | ND_DATA, &mb->id);
 	}
 
 	return ml;
 }
 
-static void rna_MetaBall_elements_remove(MetaBall *mb, ReportList *reports, MetaElem *ml)
+static void rna_MetaBall_elements_remove(MetaBall *mb, ReportList *reports, PointerRNA *ml_ptr)
 {
-	int found = 0;
+	MetaElem *ml = ml_ptr->data;
 
-	found = BLI_remlink_safe(&mb->elems, ml);
-
-	if (!found) {
-		BKE_reportf(reports, RPT_ERROR, "Metaball \"%s\" does not contain spline given", mb->id.name+2);
+	if (BLI_remlink_safe(&mb->elems, ml) == FALSE) {
+		BKE_reportf(reports, RPT_ERROR, "Metaball '%s' does not contain spline given", mb->id.name + 2);
 		return;
 	}
 
 	MEM_freeN(ml);
-	/* invalidate pointer!, no can do */
+	RNA_POINTER_INVALIDATE(ml_ptr);
 
 	/* cheating way for importers to avoid slow updates */
 	if (mb->id.us > 0) {
 		DAG_id_tag_update(&mb->id, 0);
-		WM_main_add_notifier(NC_GEOM|ND_DATA, &mb->id);
+		WM_main_add_notifier(NC_GEOM | ND_DATA, &mb->id);
 	}
 }
 
@@ -154,7 +153,7 @@ static void rna_MetaBall_elements_clear(MetaBall *mb)
 	/* cheating way for importers to avoid slow updates */
 	if (mb->id.us > 0) {
 		DAG_id_tag_update(&mb->id, 0);
-		WM_main_add_notifier(NC_GEOM|ND_DATA, &mb->id);
+		WM_main_add_notifier(NC_GEOM | ND_DATA, &mb->id);
 	}
 }
 
@@ -188,7 +187,7 @@ static void rna_def_metaelement(BlenderRNA *brna)
 	RNA_def_property_ui_text(prop, "Rotation", "Normalized quaternion rotation");
 	RNA_def_property_update(prop, 0, "rna_MetaBall_update_rotation");
 
-	prop = RNA_def_property(srna, "radius", PROP_FLOAT, PROP_UNSIGNED|PROP_UNIT_LENGTH);
+	prop = RNA_def_property(srna, "radius", PROP_FLOAT, PROP_UNSIGNED | PROP_UNIT_LENGTH);
 	RNA_def_property_float_sdna(prop, NULL, "rad");
 	RNA_def_property_ui_text(prop, "Radius", "");
 	RNA_def_property_range(prop, 0.0f, FLT_MAX);
@@ -254,7 +253,8 @@ static void rna_def_metaball_elements(BlenderRNA *brna, PropertyRNA *cprop)
 	RNA_def_function_ui_description(func, "Remove an element from the metaball");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
 	parm = RNA_def_pointer(func, "element", "MetaElement", "", "The element to remove");
-	RNA_def_property_flag(parm, PROP_REQUIRED|PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL | PROP_RNAPTR);
+	RNA_def_property_clear_flag(parm, PROP_THICK_WRAP);
 
 	func = RNA_def_function(srna, "clear", "rna_MetaBall_elements_clear");
 	RNA_def_function_ui_description(func, "Remove all elements from the metaball");
@@ -273,7 +273,8 @@ static void rna_def_metaball(BlenderRNA *brna)
 		{MB_UPDATE_HALFRES, "HALFRES", 0, "Half", "While editing, update metaball in half resolution"},
 		{MB_UPDATE_FAST, "FAST", 0, "Fast", "While editing, update metaball without polygonization"},
 		{MB_UPDATE_NEVER, "NEVER", 0, "Never", "While editing, don't update metaball at all"},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 	
 	srna = RNA_def_struct(brna, "MetaBall", "ID");
 	RNA_def_struct_ui_text(srna, "MetaBall", "Metaball datablock to defined blobby surfaces");
@@ -333,7 +334,7 @@ static void rna_def_metaball(BlenderRNA *brna)
 	
 	/* not supported yet */
 #if 0
-	prop= RNA_def_property(srna, "texspace_rot", PROP_FLOAT, PROP_EULER);
+	prop = RNA_def_property(srna, "texspace_rot", PROP_FLOAT, PROP_EULER);
 	RNA_def_property_float(prop, NULL, "rot");
 	RNA_def_property_ui_text(prop, "Texture Space Rotation", "Texture space rotation");
 	RNA_def_property_editable_func(prop, "rna_Meta_texspace_editable");
@@ -350,6 +351,8 @@ static void rna_def_metaball(BlenderRNA *brna)
 	
 	/* anim */
 	rna_def_animdata_common(srna);
+
+	RNA_api_meta(srna);
 }
 
 void RNA_def_meta(BlenderRNA *brna)
