@@ -34,217 +34,88 @@
 #include <string.h>
 #include <time.h>
 
+#include "BLI_utildefines.h"
+
 #include "RNA_define.h"
 
-#include "DNA_object_types.h"
+#include "DNA_constraint_types.h"
 #include "DNA_modifier_types.h"
+#include "DNA_object_types.h"
+
+#include "rna_internal.h"  /* own include */
+
+static EnumPropertyItem space_items[] = {
+	{CONSTRAINT_SPACE_WORLD,    "WORLD", 0, "World Space",
+	                            "The most gobal space in Blender"},
+	{CONSTRAINT_SPACE_POSE,     "POSE", 0, "Pose Space",
+	                            "The pose space of a bone (its armature's object space)"},
+	{CONSTRAINT_SPACE_PARLOCAL, "LOCAL_WITH_PARENT", 0, "Local With Parent",
+	                            "The local space of a bone's parent bone"},
+	{CONSTRAINT_SPACE_LOCAL,    "LOCAL", 0, "Local Space",
+	                            "The local space of an object/bone"},
+	{0, NULL, 0, NULL, NULL}
+};
 
 #ifdef RNA_RUNTIME
+
 #include "BLI_math.h"
 
-#include "BKE_main.h"
-#include "BKE_global.h"
-#include "BKE_context.h"
-#include "BKE_report.h"
-#include "BKE_object.h"
-#include "BKE_mesh.h"
-#include "BKE_DerivedMesh.h"
-#include "BKE_bvhutils.h"
-
-#include "BKE_customdata.h"
 #include "BKE_anim.h"
+#include "BKE_bvhutils.h"
+#include "BKE_cdderivedmesh.h"
+#include "BKE_constraint.h"
+#include "BKE_context.h"
+#include "BKE_customdata.h"
 #include "BKE_depsgraph.h"
-#include "BKE_displist.h"
 #include "BKE_font.h"
+#include "BKE_global.h"
+#include "BKE_main.h"
+#include "BKE_mesh.h"
 #include "BKE_mball.h"
 #include "BKE_modifier.h"
-#include "BKE_cdderivedmesh.h"
+#include "BKE_object.h"
+#include "BKE_report.h"
 
-#include "DNA_mesh_types.h"
-#include "DNA_scene_types.h"
-#include "DNA_meshdata_types.h"
 #include "DNA_curve_types.h"
-#include "DNA_modifier_types.h"
-#include "DNA_constraint_types.h"
+#include "DNA_mesh_types.h"
+#include "DNA_meshdata_types.h"
+#include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
 
 #include "MEM_guardedalloc.h"
 
+/* Convert a given matrix from a space to another (using the object and/or a bone as reference). */
+static void rna_Scene_mat_convert_space(Object *ob, ReportList *reports, bPoseChannel *pchan,
+                                        float *mat, float *mat_ret, int from, int to)
+{
+	copy_m4_m4((float (*)[4])mat_ret, (float (*)[4])mat);
+
+	/* Error in case of invalid from/to values when pchan is NULL */
+	if (pchan == NULL) {
+		if (ELEM(from, CONSTRAINT_SPACE_POSE, CONSTRAINT_SPACE_PARLOCAL)) {
+			const char *identifier = NULL;
+			RNA_enum_identifier(space_items, from, &identifier);
+			BKE_reportf(reports, RPT_ERROR, "'from_space' '%s' is invalid when no pose bone is given!", identifier);
+			return;
+		}
+		if (ELEM(to, CONSTRAINT_SPACE_POSE, CONSTRAINT_SPACE_PARLOCAL)) {
+			const char *identifier = NULL;
+			RNA_enum_identifier(space_items, to, &identifier);
+			BKE_reportf(reports, RPT_ERROR, "'to_space' '%s' is invalid when no pose bone is given!", identifier);
+			return;
+		}
+	}
+
+	BKE_constraint_mat_convertspace(ob, pchan, (float (*)[4])mat_ret, from, to);
+}
+
 /* copied from Mesh_getFromObject and adapted to RNA interface */
 /* settings: 0 - preview, 1 - render */
-Mesh *rna_Object_to_mesh(Object *ob, ReportList *reports, Scene *sce, int apply_modifiers, int settings)
+static Mesh *rna_Object_to_mesh(
+        Object *ob, ReportList *reports, Scene *sce,
+        int apply_modifiers, int settings, int calc_tessface)
 {
-	Mesh *tmpmesh;
-	Curve *tmpcu = NULL, *copycu;
-	Object *tmpobj = NULL;
-	int render = settings == eModifierMode_Render, i;
-	int cage = !apply_modifiers;
-
-	/* perform the mesh extraction based on type */
-	switch (ob->type) {
-	case OB_FONT:
-	case OB_CURVE:
-	case OB_SURF:
-
-		/* copies object and modifiers (but not the data) */
-		tmpobj = copy_object(ob);
-		tmpcu = (Curve *)tmpobj->data;
-		tmpcu->id.us--;
-
-		/* if getting the original caged mesh, delete object modifiers */
-		if ( cage )
-			object_free_modifiers(tmpobj);
-
-		/* copies the data */
-		copycu = tmpobj->data = copy_curve( (Curve *) ob->data );
-
-		/* temporarily set edit so we get updates from edit mode, but
-		 * also because for text datablocks copying it while in edit
-		 * mode gives invalid data structures */
-		copycu->editfont = tmpcu->editfont;
-		copycu->editnurb = tmpcu->editnurb;
-
-		/* get updated display list, and convert to a mesh */
-		makeDispListCurveTypes( sce, tmpobj, 0 );
-
-		copycu->editfont = NULL;
-		copycu->editnurb = NULL;
-
-		nurbs_to_mesh( tmpobj );
-
-		/* nurbs_to_mesh changes the type to a mesh, check it worked */
-		if (tmpobj->type != OB_MESH) {
-			free_libblock_us( &(G.main->object), tmpobj );
-			BKE_report(reports, RPT_ERROR, "cant convert curve to mesh. Does the curve have any segments?");
-			return NULL;
-		}
-		tmpmesh = tmpobj->data;
-		free_libblock_us( &G.main->object, tmpobj );
-		break;
-
-	case OB_MBALL: {
-		/* metaballs don't have modifiers, so just convert to mesh */
-		Object *basis_ob = find_basis_mball(sce, ob);
-		/* todo, re-generatre for render-res */
-		/* metaball_polygonize(scene, ob) */
-
-		if (ob != basis_ob)
-			return NULL; /* only do basis metaball */
-
-		tmpmesh = add_mesh("Mesh");
-			
-		if (render) {
-			ListBase disp = {NULL, NULL};
-			makeDispListMBall_forRender(sce, ob, &disp);
-			mball_to_mesh(&disp, tmpmesh);
-			freedisplist(&disp);
-		}
-		else
-			mball_to_mesh(&ob->disp, tmpmesh);
-		break;
-
-	}
-	case OB_MESH:
-		/* copies object and modifiers (but not the data) */
-		if (cage) {
-			/* copies the data */
-			tmpmesh = copy_mesh( ob->data );
-		/* if not getting the original caged mesh, get final derived mesh */
-		}
-		else {
-			/* Make a dummy mesh, saves copying */
-			DerivedMesh *dm;
-			/* CustomDataMask mask = CD_MASK_BAREMESH|CD_MASK_MTFACE|CD_MASK_MCOL; */
-			CustomDataMask mask = CD_MASK_MESH; /* this seems more suitable, exporter,
-												 * for example, needs CD_MASK_MDEFORMVERT */
-			
-			/* Write the display mesh into the dummy mesh */
-			if (render)
-				dm = mesh_create_derived_render(sce, ob, mask);
-			else
-				dm = mesh_create_derived_view(sce, ob, mask);
-			
-			tmpmesh = add_mesh("Mesh");
-			DM_to_mesh(dm, tmpmesh, ob);
-			dm->release(dm);
-		}
-		
-		break;
-	default:
-		BKE_report(reports, RPT_ERROR, "Object does not have geometry data");
-		return NULL;
-	}
-
-	/* Copy materials to new mesh */
-	switch (ob->type) {
-	case OB_SURF:
-	case OB_FONT:
-	case OB_CURVE:
-		tmpmesh->totcol = tmpcu->totcol;
-		
-		/* free old material list (if it exists) and adjust user counts */
-		if ( tmpcu->mat ) {
-			for ( i = tmpcu->totcol; i-- > 0; ) {
-				/* are we an object material or data based? */
-
-				tmpmesh->mat[i] = ob->matbits[i] ? ob->mat[i] : tmpcu->mat[i];
-
-				if (tmpmesh->mat[i]) {
-					tmpmesh->mat[i]->id.us++;
-				}
-			}
-		}
-		break;
-
-#if 0
-	/* Crashes when assigning the new material, not sure why */
-	case OB_MBALL:
-		tmpmb = (MetaBall *)ob->data;
-		tmpmesh->totcol = tmpmb->totcol;
-		
-		/* free old material list (if it exists) and adjust user counts */
-		if ( tmpmb->mat ) {
-			for ( i = tmpmb->totcol; i-- > 0; ) {
-				tmpmesh->mat[i] = tmpmb->mat[i]; /* CRASH HERE ??? */
-				if (tmpmesh->mat[i]) {
-					tmpmb->mat[i]->id.us++;
-				}
-			}
-		}
-		break;
-#endif
-
-	case OB_MESH:
-		if (!cage) {
-			Mesh *origmesh = ob->data;
-			tmpmesh->flag = origmesh->flag;
-			tmpmesh->mat = MEM_dupallocN(origmesh->mat);
-			tmpmesh->totcol = origmesh->totcol;
-			tmpmesh->smoothresh = origmesh->smoothresh;
-			if ( origmesh->mat ) {
-				for ( i = origmesh->totcol; i-- > 0; ) {
-					/* are we an object material or data based? */
-					tmpmesh->mat[i] = ob->matbits[i] ? ob->mat[i] : origmesh->mat[i];
-
-					if (tmpmesh->mat[i]) {
-						tmpmesh->mat[i]->id.us++;
-					}
-				}
-			}
-		}
-		break;
-	} /* end copy materials */
-
-	/* cycles and exporters rely on this still */
-	BKE_mesh_tessface_ensure(tmpmesh);
-
-	/* we don't assign it to anything */
-	tmpmesh->id.us--;
-	
-	/* make sure materials get updated in objects */
-	test_object_materials(&tmpmesh->id);
-
-	return tmpmesh;
+	return rna_Main_meshes_new_from_object(G.main, reports, sce, ob, apply_modifiers, settings, calc_tessface);
 }
 
 /* mostly a copy from convertblender.c */
@@ -273,11 +144,11 @@ static void dupli_render_particle_set(Scene *scene, Object *ob, int level, int e
 			}
 		}
 
-		if (level == 0 && enable) {
+		if (enable) {
 			/* this is to make sure we get render level duplis in groups:
 			 * the derivedmesh must be created before init_render_mesh,
 			 * since object_duplilist does dupliparticles before that */
-			dm = mesh_create_derived_render(scene, ob, CD_MASK_BAREMESH|CD_MASK_MTFACE|CD_MASK_MCOL);
+			dm = mesh_create_derived_render(scene, ob, CD_MASK_BAREMESH | CD_MASK_MTFACE | CD_MASK_MCOL);
 			dm->release(dm);
 
 			for (psys = ob->particlesystem.first; psys; psys = psys->next)
@@ -289,11 +160,13 @@ static void dupli_render_particle_set(Scene *scene, Object *ob, int level, int e
 	group = ob->dup_group;
 
 	for (go = group->gobject.first; go; go = go->next)
-		dupli_render_particle_set(scene, go->ob, level+1, enable);
+		dupli_render_particle_set(scene, go->ob, level + 1, enable);
 }
 /* When no longer needed, duplilist should be freed with Object.free_duplilist */
-void rna_Object_create_duplilist(Object *ob, ReportList *reports, Scene *sce)
+static void rna_Object_create_duplilist(Object *ob, ReportList *reports, Scene *sce, int settings)
 {
+	int for_render = settings == eModifierMode_Render;
+
 	if (!(ob->transflag & OB_DUPLI)) {
 		BKE_report(reports, RPT_ERROR, "Object does not have duplis");
 		return;
@@ -301,20 +174,20 @@ void rna_Object_create_duplilist(Object *ob, ReportList *reports, Scene *sce)
 
 	/* free duplilist if a user forgets to */
 	if (ob->duplilist) {
-		BKE_reportf(reports, RPT_WARNING, "Object.dupli_list has not been freed");
+		BKE_report(reports, RPT_WARNING, "Object.dupli_list has not been freed");
 
 		free_object_duplilist(ob->duplilist);
 		ob->duplilist = NULL;
 	}
-	if (G.rendering)
+	if (G.is_rendering)
 		dupli_render_particle_set(sce, ob, 0, 1);
-	ob->duplilist = object_duplilist(sce, ob);
-	if (G.rendering)
+	ob->duplilist = object_duplilist(sce, ob, for_render);
+	if (G.is_rendering)
 		dupli_render_particle_set(sce, ob, 0, 0);
 	/* ob->duplilist should now be freed with Object.free_duplilist */
 }
 
-void rna_Object_free_duplilist(Object *ob)
+static void rna_Object_free_duplilist(Object *ob)
 {
 	if (ob->duplilist) {
 		free_object_duplilist(ob->duplilist);
@@ -328,21 +201,21 @@ static PointerRNA rna_Object_shape_key_add(Object *ob, bContext *C, ReportList *
 	Scene *scene = CTX_data_scene(C);
 	KeyBlock *kb = NULL;
 
-	if ((kb = object_insert_shape_key(scene, ob, name, from_mix))) {
+	if ((kb = BKE_object_insert_shape_key(scene, ob, name, from_mix))) {
 		PointerRNA keyptr;
 
 		RNA_pointer_create((ID *)ob->data, &RNA_ShapeKey, kb, &keyptr);
-		WM_event_add_notifier(C, NC_OBJECT|ND_DRAW, ob);
+		WM_event_add_notifier(C, NC_OBJECT | ND_DRAW, ob);
 		
 		return keyptr;
 	}
 	else {
-		BKE_reportf(reports, RPT_ERROR, "Object \"%s\"does not support shapes", ob->id.name+2);
+		BKE_reportf(reports, RPT_ERROR, "Object '%s' does not support shapes", ob->id.name + 2);
 		return PointerRNA_NULL;
 	}
 }
 
-int rna_Object_is_visible(Object *ob, Scene *sce)
+static int rna_Object_is_visible(Object *ob, Scene *sce)
 {
 	return !(ob->restrictflag & OB_RESTRICT_VIEW) && (ob->lay & sce->lay);
 }
@@ -352,19 +225,19 @@ static void rna_Mesh_assign_verts_to_group(Object *ob, bDeformGroup *group, int
                                            float weight, int assignmode)
 {
 	if (ob->type != OB_MESH) {
-		BKE_report(reports, RPT_ERROR, "Object should be of MESH type");
+		BKE_report(reports, RPT_ERROR, "Object should be of mesh type");
 		return;
 	}
 
-	Mesh *me = (Mesh*)ob->data;
+	Mesh *me = (Mesh *)ob->data;
 	int group_index = BLI_findlink(&ob->defbase, group);
 	if (group_index == -1) {
-		BKE_report(reports, RPT_ERROR, "No deform groups assigned to mesh");
+		BKE_report(reports, RPT_ERROR, "No vertex groups assigned to mesh");
 		return;
 	}
 
 	if (assignmode != WEIGHT_REPLACE && assignmode != WEIGHT_ADD && assignmode != WEIGHT_SUBTRACT) {
-		BKE_report(reports, RPT_ERROR, "Bad assignment mode" );
+		BKE_report(reports, RPT_ERROR, "Bad assignment mode");
 		return;
 	}
 
@@ -385,13 +258,13 @@ static void rna_Mesh_assign_verts_to_group(Object *ob, bDeformGroup *group, int
 #endif
 
 /* BMESH_TODO, return polygon index, not tessface */
-void rna_Object_ray_cast(Object *ob, ReportList *reports, float ray_start[3], float ray_end[3],
-                         float r_location[3], float r_normal[3], int *index)
+static void rna_Object_ray_cast(Object *ob, ReportList *reports, float ray_start[3], float ray_end[3],
+                                float r_location[3], float r_normal[3], int *index)
 {
 	BVHTreeFromMesh treeData = {NULL};
 	
 	if (ob->derivedFinal == NULL) {
-		BKE_reportf(reports, RPT_ERROR, "object \"%s\" has no mesh data to be used for ray casting", ob->id.name+2);
+		BKE_reportf(reports, RPT_ERROR, "Object '%s' has no mesh data to be used for ray casting", ob->id.name + 2);
 		return;
 	}
 
@@ -399,7 +272,7 @@ void rna_Object_ray_cast(Object *ob, ReportList *reports, float ray_start[3], fl
 	bvhtree_from_mesh_faces(&treeData, ob->derivedFinal, 0.0f, 4, 6);
 
 	if (treeData.tree == NULL) {
-		BKE_reportf(reports, RPT_ERROR, "object \"%s\" could not create internal data for ray casting", ob->id.name+2);
+		BKE_reportf(reports, RPT_ERROR, "Object '%s' could not create internal data for ray casting", ob->id.name + 2);
 		return;
 	}
 	else {
@@ -411,7 +284,8 @@ void rna_Object_ray_cast(Object *ob, ReportList *reports, float ray_start[3], fl
 		hit.index = -1;
 		
 		if (BLI_bvhtree_ray_cast(treeData.tree, ray_start, ray_nor, 0.0f, &hit,
-		                         treeData.raycast_callback, &treeData) != -1) {
+		                         treeData.raycast_callback, &treeData) != -1)
+		{
 			if (hit.dist <= dist) {
 				copy_v3_v3(r_location, hit.co);
 				copy_v3_v3(r_normal, hit.no);
@@ -426,14 +300,14 @@ void rna_Object_ray_cast(Object *ob, ReportList *reports, float ray_start[3], fl
 	*index = -1;
 }
 
-void rna_Object_closest_point_on_mesh(Object *ob, ReportList *reports, float point_co[3], float max_dist,
-                                      float n_location[3], float n_normal[3], int *index)
+static void rna_Object_closest_point_on_mesh(Object *ob, ReportList *reports, float point_co[3], float max_dist,
+                                             float n_location[3], float n_normal[3], int *index)
 {
 	BVHTreeFromMesh treeData = {NULL};
 	
 	if (ob->derivedFinal == NULL) {
-		BKE_reportf(reports, RPT_ERROR, "object \"%s\" has no mesh data to be used for finding nearest point",
-		            ob->id.name+2);
+		BKE_reportf(reports, RPT_ERROR, "Object '%s' has no mesh data to be used for finding nearest point",
+		            ob->id.name + 2);
 		return;
 	}
 
@@ -441,8 +315,8 @@ void rna_Object_closest_point_on_mesh(Object *ob, ReportList *reports, float poi
 	bvhtree_from_mesh_faces(&treeData, ob->derivedFinal, 0.0f, 4, 6);
 
 	if (treeData.tree == NULL) {
-		BKE_reportf(reports, RPT_ERROR, "object \"%s\" could not create internal data for finding nearest point",
-		            ob->id.name+2);
+		BKE_reportf(reports, RPT_ERROR, "Object '%s' could not create internal data for finding nearest point",
+		            ob->id.name + 2);
 		return;
 	}
 	else {
@@ -466,14 +340,19 @@ void rna_Object_closest_point_on_mesh(Object *ob, ReportList *reports, float poi
 
 /* ObjectBase */
 
-void rna_ObjectBase_layers_from_view(Base *base, View3D *v3d)
+static void rna_ObjectBase_layers_from_view(Base *base, View3D *v3d)
 {
 	base->lay = base->object->lay = v3d->lay;
 }
 
-int rna_Object_is_modified(Object *ob, Scene *scene, int settings)
+static int rna_Object_is_modified(Object *ob, Scene *scene, int settings)
 {
-	return object_is_modified(scene, ob) & settings;
+	return BKE_object_is_modified(scene, ob) & settings;
+}
+
+static int rna_Object_is_deform_modified(Object *ob, Scene *scene, int settings)
+{
+	return BKE_object_is_deform_modified(scene, ob) & settings;
 }
 
 #ifndef NDEBUG
@@ -514,7 +393,7 @@ void rna_Object_dm_info(struct Object *ob, int type, char *result)
 }
 #endif /* NDEBUG */
 
-#else
+#else /* RNA_RUNTIME */
 
 void RNA_api_object(StructRNA *srna)
 {
@@ -527,6 +406,8 @@ void RNA_api_object(StructRNA *srna)
 		{0, NULL, 0, NULL, NULL}
 	};
 
+	static int rna_matrix_dimsize_4x4[] = {4, 4};
+
 #ifndef NDEBUG
 	static EnumPropertyItem mesh_dm_info_items[] = {
 		{0, "SOURCE", 0, "Source", "Source mesh"},
@@ -536,16 +417,36 @@ void RNA_api_object(StructRNA *srna)
 	};
 #endif
 
+	/* Matrix space conversion */
+	func = RNA_def_function(srna, "convert_space", "rna_Scene_mat_convert_space");
+	RNA_def_function_ui_description(func, "Convert (transform) the given matrix from one space to another");
+	RNA_def_function_flag(func, FUNC_USE_REPORTS);
+	parm = RNA_def_pointer(func, "pose_bone", "PoseBone", "",
+	                       "Bone to use to define spaces (may be None, in which case only the two 'WORLD' and "
+	                       "'LOCAL' spaces are usable)");
+	parm = RNA_def_property(func, "matrix", PROP_FLOAT, PROP_MATRIX);
+	RNA_def_property_multi_array(parm, 2, rna_matrix_dimsize_4x4);
+	RNA_def_property_ui_text(parm, "", "The matrix to transform");
+	parm = RNA_def_property(func, "matrix_return", PROP_FLOAT, PROP_MATRIX);
+	RNA_def_property_multi_array(parm, 2, rna_matrix_dimsize_4x4);
+	RNA_def_property_ui_text(parm, "", "The transformed matrix");
+	RNA_def_function_output(func, parm);
+	parm = RNA_def_enum(func, "from_space", space_items, CONSTRAINT_SPACE_WORLD, "",
+	                    "The space in which 'matrix' is currently");
+	parm = RNA_def_enum(func, "to_space", space_items, CONSTRAINT_SPACE_WORLD, "",
+	                    "The space to which you want to transform 'matrix'");
+
 	/* mesh */
 	func = RNA_def_function(srna, "to_mesh", "rna_Object_to_mesh");
 	RNA_def_function_ui_description(func, "Create a Mesh datablock with modifiers applied");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
 	parm = RNA_def_pointer(func, "scene", "Scene", "", "Scene within which to evaluate modifiers");
-	RNA_def_property_flag(parm, PROP_REQUIRED|PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL);
 	parm = RNA_def_boolean(func, "apply_modifiers", 0, "", "Apply modifiers");
 	RNA_def_property_flag(parm, PROP_REQUIRED);
 	parm = RNA_def_enum(func, "settings", mesh_type_items, 0, "", "Modifier settings to apply");
 	RNA_def_property_flag(parm, PROP_REQUIRED);
+	RNA_def_boolean(func, "calc_tessface", true, "Calculate Tessellation", "Calculate tessellation faces");
 	parm = RNA_def_pointer(func, "mesh", "Mesh", "",
 	                       "Mesh created from object, remove it if it is only used for export");
 	RNA_def_function_return(func, parm);
@@ -553,10 +454,11 @@ void RNA_api_object(StructRNA *srna)
 	/* duplis */
 	func = RNA_def_function(srna, "dupli_list_create", "rna_Object_create_duplilist");
 	RNA_def_function_ui_description(func, "Create a list of dupli objects for this object, needs to "
-	                                      "be freed manually with free_dupli_list to restore the "
-	                                      "objects real matrix and layers");
+	                                "be freed manually with free_dupli_list to restore the "
+	                                "objects real matrix and layers");
 	parm = RNA_def_pointer(func, "scene", "Scene", "", "Scene within which to evaluate duplis");
-	RNA_def_property_flag(parm, PROP_REQUIRED|PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL);
+	RNA_def_enum(func, "settings", mesh_type_items, 0, "", "Generate texture coordinates for rendering");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
 
 	func = RNA_def_function(srna, "dupli_list_clear", "rna_Object_free_duplilist");
@@ -571,7 +473,7 @@ void RNA_api_object(StructRNA *srna)
 	/* Shape key */
 	func = RNA_def_function(srna, "shape_key_add", "rna_Object_shape_key_add");
 	RNA_def_function_ui_description(func, "Add shape key to an object");
-	RNA_def_function_flag(func, FUNC_USE_CONTEXT|FUNC_USE_REPORTS);
+	RNA_def_function_flag(func, FUNC_USE_CONTEXT | FUNC_USE_REPORTS);
 	RNA_def_string(func, "name", "Key", 0, "", "Unique name for the new keylock"); /* optional */
 	RNA_def_boolean(func, "from_mix", 1, "", "Create new shape from existing mix of shapes");
 	parm = RNA_def_pointer(func, "key", "ShapeKey", "", "New shape keyblock");
@@ -630,7 +532,7 @@ void RNA_api_object(StructRNA *srna)
 	func = RNA_def_function(srna, "is_visible", "rna_Object_is_visible");
 	RNA_def_function_ui_description(func, "Determine if object is visible in a given scene");
 	parm = RNA_def_pointer(func, "scene", "Scene", "", "");
-	RNA_def_property_flag(parm, PROP_REQUIRED|PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL);
 	parm = RNA_def_boolean(func, "result", 0, "", "Object visibility");
 	RNA_def_function_return(func, parm);
 
@@ -638,12 +540,20 @@ void RNA_api_object(StructRNA *srna)
 	func = RNA_def_function(srna, "is_modified", "rna_Object_is_modified");
 	RNA_def_function_ui_description(func, "Determine if this object is modified from the base mesh data");
 	parm = RNA_def_pointer(func, "scene", "Scene", "", "");
-	RNA_def_property_flag(parm, PROP_REQUIRED|PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL);
 	parm = RNA_def_enum(func, "settings", mesh_type_items, 0, "", "Modifier settings to apply");
 	RNA_def_property_flag(parm, PROP_REQUIRED);
 	parm = RNA_def_boolean(func, "result", 0, "", "Object visibility");
 	RNA_def_function_return(func, parm);
 
+	func = RNA_def_function(srna, "is_deform_modified", "rna_Object_is_deform_modified");
+	RNA_def_function_ui_description(func, "Determine if this object is modified by a deformation from the base mesh data");
+	parm = RNA_def_pointer(func, "scene", "Scene", "", "");
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL);
+	parm = RNA_def_enum(func, "settings", mesh_type_items, 0, "", "Modifier settings to apply");
+	RNA_def_property_flag(parm, PROP_REQUIRED);
+	parm = RNA_def_boolean(func, "result", 0, "", "Object visibility");
+	RNA_def_function_return(func, parm);
 
 #ifndef NDEBUG
 	/* mesh */
@@ -669,8 +579,7 @@ void RNA_api_object_base(StructRNA *srna)
 	RNA_def_function_ui_description(func,
 	                                "Sets the object layers from a 3D View (use when adding an object in local view)");
 	parm = RNA_def_pointer(func, "view", "SpaceView3D", "", "");
-	RNA_def_property_flag(parm, PROP_REQUIRED|PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL);
 }
 
-#endif
-
+#endif /* RNA_RUNTIME */
