@@ -24,11 +24,12 @@
  *  \ingroup RNA
  */
 
-
 #include <stdlib.h>
 
 #include "DNA_scene_types.h"
 
+#include "BLI_path_util.h"
+
 #include "RNA_define.h"
 #include "RNA_enum_types.h"
 
@@ -37,7 +38,6 @@
 #include "RE_engine.h"
 #include "RE_pipeline.h"
 
-#include "BKE_utildefines.h"
 
 #ifdef RNA_RUNTIME
 
@@ -50,12 +50,12 @@
 
 /* RenderEngine Callbacks */
 
-void engine_tag_redraw(RenderEngine *engine)
+static void engine_tag_redraw(RenderEngine *engine)
 {
 	engine->flag |= RE_ENGINE_DO_DRAW;
 }
 
-void engine_tag_update(RenderEngine *engine)
+static void engine_tag_update(RenderEngine *engine)
 {
 	engine->flag |= RE_ENGINE_DO_UPDATE;
 }
@@ -129,6 +129,24 @@ static void engine_view_draw(RenderEngine *engine, const struct bContext *contex
 	RNA_parameter_list_free(&list);
 }
 
+static void engine_update_script_node(RenderEngine *engine, struct bNodeTree *ntree, struct bNode *node)
+{
+	extern FunctionRNA rna_RenderEngine_update_script_node_func;
+	PointerRNA ptr, nodeptr;
+	ParameterList list;
+	FunctionRNA *func;
+
+	RNA_pointer_create(NULL, engine->type->ext.srna, engine, &ptr);
+	RNA_pointer_create((ID *)ntree, &RNA_Node, node, &nodeptr);
+	func = &rna_RenderEngine_update_script_node_func;
+
+	RNA_parameter_list_create(&list, &ptr, func);
+	RNA_parameter_set_lookup(&list, "node", &nodeptr);
+	engine->type->ext.call(NULL, &ptr, func, &list);
+
+	RNA_parameter_list_free(&list);
+}
+
 /* RenderEngine registration */
 
 static void rna_RenderEngine_unregister(Main *UNUSED(bmain), StructRNA *type)
@@ -149,7 +167,7 @@ static StructRNA *rna_RenderEngine_register(Main *bmain, ReportList *reports, vo
 	RenderEngineType *et, dummyet = {NULL};
 	RenderEngine dummyengine = {NULL};
 	PointerRNA dummyptr;
-	int have_function[4];
+	int have_function[5];
 
 	/* setup dummy engine & engine type to store static properties in */
 	dummyengine.type = &dummyet;
@@ -160,7 +178,7 @@ static StructRNA *rna_RenderEngine_register(Main *bmain, ReportList *reports, vo
 		return NULL;
 
 	if (strlen(identifier) >= sizeof(dummyet.idname)) {
-		BKE_reportf(reports, RPT_ERROR, "registering render engine class: '%s' is too long, maximum length is %d",
+		BKE_reportf(reports, RPT_ERROR, "Registering render engine class: '%s' is too long, maximum length is %d",
 		            identifier, (int)sizeof(dummyet.idname));
 		return NULL;
 	}
@@ -178,16 +196,17 @@ static StructRNA *rna_RenderEngine_register(Main *bmain, ReportList *reports, vo
 	et = MEM_callocN(sizeof(RenderEngineType), "python render engine");
 	memcpy(et, &dummyet, sizeof(dummyet));
 
-	et->ext.srna = RNA_def_struct(&BLENDER_RNA, et->idname, "RenderEngine");
+	et->ext.srna = RNA_def_struct_ptr(&BLENDER_RNA, et->idname, &RNA_RenderEngine);
 	et->ext.data = data;
 	et->ext.call = call;
 	et->ext.free = free;
 	RNA_struct_blender_type_set(et->ext.srna, et);
 
-	et->update = (have_function[0])? engine_update: NULL;
-	et->render = (have_function[1])? engine_render: NULL;
-	et->view_update = (have_function[2])? engine_view_update: NULL;
-	et->view_draw = (have_function[3])? engine_view_draw: NULL;
+	et->update = (have_function[0]) ? engine_update : NULL;
+	et->render = (have_function[1]) ? engine_render : NULL;
+	et->view_update = (have_function[2]) ? engine_view_update : NULL;
+	et->view_draw = (have_function[3]) ? engine_view_draw : NULL;
+	et->update_script_node = (have_function[4]) ? engine_update_script_node : NULL;
 
 	BLI_addtail(&R_engines, et);
 
@@ -200,66 +219,66 @@ static void **rna_RenderEngine_instance(PointerRNA *ptr)
 	return &engine->py_instance;
 }
 
-static StructRNA* rna_RenderEngine_refine(PointerRNA *ptr)
+static StructRNA *rna_RenderEngine_refine(PointerRNA *ptr)
 {
-	RenderEngine *engine = (RenderEngine*)ptr->data;
-	return (engine->type && engine->type->ext.srna)? engine->type->ext.srna: &RNA_RenderEngine;
+	RenderEngine *engine = (RenderEngine *)ptr->data;
+	return (engine->type && engine->type->ext.srna) ? engine->type->ext.srna : &RNA_RenderEngine;
 }
 
 static void rna_RenderResult_layers_begin(CollectionPropertyIterator *iter, PointerRNA *ptr)
 {
-	RenderResult *rr = (RenderResult*)ptr->data;
+	RenderResult *rr = (RenderResult *)ptr->data;
 	rna_iterator_listbase_begin(iter, &rr->layers, NULL);
 }
 
 static void rna_RenderLayer_passes_begin(CollectionPropertyIterator *iter, PointerRNA *ptr)
 {
-	RenderLayer *rl = (RenderLayer*)ptr->data;
+	RenderLayer *rl = (RenderLayer *)ptr->data;
 	rna_iterator_listbase_begin(iter, &rl->passes, NULL);
 }
 
 static int rna_RenderLayer_rect_get_length(PointerRNA *ptr, int length[RNA_MAX_ARRAY_DIMENSION])
 {
-	RenderLayer *rl = (RenderLayer*)ptr->data;
+	RenderLayer *rl = (RenderLayer *)ptr->data;
 
-	length[0] = rl->rectx*rl->recty;
+	length[0] = rl->rectx * rl->recty;
 	length[1] = 4;
 
-	return length[0]*length[1];
+	return length[0] * length[1];
 }
 
 static void rna_RenderLayer_rect_get(PointerRNA *ptr, float *values)
 {
-	RenderLayer *rl = (RenderLayer*)ptr->data;
-	memcpy(values, rl->rectf, sizeof(float)*rl->rectx*rl->recty*4);
+	RenderLayer *rl = (RenderLayer *)ptr->data;
+	memcpy(values, rl->rectf, sizeof(float) * rl->rectx * rl->recty * 4);
 }
 
 void rna_RenderLayer_rect_set(PointerRNA *ptr, const float *values)
 {
-	RenderLayer *rl = (RenderLayer*)ptr->data;
-	memcpy(rl->rectf, values, sizeof(float)*rl->rectx*rl->recty*4);
+	RenderLayer *rl = (RenderLayer *)ptr->data;
+	memcpy(rl->rectf, values, sizeof(float) * rl->rectx * rl->recty * 4);
 }
 
 static int rna_RenderPass_rect_get_length(PointerRNA *ptr, int length[RNA_MAX_ARRAY_DIMENSION])
 {
-	RenderPass *rpass = (RenderPass*)ptr->data;
+	RenderPass *rpass = (RenderPass *)ptr->data;
 
-	length[0] = rpass->rectx*rpass->recty;
+	length[0] = rpass->rectx * rpass->recty;
 	length[1] = rpass->channels;
 
-	return length[0]*length[1];
+	return length[0] * length[1];
 }
 
 static void rna_RenderPass_rect_get(PointerRNA *ptr, float *values)
 {
-	RenderPass *rpass = (RenderPass*)ptr->data;
-	memcpy(values, rpass->rect, sizeof(float)*rpass->rectx*rpass->recty*rpass->channels);
+	RenderPass *rpass = (RenderPass *)ptr->data;
+	memcpy(values, rpass->rect, sizeof(float) * rpass->rectx * rpass->recty * rpass->channels);
 }
 
 void rna_RenderPass_rect_set(PointerRNA *ptr, const float *values)
 {
-	RenderPass *rpass = (RenderPass*)ptr->data;
-	memcpy(rpass->rect, values, sizeof(float)*rpass->rectx*rpass->recty*rpass->channels);
+	RenderPass *rpass = (RenderPass *)ptr->data;
+	memcpy(rpass->rect, values, sizeof(float) * rpass->rectx * rpass->recty * rpass->channels);
 }
 
 #else /* RNA_RUNTIME */
@@ -300,6 +319,13 @@ static void rna_def_render_engine(BlenderRNA *brna)
 	RNA_def_function_flag(func, FUNC_REGISTER_OPTIONAL);
 	RNA_def_pointer(func, "context", "Context", "", "");
 
+	/* shader script callbacks */
+	func = RNA_def_function(srna, "update_script_node", NULL);
+	RNA_def_function_ui_description(func, "Compile shader script node");
+	RNA_def_function_flag(func, FUNC_REGISTER_OPTIONAL | FUNC_ALLOW_WRITE);
+	prop = RNA_def_pointer(func, "node", "Node", "", "");
+	RNA_def_property_flag(prop, PROP_RNAPTR);
+
 	/* tag for redraw */
 	RNA_def_function(srna, "tag_redraw", "engine_tag_redraw");
 	RNA_def_function_ui_description(func, "Request redraw for viewport rendering");
@@ -317,6 +343,7 @@ static void rna_def_render_engine(BlenderRNA *brna)
 	RNA_def_property_flag(prop, PROP_REQUIRED);
 	prop = RNA_def_int(func, "h", 0, 0, INT_MAX, "Height", "", 0, INT_MAX);
 	RNA_def_property_flag(prop, PROP_REQUIRED);
+	RNA_def_string(func, "layer", "", 0, "Layer", "Single layer to get render result for");  /* NULL ok here */
 	prop = RNA_def_pointer(func, "result", "RenderResult", "Result", "");
 	RNA_def_function_return(func, prop);
 
@@ -327,6 +354,7 @@ static void rna_def_render_engine(BlenderRNA *brna)
 	func = RNA_def_function(srna, "end_result", "RE_engine_end_result");
 	prop = RNA_def_pointer(func, "result", "RenderResult", "Result", "");
 	RNA_def_property_flag(prop, PROP_REQUIRED);
+	RNA_def_boolean(func, "cancel", 0, "Cancel", "Don't merge back results");
 
 	func = RNA_def_function(srna, "test_break", "RE_engine_test_break");
 	prop = RNA_def_boolean(func, "do_break", 0, "Break", "");
@@ -342,6 +370,11 @@ static void rna_def_render_engine(BlenderRNA *brna)
 	prop = RNA_def_float(func, "progress", 0, 0.0f, 1.0f, "", "Percentage of render that's done", 0.0f, 1.0f);
 	RNA_def_property_flag(prop, PROP_REQUIRED);
 
+	func = RNA_def_function(srna, "update_memory_stats", "RE_engine_update_memory_stats");
+	RNA_def_float(func, "memory_used", 0, 0.0f, FLT_MAX, "", "Current memory usage in megabytes", 0.0f, FLT_MAX);
+	RNA_def_float(func, "memory_peak", 0, 0.0f, FLT_MAX, "", "Peak memory usage in megabytes", 0.0f, FLT_MAX);
+	RNA_def_property_flag(prop, PROP_REQUIRED);
+
 	func = RNA_def_function(srna, "report", "RE_engine_report");
 	prop = RNA_def_enum_flag(func, "type", wm_report_items, 0, "Type", "");
 	RNA_def_property_flag(prop, PROP_REQUIRED);
@@ -360,13 +393,29 @@ static void rna_def_render_engine(BlenderRNA *brna)
 	RNA_def_property_pointer_sdna(prop, NULL, "camera_override");
 	RNA_def_property_struct_type(prop, "Object");
 
+	prop = RNA_def_property(srna, "tile_x", PROP_INT, PROP_UNSIGNED);
+	RNA_def_property_int_sdna(prop, NULL, "tile_x");
+	prop = RNA_def_property(srna, "tile_y", PROP_INT, PROP_UNSIGNED);
+	RNA_def_property_int_sdna(prop, NULL, "tile_y");
+
+	prop = RNA_def_property(srna, "resolution_x", PROP_INT, PROP_NONE);
+	RNA_def_property_int_sdna(prop, NULL, "resolution_x");
+	RNA_def_property_clear_flag(prop, PROP_EDITABLE);
+
+	prop = RNA_def_property(srna, "resolution_y", PROP_INT, PROP_NONE);
+	RNA_def_property_int_sdna(prop, NULL, "resolution_y");
+	RNA_def_property_clear_flag(prop, PROP_EDITABLE);
+
+	prop = RNA_def_property(srna, "use_highlight_tiles", PROP_BOOLEAN, PROP_NONE);
+	RNA_def_property_boolean_sdna(prop, NULL, "flag", RE_ENGINE_HIGHLIGHT_TILES);
+
 	/* registration */
 
 	prop = RNA_def_property(srna, "bl_idname", PROP_STRING, PROP_NONE);
 	RNA_def_property_string_sdna(prop, NULL, "type->idname");
-	RNA_def_property_flag(prop, PROP_REGISTER|PROP_NEVER_CLAMP);
+	RNA_def_property_flag(prop, PROP_REGISTER | PROP_NEVER_CLAMP);
 
-	prop = RNA_def_property(srna, "bl_label", PROP_STRING, PROP_TRANSLATE);
+	prop = RNA_def_property(srna, "bl_label", PROP_STRING, PROP_NONE);
 	RNA_def_property_string_sdna(prop, NULL, "type->name");
 	RNA_def_property_flag(prop, PROP_REGISTER);
 
@@ -398,8 +447,8 @@ static void rna_def_render_result(BlenderRNA *brna)
 	RNA_def_function_ui_description(func, "Copies the pixels of this render result from an image file");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
 	parm = RNA_def_string_file_name(func, "filename", "", FILE_MAX, "File Name",
-	                               "Filename to load into this render tile, must be no smaller than "
-	                               "the render result");
+	                                "Filename to load into this render tile, must be no smaller than "
+	                                "the render result");
 	RNA_def_property_flag(parm, PROP_REQUIRED);
 
 	RNA_define_verify_sdna(0);
@@ -434,7 +483,7 @@ static void rna_def_render_layer(BlenderRNA *brna)
 	RNA_def_function_ui_description(func, "Copies the pixels of this renderlayer from an image file");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
 	prop = RNA_def_string(func, "filename", "", 0, "Filename",
-	                     "Filename to load into this render tile, must be no smaller than the renderlayer");
+	                      "Filename to load into this render tile, must be no smaller than the renderlayer");
 	RNA_def_property_flag(prop, PROP_REQUIRED);
 	RNA_def_int(func, "x", 0, 0, INT_MAX, "Offset X",
 	            "Offset the position to copy from if the image is larger than the render layer", 0, INT_MAX);
@@ -492,7 +541,8 @@ static void rna_def_render_pass(BlenderRNA *brna)
 		{SCE_PASS_TRANSM_DIRECT, "TRANSMISSION_DIRECT", 0, "Transmission Direct", ""},
 		{SCE_PASS_TRANSM_INDIRECT, "TRANSMISSION_INDIRECT", 0, "Transmission Indirect", ""},
 		{SCE_PASS_TRANSM_COLOR, "TRANSMISSION_COLOR", 0, "Transmission Color", ""},
-		{0, NULL, 0, NULL, NULL}};
+		{0, NULL, 0, NULL, NULL}
+	};
 	
 	srna = RNA_def_struct(brna, "RenderPass", NULL);
 	RNA_def_struct_ui_text(srna, "Render Pass", "");
