@@ -53,7 +53,8 @@
 #include "BKE_mesh.h"
 #include "BKE_modifier.h"
 #include "BKE_object.h"
-#include "BKE_tessmesh.h"
+
+#include "bmesh.h"
 
 #include "depsgraph_private.h"
 
@@ -63,7 +64,7 @@
 
 static void initData(ModifierData *md)
 {
-	ArrayModifierData *amd = (ArrayModifierData*) md;
+	ArrayModifierData *amd = (ArrayModifierData *) md;
 
 	/* default to 2 duplicates distributed along the x-axis by an
 	 * offset of 1 object-width
@@ -82,8 +83,8 @@ static void initData(ModifierData *md)
 
 static void copyData(ModifierData *md, ModifierData *target)
 {
-	ArrayModifierData *amd = (ArrayModifierData*) md;
-	ArrayModifierData *tamd = (ArrayModifierData*) target;
+	ArrayModifierData *amd = (ArrayModifierData *) md;
+	ArrayModifierData *tamd = (ArrayModifierData *) target;
 
 	tamd->start_cap = amd->start_cap;
 	tamd->end_cap = amd->end_cap;
@@ -100,11 +101,11 @@ static void copyData(ModifierData *md, ModifierData *target)
 }
 
 static void foreachObjectLink(
-						ModifierData *md, Object *ob,
-	 void (*walk)(void *userData, Object *ob, Object **obpoin),
-		void *userData)
+        ModifierData *md, Object *ob,
+        void (*walk)(void *userData, Object *ob, Object **obpoin),
+        void *userData)
 {
-	ArrayModifierData *amd = (ArrayModifierData*) md;
+	ArrayModifierData *amd = (ArrayModifierData *) md;
 
 	walk(userData, ob, &amd->start_cap);
 	walk(userData, ob, &amd->end_cap);
@@ -113,9 +114,9 @@ static void foreachObjectLink(
 }
 
 static void updateDepgraph(ModifierData *md, DagForest *forest,
-	struct Scene *UNUSED(scene), Object *UNUSED(ob), DagNode *obNode)
+                           struct Scene *UNUSED(scene), Object *UNUSED(ob), DagNode *obNode)
 {
-	ArrayModifierData *amd = (ArrayModifierData*) md;
+	ArrayModifierData *amd = (ArrayModifierData *) md;
 
 	if (amd->start_cap) {
 		DagNode *curNode = dag_get_node(forest, amd->start_cap);
@@ -153,7 +154,7 @@ static float vertarray_size(MVert *mvert, int numVerts, int axis)
 
 	/* find the minimum and maximum coordinates on the desired axis */
 	min_co = max_co = mvert->co[axis];
-	++mvert;
+	mvert++;
 	for (i = 1; i < numVerts; ++i, ++mvert) {
 		if (mvert->co[axis] < min_co) min_co = mvert->co[axis];
 		if (mvert->co[axis] > max_co) max_co = mvert->co[axis];
@@ -163,8 +164,8 @@ static float vertarray_size(MVert *mvert, int numVerts, int axis)
 }
 
 static int *find_doubles_index_map(BMesh *bm, BMOperator *dupe_op,
-								   const ArrayModifierData *amd,
-								   int *index_map_length)
+                                   const ArrayModifierData *amd,
+                                   int *index_map_length)
 {
 	BMOperator find_op;
 	BMOIter oiter;
@@ -172,31 +173,31 @@ static int *find_doubles_index_map(BMesh *bm, BMOperator *dupe_op,
 	BMElem *ele;
 	int *index_map, i;
 
-	BMO_op_initf(bm, &find_op,
-				 "finddoubles verts=%av dist=%f keepverts=%s",
-				 amd->merge_dist, dupe_op, "geom");
+	BMO_op_initf(bm, &find_op, (BMO_FLAG_DEFAULTS & ~BMO_FLAG_RESPECT_HIDE),
+	             "find_doubles verts=%av dist=%f keep_verts=%s",
+	             amd->merge_dist, dupe_op, "geom");
 
 	BMO_op_exec(bm, &find_op);
-			
+
 	i = 0;
-	BMO_ITER (ele, &oiter, bm, dupe_op, "geom", BM_ALL) {
+	BMO_ITER (ele, &oiter, dupe_op->slots_in, "geom", BM_ALL) {
 		BM_elem_index_set(ele, i); /* set_dirty */
 		i++;
 	}
 
-	BMO_ITER (ele, &oiter, bm, dupe_op, "newout", BM_ALL) {
+	BMO_ITER (ele, &oiter, dupe_op->slots_out, "geom.out", BM_ALL) {
 		BM_elem_index_set(ele, i); /* set_dirty */
 		i++;
 	}
 	/* above loops over all, so set all to dirty, if this is somehow
-	 * setting valid values, this line can be remvoed - campbell */
+	 * setting valid values, this line can be removed - campbell */
 	bm->elem_index_dirty |= BM_VERT | BM_EDGE | BM_FACE;
 
 	(*index_map_length) = i;
 	index_map = MEM_callocN(sizeof(int) * (*index_map_length), "index_map");
 
 	/*element type argument doesn't do anything here*/
-	BMO_ITER (v, &oiter, bm, &find_op, "targetmapout", 0) {
+	BMO_ITER (v, &oiter, find_op.slots_out, "targetmap.out", 0) {
 		v2 = BMO_iter_map_value_p(&oiter);
 
 		index_map[BM_elem_index_get(v)] = BM_elem_index_get(v2) + 1;
@@ -214,15 +215,19 @@ static int *find_doubles_index_map(BMesh *bm, BMOperator *dupe_op,
  *
  * All verts will be tagged on exit.
  */
-static void bm_merge_dm_transform(BMesh* bm, DerivedMesh *dm, float mat[4][4],
-								  const ArrayModifierData *amd,
-								  BMOperator *dupe_op,
-								  const char *dupe_slot_name,
-								  BMOperator *weld_op)
+static void bm_merge_dm_transform(BMesh *bm, DerivedMesh *dm, float mat[4][4],
+                                  const ArrayModifierData *amd,
+                                  BMOperator *dupe_op,
+                                  BMOpSlot dupe_op_slot_args[BMO_OP_MAX_SLOTS], const char *dupe_slot_name,
+                                  BMOperator *weld_op)
 {
-	BMVert *v, *v2;
+	const int is_input = (dupe_op->slots_in == dupe_op_slot_args);
+	BMVert *v, *v2, *v3;
 	BMIter iter;
 
+	/* Add the DerivedMesh's elements to the BMesh. The pre-existing
+	 * elements were already tagged, so the new elements can be
+	 * identified by not having the BM_ELEM_TAG flag set. */
 	DM_to_bmesh_ex(dm, bm);
 
 	if (amd->flags & MOD_ARR_MERGE) {
@@ -230,14 +235,27 @@ static void bm_merge_dm_transform(BMesh* bm, DerivedMesh *dm, float mat[4][4],
 		
 		BMOIter oiter;
 		BMOperator find_op;
+		BMOpSlot *slot_targetmap;
 
-		BMO_op_initf(bm, &find_op,
-					 "finddoubles verts=%Hv dist=%f keepverts=%s",
-					 BM_ELEM_TAG, amd->merge_dist,
-					 dupe_op, dupe_slot_name);
+		BMO_op_initf(bm, &find_op, (BMO_FLAG_DEFAULTS & ~BMO_FLAG_RESPECT_HIDE),
+		             is_input ?  /* ugh */
+		             "find_doubles verts=%Hv dist=%f keep_verts=%s" :
+		             "find_doubles verts=%Hv dist=%f keep_verts=%S",
+		             BM_ELEM_TAG, amd->merge_dist,
+		             dupe_op, dupe_slot_name);
 
 		/* append the dupe's geom to the findop input verts */
-		BMO_slot_buffer_append(&find_op, "verts", dupe_op, dupe_slot_name);
+		if (is_input) {
+			BMO_slot_buffer_append(&find_op, slots_in, "verts",
+			                       dupe_op,  slots_in, dupe_slot_name);
+		}
+		else if (dupe_op->slots_out == dupe_op_slot_args) {
+			BMO_slot_buffer_append(&find_op, slots_in,  "verts",
+			                       dupe_op,  slots_out, dupe_slot_name);
+		}
+		else {
+			BLI_assert(0);
+		}
 
 		/* transform and tag verts */
 		BM_ITER_MESH (v, &iter, bm, BM_VERTS_OF_MESH) {
@@ -249,10 +267,17 @@ static void bm_merge_dm_transform(BMesh* bm, DerivedMesh *dm, float mat[4][4],
 
 		BMO_op_exec(bm, &find_op);
 
+		slot_targetmap = BMO_slot_get(weld_op->slots_in, "targetmap");
+
 		/* add new merge targets to weld operator */
-		BMO_ITER (v, &oiter, bm, &find_op, "targetmapout", 0) {
+		BMO_ITER (v, &oiter, find_op.slots_out, "targetmap.out", 0) {
 			v2 = BMO_iter_map_value_p(&oiter);
-			BMO_slot_map_ptr_insert(bm, weld_op, "targetmap", v, v2);
+			/* check in case the target vertex (v2) is already marked
+			 * for merging */
+			while ((v3 = BMO_slot_map_elem_get(slot_targetmap, v2))) {
+				v2 = v3;
+			}
+			BMO_slot_map_elem_insert(weld_op, slot_targetmap, v, v2);
 		}
 
 		BMO_op_finish(bm, &find_op);
@@ -268,30 +293,33 @@ static void bm_merge_dm_transform(BMesh* bm, DerivedMesh *dm, float mat[4][4],
 	}
 }
 
-static void merge_first_last(BMesh* bm,
-							 const ArrayModifierData *amd,
-							 BMOperator *dupe_first,
-							 BMOperator *dupe_last,
-							 BMOperator *weld_op)
+static void merge_first_last(BMesh *bm,
+                             const ArrayModifierData *amd,
+                             BMOperator *dupe_first,
+                             BMOperator *dupe_last,
+                             BMOperator *weld_op)
 {
 	BMOperator find_op;
 	BMOIter oiter;
 	BMVert *v, *v2;
+	BMOpSlot *slot_targetmap;
 
-	BMO_op_initf(bm, &find_op,
-				 "finddoubles verts=%s dist=%f keepverts=%s",
-				 dupe_first, "geom", amd->merge_dist,
-				 dupe_first, "geom");
+	BMO_op_initf(bm, &find_op, (BMO_FLAG_DEFAULTS & ~BMO_FLAG_RESPECT_HIDE),
+	             "find_doubles verts=%s dist=%f keep_verts=%s",
+	             dupe_first, "geom", amd->merge_dist,
+	             dupe_first, "geom");
 
 	/* append the last dupe's geom to the findop input verts */
-	BMO_slot_buffer_append(&find_op, "verts", dupe_last, "newout");
+	BMO_slot_buffer_append(&find_op,  slots_in,  "verts",
+	                       dupe_last, slots_out, "geom.out");
 
 	BMO_op_exec(bm, &find_op);
 
 	/* add new merge targets to weld operator */
-	BMO_ITER (v, &oiter, bm, &find_op, "targetmapout", 0) {
+	slot_targetmap = BMO_slot_get(weld_op->slots_in, "targetmap");
+	BMO_ITER (v, &oiter, find_op.slots_out, "targetmap.out", 0) {
 		v2 = BMO_iter_map_value_p(&oiter);
-		BMO_slot_map_ptr_insert(bm, weld_op, "targetmap", v, v2);
+		BMO_slot_map_elem_insert(weld_op, slot_targetmap, v, v2);
 	}
 
 	BMO_op_finish(bm, &find_op);
@@ -302,10 +330,11 @@ static DerivedMesh *arrayModifier_doArray(ArrayModifierData *amd,
                                           int UNUSED(initFlags))
 {
 	DerivedMesh *result;
-	BMEditMesh *em = DM_to_editbmesh(dm, NULL, FALSE);
+	BMesh *bm = DM_to_bmesh(dm);
 	BMOperator first_dupe_op, dupe_op, old_dupe_op, weld_op;
 	BMVert **first_geom = NULL;
-	int i, j, indexLen;
+	int i, j;
+	int index_len = -1;  /* initialize to an invalid value */
 	/* offset matrix */
 	float offset[4][4];
 	float final_offset[4][4];
@@ -315,6 +344,7 @@ static DerivedMesh *arrayModifier_doArray(ArrayModifierData *amd,
 	int *indexMap = NULL;
 	DerivedMesh *start_cap = NULL, *end_cap = NULL;
 	MVert *src_mvert;
+	BMOpSlot *slot_targetmap = NULL;  /* for weldop */
 
 	/* need to avoid infinite recursion here */
 	if (amd->start_cap && amd->start_cap != ob)
@@ -331,8 +361,7 @@ static DerivedMesh *arrayModifier_doArray(ArrayModifierData *amd,
 		add_v3_v3v3(offset[3], offset[3], amd->offset);
 	if (amd->offset_type & MOD_ARR_OFF_RELATIVE) {
 		for (j = 0; j < 3; j++)
-			offset[3][j] += amd->scale[j] * vertarray_size(src_mvert,
-					maxVerts, j);
+			offset[3][j] += amd->scale[j] * vertarray_size(src_mvert, maxVerts, j);
 	}
 
 	if ((amd->offset_type & MOD_ARR_OFF_OBJ) && (amd->offset_ob)) {
@@ -356,15 +385,15 @@ static DerivedMesh *arrayModifier_doArray(ArrayModifierData *amd,
 			float tmp_mat[3][3];
 			float scale;
 			
-			object_to_mat3(amd->curve_ob, tmp_mat);
+			BKE_object_to_mat3(amd->curve_ob, tmp_mat);
 			scale = mat3_to_scale(tmp_mat);
 				
 			if (!cu->path) {
 				cu->flag |= CU_PATH; // needed for path & bevlist
-				makeDispListCurveTypes(scene, amd->curve_ob, 0);
+				BKE_displist_make_curveTypes(scene, amd->curve_ob, 0);
 			}
 			if (cu->path)
-				length = scale*cu->path->totdist;
+				length = scale * cu->path->totdist;
 		}
 	}
 
@@ -388,7 +417,7 @@ static DerivedMesh *arrayModifier_doArray(ArrayModifierData *amd,
 	/* calculate the offset matrix of the final copy (for merging) */
 	unit_m4(final_offset);
 
-	for (j=0; j < count - 1; j++) {
+	for (j = 0; j < count - 1; j++) {
 		mult_m4_m4m4(tmp_mat, offset, final_offset);
 		copy_m4_m4(final_offset, tmp_mat);
 	}
@@ -399,30 +428,39 @@ static DerivedMesh *arrayModifier_doArray(ArrayModifierData *amd,
 	 * cleaner way to do this. One possibility: a "mirror" BMOp would
 	 * certainly help by compressing it all into one top-level BMOp that
 	 * executes a lot of second-level BMOps. */
-	BMO_push(em->bm, NULL);
-	bmesh_edit_begin(em->bm, 0);
+	BM_mesh_elem_toolflags_ensure(bm);
+	BMO_push(bm, NULL);
+	bmesh_edit_begin(bm, 0);
 
-	if (amd->flags & MOD_ARR_MERGE)
-		BMO_op_init(em->bm, &weld_op, "weldverts");
+	if (amd->flags & MOD_ARR_MERGE) {
+		BMO_op_init(bm, &weld_op, (BMO_FLAG_DEFAULTS & ~BMO_FLAG_RESPECT_HIDE),
+		            "weld_verts");
 
-	BMO_op_initf(em->bm, &dupe_op, "dupe geom=%avef");
+		slot_targetmap = BMO_slot_get(weld_op.slots_in, "targetmap");
+	}
+
+	BMO_op_initf(bm, &dupe_op, (BMO_FLAG_DEFAULTS & ~BMO_FLAG_RESPECT_HIDE),
+	             "duplicate geom=%avef");
 	first_dupe_op = dupe_op;
 
-	for (j=0; j < count - 1; j++) {
+	for (j = 0; j < count - 1; j++) {
 		BMVert *v, *v2, *v3;
 		BMOpSlot *geom_slot;
-		BMOpSlot *newout_slot;
+		BMOpSlot *geom_out_slot;
 		BMOIter oiter;
 
-		if (j != 0)
-			BMO_op_initf(em->bm, &dupe_op, "dupe geom=%s", &old_dupe_op, "newout");
-		BMO_op_exec(em->bm, &dupe_op);
+		if (j != 0) {
+			BMO_op_initf(bm, &dupe_op,
+			             (BMO_FLAG_DEFAULTS & ~BMO_FLAG_RESPECT_HIDE),
+			             "duplicate geom=%S", &old_dupe_op, "geom.out");
+		}
+		BMO_op_exec(bm, &dupe_op);
 
-		geom_slot = BMO_slot_get(&dupe_op, "geom");
-		newout_slot = BMO_slot_get(&dupe_op, "newout");
+		geom_slot   = BMO_slot_get(dupe_op.slots_in,  "geom");
+		geom_out_slot = BMO_slot_get(dupe_op.slots_out, "geom.out");
 
 		if ((amd->flags & MOD_ARR_MERGEFINAL) && j == 0) {
-			int first_geom_bytes = sizeof(BMVert*) * geom_slot->len;
+			int first_geom_bytes = sizeof(BMVert *) * geom_slot->len;
 				
 			/* make a copy of the initial geometry ordering so the
 			 * last duplicate can be merged into it */
@@ -431,33 +469,36 @@ static DerivedMesh *arrayModifier_doArray(ArrayModifierData *amd,
 		}
 
 		/* apply transformation matrix */
-		BMO_ITER (v, &oiter, em->bm, &dupe_op, "newout", BM_VERT) {
+		BMO_ITER (v, &oiter, dupe_op.slots_out, "geom.out", BM_VERT) {
 			mul_m4_v3(offset, v->co);
 		}
 
 		if (amd->flags & MOD_ARR_MERGE) {
 			/*calculate merge mapping*/
 			if (j == 0) {
-				indexMap = find_doubles_index_map(em->bm, &dupe_op,
-												  amd, &indexLen);
+				indexMap = find_doubles_index_map(bm, &dupe_op,
+				                                  amd, &index_len);
 			}
 
 			#define _E(s, i) ((BMVert **)(s)->data.buf)[i]
 
-			for (i=0; i<indexLen; i++) {
+			/* ensure this is set */
+			BLI_assert(index_len != -1);
+
+			for (i = 0; i < index_len; i++) {
 				if (!indexMap[i]) continue;
 
-				/* merge v (from 'newout') into v2 (from old 'geom') */
-				v = _E(newout_slot, i - geom_slot->len);
-				v2 = _E(geom_slot, indexMap[i]-1);
+				/* merge v (from 'geom.out') into v2 (from old 'geom') */
+				v = _E(geom_out_slot, i - geom_slot->len);
+				v2 = _E(geom_slot, indexMap[i] - 1);
 
 				/* check in case the target vertex (v2) is already marked
 				 * for merging */
-				while ((v3 = BMO_slot_map_ptr_get(em->bm, &weld_op, "targetmap", v2))) {
+				while ((v3 = BMO_slot_map_elem_get(slot_targetmap, v2))) {
 					v2 = v3;
 				}
 
-				BMO_slot_map_ptr_insert(em->bm, &weld_op, "targetmap", v, v2);
+				BMO_slot_map_elem_insert(&weld_op, slot_targetmap, v, v2);
 			}
 
 			#undef _E
@@ -469,7 +510,7 @@ static DerivedMesh *arrayModifier_doArray(ArrayModifierData *amd,
 			first_dupe_op = dupe_op;
 		
 		if (j >= 2)
-			BMO_op_finish(em->bm, &old_dupe_op);
+			BMO_op_finish(bm, &old_dupe_op);
 		old_dupe_op = dupe_op;
 	}
 
@@ -482,45 +523,45 @@ static DerivedMesh *arrayModifier_doArray(ArrayModifierData *amd,
 		 * loop) the offset between first and last is different from
 		 * dupe X to dupe X+1. */
 
-		merge_first_last(em->bm, amd, &first_dupe_op, &dupe_op, &weld_op);
+		merge_first_last(bm, amd, &first_dupe_op, &dupe_op, &weld_op);
 	}
 
 	/* start capping */
 	if (start_cap || end_cap) {
-		BM_mesh_elem_hflag_enable_all(em->bm, BM_VERT, BM_ELEM_TAG, FALSE);
+		BM_mesh_elem_hflag_enable_all(bm, BM_VERT, BM_ELEM_TAG, FALSE);
 
 		if (start_cap) {
 			float startoffset[4][4];
 			invert_m4_m4(startoffset, offset);
-			bm_merge_dm_transform(em->bm, start_cap, startoffset, amd,
-				                  &first_dupe_op, "geom", &weld_op);
+			bm_merge_dm_transform(bm, start_cap, startoffset, amd,
+			                      &first_dupe_op, first_dupe_op.slots_in, "geom", &weld_op);
 		}
 
 		if (end_cap) {
 			float endoffset[4][4];
 			mult_m4_m4m4(endoffset, offset, final_offset);
-			bm_merge_dm_transform(em->bm, end_cap, endoffset, amd,
-				&dupe_op, count == 1 ? "geom" : "newout", &weld_op);
+			bm_merge_dm_transform(bm, end_cap, endoffset, amd,
+			                      &dupe_op, (count == 1) ? dupe_op.slots_in : dupe_op.slots_out,
+			                      (count == 1) ? "geom" : "geom.out", &weld_op);
 		}
 	}
 	/* done capping */
 
 	/* free remaining dupe operators */
-	BMO_op_finish(em->bm, &first_dupe_op);
+	BMO_op_finish(bm, &first_dupe_op);
 	if (count > 2)
-		BMO_op_finish(em->bm, &dupe_op);
+		BMO_op_finish(bm, &dupe_op);
 
 	/* run merge operator */
 	if (amd->flags & MOD_ARR_MERGE) {
-		BMO_op_exec(em->bm, &weld_op);
-		BMO_op_finish(em->bm, &weld_op);
+		BMO_op_exec(bm, &weld_op);
+		BMO_op_finish(bm, &weld_op);
 	}
 
 	/* Bump the stack level back down to match the adjustment up above */
-	BMO_pop(em->bm);
+	BMO_pop(bm);
 
-	BLI_assert(em->looptris == NULL);
-	result = CDDM_from_BMEditMesh(em, NULL, FALSE, FALSE);
+	result = CDDM_from_bmesh(bm, FALSE);
 
 	if ((amd->offset_type & MOD_ARR_OFF_OBJ) && (amd->offset_ob)) {
 		/* Update normals in case offset object has rotation. */
@@ -531,8 +572,8 @@ static DerivedMesh *arrayModifier_doArray(ArrayModifierData *amd,
 		CDDM_calc_normals(result);
 	}
 
-	BMEdit_Free(em);
-	MEM_freeN(em);
+	BM_mesh_free(bm);
+
 	if (indexMap)
 		MEM_freeN(indexMap);
 	if (first_geom)
@@ -542,12 +583,11 @@ static DerivedMesh *arrayModifier_doArray(ArrayModifierData *amd,
 }
 
 static DerivedMesh *applyModifier(ModifierData *md, Object *ob,
-						DerivedMesh *dm,
-						int UNUSED(useRenderParams),
-						int UNUSED(isFinalCalc))
+                                  DerivedMesh *dm,
+                                  ModifierApplyFlag UNUSED(flag))
 {
 	DerivedMesh *result;
-	ArrayModifierData *amd = (ArrayModifierData*) md;
+	ArrayModifierData *amd = (ArrayModifierData *) md;
 
 	result = arrayModifier_doArray(amd, md->scene, ob, dm, 0);
 
@@ -558,10 +598,10 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob,
 }
 
 static DerivedMesh *applyModifierEM(ModifierData *md, Object *ob,
-						struct BMEditMesh *UNUSED(editData),
-						DerivedMesh *dm)
+                                    struct BMEditMesh *UNUSED(editData),
+                                    DerivedMesh *dm)
 {
-	return applyModifier(md, ob, dm, 0, 1);
+	return applyModifier(md, ob, dm, MOD_APPLY_USECACHE);
 }
 
 
@@ -570,11 +610,11 @@ ModifierTypeInfo modifierType_Array = {
 	/* structName */        "ArrayModifierData",
 	/* structSize */        sizeof(ArrayModifierData),
 	/* type */              eModifierTypeType_Constructive,
-	/* flags */             eModifierTypeFlag_AcceptsMesh
-							| eModifierTypeFlag_SupportsMapping
-							| eModifierTypeFlag_SupportsEditmode
-							| eModifierTypeFlag_EnableInEditmode
-							| eModifierTypeFlag_AcceptsCVs,
+	/* flags */             eModifierTypeFlag_AcceptsMesh |
+	                        eModifierTypeFlag_SupportsMapping |
+	                        eModifierTypeFlag_SupportsEditmode |
+	                        eModifierTypeFlag_EnableInEditmode |
+	                        eModifierTypeFlag_AcceptsCVs,
 
 	/* copyData */          copyData,
 	/* deformVerts */       NULL,
