@@ -55,7 +55,7 @@
 
 static void initData(ModifierData *md) 
 {
-	CollisionModifierData *collmd = (CollisionModifierData*) md;
+	CollisionModifierData *collmd = (CollisionModifierData *) md;
 	
 	collmd->x = NULL;
 	collmd->xnew = NULL;
@@ -69,10 +69,9 @@ static void initData(ModifierData *md)
 
 static void freeData(ModifierData *md)
 {
-	CollisionModifierData *collmd = (CollisionModifierData*) md;
+	CollisionModifierData *collmd = (CollisionModifierData *) md;
 	
-	if (collmd) 
-	{
+	if (collmd) {
 		if (collmd->bvhtree)
 			BLI_bvhtree_free(collmd->bvhtree);
 		if (collmd->x)
@@ -106,57 +105,51 @@ static int dependsOnTime(ModifierData *UNUSED(md))
 }
 
 static void deformVerts(ModifierData *md, Object *ob,
-						DerivedMesh *derivedData,
-						float (*vertexCos)[3],
-						int UNUSED(numVerts),
-						int UNUSED(useRenderParams),
-						int UNUSED(isFinalCalc))
+                        DerivedMesh *derivedData,
+                        float (*vertexCos)[3],
+                        int UNUSED(numVerts),
+                        ModifierApplyFlag UNUSED(flag))
 {
-	CollisionModifierData *collmd = (CollisionModifierData*) md;
+	CollisionModifierData *collmd = (CollisionModifierData *) md;
 	DerivedMesh *dm = NULL;
 	MVert *tempVert = NULL;
 	
 	/* if possible use/create DerivedMesh */
 	if (derivedData) dm = CDDM_copy(derivedData);
-	else if (ob->type==OB_MESH) dm = CDDM_from_mesh(ob->data, ob);
+	else if (ob->type == OB_MESH) dm = CDDM_from_mesh(ob->data, ob);
 	
-	if (!ob->pd)
-	{
+	if (!ob->pd) {
 		printf("CollisionModifier deformVerts: Should not happen!\n");
 		return;
 	}
 	
-	if (dm)
-	{
+	if (dm) {
 		float current_time = 0;
 		unsigned int numverts = 0;
 
 		CDDM_apply_vert_coords(dm, vertexCos);
 		CDDM_calc_normals(dm);
 		
-		current_time = BKE_curframe(md->scene);
+		current_time = BKE_scene_frame_get(md->scene);
 		
-		if (G.rt > 0)
+		if (G.debug_value > 0)
 			printf("current_time %f, collmd->time_xnew %f\n", current_time, collmd->time_xnew);
 		
-		numverts = dm->getNumVerts ( dm );
+		numverts = dm->getNumVerts(dm);
 		
-		if ((current_time > collmd->time_xnew)|| (BKE_ptcache_get_continue_physics()))
-		{
+		if (current_time > collmd->time_xnew) {
 			unsigned int i;
 
-			// check if mesh has changed
+			/* check if mesh has changed */
 			if (collmd->x && (numverts != collmd->numverts))
 				freeData((ModifierData *)collmd);
-			
-			if (collmd->time_xnew == -1000) // first time
-			{
-				collmd->x = dm->dupVertArray(dm); // frame start position
-				
-				for ( i = 0; i < numverts; i++ )
-				{
-					// we save global positions
-					mul_m4_v3( ob->obmat, collmd->x[i].co );
+
+			if (collmd->time_xnew == -1000) { /* first time */
+				collmd->x = dm->dupVertArray(dm); /* frame start position */
+
+				for (i = 0; i < numverts; i++) {
+					/* we save global positions */
+					mul_m4_v3(ob->obmat, collmd->x[i].co);
 				}
 				
 				collmd->xnew = MEM_dupallocN(collmd->x); // frame end position
@@ -171,27 +164,27 @@ static void deformVerts(ModifierData *md, Object *ob,
 				collmd->mfaces = dm->dupTessFaceArray(dm);
 				collmd->numfaces = dm->getNumTessFaces(dm);
 				
-				// create bounding box hierarchy
+				/* create bounding box hierarchy */
 				collmd->bvhtree = bvhtree_build_from_mvert(collmd->mfaces, collmd->numfaces, collmd->x, numverts, ob->pd->pdef_sboft);
-				
+
 				collmd->time_x = collmd->time_xnew = current_time;
 			}
 			else if (numverts == collmd->numverts) {
-				// put positions to old positions
+				/* put positions to old positions */
 				tempVert = collmd->x;
 				collmd->x = collmd->xnew;
 				collmd->xnew = tempVert;
 				collmd->time_x = collmd->time_xnew;
-				
-				memcpy(collmd->xnew, dm->getVertArray(dm), numverts*sizeof(MVert));
-				
+
+				memcpy(collmd->xnew, dm->getVertArray(dm), numverts * sizeof(MVert));
+
 				for (i = 0; i < numverts; i++) {
-					// we save global positions
-					mul_m4_v3( ob->obmat, collmd->xnew[i].co );
+					/* we save global positions */
+					mul_m4_v3(ob->obmat, collmd->xnew[i].co);
 				}
 				
-				memcpy(collmd->current_xnew, collmd->x, numverts*sizeof(MVert));
-				memcpy(collmd->current_x, collmd->x, numverts*sizeof(MVert));
+				memcpy(collmd->current_xnew, collmd->x, numverts * sizeof(MVert));
+				memcpy(collmd->current_x, collmd->x, numverts * sizeof(MVert));
 				
 				/* check if GUI setting has changed for bvh */
 				if (collmd->bvhtree) {
@@ -207,8 +200,8 @@ static void deformVerts(ModifierData *md, Object *ob,
 					collmd->bvhtree = bvhtree_build_from_mvert(collmd->mfaces, collmd->numfaces, collmd->current_x, numverts, ob->pd->pdef_sboft);
 				}
 				else {
-					// recalc static bounding boxes
-					bvhtree_update_from_mvert ( collmd->bvhtree, collmd->mfaces, collmd->numfaces, collmd->current_x, collmd->current_xnew, collmd->numverts, 1 );
+					/* recalc static bounding boxes */
+					bvhtree_update_from_mvert(collmd->bvhtree, collmd->mfaces, collmd->numfaces, collmd->current_x, collmd->current_xnew, collmd->numverts, 1);
 				}
 				
 				collmd->time_xnew = current_time;
@@ -238,8 +231,8 @@ ModifierTypeInfo modifierType_Collision = {
 	/* structName */        "CollisionModifierData",
 	/* structSize */        sizeof(CollisionModifierData),
 	/* type */              eModifierTypeType_OnlyDeform,
-	/* flags */             eModifierTypeFlag_AcceptsMesh
-							| eModifierTypeFlag_Single,
+	/* flags */             eModifierTypeFlag_AcceptsMesh |
+	                        eModifierTypeFlag_Single,
 
 	/* copyData */          NULL,
 	/* deformVerts */       deformVerts,
