@@ -73,9 +73,9 @@ static void get_vert2geom_distance(int numVerts, float (*v_cos)[3],
 	BVHTreeFromMesh treeData_v = NULL_BVHTreeFromMesh;
 	BVHTreeFromMesh treeData_e = NULL_BVHTreeFromMesh;
 	BVHTreeFromMesh treeData_f = NULL_BVHTreeFromMesh;
-	BVHTreeNearest  nearest_v  = NULL_BVHTreeNearest;
-	BVHTreeNearest  nearest_e  = NULL_BVHTreeNearest;
-	BVHTreeNearest  nearest_f  = NULL_BVHTreeNearest;
+	BVHTreeNearest nearest_v   = NULL_BVHTreeNearest;
+	BVHTreeNearest nearest_e   = NULL_BVHTreeNearest;
+	BVHTreeNearest nearest_f   = NULL_BVHTreeNearest;
 
 	if (dist_v) {
 		/* Create a bvh-tree of the given target's verts. */
@@ -107,12 +107,12 @@ static void get_vert2geom_distance(int numVerts, float (*v_cos)[3],
 	/*nearest_v.dist  = nearest_e.dist  = nearest_f.dist  = FLT_MAX;*/
 	/* Find the nearest vert/edge/face. */
 #ifndef __APPLE__
-#pragma omp parallel for default(none) private(i) firstprivate(nearest_v,nearest_e,nearest_f) \
-                         shared(treeData_v,treeData_e,treeData_f,numVerts,v_cos,dist_v,dist_e, \
-                                dist_f,loc2trgt) \
+#pragma omp parallel for default(none) private(i) firstprivate(nearest_v, nearest_e, nearest_f) \
+                         shared(treeData_v, treeData_e, treeData_f, numVerts, v_cos, dist_v, dist_e, \
+                                dist_f, loc2trgt) \
                          schedule(static)
 #endif
-	for (i = 0; i < numVerts; ++i) {
+	for (i = 0; i < numVerts; i++) {
 		float tmp_co[3];
 
 		/* Convert the vertex to tree coordinates. */
@@ -159,11 +159,11 @@ static void get_vert2geom_distance(int numVerts, float (*v_cos)[3],
  * Note that it works in final world space (i.e. with constraints etc. applied).
  */
 static void get_vert2ob_distance(int numVerts, float (*v_cos)[3], float *dist,
-                                 Object* ob, Object* obr)
+                                 Object *ob, Object *obr)
 {
 	/* Vertex and ref object coordinates. */
 	float v_wco[3];
-	unsigned int i= numVerts;
+	unsigned int i = numVerts;
 
 	while (i-- > 0) {
 		/* Get world-coordinates of the vertex (constraints and anim included). */
@@ -177,18 +177,18 @@ static void get_vert2ob_distance(int numVerts, float (*v_cos)[3], float *dist,
  * Returns the real distance between an object and another reference object.
  * Note that it works in final world space (i.e. with constraints etc. applied).
  */
-static float get_ob2ob_distance(const Object* ob, const Object* obr)
+static float get_ob2ob_distance(const Object *ob, const Object *obr)
 {
 	return len_v3v3(ob->obmat[3], obr->obmat[3]); 
 }
 
 /**
- * Maps distances to weights, with an optionnal "smoothing" mapping.
+ * Maps distances to weights, with an optional "smoothing" mapping.
  */
-void do_map(float *weights, const int nidx, const float min_d, const float max_d, short mode)
+static void do_map(float *weights, const int nidx, const float min_d, const float max_d, short mode)
 {
-	const float range_inv= 1.0f / (max_d - min_d); /* invert since multiplication is faster */
-	unsigned int i= nidx;
+	const float range_inv = 1.0f / (max_d - min_d); /* invert since multiplication is faster */
+	unsigned int i = nidx;
 	if (max_d == min_d) {
 		while (i-- > 0) {
 			weights[i] = (weights[i] >= max_d) ? 1.0f : 0.0f; /* "Step" behavior... */
@@ -196,16 +196,16 @@ void do_map(float *weights, const int nidx, const float min_d, const float max_d
 	}
 	else if (max_d > min_d) {
 		while (i-- > 0) {
-			if     (weights[i] >= max_d) weights[i]= 1.0f; /* most likely case first */
-			else if (weights[i] <= min_d) weights[i]= 0.0f;
-			else                         weights[i]= (weights[i] - min_d) * range_inv;
+			if     (weights[i] >= max_d) weights[i] = 1.0f;  /* most likely case first */
+			else if (weights[i] <= min_d) weights[i] = 0.0f;
+			else weights[i] = (weights[i] - min_d) * range_inv;
 		}
 	}
 	else {
 		while (i-- > 0) {
-			if     (weights[i] <= max_d) weights[i]= 1.0f; /* most likely case first */
-			else if (weights[i] >= min_d) weights[i]= 0.0f;
-			else                         weights[i]= (weights[i] - min_d) * range_inv;
+			if     (weights[i] <= max_d) weights[i] = 1.0f;  /* most likely case first */
+			else if (weights[i] >= min_d) weights[i] = 0.0f;
+			else weights[i] = (weights[i] - min_d) * range_inv;
 		}
 	}
 
@@ -219,7 +219,7 @@ void do_map(float *weights, const int nidx, const float min_d, const float max_d
  **************************************/
 static void initData(ModifierData *md)
 {
-	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData*) md;
+	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData *) md;
 
 	wmd->proximity_mode       = MOD_WVG_PROXIMITY_OBJECT;
 	wmd->proximity_flags      = MOD_WVG_PROXIMITY_GEOM_VERTS;
@@ -234,8 +234,8 @@ static void initData(ModifierData *md)
 
 static void copyData(ModifierData *md, ModifierData *target)
 {
-	WeightVGProximityModifierData *wmd  = (WeightVGProximityModifierData*) md;
-	WeightVGProximityModifierData *twmd = (WeightVGProximityModifierData*) target;
+	WeightVGProximityModifierData *wmd  = (WeightVGProximityModifierData *) md;
+	WeightVGProximityModifierData *twmd = (WeightVGProximityModifierData *) target;
 
 	BLI_strncpy(twmd->defgrp_name, wmd->defgrp_name, sizeof(twmd->defgrp_name));
 	twmd->proximity_mode         = wmd->proximity_mode;
@@ -257,7 +257,7 @@ static void copyData(ModifierData *md, ModifierData *target)
 
 static CustomDataMask requiredDataMask(Object *UNUSED(ob), ModifierData *md)
 {
-	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData*) md;
+	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData *) md;
 	CustomDataMask dataMask = 0;
 
 	/* We need vertex groups! */
@@ -274,7 +274,7 @@ static CustomDataMask requiredDataMask(Object *UNUSED(ob), ModifierData *md)
 
 static int dependsOnTime(ModifierData *md)
 {
-	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData*) md;
+	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData *) md;
 
 	if (wmd->mask_texture)
 		return BKE_texture_dependsOnTime(wmd->mask_texture);
@@ -285,14 +285,14 @@ static void foreachObjectLink(ModifierData *md, Object *ob,
                               void (*walk)(void *userData, Object *ob, Object **obpoin),
                               void *userData)
 {
-	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData*) md;
+	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData *) md;
 	walk(userData, ob, &wmd->proximity_ob_target);
 	walk(userData, ob, &wmd->mask_tex_map_obj);
 }
 
 static void foreachIDLink(ModifierData *md, Object *ob, IDWalkFunc walk, void *userData)
 {
-	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData*) md;
+	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData *) md;
 
 	walk(userData, ob, (ID **)&wmd->mask_texture);
 
@@ -307,30 +307,30 @@ static void foreachTexLink(ModifierData *md, Object *ob, TexWalkFunc walk, void
 static void updateDepgraph(ModifierData *md, DagForest *forest, struct Scene *UNUSED(scene),
                            Object *UNUSED(ob), DagNode *obNode)
 {
-	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData*) md;
+	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData *) md;
 	DagNode *curNode;
 
 	if (wmd->proximity_ob_target) {
 		curNode = dag_get_node(forest, wmd->proximity_ob_target);
-		dag_add_relation(forest, curNode, obNode, DAG_RL_DATA_DATA|DAG_RL_OB_DATA,
+		dag_add_relation(forest, curNode, obNode, DAG_RL_DATA_DATA | DAG_RL_OB_DATA,
 		                 "WeightVGProximity Modifier");
 	}
 
 	if (wmd->mask_tex_map_obj && wmd->mask_tex_mapping == MOD_DISP_MAP_OBJECT) {
 		curNode = dag_get_node(forest, wmd->mask_tex_map_obj);
 
-		dag_add_relation(forest, curNode, obNode, DAG_RL_DATA_DATA|DAG_RL_OB_DATA,
+		dag_add_relation(forest, curNode, obNode, DAG_RL_DATA_DATA | DAG_RL_OB_DATA,
 		                 "WeightVGProximity Modifier");
 	}
 
 	if (wmd->mask_tex_mapping == MOD_DISP_MAP_GLOBAL)
-		dag_add_relation(forest, obNode, obNode, DAG_RL_DATA_DATA|DAG_RL_OB_DATA,
+		dag_add_relation(forest, obNode, obNode, DAG_RL_DATA_DATA | DAG_RL_OB_DATA,
 		                 "WeightVGProximity Modifier");
 }
 
 static int isDisabled(ModifierData *md, int UNUSED(useRenderParams))
 {
-	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData*) md;
+	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData *) md;
 	/* If no vertex group, bypass. */
 	if (wmd->defgrp_name[0] == '\0') return 1;
 	/* If no target object, bypass. */
@@ -338,19 +338,19 @@ static int isDisabled(ModifierData *md, int UNUSED(useRenderParams))
 }
 
 static DerivedMesh *applyModifier(ModifierData *md, Object *ob, DerivedMesh *derivedData,
-                                  int UNUSED(useRenderParams), int UNUSED(isFinalCalc))
+                                  ModifierApplyFlag UNUSED(flag))
 {
-	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData*) md;
+	WeightVGProximityModifierData *wmd = (WeightVGProximityModifierData *) md;
 	DerivedMesh *dm = derivedData;
 	MDeformVert *dvert = NULL;
 	MDeformWeight **dw, **tdw;
 	int numVerts;
 	float (*v_cos)[3] = NULL; /* The vertices coordinates. */
 	Object *obr = NULL; /* Our target object. */
-	int defgrp_idx;
+	int defgrp_index;
 	float *tw = NULL;
 	float *org_w = NULL;
-	float *new_w =NULL;
+	float *new_w = NULL;
 	int *tidx, *indices = NULL;
 	int numIdx = 0;
 	int i;
@@ -378,8 +378,8 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob, DerivedMesh *der
 		return dm;
 
 	/* Get vgroup idx from its name. */
-	defgrp_idx = defgroup_name_index(ob, wmd->defgrp_name);
-	if (defgrp_idx < 0)
+	defgrp_index = defgroup_name_index(ob, wmd->defgrp_name);
+	if (defgrp_index == -1)
 		return dm;
 
 	dvert = CustomData_duplicate_referenced_layer(&dm->vertData, CD_MDEFORMVERT, numVerts);
@@ -392,9 +392,9 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob, DerivedMesh *der
 	 */
 	tidx = MEM_mallocN(sizeof(int) * numVerts, "WeightVGProximity Modifier, tidx");
 	tw = MEM_mallocN(sizeof(float) * numVerts, "WeightVGProximity Modifier, tw");
-	tdw = MEM_mallocN(sizeof(MDeformWeight*) * numVerts, "WeightVGProximity Modifier, tdw");
+	tdw = MEM_mallocN(sizeof(MDeformWeight *) * numVerts, "WeightVGProximity Modifier, tdw");
 	for (i = 0; i < numVerts; i++) {
-		MDeformWeight *_dw = defvert_find_index(&dvert[i], defgrp_idx);
+		MDeformWeight *_dw = defvert_find_index(&dvert[i], defgrp_index);
 		if (_dw) {
 			tidx[numIdx] = i;
 			tw[numIdx] = _dw->weight;
@@ -413,8 +413,8 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob, DerivedMesh *der
 		memcpy(indices, tidx, sizeof(int) * numIdx);
 		org_w = MEM_mallocN(sizeof(float) * numIdx, "WeightVGProximity Modifier, org_w");
 		memcpy(org_w, tw, sizeof(float) * numIdx);
-		dw = MEM_mallocN(sizeof(MDeformWeight*) * numIdx, "WeightVGProximity Modifier, dw");
-		memcpy(dw, tdw, sizeof(MDeformWeight*) * numIdx);
+		dw = MEM_mallocN(sizeof(MDeformWeight *) * numIdx, "WeightVGProximity Modifier, dw");
+		memcpy(dw, tdw, sizeof(MDeformWeight *) * numIdx);
 		MEM_freeN(tw);
 		MEM_freeN(tdw);
 	}
@@ -458,9 +458,9 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob, DerivedMesh *der
 				if (ELEM3(obr->type, OB_CURVE, OB_SURF, OB_FONT))
 					target_dm = CDDM_from_curve(obr);
 				else if (obr->type == OB_MESH) {
-					Mesh *me = (Mesh*)obr->data;
+					Mesh *me = (Mesh *)obr->data;
 					if (me->edit_btmesh)
-						target_dm = CDDM_from_BMEditMesh(me->edit_btmesh, me, FALSE, FALSE);
+						target_dm = CDDM_from_editbmesh(me->edit_btmesh, FALSE, FALSE);
 					else
 						target_dm = CDDM_from_mesh(me, obr);
 				}
@@ -470,19 +470,19 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob, DerivedMesh *der
 			/* We must check that we do have a valid target_dm! */
 			if (target_dm) {
 				SpaceTransform loc2trgt;
-				float *dists_v = use_trgt_verts ? MEM_mallocN(sizeof(float) * numIdx, "WeightVGProximity Modifier, dists_v") : NULL;
-				float *dists_e = use_trgt_edges ? MEM_mallocN(sizeof(float) * numIdx, "WeightVGProximity Modifier, dists_e") : NULL;
-				float *dists_f = use_trgt_faces ? MEM_mallocN(sizeof(float) * numIdx, "WeightVGProximity Modifier, dists_f") : NULL;
+				float *dists_v = use_trgt_verts ? MEM_mallocN(sizeof(float) * numIdx, "dists_v") : NULL;
+				float *dists_e = use_trgt_edges ? MEM_mallocN(sizeof(float) * numIdx, "dists_e") : NULL;
+				float *dists_f = use_trgt_faces ? MEM_mallocN(sizeof(float) * numIdx, "dists_f") : NULL;
 
-				space_transform_setup(&loc2trgt, ob, obr);
+				SPACE_TRANSFORM_SETUP(&loc2trgt, ob, obr);
 				get_vert2geom_distance(numIdx, v_cos, dists_v, dists_e, dists_f,
 				                       target_dm, &loc2trgt);
 				for (i = 0; i < numIdx; i++) {
 					new_w[i] = dists_v ? dists_v[i] : FLT_MAX;
 					if (dists_e)
-						new_w[i] = minf(dists_e[i], new_w[i]);
+						new_w[i] = min_ff(dists_e[i], new_w[i]);
 					if (dists_f)
-						new_w[i] = minf(dists_f[i], new_w[i]);
+						new_w[i] = min_ff(dists_f[i], new_w[i]);
 				}
 				if (free_target_dm) target_dm->release(target_dm);
 				if (dists_v) MEM_freeN(dists_v);
@@ -509,7 +509,7 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob, DerivedMesh *der
 	                 wmd->mask_tex_map_obj, wmd->mask_tex_uvlayer_name);
 
 	/* Update vgroup. Note we never add nor remove vertices from vgroup here. */
-	weightvg_update_vg(dvert, defgrp_idx, dw, numIdx, indices, org_w, FALSE, 0.0f, FALSE, 0.0f);
+	weightvg_update_vg(dvert, defgrp_index, dw, numIdx, indices, org_w, FALSE, 0.0f, FALSE, 0.0f);
 
 	/* If weight preview enabled... */
 #if 0 /* XXX Currently done in mod stack :/ */
@@ -537,7 +537,7 @@ static DerivedMesh *applyModifierEM(ModifierData *md, Object *ob,
                                     struct BMEditMesh *UNUSED(editData),
                                     DerivedMesh *derivedData)
 {
-	return applyModifier(md, ob, derivedData, 0, 1);
+	return applyModifier(md, ob, derivedData, MOD_APPLY_USECACHE);
 }
 
 
@@ -546,10 +546,10 @@ ModifierTypeInfo modifierType_WeightVGProximity = {
 	/* structName */        "WeightVGProximityModifierData",
 	/* structSize */        sizeof(WeightVGProximityModifierData),
 	/* type */              eModifierTypeType_NonGeometrical,
-	/* flags */             eModifierTypeFlag_AcceptsMesh
-	                       |eModifierTypeFlag_SupportsMapping
-	                       |eModifierTypeFlag_SupportsEditmode
-	                       |eModifierTypeFlag_UsesPreview,
+	/* flags */             eModifierTypeFlag_AcceptsMesh |
+	                        eModifierTypeFlag_SupportsMapping |
+	                        eModifierTypeFlag_SupportsEditmode |
+	                        eModifierTypeFlag_UsesPreview,
 
 	/* copyData */          copyData,
 	/* deformVerts */       NULL,
