@@ -34,47 +34,17 @@
 #include "node_composite_util.h"
 
 /* **************** Gamma Tools  ******************** */
-  
-static bNodeSocketTemplate cmp_node_gamma_in[]= {
-	{	SOCK_RGBA, 1, "Image",			1.0f, 1.0f, 1.0f, 1.0f},
-	{	SOCK_FLOAT, 1, "Gamma",			1.0f, 0.0f, 0.0f, 0.0f, 0.001f, 10.0f, PROP_UNSIGNED},
+
+static bNodeSocketTemplate cmp_node_gamma_in[] = {
+	{	SOCK_RGBA, 1, N_("Image"),			1.0f, 1.0f, 1.0f, 1.0f},
+	{	SOCK_FLOAT, 1, N_("Gamma"),			1.0f, 0.0f, 0.0f, 0.0f, 0.001f, 10.0f, PROP_UNSIGNED},
 	{	-1, 0, ""	}
 };
-static bNodeSocketTemplate cmp_node_gamma_out[]= {
-	{	SOCK_RGBA, 0, "Image"},
+static bNodeSocketTemplate cmp_node_gamma_out[] = {
+	{	SOCK_RGBA, 0, N_("Image")},
 	{	-1, 0, ""	}
 };
 
-static void do_gamma(bNode *UNUSED(node), float *out, float *in, float *fac)
-{
-	int i=0;
-	for (i=0; i<3; i++) {
-		/* check for negative to avoid nan's */
-		out[i] = (in[i] > 0.0f)? powf(in[i],fac[0]): in[i];
-	}
-	out[3] = in[3];
-}
-static void node_composit_exec_gamma(void *UNUSED(data), bNode *node, bNodeStack **in, bNodeStack **out)
-{
-	/* stack order in: Fac, Image */
-	/* stack order out: Image */
-	if (out[0]->hasoutput==0) return;
-	
-	/* input no image? then only color operation */
-	if (in[0]->data==NULL) {
-		do_gamma(node, out[0]->vec, in[0]->vec, in[1]->vec);
-	}
-	else {
-		/* make output size of input image */
-		CompBuf *cbuf= in[0]->data;
-		CompBuf *stackbuf= alloc_compbuf(cbuf->x, cbuf->y, CB_RGBA, 1); // allocs
-		
-		composit2_pixel_processor(node, stackbuf, cbuf, in[0]->vec, in[1]->data, in[1]->vec, do_gamma, CB_RGBA, CB_VAL);
-
-		out[0]->data= stackbuf;
-	}
-}
-
 void register_node_type_cmp_gamma(bNodeTreeType *ttype)
 {
 	static bNodeType ntype;
@@ -82,7 +52,6 @@ void register_node_type_cmp_gamma(bNodeTreeType *ttype)
 	node_type_base(ttype, &ntype, CMP_NODE_GAMMA, "Gamma", NODE_CLASS_OP_COLOR, NODE_OPTIONS);
 	node_type_socket_templates(&ntype, cmp_node_gamma_in, cmp_node_gamma_out);
 	node_type_size(&ntype, 140, 100, 320);
-	node_type_exec(&ntype, node_composit_exec_gamma);
 	
 	nodeRegisterType(ttype, &ntype);
 }
