@@ -32,55 +32,17 @@
 #include "node_composite_util.h"
 
 /* **************** MIX RGB ******************** */
-static bNodeSocketTemplate cmp_node_mix_rgb_in[]= {
-	{	SOCK_FLOAT, 1, "Fac",			1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 5.0f, PROP_FACTOR},
-	{	SOCK_RGBA, 1, "Image",			1.0f, 1.0f, 1.0f, 1.0f},
-	{	SOCK_RGBA, 1, "Image",			1.0f, 1.0f, 1.0f, 1.0f},
+static bNodeSocketTemplate cmp_node_mix_rgb_in[] = {
+	{	SOCK_FLOAT, 1, N_("Fac"),			1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 5.0f, PROP_FACTOR},
+	{	SOCK_RGBA, 1, N_("Image"),			1.0f, 1.0f, 1.0f, 1.0f},
+	{	SOCK_RGBA, 1, N_("Image"),			1.0f, 1.0f, 1.0f, 1.0f},
 	{	-1, 0, ""	}
 };
-static bNodeSocketTemplate cmp_node_mix_rgb_out[]= {
-	{	SOCK_RGBA, 0, "Image"},
+static bNodeSocketTemplate cmp_node_mix_rgb_out[] = {
+	{	SOCK_RGBA, 0, N_("Image")},
 	{	-1, 0, ""	}
 };
 
-static void do_mix_rgb(bNode *node, float *out, float *in1, float *in2, float *fac)
-{
-	float col[3];
-	
-	copy_v3_v3(col, in1);
-	if (node->custom2)
-		ramp_blend(node->custom1, col, in2[3]*fac[0], in2);
-	else
-		ramp_blend(node->custom1, col, fac[0], in2);
-	copy_v3_v3(out, col);
-	out[3]= in1[3];
-}
-
-static void node_composit_exec_mix_rgb(void *data, bNode *node, bNodeStack **in, bNodeStack **out)
-{
-	/* stack order in: fac, Image, Image */
-	/* stack order out: Image */
-	float *fac= in[0]->vec;
-	
-	if (out[0]->hasoutput==0) return;
-	
-	/* input no image? then only color operation */
-	if (in[1]->data==NULL && in[2]->data==NULL) {
-		do_mix_rgb(node, out[0]->vec, in[1]->vec, in[2]->vec, fac);
-	}
-	else {
-		/* make output size of first available input image */
-		CompBuf *cbuf= in[1]->data?in[1]->data:in[2]->data;
-		CompBuf *stackbuf= alloc_compbuf(cbuf->x, cbuf->y, CB_RGBA, 1); /* allocs */
-		
-		composit3_pixel_processor(node, stackbuf, in[1]->data, in[1]->vec, in[2]->data, in[2]->vec, in[0]->data, fac, do_mix_rgb, CB_RGBA, CB_RGBA, CB_VAL);
-		
-		out[0]->data= stackbuf;
-		
-		generate_preview(data, node, out[0]->data);
-	}
-}
-
 /* custom1 = mix type */
 void register_node_type_cmp_mix_rgb(bNodeTreeType *ttype)
 {
@@ -90,7 +52,6 @@ void register_node_type_cmp_mix_rgb(bNodeTreeType *ttype)
 	node_type_socket_templates(&ntype, cmp_node_mix_rgb_in, cmp_node_mix_rgb_out);
 	node_type_size(&ntype, 110, 60, 120);
 	node_type_label(&ntype, node_blend_label);
-	node_type_exec(&ntype, node_composit_exec_mix_rgb);
 
 	nodeRegisterType(ttype, &ntype);
 }
