@@ -37,21 +37,9 @@
 
 struct bNodeTree;
 
-struct bNodeSocket *node_group_add_extern_socket(struct bNodeTree *ntree, ListBase *lb, int in_out, struct bNodeSocket *gsock);
-
 void node_group_init(struct bNodeTree *ntree, struct bNode *node, struct bNodeTemplate *ntemp);
-void node_forloop_init(struct bNodeTree *ntree, struct bNode *node, struct bNodeTemplate *ntemp);
-void node_whileloop_init(struct bNodeTree *ntree, struct bNode *node, struct bNodeTemplate *ntemp);
-
-void node_forloop_init_tree(struct bNodeTree *ntree);
-void node_whileloop_init_tree(struct bNodeTree *ntree);
-
 const char *node_group_label(struct bNode *node);
-
 struct bNodeTemplate node_group_template(struct bNode *node);
-struct bNodeTemplate node_forloop_template(struct bNode *node);
-struct bNodeTemplate node_whileloop_template(struct bNode *node);
-
 int node_group_valid(struct bNodeTree *ntree, struct bNodeTemplate *ntemp);
 void node_group_verify(struct bNodeTree *ntree, struct bNode *node, struct ID *id);
 
@@ -59,6 +47,6 @@ struct bNodeTree *node_group_edit_get(struct bNode *node);
 struct bNodeTree *node_group_edit_set(struct bNode *node, int edit);
 void node_group_edit_clear(bNode *node);
 
-void node_loop_update_tree(struct bNodeTree *ngroup);
+void ntree_update_reroute_nodes(struct bNodeTree *ntree);
 
 #endif
