@@ -46,7 +46,9 @@
 /* for a given socket, find the actual stack entry */
 bNodeStack *node_get_socket_stack(bNodeStack *stack, bNodeSocket *sock)
 {
-	return stack + sock->stack_index;
+	if (stack && sock)
+		return stack + sock->stack_index;
+	return NULL;
 }
 
 void node_get_stack(bNode *node, bNodeStack *stack, bNodeStack **in, bNodeStack **out)
@@ -67,7 +69,7 @@ void node_get_stack(bNode *node, bNodeStack *stack, bNodeStack **in, bNodeStack
 	}
 }
 
-void node_init_input_index(bNodeSocket *sock, int *index)
+static void node_init_input_index(bNodeSocket *sock, int *index)
 {
 	if (sock->link && sock->link->fromsock) {
 		sock->stack_index = sock->link->fromsock->stack_index;
@@ -77,16 +79,31 @@ void node_init_input_index(bNodeSocket *sock, int *index)
 	}
 }
 
-void node_init_output_index(bNodeSocket *sock, int *index)
+static void node_init_output_index(bNodeSocket *sock, int *index, ListBase *internal_links)
 {
-	sock->stack_index = (*index)++;
+	if (internal_links) {
+		bNodeLink *link;
+		/* copy the stack index from internally connected input to skip the node */
+		for (link = internal_links->first; link; link = link->next) {
+			if (link->tosock == sock) {
+				sock->stack_index = link->fromsock->stack_index;
+				break;
+			}
+		}
+		/* if not internally connected, assign a new stack index anyway to avoid bad stack access */
+		if (!link)
+			sock->stack_index = (*index)++;
+	}
+	else {
+		sock->stack_index = (*index)++;
+	}
 }
 
 /* basic preparation of socket stacks */
 static struct bNodeStack *setup_stack(bNodeStack *stack, bNodeSocket *sock)
 {
 	bNodeStack *ns = node_get_socket_stack(stack, sock);
-	float null_value[4]= {0.0f, 0.0f, 0.0f, 0.0f};
+	float null_value[4] = {0.0f, 0.0f, 0.0f, 0.0f};
 	
 	/* don't mess with remote socket stacks, these are initialized by other nodes! */
 	if (sock->link)
@@ -97,13 +114,13 @@ static struct bNodeStack *setup_stack(bNodeStack *stack, bNodeSocket *sock)
 	if (sock->default_value) {
 		switch (sock->type) {
 		case SOCK_FLOAT:
-			ns->vec[0] = ((bNodeSocketValueFloat*)sock->default_value)->value;
+			ns->vec[0] = ((bNodeSocketValueFloat *)sock->default_value)->value;
 			break;
 		case SOCK_VECTOR:
-			copy_v3_v3(ns->vec, ((bNodeSocketValueVector*)sock->default_value)->value);
+			copy_v3_v3(ns->vec, ((bNodeSocketValueVector *)sock->default_value)->value);
 			break;
 		case SOCK_RGBA:
-			copy_v4_v4(ns->vec, ((bNodeSocketValueRGBA*)sock->default_value)->value);
+			copy_v4_v4(ns->vec, ((bNodeSocketValueRGBA *)sock->default_value)->value);
 			break;
 		}
 	}
@@ -131,7 +148,7 @@ bNodeTreeExec *ntree_exec_begin(bNodeTree *ntree)
 	bNodeExec *nodeexec;
 	bNodeSocket *sock, *gsock;
 	bNodeStack *ns;
-	int index= 0;
+	int index;
 	bNode **nodelist;
 	int totnodes, n;
 	
@@ -146,10 +163,11 @@ bNodeTreeExec *ntree_exec_begin(bNodeTree *ntree)
 	/* backpointer to node tree */
 	exec->nodetree = ntree;
 	
+	/* set stack indices */
+	index = 0;
 	/* group inputs essentially work as outputs */
 	for (gsock=ntree->inputs.first; gsock; gsock = gsock->next)
-		node_init_output_index(gsock, &index);
-	/* set stack indexes */
+		node_init_output_index(gsock, &index, NULL);
 	for (n=0; n < totnodes; ++n) {
 		node = nodelist[n];
 		
@@ -158,8 +176,15 @@ bNodeTreeExec *ntree_exec_begin(bNodeTree *ntree)
 		/* init node socket stack indexes */
 		for (sock=node->inputs.first; sock; sock=sock->next)
 			node_init_input_index(sock, &index);
-		for (sock=node->outputs.first; sock; sock=sock->next)
-			node_init_output_index(sock, &index);
+		
+		if (node->flag & NODE_MUTED || node->type == NODE_REROUTE) {
+			for (sock=node->outputs.first; sock; sock=sock->next)
+				node_init_output_index(sock, &index, &node->internal_links);
+		}
+		else {
+			for (sock=node->outputs.first; sock; sock=sock->next)
+				node_init_output_index(sock, &index, NULL);
+		}
 	}
 	/* group outputs essentially work as inputs */
 	for (gsock=ntree->outputs.first; gsock; gsock = gsock->next)
@@ -223,8 +248,9 @@ void ntree_exec_end(bNodeTreeExec *exec)
 		MEM_freeN(exec->stack);
 	
 	for (n=0, nodeexec= exec->nodeexec; n < exec->totnodes; ++n, ++nodeexec) {
-		if (nodeexec->node->typeinfo->freeexecfunc)
-			nodeexec->node->typeinfo->freeexecfunc(nodeexec->node, nodeexec->data);
+		if (nodeexec->node->typeinfo)
+			if (nodeexec->node->typeinfo->freeexecfunc)
+				nodeexec->node->typeinfo->freeexecfunc(nodeexec->node, nodeexec->data);
 	}
 	
 	if (exec->nodeexec)
@@ -233,7 +259,7 @@ void ntree_exec_end(bNodeTreeExec *exec)
 	MEM_freeN(exec);
 }
 
-/**** Compositor/Material/Texture trees ****/
+/**** Material/Texture trees ****/
 
 bNodeThreadStack *ntreeGetThreadStack(bNodeTreeExec *exec, int thread)
 {
@@ -242,7 +268,7 @@ bNodeThreadStack *ntreeGetThreadStack(bNodeTreeExec *exec, int thread)
 	
 	for (nts=lb->first; nts; nts=nts->next) {
 		if (!nts->used) {
-			nts->used= 1;
+			nts->used = TRUE;
 			break;
 		}
 	}
@@ -250,7 +276,7 @@ bNodeThreadStack *ntreeGetThreadStack(bNodeTreeExec *exec, int thread)
 	if (!nts) {
 		nts= MEM_callocN(sizeof(bNodeThreadStack), "bNodeThreadStack");
 		nts->stack= MEM_dupallocN(exec->stack);
-		nts->used= 1;
+		nts->used = TRUE;
 		BLI_addtail(lb, nts);
 	}
 
@@ -262,33 +288,7 @@ void ntreeReleaseThreadStack(bNodeThreadStack *nts)
 	nts->used = 0;
 }
 
-void ntreeExecNodes(bNodeTreeExec *exec, void *callerdata, int thread)
-{
-	bNodeStack *nsin[MAX_SOCKET];	/* arbitrary... watch this */
-	bNodeStack *nsout[MAX_SOCKET];	/* arbitrary... watch this */
-	bNodeExec *nodeexec;
-	bNode *node;
-	int n;
-	
-	/* nodes are presorted, so exec is in order of list */
-	
-	for (n=0, nodeexec= exec->nodeexec; n < exec->totnodes; ++n, ++nodeexec) {
-		node = nodeexec->node;
-		if (node->need_exec) {
-			node_get_stack(node, exec->stack, nsin, nsout);
-			/* Handle muted nodes...
-			 * If the mute func is not set, assume the node should never be muted,
-			 * and hence execute it!
-			 */
-			if (node->typeinfo->execfunc)
-				node->typeinfo->execfunc(callerdata, node, nsin, nsout);
-			else if (node->typeinfo->newexecfunc)
-				node->typeinfo->newexecfunc(callerdata, thread, node, nodeexec->data, nsin, nsout);
-		}
-	}
-}
-
-void ntreeExecThreadNodes(bNodeTreeExec *exec, bNodeThreadStack *nts, void *callerdata, int thread)
+bool ntreeExecThreadNodes(bNodeTreeExec *exec, bNodeThreadStack *nts, void *callerdata, int thread)
 {
 	bNodeStack *nsin[MAX_SOCKET];	/* arbitrary... watch this */
 	bNodeStack *nsout[MAX_SOCKET];	/* arbitrary... watch this */
@@ -306,10 +306,15 @@ void ntreeExecThreadNodes(bNodeTreeExec *exec, bNodeThreadStack *nts, void *call
 			 * If the mute func is not set, assume the node should never be muted,
 			 * and hence execute it!
 			 */
+//			if (node->typeinfo->compatibility == NODE_NEW_SHADING)
+//				return false;
 			if (node->typeinfo->execfunc)
 				node->typeinfo->execfunc(callerdata, node, nsin, nsout);
 			else if (node->typeinfo->newexecfunc)
 				node->typeinfo->newexecfunc(callerdata, thread, node, nodeexec->data, nsin, nsout);
 		}
 	}
+	
+	/* signal to that all went OK, for render */
+	return true;
 }
