@@ -29,6 +29,7 @@
  *  \ingroup nodes
  */
 
+#include <limits.h>
 
 #include "DNA_action_types.h"
 #include "DNA_node_types.h"
@@ -57,17 +58,25 @@ void node_free_curves(bNode *node)
 
 void node_free_standard_storage(bNode *node)
 {
-	MEM_freeN(node->storage);
+	if (node->storage) {
+		MEM_freeN(node->storage);
+	}
 }
 
 void node_copy_curves(bNode *orig_node, bNode *new_node)
 {
-	new_node->storage= curvemapping_copy(orig_node->storage);
+	new_node->storage = curvemapping_copy(orig_node->storage);
 }
 
 void node_copy_standard_storage(bNode *orig_node, bNode *new_node)
 {
-	new_node->storage= MEM_dupallocN(orig_node->storage);
+	new_node->storage = MEM_dupallocN(orig_node->storage);
+}
+
+void *node_initexec_curves(bNode *node)
+{
+	curvemapping_initialize(node->storage);
+	return NULL;  /* unused return */
 }
 
 /**** Labels ****/
@@ -100,56 +109,69 @@ const char *node_filter_label(bNode *node)
 	return IFACE_(name);
 }
 
-ListBase node_internal_connect_default(bNodeTree *ntree, bNode *node)
+void node_update_internal_links_default(bNodeTree *ntree, bNode *node)
 {
-	ListBase ret;
 	bNodeSocket *fromsock_first=NULL, *tosock_first=NULL;	/* used for fallback link if no other reconnections are found */
 	int datatype;
 	int num_links_in = 0, num_links_out = 0, num_reconnect = 0;
 
-	ret.first = ret.last = NULL;
-
 	/* Security check! */
 	if (!ntree)
-		return ret;
+		return;
 
 	for (datatype=0; datatype < NUM_SOCKET_TYPES; ++datatype) {
-		bNodeSocket *fromsock=NULL, *tosock=NULL;
+		bNodeSocket *fromsock, *tosock;
+		int fromindex, toindex;
 		bNodeLink *link;
 		
 		/* Connect the first input of each type with outputs of the same type. */
 		
+		fromindex = INT_MAX;
+		fromsock = NULL;
 		for (link=ntree->links.first; link; link=link->next) {
 			if (link->tonode == node && link->tosock->type == datatype) {
-				fromsock = link->tosock;
-				++num_links_in;
-				if (!fromsock_first)
-					fromsock_first = fromsock;
-				break;
+				int index = BLI_findindex(&node->inputs, link->tosock);
+				if (index < fromindex) {
+					fromindex = index;
+					fromsock = link->tosock;
+				}
 			}
 		}
+		if (fromsock) {
+			++num_links_in;
+			if (!fromsock_first)
+				fromsock_first = fromsock;
+		}
 		
+		toindex = INT_MAX;
+		tosock = NULL;
 		for (link=ntree->links.first; link; link=link->next) {
 			if (link->fromnode == node && link->fromsock->type == datatype) {
-				tosock = link->fromsock;
-				++num_links_out;
-				if (!tosock_first)
-					tosock_first = tosock;
-				
-				if (fromsock) {
-					bNodeLink *ilink = MEM_callocN(sizeof(bNodeLink), "internal node link");
-					ilink->fromnode = node;
-					ilink->fromsock = fromsock;
-					ilink->tonode = node;
-					ilink->tosock = tosock;
-					/* internal link is always valid */
-					ilink->flag |= NODE_LINK_VALID;
-					BLI_addtail(&ret, ilink);
-					
-					++num_reconnect;
+				int index = BLI_findindex(&node->outputs, link->fromsock);
+				if (index < toindex) {
+					toindex = index;
+					tosock = link->fromsock;
 				}
 			}
 		}
+		if (tosock) {
+			++num_links_out;
+			if (!tosock_first)
+				tosock_first = tosock;
+			
+			if (fromsock) {
+				bNodeLink *ilink = MEM_callocN(sizeof(bNodeLink), "internal node link");
+				ilink->fromnode = node;
+				ilink->fromsock = fromsock;
+				ilink->tonode = node;
+				ilink->tosock = tosock;
+				/* internal link is always valid */
+				ilink->flag |= NODE_LINK_VALID;
+				BLI_addtail(&node->internal_links, ilink);
+				
+				++num_reconnect;
+			}
+		}
 	}
 	
 	/* if there is one input and one output link, but no reconnections by type,
@@ -163,8 +185,6 @@ ListBase node_internal_connect_default(bNodeTree *ntree, bNode *node)
 		ilink->tosock = tosock_first;
 		/* internal link is always valid */
 		ilink->flag |= NODE_LINK_VALID;
-		BLI_addtail(&ret, ilink);
+		BLI_addtail(&node->internal_links, ilink);
 	}
-	
-	return ret;
 }
