@@ -49,12 +49,12 @@
 #include "BKE_main.h"
 #include "BKE_node.h"
 #include "BKE_scene.h"
-#include "BKE_utildefines.h"
 
 #include "GPU_material.h"
 
 #include "RE_shader_ext.h"
 
+#include "node_common.h"
 #include "node_exec.h"
 #include "node_util.h"
 #include "node_shader_util.h"
@@ -65,45 +65,46 @@ static void foreach_nodetree(Main *main, void *calldata, bNodeTreeCallback func)
 	Lamp *la;
 	World *wo;
 
-	for (ma= main->mat.first; ma; ma= ma->id.next)
+	for (ma = main->mat.first; ma; ma = ma->id.next)
 		if (ma->nodetree)
 			func(calldata, &ma->id, ma->nodetree);
 
-	for (la= main->lamp.first; la; la= la->id.next)
+	for (la = main->lamp.first; la; la = la->id.next)
 		if (la->nodetree)
 			func(calldata, &la->id, la->nodetree);
 
-	for (wo= main->world.first; wo; wo= wo->id.next)
+	for (wo = main->world.first; wo; wo = wo->id.next)
 		if (wo->nodetree)
 			func(calldata, &wo->id, wo->nodetree);
 }
 
 static void foreach_nodeclass(Scene *scene, void *calldata, bNodeClassCallback func)
 {
-	func(calldata, NODE_CLASS_INPUT, IFACE_("Input"));
-	func(calldata, NODE_CLASS_OUTPUT, IFACE_("Output"));
+	func(calldata, NODE_CLASS_INPUT, N_("Input"));
+	func(calldata, NODE_CLASS_OUTPUT, N_("Output"));
 
-	if (scene_use_new_shading_nodes(scene)) {
-		func(calldata, NODE_CLASS_SHADER, IFACE_("Shader"));
-		func(calldata, NODE_CLASS_TEXTURE, IFACE_("Texture"));
+	if (BKE_scene_use_new_shading_nodes(scene)) {
+		func(calldata, NODE_CLASS_SHADER, N_("Shader"));
+		func(calldata, NODE_CLASS_TEXTURE, N_("Texture"));
 	}
-
-	func(calldata, NODE_CLASS_OP_COLOR, IFACE_("Color"));
-	func(calldata, NODE_CLASS_OP_VECTOR, IFACE_("Vector"));
-	func(calldata, NODE_CLASS_CONVERTOR, IFACE_("Convertor"));
-	func(calldata, NODE_CLASS_GROUP, IFACE_("Group"));
-	func(calldata, NODE_CLASS_LAYOUT, IFACE_("Layout"));
+	
+	func(calldata, NODE_CLASS_OP_COLOR, N_("Color"));
+	func(calldata, NODE_CLASS_OP_VECTOR, N_("Vector"));
+	func(calldata, NODE_CLASS_CONVERTOR, N_("Convertor"));
+	func(calldata, NODE_CLASS_SCRIPT, N_("Script"));
+	func(calldata, NODE_CLASS_GROUP, N_("Group"));
+	func(calldata, NODE_CLASS_LAYOUT, N_("Layout"));
 }
 
 static void localize(bNodeTree *localtree, bNodeTree *UNUSED(ntree))
 {
 	bNode *node, *node_next;
 	
-	/* replace muted nodes by internal links */
-	for (node= localtree->nodes.first; node; node= node_next) {
+	/* replace muted nodes and reroute nodes by internal links */
+	for (node = localtree->nodes.first; node; node = node_next) {
 		node_next = node->next;
 		
-		if (node->flag & NODE_MUTED) {
+		if (node->flag & NODE_MUTED || node->type == NODE_REROUTE) {
 			nodeInternalRelink(localtree, node);
 			nodeFreeNode(localtree, node);
 		}
@@ -115,15 +116,15 @@ static void local_sync(bNodeTree *localtree, bNodeTree *ntree)
 	bNode *lnode;
 	
 	/* copy over contents of previews */
-	for (lnode= localtree->nodes.first; lnode; lnode= lnode->next) {
+	for (lnode = localtree->nodes.first; lnode; lnode = lnode->next) {
 		if (ntreeNodeExists(ntree, lnode->new_node)) {
-			bNode *node= lnode->new_node;
+			bNode *node = lnode->new_node;
 			
 			if (node->preview && node->preview->rect) {
 				if (lnode->preview && lnode->preview->rect) {
-					int xsize= node->preview->xsize;
-					int ysize= node->preview->ysize;
-					memcpy(node->preview->rect, lnode->preview->rect, 4*xsize + xsize*ysize*sizeof(char)*4);
+					int xsize = node->preview->xsize;
+					int ysize = node->preview->ysize;
+					memcpy(node->preview->rect, lnode->preview->rect, 4 * xsize + xsize * ysize * sizeof(char) * 4);
 				}
 			}
 		}
@@ -133,6 +134,8 @@ static void local_sync(bNodeTree *localtree, bNodeTree *ntree)
 static void update(bNodeTree *ntree)
 {
 	ntreeSetOutput(ntree);
+	
+	ntree_update_reroute_nodes(ntree);
 }
 
 bNodeTreeType ntreeType_Shader = {
@@ -151,7 +154,7 @@ bNodeTreeType ntreeType_Shader = {
 	/* update */			update,
 	/* update_node */		NULL,
 	/* validate_link */		NULL,
-	/* internal_connect */	node_internal_connect_default
+	/* update_internal_links */	node_update_internal_links_default
 };
 
 /* GPU material from shader nodes */
@@ -173,7 +176,7 @@ void (*node_shader_lamp_loop)(struct ShadeInput *, struct ShadeResult *);
 
 void set_node_shader_lamp_loop(void (*lamp_loop_func)(ShadeInput *, ShadeResult *))
 {
-	node_shader_lamp_loop= lamp_loop_func;
+	node_shader_lamp_loop = lamp_loop_func;
 }
 
 
@@ -200,10 +203,10 @@ bNodeTreeExec *ntreeShaderBeginExecTree(bNodeTree *ntree, int use_tree_data)
 	exec = ntree_exec_begin(ntree);
 	
 	/* allocate the thread stack listbase array */
-	exec->threadstack= MEM_callocN(BLENDER_MAX_THREADS*sizeof(ListBase), "thread stack array");
+	exec->threadstack = MEM_callocN(BLENDER_MAX_THREADS * sizeof(ListBase), "thread stack array");
 	
-	for (node= exec->nodetree->nodes.first; node; node= node->next)
-		node->need_exec= 1;
+	for (node = exec->nodetree->nodes.first; node; node = node->next)
+		node->need_exec = 1;
 	
 	if (use_tree_data) {
 		/* XXX this should not be necessary, but is still used for cmp/sha/tex nodes,
@@ -221,19 +224,19 @@ bNodeTreeExec *ntreeShaderBeginExecTree(bNodeTree *ntree, int use_tree_data)
 void ntreeShaderEndExecTree(bNodeTreeExec *exec, int use_tree_data)
 {
 	if (exec) {
-		bNodeTree *ntree= exec->nodetree;
+		bNodeTree *ntree = exec->nodetree;
 		bNodeThreadStack *nts;
 		int a;
 		
 		if (exec->threadstack) {
-			for (a=0; a<BLENDER_MAX_THREADS; a++) {
-				for (nts=exec->threadstack[a].first; nts; nts=nts->next)
+			for (a = 0; a < BLENDER_MAX_THREADS; a++) {
+				for (nts = exec->threadstack[a].first; nts; nts = nts->next)
 					if (nts->stack) MEM_freeN(nts->stack);
 				BLI_freelistN(&exec->threadstack[a]);
 			}
 			
 			MEM_freeN(exec->threadstack);
-			exec->threadstack= NULL;
+			exec->threadstack = NULL;
 		}
 		
 		ntree_exec_end(exec);
@@ -245,7 +248,8 @@ void ntreeShaderEndExecTree(bNodeTreeExec *exec, int use_tree_data)
 	}
 }
 
-void ntreeShaderExecTree(bNodeTree *ntree, ShadeInput *shi, ShadeResult *shr)
+/* only for Blender internal */
+bool ntreeShaderExecTree(bNodeTree *ntree, ShadeInput *shi, ShadeResult *shr)
 {
 	ShaderCallData scd;
 	/**
@@ -255,10 +259,11 @@ void ntreeShaderExecTree(bNodeTree *ntree, ShadeInput *shi, ShadeResult *shr)
 	Material *mat = shi->mat;
 	bNodeThreadStack *nts = NULL;
 	bNodeTreeExec *exec = ntree->execdata;
+	int compat;
 	
 	/* convert caller data to struct */
-	scd.shi= shi;
-	scd.shr= shr;
+	scd.shi = shi;
+	scd.shr = shr;
 	
 	/* each material node has own local shaderesult, with optional copying */
 	memset(shr, 0, sizeof(ShadeResult));
@@ -273,14 +278,18 @@ void ntreeShaderExecTree(bNodeTree *ntree, ShadeInput *shi, ShadeResult *shr)
 		exec = ntree->execdata;
 	}
 	
-	nts= ntreeGetThreadStack(exec, shi->thread);
-	ntreeExecThreadNodes(exec, nts, &scd, shi->thread);
+	nts = ntreeGetThreadStack(exec, shi->thread);
+	compat = ntreeExecThreadNodes(exec, nts, &scd, shi->thread);
 	ntreeReleaseThreadStack(nts);
 	
 	// \note: set material back to preserved material
 	shi->mat = mat;
+		
 	/* better not allow negative for now */
-	if (shr->combined[0]<0.0f) shr->combined[0]= 0.0f;
-	if (shr->combined[1]<0.0f) shr->combined[1]= 0.0f;
-	if (shr->combined[2]<0.0f) shr->combined[2]= 0.0f;
+	if (shr->combined[0] < 0.0f) shr->combined[0] = 0.0f;
+	if (shr->combined[1] < 0.0f) shr->combined[1] = 0.0f;
+	if (shr->combined[2] < 0.0f) shr->combined[2] = 0.0f;
+	
+	/* if compat is zero, it has been using non-compatible nodes */
+	return compat;
 }
