@@ -33,10 +33,10 @@
 #include "node_shader_util.h"
 
 /* **************** CAMERA INFO  ******************** */
-static bNodeSocketTemplate sh_node_camera_out[]= {
-	{	SOCK_VECTOR, 0, "View Vector"},
-	{	SOCK_FLOAT, 0, "View Z Depth"},
-	{	SOCK_FLOAT, 0, "View Distance"},
+static bNodeSocketTemplate sh_node_camera_out[] = {
+	{	SOCK_VECTOR, 0, N_("View Vector")},
+	{	SOCK_FLOAT, 0, N_("View Z Depth")},
+	{	SOCK_FLOAT, 0, N_("View Distance")},
 	{	-1, 0, ""	}
 };
 
@@ -47,8 +47,8 @@ static void node_shader_exec_camera(void *data, bNode *UNUSED(node), bNodeStack
 		ShadeInput *shi= ((ShaderCallData *)data)->shi;  /* Data we need for shading. */
 		
 		copy_v3_v3(out[0]->vec, shi->co);		/* get view vector */
-		out[1]->vec[0]= fabs(shi->co[2]);		/* get view z-depth */
-		out[2]->vec[0]= normalize_v3(out[0]->vec);	/* get view distance */
+		out[1]->vec[0] = fabs(shi->co[2]);		/* get view z-depth */
+		out[2]->vec[0] = normalize_v3(out[0]->vec);	/* get view distance */
 	}
 }
 
