@@ -37,16 +37,16 @@
 /* **************** GEOMETRY  ******************** */
 
 /* output socket type definition */
-static bNodeSocketTemplate sh_node_geom_out[]= {
-	{	SOCK_VECTOR, 0, "Global"},
-	{	SOCK_VECTOR, 0, "Local"},
-	{	SOCK_VECTOR, 0, "View"},
-	{	SOCK_VECTOR, 0, "Orco"},
-	{	SOCK_VECTOR, 0, "UV"},
-	{	SOCK_VECTOR, 0, "Normal"},
-	{	SOCK_RGBA,   0, "Vertex Color"},
-	{	SOCK_FLOAT,   0, "Vertex Alpha"},
-	{	SOCK_FLOAT,   0, "Front/Back"},
+static bNodeSocketTemplate sh_node_geom_out[] = {
+	{	SOCK_VECTOR, 0, N_("Global")},
+	{	SOCK_VECTOR, 0, N_("Local")},
+	{	SOCK_VECTOR, 0, N_("View")},
+	{	SOCK_VECTOR, 0, N_("Orco")},
+	{	SOCK_VECTOR, 0, N_("UV")},
+	{	SOCK_VECTOR, 0, N_("Normal")},
+	{	SOCK_RGBA,   0, N_("Vertex Color")},
+	{	SOCK_FLOAT,   0, N_("Vertex Alpha")},
+	{	SOCK_FLOAT,   0, N_("Front/Back")},
 	{	-1, 0, ""	}
 };
 
@@ -92,12 +92,12 @@ static void node_shader_exec_geom(void *data, bNode *node, bNodeStack **UNUSED(i
 			}
 
 			copy_v3_v3(out[GEOM_OUT_VCOL]->vec, scol->col);
-			out[GEOM_OUT_VCOL]->vec[3]= scol->col[3];
-			out[GEOM_OUT_VCOL_ALPHA]->vec[0]= scol->col[3];
+			out[GEOM_OUT_VCOL]->vec[3] = scol->col[3];
+			out[GEOM_OUT_VCOL_ALPHA]->vec[0] = scol->col[3];
 		}
 		else {
 			memcpy(out[GEOM_OUT_VCOL]->vec, defaultvcol, sizeof(defaultvcol));
-			out[GEOM_OUT_VCOL_ALPHA]->vec[0]= 1.0f;
+			out[GEOM_OUT_VCOL_ALPHA]->vec[0] = 1.0f;
 		}
 		
 		if (shi->osatex) {
@@ -116,11 +116,11 @@ static void node_shader_exec_geom(void *data, bNode *node, bNodeStack **UNUSED(i
 		}
 		
 		/* front/back, normal flipping was stored */
-		out[GEOM_OUT_FRONTBACK]->vec[0]= (shi->flippednor)? 0.0f: 1.0f;
+		out[GEOM_OUT_FRONTBACK]->vec[0] = (shi->flippednor)? 0.0f: 1.0f;
 	}
 }
 
-static void node_shader_init_geometry(bNodeTree *UNUSED(ntree), bNode* node, bNodeTemplate *UNUSED(ntemp))
+static void node_shader_init_geometry(bNodeTree *UNUSED(ntree), bNode *node, bNodeTemplate *UNUSED(ntemp))
 {
 	node->storage= MEM_callocN(sizeof(NodeGeometry), "NodeGeometry");
 }
