@@ -34,16 +34,16 @@
 
 
 /* **************** Hue Saturation ******************** */
-static bNodeSocketTemplate sh_node_hue_sat_in[]= {
-	{	SOCK_FLOAT, 1, "Hue",			0.5f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, PROP_NONE},
-	{	SOCK_FLOAT, 1, "Saturation",		1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 2.0f, PROP_FACTOR},
-	{	SOCK_FLOAT, 1, "Value",			1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 2.0f, PROP_FACTOR},
-	{	SOCK_FLOAT, 1, "Fac",			1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, PROP_FACTOR},
-	{	SOCK_RGBA, 1, "Color",			0.8f, 0.8f, 0.8f, 1.0f},
+static bNodeSocketTemplate sh_node_hue_sat_in[] = {
+	{	SOCK_FLOAT, 1, N_("Hue"),			0.5f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, PROP_NONE},
+	{	SOCK_FLOAT, 1, N_("Saturation"),		1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 2.0f, PROP_FACTOR},
+	{	SOCK_FLOAT, 1, N_("Value"),			1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 2.0f, PROP_FACTOR},
+	{	SOCK_FLOAT, 1, N_("Fac"),			1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, PROP_FACTOR},
+	{	SOCK_RGBA, 1, N_("Color"),			0.8f, 0.8f, 0.8f, 1.0f},
 	{	-1, 0, ""	}
 };
-static bNodeSocketTemplate sh_node_hue_sat_out[]= {
-	{	SOCK_RGBA, 0, "Color"},
+static bNodeSocketTemplate sh_node_hue_sat_out[] = {
+	{	SOCK_RGBA, 0, N_("Color")},
 	{	-1, 0, ""	}
 };
 
@@ -60,9 +60,9 @@ static void do_hue_sat_fac(bNode *UNUSED(node), float *out, float *hue, float *s
 		hsv[2]*= *val;
 		hsv_to_rgb(hsv[0], hsv[1], hsv[2], col, col+1, col+2);
 		
-		out[0]= mfac*in[0] + *fac*col[0];
-		out[1]= mfac*in[1] + *fac*col[1];
-		out[2]= mfac*in[2] + *fac*col[2];
+		out[0] = mfac*in[0] + *fac*col[0];
+		out[1] = mfac*in[1] + *fac*col[1];
+		out[2] = mfac*in[2] + *fac*col[2];
 	}
 	else {
 		copy_v4_v4(out, in);
