@@ -33,13 +33,13 @@
 #include "node_shader_util.h"
 
 /* **************** MAPPING  ******************** */
-static bNodeSocketTemplate sh_node_mapping_in[]= {
-	{	SOCK_VECTOR, 1, "Vector",	0.0f, 0.0f, 0.0f, 1.0f, -1.0f, 1.0f, PROP_NONE},
+static bNodeSocketTemplate sh_node_mapping_in[] = {
+	{	SOCK_VECTOR, 1, N_("Vector"),	0.0f, 0.0f, 0.0f, 1.0f, -1.0f, 1.0f, PROP_NONE},
 	{	-1, 0, ""	}
 };
 
-static bNodeSocketTemplate sh_node_mapping_out[]= {
-	{	SOCK_VECTOR, 0, "Vector"},
+static bNodeSocketTemplate sh_node_mapping_out[] = {
+	{	SOCK_VECTOR, 0, N_("Vector")},
 	{	-1, 0, ""	}
 };
 
@@ -55,19 +55,19 @@ static void node_shader_exec_mapping(void *UNUSED(data), bNode *node, bNodeStack
 	mul_m4_v3(texmap->mat, vec);
 	
 	if (texmap->flag & TEXMAP_CLIP_MIN) {
-		if (vec[0]<texmap->min[0]) vec[0]= texmap->min[0];
-		if (vec[1]<texmap->min[1]) vec[1]= texmap->min[1];
-		if (vec[2]<texmap->min[2]) vec[2]= texmap->min[2];
+		if (vec[0]<texmap->min[0]) vec[0] = texmap->min[0];
+		if (vec[1]<texmap->min[1]) vec[1] = texmap->min[1];
+		if (vec[2]<texmap->min[2]) vec[2] = texmap->min[2];
 	}
 	if (texmap->flag & TEXMAP_CLIP_MAX) {
-		if (vec[0]>texmap->max[0]) vec[0]= texmap->max[0];
-		if (vec[1]>texmap->max[1]) vec[1]= texmap->max[1];
-		if (vec[2]>texmap->max[2]) vec[2]= texmap->max[2];
+		if (vec[0]>texmap->max[0]) vec[0] = texmap->max[0];
+		if (vec[1]>texmap->max[1]) vec[1] = texmap->max[1];
+		if (vec[2]>texmap->max[2]) vec[2] = texmap->max[2];
 	}
 }
 
 
-static void node_shader_init_mapping(bNodeTree *UNUSED(ntree), bNode* node, bNodeTemplate *UNUSED(ntemp))
+static void node_shader_init_mapping(bNodeTree *UNUSED(ntree), bNode *node, bNodeTemplate *UNUSED(ntemp))
 {
 	node->storage= add_tex_mapping();
 }
@@ -77,7 +77,7 @@ static int gpu_shader_mapping(GPUMaterial *mat, bNode *node, GPUNodeStack *in, G
 	TexMapping *texmap= node->storage;
 	float domin= (texmap->flag & TEXMAP_CLIP_MIN) != 0;
 	float domax= (texmap->flag & TEXMAP_CLIP_MAX) != 0;
-	GPUNodeLink *tmat = GPU_uniform((float*)texmap->mat);
+	GPUNodeLink *tmat = GPU_uniform((float *)texmap->mat);
 	GPUNodeLink *tmin = GPU_uniform(texmap->min);
 	GPUNodeLink *tmax = GPU_uniform(texmap->max);
 	GPUNodeLink *tdomin = GPU_uniform(&domin);
