@@ -4,7 +4,7 @@
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. 
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -45,6 +45,7 @@
 #include "BKE_main.h"
 #include "BKE_node.h"
 
+#include "node_common.h"
 #include "node_exec.h"
 #include "node_util.h"
 #include "NOD_texture.h"
@@ -57,7 +58,7 @@
 static void foreach_nodetree(Main *main, void *calldata, bNodeTreeCallback func)
 {
 	Tex *tx;
-	for (tx= main->tex.first; tx; tx= tx->id.next) {
+	for (tx = main->tex.first; tx; tx = tx->id.next) {
 		if (tx->nodetree) {
 			func(calldata, &tx->id, tx->nodetree);
 		}
@@ -66,26 +67,26 @@ static void foreach_nodetree(Main *main, void *calldata, bNodeTreeCallback func)
 
 static void foreach_nodeclass(Scene *UNUSED(scene), void *calldata, bNodeClassCallback func)
 {
-	func(calldata, NODE_CLASS_INPUT, IFACE_("Input"));
-	func(calldata, NODE_CLASS_OUTPUT, IFACE_("Output"));
-	func(calldata, NODE_CLASS_OP_COLOR, IFACE_("Color"));
-	func(calldata, NODE_CLASS_PATTERN, IFACE_("Patterns"));
-	func(calldata, NODE_CLASS_TEXTURE, IFACE_("Textures"));
-	func(calldata, NODE_CLASS_CONVERTOR, IFACE_("Convertor"));
-	func(calldata, NODE_CLASS_DISTORT, IFACE_("Distort"));
-	func(calldata, NODE_CLASS_GROUP, IFACE_("Group"));
-	func(calldata, NODE_CLASS_LAYOUT, IFACE_("Layout"));
+	func(calldata, NODE_CLASS_INPUT, N_("Input"));
+	func(calldata, NODE_CLASS_OUTPUT, N_("Output"));
+	func(calldata, NODE_CLASS_OP_COLOR, N_("Color"));
+	func(calldata, NODE_CLASS_PATTERN, N_("Patterns"));
+	func(calldata, NODE_CLASS_TEXTURE, N_("Textures"));
+	func(calldata, NODE_CLASS_CONVERTOR, N_("Convertor"));
+	func(calldata, NODE_CLASS_DISTORT, N_("Distort"));
+	func(calldata, NODE_CLASS_GROUP, N_("Group"));
+	func(calldata, NODE_CLASS_LAYOUT, N_("Layout"));
 }
 
 static void localize(bNodeTree *localtree, bNodeTree *UNUSED(ntree))
 {
 	bNode *node, *node_next;
 	
-	/* replace muted nodes by internal links */
-	for (node= localtree->nodes.first; node; node= node_next) {
+	/* replace muted nodes and reroute nodes by internal links */
+	for (node = localtree->nodes.first; node; node = node_next) {
 		node_next = node->next;
 		
-		if (node->flag & NODE_MUTED) {
+		if (node->flag & NODE_MUTED || node->type == NODE_REROUTE) {
 			nodeInternalRelink(localtree, node);
 			nodeFreeNode(localtree, node);
 		}
@@ -97,21 +98,26 @@ static void local_sync(bNodeTree *localtree, bNodeTree *ntree)
 	bNode *lnode;
 	
 	/* copy over contents of previews */
-	for (lnode= localtree->nodes.first; lnode; lnode= lnode->next) {
+	for (lnode = localtree->nodes.first; lnode; lnode = lnode->next) {
 		if (ntreeNodeExists(ntree, lnode->new_node)) {
-			bNode *node= lnode->new_node;
+			bNode *node = lnode->new_node;
 			
 			if (node->preview && node->preview->rect) {
 				if (lnode->preview && lnode->preview->rect) {
-					int xsize= node->preview->xsize;
-					int ysize= node->preview->ysize;
-					memcpy(node->preview->rect, lnode->preview->rect, 4*xsize + xsize*ysize*sizeof(char)*4);
+					int xsize = node->preview->xsize;
+					int ysize = node->preview->ysize;
+					memcpy(node->preview->rect, lnode->preview->rect, 4 * xsize + xsize * ysize * sizeof(char) * 4);
 				}
 			}
 		}
 	}
 }
 
+static void update(bNodeTree *ntree)
+{
+	ntree_update_reroute_nodes(ntree);
+}
+
 bNodeTreeType ntreeType_Texture = {
 	/* type */				NTREE_TEXTURE,
 	/* id_name */			"NTTexture Nodetree",
@@ -125,25 +131,25 @@ bNodeTreeType ntreeType_Texture = {
 	/* localize */			localize,
 	/* local_sync */		local_sync,
 	/* local_merge */		NULL,
-	/* update */			NULL,
+	/* update */			update,
 	/* update_node */		NULL,
 	/* validate_link */		NULL,
-	/* internal_connect */	node_internal_connect_default
+	/* update_internal_links */	node_update_internal_links_default
 };
 
 int ntreeTexTagAnimated(bNodeTree *ntree)
 {
 	bNode *node;
 	
-	if (ntree==NULL) return 0;
+	if (ntree == NULL) return 0;
 	
-	for (node= ntree->nodes.first; node; node= node->next) {
-		if (node->type==TEX_NODE_CURVE_TIME) {
+	for (node = ntree->nodes.first; node; node = node->next) {
+		if (node->type == TEX_NODE_CURVE_TIME) {
 			nodeUpdate(ntree, node);
 			return 1;
 		}
-		else if (node->type==NODE_GROUP) {
-			if ( ntreeTexTagAnimated((bNodeTree *)node->id) ) {
+		else if (node->type == NODE_GROUP) {
+			if (ntreeTexTagAnimated((bNodeTree *)node->id) ) {
 				return 1;
 			}
 		}
@@ -172,10 +178,10 @@ bNodeTreeExec *ntreeTexBeginExecTree(bNodeTree *ntree, int use_tree_data)
 	exec = ntree_exec_begin(ntree);
 	
 	/* allocate the thread stack listbase array */
-	exec->threadstack= MEM_callocN(BLENDER_MAX_THREADS*sizeof(ListBase), "thread stack array");
+	exec->threadstack = MEM_callocN(BLENDER_MAX_THREADS * sizeof(ListBase), "thread stack array");
 	
-	for (node= exec->nodetree->nodes.first; node; node= node->next)
-		node->need_exec= 1;
+	for (node = exec->nodetree->nodes.first; node; node = node->next)
+		node->need_exec = 1;
 	
 	if (use_tree_data) {
 		/* XXX this should not be necessary, but is still used for cmp/sha/tex nodes,
@@ -194,9 +200,9 @@ static void tex_free_delegates(bNodeTreeExec *exec)
 	bNodeStack *ns;
 	int th, a;
 	
-	for (th=0; th<BLENDER_MAX_THREADS; th++)
-		for (nts=exec->threadstack[th].first; nts; nts=nts->next)
-			for (ns= nts->stack, a=0; a<exec->stacksize; a++, ns++)
+	for (th = 0; th < BLENDER_MAX_THREADS; th++)
+		for (nts = exec->threadstack[th].first; nts; nts = nts->next)
+			for (ns = nts->stack, a = 0; a < exec->stacksize; a++, ns++)
 				if (ns->data && !ns->is_copy)
 					MEM_freeN(ns->data);
 }
@@ -207,21 +213,21 @@ static void tex_free_delegates(bNodeTreeExec *exec)
 void ntreeTexEndExecTree(bNodeTreeExec *exec, int use_tree_data)
 {
 	if (exec) {
-		bNodeTree *ntree= exec->nodetree;
+		bNodeTree *ntree = exec->nodetree;
 		bNodeThreadStack *nts;
 		int a;
 		
 		if (exec->threadstack) {
 			tex_free_delegates(exec);
 			
-			for (a=0; a<BLENDER_MAX_THREADS; a++) {
-				for (nts=exec->threadstack[a].first; nts; nts=nts->next)
+			for (a = 0; a < BLENDER_MAX_THREADS; a++) {
+				for (nts = exec->threadstack[a].first; nts; nts = nts->next)
 					if (nts->stack) MEM_freeN(nts->stack);
 				BLI_freelistN(&exec->threadstack[a]);
 			}
 			
 			MEM_freeN(exec->threadstack);
-			exec->threadstack= NULL;
+			exec->threadstack = NULL;
 		}
 		
 		ntree_exec_end(exec);
@@ -234,24 +240,24 @@ void ntreeTexEndExecTree(bNodeTreeExec *exec, int use_tree_data)
 }
 
 int ntreeTexExecTree(
-	bNodeTree *nodes,
-	TexResult *texres,
-	float *co,
-	float *dxt, float *dyt,
-	int osatex,
-	short thread, 
-	Tex *UNUSED(tex), 
-	short which_output, 
-	int cfra,
-	int preview,
-	ShadeInput *shi,
-	MTex *mtex
-) {
+        bNodeTree *nodes,
+        TexResult *texres,
+        float co[3],
+        float dxt[3], float dyt[3],
+        int osatex,
+        const short thread,
+        Tex *UNUSED(tex),
+        short which_output,
+        int cfra,
+        int preview,
+        ShadeInput *shi,
+        MTex *mtex)
+{
 	TexCallData data;
-	float *nor= texres->nor;
+	float *nor = texres->nor;
 	int retval = TEX_INT;
 	bNodeThreadStack *nts = NULL;
-	bNodeTreeExec *exec= nodes->execdata;
+	bNodeTreeExec *exec = nodes->execdata;
 
 	data.co = co;
 	data.dxt = dxt;
@@ -261,9 +267,9 @@ int ntreeTexExecTree(
 	data.do_preview = preview;
 	data.thread = thread;
 	data.which_output = which_output;
-	data.cfra= cfra;
-	data.mtex= mtex;
-	data.shi= shi;
+	data.cfra = cfra;
+	data.mtex = mtex;
+	data.shi = shi;
 	
 	/* ensure execdata is only initialized once */
 	if (!exec) {
@@ -272,18 +278,18 @@ int ntreeTexExecTree(
 			ntreeTexBeginExecTree(nodes, 1);
 		BLI_unlock_thread(LOCK_NODES);
 
-		exec= nodes->execdata;
+		exec = nodes->execdata;
 	}
 	
-	nts= ntreeGetThreadStack(exec, thread);
+	nts = ntreeGetThreadStack(exec, thread);
 	ntreeExecThreadNodes(exec, nts, &data, thread);
 	ntreeReleaseThreadStack(nts);
 
 	if (texres->nor) retval |= TEX_NOR;
 	retval |= TEX_RGB;
 	/* confusing stuff; the texture output node sets this to NULL to indicate no normal socket was set
-	   however, the texture code checks this for other reasons (namely, a normal is required for material) */
-	texres->nor= nor;
+	 * however, the texture code checks this for other reasons (namely, a normal is required for material) */
+	texres->nor = nor;
 
 	return retval;
 }
