@@ -4,7 +4,7 @@
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. 
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,28 +29,26 @@
  *  \ingroup nodes
  */
 
- 
+
 /*
-	HOW TEXTURE NODES WORK
-
-	In contrast to Shader nodes, which place a color into the output
-	stack when executed, Texture nodes place a TexDelegate* there. To
-	obtain a color value from this, a node further up the chain reads
-	the TexDelegate* from its input stack, and uses tex_call_delegate to
-	retrieve the color from the delegate.
- 
-	comments: (ton)
-
-	This system needs recode, a node system should rely on the stack, and 
-	callbacks for nodes only should evaluate own node, not recursively go
-	over other previous ones.
-*/
+ * HOW TEXTURE NODES WORK
+ *
+ * In contrast to Shader nodes, which place a color into the output
+ * stack when executed, Texture nodes place a TexDelegate* there. To
+ * obtain a color value from this, a node further up the chain reads
+ * the TexDelegate* from its input stack, and uses tex_call_delegate to
+ * retrieve the color from the delegate.
+ *
+ * comments: (ton)
+ *
+ * This system needs recode, a node system should rely on the stack, and
+ * callbacks for nodes only should evaluate own node, not recursively go
+ * over other previous ones.
+ */
 
 #include <assert.h>
 #include "node_texture_util.h"
 
-#define PREV_RES 128 /* default preview resolution */
-
 static void tex_call_delegate(TexDelegate *dg, float *out, TexParams *params, short thread)
 {
 	if (dg->node->need_exec) {
@@ -88,9 +86,9 @@ void tex_input_rgba(float *out, bNodeStack *in, TexParams *params, short thread)
 	}
 	
 	if (in->hasoutput && in->sockettype == SOCK_VECTOR) {
-		out[0] = out[0] * .5f + .5f;
-		out[1] = out[1] * .5f + .5f;
-		out[2] = out[2] * .5f + .5f;
+		out[0] = out[0] * 0.5f + 0.5f;
+		out[1] = out[1] * 0.5f + 0.5f;
+		out[2] = out[2] * 0.5f + 0.5f;
 		out[3] = 1;
 	}
 }
@@ -114,13 +112,13 @@ void params_from_cdata(TexParams *out, TexCallData *in)
 	out->mtex = in->mtex;
 }
 
-void tex_do_preview(bNode *node, float *co, float *col)
+void tex_do_preview(bNode *node, const float coord[2], const float col[4])
 {
-	bNodePreview *preview= node->preview;
+	bNodePreview *preview = node->preview;
 
 	if (preview) {
-		int xs= ((co[0] + 1.0f)*0.5f)*preview->xsize;
-		int ys= ((co[1] + 1.0f)*0.5f)*preview->ysize;
+		int xs = ((coord[0] + 1.0f) * 0.5f) * preview->xsize;
+		int ys = ((coord[1] + 1.0f) * 0.5f) * preview->ysize;
 
 		nodeAddToPreview(node, col, xs, ys, 0); /* 0 = no color management */
 	}
@@ -134,19 +132,19 @@ void tex_output(bNode *node, bNodeStack **in, bNodeStack *out, TexFn texfn, TexC
 		dg = out->data = MEM_mallocN(sizeof(TexDelegate), "tex delegate");
 	else
 		dg = out->data;
-	
-	dg->cdata= cdata;
+
+	dg->cdata = cdata;
 	dg->fn = texfn;
 	dg->node = node;
-	memcpy(dg->in, in, MAX_SOCKET * sizeof(bNodeStack*));
+	memcpy(dg->in, in, MAX_SOCKET * sizeof(bNodeStack *));
 	dg->type = out->sockettype;
 }
 
 void ntreeTexCheckCyclics(struct bNodeTree *ntree)
 {
 	bNode *node;
-	for (node= ntree->nodes.first; node; node= node->next) {
-		
+	for (node = ntree->nodes.first; node; node = node->next) {
+
 		if (node->type == TEX_NODE_TEXTURE && node->id) {
 			/* custom2 stops the node from rendering */
 			if (node->custom1) {
