@@ -34,14 +34,14 @@
 #include "NOD_texture.h"
 #include <math.h>
 
-static bNodeSocketTemplate inputs[]= {
-	{ SOCK_RGBA, 1, "Color1", 1.0f, 0.0f, 0.0f, 1.0f },
-	{ SOCK_RGBA, 1, "Color2", 1.0f, 1.0f, 1.0f, 1.0f },
-	{ SOCK_FLOAT, 1, "Size",   0.5f, 0.0f, 0.0f, 0.0f,  0.0f, 100.0f, PROP_UNSIGNED },
+static bNodeSocketTemplate inputs[] = {
+	{ SOCK_RGBA, 1, N_("Color1"), 1.0f, 0.0f, 0.0f, 1.0f },
+	{ SOCK_RGBA, 1, N_("Color2"), 1.0f, 1.0f, 1.0f, 1.0f },
+	{ SOCK_FLOAT, 1, N_("Size"),   0.5f, 0.0f, 0.0f, 0.0f,  0.0f, 100.0f, PROP_UNSIGNED },
 	{ -1, 0, "" }
 };
-static bNodeSocketTemplate outputs[]= {
-	{ SOCK_RGBA, 0, "Color" },
+static bNodeSocketTemplate outputs[] = {
+	{ SOCK_RGBA, 0, N_("Color") },
 	{ -1, 0, "" }
 };
 
@@ -62,7 +62,7 @@ static void colorfn(float *out, TexParams *p, bNode *UNUSED(node), bNodeStack **
 	}
 	else {
 		tex_input_rgba(out, in[1], p, thread);
-	} 
+	}
 }
 
 static void exec(void *data, bNode *node, bNodeStack **in, bNodeStack **out)
