@@ -36,8 +36,8 @@
 /* **************** CURVE Time  ******************** */
 
 /* custom1 = sfra, custom2 = efra */
-static bNodeSocketTemplate time_outputs[]= {
-	{ SOCK_FLOAT, 0, "Value" },
+static bNodeSocketTemplate time_outputs[] = {
+	{ SOCK_FLOAT, 0, N_("Value") },
 	{ -1, 0, "" }
 };
 
@@ -49,6 +49,7 @@ static void time_colorfn(float *out, TexParams *p, bNode *node, bNodeStack **UNU
 	if (node->custom1 < node->custom2)
 		fac = (p->cfra - node->custom1)/(float)(node->custom2-node->custom1);
 	
+	curvemapping_initialize(node->storage);
 	fac = curvemapping_evaluateF(node->storage, 0, fac);
 	out[0] = CLAMPIS(fac, 0.0f, 1.0f);
 }
@@ -59,7 +60,7 @@ static void time_exec(void *data, bNode *node, bNodeStack **in, bNodeStack **out
 }
 
 
-static void time_init(bNodeTree *UNUSED(ntree), bNode* node, bNodeTemplate *UNUSED(ntemp))
+static void time_init(bNodeTree *UNUSED(ntree), bNode *node, bNodeTemplate *UNUSED(ntemp))
 {
 	node->custom1= 1;
 	node->custom2= 250;
@@ -76,18 +77,19 @@ void register_node_type_tex_curve_time(bNodeTreeType *ttype)
 	node_type_init(&ntype, time_init);
 	node_type_storage(&ntype, "CurveMapping", node_free_curves, node_copy_curves);
 	node_type_exec(&ntype, time_exec);
+	node_type_exec_new(&ntype, node_initexec_curves, NULL, NULL);  /* only for its initexec func */
 	
 	nodeRegisterType(ttype, &ntype);
 }
 
 /* **************** CURVE RGB  ******************** */
-static bNodeSocketTemplate rgb_inputs[]= {
-	{	SOCK_RGBA, 1, "Color",	0.0f, 0.0f, 0.0f, 1.0f},
+static bNodeSocketTemplate rgb_inputs[] = {
+	{	SOCK_RGBA, 1, N_("Color"),	0.0f, 0.0f, 0.0f, 1.0f},
 	{	-1, 0, ""	}
 };
 
-static bNodeSocketTemplate rgb_outputs[]= {
-	{	SOCK_RGBA, 0, "Color"},
+static bNodeSocketTemplate rgb_outputs[] = {
+	{	SOCK_RGBA, 0, N_("Color")},
 	{	-1, 0, ""	}
 };
 
@@ -105,7 +107,7 @@ static void rgb_exec(void *data, bNode *node, bNodeStack **in, bNodeStack **out)
 	tex_output(node, in, out[0], &rgb_colorfn, data);
 }
 
-static void rgb_init(bNodeTree *UNUSED(ntree), bNode* node, bNodeTemplate *UNUSED(ntemp))
+static void rgb_init(bNodeTree *UNUSED(ntree), bNode *node, bNodeTemplate *UNUSED(ntemp))
 {
 	node->storage= curvemapping_add(4, 0.0f, 0.0f, 1.0f, 1.0f);
 }
@@ -120,6 +122,7 @@ void register_node_type_tex_curve_rgb(bNodeTreeType *ttype)
 	node_type_init(&ntype, rgb_init);
 	node_type_storage(&ntype, "CurveMapping", node_free_curves, node_copy_curves);
 	node_type_exec(&ntype, rgb_exec);
+	node_type_exec_new(&ntype, node_initexec_curves, NULL, NULL);  /* only for its initexec func */
 	
 	nodeRegisterType(ttype, &ntype);
 }
