@@ -33,8 +33,8 @@
 #include "node_texture_util.h"
 #include "NOD_texture.h"
 
-static bNodeSocketTemplate outputs[]= {
-	{ SOCK_RGBA, 0, "Image"},
+static bNodeSocketTemplate outputs[] = {
+	{ SOCK_RGBA, 0, N_("Image")},
 	{ -1, 0, "" }
 };
 
@@ -46,7 +46,7 @@ static void colorfn(float *out, TexParams *p, bNode *node, bNodeStack **UNUSED(i
 	ImageUser *iuser= (ImageUser *)node->storage;
 	
 	if ( ima ) {
-		ImBuf *ibuf = BKE_image_get_ibuf(ima, iuser);
+		ImBuf *ibuf = BKE_image_acquire_ibuf(ima, iuser, NULL);
 		if ( ibuf ) {
 			float xsize, ysize;
 			float xoff, yoff;
@@ -76,7 +76,9 @@ static void colorfn(float *out, TexParams *p, bNode *node, bNodeStack **UNUSED(i
 			while ( py >= ibuf->y ) py -= ibuf->y;
 			
 			result = ibuf->rect_float + py*ibuf->x*4 + px*4;
-			copy_v4_v4( out, result );
+			copy_v4_v4(out, result);
+
+			BKE_image_release_ibuf(ima, ibuf, NULL);
 		}
 	}
 }
@@ -86,7 +88,7 @@ static void exec(void *data, bNode *node, bNodeStack **in, bNodeStack **out)
 	tex_output(node, in, out[0], &colorfn, data);
 }
 
-static void init(bNodeTree *UNUSED(ntree), bNode* node, bNodeTemplate *UNUSED(ntemp))
+static void init(bNodeTree *UNUSED(ntree), bNode *node, bNodeTemplate *UNUSED(ntemp))
 {
 	ImageUser *iuser= MEM_callocN(sizeof(ImageUser), "node image user");
 	node->storage= iuser;
