@@ -35,15 +35,15 @@
 
 
 /* **************** SCALAR MATH ******************** */ 
-static bNodeSocketTemplate inputs[]= { 
-	{ SOCK_FLOAT, 1, "Value", 0.5f, 0.5f, 0.5f, 1.0f, -100.0f, 100.0f, PROP_NONE}, 
-	{ SOCK_FLOAT, 1, "Value", 0.5f, 0.5f, 0.5f, 1.0f, -100.0f, 100.0f, PROP_NONE}, 
-	{ -1, 0, "" } 
+static bNodeSocketTemplate inputs[] = {
+	{ SOCK_FLOAT, 1, N_("Value"), 0.5f, 0.5f, 0.5f, 1.0f, -100.0f, 100.0f, PROP_NONE},
+	{ SOCK_FLOAT, 1, N_("Value"), 0.5f, 0.5f, 0.5f, 1.0f, -100.0f, 100.0f, PROP_NONE},
+	{ -1, 0, "" }
 };
 
-static bNodeSocketTemplate outputs[]= { 
-	{ SOCK_FLOAT, 0, "Value"}, 
-	{ -1, 0, "" } 
+static bNodeSocketTemplate outputs[] = {
+	{ SOCK_FLOAT, 0, N_("Value")},
+	{ -1, 0, "" }
 };
 
 static void valuefn(float *out, TexParams *p, bNode *node, bNodeStack **in, short thread)
@@ -51,7 +51,7 @@ static void valuefn(float *out, TexParams *p, bNode *node, bNodeStack **in, shor
 	float in0 = tex_input_value(in[0], p, thread);
 	float in1 = tex_input_value(in[1], p, thread);
 	
-	switch(node->custom1) {
+	switch (node->custom1) {
 	
 	case 0: /* Add */
 		*out= in0 + in1; 
@@ -112,7 +112,7 @@ static void valuefn(float *out, TexParams *p, bNode *node, bNodeStack **in, shor
 		{
 			/* Only raise negative numbers by full integers */
 			if ( in0 >= 0 ) {
-				out[0]= pow(in0, in1);
+				out[0] = pow(in0, in1);
 			}
 			else {
 				float y_mod_1 = fmod(in1, 1);
@@ -178,7 +178,7 @@ static void valuefn(float *out, TexParams *p, bNode *node, bNodeStack **in, shor
 		fprintf(stderr,
 			"%s:%d: unhandeld value in switch statement: %d\n",
 			__FILE__, __LINE__, node->custom1);
-	} 
+	}
 }
 
 static void exec(void *data, bNode *node, bNodeStack **in, bNodeStack **out)
