@@ -34,11 +34,11 @@
 #include "NOD_texture.h"
 #include <math.h>
 
-static bNodeSocketTemplate inputs[]= {
-	{ SOCK_RGBA, 1, "Color", 1.0f, 0.0f, 0.0f, 1.0f },
+static bNodeSocketTemplate inputs[] = {
+	{ SOCK_RGBA, 1, N_("Color"), 1.0f, 0.0f, 0.0f, 1.0f },
 	{ -1, 0, "" }
 };
-static bNodeSocketTemplate outputs[]= {
+static bNodeSocketTemplate outputs[] = {
 	{ -1, 0, "" }
 };
 
@@ -66,7 +66,7 @@ void register_node_type_tex_viewer(bNodeTreeType *ttype)
 	node_type_exec(&ntype, exec);
 	
 	/* Do not allow muting viewer node. */
-	node_type_internal_connect(&ntype, NULL);
+	node_type_internal_links(&ntype, NULL);
 	
 	nodeRegisterType(ttype, &ntype);
 }
