@@ -23,7 +23,7 @@
  * ***** END GPL LICENSE BLOCK *****
  */
 
-/** \file blender/python/bmesh/bmesh_py_api.c
+/** \file blender/python/bmesh/bmesh_py_utils.c
  *  \ingroup pybmesh
  *
  * This file defines the 'bmesh.utils' module.
@@ -39,9 +39,7 @@
 #include "../mathutils/mathutils.h"
 
 #include "bmesh.h"
-
 #include "bmesh_py_types.h"
-
 #include "bmesh_py_utils.h" /* own include */
 
 
@@ -92,7 +90,7 @@ static PyObject *bpy_bm_utils_vert_collapse_edge(PyObject *UNUSED(self), PyObjec
 
 	bm = py_edge->bm;
 
-	e_new = BM_vert_collapse_edge(bm, py_edge->e, py_vert->v, TRUE);
+	e_new = BM_vert_collapse_edge(bm, py_edge->e, py_vert->v, true);
 
 	if (e_new) {
 		return BPy_BMEdge_CreatePyObject(bm, e_new);
@@ -108,7 +106,7 @@ static PyObject *bpy_bm_utils_vert_collapse_edge(PyObject *UNUSED(self), PyObjec
 PyDoc_STRVAR(bpy_bm_utils_vert_collapse_faces_doc,
 ".. method:: vert_collapse_faces(vert, edge, fac, join_faces)\n"
 "\n"
-"   Split an edge, return the newly created data.\n"
+"   Collapses a vertex that has only two manifold edges onto a vertex it shares an edge with.\n"
 "\n"
 "   :arg vert: The vert that will be collapsed.\n"
 "   :type vert: :class:`bmesh.types.BMVert`\n"
@@ -158,14 +156,14 @@ static PyObject *bpy_bm_utils_vert_collapse_faces(PyObject *UNUSED(self), PyObje
 
 	bm = py_edge->bm;
 
-	e_new = BM_vert_collapse_faces(bm, py_edge->e, py_vert->v, CLAMPIS(fac, 0.0f, 1.0f), do_join_faces, TRUE);
+	e_new = BM_vert_collapse_faces(bm, py_edge->e, py_vert->v, CLAMPIS(fac, 0.0f, 1.0f), do_join_faces, true);
 
 	if (e_new) {
 		return BPy_BMEdge_CreatePyObject(bm, e_new);
 	}
 	else {
 		PyErr_SetString(PyExc_ValueError,
-		                "vert_collapse_edge(vert, edge): no new edge created, internal error");
+		                "vert_collapse_faces(vert, edge): no new edge created, internal error");
 		return NULL;
 	}
 }
@@ -241,7 +239,7 @@ static PyObject *bpy_bm_utils_vert_separate(PyObject *UNUSED(self), PyObject *ar
 
 	edge_array = BPy_BMElem_PySeq_As_Array(&bm, edge_seq, 0, PY_SSIZE_T_MAX,
 	                                       &edge_array_len, BM_EDGE,
-	                                       TRUE, TRUE, "vert_separate(...)");
+	                                       true, true, "vert_separate(...)");
 
 	if (edge_array == NULL) {
 		return NULL;
@@ -340,7 +338,7 @@ PyDoc_STRVAR(bpy_bm_utils_edge_rotate_doc,
 static PyObject *bpy_bm_utils_edge_rotate(PyObject *UNUSED(self), PyObject *args)
 {
 	BPy_BMEdge *py_edge;
-	int do_ccw = FALSE;
+	int do_ccw = false;
 
 	BMesh *bm;
 	BMEdge *e_new = NULL;
@@ -370,7 +368,7 @@ static PyObject *bpy_bm_utils_edge_rotate(PyObject *UNUSED(self), PyObject *args
 PyDoc_STRVAR(bpy_bm_utils_face_split_doc,
 ".. method:: face_split(face, vert_a, vert_b, coords=(), use_exist=True, example=None)\n"
 "\n"
-"   Split an edge, return the newly created data.\n"
+"   Face split with optional intermediate points.\n"
 "\n"
 "   :arg face: The face to cut.\n"
 "   :type face: :class:`bmesh.types.BMFace`\n"
@@ -398,7 +396,7 @@ static PyObject *bpy_bm_utils_face_split(PyObject *UNUSED(self), PyObject *args,
 
 	/* optional */
 	PyObject *py_coords = NULL;
-	int edge_exists = TRUE;
+	int edge_exists = true;
 	BPy_BMEdge *py_edge_example = NULL;
 
 	float *coords;
@@ -428,8 +426,8 @@ static PyObject *bpy_bm_utils_face_split(PyObject *UNUSED(self), PyObject *args,
 	}
 
 	/* this doubles for checking that the verts are in the same mesh */
-	if (BM_vert_in_face(py_face->f, py_vert_a->v) == FALSE ||
-	    BM_vert_in_face(py_face->f, py_vert_b->v) == FALSE)
+	if (BM_vert_in_face(py_face->f, py_vert_a->v) == false ||
+	    BM_vert_in_face(py_face->f, py_vert_b->v) == false)
 	{
 		PyErr_SetString(PyExc_ValueError,
 		                "face_split(...): one of the verts passed is not found in the face");
@@ -457,6 +455,7 @@ static PyObject *bpy_bm_utils_face_split(PyObject *UNUSED(self), PyObject *args,
 		                        py_vert_a->v, py_vert_b->v,
 		                        (float (*)[3])coords, ncoords,
 		                        &l_new, py_edge_example ? py_edge_example->e : NULL);
+		PyMem_Free(coords);
 	}
 	else {
 		f_new = BM_face_split(bm, py_face->f,
@@ -497,7 +496,7 @@ static PyObject *bpy_bm_utils_face_join(PyObject *UNUSED(self), PyObject *args)
 	BMFace **face_array;
 	Py_ssize_t face_seq_len = 0;
 	BMFace *f_new;
-	int do_remove = TRUE;
+	int do_remove = true;
 
 	if (!PyArg_ParseTuple(args, "O|i:face_join", &py_face_array, &do_remove)) {
 		return NULL;
@@ -505,7 +504,7 @@ static PyObject *bpy_bm_utils_face_join(PyObject *UNUSED(self), PyObject *args)
 
 	face_array = BPy_BMElem_PySeq_As_Array(&bm, py_face_array, 2, PY_SSIZE_T_MAX,
 	                                       &face_seq_len, BM_FACE,
-	                                       TRUE, TRUE, "face_join(...)");
+	                                       true, true, "face_join(...)");
 
 	if (face_array == NULL) {
 		return NULL; /* error will be set */
@@ -558,16 +557,10 @@ static PyObject *bpy_bm_utils_face_vert_separate(PyObject *UNUSED(self), PyObjec
 		return NULL;
 	}
 
-	BPY_BM_CHECK_OBJ(py_face);
-	BPY_BM_CHECK_OBJ(py_vert);
-
 	bm = py_face->bm;
 
-	if (bm != py_vert->bm) {
-		PyErr_SetString(PyExc_ValueError,
-		                "mesh elements are from different meshes");
-		return NULL;
-	}
+	BPY_BM_CHECK_OBJ(py_face);
+	BPY_BM_CHECK_SOURCE_OBJ(py_vert, bm, "face_vert_separate()");
 
 	l = BM_face_vert_share_loop(py_face->f, py_vert->v);
 
@@ -652,34 +645,34 @@ static PyObject *bpy_bm_utils_loop_separate(PyObject *UNUSED(self), BPy_BMLoop *
 
 
 static struct PyMethodDef BPy_BM_utils_methods[] = {
-    {"vert_collapse_edge",  (PyCFunction)bpy_bm_utils_vert_collapse_edge,  METH_VARARGS, bpy_bm_utils_vert_collapse_edge_doc},
-    {"vert_collapse_faces", (PyCFunction)bpy_bm_utils_vert_collapse_faces, METH_VARARGS, bpy_bm_utils_vert_collapse_faces_doc},
-    {"vert_dissolve",       (PyCFunction)bpy_bm_utils_vert_dissolve,       METH_VARARGS, bpy_bm_utils_vert_dissolve_doc}, /* could use METH_O */
-    {"vert_separate",       (PyCFunction)bpy_bm_utils_vert_separate,       METH_VARARGS, bpy_bm_utils_vert_separate_doc},
-    {"edge_split",          (PyCFunction)bpy_bm_utils_edge_split,          METH_VARARGS, bpy_bm_utils_edge_split_doc},
-    {"edge_rotate",         (PyCFunction)bpy_bm_utils_edge_rotate,         METH_VARARGS, bpy_bm_utils_edge_rotate_doc},
-    {"face_split",          (PyCFunction)bpy_bm_utils_face_split,          METH_VARARGS | METH_KEYWORDS, bpy_bm_utils_face_split_doc},
-    {"face_join",           (PyCFunction)bpy_bm_utils_face_join,           METH_VARARGS, bpy_bm_utils_face_join_doc},
-    {"face_vert_separate",  (PyCFunction)bpy_bm_utils_face_vert_separate,  METH_VARARGS, bpy_bm_utils_face_vert_separate_doc},
-    {"face_flip",           (PyCFunction)bpy_bm_utils_face_flip,           METH_O,       bpy_bm_utils_face_flip_doc},
-    {"loop_separate",       (PyCFunction)bpy_bm_utils_loop_separate,       METH_O,       bpy_bm_utils_loop_separate_doc},
-    {NULL, NULL, 0, NULL}
+	{"vert_collapse_edge",  (PyCFunction)bpy_bm_utils_vert_collapse_edge,  METH_VARARGS, bpy_bm_utils_vert_collapse_edge_doc},
+	{"vert_collapse_faces", (PyCFunction)bpy_bm_utils_vert_collapse_faces, METH_VARARGS, bpy_bm_utils_vert_collapse_faces_doc},
+	{"vert_dissolve",       (PyCFunction)bpy_bm_utils_vert_dissolve,       METH_VARARGS, bpy_bm_utils_vert_dissolve_doc}, /* could use METH_O */
+	{"vert_separate",       (PyCFunction)bpy_bm_utils_vert_separate,       METH_VARARGS, bpy_bm_utils_vert_separate_doc},
+	{"edge_split",          (PyCFunction)bpy_bm_utils_edge_split,          METH_VARARGS, bpy_bm_utils_edge_split_doc},
+	{"edge_rotate",         (PyCFunction)bpy_bm_utils_edge_rotate,         METH_VARARGS, bpy_bm_utils_edge_rotate_doc},
+	{"face_split",          (PyCFunction)bpy_bm_utils_face_split,          METH_VARARGS | METH_KEYWORDS, bpy_bm_utils_face_split_doc},
+	{"face_join",           (PyCFunction)bpy_bm_utils_face_join,           METH_VARARGS, bpy_bm_utils_face_join_doc},
+	{"face_vert_separate",  (PyCFunction)bpy_bm_utils_face_vert_separate,  METH_VARARGS, bpy_bm_utils_face_vert_separate_doc},
+	{"face_flip",           (PyCFunction)bpy_bm_utils_face_flip,           METH_O,       bpy_bm_utils_face_flip_doc},
+	{"loop_separate",       (PyCFunction)bpy_bm_utils_loop_separate,       METH_O,       bpy_bm_utils_loop_separate_doc},
+	{NULL, NULL, 0, NULL}
 };
 
 
-PyDoc_STRVAR(BPy_BM_doc,
+PyDoc_STRVAR(BPy_BM_utils_doc,
 "This module provides access to blenders bmesh data structures."
 );
-static struct PyModuleDef BPy_BM_types_module_def = {
-    PyModuleDef_HEAD_INIT,
-    "bmesh.utils",  /* m_name */
-    BPy_BM_doc,  /* m_doc */
-    0,  /* m_size */
-    BPy_BM_utils_methods,  /* m_methods */
-    NULL,  /* m_reload */
-    NULL,  /* m_traverse */
-    NULL,  /* m_clear */
-    NULL,  /* m_free */
+static struct PyModuleDef BPy_BM_utils_module_def = {
+	PyModuleDef_HEAD_INIT,
+	"bmesh.utils",  /* m_name */
+	BPy_BM_utils_doc,  /* m_doc */
+	0,  /* m_size */
+	BPy_BM_utils_methods,  /* m_methods */
+	NULL,  /* m_reload */
+	NULL,  /* m_traverse */
+	NULL,  /* m_clear */
+	NULL,  /* m_free */
 };
 
 
@@ -687,7 +680,7 @@ PyObject *BPyInit_bmesh_utils(void)
 {
 	PyObject *submodule;
 
-	submodule = PyModule_Create(&BPy_BM_types_module_def);
+	submodule = PyModule_Create(&BPy_BM_utils_module_def);
 
 	return submodule;
 }
