@@ -23,10 +23,13 @@
  *  \ingroup pythonintern
  */
 
- 
+#ifndef __BPY_H__
+
 void BPy_init_modules(void);
 extern PyObject *bpy_package_py;
 
 /* bpy_interface_atexit.c */
 void BPY_atexit_register(void);
 void BPY_atexit_unregister(void);
+
+#endif  /* __BPY_H__ */
