@@ -1,5 +1,4 @@
 /*
- *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -32,12 +31,15 @@
 
 #include "BLI_math.h"
 #include "BLI_utildefines.h"
-#include "BLI_dynstr.h"
+
+#ifndef MATH_STANDALONE
+#  include "BLI_dynstr.h"
+#endif
 
 #define COLOR_SIZE 3
 
-//----------------------------------mathutils.Color() -------------------
-//makes a new color for you to play with
+/* ----------------------------------mathutils.Color() ------------------- */
+/* makes a new color for you to play with */
 static PyObject *Color_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
 	float col[3] = {0.0f, 0.0f, 0.0f};
@@ -65,7 +67,7 @@ static PyObject *Color_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	return Color_CreatePyObject(col, Py_NEW, type);
 }
 
-//-----------------------------METHODS----------------------------
+/* -----------------------------METHODS---------------------------- */
 
 /* note: BaseMath_ReadCallback must be called beforehand */
 static PyObject *Color_ToTupleExt(ColorObject *self, int ndigits)
@@ -114,8 +116,8 @@ static PyObject *Color_deepcopy(ColorObject *self, PyObject *args)
 	return Color_copy(self);
 }
 
-//----------------------------print object (internal)--------------
-//print the object to screen
+/* ----------------------------print object (internal)-------------- */
+/* print the object to screen */
 
 static PyObject *Color_repr(ColorObject *self)
 {
@@ -132,6 +134,7 @@ static PyObject *Color_repr(ColorObject *self)
 	return ret;
 }
 
+#ifndef MATH_STANDALONE
 static PyObject *Color_str(ColorObject *self)
 {
 	DynStr *ds;
@@ -146,9 +149,10 @@ static PyObject *Color_str(ColorObject *self)
 
 	return mathutils_dynstr_to_py(ds); /* frees ds */
 }
+#endif
 
-//------------------------tp_richcmpr
-//returns -1 execption, 0 false, 1 true
+/* ------------------------tp_richcmpr */
+/* returns -1 exception, 0 false, 1 true */
 static PyObject *Color_richcmpr(PyObject *a, PyObject *b, int op)
 {
 	PyObject *res;
@@ -185,15 +189,15 @@ static PyObject *Color_richcmpr(PyObject *a, PyObject *b, int op)
 	return Py_INCREF(res), res;
 }
 
-//---------------------SEQUENCE PROTOCOLS------------------------
-//----------------------------len(object)------------------------
-//sequence length
+/* ---------------------SEQUENCE PROTOCOLS------------------------ */
+/* ----------------------------len(object)------------------------ */
+/* sequence length */
 static int Color_len(ColorObject *UNUSED(self))
 {
 	return COLOR_SIZE;
 }
-//----------------------------object[]---------------------------
-//sequence accessor (get)
+/* ----------------------------object[]--------------------------- */
+/* sequence accessor (get) */
 static PyObject *Color_item(ColorObject *self, int i)
 {
 	if (i < 0) i = COLOR_SIZE - i;
@@ -211,13 +215,13 @@ static PyObject *Color_item(ColorObject *self, int i)
 	return PyFloat_FromDouble(self->col[i]);
 
 }
-//----------------------------object[]-------------------------
-//sequence accessor (set)
+/* ----------------------------object[]------------------------- */
+/* sequence accessor (set) */
 static int Color_ass_item(ColorObject *self, int i, PyObject *value)
 {
 	float f = PyFloat_AsDouble(value);
 
-	if (f == -1 && PyErr_Occurred()) { // parsed item not a number
+	if (f == -1 && PyErr_Occurred()) {  /* parsed item not a number */
 		PyErr_SetString(PyExc_TypeError,
 		                "color[item] = x: "
 		                "argument not a number");
@@ -239,8 +243,8 @@ static int Color_ass_item(ColorObject *self, int i, PyObject *value)
 
 	return 0;
 }
-//----------------------------object[z:y]------------------------
-//sequence slice (get)
+/* ----------------------------object[z:y]------------------------ */
+/* sequence slice (get) */
 static PyObject *Color_slice(ColorObject *self, int begin, int end)
 {
 	PyObject *tuple;
@@ -261,8 +265,8 @@ static PyObject *Color_slice(ColorObject *self, int begin, int end)
 
 	return tuple;
 }
-//----------------------------object[z:y]------------------------
-//sequence slice (set)
+/* ----------------------------object[z:y]------------------------ */
+/* sequence slice (set) */
 static int Color_ass_slice(ColorObject *self, int begin, int end, PyObject *seq)
 {
 	int i, size;
@@ -307,7 +311,7 @@ static PyObject *Color_subscript(ColorObject *self, PyObject *item)
 	else if (PySlice_Check(item)) {
 		Py_ssize_t start, stop, step, slicelength;
 
-		if (PySlice_GetIndicesEx((void *)item, COLOR_SIZE, &start, &stop, &step, &slicelength) < 0)
+		if (PySlice_GetIndicesEx(item, COLOR_SIZE, &start, &stop, &step, &slicelength) < 0)
 			return NULL;
 
 		if (slicelength <= 0) {
@@ -343,7 +347,7 @@ static int Color_ass_subscript(ColorObject *self, PyObject *item, PyObject *valu
 	else if (PySlice_Check(item)) {
 		Py_ssize_t start, stop, step, slicelength;
 
-		if (PySlice_GetIndicesEx((void *)item, COLOR_SIZE, &start, &stop, &step, &slicelength) < 0)
+		if (PySlice_GetIndicesEx(item, COLOR_SIZE, &start, &stop, &step, &slicelength) < 0)
 			return -1;
 
 		if (step == 1)
@@ -362,7 +366,7 @@ static int Color_ass_subscript(ColorObject *self, PyObject *item, PyObject *valu
 	}
 }
 
-//-----------------PROTCOL DECLARATIONS--------------------------
+/* -----------------PROTCOL DECLARATIONS-------------------------- */
 static PySequenceMethods Color_SeqMethods = {
 	(lenfunc) Color_len,                    /* sq_length */
 	(binaryfunc) NULL,                      /* sq_concat */
@@ -566,7 +570,7 @@ static PyObject *Color_div(PyObject *v1, PyObject *v2)
 	return NULL;
 }
 
-/* mulplication in-place: obj *= obj */
+/* multiplication in-place: obj *= obj */
 static PyObject *Color_imul(PyObject *v1, PyObject *v2)
 {
 	ColorObject *color = (ColorObject *)v1;
@@ -592,7 +596,7 @@ static PyObject *Color_imul(PyObject *v1, PyObject *v2)
 	return v1;
 }
 
-/* mulplication in-place: obj *= obj */
+/* multiplication in-place: obj *= obj */
 static PyObject *Color_idiv(PyObject *v1, PyObject *v2)
 {
 	ColorObject *color = (ColorObject *)v1;
@@ -645,8 +649,8 @@ static PyNumberMethods Color_NumMethods = {
 	NULL,               /*nb_remainder*/
 	NULL,               /*nb_divmod*/
 	NULL,               /*nb_power*/
-	(unaryfunc) Color_neg, /*nb_negative*/
-	(unaryfunc) NULL,   /*tp_positive*/
+	(unaryfunc) Color_neg,   /*nb_negative*/
+	(unaryfunc) Color_copy,  /*tp_positive*/
 	(unaryfunc) NULL,   /*tp_absolute*/
 	(inquiry)   NULL,   /*tp_bool*/
 	(unaryfunc) NULL,   /*nb_invert*/
@@ -724,10 +728,10 @@ static int Color_channel_hsv_set(ColorObject *self, PyObject *value, void *type)
 	if (BaseMath_ReadCallback(self) == -1)
 		return -1;
 
-	rgb_to_hsv(self->col[0], self->col[1], self->col[2], &(hsv[0]), &(hsv[1]), &(hsv[2]));
+	rgb_to_hsv_v(self->col, hsv);
 	CLAMP(f, 0.0f, 1.0f);
 	hsv[i] = f;
-	hsv_to_rgb(hsv[0], hsv[1], hsv[2], &(self->col[0]), &(self->col[1]), &(self->col[2]));
+	hsv_to_rgb_v(hsv, self->col);
 
 	if (BaseMath_WriteCallback(self) == -1)
 		return -1;
@@ -765,7 +769,7 @@ static int Color_hsv_set(ColorObject *self, PyObject *value, void *UNUSED(closur
 	CLAMP(hsv[1], 0.0f, 1.0f);
 	CLAMP(hsv[2], 0.0f, 1.0f);
 
-	hsv_to_rgb(hsv[0], hsv[1], hsv[2], &(self->col[0]), &(self->col[1]), &(self->col[2]));
+	hsv_to_rgb_v(hsv, self->col);
 
 	if (BaseMath_WriteCallback(self) == -1)
 		return -1;
@@ -793,7 +797,7 @@ static PyGetSetDef Color_getseters[] = {
 };
 
 
-//-----------------------METHOD DEFINITIONS ----------------------
+/* -----------------------METHOD DEFINITIONS ---------------------- */
 static struct PyMethodDef Color_methods[] = {
 	{"copy", (PyCFunction) Color_copy, METH_NOARGS, Color_copy_doc},
 	{"__copy__", (PyCFunction) Color_copy, METH_NOARGS, Color_copy_doc},
@@ -801,65 +805,69 @@ static struct PyMethodDef Color_methods[] = {
 	{NULL, NULL, 0, NULL}
 };
 
-//------------------PY_OBECT DEFINITION--------------------------
+/* ------------------PY_OBECT DEFINITION-------------------------- */
 PyDoc_STRVAR(color_doc,
 "This object gives access to Colors in Blender."
 );
 PyTypeObject color_Type = {
 	PyVarObject_HEAD_INIT(NULL, 0)
-	"Color",                        //tp_name
-	sizeof(ColorObject),            //tp_basicsize
-	0,                              //tp_itemsize
-	(destructor)BaseMathObject_dealloc,     //tp_dealloc
-	NULL,                           //tp_print
-	NULL,                           //tp_getattr
-	NULL,                           //tp_setattr
-	NULL,                           //tp_compare
-	(reprfunc) Color_repr,          //tp_repr
-	&Color_NumMethods,              //tp_as_number
-	&Color_SeqMethods,              //tp_as_sequence
-	&Color_AsMapping,               //tp_as_mapping
-	NULL,                           //tp_hash
-	NULL,                           //tp_call
-	(reprfunc) Color_str,           //tp_str
-	NULL,                           //tp_getattro
-	NULL,                           //tp_setattro
-	NULL,                           //tp_as_buffer
-	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC, //tp_flags
-	color_doc, //tp_doc
-	(traverseproc)BaseMathObject_traverse,  //tp_traverse
-	(inquiry)BaseMathObject_clear,  //tp_clear
-	(richcmpfunc)Color_richcmpr,    //tp_richcompare
-	0,                              //tp_weaklistoffset
-	NULL,                           //tp_iter
-	NULL,                           //tp_iternext
-	Color_methods,                  //tp_methods
-	NULL,                           //tp_members
-	Color_getseters,                //tp_getset
-	NULL,                           //tp_base
-	NULL,                           //tp_dict
-	NULL,                           //tp_descr_get
-	NULL,                           //tp_descr_set
-	0,                              //tp_dictoffset
-	NULL,                           //tp_init
-	NULL,                           //tp_alloc
-	Color_new,                      //tp_new
-	NULL,                           //tp_free
-	NULL,                           //tp_is_gc
-	NULL,                           //tp_bases
-	NULL,                           //tp_mro
-	NULL,                           //tp_cache
-	NULL,                           //tp_subclasses
-	NULL,                           //tp_weaklist
-	NULL                            //tp_del
+	"Color",                        /* tp_name */
+	sizeof(ColorObject),            /* tp_basicsize */
+	0,                              /* tp_itemsize */
+	(destructor)BaseMathObject_dealloc,  /* tp_dealloc */
+	NULL,                           /* tp_print */
+	NULL,                           /* tp_getattr */
+	NULL,                           /* tp_setattr */
+	NULL,                           /* tp_compare */
+	(reprfunc) Color_repr,          /* tp_repr */
+	&Color_NumMethods,              /* tp_as_number */
+	&Color_SeqMethods,              /* tp_as_sequence */
+	&Color_AsMapping,               /* tp_as_mapping */
+	NULL,                           /* tp_hash */
+	NULL,                           /* tp_call */
+#ifndef MATH_STANDALONE
+	(reprfunc) Color_str,           /* tp_str */
+#else
+	NULL,                           /* tp_str */
+#endif
+	NULL,                           /* tp_getattro */
+	NULL,                           /* tp_setattro */
+	NULL,                           /* tp_as_buffer */
+	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC, /* tp_flags */
+	color_doc, /* tp_doc */
+	(traverseproc)BaseMathObject_traverse,  /* tp_traverse */
+	(inquiry)BaseMathObject_clear,  /* tp_clear */
+	(richcmpfunc)Color_richcmpr,    /* tp_richcompare */
+	0,                              /* tp_weaklistoffset */
+	NULL,                           /* tp_iter */
+	NULL,                           /* tp_iternext */
+	Color_methods,                  /* tp_methods */
+	NULL,                           /* tp_members */
+	Color_getseters,                /* tp_getset */
+	NULL,                           /* tp_base */
+	NULL,                           /* tp_dict */
+	NULL,                           /* tp_descr_get */
+	NULL,                           /* tp_descr_set */
+	0,                              /* tp_dictoffset */
+	NULL,                           /* tp_init */
+	NULL,                           /* tp_alloc */
+	Color_new,                      /* tp_new */
+	NULL,                           /* tp_free */
+	NULL,                           /* tp_is_gc */
+	NULL,                           /* tp_bases */
+	NULL,                           /* tp_mro */
+	NULL,                           /* tp_cache */
+	NULL,                           /* tp_subclasses */
+	NULL,                           /* tp_weaklist */
+	NULL                            /* tp_del */
 };
-//------------------------Color_CreatePyObject (internal)-------------
-//creates a new color object
+/* ------------------------Color_CreatePyObject (internal)------------- */
+/* creates a new color object */
 /* pass Py_WRAP - if vector is a WRAPPER for data allocated by BLENDER
  *  (i.e. it was allocated elsewhere by MEM_mallocN())
  *   pass Py_NEW - if vector is not a WRAPPER and managed by PYTHON
  *  (i.e. it must be created here with PyMEM_malloc())*/
-PyObject *Color_CreatePyObject(float *col, int type, PyTypeObject *base_type)
+PyObject *Color_CreatePyObject(float col[3], int type, PyTypeObject *base_type)
 {
 	ColorObject *self;
 
