@@ -35,7 +35,10 @@
 
 #include "BLI_math.h"
 #include "BLI_utildefines.h"
-#include "BLI_dynstr.h"
+
+#ifndef MATH_STANDALONE
+#  include "BLI_dynstr.h"
+#endif
 
 #define QUAT_SIZE 4
 
@@ -44,7 +47,7 @@ static void      quat__axis_angle_sanitize(float axis[3], float *angle);
 static PyObject *Quaternion_copy(QuaternionObject *self);
 static PyObject *Quaternion_deepcopy(QuaternionObject *self, PyObject *args);
 
-//-----------------------------METHODS------------------------------
+/* -----------------------------METHODS------------------------------ */
 
 /* note: BaseMath_ReadCallback must be called beforehand */
 static PyObject *Quaternion_to_tuple_ext(QuaternionObject *self, int ndigits)
@@ -124,7 +127,7 @@ static PyObject *Quaternion_to_euler(QuaternionObject *self, PyObject *args)
 
 	return Euler_CreatePyObject(eul, order, Py_NEW, NULL);
 }
-//----------------------------Quaternion.toMatrix()------------------
+
 PyDoc_STRVAR(Quaternion_to_matrix_doc,
 ".. method:: to_matrix()\n"
 "\n"
@@ -144,7 +147,6 @@ static PyObject *Quaternion_to_matrix(QuaternionObject *self)
 	return Matrix_CreatePyObject(mat, 3, 3, Py_NEW, NULL);
 }
 
-//----------------------------Quaternion.toMatrix()------------------
 PyDoc_STRVAR(Quaternion_to_axis_angle_doc,
 ".. method:: to_axis_angle()\n"
 "\n"
@@ -176,8 +178,6 @@ static PyObject *Quaternion_to_axis_angle(QuaternionObject *self)
 	return ret;
 }
 
-
-//----------------------------Quaternion.cross(other)------------------
 PyDoc_STRVAR(Quaternion_cross_doc,
 ".. method:: cross(other)\n"
 "\n"
@@ -196,7 +196,8 @@ static PyObject *Quaternion_cross(QuaternionObject *self, PyObject *value)
 		return NULL;
 
 	if (mathutils_array_parse(tquat, QUAT_SIZE, QUAT_SIZE, value,
-	                          "Quaternion.cross(other), invalid 'other' arg") == -1) {
+	                          "Quaternion.cross(other), invalid 'other' arg") == -1)
+	{
 		return NULL;
 	}
 
@@ -204,7 +205,6 @@ static PyObject *Quaternion_cross(QuaternionObject *self, PyObject *value)
 	return Quaternion_CreatePyObject(quat, Py_NEW, Py_TYPE(self));
 }
 
-//----------------------------Quaternion.dot(other)------------------
 PyDoc_STRVAR(Quaternion_dot_doc,
 ".. method:: dot(other)\n"
 "\n"
@@ -334,8 +334,8 @@ static PyObject *Quaternion_rotate(QuaternionObject *self, PyObject *value)
 	Py_RETURN_NONE;
 }
 
-//----------------------------Quaternion.normalize()----------------
-//normalize the axis of rotation of [theta, vector]
+/* ----------------------------Quaternion.normalize()---------------- */
+/* normalize the axis of rotation of [theta, vector] */
 PyDoc_STRVAR(Quaternion_normalize_doc,
 ".. function:: normalize()\n"
 "\n"
@@ -364,7 +364,6 @@ static PyObject *Quaternion_normalized(QuaternionObject *self)
 	return quat__apply_to_copy((PyNoArgsFunction)Quaternion_normalize, self);
 }
 
-//----------------------------Quaternion.invert()------------------
 PyDoc_STRVAR(Quaternion_invert_doc,
 ".. function:: invert()\n"
 "\n"
@@ -393,7 +392,6 @@ static PyObject *Quaternion_inverted(QuaternionObject *self)
 	return quat__apply_to_copy((PyNoArgsFunction)Quaternion_invert, self);
 }
 
-//----------------------------Quaternion.identity()-----------------
 PyDoc_STRVAR(Quaternion_identity_doc,
 ".. function:: identity()\n"
 "\n"
@@ -412,7 +410,7 @@ static PyObject *Quaternion_identity(QuaternionObject *self)
 	(void)BaseMath_WriteCallback(self);
 	Py_RETURN_NONE;
 }
-//----------------------------Quaternion.negate()-------------------
+
 PyDoc_STRVAR(Quaternion_negate_doc,
 ".. function:: negate()\n"
 "\n"
@@ -431,7 +429,7 @@ static PyObject *Quaternion_negate(QuaternionObject *self)
 	(void)BaseMath_WriteCallback(self);
 	Py_RETURN_NONE;
 }
-//----------------------------Quaternion.conjugate()----------------
+
 PyDoc_STRVAR(Quaternion_conjugate_doc,
 ".. function:: conjugate()\n"
 "\n"
@@ -460,7 +458,6 @@ static PyObject *Quaternion_conjugated(QuaternionObject *self)
 	return quat__apply_to_copy((PyNoArgsFunction)Quaternion_conjugate, self);
 }
 
-//----------------------------Quaternion.copy()----------------
 PyDoc_STRVAR(Quaternion_copy_doc,
 ".. function:: copy()\n"
 "\n"
@@ -486,8 +483,7 @@ static PyObject *Quaternion_deepcopy(QuaternionObject *self, PyObject *args)
 	return Quaternion_copy(self);
 }
 
-//----------------------------print object (internal)--------------
-//print the object to screen
+/* print the object to screen */
 static PyObject *Quaternion_repr(QuaternionObject *self)
 {
 	PyObject *ret, *tuple;
@@ -503,6 +499,7 @@ static PyObject *Quaternion_repr(QuaternionObject *self)
 	return ret;
 }
 
+#ifndef MATH_STANDALONE
 static PyObject *Quaternion_str(QuaternionObject *self)
 {
 	DynStr *ds;
@@ -517,6 +514,7 @@ static PyObject *Quaternion_str(QuaternionObject *self)
 
 	return mathutils_dynstr_to_py(ds); /* frees ds */
 }
+#endif
 
 static PyObject *Quaternion_richcmpr(PyObject *a, PyObject *b, int op)
 {
@@ -554,15 +552,15 @@ static PyObject *Quaternion_richcmpr(PyObject *a, PyObject *b, int op)
 	return Py_INCREF(res), res;
 }
 
-//---------------------SEQUENCE PROTOCOLS------------------------
-//----------------------------len(object)------------------------
-//sequence length
+/* ---------------------SEQUENCE PROTOCOLS------------------------ */
+/* ----------------------------len(object)------------------------ */
+/* sequence length */
 static int Quaternion_len(QuaternionObject *UNUSED(self))
 {
 	return QUAT_SIZE;
 }
-//----------------------------object[]---------------------------
-//sequence accessor (get)
+/* ----------------------------object[]--------------------------- */
+/* sequence accessor (get) */
 static PyObject *Quaternion_item(QuaternionObject *self, int i)
 {
 	if (i < 0) i = QUAT_SIZE - i;
@@ -580,8 +578,8 @@ static PyObject *Quaternion_item(QuaternionObject *self, int i)
 	return PyFloat_FromDouble(self->quat[i]);
 
 }
-//----------------------------object[]-------------------------
-//sequence accessor (set)
+/* ----------------------------object[]------------------------- */
+/* sequence accessor (set) */
 static int Quaternion_ass_item(QuaternionObject *self, int i, PyObject *ob)
 {
 	float scalar = (float)PyFloat_AsDouble(ob);
@@ -607,8 +605,8 @@ static int Quaternion_ass_item(QuaternionObject *self, int i, PyObject *ob)
 
 	return 0;
 }
-//----------------------------object[z:y]------------------------
-//sequence slice (get)
+/* ----------------------------object[z:y]------------------------ */
+/* sequence slice (get) */
 static PyObject *Quaternion_slice(QuaternionObject *self, int begin, int end)
 {
 	PyObject *tuple;
@@ -629,8 +627,8 @@ static PyObject *Quaternion_slice(QuaternionObject *self, int begin, int end)
 
 	return tuple;
 }
-//----------------------------object[z:y]------------------------
-//sequence slice (set)
+/* ----------------------------object[z:y]------------------------ */
+/* sequence slice (set) */
 static int Quaternion_ass_slice(QuaternionObject *self, int begin, int end, PyObject *seq)
 {
 	int i, size;
@@ -677,7 +675,7 @@ static PyObject *Quaternion_subscript(QuaternionObject *self, PyObject *item)
 	else if (PySlice_Check(item)) {
 		Py_ssize_t start, stop, step, slicelength;
 
-		if (PySlice_GetIndicesEx((void *)item, QUAT_SIZE, &start, &stop, &step, &slicelength) < 0)
+		if (PySlice_GetIndicesEx(item, QUAT_SIZE, &start, &stop, &step, &slicelength) < 0)
 			return NULL;
 
 		if (slicelength <= 0) {
@@ -714,7 +712,7 @@ static int Quaternion_ass_subscript(QuaternionObject *self, PyObject *item, PyOb
 	else if (PySlice_Check(item)) {
 		Py_ssize_t start, stop, step, slicelength;
 
-		if (PySlice_GetIndicesEx((void *)item, QUAT_SIZE, &start, &stop, &step, &slicelength) < 0)
+		if (PySlice_GetIndicesEx(item, QUAT_SIZE, &start, &stop, &step, &slicelength) < 0)
 			return -1;
 
 		if (step == 1)
@@ -733,9 +731,9 @@ static int Quaternion_ass_subscript(QuaternionObject *self, PyObject *item, PyOb
 	}
 }
 
-//------------------------NUMERIC PROTOCOLS----------------------
-//------------------------obj + obj------------------------------
-//addition
+/* ------------------------NUMERIC PROTOCOLS---------------------- */
+/* ------------------------obj + obj------------------------------ */
+/* addition */
 static PyObject *Quaternion_add(PyObject *q1, PyObject *q2)
 {
 	float quat[QUAT_SIZE];
@@ -757,8 +755,8 @@ static PyObject *Quaternion_add(PyObject *q1, PyObject *q2)
 	add_qt_qtqt(quat, quat1->quat, quat2->quat, 1.0f);
 	return Quaternion_CreatePyObject(quat, Py_NEW, Py_TYPE(q1));
 }
-//------------------------obj - obj------------------------------
-//subtraction
+/* ------------------------obj - obj------------------------------ */
+/* subtraction */
 static PyObject *Quaternion_sub(PyObject *q1, PyObject *q2)
 {
 	int x;
@@ -794,8 +792,8 @@ static PyObject *quat_mul_float(QuaternionObject *quat, const float scalar)
 	return Quaternion_CreatePyObject(tquat, Py_NEW, Py_TYPE(quat));
 }
 
-//------------------------obj * obj------------------------------
-//mulplication
+/*------------------------obj * obj------------------------------
+ * multiplication */
 static PyObject *Quaternion_mul(PyObject *q1, PyObject *q2)
 {
 	float quat[QUAT_SIZE], scalar;
@@ -874,7 +872,7 @@ static PyObject *Quaternion_neg(QuaternionObject *self)
 }
 
 
-//-----------------PROTOCOL DECLARATIONS--------------------------
+/* -----------------PROTOCOL DECLARATIONS-------------------------- */
 static PySequenceMethods Quaternion_SeqMethods = {
 	(lenfunc) Quaternion_len,               /* sq_length */
 	(binaryfunc) NULL,                      /* sq_concat */
@@ -902,7 +900,7 @@ static PyNumberMethods Quaternion_NumMethods = {
 	NULL,                           /*nb_divmod*/
 	NULL,                           /*nb_power*/
 	(unaryfunc)     Quaternion_neg, /*nb_negative*/
-	(unaryfunc)     0,  /*tp_positive*/
+	(unaryfunc)     Quaternion_copy,/*tp_positive*/
 	(unaryfunc)     0,  /*tp_absolute*/
 	(inquiry)   0,      /*tp_bool*/
 	(unaryfunc) 0,      /*nb_invert*/
@@ -1059,7 +1057,7 @@ static int Quaternion_axis_vector_set(QuaternionObject *self, PyObject *value, v
 	return 0;
 }
 
-//----------------------------------mathutils.Quaternion() --------------
+/* ----------------------------------mathutils.Quaternion() -------------- */
 static PyObject *Quaternion_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
 	PyObject *seq = NULL;
@@ -1084,12 +1082,15 @@ static PyObject *Quaternion_new(PyTypeObject *type, PyObject *args, PyObject *kw
 				return NULL;
 			break;
 		case 2:
-			if (mathutils_array_parse(quat, 3, 3, seq, "mathutils.Quaternion()") == -1)
+		{
+			float axis[3];
+			if (mathutils_array_parse(axis, 3, 3, seq, "mathutils.Quaternion()") == -1)
 				return NULL;
 			angle = angle_wrap_rad(angle); /* clamp because of precision issues */
-			axis_angle_to_quat(quat, quat, angle);
+			axis_angle_to_quat(quat, axis, angle);
 			break;
 			/* PyArg_ParseTuple assures no more then 2 */
+		}
 	}
 	return Quaternion_CreatePyObject(quat, Py_NEW, type);
 }
@@ -1136,7 +1137,7 @@ static void quat__axis_angle_sanitize(float axis[3], float *angle)
 	}
 }
 
-//-----------------------METHOD DEFINITIONS ----------------------
+/* -----------------------METHOD DEFINITIONS ---------------------- */
 static struct PyMethodDef Quaternion_methods[] = {
 	/* in place only */
 	{"identity", (PyCFunction) Quaternion_identity, METH_NOARGS, Quaternion_identity_doc},
@@ -1186,60 +1187,64 @@ static PyGetSetDef Quaternion_getseters[] = {
 	{NULL, NULL, NULL, NULL, NULL}  /* Sentinel */
 };
 
-//------------------PY_OBECT DEFINITION--------------------------
+/* ------------------PY_OBECT DEFINITION-------------------------- */
 PyDoc_STRVAR(quaternion_doc,
 "This object gives access to Quaternions in Blender."
 );
 PyTypeObject quaternion_Type = {
 	PyVarObject_HEAD_INIT(NULL, 0)
-	"Quaternion",             //tp_name
-	sizeof(QuaternionObject),           //tp_basicsize
-	0,                                  //tp_itemsize
-	(destructor)BaseMathObject_dealloc, //tp_dealloc
-	NULL,                               //tp_print
-	NULL,                               //tp_getattr
-	NULL,                               //tp_setattr
-	NULL,                               //tp_compare
-	(reprfunc) Quaternion_repr,         //tp_repr
-	&Quaternion_NumMethods,             //tp_as_number
-	&Quaternion_SeqMethods,             //tp_as_sequence
-	&Quaternion_AsMapping,              //tp_as_mapping
-	NULL,                               //tp_hash
-	NULL,                               //tp_call
-	(reprfunc) Quaternion_str,          //tp_str
-	NULL,                               //tp_getattro
-	NULL,                               //tp_setattro
-	NULL,                               //tp_as_buffer
-	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC, //tp_flags
-	quaternion_doc, //tp_doc
-	(traverseproc)BaseMathObject_traverse,  //tp_traverse
-	(inquiry)BaseMathObject_clear,  //tp_clear
-	(richcmpfunc)Quaternion_richcmpr,   //tp_richcompare
-	0,                                  //tp_weaklistoffset
-	NULL,                               //tp_iter
-	NULL,                               //tp_iternext
-	Quaternion_methods,                 //tp_methods
-	NULL,                               //tp_members
-	Quaternion_getseters,               //tp_getset
-	NULL,                               //tp_base
-	NULL,                               //tp_dict
-	NULL,                               //tp_descr_get
-	NULL,                               //tp_descr_set
-	0,                                  //tp_dictoffset
-	NULL,                               //tp_init
-	NULL,                               //tp_alloc
-	Quaternion_new,                     //tp_new
-	NULL,                               //tp_free
-	NULL,                               //tp_is_gc
-	NULL,                               //tp_bases
-	NULL,                               //tp_mro
-	NULL,                               //tp_cache
-	NULL,                               //tp_subclasses
-	NULL,                               //tp_weaklist
-	NULL,                               //tp_del
+	"Quaternion",                       /* tp_name */
+	sizeof(QuaternionObject),           /* tp_basicsize */
+	0,                                  /* tp_itemsize */
+	(destructor)BaseMathObject_dealloc, /* tp_dealloc */
+	NULL,                               /* tp_print */
+	NULL,                               /* tp_getattr */
+	NULL,                               /* tp_setattr */
+	NULL,                               /* tp_compare */
+	(reprfunc) Quaternion_repr,         /* tp_repr */
+	&Quaternion_NumMethods,             /* tp_as_number */
+	&Quaternion_SeqMethods,             /* tp_as_sequence */
+	&Quaternion_AsMapping,              /* tp_as_mapping */
+	NULL,                               /* tp_hash */
+	NULL,                               /* tp_call */
+#ifndef MATH_STANDALONE
+	(reprfunc) Quaternion_str,          /* tp_str */
+#else
+	NULL,                               /* tp_str */
+#endif
+	NULL,                               /* tp_getattro */
+	NULL,                               /* tp_setattro */
+	NULL,                               /* tp_as_buffer */
+	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC, /* tp_flags */
+	quaternion_doc, /* tp_doc */
+	(traverseproc)BaseMathObject_traverse,  /* tp_traverse */
+	(inquiry)BaseMathObject_clear,  /* tp_clear */
+	(richcmpfunc)Quaternion_richcmpr,   /* tp_richcompare */
+	0,                                  /* tp_weaklistoffset */
+	NULL,                               /* tp_iter */
+	NULL,                               /* tp_iternext */
+	Quaternion_methods,                 /* tp_methods */
+	NULL,                               /* tp_members */
+	Quaternion_getseters,               /* tp_getset */
+	NULL,                               /* tp_base */
+	NULL,                               /* tp_dict */
+	NULL,                               /* tp_descr_get */
+	NULL,                               /* tp_descr_set */
+	0,                                  /* tp_dictoffset */
+	NULL,                               /* tp_init */
+	NULL,                               /* tp_alloc */
+	Quaternion_new,                     /* tp_new */
+	NULL,                               /* tp_free */
+	NULL,                               /* tp_is_gc */
+	NULL,                               /* tp_bases */
+	NULL,                               /* tp_mro */
+	NULL,                               /* tp_cache */
+	NULL,                               /* tp_subclasses */
+	NULL,                               /* tp_weaklist */
+	NULL,                               /* tp_del */
 };
-//------------------------Quaternion_CreatePyObject (internal)-------------
-//creates a new quaternion object
+/* ------------------------Quaternion_CreatePyObject (internal)------------- */
+/* creates a new quaternion object */
 /*pass Py_WRAP - if vector is a WRAPPER for data allocated by BLENDER
  * (i.e. it was allocated elsewhere by MEM_mallocN())
  *  pass Py_NEW - if vector is not a WRAPPER and managed by PYTHON
@@ -1262,7 +1267,7 @@ PyObject *Quaternion_CreatePyObject(float *quat, int type, PyTypeObject *base_ty
 		}
 		else if (type == Py_NEW) {
 			self->quat = PyMem_Malloc(QUAT_SIZE * sizeof(float));
-			if (!quat) { //new empty
+			if (!quat) {  /* new empty */
 				unit_qt(self->quat);
 			}
 			else {
