@@ -1,5 +1,4 @@
 /*
- *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -47,14 +46,12 @@
 #include "BLI_math.h"
 #include "BLI_utildefines.h"
 
-#define SWAP_FLOAT(a, b, tmp) tmp = a; a = b; b = tmp
-
 /*-------------------------DOC STRINGS ---------------------------*/
 PyDoc_STRVAR(M_Geometry_doc,
 "The Blender geometry module"
 );
 
-//---------------------------------INTERSECTION FUNCTIONS--------------------
+/* ---------------------------------INTERSECTION FUNCTIONS-------------------- */
 
 PyDoc_STRVAR(M_Geometry_intersect_ray_tri_doc,
 ".. function:: intersect_ray_tri(v1, v2, v3, ray, orig, clip=True)\n"
@@ -254,10 +251,86 @@ static PyObject *M_Geometry_intersect_line_line(PyObject *UNUSED(self), PyObject
 	}
 }
 
+/* Line-Line intersection using algorithm from mathworld.wolfram.com */
 
+PyDoc_STRVAR(M_Geometry_intersect_sphere_sphere_2d_doc,
+".. function:: intersect_sphere_sphere_2d(p_a, radius_a, p_b, radius_b)\n"
+"\n"
+"   Returns 2 points on between intersecting circles.\n"
+"\n"
+"   :arg p_a: Center of the first circle\n"
+"   :type p_a: :class:`mathutils.Vector`\n"
+"   :arg radius_a: Radius of the first circle\n"
+"   :type radius_a: float\n"
+"   :arg p_b: Center of the second circle\n"
+"   :type p_b: :class:`mathutils.Vector`\n"
+"   :arg radius_b: Radius of the second circle\n"
+"   :type radius_b: float\n"
+"   :rtype: tuple of :class:`mathutils.Vector`'s or None when there is no intersection\n"
+);
+static PyObject *M_Geometry_intersect_sphere_sphere_2d(PyObject *UNUSED(self), PyObject *args)
+{
+	PyObject *ret;
+	VectorObject *vec_a, *vec_b;
+	float *v_a, *v_b;
+	float rad_a, rad_b;
+	float v_ab[2];
+	float dist;
 
+	if (!PyArg_ParseTuple(args, "O!fO!f:intersect_sphere_sphere_2d",
+	                      &vector_Type, &vec_a, &rad_a,
+	                      &vector_Type, &vec_b, &rad_b))
+	{
+		return NULL;
+	}
+
+	if (BaseMath_ReadCallback(vec_a) == -1 ||
+	    BaseMath_ReadCallback(vec_b) == -1)
+	{
+		return NULL;
+	}
+
+	ret = PyTuple_New(2);
+
+	v_a = vec_a->vec;
+	v_b = vec_b->vec;
+
+	sub_v2_v2v2(v_ab, v_b, v_a);
+	dist = len_v2(v_ab);
+
+	if (/* out of range */
+	    (dist > rad_a + rad_b) ||
+	    /* fully-contained in the other */
+	    (dist < abs(rad_a - rad_b)) ||
+	    /* co-incident */
+	    (dist < FLT_EPSILON))
+	{
+		/* out of range */
+		PyTuple_SET_ITEM(ret, 0,  Py_None); Py_INCREF(Py_None);
+		PyTuple_SET_ITEM(ret, 1,  Py_None); Py_INCREF(Py_None);
+	}
+	else {
+		const float dist_delta = ((rad_a * rad_a) - (rad_b * rad_b) + (dist * dist)) / (2.0f * dist);
+		const float h = powf(fabsf((rad_a * rad_a) - (dist_delta * dist_delta)), 0.5f);
+		float i_cent[2];
+		float i1[2], i2[2];
+
+		i_cent[0] = v_a[0] + ((v_ab[0] * dist_delta) / dist);
+		i_cent[1] = v_a[1] + ((v_ab[1] * dist_delta) / dist);
+
+		i1[0] = i_cent[0] + h * v_ab[1] / dist;
+		i1[1] = i_cent[1] - h * v_ab[0] / dist;
+
+		i2[0] = i_cent[0] - h * v_ab[1] / dist;
+		i2[1] = i_cent[1] + h * v_ab[0] / dist;
+
+		PyTuple_SET_ITEM(ret, 0, Vector_CreatePyObject(i1, 2, Py_NEW, NULL));
+		PyTuple_SET_ITEM(ret, 1, Vector_CreatePyObject(i2, 2, Py_NEW, NULL));
+	}
+
+	return ret;
+}
 
-//----------------------------geometry.normal() -------------------
 PyDoc_STRVAR(M_Geometry_normal_doc,
 ".. function:: normal(v1, v2, v3, v4=None)\n"
 "\n"
@@ -341,7 +414,7 @@ static PyObject *M_Geometry_normal(PyObject *UNUSED(self), PyObject *args)
 	return Vector_CreatePyObject(n, 3, Py_NEW, NULL);
 }
 
-//--------------------------------- AREA FUNCTIONS--------------------
+/* --------------------------------- AREA FUNCTIONS-------------------- */
 
 PyDoc_STRVAR(M_Geometry_area_tri_doc,
 ".. function:: area_tri(v1, v2, v3)\n"
@@ -729,6 +802,7 @@ static PyObject *M_Geometry_intersect_point_line(PyObject *UNUSED(self), PyObjec
 	float pt_in[3], pt_out[3], l1[3], l2[3];
 	float lambda;
 	PyObject *ret;
+	int size = 2;
 	
 	if (!PyArg_ParseTuple(args, "O!O!O!:intersect_point_line",
 	                      &vector_Type, &pt,
@@ -746,20 +820,20 @@ static PyObject *M_Geometry_intersect_point_line(PyObject *UNUSED(self), PyObjec
 	}
 
 	/* accept 2d verts */
-	if (pt->size == 3) {     copy_v3_v3(pt_in, pt->vec); }
-	else { pt_in[2] = 0.0f;  copy_v2_v2(pt_in, pt->vec); }
+	if (pt->size >= 3)     { copy_v3_v3(pt_in, pt->vec); size = 3; }
+	else                   { copy_v2_v2(pt_in, pt->vec); pt_in[2] = 0.0f; }
 	
-	if (line_1->size == 3) { copy_v3_v3(l1, line_1->vec); }
-	else { l1[2] = 0.0f;     copy_v2_v2(l1, line_1->vec); }
+	if (line_1->size >= 3) { copy_v3_v3(l1, line_1->vec); size = 3; }
+	else                   { copy_v2_v2(l1, line_1->vec); l1[2] = 0.0f; }
 	
-	if (line_2->size == 3) { copy_v3_v3(l2, line_2->vec); }
-	else { l2[2] = 0.0f;     copy_v2_v2(l2, line_2->vec); }
+	if (line_2->size >= 3) { copy_v3_v3(l2, line_2->vec); size = 3; }
+	else                   { copy_v2_v2(l2, line_2->vec); l2[2] = 0.0f; }
 	
 	/* do the calculation */
 	lambda = closest_to_line_v3(pt_out, pt_in, l1, l2);
 	
 	ret = PyTuple_New(2);
-	PyTuple_SET_ITEM(ret, 0, Vector_CreatePyObject(pt_out, 3, Py_NEW, NULL));
+	PyTuple_SET_ITEM(ret, 0, Vector_CreatePyObject(pt_out, size, Py_NEW, NULL));
 	PyTuple_SET_ITEM(ret, 1, PyFloat_FromDouble(lambda));
 	return ret;
 }
@@ -808,6 +882,7 @@ PyDoc_STRVAR(M_Geometry_intersect_point_quad_2d_doc,
 "\n"
 "   Takes 5 vectors (using only the x and y coordinates): one is the point and the next 4 define the quad, \n"
 "   only the x and y are used from the vectors. Returns 1 if the point is within the quad, otherwise 0.\n"
+"   Works only with convex quads without singular edges."
 "\n"
 "   :arg pt: Point\n"
 "   :type pt: :class:`mathutils.Vector`\n"
@@ -944,6 +1019,111 @@ static PyObject *M_Geometry_barycentric_transform(PyObject *UNUSED(self), PyObje
 	return Vector_CreatePyObject(vec, 3, Py_NEW, NULL);
 }
 
+PyDoc_STRVAR(M_Geometry_points_in_planes_doc,
+".. function:: points_in_planes(planes)\n"
+"\n"
+"   Returns a list of points inside all planes given and a list of index values for the planes used.\n"
+"\n"
+"   :arg planes: List of planes (4D vectors).\n"
+"   :type planes: list of :class:`mathutils.Vector`\n"
+"   :return: two lists, once containing the vertices inside the planes, another containing the plane indicies used\n"
+"   :rtype: pair of lists\n"
+);
+/* note: this function could be optimized by some spatial structure */
+static PyObject *M_Geometry_points_in_planes(PyObject *UNUSED(self), PyObject *args)
+{
+	PyObject *py_planes;
+	float (*planes)[4];
+	unsigned int planes_len;
+
+	if (!PyArg_ParseTuple(args, "O:points_in_planes",
+	                      &py_planes))
+	{
+		return NULL;
+	}
+
+	if ((planes_len = mathutils_array_parse_alloc_v((float **)&planes, 4, py_planes, "points_in_planes")) == -1) {
+		return NULL;
+	}
+	else {
+		/* note, this could be refactored into plain C easy - py bits are noted */
+		const float eps = 0.0001f;
+		const unsigned int len = (unsigned int)planes_len;
+		unsigned int i, j, k, l;
+
+		float n1n2[3], n2n3[3], n3n1[3];
+		float potentialVertex[3];
+		char *planes_used = PyMem_Malloc(sizeof(char) * len);
+
+		/* python */
+		PyObject *py_verts = PyList_New(0);
+		PyObject *py_plene_index = PyList_New(0);
+
+		memset(planes_used, 0, sizeof(char) * len);
+
+		for (i = 0; i < len; i++) {
+			const float *N1 = planes[i];
+			for (j = i + 1; j < len; j++) {
+				const float *N2 = planes[j];
+				cross_v3_v3v3(n1n2, N1, N2);
+				if (len_squared_v3(n1n2) > eps) {
+					for (k = j + 1; k < len; k++) {
+						const float *N3 = planes[k];
+						cross_v3_v3v3(n2n3, N2, N3);
+						if (len_squared_v3(n2n3) > eps) {
+							cross_v3_v3v3(n3n1, N3, N1);
+							if (len_squared_v3(n3n1) > eps) {
+								const float quotient = dot_v3v3(N1, n2n3);
+								if (fabsf(quotient) > eps) {
+									/* potentialVertex = (n2n3 * N1[3] + n3n1 * N2[3] + n1n2 * N3[3]) * (-1.0 / quotient); */
+									const float quotient_ninv = -1.0f / quotient;
+									potentialVertex[0] = ((n2n3[0] * N1[3]) + (n3n1[0] * N2[3]) + (n1n2[0] * N3[3])) * quotient_ninv;
+									potentialVertex[1] = ((n2n3[1] * N1[3]) + (n3n1[1] * N2[3]) + (n1n2[1] * N3[3])) * quotient_ninv;
+									potentialVertex[2] = ((n2n3[2] * N1[3]) + (n3n1[2] * N2[3]) + (n1n2[2] * N3[3])) * quotient_ninv;
+									for (l = 0; l < len; l++) {
+										const float *NP = planes[l];
+										if ((dot_v3v3(NP, potentialVertex) + NP[3]) > 0.000001f) {
+											break;
+										}
+									}
+
+									if (l == len) { /* ok */
+										/* python */
+										PyObject *item = Vector_CreatePyObject(potentialVertex, 3, Py_NEW, NULL);
+										PyList_Append(py_verts, item);
+										Py_DECREF(item);
+
+										planes_used[i] = planes_used[j] = planes_used[k] = TRUE;
+									}
+								}
+							}
+						}
+					}
+				}
+			}
+		}
+
+		PyMem_Free(planes);
+
+		/* now make a list of used planes */
+		for (i = 0; i < len; i++) {
+			if (planes_used[i]) {
+				PyObject *item = PyLong_FromLong(i);
+				PyList_Append(py_plene_index, item);
+				Py_DECREF(item);
+			}
+		}
+		PyMem_Free(planes_used);
+
+		{
+			PyObject *ret = PyTuple_New(2);
+			PyTuple_SET_ITEM(ret, 0, py_verts);
+			PyTuple_SET_ITEM(ret, 1, py_plene_index);
+			return ret;
+		}
+	}
+}
+
 #ifndef MATH_STANDALONE
 
 PyDoc_STRVAR(M_Geometry_interpolate_bezier_doc,
@@ -1002,7 +1182,7 @@ static PyObject *M_Geometry_interpolate_bezier(PyObject *UNUSED(self), PyObject
 		return NULL;
 	}
 
-	dims = MAX4(vec_k1->size, vec_h1->size, vec_h2->size, vec_k2->size);
+	dims = max_iiii(vec_k1->size, vec_h1->size, vec_h2->size, vec_k2->size);
 
 	for (i = 0; i < vec_k1->size; i++) k1[i] = vec_k1->vec[i];
 	for (i = 0; i < vec_h1->size; i++) h1[i] = vec_h1->vec[i];
@@ -1011,7 +1191,7 @@ static PyObject *M_Geometry_interpolate_bezier(PyObject *UNUSED(self), PyObject
 
 	coord_array = MEM_callocN(dims * (resolu) * sizeof(float), "interpolate_bezier");
 	for (i = 0; i < dims; i++) {
-		forward_diff_bezier(k1[i], h1[i], h2[i], k2[i], coord_array + i, resolu - 1, sizeof(float) * dims);
+		BKE_curve_forward_diff_bezier(k1[i], h1[i], h2[i], k2[i], coord_array + i, resolu - 1, sizeof(float) * dims);
 	}
 
 	list = PyList_New(resolu);
@@ -1056,7 +1236,7 @@ static PyObject *M_Geometry_tessellate_polygon(PyObject *UNUSED(self), PyObject
 	for (i = 0; i < len_polylines; i++) {
 		polyLine = PySequence_GetItem(polyLineSeq, i);
 		if (!PySequence_Check(polyLine)) {
-			freedisplist(&dispbase);
+			BKE_displist_free(&dispbase);
 			Py_XDECREF(polyLine); /* may be null so use Py_XDECREF*/
 			PyErr_SetString(PyExc_TypeError,
 			                "One or more of the polylines is not a sequence of mathutils.Vector's");
@@ -1110,7 +1290,7 @@ static PyObject *M_Geometry_tessellate_polygon(PyObject *UNUSED(self), PyObject
 	}
 
 	if (ls_error) {
-		freedisplist(&dispbase); /* possible some dl was allocated */
+		BKE_displist_free(&dispbase); /* possible some dl was allocated */
 		PyErr_SetString(PyExc_TypeError,
 		                "A point in one of the polylines "
 		                "is not a mathutils.Vector type");
@@ -1118,7 +1298,7 @@ static PyObject *M_Geometry_tessellate_polygon(PyObject *UNUSED(self), PyObject
 	}
 	else if (totpoints) {
 		/* now make the list to return */
-		filldisplist(&dispbase, &dispbase, 0);
+		BKE_displist_fill(&dispbase, &dispbase, 0);
 
 		/* The faces are stored in a new DisplayList
 		 * thats added to the head of the listbase */
@@ -1126,7 +1306,7 @@ static PyObject *M_Geometry_tessellate_polygon(PyObject *UNUSED(self), PyObject
 
 		tri_list = PyList_New(dl->parts);
 		if (!tri_list) {
-			freedisplist(&dispbase);
+			BKE_displist_free(&dispbase);
 			PyErr_SetString(PyExc_RuntimeError,
 			                "failed to make a new list");
 			return NULL;
@@ -1139,11 +1319,11 @@ static PyObject *M_Geometry_tessellate_polygon(PyObject *UNUSED(self), PyObject
 			dl_face += 3;
 			index++;
 		}
-		freedisplist(&dispbase);
+		BKE_displist_free(&dispbase);
 	}
 	else {
 		/* no points, do this so scripts don't barf */
-		freedisplist(&dispbase); /* possible some dl was allocated */
+		BKE_displist_free(&dispbase); /* possible some dl was allocated */
 		tri_list = PyList_New(0);
 	}
 
@@ -1151,11 +1331,11 @@ static PyObject *M_Geometry_tessellate_polygon(PyObject *UNUSED(self), PyObject
 }
 
 
-static int boxPack_FromPyObject(PyObject *value, boxPack **boxarray)
+static int boxPack_FromPyObject(PyObject *value, BoxPack **boxarray)
 {
 	Py_ssize_t len, i;
 	PyObject *list_item, *item_1, *item_2;
-	boxPack *box;
+	BoxPack *box;
 
 
 	/* Error checking must already be done */
@@ -1167,7 +1347,7 @@ static int boxPack_FromPyObject(PyObject *value, boxPack **boxarray)
 
 	len = PyList_GET_SIZE(value);
 
-	*boxarray = MEM_mallocN(len * sizeof(boxPack), "boxPack box");
+	*boxarray = MEM_mallocN(len * sizeof(BoxPack), "BoxPack box");
 
 
 	for (i = 0; i < len; i++) {
@@ -1202,11 +1382,11 @@ static int boxPack_FromPyObject(PyObject *value, boxPack **boxarray)
 	return 0;
 }
 
-static void boxPack_ToPyObject(PyObject *value, boxPack **boxarray)
+static void boxPack_ToPyObject(PyObject *value, BoxPack **boxarray)
 {
 	Py_ssize_t len, i;
 	PyObject *list_item;
-	boxPack *box;
+	BoxPack *box;
 
 	len = PyList_GET_SIZE(value);
 
@@ -1244,13 +1424,13 @@ static PyObject *M_Geometry_box_pack_2d(PyObject *UNUSED(self), PyObject *boxlis
 
 	len = PyList_GET_SIZE(boxlist);
 	if (len) {
-		boxPack *boxarray = NULL;
+		BoxPack *boxarray = NULL;
 		if (boxPack_FromPyObject(boxlist, &boxarray) == -1) {
 			return NULL; /* exception set */
 		}
 
 		/* Non Python function */
-		boxPack2D(boxarray, len, &tot_width, &tot_height);
+		BLI_box_pack_2D(boxarray, len, &tot_width, &tot_height);
 
 		boxPack_ToPyObject(boxlist, &boxarray);
 	}
@@ -1276,9 +1456,11 @@ static PyMethodDef M_Geometry_methods[] = {
 	{"intersect_line_sphere", (PyCFunction) M_Geometry_intersect_line_sphere, METH_VARARGS, M_Geometry_intersect_line_sphere_doc},
 	{"intersect_line_sphere_2d", (PyCFunction) M_Geometry_intersect_line_sphere_2d, METH_VARARGS, M_Geometry_intersect_line_sphere_2d_doc},
 	{"distance_point_to_plane", (PyCFunction) M_Geometry_distance_point_to_plane, METH_VARARGS, M_Geometry_distance_point_to_plane_doc},
+	{"intersect_sphere_sphere_2d", (PyCFunction) M_Geometry_intersect_sphere_sphere_2d, METH_VARARGS, M_Geometry_intersect_sphere_sphere_2d_doc},
 	{"area_tri", (PyCFunction) M_Geometry_area_tri, METH_VARARGS, M_Geometry_area_tri_doc},
 	{"normal", (PyCFunction) M_Geometry_normal, METH_VARARGS, M_Geometry_normal_doc},
 	{"barycentric_transform", (PyCFunction) M_Geometry_barycentric_transform, METH_VARARGS, M_Geometry_barycentric_transform_doc},
+	{"points_in_planes", (PyCFunction) M_Geometry_points_in_planes, METH_VARARGS, M_Geometry_points_in_planes_doc},
 #ifndef MATH_STANDALONE
 	{"interpolate_bezier", (PyCFunction) M_Geometry_interpolate_bezier, METH_VARARGS, M_Geometry_interpolate_bezier_doc},
 	{"tessellate_polygon", (PyCFunction) M_Geometry_tessellate_polygon, METH_O, M_Geometry_tessellate_polygon_doc},
