@@ -61,8 +61,8 @@
 #import <QTKit/QTKit.h>
 #include <AudioToolbox/AudioToolbox.h>
 
-#if (MAC_OS_X_VERSION_MIN_REQUIRED <= MAC_OS_X_VERSION_10_4) || !__LP64__
-#error 64 bit build & OSX 10.5 minimum are needed for QTKit
+#if MAC_OS_X_VERSION_MIN_REQUIRED <= 1040
+#error OSX 10.5 minimum is needed for QTKit
 #endif
 
 #include "quicktime_import.h"
@@ -88,8 +88,8 @@ typedef struct QuicktimeExport {
 	AudioStreamBasicDescription audioInputFormat, audioOutputFormat;
 	AudioStreamPacketDescription *audioOutputPktDesc;
 	SInt64 audioFilePos;
-	char* audioInputBuffer;
-	char* audioOutputBuffer;
+	char *audioInputBuffer;
+	char *audioOutputBuffer;
 	UInt32 audioCodecMaxOutputPacketSize;
 	UInt64 audioTotalExportedFrames, audioTotalSavedFrames;
 	UInt64 audioLastFrame;
@@ -143,7 +143,7 @@ QuicktimeCodecTypeDesc* quicktime_get_videocodecType_desc(int indexValue)
 int quicktime_rnatmpvalue_from_videocodectype(int codecType)
 {
 	int i;
-	for (i=0;i<qtVideoCodecCount;i++) {
+	for (i = 0; i < qtVideoCodecCount; i++) {
 		if (qtVideoCodecList[i].codecType == codecType)
 			return qtVideoCodecList[i].rnatmpvalue;
 	}
@@ -154,12 +154,12 @@ int quicktime_rnatmpvalue_from_videocodectype(int codecType)
 int quicktime_videocodecType_from_rnatmpvalue(int rnatmpvalue)
 {
 	int i;
-	for (i=0;i<qtVideoCodecCount;i++) {
+	for (i = 0; i < qtVideoCodecCount; i++) {
 		if (qtVideoCodecList[i].rnatmpvalue == rnatmpvalue)
 			return qtVideoCodecList[i].codecType;
 	}
 	
-	return 0;	
+	return 0;
 }
 
 /* Audio codec */
@@ -188,7 +188,7 @@ QuicktimeCodecTypeDesc* quicktime_get_audiocodecType_desc(int indexValue)
 int quicktime_rnatmpvalue_from_audiocodectype(int codecType)
 {
 	int i;
-	for (i=0;i<qtAudioCodecCount;i++) {
+	for (i = 0; i < qtAudioCodecCount; i++) {
 		if (qtAudioCodecList[i].codecType == codecType)
 			return qtAudioCodecList[i].rnatmpvalue;
 	}
@@ -199,12 +199,12 @@ int quicktime_rnatmpvalue_from_audiocodectype(int codecType)
 int quicktime_audiocodecType_from_rnatmpvalue(int rnatmpvalue)
 {
 	int i;
-	for (i=0;i<qtAudioCodecCount;i++) {
+	for (i = 0; i < qtAudioCodecCount; i++) {
 		if (qtAudioCodecList[i].rnatmpvalue == rnatmpvalue)
 			return qtAudioCodecList[i].codecType;
 	}
 	
-	return 0;	
+	return 0;
 }
 
 
@@ -212,7 +212,7 @@ static NSString *stringWithCodecType(int codecType)
 {
 	char str[5];
 	
-	*((int*)str) = EndianU32_NtoB(codecType);
+	*((int *)str) = EndianU32_NtoB(codecType);
 	str[4] = 0;
 	
 	return [NSString stringWithCString:str encoding:NSASCIIStringEncoding];
@@ -235,7 +235,7 @@ void makeqtstring (RenderData *rd, char *string)
 
 void filepath_qt(char *string, RenderData *rd)
 {
-	if (string==NULL) return;
+	if (string == NULL) return;
 	
 	strcpy(string, rd->pic);
 	BLI_path_abs(string, G.main->name);
@@ -313,22 +313,22 @@ int start_qt(struct Scene *scene, struct RenderData *rd, int rectx, int recty, R
 	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 	NSError *error;
 	char name[1024];
-	int success= 1;
-	OSStatus err=noErr;
+	int success = 1;
+	OSStatus err = noErr;
 
 	if(qtexport == NULL) qtexport = MEM_callocN(sizeof(QuicktimeExport), "QuicktimeExport");
 	
-	[QTMovie enterQTKitOnThread];		
+	[QTMovie enterQTKitOnThread];
 	
 	/* Check first if the QuickTime 7.2.1 initToWritableFile: method is available */
 	if ([[[[QTMovie alloc] init] autorelease] respondsToSelector:@selector(initToWritableFile:error:)] != YES) {
 		BKE_report(reports, RPT_ERROR, "\nUnable to create quicktime movie, need Quicktime rev 7.2.1 or later");
-		success= 0;
+		success = 0;
 	}
 	else {
 		makeqtstring(rd, name);
 		qtexport->filename = [[NSString alloc] initWithCString:name
-								  encoding:[NSString defaultCStringEncoding]];
+		                                                       encoding:[NSString defaultCStringEncoding]];
 		qtexport->movie = nil;
 		qtexport->audioFile = NULL;
 
@@ -354,7 +354,7 @@ int start_qt(struct Scene *scene, struct RenderData *rd, int rectx, int recty, R
 					strcpy(extension,".aiff");
 					break;
 			}
-					
+
 			tmpnam(name);
 			strcat(name, extension);
 			outputFileURL = CFURLCreateFromFileSystemRepresentation(kCFAllocatorDefault,(UInt8*) name, strlen(name), false);
@@ -362,7 +362,7 @@ int start_qt(struct Scene *scene, struct RenderData *rd, int rectx, int recty, R
 			if (outputFileURL) {
 				
 				qtexport->audioFileName = [[NSString alloc] initWithCString:name
-															 encoding:[NSString defaultCStringEncoding]];
+				                                                            encoding:[NSString defaultCStringEncoding]];
 				
 				qtexport->audioInputFormat.mSampleRate = U.audiorate;
 				qtexport->audioInputFormat.mFormatID = kAudioFormatLinearPCM;
@@ -400,13 +400,13 @@ int start_qt(struct Scene *scene, struct RenderData *rd, int rectx, int recty, R
 				qtexport->audioInputFormat.mFormatFlags |= kLinearPCMFormatFlagIsPacked;
 				
 				
-				/*Ouput format*/
+				/*Output format*/
 				qtexport->audioOutputFormat.mFormatID = rd->qtcodecsettings.audiocodecType;
 				//TODO: set audio channels
 				qtexport->audioOutputFormat.mChannelsPerFrame = 2;
 				qtexport->audioOutputFormat.mSampleRate = rd->qtcodecsettings.audioSampleRate;
 				
-				/* Default value for compressed formats, overriden after if not the case */
+				/* Default value for compressed formats, overridden after if not the case */
 				qtexport->audioOutputFormat.mFramesPerPacket = 0;
 				qtexport->audioOutputFormat.mBytesPerFrame = 0;
 				qtexport->audioOutputFormat.mBytesPerPacket = 0;
@@ -471,7 +471,7 @@ int start_qt(struct Scene *scene, struct RenderData *rd, int rectx, int recty, R
 						qtexport->audioOutputFormat.mBytesPerFrame = qtexport->audioOutputFormat.mBytesPerPacket;
 						break;
 				}
-												
+
 				err = AudioFileCreateWithURL(outputFileURL, audioFileType, &qtexport->audioOutputFormat, kAudioFileFlags_EraseFile, &qtexport->audioFile);
 				CFRelease(outputFileURL);
 				
@@ -489,27 +489,27 @@ int start_qt(struct Scene *scene, struct RenderData *rd, int rectx, int recty, R
 					else {
 						UInt32 prop,propSize;
 						/* Set up codec properties */
-						if (rd->qtcodecsettings.audiocodecType == kAudioFormatMPEG4AAC) { /*Lossy compressed format*/
+						if (rd->qtcodecsettings.audiocodecType == kAudioFormatMPEG4AAC) {  /* Lossy compressed format */
 							prop = rd->qtcodecsettings.audioBitRate;
 							AudioConverterSetProperty(qtexport->audioConverter, kAudioConverterEncodeBitRate,
-													  sizeof(prop), &prop);
+							                          sizeof(prop), &prop);
 							
 							if (rd->qtcodecsettings.audioCodecFlags & QTAUDIO_FLAG_CODEC_ISCBR)
 								prop = kAudioCodecBitRateControlMode_Constant;
 							else
 								prop = kAudioCodecBitRateControlMode_LongTermAverage;
 							AudioConverterSetProperty(qtexport->audioConverter, kAudioCodecPropertyBitRateControlMode,
-															sizeof(prop), &prop);
+							                          sizeof(prop), &prop);
 						}
 						/* Conversion quality : if performance impact then offer degraded option */
-						if ((rd->qtcodecsettings.audioCodecFlags & QTAUDIO_FLAG_RESAMPLE_NOHQ) == 0) {							
+						if ((rd->qtcodecsettings.audioCodecFlags & QTAUDIO_FLAG_RESAMPLE_NOHQ) == 0) {
 							prop = kAudioConverterSampleRateConverterComplexity_Mastering;
 							AudioConverterSetProperty(qtexport->audioConverter, kAudioConverterSampleRateConverterComplexity,
-													  sizeof(prop), &prop);
+							                          sizeof(prop), &prop);
 							
 							prop = kAudioConverterQuality_Max;
 							AudioConverterSetProperty(qtexport->audioConverter, kAudioConverterSampleRateConverterQuality,
-													  sizeof(prop), &prop);
+							                          sizeof(prop), &prop);
 						}
 						
 						write_cookie(qtexport->audioConverter, qtexport->audioFile);
@@ -517,32 +517,33 @@ int start_qt(struct Scene *scene, struct RenderData *rd, int rectx, int recty, R
 						/* Allocate output buffer */
 						if (qtexport->audioOutputFormat.mBytesPerPacket ==0) /* VBR */
 							AudioConverterGetProperty(qtexport->audioConverter, kAudioConverterPropertyMaximumOutputPacketSize,
-												  &propSize, &qtexport->audioCodecMaxOutputPacketSize);
+							                          &propSize, &qtexport->audioCodecMaxOutputPacketSize);
 						else
 							qtexport->audioCodecMaxOutputPacketSize = qtexport->audioOutputFormat.mBytesPerPacket;
 						
 						qtexport->audioInputBuffer = MEM_mallocN(AUDIOOUTPUTBUFFERSIZE, "qt_audio_inputPacket");
 						qtexport->audioOutputBuffer = MEM_mallocN(AUDIOOUTPUTBUFFERSIZE, "qt_audio_outputPacket");
-						qtexport->audioOutputPktDesc = MEM_mallocN(sizeof(AudioStreamPacketDescription)*AUDIOOUTPUTBUFFERSIZE/qtexport->audioCodecMaxOutputPacketSize,
-																   "qt_audio_pktdesc");
+						qtexport->audioOutputPktDesc = MEM_mallocN(sizeof(AudioStreamPacketDescription) * AUDIOOUTPUTBUFFERSIZE / qtexport->audioCodecMaxOutputPacketSize,
+						                                           "qt_audio_pktdesc");
 					}
 				}
 			}
-			
+
 			if (err == noErr) {
-				qtexport->videoTempFileName = [[NSString alloc] initWithCString:tmpnam(nil) 
-															 encoding:[NSString defaultCStringEncoding]];			
-				if (qtexport->videoTempFileName)
+				qtexport->videoTempFileName = [[NSString alloc] initWithCString:tmpnam(nil)
+				                                                encoding:[NSString defaultCStringEncoding]];
+				if (qtexport->videoTempFileName) {
 					qtexport->movie = [[QTMovie alloc] initToWritableFile:qtexport->videoTempFileName error:&error];
+				}
 
 			}
 		}
 		else
 			qtexport->movie = [[QTMovie alloc] initToWritableFile:qtexport->filename error:&error];
-			
+
 		if(qtexport->movie == nil) {
 			BKE_report(reports, RPT_ERROR, "Unable to create quicktime movie.");
-			success= 0;
+			success = 0;
 			if (qtexport->filename) [qtexport->filename release];
 			qtexport->filename = nil;
 			if (qtexport->audioFileName) [qtexport->audioFileName release];
@@ -556,23 +557,25 @@ int start_qt(struct Scene *scene, struct RenderData *rd, int rectx, int recty, R
 			[qtexport->movie setAttribute:[NSNumber numberWithBool:YES] forKey:QTMovieEditableAttribute];
 			[qtexport->movie setAttribute:@"Made with Blender" forKey:QTMovieCopyrightAttribute];
 			
-			qtexport->frameDuration = QTMakeTime(rd->frs_sec_base*1000, rd->frs_sec*1000);
+			qtexport->frameDuration = QTMakeTime(rd->frs_sec_base * 1000, rd->frs_sec * 1000);
 			
 			/* specifying the codec attributes : try to retrieve them from render data first*/
 			if (rd->qtcodecsettings.codecType) {
-				qtexport->frameAttributes = [NSDictionary dictionaryWithObjectsAndKeys:
-											 stringWithCodecType(rd->qtcodecsettings.codecType),
-											 QTAddImageCodecType,
-											 [NSNumber numberWithLong:((rd->qtcodecsettings.codecSpatialQuality)*codecLosslessQuality)/100],
-											 QTAddImageCodecQuality,
-											 nil];
+				qtexport->frameAttributes = [
+				        NSDictionary dictionaryWithObjectsAndKeys:
+				        stringWithCodecType(rd->qtcodecsettings.codecType),
+				        QTAddImageCodecType,
+				        [NSNumber numberWithLong:((rd->qtcodecsettings.codecSpatialQuality)*codecLosslessQuality)/100],
+				        QTAddImageCodecQuality,
+				        nil];
 			}
 			else {
-				qtexport->frameAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"jpeg",
-											 QTAddImageCodecType,
-											 [NSNumber numberWithLong:codecHighQuality],
-											 QTAddImageCodecQuality,
-											 nil];
+				qtexport->frameAttributes = [
+				        NSDictionary dictionaryWithObjectsAndKeys:@"jpeg",
+				        QTAddImageCodecType,
+				        [NSNumber numberWithLong:codecHighQuality],
+				        QTAddImageCodecQuality,
+				        nil];
 			}
 			[qtexport->frameAttributes retain];
 			
@@ -585,7 +588,7 @@ int start_qt(struct Scene *scene, struct RenderData *rd, int rectx, int recty, R
 				specs.rate = U.audiorate;
 				qtexport->audioInputDevice = AUD_openReadDevice(specs);
 				AUD_playDevice(qtexport->audioInputDevice, scene->sound_scene, rd->sfra * rd->frs_sec_base / rd->frs_sec);
-								
+
 				qtexport->audioOutputPktPos = 0;
 				qtexport->audioTotalExportedFrames = 0;
 				qtexport->audioTotalSavedFrames = 0;
@@ -623,8 +626,8 @@ int append_qt(struct RenderData *rd, int start_frame, int frame, int *pixels, in
 		return 0;
 	}
 	
-	from_Ptr = (unsigned char*)pixels;
-	to_Ptr = (unsigned char*)[blBitmapFormatImage bitmapData];
+	from_Ptr = (unsigned char *)pixels;
+	to_Ptr = (unsigned char *)[blBitmapFormatImage bitmapData];
 	for (y = 0; y < recty; y++) {
 		to_i = (recty-y-1)*rectx;
 		from_i = y*rectx;
@@ -646,7 +649,7 @@ int append_qt(struct RenderData *rd, int start_frame, int frame, int *pixels, in
 	if (qtexport->audioFile) {
 		UInt32 audioPacketsConverted;
 		/* Append audio */
-		while (qtexport->audioTotalExportedFrames < qtexport->audioLastFrame) {	
+		while (qtexport->audioTotalExportedFrames < qtexport->audioLastFrame) {
 
 			qtexport->audioBufferList.mNumberBuffers = 1;
 			qtexport->audioBufferList.mBuffers[0].mNumberChannels = qtexport->audioOutputFormat.mChannelsPerFrame;
@@ -655,13 +658,13 @@ int append_qt(struct RenderData *rd, int start_frame, int frame, int *pixels, in
 			audioPacketsConverted = AUDIOOUTPUTBUFFERSIZE / qtexport->audioCodecMaxOutputPacketSize;
 			
 			err = AudioConverterFillComplexBuffer(qtexport->audioConverter, AudioConverterInputCallback,
-											NULL, &audioPacketsConverted, &qtexport->audioBufferList, qtexport->audioOutputPktDesc);
+			                                      NULL, &audioPacketsConverted, &qtexport->audioBufferList, qtexport->audioOutputPktDesc);
 			if (audioPacketsConverted) {
 				AudioFileWritePackets(qtexport->audioFile, false, qtexport->audioBufferList.mBuffers[0].mDataByteSize,
-									  qtexport->audioOutputPktDesc, qtexport->audioOutputPktPos, &audioPacketsConverted, qtexport->audioOutputBuffer);
+				        qtexport->audioOutputPktDesc, qtexport->audioOutputPktPos, &audioPacketsConverted, qtexport->audioOutputBuffer);
 				qtexport->audioOutputPktPos += audioPacketsConverted;
 				
-				if (qtexport->audioOutputFormat.mFramesPerPacket) { 
+				if (qtexport->audioOutputFormat.mFramesPerPacket) {
 					// this is the common case: format has constant frames per packet
 					qtexport->audioTotalSavedFrames += (audioPacketsConverted * qtexport->audioOutputFormat.mFramesPerPacket);
 				}
@@ -682,7 +685,7 @@ int append_qt(struct RenderData *rd, int start_frame, int frame, int *pixels, in
 
 		}
 	}
-	[pool drain];	
+	[pool drain];
 
 	return 1;
 }
@@ -751,8 +754,8 @@ void end_qt(void)
 			}
 			
 			/* Save file */
-			dict = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] 
-											   forKey:QTMovieFlatten];
+			dict = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES]
+			        forKey:QTMovieFlatten];
 
 			if (dict) {
 				[qtexport->movie writeToFile:qtexport->filename withAttributes:dict];
