@@ -35,6 +35,8 @@
 #include "DNA_listBase.h"
 #include "RNA_types.h"
 
+struct bNode;
+struct bNodeTree;
 struct Object;
 struct Render;
 struct RenderEngine;
@@ -58,6 +60,9 @@ struct Scene;
 #define RE_ENGINE_PREVIEW		2
 #define RE_ENGINE_DO_DRAW		4
 #define RE_ENGINE_DO_UPDATE		8
+#define RE_ENGINE_RENDERING		16
+#define RE_ENGINE_HIGHLIGHT_TILES	32
+#define RE_ENGINE_USED_FOR_VIEWPORT	64
 
 extern ListBase R_engines;
 
@@ -75,6 +80,8 @@ typedef struct RenderEngineType {
 	void (*view_update)(struct RenderEngine *engine, const struct bContext *context);
 	void (*view_draw)(struct RenderEngine *engine, const struct bContext *context);
 
+	void (*update_script_node)(struct RenderEngine *engine, struct bNodeTree *ntree, struct bNode *node);
+
 	/* RNA integration */
 	ExtensionRNA ext;
 } RenderEngineType;
@@ -86,24 +93,33 @@ typedef struct RenderEngine {
 	int flag;
 	struct Object *camera_override;
 
+	int tile_x;
+	int tile_y;
+
 	struct Render *re;
 	ListBase fullresult;
 	char *text;
+
+	int resolution_x, resolution_y;
+
+	struct ReportList *reports;
 } RenderEngine;
 
 RenderEngine *RE_engine_create(RenderEngineType *type);
+RenderEngine *RE_engine_create_ex(RenderEngineType *type, int use_for_viewport);
 void RE_engine_free(RenderEngine *engine);
 
 void RE_layer_load_from_file(struct RenderLayer *layer, struct ReportList *reports, const char *filename, int x, int y);
 void RE_result_load_from_file(struct RenderResult *result, struct ReportList *reports, const char *filename);
 
-struct RenderResult *RE_engine_begin_result(RenderEngine *engine, int x, int y, int w, int h);
+struct RenderResult *RE_engine_begin_result(RenderEngine *engine, int x, int y, int w, int h, const char *layername);
 void RE_engine_update_result(RenderEngine *engine, struct RenderResult *result);
-void RE_engine_end_result(RenderEngine *engine, struct RenderResult *result);
+void RE_engine_end_result(RenderEngine *engine, struct RenderResult *result, int cancel);
 
 int RE_engine_test_break(RenderEngine *engine);
 void RE_engine_update_stats(RenderEngine *engine, const char *stats, const char *info);
 void RE_engine_update_progress(RenderEngine *engine, float progress);
+void RE_engine_update_memory_stats(RenderEngine *engine, float mem_used, float mem_peak);
 void RE_engine_report(RenderEngine *engine, int type, const char *msg);
 
 int RE_engine_render(struct Render *re, int do_all);
@@ -117,5 +133,7 @@ void RE_engines_exit(void);
 
 RenderEngineType *RE_engines_find(const char *idname);
 
+void RE_engine_get_current_tiles(struct Render *re, int *total_tiles_r, rcti **tiles_r);
+
 #endif /* __RE_ENGINE_H__ */
 
