@@ -23,6 +23,9 @@
  * ***** END GPL LICENSE BLOCK *****
  */
 
-void prepare_ocean_tex_modifier(struct OceanTex *ot);
+#ifndef __TEXTURE_OCEAN_H__
+#define __TEXTURE_OCEAN_H__
 
-int ocean_texture(struct Tex *tex, float *texvec, struct TexResult *texres);
+int ocean_texture(struct Tex *tex, const float texvec[2], struct TexResult *texres);
+
+#endif  /* __TEXTURE_OCEAN_H__ */
