@@ -81,27 +81,25 @@ extern struct Render R;
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 static int test_break(void *data)
 {
-	Render *re = (Render*)data;
+	Render *re = (Render *)data;
 	return re->test_break(re->tbh);
 }
 
 static void RE_rayobject_config_control(RayObject *r, Render *re)
 {
-	if (RE_rayobject_isRayAPI(r))
-	{
-		r = RE_rayobject_align( r );
+	if (RE_rayobject_isRayAPI(r)) {
+		r = RE_rayobject_align(r);
 		r->control.data = re;
 		r->control.test_break = test_break;
 	}
 }
 
-static RayObject*  RE_rayobject_create(Render *re, int type, int size)
+RayObject *RE_rayobject_create(int type, int size, int octree_resolution)
 {
 	RayObject * res = NULL;
 
-	if (type == R_RAYSTRUCTURE_AUTO)
-	{
-		//TODO
+	if (type == R_RAYSTRUCTURE_AUTO) {
+		/* TODO */
 		//if (detect_simd())
 #ifdef __SSE__
 		type = BLI_cpu_support_sse2()? R_RAYSTRUCTURE_SIMD_SVBVH: R_RAYSTRUCTURE_VBVH;
@@ -111,8 +109,7 @@ static RayObject*  RE_rayobject_create(Render *re, int type, int size)
 	}
 	
 #ifndef __SSE__
-	if (type == R_RAYSTRUCTURE_SIMD_SVBVH || type == R_RAYSTRUCTURE_SIMD_QBVH)
-	{
+	if (type == R_RAYSTRUCTURE_SIMD_SVBVH || type == R_RAYSTRUCTURE_SIMD_QBVH) {
 		puts("Warning: Using VBVH (SSE was disabled at compile time)");
 		type = R_RAYSTRUCTURE_VBVH;
 	}
@@ -120,7 +117,7 @@ static RayObject*  RE_rayobject_create(Render *re, int type, int size)
 	
 		
 	if (type == R_RAYSTRUCTURE_OCTREE) //TODO dynamic ocres
-		res = RE_rayobject_octree_create(re->r.ocres, size);
+		res = RE_rayobject_octree_create(octree_resolution, size);
 	else if (type == R_RAYSTRUCTURE_BLIBVH)
 		res = RE_rayobject_blibvh_create(size);
 	else if (type == R_RAYSTRUCTURE_VBVH)
@@ -132,10 +129,18 @@ static RayObject*  RE_rayobject_create(Render *re, int type, int size)
 	else
 		res = RE_rayobject_vbvh_create(size);	//Fallback
 	
+	return res;
+}
+
+static RayObject* rayobject_create(Render *re, int type, int size)
+{
+	RayObject * res = NULL;
+
+	res = RE_rayobject_create(type, size, re->r.ocres);
 	
 	if (res)
 		RE_rayobject_config_control(res, re);
-	
+
 	return res;
 }
 
@@ -148,37 +153,30 @@ void freeraytree(Render *re)
 {
 	ObjectInstanceRen *obi;
 	
-	if (re->raytree)
-	{
+	if (re->raytree) {
 		RE_rayobject_free(re->raytree);
 		re->raytree = NULL;
 	}
-	if (re->rayfaces)
-	{
+	if (re->rayfaces) {
 		MEM_freeN(re->rayfaces);
 		re->rayfaces = NULL;
 	}
-	if (re->rayprimitives)
-	{
+	if (re->rayprimitives) {
 		MEM_freeN(re->rayprimitives);
 		re->rayprimitives = NULL;
 	}
 
-	for (obi=re->instancetable.first; obi; obi=obi->next)
-	{
+	for (obi=re->instancetable.first; obi; obi=obi->next) {
 		ObjectRen *obr = obi->obr;
-		if (obr->raytree)
-		{
+		if (obr->raytree) {
 			RE_rayobject_free(obr->raytree);
 			obr->raytree = NULL;
 		}
-		if (obr->rayfaces)
-		{
+		if (obr->rayfaces) {
 			MEM_freeN(obr->rayfaces);
 			obr->rayfaces = NULL;
 		}
-		if (obi->raytree)
-		{
+		if (obi->raytree) {
 			RE_rayobject_free(obi->raytree);
 			obi->raytree = NULL;
 		}
@@ -187,7 +185,7 @@ void freeraytree(Render *re)
 #ifdef RE_RAYCOUNTER
 	{
 		RayCounter sum;
-		memset( &sum, 0, sizeof(sum) );
+		memset(&sum, 0, sizeof(sum));
 		int i;
 		for (i=0; i<BLENDER_MAX_THREADS; i++)
 			RE_RC_MERGE(&sum, re_rc_counter+i);
@@ -226,14 +224,13 @@ static int is_raytraceable(Render *re, ObjectInstanceRen *obi)
 
 RayObject* makeraytree_object(Render *re, ObjectInstanceRen *obi)
 {
-	//TODO
-	// out-of-memory safeproof
-	// break render
-	// update render stats
+	/*TODO
+	 * out-of-memory safeproof
+	 * break render
+	 * update render stats */
 	ObjectRen *obr = obi->obr;
-	
-	if (obr->raytree == NULL)
-	{
+
+	if (obr->raytree == NULL) {
 		RayObject *raytree;
 		RayFace *face = NULL;
 		VlakPrimitive *vlakprimitive = NULL;
@@ -241,8 +238,7 @@ RayObject* makeraytree_object(Render *re, ObjectInstanceRen *obi)
 		
 		//Count faces
 		int faces = 0;
-		for (v=0;v<obr->totvlak;v++)
-		{
+		for (v=0;v<obr->totvlak;v++) {
 			VlakRen *vlr = obr->vlaknodes[v>>8].vlak + (v&255);
 			if (is_raytraceable_vlr(re, vlr))
 				faces++;
@@ -251,32 +247,30 @@ RayObject* makeraytree_object(Render *re, ObjectInstanceRen *obi)
 		if (faces == 0)
 			return NULL;
 
-		//Create Ray cast accelaration structure		
-		raytree = RE_rayobject_create( re,  re->r.raytrace_structure, faces );
+		//Create Ray cast accelaration structure
+		raytree = rayobject_create( re,  re->r.raytrace_structure, faces );
 		if (  (re->r.raytrace_options & R_RAYTRACE_USE_LOCAL_COORDS) )
-			vlakprimitive = obr->rayprimitives = (VlakPrimitive*)MEM_callocN(faces*sizeof(VlakPrimitive), "ObjectRen primitives");
+			vlakprimitive = obr->rayprimitives = (VlakPrimitive *)MEM_callocN(faces * sizeof(VlakPrimitive), "ObjectRen primitives");
 		else
-			face = obr->rayfaces = (RayFace*)MEM_callocN(faces*sizeof(RayFace), "ObjectRen faces");
+			face = obr->rayfaces = (RayFace *)MEM_callocN(faces * sizeof(RayFace), "ObjectRen faces");
 
 		obr->rayobi = obi;
 		
-		for (v=0;v<obr->totvlak;v++)
-		{
+		for (v=0;v<obr->totvlak;v++) {
 			VlakRen *vlr = obr->vlaknodes[v>>8].vlak + (v&255);
-			if (is_raytraceable_vlr(re, vlr))
-			{
+			if (is_raytraceable_vlr(re, vlr)) {
 				if ((re->r.raytrace_options & R_RAYTRACE_USE_LOCAL_COORDS)) {
-					RE_rayobject_add( raytree, RE_vlakprimitive_from_vlak( vlakprimitive, obi, vlr ) );
+					RE_rayobject_add(raytree, RE_vlakprimitive_from_vlak(vlakprimitive, obi, vlr));
 					vlakprimitive++;
 				}
 				else {
 					RE_rayface_from_vlak(face, obi, vlr);
-					RE_rayobject_add( raytree, RE_rayobject_unalignRayFace(face) );
+					RE_rayobject_add(raytree, RE_rayobject_unalignRayFace(face));
 					face++;
 				}
 			}
 		}
-		RE_rayobject_done( raytree );
+		RE_rayobject_done(raytree);
 
 		/* in case of cancel during build, raytree is not usable */
 		if (test_break(re))
@@ -286,8 +280,7 @@ RayObject* makeraytree_object(Render *re, ObjectInstanceRen *obi)
 	}
 
 	if (obr->raytree) {
-		if ((obi->flag & R_TRANSFORMED) && obi->raytree == NULL)
-		{
+		if ((obi->flag & R_TRANSFORMED) && obi->raytree == NULL) {
 			obi->transform_primitives = 0;
 			obi->raytree = RE_rayobject_instance_create( obr->raytree, obi->mat, obi, obi->obr->rayobi );
 		}
@@ -299,16 +292,13 @@ RayObject* makeraytree_object(Render *re, ObjectInstanceRen *obi)
 
 static int has_special_rayobject(Render *re, ObjectInstanceRen *obi)
 {
-	if ( (obi->flag & R_TRANSFORMED) && (re->r.raytrace_options & R_RAYTRACE_USE_INSTANCES) )
-	{
+	if ( (obi->flag & R_TRANSFORMED) && (re->r.raytrace_options & R_RAYTRACE_USE_INSTANCES) ) {
 		ObjectRen *obr = obi->obr;
 		int v, faces = 0;
 		
-		for (v=0;v<obr->totvlak;v++)
-		{
+		for (v=0;v<obr->totvlak;v++) {
 			VlakRen *vlr = obr->vlaknodes[v>>8].vlak + (v&255);
-			if (is_raytraceable_vlr(re, vlr))
-			{
+			if (is_raytraceable_vlr(re, vlr)) {
 				faces++;
 				if (faces > 4)
 					return 1;
@@ -329,8 +319,7 @@ static void makeraytree_single(Render *re)
 	int faces = 0, obs = 0, special = 0;
 
 	for (obi=re->instancetable.first; obi; obi=obi->next)
-	if (is_raytraceable(re, obi))
-	{
+	if (is_raytraceable(re, obi)) {
 		ObjectRen *obr = obi->obr;
 		obs++;
 		
@@ -339,8 +328,7 @@ static void makeraytree_single(Render *re)
 		}
 		else {
 			int v;
-			for (v=0;v<obr->totvlak;v++)
-			{
+			for (v=0;v<obr->totvlak;v++) {
 				VlakRen *vlr = obr->vlaknodes[v>>8].vlak + (v&255);
 				if (is_raytraceable_vlr(re, vlr))
 					faces++;
@@ -348,26 +336,23 @@ static void makeraytree_single(Render *re)
 		}
 	}
 	
-	if (faces + special == 0)
-	{
+	if (faces + special == 0) {
 		re->raytree = RE_rayobject_empty_create();
 		return;
 	}
 	
 	//Create raytree
-	raytree = re->raytree = RE_rayobject_create( re, re->r.raytrace_structure, faces+special );
+	raytree = re->raytree = rayobject_create( re, re->r.raytrace_structure, faces+special );
 
-	if ( (re->r.raytrace_options & R_RAYTRACE_USE_LOCAL_COORDS) )
-	{
-		vlakprimitive = re->rayprimitives = (VlakPrimitive*)MEM_callocN(faces*sizeof(VlakPrimitive), "Raytrace vlak-primitives");
+	if ( (re->r.raytrace_options & R_RAYTRACE_USE_LOCAL_COORDS) ) {
+		vlakprimitive = re->rayprimitives = (VlakPrimitive *)MEM_callocN(faces * sizeof(VlakPrimitive), "Raytrace vlak-primitives");
 	}
 	else {
-		face = re->rayfaces	= (RayFace*)MEM_callocN(faces*sizeof(RayFace), "Render ray faces");
+		face = re->rayfaces	= (RayFace *)MEM_callocN(faces * sizeof(RayFace), "Render ray faces");
 	}
 	
 	for (obi=re->instancetable.first; obi; obi=obi->next)
-	if (is_raytraceable(re, obi))
-	{
+	if (is_raytraceable(re, obi)) {
 		if (test_break(re))
 			break;
 
@@ -378,30 +363,27 @@ static void makeraytree_single(Render *re)
 				break;
 
 			if (obj)
-				RE_rayobject_add( re->raytree, obj );
+				RE_rayobject_add(re->raytree, obj);
 		}
 		else {
 			int v;
 			ObjectRen *obr = obi->obr;
 			
-			if (obi->flag & R_TRANSFORMED)
-			{
+			if (obi->flag & R_TRANSFORMED) {
 				obi->transform_primitives = 1;
 			}
 
-			for (v=0;v<obr->totvlak;v++)
-			{
+			for (v=0;v<obr->totvlak;v++) {
 				VlakRen *vlr = obr->vlaknodes[v>>8].vlak + (v&255);
 				if (is_raytraceable_vlr(re, vlr)) {
 					if ((re->r.raytrace_options & R_RAYTRACE_USE_LOCAL_COORDS)) {
 						RayObject *obj = RE_vlakprimitive_from_vlak( vlakprimitive, obi, vlr );
-						RE_rayobject_add( raytree, obj );
+						RE_rayobject_add(raytree, obj);
 						vlakprimitive++;
 					}
 					else {
 						RE_rayface_from_vlak(face, obi, vlr);
-						if ((obi->flag & R_TRANSFORMED))
-						{
+						if ((obi->flag & R_TRANSFORMED)) {
 							mul_m4_v3(obi->mat, face->v1);
 							mul_m4_v3(obi->mat, face->v2);
 							mul_m4_v3(obi->mat, face->v3);
@@ -409,7 +391,7 @@ static void makeraytree_single(Render *re)
 								mul_m4_v3(obi->mat, face->v4);
 						}
 
-						RE_rayobject_add( raytree, RE_rayobject_unalignRayFace(face) );
+						RE_rayobject_add(raytree, RE_rayobject_unalignRayFace(face));
 						face++;
 					}
 				}
@@ -417,12 +399,11 @@ static void makeraytree_single(Render *re)
 		}
 	}
 	
-	if (!test_break(re))
-	{	
+	if (!test_break(re)) {
 		re->i.infostr= "Raytree.. building";
 		re->stats_draw(re->sdh, &re->i);
 
-		RE_rayobject_done( raytree );	
+		RE_rayobject_done(raytree);
 	}
 }
 
@@ -448,26 +429,29 @@ void makeraytree(Render *re)
 		re->stats_draw(re->sdh, &re->i);
 	}
 	else {
-		//Calculate raytree max_size
-		//This is ONLY needed to kept a bogus behavior of SUN and HEMI lights
+		/* Calculate raytree max_size
+		 * This is ONLY needed to kept a bogus behavior of SUN and HEMI lights */
 		INIT_MINMAX(min, max);
-		RE_rayobject_merge_bb( re->raytree, min, max );
-		for (i=0; i<3; i++)
-		{
+		RE_rayobject_merge_bb(re->raytree, min, max);
+		if (min[0] > max[0]) {  /* empty raytree */
+			zero_v3(min);
+			zero_v3(max);
+		}
+		for (i=0; i<3; i++) {
+			/* TODO: explain why add top both min and max??? */
 			min[i] += 0.01f;
 			max[i] += 0.01f;
 			sub[i] = max[i]-min[i];
 		}
 
-		re->maxdist = dot_v3v3(sub, sub);
-		if (re->maxdist > 0.0f) re->maxdist= sqrt(re->maxdist);
+		re->maxdist = len_v3(sub);
 
 		re->i.infostr= "Raytree finished";
 		re->stats_draw(re->sdh, &re->i);
 	}
 
 #ifdef RE_RAYCOUNTER
-	memset( re_rc_counter, 0, sizeof(re_rc_counter) );
+	memset(re_rc_counter, 0, sizeof(re_rc_counter));
 #endif
 }
 
@@ -516,8 +500,8 @@ static void shade_ray_set_derivative(ShadeInput *shi)
 
 void shade_ray(Isect *is, ShadeInput *shi, ShadeResult *shr)
 {
-	ObjectInstanceRen *obi= (ObjectInstanceRen*)is->hit.ob;
-	VlakRen *vlr= (VlakRen*)is->hit.face;
+	ObjectInstanceRen *obi = (ObjectInstanceRen *)is->hit.ob;
+	VlakRen *vlr = (VlakRen *)is->hit.face;
 	
 	/* set up view vector */
 	copy_v3_v3(shi->view, is->dir);
@@ -577,7 +561,7 @@ void shade_ray(Isect *is, ShadeInput *shi, ShadeResult *shr)
 		
 		/* raytrace likes to separate the spec color */
 		sub_v3_v3v3(shr->diff, shr->combined, shr->spec);
-	}	
+	}
 
 }
 
@@ -592,12 +576,12 @@ static int refraction(float refract[3], const float n[3], const float view[3], f
 	if (dot>0.0f) {
 		index = 1.0f/index;
 		fac= 1.0f - (1.0f - dot*dot)*index*index;
-		if (fac<= 0.0f) return 0;
+		if (fac <= 0.0f) return 0;
 		fac= -dot*index + sqrtf(fac);
 	}
 	else {
 		fac= 1.0f - (1.0f - dot*dot)*index*index;
-		if (fac<= 0.0f) return 0;
+		if (fac <= 0.0f) return 0;
 		fac= -dot*index - sqrtf(fac);
 	}
 
@@ -632,7 +616,7 @@ static void reflection(float ref[3], float n[3], const float view[3], const floa
 }
 
 #if 0
-static void color_combine(float *result, float fac1, float fac2, float *col1, float *col2)
+static void color_combine(float *result, float fac1, float fac2, float col1[3], float col2[3])
 {
 	float col1t[3], col2t[3];
 	
@@ -660,7 +644,7 @@ static float shade_by_transmission(Isect *is, ShadeInput *shi, ShadeResult *shr)
 	   
 	if (shi->mat->tx_limit <= 0.0f) {
 		d= 1.0f;
-	} 
+	}
 	else {
 		float p;
 
@@ -720,7 +704,7 @@ static void traceray(ShadeInput *origshi, ShadeResult *origshr, short depth, con
 	float dist_mir = origshi->mat->dist_mir;
 	
 	copy_v3_v3(isec.start, start);
-	copy_v3_v3(isec.dir, dir );
+	copy_v3_v3(isec.dir, dir);
 	isec.dist = dist_mir > 0 ? dist_mir : RE_RAYTRACE_MAXDIST;
 	isec.mode= RE_RAY_MIRROR;
 	isec.check = RE_CHECK_VLR_RENDER;
@@ -749,7 +733,7 @@ static void traceray(ShadeInput *origshi, ShadeResult *origshr, short depth, con
 		shi.lay= origshi->lay;
 		shi.passflag= SCE_PASS_COMBINED; /* result of tracing needs no pass info */
 		shi.combinedflag= 0xFFFFFF;		 /* ray trace does all options */
-		//shi.do_preview= 0; // memset above, so don't need this
+		//shi.do_preview = FALSE; // memset above, so don't need this
 		shi.light_override= origshi->light_override;
 		shi.mat_override= origshi->mat_override;
 		
@@ -761,14 +745,14 @@ static void traceray(ShadeInput *origshi, ShadeResult *origshr, short depth, con
 		if (depth>0) {
 			float fr, fg, fb, f, f1;
 
-			if ((shi.mat->mode_l & MA_TRANSP) && shr.alpha < 1.0f && (shi.mat->mode_l & (MA_ZTRANSP | MA_RAYTRANSP))) { 
+			if ((shi.mat->mode_l & MA_TRANSP) && shr.alpha < 1.0f && (shi.mat->mode_l & (MA_ZTRANSP | MA_RAYTRANSP))) {
 				float nf, f, refract[3], tracol[4];
 				
 				tracol[0]= shi.r;
 				tracol[1]= shi.g;
 				tracol[2]= shi.b;
-				tracol[3]= col[3];	// we pass on and accumulate alpha
-				
+				tracol[3]= col[3];	/* we pass on and accumulate alpha */
+
 				if ((shi.mat->mode & MA_TRANSP) && (shi.mat->mode & MA_RAYTRANSP)) {
 					/* don't overwrite traflag, it's value is used in mirror reflection */
 					int new_traflag = traflag;
@@ -911,12 +895,12 @@ static void DP_energy(float *table, float vec[2], int tot, float xsize, float ys
 	}
 	vec[0] += 0.1f*min*result[0]/(float)tot;
 	vec[1] += 0.1f*min*result[1]/(float)tot;
-	// cyclic clamping
+	/* cyclic clamping */
 	vec[0]= vec[0] - xsize*floorf(vec[0]/xsize + 0.5f);
 	vec[1]= vec[1] - ysize*floorf(vec[1]/ysize + 0.5f);
 }
 
-// random offset of 1 in 2
+/* random offset of 1 in 2 */
 static void jitter_plane_offset(float *jitter1, float *jitter2, int tot, float sizex, float sizey, float ofsx, float ofsy)
 {
 	float dsizex= sizex*ofsx;
@@ -966,7 +950,7 @@ void init_jitter_plane(LampRen *lar)
 				DP_energy(lar->jitter, fp, tot, lar->area_size, lar->area_sizey);
 			}
 		}
-	}	
+	}
 	/* create the dithered tables (could just check lamp type!) */
 	jitter_plane_offset(lar->jitter, lar->jitter+2*tot, tot, lar->area_size, lar->area_sizey, 0.5f, 0.0f);
 	jitter_plane_offset(lar->jitter, lar->jitter+4*tot, tot, lar->area_size, lar->area_sizey, 0.5f, 0.5f);
@@ -1002,16 +986,14 @@ static float *give_jitter_plane(LampRen *lar, int thread, int xs, int ys)
 
 static void halton_sample(double *ht_invprimes, double *ht_nums, double *v)
 {
-	// incremental halton sequence generator, from:
-	// "Instant Radiosity", Keller A.
+	/* incremental halton sequence generator, from:
+	 * "Instant Radiosity", Keller A. */
 	unsigned int i;
-	
-	for (i = 0; i < 2; i++)
-	{
+
+	for (i = 0; i < 2; i++) {
 		double r = fabs((1.0 - ht_nums[i]) - 1e-10);
 		
-		if (ht_invprimes[i] >= r)
-		{
+		if (ht_invprimes[i] >= r) {
 			double lasth;
 			double h = ht_invprimes[i];
 			
@@ -1065,8 +1047,7 @@ static struct QMCSampler *QMC_initSampler(int type, int tot)
 
 static void QMC_initPixel(QMCSampler *qsa, int thread)
 {
-	if (qsa->type==SAMP_TYPE_HAMMERSLEY)
-	{
+	if (qsa->type==SAMP_TYPE_HAMMERSLEY) {
 		/* hammersley sequence is fixed, already created in QMCSampler init.
 		 * per pixel, gets a random offset. We create separate offsets per thread, for write-safety */
 		qsa->offs[thread][0] = 0.5f * BLI_thread_frand(thread);
@@ -1074,7 +1055,7 @@ static void QMC_initPixel(QMCSampler *qsa, int thread)
 	}
 	else { 	/* SAMP_TYPE_HALTON */
 		
-		/* generate a new randomised halton sequence per pixel
+		/* generate a new randomized halton sequence per pixel
 		 * to alleviate qmc artifacts and make it reproducible 
 		 * between threads/frames */
 		double ht_invprimes[2], ht_nums[2];
@@ -1182,7 +1163,7 @@ static void QMC_sampleHemiCosine(float vec[3], QMCSampler *qsa, int thread, int
 	
 	QMC_getSample(s, qsa, thread, num);
 	
-	phi = s[0]*2.f*M_PI;	
+	phi = s[0]*2.f*M_PI;
 	sqr = s[1]*sqrt(2-s[1]*s[1]);
 
 	vec[0] = cos(phi)*sqr;
@@ -1207,13 +1188,13 @@ static QMCSampler *get_thread_qmcsampler(Render *re, int thread, int type, int t
 
 	for (qsa=re->qmcsamplers[thread].first; qsa; qsa=qsa->next) {
 		if (qsa->type == type && qsa->tot == tot && !qsa->used) {
-			qsa->used= 1;
+			qsa->used = TRUE;
 			return qsa;
 		}
 	}
 
 	qsa= QMC_initSampler(type, tot);
-	qsa->used= 1;
+	qsa->used = TRUE;
 	BLI_addtail(&re->qmcsamplers[thread], qsa);
 
 	return qsa;
@@ -1270,7 +1251,7 @@ static int adaptive_sample_contrast_val(int samples, float prev, float val, floa
 	/* if the last sample's contribution to the total value was below a small threshold
 	 * (i.e. the samples taken are very similar), then taking more samples that are probably 
 	 * going to be the same is wasting effort */
-	if (fabsf( prev/(float)(samples-1) - val/(float)samples ) < thresh) {
+	if (fabsf(prev / (float)(samples - 1) - val / (float)samples ) < thresh) {
 		return 1;
 	}
 	else
@@ -1326,7 +1307,7 @@ static void trace_refract(float col[4], ShadeInput *shi, ShadeResult *shr)
 		max_samples = 1;
 	
 
-	while (samples < max_samples) {		
+	while (samples < max_samples) {
 		if (refraction(v_refract, shi->vn, shi->view, shi->ang)) {
 			traflag |= RAY_INSIDE;
 		}
@@ -1345,7 +1326,7 @@ static void trace_refract(float col[4], ShadeInput *shi, ShadeResult *shr)
 			/* get a quasi-random vector from a phong-weighted disc */
 			QMC_samplePhong(samp3d, qsa, shi->thread, samples, blur);
 						
-			ortho_basis_v3v3_v3( orthx, orthy,v_refract);
+			ortho_basis_v3v3_v3(orthx, orthy, v_refract);
 			mul_v3_fl(orthx, samp3d[0]);
 			mul_v3_fl(orthy, samp3d[1]);
 				
@@ -1377,8 +1358,7 @@ static void trace_refract(float col[4], ShadeInput *shi, ShadeResult *shr)
 		samples++;
 		
 		/* adaptive sampling */
-		if (adapt_thresh < 1.0f && samples > max_samples/2)
-		{
+		if (adapt_thresh < 1.0f && samples > max_samples/2) {
 			if (adaptive_sample_variance(samples, col, colsq, adapt_thresh))
 				break;
 			
@@ -1442,7 +1422,7 @@ static void trace_reflect(float col[3], ShadeInput *shi, ShadeResult *shr, float
 				mul_v3_fl(orthy, samp3d[1]*aniso);
 			}
 			else {
-				ortho_basis_v3v3_v3( orthx, orthy,shi->vn);
+				ortho_basis_v3v3_v3(orthx, orthy, shi->vn);
 				mul_v3_fl(orthx, samp3d[0]);
 				mul_v3_fl(orthy, samp3d[1]);
 			}
@@ -1479,8 +1459,7 @@ static void trace_reflect(float col[3], ShadeInput *shi, ShadeResult *shr, float
 		samples++;
 
 		/* adaptive sampling */
-		if (adapt_thresh > 0.0f && samples > max_samples/3)
-		{
+		if (adapt_thresh > 0.0f && samples > max_samples/3) {
 			if (adaptive_sample_variance(samples, col, colsq, adapt_thresh))
 				break;
 			
@@ -1516,8 +1495,8 @@ void ray_trace(ShadeInput *shi, ShadeResult *shr)
 	float diff[3];
 	int do_tra, do_mir;
 	
-	do_tra= ((shi->mode & MA_TRANSP) && (shi->mode & MA_RAYTRANSP) && shr->alpha!=1.0f && (shi->depth <= shi->mat->ray_depth_tra));
-	do_mir= ((shi->mat->mode & MA_RAYMIRROR) && shi->ray_mirror!=0.0f && (shi->depth <= shi->mat->ray_depth));
+	do_tra = ((shi->mode & MA_TRANSP) && (shi->mode & MA_RAYTRANSP) && shr->alpha != 1.0f && (shi->depth <= shi->mat->ray_depth_tra));
+	do_mir = ((shi->mat->mode & MA_RAYMIRROR) && shi->ray_mirror != 0.0f && (shi->depth <= shi->mat->ray_depth));
 	
 	/* raytrace mirror and refract like to separate the spec color */
 	if (shi->combinedflag & SCE_PASS_SPEC)
@@ -1548,7 +1527,7 @@ void ray_trace(ShadeInput *shi, ShadeResult *shr)
 		if (!(shi->combinedflag & SCE_PASS_REFRACT))
 			sub_v3_v3v3(diff, diff, shr->refr);
 		
-		shr->alpha= MIN2(1.0f, tracol[3]);
+		shr->alpha = min_ff(1.0f, tracol[3]);
 	}
 	
 	if (do_mir) {
@@ -1712,7 +1691,7 @@ static int UNUSED_FUNCTION(ray_trace_shadow_rad)(ShadeInput *ship, ShadeResult *
 			vec[2]-= vec[2];
 		}
 
-		copy_v3_v3(isec.dir, vec );
+		copy_v3_v3(isec.dir, vec);
 		isec.dist = RE_RAYTRACE_MAXDIST;
 
 		if (RE_rayobject_raycast(R.raytree, &isec)) {
@@ -1845,7 +1824,7 @@ static float *sphere_sampler(int type, int resol, int thread, int xs, int ys, in
 		float *sphere;
 		int a;
 		
-		// always returns table
+		/* always returns table */
 		sphere= threadsafe_table_sphere(0, thread, xs, ys, tot);
 
 		/* total random sampling. NOT THREADSAFE! (should be removed, is not useful) */
@@ -1855,12 +1834,12 @@ static float *sphere_sampler(int type, int resol, int thread, int xs, int ys, in
 		}
 		
 		return sphere;
-	} 
+	}
 	else {
 		float *sphere;
 		float *vec1;
 		
-		// returns table if xs and ys were equal to last call, and not resetting
+		/* returns table if xs and ys were equal to last call, and not resetting */
 		sphere= (reset)? NULL: threadsafe_table_sphere(1, thread, xs, ys, tot);
 		if (sphere==NULL) {
 			float cosfi, sinfi, cost, sint;
@@ -1869,7 +1848,7 @@ static float *sphere_sampler(int type, int resol, int thread, int xs, int ys, in
 
 			sphere= threadsafe_table_sphere(0, thread, xs, ys, tot);
 			
-			// random rotation
+			/* random rotation */
 			ang= BLI_thread_frand(thread);
 			sinfi= sin(ang); cosfi= cos(ang);
 			ang= BLI_thread_frand(thread);
@@ -1880,7 +1859,7 @@ static float *sphere_sampler(int type, int resol, int thread, int xs, int ys, in
 			for (a=0; a<tot; a++, vec+=3, vec1+=3) {
 				vec1[0]= cost*cosfi*vec[0] - sinfi*vec[1] + sint*cosfi*vec[2];
 				vec1[1]= cost*sinfi*vec[0] + cosfi*vec[1] + sint*sinfi*vec[2];
-				vec1[2]= -sint*vec[0] + cost*vec[2];			
+				vec1[2]= -sint*vec[0] + cost*vec[2];
 			}
 		}
 		return sphere;
@@ -1922,7 +1901,7 @@ static void ray_ao_qmc(ShadeInput *shi, float ao[3], float env[3])
 	isec.lay= -1;
 	
 	copy_v3_v3(isec.start, shi->co);
-	RE_rayobject_hint_bb( R.raytree, &point_hint, isec.start, isec.start );
+	RE_rayobject_hint_bb(R.raytree, &point_hint, isec.start, isec.start);
 	isec.hint = &point_hint;
 
 	zero_v3(ao);
@@ -1946,7 +1925,7 @@ static void ray_ao_qmc(ShadeInput *shi, float ao[3], float env[3])
 		copy_v3_v3(nrm, shi->facenor);
 	}
 
-	ortho_basis_v3v3_v3( up, side,nrm);
+	ortho_basis_v3v3_v3(up, side, nrm);
 	
 	/* sampling init */
 	if (R.wrld.ao_samp_method==WO_AOSAMP_HALTON) {
@@ -2014,7 +1993,7 @@ static void ray_ao_qmc(ShadeInput *shi, float ao[3], float env[3])
 		samples++;
 		
 		if (qsa && qsa->type == SAMP_TYPE_HALTON) {
-			/* adaptive sampling - consider samples below threshold as in shadow (or vice versa) and exit early */		
+			/* adaptive sampling - consider samples below threshold as in shadow (or vice versa) and exit early */
 			if (adapt_thresh > 0.0f && (samples > max_samples/2) ) {
 				
 				if (adaptive_sample_contrast_val(samples, prev, fac, adapt_thresh)) {
@@ -2062,7 +2041,7 @@ static void ray_ao_spheresamp(ShadeInput *shi, float ao[3], float env[3])
 	isec.lay= -1;
 
 	copy_v3_v3(isec.start, shi->co);
-	RE_rayobject_hint_bb( R.raytree, &point_hint, isec.start, isec.start );
+	RE_rayobject_hint_bb(R.raytree, &point_hint, isec.start, isec.start);
 	isec.hint = &point_hint;
 
 	zero_v3(ao);
@@ -2089,7 +2068,7 @@ static void ray_ao_spheresamp(ShadeInput *shi, float ao[3], float env[3])
 	 * for strand render we always require a new sampler because x/y are not set */
 	vec= sphere_sampler(R.wrld.aomode, resol, shi->thread, shi->xs, shi->ys, shi->strand != NULL);
 	
-	// warning: since we use full sphere now, and dotproduct is below, we do twice as much
+	/* warning: since we use full sphere now, and dotproduct is below, we do twice as much */
 	tot= 2*resol*resol;
 
 	if (envcolor == WO_AOSKYTEX) {
@@ -2149,7 +2128,7 @@ static void ray_ao_spheresamp(ShadeInput *shi, float ao[3], float env[3])
 				skyadded++;
 			}
 		}
-		// samples
+		/* samples */
 		vec+= 3;
 	}
 	
@@ -2228,7 +2207,7 @@ static void ray_shadow_qmc(ShadeInput *shi, LampRen *lar, const float lampco[3],
 	float adapt_thresh = lar->adapt_thresh;
 	int min_adapt_samples=4, max_samples = lar->ray_totsamp;
 	float *co;
-	int do_soft=1, full_osa=0, i;
+	int do_soft = TRUE, full_osa = FALSE, i;
 
 	float min[3], max[3];
 	RayHint bb_hint;
@@ -2243,8 +2222,8 @@ static void ray_shadow_qmc(ShadeInput *shi, LampRen *lar, const float lampco[3],
 	else
 		shadfac[3]= 1.0f;
 	
-	if (lar->ray_totsamp < 2) do_soft = 0;
-	if ((R.r.mode & R_OSA) && (R.osa > 0) && (shi->vlr->flag & R_FULL_OSA)) full_osa = 1;
+	if (lar->ray_totsamp < 2) do_soft = FALSE;
+	if ((R.r.mode & R_OSA) && (R.osa > 0) && (shi->vlr->flag & R_FULL_OSA)) full_osa = TRUE;
 	
 	if (full_osa) {
 		if (do_soft) max_samples  = max_samples/R.osa + 1;
@@ -2267,11 +2246,10 @@ static void ray_shadow_qmc(ShadeInput *shi, LampRen *lar, const float lampco[3],
 	QMC_initPixel(qsa, shi->thread);
 
 	INIT_MINMAX(min, max);
-	for (i=0; i<totjitco; i++)
-	{
-		DO_MINMAX(jitco[i], min, max);
+	for (i = 0; i < totjitco; i++) {
+		minmax_v3v3_v3(min, max, jitco[i]);
 	}
-	RE_rayobject_hint_bb( R.raytree, &bb_hint, min, max);
+	RE_rayobject_hint_bb(R.raytree, &bb_hint, min, max);
 	
 	isec->hint = &bb_hint;
 	isec->check = RE_CHECK_VLR_RENDER;
@@ -2296,10 +2274,10 @@ static void ray_shadow_qmc(ShadeInput *shi, LampRen *lar, const float lampco[3],
 				/* calc tangent plane vectors */
 				sub_v3_v3v3(v, co, lampco);
 				normalize_v3(v);
-				ortho_basis_v3v3_v3( ru, rv,v);
+				ortho_basis_v3v3_v3(ru, rv, v);
 				
 				/* sampling, returns quasi-random vector in area_size disc */
-				QMC_sampleDisc(samp3d, qsa, shi->thread, samples,lar->area_size);
+				QMC_sampleDisc(samp3d, qsa, shi->thread, samples, lar->area_size);
 
 				/* distribute disc samples across the tangent plane */
 				s[0] = samp3d[0]*ru[0] + samp3d[1]*rv[0];
@@ -2420,7 +2398,7 @@ static void ray_shadow_jitter(ShadeInput *shi, LampRen *lar, const float lampco[
 	copy_v3_v3(isec->start, shi->co);
 	isec->orig.ob   = shi->obi;
 	isec->orig.face = shi->vlr;
-	RE_rayobject_hint_bb( R.raytree, &point_hint, isec->start, isec->start );
+	RE_rayobject_hint_bb(R.raytree, &point_hint, isec->start, isec->start);
 	isec->hint = &point_hint;
 	
 	while (a--) {
@@ -2470,7 +2448,7 @@ static void ray_shadow_jitter(ShadeInput *shi, LampRen *lar, const float lampco[
 		shadfac[3] /= div;
 	}
 	else {
-		// sqrt makes nice umbra effect
+		/* sqrt makes nice umbra effect */
 		if (lar->ray_samp_type & LA_SAMP_UMBRA)
 			shadfac[3]= sqrt(1.0f-fac/div);
 		else
@@ -2535,8 +2513,8 @@ void ray_shadow(ShadeInput *shi, LampRen *lar, float shadfac[4])
 			isec.orig.ob   = shi->obi;
 			isec.orig.face = shi->vlr;
 			
-			shadfac[3]= 1.0f; // 1.0=full light
-			
+			shadfac[3]= 1.0f;  /* 1.0=full light */
+
 			/* set up isec.dir */
 			copy_v3_v3(isec.start, shi->co);
 			sub_v3_v3v3(isec.dir, lampco, isec.start);
@@ -2566,7 +2544,7 @@ void ray_shadow(ShadeInput *shi, LampRen *lar, float shadfac[4])
 
 #if 0
 /* only when face points away from lamp, in direction of lamp, trace ray and find first exit point */
-static void ray_translucent(ShadeInput *shi, LampRen *lar, float *distfac, float *co)
+static void ray_translucent(ShadeInput *shi, LampRen *lar, float *distfac, float co[3])
 {
 	Isect isec;
 	float lampco[3];
