@@ -38,10 +38,6 @@
 #include "DNA_lamp_types.h"
 #include "DNA_material_types.h"
 
-#include "BKE_global.h"
-#include "BKE_scene.h"
-
-
 #include "BLI_math.h"
 #include "BLI_blenlib.h"
 #include "BLI_jitter.h"
@@ -49,6 +45,9 @@
 #include "BLI_rand.h"
 #include "BLI_utildefines.h"
 
+#include "BKE_global.h"
+#include "BKE_scene.h"
+
 #include "PIL_time.h"
 
 #include "renderpipeline.h"
@@ -72,6 +71,9 @@
 #  define ACOMP	3
 #endif
 
+#define RCT_SIZE_X(rct)       ((rct)->xmax - (rct)->xmin)
+#define RCT_SIZE_Y(rct)       ((rct)->ymax - (rct)->ymin)
+
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 /* defined in pipeline.c, is hardcopy of active dynamic allocated Render */
 /* only to be used here in this file, it's for speed */
@@ -86,7 +88,7 @@ extern struct Render R;
 
 static void copy_to_ztile(int *rectz, int size, int x1, int y1, int tile, char *r1)
 {
-	int len4, *rz;	
+	int len4, *rz;
 	int x2, y2;
 	
 	x2= x1+tile;
@@ -108,7 +110,7 @@ static void copy_to_ztile(int *rectz, int size, int x1, int y1, int tile, char *
 #if 0
 static int sizeoflampbuf(ShadBuf *shb)
 {
-	int num,count=0;
+	int num, count=0;
 	char *cp;
 	
 	cp= shb->cbuf;
@@ -139,7 +141,7 @@ static float *give_jitter_tab(int samp)
 
 	if (ctab[samp]==0) {
 		ctab[samp]= 1;
-		BLI_initjit(jit[offset], samp*samp);
+		BLI_jitter_init(jit[offset], samp*samp);
 	}
 		
 	return jit[offset];
@@ -156,9 +158,9 @@ static void make_jitter_weight_tab(Render *re, ShadBuf *shb, short filtertype)
 	
 	for (jit= shb->jit, a=0; a<tot; a++, jit+=2) {
 		if (filtertype==LA_SHADBUF_TENT)
-			shb->weight[a]= 0.71f - sqrt(jit[0]*jit[0] + jit[1]*jit[1]);
+			shb->weight[a] = 0.71f - sqrtf(jit[0] * jit[0] + jit[1] * jit[1]);
 		else if (filtertype==LA_SHADBUF_GAUSS)
-			shb->weight[a]= RE_filter_value(R_FILTER_GAUSS, 1.8f*sqrt(jit[0]*jit[0] + jit[1]*jit[1]));
+			shb->weight[a] = RE_filter_value(R_FILTER_GAUSS, 1.8f * sqrtf(jit[0] * jit[0] + jit[1] * jit[1]));
 		else
 			shb->weight[a]= 1.0f;
 		
@@ -193,7 +195,7 @@ static int compress_deepsamples(DeepSample *dsample, int tot, float epsilon)
 #if 0
 	if (print) {
 		for (a=0, ds=dsample; a<tot; a++, ds++)
-			printf("%lf,%f ", ds->z/(double)0x7FFFFFFF, ds->v);
+			printf("%lf, %f ", ds->z/(double)0x7FFFFFFF, ds->v);
 		printf("\n");
 	}
 #endif
@@ -214,15 +216,15 @@ static int compress_deepsamples(DeepSample *dsample, int tot, float epsilon)
 			if (ds->z == newds->z) {
 				/* still in same z position, simply check
 				 * visibility difference against epsilon */
-				if (!(fabs(newds->v - ds->v) <= epsilon)) {
+				if (!(fabsf(newds->v - ds->v) <= epsilon)) {
 					break;
 				}
 			}
 			else {
 				/* compute slopes */
-				div= (double)0x7FFFFFFF/((double)ds->z - (double)newds->z);
-				min= ((ds->v - epsilon) - newds->v)*div;
-				max= ((ds->v + epsilon) - newds->v)*div;
+				div= (double)0x7FFFFFFF / ((double)ds->z - (double)newds->z);
+				min= (double)((ds->v - epsilon) - newds->v) * div;
+				max= (double)((ds->v + epsilon) - newds->v) * div;
 
 				/* adapt existing slopes */
 				if (first) {
@@ -261,8 +263,8 @@ static int compress_deepsamples(DeepSample *dsample, int tot, float epsilon)
 		}
 		else {
 			/* compute visibility at center between slopes at z */
-			slope= (slopemin+slopemax)*0.5f;
-			v= newds->v + slope*((z - newds->z)/(double)0x7FFFFFFF);
+			slope = (slopemin + slopemax) * 0.5;
+			v = (double)newds->v + slope * ((double)(z - newds->z) / (double)0x7FFFFFFF);
 		}
 
 		newds++;
@@ -278,7 +280,7 @@ static int compress_deepsamples(DeepSample *dsample, int tot, float epsilon)
 #if 0
 	if (print) {
 		for (a=0, ds=dsample; a<newtot; a++, ds++)
-			printf("%lf,%f ", ds->z/(double)0x7FFFFFFF, ds->v);
+			printf("%lf, %f ", ds->z/(double)0x7FFFFFFF, ds->v);
 		printf("\n");
 	}
 #endif
@@ -319,11 +321,11 @@ static void compress_deepshadowbuf(Render *re, ShadBuf *shb, APixstr *apixbuf, A
 	int a, b, c, tot, minz, found, prevtot, newtot;
 	int sampletot[RE_MAX_OSA], totsample = 0, totsamplec = 0;
 	
-	shsample= MEM_callocN( sizeof(ShadSampleBuf), "shad sample buf");
+	shsample= MEM_callocN(sizeof(ShadSampleBuf), "shad sample buf");
 	BLI_addtail(&shb->buffers, shsample);
 
-	shsample->totbuf= MEM_callocN(sizeof(int)*size*size, "deeptotbuf");
-	shsample->deepbuf= MEM_callocN(sizeof(DeepSample*)*size*size, "deepbuf");
+	shsample->totbuf = MEM_callocN(sizeof(int) * size * size, "deeptotbuf");
+	shsample->deepbuf = MEM_callocN(sizeof(DeepSample *) * size * size, "deepbuf");
 
 	ap= apixbuf;
 	aps= apixbufstrand;
@@ -511,11 +513,11 @@ static void compress_shadowbuf(ShadBuf *shb, int *rectz, int square)
 	int a, x, y, minx, miny, byt1, byt2;
 	char *rc, *rcline, *ctile, *zt;
 	
-	shsample= MEM_callocN( sizeof(ShadSampleBuf), "shad sample buf");
+	shsample= MEM_callocN(sizeof(ShadSampleBuf), "shad sample buf");
 	BLI_addtail(&shb->buffers, shsample);
 	
-	shsample->zbuf= MEM_mallocN( sizeof(uintptr_t)*(size*size)/256, "initshadbuf2");
-	shsample->cbuf= MEM_callocN( (size*size)/256, "initshadbuf3");
+	shsample->zbuf= MEM_mallocN(sizeof(uintptr_t)*(size*size)/256, "initshadbuf2");
+	shsample->cbuf= MEM_callocN((size*size)/256, "initshadbuf3");
 	
 	ztile= (uintptr_t *)shsample->zbuf;
 	ctile= shsample->cbuf;
@@ -525,28 +527,28 @@ static void compress_shadowbuf(ShadBuf *shb, int *rectz, int square)
 	
 	for (y=0; y<size; y+=16) {
 		if (y< size/2) miny= y+15-size/2;
-		else miny= y-size/2;	
+		else miny= y-size/2;
 		
 		for (x=0; x<size; x+=16) {
 			
 			/* is tile within spotbundle? */
 			a= size/2;
 			if (x< a) minx= x+15-a;
-			else minx= x-a;	
+			else minx= x-a;
 			
 			dist= sqrt( (float)(minx*minx+miny*miny) );
 			
 			if (square==0 && dist>(float)(a+12)) {	/* 12, tested with a onlyshadow lamp */
 				a= 256; verg= 0; /* 0x80000000; */ /* 0x7FFFFFFF; */
 				rz1= (&verg)+1;
-			} 
+			}
 			else {
 				copy_to_ztile(rectz, size, x, y, 16, rcline);
 				rz1= (int *)rcline;
 				
 				verg= (*rz1 & 0xFFFFFF00);
 				
-				for (a=0;a<256;a++,rz1++) {
+				for (a=0;a<256;a++, rz1++) {
 					if ( (*rz1 & 0xFFFFFF00) !=verg) break;
 				}
 			}
@@ -564,7 +566,7 @@ static void compress_shadowbuf(ShadBuf *shb, int *rectz, int square)
 				verg1= rc[BCOMP];
 				rc+= 4;
 				byt1= 1; byt2= 1;
-				for (a=1;a<256;a++,rc+=4) {
+				for (a=1;a<256;a++, rc+=4) {
 					byt1 &= (verg==rc[ACOMP]);
 					byt2 &= (verg1==rc[BCOMP]);
 					
@@ -582,7 +584,7 @@ static void compress_shadowbuf(ShadBuf *shb, int *rectz, int square)
 				}
 				else if (byt1) {		/* only store short */
 					*ctile= 2;
-					*ztile= (uintptr_t)MEM_mallocN(2*256+4,"Tile2");
+					*ztile= (uintptr_t)MEM_mallocN(2*256+4, "Tile2");
 					rz= (int *)*ztile;
 					*rz= *rz1;
 					
@@ -595,7 +597,7 @@ static void compress_shadowbuf(ShadBuf *shb, int *rectz, int square)
 				}
 				else {			/* store triple */
 					*ctile= 3;
-					*ztile= (uintptr_t)MEM_mallocN(3*256,"Tile3");
+					*ztile= (uintptr_t)MEM_mallocN(3*256, "Tile3");
 
 					zt= (char *)*ztile;
 					rc= rcline;
@@ -634,7 +636,7 @@ static void shadowbuf_autoclip(Render *re, LampRen *lar)
 
 	maxtotvert= 0;
 	for (obr=re->objecttable.first; obr; obr=obr->next)
-		maxtotvert= MAX2(obr->totvert, maxtotvert);
+		maxtotvert = max_ii(obr->totvert, maxtotvert);
 
 	clipflag= MEM_callocN(sizeof(char)*maxtotvert, "autoclipflag");
 
@@ -666,8 +668,8 @@ static void shadowbuf_autoclip(Render *re, LampRen *lar)
 				clipflag[vlr->v2->index]= 1;
 				clipflag[vlr->v3->index]= 1;
 				if (vlr->v4) clipflag[vlr->v4->index]= 1;
-			}				
-		}		
+			}
+		}
 		
 		/* calculate min and max */
 		for (a=0; a< obr->totvert;a++) {
@@ -775,11 +777,11 @@ void makeshadowbuf(Render *re, LampRen *lar)
 	 * transforming from observer view to lamp view, including lamp window matrix */
 	
 	angle= saacos(lar->spotsi);
-	temp= 0.5f*shb->size*cos(angle)/sin(angle);
+	temp = 0.5f * shb->size * cosf(angle) / sinf(angle);
 	shb->pixsize= (shb->d)/temp;
 	wsize= shb->pixsize*(shb->size/2.0f);
 	
-	perspective_m4( shb->winmat,-wsize, wsize, -wsize, wsize, shb->d, shb->clipend);
+	perspective_m4(shb->winmat, -wsize, wsize, -wsize, wsize, shb->d, shb->clipend);
 	mult_m4_m4m4(shb->persmat, shb->winmat, shb->viewmat);
 
 	if (ELEM3(lar->buftype, LA_SHADBUF_REGULAR, LA_SHADBUF_HALFWAY, LA_SHADBUF_DEEP)) {
@@ -809,7 +811,7 @@ void makeshadowbuf(Render *re, LampRen *lar)
 
 static void *do_shadow_thread(void *re_v)
 {
-	Render *re= (Render*)re_v;
+	Render *re = (Render *)re_v;
 	LampRen *lar;
 
 	do {
@@ -848,15 +850,15 @@ void threaded_makeshadowbufs(Render *re)
 	int (*test_break)(void *);
 
 	/* count number of threads to use */
-	if (G.rendering) {
+	if (G.is_rendering) {
 		for (lar=re->lampren.first; lar; lar= lar->next)
 			if (lar->shb)
 				totthread++;
 		
-		totthread= MIN2(totthread, re->r.threads);
+		totthread = min_ii(totthread, re->r.threads);
 	}
 	else
-		totthread= 1; /* preview render */
+		totthread = 1; /* preview render */
 
 	if (totthread <= 1) {
 		for (lar=re->lampren.first; lar; lar= lar->next) {
@@ -1176,8 +1178,8 @@ float testshadowbuf(Render *re, ShadBuf *shb, const float co[3], const float dxc
 	dy[0]= xs1 - dy[0];
 	dy[1]= ys1 - dy[1];
 	
-	xres= fac*(fabs(dx[0]) + fabs(dy[0]));
-	yres= fac*(fabs(dx[1]) + fabs(dy[1]));
+	xres = fac * (fabsf(dx[0]) + fabsf(dy[0]));
+	yres = fac * (fabsf(dx[1]) + fabsf(dy[1]));
 	if (xres<1.0f) xres= 1.0f;
 	if (yres<1.0f) yres= 1.0f;
 	
@@ -1192,7 +1194,7 @@ float testshadowbuf(Render *re, ShadBuf *shb, const float co[3], const float dxc
 			if (firstreadshadbuf(shb, shsample, &rz, (int)(xs1+xres), (int)ys1, 1)) {
 				if (firstreadshadbuf(shb, shsample, &rz, (int)xs1, (int)(ys1+yres), 1)) {
 					if (firstreadshadbuf(shb, shsample, &rz, (int)(xs1+xres), (int)(ys1+yres), 1)) {
-						return readshadowbuf(shb, shsample, bias,(int)xs1, (int)ys1, zs);
+						return readshadowbuf(shb, shsample, bias, (int)xs1, (int)ys1, zs);
 					}
 				}
 			}
@@ -1299,7 +1301,7 @@ float shadow_halo(LampRen *lar, const float p1[3], const float p2[3])
 	ShadBuf *shb= lar->shb;
 	ShadSampleBuf *shsample;
 	float co[4], siz;
-	float labda, labdao, labdax, labday, ldx, ldy;
+	float lambda, lambda_o, lambda_x, lambda_y, ldx, ldy;
 	float zf, xf1, yf1, zf1, xf2, yf2, zf2;
 	float count, lightcount;
 	int x, y, z, xs1, ys1;
@@ -1333,68 +1335,68 @@ float shadow_halo(LampRen *lar, const float p1[3], const float p2[3])
 
 	if (xf1 != xf2) {
 		if (xf2-xf1 > 0.0f) {
-			labdax= (xf1-xs1-1.0f)/(xf1-xf2);
+			lambda_x= (xf1-xs1-1.0f)/(xf1-xf2);
 			ldx= -shb->shadhalostep/(xf1-xf2);
 			dx= shb->shadhalostep;
 		}
 		else {
-			labdax= (xf1-xs1)/(xf1-xf2);
+			lambda_x= (xf1-xs1)/(xf1-xf2);
 			ldx= shb->shadhalostep/(xf1-xf2);
 			dx= -shb->shadhalostep;
 		}
 	}
 	else {
-		labdax= 1.0;
+		lambda_x= 1.0;
 		ldx= 0.0;
 	}
 
 	if (yf1 != yf2) {
 		if (yf2-yf1 > 0.0f) {
-			labday= (yf1-ys1-1.0f)/(yf1-yf2);
+			lambda_y= (yf1-ys1-1.0f)/(yf1-yf2);
 			ldy= -shb->shadhalostep/(yf1-yf2);
 			dy= shb->shadhalostep;
 		}
 		else {
-			labday= (yf1-ys1)/(yf1-yf2);
+			lambda_y= (yf1-ys1)/(yf1-yf2);
 			ldy= shb->shadhalostep/(yf1-yf2);
 			dy= -shb->shadhalostep;
 		}
 	}
 	else {
-		labday= 1.0;
+		lambda_y= 1.0;
 		ldy= 0.0;
 	}
 	
 	x= xs1;
 	y= ys1;
-	labda= count= lightcount= 0.0;
+	lambda= count= lightcount= 0.0;
 
 /* printf("start %x %x	\n", (int)(0x7FFFFFFF*zf1), (int)(0x7FFFFFFF*zf2)); */
 
 	while (1) {
-		labdao= labda;
+		lambda_o= lambda;
 		
-		if (labdax==labday) {
-			labdax+= ldx;
+		if (lambda_x==lambda_y) {
+			lambda_x+= ldx;
 			x+= dx;
-			labday+= ldy;
+			lambda_y+= ldy;
 			y+= dy;
 		}
 		else {
-			if (labdax<labday) {
-				labdax+= ldx;
+			if (lambda_x<lambda_y) {
+				lambda_x+= ldx;
 				x+= dx;
 			}
 			else {
-				labday+= ldy;
+				lambda_y+= ldy;
 				y+= dy;
 			}
 		}
 		
-		labda= MIN2(labdax, labday);
-		if (labda==labdao || labda>=1.0f) break;
+		lambda = min_ff(lambda_x, lambda_y);
+		if (lambda==lambda_o || lambda>=1.0f) break;
 		
-		zf= zf1 + labda*(zf2-zf1);
+		zf= zf1 + lambda*(zf2-zf1);
 		count+= (float)shb->totbuf;
 
 		if (zf<= -1.0f) lightcount += 1.0f;	/* close to the spot */
@@ -1449,7 +1451,7 @@ typedef struct ISBBranch {
 
 typedef struct BSPFace {
 	Boxf box;
-	float *v1, *v2, *v3, *v4;
+	const float *v1, *v2, *v3, *v4;
 	int obi;		/* object for face lookup */
 	int facenr;		/* index to retrieve VlakRen */
 	int type;		/* only for strand now */
@@ -1496,7 +1498,7 @@ static void bound_rectf(rctf *box, const float v1[2])
 static void isb_bsp_split_init(ISBBranch *root, MemArena *mem, int level)
 {
 	
-	/* if level > 0 we create new branches and go deeper*/
+	/* if level > 0 we create new branches and go deeper */
 	if (level > 0) {
 		ISBBranch *left, *right;
 		int i;
@@ -1506,10 +1508,10 @@ static void isb_bsp_split_init(ISBBranch *root, MemArena *mem, int level)
 		root->divider[1]= 0.5f*(root->box.ymin+root->box.ymax);
 		
 		/* find best splitpoint */
-		if (root->box.xmax-root->box.xmin > root->box.ymax-root->box.ymin)
-			i= root->index= 0;
+		if (RCT_SIZE_X(&root->box) > RCT_SIZE_Y(&root->box))
+			i = root->index = 0;
 		else
-			i= root->index= 1;
+			i = root->index = 1;
 		
 		left= root->left= BLI_memarena_alloc(mem, sizeof(ISBBranch));
 		right= root->right= BLI_memarena_alloc(mem, sizeof(ISBBranch));
@@ -1551,19 +1553,19 @@ static void isb_bsp_split(ISBBranch *root, MemArena *mem)
 	root->divider[1]/= BSPMAX_SAMPLE;
 	
 	/* find best splitpoint */
-	if (root->box.xmax-root->box.xmin > root->box.ymax-root->box.ymin)
-		i= root->index= 0;
+	if (RCT_SIZE_X(&root->box) > RCT_SIZE_Y(&root->box))
+		i = root->index = 0;
 	else
-		i= root->index= 1;
+		i = root->index = 1;
 	
 	/* new branches */
 	left= root->left= BLI_memarena_alloc(mem, sizeof(ISBBranch));
 	right= root->right= BLI_memarena_alloc(mem, sizeof(ISBBranch));
 
 	/* new sample array */
-	left->samples= BLI_memarena_alloc(mem, BSPMAX_SAMPLE*sizeof(void *));
-	right->samples= samples; // tmp
-	
+	left->samples = BLI_memarena_alloc(mem, BSPMAX_SAMPLE*sizeof(void *));
+	right->samples = samples;  /* tmp */
+
 	/* split samples */
 	for (a=BSPMAX_SAMPLE-1; a>=0; a--) {
 		int comp= 0;
@@ -1660,9 +1662,9 @@ static void bspface_init_strand(BSPFace *face)
 	
 	face->len= face->rc[0]*face->rc[0]+ face->rc[1]*face->rc[1];
 	
-	if (face->len!=0.0f) {
-		face->radline_end= face->radline/sqrt(face->len);
-		face->len= 1.0f/face->len;
+	if (face->len != 0.0f) {
+		face->radline_end = face->radline / sqrtf(face->len);
+		face->len = 1.0f / face->len;
 	}
 }
 
@@ -1683,21 +1685,21 @@ static int point_behind_strand(const float p[3], BSPFace *face)
 			return 1;
 	}
 	else {
-		float labda= ( face->rc[0]*(p[0]-face->vec1[0]) + face->rc[1]*(p[1]-face->vec1[1]) )*face->len;
+		float lambda= ( face->rc[0]*(p[0]-face->vec1[0]) + face->rc[1]*(p[1]-face->vec1[1]) )*face->len;
 		
-		if (labda > -face->radline_end && labda < 1.0f+face->radline_end) {
+		if (lambda > -face->radline_end && lambda < 1.0f+face->radline_end) {
 			/* hesse for dist: */
 			//dist= (float)(fabs( (p[0]-vec2[0])*rc[1] + (p[1]-vec2[1])*rc[0])/len);
 			
-			pt[0]= labda*face->rc[0]+face->vec1[0];
-			pt[1]= labda*face->rc[1]+face->vec1[1];
+			pt[0]= lambda*face->rc[0]+face->vec1[0];
+			pt[1]= lambda*face->rc[1]+face->vec1[1];
 			
 			rc[0]= pt[0]-p[0];
 			rc[1]= pt[1]-p[1];
 			dist= (float)sqrt(rc[0]*rc[0]+ rc[1]*rc[1]);
 			
 			if (dist < face->radline) {
-				float zval= face->vec1[2] + labda*face->rc[2];
+				float zval= face->vec1[2] + lambda*face->rc[2];
 				if (p[2] > zval)
 					return 1;
 			}
@@ -1815,7 +1817,7 @@ static void isb_bsp_face_inside(ISBBranch *bspn, BSPFace *face)
 			return;
 		
 		/* if face boundbox is outside of branch rect, give up */
-		if (0==BLI_isect_rctf((rctf *)&face->box, (rctf *)&bspn->box, NULL))
+		if (0==BLI_rctf_isect((rctf *)&face->box, (rctf *)&bspn->box, NULL))
 			return;
 		
 		/* test all points inside branch */
@@ -1824,7 +1826,7 @@ static void isb_bsp_face_inside(ISBBranch *bspn, BSPFace *face)
 			
 			if ((samp->facenr!=face->facenr || samp->obi!=face->obi) && samp->shadfac) {
 				if (face->box.zmin < samp->zco[2]) {
-					if (BLI_in_rctf((rctf *)&face->box, samp->zco[0], samp->zco[1])) {
+					if (BLI_rctf_isect_pt_v((rctf *)&face->box, samp->zco)) {
 						int inshadow= 0;
 						
 						if (face->type) {
@@ -1864,11 +1866,12 @@ static void isb_bsp_recalc_box(ISBBranch *root)
 		init_box(&root->box);
 		for (a=root->totsamp-1; a>=0; a--)
 			bound_boxf(&root->box, root->samples[a]->zco);
-	}	
+	}
 }
 
 /* callback function for zbuf clip */
-static void isb_bsp_test_strand(ZSpan *zspan, int obi, int zvlnr, float *v1, float *v2, float *v3, float *v4)
+static void isb_bsp_test_strand(ZSpan *zspan, int obi, int zvlnr,
+                                const float *v1, const float *v2, const float *v3, const float *v4)
 {
 	BSPFace face;
 	
@@ -1902,7 +1905,8 @@ static void isb_bsp_test_strand(ZSpan *zspan, int obi, int zvlnr, float *v1, flo
 }
 
 /* callback function for zbuf clip */
-static void isb_bsp_test_face(ZSpan *zspan, int obi, int zvlnr, float *v1, float *v2, float *v3, float *v4) 
+static void isb_bsp_test_face(ZSpan *zspan, int obi, int zvlnr,
+                              const float *v1, const float *v2, const float *v3, const float *v4)
 {
 	BSPFace face;
 	
@@ -2035,7 +2039,7 @@ static void isb_bsp_fillfaces(Render *re, LampRen *lar, ISBBranch *root)
 					if (vlr->v4)
 						c4= testclip(hoco[3]); 
 					
-					/* ***** NO WIRE YET */			
+					/* ***** NO WIRE YET */
 					if (ma->material_type == MA_TYPE_WIRE) {
 						if (vlr->v4)
 							zbufclipwire(&zspan, i, a+1, vlr->ec, hoco[0], hoco[1], hoco[2], hoco[3], c1, c2, c3, c4);
@@ -2074,7 +2078,7 @@ static int viewpixel_to_lampbuf(ShadBuf *shb, ObjectInstanceRen *obi, VlakRen *v
 	dface = dot_v3v3(v1, nor);
 	hoco[3]= 1.0f;
 	
-	/* ortho viewplane cannot intersect using view vector originating in (0,0,0) */
+	/* ortho viewplane cannot intersect using view vector originating in (0, 0, 0) */
 	if (R.r.mode & R_ORTHO) {
 		/* x and y 3d coordinate can be derived from pixel coord and winmat */
 		float fx= 2.0f/(R.winx*R.winmat[0][0]);
@@ -2160,8 +2164,8 @@ static int isb_add_samples(RenderPart *pa, ISBBranch *root, MemArena *memarena,
 	int sample, bsp_err= 0;
 	
 	/* bsp split doesn't like to handle regular sequences */
-	xcos= MEM_mallocN( pa->rectx*sizeof(int), "xcos");
-	ycos= MEM_mallocN( pa->recty*sizeof(int), "ycos");
+	xcos= MEM_mallocN(pa->rectx*sizeof(int), "xcos");
+	ycos= MEM_mallocN(pa->recty*sizeof(int), "ycos");
 	for (xi=0; xi<pa->rectx; xi++)
 		xcos[xi]= xi;
 	for (yi=0; yi<pa->recty; yi++)
@@ -2182,7 +2186,7 @@ static int isb_add_samples(RenderPart *pa, ISBBranch *root, MemArena *memarena,
 			}
 			if (bsp_err) break;
 		}
-	}	
+	}
 	
 	MEM_freeN(xcos);
 	MEM_freeN(ycos);
@@ -2364,8 +2368,8 @@ static int isb_add_samples_transp(RenderPart *pa, ISBBranch *root, MemArena *mem
 	int sample, bsp_err= 0;
 	
 	/* bsp split doesn't like to handle regular sequences */
-	xcos= MEM_mallocN( pa->rectx*sizeof(int), "xcos");
-	ycos= MEM_mallocN( pa->recty*sizeof(int), "ycos");
+	xcos= MEM_mallocN(pa->rectx*sizeof(int), "xcos");
+	ycos= MEM_mallocN(pa->recty*sizeof(int), "ycos");
 	for (xi=0; xi<pa->rectx; xi++)
 		xcos[xi]= xi;
 	for (yi=0; yi<pa->recty; yi++)
@@ -2389,7 +2393,7 @@ static int isb_add_samples_transp(RenderPart *pa, ISBBranch *root, MemArena *mem
 			}
 			if (bsp_err) break;
 		}
-	}	
+	}
 	
 	MEM_freeN(xcos);
 	MEM_freeN(ycos);
