@@ -61,11 +61,6 @@
 #include "strand.h"
 #include "zbuf.h"
 
-/* to be removed */
-void hoco_to_zco(ZSpan *zspan, float *zco, float *hoco);
-void zspan_scanconvert_strand(ZSpan *zspan, void *handle, float *v1, float *v2, float *v3, void (*func)(void *, int, int, float, float, float) );
-void zbufsinglewire(ZSpan *zspan, int obi, int zvlnr, float *ho1, float *ho2);
-
 /* *************** */
 
 static float strand_eval_width(Material *ma, float strandco)
@@ -333,8 +328,8 @@ StrandShadeCache *strand_shade_cache_create(void)
 	StrandShadeCache *cache;
 
 	cache= MEM_callocN(sizeof(StrandShadeCache), "StrandShadeCache");
-	cache->resulthash= BLI_ghash_new(BLI_ghashutil_pairhash, BLI_ghashutil_paircmp, "strand_shade_cache_create1 gh");
-	cache->refcounthash= BLI_ghash_new(BLI_ghashutil_pairhash, BLI_ghashutil_paircmp, "strand_shade_cache_create2 gh");
+	cache->resulthash= BLI_ghash_pair_new("strand_shade_cache_create1 gh");
+	cache->refcounthash= BLI_ghash_pair_new("strand_shade_cache_create2 gh");
 	cache->memarena= BLI_memarena_new(BLI_MEMARENA_STD_BUFSIZE, "strand shade cache arena");
 	
 	return cache;
@@ -400,7 +395,7 @@ void strand_shade_segment(Render *re, StrandShadeCache *cache, StrandSegment *ss
 	interpolate_shade_result(&shr1, &shr2, t, ssamp->shr, addpassflag);
 
 	/* apply alpha along width */
-	if (sseg->buffer->widthfade != 0.0f) {
+	if (sseg->buffer->widthfade != -1.0f) {
 		s = 1.0f - powf(fabsf(s), sseg->buffer->widthfade);
 
 		strand_apply_shaderesult_alpha(ssamp->shr, s);
@@ -454,6 +449,7 @@ typedef struct StrandPart {
 	int sample;
 	int shadow;
 	float (*jit)[2];
+	int samples;
 
 	StrandSegment *segment;
 	float t[3], s[3];
@@ -480,13 +476,13 @@ static int compare_strand_segment(const void *poin1, const void *poin2)
 		return 1;
 }
 
-static void do_strand_point_project(float winmat[][4], ZSpan *zspan, float *co, float *hoco, float *zco)
+static void do_strand_point_project(float winmat[4][4], ZSpan *zspan, float *co, float *hoco, float *zco)
 {
 	projectvert(co, winmat, hoco);
 	hoco_to_zco(zspan, zco, hoco);
 }
 
-static void strand_project_point(float winmat[][4], float winx, float winy, StrandPoint *spoint)
+static void strand_project_point(float winmat[4][4], float winx, float winy, StrandPoint *spoint)
 {
 	float div;
 
@@ -503,7 +499,7 @@ static APixstrand *addpsmainAstrand(ListBase *lb)
 
 	psm= MEM_mallocN(sizeof(APixstrMain), "addpsmainA");
 	BLI_addtail(lb, psm);
-	psm->ps= MEM_callocN(4096*sizeof(APixstrand),"pixstr");
+	psm->ps = MEM_callocN(4096 * sizeof(APixstrand), "pixstr");
 
 	return psm->ps;
 }
@@ -526,7 +522,7 @@ static APixstrand *addpsAstrand(ZSpan *zspan)
 
 static void do_strand_fillac(void *handle, int x, int y, float u, float v, float z)
 {
-	StrandPart *spart= (StrandPart*)handle;
+	StrandPart *spart= (StrandPart *)handle;
 	StrandShadeCache *cache= spart->cache;
 	StrandSegment *sseg= spart->segment;
 	APixstrand *apn, *apnew;
@@ -550,7 +546,7 @@ static void do_strand_fillac(void *handle, int x, int y, float u, float v, float
 		bufferz= 0x7FFFFFFF;
 		if (spart->rectmask) maskz= 0x7FFFFFFF;
 		
-		if (*rd) {	
+		if (*rd) {
 			for (ps= (PixStr *)(*rd); ps; ps= ps->next) {
 				if (mask & ps->mask) {
 					bufferz= ps->z;
@@ -569,16 +565,16 @@ static void do_strand_fillac(void *handle, int x, int y, float u, float v, float
 
 #define CHECK_ADD(n) \
 	if (apn->p[n]==strnr && apn->obi[n]==obi && apn->seg[n]==seg) \
-	{ if (!(apn->mask[n] & mask)) { apn->mask[n] |= mask; apn->v[n] += t; apn->u[n] += s; } break; }
+	{ if (!(apn->mask[n] & mask)) { apn->mask[n] |= mask; apn->v[n] += t; apn->u[n] += s; } break; } (void)0
 #define CHECK_ASSIGN(n) \
 	if (apn->p[n]==0) \
-	{apn->obi[n]= obi; apn->p[n]= strnr; apn->z[n]= zverg; apn->mask[n]= mask; apn->v[n]= t; apn->u[n]= s; apn->seg[n]= seg; break; }
+	{apn->obi[n]= obi; apn->p[n]= strnr; apn->z[n]= zverg; apn->mask[n]= mask; apn->v[n]= t; apn->u[n]= s; apn->seg[n]= seg; break; } (void)0
 
 	/* add to pixel list */
 	if (zverg < bufferz && (spart->totapixbuf[offset] < MAX_ZROW)) {
 		if (!spart->rectmask || zverg > maskz) {
-			t = u*spart->t[0] + v*spart->t[1] + (1.0f-u-v)*spart->t[2];
-			s = fabs(u*spart->s[0] + v*spart->s[1] + (1.0f-u-v)*spart->s[2]);
+			t = u * spart->t[0] + v * spart->t[1] + (1.0f - u - v) * spart->t[2];
+			s = fabsf(u * spart->s[0] + v * spart->s[1] + (1.0f - u - v) * spart->s[2]);
 
 			apn= spart->apixbuf + offset;
 			while (apn) {
@@ -607,7 +603,7 @@ static void do_strand_fillac(void *handle, int x, int y, float u, float v, float
 }
 
 /* width is calculated in hoco space, to ensure strands are visible */
-static int strand_test_clip(float winmat[][4], ZSpan *UNUSED(zspan), float *bounds, float *co, float *zcomp, float widthx, float widthy)
+static int strand_test_clip(float winmat[4][4], ZSpan *UNUSED(zspan), float *bounds, float *co, float *zcomp, float widthx, float widthy)
 {
 	float hoco[4];
 	int clipflag= 0;
@@ -667,31 +663,27 @@ static void do_scanconvert_strand(Render *UNUSED(re), StrandPart *spart, ZSpan *
 	zspan_scanconvert_strand(zspan, spart, jco1, jco3, jco4, do_strand_fillac);
 }
 
-static void strand_render(Render *re, StrandSegment *sseg, float winmat[][4], StrandPart *spart, ZSpan *zspan, int totzspan, StrandPoint *p1, StrandPoint *p2)
+static void strand_render(Render *re, StrandSegment *sseg, float winmat[4][4], StrandPart *spart, ZSpan *zspan, int totzspan, StrandPoint *p1, StrandPoint *p2)
 {
 	if (spart) {
 		float t= p2->t;
 		float dt= p2->t - p1->t;
 		int a;
 
-		if (re->osa) {
-			for (a=0; a<re->osa; a++)
-				do_scanconvert_strand(re, spart, zspan, t, dt, p1->zco2, p1->zco1, p2->zco1, p2->zco2, a);
-		}
-		else
-			do_scanconvert_strand(re, spart, zspan, t, dt, p1->zco2, p1->zco1, p2->zco1, p2->zco2, 0);
+		for (a=0; a<spart->samples; a++)
+			do_scanconvert_strand(re, spart, zspan, t, dt, p1->zco2, p1->zco1, p2->zco1, p2->zco2, a);
 	}
 	else {
 		float hoco1[4], hoco2[4];
 		int a, obi, index;
-  
+
 		obi= sseg->obi - re->objectinstance;
 		index= sseg->strand->index;
 
 		projectvert(p1->co, winmat, hoco1);
 		projectvert(p2->co, winmat, hoco2);
 
-  
+
 		for (a=0; a<totzspan; a++) {
 #if 0
 			/* render both strand and single pixel wire to counter aliasing */
@@ -704,7 +696,7 @@ static void strand_render(Render *re, StrandSegment *sseg, float winmat[][4], St
 	}
 }
 
-static int strand_segment_recursive(Render *re, float winmat[][4], StrandPart *spart, ZSpan *zspan, int totzspan, StrandSegment *sseg, StrandPoint *p1, StrandPoint *p2, int depth)
+static int strand_segment_recursive(Render *re, float winmat[4][4], StrandPart *spart, ZSpan *zspan, int totzspan, StrandSegment *sseg, StrandPoint *p1, StrandPoint *p2, int depth)
 {
 	StrandPoint p;
 	StrandBuffer *buffer= sseg->buffer;
@@ -753,7 +745,7 @@ static int strand_segment_recursive(Render *re, float winmat[][4], StrandPart *s
 	return 1;
 }
 
-void render_strand_segment(Render *re, float winmat[][4], StrandPart *spart, ZSpan *zspan, int totzspan, StrandSegment *sseg)
+void render_strand_segment(Render *re, float winmat[4][4], StrandPart *spart, ZSpan *zspan, int totzspan, StrandSegment *sseg)
 {
 	StrandBuffer *buffer= sseg->buffer;
 	StrandPoint *p1= &sseg->point1;
@@ -791,7 +783,7 @@ void render_strand_segment(Render *re, float winmat[][4], StrandPart *spart, ZSp
 }
 
 /* render call to fill in strands */
-int zbuffer_strands_abuf(Render *re, RenderPart *pa, APixstrand *apixbuf, ListBase *apsmbase, unsigned int lay, int UNUSED(negzmask), float winmat[][4], int winx, int winy, int UNUSED(sample), float (*jit)[2], float clipcrop, int shadow, StrandShadeCache *cache)
+int zbuffer_strands_abuf(Render *re, RenderPart *pa, APixstrand *apixbuf, ListBase *apsmbase, unsigned int lay, int UNUSED(negzmask), float winmat[4][4], int winx, int winy, int samples, float (*jit)[2], float clipcrop, int shadow, StrandShadeCache *cache)
 {
 	ObjectRen *obr;
 	ObjectInstanceRen *obi;
@@ -825,6 +817,7 @@ int zbuffer_strands_abuf(Render *re, RenderPart *pa, APixstrand *apixbuf, ListBa
 	spart.cache= cache;
 	spart.shadow= shadow;
 	spart.jit= jit;
+	spart.samples= samples;
 
 	zbuf_alloc_span(&zspan, pa->rectx, pa->recty, clipcrop);
 
@@ -983,7 +976,7 @@ int zbuffer_strands_abuf(Render *re, RenderPart *pa, APixstrand *apixbuf, ListBa
 
 /* *************** */
 
-StrandSurface *cache_strand_surface(Render *re, ObjectRen *obr, DerivedMesh *dm, float mat[][4], int timeoffset)
+StrandSurface *cache_strand_surface(Render *re, ObjectRen *obr, DerivedMesh *dm, float mat[4][4], int timeoffset)
 {
 	StrandSurface *mesh;
 	MFace *mface;
@@ -1060,21 +1053,22 @@ void free_strand_surface(Render *re)
 	BLI_freelistN(&re->strandsurface);
 }
 
-void strand_minmax(StrandRen *strand, float *min, float *max, float width)
+void strand_minmax(StrandRen *strand, float min[3], float max[3], const float width)
 {
 	StrandVert *svert;
-	float vec[3], width2= 2.0f*width;
+	const float width2 = width * 2.0f;
+	float vec[3];
 	int a;
 
 	for (a=0, svert=strand->vert; a<strand->totvert; a++, svert++) {
 		copy_v3_v3(vec, svert->co);
-		DO_MINMAX(vec, min, max);
+		minmax_v3v3_v3(min, max, vec);
 		
 		if (width!=0.0f) {
-			vec[0]+= width; vec[1]+= width; vec[2]+= width;
-			DO_MINMAX(vec, min, max);
-			vec[0]-= width2; vec[1]-= width2; vec[2]-= width2;
-			DO_MINMAX(vec, min, max);
+			add_v3_fl(vec, width);
+			minmax_v3v3_v3(min, max, vec);
+			add_v3_fl(vec, -width2);
+			minmax_v3v3_v3(min, max, vec);
 		}
 	}
 }
