@@ -92,10 +92,10 @@ static int intersect_outside_volume(RayObject *tree, Isect *isect, float *offset
 }
 
 /* Uses ray tracing to check if a point is inside or outside an ObjectInstanceRen */
-static int point_inside_obi(RayObject *tree, ObjectInstanceRen *UNUSED(obi), float *co)
+static int point_inside_obi(RayObject *tree, ObjectInstanceRen *UNUSED(obi), const float co[3])
 {
 	Isect isect= {{0}};
-	float dir[3] = {0.0f,0.0f,1.0f};
+	float dir[3] = {0.0f, 0.0f, 1.0f};
 	int final_depth=0, depth=0, limit=20;
 	
 	/* set up the isect */
@@ -118,7 +118,7 @@ static int point_inside_obi(RayObject *tree, ObjectInstanceRen *UNUSED(obi), flo
 }
 
 /* find the bounding box of an objectinstance in global space */
-void global_bounds_obi(Render *re, ObjectInstanceRen *obi, float *bbmin, float *bbmax)
+void global_bounds_obi(Render *re, ObjectInstanceRen *obi, float bbmin[3], float bbmax[3])
 {
 	ObjectRen *obr = obi->obr;
 	VolumePrecache *vp = obi->volume_precache;
@@ -150,7 +150,7 @@ void global_bounds_obi(Render *re, ObjectInstanceRen *obi, float *bbmin, float *
 		/* convert to global space */
 		mul_m4_v3(re->viewinv, co);
 		
-		DO_MINMAX(co, vp->bbmin, vp->bbmax);
+		minmax_v3v3_v3(vp->bbmin, vp->bbmax, co);
 	}
 	
 	copy_v3_v3(bbmin, vp->bbmin);
@@ -177,7 +177,7 @@ static float get_avg_surrounds(float *cache, int *res, int xx, int yy, int zz)
 					for (x=-1; x <= 1; x++) {
 						x_ = xx+x;
 						if (x_ >= 0 && x_ <= res[0]-1) {
-							const int i= V_I(x_, y_, z_, res);
+							const int i = BLI_VOXEL_INDEX(x_, y_, z_, res);
 							
 							if (cache[i] > 0.0f) {
 								tot += cache[i];
@@ -208,7 +208,7 @@ static void lightcache_filter(VolumePrecache *vp)
 		for (y=0; y < vp->res[1]; y++) {
 			for (x=0; x < vp->res[0]; x++) {
 				/* trigger for outside mesh */
-				const int i= V_I(x, y, z, vp->res);
+				const int i = BLI_VOXEL_INDEX(x, y, z, vp->res);
 				
 				if (vp->data_r[i] < -0.f)
 					vp->data_r[i] = get_avg_surrounds(vp->data_r, vp->res, x, y, z);
@@ -240,7 +240,7 @@ static void lightcache_filter2(VolumePrecache *vp)
 		for (y=0; y < vp->res[1]; y++) {
 			for (x=0; x < vp->res[0]; x++) {
 				/* trigger for outside mesh */
-				const int i= V_I(x, y, z, vp->res);
+				const int i = BLI_VOXEL_INDEX(x, y, z, vp->res);
 				if (vp->data_r[i] < -0.f)
 					new_r[i] = get_avg_surrounds(vp->data_r, vp->res, x, y, z);
 				if (vp->data_g[i] < -0.f)
@@ -261,16 +261,16 @@ static void lightcache_filter2(VolumePrecache *vp)
 }
 #endif
 
-BLI_INLINE int ms_I(int x, int y, int z, int *n) //has a pad of 1 voxel surrounding the core for boundary simulation
-{ 
+BLI_INLINE int ms_I(int x, int y, int z, int *n) /* has a pad of 1 voxel surrounding the core for boundary simulation */
+{
 	/* different ordering to light cache */
-	return x*(n[1]+2)*(n[2]+2) + y*(n[2]+2) + z; 	
+	return x*(n[1]+2)*(n[2]+2) + y*(n[2]+2) + z;
 }
 
-BLI_INLINE int v_I_pad(int x, int y, int z, int *n) //has a pad of 1 voxel surrounding the core for boundary simulation
-{ 
+BLI_INLINE int v_I_pad(int x, int y, int z, int *n) /* has a pad of 1 voxel surrounding the core for boundary simulation */
+{
 	/* same ordering to light cache, with padding */
-	return z*(n[1]+2)*(n[0]+2) + y*(n[0]+2) + x;  	
+	return z*(n[1]+2)*(n[0]+2) + y*(n[0]+2) + x;
 }
 
 BLI_INLINE int lc_to_ms_I(int x, int y, int z, int *n)
@@ -291,7 +291,7 @@ static float total_ss_energy(Render *re, int do_test_break, VolumePrecache *vp)
 	for (z=0; z < res[2]; z++) {
 		for (y=0; y < res[1]; y++) {
 			for (x=0; x < res[0]; x++) {
-				const int i=V_I(x, y, z, res);
+				const int i = BLI_VOXEL_INDEX(x, y, z, res);
 			
 				if (vp->data_r[i] > 0.f) energy += vp->data_r[i];
 				if (vp->data_g[i] > 0.f) energy += vp->data_g[i];
@@ -313,7 +313,7 @@ static float total_ms_energy(Render *re, int do_test_break, float *sr, float *sg
 	for (z=1;z<=res[2];z++) {
 		for (y=1;y<=res[1];y++) {
 			for (x=1;x<=res[0];x++) {
-				const int i = ms_I(x,y,z,res);
+				const int i = ms_I(x, y, z, res);
 				
 				if (sr[i] > 0.f) energy += sr[i];
 				if (sg[i] > 0.f) energy += sg[i];
@@ -334,16 +334,12 @@ static void ms_diffuse(Render *re, int do_test_break, float *x0, float *x, float
 	size_t size = n[0]*n[1]*n[2];
 	const float a = dt*diff*size;
 	
-	for (l=0; l<20; l++)
-	{
-		for (k=1; k<=n[2]; k++)
-		{
-			for (j=1; j<=n[1]; j++)
-			{
-				for (i=1; i<=n[0]; i++)
-				{
-				   x[v_I_pad(i,j,k,n)] = (x0[v_I_pad(i,j,k,n)]) + a*(	x0[v_I_pad(i-1,j,k,n)]+ x0[v_I_pad(i+1,j,k,n)]+ x0[v_I_pad(i,j-1,k,n)]+
-																		x0[v_I_pad(i,j+1,k,n)]+ x0[v_I_pad(i,j,k-1,n)]+x0[v_I_pad(i,j,k+1,n)]
+	for (l=0; l<20; l++) {
+		for (k=1; k<=n[2]; k++) {
+			for (j=1; j<=n[1]; j++) {
+				for (i=1; i<=n[0]; i++) {
+				   x[v_I_pad(i, j, k, n)] = (x0[v_I_pad(i, j, k, n)]) + a*(	x0[v_I_pad(i-1, j, k, n)]+ x0[v_I_pad(i+1, j, k, n)]+ x0[v_I_pad(i, j-1, k, n)]+
+																		x0[v_I_pad(i, j+1, k, n)]+ x0[v_I_pad(i, j, k-1, n)]+x0[v_I_pad(i, j, k+1, n)]
 																		) / (1+6*a);
 				}
 			}
@@ -358,7 +354,7 @@ static void ms_diffuse(Render *re, int do_test_break, float *x0, float *x, float
 static void multiple_scattering_diffusion(Render *re, VolumePrecache *vp, Material *ma)
 {
 	const float diff = ma->vol.ms_diff * 0.001f; 	/* compensate for scaling for a nicer UI range */
-	const int simframes = (int)(ma->vol.ms_spread * (float)MAX3(vp->res[0], vp->res[1], vp->res[2]));
+	const int simframes = (int)(ma->vol.ms_spread * (float)max_iii(vp->res[0], vp->res[1], vp->res[2]));
 	const int shade_type = ma->vol.shade_type;
 	float fac = ma->vol.ms_intensity;
 	
@@ -384,20 +380,16 @@ static void multiple_scattering_diffusion(Render *re, VolumePrecache *vp, Materi
 	energy_ss = total_ss_energy(re, do_test_break, vp);
 	
 	/* Scattering as diffusion pass */
-	for (m=0; m<simframes; m++)
-	{
+	for (m=0; m<simframes; m++) {
 		/* add sources */
-		for (z=1; z<=n[2]; z++)
-		{
-			for (y=1; y<=n[1]; y++)
-			{
-				for (x=1; x<=n[0]; x++)
-				{
-					const int i = lc_to_ms_I(x, y ,z, n);	//lc index					
+		for (z=1; z<=n[2]; z++) {
+			for (y=1; y<=n[1]; y++) {
+				for (x=1; x<=n[0]; x++) {
+					const int i = lc_to_ms_I(x, y, z, n);	//lc index
 					const int j = ms_I(x, y, z, n);			//ms index
 					
 					time= PIL_check_seconds_timer();
-					c++;										
+					c++;
 					if (vp->data_r[i] > 0.0f)
 						sr[j] += vp->data_r[i];
 					if (vp->data_g[i] > 0.0f)
@@ -422,9 +414,9 @@ static void multiple_scattering_diffusion(Render *re, VolumePrecache *vp, Materi
 
 		if (re->test_break(re->tbh)) break;
 
-		SWAP(float *,sr,sr0);
-		SWAP(float *,sg,sg0);
-		SWAP(float *,sb,sb0);
+		SWAP(float *, sr, sr0);
+		SWAP(float *, sg, sg0);
+		SWAP(float *, sb, sb0);
 
 		/* main diffusion simulation */
 		ms_diffuse(re, do_test_break, sr0, sr, diff, n);
@@ -448,13 +440,10 @@ static void multiple_scattering_diffusion(Render *re, VolumePrecache *vp, Materi
 		origf = 0.0f;
 	}
 
-	for (z=1;z<=n[2];z++)
-	{
-		for (y=1;y<=n[1];y++)
-		{
-			for (x=1;x<=n[0];x++)
-			{
-				const int i = lc_to_ms_I(x, y ,z, n);	//lc index					
+	for (z=1;z<=n[2];z++) {
+		for (y=1;y<=n[1];y++) {
+			for (x=1;x<=n[0];x++) {
+				const int i = lc_to_ms_I(x, y, z, n);	//lc index
 				const int j = ms_I(x, y, z, n);			//ms index
 				
 				vp->data_r[i] = origf * vp->data_r[i] + fac * sr[j];
@@ -476,7 +465,7 @@ static void multiple_scattering_diffusion(Render *re, VolumePrecache *vp, Materi
 
 
 
-#if 0 // debug stuff
+#if 0  /* debug stuff */
 static void *vol_precache_part_test(void *data)
 {
 	VolPrecachePart *pa = data;
@@ -504,7 +493,7 @@ typedef struct VolPrecacheQueue {
  */
 static void *vol_precache_part(void *data)
 {
-	VolPrecacheQueue *queue = (VolPrecacheQueue*)data;
+	VolPrecacheQueue *queue = (VolPrecacheQueue *)data;
 	VolPrecachePart *pa;
 
 	while ((pa = BLI_thread_queue_pop(queue->work))) {
@@ -537,10 +526,10 @@ static void *vol_precache_part(void *data)
 					
 					/* convert from world->camera space for shading */
 					mul_v3_m4v3(cco, pa->viewmat, co);
-					
-					i= V_I(x, y, z, res);
-					
-					// don't bother if the point is not inside the volume mesh
+
+					i = BLI_VOXEL_INDEX(x, y, z, res);
+
+					/* don't bother if the point is not inside the volume mesh */
 					if (!point_inside_obi(tree, obi, cco)) {
 						obi->volume_precache->data_r[i] = -1.0f;
 						obi->volume_precache->data_g[i] = -1.0f;
@@ -574,7 +563,7 @@ static void precache_setup_shadeinput(Render *re, ObjectInstanceRen *obi, Materi
 	shi->mask= 1;
 	shi->mat = ma;
 	shi->vlr = NULL;
-	memcpy(&shi->r, &shi->mat->r, 23*sizeof(float));	// note, keep this synced with render_types.h
+	memcpy(&shi->r, &shi->mat->r, 23*sizeof(float));	/* note, keep this synced with render_types.h */
 	shi->har= shi->mat->har;
 	shi->obi= obi;
 	shi->obr= obi->obr;
@@ -663,7 +652,7 @@ static int precache_resolution(Render *re, VolumePrecache *vp, ObjectInstanceRen
 	global_bounds_obi(re, obi, bbmin, bbmax);
 	sub_v3_v3v3(dim, bbmax, bbmin);
 	
-	div = MAX3(dim[0], dim[1], dim[2]);
+	div = max_fff(dim[0], dim[1], dim[2]);
 	dim[0] /= div;
 	dim[1] /= div;
 	dim[2] /= div;
@@ -768,7 +757,8 @@ static void vol_precache_objectinstance_threads(Render *re, ObjectInstanceRen *o
 	BLI_freelistN(&re->volume_precache_parts);
 	
 	if (tree) {
-		//TODO: makeraytree_object creates a tree and saves it on OBI, if we free this tree we should also clear other pointers to it
+		/* TODO: makeraytree_object creates a tree and saves it on OBI,
+		 * if we free this tree we should also clear other pointers to it */
 		//RE_rayobject_free(tree);
 		//tree= NULL;
 	}
@@ -836,7 +826,7 @@ void free_volume_precache(Render *re)
 	BLI_freelistN(&re->volumes);
 }
 
-int point_inside_volume_objectinstance(Render *re, ObjectInstanceRen *obi, float *co)
+int point_inside_volume_objectinstance(Render *re, ObjectInstanceRen *obi, const float co[3])
 {
 	RayObject *tree;
 	int inside=0;
