@@ -61,7 +61,7 @@ static GHOST_TStandardCursor convert_cursor(int curs)
 		case CURSOR_FACESEL:    return GHOST_kStandardCursorRightArrow;
 		case CURSOR_WAIT:       return GHOST_kStandardCursorWait;
 		case CURSOR_EDIT:       return GHOST_kStandardCursorCrosshair;
-		case CURSOR_HELP:		
+		case CURSOR_HELP:
 #ifdef __APPLE__
 			return GHOST_kStandardCursorLeftRight;
 #else
@@ -179,34 +179,51 @@ void WM_cursor_wait(int val)
 	}
 }
 
-void WM_cursor_grab(wmWindow *win, int wrap, int hide, int *bounds)
+/**
+ * \param bounds can be NULL
+ */
+void WM_cursor_grab_enable(wmWindow *win, int wrap, int hide, int bounds[4])
 {
 	/* Only grab cursor when not running debug.
 	 * It helps not to get a stuck WM when hitting a breakpoint  
 	 * */
 	GHOST_TGrabCursorMode mode = GHOST_kGrabNormal;
+	float fac = GHOST_GetNativePixelSize(win->ghostwin);
 
-	if (hide) mode = GHOST_kGrabHide;
-	else if (wrap) mode = GHOST_kGrabWrap;
+	/* in case pixel coords differ from window/mouse coords */
+	if (bounds) {
+		bounds[0] /= fac;
+		bounds[1] /= fac;
+		bounds[2] /= fac;
+		bounds[3] /= fac;
+	}
+	
+	if (hide) {
+		mode = GHOST_kGrabHide;
+	}
+	else if (wrap) {
+		mode = GHOST_kGrabWrap;
+	}
 	if ((G.debug & G_DEBUG) == 0) {
-		if (win && win->ghostwin) {
+		if (win->ghostwin) {
 			const GHOST_TabletData *tabletdata = GHOST_GetTabletData(win->ghostwin);
-			// Note: There is no tabletdata on Windows if no tablet device is connected.
+			
+			/* Note: There is no tabletdata on Windows if no tablet device is connected. */
 			if (!tabletdata)
-				GHOST_SetCursorGrab(win->ghostwin, mode, bounds);
+				GHOST_SetCursorGrab(win->ghostwin, mode, bounds, NULL);
 			else if (tabletdata->Active == GHOST_kTabletModeNone)
-				GHOST_SetCursorGrab(win->ghostwin, mode, bounds);
+				GHOST_SetCursorGrab(win->ghostwin, mode, bounds, NULL);
 
 			win->grabcursor = mode;
 		}
 	}
 }
 
-void WM_cursor_ungrab(wmWindow *win)
+void WM_cursor_grab_disable(wmWindow *win, int mouse_ungrab_xy[2])
 {
 	if ((G.debug & G_DEBUG) == 0) {
 		if (win && win->ghostwin) {
-			GHOST_SetCursorGrab(win->ghostwin, GHOST_kGrabDisable, NULL);
+			GHOST_SetCursorGrab(win->ghostwin, GHOST_kGrabDisable, NULL, mouse_ungrab_xy);
 			win->grabcursor = GHOST_kGrabDisable;
 		}
 	}
@@ -239,7 +256,7 @@ int wm_cursor_arrow_move(wmWindow *win, wmEvent *event)
 
 
 /* afer this you can call restore too */
-void WM_timecursor(wmWindow *win, int nr)
+void WM_cursor_time(wmWindow *win, int nr)
 {
 	/* 10 8x8 digits */
 	static char number_bitmaps[10][8] = {
@@ -300,7 +317,7 @@ void WM_timecursor(wmWindow *win, int nr)
  * (3 bytes = 17 bits rounded up to nearest whole byte).  Pad extra bits
  * in mask with 0's.
  *
- * Setting big_bm=NULL disables the large version of the cursor.
+ * Setting big_bm = NULL disables the large version of the cursor.
  *
  * *******************************************************************
  *
@@ -314,7 +331,7 @@ void WM_timecursor(wmWindow *win, int nr)
 /* Because defining a cursor mixes declarations and executable code
  * each cursor needs it's own scoping block or it would be split up
  * over several hundred lines of code.  To enforce/document this better
- * I define 2 pretty braindead macros so it's obvious what the extra "[]"
+ * I define 2 pretty brain-dead macros so it's obvious what the extra "[]"
  * are for */
 
 #define BEGIN_CURSOR_BLOCK {
@@ -557,7 +574,7 @@ BEGIN_CURSOR_BLOCK
 	BlenderCursor[BC_CROSSCURSOR] = &CrossCursor;
 END_CURSOR_BLOCK
 
-	/********************** EditCross Cursor ***********************/	
+	/********************** EditCross Cursor ***********************/
 BEGIN_CURSOR_BLOCK
 	static char editcross_sbm[] = {
 		0x0e,  0x00,  0x11,  0x00,  0x1d,  0x00,  0x19,  0x03,
@@ -772,7 +789,7 @@ BEGIN_CURSOR_BLOCK
 END_CURSOR_BLOCK
 	
 
-	/********************** TextEdit Cursor ***********************/	
+	/********************** TextEdit Cursor ***********************/
 BEGIN_CURSOR_BLOCK
 	static char textedit_sbm[] = {
 		0xe0,  0x03,  0x10,  0x04,  0x60,  0x03,  0x40,  0x01,
@@ -805,7 +822,7 @@ BEGIN_CURSOR_BLOCK
 END_CURSOR_BLOCK
 
 
-	/********************** Paintbrush Cursor ***********************/	
+	/********************** Paintbrush Cursor ***********************/
 BEGIN_CURSOR_BLOCK
 	static char paintbrush_sbm[] = {
 
