@@ -45,9 +45,11 @@
 #include "BLI_utildefines.h"
 #include "BLI_math_base.h"
 
+#include "BIF_gl.h"
+
 #include "BKE_context.h"
 #include "BKE_global.h"
-
+#include "BKE_screen.h"
 
 #include "GHOST_C-api.h"
 
@@ -83,7 +85,7 @@ static void wm_paintcursor_draw(bContext *C, ARegion *ar)
 		bScreen *screen = win->screen;
 		wmPaintCursor *pc;
 
-		if (screen->subwinactive == ar->swinid) {
+		if (ar->swinid && screen->subwinactive == ar->swinid) {
 			for (pc = wm->paintcursors.first; pc; pc = pc->next) {
 				if (pc->poll == NULL || pc->poll(C)) {
 					ARegion *ar_other = CTX_wm_region(C);
@@ -130,7 +132,7 @@ static void wm_region_test_render_do_draw(ScrArea *sa, ARegion *ar)
 		RenderEngine *engine = (rv3d) ? rv3d->render_engine : NULL;
 
 		if (engine && (engine->flag & RE_ENGINE_DO_DRAW)) {
-			ar->do_draw = 1;
+			ar->do_draw = TRUE;
 			engine->flag &= ~RE_ENGINE_DO_DRAW;
 		}
 	}
@@ -193,7 +195,7 @@ static void wm_flush_regions_down(bScreen *screen, rcti *dirty)
 
 	for (sa = screen->areabase.first; sa; sa = sa->next) {
 		for (ar = sa->regionbase.first; ar; ar = ar->next) {
-			if (BLI_isect_rcti(dirty, &ar->winrct, NULL)) {
+			if (BLI_rcti_isect(dirty, &ar->winrct, NULL)) {
 				ar->do_draw = RGN_DRAW;
 				memset(&ar->drawrct, 0, sizeof(ar->drawrct));
 				ar->swap = WIN_NONE_OK;
@@ -208,7 +210,7 @@ static void wm_flush_regions_up(bScreen *screen, rcti *dirty)
 	ARegion *ar;
 	
 	for (ar = screen->regionbase.first; ar; ar = ar->next) {
-		if (BLI_isect_rcti(dirty, &ar->winrct, NULL)) {
+		if (BLI_rcti_isect(dirty, &ar->winrct, NULL)) {
 			ar->do_draw = RGN_DRAW;
 			memset(&ar->drawrct, 0, sizeof(ar->drawrct));
 			ar->swap = WIN_NONE_OK;
@@ -430,22 +432,22 @@ static int wm_triple_gen_textures(wmWindow *win, wmDrawTriple *triple)
 		triple->target = GL_TEXTURE_RECTANGLE_ARB;
 		triple->nx = 1;
 		triple->ny = 1;
-		triple->x[0] = win->sizex;
-		triple->y[0] = win->sizey;
+		triple->x[0] = WM_window_pixels_x(win);
+		triple->y[0] = WM_window_pixels_y(win);
 	}
 	else if (GPU_non_power_of_two_support()) {
 		triple->target = GL_TEXTURE_2D;
 		triple->nx = 1;
 		triple->ny = 1;
-		triple->x[0] = win->sizex;
-		triple->y[0] = win->sizey;
+		triple->x[0] = WM_window_pixels_x(win);
+		triple->y[0] = WM_window_pixels_y(win);
 	}
 	else {
 		triple->target = GL_TEXTURE_2D;
 		triple->nx = 0;
 		triple->ny = 0;
-		split_width(win->sizex, MAX_N_TEX, triple->x, &triple->nx);
-		split_width(win->sizey, MAX_N_TEX, triple->y, &triple->ny);
+		split_width(WM_window_pixels_x(win), MAX_N_TEX, triple->x, &triple->nx);
+		split_width(WM_window_pixels_y(win), MAX_N_TEX, triple->y, &triple->ny);
 	}
 
 	/* generate texture names */
@@ -475,7 +477,7 @@ static int wm_triple_gen_textures(wmWindow *win, wmDrawTriple *triple)
 			glTexImage2D(triple->target, 0, GL_RGB8, triple->x[x], triple->y[y], 0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
 			glTexParameteri(triple->target, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 			glTexParameteri(triple->target, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-			// glColor still used with this enabled?
+			/* The current color is ignored if the GL_REPLACE texture environment is used. */
 			// glTexEnvi(triple->target, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 			glBindTexture(triple->target, 0);
 
@@ -490,7 +492,7 @@ static int wm_triple_gen_textures(wmWindow *win, wmDrawTriple *triple)
 	return 1;
 }
 
-static void wm_triple_draw_textures(wmWindow *win, wmDrawTriple *triple)
+static void wm_triple_draw_textures(wmWindow *win, wmDrawTriple *triple, float alpha)
 {
 	float halfx, halfy, ratiox, ratioy;
 	int x, y, sizex, sizey, offx, offy;
@@ -499,14 +501,14 @@ static void wm_triple_draw_textures(wmWindow *win, wmDrawTriple *triple)
 
 	for (y = 0, offy = 0; y < triple->ny; offy += triple->y[y], y++) {
 		for (x = 0, offx = 0; x < triple->nx; offx += triple->x[x], x++) {
-			sizex = (x == triple->nx - 1) ? win->sizex - offx : triple->x[x];
-			sizey = (y == triple->ny - 1) ? win->sizey - offy : triple->y[y];
+			sizex = (x == triple->nx - 1) ? WM_window_pixels_x(win) - offx : triple->x[x];
+			sizey = (y == triple->ny - 1) ? WM_window_pixels_y(win) - offy : triple->y[y];
 
 			/* wmOrtho for the screen has this same offset */
 			ratiox = sizex;
 			ratioy = sizey;
-			halfx = 0.375f;
-			halfy = 0.375f;
+			halfx = GLA_PIXEL_OFS;
+			halfy = GLA_PIXEL_OFS;
 
 			/* texture rectangle has unnormalized coordinates */
 			if (triple->target == GL_TEXTURE_2D) {
@@ -518,7 +520,7 @@ static void wm_triple_draw_textures(wmWindow *win, wmDrawTriple *triple)
 
 			glBindTexture(triple->target, triple->bind[x + y * triple->nx]);
 
-			glColor3f(1.0f, 1.0f, 1.0f);
+			glColor4f(1.0f, 1.0f, 1.0f, alpha);
 			glBegin(GL_QUADS);
 			glTexCoord2f(halfx, halfy);
 			glVertex2f(offx, offy);
@@ -545,8 +547,8 @@ static void wm_triple_copy_textures(wmWindow *win, wmDrawTriple *triple)
 
 	for (y = 0, offy = 0; y < triple->ny; offy += triple->y[y], y++) {
 		for (x = 0, offx = 0; x < triple->nx; offx += triple->x[x], x++) {
-			sizex = (x == triple->nx - 1) ? win->sizex - offx : triple->x[x];
-			sizey = (y == triple->ny - 1) ? win->sizey - offy : triple->y[y];
+			sizex = (x == triple->nx - 1) ? WM_window_pixels_x(win) - offx : triple->x[x];
+			sizey = (y == triple->ny - 1) ? WM_window_pixels_y(win) - offy : triple->y[y];
 
 			glBindTexture(triple->target, triple->bind[x + y * triple->nx]);
 			glCopyTexSubImage2D(triple->target, 0, 0, 0, offx, offy, sizex, sizey);
@@ -556,6 +558,20 @@ static void wm_triple_copy_textures(wmWindow *win, wmDrawTriple *triple)
 	glBindTexture(triple->target, 0);
 }
 
+static void wm_draw_region_blend(wmWindow *win, ARegion *ar)
+{
+	float fac = ED_region_blend_factor(ar);
+	
+	/* region blend always is 1, except when blend timer is running */
+	if (fac < 1.0f) {
+		wmSubWindowScissorSet(win, win->screen->mainwin, &ar->winrct);
+
+		glEnable(GL_BLEND);
+		wm_triple_draw_textures(win, win->drawdata, 1.0f - fac);
+		glDisable(GL_BLEND);
+	}
+}
+
 static void wm_method_draw_triple(bContext *C, wmWindow *win)
 {
 	wmWindowManager *wm = CTX_wm_manager(C);
@@ -571,7 +587,7 @@ static void wm_method_draw_triple(bContext *C, wmWindow *win)
 
 		wmSubWindowSet(win, screen->mainwin);
 
-		wm_triple_draw_textures(win, win->drawdata);
+		wm_triple_draw_textures(win, win->drawdata, 1.0f);
 	}
 	else {
 		win->drawdata = MEM_callocN(sizeof(wmDrawTriple), "wmDrawTriple");
@@ -590,11 +606,14 @@ static void wm_method_draw_triple(bContext *C, wmWindow *win)
 
 		for (ar = sa->regionbase.first; ar; ar = ar->next) {
 			if (ar->swinid && ar->do_draw) {
-				CTX_wm_region_set(C, ar);
-				ED_region_do_draw(C, ar);
-				ED_area_overdraw_flush(sa, ar);
-				CTX_wm_region_set(C, NULL);
-				copytex = 1;
+				
+				if (ar->overlap == 0) {
+					CTX_wm_region_set(C, ar);
+					ED_region_do_draw(C, ar);
+					ED_area_overdraw_flush(sa, ar);
+					CTX_wm_region_set(C, NULL);
+					copytex = 1;
+				}
 			}
 		}
 		
@@ -609,10 +628,27 @@ static void wm_method_draw_triple(bContext *C, wmWindow *win)
 		wm_triple_copy_textures(win, triple);
 	}
 
+	/* draw overlapping area regions (always like popups) */
+	for (sa = screen->areabase.first; sa; sa = sa->next) {
+		CTX_wm_area_set(C, sa);
+		
+		for (ar = sa->regionbase.first; ar; ar = ar->next) {
+			if (ar->swinid && ar->overlap) {
+				CTX_wm_region_set(C, ar);
+				ED_region_do_draw(C, ar);
+				CTX_wm_region_set(C, NULL);
+				
+				wm_draw_region_blend(win, ar);
+			}
+		}
+
+		CTX_wm_area_set(C, NULL);
+	}
+
 	/* after area regions so we can do area 'overlay' drawing */
 	ED_screen_draw(win);
 
-	/* draw overlapping regions */
+	/* draw floating regions (menus) */
 	for (ar = screen->regionbase.first; ar; ar = ar->next) {
 		if (ar->swinid) {
 			CTX_wm_menu_set(C, ar);
@@ -630,7 +666,7 @@ static void wm_method_draw_triple(bContext *C, wmWindow *win)
 	if (paintcursor && wm->paintcursors.first) {
 		for (sa = screen->areabase.first; sa; sa = sa->next) {
 			for (ar = sa->regionbase.first; ar; ar = ar->next) {
-				if (ar->swinid == screen->subwinactive) {
+				if (ar->swinid && ar->swinid == screen->subwinactive) {
 					CTX_wm_area_set(C, sa);
 					CTX_wm_region_set(C, ar);
 
@@ -651,9 +687,9 @@ static void wm_method_draw_triple(bContext *C, wmWindow *win)
 	if (wm->drags.first) {
 		wm_drags_draw(C, win, NULL);
 	}
-
 }
 
+
 /****************** main update call **********************/
 
 /* quick test to prevent changing window drawable */
@@ -661,15 +697,15 @@ static int wm_draw_update_test_window(wmWindow *win)
 {
 	ScrArea *sa;
 	ARegion *ar;
-	int do_draw = 0;
+	int do_draw = FALSE;
 
 	for (ar = win->screen->regionbase.first; ar; ar = ar->next) {
 		if (ar->do_draw_overlay) {
 			wm_tag_redraw_overlay(win, ar);
-			ar->do_draw_overlay = 0;
+			ar->do_draw_overlay = FALSE;
 		}
 		if (ar->swinid && ar->do_draw)
-			do_draw = 1;
+			do_draw = TRUE;
 	}
 
 	for (sa = win->screen->areabase.first; sa; sa = sa->next) {
@@ -677,7 +713,7 @@ static int wm_draw_update_test_window(wmWindow *win)
 			wm_region_test_render_do_draw(sa, ar);
 
 			if (ar->swinid && ar->do_draw)
-				do_draw = 1;
+				do_draw = TRUE;
 		}
 	}
 
@@ -710,12 +746,14 @@ static int wm_automatic_draw_method(wmWindow *win)
 		if (GPU_type_matches(GPU_DEVICE_ATI, GPU_OS_UNIX, GPU_DRIVER_OPENSOURCE))
 			return USER_DRAW_OVERLAP;
 		/* also Intel drivers are slow */
+#if 0	/* 2.64 BCon3 period, let's try if intel now works... */
 		else if (GPU_type_matches(GPU_DEVICE_INTEL, GPU_OS_UNIX, GPU_DRIVER_ANY))
 			return USER_DRAW_OVERLAP;
 		else if (GPU_type_matches(GPU_DEVICE_INTEL, GPU_OS_WIN, GPU_DRIVER_ANY))
 			return USER_DRAW_OVERLAP_FLIP;
 		else if (GPU_type_matches(GPU_DEVICE_INTEL, GPU_OS_MAC, GPU_DRIVER_ANY))
 			return USER_DRAW_OVERLAP_FLIP;
+#endif
 		/* Windows software driver darkens color on each redraw */
 		else if (GPU_type_matches(GPU_DEVICE_SOFTWARE, GPU_OS_WIN, GPU_DRIVER_SOFTWARE))
 			return USER_DRAW_OVERLAP_FLIP;
@@ -731,13 +769,21 @@ static int wm_automatic_draw_method(wmWindow *win)
 		return win->drawmethod;
 }
 
+int WM_is_draw_triple(wmWindow *win)
+{
+	/* function can get called before this variable is set in drawing code below */
+	if (win->drawmethod != U.wmdrawmethod)
+		win->drawmethod = U.wmdrawmethod;
+	return USER_DRAW_TRIPLE == wm_automatic_draw_method(win);
+}
+
 void wm_tag_redraw_overlay(wmWindow *win, ARegion *ar)
 {
 	/* for draw triple gestures, paint cursors don't need region redraw */
 	if (ar && win) {
 		if (wm_automatic_draw_method(win) != USER_DRAW_TRIPLE)
 			ED_region_tag_redraw(ar);
-		win->screen->do_draw_paintcursor = 1;
+		win->screen->do_draw_paintcursor = TRUE;
 	}
 }
 
@@ -750,6 +796,15 @@ void wm_draw_update(bContext *C)
 	GPU_free_unused_buffers();
 	
 	for (win = wm->windows.first; win; win = win->next) {
+		int state = GHOST_GetWindowState(win->ghostwin);
+
+		if (state == GHOST_kWindowStateMinimized) {
+			/* do not update minimized windows, it gives issues on intel drivers (see [#33223])
+			 * anyway, it seems logical to skip update for invisible windows
+			 */
+			continue;
+		}
+
 		if (win->drawmethod != U.wmdrawmethod) {
 			wm_draw_window_clear(win);
 			win->drawmethod = U.wmdrawmethod;
@@ -778,9 +833,9 @@ void wm_draw_update(bContext *C)
 			else // if (drawmethod == USER_DRAW_TRIPLE)
 				wm_method_draw_triple(C, win);
 
-			win->screen->do_draw_gesture = 0;
-			win->screen->do_draw_paintcursor = 0;
-			win->screen->do_draw_drag = 0;
+			win->screen->do_draw_gesture = FALSE;
+			win->screen->do_draw_paintcursor = FALSE;
+			win->screen->do_draw_drag = FALSE;
 		
 			wm_window_swap_buffers(win);
 
@@ -816,7 +871,7 @@ void wm_draw_region_clear(wmWindow *win, ARegion *ar)
 	if (ELEM(drawmethod, USER_DRAW_OVERLAP, USER_DRAW_OVERLAP_FLIP))
 		wm_flush_regions_down(win->screen, &ar->winrct);
 
-	win->screen->do_draw = 1;
+	win->screen->do_draw = TRUE;
 }
 
 void WM_redraw_windows(bContext *C)
