@@ -56,6 +56,7 @@
 #include "BLI_blenlib.h"
 #include "BLI_linklist.h"
 #include "BLI_utildefines.h"
+#include "BLI_threads.h"
 #include "BLI_callbacks.h"
 
 #include "BLF_translation.h"
@@ -77,6 +78,7 @@
 #include "BKE_global.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
+#include "BKE_multires.h"
 #include "BKE_packedFile.h"
 #include "BKE_report.h"
 #include "BKE_sound.h"
@@ -142,7 +144,7 @@ static void wm_window_match_init(bContext *C, ListBase *wmlist)
 	/* code copied from wm_init_exit.c */
 	for (wm = wmlist->first; wm; wm = wm->id.next) {
 		
-		WM_jobs_stop_all(wm);
+		WM_jobs_kill_all(wm);
 		
 		for (win = wm->windows.first; win; win = win->next) {
 		
@@ -188,9 +190,12 @@ static void wm_window_match_do(bContext *C, ListBase *oldwmlist)
 	
 	/* cases 1 and 2 */
 	if (oldwmlist->first == NULL) {
-		if (G.main->wm.first) ;  /* nothing todo */
-		else
+		if (G.main->wm.first) {
+			/* nothing todo */
+		}
+		else {
 			wm_add_default(C);
+		}
 	}
 	else {
 		/* cases 3 and 4 */
@@ -278,7 +283,9 @@ static void wm_window_match_do(bContext *C, ListBase *oldwmlist)
 /* in case UserDef was read, we re-initialize all, and do versioning */
 static void wm_init_userdef(bContext *C)
 {
+	/* versioning is here */
 	UI_init_userdef();
+	
 	MEM_CacheLimiter_set_maximum(((size_t)U.memcachelimit) * 1024 * 1024);
 	sound_init(CTX_data_main(C));
 
@@ -295,6 +302,8 @@ static void wm_init_userdef(bContext *C)
 
 	/* update tempdir from user preferences */
 	BLI_init_temporary_dir(U.tempdir);
+
+	BKE_userdef_state();
 }
 
 
@@ -315,7 +324,7 @@ static int wm_read_exotic(Scene *UNUSED(scene), const char *name)
 	char header[7];
 	int retval;
 
-	// make sure we're not trying to read a directory....
+	/* make sure we're not trying to read a directory.... */
 
 	len = strlen(name);
 	if (ELEM(name[len - 1], '/', '\\')) {
@@ -333,8 +342,9 @@ static int wm_read_exotic(Scene *UNUSED(scene), const char *name)
 				retval = BKE_READ_EXOTIC_OK_BLEND;
 			}
 			else {
-				//XXX waitcursor(1);
 #if 0           /* historic stuff - no longer used */
+				WM_cursor_wait(TRUE);
+
 				if (is_foo_format(name)) {
 					read_foo(name);
 					retval = BKE_READ_EXOTIC_OK_OTHER;
@@ -344,7 +354,9 @@ static int wm_read_exotic(Scene *UNUSED(scene), const char *name)
 				{
 					retval = BKE_READ_EXOTIC_FAIL_FORMAT;
 				}
-				//XXX waitcursor(0);
+#if 0
+				WM_cursor_wait(FALSE);
+#endif
 			}
 		}
 	}
@@ -352,7 +364,7 @@ static int wm_read_exotic(Scene *UNUSED(scene), const char *name)
 	return retval;
 }
 
-void WM_read_file(bContext *C, const char *filepath, ReportList *reports)
+void WM_file_read(bContext *C, const char *filepath, ReportList *reports)
 {
 	int retval;
 
@@ -361,7 +373,7 @@ void WM_read_file(bContext *C, const char *filepath, ReportList *reports)
 
 	WM_cursor_wait(1);
 
-	BLI_exec_cb(CTX_data_main(C), NULL, BLI_CB_EVT_LOAD_PRE);
+	BLI_callback_exec(CTX_data_main(C), NULL, BLI_CB_EVT_LOAD_PRE);
 
 	/* first try to append data from exotic file formats... */
 	/* it throws error box when file doesn't exist and returns -1 */
@@ -373,10 +385,15 @@ void WM_read_file(bContext *C, const char *filepath, ReportList *reports)
 		int G_f = G.f;
 		ListBase wmbase;
 
+		/* assume automated tasks with background, don't write recent file list */
+		const int do_history = (G.background == FALSE) && (CTX_wm_manager(C)->op_undo_depth == 0);
+
 		/* put aside screens to match with persistent windows later */
 		/* also exit screens and editors */
 		wm_window_match_init(C, &wmbase); 
 		
+		/* confusing this global... */
+		G.relbase_valid = 1;
 		retval = BKE_read_file(C, filepath, reports);
 		G.save_over = 1;
 
@@ -390,8 +407,6 @@ void WM_read_file(bContext *C, const char *filepath, ReportList *reports)
 		/* match the read WM with current WM */
 		wm_window_match_do(C, &wmbase);
 		WM_check(C); /* opens window(s), checks keymaps */
-		
-// XXX		mainwindow_set_filename_to_title(G.main->name);
 
 		if (retval == BKE_READ_FILE_OK_USERPREFS) {
 			/* in case a userdef is read from regular .blend */
@@ -399,9 +414,9 @@ void WM_read_file(bContext *C, const char *filepath, ReportList *reports)
 		}
 		
 		if (retval != BKE_READ_FILE_FAIL) {
-			G.relbase_valid = 1;
-			if (!G.background) /* assume automated tasks with background, don't write recent file list */
+			if (do_history) {
 				write_history();
+			}
 		}
 
 
@@ -421,7 +436,7 @@ void WM_read_file(bContext *C, const char *filepath, ReportList *reports)
 #endif
 
 		/* important to do before NULL'ing the context */
-		BLI_exec_cb(CTX_data_main(C), NULL, BLI_CB_EVT_LOAD_POST);
+		BLI_callback_exec(CTX_data_main(C), NULL, BLI_CB_EVT_LOAD_POST);
 
 		if (!G.background) {
 			/* in background mode this makes it hard to load
@@ -440,33 +455,31 @@ void WM_read_file(bContext *C, const char *filepath, ReportList *reports)
 				if (sce->r.engine[0] &&
 				    BLI_findstring(&R_engines, sce->r.engine, offsetof(RenderEngineType, idname)) == NULL)
 				{
-					BKE_reportf(reports, RPT_WARNING,
-					            "Engine not available: '%s' for scene: %s, "
-					            "an addon may need to be installed or enabled",
+					BKE_reportf(reports, RPT_ERROR, "Engine '%s' not available for scene '%s' "
+					            "(an addon may need to be installed or enabled)",
 					            sce->r.engine, sce->id.name + 2);
 				}
 			}
 		}
 #endif
 
-		// XXX		undo_editmode_clear();
 		BKE_reset_undo();
 		BKE_write_undo(C, "original");  /* save current state */
 	}
 	else if (retval == BKE_READ_EXOTIC_OK_OTHER)
 		BKE_write_undo(C, "Import file");
 	else if (retval == BKE_READ_EXOTIC_FAIL_OPEN) {
-		BKE_reportf(reports, RPT_ERROR, IFACE_("Can't read file: \"%s\", %s."), filepath,
-		            errno ? strerror(errno) : IFACE_("Unable to open the file"));
+		BKE_reportf(reports, RPT_ERROR, "Cannot read file '%s': %s", filepath,
+		            errno ? strerror(errno) : TIP_("unable to open the file"));
 	}
 	else if (retval == BKE_READ_EXOTIC_FAIL_FORMAT) {
-		BKE_reportf(reports, RPT_ERROR, IFACE_("File format is not supported in file: \"%s\"."), filepath);
+		BKE_reportf(reports, RPT_ERROR, "File format is not supported in file '%s'", filepath);
 	}
 	else if (retval == BKE_READ_EXOTIC_FAIL_PATH) {
-		BKE_reportf(reports, RPT_ERROR, IFACE_("File path invalid: \"%s\"."), filepath);
+		BKE_reportf(reports, RPT_ERROR, "File path '%s' invalid", filepath);
 	}
 	else {
-		BKE_reportf(reports, RPT_ERROR, IFACE_("Unknown error loading: \"%s\"."), filepath);
+		BKE_reportf(reports, RPT_ERROR, "Unknown error loading '%s'", filepath);
 		BLI_assert(!"invalid 'retval'");
 	}
 
@@ -477,23 +490,26 @@ void WM_read_file(bContext *C, const char *filepath, ReportList *reports)
 
 /* called on startup,  (context entirely filled with NULLs) */
 /* or called for 'New File' */
-/* op can be NULL */
-int WM_read_homefile(bContext *C, ReportList *UNUSED(reports), short from_memory)
+/* both startup.blend and userpref.blend are checked */
+int wm_homefile_read(bContext *C, ReportList *UNUSED(reports), short from_memory)
 {
 	ListBase wmbase;
-	char tstr[FILE_MAX];
+	char startstr[FILE_MAX];
+	char prefstr[FILE_MAX];
 	int success = 0;
-	
-	free_ttfont(); /* still weird... what does it here? */
-		
+
+	BLI_callback_exec(CTX_data_main(C), NULL, BLI_CB_EVT_LOAD_PRE);
+
 	G.relbase_valid = 0;
 	if (!from_memory) {
 		char *cfgdir = BLI_get_folder(BLENDER_USER_CONFIG, NULL);
 		if (cfgdir) {
-			BLI_make_file_string(G.main->name, tstr, cfgdir, BLENDER_STARTUP_FILE);
+			BLI_make_file_string(G.main->name, startstr, cfgdir, BLENDER_STARTUP_FILE);
+			BLI_make_file_string(G.main->name, prefstr, cfgdir, BLENDER_USERPREF_FILE);
 		}
 		else {
-			tstr[0] = '\0';
+			startstr[0] = '\0';
+			prefstr[0] = '\0';
 			from_memory = 1;
 		}
 	}
@@ -504,14 +520,17 @@ int WM_read_homefile(bContext *C, ReportList *UNUSED(reports), short from_memory
 	/* put aside screens to match with persistent windows later */
 	wm_window_match_init(C, &wmbase); 
 	
-	if (!from_memory && BLI_exists(tstr)) {
-		success = (BKE_read_file(C, tstr, NULL) != BKE_READ_FILE_FAIL);
-		
+	if (!from_memory) {
+		if (BLI_exists(startstr)) {
+			success = (BKE_read_file(C, startstr, NULL) != BKE_READ_FILE_FAIL);
+		}
+
 		if (U.themes.first == NULL) {
-			printf("\nError: No valid "STRINGIFY (BLENDER_STARTUP_FILE)", fall back to built-in default.\n\n");
+			printf("\nNote: No (valid) '%s' found, fall back to built-in default.\n\n", startstr);
 			success = 0;
 		}
 	}
+
 	if (success == 0) {
 		success = BKE_read_file_from_memory(C, datatoc_startup_blend, datatoc_startup_blend_size, NULL);
 		if (wmbase.first == NULL) wm_clear_default_size(C);
@@ -523,6 +542,12 @@ int WM_read_homefile(bContext *C, ReportList *UNUSED(reports), short from_memory
 #endif
 	}
 	
+	/* check new prefs only after startup.blend was finished */
+	if (!from_memory && BLI_exists(prefstr)) {
+		int done = BKE_read_file_userdef(prefstr, NULL);
+		if (done) printf("Read new prefs: %s\n", prefstr);
+	}
+	
 	/* prevent buggy files that had G_FILE_RELATIVE_REMAP written out by mistake. Screws up autosaves otherwise
 	 * can remove this eventually, only in a 2.53 and older, now its not written */
 	G.fileflags &= ~G_FILE_RELATIVE_REMAP;
@@ -542,8 +567,7 @@ int WM_read_homefile(bContext *C, ReportList *UNUSED(reports), short from_memory
 	/* XXX */
 	G.save_over = 0;    // start with save preference untitled.blend
 	G.fileflags &= ~G_FILE_AUTOPLAY;    /*  disable autoplay in startup.blend... */
-//	mainwindow_set_filename_to_title("");	// empty string re-initializes title to "Blender"
-	
+
 //	refresh_interface_font();
 	
 //	undo_editmode_clear();
@@ -564,6 +588,9 @@ int WM_read_homefile(bContext *C, ReportList *UNUSED(reports), short from_memory
 	}
 #endif
 
+	/* important to do before NULL'ing the context */
+	BLI_callback_exec(CTX_data_main(C), NULL, BLI_CB_EVT_LOAD_POST);
+
 	WM_event_add_notifier(C, NC_WM | ND_FILEREAD, NULL);
 
 	/* in background mode the scene will stay NULL */
@@ -574,13 +601,13 @@ int WM_read_homefile(bContext *C, ReportList *UNUSED(reports), short from_memory
 	return TRUE;
 }
 
-int WM_read_homefile_exec(bContext *C, wmOperator *op)
+int wm_homefile_read_exec(bContext *C, wmOperator *op)
 {
 	int from_memory = strcmp(op->type->idname, "WM_OT_read_factory_settings") == 0;
-	return WM_read_homefile(C, op->reports, from_memory) ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
+	return wm_homefile_read(C, op->reports, from_memory) ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
 }
 
-void WM_read_history(void)
+void wm_read_history(void)
 {
 	char name[FILE_MAX];
 	LinkNode *l, *lines;
@@ -620,6 +647,10 @@ static void write_history(void)
 	FILE *fp;
 	int i;
 
+	/* no write history for recovered startup files */
+	if (G.main->name[0] == 0)
+		return;
+	
 	/* will be NULL in background mode */
 	user_config_dir = BLI_get_folder_create(BLENDER_USER_CONFIG, NULL);
 	if (!user_config_dir)
@@ -698,14 +729,14 @@ static ImBuf *blend_file_thumb(Scene *scene, bScreen *screen, int **thumb_pt)
 	if (scene->camera) {
 		ibuf = ED_view3d_draw_offscreen_imbuf_simple(scene, scene->camera,
 		                                             BLEN_THUMB_SIZE * 2, BLEN_THUMB_SIZE * 2,
-		                                             IB_rect, OB_SOLID, FALSE, err_out);
+		                                             IB_rect, OB_SOLID, FALSE, FALSE, R_ADDSKY, err_out);
 	}
 	else {
 		ibuf = ED_view3d_draw_offscreen_imbuf(scene, v3d, ar, BLEN_THUMB_SIZE * 2, BLEN_THUMB_SIZE * 2,
-		                                      IB_rect, FALSE, err_out);
+		                                      IB_rect, FALSE, R_ADDSKY, err_out);
 	}
 
-	if (ibuf) {		
+	if (ibuf) {
 		float aspect = (scene->r.xsch * scene->r.xasp) / (scene->r.ysch * scene->r.yasp);
 
 		/* dirty oversampling */
@@ -752,12 +783,11 @@ int write_crash_blend(void)
 	}
 }
 
-int WM_write_file(bContext *C, const char *target, int fileflags, ReportList *reports, int copy)
+int wm_file_write(bContext *C, const char *target, int fileflags, ReportList *reports)
 {
 	Library *li;
 	int len;
 	char filepath[FILE_MAX];
-
 	int *thumb = NULL;
 	ImBuf *ibuf_thumb = NULL;
 
@@ -780,18 +810,18 @@ int WM_write_file(bContext *C, const char *target, int fileflags, ReportList *re
 	/* send the OnSave event */
 	for (li = G.main->library.first; li; li = li->id.next) {
 		if (BLI_path_cmp(li->filepath, filepath) == 0) {
-			BKE_reportf(reports, RPT_ERROR, "Can't overwrite used library '%.240s'", filepath);
+			BKE_reportf(reports, RPT_ERROR, "Cannot overwrite used library '%.240s'", filepath);
 			return -1;
 		}
 	}
 
 	/* blend file thumbnail */
 	/* save before exit_editmode, otherwise derivedmeshes for shared data corrupt #27765) */
-	if (U.flag & USER_SAVE_PREVIEWS) {
+	if ((U.flag & USER_SAVE_PREVIEWS) && BLI_thread_is_main()) {
 		ibuf_thumb = blend_file_thumb(CTX_data_scene(C), CTX_wm_screen(C), &thumb);
 	}
 
-	BLI_exec_cb(G.main, NULL, BLI_CB_EVT_SAVE_PRE);
+	BLI_callback_exec(G.main, NULL, BLI_CB_EVT_SAVE_PRE);
 
 	/* operator now handles overwrite checks */
 
@@ -807,8 +837,16 @@ int WM_write_file(bContext *C, const char *target, int fileflags, ReportList *re
 	
 	fileflags |= G_FILE_HISTORY; /* write file history */
 
+	/* first time saving */
+	/* XXX temp solution to solve bug, real fix coming (ton) */
+	if (G.main->name[0] == 0)
+		BLI_strncpy(G.main->name, filepath, sizeof(G.main->name));
+	
+	/* XXX temp solution to solve bug, real fix coming (ton) */
+	G.main->recovered = 0;
+	
 	if (BLO_write_file(CTX_data_main(C), filepath, fileflags, reports, thumb)) {
-		if (!copy) {
+		if (!(fileflags & G_FILE_SAVE_COPY)) {
 			G.relbase_valid = 1;
 			BLI_strncpy(G.main->name, filepath, sizeof(G.main->name));  /* is guaranteed current file */
 	
@@ -826,7 +864,7 @@ int WM_write_file(bContext *C, const char *target, int fileflags, ReportList *re
 			write_history();
 		}
 
-		BLI_exec_cb(G.main, NULL, BLI_CB_EVT_SAVE_POST);
+		BLI_callback_exec(G.main, NULL, BLI_CB_EVT_SAVE_POST);
 
 		/* run this function after because the file cant be written before the blend is */
 		if (ibuf_thumb) {
@@ -851,7 +889,7 @@ int WM_write_file(bContext *C, const char *target, int fileflags, ReportList *re
 }
 
 /* operator entry */
-int WM_write_homefile(bContext *C, wmOperator *op)
+int wm_homefile_write_exec(bContext *C, wmOperator *op)
 {
 	wmWindowManager *wm = CTX_wm_manager(C);
 	wmWindow *win = CTX_wm_window(C);
@@ -871,7 +909,7 @@ int WM_write_homefile(bContext *C, wmOperator *op)
 	/*  force save as regular blend file */
 	fileflags = G.fileflags & ~(G_FILE_COMPRESS | G_FILE_AUTOPLAY | G_FILE_LOCK | G_FILE_SIGN | G_FILE_HISTORY);
 
-	if (BLO_write_file(CTX_data_main(C), filepath, fileflags, op->reports, NULL) == 0) {
+	if (BLO_write_file(CTX_data_main(C), filepath, fileflags | G_FILE_USERPREFS, op->reports, NULL) == 0) {
 		printf("fail\n");
 		return OPERATOR_CANCELLED;
 	}
@@ -883,6 +921,28 @@ int WM_write_homefile(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
+/* Only save the prefs block. operator entry */
+int wm_userpref_write_exec(bContext *C, wmOperator *op)
+{
+	wmWindowManager *wm = CTX_wm_manager(C);
+	char filepath[FILE_MAX];
+	
+	/* update keymaps in user preferences */
+	WM_keyconfig_update(wm);
+	
+	BLI_make_file_string("/", filepath, BLI_get_folder_create(BLENDER_USER_CONFIG, NULL), BLENDER_USERPREF_FILE);
+	printf("trying to save userpref at %s ", filepath);
+	
+	if (BKE_write_file_userdef(filepath, op->reports) == 0) {
+		printf("fail\n");
+		return OPERATOR_CANCELLED;
+	}
+	
+	printf("ok\n");
+	
+	return OPERATOR_FINISHED;
+}
+
 /************************ autosave ****************************/
 
 void wm_autosave_location(char *filepath)
@@ -926,7 +986,8 @@ void wm_autosave_timer(const bContext *C, wmWindowManager *wm, wmTimer *UNUSED(w
 	wmWindow *win;
 	wmEventHandler *handler;
 	char filepath[FILE_MAX];
-	int fileflags;
+	
+	Scene *scene = CTX_data_scene(C);
 
 	WM_event_remove_timer(wm, NULL, wm->autosavetimer);
 
@@ -939,15 +1000,27 @@ void wm_autosave_timer(const bContext *C, wmWindowManager *wm, wmTimer *UNUSED(w
 			}
 		}
 	}
-	
+
+	if (scene) {
+		Object *ob = OBACT;
+
+		if (ob && ob->mode & OB_MODE_SCULPT)
+			multires_force_update(ob);
+	}
+
 	wm_autosave_location(filepath);
 
-	/*  force save as regular blend file */
-	fileflags = G.fileflags & ~(G_FILE_COMPRESS | G_FILE_AUTOPLAY | G_FILE_LOCK | G_FILE_SIGN | G_FILE_HISTORY);
-
-	/* no error reporting to console */
-	BLO_write_file(CTX_data_main(C), filepath, fileflags, NULL, NULL);
+	if (U.uiflag & USER_GLOBALUNDO) {
+		/* fast save of last undobuffer, now with UI */
+		BKE_undo_save_file(filepath);
+	}
+	else {
+		/*  save as regular blend file */
+		int fileflags = G.fileflags & ~(G_FILE_COMPRESS | G_FILE_AUTOPLAY | G_FILE_LOCK | G_FILE_SIGN | G_FILE_HISTORY);
 
+		/* no error reporting to console */
+		BLO_write_file(CTX_data_main(C), filepath, fileflags, NULL, NULL);
+	}
 	/* do timer after file write, just in case file write takes a long time */
 	wm->autosavetimer = WM_event_add_timer(wm, NULL, TIMERAUTOSAVE, U.savetime * 60.0);
 }
@@ -968,7 +1041,7 @@ void wm_autosave_delete(void)
 
 	if (BLI_exists(filename)) {
 		char str[FILE_MAX];
-		BLI_make_file_string("/", str, BLI_temporary_dir(), "quit.blend");
+		BLI_make_file_string("/", str, BLI_temporary_dir(), BLENDER_QUIT_FILE);
 
 		/* if global undo; remove tempsave, otherwise rename */
 		if (U.uiflag & USER_GLOBALUNDO) BLI_delete(filename, 0, 0);
@@ -981,6 +1054,9 @@ void wm_autosave_read(bContext *C, ReportList *reports)
 	char filename[FILE_MAX];
 
 	wm_autosave_location(filename);
-	WM_read_file(C, filename, reports);
+	WM_file_read(C, filename, reports);
 }
 
+
+
+
