@@ -113,6 +113,9 @@ void WM_gesture_end(bContext *C, wmGesture *gesture)
 		win->tweak = NULL;
 	BLI_remlink(&win->gesture, gesture);
 	MEM_freeN(gesture->customdata);
+	if (gesture->userdata) {
+		MEM_freeN(gesture->userdata);
+	}
 	MEM_freeN(gesture);
 }
 
@@ -130,8 +133,8 @@ int wm_gesture_evaluate(wmGesture *gesture)
 {
 	if (gesture->type == WM_GESTURE_TWEAK) {
 		rcti *rect = gesture->customdata;
-		int dx = rect->xmax - rect->xmin;
-		int dy = rect->ymax - rect->ymin;
+		int dx = BLI_rcti_size_x(rect);
+		int dy = BLI_rcti_size_y(rect);
 		if (ABS(dx) + ABS(dy) > U.tweak_threshold) {
 			int theta = (int)floor(4.0f * atan2f((float)dy, (float)dx) / (float)M_PI + 0.5f);
 			int val = EVT_GESTURE_W;
@@ -154,7 +157,7 @@ int wm_gesture_evaluate(wmGesture *gesture)
 			if (val == 6) printf("tweak south-west\n");
 			if (val == 7) printf("tweak west\n");
 			if (val == 8) printf("tweak north-west\n");
-#endif			
+#endif
 			return val;
 		}
 	}
@@ -224,19 +227,19 @@ static void wm_gesture_draw_circle(wmGesture *gt)
 	glutil_draw_lined_arc(0.0, M_PI * 2.0, rect->xmax, 40);
 	
 	glDisable(GL_LINE_STIPPLE);
-	glTranslatef((float)-rect->xmin, (float)-rect->ymin, 0.0f);
+	glTranslatef(-rect->xmin, -rect->ymin, 0.0f);
 	
 }
 
 static void draw_filled_lasso(wmGesture *gt)
 {
 	ScanFillContext sf_ctx;
-	ScanFillVert *v = NULL, *lastv = NULL, *firstv = NULL;
-	ScanFillFace *efa;
+	ScanFillVert *sf_vert = NULL, *sf_vert_last = NULL, *sf_vert_first = NULL;
+	ScanFillFace *sf_tri;
 	short *lasso = (short *)gt->customdata;
 	int i;
 	
-	BLI_begin_edgefill(&sf_ctx);
+	BLI_scanfill_begin(&sf_ctx);
 	for (i = 0; i < gt->points; i++, lasso += 2) {
 		float co[3];
 
@@ -244,31 +247,31 @@ static void draw_filled_lasso(wmGesture *gt)
 		co[1] = (float)lasso[1];
 		co[2] = 0.0f;
 
-		v = BLI_addfillvert(&sf_ctx, co);
-		if (lastv)
-			/* e = */ /* UNUSED */ BLI_addfilledge(&sf_ctx, lastv, v);
-		lastv = v;
-		if (firstv == NULL) firstv = v;
+		sf_vert = BLI_scanfill_vert_add(&sf_ctx, co);
+		if (sf_vert_last)
+			/* e = */ /* UNUSED */ BLI_scanfill_edge_add(&sf_ctx, sf_vert_last, sf_vert);
+		sf_vert_last = sf_vert;
+		if (sf_vert_first == NULL) sf_vert_first = sf_vert;
 	}
 	
 	/* highly unlikely this will fail, but could crash if (gt->points == 0) */
-	if (firstv) {
-		float zvec[3] = {0.0f, 0.0f, 1.0f};
-		BLI_addfilledge(&sf_ctx, firstv, v);
-		BLI_edgefill_ex(&sf_ctx, FALSE, zvec);
+	if (sf_vert_first) {
+		const float zvec[3] = {0.0f, 0.0f, 1.0f};
+		BLI_scanfill_edge_add(&sf_ctx, sf_vert_first, sf_vert);
+		BLI_scanfill_calc_ex(&sf_ctx, BLI_SCANFILL_CALC_REMOVE_DOUBLES | BLI_SCANFILL_CALC_HOLES, zvec);
 	
 		glEnable(GL_BLEND);
 		glColor4f(1.0, 1.0, 1.0, 0.05);
 		glBegin(GL_TRIANGLES);
-		for (efa = sf_ctx.fillfacebase.first; efa; efa = efa->next) {
-			glVertex2fv(efa->v1->co);
-			glVertex2fv(efa->v2->co);
-			glVertex2fv(efa->v3->co);
+		for (sf_tri = sf_ctx.fillfacebase.first; sf_tri; sf_tri = sf_tri->next) {
+			glVertex2fv(sf_tri->v1->co);
+			glVertex2fv(sf_tri->v2->co);
+			glVertex2fv(sf_tri->v3->co);
 		}
 		glEnd();
 		glDisable(GL_BLEND);
 	
-		BLI_end_edgefill(&sf_ctx);
+		BLI_scanfill_end(&sf_ctx);
 	}
 }
 
@@ -306,17 +309,19 @@ static void wm_gesture_draw_lasso(wmGesture *gt)
 static void wm_gesture_draw_cross(wmWindow *win, wmGesture *gt)
 {
 	rcti *rect = (rcti *)gt->customdata;
-	
+	int winsizex = WM_window_pixels_x(win);
+	int winsizey = WM_window_pixels_y(win);
+
 	glEnable(GL_LINE_STIPPLE);
 	glColor3ub(96, 96, 96);
 	glLineStipple(1, 0xCCCC);
-	sdrawline(rect->xmin - win->sizex, rect->ymin, rect->xmin + win->sizex, rect->ymin);
-	sdrawline(rect->xmin, rect->ymin - win->sizey, rect->xmin, rect->ymin + win->sizey);
+	sdrawline(rect->xmin - winsizex, rect->ymin, rect->xmin + winsizex, rect->ymin);
+	sdrawline(rect->xmin, rect->ymin - winsizey, rect->xmin, rect->ymin + winsizey);
 	
 	glColor3ub(255, 255, 255);
 	glLineStipple(1, 0x3333);
-	sdrawline(rect->xmin - win->sizex, rect->ymin, rect->xmin + win->sizex, rect->ymin);
-	sdrawline(rect->xmin, rect->ymin - win->sizey, rect->xmin, rect->ymin + win->sizey);
+	sdrawline(rect->xmin - winsizex, rect->ymin, rect->xmin + winsizex, rect->ymin);
+	sdrawline(rect->xmin, rect->ymin - winsizey, rect->xmin, rect->ymin + winsizey);
 	glDisable(GL_LINE_STIPPLE);
 }
 
@@ -331,7 +336,7 @@ void wm_gesture_draw(wmWindow *win)
 		
 		if (gt->type == WM_GESTURE_RECT)
 			wm_gesture_draw_rect(gt);
-//		else if (gt->type==WM_GESTURE_TWEAK)
+//		else if (gt->type == WM_GESTURE_TWEAK)
 //			wm_gesture_draw_line(gt);
 		else if (gt->type == WM_GESTURE_CIRCLE)
 			wm_gesture_draw_circle(gt);
@@ -357,7 +362,7 @@ void wm_gesture_tag_redraw(bContext *C)
 	ARegion *ar = CTX_wm_region(C);
 	
 	if (screen)
-		screen->do_draw_gesture = 1;
+		screen->do_draw_gesture = TRUE;
 
 	wm_tag_redraw_overlay(win, ar);
 }
