@@ -116,8 +116,8 @@ void wm_subwindow_getsize(wmWindow *win, int swinid, int *x, int *y)
 	wmSubWindow *swin = swin_from_swinid(win, swinid);
 
 	if (swin) {
-		*x = swin->winrct.xmax - swin->winrct.xmin + 1;
-		*y = swin->winrct.ymax - swin->winrct.ymin + 1;
+		*x = BLI_rcti_size_x(&swin->winrct) + 1;
+		*y = BLI_rcti_size_y(&swin->winrct) + 1;
 	}
 }
 
@@ -131,7 +131,7 @@ void wm_subwindow_getorigin(wmWindow *win, int swinid, int *x, int *y)
 	}
 }
 
-void wm_subwindow_getmatrix(wmWindow *win, int swinid, float mat[][4])
+void wm_subwindow_getmatrix(wmWindow *win, int swinid, float mat[4][4])
 {
 	wmSubWindow *swin = swin_from_swinid(win, swinid);
 
@@ -141,7 +141,7 @@ void wm_subwindow_getmatrix(wmWindow *win, int swinid, float mat[][4])
 			int width, height;
 
 			wm_subwindow_getsize(win, swin->swinid, &width, &height);
-			orthographic_m4(mat, -0.375f, (float)width - 0.375f, -0.375f, (float)height - 0.375f, -100, 100);
+			orthographic_m4(mat, -GLA_PIXEL_OFS, (float)width - GLA_PIXEL_OFS, -GLA_PIXEL_OFS, (float)height - GLA_PIXEL_OFS, -100, 100);
 		}
 		else
 			glGetFloatv(GL_PROJECTION_MATRIX, (float *)mat);
@@ -149,7 +149,7 @@ void wm_subwindow_getmatrix(wmWindow *win, int swinid, float mat[][4])
 }
 
 /* always sets pixel-precise 2D window/view matrices */
-/* coords is in whole pixels. xmin = 15, xmax= 16: means window is 2 pix big */
+/* coords is in whole pixels. xmin = 15, xmax = 16: means window is 2 pix big */
 int wm_subwindow_open(wmWindow *win, rcti *winrct)
 {
 	wmSubWindow *swin;
@@ -163,10 +163,6 @@ int wm_subwindow_open(wmWindow *win, rcti *winrct)
 	win->curswin = swin = MEM_callocN(sizeof(wmSubWindow), "swinopen");
 	BLI_addtail(&win->subwindows, swin);
 	
-	if (G.debug & G_DEBUG_EVENTS) {
-		printf("%s: swin %d added\n", __func__, freewinid);
-	}
-
 	swin->swinid = freewinid;
 	swin->winrct = *winrct;
 
@@ -175,7 +171,7 @@ int wm_subwindow_open(wmWindow *win, rcti *winrct)
 	
 	/* extra service */
 	wm_subwindow_getsize(win, swin->swinid, &width, &height);
-	wmOrtho2(-0.375f, (float)width - 0.375f, -0.375f, (float)height - 0.375f);
+	wmOrtho2(-GLA_PIXEL_OFS, (float)width - GLA_PIXEL_OFS, -GLA_PIXEL_OFS, (float)height - GLA_PIXEL_OFS);
 	glLoadIdentity();
 
 	return swin->swinid;
@@ -192,11 +188,10 @@ void wm_subwindow_close(wmWindow *win, int swinid)
 		wm_subwindow_free(swin);
 		BLI_remlink(&win->subwindows, swin);
 		MEM_freeN(swin);
-	} 
-	else {
-		printf("wm_subwindow_close: Internal error, bad winid: %d\n", swinid);
 	}
-
+	else {
+		printf("%s: Internal error, bad winid: %d\n", __func__, swinid);
+	}
 }
 
 /* pixels go from 0-99 for a 100 pixel window */
@@ -222,22 +217,22 @@ void wm_subwindow_position(wmWindow *win, int swinid, rcti *winrct)
 		 * fixed it). - zr  (2001!)
 		 */
 		
-		if (swin->winrct.xmax > win->sizex)
-			swin->winrct.xmax = win->sizex;
-		if (swin->winrct.ymax > win->sizey)
-			swin->winrct.ymax = win->sizey;
+		if (swin->winrct.xmax > WM_window_pixels_x(win))
+			swin->winrct.xmax = WM_window_pixels_x(win);
+		if (swin->winrct.ymax > WM_window_pixels_y(win))
+			swin->winrct.ymax = WM_window_pixels_y(win);
 		
 		/* extra service */
 		wmSubWindowSet(win, swinid);
 		wm_subwindow_getsize(win, swinid, &width, &height);
-		wmOrtho2(-0.375f, (float)width - 0.375f, -0.375f, (float)height - 0.375f);
+		wmOrtho2(-GLA_PIXEL_OFS, (float)width - GLA_PIXEL_OFS, -GLA_PIXEL_OFS, (float)height - GLA_PIXEL_OFS);
 	}
 	else {
-		printf("wm_subwindow_position: Internal error, bad winid: %d\n", swinid);
+		printf("%s: Internal error, bad winid: %d\n", __func__, swinid);
 	}
 }
 
-/* ---------------- WM versions of OpenGL calls, using glBlah() syntax ------------------------ */
+/* ---------------- WM versions of OpenGL style API calls ------------------------ */
 /* ----------------- exported in WM_api.h ------------------------------------------------------ */
 
 /* internal state, no threaded opengl! XXX */
@@ -250,32 +245,31 @@ void wmSubWindowScissorSet(wmWindow *win, int swinid, rcti *srct)
 	_curswin = swin_from_swinid(win, swinid);
 	
 	if (_curswin == NULL) {
-		printf("wmSubWindowSet %d: doesn't exist\n", swinid);
+		printf("%s %d: doesn't exist\n", __func__, swinid);
 		return;
 	}
 	
 	win->curswin = _curswin;
 	_curwindow = win;
 	
-	width = _curswin->winrct.xmax - _curswin->winrct.xmin + 1;
-	height = _curswin->winrct.ymax - _curswin->winrct.ymin + 1;
+	width  = BLI_rcti_size_x(&_curswin->winrct) + 1;
+	height = BLI_rcti_size_y(&_curswin->winrct) + 1;
 	glViewport(_curswin->winrct.xmin, _curswin->winrct.ymin, width, height);
-
+	
 	if (srct) {
-		width = srct->xmax - srct->xmin + 1;
-		height = srct->ymax - srct->ymin + 1;
+		int width  = BLI_rcti_size_x(srct) + 1; /* only here */
+		int height = BLI_rcti_size_y(srct) + 1;
 		glScissor(srct->xmin, srct->ymin, width, height);
 	}
 	else
 		glScissor(_curswin->winrct.xmin, _curswin->winrct.ymin, width, height);
 	
-	wmOrtho2(-0.375f, (float)width - 0.375f, -0.375f, (float)height - 0.375f);
+	wmOrtho2(-GLA_PIXEL_OFS, (float)width - GLA_PIXEL_OFS, -GLA_PIXEL_OFS, (float)height - GLA_PIXEL_OFS);
 	glLoadIdentity();
-
+	
 	glFlush();
 }
 
-
 /* enable the WM versions of opengl calls */
 void wmSubWindowSet(wmWindow *win, int swinid)
 {
@@ -332,12 +326,12 @@ unsigned int index_to_framebuffer(int index)
 			break;
 		case 24:
 			break;
-		default: // 18 bits...
+		default: /* 18 bits... */
 			i = ((i & 0x3F000) << 6) + ((i & 0xFC0) << 4) + ((i & 0x3F) << 2);
 			i |= 0x010101;
 			break;
 	}
-	
+
 	return i;
 }
 
@@ -366,7 +360,7 @@ unsigned int index_to_framebuffer(int index)
 			break;
 		case 24:
 			break;
-		default:    // 18 bits...
+		default:    /* 18 bits... */
 			i = ((i & 0x3F000) << 6) + ((i & 0xFC0) << 4) + ((i & 0x3F) << 2);
 			i |= 0x030303;
 			break;
@@ -377,7 +371,7 @@ unsigned int index_to_framebuffer(int index)
 
 #endif
 
-void WM_set_framebuffer_index_color(int index)
+void WM_framebuffer_index_set(int index)
 {
 	const int col = index_to_framebuffer(index);
 	cpack(col);
@@ -399,7 +393,7 @@ int WM_framebuffer_to_index(unsigned int col)
 			return col & 0xFFFFFF;
 		default: // 18 bits...
 			return ((col & 0xFC0000) >> 6) + ((col & 0xFC00) >> 4) + ((col & 0xFC) >> 2);
-	}		
+	}
 }
 
 
