@@ -43,6 +43,7 @@
 
 #include "GHOST_C-api.h"
 
+#include "BLI_math.h"
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
 
@@ -75,6 +76,11 @@
 
 #include "UI_interface.h"
 
+/* for assert */
+#ifndef NDEBUG
+#  include "BLI_threads.h"
+#endif
+
 /* the global to talk to ghost */
 static GHOST_SystemHandle g_system = NULL;
 
@@ -91,7 +97,9 @@ static struct WMInitStruct {
 
 	int windowstate;
 	WinOverrideFlag override_flag;
-} wm_init_state = {0, 0, 0, 0, GHOST_kWindowStateNormal, 0};
+	
+	int native_pixels;
+} wm_init_state = {0, 0, 0, 0, GHOST_kWindowStateNormal, 0, 1};
 
 /* ******** win open & close ************ */
 
@@ -107,6 +115,17 @@ void wm_get_screensize(int *width_r, int *height_r)
 	*height_r = uiheight;
 }
 
+/* size of all screens, useful since the mouse is bound by this */
+void wm_get_screensize_all(int *width_r, int *height_r)
+{
+	unsigned int uiwidth;
+	unsigned int uiheight;
+
+	GHOST_GetAllDisplayDimensions(g_system, &uiwidth, &uiheight);
+	*width_r = uiwidth;
+	*height_r = uiheight;
+}
+
 /* keeps offset and size within monitor bounds */
 /* XXX solve dual screen... */
 static void wm_window_check_position(rcti *rect)
@@ -164,7 +183,7 @@ void wm_window_free(bContext *C, wmWindowManager *wm, wmWindow *win)
 
 		if (CTX_wm_window(C) == win)
 			CTX_wm_window_set(C, NULL);
-	}	
+	}
 
 	/* always set drawable and active to NULL,
 	 * prevents non-drawable state of main windows (bugs #22967 and #25071, possibly #22477 too) */
@@ -238,10 +257,10 @@ wmWindow *wm_window_copy(bContext *C, wmWindow *winorig)
 	BLI_strncpy(win->screenname, win->screen->id.name + 2, sizeof(win->screenname));
 	win->screen->winid = win->winid;
 
-	win->screen->do_refresh = 1;
-	win->screen->do_draw = 1;
+	win->screen->do_refresh = TRUE;
+	win->screen->do_draw = TRUE;
 
-	win->drawmethod = -1;
+	win->drawmethod = U.wmdrawmethod;
 	win->drawdata = NULL;
 	
 	return win;
@@ -251,67 +270,65 @@ wmWindow *wm_window_copy(bContext *C, wmWindow *winorig)
 void wm_window_close(bContext *C, wmWindowManager *wm, wmWindow *win)
 {
 	wmWindow *tmpwin;
-	bScreen *screen = win->screen;
+	int do_exit = 0;
+	
+	/* first check if we have to quit (there are non-temp remaining windows) */
+	for (tmpwin = wm->windows.first; tmpwin; tmpwin = tmpwin->next) {
+		if (tmpwin == win)
+			continue;
+		if (tmpwin->screen->temp == 0)
+			break;
+	}
+
+	if (tmpwin == NULL)
+		do_exit = 1;
 	
-	/* first check if we have any non-temp remaining windows */
 	if ((U.uiflag & USER_QUIT_PROMPT) && !wm->file_saved) {
-		if (wm->windows.first) {
-			for (tmpwin = wm->windows.first; tmpwin; tmpwin = tmpwin->next) {
-				if (tmpwin == win)
-					continue;
-				if (tmpwin->screen->temp == 0)
-					break;
-			}
-			if (tmpwin == NULL) {
-				if (!GHOST_confirmQuit(win->ghostwin))
-					return;
-			}
+		if (do_exit) {
+			if (!GHOST_confirmQuit(win->ghostwin))
+				return;
 		}
 	}
 
-	BLI_remlink(&wm->windows, win);
-	
-	wm_draw_window_clear(win);
-	CTX_wm_window_set(C, win);  /* needed by handlers */
-	WM_event_remove_handlers(C, &win->handlers);
-	WM_event_remove_handlers(C, &win->modalhandlers);
-	ED_screen_exit(C, win, win->screen); 
-	
-	wm_window_free(C, wm, win);
-	
-	/* if temp screen, delete it after window free (it stops jobs that can access it) */
-	if (screen->temp) {
-		Main *bmain = CTX_data_main(C);
-		free_libblock(&bmain->screen, screen);
-	}
-	
-	/* check remaining windows */
-	if (wm->windows.first) {
-		for (win = wm->windows.first; win; win = win->next)
-			if (win->screen->temp == 0)
-				break;
-		/* in this case we close all */
-		if (win == NULL)
-			WM_exit(C);
-	}
-	else
+	/* let WM_exit do all freeing, for correct quit.blend save */
+	if (do_exit) {
 		WM_exit(C);
+	}
+	else {
+		bScreen *screen = win->screen;
+		
+		BLI_remlink(&wm->windows, win);
+		
+		wm_draw_window_clear(win);
+		
+		CTX_wm_window_set(C, win);  /* needed by handlers */
+		WM_event_remove_handlers(C, &win->handlers);
+		WM_event_remove_handlers(C, &win->modalhandlers);
+		ED_screen_exit(C, win, win->screen); 
+		
+		wm_window_free(C, wm, win);
+	
+		/* if temp screen, delete it after window free (it stops jobs that can access it) */
+		if (screen->temp) {
+			Main *bmain = CTX_data_main(C);
+			BKE_libblock_free(&bmain->screen, screen);
+		}
+	}		
 }
 
 void wm_window_title(wmWindowManager *wm, wmWindow *win)
 {
-	/* handle the 'temp' window, only set title when not set before */
 	if (win->screen && win->screen->temp) {
-		char *title = GHOST_GetTitle(win->ghostwin);
-		if (title == NULL || title[0] == 0)
-			GHOST_SetTitle(win->ghostwin, "Blender");
+		/* nothing to do for 'temp' windows,
+		 * because WM_window_open_temp always sets window title  */
 	}
 	else {
 		
 		/* this is set to 1 if you don't have startup.blend open */
 		if (G.save_over && G.main->name[0]) {
-			char str[sizeof(G.main->name) + 12];
-			BLI_snprintf(str, sizeof(str), "Blender%s [%s]", wm->file_saved ? "" : "*", G.main->name);
+			char str[sizeof(G.main->name) + 24];
+			BLI_snprintf(str, sizeof(str), "Blender%s [%s%s]", wm->file_saved ? "" : "*", G.main->name,
+			             G.main->recovered ? " (Recovered)" : "");
 			GHOST_SetTitle(win->ghostwin, str);
 		}
 		else
@@ -335,27 +352,27 @@ void wm_window_title(wmWindowManager *wm, wmWindow *win)
 static void wm_window_add_ghostwindow(const char *title, wmWindow *win)
 {
 	GHOST_WindowHandle ghostwin;
+	static int multisamples = -1;
 	int scr_w, scr_h, posy;
 	
+	/* force setting multisamples only once, it requires restart - and you cannot 
+	 * mix it, either all windows have it, or none (tested in OSX opengl) */
+	if (multisamples == -1)
+		multisamples = U.ogl_multisamples;
+	
 	wm_get_screensize(&scr_w, &scr_h);
 	posy = (scr_h - win->posy - win->sizey);
 	
-#if defined(__APPLE__) && !defined(GHOST_COCOA)
-	{
-		extern int macPrefState; /* creator.c */
-		initial_state += macPrefState;
-	}
-#endif
-	/* Disable AA for now, as GL_SELECT (used for border, lasso, ... select)
-	 * doesn't work well when AA is initialized, even if not used. */
 	ghostwin = GHOST_CreateWindow(g_system, title,
 	                              win->posx, posy, win->sizex, win->sizey,
 	                              (GHOST_TWindowState)win->windowstate,
 	                              GHOST_kDrawingContextTypeOpenGL,
 	                              0 /* no stereo */,
-	                              0 /* no AA */);
+	                              multisamples /* AA */);
 	
 	if (ghostwin) {
+		GHOST_RectangleHandle bounds;
+		
 		/* needed so we can detect the graphics card below */
 		GPU_extensions_init();
 		
@@ -368,20 +385,31 @@ static void wm_window_add_ghostwindow(const char *title, wmWindow *win)
 		if (win->eventstate == NULL)
 			win->eventstate = MEM_callocN(sizeof(wmEvent), "window event state");
 		
-		/* until screens get drawn, make it nice grey */
-		glClearColor(.55, .55, .55, 0.0);
+		/* until screens get drawn, make it nice gray */
+		glClearColor(0.55, 0.55, 0.55, 0.0);
 		/* Crash on OSS ATI: bugs.launchpad.net/ubuntu/+source/mesa/+bug/656100 */
 		if (!GPU_type_matches(GPU_DEVICE_ATI, GPU_OS_UNIX, GPU_DRIVER_OPENSOURCE)) {
 			glClear(GL_COLOR_BUFFER_BIT);
 		}
+		
+		/* displays with larger native pixels, like Macbook. Used to scale dpi with */
+		/* needed here, because it's used before it reads userdef */
+		U.pixelsize = GHOST_GetNativePixelSize(win->ghostwin);
+		BKE_userdef_state();
+		
+		/* store actual window size in blender window */
+		bounds = GHOST_GetClientBounds(win->ghostwin);
+		win->sizex = GHOST_GetWidthRectangle(bounds);
+		win->sizey = GHOST_GetHeightRectangle(bounds);
+		GHOST_DisposeRectangle(bounds);
 
+		
 		wm_window_swap_buffers(win);
 		
 		//GHOST_SetWindowState(ghostwin, GHOST_kWindowStateModified);
 		
 		/* standard state vars for window */
 		glEnable(GL_SCISSOR_TEST);
-		
 		GPU_state_init();
 	}
 }
@@ -402,17 +430,26 @@ void wm_window_add_ghostwindows(wmWindowManager *wm)
 		wm_get_screensize(&wm_init_state.size_x, &wm_init_state.size_y);
 		
 #if defined(__APPLE__) && !defined(GHOST_COCOA)
-//Cocoa provides functions to get correct max window size
+		/* Cocoa provides functions to get correct max window size */
 		{
 			extern void wm_set_apple_prefsize(int, int);    /* wm_apple.c */
 			
 			wm_set_apple_prefsize(wm_init_state.size_x, wm_init_state.size_y);
 		}
 #else
+		/* note!, this isnt quite correct, active screen maybe offset 1000s if PX,
+		 * we'd need a wm_get_screensize like function that gives offset,
+		 * in practice the window manager will likely move to the correct monitor */
 		wm_init_state.start_x = 0;
 		wm_init_state.start_y = 0;
-		
 #endif
+
+#if !defined(__APPLE__) && !defined(WIN32)  /* X11 */
+		/* X11, start maximized but use default same size */
+		wm_init_state.size_x = min_ii(wm_init_state.size_x, WM_WIN_INIT_SIZE_X);
+		wm_init_state.size_y = min_ii(wm_init_state.size_y, WM_WIN_INIT_SIZE_Y);
+#endif
+
 	}
 	
 	for (win = wm->windows.first; win; win = win->next) {
@@ -422,6 +459,20 @@ void wm_window_add_ghostwindows(wmWindowManager *wm)
 				win->posy = wm_init_state.start_y;
 				win->sizex = wm_init_state.size_x;
 				win->sizey = wm_init_state.size_y;
+
+#if !defined(__APPLE__) && !defined(WIN32)  /* X11 */
+				if (wm_init_state.override_flag & WIN_OVERRIDE_GEOM) {
+					/* we can't properly resize a maximized window */
+					win->windowstate = GHOST_kWindowStateNormal;
+				}
+				else {
+					/* loading without userpref, default to maximized */
+					win->windowstate = GHOST_kWindowStateMaximized;
+				}
+#else
+				win->windowstate = GHOST_kWindowStateNormal;
+#endif
+
 				wm_init_state.override_flag &= ~WIN_OVERRIDE_GEOM;
 			}
 
@@ -464,10 +515,10 @@ wmWindow *WM_window_open(bContext *C, rcti *rect)
 	
 	win->posx = rect->xmin;
 	win->posy = rect->ymin;
-	win->sizex = rect->xmax - rect->xmin;
-	win->sizey = rect->ymax - rect->ymin;
+	win->sizex = BLI_rcti_size_x(rect);
+	win->sizey = BLI_rcti_size_y(rect);
 
-	win->drawmethod = -1;
+	win->drawmethod = U.wmdrawmethod;
 	win->drawdata = NULL;
 	
 	WM_check(C);
@@ -477,7 +528,7 @@ wmWindow *WM_window_open(bContext *C, rcti *rect)
 
 /* uses screen->temp tag to define what to do, currently it limits
  * to only one "temp" window for render out, preferences, filewindow, etc */
-/* type is #define in WM_api.h */
+/* type is defined in WM_api.h */
 
 void WM_window_open_temp(bContext *C, rcti *position, int type)
 {
@@ -500,8 +551,8 @@ void WM_window_open_temp(bContext *C, rcti *position, int type)
 		win->posy = position->ymin;
 	}
 	
-	win->sizex = position->xmax - position->xmin;
-	win->sizey = position->ymax - position->ymin;
+	win->sizex = BLI_rcti_size_x(position);
+	win->sizey = BLI_rcti_size_y(position);
 	
 	if (win->ghostwin) {
 		wm_window_set_size(win, win->sizex, win->sizey);
@@ -529,6 +580,7 @@ void WM_window_open_temp(bContext *C, rcti *position, int type)
 	}
 	
 	ED_screen_set(C, win->screen);
+	ED_screen_refresh(CTX_wm_manager(C), win); /* test scale */
 	
 	if (sa->spacetype == SPACE_IMAGE)
 		GHOST_SetTitle(win->ghostwin, IFACE_("Blender Render"));
@@ -577,12 +629,29 @@ int wm_window_fullscreen_toggle_exec(bContext *C, wmOperator *UNUSED(op))
 
 /* ************ events *************** */
 
-typedef enum
+static void wm_convert_cursor_position(wmWindow *win, int *x, int *y)
 {
-	SHIFT = 's',
-	CONTROL = 'c',
-	ALT = 'a',
-	OS = 'C'
+	float fac = GHOST_GetNativePixelSize(win->ghostwin);
+	
+	GHOST_ScreenToClient(win->ghostwin, *x, *y, x, y);
+	*x *= fac;
+	
+	*y = (win->sizey - 1) - *y;
+	*y *= fac;
+}
+
+
+void wm_get_cursor_position(wmWindow *win, int *x, int *y)
+{
+	GHOST_GetCursorPosition(g_system, x, y);
+	wm_convert_cursor_position(win, x, y);
+}
+
+typedef enum {
+	SHIFT    = 's',
+	CONTROL  = 'c',
+	ALT      = 'a',
+	OS       = 'C'
 } modifierKeyType;
 
 /* check if specified modifier key type is pressed */
@@ -622,20 +691,25 @@ void wm_window_make_drawable(bContext *C, wmWindow *win)
 	wmWindowManager *wm = CTX_wm_manager(C);
 
 	if (win != wm->windrawable && win->ghostwin) {
-//		win->lmbut= 0;	/* keeps hanging when mousepressed while other window opened */
+//		win->lmbut = 0;	/* keeps hanging when mousepressed while other window opened */
 		
 		wm->windrawable = win;
 		if (G.debug & G_DEBUG_EVENTS) {
 			printf("%s: set drawable %d\n", __func__, win->winid);
 		}
 		GHOST_ActivateWindowDrawingContext(win->ghostwin);
+		
+		/* this can change per window */
+		U.pixelsize = GHOST_GetNativePixelSize(win->ghostwin);
+		BKE_userdef_state();
 	}
 }
 
 /* called by ghost, here we handle events for windows themselves or send to event system */
-static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private) 
+/* mouse coordinate converversion happens here */
+static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr)
 {
-	bContext *C = private;
+	bContext *C = C_void_ptr;
 	wmWindowManager *wm = CTX_wm_manager(C);
 	GHOST_TEventType type = GHOST_GetEventType(evt);
 	int time = GHOST_GetEventTime(evt);
@@ -649,15 +723,15 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 		wmWindow *win;
 		
 		if (!ghostwin) {
-			// XXX - should be checked, why are we getting an event here, and
-			//	what is it?
+			/* XXX - should be checked, why are we getting an event here, and */
+			/* what is it? */
 			puts("<!> event has no window");
 			return 1;
 		}
 		else if (!GHOST_ValidWindow(g_system, ghostwin)) {
-			// XXX - should be checked, why are we getting an event here, and
-			//	what is it?
-			puts("<!> event has invalid window");			
+			/* XXX - should be checked, why are we getting an event here, and */
+			/* what is it? */
+			puts("<!> event has invalid window");
 			return 1;
 		}
 		else {
@@ -668,11 +742,20 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 			case GHOST_kEventWindowDeactivate:
 				wm_event_add_ghostevent(wm, win, type, time, data);
 				win->active = 0; /* XXX */
+				
+				/* clear modifiers for inactive windows */
+				win->eventstate->alt = 0;
+				win->eventstate->ctrl = 0;
+				win->eventstate->shift = 0;
+				win->eventstate->oskey = 0;
+				win->eventstate->keymodifier = 0;
+
 				break;
 			case GHOST_kEventWindowActivate: 
 			{
 				GHOST_TEventKeyData kdata;
-				int cx, cy, wx, wy;
+				wmEvent event;
+				int wx, wy;
 				
 				wm->winactive = win; /* no context change! c->wm->windrawable is drawable, or for area queues */
 				
@@ -680,6 +763,9 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 //				window_handle(win, INPUTCHANGE, win->active);
 				
 				/* bad ghost support for modifier keys... so on activate we set the modifiers again */
+
+				/* TODO: This is not correct since a modifier may be held when a window is activated...
+				 * better solve this at ghost level. attempted fix r54450 but it caused bug [#34255] */
 				kdata.ascii = '\0';
 				kdata.utf8_buf[0] = '\0';
 				if (win->eventstate->shift && !query_qual(SHIFT)) {
@@ -702,22 +788,40 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 				win->eventstate->keymodifier = 0;
 				
 				/* entering window, update mouse pos. but no event */
-				GHOST_GetCursorPosition(g_system, &wx, &wy);
-				
-				GHOST_ScreenToClient(win->ghostwin, wx, wy, &cx, &cy);
-				win->eventstate->x = cx;
-				win->eventstate->y = (win->sizey - 1) - cy;
+				wm_get_cursor_position(win,  &wx, &wy);
+
+				win->eventstate->x = wx;
+				win->eventstate->y = wy;
 				
 				win->addmousemove = 1;   /* enables highlighted buttons */
 				
 				wm_window_make_drawable(C, win);
+
+				/* window might be focused by mouse click in configuration of window manager
+				 * when focus is not following mouse
+				 * click could have been done on a button and depending on window manager settings
+				 * click would be passed to blender or not, but in any case button under cursor
+				 * should be activated, so at max next click on button without moving mouse
+				 * would trigger it's handle function
+				 * currently it seems to be common practice to generate new event for, but probably
+				 * we'll need utility function for this? (sergey)
+				 */
+				event = *(win->eventstate);
+				event.type = MOUSEMOVE;
+				event.prevx = event.x;
+				event.prevy = event.y;
+
+				wm_event_add(win, &event);
+
 				break;
 			}
-			case GHOST_kEventWindowClose: {
+			case GHOST_kEventWindowClose:
+			{
 				wm_window_close(C, wm, win);
 				break;
 			}
-			case GHOST_kEventWindowUpdate: {
+			case GHOST_kEventWindowUpdate:
+			{
 				if (G.debug & G_DEBUG_EVENTS) {
 					printf("%s: ghost redraw %d\n", __func__, win->winid);
 				}
@@ -728,11 +832,17 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 				break;
 			}
 			case GHOST_kEventWindowSize:
-			case GHOST_kEventWindowMove: {
+			case GHOST_kEventWindowMove:
+			{
 				GHOST_TWindowState state;
 				state = GHOST_GetWindowState(win->ghostwin);
 				win->windowstate = state;
 
+				/* stop screencast if resize */
+				if (type == GHOST_kEventWindowSize) {
+					WM_jobs_stop(CTX_wm_manager(C), win->screen, NULL);
+				}
+				
 				/* win32: gives undefined window size when minimized */
 				if (state != GHOST_kWindowStateMinimized) {
 					GHOST_RectangleHandle client_rect;
@@ -744,7 +854,7 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 					
 					GHOST_DisposeRectangle(client_rect);
 					
-					wm_get_screensize(&scr_w, &scr_h);
+					wm_get_screensize_all(&scr_w, &scr_h);
 					sizex = r - l;
 					sizey = b - t;
 					posx = l;
@@ -831,18 +941,16 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 			{
 				wmEvent event;
 				GHOST_TEventDragnDropData *ddd = GHOST_GetEventData(evt);
-				int cx, cy, wx, wy;
+				int wx, wy;
 				
 				/* entering window, update mouse pos */
-				GHOST_GetCursorPosition(g_system, &wx, &wy);
-				
-				GHOST_ScreenToClient(win->ghostwin, wx, wy, &cx, &cy);
-				win->eventstate->x = cx;
-				win->eventstate->y = (win->sizey - 1) - cy;
+				wm_get_cursor_position(win, &wx, &wy);
+				win->eventstate->x = wx;
+				win->eventstate->y = wy;
 				
 				event = *(win->eventstate);  /* copy last state, like mouse coords */
 				
-				// activate region
+				/* activate region */
 				event.type = MOUSEMOVE;
 				event.prevx = event.x;
 				event.prevy = event.y;
@@ -877,15 +985,37 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private)
 						
 						WM_event_start_drag(C, icon, WM_DRAG_PATH, stra->strings[a], 0.0);
 						/* void poin should point to string, it makes a copy */
-						break; // only one drop element supported now 
+						break; /* only one drop element supported now */
 					}
 				}
 				
-				
-				
 				break;
 			}
-			
+			case GHOST_kEventNativeResolutionChange:
+				// printf("change, pixel size %f\n", GHOST_GetNativePixelSize(win->ghostwin));
+				
+				U.pixelsize = GHOST_GetNativePixelSize(win->ghostwin);
+				BKE_userdef_state();
+				WM_event_add_notifier(C, NC_SCREEN | NA_EDITED, NULL);
+				WM_event_add_notifier(C, NC_WINDOW | NA_EDITED, NULL);
+
+				break;
+			case GHOST_kEventTrackpad:
+			{
+				GHOST_TEventTrackpadData *pd = data;
+				
+				wm_convert_cursor_position(win, &pd->x, &pd->y);
+				wm_event_add_ghostevent(wm, win, type, time, data);
+				break;
+			}
+			case GHOST_kEventCursorMove:
+			{
+				GHOST_TEventCursorData *cd = data;
+				
+				wm_convert_cursor_position(win, &cd->x, &cd->y);
+				wm_event_add_ghostevent(wm, win, type, time, data);
+				break;
+			}
 			default:
 				wm_event_add_ghostevent(wm, win, type, time, data);
 				break;
@@ -914,12 +1044,12 @@ static int wm_window_timer(const bContext *C)
 		wtnext = wt->next; /* in case timer gets removed */
 		win = wt->win;
 
-		if (wt->sleep== 0) {
+		if (wt->sleep == 0) {
 			if (time > wt->ntime) {
 				wt->delta = time - wt->ltime;
 				wt->duration += wt->delta;
 				wt->ltime = time;
-				wt->ntime = wt->stime + wt->timestep *ceil(wt->duration / wt->timestep);
+				wt->ntime = wt->stime + wt->timestep * ceil(wt->duration / wt->timestep);
 
 				if (wt->event_type == TIMERJOBS)
 					wm_jobs_timer(C, wm, wt);
@@ -929,6 +1059,8 @@ static int wm_window_timer(const bContext *C)
 					wmEvent event = *(win->eventstate);
 					
 					event.type = wt->event_type;
+					event.val = 0;
+					event.keymodifier = 0;
 					event.custom = EVT_DATA_TIMER;
 					event.customdata = wt;
 					wm_event_add(win, &event);
@@ -943,8 +1075,12 @@ static int wm_window_timer(const bContext *C)
 
 void wm_window_process_events(const bContext *C) 
 {
-	int hasevent = GHOST_ProcessEvents(g_system, 0); /* 0 is no wait */
-	
+	int hasevent;
+
+	BLI_assert(BLI_thread_is_main());
+
+	hasevent = GHOST_ProcessEvents(g_system, 0); /* 0 is no wait */
+
 	if (hasevent)
 		GHOST_DispatchEvents(g_system);
 	
@@ -966,11 +1102,13 @@ void wm_window_testbreak(void)
 {
 	static double ltime = 0;
 	double curtime = PIL_check_seconds_timer();
-	
+
+	BLI_assert(BLI_thread_is_main());
+
 	/* only check for breaks every 50 milliseconds
 	 * if we get called more often.
 	 */
-	if ((curtime - ltime) > .05) {
+	if ((curtime - ltime) > 0.05) {
 		int hasevent = GHOST_ProcessEvents(g_system, 0); /* 0 is no wait */
 		
 		if (hasevent)
@@ -989,7 +1127,11 @@ void wm_ghost_init(bContext *C)
 		
 		g_system = GHOST_CreateSystem();
 		GHOST_AddEventConsumer(g_system, consumer);
-	}	
+		
+		if (wm_init_state.native_pixels) {
+			GHOST_UseNativePixels();
+		}
+	}
 }
 
 void wm_ghost_exit(void)
@@ -1040,6 +1182,8 @@ void WM_event_remove_timer(wmWindowManager *wm, wmWindow *UNUSED(win), wmTimer *
 		if (wt == timer)
 			break;
 	if (wt) {
+		wmWindow *win;
+		
 		if (wm->reports.reporttimer == wt)
 			wm->reports.reporttimer = NULL;
 		
@@ -1047,6 +1191,17 @@ void WM_event_remove_timer(wmWindowManager *wm, wmWindow *UNUSED(win), wmTimer *
 		if (wt->customdata)
 			MEM_freeN(wt->customdata);
 		MEM_freeN(wt);
+		
+		/* there might be events in queue with this timer as customdata */
+		for (win = wm->windows.first; win; win = win->next) {
+			wmEvent *event;
+			for (event = win->queue.first; event; event = event->next) {
+				if (event->customdata == wt) {
+					event->customdata = NULL;
+					event->type = EVENT_NONE;	/* timer users customdata, dont want NULL == NULL */
+				}
+			}
+		}
 	}
 }
 
@@ -1064,7 +1219,7 @@ char *WM_clipboard_text_get(int selection)
 		return NULL;
 	
 	/* always convert from \r\n to \n */
-	newbuf = MEM_callocN(strlen(buf) + 1, "WM_clipboard_text_get");
+	newbuf = MEM_callocN(strlen(buf) + 1, __func__);
 
 	for (p = buf, p2 = newbuf; *p; p++) {
 		if (*p != '\r')
@@ -1130,23 +1285,6 @@ void wm_window_get_position(wmWindow *win, int *posx_r, int *posy_r)
 	*posy_r = win->posy;
 }
 
-void wm_window_get_size(wmWindow *win, int *width_r, int *height_r) 
-{
-	*width_r = win->sizex;
-	*height_r = win->sizey;
-}
-
-/* exceptional case: - splash is called before events are processed
- * this means we don't actually know the window size so get this from GHOST */
-void wm_window_get_size_ghost(wmWindow *win, int *width_r, int *height_r)
-{
-	GHOST_RectangleHandle bounds = GHOST_GetClientBounds(win->ghostwin);
-	*width_r = GHOST_GetWidthRectangle(bounds);
-	*height_r = GHOST_GetHeightRectangle(bounds);
-	
-	GHOST_DisposeRectangle(bounds);
-}
-
 void wm_window_set_size(wmWindow *win, int width, int height) 
 {
 	GHOST_SetClientSize(win->ghostwin, width, height);
@@ -1174,45 +1312,47 @@ void wm_window_swap_buffers(wmWindow *win)
 #endif
 }
 
-void wm_get_cursor_position(wmWindow *win, int *x, int *y)
-{
-	GHOST_GetCursorPosition(g_system, x, y);
-	GHOST_ScreenToClient(win->ghostwin, *x, *y, x, y);
-	*y = (win->sizey - 1) - *y;
-}
 
 /* ******************* exported api ***************** */
 
 
 /* called whem no ghost system was initialized */
-void WM_setprefsize(int stax, int stay, int sizx, int sizy)
+void WM_init_state_size_set(int stax, int stay, int sizx, int sizy)
 {
 	wm_init_state.start_x = stax; /* left hand pos */
 	wm_init_state.start_y = stay; /* bottom pos */
-	wm_init_state.size_x = sizx;
-	wm_init_state.size_y = sizy;
+	wm_init_state.size_x = sizx < 640 ? 640 : sizx;
+	wm_init_state.size_y = sizy < 480 ? 480 : sizy;
 	wm_init_state.override_flag |= WIN_OVERRIDE_GEOM;
 }
 
 /* for borderless and border windows set from command-line */
-void WM_setinitialstate_fullscreen(void)
+void WM_init_state_fullscreen_set(void)
 {
 	wm_init_state.windowstate = GHOST_kWindowStateFullScreen;
 	wm_init_state.override_flag |= WIN_OVERRIDE_WINSTATE;
 }
 
-void WM_setinitialstate_normal(void)
+void WM_init_state_normal_set(void)
 {
 	wm_init_state.windowstate = GHOST_kWindowStateNormal;
 	wm_init_state.override_flag |= WIN_OVERRIDE_WINSTATE;
 }
 
+void WM_init_native_pixels(int do_it)
+{
+	wm_init_state.native_pixels = do_it;
+}
+
 /* This function requires access to the GHOST_SystemHandle (g_system) */
 void WM_cursor_warp(wmWindow *win, int x, int y)
 {
 	if (win && win->ghostwin) {
+		float f = GHOST_GetNativePixelSize(win->ghostwin);
 		int oldx = x, oldy = y;
 
+		x = x / f;
+		y = y / f;
 		y = win->sizey - y - 1;
 
 		GHOST_ClientToScreen(win->ghostwin, x, y, &x, &y);
@@ -1223,3 +1363,34 @@ void WM_cursor_warp(wmWindow *win, int x, int y)
 	}
 }
 
+/**
+ * Get the cursor pressure, in most cases you'll want to use wmTabletData from the event
+ */
+float WM_cursor_pressure(const struct wmWindow *win)
+{
+	const GHOST_TabletData *td = GHOST_GetTabletData(win->ghostwin);
+	/* if there's tablet data from an active tablet device then add it */
+	if ((td != NULL) && td->Active != GHOST_kTabletModeNone) {
+		return td->Pressure;
+	}
+	else {
+		return -1.0f;
+	}
+}
+
+/* support for native pixel size */
+/* mac retina opens window in size X, but it has up to 2 x more pixels */
+int WM_window_pixels_x(wmWindow *win)
+{
+	float f = GHOST_GetNativePixelSize(win->ghostwin);
+	
+	return (int)(f * (float)win->sizex);
+}
+
+int WM_window_pixels_y(wmWindow *win)
+{
+	float f = GHOST_GetNativePixelSize(win->ghostwin);
+	
+	return (int)(f * (float)win->sizey);
+	
+}
