@@ -31,7 +31,13 @@
 #ifndef __WM_FILES_H__
 #define __WM_FILES_H__
 
-void WM_read_history(void);
+void		wm_read_history(void);
+int			wm_file_write(struct bContext *C, const char *target, int fileflags, struct ReportList *reports);
+int			wm_homefile_read_exec(struct bContext *C, struct wmOperator *op);
+int			wm_homefile_read(struct bContext *C, struct ReportList *reports, short from_memory);
+int			wm_homefile_write_exec(struct bContext *C, struct wmOperator *op);
+int			wm_userpref_write_exec(struct bContext *C, struct wmOperator *op);
+
 
 #endif /* __WM_FILES_H__ */
 
