@@ -31,23 +31,38 @@
 
 
 #if defined(__linux__) && defined(__GNUC__)
-#define _GNU_SOURCE
-#include <fenv.h>
+#  define _GNU_SOURCE
+#  include <fenv.h>
 #endif
 
 #if (defined(__APPLE__) && (defined(__i386__) || defined(__x86_64__)))
-#define OSX_SSE_FPE
-#include <xmmintrin.h>
+#  define OSX_SSE_FPE
+#  include <xmmintrin.h>
+#endif
+
+/* crash handler */
+#ifdef WIN32
+#  include <process.h> /* getpid */
+#else
+#  include <unistd.h> /* getpid */
 #endif
 
 #ifdef WIN32
-#include <Windows.h>
-#include "utfconv.h"
+#  include <windows.h>
+#  include "utfconv.h"
+#endif
+
+/* for backtrace */
+#if defined(__linux__) || defined(__APPLE__)
+#  include <execinfo.h>
+#elif defined(_MSV_VER)
+#  include <DbgHelp.h>
 #endif
 
 #include <stdlib.h>
 #include <stddef.h>
 #include <string.h>
+#include <errno.h>
 
 /* This little block needed for linking to Blender... */
 
@@ -68,12 +83,12 @@
 
 #include "BLI_blenlib.h"
 
-#include "BKE_utildefines.h"
 #include "BKE_blender.h"
 #include "BKE_context.h"
 #include "BKE_depsgraph.h" /* for DAG_on_visible_update */
 #include "BKE_font.h"
 #include "BKE_global.h"
+#include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_material.h"
 #include "BKE_packedFile.h"
@@ -92,7 +107,6 @@
 #include "RE_engine.h"
 #include "RE_pipeline.h"
 
-//XXX #include "playanim_ext.h"
 #include "ED_datafiles.h"
 
 #include "WM_api.h"
@@ -105,30 +119,30 @@
 #include "BLI_scanfill.h" /* for BLI_setErrorCallBack, TODO, move elsewhere */
 
 #ifdef WITH_BUILDINFO_HEADER
-#define BUILD_DATE
+#  define BUILD_DATE
 #endif
 
 /* for passing information between creator and gameengine */
 #ifdef WITH_GAMEENGINE
-#include "BL_System.h"
+#  include "BL_System.h"
 #else /* dummy */
-#define SYS_SystemHandle int
+#  define SYS_SystemHandle int
 #endif
 
 #include <signal.h>
 
 #ifdef __FreeBSD__
-# include <sys/types.h>
-# include <floatingpoint.h>
-# include <sys/rtprio.h>
+#  include <sys/types.h>
+#  include <floatingpoint.h>
+#  include <sys/rtprio.h>
 #endif
 
 #ifdef WITH_BINRELOC
-#include "binreloc.h"
+#  include "binreloc.h"
 #endif
 
 #ifdef WITH_LIBMV
-#include "libmv-capi.h"
+#  include "libmv-capi.h"
 #endif
 
 /* from buildinfo.c */
@@ -145,20 +159,28 @@ extern char build_system[];
 #endif
 
 /*	Local Function prototypes */
+#ifdef WITH_PYTHON_MODULE
+int  main_python_enter(int argc, const char **argv);
+void main_python_exit(void);
+#else
 static int print_help(int argc, const char **argv, void *data);
 static int print_version(int argc, const char **argv, void *data);
+#endif
 
 /* for the callbacks: */
 
-extern int pluginapi_force_ref(void);  /* from blenpluginapi:pluginapi.c */
-
-#define BLEND_VERSION_STRING_FMT                                              \
-	"Blender %d.%02d (sub %d)\n",                                             \
-	BLENDER_VERSION/100, BLENDER_VERSION%100, BLENDER_SUBVERSION              \
+#define BLEND_VERSION_FMT         "Blender %d.%02d (sub %d)"
+#define BLEND_VERSION_ARG         BLENDER_VERSION / 100, BLENDER_VERSION % 100, BLENDER_SUBVERSION
+/* pass directly to printf */
+#define BLEND_VERSION_STRING_FMT  BLEND_VERSION_FMT "\n", BLEND_VERSION_ARG
 
 /* Initialize callbacks for the modules that need them */
 static void setCallbacks(void); 
 
+#ifndef WITH_PYTHON_MODULE
+
+static bool use_crash_handler = true;
+
 /* set breakpoints here when running in debug mode, useful to catch floating point errors */
 #if defined(__linux__) || defined(_WIN32) || defined(OSX_SSE_FPE)
 static void fpe_handler(int UNUSED(sig))
@@ -167,13 +189,12 @@ static void fpe_handler(int UNUSED(sig))
 }
 #endif
 
-#ifndef WITH_PYTHON_MODULE
 /* handling ctrl-c event in console */
 static void blender_esc(int sig)
 {
 	static int count = 0;
 	
-	G.afbreek = 1;  /* forces render loop to read queue, not sure if its needed */
+	G.is_break = TRUE;  /* forces render loop to read queue, not sure if its needed */
 	
 	if (sig == 2) {
 		if (count) {
@@ -184,7 +205,6 @@ static void blender_esc(int sig)
 		count++;
 	}
 }
-#endif
 
 static int print_version(int UNUSED(argc), const char **UNUSED(argv), void *UNUSED(data))
 {
@@ -248,6 +268,7 @@ static int print_help(int UNUSED(argc), const char **UNUSED(argv), void *data)
 	printf("Misc Options:\n");
 	BLI_argsPrintArgDoc(ba, "--debug");
 	BLI_argsPrintArgDoc(ba, "--debug-fpe");
+	BLI_argsPrintArgDoc(ba, "--disable-crash-handler");
 
 #ifdef WITH_FFMPEG
 	BLI_argsPrintArgDoc(ba, "--debug-ffmpeg");
@@ -263,7 +284,6 @@ static int print_help(int UNUSED(argc), const char **UNUSED(argv), void *data)
 	BLI_argsPrintArgDoc(ba, "--env-system-config");
 	BLI_argsPrintArgDoc(ba, "--env-system-datafiles");
 	BLI_argsPrintArgDoc(ba, "--env-system-scripts");
-	BLI_argsPrintArgDoc(ba, "--env-system-plugins");
 	BLI_argsPrintArgDoc(ba, "--env-system-python");
 	printf("\n");
 	BLI_argsPrintArgDoc(ba, "-nojoystick");
@@ -283,6 +303,7 @@ static int print_help(int UNUSED(argc), const char **UNUSED(argv), void *data)
 	printf("\n");
 
 	BLI_argsPrintArgDoc(ba, "--python");
+	BLI_argsPrintArgDoc(ba, "--python-text");
 	BLI_argsPrintArgDoc(ba, "--python-console");
 	BLI_argsPrintArgDoc(ba, "--addons");
 
@@ -353,6 +374,12 @@ static int disable_python(int UNUSED(argc), const char **UNUSED(argv), void *UNU
 	return 0;
 }
 
+static int disable_crash_handler(int UNUSED(argc), const char **UNUSED(argv), void *UNUSED(data))
+{
+	use_crash_handler = false;
+	return 0;
+}
+
 static int background_mode(int UNUSED(argc), const char **UNUSED(argv), void *UNUSED(data))
 {
 	G.background = 1;
@@ -388,6 +415,19 @@ static int debug_mode_libmv(int UNUSED(argc), const char **UNUSED(argv), void *U
 }
 #endif
 
+static int set_debug_value(int argc, const char **argv, void *UNUSED(data))
+{
+	if (argc > 1) {
+		G.debug_value = atoi(argv[1]);
+
+		return 1;
+	}
+	else {
+		printf("\nError: you must specify debug value to set.\n");
+		return 0;
+	}
+}
+
 static int set_fpe(int UNUSED(argc), const char **UNUSED(argv), void *UNUSED(data))
 {
 #if defined(__linux__) || defined(_WIN32) || defined(OSX_SSE_FPE)
@@ -413,6 +453,148 @@ static int set_fpe(int UNUSED(argc), const char **UNUSED(argv), void *UNUSED(dat
 	return 0;
 }
 
+#if defined(__linux__) || defined(__APPLE__)
+
+/* Unix */
+static void blender_crash_handler_backtrace(FILE *fp)
+{
+#define SIZE 100
+	void *buffer[SIZE];
+	int nptrs;
+	char **strings;
+	int i;
+
+	fputs("\n# backtrace\n", fp);
+
+	/* include a backtrace for good measure */
+	nptrs = backtrace(buffer, SIZE);
+	strings = backtrace_symbols(buffer, nptrs);
+	for (i = 0; i < nptrs; i++) {
+		fputs(strings[i], fp);
+		fputc('\n', fp);
+	}
+
+	free(strings);
+#undef SIZE
+}
+
+#elif defined(_MSV_VER)
+
+static void blender_crash_handler_backtrace(FILE *fp)
+{
+	(void)fp;
+
+#if 0
+#define MAXSYMBOL 256
+	unsigned short	i;
+	void *stack[SIZE];
+	unsigned short nframes;
+	SYMBOL_INFO	*symbolinfo;
+	HANDLE process;
+
+	process = GetCurrentProcess();
+
+	SymInitialize(process, NULL, TRUE);
+
+	nframes = CaptureStackBackTrace(0, SIZE, stack, NULL);
+	symbolinfo = MEM_callocN(sizeof(SYMBOL_INFO) + MAXSYMBOL * sizeof( char ), "crash Symbol table");
+	symbolinfo->MaxNameLen = MAXSYMBOL - 1;
+	symbolinfo->SizeOfStruct = sizeof(SYMBOL_INFO);
+
+	for (i = 0; i < nframes; i++) {
+		SymFromAddr(process, ( DWORD64 )( stack[ i ] ), 0, symbolinfo);
+
+		fprintf(fp, "%u: %s - 0x%0X\n", nframes - i - 1, symbolinfo->Name, symbolinfo->Address);
+	}
+
+	MEM_freeN(symbolinfo);
+#endif
+}
+
+#else  /* non msvc/osx/linux */
+
+static void blender_crash_handler_backtrace(FILE *fp)
+{
+	(void)fp;
+}
+
+#endif
+
+static void blender_crash_handler(int signum)
+{
+
+#if 0
+	{
+		char fname[FILE_MAX];
+
+		if (!G.main->name[0]) {
+			BLI_make_file_string("/", fname, BLI_temporary_dir(), "crash.blend");
+		}
+		else {
+			BLI_strncpy(fname, G.main->name, sizeof(fname));
+			BLI_replace_extension(fname, sizeof(fname), ".crash.blend");
+		}
+
+		printf("Writing: %s\n", fname);
+		fflush(stdout);
+
+		BKE_undo_save_file(fname);
+	}
+#endif
+
+	FILE *fp;
+	char header[512];
+	wmWindowManager *wm = G.main->wm.first;
+
+	char fname[FILE_MAX];
+
+	if (!G.main->name[0]) {
+		BLI_join_dirfile(fname, sizeof(fname), BLI_temporary_dir(), "blender.crash.txt");
+	}
+	else {
+		BLI_join_dirfile(fname, sizeof(fname), BLI_temporary_dir(), BLI_path_basename(G.main->name));
+		BLI_replace_extension(fname, sizeof(fname), ".crash.txt");
+	}
+
+	printf("Writing: %s\n", fname);
+	fflush(stdout);
+
+	BLI_snprintf(header, sizeof(header), "# " BLEND_VERSION_FMT ", Revision: %s\n", BLEND_VERSION_ARG,
+#ifdef BUILD_DATE
+	             build_rev
+#else
+	             "Unknown"
+#endif
+	             );
+
+	/* open the crash log */
+	errno = 0;
+	fp = BLI_fopen(fname, "wb");
+	if (fp == NULL) {
+		fprintf(stderr, "Unable to save '%s': %s\n",
+		        fname, errno ? strerror(errno) : "Unknown error opening file");
+	}
+	else {
+		if (wm) {
+			BKE_report_write_file_fp(fp, &wm->reports, header);
+		}
+
+		blender_crash_handler_backtrace(fp);
+
+		fclose(fp);
+	}
+
+
+	/* really crash */
+	signal(signum, SIG_DFL);
+#ifndef WIN32
+	kill(getpid(), signum);
+#else
+	TerminateProcess(GetCurrentProcess(), signum);
+#endif
+}
+
+
 static int set_factory_startup(int UNUSED(argc), const char **UNUSED(argv), void *UNUSED(data))
 {
 	G.factory_startup = 1;
@@ -441,16 +623,12 @@ static int set_env(int argc, const char **argv, void *UNUSED(data))
 	return 1;
 }
 
-static int playback_mode(int UNUSED(argc), const char **UNUSED(argv), void *UNUSED(data))
+static int playback_mode(int argc, const char **argv, void *UNUSED(data))
 {
 	/* not if -b was given first */
 	if (G.background == 0) {
-#if 0   /* TODO, bring player back? */
-		playanim(argc, argv); /* not the same argc and argv as before */
-#else
-		fprintf(stderr, "Playback mode not supported in blender 2.6x\n");
-		exit(0);
-#endif
+		WM_main_playanim(argc, argv); /* not the same argc and argv as before */
+		exit(0); /* 2.4x didn't do this */
 	}
 
 	return -2;
@@ -470,20 +648,26 @@ static int prefsize(int argc, const char **argv, void *UNUSED(data))
 	sizx = atoi(argv[3]);
 	sizy = atoi(argv[4]);
 
-	WM_setprefsize(stax, stay, sizx, sizy);
+	WM_init_state_size_set(stax, stay, sizx, sizy);
 
 	return 4;
 }
 
+static int native_pixels(int UNUSED(argc), const char **UNUSED(argv), void *UNUSED(data))
+{
+	WM_init_native_pixels(0);
+	return 0;
+}
+
 static int with_borders(int UNUSED(argc), const char **UNUSED(argv), void *UNUSED(data))
 {
-	WM_setinitialstate_normal();
+	WM_init_state_normal_set();
 	return 0;
 }
 
 static int without_borders(int UNUSED(argc), const char **UNUSED(argv), void *UNUSED(data))
 {
-	WM_setinitialstate_fullscreen();
+	WM_init_state_fullscreen_set();
 	return 0;
 }
 
@@ -603,7 +787,7 @@ static int set_engine(int argc, const char **argv, void *data)
 static int set_image_type(int argc, const char **argv, void *data)
 {
 	bContext *C = data;
-	if (argc >= 1) {
+	if (argc > 1) {
 		const char *imtype = argv[1];
 		Scene *scene = CTX_data_scene(C);
 		if (scene) {
@@ -629,7 +813,7 @@ static int set_image_type(int argc, const char **argv, void *data)
 
 static int set_threads(int argc, const char **argv, void *UNUSED(data))
 {
-	if (argc >= 1) {
+	if (argc > 1) {
 		if (G.background) {
 			RE_set_max_threads(atoi(argv[1]));
 		}
@@ -644,6 +828,25 @@ static int set_threads(int argc, const char **argv, void *UNUSED(data))
 	}
 }
 
+static int set_verbosity(int argc, const char **argv, void *UNUSED(data))
+{
+	if (argc > 1) {
+		int level = atoi(argv[1]);
+
+#ifdef WITH_LIBMV
+		libmv_setLoggingVerbosity(level);
+#else
+		(void)level;
+#endif
+
+		return 1;
+	}
+	else {
+		printf("\nError: you must specify a verbosity level.\n");
+		return 0;
+	}
+}
+
 static int set_extension(int argc, const char **argv, void *data)
 {
 	bContext *C = data;
@@ -722,7 +925,7 @@ static int set_ge_parameters(int argc, const char **argv, void *data)
 			}
 
 
-		} /* if (*(argv[a+1]) == '=') */
+		} /* if (*(argv[a + 1]) == '=') */
 	}
 
 	return a;
@@ -795,7 +998,7 @@ static int set_scene(int argc, const char **argv, void *data)
 {
 	if (argc > 1) {
 		bContext *C = data;
-		Scene *scene = set_scene_name(CTX_data_main(C), argv[1]);
+		Scene *scene = BKE_scene_set_name(CTX_data_main(C), argv[1]);
 		if (scene) {
 			CTX_data_scene_set(C, scene);
 		}
@@ -888,11 +1091,11 @@ static int set_skip_frame(int argc, const char **argv, void *data)
 			_cmd;                                                             \
 		}                                                                     \
 		CTX_data_scene_set(C, prevscene);                                     \
-	}                                                                         \
+	} (void)0                                                                 \
 
 #endif /* WITH_PYTHON */
 
-static int run_python(int argc, const char **argv, void *data)
+static int run_python_file(int argc, const char **argv, void *data)
 {
 #ifdef WITH_PYTHON
 	bContext *C = data;
@@ -904,12 +1107,42 @@ static int run_python(int argc, const char **argv, void *data)
 		BLI_strncpy(filename, argv[1], sizeof(filename));
 		BLI_path_cwd(filename);
 
-		BPY_CTX_SETUP(BPY_filepath_exec(C, filename, NULL))
+		BPY_CTX_SETUP(BPY_filepath_exec(C, filename, NULL));
 
 		return 1;
 	}
 	else {
-		printf("\nError: you must specify a Python script after '-P / --python'.\n");
+		printf("\nError: you must specify a filepath after '%s'.\n", argv[0]);
+		return 0;
+	}
+#else
+	(void)argc; (void)argv; (void)data; /* unused */
+	printf("This blender was built without python support\n");
+	return 0;
+#endif /* WITH_PYTHON */
+}
+
+static int run_python_text(int argc, const char **argv, void *data)
+{
+#ifdef WITH_PYTHON
+	bContext *C = data;
+
+	/* workaround for scripts not getting a bpy.context.scene, causes internal errors elsewhere */
+	if (argc > 1) {
+		/* Make the path absolute because its needed for relative linked blends to be found */
+		struct Text *text = (struct Text *)BKE_libblock_find_name(ID_TXT, argv[1]);
+
+		if (text) {
+			BPY_CTX_SETUP(BPY_text_exec(C, text, NULL, false));
+			return 1;
+		}
+		else {
+			printf("\nError: text block not found %s.\n", argv[1]);
+			return 1;
+		}
+	}
+	else {
+		printf("\nError: you must specify a text block after '%s'.\n", argv[0]);
 		return 0;
 	}
 #else
@@ -924,7 +1157,7 @@ static int run_python_console(int UNUSED(argc), const char **argv, void *data)
 #ifdef WITH_PYTHON
 	bContext *C = data;
 
-	BPY_CTX_SETUP(BPY_string_exec(C, "__import__('code').interact()"))
+	BPY_CTX_SETUP(BPY_string_exec(C, "__import__('code').interact()"));
 
 	return 0;
 #else
@@ -956,7 +1189,6 @@ static int set_addons(int argc, const char **argv, void *data)
 	}
 }
 
-
 static int load_file(int UNUSED(argc), const char **argv, void *data)
 {
 	bContext *C = data;
@@ -1001,7 +1233,7 @@ static int load_file(int UNUSED(argc), const char **argv, void *data)
 			return -1;
 		}
 
-		/* WM_read_file() runs normally but since we're in background mode do here */
+		/* WM_file_read() runs normally but since we're in background mode do here */
 #ifdef WITH_PYTHON
 		/* run any texts that were loaded in and flagged as modules */
 		BPY_driver_reset();
@@ -1018,7 +1250,7 @@ static int load_file(int UNUSED(argc), const char **argv, void *data)
 		 * a file - this should do everything a 'load file' does */
 		ReportList reports;
 		BKE_reports_init(&reports, RPT_PRINT);
-		WM_read_file(C, filename, &reports);
+		WM_file_read(C, filename, &reports);
 		BKE_reports_clear(&reports);
 	}
 
@@ -1053,7 +1285,9 @@ static void setupArguments(bContext *C, bArgs *ba, SYS_SystemHandle *syshandle)
 		"\n\t\t-p <sx> <sy>\tOpen with lower left corner at <sx>, <sy>"
 		"\n\t\t-m\t\tRead from disk (Don't buffer)"
 		"\n\t\t-f <fps> <fps-base>\t\tSpecify FPS to start with"
-		"\n\t\t-j <frame>\tSet frame step to <frame>";
+		"\n\t\t-j <frame>\tSet frame step to <frame>"
+		"\n\t\t-s <frame>\tPlay from <frame>"
+		"\n\t\t-e <frame>\tPlay until <frame>";
 
 	static char game_doc[] = "Game Engine specific options"
 		"\n\t-g fixedtime\t\tRun on 50 hertz without dropping frames"
@@ -1088,7 +1322,9 @@ static void setupArguments(bContext *C, bArgs *ba, SYS_SystemHandle *syshandle)
 #endif
 
 	BLI_argsAdd(ba, 1, "-y", "--enable-autoexec", "\n\tEnable automatic python script execution" PY_ENABLE_AUTO, enable_python, NULL);
-	BLI_argsAdd(ba, 1, "-Y", "--disable-autoexec", "\n\tDisable automatic python script execution (pydrivers, pyconstraints, pynodes)" PY_DISABLE_AUTO, disable_python, NULL);
+	BLI_argsAdd(ba, 1, "-Y", "--disable-autoexec", "\n\tDisable automatic python script execution (pydrivers & startup scripts)" PY_DISABLE_AUTO, disable_python, NULL);
+
+	BLI_argsAdd(ba, 1, NULL, "--disable-crash-handler", "\n\tDisable the crash handler", disable_crash_handler, NULL);
 
 #undef PY_ENABLE_AUTO
 #undef PY_DISABLE_AUTO
@@ -1098,11 +1334,13 @@ static void setupArguments(bContext *C, bArgs *ba, SYS_SystemHandle *syshandle)
 	BLI_argsAdd(ba, 1, "-a", NULL, playback_doc, playback_mode, NULL);
 
 	BLI_argsAdd(ba, 1, "-d", "--debug", debug_doc, debug_mode, ba);
+
 #ifdef WITH_FFMPEG
 	BLI_argsAdd(ba, 1, NULL, "--debug-ffmpeg", "\n\tEnable debug messages from FFmpeg library", debug_mode_generic, (void *)G_DEBUG_FFMPEG);
 #endif
 	BLI_argsAdd(ba, 1, NULL, "--debug-python", "\n\tEnable debug messages for python", debug_mode_generic, (void *)G_DEBUG_PYTHON);
 	BLI_argsAdd(ba, 1, NULL, "--debug-events", "\n\tEnable debug messages for the event system", debug_mode_generic, (void *)G_DEBUG_EVENTS);
+	BLI_argsAdd(ba, 1, NULL, "--debug-handlers", "\n\tEnable debug messages for event handling", debug_mode_generic, (void *)G_DEBUG_HANDLERS);
 	BLI_argsAdd(ba, 1, NULL, "--debug-wm",     "\n\tEnable debug messages for the window manager", debug_mode_generic, (void *)G_DEBUG_WM);
 	BLI_argsAdd(ba, 1, NULL, "--debug-all",    "\n\tEnable all debug messages (excludes libmv)", debug_mode_generic, (void *)G_DEBUG_ALL);
 
@@ -1112,21 +1350,26 @@ static void setupArguments(bContext *C, bArgs *ba, SYS_SystemHandle *syshandle)
 	BLI_argsAdd(ba, 1, NULL, "--debug-libmv", "\n\tEnable debug messages from libmv library", debug_mode_libmv, NULL);
 #endif
 
+	BLI_argsAdd(ba, 1, NULL, "--debug-value", "<value>\n\tSet debug value of <value> on startup\n", set_debug_value, NULL);
+	BLI_argsAdd(ba, 1, NULL, "--debug-jobs",  "\n\tEnable time profiling for background jobs.", debug_mode_generic, (void *)G_DEBUG_JOBS);
+
+	BLI_argsAdd(ba, 1, NULL, "--verbose", "<verbose>\n\tSet logging verbosity level.", set_verbosity, NULL);
+
 	BLI_argsAdd(ba, 1, NULL, "--factory-startup", "\n\tSkip reading the "STRINGIFY (BLENDER_STARTUP_FILE)" in the users home directory", set_factory_startup, NULL);
 
 	/* TODO, add user env vars? */
 	BLI_argsAdd(ba, 1, NULL, "--env-system-datafiles",  "\n\tSet the "STRINGIFY_ARG (BLENDER_SYSTEM_DATAFILES)" environment variable", set_env, NULL);
 	BLI_argsAdd(ba, 1, NULL, "--env-system-scripts",    "\n\tSet the "STRINGIFY_ARG (BLENDER_SYSTEM_SCRIPTS)" environment variable", set_env, NULL);
-	BLI_argsAdd(ba, 1, NULL, "--env-system-plugins",    "\n\tSet the "STRINGIFY_ARG (BLENDER_SYSTEM_PLUGINS)" environment variable", set_env, NULL);
 	BLI_argsAdd(ba, 1, NULL, "--env-system-python",     "\n\tSet the "STRINGIFY_ARG (BLENDER_SYSTEM_PYTHON)" environment variable", set_env, NULL);
 
 	/* second pass: custom window stuff */
 	BLI_argsAdd(ba, 2, "-p", "--window-geometry", "<sx> <sy> <w> <h>\n\tOpen with lower left corner at <sx>, <sy> and width and height as <w>, <h>", prefsize, NULL);
 	BLI_argsAdd(ba, 2, "-w", "--window-border", "\n\tForce opening with borders (default)", with_borders, NULL);
 	BLI_argsAdd(ba, 2, "-W", "--window-borderless", "\n\tForce opening without borders", without_borders, NULL);
-	BLI_argsAdd(ba, 2, "-con", "--start-console", "\n\tStart with the console window open (ignored if -b is set)", start_with_console, NULL);
+	BLI_argsAdd(ba, 2, "-con", "--start-console", "\n\tStart with the console window open (ignored if -b is set), (Windows only)", start_with_console, NULL);
 	BLI_argsAdd(ba, 2, "-R", NULL, "\n\tRegister .blend extension, then exit (Windows only)", register_extension, NULL);
 	BLI_argsAdd(ba, 2, "-r", NULL, "\n\tSilently register .blend extension, then exit (Windows only)", register_extension, ba);
+	BLI_argsAdd(ba, 2, NULL, "--no-native-pixels", "\n\tDo not use native pixel size, for high resolution displays (MacBook 'Retina')", native_pixels, ba);
 
 	/* third pass: disabling things and forcing settings */
 	BLI_argsAddCase(ba, 3, "-nojoystick", 1, NULL, 0, "\n\tDisable joystick support", no_joystick, syshandle);
@@ -1142,7 +1385,8 @@ static void setupArguments(bContext *C, bArgs *ba, SYS_SystemHandle *syshandle)
 	BLI_argsAdd(ba, 4, "-s", "--frame-start", "<frame>\n\tSet start to frame <frame> (use before the -a argument)", set_start_frame, C);
 	BLI_argsAdd(ba, 4, "-e", "--frame-end", "<frame>\n\tSet end to frame <frame> (use before the -a argument)", set_end_frame, C);
 	BLI_argsAdd(ba, 4, "-j", "--frame-jump", "<frames>\n\tSet number of frames to step forward after each rendered frame", set_skip_frame, C);
-	BLI_argsAdd(ba, 4, "-P", "--python", "<filename>\n\tRun the given Python script (filename or Blender Text)", run_python, C);
+	BLI_argsAdd(ba, 4, "-P", "--python", "<filename>\n\tRun the given Python script file", run_python_file, C);
+	BLI_argsAdd(ba, 4, NULL, "--python-text", "<name>\n\tRun the given Python script text block", run_python_text, C);
 	BLI_argsAdd(ba, 4, NULL, "--python-console", "\n\tRun blender with an interactive console", run_python_console, C);
 	BLI_argsAdd(ba, 4, NULL, "--addons", "\n\tComma separated list of addons (no spaces)", set_addons, C);
 
@@ -1154,18 +1398,18 @@ static void setupArguments(bContext *C, bArgs *ba, SYS_SystemHandle *syshandle)
 	BLI_argsAdd(ba, 4, "-x", "--use-extension", "<bool>\n\tSet option to add the file extension to the end of the file", set_extension, C);
 
 }
+#endif /* WITH_PYTHON_MODULE */
 
 #ifdef WITH_PYTHON_MODULE
 /* allow python module to call main */
-#define main main_python_enter
+#  define main main_python_enter
 static void *evil_C = NULL;
 
-#ifdef __APPLE__
-/* environ is not available in mac shared libraries */
-#include <crt_externs.h>
+#  ifdef __APPLE__
+     /* environ is not available in mac shared libraries */
+#    include <crt_externs.h>
 char **environ = NULL;
-#endif
-
+#  endif
 #endif
 
 
@@ -1175,9 +1419,12 @@ int main(int argc, const char **UNUSED(argv_c)) /* Do not mess with const */
 int main(int argc, const char **argv)
 #endif
 {
-	SYS_SystemHandle syshandle;
 	bContext *C = CTX_create();
+	SYS_SystemHandle syshandle;
+
+#ifndef WITH_PYTHON_MODULE
 	bArgs *ba;
+#endif
 
 #ifdef WIN32
 	wchar_t **argv_16 = CommandLineToArgvW(GetCommandLineW(), &argc);
@@ -1187,7 +1434,7 @@ int main(int argc, const char **argv)
 		argv[argci] = alloc_utf_8_from_16(argv_16[argci], 0);
 	}
 	LocalFree(argv_16);
-#endif	
+#endif
 
 #ifdef WITH_PYTHON_MODULE
 #ifdef __APPLE__
@@ -1234,21 +1481,16 @@ int main(int argc, const char **argv)
 
 	BLI_threadapi_init();
 
-	RNA_init();
-	RE_engines_init();
-
-	/* Hack - force inclusion of the plugin api functions,
-	 * see blenpluginapi:pluginapi.c
-	 */
-	pluginapi_force_ref();
-
-	init_nodesystem();
-	
 	initglobals();  /* blender.c */
 
 	IMB_init();
+	BKE_images_init();
 
-	BLI_cb_init();
+#ifdef WITH_FFMPEG
+	IMB_ffmpeg_init();
+#endif
+
+	BLI_callback_global_init();
 
 #ifdef WITH_GAMEENGINE
 	syshandle = SYS_GetSystem();
@@ -1257,20 +1499,40 @@ int main(int argc, const char **argv)
 #endif
 
 	/* first test for background */
+#ifndef WITH_PYTHON_MODULE
 	ba = BLI_argsInit(argc, (const char **)argv); /* skip binary path */
 	setupArguments(C, ba, &syshandle);
 
 	BLI_argsParse(ba, 1, NULL, NULL);
 
+	if (use_crash_handler) {
+		/* after parsing args */
+		signal(SIGSEGV, blender_crash_handler);
+	}
+#else
+	G.factory_startup = true;  /* using preferences or user startup makes no sense for py-as-module */
+	(void)syshandle;
+#endif
+
+	/* after level 1 args, this is so playanim skips RNA init */
+	RNA_init();
+
+	RE_engines_init();
+	init_nodesystem();
+	/* end second init */
+
+
 #if defined(WITH_PYTHON_MODULE) || defined(WITH_HEADLESS)
-	G.background = 1; /* python module mode ALWAYS runs in background mode (for now) */
+	G.background = true; /* python module mode ALWAYS runs in background mode (for now) */
 #else
 	/* for all platforms, even windos has it! */
-	if (G.background) signal(SIGINT, blender_esc);  /* ctrl c out bg render */
+	if (G.background) {
+		signal(SIGINT, blender_esc);  /* ctrl c out bg render */
+	}
 #endif
 
 	/* background render uses this font too */
-	BKE_font_register_builtin(datatoc_Bfont, datatoc_Bfont_size);
+	BKE_vfont_builtin_register(datatoc_bfont_pfb, datatoc_bfont_pfb_size);
 
 	/* Initialize ffmpeg if built in, also needed for bg mode if videos are
 	 * rendered via ffmpeg */
@@ -1279,9 +1541,10 @@ int main(int argc, const char **argv)
 	init_def_material();
 
 	if (G.background == 0) {
+#ifndef WITH_PYTHON_MODULE
 		BLI_argsParse(ba, 2, NULL, NULL);
 		BLI_argsParse(ba, 3, NULL, NULL);
-
+#endif
 		WM_init(C, argc, (const char **)argv);
 
 		/* this is properly initialized with user defs, but this is default */
@@ -1293,7 +1556,9 @@ int main(int argc, const char **argv)
 #endif
 	}
 	else {
+#ifndef WITH_PYTHON_MODULE
 		BLI_argsParse(ba, 3, NULL, NULL);
+#endif
 
 		WM_init(C, argc, (const char **)argv);
 
@@ -1308,7 +1573,7 @@ int main(int argc, const char **argv)
 	 * WM_init() before #BPY_python_start() crashes Blender at startup.
 	 */
 
-	// TODO - U.pythondir
+	/* TODO - U.pythondir */
 #else
 	printf("\n* WARNING * - Blender compiled without Python!\nthis is not intended for typical usage\n\n");
 #endif
@@ -1317,13 +1582,16 @@ int main(int argc, const char **argv)
 	WM_keymap_init(C);
 
 	/* OK we are ready for it */
+#ifndef WITH_PYTHON_MODULE
 	BLI_argsParse(ba, 4, load_file, C);
+#endif
 
+#ifndef WITH_PYTHON_MODULE
 	BLI_argsFree(ba);
+#endif
 
 #ifdef WIN32
-	while (argci)
-	{
+	while (argci) {
 		free(argv[--argci]);
 	}
 	MEM_freeN(argv);
@@ -1351,7 +1619,7 @@ int main(int argc, const char **argv)
 	WM_main(C);
 
 	return 0;
-} /* end of int main(argc,argv)	*/
+} /* end of int main(argc, argv)	*/
 
 #ifdef WITH_PYTHON_MODULE
 void main_python_exit(void)
