@@ -32,8 +32,8 @@
 #ifndef __KX_BLENDERINPUTDEVICE_H__
 #define __KX_BLENDERINPUTDEVICE_H__
 
-#if defined(WIN32) && !defined(FREE_WINDOWS)
-#pragma warning(disable : 4786)  // shut off 255 char limit debug template warning
+#ifdef _MSC_VER
+#  pragma warning(disable:4786)  // shut off 255 char limit debug template warning
 #endif
 
 #include <map>
@@ -48,8 +48,9 @@
 #endif
 
 /**
- Base Class for Blender specific inputdevices. Blender specific inputdevices are used when the gameengine is running in embedded mode instead of standalone mode.
-*/
+ * Base Class for Blender specific inputdevices.
+ * Blender specific inputdevices are used when the gameengine is running in embedded mode instead of standalone mode.
+ */
 class BL_BlenderInputDevice : public SCA_IInputDevice
 {
 public:
@@ -72,10 +73,8 @@ public:
 
 
 #ifdef WITH_CXX_GUARDEDALLOC
-public:
-	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:BL_BlenderInputDevice"); }
-	void operator delete(void *mem) { MEM_freeN(mem); }
+	MEM_CXX_CLASS_ALLOC_FUNCS("GE:BL_BlenderInputDevice")
 #endif
 };
-#endif //__KX_BLENDERINPUTDEVICE_H__
 
+#endif  /* __KX_BLENDERINPUTDEVICE_H__ */
