@@ -96,7 +96,7 @@ BL_ActionActuator::BL_ActionActuator(SCA_IObject* gameobj,
 	m_userpose(NULL),
 	m_action(action),
 	m_propname(propname),
-	m_framepropname(framepropname)		
+	m_framepropname(framepropname)
 {
 	if (!end_reset)
 		m_flag |= ACT_FLAG_CONTINUE;
@@ -140,12 +140,10 @@ void BL_ActionActuator::SetLocalTime(float curtime)
 	// Handle wrap around
 	if (m_localtime < min(m_startframe, m_endframe) || m_localtime > max(m_startframe, m_endframe))
 	{
-		switch(m_playtype)
-		{
+		switch (m_playtype) {
 		case ACT_ACTION_PLAY:
 			// Clamp
 			m_localtime = m_endframe;
-			((KX_GameObject*)GetParent())->StopAction(m_layer);
 			break;
 		case ACT_ACTION_LOOP_END:
 			// Put the time back to the beginning
@@ -197,8 +195,7 @@ bool BL_ActionActuator::Update(double curtime, bool frame)
 		return false;
 
 	// Convert our playtype to one that BL_Action likes
-	switch(m_playtype)
-	{
+	switch (m_playtype) {
 		case ACT_ACTION_LOOP_END:
 		case ACT_ACTION_LOOP_STOP:
 			playtype = BL_Action::ACT_MODE_LOOP;
@@ -300,7 +297,7 @@ bool BL_ActionActuator::Update(double curtime, bool frame)
 		m_flag |= ACT_FLAG_ATTEMPT_PLAY;
 	}
 	else if ((m_flag & ACT_FLAG_ACTIVE) && bNegativeEvent)
-	{	
+	{
 		m_flag &= ~ACT_FLAG_ATTEMPT_PLAY;
 		m_localtime = obj->GetActionFrame(m_layer);
 		bAction *curr_action = obj->GetCurrentAction(m_layer);
@@ -313,8 +310,7 @@ bool BL_ActionActuator::Update(double curtime, bool frame)
 			return false;
 		}
 
-		switch(m_playtype)
-		{
+		switch (m_playtype) {
 			case ACT_ACTION_LOOP_STOP:
 				obj->StopAction(m_layer); // Stop the action after getting the frame
 
@@ -339,7 +335,7 @@ bool BL_ActionActuator::Update(double curtime, bool frame)
 		}
 	}
 
-	return true;
+	return m_flag & ACT_FLAG_ACTIVE;
 }
 
 #ifdef WITH_PYTHON
@@ -348,8 +344,11 @@ bool BL_ActionActuator::Update(double curtime, bool frame)
 /* Python functions                                                          */
 /* ------------------------------------------------------------------------- */
 
-PyObject* BL_ActionActuator::PyGetChannel(PyObject* value)
+PyObject *BL_ActionActuator::PyGetChannel(PyObject *value)
 {
+	PyErr_SetString(PyExc_NotImplementedError, "BL_ActionActuator.getChannel() no longer works, please use BL_ArmatureObject.channels instead");
+	return NULL;
+#if 0 // XXX To be removed in a later version (first removed in 2.64)
 	const char *string= _PyUnicode_AsString(value);
 
 	if (GetParent()->GetGameObjectType() != SCA_IObject::OBJ_ARMATURE)
@@ -370,10 +369,10 @@ PyObject* BL_ActionActuator::PyGetChannel(PyObject* value)
 		obj->GetPose(&m_pose); /* Get the underlying pose from the armature */
 	}
 	
-	// get_pose_channel accounts for NULL pose, run on both in case one exists but
+	// BKE_pose_channel_find_name accounts for NULL pose, run on both in case one exists but
 	// the channel doesnt
-	if (		!(pchan=get_pose_channel(m_userpose, string)) &&
-			!(pchan=get_pose_channel(m_pose, string))  )
+	if (		!(pchan=BKE_pose_channel_find_name(m_userpose, string)) &&
+			!(pchan=BKE_pose_channel_find_name(m_pose, string))  )
 	{
 		PyErr_SetString(PyExc_ValueError, "channel doesnt exist");
 		return NULL;
@@ -407,6 +406,7 @@ PyObject* BL_ActionActuator::PyGetChannel(PyObject* value)
 		pchan->size[0], pchan->size[1], pchan->size[2],
 		pchan->quat[0], pchan->quat[1], pchan->quat[2], pchan->quat[3] );
 #endif
+#endif
 }
 
 /*     setChannel                                                         */
@@ -416,6 +416,10 @@ KX_PYMETHODDEF_DOC(BL_ActionActuator, setChannel,
 "\t - matrix    : A 4x4 matrix specifying the overriding transformation\n"
 "\t               as an offset from the bone's rest position.\n")
 {
+	PyErr_SetString(PyExc_NotImplementedError, "BL_ActionActuator.setChannel() no longer works, please use BL_ArmatureObject.channels instead");
+	return NULL;
+
+#if 0 // XXX To be removed in a later version (first removed in 2.64)
 	BL_ArmatureObject *obj = (BL_ArmatureObject*)GetParent();
 	char *string;
 	PyObject *pymat= NULL;
@@ -457,8 +461,8 @@ KX_PYMETHODDEF_DOC(BL_ActionActuator, setChannel,
 				obj->GetPose(&m_pose); /* Get the underlying pose from the armature */
 			game_copy_pose(&m_userpose, m_pose, 0);
 		}
-		// pchan= verify_pose_channel(m_userpose, string); // adds the channel if its not there.
-		pchan= get_pose_channel(m_userpose, string); // adds the channel if its not there.
+		// pchan= BKE_pose_channel_verify(m_userpose, string); // adds the channel if its not there.
+		pchan= BKE_pose_channel_find_name(m_userpose, string); // adds the channel if its not there.
 		
 		if (pchan) {
 			copy_v3_v3(pchan->loc, matrix[3]);
@@ -480,14 +484,14 @@ KX_PYMETHODDEF_DOC(BL_ActionActuator, setChannel,
 				obj->GetPose(&m_pose); /* Get the underlying pose from the armature */
 			game_copy_pose(&m_userpose, m_pose, 0);
 		}
-		// pchan= verify_pose_channel(m_userpose, string);
-		pchan= get_pose_channel(m_userpose, string); // adds the channel if its not there.
+		// pchan= BKE_pose_channel_verify(m_userpose, string);
+		pchan= BKE_pose_channel_find_name(m_userpose, string); // adds the channel if its not there.
 		
 		// for some reason loc.setValue(pchan->loc) fails
 		if (pchan) {
-			pchan->loc[0]= loc[0]; pchan->loc[1]= loc[1]; pchan->loc[2]= loc[2];
-			pchan->size[0]= size[0]; pchan->size[1]= size[1]; pchan->size[2]= size[2];
-			pchan->quat[0]= quat[3]; pchan->quat[1]= quat[0]; pchan->quat[2]= quat[1]; pchan->quat[3]= quat[2]; /* notice xyzw -> wxyz is intentional */
+			pchan->loc[0] = loc[0]; pchan->loc[1] = loc[1]; pchan->loc[2] = loc[2];
+			pchan->size[0] = size[0]; pchan->size[1] = size[1]; pchan->size[2] = size[2];
+			pchan->quat[0] = quat[3]; pchan->quat[1] = quat[0]; pchan->quat[2] = quat[1]; pchan->quat[3] = quat[2]; /* notice xyzw -> wxyz is intentional */
 		}
 	}
 	
@@ -497,6 +501,7 @@ KX_PYMETHODDEF_DOC(BL_ActionActuator, setChannel,
 	}
 	
 	Py_RETURN_NONE;
+#endif
 }
 
 /* ------------------------------------------------------------------------- */
@@ -547,15 +552,15 @@ PyAttributeDef BL_ActionActuator::Attributes[] = {
 	{ NULL }	//Sentinel
 };
 
-PyObject* BL_ActionActuator::pyattr_get_action(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *BL_ActionActuator::pyattr_get_action(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	BL_ActionActuator* self= static_cast<BL_ActionActuator*>(self_v);
+	BL_ActionActuator* self = static_cast<BL_ActionActuator*>(self_v);
 	return PyUnicode_FromString(self->GetAction() ? self->GetAction()->id.name+2 : "");
 }
 
 int BL_ActionActuator::pyattr_set_action(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef, PyObject *value)
 {
-	BL_ActionActuator* self= static_cast<BL_ActionActuator*>(self_v);
+	BL_ActionActuator* self = static_cast<BL_ActionActuator*>(self_v);
 	
 	if (!PyUnicode_Check(value))
 	{
@@ -581,9 +586,13 @@ int BL_ActionActuator::pyattr_set_action(void *self_v, const KX_PYATTRIBUTE_DEF
 
 }
 
-PyObject* BL_ActionActuator::pyattr_get_channel_names(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *BL_ActionActuator::pyattr_get_channel_names(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	BL_ActionActuator* self= static_cast<BL_ActionActuator*>(self_v);
+	PyErr_SetString(PyExc_NotImplementedError, "BL_ActionActuator.channelNames no longer works, please use BL_ArmatureObject.channels instead");
+	return NULL;
+
+#if 0 // XXX To be removed in a later version (first removed in 2.64)
+	BL_ActionActuator* self = static_cast<BL_ActionActuator*>(self_v);
 	PyObject *ret= PyList_New(0);
 	PyObject *item;
 	
@@ -605,17 +614,18 @@ PyObject* BL_ActionActuator::pyattr_get_channel_names(void *self_v, const KX_PYA
 	}
 	
 	return ret;
+#endif
 }
 
-PyObject* BL_ActionActuator::pyattr_get_use_continue(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *BL_ActionActuator::pyattr_get_use_continue(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	BL_ActionActuator* self= static_cast<BL_ActionActuator*>(self_v);
+	BL_ActionActuator* self = static_cast<BL_ActionActuator*>(self_v);
 	return PyBool_FromLong(self->m_flag & ACT_FLAG_CONTINUE);
 }
 
 int BL_ActionActuator::pyattr_set_use_continue(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef, PyObject *value)
 {
-	BL_ActionActuator* self= static_cast<BL_ActionActuator*>(self_v);
+	BL_ActionActuator* self = static_cast<BL_ActionActuator*>(self_v);
 	
 	if (PyObject_IsTrue(value))
 		self->m_flag |= ACT_FLAG_CONTINUE;
@@ -625,15 +635,15 @@ int BL_ActionActuator::pyattr_set_use_continue(void *self_v, const KX_PYATTRIBUT
 	return PY_SET_ATTR_SUCCESS;
 }
 
-PyObject* BL_ActionActuator::pyattr_get_frame(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *BL_ActionActuator::pyattr_get_frame(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	BL_ActionActuator* self= static_cast<BL_ActionActuator*>(self_v);
+	BL_ActionActuator* self = static_cast<BL_ActionActuator*>(self_v);
 	return PyFloat_FromDouble(((KX_GameObject*)self->m_gameobj)->GetActionFrame(self->m_layer));
 }
 
 int BL_ActionActuator::pyattr_set_frame(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef, PyObject *value)
 {
-	BL_ActionActuator* self= static_cast<BL_ActionActuator*>(self_v);
+	BL_ActionActuator* self = static_cast<BL_ActionActuator*>(self_v);
 	
 	((KX_GameObject*)self->m_gameobj)->SetActionFrame(self->m_layer, PyFloat_AsDouble(value));
 	
