@@ -61,7 +61,7 @@
 /** 
  * Move here pose function for game engine so that we can mix with GE objects
  * Principle is as follow:
- * Use Blender structures so that where_is_pose can be used unchanged
+ * Use Blender structures so that BKE_pose_where_is can be used unchanged
  * Copy the constraint so that they can be enabled/disabled/added/removed at runtime
  * Don't copy the constraints for the pose used by the Action actuator, it does not need them.
  * Scan the constraint structures so that the KX equivalent of target objects are identified and 
@@ -84,7 +84,7 @@ void game_copy_pose(bPose **dst, bPose *src, int copy_constraint)
 		return;
 	}
 	else if (*dst==src) {
-		printf("copy_pose source and target are the same\n");
+		printf("BKE_pose_copy_data source and target are the same\n");
 		*dst=NULL;
 		return;
 	}
@@ -105,14 +105,15 @@ void game_copy_pose(bPose **dst, bPose *src, int copy_constraint)
 	for (; pchan; pchan=pchan->next, outpchan=outpchan->next)
 		BLI_ghash_insert(ghash, pchan, outpchan);
 
-	for (pchan=(bPoseChannel*)out->chanbase.first; pchan; pchan=(bPoseChannel*)pchan->next) {
+	for (pchan = (bPoseChannel*)out->chanbase.first; pchan; pchan = pchan->next) {
 		pchan->parent= (bPoseChannel*)BLI_ghash_lookup(ghash, pchan->parent);
 		pchan->child= (bPoseChannel*)BLI_ghash_lookup(ghash, pchan->child);
 
 		if (copy_constraint) {
 			ListBase listb;
 			// copy all constraint for backward compatibility
-			copy_constraints(&listb, &pchan->constraints, FALSE);  // copy_constraints NULLs listb, no need to make extern for this operation.
+			// BKE_copy_constraints NULLs listb, no need to make extern for this operation.
+			BKE_copy_constraints(&listb, &pchan->constraints, FALSE);
 			pchan->constraints= listb;
 		} else {
 			pchan->constraints.first = NULL;
@@ -129,7 +130,7 @@ void game_copy_pose(bPose **dst, bPose *src, int copy_constraint)
 
 	BLI_ghash_free(ghash, NULL, NULL);
 	// set acceleration structure for channel lookup
-	make_pose_channels_hash(out);
+	BKE_pose_channels_hash_make(out);
 	*dst=out;
 }
 
@@ -186,7 +187,10 @@ void game_blend_poses(bPose *dst, bPose *src, float srcweight/*, short mode*/)
 			if (schan->rotmode)
 				dchan->eul[i] = (dchan->eul[i]*dstweight) + (schan->eul[i]*srcweight);
 		}
-		for (dcon= (bConstraint*)dchan->constraints.first, scon= (bConstraint*)schan->constraints.first; dcon && scon; dcon= (bConstraint*)dcon->next, scon= (bConstraint*)scon->next) {
+		for (dcon= (bConstraint *)dchan->constraints.first, scon= (bConstraint *)schan->constraints.first;
+		     dcon && scon;
+		     dcon = dcon->next, scon = scon->next)
+		{
 			/* no 'add' option for constraint blending */
 			dcon->enforce= dcon->enforce*(1.0f-srcweight) + scon->enforce*srcweight;
 		}
@@ -200,7 +204,7 @@ void game_free_pose(bPose *pose)
 {
 	if (pose) {
 		/* free pose-channels and constraints */
-		free_pose_channels(pose);
+		BKE_pose_channels_free(pose);
 		
 		/* free IK solver state */
 		BIK_clear_data(pose);
@@ -225,7 +229,7 @@ BL_ArmatureObject::BL_ArmatureObject(
 	m_poseChannels(),
 	m_objArma(armature),
 	m_framePose(NULL),
-	m_scene(scene), // maybe remove later. needed for where_is_pose
+	m_scene(scene), // maybe remove later. needed for BKE_pose_where_is
 	m_lastframe(0.0),
 	m_timestep(0.040),
 	m_activeAct(NULL),
@@ -282,8 +286,8 @@ void BL_ArmatureObject::LoadConstraints(KX_BlenderSceneConverter* converter)
 	KX_GameObject* gamesubtarget;
 
 	// and locate the constraint
-	for (pchan = (bPoseChannel*)m_pose->chanbase.first; pchan; pchan=(bPoseChannel*)pchan->next) {
-		for (pcon = (bConstraint*)pchan->constraints.first; pcon; pcon=(bConstraint*)pcon->next) {
+	for (pchan = (bPoseChannel *)m_pose->chanbase.first; pchan; pchan = pchan->next) {
+		for (pcon = (bConstraint *)pchan->constraints.first; pcon; pcon = pcon->next) {
 			if (pcon->flag & CONSTRAINT_DISABLE)
 				continue;
 			// which constraint should we support?
@@ -301,7 +305,7 @@ void BL_ArmatureObject::LoadConstraints(KX_BlenderSceneConverter* converter)
 			case CONSTRAINT_TYPE_TRANSFORM:
 			case CONSTRAINT_TYPE_DISTLIMIT:
 			case CONSTRAINT_TYPE_TRANSLIKE:
-				cti = constraint_get_typeinfo(pcon);
+				cti = BKE_constraint_get_typeinfo(pcon);
 				gametarget = gamesubtarget = NULL;
 				if (cti && cti->get_constraint_targets) {
 					ListBase listb = { NULL, NULL };
@@ -315,7 +319,7 @@ void BL_ArmatureObject::LoadConstraints(KX_BlenderSceneConverter* converter)
 						}
 						if (target->next != NULL) {
 							// secondary target
-							target = (bConstraintTarget*)target->next;
+							target = target->next;
 							if (target->tar && target->tar != m_objArma) {
 								// only track external object
 								blendtarget = target->tar;
@@ -430,7 +434,7 @@ void BL_ArmatureObject::ProcessReplica()
 
 	m_pose = NULL;
 	m_framePose = NULL;
-	game_copy_pose(&m_pose, pose, 1);	
+	game_copy_pose(&m_pose, pose, 1);
 }
 
 void BL_ArmatureObject::ReParentLogic()
@@ -477,7 +481,7 @@ void BL_ArmatureObject::ApplyPose()
 		}
 		// update ourself
 		UpdateBlenderObjectMatrix(m_objArma);
-		where_is_pose(m_scene, m_objArma); // XXX
+		BKE_pose_where_is(m_scene, m_objArma); // XXX
 		// restore ourself
 		memcpy(m_objArma->obmat, m_obmat, sizeof(m_obmat));
 		// restore active targets
@@ -521,7 +525,7 @@ bool BL_ArmatureObject::SetActiveAction(BL_ActionActuator *act, short priority,
 				SetPose(m_framePose);
 				if (m_activeAct && (m_activeAct!=act))
 					/* Reset the blend timer since this new action cancels the old one */
-					m_activeAct->SetBlendTime(0.0);	
+					m_activeAct->SetBlendTime(0.0);
 			}
 			m_activeAct = act;
 			m_activePriority = priority;
@@ -529,7 +533,7 @@ bool BL_ArmatureObject::SetActiveAction(BL_ActionActuator *act, short priority,
 		
 			return true;
 		}
-		else{
+		else {
 			act->SetBlendTime(0.0);
 			return false;
 		}
@@ -590,7 +594,7 @@ bool BL_ArmatureObject::GetBoneMatrix(Bone* bone, MT_Matrix4x4& matrix)
 	bPoseChannel *pchan;
 
 	ApplyPose();
-	pchan = get_pose_channel(m_objArma->pose, bone->name);
+	pchan = BKE_pose_channel_find_name(m_objArma->pose, bone->name);
 	if (pchan)
 		matrix.setValue(&pchan->pose_mat[0][0]);
 	RestorePose();
@@ -636,7 +640,6 @@ PyTypeObject BL_ArmatureObject::Type = {
 };
 
 PyMethodDef BL_ArmatureObject::Methods[] = {
-
 	KX_PYMETHODTABLE_NOARGS(BL_ArmatureObject, update),
 	{NULL,NULL} //Sentinel
 };
@@ -648,12 +651,12 @@ PyAttributeDef BL_ArmatureObject::Attributes[] = {
 	{NULL} //Sentinel
 };
 
-PyObject* BL_ArmatureObject::pyattr_get_constraints(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *BL_ArmatureObject::pyattr_get_constraints(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
 	return KX_PythonSeq_CreatePyObject((static_cast<BL_ArmatureObject*>(self_v))->m_proxy, KX_PYGENSEQ_OB_TYPE_CONSTRAINTS);
 }
 
-PyObject* BL_ArmatureObject::pyattr_get_channels(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *BL_ArmatureObject::pyattr_get_channels(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
 	BL_ArmatureObject* self = static_cast<BL_ArmatureObject*>(self_v);
 	self->LoadChannels(); // make sure we have the channels
