@@ -16,13 +16,13 @@
  *  \ingroup expressions
  */
 
-#if defined(WIN32) && !defined(FREE_WINDOWS)
-#pragma warning (disable:4786)
-#endif //WIN32
-
 #ifndef __VALUE_H__
 #define __VALUE_H__
 
+#ifdef _MSC_VER
+#  pragma warning (disable:4786)
+#endif
+
 #include <map>		// array functionality for the propertylist
 #include "STR_String.h"	// STR_String class
 
@@ -53,14 +53,6 @@ using namespace std;
 #define	assertd(exp)			((void)NULL)
 #endif
 
-
-#ifndef USE_PRAGMA_ONCE
-#ifdef WIN32
-	#pragma once
-
-#endif //WIN32
-#endif
-
 enum VALUE_OPERATOR {
 	
 	VALUE_MOD_OPERATOR,			// %
@@ -158,15 +150,13 @@ class CAction
 public:
 	CAction() {
 	};
-	virtual ~CAction(){
+	virtual ~CAction() {
 	};
 	virtual void Execute() const =0;
 	
 	
 #ifdef WITH_CXX_GUARDEDALLOC
-public:
-	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:CAction"); }
-	void operator delete( void *mem ) { MEM_freeN(mem); }
+	MEM_CXX_CLASS_ALLOC_FUNCS("GE:CAction")
 #endif
 };
 
@@ -220,22 +210,22 @@ public:
 	CValue();
 
 #ifdef WITH_PYTHON
-	//static PyObject*	PyMake(PyObject*,PyObject*);
+	//static PyObject *PyMake(PyObject *, PyObject *);
 	virtual PyObject *py_repr(void)
 	{
 		return PyUnicode_From_STR_String(GetText());
 	}
 
-	virtual PyObject*	ConvertValueToPython() {
+	virtual PyObject *ConvertValueToPython() {
 		return NULL;
 	}
 
-	virtual CValue*	ConvertPythonToValue(PyObject* pyobj, const char *error_prefix);
+	virtual CValue *ConvertPythonToValue(PyObject *pyobj, const char *error_prefix);
 	
-	static PyObject * pyattr_get_name(void * self, const KX_PYATTRIBUTE_DEF * attrdef);
+	static PyObject *pyattr_get_name(void * self, const KX_PYATTRIBUTE_DEF * attrdef);
 	
-	virtual PyObject* ConvertKeysToPython( void );
-#endif // WITH_PYTHON
+	virtual PyObject *ConvertKeysToPython( void );
+#endif  /* WITH_PYTHON */
 
 	
 	
@@ -252,13 +242,13 @@ public:
 	};
 
 	/// Reference Counting
-	int					GetRefCount()											
+	int GetRefCount()
 	{ 
 		return m_refcount; 
 	}
 
 	// Add a reference to this value
-	CValue*				AddRef()												
+	CValue *AddRef()
 	{
 		// Increase global reference count, used to see at the end of the program
 		// if all CValue-derived classes have been dereferenced to 0
@@ -271,7 +261,7 @@ public:
 	}
 
 	// Release a reference to this value (when reference count reaches 0, the value is removed from the heap)
-	int			Release()								
+	int			Release()
 	{
 		// Decrease global reference count, used to see at the end of the program
 		// if all CValue-derived classes have been dereferenced to 0
@@ -356,16 +346,16 @@ public:
 	virtual void		SetCustomFlag2(bool bCustomFlag)						{ m_ValFlags.CustomFlag2 = bCustomFlag;}
 	virtual bool		IsCustomFlag2()											{ return m_ValFlags.CustomFlag2;}
 
-protected:																		
+protected:
 	virtual void		DisableRefCount();										// Disable reference counting for this value
-	//virtual void		AddDataToReplica(CValue* replica);						
+	//virtual void		AddDataToReplica(CValue* replica);
 	virtual				~CValue();
 private:
-	// Member variables															
+	// Member variables
 	std::map<STR_String,CValue*>*		m_pNamedPropertyArray;									// Properties for user/game etc
 	ValueFlags			m_ValFlags;												// Frequently used flags in a bitfield (low memoryusage)
-	int					m_refcount;												// Reference Counter	
-	static	double m_sZeroVec[3];	
+	int					m_refcount;												// Reference Counter
+	static	double m_sZeroVec[3];
 
 };
 
@@ -436,13 +426,9 @@ public:
 protected:
 	STR_String					m_strNewName;				    // Identification
 
-
 #ifdef WITH_CXX_GUARDEDALLOC
-public:
-	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:CPropValue"); }
-	void operator delete( void *mem ) { MEM_freeN(mem); }
+	MEM_CXX_CLASS_ALLOC_FUNCS("GE:CPropValue")
 #endif
 };
 
-#endif // !defined _VALUEBASECLASS_H
-
+#endif  /* __VALUE_H__ */
