@@ -86,11 +86,8 @@ protected:
 
 
 #ifdef WITH_CXX_GUARDEDALLOC
-public:
-	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:CVectorValue"); }
-	void operator delete( void *mem ) { MEM_freeN(mem); }
+	MEM_CXX_CLASS_ALLOC_FUNCS("GE:CVectorValue")
 #endif
 };
 
-#endif // !defined __VECTORVALUE_H__
-
+#endif  /* __VECTORVALUE_H__ */
