@@ -74,11 +74,8 @@ public:
 
 
 #ifdef WITH_CXX_GUARDEDALLOC
-public:
-	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:SCA_IScene"); }
-	void operator delete( void *mem ) { MEM_freeN(mem); }
+	MEM_CXX_CLASS_ALLOC_FUNCS("GE:SCA_IScene")
 #endif
 };
 
-#endif //__SCA_ISCENE_H__
-
+#endif  /* __SCA_ISCENE_H__ */
