@@ -71,21 +71,19 @@ SCA_PythonController::SCA_PythonController(SCA_IObject* gameobj, int mode)
 	
 }
 
-/*
+#if 0
 //debugging
-CValue*		SCA_PythonController::AddRef()
+CValue *SCA_PythonController::AddRef()
 {
 	//printf("AddRef refcount = %i\n",GetRefCount());
 	return CValue::AddRef();
 }
-int			SCA_PythonController::Release()
+int SCA_PythonController::Release()
 {
 	//printf("Release refcount = %i\n",GetRefCount());
 	return CValue::Release();
 }
-*/
-
-
+#endif
 
 SCA_PythonController::~SCA_PythonController()
 {
@@ -122,13 +120,14 @@ CValue* SCA_PythonController::GetReplica()
 	if (m_pythondictionary)
 		replica->m_pythondictionary = PyDict_Copy(m_pythondictionary);
 		
-	/*
+#if 0
 	// The other option is to incref the replica->m_pythondictionary -
 	// the replica objects can then share data.
 	if (m_pythondictionary)
 		Py_INCREF(replica->m_pythondictionary);
-	*/
 #endif
+
+#endif /* WITH_PYTHON */
 	
 	// this will copy properties and so on...
 	replica->ProcessReplica();
@@ -169,18 +168,22 @@ void SCA_PythonController::SetNamespace(PyObject*	pythondictionary)
 }
 #endif
 
-int SCA_PythonController::IsTriggered(class SCA_ISensor* sensor)
+bool SCA_PythonController::IsTriggered(class SCA_ISensor* sensor)
 {
 	if (std::find(m_triggeredSensors.begin(), m_triggeredSensors.end(), sensor) != 
 		m_triggeredSensors.end())
-		return 1;
-	return 0;
+	{
+		return true;
+	}
+	else {
+		return false;
+	}
 }
 
 #ifdef WITH_PYTHON
 
 /* warning, self is not the SCA_PythonController, its a PyObjectPlus_Proxy */
-PyObject* SCA_PythonController::sPyGetCurrentController(PyObject *self)
+PyObject *SCA_PythonController::sPyGetCurrentController(PyObject *self)
 {
 	if (m_sCurrentController==NULL)
 	{
@@ -209,7 +212,7 @@ SCA_IActuator* SCA_PythonController::LinkedActuatorFromPy(PyObject *value)
 	else if (PyObject_TypeCheck(value, &SCA_IActuator::Type)) {
 		PyObjectPlus *value_plus= BGE_PROXY_REF(value);
 		for (it = lacts.begin(); it!= lacts.end(); ++it) {
-			if ( static_cast<SCA_IActuator*>(value_plus) == (*it) ) {
+			if (static_cast<SCA_IActuator*>(value_plus) == (*it)) {
 				return *it;
 			}
 		}
@@ -272,7 +275,7 @@ void SCA_PythonController::ErrorPrint(const char *error_msg)
 }
 
 bool SCA_PythonController::Compile()
-{	
+{
 	//printf("py script modified '%s'\n", m_scriptName.Ptr());
 	m_bModified= false;
 	
@@ -375,66 +378,65 @@ void SCA_PythonController::Trigger(SCA_LogicManager* logicmgr)
 	m_sCurrentLogicManager = logicmgr;
 	
 	PyObject *excdict=		NULL;
-	PyObject* resultobj=	NULL;
+	PyObject *resultobj=	NULL;
 	
-	switch(m_mode) {
-	case SCA_PYEXEC_SCRIPT:
-	{
-		if (m_bModified)
-			if (Compile()==false) // sets m_bModified to false
+	switch (m_mode) {
+		case SCA_PYEXEC_SCRIPT:
+		{
+			if (m_bModified)
+				if (Compile()==false) // sets m_bModified to false
+					return;
+			if (!m_bytecode)
 				return;
-		if (!m_bytecode)
-			return;
-		
-		/*
-		 * This part here with excdict is a temporary patch
-		 * to avoid python/gameengine crashes when python
-		 * inadvertently holds references to game objects
-		 * in global variables.
-		 * 
-		 * The idea is always make a fresh dictionary, and
-		 * destroy it right after it is used to make sure
-		 * python won't hold any gameobject references.
-		 * 
-		 * Note that the PyDict_Clear _is_ necessary before
-		 * the Py_DECREF() because it is possible for the
-		 * variables inside the dictionary to hold references
-		 * to the dictionary (ie. generate a cycle), so we
-		 * break it by hand, then DECREF (which in this case
-		 * should always ensure excdict is cleared).
-		 */
 
-		excdict= PyDict_Copy(m_pythondictionary);
+			/*
+			 * This part here with excdict is a temporary patch
+			 * to avoid python/gameengine crashes when python
+			 * inadvertently holds references to game objects
+			 * in global variables.
+			 *
+			 * The idea is always make a fresh dictionary, and
+			 * destroy it right after it is used to make sure
+			 * python won't hold any gameobject references.
+			 *
+			 * Note that the PyDict_Clear _is_ necessary before
+			 * the Py_DECREF() because it is possible for the
+			 * variables inside the dictionary to hold references
+			 * to the dictionary (ie. generate a cycle), so we
+			 * break it by hand, then DECREF (which in this case
+			 * should always ensure excdict is cleared).
+			 */
 
-		resultobj = PyEval_EvalCode((PyObject *)m_bytecode, excdict, excdict);
+			excdict= PyDict_Copy(m_pythondictionary);
 
-		/* PyRun_SimpleString(m_scriptText.Ptr()); */
-		break;
-	}
-	case SCA_PYEXEC_MODULE:
-	{
-		if (m_bModified || m_debug)
-			if (Import()==false) // sets m_bModified to false
-				return;
-		if (!m_function)
-			return;
-		
-		PyObject *args= NULL;
-		
-		if (m_function_argc==1) {
-			args = PyTuple_New(1);
-			PyTuple_SET_ITEM(args, 0, GetProxy());
+			resultobj = PyEval_EvalCode((PyObject *)m_bytecode, excdict, excdict);
+
+			/* PyRun_SimpleString(m_scriptText.Ptr()); */
+			break;
 		}
-		
-		resultobj = PyObject_CallObject(m_function, args);
-		Py_XDECREF(args);
-		break;
-	}
-	
+		case SCA_PYEXEC_MODULE:
+		{
+			if (m_bModified || m_debug)
+				if (Import()==false) // sets m_bModified to false
+					return;
+			if (!m_function)
+				return;
+
+			PyObject *args= NULL;
+
+			if (m_function_argc==1) {
+				args = PyTuple_New(1);
+				PyTuple_SET_ITEM(args, 0, GetProxy());
+			}
+
+			resultobj = PyObject_CallObject(m_function, args);
+			Py_XDECREF(args);
+			break;
+		}
+
 	} /* end switch */
-	
-	
-	
+
+
 	/* Free the return value and print the error */
 	if (resultobj)
 		Py_DECREF(resultobj);
@@ -448,13 +450,13 @@ void SCA_PythonController::Trigger(SCA_LogicManager* logicmgr)
 		// This doesn't appear to be needed anymore
 		//PyDict_Clear(excdict);
 		Py_DECREF(excdict);
-	}	
+	}
 	
 	m_triggeredSensors.clear();
 	m_sCurrentController = NULL;
 }
 
-PyObject* SCA_PythonController::PyActivate(PyObject *value)
+PyObject *SCA_PythonController::PyActivate(PyObject *value)
 {
 	if (m_sCurrentController != this) {
 		PyErr_SetString(PyExc_SystemError, "Cannot add an actuator from a non-active controller");
@@ -469,7 +471,7 @@ PyObject* SCA_PythonController::PyActivate(PyObject *value)
 	Py_RETURN_NONE;
 }
 
-PyObject* SCA_PythonController::PyDeActivate(PyObject *value)
+PyObject *SCA_PythonController::PyDeActivate(PyObject *value)
 {
 	if (m_sCurrentController != this) {
 		PyErr_SetString(PyExc_SystemError, "Cannot add an actuator from a non-active controller");
@@ -484,12 +486,12 @@ PyObject* SCA_PythonController::PyDeActivate(PyObject *value)
 	Py_RETURN_NONE;
 }
 
-PyObject* SCA_PythonController::pyattr_get_script(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *SCA_PythonController::pyattr_get_script(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	//SCA_PythonController* self= static_cast<SCA_PythonController*>(static_cast<SCA_IController*>(static_cast<SCA_ILogicBrick*>(static_cast<CValue*>(static_cast<PyObjectPlus*>(self_v)))));
+	//SCA_PythonController* self = static_cast<SCA_PythonController*>(static_cast<SCA_IController*>(static_cast<SCA_ILogicBrick*>(static_cast<CValue*>(static_cast<PyObjectPlus*>(self_v)))));
 	// static_cast<void *>(dynamic_cast<Derived *>(obj)) - static_cast<void *>(obj)
 
-	SCA_PythonController* self= static_cast<SCA_PythonController*>(self_v);
+	SCA_PythonController* self = static_cast<SCA_PythonController*>(self_v);
 	return PyUnicode_From_STR_String(self->m_scriptText);
 }
 
@@ -497,7 +499,7 @@ PyObject* SCA_PythonController::pyattr_get_script(void *self_v, const KX_PYATTRI
 
 int SCA_PythonController::pyattr_set_script(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef, PyObject *value)
 {
-	SCA_PythonController* self= static_cast<SCA_PythonController*>(self_v);
+	SCA_PythonController* self = static_cast<SCA_PythonController*>(self_v);
 	
 	const char *scriptArg = _PyUnicode_AsString(value);
 	
@@ -507,7 +509,7 @@ int SCA_PythonController::pyattr_set_script(void *self_v, const KX_PYATTRIBUTE_D
 	}
 
 	/* set scripttext sets m_bModified to true, 
-		so next time the script is needed, a reparse into byte code is done */
+	 * so next time the script is needed, a reparse into byte code is done */
 	self->SetScriptText(scriptArg);
 		
 	return PY_SET_ATTR_SUCCESS;
