@@ -38,7 +38,7 @@
 #ifdef WIN32
 	#pragma warning (disable:4786) // suppress stl-MSVC debug info warning
 	#include <windows.h>
-#endif // WIN32
+#endif  /* WIN32 */
 
 #include "GL/glew.h"
 
@@ -87,9 +87,11 @@ protected:
 	/** Height of the context. */
 	int m_height;
 	/** Rect that defines the area used for rendering,
-	    relative to the context */
+	 * relative to the context */
 	RAS_Rect m_displayarea;
 
+	int m_viewport[4];
+
 	/** Storage for the banners to display. */
 	TBannerMap m_banners;
 	/** State of banner display. */
@@ -103,7 +105,7 @@ public:
 
 	void Resize(int width, int height);
 
-	virtual void ResizeWindow(int width, int height){};
+	virtual void ResizeWindow(int width, int height) {}
 
 	/**
 	 * \section Methods inherited from abstract base class RAS_ICanvas.
@@ -153,6 +155,8 @@ public:
 	);
 	
 	void SetViewPort(int x1, int y1, int x2, int y2);
+	void UpdateViewPort(int x1, int y1, int x2, int y2);
+	const int *GetViewPort();
 
 	void ClearColor(float r, float g, float b, float a);
 
@@ -163,7 +167,7 @@ public:
 	
 	void SetMouseState(RAS_MouseState mousestate)
 	{
-		// not yet		
+		// not yet
 	}
 
 	void SetMousePosition(int x, int y)
@@ -250,7 +254,7 @@ protected:
 		int oldWriteMask;
 	};
 
-		void			
+		void
 	PushRenderState(
 		CanvasRenderState & render_state
 	);
@@ -270,5 +274,4 @@ protected:
 	static TBannerId s_bannerId;
 };
 
-#endif // __GPC_CANVAS_H__
-
+#endif  /* __GPC_CANVAS_H__ */
