@@ -73,9 +73,6 @@ void printfGlyph(bmGlyph * glyph)
 }
 */
 
-#define MAX2(x,y)          ( (x)>(y) ? (x) : (y) )
-#define MAX3(x,y,z)                MAX2( MAX2((x),(y)) , (z) )  
-
 void calcAlpha(ImBuf * ibuf)
 {
 	int i;
@@ -104,13 +101,13 @@ void readBitmapFontVersion0(ImBuf * ibuf, unsigned char * rect, int step)
 	ysize = (bytes + (ibuf->x - 1)) / ibuf->x;
 	
 	if (ysize < ibuf->y) {
-		// we're first going to copy all data into a liniar buffer.
+		// we're first going to copy all data into a linear buffer.
 		// step can be 4 or 1 bytes, and the data is not sequential because
 		// the bitmap was flipped vertically.
 		
 		buffer = (unsigned char*)MEM_mallocN(bytes, "readBitmapFontVersion0:buffer");
 		
-		index = 0;	
+		index = 0;
 		for (i = 0; i < bytes; i++) {
 			buffer[i] = rect[index];
 			index += step;
