@@ -33,8 +33,8 @@
 #define __GPG_CANVAS_H__
 
 #ifdef WIN32
-#pragma warning (disable : 4786)
-#endif // WIN32
+#pragma warning (disable:4786)
+#endif  /* WIN32 */
 
 #include "GPC_Canvas.h"
 
@@ -66,5 +66,4 @@ public:
 	void EndDraw() {};
 };
 
-#endif // __GPG_CANVAS_H__
-
+#endif  /* __GPG_CANVAS_H__ */
