@@ -47,6 +47,7 @@
 #endif // __APPLE__
 #include "KX_KetsjiEngine.h"
 #include "KX_PythonInit.h"
+#include "KX_PythonMain.h"
 
 /**********************************
 * Begin Blender include block
@@ -56,10 +57,11 @@ extern "C"
 {
 #endif  // __cplusplus
 #include "MEM_guardedalloc.h"
-#include "BKE_blender.h"	
-#include "BKE_global.h"	
-#include "BKE_icons.h"	
-#include "BKE_node.h"	
+#include "BKE_blender.h"
+#include "BKE_global.h"
+#include "BKE_icons.h"
+#include "BKE_image.h"
+#include "BKE_node.h"
 #include "BKE_report.h"
 #include "BKE_library.h"
 #include "BLI_threads.h"
@@ -97,7 +99,6 @@ extern char datatoc_bfont_ttf[];
 #include "RAS_IRasterizer.h"
 
 #include "BKE_main.h"
-#include "BKE_utildefines.h"
 
 #include "RNA_define.h"
 
@@ -118,13 +119,12 @@ static void mem_error_cb(const char *errorStr)
 }
 
 #ifdef WIN32
-typedef enum 
-{
-  SCREEN_SAVER_MODE_NONE = 0,
-  SCREEN_SAVER_MODE_PREVIEW,
-  SCREEN_SAVER_MODE_SAVER,
-  SCREEN_SAVER_MODE_CONFIGURATION,
-  SCREEN_SAVER_MODE_PASSWORD,
+typedef enum {
+	SCREEN_SAVER_MODE_NONE = 0,
+	SCREEN_SAVER_MODE_PREVIEW,
+	SCREEN_SAVER_MODE_SAVER,
+	SCREEN_SAVER_MODE_CONFIGURATION,
+	SCREEN_SAVER_MODE_PASSWORD,
 } ScreenSaverMode;
 
 static ScreenSaverMode scr_saver_mode = SCREEN_SAVER_MODE_NONE;
@@ -173,7 +173,7 @@ static BOOL scr_saver_init(int argc, char **argv)
 
 #endif /* WIN32 */
 
-void usage(const char* program, bool isBlenderPlayer)
+static void usage(const char* program, bool isBlenderPlayer)
 {
 	const char * consoleoption;
 	const char * example_filename = "";
@@ -195,7 +195,7 @@ void usage(const char* program, bool isBlenderPlayer)
 	}
 	
 	printf("usage:   %s [-w [w h l t]] [-f [fw fh fb ff]] %s[-g gamengineoptions] "
-		"[-s stereomode] [-m aasamples] %s\n", program, consoleoption, example_filename);
+	       "[-s stereomode] [-m aasamples] %s\n", program, consoleoption, example_filename);
 	printf("  -h: Prints this command summary\n\n");
 	printf("  -w: display in a window\n");
 	printf("       --Optional parameters--\n"); 
@@ -224,7 +224,7 @@ void usage(const char* program, bool isBlenderPlayer)
 	printf("       --Optional parameters--\n");
 	printf("       angle    = field of view in degrees\n");
 	printf("       tilt     = tilt angle in degrees\n");
-	printf("       warpdata = a file to use for warping the image (absolute path)\n");	
+	printf("       warpdata = a file to use for warping the image (absolute path)\n");
 	printf("       mode: fisheye                (Fisheye)\n");
 	printf("             truncatedfront         (Front-Truncated)\n");
 	printf("             truncatedrear          (Rear-Truncated)\n");
@@ -258,10 +258,10 @@ void usage(const char* program, bool isBlenderPlayer)
 static void get_filename(int argc, char **argv, char *filename)
 {
 #ifdef __APPLE__
-/* On Mac we park the game file (called game.blend) in the application bundle.
-* The executable is located in the bundle as well.
-* Therefore, we can locate the game relative to the executable.
-	*/
+	/* On Mac we park the game file (called game.blend) in the application bundle.
+	 * The executable is located in the bundle as well.
+	 * Therefore, we can locate the game relative to the executable.
+	 */
 	int srclen = ::strlen(argv[0]);
 	int len = 0;
 	char *gamefile = NULL;
@@ -333,6 +333,40 @@ static BlendFileData *load_game_data(const char *progname, char *filename = NULL
 	return bfd;
 }
 
+static bool GPG_NextFrame(GHOST_ISystem* system, GPG_Application *app, int &exitcode, STR_String &exitstring, GlobalSettings *gs)
+{
+	bool run = true;
+	system->processEvents(false);
+	system->dispatchEvents();
+	app->EngineNextFrame();
+	if ((exitcode = app->getExitRequested())) {
+		run = false;
+		exitstring = app->getExitString();
+		*gs = *app->getGlobalSettings();
+	}
+	return run;
+}
+
+struct GPG_NextFrameState {
+	GHOST_ISystem* system;
+	GPG_Application *app;
+	GlobalSettings *gs;
+} gpg_nextframestate;
+
+static int GPG_PyNextFrame(void *state0)
+{
+	GPG_NextFrameState *state = (GPG_NextFrameState *) state0;
+	int exitcode;
+	STR_String exitstring;
+	bool run = GPG_NextFrame(state->system, state->app, exitcode, exitstring, state->gs);
+	if (run) return 0;  
+	else {
+		if (exitcode) 
+			fprintf(stderr, "Exit code %d: %s\n", exitcode, exitstring.ReadPtr());
+		return 1;
+	}
+}
+
 int main(int argc, char** argv)
 {
 	int i;
@@ -413,11 +447,15 @@ int main(int argc, char** argv)
 	G.main = NULL;
 
 	IMB_init();
+	BKE_images_init();
+
+#ifdef WITH_FFMPEG
+	IMB_ffmpeg_init();
+#endif
 
 	// Setup builtin font for BLF (mostly copied from creator.c, wm_init_exit.c and interface_style.c)
 	BLF_init(11, U.dpi);
 	BLF_lang_init();
-	BLF_lang_encoding("");
 	BLF_lang_set("");
 
 	BLF_load_mem("default", (unsigned char*)datatoc_bfont_ttf, datatoc_bfont_ttf_size);
@@ -437,7 +475,7 @@ int main(int argc, char** argv)
 			break;
 		case SCREEN_SAVER_MODE_PASSWORD:
 			/* This is W95 only, which we currently do not support.
-			   Fall-back to normal screen saver behavior in that case... */
+			 * Fall-back to normal screen saver behavior in that case... */
 		case SCREEN_SAVER_MODE_SAVER:
 			fullScreen = true;
 			fullScreenParFound = true;
@@ -458,6 +496,8 @@ int main(int argc, char** argv)
 
 	// XXX this one too
 	U.anisotropic_filter = 2;
+	// enable fast mipmap generation
+	U.use_gpu_mipmap = 1;
 
 	sound_init_once();
 
@@ -476,7 +516,7 @@ int main(int argc, char** argv)
 
 	{
 #if defined(DEBUG)
-		printf("argv[%d] = '%s'   , %i\n", i, argv[i],argc);
+		printf("argv[%d] = '%s', %i\n", i, argv[i],argc);
 #endif
 		if (argv[i][0] == '-')
 		{
@@ -722,6 +762,7 @@ int main(int argc, char** argv)
 		}
 
 		GPU_set_anisotropic(U.anisotropic_filter);
+		GPU_set_gpu_mipmapping(U.use_gpu_mipmap);
 		
 		// Create the system
 		if (GHOST_ISystem::createSystem() == GHOST_kSuccess)
@@ -848,7 +889,7 @@ int main(int argc, char** argv)
 						{
 							if (scene->gm.stereoflag == STEREO_ENABLED) {
 								stereomode = (RAS_IRasterizer::StereoMode) scene->gm.stereomode;
-								if (stereomode != RAS_IRasterizer::RAS_STEREO_QUADBUFFERED)
+								if (stereomode == RAS_IRasterizer::RAS_STEREO_QUADBUFFERED)
 									stereoWindow = true;
 							}
 						}
@@ -870,7 +911,7 @@ int main(int argc, char** argv)
 							if (domeWarp)
 							{
 								//XXX to do: convert relative to absolute path
-								domeText= add_text(domeWarp, "");
+								domeText= BKE_text_load(G.main, domeWarp, "");
 								if (!domeText)
 									printf("error: invalid warpdata text file - %s\n", domeWarp);
 								else
@@ -967,15 +1008,39 @@ int main(int argc, char** argv)
 						
 						// Enter main loop
 						bool run = true;
-						while (run)
+						char *python_main = NULL;
+						pynextframestate.state = NULL;
+						pynextframestate.func = NULL;
+#ifdef WITH_PYTHON
+						python_main = KX_GetPythonMain(scene);
+#endif // WITH_PYTHON
+						if (python_main) 
 						{
-							system->processEvents(false);
-							system->dispatchEvents();
-							if ((exitcode = app.getExitRequested()))
+							char *python_code = KX_GetPythonCode(maggie, python_main);
+							if (python_code)
 							{
-								run = false;
-								exitstring = app.getExitString();
-								gs = *app.getGlobalSettings();
+#ifdef WITH_PYTHON
+								gpg_nextframestate.system = system;
+								gpg_nextframestate.app = &app;
+								gpg_nextframestate.gs = &gs;
+								pynextframestate.state = &gpg_nextframestate;
+								pynextframestate.func = &GPG_PyNextFrame;
+
+								printf("Yielding control to Python script '%s'...\n", python_main);
+								PyRun_SimpleString(python_code);
+								printf("Exit Python script '%s'\n", python_main);
+#endif // WITH_PYTHON
+								MEM_freeN(python_code);
+							}
+							else {
+								fprintf(stderr, "ERROR: cannot yield control to Python: no Python text data block named '%s'\n", python_main);
+							}
+						}
+						else
+						{
+							while (run)
+							{
+								run = GPG_NextFrame(system, &app, exitcode, exitstring, &gs);
 							}
 						}
 						app.StopGameEngine();
@@ -985,6 +1050,7 @@ int main(int argc, char** argv)
 						system->removeEventConsumer(&app);
 
 						BLO_blendfiledata_free(bfd);
+						if (python_main) MEM_freeN(python_main);
 					}
 				} while (exitcode == KX_EXIT_REQUEST_RESTART_GAME || exitcode == KX_EXIT_REQUEST_START_OTHER_GAME);
 			}
@@ -1006,9 +1072,11 @@ int main(int argc, char** argv)
 
 #ifdef WITH_INTERNATIONAL
 	BLF_free_unifont();
+	BLF_lang_free();
 #endif
 
 	IMB_exit();
+	BKE_images_exit();
 	free_nodesystem();
 
 	SYS_DeleteSystem(syshandle);
