@@ -36,7 +36,6 @@
 
 // These three are for getting the action from the logic manager
 #include "KX_Scene.h"
-#include "KX_PythonInit.h"
 #include "SCA_LogicManager.h"
 
 extern "C" {
@@ -44,6 +43,10 @@ extern "C" {
 #include "BKE_action.h"
 #include "RNA_access.h"
 #include "RNA_define.h"
+
+// Needed for material IPOs
+#include "BKE_material.h"
+#include "DNA_material_types.h"
 }
 
 BL_Action::BL_Action(class KX_GameObject* gameobj)
@@ -127,13 +130,15 @@ bool BL_Action::Play(const char* name,
 
 	// Only start playing a new action if we're done, or if
 	// the new action has a higher priority
-	if (priority != 0 && !IsDone() && priority >= m_priority)
+	if (!IsDone() && priority > m_priority)
 		return false;
 	m_priority = priority;
 	bAction* prev_action = m_action;
 
+	KX_Scene* kxscene = m_obj->GetScene();
+
 	// First try to load the action
-	m_action = (bAction*)KX_GetActiveScene()->GetLogicManager()->GetActionByName(name);
+	m_action = (bAction*)kxscene->GetLogicManager()->GetActionByName(name);
 	if (!m_action)
 	{
 		printf("Failed to load action: %s\n", name);
@@ -141,33 +146,77 @@ bool BL_Action::Play(const char* name,
 		return false;
 	}
 
-	if (prev_action != m_action)
-	{
-		// First get rid of any old controllers
-		ClearControllerList();
+	// If we have the same settings, don't play again
+	// This is to resolve potential issues with pulses on sensors such as the ones
+	// reported in bug #29412. The fix is here so it works for both logic bricks and Python.
+	// However, this may eventually lead to issues where a user wants to override an already
+	// playing action with the same action and settings. If this becomes an issue,
+	// then this fix may have to be re-evaluated.
+	if (!IsDone() && m_action == prev_action && m_startframe == start && m_endframe == end
+			&& m_priority == priority && m_speed == playback_speed)
+		return false;
 
-		// Create an SG_Controller
-		SG_Controller *sg_contr = BL_CreateIPO(m_action, m_obj, KX_GetActiveScene()->GetSceneConverter());
+	// First get rid of any old controllers
+	ClearControllerList();
+
+	// Create an SG_Controller
+	SG_Controller *sg_contr = BL_CreateIPO(m_action, m_obj, kxscene->GetSceneConverter());
+	m_sg_contr_list.push_back(sg_contr);
+	m_obj->GetSGNode()->AddSGController(sg_contr);
+	sg_contr->SetObject(m_obj->GetSGNode());
+
+	// Try obcolor
+	sg_contr = BL_CreateObColorIPO(m_action, m_obj, kxscene->GetSceneConverter());
+	if (sg_contr) {
 		m_sg_contr_list.push_back(sg_contr);
 		m_obj->GetSGNode()->AddSGController(sg_contr);
 		sg_contr->SetObject(m_obj->GetSGNode());
+	}
 
-		// Extra controllers
-		if (m_obj->GetGameObjectType() == SCA_IObject::OBJ_LIGHT)
-		{
-			sg_contr = BL_CreateLampIPO(m_action, m_obj, KX_GetActiveScene()->GetSceneConverter());
-			m_sg_contr_list.push_back(sg_contr);
-			m_obj->GetSGNode()->AddSGController(sg_contr);
-			sg_contr->SetObject(m_obj->GetSGNode());
+	// Now try materials
+	if (m_obj->GetBlenderObject()->totcol==1) {
+		Material *mat = give_current_material(m_obj->GetBlenderObject(), 1);
+		if (mat) {
+			sg_contr = BL_CreateMaterialIpo(m_action, mat, 0, m_obj, kxscene->GetSceneConverter());
+			if (sg_contr) {
+				m_sg_contr_list.push_back(sg_contr);
+				m_obj->GetSGNode()->AddSGController(sg_contr);
+				sg_contr->SetObject(m_obj->GetSGNode());
+			}
 		}
-		else if (m_obj->GetGameObjectType() == SCA_IObject::OBJ_CAMERA)
-		{
-			sg_contr = BL_CreateCameraIPO(m_action, m_obj, KX_GetActiveScene()->GetSceneConverter());
-			m_sg_contr_list.push_back(sg_contr);
-			m_obj->GetSGNode()->AddSGController(sg_contr);
-			sg_contr->SetObject(m_obj->GetSGNode());
+	} else {
+		Material *mat;
+		STR_HashedString matname;
+
+		for (int matidx = 1; matidx <= m_obj->GetBlenderObject()->totcol; ++matidx) {
+			mat = give_current_material(m_obj->GetBlenderObject(), matidx);
+			if (mat) {
+				matname = mat->id.name;
+				sg_contr = BL_CreateMaterialIpo(m_action, mat, matname.hash(), m_obj, kxscene->GetSceneConverter());
+				if (sg_contr) {
+					m_sg_contr_list.push_back(sg_contr);
+					m_obj->GetSGNode()->AddSGController(sg_contr);
+					sg_contr->SetObject(m_obj->GetSGNode());
+				}
+			}
 		}
 	}
+
+	// Extra controllers
+	if (m_obj->GetGameObjectType() == SCA_IObject::OBJ_LIGHT)
+	{
+		sg_contr = BL_CreateLampIPO(m_action, m_obj, kxscene->GetSceneConverter());
+		m_sg_contr_list.push_back(sg_contr);
+		m_obj->GetSGNode()->AddSGController(sg_contr);
+		sg_contr->SetObject(m_obj->GetSGNode());
+	}
+	else if (m_obj->GetGameObjectType() == SCA_IObject::OBJ_CAMERA)
+	{
+		sg_contr = BL_CreateCameraIPO(m_action, m_obj, kxscene->GetSceneConverter());
+		m_sg_contr_list.push_back(sg_contr);
+		m_obj->GetSGNode()->AddSGController(sg_contr);
+		sg_contr->SetObject(m_obj->GetSGNode());
+	}
 	
 	m_ipo_flags = ipo_flags;
 	InitIPO();
@@ -196,7 +245,7 @@ bool BL_Action::Play(const char* name,
 	}
 
 	// Now that we have an action, we have something we can play
-	m_starttime = KX_GetActiveEngine()->GetFrameTime();
+	m_starttime = -1.f; // We get the start time on our first update
 	m_startframe = m_localtime = start;
 	m_endframe = end;
 	m_blendin = blendin;
@@ -328,6 +377,11 @@ void BL_Action::Update(float curtime)
 
 	curtime -= KX_KetsjiEngine::GetSuspendedDelta();
 
+	// Grab the start time here so we don't end up with a negative m_localtime when
+	// suspending and resuming scenes.
+	if (m_starttime < 0)
+		m_starttime = curtime;
+
 	if (m_calc_localtime)
 		SetLocalTime(curtime);
 	else
@@ -337,29 +391,27 @@ void BL_Action::Update(float curtime)
 	}
 
 	// Handle wrap around
-	if (m_localtime < min(m_startframe, m_endframe) || m_localtime > max(m_startframe, m_endframe))
-	{
-		switch(m_playmode)
-		{
-		case ACT_MODE_PLAY:
-			// Clamp
-			m_localtime = m_endframe;
-			m_done = true;
-			break;
-		case ACT_MODE_LOOP:
-			// Put the time back to the beginning
-			m_localtime = m_startframe;
-			m_starttime = curtime;
-			break;
-		case ACT_MODE_PING_PONG:
-			// Swap the start and end frames
-			float temp = m_startframe;
-			m_startframe = m_endframe;
-			m_endframe = temp;
+	if (m_localtime < min(m_startframe, m_endframe) || m_localtime > max(m_startframe, m_endframe)) {
+		switch (m_playmode) {
+			case ACT_MODE_PLAY:
+				// Clamp
+				m_localtime = m_endframe;
+				m_done = true;
+				break;
+			case ACT_MODE_LOOP:
+				// Put the time back to the beginning
+				m_localtime = m_startframe;
+				m_starttime = curtime;
+				break;
+			case ACT_MODE_PING_PONG:
+				// Swap the start and end frames
+				float temp = m_startframe;
+				m_startframe = m_endframe;
+				m_endframe = temp;
 
-			m_starttime = curtime;
+				m_starttime = curtime;
 
-			break;
+				break;
 		}
 	}
 
@@ -445,4 +497,7 @@ void BL_Action::Update(float curtime)
 
 		m_obj->UpdateIPO(m_localtime, m_ipo_flags & ACT_IPOFLAG_CHILD);
 	}
+
+	if (m_done)
+		ClearControllerList();
 }
