@@ -132,11 +132,8 @@ public:
 	};
 
 #ifdef WITH_CXX_GUARDEDALLOC
-public:
-	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:BL_Action"); }
-	void operator delete( void *mem ) { MEM_freeN(mem); }
+	MEM_CXX_CLASS_ALLOC_FUNCS("GE:BL_Action")
 #endif
 };
 
-#endif //BL_ACTION
-
+#endif  /* BL_ACTION */
