@@ -6,6 +6,8 @@
 #include "DNA_material_types.h"
 #include "DNA_scene_types.h"
 
+#include "BLI_utildefines.h"
+
 #include "BKE_global.h"
 #include "BKE_main.h"
 #include "BKE_DerivedMesh.h"
@@ -78,7 +80,7 @@ void BL_BlenderShader::SetAttribs(RAS_IRasterizer* ras, const BL_Material *mat)
 {
 	GPUVertexAttribs attribs;
 	GPUMaterial *gpumat;
-	int i, attrib_num;
+	int i, attrib_num, uv = 0;
 
 	ras->SetAttribNum(0);
 
@@ -93,21 +95,15 @@ void BL_BlenderShader::SetAttribs(RAS_IRasterizer* ras, const BL_Material *mat)
 
 		ras->SetTexCoordNum(0);
 		ras->SetAttribNum(attrib_num);
-		for (i=0; i<attrib_num; i++)
+		for (i = 0; i < attrib_num; i++)
 			ras->SetAttrib(RAS_IRasterizer::RAS_TEXCO_DISABLE, i);
 
 		for (i = 0; i < attribs.totlayer; i++) {
 			if (attribs.layer[i].glindex > attrib_num)
 				continue;
 
-			if (attribs.layer[i].type == CD_MTFACE) {
-				if (!mat->uvName.IsEmpty() && strcmp(mat->uvName.ReadPtr(), attribs.layer[i].name) == 0)
-					ras->SetAttrib(RAS_IRasterizer::RAS_TEXCO_UV1, attribs.layer[i].glindex);
-				else if (!mat->uv2Name.IsEmpty() && strcmp(mat->uv2Name.ReadPtr(), attribs.layer[i].name) == 0)
-					ras->SetAttrib(RAS_IRasterizer::RAS_TEXCO_UV2, attribs.layer[i].glindex);
-				else
-					ras->SetAttrib(RAS_IRasterizer::RAS_TEXCO_UV1, attribs.layer[i].glindex);
-			}
+			if (attribs.layer[i].type == CD_MTFACE)
+				ras->SetAttrib(RAS_IRasterizer::RAS_TEXCO_UV, attribs.layer[i].glindex, uv++);
 			else if (attribs.layer[i].type == CD_TANGENT)
 				ras->SetAttrib(RAS_IRasterizer::RAS_TEXTANGENT, attribs.layer[i].glindex);
 			else if (attribs.layer[i].type == CD_ORCO)
@@ -143,9 +139,9 @@ void BL_BlenderShader::Update(const RAS_MeshSlot & ms, RAS_IRasterizer* rasty )
 	viewinv.getValue((float*)viewinvmat);
 
 	if (ms.m_bObjectColor)
-		ms.m_RGBAcolor.getValue((float*)obcol);
+		ms.m_RGBAcolor.getValue((float *)obcol);
 	else
-		obcol[0]= obcol[1]= obcol[2]= obcol[3]= 1.0f;
+		obcol[0] = obcol[1] = obcol[2] = obcol[3] = 1.0f;
 
 	float auto_bump_scale = ms.m_pDerivedMesh!=0 ? ms.m_pDerivedMesh->auto_bump_scale : 1.0f;
 	GPU_material_bind_uniforms(gpumat, obmat, viewmat, viewinvmat, obcol, auto_bump_scale);
