@@ -97,10 +97,8 @@ public:
 	
 	
 #ifdef WITH_CXX_GUARDEDALLOC
-public:
-	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:BL_BlenderShader"); }
-	void operator delete( void *mem ) { MEM_freeN(mem); }
+	MEM_CXX_CLASS_ALLOC_FUNCS("GE:BL_BlenderShader")
 #endif
 };
 
-#endif//__BL_BLENDERSHADER_H__
+#endif /* __BL_BLENDERSHADER_H__ */
