@@ -60,7 +60,8 @@ KX_BlenderMaterial::KX_BlenderMaterial()
 void KX_BlenderMaterial::Initialize(
 	KX_Scene *scene,
 	BL_Material *data,
-	GameSettings *game)
+	GameSettings *game,
+	int lightlayer)
 {
 	RAS_IPolyMaterial::Initialize(
 		data->texname[0],
@@ -84,6 +85,7 @@ void KX_BlenderMaterial::Initialize(
 	mModified = 0;
 	mConstructed = false;
 	mPass = 0;
+	mLightLayer = lightlayer;
 	// --------------------------------
 	// RAS_IPolyMaterial variables...
 	m_flag |= RAS_BLENDERMAT;
@@ -92,16 +94,11 @@ void KX_BlenderMaterial::Initialize(
 	m_flag |= (mMaterial->glslmat)? RAS_BLENDERGLSL: 0;
 	m_flag |= ((mMaterial->ras_mode & CAST_SHADOW)!=0)? RAS_CASTSHADOW: 0;
 
-	// figure max
-	int enabled = mMaterial->num_enabled;
-	int max = BL_Texture::GetMaxUnits();
-	mMaterial->num_enabled = enabled>=max?max:enabled;
-
 	// test the sum of the various modes for equality
 	// so we can ether accept or reject this material
 	// as being equal, this is rather important to
 	// prevent material bleeding
-	for (int i=0; i<mMaterial->num_enabled; i++) {
+	for (int i=0; i<BL_Texture::GetMaxUnits(); i++) {
 		m_multimode	+= (mMaterial->flag[i] + mMaterial->blend_mode[i]);
 	}
 	m_multimode += mMaterial->IdMode+ (mMaterial->ras_mode & ~(USE_LIGHT));
@@ -130,10 +127,10 @@ unsigned int* KX_BlenderMaterial::GetMCol(void) const
 void KX_BlenderMaterial::GetMaterialRGBAColor(unsigned char *rgba) const
 {
 	if (mMaterial) {
-		*rgba++ = (unsigned char) (mMaterial->matcolor[0]*255.0);
-		*rgba++ = (unsigned char) (mMaterial->matcolor[1]*255.0);
-		*rgba++ = (unsigned char) (mMaterial->matcolor[2]*255.0);
-		*rgba++ = (unsigned char) (mMaterial->matcolor[3]*255.0);
+		*rgba++ = (unsigned char)(mMaterial->matcolor[0] * 255.0f);
+		*rgba++ = (unsigned char)(mMaterial->matcolor[1] * 255.0f);
+		*rgba++ = (unsigned char)(mMaterial->matcolor[2] * 255.0f);
+		*rgba++ = (unsigned char)(mMaterial->matcolor[3] * 255.0f);
 	} else
 		RAS_IPolyMaterial::GetMaterialRGBAColor(rgba);
 }
@@ -154,29 +151,23 @@ void KX_BlenderMaterial::ReleaseMaterial()
 		mBlenderShader->ReloadMaterial();
 }
 
-void KX_BlenderMaterial::OnConstruction(int layer)
+void KX_BlenderMaterial::InitTextures()
 {
-	if (mConstructed)
-		// when material are reused between objects
-		return;
-	
-	if (mMaterial->glslmat)
-		SetBlenderGLSLShader(layer);
-
 	// for each unique material...
 	int i;
-	for (i=0; i<mMaterial->num_enabled; i++) {
+	for (i=0; i<BL_Texture::GetMaxUnits(); i++) {
 		if ( mMaterial->mapping[i].mapping & USEENV ) {
 			if (!GLEW_ARB_texture_cube_map) {
 				spit("CubeMap textures not supported");
 				continue;
 			}
 			if (!mTextures[i].InitCubeMap(i, mMaterial->cubemap[i] ) )
-				spit("unable to initialize image("<<i<<") in "<< 
-						 mMaterial->matname<< ", image will not be available");
-		} 
-	
-		else {
+				spit("unable to initialize image("<<i<<") in "<<
+				     mMaterial->matname<< ", image will not be available");
+		}
+		/* If we're using glsl materials, the textures are handled by bf_gpu, so don't load them twice!
+		 * However, if we're using a custom shader, then we still need to load the textures ourselves. */
+		else if (!mMaterial->glslmat || mShader) {
 			if ( mMaterial->img[i] ) {
 				if ( ! mTextures[i].InitFromImage(i, mMaterial->img[i], (mMaterial->flag[i] &MIPMAP)!=0 ))
 					spit("unable to initialize image("<<i<<") in "<< 
@@ -184,6 +175,18 @@ void KX_BlenderMaterial::OnConstruction(int layer)
 			}
 		}
 	}
+}
+
+void KX_BlenderMaterial::OnConstruction()
+{
+	if (mConstructed)
+		// when material are reused between objects
+		return;
+	
+	if (mMaterial->glslmat)
+		SetBlenderGLSLShader();
+
+	InitTextures();
 
 	mBlendFunc[0] =0;
 	mBlendFunc[1] =0;
@@ -228,7 +231,8 @@ void KX_BlenderMaterial::OnExit()
 	}
 
 	BL_Texture::ActivateFirst();
-	for (int i=0; i<mMaterial->num_enabled; i++) {
+	for (int i=0; i<BL_Texture::GetMaxUnits(); i++) {
+		if (!mTextures[i].Ok()) continue;
 		BL_Texture::ActivateUnit(i);
 		mTextures[i].DeleteTex();
 		mTextures[i].DisableUnit();
@@ -267,7 +271,7 @@ void KX_BlenderMaterial::setShaderData( bool enable, RAS_IRasterizer *ras)
 	mShader->ApplyShader();
 
 	// for each enabled unit
-	for (i=0; i<mMaterial->num_enabled; i++) {
+	for (i=0; i<BL_Texture::GetMaxUnits(); i++) {
 		if (!mTextures[i].Ok()) continue;
 		mTextures[i].ActivateTexture();
 		mTextures[0].SetMapping(mMaterial->mapping[i].mapping);
@@ -343,7 +347,7 @@ void KX_BlenderMaterial::setTexData( bool enable, RAS_IRasterizer *ras)
 	}
 
 	int mode = 0,i=0;
-	for (i=0; (i<mMaterial->num_enabled && i<MAXTEX); i++) {
+	for (i=0; i<BL_Texture::GetMaxUnits(); i++) {
 		if ( !mTextures[i].Ok() ) continue;
 
 		mTextures[i].ActivateTexture();
@@ -396,6 +400,8 @@ KX_BlenderMaterial::ActivatShaders(
 	
 		if (rasty->GetDrawingMode() == RAS_IRasterizer::KX_TEXTURED)
 			tmp->setShaderData(true, rasty);
+		else if (rasty->GetDrawingMode() == RAS_IRasterizer::KX_SHADOW && IsAlpha() && !rasty->GetUsingOverrideShader())
+			tmp->setShaderData(true, rasty);
 		else
 			tmp->setShaderData(false, rasty);
 
@@ -406,7 +412,7 @@ KX_BlenderMaterial::ActivatShaders(
 
 		if ((mMaterial->ras_mode &WIRE) ||
 		    (rasty->GetDrawingMode() <= RAS_IRasterizer::KX_WIREFRAME))
-		{		
+		{
 			if (mMaterial->ras_mode &WIRE) 
 				rasty->SetCullFace(false);
 			rasty->SetLines(true);
@@ -441,6 +447,8 @@ KX_BlenderMaterial::ActivateBlenderShaders(
 	
 		if (rasty->GetDrawingMode() == RAS_IRasterizer::KX_TEXTURED)
 			tmp->setBlenderShaderData(true, rasty);
+		else if (rasty->GetDrawingMode() == RAS_IRasterizer::KX_SHADOW && IsAlpha() && !rasty->GetUsingOverrideShader())
+			tmp->setBlenderShaderData(true, rasty);
 		else
 			tmp->setBlenderShaderData(false, rasty);
 
@@ -451,7 +459,7 @@ KX_BlenderMaterial::ActivateBlenderShaders(
 
 		if ((mMaterial->ras_mode &WIRE) ||
 		    (rasty->GetDrawingMode() <= RAS_IRasterizer::KX_WIREFRAME))
-		{		
+		{
 			if (mMaterial->ras_mode &WIRE) 
 				rasty->SetCullFace(false);
 			rasty->SetLines(true);
@@ -490,6 +498,8 @@ KX_BlenderMaterial::ActivateMat(
 
 		if (rasty->GetDrawingMode() == RAS_IRasterizer::KX_TEXTURED)
 			tmp->setTexData( true,rasty  );
+		else if (rasty->GetDrawingMode() == RAS_IRasterizer::KX_SHADOW && IsAlpha() && !rasty->GetUsingOverrideShader())
+			tmp->setTexData(true, rasty);
 		else
 			tmp->setTexData( false,rasty);
 
@@ -500,7 +510,7 @@ KX_BlenderMaterial::ActivateMat(
 
 		if ((mMaterial->ras_mode &WIRE) ||
 		    (rasty->GetDrawingMode() <= RAS_IRasterizer::KX_WIREFRAME))
-		{		
+		{
 			if (mMaterial->ras_mode &WIRE) 
 				rasty->SetCullFace(false);
 			rasty->SetLines(true);
@@ -608,7 +618,7 @@ void KX_BlenderMaterial::ActivatGLMaterials( RAS_IRasterizer* rasty )const
 			mMaterial->matcolor[2]*mMaterial->ref+mMaterial->emit,
 			1.0f);
 
-		rasty->SetEmissive(	
+		rasty->SetEmissive(
 			mMaterial->matcolor[0]*mMaterial->emit,
 			mMaterial->matcolor[1]*mMaterial->emit,
 			mMaterial->matcolor[2]*mMaterial->emit,
@@ -624,7 +634,8 @@ void KX_BlenderMaterial::ActivatGLMaterials( RAS_IRasterizer* rasty )const
 
 void KX_BlenderMaterial::ActivateTexGen(RAS_IRasterizer *ras) const
 {
-	if (ras->GetDrawingMode() == RAS_IRasterizer::KX_TEXTURED) {
+	if (ras->GetDrawingMode() == RAS_IRasterizer::KX_TEXTURED || 
+		(ras->GetDrawingMode() == RAS_IRasterizer::KX_SHADOW && IsAlpha() && !ras->GetUsingOverrideShader())) {
 		ras->SetAttribNum(0);
 		if (mShader && GLEW_ARB_shader_objects) {
 			if (mShader->GetAttribute() == BL_Shader::SHD_TANGENT) {
@@ -636,16 +647,9 @@ void KX_BlenderMaterial::ActivateTexGen(RAS_IRasterizer *ras) const
 
 		ras->SetTexCoordNum(mMaterial->num_enabled);
 
-		for (int i=0; i<mMaterial->num_enabled; i++) {
+		for (int i=0; i<BL_Texture::GetMaxUnits(); i++) {
 			int mode = mMaterial->mapping[i].mapping;
 
-			if (mode &USECUSTOMUV)
-			{
-				if (!mMaterial->mapping[i].uvCoName.IsEmpty())
-					ras->SetTexCoord(RAS_IRasterizer::RAS_TEXCO_UV2, i);
-				continue;
-			}
-
 			if ( mode &(USEREFL|USEOBJ))
 				ras->SetTexCoord(RAS_IRasterizer::RAS_TEXCO_GEN, i);
 			else if (mode &USEORCO)
@@ -653,7 +657,7 @@ void KX_BlenderMaterial::ActivateTexGen(RAS_IRasterizer *ras) const
 			else if (mode &USENORM)
 				ras->SetTexCoord(RAS_IRasterizer::RAS_TEXCO_NORM, i);
 			else if (mode &USEUV)
-				ras->SetTexCoord(RAS_IRasterizer::RAS_TEXCO_UV1, i);
+				ras->SetTexCoord(RAS_IRasterizer::RAS_TEXCO_UV, i);
 			else if (mode &USETANG)
 				ras->SetTexCoord(RAS_IRasterizer::RAS_TEXTANGENT, i);
 			else 
@@ -720,7 +724,7 @@ void KX_BlenderMaterial::setObjectMatrixData(int i, RAS_IRasterizer *ras)
 	GLenum plane = GL_EYE_PLANE;
 
 	// figure plane gen
-	float proj[4]= {0.f,0.f,0.f,0.f};
+	float proj[4] = {0.f,0.f,0.f,0.f};
 	GetProjPlane(mMaterial, i, 0, proj);
 	glTexGenfv(GL_S, plane, proj);
 	
@@ -779,10 +783,19 @@ void KX_BlenderMaterial::UpdateIPO(
 	mMaterial->ref			= (float)(ref);
 }
 
-void KX_BlenderMaterial::SetBlenderGLSLShader(int layer)
+void KX_BlenderMaterial::Replace_IScene(SCA_IScene *val)
+{
+	mScene= static_cast<KX_Scene *>(val);
+	if (mBlenderShader)
+		mBlenderShader->SetScene(mScene);
+	
+	OnConstruction();
+}
+
+void KX_BlenderMaterial::SetBlenderGLSLShader()
 {
 	if (!mBlenderShader)
-		mBlenderShader = new BL_BlenderShader(mScene, mMaterial->material, layer);
+		mBlenderShader = new BL_BlenderShader(mScene, mMaterial->material, mLightLayer);
 
 	if (!mBlenderShader->Ok()) {
 		delete mBlenderShader;
@@ -829,29 +842,29 @@ PyTypeObject KX_BlenderMaterial::Type = {
 	py_base_new
 };
 
-PyObject* KX_BlenderMaterial::pyattr_get_shader(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *KX_BlenderMaterial::pyattr_get_shader(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	KX_BlenderMaterial* self= static_cast<KX_BlenderMaterial*>(self_v);
+	KX_BlenderMaterial* self = static_cast<KX_BlenderMaterial*>(self_v);
 	return self->PygetShader(NULL, NULL);
 }
 
-PyObject* KX_BlenderMaterial::pyattr_get_materialIndex(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *KX_BlenderMaterial::pyattr_get_materialIndex(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	KX_BlenderMaterial* self= static_cast<KX_BlenderMaterial*>(self_v);
-	return PyLong_FromSsize_t(self->GetMaterialIndex());
+	KX_BlenderMaterial* self = static_cast<KX_BlenderMaterial*>(self_v);
+	return PyLong_FromLong(self->GetMaterialIndex());
 }
 
-PyObject* KX_BlenderMaterial::pyattr_get_blending(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *KX_BlenderMaterial::pyattr_get_blending(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	KX_BlenderMaterial* self= static_cast<KX_BlenderMaterial*>(self_v);
+	KX_BlenderMaterial* self = static_cast<KX_BlenderMaterial*>(self_v);
 	unsigned int* bfunc = self->getBlendFunc();
 	return Py_BuildValue("(ll)", (long int)bfunc[0], (long int)bfunc[1]);
 }
 
 int KX_BlenderMaterial::pyattr_set_blending(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef, PyObject *value)
 {
-	KX_BlenderMaterial* self= static_cast<KX_BlenderMaterial*>(self_v);
-	PyObject* obj = self->PysetBlending(value, NULL);
+	KX_BlenderMaterial* self = static_cast<KX_BlenderMaterial*>(self_v);
+	PyObject *obj = self->PysetBlending(value, NULL);
 	if (obj)
 	{
 		Py_DECREF(obj);
@@ -878,7 +891,7 @@ KX_PYMETHODDEF_DOC( KX_BlenderMaterial, getShader , "getShader()")
 		Py_RETURN_NONE;
 	}
 
-	if (!GLEW_ARB_shader_objects)  {
+	if (!GLEW_ARB_shader_objects) {
 		if (!mModified)
 			spit("GLSL not supported");
 		mModified = true;
@@ -891,6 +904,9 @@ KX_PYMETHODDEF_DOC( KX_BlenderMaterial, getShader , "getShader()")
 		if (!mShader && !mModified) {
 			mShader = new BL_Shader();
 			mModified = true;
+
+			// Using a custom shader, make sure to initialize textures
+			InitTextures();
 		}
 
 		if (mShader && !mShader->GetError()) {
@@ -898,8 +914,8 @@ KX_PYMETHODDEF_DOC( KX_BlenderMaterial, getShader , "getShader()")
 			mMaterial->SetSharedMaterial(true);
 			mScene->GetBucketManager()->ReleaseDisplayLists(this);
 			return mShader->GetProxy();
-		}else
-		{
+		}
+		else {
 			// decref all references to the object
 			// then delete it!
 			// We will then go back to fixed functionality
@@ -917,7 +933,7 @@ KX_PYMETHODDEF_DOC( KX_BlenderMaterial, getShader , "getShader()")
 
 KX_PYMETHODDEF_DOC( KX_BlenderMaterial, getMaterialIndex, "getMaterialIndex()")
 {
-	return PyLong_FromSsize_t( GetMaterialIndex() );
+	return PyLong_FromLong(GetMaterialIndex());
 }
 
 KX_PYMETHODDEF_DOC( KX_BlenderMaterial, getTexture, "getTexture( index )" )
