@@ -4,7 +4,7 @@
 //under visual studio the #define in KX_ConvertPhysicsObject.h is quicker for recompilation
 #include "KX_ConvertPhysicsObject.h"
 
-#ifdef USE_BULLET
+#ifdef WITH_BULLET
 
 #include "KX_BulletPhysicsController.h"
 
@@ -20,8 +20,8 @@
 #include "BulletSoftBody/btSoftBody.h"
 
 
-KX_BulletPhysicsController::KX_BulletPhysicsController (const CcdConstructionInfo& ci, bool dyna, bool sensor, bool compound)
-: KX_IPhysicsController(dyna,sensor,compound,(PHY_IPhysicsController*)this),
+KX_BulletPhysicsController::KX_BulletPhysicsController (const CcdConstructionInfo& ci, bool dyna, bool sensor, bool character, bool compound)
+: KX_IPhysicsController(dyna,sensor,character,compound,(PHY_IPhysicsController*)this),
 CcdPhysicsController(ci),
 m_savedCollisionFlags(0),
 m_savedCollisionFilterGroup(0),
@@ -75,6 +75,11 @@ void  KX_BulletPhysicsController::SetLinVelocityMin(float val)
 	CcdPhysicsController::SetLinVelocityMin(val);
 }
 
+void KX_BulletPhysicsController::Jump()
+{
+	CcdPhysicsController::Jump();
+}
+
 float KX_BulletPhysicsController::GetLinVelocityMax()
 {
 	return (float)CcdPhysicsController::GetLinVelocityMax();
@@ -119,6 +124,11 @@ void	KX_BulletPhysicsController::RelativeTranslate(const MT_Vector3& dloc,bool l
 
 }
 
+void	KX_BulletPhysicsController::SetWalkDirection(const MT_Vector3& dloc,bool local)
+{
+	CcdPhysicsController::SetWalkDirection(dloc[0],dloc[1],dloc[2],local);
+}
+
 void	KX_BulletPhysicsController::RelativeRotate(const MT_Matrix3x3& drot,bool local)
 {
 	float	rotval[9];
@@ -155,6 +165,13 @@ MT_Vector3 KX_BulletPhysicsController::GetVelocity(const MT_Point3& pos)
 	return MT_Vector3(linVel[0],linVel[1],linVel[2]);
 }
 
+MT_Vector3 KX_BulletPhysicsController::GetWalkDirection()
+{
+	float dir[3];
+	CcdPhysicsController::GetWalkDirection(dir[0], dir[1], dir[2]);
+	return MT_Vector3(dir[0], dir[1], dir[2]);
+}
+
 void	KX_BulletPhysicsController::SetAngularVelocity(const MT_Vector3& ang_vel,bool local)
 {
 	CcdPhysicsController::SetAngularVelocity(ang_vel.x(),ang_vel.y(),ang_vel.z(),local);
@@ -232,6 +249,7 @@ MT_Vector3	KX_BulletPhysicsController::getReactionForce()
 }
 void	KX_BulletPhysicsController::setRigidBody(bool rigid)
 {
+	CcdPhysicsController::setRigidBody(rigid);
 }
 
 /* This function dynamically adds the collision shape of another controller to
@@ -268,7 +286,7 @@ void    KX_BulletPhysicsController::AddCompoundChild(KX_IPhysicsController* chil
 	rootScale[2] = 1.0/rootScale[2];
 	// relative scale = child_scale/parent_scale
 	btVector3 relativeScale = childShape->getLocalScaling()*rootScale;
-	btMatrix3x3 rootRotInverse = rootTrans.getBasis().transpose();	
+	btMatrix3x3 rootRotInverse = rootTrans.getBasis().transpose();
 	// relative pos = parent_rot^-1 * ((parent_pos-child_pos)/parent_scale)
 	btVector3 relativePos = rootRotInverse*((childTrans.getOrigin()-rootTrans.getOrigin())*rootScale);
 	// relative rot = parent_rot^-1 * child_rot
@@ -472,7 +490,7 @@ SG_Controller*	KX_BulletPhysicsController::GetReplica(class SG_Node* destnode)
 void	KX_BulletPhysicsController::SetSumoTransform(bool nondynaonly)
 {
 
-	if (!m_bDyna && !m_bSensor)
+	if (!m_bDyna && !m_bSensor && !m_bCharacter)
 	{
 		btCollisionObject* object = GetRigidBody();
 		object->setActivationState(ACTIVE_TAG);
@@ -535,4 +553,4 @@ const char* KX_BulletPhysicsController::getName()
 	return 0;
 }
 
-#endif // USE_BULLET
+#endif // WITH_BULLET
