@@ -30,11 +30,10 @@
  *  \ingroup ketsji
  */
 
-
-#if defined(WIN32) && !defined(FREE_WINDOWS)
-// This warning tells us about truncation of __long__ stl-generated names.
-// It can occasionally cause DevStudio to have internal compiler warnings.
-#pragma warning( disable : 4786 )     
+#ifdef _MSC_VER
+  /* This warning tells us about truncation of __long__ stl-generated names.
+   * It can occasionally cause DevStudio to have internal compiler warnings. */
+#  pragma warning(disable:4786)
 #endif
 
 #include "MT_Point3.h"
@@ -60,14 +59,14 @@
 /* ------------------------------------------------------------------------- */
 
 KX_MouseFocusSensor::KX_MouseFocusSensor(SCA_MouseManager* eventmgr, 
-										 int startx,
-										 int starty,
-										 short int mousemode,
-										 int focusmode,
-										 bool bTouchPulse,
-										 KX_Scene* kxscene,
-										 KX_KetsjiEngine *kxengine,
-										 SCA_IObject* gameobj)
+                                         int startx,
+                                         int starty,
+                                         short int mousemode,
+                                         int focusmode,
+                                         bool bTouchPulse,
+                                         KX_Scene* kxscene,
+                                         KX_KetsjiEngine *kxengine,
+                                         SCA_IObject* gameobj)
 	: SCA_MouseSensor(eventmgr, startx, starty, mousemode, gameobj),
 	  m_focusmode(focusmode),
 	  m_bTouchPulse(bTouchPulse),
@@ -142,7 +141,7 @@ bool KX_MouseFocusSensor::RayHit(KX_ClientObjectInfo* client_info, KX_RayCast* r
 	KX_GameObject* hitKXObj = client_info->m_gameobject;
 	
 	/* Is this me? In the ray test, there are a lot of extra checks
-	 * for aliasing artefacts from self-hits. That doesn't happen
+	 * for aliasing artifacts from self-hits. That doesn't happen
 	 * here, so a simple test suffices. Or does the camera also get
 	 * self-hits? (No, and the raysensor shouldn't do it either, since
 	 * self-hits are excluded by setting the correct ignore-object.)
@@ -239,7 +238,7 @@ bool KX_MouseFocusSensor::ParentObjectHasFocusCamera(KX_Camera *cam)
 	
 	/*	build the from and to point in normalized device coordinates 
 	 *	Normalized device coordinates are [-1,1] in x, y, z
-	 *	
+	 *
 	 *	The actual z coordinates used don't have to be exact just infront and 
 	 *	behind of the near and far clip planes.
 	 */ 
@@ -309,8 +308,7 @@ bool KX_MouseFocusSensor::ParentObjectHasFocus()
 	list<class KX_Camera*>* cameras = m_kxscene->GetCameras();
 	list<KX_Camera*>::iterator it = cameras->begin();
 	
-	while(it != cameras->end())
-	{
+	while (it != cameras->end()) {
 		if (((*it) != cam) && (*it)->GetViewport())
 			if (ParentObjectHasFocusCamera(*it))
 				return true;
@@ -392,30 +390,30 @@ PyAttributeDef KX_MouseFocusSensor::Attributes[] = {
 };
 
 /* Attributes */
-PyObject* KX_MouseFocusSensor::pyattr_get_ray_source(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *KX_MouseFocusSensor::pyattr_get_ray_source(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	KX_MouseFocusSensor* self= static_cast<KX_MouseFocusSensor*>(self_v);
+	KX_MouseFocusSensor* self = static_cast<KX_MouseFocusSensor*>(self_v);
 	return PyObjectFrom(self->RaySource());
 }
 
-PyObject* KX_MouseFocusSensor::pyattr_get_ray_target(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *KX_MouseFocusSensor::pyattr_get_ray_target(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	KX_MouseFocusSensor* self= static_cast<KX_MouseFocusSensor*>(self_v);
+	KX_MouseFocusSensor* self = static_cast<KX_MouseFocusSensor*>(self_v);
 	return PyObjectFrom(self->RayTarget());
 }
 
-PyObject* KX_MouseFocusSensor::pyattr_get_ray_direction(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *KX_MouseFocusSensor::pyattr_get_ray_direction(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	KX_MouseFocusSensor* self= static_cast<KX_MouseFocusSensor*>(self_v);
+	KX_MouseFocusSensor* self = static_cast<KX_MouseFocusSensor*>(self_v);
 	MT_Vector3 dir = self->RayTarget() - self->RaySource();
 	if (MT_fuzzyZero(dir))	dir.setValue(0,0,0);
 	else					dir.normalize();
 	return PyObjectFrom(dir);
 }
 
-PyObject* KX_MouseFocusSensor::pyattr_get_hit_object(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *KX_MouseFocusSensor::pyattr_get_hit_object(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	KX_MouseFocusSensor* self= static_cast<KX_MouseFocusSensor*>(self_v);
+	KX_MouseFocusSensor* self = static_cast<KX_MouseFocusSensor*>(self_v);
 	
 	if (self->m_hitObject)
 		return self->m_hitObject->GetProxy();
@@ -423,21 +421,21 @@ PyObject* KX_MouseFocusSensor::pyattr_get_hit_object(void *self_v, const KX_PYAT
 	Py_RETURN_NONE;
 }
 
-PyObject* KX_MouseFocusSensor::pyattr_get_hit_position(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *KX_MouseFocusSensor::pyattr_get_hit_position(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	KX_MouseFocusSensor* self= static_cast<KX_MouseFocusSensor*>(self_v);
+	KX_MouseFocusSensor* self = static_cast<KX_MouseFocusSensor*>(self_v);
 	return PyObjectFrom(self->HitPosition());
 }
 
-PyObject* KX_MouseFocusSensor::pyattr_get_hit_normal(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *KX_MouseFocusSensor::pyattr_get_hit_normal(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	KX_MouseFocusSensor* self= static_cast<KX_MouseFocusSensor*>(self_v);
+	KX_MouseFocusSensor* self = static_cast<KX_MouseFocusSensor*>(self_v);
 	return PyObjectFrom(self->HitNormal());
 }
 
-PyObject* KX_MouseFocusSensor::pyattr_get_hit_uv(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+PyObject *KX_MouseFocusSensor::pyattr_get_hit_uv(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
 {
-	KX_MouseFocusSensor* self= static_cast<KX_MouseFocusSensor*>(self_v);
+	KX_MouseFocusSensor* self = static_cast<KX_MouseFocusSensor*>(self_v);
 	return PyObjectFrom(self->HitUV());
 }
 
