@@ -36,16 +36,16 @@
 
 
 /**
-	PHY_IPhysicsController is the abstract simplified Interface to a physical object.
-	It contains the IMotionState and IDeformableMesh Interfaces.
-*/
+ * PHY_IPhysicsController is the abstract simplified Interface to a physical object.
+ * It contains the IMotionState and IDeformableMesh Interfaces.
+ */
 class PHY_IGraphicController : public PHY_IController
 {
 	public:
 		virtual ~PHY_IGraphicController();
 		/**
-			SynchronizeMotionStates ynchronizes dynas, kinematic and deformable entities (and do 'late binding')
-		*/
+		 * SynchronizeMotionStates ynchronizes dynas, kinematic and deformable entities (and do 'late binding')
+		 */
 		virtual bool SetGraphicTransform()=0;
 		virtual void Activate(bool active=true)=0;
 		virtual void setLocalAabb(const PHY__Vector3& aabbMin,const PHY__Vector3& aabbMax)=0;
@@ -54,10 +54,8 @@ class PHY_IGraphicController : public PHY_IController
 		virtual PHY_IGraphicController*	GetReplica(class PHY_IMotionState* motionstate) {return 0;}
 
 #ifdef WITH_CXX_GUARDEDALLOC
-	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:PHY_IController"); }
-	void operator delete( void *mem ) { MEM_freeN(mem); }
+	MEM_CXX_CLASS_ALLOC_FUNCS("GE:PHY_IController")
 #endif
 };
 
-#endif //__PHY_IGRAPHICCONTROLLER_H__
-
+#endif  /* __PHY_IGRAPHICCONTROLLER_H__ */
