@@ -38,21 +38,21 @@ class PHY_IMotionState;
 class PHY_IPhysicsEnvironment;
 
 /**
-	PHY_IPhysicsController is the abstract simplified Interface to a physical object.
-	It contains the IMotionState and IDeformableMesh Interfaces.
-*/
+ * PHY_IPhysicsController is the abstract simplified Interface to a physical object.
+ * It contains the IMotionState and IDeformableMesh Interfaces.
+ */
 class PHY_IPhysicsController : public PHY_IController
 {
 
 	public:
 		virtual ~PHY_IPhysicsController();
 		/**
-			SynchronizeMotionStates ynchronizes dynas, kinematic and deformable entities (and do 'late binding')
-		*/
+		 * SynchronizeMotionStates ynchronizes dynas, kinematic and deformable entities (and do 'late binding')
+		 */
 		virtual bool		SynchronizeMotionStates(float time)=0;
 		/**
-			WriteMotionStateToDynamics ynchronizes dynas, kinematic and deformable entities (and do 'late binding')
-		*/
+		 * WriteMotionStateToDynamics ynchronizes dynas, kinematic and deformable entities (and do 'late binding')
+		 */
 		
 		virtual void		WriteMotionStateToDynamics(bool nondynaonly)=0;
 		virtual	void		WriteDynamicsToMotionState()=0;
@@ -102,12 +102,10 @@ class PHY_IPhysicsController : public PHY_IController
 		
 		PHY__Vector3	GetWorldPosition(PHY__Vector3& localpos);
 
+
 #ifdef WITH_CXX_GUARDEDALLOC
-public:
-	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:PHY_IPhysicsController"); }
-	void operator delete( void *mem ) { MEM_freeN(mem); }
+	MEM_CXX_CLASS_ALLOC_FUNCS("GE:PHY_IPhysicsController")
 #endif
 };
 
-#endif //__PHY_IPHYSICSCONTROLLER_H__
-
+#endif  /* __PHY_IPHYSICSCONTROLLER_H__ */
