@@ -29,13 +29,11 @@
  *  \ingroup bgerast
  */
 
-
-#if defined(WIN32) && !defined(FREE_WINDOWS)
-// don't show these anoying STL warnings
-#pragma warning (disable:4786)
+#ifdef _MSC_VER
+   /* don't show these anoying STL warnings */
+#  pragma warning (disable:4786)
 #endif
 
-#include "CTR_Map.h"
 #include "RAS_MaterialBucket.h"
 #include "STR_HashedString.h"
 #include "RAS_MeshObject.h"
@@ -130,7 +128,7 @@ void RAS_BucketManager::OrderBuckets(const MT_Transform& cameratrans, BucketList
 		RAS_MaterialBucket* bucket = *bit;
 		RAS_MeshSlot* ms;
 		// remove the mesh slot form the list, it culls them automatically for next frame
-		while((ms = bucket->GetNextActiveMeshSlot())) {
+		while ((ms = bucket->GetNextActiveMeshSlot())) {
 			slots[i++].set(ms, bucket, pnorm);
 		}
 	}
@@ -150,14 +148,15 @@ void RAS_BucketManager::RenderAlphaBuckets(
 	// Having depth masks disabled/enabled gives different artifacts in
 	// case no sorting is done or is done inexact. For compatibility, we
 	// disable it.
-	rasty->SetDepthMask(RAS_IRasterizer::KX_DEPTHMASK_DISABLED);
+	if (rasty->GetDrawingMode() != RAS_IRasterizer::KX_SHADOW)
+		rasty->SetDepthMask(RAS_IRasterizer::KX_DEPTHMASK_DISABLED);
 
 	OrderBuckets(cameratrans, m_AlphaBuckets, slots, true);
 	
 	for (sit=slots.begin(); sit!=slots.end(); ++sit) {
 		rendertools->SetClientObject(rasty, sit->m_ms->m_clientObj);
 
-		while(sit->m_bucket->ActivateMaterial(cameratrans, rasty, rendertools))
+		while (sit->m_bucket->ActivateMaterial(cameratrans, rasty, rendertools))
 			sit->m_bucket->RenderMeshSlot(cameratrans, rasty, rendertools, *(sit->m_ms));
 
 		// make this mesh slot culled automatically for next frame
@@ -180,8 +179,7 @@ void RAS_BucketManager::RenderSolidBuckets(
 		RAS_MaterialBucket* bucket = *bit;
 		RAS_MeshSlot* ms;
 		// remove the mesh slot form the list, it culls them automatically for next frame
-		while((ms = bucket->GetNextActiveMeshSlot()))
-		{
+		while ((ms = bucket->GetNextActiveMeshSlot())) {
 			rendertools->SetClientObject(rasty, ms->m_clientObj);
 			while (bucket->ActivateMaterial(cameratrans, rasty, rendertools))
 				bucket->RenderMeshSlot(cameratrans, rasty, rendertools, *ms);
@@ -220,7 +218,7 @@ void RAS_BucketManager::RenderSolidBuckets(
 	for (sit=slots.begin(); sit!=slots.end(); ++sit) {
 		rendertools->SetClientObject(rasty, sit->m_ms->m_clientObj);
 
-		while(sit->m_bucket->ActivateMaterial(cameratrans, rasty, rendertools))
+		while (sit->m_bucket->ActivateMaterial(cameratrans, rasty, rendertools))
 			sit->m_bucket->RenderMeshSlot(cameratrans, rasty, rendertools, *(sit->m_ms));
 	}
 #endif
@@ -232,8 +230,26 @@ void RAS_BucketManager::Renderbuckets(
 	/* beginning each frame, clear (texture/material) caching information */
 	rasty->ClearCachingInfo();
 
-	RenderSolidBuckets(cameratrans, rasty, rendertools);	
-	RenderAlphaBuckets(cameratrans, rasty, rendertools);	
+	RenderSolidBuckets(cameratrans, rasty, rendertools);
+	RenderAlphaBuckets(cameratrans, rasty, rendertools);
+
+	/* All meshes should be up to date now */
+	/* Don't do this while processing buckets because some meshes are split between buckets */
+	BucketList::iterator bit;
+	list<RAS_MeshSlot>::iterator mit;
+	for (bit = m_SolidBuckets.begin(); bit != m_SolidBuckets.end(); ++bit) {
+		// RAS_MaterialBucket *bucket = *bit;  /* UNUSED */
+		for (mit = (*bit)->msBegin(); mit != (*bit)->msEnd(); ++mit) {
+			mit->m_mesh->SetMeshModified(false);
+		}
+	}
+	for (bit = m_AlphaBuckets.begin(); bit != m_AlphaBuckets.end(); ++bit) {
+		// RAS_MaterialBucket* bucket = *bit;  /* UNUSED */
+		for (mit = (*bit)->msBegin(); mit != (*bit)->msEnd(); ++mit) {
+			mit->m_mesh->SetMeshModified(false);
+		}
+	}
+	
 
 	rendertools->SetClientObject(rasty, NULL);
 }
