@@ -43,6 +43,10 @@
 #include "MT_CmMatrix4x4.h"
 #include "RAS_IRenderTools.h" // rendering text
 
+#include "RAS_StorageIM.h"
+#include "RAS_StorageVA.h"
+#include "RAS_StorageVBO.h"
+
 #include "GPU_draw.h"
 #include "GPU_material.h"
 #include "GPU_extensions.h"
@@ -52,7 +56,10 @@
 #include "DNA_material_types.h"
 #include "DNA_scene_types.h"
 
-#include "BKE_DerivedMesh.h"
+extern "C"{
+	#include "BLI_utildefines.h"
+	#include "BKE_DerivedMesh.h"
+}
 
 #ifndef M_PI
 #define M_PI		3.14159265358979323846
@@ -71,7 +78,7 @@ static GLuint right_eye_vinterlace_mask[32];
  */
 static GLuint hinterlace_mask[33];
 
-RAS_OpenGLRasterizer::RAS_OpenGLRasterizer(RAS_ICanvas* canvas)
+RAS_OpenGLRasterizer::RAS_OpenGLRasterizer(RAS_ICanvas* canvas, int storage)
 	:RAS_IRasterizer(canvas),
 	m_2DCanvas(canvas),
 	m_fogenabled(false),
@@ -86,11 +93,13 @@ RAS_OpenGLRasterizer::RAS_OpenGLRasterizer(RAS_ICanvas* canvas)
 	m_noOfScanlines(32),
 	m_motionblur(0),
 	m_motionblurvalue(-1.0),
+	m_usingoverrideshader(false),
 	m_texco_num(0),
 	m_attrib_num(0),
 	//m_last_alphablend(GPU_BLEND_SOLID),
 	m_last_frontface(true),
-	m_materialCachingInfo(0)
+	m_materialCachingInfo(0),
+	m_storage_type(storage)
 {
 	m_viewmatrix.setIdentity();
 	m_viewinvmatrix.setIdentity();
@@ -104,6 +113,24 @@ RAS_OpenGLRasterizer::RAS_OpenGLRasterizer(RAS_ICanvas* canvas)
 	hinterlace_mask[32] = 0;
 
 	m_prevafvalue = GPU_get_anisotropic();
+
+	if (m_storage_type == RAS_VBO /*|| m_storage_type == RAS_AUTO_STORAGE && GLEW_ARB_vertex_buffer_object*/)
+	{
+		m_storage = new RAS_StorageVBO(&m_texco_num, m_texco, &m_attrib_num, m_attrib, m_attrib_layer);
+		m_failsafe_storage = new RAS_StorageIM(&m_texco_num, m_texco, &m_attrib_num, m_attrib, m_attrib_layer);
+		m_storage_type = RAS_VBO;
+	}
+	else if ((m_storage_type == RAS_VA) || (m_storage_type == RAS_AUTO_STORAGE && GLEW_VERSION_1_1))
+	{
+		m_storage = new RAS_StorageVA(&m_texco_num, m_texco, &m_attrib_num, m_attrib, m_attrib_layer);
+		m_failsafe_storage = new RAS_StorageIM(&m_texco_num, m_texco, &m_attrib_num, m_attrib, m_attrib_layer);
+		m_storage_type = RAS_VA;
+	}
+	else
+	{
+		m_storage = m_failsafe_storage = new RAS_StorageIM(&m_texco_num, m_texco, &m_attrib_num, m_attrib, m_attrib_layer);
+		m_storage_type = RAS_IMMEDIATE;
+	}
 }
 
 
@@ -112,10 +139,16 @@ RAS_OpenGLRasterizer::~RAS_OpenGLRasterizer()
 {
 	// Restore the previous AF value
 	GPU_set_anisotropic(m_prevafvalue);
+	if (m_failsafe_storage && m_failsafe_storage != m_storage)
+		delete m_failsafe_storage;
+
+	if (m_storage)
+		delete m_storage;
 }
 
 bool RAS_OpenGLRasterizer::Init()
 {
+	bool storage_init;
 	GPU_state_init();
 
 
@@ -143,7 +176,9 @@ bool RAS_OpenGLRasterizer::Init()
 
 	glShadeModel(GL_SMOOTH);
 
-	return true;
+	storage_init = m_storage->Init();
+
+	return true && storage_init;
 }
 
 
@@ -239,10 +274,10 @@ void RAS_OpenGLRasterizer::DisplayFog()
 		glFogf(GL_FOG_DENSITY, 0.1f);
 		glFogf(GL_FOG_START, m_fogstart);
 		glFogf(GL_FOG_END, m_fogstart + m_fogdist);
-		params[0]= m_fogr;
-		params[1]= m_fogg;
-		params[2]= m_fogb;
-		params[3]= 0.0;
+		params[0] = m_fogr;
+		params[1] = m_fogg;
+		params[2] = m_fogb;
+		params[3] = 0.0;
 		glFogfv(GL_FOG_COLOR, params); 
 		glEnable(GL_FOG);
 	} 
@@ -264,6 +299,8 @@ bool RAS_OpenGLRasterizer::SetMaterial(const RAS_IPolyMaterial& mat)
 void RAS_OpenGLRasterizer::Exit()
 {
 
+	m_storage->Exit();
+
 	glEnable(GL_CULL_FACE);
 	glEnable(GL_DEPTH_TEST);
 	glClearDepth(1.0); 
@@ -286,7 +323,7 @@ void RAS_OpenGLRasterizer::Exit()
 bool RAS_OpenGLRasterizer::BeginFrame(int drawingmode, double time)
 {
 	m_time = time;
-	m_drawingmode = drawingmode;
+	SetDrawingMode(drawingmode);
 
 	// Blender camera routine destroys the settings
 	if (m_drawingmode < KX_SOLID)
@@ -325,6 +362,11 @@ void RAS_OpenGLRasterizer::SetDrawingMode(int drawingmode)
 
 	if (m_drawingmode == KX_WIREFRAME)
 		glDisable(GL_CULL_FACE);
+
+	m_storage->SetDrawingMode(drawingmode);
+	if (m_failsafe_storage && m_failsafe_storage != m_storage) {
+		m_failsafe_storage->SetDrawingMode(drawingmode);
+	}
 }
 
 int RAS_OpenGLRasterizer::GetDrawingMode()
@@ -359,7 +401,7 @@ void RAS_OpenGLRasterizer::ClearCachingInfo(void)
 
 void RAS_OpenGLRasterizer::FlushDebugShapes()
 {
-	if (!m_debugShapes.size())
+	if (m_debugShapes.empty())
 		return;
 
 	// DrawDebugLines
@@ -438,7 +480,7 @@ void RAS_OpenGLRasterizer::EndFrame()
 	glDisable(GL_MULTISAMPLE_ARB);
 
 	m_2DCanvas->EndFrame();
-}	
+}
 
 void RAS_OpenGLRasterizer::SetRenderArea()
 {
@@ -447,8 +489,7 @@ void RAS_OpenGLRasterizer::SetRenderArea()
 	switch (m_stereomode)
 	{
 		case RAS_STEREO_ABOVEBELOW:
-			switch(m_curreye)
-			{
+			switch (m_curreye) {
 				case RAS_STEREO_LEFTEYE:
 					// upper half of window
 					area.SetLeft(0);
@@ -533,10 +574,10 @@ void RAS_OpenGLRasterizer::SetEye(const StereoEye eye)
 			glDrawBuffer(m_curreye == RAS_STEREO_LEFTEYE ? GL_BACK_LEFT : GL_BACK_RIGHT);
 			break;
 		case RAS_STEREO_ANAGLYPH:
-			if (m_curreye == RAS_STEREO_LEFTEYE)
-			{
+			if (m_curreye == RAS_STEREO_LEFTEYE) {
 				glColorMask(GL_FALSE, GL_TRUE, GL_TRUE, GL_FALSE);
-			} else {
+			}
+			else {
 				//glAccum(GL_LOAD, 1.0);
 				glColorMask(GL_TRUE, GL_FALSE, GL_FALSE, GL_FALSE);
 				ClearDepthBuffer();
@@ -664,7 +705,7 @@ void RAS_OpenGLRasterizer::IndexPrimitives_3DText(RAS_MeshSlot& ms,
 				glattrib = -1;
 				if (GLEW_ARB_vertex_program)
 					for (unit=0; unit<m_attrib_num; unit++)
-						if (m_attrib[unit] == RAS_TEXCO_UV1)
+						if (m_attrib[unit] == RAS_TEXCO_UV)
 							glattrib = unit;
 				
 				rendertools->RenderText(polymat->GetDrawingMode(), polymat,
@@ -699,272 +740,38 @@ void RAS_OpenGLRasterizer::SetTexCoord(TexCoGen coords, int unit)
 		m_texco[unit] = coords;
 }
 
-void RAS_OpenGLRasterizer::SetAttrib(TexCoGen coords, int unit)
+void RAS_OpenGLRasterizer::SetAttrib(TexCoGen coords, int unit, int layer)
 {
 	// this changes from material to material
-	if (unit < RAS_MAX_ATTRIB)
+	if (unit < RAS_MAX_ATTRIB) {
 		m_attrib[unit] = coords;
-}
-
-void RAS_OpenGLRasterizer::TexCoord(const RAS_TexVert &tv)
-{
-	int unit;
-
-	if (GLEW_ARB_multitexture) {
-		for (unit=0; unit<m_texco_num; unit++) {
-			if (tv.getFlag() & RAS_TexVert::SECOND_UV && (int)tv.getUnit() == unit) {
-				glMultiTexCoord2fvARB(GL_TEXTURE0_ARB+unit, tv.getUV2());
-				continue;
-			}
-			switch(m_texco[unit]) {
-			case RAS_TEXCO_ORCO:
-			case RAS_TEXCO_GLOB:
-				glMultiTexCoord3fvARB(GL_TEXTURE0_ARB+unit, tv.getXYZ());
-				break;
-			case RAS_TEXCO_UV1:
-				glMultiTexCoord2fvARB(GL_TEXTURE0_ARB+unit, tv.getUV1());
-				break;
-			case RAS_TEXCO_NORM:
-				glMultiTexCoord3fvARB(GL_TEXTURE0_ARB+unit, tv.getNormal());
-				break;
-			case RAS_TEXTANGENT:
-				glMultiTexCoord4fvARB(GL_TEXTURE0_ARB+unit, tv.getTangent());
-				break;
-			case RAS_TEXCO_UV2:
-				glMultiTexCoord2fvARB(GL_TEXTURE0_ARB+unit, tv.getUV2());
-				break;
-			default:
-				break;
-			}
-		}
+		m_attrib_layer[unit] = layer;
 	}
-
-	if (GLEW_ARB_vertex_program) {
-		for (unit=0; unit<m_attrib_num; unit++) {
-			switch(m_attrib[unit]) {
-			case RAS_TEXCO_ORCO:
-			case RAS_TEXCO_GLOB:
-				glVertexAttrib3fvARB(unit, tv.getXYZ());
-				break;
-			case RAS_TEXCO_UV1:
-				glVertexAttrib2fvARB(unit, tv.getUV1());
-				break;
-			case RAS_TEXCO_NORM:
-				glVertexAttrib3fvARB(unit, tv.getNormal());
-				break;
-			case RAS_TEXTANGENT:
-				glVertexAttrib4fvARB(unit, tv.getTangent());
-				break;
-			case RAS_TEXCO_UV2:
-				glVertexAttrib2fvARB(unit, tv.getUV2());
-				break;
-			case RAS_TEXCO_VCOL:
-				glVertexAttrib4ubvARB(unit, tv.getRGBA());
-				break;
-			default:
-				break;
-			}
-		}
-	}
-
 }
 
 void RAS_OpenGLRasterizer::IndexPrimitives(RAS_MeshSlot& ms)
 {
-	IndexPrimitivesInternal(ms, false);
+	if (ms.m_pDerivedMesh)
+		m_failsafe_storage->IndexPrimitives(ms);
+	else
+		m_storage->IndexPrimitives(ms);
 }
 
 void RAS_OpenGLRasterizer::IndexPrimitivesMulti(RAS_MeshSlot& ms)
 {
-	IndexPrimitivesInternal(ms, true);
-}
-
-static bool current_wireframe;
-static RAS_MaterialBucket *current_bucket;
-static RAS_IPolyMaterial *current_polymat;
-static RAS_MeshSlot *current_ms;
-static RAS_MeshObject *current_mesh;
-static int current_blmat_nr;
-static GPUVertexAttribs current_gpu_attribs;
-static Image *current_image;
-static int CheckMaterialDM(int matnr, void *attribs)
-{
-	// only draw the current material
-	if (matnr != current_blmat_nr)
-		return 0;
-	GPUVertexAttribs *gattribs = (GPUVertexAttribs *)attribs;
-	if (gattribs)
-		memcpy(gattribs, &current_gpu_attribs, sizeof(GPUVertexAttribs));
-	return 1;
-}
-
-/*
-static int CheckTexfaceDM(void *mcol, int index)
-{
-
-	// index is the original face index, retrieve the polygon
-	RAS_Polygon* polygon = (index >= 0 && index < current_mesh->NumPolygons()) ?
-		current_mesh->GetPolygon(index) : NULL;
-	if (polygon && polygon->GetMaterial() == current_bucket) {
-		// must handle color.
-		if (current_wireframe)
-			return 2;
-		if (current_ms->m_bObjectColor) {
-			MT_Vector4& rgba = current_ms->m_RGBAcolor;
-			glColor4d(rgba[0], rgba[1], rgba[2], rgba[3]);
-			// don't use mcol
-			return 2;
-		}
-		if (!mcol) {
-			// we have to set the color from the material
-			unsigned char rgba[4];
-			current_polymat->GetMaterialRGBAColor(rgba);
-			glColor4ubv((const GLubyte *)rgba);
-			return 2;
-		}
-		return 1;
-	}
-	return 0;
-}
-*/
-
-static DMDrawOption CheckTexDM(MTFace *tface, int has_mcol, int matnr)
-{
-
-	// index is the original face index, retrieve the polygon
-	if (matnr == current_blmat_nr &&
-		(tface == NULL || tface->tpage == current_image)) {
-		// must handle color.
-		if (current_wireframe)
-			return DM_DRAW_OPTION_NO_MCOL;
-		if (current_ms->m_bObjectColor) {
-			MT_Vector4& rgba = current_ms->m_RGBAcolor;
-			glColor4d(rgba[0], rgba[1], rgba[2], rgba[3]);
-			// don't use mcol
-			return DM_DRAW_OPTION_NO_MCOL;
-		}
-		if (!has_mcol) {
-			// we have to set the color from the material
-			unsigned char rgba[4];
-			current_polymat->GetMaterialRGBAColor(rgba);
-			glColor4ubv((const GLubyte *)rgba);
-			return DM_DRAW_OPTION_NO_MCOL;
-		}
-		return DM_DRAW_OPTION_NORMAL;
-	}
-	return DM_DRAW_OPTION_SKIP;
-}
-
-void RAS_OpenGLRasterizer::IndexPrimitivesInternal(RAS_MeshSlot& ms, bool multi)
-{ 
-	bool obcolor = ms.m_bObjectColor;
-	bool wireframe = m_drawingmode <= KX_WIREFRAME;
-	MT_Vector4& rgba = ms.m_RGBAcolor;
-	RAS_MeshSlot::iterator it;
-
-	if (ms.m_pDerivedMesh) {
-		// mesh data is in derived mesh, 
-		current_bucket = ms.m_bucket;
-		current_polymat = current_bucket->GetPolyMaterial();
-		current_ms = &ms;
-		current_mesh = ms.m_mesh;
-		current_wireframe = wireframe;
-		// MCol *mcol = (MCol*)ms.m_pDerivedMesh->getFaceDataArray(ms.m_pDerivedMesh, CD_MCOL); /* UNUSED */
-
-		// handle two-side
-		if (current_polymat->GetDrawingMode() & RAS_IRasterizer::KX_BACKCULL)
-			this->SetCullFace(true);
-		else
-			this->SetCullFace(false);
-
-		if (current_polymat->GetFlag() & RAS_BLENDERGLSL) {
-			// GetMaterialIndex return the original mface material index, 
-			// increment by 1 to match what derived mesh is doing
-			current_blmat_nr = current_polymat->GetMaterialIndex()+1;
-			// For GLSL we need to retrieve the GPU material attribute
-			Material* blmat = current_polymat->GetBlenderMaterial();
-			Scene* blscene = current_polymat->GetBlenderScene();
-			if (!wireframe && blscene && blmat)
-				GPU_material_vertex_attributes(GPU_material_from_blender(blscene, blmat), &current_gpu_attribs);
-			else
-				memset(&current_gpu_attribs, 0, sizeof(current_gpu_attribs));
-			// DM draw can mess up blending mode, restore at the end
-			int current_blend_mode = GPU_get_material_alpha_blend();
-			ms.m_pDerivedMesh->drawFacesGLSL(ms.m_pDerivedMesh, CheckMaterialDM);
-			GPU_set_material_alpha_blend(current_blend_mode);
-		} else {
-			//ms.m_pDerivedMesh->drawMappedFacesTex(ms.m_pDerivedMesh, CheckTexfaceDM, mcol);
-			current_blmat_nr = current_polymat->GetMaterialIndex();
-			current_image = current_polymat->GetBlenderImage();
-			ms.m_pDerivedMesh->drawFacesTex(ms.m_pDerivedMesh, CheckTexDM, NULL, NULL);
-		}
-		return;
-	}
-	// iterate over display arrays, each containing an index + vertex array
-	for (ms.begin(it); !ms.end(it); ms.next(it)) {
-		RAS_TexVert *vertex;
-		size_t i, j, numvert;
-		
-		numvert = it.array->m_type;
-
-		if (it.array->m_type == RAS_DisplayArray::LINE) {
-			// line drawing
-			glBegin(GL_LINES);
-
-			for (i=0; i<it.totindex; i+=2)
-			{
-				vertex = &it.vertex[it.index[i]];
-				glVertex3fv(vertex->getXYZ());
-
-				vertex = &it.vertex[it.index[i+1]];
-				glVertex3fv(vertex->getXYZ());
-			}
-
-			glEnd();
-		}
-		else {
-			// triangle and quad drawing
-			if (it.array->m_type == RAS_DisplayArray::TRIANGLE)
-				glBegin(GL_TRIANGLES);
-			else
-				glBegin(GL_QUADS);
-
-			for (i=0; i<it.totindex; i+=numvert)
-			{
-				if (obcolor)
-					glColor4d(rgba[0], rgba[1], rgba[2], rgba[3]);
-
-				for (j=0; j<numvert; j++) {
-					vertex = &it.vertex[it.index[i+j]];
-
-					if (!wireframe) {
-						if (!obcolor)
-							glColor4ubv((const GLubyte *)(vertex->getRGBA()));
-
-						glNormal3fv(vertex->getNormal());
-
-						if (multi)
-							TexCoord(*vertex);
-						else
-							glTexCoord2fv(vertex->getUV1());
-					}
-
-					glVertex3fv(vertex->getXYZ());
-				}
-			}
-
-			glEnd();
-		}
-	}
+	if (ms.m_pDerivedMesh)
+		m_failsafe_storage->IndexPrimitivesMulti(ms);
+	else
+		m_storage->IndexPrimitivesMulti(ms);
 }
 
 void RAS_OpenGLRasterizer::SetProjectionMatrix(MT_CmMatrix4x4 &mat)
 {
 	glMatrixMode(GL_PROJECTION);
-	double* matrix = &mat(0,0);
+	double* matrix = &mat(0, 0);
 	glLoadMatrixd(matrix);
 
-	m_camortho= (mat(3, 3) != 0.0f);
+	m_camortho = (mat(3, 3) != 0.0);
 }
 
 void RAS_OpenGLRasterizer::SetProjectionMatrix(const MT_Matrix4x4 & mat)
@@ -974,9 +781,9 @@ void RAS_OpenGLRasterizer::SetProjectionMatrix(const MT_Matrix4x4 & mat)
 	/* Get into argument. Looks a bit dodgy, but it's ok. */
 	mat.getValue(matrix);
 	/* Internally, MT_Matrix4x4 uses doubles (MT_Scalar). */
-	glLoadMatrixd(matrix);	
+	glLoadMatrixd(matrix);
 
-	m_camortho= (mat[3][3] != 0.0f);
+	m_camortho= (mat[3][3] != 0.0);
 }
 
 MT_Matrix4x4 RAS_OpenGLRasterizer::GetFrustumMatrix(
@@ -1001,13 +808,12 @@ MT_Matrix4x4 RAS_OpenGLRasterizer::GetFrustumMatrix(
 			// if Rasterizer.setFocalLength is not called we use the camera focallength
 			if (!m_setfocallength)
 				// if focallength is null we use a value known to be reasonable
-				m_focallength = (focallength == 0.f) ? m_eyeseparation * 30.0
+				m_focallength = (focallength == 0.f) ? m_eyeseparation * 30.0f
 					: focallength;
 
 			near_div_focallength = frustnear / m_focallength;
-			offset = 0.5 * m_eyeseparation * near_div_focallength;
-			switch(m_curreye)
-			{
+			offset = 0.5f * m_eyeseparation * near_div_focallength;
+			switch (m_curreye) {
 				case RAS_STEREO_LEFTEYE:
 						left += offset;
 						right += offset;
@@ -1077,8 +883,7 @@ void RAS_OpenGLRasterizer::SetViewMatrix(const MT_Matrix4x4 &mat,
 		// vector between eyes
 		eyeline = viewDir.cross(viewupVec);
 
-		switch(m_curreye)
-		{
+		switch (m_curreye) {
 			case RAS_STEREO_LEFTEYE:
 				{
 				// translate to left by half the eye distance
@@ -1258,3 +1063,14 @@ short RAS_OpenGLRasterizer::GetAnisotropicFiltering()
 {
 	return (short)GPU_get_anisotropic();
 }
+
+void RAS_OpenGLRasterizer::SetUsingOverrideShader(bool val)
+{
+	m_usingoverrideshader = val;
+}
+
+bool RAS_OpenGLRasterizer::GetUsingOverrideShader()
+{
+	return m_usingoverrideshader;
+}
+
