@@ -48,23 +48,21 @@ class RAS_TexVert
 {
 	
 	float			m_localxyz[3];	// 3*4 = 12
-	float			m_uv1[2];		// 2*4 =  8
-	float			m_uv2[2];		// 2*4 =  8
+	float			m_uvs[8][2];	// 8*2*4=64		//8 = MAX_UNIT
 	unsigned int	m_rgba;			//        4
-	float			m_tangent[4];   // 4*4 =  16
-	float			m_normal[3];	// 3*4 =  12
+	float			m_tangent[4];   // 4*4 = 16
+	float			m_normal[3];	// 3*4 = 12
 	short			m_flag;			//        2
 	short			m_softBodyIndex;		//2
 	unsigned int	m_unit;			//		  4
 	unsigned int	m_origindex;		//    4
 									//---------
-									//       56+6+8+2=72
-	// 32 bytes total size, fits nice = 56 = not fit nice.
+									//      120
+	// 32 bytes total size, fits nice = 120 = not fit nice.
 
 public:
 	enum {
 		FLAT = 1,
-		SECOND_UV = 2,
 		MAX_UNIT = 8
 	};
 
@@ -74,8 +72,7 @@ public:
 	RAS_TexVert()// :m_xyz(0,0,0),m_uv(0,0),m_rgba(0)
 	{}
 	RAS_TexVert(const MT_Point3& xyz,
-				const MT_Point2& uv,
-				const MT_Point2& uv2,
+				const MT_Point2 uvs[MAX_UNIT],
 				const MT_Vector4& tangent,
 				const unsigned int rgba,
 				const MT_Vector3& normal,
@@ -83,12 +80,8 @@ public:
 				const unsigned int origindex);
 	~RAS_TexVert() {};
 
-	const float* getUV1 () const { 
-		return m_uv1;
-	};
-
-	const float* getUV2 () const { 
-		return m_uv2;
+	const float* getUV (int unit) const {
+		return m_uvs[unit];
 	};
 
 	const float* getXYZ() const { 
@@ -122,9 +115,9 @@ public:
 	}
 
 	void				SetXYZ(const MT_Point3& xyz);
-	void				SetXYZ(const float *xyz);
-	void				SetUV(const MT_Point2& uv);
-	void				SetUV2(const MT_Point2& uv);
+	void				SetXYZ(const float xyz[3]);
+	void				SetUV(int index, const MT_Point2& uv);
+	void				SetUV(int index, const float uv[2]);
 
 	void				SetRGBA(const unsigned int rgba);
 	void				SetNormal(const MT_Vector3& normal);
@@ -137,17 +130,16 @@ public:
 
 	void				Transform(const class MT_Matrix4x4& mat,
 	                              const class MT_Matrix4x4& nmat);
+	void				TransformUV(int index, const MT_Matrix4x4& mat);
 
 	// compare two vertices, to test if they can be shared, used for
 	// splitting up based on uv's, colors, etc
 	bool				closeTo(const RAS_TexVert* other);
-	
+
+
 #ifdef WITH_CXX_GUARDEDALLOC
-public:
-	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:RAS_TexVert"); }
-	void operator delete( void *mem ) { MEM_freeN(mem); }
+	MEM_CXX_CLASS_ALLOC_FUNCS("GE:RAS_TexVert")
 #endif
 };
 
-#endif //__RAS_TEXVERT_H__
-
+#endif  /* __RAS_TEXVERT_H__ */
