@@ -1,24 +1,28 @@
 /*
------------------------------------------------------------------------------
-This source file is part of VideoTexture library
-
-Copyright (c) 2007 The Zdeno Ash Miklas
-
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU Lesser General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
-
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place - Suite 330, Boston, MA 02111-1307, USA, or go to
-http://www.gnu.org/copyleft/lesser.txt.
------------------------------------------------------------------------------
-*/
+ * ***** BEGIN GPL LICENSE BLOCK *****
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software  Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *
+ * Copyright (c) 2007 The Zdeno Ash Miklas
+ *
+ * This source file is part of VideoTexture library
+ *
+ * Contributor(s):
+ *
+ * ***** END GPL LICENSE BLOCK *****
+ */
 
 /** \file gameengine/VideoTexture/ImageBuff.cpp
  *  \ingroup bgevideotex
@@ -26,7 +30,7 @@ http://www.gnu.org/copyleft/lesser.txt.
 
 // implementation
 
-#include <PyObjectPlus.h>
+#include "PyObjectPlus.h"
 #include <structmember.h>
 
 #include "ImageBuff.h"
@@ -47,7 +51,7 @@ FilterRGB24 defFilter;
 // forward declaration;
 extern PyTypeObject ImageBuffType;
 
-static int ImageBuff_init (PyObject * pySelf, PyObject * args, PyObject * kwds)
+static int ImageBuff_init (PyObject *pySelf, PyObject *args, PyObject *kwds)
 {
 	short width = -1;
 	short height = -1;
@@ -55,7 +59,7 @@ static int ImageBuff_init (PyObject * pySelf, PyObject * args, PyObject * kwds)
 	PyObject *py_scale = Py_False;
 	ImageBuff *image;
 
-	PyImage * self = reinterpret_cast<PyImage*>(pySelf);
+	PyImage *self = reinterpret_cast<PyImage*>(pySelf);
 	// create source object
 	if (self->m_image != NULL) 
 		delete self->m_image;
@@ -144,7 +148,7 @@ void ImageBuff::clear (short width, short height, unsigned char color)
 // img must point to a array of RGBA data of size width*height
 void ImageBuff::plot (unsigned char * img, short width, short height, short x, short y, short mode)
 {
-	struct ImBuf* tmpbuf;
+	struct ImBuf *tmpbuf;
 
 	if (m_size[0] == 0 || m_size[1] == 0 || width <= 0 || height <= 0)
 		return;
@@ -190,7 +194,7 @@ void ImageBuff::plot (ImageBuff* img, short x, short y, short mode)
 
 
 // cast Image pointer to ImageBuff
-inline ImageBuff * getImageBuff (PyImage * self)
+inline ImageBuff * getImageBuff (PyImage *self)
 { return static_cast<ImageBuff*>(self->m_image); }
 
 
@@ -245,7 +249,7 @@ static bool testBGLBuffer(Buffer* buffer, int width, int height, unsigned int pi
 
 
 // load image
-static PyObject * load (PyImage * self, PyObject * args)
+static PyObject *load(PyImage *self, PyObject *args)
 {
 	// parameters: string image buffer, its size, width, height
 	Py_buffer buffer;
@@ -306,10 +310,10 @@ static PyObject * load (PyImage * self, PyObject * args)
 	}
 	if (PyErr_Occurred())
 		return NULL;
-	Py_RETURN_NONE;	
+	Py_RETURN_NONE;
 }
 
-static PyObject * plot (PyImage * self, PyObject * args)
+static PyObject *plot (PyImage *self, PyObject *args)
 {
 	PyImage * other;
 	Buffer* bglBuffer;
@@ -332,7 +336,7 @@ static PyObject * plot (PyImage * self, PyObject * args)
 		PyBuffer_Release(&buffer);
 		if (PyErr_Occurred())
 			return NULL;
-		Py_RETURN_NONE;	
+		Py_RETURN_NONE;
 	}
 	PyErr_Clear();
 	// try the other format
@@ -354,7 +358,7 @@ static PyObject * plot (PyImage * self, PyObject * args)
 	}
 	if (PyErr_Occurred())
 		return NULL;
-	Py_RETURN_NONE;	
+	Py_RETURN_NONE;
 }
 
 // methods structure
